dofile_once("mods/ArcaneReliquary/files/scripts/utilities/arcanereliquary_utilities.lua")
dofile_once("mods/ArcaneReliquary/files/scripts/lists/relics_list.lua")

function spawnItemsFromChestandKillEntities(chest_name, chest_id, key_id)
	
	-- settings for each compatible chest type
	local message = ""
	local text_decoration = ""
	local emitter_path = ""
	if(chest_name == "relic_chest_red") then
		message = "You unlocked a Red Relic Chest!"
		text_decoration = "mods/ArcaneReliquary/files/other_gfx/custom_decoration_chest_red.png"
		emitter_path = "mods/ArcaneReliquary/files/entity_definitions/image_emitters/chest_effect_red.xml"
	elseif(chest_name == "relic_chest_purple") then
		message = "You unlocked a Purple Relic Chest!"
		text_decoration = "mods/ArcaneReliquary/files/other_gfx/custom_decoration_chest_purple.png"
		emitter_path = "mods/ArcaneReliquary/files/entity_definitions/image_emitters/chest_effect_purple.xml"
	elseif(chest_name == "relic_chest_black") then
		message = "You unlocked a Black Relic Chest!"
		text_decoration = "mods/ArcaneReliquary/files/other_gfx/custom_decoration_chest_black.png"
		emitter_path = "mods/ArcaneReliquary/files/entity_definitions/image_emitters/chest_effect_black.xml"
	end
	
	-- gives user feedback according to settings above
	GamePrint(message)
	GamePrintImportant(message, "Chest unlocked", text_decoration)
	local x, y = EntityGetTransform(chest_id)
	EntityLoad(emitter_path, x, y)
	EntityKill(chest_id)
	EntityKill(key_id)
	
	local y_offset = 5
	-- items in each chest type
	if(chest_name == "relic_chest_red") then
		-- red chest spawns non-leveled relics once unlocked
		for i=1,2 do
			local relic = getRandomRelic()
			EntityLoad(getRelicsBaseDir() .. relic.name .. ".xml", x, y - y_offset)
			
			-- if spawned relic requires another relic, there's a 50% chance it will also spawn
			if(relic.requires_relic ~= nil) then
				if(Random(1,2) == 1) then
					local required_relic = getRelic(relic.requires_relic)
					EntityLoad(getRelicsBaseDir() .. required_relic.name .. ".xml", x, y - 5)
				end
			end
		end
	elseif(chest_name == "relic_chest_purple") then
		-- purple chest spawns random leveled relics, when applicable, once unlocked
		for i=1,3 do
			local relic = getRandomUpgradableRelic()
			local relic_id = EntityLoad(getRelicsBaseDir() .. relic.name .. ".xml", x, y - y_offset)
			setRelicLevel(relic, relic_id, Random(1,getRelicMaxUpgradeLevel()))

			-- if spawned relic requires another relic, there's a 50% chance it will also spawn
			if(relic.requires_relic ~= nil) then
				if(Random(1,2) == 1) then
					local required_relic = getRelic(relic.requires_relic)
					EntityLoad(getRelicsBaseDir() .. required_relic.name .. ".xml", x, y - 5)
				end
			end
		end
	elseif(chest_name == "relic_chest_black") then
		-- black chests spawn max level relics, when applicable, once unlocked
		for i=1,4 do
			local relic = getRandomUpgradableRelic()
			local relic_id = EntityLoad(getRelicsBaseDir() .. relic.name .. ".xml", x, y - y_offset)
			setRelicLevel(relic, relic_id, 10)
			
			-- if spawned relic requires another relic, there's a 50% chance it will also spawn
			if(relic.requires_relic ~= nil) then
				if(Random(1,2) == 1) then
					local required_relic = getRelic(relic.requires_relic)
					EntityLoad(getRelicsBaseDir() .. required_relic.name .. ".xml", x, y - 5)
				end
			end
		end
	end
	
end

-- obtains id of upgrade stone
local chest_key_id = GetUpdatedEntityID()
local chest_key_name = EntityGetName(chest_key_id)

-- detects relics nearby
local key_x, key_y = EntityGetTransform(chest_key_id)
local locked_chests = EntityGetInRadiusWithTag( key_x, key_y, 20, "locked_relic_chest" )

-- obtains necessary data
local locked_chest_id = locked_chests[1]
local locked_chest_name = EntityGetName(locked_chest_id)

-- red keys unlocks just red chests
if(chest_key_name == "chest_key_red" and locked_chest_name == "relic_chest_red") then 
	spawnItemsFromChestandKillEntities(locked_chest_name, locked_chest_id, chest_key_id)
-- purple keys unlocks red and purple chests
elseif(chest_key_name == "chest_key_purple" and (locked_chest_name == "relic_chest_purple" or locked_chest_name == "relic_chest_red")) then 
	spawnItemsFromChestandKillEntities(locked_chest_name, locked_chest_id, chest_key_id)
-- black keys unlocks red, purple and black chests
elseif(chest_key_name == "chest_key_black" and (locked_chest_name == "relic_chest_black" or locked_chest_name == "relic_chest_purple" or locked_chest_name == "relic_chest_red")) then
	spawnItemsFromChestandKillEntities(locked_chest_name, locked_chest_id, chest_key_id)
end