dofile_once("mods/ArcaneReliquary/files/scripts/utilities/arcanereliquary_utilities.lua")

function collision_trigger(colliding_entity_id)
	
	local entity_id = GetUpdatedEntityID()

	if(not EntityHasTag(colliding_entity_id, "item_physics")) then return end -- making sure entity that triggered this was a projectile
	if(not isItemDropped(colliding_entity_id)) then return end -- also doesn't trigger if the item is not out of the player's inventory
	if(not string.find(EntityGetName(colliding_entity_id), "seed_")) then return end -- ignores items that are not seeds with the "seed_" prefix as its entity name
	
	local x, y = EntityGetTransform(colliding_entity_id)
	
	-- Loads audiovisual feedback
	EntityLoad("mods/ArcaneReliquary/files/entity_definitions/particle_explosions/enchanted_pot_plant_seed_explosion.xml", x, y)
	local ic_id = EntityGetFirstComponentIncludingDisabled(colliding_entity_id, "ItemComponent")
	local ui_name = ComponentGetValue2(ic_id, "item_name")
	
	-- sets planted seed config
	setInternalVariableValue(entity_id, "has_seed", "value_bool", true)
	
	-- sets appropriate sprite file, grow material name and reward type according to seed type
	local sc_id = EntityGetFirstComponentIncludingDisabled(entity_id, "SpriteComponent")
	if(EntityGetName(colliding_entity_id) == "seed_water") then
		ComponentSetValue( sc_id, "image_file", "mods/ArcaneReliquary/files/creatures_gfx/pot_water.xml")
		setInternalVariableValue(entity_id, "grow_material_name", "value_string", "water")
		setInternalVariableValue(entity_id, "reward_type", "value_string", "spell_card")
		
		GamePrint(ui_name .. " planted on Enchanted Pot, make it consume water to grow it.")
	elseif(EntityGetName(colliding_entity_id) == "seed_lava") then
		ComponentSetValue( sc_id, "image_file", "mods/ArcaneReliquary/files/creatures_gfx/pot_lava.xml")
		setInternalVariableValue(entity_id, "grow_material_name", "value_string", "lava")
		setInternalVariableValue(entity_id, "reward_type", "value_string", "perk_card")
		
		GamePrint(ui_name .. " planted on Enchanted Pot, make it consume lava to grow it.")
	elseif(EntityGetName(colliding_entity_id) == "seed_poly") then
		ComponentSetValue( sc_id, "image_file", "mods/ArcaneReliquary/files/creatures_gfx/pot_poly.xml")
		setInternalVariableValue(entity_id, "grow_material_name", "value_string", "magic_liquid_polymorph")
		setInternalVariableValue(entity_id, "reward_type", "value_string", "random_wand")
		
		GamePrint(ui_name .. " planted on Enchanted Pot, make it consume regular polymorphgine to grow it.")
	elseif(EntityGetName(colliding_entity_id) == "seed_blood") then
		ComponentSetValue( sc_id, "image_file", "mods/ArcaneReliquary/files/creatures_gfx/pot_blood.xml")
		setInternalVariableValue(entity_id, "grow_material_name", "value_string", "blood")
		setInternalVariableValue(entity_id, "reward_type", "value_string", "random_vanilla_item")
		
		GamePrint(ui_name .. " planted on Enchanted Pot, make it consume red blood to grow it.")
	elseif(EntityGetName(colliding_entity_id) == "seed_toxic") then
		ComponentSetValue( sc_id, "image_file", "mods/ArcaneReliquary/files/creatures_gfx/pot_toxic.xml")
		setInternalVariableValue(entity_id, "grow_material_name", "value_string", "radioactive_liquid")
		setInternalVariableValue(entity_id, "reward_type", "value_string", "leveled_relic")
		
		GamePrint(ui_name .. " planted on Enchanted Pot, make it consume toxic sludge to grow it.")
	elseif(EntityGetName(colliding_entity_id) == "seed_purple_slime") then
		ComponentSetValue( sc_id, "image_file", "mods/ArcaneReliquary/files/creatures_gfx/pot_slime.xml")
		setInternalVariableValue(entity_id, "grow_material_name", "value_string", "slime")
		setInternalVariableValue(entity_id, "reward_type", "value_string", "random_chest")
		
		GamePrint(ui_name .. " planted on Enchanted Pot, make it consume slime to grow it.")
	end
	
	-- advances to stage1 animation
	ComponentSetValue( sc_id, "rect_animation", "stage1")
	
	-- disables collision trigger on enchanted pot so no other seeds will be collected until it is enabled again
	toggleCollisionTrigger(entity_id, "seed_get_collision_trigger", false)
	-- enables sucker
	toggleMaterialSucker(entity_id, "material_suck", true)
	
	-- kills seed
	EntityKill(colliding_entity_id)
end