dofile_once("mods/ArcaneReliquary/files/scripts/utilities/arcanereliquary_utilities.lua")
dofile_once("mods/ArcaneReliquary/files/scripts/lists/relic_precursor_materials_list.lua")

function collision_trigger(colliding_entity_id)

	local projectile_id = GetUpdatedEntityID()
	
	local shooter_id = getPlayerEntity()
	
	local x, y = EntityGetTransform(colliding_entity_id)
	SetRandomSeed( GameGetFrameNum() + projectile_id, x + y + colliding_entity_id)
	
	-- internal variables used to handle enemy creatures and projectiles 
	local projectile_definition_path = getInternalVariableValue(projectile_id, "projectile_to_shoot", "value_string")
	local deflect_explosion_definition = getInternalVariableValue(projectile_id, "deflect_explosion_definition", "value_string")
	local projectile_explosion_to_load = getInternalVariableValue(projectile_id, "projectile_explosion_to_load", "value_string")
	
	if(not EntityHasTag(colliding_entity_id, "projectile") and not EntityHasTag(colliding_entity_id, "enemy")) then return end -- making sure entity that triggered this was a projectile or enemy
	
	-- enemy projectiles handling
	if(EntityHasTag(colliding_entity_id, "projectile")) then
		
		-- conditions to prevent friendly projectiles from being redirected
		if(entityHasSimplifiedInternalVariable(colliding_entity_id, "relic_effect_projectile")) then return end
		if(string.find(EntityGetName(colliding_entity_id), "rotating_shield_projectile")) then return end
		if(entityHasSimplifiedInternalVariable(colliding_entity_id, "deflected")) then return end
		
		if(getProjectileShooter(colliding_entity_id) ~= shooter_id) then
			
			-- Loads audiovisual feedback
			GamePlaySound( "data/audio/Desktop/projectiles.snd", "player_projectiles/shield/block", x, y )
			EntityLoad(deflect_explosion_definition, x, y)
			
			-- obtains settings from colliding projectile
			local projectile_vel_x, projectile_vel_y = getProjectileVelocitiesandGravity(colliding_entity_id)
			local new_default_vel_x = projectile_vel_x * -1
			local new_default_vel_y = projectile_vel_y * -1
			
			-- kills colliding projectile
			projectileToggleExplosionOnDeath(colliding_entity_id, false)
			EntityKill(colliding_entity_id)
			
			local root_projectiles_amount = Random(8,10)
			local vel_variation_threshold = 150
			
			-- loads random amount of projectiles similar to the one that triggered this callback
			for i=1,root_projectiles_amount do
				local new_projectile_id = EntityLoad(projectile_definition_path, x, y)
				
				-- Inverts projectile velocity, turning it back to the direction it came
				setProjectileVelocitiesandGravity(new_projectile_id, new_default_vel_x + Random(-vel_variation_threshold, vel_variation_threshold), new_default_vel_y  + Random(-vel_variation_threshold, vel_variation_threshold))

				-- Sets the shooter of the deflected projectile as the creature under the effect
				setProjectileShooter(new_projectile_id, shooter_id)

				-- Flags projectile as deflected to prevent it from changing again
				addNewSimplifiedInternalVariable(new_projectile_id, "deflected")
			end
		end
	end
	
	-- enemy creatures handling
	if(EntityHasTag(colliding_entity_id, "enemy")) then
		SetRandomSeed( GameGetFrameNum() + projectile_id, x + y + colliding_entity_id)
		if(Random(1,4) == 1) then
			-- kills colliding creature
			killEntitywithCustomMessage(colliding_entity_id, "Converted into projectile barrage")
			doArtificialDeathLogic(colliding_entity_id, 1, false, getRelicPrecursorMaterialsList(), getPrecursorItemsBaseDir())
			
			-- Loads audiovisual feedback
			GamePlaySound( "data/audio/Desktop/projectiles.snd", "player_projectiles/shield/block", x, y )
			EntityLoad(deflect_explosion_definition, x, y)
			
			-- Loads projectile explosion
			EntityLoad(projectile_explosion_to_load, x, y)
		end
	end
end