dofile_once("mods/ArcaneReliquary/files/scripts/utilities/arcanereliquary_utilities.lua")
dofile_once("mods/ArcaneReliquary/files/scripts/lists/custom_effects_list.lua")

-- main function that calls all ability specific functions
function companionAbilities(companion_id, is_monk_hand)
	deflectingShield(companion_id, is_monk_hand)
	contactDamage(companion_id)
	electrocution(companion_id)
	freeze(companion_id)
	poly(companion_id)
	projectileGuide(companion_id)
	customEffect_exploding(companion_id)
	customEffect_bleeding(companion_id)
	customEffect_slash(companion_id)
	customEffect_slap(companion_id)
	customEffect_wind(companion_id)
	customEffect_time_bomb(companion_id)
    customEffect_attraction(companion_id)
end

-- deflecting shield ability
function deflectingShield(companion_id, is_monk_hand)

	-- shield appears only if the host entity has the required variable and the companion entity is not flagged to negate it
	if(entityHasSimplifiedInternalVariable(EntityGetParent(companion_id), "companion_shield") and not entityHasSimplifiedInternalVariable(companion_id, "no_energy_shield")) then
		
		local pos_x, pos_y = EntityGetTransform( companion_id )
		if(is_monk_hand) then pos_y = pos_y - 5 end -- heigh adjustment for monk hand companions
		local field_id = EntityLoad("mods/ArcaneReliquary/files/entity_definitions/others/deflecting_shield_companion.xml", pos_x, pos_y)
		local field_x, field_y = EntityGetTransform(field_id)
		local projectiles = EntityGetInRadiusWithTag(field_x, field_y, 8, "projectile")

		-- deflects projectiles
		for i, projectile_id in ipairs(projectiles) do
			for i, component in ipairs(EntityGetComponent(projectile_id, "ProjectileComponent") or {}) do
				if ComponentGetValue2(component, "mWhoShot") ~= EntityGetParent(companion_id) then -- projectiles are only deflected if not shot by the root entity related to companion
					
					-- check to exclude projectiles shot by relic effects
					if(entityHasSimplifiedInternalVariable(projectile_id, "relic_effect_projectile")) then return end
					
					local x, y = EntityGetTransform(projectile_id)

					-- First checks if the projectile has already been deflected, returns to the caller if so
					if(entityHasSimplifiedInternalVariable(projectile_id, "deflected")) then return end
					
					-- Then checks if the projectile contains the shield_bypass flag, which also excludes it from deflection mechanic
					if(entityHasSimplifiedInternalVariable(projectile_id, "shield_bypass")) then return end
					
					-- check to prevent rotating projectiles from being deflected
					if(string.find(EntityGetName(projectile_id), "rotating_shield_projectile")) then return end

					-- Loads audiovisual feedback
					GamePlaySound( "data/audio/Desktop/projectiles.snd", "player_projectiles/shield/block", x, y )
					EntityLoad("mods/ArcaneReliquary/files/entity_definitions/particle_explosions/shield_deflect_explosion_green.xml", x, y)

					local projectile_vel_x, projectile_vel_y = getProjectileVelocitiesandGravity(projectile_id)

					-- Inverts projectile velocity, turning it back to the direction it came
					setProjectileVelocitiesandGravity(projectile_id, (projectile_vel_x + 1.5) * -1, (projectile_vel_y + 1.5) * -1)

					-- Sets the shooter as the root entity related to companion
					setProjectileShooter(projectile_id, EntityGetParent(companion_id))

					-- Flags projectile as deflected to prevent it from changing again
					addNewSimplifiedInternalVariable(projectile_id, "deflected")
					
				end
			end
		end
	end
end

-- contact damage ability
function contactDamage(companion_id)

	-- ability active only if the host entity possesses the required variable and the companion entity is not flagged to negate it
	if(entityHasSimplifiedInternalVariable(EntityGetParent(companion_id), "companion_contact_damage") and not entityHasSimplifiedInternalVariable(companion_id, "no_contact_damage")) then
	
		local pos_x, pos_y = EntityGetTransform( companion_id )
		local creatures = EntityGetInRadiusWithTag(pos_x, pos_y, 7, "enemy")

		for i, creature_id in ipairs(creatures) do

			-- Damages enemies randomly
			SetRandomSeed( GameGetFrameNum(), math.random(1,50))
			if(Random(1,2) == 1) then
				-- Loads audiovisual feedback
				GamePlaySound( "data/audio/Desktop/animals.snd", "animals/damage/physics_hit", pos_x, pos_y )
				EntityLoad("mods/ArcaneReliquary/files/entity_definitions/particle_explosions/shield_damage_explosion_pink.xml", pos_x, pos_y)
				
				local damageThreshold = Randomf(0.02, 0.05)
				
				EntityInflictDamage( creature_id, damageThreshold, "DAMAGE_PHYSICS_HIT", "Player Companion Contact Damage", "NORMAL", 0, 0)
			end
		end
	end
end

-- electrocution ability
function electrocution(companion_id)

	-- ability active only if the host entity possesses the required variable
	if(entityHasSimplifiedInternalVariable(EntityGetParent(companion_id), "companion_contact_electrocution")) then
	
		local pos_x, pos_y = EntityGetTransform( companion_id )
		local creatures = EntityGetInRadiusWithTag(pos_x, pos_y, 7, "enemy")
		
		for i, creature_id in ipairs(creatures) do
			
			-- Applies effect on enemy
			SetRandomSeed( GameGetFrameNum(), math.random(1,50))
			if(Random(1,2) == 1) then
				-- Loads audiovisual feedback
				GamePlaySound( "data/audio/Desktop/projectiles.snd", "player_projectiles/shield/block", pos_x, pos_y )
				EntityLoad("mods/ArcaneReliquary/files/entity_definitions/particle_explosions/shield_damage_explosion_blue.xml", pos_x, pos_y)
				
				LoadGameEffectEntityTo( creature_id, "data/entities/misc/effect_electricity.xml" )
			end
		end
	end
end

-- freeze ability
function freeze(companion_id)

	-- ability active only if the host entity possesses the required variable
	if(entityHasSimplifiedInternalVariable(EntityGetParent(companion_id), "companion_contact_freeze")) then
	
		local pos_x, pos_y = EntityGetTransform( companion_id )
		local creatures = EntityGetInRadiusWithTag(pos_x, pos_y, 7, "enemy")
		
		for i, creature_id in ipairs(creatures) do
			
			-- Applies effect on enemy
			SetRandomSeed( GameGetFrameNum(), math.random(1,50))
			if(Random(1,2) == 1) then
				-- Loads audiovisual feedback
				GamePlaySound( "data/audio/Desktop/projectiles.snd", "player_projectiles/shield/block", pos_x, pos_y )
				EntityLoad("mods/ArcaneReliquary/files/entity_definitions/particle_explosions/shield_damage_explosion_blue.xml", pos_x, pos_y)
				
				LoadGameEffectEntityTo( creature_id, "data/entities/misc/effect_frozen.xml" )
			end
		end
	end
end

-- polymorph ability
function poly(companion_id)

	-- ability active only if the host entity possesses the required variable
	if(entityHasSimplifiedInternalVariable(EntityGetParent(companion_id), "companion_contact_poly")) then
	
		local pos_x, pos_y = EntityGetTransform( companion_id )
		local creatures = EntityGetInRadiusWithTag(pos_x, pos_y, 7, "enemy")
		
		for i, creature_id in ipairs(creatures) do
			
			-- Applies effect on enemy
			SetRandomSeed( GameGetFrameNum(), math.random(1,50))
			if(Random(1,2) == 1) then
				-- Loads audiovisual feedback
				GamePlaySound( "data/audio/Desktop/projectiles.snd", "player_projectiles/shield/block", pos_x, pos_y )
				EntityLoad("mods/ArcaneReliquary/files/entity_definitions/particle_explosions/shield_damage_explosion_pink.xml", pos_x, pos_y)
				
				LoadGameEffectEntityTo( creature_id, "data/entities/misc/effect_polymorph.xml" )
			end
		end
	end
end

-- projectile guide ability
function projectileGuide(companion_id)
	-- ability active only if the host entity possesses the required variable and the companion entity is flagged as such
	if(entityHasSimplifiedInternalVariable(EntityGetParent(companion_id), "companion_projectile_guide") and entityHasSimplifiedInternalVariable(companion_id, "can_projectile_guide")) then
		local pos_x, pos_y = EntityGetTransform( companion_id )
		local projectiles = EntityGetInRadiusWithTag(pos_x, pos_y, 7, "projectile")
		
		for i, projectile_id in ipairs(projectiles) do
			if(getProjectileShooter(projectile_id) == EntityGetParent(companion_id)) then
				
				-- ignores projectile if already redirected
				if(entityHasSimplifiedInternalVariable(projectile_id, "deflected")) then return end
				
				-- locate nearest target
				local target, target_x, target_y
				local min_dist = 9999
				for _,id in pairs(EntityGetInRadiusWithTag(pos_x, pos_y, 250, "enemy")) do
					local x, y = EntityGetFirstHitboxCenter( id )
					local dist = get_distance(pos_x, pos_y, x, y)
					if dist < min_dist then
						min_dist = dist
						target = id
						target_x = x
						target_y = y
					end
				end
				
				if not target then return end

				local target_x, target_y = EntityGetTransform(target)

				-- direction
				local vel_x, vel_y = vec_sub(target_x, target_y, pos_x, pos_y)
				vel_x,vel_y = vec_normalize(vel_x, vel_y)

				-- scatter
				local scatter = 0.1
				SetRandomSeed(pos_x + GameGetFrameNum(), pos_y)
				vel_x,vel_y = vec_rotate(vel_x,vel_y, rand(-scatter, scatter))

				-- apply velocity & shoot
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				vel_x,vel_y = vec_mult(vel_x,vel_y, Random(650,850))
				
				-- direct projectile towards target
				setProjectileVelocitiesandGravity(projectile_id, vel_x, vel_y)
				
				-- Loads audiovisual feedback
				local projectile_x, projectile_y = EntityGetTransform( projectile_id )
				GamePlaySound( "data/audio/Desktop/projectiles.snd", "player_projectiles/shield/block", projectile_x, projectile_y )
				EntityLoad("mods/ArcaneReliquary/files/entity_definitions/particle_explosions/shield_deflect_explosion_red.xml", projectile_x, projectile_y)
				
				-- Doubles base damage
				multiplyProjectileBaseDamage(projectile_id, 2)
				addBuffVisualIndicator_Projectile(projectile_id, "spark_red")
				
				-- Flags projectile as deflected to prevent it from changing again
			    addNewSimplifiedInternalVariable(projectile_id, "deflected")
					
			end
		end
	end
end

-- exploding ability
function customEffect_exploding(companion_id)

	-- ability active only if the host entity possesses the required variable
	if(entityHasSimplifiedInternalVariable(EntityGetParent(companion_id), "companion_contact_exploding")) then
	
		local pos_x, pos_y = EntityGetTransform( companion_id )
		local creatures = EntityGetInRadiusWithTag(pos_x, pos_y, 7, "enemy")
		
		for i, creature_id in ipairs(creatures) do
			
			-- Applies effect on enemy
			SetRandomSeed( GameGetFrameNum(), math.random(1,50))
			if(Random(1,2) == 1) then
				-- Loads audiovisual feedback
				GamePlaySound( "data/audio/Desktop/projectiles.snd", "player_projectiles/shield/block", pos_x, pos_y )
				EntityLoad("mods/ArcaneReliquary/files/entity_definitions/particle_explosions/shield_damage_explosion_orange.xml", pos_x, pos_y)
				
				LoadGameEffectEntityTo( creature_id, getCustomEffect("effect_exploding").path )
			end
		end
	end
end

-- bleeding ability
function customEffect_bleeding(companion_id)

	-- ability active only if the host entity possesses the required variable
	if(entityHasSimplifiedInternalVariable(EntityGetParent(companion_id), "companion_contact_bleeding")) then
	
		local pos_x, pos_y = EntityGetTransform( companion_id )
		local creatures = EntityGetInRadiusWithTag(pos_x, pos_y, 7, "enemy")
		
		for i, creature_id in ipairs(creatures) do
			
			-- Applies effect on enemy
			SetRandomSeed( GameGetFrameNum(), math.random(1,50))
			if(Random(1,2) == 1) then
				-- Loads audiovisual feedback
				GamePlaySound( "data/audio/Desktop/projectiles.snd", "player_projectiles/shield/block", pos_x, pos_y )
				EntityLoad("mods/ArcaneReliquary/files/entity_definitions/particle_explosions/shield_damage_explosion_red.xml", pos_x, pos_y)
				
				LoadGameEffectEntityTo( creature_id, getCustomEffect("effect_bleeding").path )
			end
		end
	end
end

-- slash ability
function customEffect_slash(companion_id)

	-- ability active only if the host entity possesses the required variable
	if(entityHasSimplifiedInternalVariable(EntityGetParent(companion_id), "companion_contact_slash")) then
	
		local pos_x, pos_y = EntityGetTransform( companion_id )
		local creatures = EntityGetInRadiusWithTag(pos_x, pos_y, 7, "enemy")
		
		for i, creature_id in ipairs(creatures) do
			
			-- Applies effect on enemy
			SetRandomSeed( GameGetFrameNum(), math.random(1,50))
			if(Random(1,2) == 1) then
				-- Loads audiovisual feedback
				GamePlaySound( "data/audio/Desktop/projectiles.snd", "player_projectiles/shield/block", pos_x, pos_y )
				EntityLoad("mods/ArcaneReliquary/files/entity_definitions/particle_explosions/shield_damage_explosion_purple.xml", pos_x, pos_y)
				
				LoadGameEffectEntityTo( creature_id, getCustomEffect("effect_slash").path )
			end
		end
	end
end

-- slap ability
function customEffect_slap(companion_id)

	-- ability active only if the host entity possesses the required variable
	if(entityHasSimplifiedInternalVariable(EntityGetParent(companion_id), "companion_contact_slap")) then
	
		local pos_x, pos_y = EntityGetTransform( companion_id )
		local creatures = EntityGetInRadiusWithTag(pos_x, pos_y, 7, "enemy")
		
		for i, creature_id in ipairs(creatures) do
			
			-- Applies effect on enemy
			SetRandomSeed( GameGetFrameNum(), math.random(1,50))
			if(Random(1,2) == 1) then
				-- Loads audiovisual feedback
				GamePlaySound( "data/audio/Desktop/projectiles.snd", "player_projectiles/shield/block", pos_x, pos_y )
				EntityLoad("mods/ArcaneReliquary/files/entity_definitions/particle_explosions/shield_damage_explosion_green.xml", pos_x, pos_y)
				
				LoadGameEffectEntityTo( creature_id, getCustomEffect("effect_slap").path )
			end
		end
	end
end

-- wind ability
function customEffect_wind(companion_id)

	-- ability active only if the host entity possesses the required variable
	if(entityHasSimplifiedInternalVariable(EntityGetParent(companion_id), "companion_contact_wind")) then
	
		local pos_x, pos_y = EntityGetTransform( companion_id )
		local creatures = EntityGetInRadiusWithTag(pos_x, pos_y, 7, "enemy")
		
		for i, creature_id in ipairs(creatures) do
			
			-- Applies effect on enemy
			SetRandomSeed( GameGetFrameNum(), math.random(1,50))
			if(Random(1,2) == 1) then
				-- Loads audiovisual feedback
				GamePlaySound( "data/audio/Desktop/projectiles.snd", "player_projectiles/shield/block", pos_x, pos_y )
				EntityLoad("mods/ArcaneReliquary/files/entity_definitions/particle_explosions/shield_damage_explosion_white.xml", pos_x, pos_y)
				
				LoadGameEffectEntityTo( creature_id, getCustomEffect("effect_wind_heavy").path )
			end
		end
	end
end

-- time bomb ability
function customEffect_time_bomb(companion_id)

	-- ability active only if the host entity possesses the required variable
	if(entityHasSimplifiedInternalVariable(EntityGetParent(companion_id), "companion_contact_time_bomb")) then
	
		local pos_x, pos_y = EntityGetTransform( companion_id )
		local creatures = EntityGetInRadiusWithTag(pos_x, pos_y, 7, "enemy")
		
		for i, creature_id in ipairs(creatures) do
			
			-- Applies effect on enemy
			SetRandomSeed( GameGetFrameNum(), math.random(1,50))
			if(Random(1,2) == 1) then
				-- Loads audiovisual feedback
				GamePlaySound( "data/audio/Desktop/projectiles.snd", "player_projectiles/shield/block", pos_x, pos_y )
				EntityLoad("mods/ArcaneReliquary/files/entity_definitions/particle_explosions/shield_damage_explosion_yellow.xml", pos_x, pos_y)
				
				LoadGameEffectEntityTo( creature_id, getCustomEffect("effect_time_bomb").path )
			end
		end
	end
end

-- attraction ability
function customEffect_attraction(companion_id)

	-- ability active only if the host entity possesses the required variable
	if(entityHasSimplifiedInternalVariable(EntityGetParent(companion_id), "companion_contact_attraction")) then
	
		local pos_x, pos_y = EntityGetTransform( companion_id )
		local creatures = EntityGetInRadiusWithTag(pos_x, pos_y, 7, "enemy")
		
		for i, creature_id in ipairs(creatures) do
			
			-- Applies effect on enemy
			SetRandomSeed( GameGetFrameNum(), math.random(1,50))
			if(Random(1,2) == 1) then
				-- Loads audiovisual feedback
				GamePlaySound( "data/audio/Desktop/projectiles.snd", "player_projectiles/shield/block", pos_x, pos_y )
				EntityLoad("mods/ArcaneReliquary/files/entity_definitions/particle_explosions/shield_damage_explosion_red.xml", pos_x, pos_y)
				
				LoadGameEffectEntityTo( creature_id, getCustomEffect("effect_attraction").path )
			end
		end
	end
end