dofile_once("data/scripts/lib/utilities.lua")
dofile_once("mods/ArcaneReliquary/files/scripts/utilities/arcanereliquary_utilities.lua")

local scatter = 0.3

local entity_id = GetUpdatedEntityID()
local root_id = EntityGetRootEntity(entity_id)
local pos_x, pos_y = EntityGetTransform( entity_id )

local projectile_definition = getInternalVariableValue(entity_id, "projectile_definition", "value_string")
local projectile_amount_min = getInternalVariableValue(entity_id, "projectile_amount_min", "value_int")
local projectile_amount_max = getInternalVariableValue(entity_id, "projectile_amount_max", "value_int")
local detection_range = getInternalVariableValue(entity_id, "detection_range", "value_int")
local projectile_velocity_min = getInternalVariableValue(entity_id, "vel_min", "value_int")
local projectile_velocity_max = getInternalVariableValue(entity_id, "vel_max", "value_int")
local target_tag = getInternalVariableValue(entity_id, "target_tag", "value_string")

-- locate nearest enemy
local enemy, enemy_x, enemy_y
local min_dist = 9999
for _,id in pairs(EntityGetInRadiusWithTag(pos_x, pos_y, detection_range, target_tag)) do
	local valid_target = true
	if(target_tag == "mortal") then
		valid_target = (EntityGetComponent(id, "GenomeDataComponent") ~= nil) and (EntityGetHerdRelation(root_id, id) < 40) and (IsPlayer(root_id) ~= IsPlayer(id))
	end
	-- is target a valid enemy
	if valid_target then
		local x, y = EntityGetFirstHitboxCenter( id )
		local dist = get_distance(pos_x, pos_y, x, y)
		if dist < min_dist then
			min_dist = dist
			enemy = id
			enemy_x = x
			enemy_y = y
		end
	end
end

if not enemy then return end

-- check los
if RaytraceSurfaces(pos_x, pos_y, enemy_x, enemy_y) then return end

-- enemies that are above companion are ignored
if(enemy_y < pos_y) then return end

local valid_horizontal_range = 20
local left_range = pos_x - valid_horizontal_range
local right_range = pos_x + valid_horizontal_range

-- enemies that are not within horizontal range are ignored
if(enemy_x < left_range or enemy_x > right_range) then return end

-- direction
local vel_x, vel_y = vec_sub(enemy_x, enemy_y, pos_x, pos_y)
vel_x,vel_y = vec_normalize(vel_x, vel_y)

SetRandomSeed(pos_x + GameGetFrameNum(), pos_y)
for i=1,Random(projectile_amount_min,projectile_amount_max) do
	-- scatter
	local vx,vy = vec_rotate(vel_x,vel_y, rand(-scatter, scatter))

	-- apply velocity & shoot
	vx,vy = vec_mult(vx,vy, rand(projectile_velocity_min, projectile_velocity_max))
	local projectile_id = shoot_projectile( entity_id, projectile_definition, pos_x, pos_y, vx, vy)
	setProjectileShooter(projectile_id, root_id) -- hacky way to prevent projectiles from hitting the player but also to prevent them from being affected by relics
end