--dofile_once("mods/ArcaneReliquary/files/scripts/utilities/arcanereliquary_utilities.lua")
dofile_once("mods/ArcaneReliquary/files/scripts/companions/companion_abilities.lua")
dofile_once("mods/ArcaneReliquary/files/scripts/lists/relics_list.lua")
dofile_once("data/scripts/lib/utilities.lua")

local entity_id = GetUpdatedEntityID()

-- config variables
local fly_height = getInternalVariableValue(entity_id, "fly_height", "value_int")
local vertical_oscilate_factor = getInternalVariableValue(entity_id, "vertical_oscilate_factor", "value_int")
local horizontal_fly_range = getInternalVariableValue(entity_id, "horizontal_fly_range", "value_int")
local is_mouse_controlled = getInternalVariableValue(entity_id, "is_mouse_controlled", "value_bool")

local lerp_amount = 0.975
local bob_h = vertical_oscilate_factor
local bob_w = horizontal_fly_range
local bob_speed_y = 0.065
local bob_speed_x = 0.01421

local pos_x, pos_y = EntityGetTransform( entity_id )

if pos_x == 0 and pos_y == 0 then
	-- get position from wand when starting
	pos_x, pos_y = EntityGetTransform(EntityGetParent(entity_id))
end

-- ghost continously lerps towards a target that floats around the parent

local target_x, target_y
if(is_mouse_controlled) then
	target_x, target_y = DEBUG_GetMouseWorld()
else
	target_x, target_y = EntityGetTransform(EntityGetParent(entity_id))
end

if target_x == nil then return end
target_y = target_y - fly_height

local time = GameGetFrameNum()
local r = ProceduralRandomf(entity_id, 0, -1, 1)

-- randomize times and speeds slightly so that multiple ghosts don't fly identically
time = time + r * 10000
bob_speed_y = bob_speed_y + (r * bob_speed_y * 0.1)
bob_speed_x = bob_speed_x + (r * bob_speed_x * 0.1)
lerp_amount = lerp_amount - (r * lerp_amount * 0.01)

-- bob
target_y = target_y + math.sin(time * bob_speed_y) * bob_h
target_x = target_x + math.sin(time * bob_speed_x) * bob_w

local dist_x = pos_x - target_x

-- move towards target
pos_x,pos_y = vec_lerp(pos_x, pos_y, target_x, target_y, lerp_amount)

EntitySetTransform( entity_id, pos_x, pos_y, 0, 1, 1)


-- plant grow config
local has_seed = getInternalVariableValue(entity_id, "has_seed", "value_bool")
if(has_seed) then
	
	local grow_material_name = getInternalVariableValue(entity_id, "grow_material_name", "value_string")
	local grow_material_amount = getMaterialAmountOnEntityInventory(entity_id, grow_material_name)
	local grow_material_amount_needed = 500
	
	--GamePrint(">>> amount of " .. grow_material_name .. " in pot: " .. grow_material_amount)

	-- check if enough material to advance to next stage
	local sc_id = EntityGetFirstComponentIncludingDisabled(entity_id, "SpriteComponent")
	local current_stage = ComponentGetValue( sc_id, "rect_animation")
	if(grow_material_amount > grow_material_amount_needed) then
		if(current_stage == "stage1") then -- if stage1, advances to stage2
			-- deducts material amount needed, then advance to next stage
			AddMaterialInventoryMaterial( entity_id, grow_material_name, grow_material_amount - grow_material_amount_needed)
			ComponentSetValue( sc_id, "rect_animation", "stage2")
		elseif(current_stage == "stage2") then -- if stage2, advances to stage3
			-- deducts material amount needed, then advance to next stage
			AddMaterialInventoryMaterial( entity_id, grow_material_name, grow_material_amount - grow_material_amount_needed)
			ComponentSetValue( sc_id, "rect_animation", "stage3")
		elseif(current_stage == "stage3") then -- final stage, resets pot and dispenses reward
			-- deducts material amount needed, resets to stage0
			AddMaterialInventoryMaterial( entity_id, grow_material_name, grow_material_amount - grow_material_amount_needed)
			ComponentSetValue( sc_id, "rect_animation", "stage0")
			-- enables collision trigger to allow it to collect more seeds
			toggleCollisionTrigger(entity_id, "seed_get_collision_trigger", true)
			-- disables sucker
			toggleMaterialSucker(entity_id, "material_suck", false)
			-- dispense reward
			local host_x, host_y = EntityGetTransform(EntityGetParent(entity_id))
			dispenseReward(getInternalVariableValue(entity_id, "reward_type", "value_string"), host_x, host_y)
			-- resets variables
			setInternalVariableValue(entity_id, "has_seed", "value_bool", false)
			setInternalVariableValue(entity_id, "reward_type", "value_string", "")
			setInternalVariableValue(entity_id, "grow_material_name", "value_string", "")
		end
	end
end

-- companion abilities that run along movement
companionAbilities(entity_id, false)

function dispenseReward(reward_type, x, y)

	--GamePrint(">>> reward type: " .. reward_type)
	
	local y_offset = 15
	
	if(reward_type == "spell_card") then -- spell card reward
		CreateItemActionEntity( getRandomActionId(), x, y - y_offset)
		
	elseif(reward_type == "perk_card") then -- perk card reward
		spawnRandomPerkCard(x, y - y_offset)
		
	elseif(reward_type == "random_wand") then -- random wand reward
		local prize_pool_items = {
			"data/entities/items/wand_level_01.xml",
			"data/entities/items/wand_level_01_better.xml",
			"data/entities/items/wand_level_01_p.xml",
			"data/entities/items/wand_level_02.xml",
			"data/entities/items/wand_level_02_better.xml",
			"data/entities/items/wand_level_02_p.xml",
			"data/entities/items/wand_level_03.xml",
			"data/entities/items/wand_level_03_better.xml",
			"data/entities/items/wand_level_03_p.xml",
			"data/entities/items/wand_level_04.xml",
			"data/entities/items/wand_level_04_better.xml",
			"data/entities/items/wand_level_04_p.xml",
			"data/entities/items/wand_level_05.xml",
			"data/entities/items/wand_level_05_better.xml",
			"data/entities/items/wand_level_05_p.xml",
			"data/entities/items/wand_level_06.xml",
			"data/entities/items/wand_level_06_better.xml",
			"data/entities/items/wand_level_06_p.xml",
			"data/entities/items/wand_level_10.xml",
		}
		EntityLoad(prize_pool_items[Random(1,#prize_pool_items)], x, y - y_offset)
		
	elseif(reward_type == "random_vanilla_item") then -- random vanilla item reward
		local prize_pool_items = {
			"data/entities/items/pickup/spell_refresh.xml",
			"data/entities/items/pickup/heart_fullhp.xml",
			"data/entities/items/pickup/beamstone.xml",
			"data/entities/items/pickup/broken_wand.xml",
			"data/entities/items/pickup/moon.xml",
			"data/entities/items/pickup/potion_random_material.xml",
			"data/entities/items/pickup/jar_of_urine.xml",
			"data/entities/items/pickup/evil_eye.xml",
			"data/entities/items/pickup/brimstone.xml",
			"data/entities/items/pickup/thunderstone.xml",
			"data/entities/items/pickup/powder_stash.xml",
			"data/entities/items/pickup/goldnugget_1000.xml",
		}
		EntityLoad(prize_pool_items[Random(1,#prize_pool_items)], x, y - y_offset)
		
	elseif(reward_type == "leveled_relic") then -- leveled relic reward
		local relic = getRandomUpgradableRelic()
		local relic_id = EntityLoad(getRelicsBaseDir() .. relic.name .. ".xml", x, y - y_offset)
		setRelicLevel(relic, relic_id, Random(1,getRelicMaxUpgradeLevel()))

		
	elseif(reward_type == "random_chest") then -- random vanilla chest or relic chest reward
		local prize_pool_items = {
			"data/entities/items/pickup/chest_random.xml",
			"data/entities/items/pickup/chest_random_super.xml",
			"mods/ArcaneReliquary/files/entity_definitions/props/relic_chest.xml",
			"mods/ArcaneReliquary/files/entity_definitions/props/relic_chest_great.xml",
			"mods/ArcaneReliquary/files/entity_definitions/props/relic_chest_chaotic.xml"
		}
		EntityLoad(prize_pool_items[Random(1,#prize_pool_items)], x, y - y_offset)
		
	end
	
	-- Loads audiovisual feedback
	EntityLoad("mods/ArcaneReliquary/files/entity_definitions/particle_explosions/enchanted_pot_dispense_reward_explosion.xml", x, y - y_offset)
end

