dofile_once("data/scripts/lib/utilities.lua")
dofile_once("mods/ArcaneReliquary/files/scripts/utilities/arcanereliquary_utilities.lua")

local scatter = 0.2

local entity_id = GetUpdatedEntityID()
local root_id = EntityGetRootEntity(entity_id)
local pos_x, pos_y = EntityGetFirstHitboxCenter( entity_id )

local projectile_definition = getInternalVariableValue(entity_id, "projectile_definition", "value_string")
local ignore_line_of_sight = getInternalVariableValue(entity_id, "ignore_line_of_sight", "value_bool")
local detection_range = getInternalVariableValue(entity_id, "detection_range", "value_int")
local projectile_velocity_min = getInternalVariableValue(entity_id, "vel_min", "value_int")
local projectile_velocity_max = getInternalVariableValue(entity_id, "vel_max", "value_int")
local kill_hand_on_shot = getInternalVariableValue(entity_id, "kill_hand_on_shot", "value_bool")
local target_tag = getInternalVariableValue(entity_id, "target_tag", "value_string")

-- locate nearest enemy
local enemy, enemy_x, enemy_y
local min_dist = 9999
for _,id in pairs(EntityGetInRadiusWithTag(pos_x, pos_y, detection_range, target_tag)) do
	local valid_target = true
	if(target_tag == "mortal") then
		valid_target = EntityGetComponent(id, "GenomeDataComponent") ~= nil and EntityGetHerdRelation(root_id, id) < 40 and IsPlayer(root_id) ~= IsPlayer(id)
	end
	-- is target a valid enemy
	if valid_target then
		local x, y = EntityGetFirstHitboxCenter( id )
		local dist = get_distance(pos_x, pos_y, x, y)
		if dist < min_dist then
			min_dist = dist
			enemy = id
			enemy_x = x
			enemy_y = y
		end
	end
end

-- check los
local can_shoot = false
if(ignore_line_of_sight) then
	can_shoot = true
else
	if enemy then can_shoot = not RaytraceSurfaces(pos_x, pos_y, enemy_x, enemy_y) end
end

-- hand/shooting state & animation control
edit_component2( entity_id, "SpriteComponent", function(comp,vars)
	-- if enemy is not visible then open hand
	local hand_check = ComponentGetValue2( comp, "rect_animation" )
	local hand_open = false
	
	if ( hand_check ~= nil ) and ( hand_check == "open" ) then
		hand_open = true
	end
	
	if not can_shoot then
		if not hand_open then
			ComponentSetValue2( comp, "rect_animation", "open")
		end
		return
	end

	-- prepare to shoot
	if hand_open then
		ComponentSetValue2( comp, "rect_animation", "close")
		can_shoot = false
		return
	end
end)

if not can_shoot then return end

-- direction
local vel_x, vel_y = vec_sub(enemy_x, enemy_y, pos_x, pos_y)
vel_x,vel_y = vec_normalize(vel_x, vel_y)

-- offset to shoot direction to avoid colliding with hand
pos_x = pos_x + vel_x * 8
pos_y = pos_y + vel_y * 8

-- scatter
SetRandomSeed(pos_x + GameGetFrameNum(), pos_y)
vel_x,vel_y = vec_rotate(vel_x,vel_y, rand(-scatter, scatter))

-- apply velocity & shoot
vel_x,vel_y = vec_mult(vel_x,vel_y, rand(projectile_velocity_min, projectile_velocity_max))
local projectile_id = shoot_projectile( entity_id, projectile_definition, pos_x, pos_y, vel_x, vel_y)
setProjectileShooter(projectile_id, root_id) -- hacky way to prevent projectiles from hitting the player but also to prevent them from being affected by relics

-- delay randomly so that multiple hands shoot at different times
ComponentSetValue(GetUpdatedComponentID(), "execute_every_n_frame", 15 + Random(10))

-- if kill_hand_on_shot, kills hand that just shot
if(kill_hand_on_shot) then
	EntityKill(entity_id)
end

