--dofile_once("mods/ArcaneReliquary/files/scripts/utilities/arcanereliquary_utilities.lua")
dofile_once("mods/ArcaneReliquary/files/scripts/companions/companion_abilities.lua")
dofile_once("data/scripts/lib/utilities.lua")

local entity_id = GetUpdatedEntityID()

-- config variables
local fly_height = getInternalVariableValue(entity_id, "fly_height", "value_int")
local vertical_oscilate_factor = getInternalVariableValue(entity_id, "vertical_oscilate_factor", "value_int")
local horizontal_fly_range = getInternalVariableValue(entity_id, "horizontal_fly_range", "value_int")
local is_mouse_controlled = getInternalVariableValue(entity_id, "is_mouse_controlled", "value_bool")

local lerp_amount = 0.975
local bob_h = vertical_oscilate_factor
local bob_w = horizontal_fly_range
local bob_speed_y = 0.065
local bob_speed_x = 0.01421

local pos_x, pos_y = EntityGetTransform( entity_id )

if pos_x == 0 and pos_y == 0 then
	-- get position from wand when starting
	pos_x, pos_y = EntityGetTransform(EntityGetParent(entity_id))
end

-- ghost continously lerps towards a target that floats around the parent

local target_x, target_y
if(is_mouse_controlled) then
	target_x, target_y = DEBUG_GetMouseWorld()
else
	target_x, target_y = EntityGetTransform(EntityGetParent(entity_id))
end

if target_x == nil then return end
target_y = target_y - fly_height

local time = GameGetFrameNum()
local r = ProceduralRandomf(entity_id, 0, -1, 1)

-- randomize times and speeds slightly so that multiple ghosts don't fly identically
time = time + r * 10000
bob_speed_y = bob_speed_y + (r * bob_speed_y * 0.1)
bob_speed_x = bob_speed_x + (r * bob_speed_x * 0.1)
lerp_amount = lerp_amount - (r * lerp_amount * 0.01)

-- bob
target_y = target_y + math.sin(time * bob_speed_y) * bob_h
target_x = target_x + math.sin(time * bob_speed_x) * bob_w

local dist_x = pos_x - target_x

-- move towards target
pos_x,pos_y = vec_lerp(pos_x, pos_y, target_x, target_y, lerp_amount)

local player_is_riding = getInternalVariableValue(entity_id, "player_is_riding", "value_bool")
if (player_is_riding and RaytraceSurfaces(pos_x, pos_y, target_x, target_y)) then return end

EntitySetTransform( entity_id, pos_x, pos_y, 0, 1, 1)

-- animation state
edit_component( entity_id, "SpriteComponent", function(comp,vars)
	local current_anim = ComponentGetValue( comp, "rect_animation")
	
	-- float when nearby and fly when further away
	local mode = "float_"
	if math.abs(dist_x) > 28 then mode = "fly_" end
	
	-- check if changing the animation is needed based on current animation and heading
	if dist_x < 2 and current_anim ~= mode.."right" then
		ComponentSetValue( comp, "rect_animation", mode.."right")
	elseif dist_x > 2 and current_anim ~= mode.."left" then
		ComponentSetValue( comp, "rect_animation", mode.."left")
	end
end)

-- companion abilities that run along movement
companionAbilities(entity_id, false)

