dofile_once("data/scripts/lib/utilities.lua")
dofile_once("mods/ArcaneReliquary/files/scripts/utilities/arcanereliquary_utilities.lua")

local entity_id    = GetUpdatedEntityID()
local attract_projectiles = getInternalVariableValue(entity_id, "attract_projectiles", "value_bool")
local attract_objects = getInternalVariableValue(entity_id, "attract_objects", "value_bool")
local attraction_range = getInternalVariableValue(entity_id, "attraction_range", "value_int")
local attracted_projectiles_shooter_exclude = getInternalVariableValue(entity_id, "attracted_projectiles_shooter_exclude", "value_bool")

entity_id = EntityGetRootEntity( entity_id )
local x, y, rot = EntityGetTransform( entity_id )


function calculate_force_at(body_x, body_y)
	local distance = math.sqrt( ( x - body_x ) ^ 2 + ( y - body_y ) ^ 2 )
	local direction = 0 - math.atan2( ( y - body_y ), ( x - body_x ) )

	local gravity_percent = ( attraction_range - distance ) / attraction_range
	local gravity_coeff = 196
	
	local fx = math.cos( direction ) * ( gravity_coeff * gravity_percent )
	local fy = -math.sin( direction ) * ( gravity_coeff * gravity_percent )

    return fx,fy
end

if(attract_projectiles) then
	-- attract projectiles
	local entities = EntityGetInRadiusWithTag(x, y, attraction_range, "projectile")
	for _,id in ipairs(entities) do
		
		-- if effect is set to exclude projectiles shot by the creature under this effect, checks it first
		local can_be_attracted = false
		if(attracted_projectiles_shooter_exclude) then
			if(getProjectileShooter(id) ~= entity_id) then
				can_be_attracted = true
			end
		else
			can_be_attracted = true
		end
		
		if(can_be_attracted) then
			local physicscomp = EntityGetFirstComponent(id, "PhysicsBody2Component") or EntityGetFirstComponent( id, "PhysicsBodyComponent")
			if physicscomp == nil and not EntityHasTag(id,"black_hole") then
				local px, py = EntityGetTransform( id )

				local velocitycomp = EntityGetFirstComponent( id, "VelocityComponent" )
				if ( velocitycomp ~= nil ) then
					local fx, fy = calculate_force_at(px, py)
					edit_component( id, "VelocityComponent", function(comp,vars)
						local vel_x,vel_y = ComponentGetValue2( comp, "mVelocity")
						
						vel_x = vel_x + fx
						vel_y = vel_y + fy

						ComponentSetValue2( comp, "mVelocity", vel_x, vel_y)
					end)
				end
			end
		end
	end
end

if(attract_objects) then
	-- force field for physics bodies
	function calculate_force_for_body( entity, body_mass, body_x, body_y, body_vel_x, body_vel_y, body_vel_angular )
		local fx, fy = calculate_force_at(body_x, body_y)

		fx = fx * 0.2 * body_mass * 2
		fy = fy * 0.2 * body_mass * 2

		return body_x,body_y,fx,fy,0 -- forcePosX,forcePosY,forceX,forceY,forceAngular
	end
	local size = attraction_range * 0.5
	PhysicsApplyForceOnArea( calculate_force_for_body, entity_id, x-size, y-size, x+size, y+size )
end

