dofile_once("data/scripts/lib/utilities.lua")
dofile_once("mods/ArcaneReliquary/files/scripts/utilities/arcanereliquary_utilities.lua")
dofile_once("mods/ArcaneReliquary/files/scripts/lists/load_on_explosion_list.lua")

local entity_id = GetUpdatedEntityID()
entity_id = EntityGetRootEntity( entity_id )
local x, y = EntityGetTransform(entity_id)
local field_id = EntityLoad("mods/ArcaneReliquary/files/entity_definitions/others/deflecting_thunder_red_shield.xml", x, y)

local target_tag = ""

if(creatureIs(entity_id) == "enemy") then
	target_tag = "player_unit"
else
	target_tag = "enemy"
end

local field_x, field_y = EntityGetTransform(field_id)

local projectiles = EntityGetInRadiusWithTag(field_x, field_y, 15, "projectile")

-- deflects projectiles
for i, projectile_id in ipairs(projectiles) do
	for i, component in ipairs(EntityGetComponent(projectile_id, "ProjectileComponent") or {}) do
		if ComponentGetValue2(component, "mWhoShot") ~= entity_id then -- projectiles are only deflected if not shot by the entity under the effect, this check is necessary otherwise the field itself will be caught within its own field
			
			-- check to exclude projectiles shot by relic effects
			if((EntityHasTag(entity_id, "player_unit")) and entityHasSimplifiedInternalVariable(projectile_id, "relic_effect_projectile")) then
				return
			end
			
			local x, y = EntityGetTransform(projectile_id)

			-- First checks if the projectile has already been deflected, returns to the caller if so
			if(entityHasSimplifiedInternalVariable(projectile_id, "deflected")) then
				return
			end
			
			-- Then checks if the projectile contains the shield_bypass flag, which also excludes it from deflection mechanic
			if(entityHasSimplifiedInternalVariable(projectile_id, "shield_bypass")) then
				return
			end
			
			-- check to prevent rotating projectiles from being deflected
			if(string.find(EntityGetName(projectile_id), "rotating_shield_projectile")) then return end

			-- Loads audiovisual feedback
			GamePlaySound( "data/audio/Desktop/projectiles.snd", "player_projectiles/shield/block", x, y )
			EntityLoad("mods/ArcaneReliquary/files/entity_definitions/particle_explosions/shield_deflect_explosion_red.xml", x, y)
			
			-- Adds thunderbolts
			addProjectileSpriteParticleTrail(projectile_id, "mods/ArcaneReliquary/files/other_gfx/spark_electric_red.xml")
			
			addNewInternalVariable(projectile_id, "projectile_definition", "value_string", "mods/ArcaneReliquary/files/entity_definitions/custom_projectiles/thunderbolt_red.xml")
			addNewInternalVariable(projectile_id, "detection_range", "value_int", 100)
			addNewInternalVariable(projectile_id, "projectile_velocity", "value_int", 950)
			addNewInternalVariable(projectile_id, "target_tag", "value_string", target_tag)
			addNewInternalVariable(projectile_id, "max_projectiles_shot", "value_int", 6)
			addNewLuaCallback(projectile_id, "overtime", "mods/ArcaneReliquary/files/scripts/projectiles/target_and_shoot_projectile.lua", 10, false, "target_and_shoot_projectile")

			local projectile_vel_x, projectile_vel_y = getProjectileVelocitiesandGravity(projectile_id)

			-- Inverts projectile velocity, turning it back to the direction it came
			comp_ids = EntityGetComponent( projectile_id, "VelocityComponent" )

			setProjectileVelocitiesandGravity(projectile_id, (projectile_vel_x + 1.5) * -1, (projectile_vel_y + 1.5) * -1)

			-- Sets the shooter of the deflected projectile as the creature under the effect
			setProjectileShooter(projectile_id, entity_id)

			-- Flags projectile as deflected to prevent it from changing again
			addNewSimplifiedInternalVariable(projectile_id, "deflected")
			
			-- if it's an enemy who's under this effect, deals small amount of damage each time a projectile is deflected
			if(EntityHasTag(entity_id, "enemy")) then
				SetRandomSeed( GameGetFrameNum(), x + y + entity_id )
				local projectile_base_damage = getProjectileBaseDamage(projectile_id)
				EntityInflictDamage( entity_id, projectile_base_damage / Random(2, 3), "DAMAGE_PHYSICS_HIT", "shield deflect damage", "NORMAL", 0, 0)
			end
			
		end
	end
end

local creatures = EntityGetInRadiusWithTag(field_x, field_y, 15, target_tag)

-- electrocutes enemies within field
for i, creature_id in ipairs(creatures) do
	if(creature_id ~= entity_id) then -- only applies effect on creatures other than the entity under this effect		
		SetRandomSeed( GameGetFrameNum(), x + y + entity_id )
		if(Random(1,3) == 1) then -- 30% chance of being electrocuted
			-- Loads audiovisual feedback
			GamePlaySound( "data/audio/Desktop/projectiles.snd", "player_projectiles/shield/block", x, y )
			EntityLoad("mods/ArcaneReliquary/files/entity_definitions/particle_explosions/shield_damage_explosion_red.xml", x, y)
			
			GetGameEffectLoadTo( creature_id, "ELECTROCUTION", true )
		end	
	end	
end