dofile_once("data/scripts/lib/utilities.lua")
dofile_once("mods/ArcaneReliquary/files/scripts/utilities/arcanereliquary_utilities.lua")
dofile_once("mods/ArcaneReliquary/files/scripts/lists/custom_effects_list.lua")

local effect_entity_id = GetUpdatedEntityID()

-- obtains radius settings
local current_radius = getInternalVariableValue(effect_entity_id, "current_radius", "value_int")
local max_radius = getInternalVariableValue(effect_entity_id, "max_radius", "value_int")
local min_radius = getInternalVariableValue(effect_entity_id, "min_radius", "value_int")
local pulsate_status = getInternalVariableValue(effect_entity_id, "pulsate_status", "value_string")

-- calculates current radius
if(pulsate_status == "increase") then
	current_radius = current_radius + 1
elseif(pulsate_status == "decrease") then
	current_radius = current_radius - 1
end

if(current_radius <= min_radius) then
	pulsate_status = "increase"
end
if(current_radius >= max_radius) then
	pulsate_status = "decrease"
end

setInternalVariableValue(effect_entity_id, "current_radius", "value_int", current_radius)
setInternalVariableValue(effect_entity_id, "pulsate_status", "value_string", pulsate_status)

--GamePrint(current_radius)

-- obtains root entity under the effect
entity_id = EntityGetRootEntity( effect_entity_id )
local x, y = EntityGetTransform(entity_id)

-- creates field and adjusts radius
local field_id = EntityLoad("mods/ArcaneReliquary/files/entity_definitions/others/pulsating_shield_decelerate.xml", x, y)

local components = EntityGetComponent( field_id, "ParticleEmitterComponent" )	
if ( components ~= nil ) then
	for key,comp_id in pairs(components) do 
		ComponentSetValue2(comp_id, "area_circle_radius", current_radius, current_radius)
	end
end

local gaec_id = EntityGetFirstComponentIncludingDisabled( field_id, "GameAreaEffectComponent" )
ComponentSetValue2(gaec_id, "radius", current_radius)

-- sets random seed for later use
SetRandomSeed( GameGetFrameNum(), x + y + entity_id)

-- detects projectiles within radius
local field_x, field_y = EntityGetTransform(field_id)
local projectiles = EntityGetInRadiusWithTag(field_x, field_y, current_radius, "projectile")

local target_tag = ""

if(creatureIs(entity_id) == "enemy") then
	target_tag = "player_unit"
else
	target_tag = "enemy"
end

local creatures = EntityGetInRadiusWithTag(field_x, field_y, current_radius, target_tag)

-- damages and knocks enemies back
for i, creature_id in ipairs(creatures) do
	if(creature_id ~= entity_id) then -- only knocks back creatures other than the entity under this effect

		-- Reduces movement speed of creature caught
		if(Random(1,2) == 1) then
			-- Loads audiovisual feedback
			GamePlaySound( "data/audio/Desktop/projectiles.snd", "player_projectiles/shield/block", x, y )
			EntityLoad("mods/ArcaneReliquary/files/entity_definitions/particle_explosions/shield_damage_explosion_green.xml", x, y)
		
			LoadGameEffectEntityTo( creature_id, getCustomEffect("effect_movement_speed_debuff").path )
		end
	end	
end