dofile_once("data/scripts/lib/utilities.lua")
dofile_once("mods/ArcaneReliquary/files/scripts/utilities/arcanereliquary_utilities.lua")

local effect_entity_id = GetUpdatedEntityID()

-- obtains radius settings
local current_radius = getInternalVariableValue(effect_entity_id, "current_radius", "value_int")
local max_radius = getInternalVariableValue(effect_entity_id, "max_radius", "value_int")
local min_radius = getInternalVariableValue(effect_entity_id, "min_radius", "value_int")
local pulsate_status = getInternalVariableValue(effect_entity_id, "pulsate_status", "value_string")
local effect_to_apply_entity_path = getInternalVariableValue(effect_entity_id, "effect_to_apply_entity_path", "value_string")
local deflect_explosion_entity_path = getInternalVariableValue(effect_entity_id, "deflect_explosion_entity_path", "value_string")
local deflect_trail_material_name = getInternalVariableValue(effect_entity_id, "deflect_trail_material_name", "value_string")
local shield_entity_path = getInternalVariableValue(effect_entity_id, "shield_entity_path", "value_string")

-- calculates current radius
if(pulsate_status == "increase") then
	current_radius = current_radius + 1
elseif(pulsate_status == "decrease") then
	current_radius = current_radius - 1
end

if(current_radius <= min_radius) then
	pulsate_status = "increase"
end
if(current_radius >= max_radius) then
	pulsate_status = "decrease"
end

setInternalVariableValue(effect_entity_id, "current_radius", "value_int", current_radius)
setInternalVariableValue(effect_entity_id, "pulsate_status", "value_string", pulsate_status)

--GamePrint(current_radius)

-- obtains root entity under the effect
entity_id = EntityGetRootEntity( effect_entity_id )
local x, y = EntityGetTransform(entity_id)

-- creates field and adjusts radius
local field_id = EntityLoad(shield_entity_path, x, y)

local components = EntityGetComponent( field_id, "ParticleEmitterComponent" )	
if ( components ~= nil ) then
	for key,comp_id in pairs(components) do 
		ComponentSetValue2(comp_id, "area_circle_radius", current_radius, current_radius)
	end
end

local gaec_id = EntityGetFirstComponentIncludingDisabled( field_id, "GameAreaEffectComponent" )
ComponentSetValue2(gaec_id, "radius", current_radius)

-- detects projectiles within radius
local field_x, field_y = EntityGetTransform(field_id)
local projectiles = EntityGetInRadiusWithTag(field_x, field_y, current_radius, "projectile")

-- deflects projectiles
for i, projectile_id in ipairs(projectiles) do
	for i, component in ipairs(EntityGetComponent(projectile_id, "ProjectileComponent") or {}) do
		if ComponentGetValue2(component, "mWhoShot") ~= entity_id then -- projectiles are only deflected if not shot by the entity under the effect, this check is necessary otherwise the field itself will be caught within its own field
			
			-- check to exclude projectiles shot by relic effects
			if((EntityHasTag(entity_id, "player_unit")) and entityHasSimplifiedInternalVariable(projectile_id, "relic_effect_projectile")) then
				return
			end
			
			local x, y = EntityGetTransform(projectile_id)

			-- First checks if the projectile has already been deflected, returns to the caller if so
			if(entityHasSimplifiedInternalVariable(projectile_id, "deflected")) then
				return
			end
			
			-- Then checks if the projectile contains the shield_bypass flag, which also excludes it from deflection mechanic
			if(entityHasSimplifiedInternalVariable(projectile_id, "shield_bypass")) then
				return
			end
			
			-- check to prevent rotating projectiles from being deflected
			if(string.find(EntityGetName(projectile_id), "rotating_shield_projectile")) then return end

			-- Loads audiovisual feedback
			GamePlaySound( "data/audio/Desktop/projectiles.snd", "player_projectiles/shield/block", x, y )
			EntityLoad(deflect_explosion_entity_path, x, y)
			
			-- Adds effect to projectile
			attachLoadEffectOntoEntitiestoProjectile(projectile_id, effect_to_apply_entity_path, deflect_trail_material_name)

			local projectile_vel_x, projectile_vel_y = getProjectileVelocitiesandGravity(projectile_id)

			-- Inverts projectile velocity, turning it back to the direction it came
			comp_ids = EntityGetComponent( projectile_id, "VelocityComponent" )

			setProjectileVelocitiesandGravity(projectile_id, (projectile_vel_x + 1.5) * -1, (projectile_vel_y + 1.5) * -1)

			-- Sets the shooter of the deflected projectile as the creature under the effect
			setProjectileShooter(projectile_id, entity_id)

			-- Flags projectile as deflected to prevent it from changing again
			addNewSimplifiedInternalVariable(projectile_id, "deflected")
			
			-- Loads audiovisual feedback
			GamePlaySound( "data/audio/Desktop/projectiles.snd", "player_projectiles/shield/block", x, y )

			
		end
	end
end

local target_tag = ""

if(creatureIs(entity_id) == "enemy") then
	target_tag = "player_unit"
else
	target_tag = "enemy"
end

local creatures = EntityGetInRadiusWithTag(field_x, field_y, current_radius, target_tag)

-- inflicts effect on creatures
for i, creature_id in ipairs(creatures) do
	if(creature_id ~= entity_id) then -- only knocks back creatures other than the entity under this effect

		-- Applies effect on creatures found
		if(Random(1,3) == 1) then
			-- Loads audiovisual feedback
			GamePlaySound( "data/audio/Desktop/projectiles.snd", "player_projectiles/shield/block", x, y )
			EntityLoad(deflect_explosion_entity_path, x, y)
		
			LoadGameEffectEntityTo( creature_id, effect_to_apply_entity_path )
		end
	end	
end