dofile_once("data/scripts/lib/utilities.lua")
dofile_once("mods/ArcaneReliquary/files/scripts/utilities/arcanereliquary_utilities.lua")

local effect_entity_id = GetUpdatedEntityID()

-- obtains radius settings
local radius = getInternalVariableValue(effect_entity_id, "radius", "value_int")
local effect_to_apply_entity_path = getInternalVariableValue(effect_entity_id, "effect_to_apply_entity_path", "value_string")
local deflect_explosion_entity_path = getInternalVariableValue(effect_entity_id, "deflect_explosion_entity_path", "value_string")
local deflect_trail_material_name = getInternalVariableValue(effect_entity_id, "deflect_trail_material_name", "value_string")
local shield_entity_path = getInternalVariableValue(effect_entity_id, "shield_entity_path", "value_string")

-- obtains root entity under the effect
entity_id = EntityGetRootEntity( effect_entity_id )
local x, y = EntityGetTransform(entity_id)

-- creates field and adjusts radius
local field_id = EntityLoad(shield_entity_path, x, y)

local components = EntityGetComponent( field_id, "ParticleEmitterComponent" )	
if ( components ~= nil ) then
	for key,comp_id in pairs(components) do 
		ComponentSetValue2(comp_id, "area_circle_radius", radius, radius)
	end
end

local gaec_id = EntityGetFirstComponentIncludingDisabled( field_id, "GameAreaEffectComponent" )
ComponentSetValue2(gaec_id, "radius", radius)

-- detects projectiles within radius
local field_x, field_y = EntityGetTransform(field_id)
local projectiles = EntityGetInRadiusWithTag(field_x, field_y, radius, "projectile")

-- deflects projectiles
for i, projectile_id in ipairs(projectiles) do
	for i, component in ipairs(EntityGetComponent(projectile_id, "ProjectileComponent") or {}) do
		if ComponentGetValue2(component, "mWhoShot") ~= entity_id then -- projectiles are only deflected if not shot by the entity under the effect, this check is necessary otherwise the field itself will be caught within its own field
			
			-- check to exclude projectiles shot by relic effects
			if((EntityHasTag(entity_id, "player_unit")) and entityHasSimplifiedInternalVariable(projectile_id, "relic_effect_projectile")) then
				return
			end
			
			local x, y = EntityGetTransform(projectile_id)

			-- First checks if the projectile has already been deflected, returns to the caller if so
			if(entityHasSimplifiedInternalVariable(projectile_id, "deflected")) then
				return
			end
			
			-- Then checks if the projectile contains the shield_bypass flag, which also excludes it from deflection mechanic
			if(entityHasSimplifiedInternalVariable(projectile_id, "shield_bypass")) then
				return
			end
			
			-- check to prevent rotating projectiles from being deflected
			if(string.find(EntityGetName(projectile_id), "rotating_shield_projectile")) then return end

			-- Loads audiovisual feedback
			GamePlaySound( "data/audio/Desktop/projectiles.snd", "player_projectiles/shield/block", x, y )
			EntityLoad(deflect_explosion_entity_path, x, y)
			
			-- obtains settings from colliding projectile
			local projectile_definition_path = getProjectileDefinitionPath(projectile_id)
			local projectile_vel_x, projectile_vel_y = getProjectileVelocitiesandGravity(projectile_id)
			local new_default_vel_x = projectile_vel_x * -1
			local new_default_vel_y = projectile_vel_y * -1
			
			-- kills colliding projectile
			projectileToggleExplosionOnDeath(projectile_id, false)
			EntityKill(projectile_id)
			
			local scattered_projectiles_amount = Random(6,8)
			local vel_variation_threshold = 200

			for i=1,scattered_projectiles_amount do
				local new_projectile_id = EntityLoad(projectile_definition_path, x, y)
				
				-- Adds effect to projectile
				attachLoadEffectOntoEntitiestoProjectile(new_projectile_id, effect_to_apply_entity_path, deflect_trail_material_name)
				
				-- Inverts projectile velocity, turning it back to the direction it came
				setProjectileVelocitiesandGravity(new_projectile_id, new_default_vel_x + Random(-vel_variation_threshold, vel_variation_threshold), new_default_vel_y  + Random(-vel_variation_threshold, vel_variation_threshold))

				-- Sets the shooter of the deflected projectile as the creature under the effect
				setProjectileShooter(new_projectile_id, shooter_id)

				-- Flags projectile as deflected to prevent it from changing again
				addNewSimplifiedInternalVariable(new_projectile_id, "deflected")
			end
		end
	end
end

local target_tag = ""

if(creatureIs(entity_id) == "enemy") then
	target_tag = "player_unit"
else
	target_tag = "enemy"
end

local creatures = EntityGetInRadiusWithTag(field_x, field_y, radius, target_tag)

-- inflicts effect on creatures
for i, creature_id in ipairs(creatures) do
	if(creature_id ~= entity_id) then -- only knocks back creatures other than the entity under this effect

		-- Applies effect on creatures found
		if(Random(1,3) == 1) then
			-- Loads audiovisual feedback
			GamePlaySound( "data/audio/Desktop/projectiles.snd", "player_projectiles/shield/block", x, y )
			EntityLoad(deflect_explosion_entity_path, x, y)
		
			LoadGameEffectEntityTo( creature_id, effect_to_apply_entity_path )
		end
	end	
end