dofile_once("data/scripts/lib/utilities.lua")
dofile_once("mods/ArcaneReliquary/files/scripts/utilities/arcanereliquary_utilities.lua")

local effect_entity_id = GetUpdatedEntityID()

-- obtains radius settings
local radius = getInternalVariableValue(effect_entity_id, "radius", "value_int")
local deflect_explosion_entity_path = getInternalVariableValue(effect_entity_id, "deflect_explosion_entity_path", "value_string")
local deflect_trail_material_name = getInternalVariableValue(effect_entity_id, "deflect_trail_material_name", "value_string")
local shield_entity_path = getInternalVariableValue(effect_entity_id, "shield_entity_path", "value_string")

-- obtains root entity under the effect
entity_id = EntityGetRootEntity( effect_entity_id )
local x, y = EntityGetTransform(entity_id)

-- creates field and adjusts radius
local field_id = EntityLoad(shield_entity_path, x, y)

local components = EntityGetComponent( field_id, "ParticleEmitterComponent" )	
if ( components ~= nil ) then
	for key,comp_id in pairs(components) do 
		ComponentSetValue2(comp_id, "area_circle_radius", radius, radius)
	end
end

local gaec_id = EntityGetFirstComponentIncludingDisabled( field_id, "GameAreaEffectComponent" )
ComponentSetValue2(gaec_id, "radius", radius)

-- detects projectiles within radius
local field_x, field_y = EntityGetTransform(field_id)
local projectiles = EntityGetInRadiusWithTag(field_x, field_y, radius, "projectile")

-- deflects projectiles
for i, projectile_id in ipairs(projectiles) do
	for i, component in ipairs(EntityGetComponent(projectile_id, "ProjectileComponent") or {}) do
		if ComponentGetValue2(component, "mWhoShot") ~= entity_id then -- projectiles are only deflected if not shot by the entity under the effect, this check is necessary otherwise the field itself will be caught within its own field
			
			-- check to exclude projectiles shot by relic effects
			if((EntityHasTag(entity_id, "player_unit")) and entityHasSimplifiedInternalVariable(projectile_id, "relic_effect_projectile")) then
				return
			end
			
			local x, y = EntityGetTransform(projectile_id)

			-- First checks if the projectile has already been deflected, returns to the caller if so
			if(entityHasSimplifiedInternalVariable(projectile_id, "deflected")) then
				return
			end
			
			-- Then checks if the projectile contains the shield_bypass flag, which also excludes it from deflection mechanic
			if(entityHasSimplifiedInternalVariable(projectile_id, "shield_bypass")) then
				return
			end
			
			-- check to prevent rotating projectiles from being deflected
			if(string.find(EntityGetName(projectile_id), "rotating_shield_projectile")) then return end

			-- Loads audiovisual feedback
			GamePlaySound( "data/audio/Desktop/projectiles.snd", "player_projectiles/shield/block", x, y )
			EntityLoad(deflect_explosion_entity_path, x, y)
			
			local projectile_vel_x, projectile_vel_y = getProjectileVelocitiesandGravity(projectile_id)
			
			-- Inverts projectile velocity, turning it back to the direction it came
			setProjectileVelocitiesandGravity(projectile_id, (projectile_vel_x + 1.5) * -1, (projectile_vel_y + 1.5) * -1)

			-- Sets the shooter of the deflected projectile as the creature under the effect
			setProjectileShooter(projectile_id, entity_id)

			-- Flags projectile as deflected to prevent it from changing again
			addNewSimplifiedInternalVariable(projectile_id, "deflected")
			
			-- Applies base damage bonus on deflected projectile
			multiplyProjectileBaseDamage(projectile_id, 15)
			addBuffVisualIndicator_Projectile(projectile_id, deflect_trail_material_name)
		end
	end
end

local target_tag = ""

if(creatureIs(entity_id) == "enemy") then
	target_tag = "player_unit"
else
	target_tag = "enemy"
end

local creatures = EntityGetInRadiusWithTag(field_x, field_y, radius, target_tag)

-- damages and knocks enemies back
for i, creature_id in ipairs(creatures) do
	if(creature_id ~= entity_id) then -- only knocks back creatures other than the entity under this effect
		-- First obtains velocity of entity under this effect
		local entity_vel_x, entity_vel_y = getCreatureVelocities(entity_id)
		
		-- then applies the same velocity to creature caught within the field
		setCreatureVelocities(creature_id, entity_vel_x, entity_vel_y)
		
		-- Loads audiovisual feedback
		GamePlaySound( "data/audio/Desktop/projectiles.snd", "player_projectiles/shield/block", x, y )
		EntityLoad("mods/ArcaneReliquary/files/entity_definitions/particle_explosions/shield_damage_explosion_pink.xml", x, y)
		
		-- Deals damage to creature within field
		local damageThreshold = 0.1
		
		EntityInflictDamage( creature_id, damageThreshold, "DAMAGE_MELEE", "knocked back", "NORMAL", 0, 0)
	end	
end