dofile_once("data/scripts/lib/utilities.lua")
dofile_once("mods/ArcaneReliquary/files/scripts/utilities/arcanereliquary_utilities.lua")

local entity_id = GetUpdatedEntityID()
entity_id = EntityGetRootEntity( entity_id )
local x, y = EntityGetTransform(entity_id)
local field_id = EntityLoad("mods/ArcaneReliquary/files/entity_definitions/others/transmuting_shield.xml", x, y)

local field_x, field_y = EntityGetTransform(field_id)

local projectiles = EntityGetInRadiusWithTag(field_x, field_y, 15, "projectile")

-- deflects projectiles
for i, projectile_id in ipairs(projectiles) do
	for i, component in ipairs(EntityGetComponent(projectile_id, "ProjectileComponent") or {}) do
		if ComponentGetValue2(component, "mWhoShot") ~= entity_id then -- projectiles are only deflected if not shot by the entity under the effect, this check is necessary otherwise the field itself will be caught within its own field
			
			-- check to exclude projectiles shot by relic effects
			if((EntityHasTag(entity_id, "player_unit")) and entityHasSimplifiedInternalVariable(projectile_id, "relic_effect_projectile")) then
				return
			end
			
			local x, y = EntityGetTransform(projectile_id)

			-- First checks if the projectile has already been deflected, returns to the caller if so
			if(entityHasSimplifiedInternalVariable(projectile_id, "deflected")) then
				return
			end
			
			-- Then checks if the projectile contains the shield_bypass flag, which also excludes it from deflection mechanic
			if(entityHasSimplifiedInternalVariable(projectile_id, "shield_bypass")) then
				return
			end
			
			-- check to prevent rotating projectiles from being deflected
			if(string.find(EntityGetName(projectile_id), "rotating_shield_projectile")) then return end

			-- Loads audiovisual feedback
			GamePlaySound( "data/audio/Desktop/projectiles.snd", "player_projectiles/shield/block", x, y )
			--EntityLoad("mods/ArcaneReliquary/files/entity_definitions/others/deflect_explosion_green.xml", x, y)
			EntityLoad("mods/ArcaneReliquary/files/entity_definitions/particle_explosions/projectile_transmute_explosion.xml", x, y)
			
			projectileToggleExplosionOnDeath(projectile_id, false)
			entityConvertToRandomMaterial(projectile_id)

			-- if it's an enemy who's under this effect, deals small amount of damage each time a projectile is deflected
			if(EntityHasTag(entity_id, "enemy")) then
				--GamePrint("### deflect damage")
				SetRandomSeed( GameGetFrameNum(), x + y + entity_id )
				local projectile_base_damage = getProjectileBaseDamage(projectile_id)
				EntityInflictDamage( entity_id, projectile_base_damage / Random(2, 3), "DAMAGE_PHYSICS_HIT", "shield deflect damage", "NORMAL", 0, 0)
			end
			
		end
	end
end