dofile_once("data/scripts/lib/coroutines.lua")
dofile_once("mods/ArcaneReliquary/files/scripts/lists/relics_list.lua")
dofile_once("mods/ArcaneReliquary/files/scripts/utilities/arcanereliquary_utilities.lua")
dofile_once("mods/ArcaneReliquary/files/scripts/lists/custom_effects_list.lua")

-- Control variables
local renderOwnedRelics = false
local renderRequiredRelics = false
local main_id = 1

local currentRelicIndex = 1

function getNewId()
	return main_id + 1
end

-- General gui elements settings
local gui_toggle = gui_toggle or GuiCreate()
local gui_toggle_id = getNewId()

local gui_drop = gui_drop or GuiCreate()
local gui_drop_id = getNewId()

local gui_next_relic = gui_next_relic or GuiCreate()
local gui_next_relic_id = getNewId()

local gui_previous_relic = gui_previous_relic or GuiCreate()
local gui_previous_relic_id = getNewId()

local gui_is_active_relic = gui_is_active_relic or GuiCreate()
local gui_is_active_relic_id = getNewId()

local gui_is_combine_exclusive = gui_is_combine_exclusive or GuiCreate()
local gui_is_combine_exclusive_id = getNewId()

local gui_relic_pics = gui_relic_pics or GuiCreate()
local gui_relic_pics_id = getNewId()

local gui_relic_pics_2nd_column = gui_relic_pics_2nd_column or GuiCreate()
local gui_relic_pics_2nd_column_id = getNewId()

local gui_relics = gui_relics or GuiCreate()
local gui_relics_id = getNewId()

local gui_required_relics = gui_required_relics or GuiCreate()
local gui_required_relics_id = getNewId()

local gui_sell_relic_store = gui_sell_relic_store or GuiCreate()

function resetIndex()
	currentRelicIndex = 1
end

-- Toggles active bounties gui on or off
function toggleRenderOwnedRelics()
	renderOwnedRelics = not renderOwnedRelics
	
	if(renderOwnedRelics) then
		GlobalsSetValue("ARCANERELIQUARY_RENDER_MAIN_GUI_FLAG", "1")
		GlobalsSetValue("BOUNTYBOARDS_RENDER_MAIN_GUI_FLAG", "0")
	else
		GlobalsSetValue("ARCANERELIQUARY_RENDER_MAIN_GUI_FLAG", "0")
		GlobalsSetValue("BOUNTYBOARDS_RENDER_MAIN_GUI_FLAG", "1")
	end
end

function toggleRenderRequiredRelics()
	renderRequiredRelics = not renderRequiredRelics
end

function renderToggleButton()
	
	-- doesn't render button if player inventory is open, or if Inventory Bags inventory is currently open
	if(isInventoryOpen() or isInventoryBagsOpen()) then return end
	
	GuiStartFrame(gui_toggle);
	GuiLayoutBeginVertical( gui_toggle, 1, 11);
	if(GuiImageButton( gui_toggle, getNewId(), -4, 0, "", getGUIIconsPath() .. "main_icon.png" )) then
		toggleRenderOwnedRelics()
	end
	GuiTooltip(gui_toggle, "Toggle Owned Relics", "Click to show / hide owned relics")
	GuiLayoutEnd( gui_toggle );
	
end

function navigateOwnedRelics(owned_relics_count, forward)
	local new_index = 0
	if(forward) then
		-- moves index forward
		new_index = currentRelicIndex + 1
		if(new_index > owned_relics_count) then
			currentRelicIndex = 1
		else
			currentRelicIndex = new_index
		end
	else
		-- moves index backwards
		new_index = currentRelicIndex - 1
		if(new_index == 0) then
			currentRelicIndex = owned_relics_count
		else
			currentRelicIndex = new_index
		end
	end
end

function renderRelicsList()
	-- if toggle variable is not true, vanilla inventory or Inventory Bags inventory is open or global flag is false, skips rendering
	-- the global flag is set to false when gui elements related to machines are currently rendering
	if(not renderOwnedRelics or isInventoryOpen() or isInventoryBagsOpen() or GlobalsGetValue("ARCANERELIQUARY_RENDER_MAIN_GUI_FLAG", "0") == "0") then return end

	GuiStartFrame(gui_relics);
	GuiLayoutBeginVertical( gui_relics, 5, 11);
	
	local owned_relics = getOwnedRelics()
	local owned_relics_count = #owned_relics
	
	if(not next(owned_relics)) then
		GuiText( gui_relics, 2, 4, "No owned relics" )
		resetIndex() -- sets index back to original value
	else
			
		local current_relic = owned_relics[currentRelicIndex]
		local relic_name = EntityGetName(current_relic)
		local relic = getRelic(relic_name)
		
		-- obtains relic level
		local item_level = getInternalVariableValue(current_relic, "relic_level", "value_int")
		local relic_level = " " .. levelNumbertoRoman(item_level)
		
		-- obtains is active flag
		local is_active = getInternalVariableValue(current_relic, "is_active", "value_bool")
		
		-- text description for current relic
		local simplified_ui_name = relic.ui_name .. relic_level
		local descriptive_ui_name = simplified_ui_name .. "\n" .. relic.effect_desc
		if(relic.is_upgradable) then
			descriptive_ui_name = descriptive_ui_name .. "\n" .. relic.upgrade_effect_desc
		else
			descriptive_ui_name = descriptive_ui_name .. "\nCan't be upgraded."
		end
	
		-- drop button and click action
		GuiStartFrame(gui_drop);
		GuiLayoutBeginVertical( gui_drop, 4, 11);
		if(GuiImageButton( gui_drop, getNewId(), -4, 0, "", getGUIIconsPath() .. "drop_icon.png" )) then
			
			-- sets index
			resetIndex()
			
			-- kills child entity and loads it back at player's feet
			GamePrint(relic.ui_name .. " dropped")
			local x, y = EntityGetTransform(getPlayerEntity())
			local physical_item = EntityLoad(getRelicsBaseDir() .. relic_name .. ".xml", x, y - 15)
			resetPhysicalRelicSettings(current_relic, physical_item, relic)
			EntityRemoveFromParent( current_relic )
			EntityKill(current_relic)
			
			-- runs drop function from relic object
			if(relic.ondrop_function ~= nil) then
				relic.ondrop_function()
			end
			
		end
		GuiTooltip(gui_drop, "Click to drop ", "")
		GuiLayoutEnd( gui_drop );
		
		-- previous relic button and click action
		GuiStartFrame(gui_previous_relic);
		GuiLayoutBeginVertical( gui_previous_relic, 6, 11);
		if(GuiImageButton( gui_previous_relic, getNewId(), -4, 0, "", getGUIIconsPath() .. "nav_previous_icon.png" )) then
			-- moves index backwards
			navigateOwnedRelics(owned_relics_count, false)
		end
		GuiTooltip(gui_previous_relic, "Click to show previous relic", "")
		GuiLayoutEnd( gui_previous_relic );
		
		-- next relic button and click action
		GuiStartFrame(gui_next_relic);
		GuiLayoutBeginVertical( gui_next_relic, 8, 11);
		if(GuiImageButton( gui_next_relic, getNewId(), -4, 0, "", getGUIIconsPath() .. "nav_next_icon.png" )) then
			-- moves index forward
			navigateOwnedRelics(owned_relics_count, true)
		end
		GuiTooltip(gui_next_relic, "Click to show next relic", "")
		GuiLayoutEnd( gui_next_relic );
		
		-- navigation controls using key inputs
		if(ModSettingGet("ArcaneReliquary.relics_cycle_item_switch_controls") == "ENABLED") then
			local cc_id = EntityGetFirstComponent(getPlayerEntity(), "ControlsComponent")
			local left_triggered = ComponentGetValueInt( cc_id, "mButtonFrameChangeItemL" ) == GameGetFrameNum()
			if(left_triggered) then navigateOwnedRelics(owned_relics_count, false) end
			local right_triggered = ComponentGetValueInt( cc_id, "mButtonFrameChangeItemR" ) == GameGetFrameNum()
			if(right_triggered) then navigateOwnedRelics(owned_relics_count, true) end
		end
		
		-- is active button and click action
		GuiStartFrame(gui_is_active_relic);
		GuiLayoutBeginVertical( gui_is_active_relic, 10, 11);
		
		local is_active_icon = "active_icon.png"
		if(not is_active) then
			is_active_icon = "inactive_icon.png"
		end
		if(GuiImageButton( gui_is_active_relic, getNewId(), -4, 0, "", getGUIIconsPath() .. is_active_icon )) then
			-- runs on drop or on pickup function according to the active state of the relic
			if(is_active) then
				if(relic.ondrop_function ~= nil) then
					relic.ondrop_function()
				end
			else
				if(relic.onpickup_function ~= nil) then
					relic.onpickup_function()
				end
			end
			-- toggles is active flag
			toggleRelicDisabled_ChildItem(current_relic)
			
		end
		GuiTooltip(gui_is_active_relic, "Click to activate or deactivate this relic ", "")
		GuiLayoutEnd( gui_is_active_relic );
		
		-- relic pic and description
		GuiImage( gui_relics, getNewId(), 45, 2, getItemUIPath() .. relic_name .. ".png", 1, 1)
		GuiText( gui_relics, 50, 2, descriptive_ui_name)
		
		-- relic requirement
		if(relic.requires_relic ~= nil) then
			GuiText( gui_relics, 50, 2, "Requires the following relic in order to function:")
			local required_relic = getRelic(relic.requires_relic)
			GuiImage( gui_relics, getNewId(), 45, 2, getItemUIPath() .. required_relic.name .. ".png", 1, 1)
			GuiTooltip(gui_relics, required_relic.ui_name .. "\n" .. required_relic.effect_desc, "")
		end
		
		-- is combine exclusive button and click action
		if(relic.combine_exclusive ~= nil and relic.combine_exclusive) then
			
			GuiStartFrame(gui_is_combine_exclusive);
			GuiLayoutBeginVertical( gui_is_combine_exclusive, 16, 11);
			if(GuiImageButton( gui_is_combine_exclusive, getNewId(), -4, 0, "", getGUIIconsPath() .. "combine_indicator_icon.png" )) then
				toggleRenderRequiredRelics()
			end
			GuiTooltip(gui_is_combine_exclusive, "Can be created by combining other \nrelics in a Relic Combine Station. Click to show them.", "")
			GuiLayoutEnd( gui_is_combine_exclusive );
			
			if(renderRequiredRelics) then
				local combine_requires_relics = relic.combine_requires_relics
				local combine_required_relics_names = {}
				for combine_required_relic in string.gmatch(combine_requires_relics, "([^,]+)") do
					table.insert(combine_required_relics_names, combine_required_relic)
				end
				
				GuiText( gui_relics, 50, 2, "This relic can be created by combining these relics:")
				for i, required_relic_name in ipairs(combine_required_relics_names) do
					local combine_required_relic = getRelic(required_relic_name)
					GuiImage( gui_relics, getNewId(), 45, 2, getItemUIPath() .. combine_required_relic.name .. ".png", 1, 1)
					GuiTooltip(gui_relics, combine_required_relic.ui_name .. "\n" .. combine_required_relic.effect_desc, "")
				end
			end
			
		end	

		-- sell button and click action, renders only when in range of a relic store
		if((isPlayerInRangeofMachine("relic_store", 30) ~= nil)) then
			GuiStartFrame(gui_sell_relic_store);
			GuiLayoutBeginVertical( gui_sell_relic_store, 19, 11);
			if(GuiImageButton( gui_sell_relic_store, getNewId(), -4, 0, "", getGUIIconsPath() .. "sell_icon.png" )) then
				-- sets index
				resetIndex()
				
				-- sells relic
				sellRelic(getRelicSellPrice(), item_level, relic)

				-- Kills child item on player
				EntityRemoveFromParent( current_relic )
				EntityKill(current_relic)
				
				-- runs drop function from relic object
				if(relic.ondrop_function ~= nil) then
					relic.ondrop_function()
				end
			end
			
			if(item_level == nil) then item_level = 0 end
			local relic_sell_value = 0
			relic_sell_value = getRelicSellPrice() + (item_level * 50)
			
			GuiTooltip(gui_sell_relic_store, "Click to sell currently selected relic for " .. relic_sell_value .. " gold.\n(" .. getRelicSellPrice() .. " + 50 per level)", "")
			GuiLayoutEnd( gui_sell_relic_store );
		end

		-- columns setup
		local max_items_per_column = 10
		local next_index = 0
		
		-- renders list of owned relics as pics
		
		-- first column
		GuiStartFrame(gui_relic_pics);
		GuiLayoutBeginVertical( gui_relic_pics, 4, 17);
		for i,owned_relic in ipairs(owned_relics) do
			GuiImage( gui_relic_pics, getNewId(), -4, 0, getItemUIPath() .. EntityGetName(owned_relic) .. ".png", 1, 1)
			if(i == currentRelicIndex) then
				GuiOptionsAddForNextWidget( gui_relic_pics, 12 )
				GuiImage( gui_relic_pics, getNewId(), -4, 0, getGUIIconsPath() .. "current_relic.png", 1, 1)
			end
			if(i == max_items_per_column) then
				next_index = i + 1
				break
			end
		end
		GuiLayoutEnd( gui_relic_pics );
		
		-- second column if more than max_items_per_column are owned
		if(next_index ~= 0) then
			GuiStartFrame(gui_relic_pics_2nd_column);
			GuiLayoutBeginVertical( gui_relic_pics_2nd_column, 8, 17);
			for i=next_index,owned_relics_count do
				GuiImage( gui_relic_pics_2nd_column, getNewId(), -4, 0, getItemUIPath() .. EntityGetName(owned_relics[i]) .. ".png", 1, 1)
				if(i == currentRelicIndex) then
					GuiOptionsAddForNextWidget( gui_relic_pics_2nd_column, 12 )
					GuiImage( gui_relic_pics_2nd_column, getNewId(), -4, 0, getGUIIconsPath() .. "current_relic.png", 1, 1)
				end
			end
			GuiLayoutEnd( gui_relic_pics_2nd_column );
		end
		
	end
	
	GuiLayoutEnd( gui_relics );
end

if gui_toggle and gui_relics then 
	async_loop(
		function() 
			if gui_toggle and gui_relics then 
				renderToggleButton()
				renderRelicsList()
			end 
			wait(0)
		end
	)
end