dofile_once("data/scripts/lib/coroutines.lua")
dofile_once("mods/ArcaneReliquary/files/scripts/lists/relics_list.lua")
dofile_once("mods/ArcaneReliquary/files/scripts/utilities/arcanereliquary_utilities.lua")

-- Control variables
local main_id = 1
local how_to_text = 
[[
Put basic relics onto the combining plate, if
a combination recipe exists, this machine will
automatically destroy them and create a new
relic. Browse the list of valid recipes to
discover which relics can be combined into others.
]]
local combine_exclusive_relics = getCombineExclusiveRelics()
local navIndex = 1

function getNewId()
	return main_id + 1
end

-- gui elements
local gui_combinestation_text = gui_combinestation_text or GuiCreate()
local gui_combinestation_howto = gui_combinestation_howto or GuiCreate()
local gui_previous_recipe = gui_previous_recipe or GuiCreate()
local gui_next_recipe = gui_next_recipe or GuiCreate()
local gui_recipe_row_1 = gui_recipe_row_1 or GuiCreate()
local gui_recipe_row_2 = gui_recipe_row_2 or GuiCreate()

function renderCombineStationGUI()

	-- doesn't render gui if player inventory is open, or if Inventory Bags inventory is currently open or if no prize machines in range
	local relic_combine_station_id = isPlayerInRangeofMachine("relic_combine_station", 30)
	if(isInventoryOpen() or isInventoryBagsOpen() or (relic_combine_station_id == nil)) then
		--GlobalsSetValue("ARCANERELIQUARY_RENDER_MAIN_GUI_FLAG", "1") -- machine gui can't render, main gui can render again
		return 
	end
	
	-- main gui is hidden when this gui is showing
	--GlobalsSetValue("ARCANERELIQUARY_RENDER_MAIN_GUI_FLAG", "0")
	
	GuiStartFrame(gui_combinestation_text);
	GuiLayoutBeginVertical( gui_combinestation_text, 23, 65);
	GuiText( gui_combinestation_text, 40, 2, "List of recipes and their resulting relics:")
	GuiLayoutEnd( gui_combinestation_text );
	
	-- how to button
	GuiStartFrame(gui_combinestation_howto);
	GuiLayoutBeginVertical( gui_combinestation_howto, 30, 61);
	GuiImage( gui_combinestation_howto, getNewId(), -4, 0, getGUIIconsPath() .. "howto_icon.png", 1, 1)
	GuiTooltip(gui_combinestation_howto, how_to_text, "")
	GuiLayoutEnd( gui_combinestation_howto );
	
	-- obtaining controls component for later use
	local cc_id = EntityGetFirstComponent(getPlayerEntity(), "ControlsComponent")
	
	-- nav up button and click action
	local left_triggered = ComponentGetValueInt( cc_id, "mButtonFrameChangeItemL" ) == GameGetFrameNum()
	GuiStartFrame(gui_previous_recipe);
	GuiLayoutBeginVertical( gui_previous_recipe, 30, 68);
	if(GuiImageButton( gui_previous_recipe, getNewId(), -4, 0, "", getGUIIconsPath() .. "nav_up_icon.png" ) or left_triggered) then
		navIndex = navIndex - 1
		if(navIndex <= 0) then
			navIndex = #combine_exclusive_relics
		end
	end
	GuiTooltip(gui_previous_recipe, "Click to navigate up", "")
	GuiLayoutEnd( gui_previous_recipe );
	
	-- recipe visual
	-- row 1, shows recipe
	GuiStartFrame(gui_recipe_row_1);
	GuiLayoutBeginHorizontal( gui_recipe_row_1, 24, 71);
	GuiText( gui_recipe_row_1, 40, 2, "Combine: ")
	
	-- builds table with recipe items, then uses table to build list of relic pics
	local recipe_items = {}
	for recipe_item_name in string.gmatch(combine_exclusive_relics[navIndex].combine_requires_relics, '([^,]+)') do
		table.insert(recipe_items, recipe_item_name)
	end
	for i,recipe_item in ipairs(recipe_items) do
		local recipe_item_relic = getRelic(recipe_item)
		GuiImage( gui_recipe_row_1, getNewId(), 2, 0, getItemUIPath() .. recipe_item_relic.name .. ".png", 1, 1)
		GuiTooltip(gui_recipe_row_1, recipe_item_relic.ui_name, "")
	end
	
	GuiLayoutEnd(gui_recipe_row_1)
	
	-- row 2, shows resulting relic
	GuiStartFrame(gui_recipe_row_2);
	GuiLayoutBeginHorizontal( gui_recipe_row_2, 24, 75);
	GuiText( gui_recipe_row_2, 40, 2, "To get: ")
	GuiImage( gui_recipe_row_2, getNewId(), 5, 0, getItemUIPath() .. combine_exclusive_relics[navIndex].name .. ".png", 1, 1)
	GuiTooltip(gui_recipe_row_2, combine_exclusive_relics[navIndex].ui_name, "")
	GuiLayoutEnd(gui_recipe_row_2)
	
	-- nav down button and click action
	local right_triggered = ComponentGetValueInt( cc_id, "mButtonFrameChangeItemR" ) == GameGetFrameNum()
	GuiStartFrame(gui_next_recipe);
	GuiLayoutBeginVertical( gui_next_recipe, 30, 78);
	if(GuiImageButton( gui_next_recipe, getNewId(), -4, 0, "", getGUIIconsPath() .. "nav_down_icon.png" ) or right_triggered) then
		navIndex = navIndex + 1
		if(navIndex >= #combine_exclusive_relics) then
			navIndex = 1
		end
	end
	GuiTooltip(gui_next_recipe, "Click to navigate down", "")
	GuiLayoutEnd( gui_next_recipe );


end

if gui_combinestation_text then 
	async_loop(
		function() 
			renderCombineStationGUI()
			wait(0)
		end
	)
end
