dofile_once("data/scripts/lib/coroutines.lua")
dofile_once("mods/ArcaneReliquary/files/scripts/lists/relics_list.lua")
dofile_once("mods/ArcaneReliquary/files/scripts/utilities/arcanereliquary_utilities.lua")

-- Control variables
local main_id = 1
local how_to_text = 
[[
Pour basic or advanced upgrade compounds into the collecting vat, then
place the relic you wish to level up onto the upgrade plate. This forge will then 
consume materials you poured into it and upgrade your relic automatically until 
there is no upgrade material left for another level.
]]

function getNewId()
	return main_id + 1
end

-- gui elements
local gui_forge_text = gui_forge_text or GuiCreate()
local gui_upgrade_compound = gui_upgrade_compound or GuiCreate()
local gui_upgrade_compound_advanced = gui_upgrade_compound_advanced or GuiCreate()
local gui_forge_howto = gui_forge_howto or GuiCreate()


function renderForgeGUI()

	-- doesn't render gui if player inventory is open, or if Inventory Bags inventory is currently open or if no prize machines in range
	local relic_forge_id = isPlayerInRangeofMachine("relic_forge", 30)
	if(isInventoryOpen() or isInventoryBagsOpen() or (relic_forge_id == nil)) then
		--GlobalsSetValue("ARCANERELIQUARY_RENDER_MAIN_GUI_FLAG", "1") -- machine gui can't render, main gui can render again
		return 
	end
	
	-- main gui is hidden when this gui is showing
	--GlobalsSetValue("ARCANERELIQUARY_RENDER_MAIN_GUI_FLAG", "0")
	
	-- obtaining info about upgrade budgets
	local relic_material_base_upgrade_cost, relic_material_advanced_upgrade_cost = getRelicUpgradeCosts()
	if(entityHasSimplifiedInternalVariable(getPlayerEntity(), "upgrade_material_budget_discount")) then
		relic_material_base_upgrade_cost = relic_material_base_upgrade_cost / 2
		relic_material_advanced_upgrade_cost = relic_material_advanced_upgrade_cost / 2
	end
	
	-- obtaining info about upgrade material names
	local upgrade_material_name, relic_advanced_upgrade_material_name = getRelicUpgradeMaterialName()
	local amount = getMaterialAmountOnEntityInventory(relic_forge_id, upgrade_material_name)
	local amount_advanced = getMaterialAmountOnEntityInventory(relic_forge_id, relic_advanced_upgrade_material_name)
	
	-- levels worth calculation
	local levels_worth_basic = amount / relic_material_base_upgrade_cost
	local levels_worth_advanced = amount_advanced / relic_material_advanced_upgrade_cost
	
	GuiStartFrame(gui_forge_text);
	GuiLayoutBeginVertical( gui_forge_text, 23, 65);
	GuiText( gui_forge_text, 40, 2, "Upgrade materials in this forge:")
	GuiLayoutEnd( gui_forge_text );
	
	GuiStartFrame(gui_upgrade_compound);
	GuiLayoutBeginHorizontal( gui_upgrade_compound, 30, 70);
	GuiImage( gui_upgrade_compound, getNewId(), -4, 0, "mods/ArcaneReliquary/files/other_items_gfx/relic_upgrade_compound_container.png", 1, 1)
	GuiTooltip(gui_upgrade_compound, "Basic upgrade compound", "")
	GuiText( gui_upgrade_compound, 10, 1, amount .. " | worth " .. math.floor(levels_worth_basic) .. " levels")
	GuiText( gui_upgrade_compound, 10, 1, "Level 1 to 5 budget: " .. relic_material_base_upgrade_cost)
	GuiLayoutEnd( gui_upgrade_compound );
	
	GuiStartFrame(gui_upgrade_compound_advanced);
	GuiLayoutBeginHorizontal( gui_upgrade_compound_advanced, 30, 73);
	GuiImage( gui_upgrade_compound_advanced, getNewId(), -4, 0, "mods/ArcaneReliquary/files/other_items_gfx/relic_upgrade_compound_container_advanced.png", 1, 1)
	GuiTooltip(gui_upgrade_compound_advanced, "Advanced upgrade compound", "")
	GuiText( gui_upgrade_compound_advanced, 10, 1, amount_advanced .. " | worth " .. math.floor(levels_worth_advanced) .. " levels")
	GuiText( gui_upgrade_compound_advanced, 10, 1, "Level 5 to 10 budget: " .. relic_material_advanced_upgrade_cost)
	GuiLayoutEnd( gui_upgrade_compound_advanced );
	
	-- how to button
	GuiStartFrame(gui_forge_howto);
	GuiLayoutBeginVertical( gui_forge_howto, 30, 61);
	GuiImage( gui_forge_howto, getNewId(), -4, 0, getGUIIconsPath() .. "howto_icon.png", 1, 1)
	GuiTooltip(gui_forge_howto, how_to_text, "")
	GuiLayoutEnd( gui_forge_howto );

end

if gui_upgrade_compound and gui_upgrade_compound_advanced then 
	async_loop(
		function() 
			renderForgeGUI()
			wait(0)
		end
	)
end
