dofile_once("data/scripts/lib/coroutines.lua")
dofile_once("mods/ArcaneReliquary/files/scripts/lists/relics_list.lua")
dofile_once("mods/ArcaneReliquary/files/scripts/utilities/arcanereliquary_utilities.lua")

-- Control variables
local main_id = 1
local maximum_claw_left = 40
local maximum_claw_right = 60
local default_claw_heigh = 25
local default_claw_x = 50
local how_to_text = "Use item switch control to move the claw left \nor right (mouse wheel if you're on PC), then \nuse the down button to attempt to grab a prize. \nDeposit relic prize chips or tokens in order to play. \nPrizes will reset each time you visit a prize machine. \nThis machine has "

local current_claw_x = default_claw_x
local current_claw_y = default_claw_heigh

local claw_descending = false
local claw_ascending = false
local claw_movement_blocked = false

local default_prize_height = 40

local default_prize1_x = 44
local default_prize2_x = 47
local default_prize3_x = 50
local default_prize4_x = 53
local default_prize5_x = 56

local prizeGrabbedId = 0
local grabOdds = 3

local relic_prize1 = getRandomRelic()
local relic_prize2 = getRandomRelic()
local relic_prize3 = getRandomRelic()
local relic_prize4 = getRandomRelic()
local relic_prize5 = getRandomRelic()

local render_prize1 = true
local render_prize2 = true
local render_prize3 = true
local render_prize4 = true
local render_prize5 = true

local claw_states = {
	{ id = 1, pic = "claw.png" },
	{ id = 2, pic = "claw_grab.png" },
	{ id = 3, pic = "claw_open.png" },
}

local claw_state = claw_states[1]

function getNewId()
	return main_id + 1
end

-- Prize machine gui elements settings
local gui_claw = gui_claw or GuiCreate()

local gui_prize1 = gui_prize1 or GuiCreate()
local gui_prize2 = gui_prize2 or GuiCreate()
local gui_prize3 = gui_prize3 or GuiCreate()
local gui_prize4 = gui_prize4 or GuiCreate()
local gui_prize5 = gui_prize5 or GuiCreate()

local gui_prize_machine_howto = gui_prize_machine_howto or GuiCreate()

function resetGame(reset_prizes)
	-- resets prizes
	if(reset_prizes) then
		relic_prize1 = getRandomRelic()
		relic_prize2 = getRandomRelic()
		relic_prize3 = getRandomRelic()
		relic_prize4 = getRandomRelic()
		relic_prize5 = getRandomRelic()
		render_prize1 = true
		render_prize2 = true
		render_prize3 = true
		render_prize4 = true
		render_prize5 = true
	end
	-- resets claw
	claw_ascending = false
	claw_descending = false
	claw_movement_blocked = false
	current_claw_y = default_claw_heigh
	current_claw_x = default_claw_x
	prizeGrabbedId = 0
	claw_state = claw_states[1]
end

function checkClawGrabPrize()

	local prizeGrabbed = 0
	
	if(current_claw_y == default_prize_height and current_claw_x == default_prize1_x) then
		prizeGrabbed = 1
	elseif(current_claw_y == default_prize_height and current_claw_x == default_prize2_x) then
		prizeGrabbed = 2
	elseif(current_claw_y == default_prize_height and current_claw_x == default_prize3_x) then
		prizeGrabbed = 3
	elseif(current_claw_y == default_prize_height and current_claw_x == default_prize4_x) then
		prizeGrabbed = 4
	elseif(current_claw_y == default_prize_height and current_claw_x == default_prize5_x) then
		prizeGrabbed = 5
	end
	
	return prizeGrabbed
end

function renderPrizeMachineGUI()

	-- doesn't render gui if player inventory is open, or if Inventory Bags inventory is currently open or if no prize machines in range
	local prize_machine_id = isPlayerInRangeofMachine("relic_prize_machine", 30)
	local x, y = EntityGetTransform(prize_machine_id)
	if(isInventoryOpen() or isInventoryBagsOpen() or (prize_machine_id == nil)) then
		resetGame(true)
		GlobalsSetValue("ARCANERELIQUARY_RENDER_MAIN_GUI_FLAG", "1") -- machine gui can't render, main gui can render again
		return 
	end
	
	-- main gui is hidden when this gui is showing
	GlobalsSetValue("ARCANERELIQUARY_RENDER_MAIN_GUI_FLAG", "0")
	
	-- obtains number of plays left in prize machine in range
	local plays = getInternalVariableValue(prize_machine_id, "plays", "value_int")
	
	-- claw controls
	local cc_id = EntityGetFirstComponent(getPlayerEntity(), "ControlsComponent")
	local left_triggered = ComponentGetValueInt( cc_id, "mButtonFrameChangeItemL" ) == GameGetFrameNum()
	local right_triggered = ComponentGetValueInt( cc_id, "mButtonFrameChangeItemR" ) == GameGetFrameNum()
	local down_triggered = ComponentGetValueInt( cc_id, "mButtonFrameDown" ) == GameGetFrameNum()
	if(left_triggered and not claw_movement_blocked) then -- if left control is triggered, claw moves left by one
		current_claw_x = current_claw_x - 1
		if(current_claw_x <= maximum_claw_left) then
			current_claw_x = maximum_claw_left
		end
	elseif(right_triggered and not claw_movement_blocked) then -- if right control is triggered, claw moves right by one
		current_claw_x = current_claw_x + 1
		if(current_claw_x >= maximum_claw_right) then
			current_claw_x = maximum_claw_right
		end
	elseif(down_triggered) then -- if down control is triggered, claw is now in descend mode, left and right controls are blocked
		-- checks whether prize machine in range contains enough plays
		if(plays >= 1) then -- if plays are available, claw is now in descend mode
			claw_descending = true
			claw_movement_blocked = true	
		else
			-- shows fail message
			message = "No plays left in this machine"				
			GamePrint(message)
			GamePrintImportant(message, "Deposit more Relic Prize Chips in this machine to get more plays", "mods/ArcaneReliquary/files/other_gfx/custom_decoration_claw_fail.png")
			-- loads audio feedback
			GamePlaySound( "data/audio/Desktop/event_cues.snd", "event_cues/chest_bad/create", x, y )
		end
		
	end
	
	if(claw_descending) then -- moves claw down
		claw_state = claw_states[3]
		current_claw_y = current_claw_y + 1
		prizeGrabbedId = checkClawGrabPrize() -- checks whether a prize has been grabbed
		--if(prizeGrabbedId ~= 0) then GamePrint("Prize grabbed: " .. prizeGrabbedId) end
	end
	
	if(current_claw_y >= default_prize_height) then -- if claw reaches prize height, reverses it to ascend mode
		claw_ascending = true
		claw_descending = false
		
		-- deducts one play from machine
		setInternalVariableValue(prize_machine_id, "plays", "value_int", plays - 1)
	end
	
	if(claw_ascending) then -- moves claw up until it reaches its default height
		claw_state = claw_states[2]
		current_claw_y = current_claw_y - 1
		if(current_claw_y <= default_claw_heigh) then
			resetGame(false) -- resets game as soon as claw reaches its default heigh again
		end
	end
	
	GuiStartFrame(gui_claw);
	GuiLayoutBeginVertical( gui_claw, current_claw_x, current_claw_y);
	GuiImage( gui_claw, getNewId(), -4, 0, getGUIIconsPath() .. claw_state.pic, 1, 1)
	GuiLayoutEnd( gui_claw );
	
	-- prizes handling
	SetRandomSeed( GameGetFrameNum(), math.random(1,50))
	
	
	-- prize 1
	local prize1_x = default_prize1_x
	local prize1_y = default_prize_height
	if(prizeGrabbedId == 1) then
		prize1_x = current_claw_x
		prize1_y = current_claw_y

		if(prize1_y == default_claw_heigh + 1 and render_prize1) then
			local prizeDispensed = (Random(1,grabOdds) == 1)
			grabPrize(relic_prize1, getRelicsBaseDir(), prize_machine_id, prizeDispensed)
			if(prizeDispensed) then render_prize1 = false end
		end
	end
	if(render_prize1) then
		GuiStartFrame(gui_prize1);
		GuiLayoutBeginVertical( gui_prize1, prize1_x, prize1_y);
		GuiImage( gui_prize1, getNewId(), -4, 0, getItemUIPath() .. relic_prize1.name .. ".png", 1, 1)
		GuiTooltip(gui_prize1, relic_prize1.ui_name, "")
		GuiLayoutEnd( gui_prize1 );
	end
	
	-- prize 2
	local prize2_x = default_prize2_x
	local prize2_y = default_prize_height
	if(prizeGrabbedId == 2) then
		prize2_x = current_claw_x
		prize2_y = current_claw_y

		if(prize2_y == default_claw_heigh + 1 and render_prize2) then
			local prizeDispensed = (Random(1,grabOdds) == 1)
			grabPrize(relic_prize2, getRelicsBaseDir(), prize_machine_id, prizeDispensed)
			if(prizeDispensed) then render_prize2 = false end
		end
	end
	if(render_prize2) then
		GuiStartFrame(gui_prize2);
		GuiLayoutBeginVertical( gui_prize2, prize2_x, prize2_y);
		GuiImage( gui_prize2, getNewId(), -4, 0, getItemUIPath() .. relic_prize2.name .. ".png", 1, 1)
		GuiTooltip(gui_prize2, relic_prize2.ui_name, "")
		GuiLayoutEnd( gui_prize2 );
	end
	
	-- prize 3
	local prize3_x = default_prize3_x
	local prize3_y = default_prize_height
	if(prizeGrabbedId == 3) then
		prize3_x = current_claw_x
		prize3_y = current_claw_y

		if(prize3_y == default_claw_heigh + 1 and render_prize3) then
			local prizeDispensed = (Random(1,grabOdds) == 1)
			grabPrize(relic_prize3, getRelicsBaseDir(), prize_machine_id, prizeDispensed)
			if(prizeDispensed) then render_prize3 = false end
		end
	end
	if(render_prize3) then
		GuiStartFrame(gui_prize3);
		GuiLayoutBeginVertical( gui_prize3, prize3_x, prize3_y);
		GuiImage( gui_prize3, getNewId(), -4, 0, getItemUIPath() .. relic_prize3.name .. ".png", 1, 1)
		GuiTooltip(gui_prize3, relic_prize3.ui_name, "")
		GuiLayoutEnd( gui_prize3 );
	end
	
	-- prize 4
	local prize4_x = default_prize4_x
	local prize4_y = default_prize_height
	if(prizeGrabbedId == 4) then
		prize4_x = current_claw_x
		prize4_y = current_claw_y

		if(prize4_y == default_claw_heigh + 1 and render_prize4) then
			local prizeDispensed = (Random(1,grabOdds) == 1)
			grabPrize(relic_prize4, getRelicsBaseDir(), prize_machine_id, prizeDispensed)
			if(prizeDispensed) then render_prize4 = false end
		end
	end
	if(render_prize4) then
		GuiStartFrame(gui_prize4);
		GuiLayoutBeginVertical( gui_prize4, prize4_x, prize4_y);
		GuiImage( gui_prize4, getNewId(), -4, 0, getItemUIPath() .. relic_prize4.name .. ".png", 1, 1)
		GuiTooltip(gui_prize4, relic_prize4.ui_name, "")
		GuiLayoutEnd( gui_prize4 );
	end
	
	-- prize 5
	local prize5_x = default_prize5_x
	local prize5_y = default_prize_height
	if(prizeGrabbedId == 5) then
		prize5_x = current_claw_x
		prize5_y = current_claw_y

		if(prize5_y == default_claw_heigh + 1 and render_prize5) then
			local prizeDispensed = (Random(1,grabOdds) == 1)
			grabPrize(relic_prize5, getRelicsBaseDir(), prize_machine_id, prizeDispensed)
			if(prizeDispensed) then render_prize5 = false end
		end
	end
	if(render_prize5) then
		GuiStartFrame(gui_prize5);
		GuiLayoutBeginVertical( gui_prize5, prize5_x, prize5_y);
		GuiImage( gui_prize5, getNewId(), -4, 0, getItemUIPath() .. relic_prize5.name .. ".png", 1, 1)
		GuiTooltip(gui_prize5, relic_prize5.ui_name, "")
		GuiLayoutEnd( gui_prize5 );
	end
	
	-- instruction text
	GuiStartFrame(gui_prize_machine_howto);
	GuiLayoutBeginVertical( gui_prize_machine_howto, 50, 45);
	GuiImage( gui_prize_machine_howto, getNewId(), -4, 0, getGUIIconsPath() .. "howto_icon.png", 1, 1)
	GuiTooltip(gui_prize_machine_howto, how_to_text .. plays .. " plays left.", "")
	GuiLayoutEnd( gui_prize_machine_howto );

end

if gui_claw then 
	async_loop(
		function() 
			renderPrizeMachineGUI()
			wait(0)
		end
	)
end