dofile_once("data/scripts/lib/coroutines.lua")
dofile_once("mods/ArcaneReliquary/files/scripts/lists/relics_list.lua")
dofile_once("mods/ArcaneReliquary/files/scripts/utilities/arcanereliquary_utilities.lua")

-- Control variables
local main_id = 1
local storeFilterEnabled = false
local store_relics_list = relics_list
local how_to_text = 
[[
Use arrow buttons or item switch control (mouse wheel if you're on PC) to navigate
through the list of relics, click the filter buttons to narrow down the list to the
category of relics you are looking for, click buy button (golden dollar sign) to buy 
selected relic or click sell button (green dollar sign) to sell the relic that is 
currently highlighted in your list of owned relics. You can also deposit relic
tokens and trade them for relics in this store.
]]

local storeIndex = 3

function getNewId()
	return main_id + 1
end

-- Relic store gui elements settings
local gui_next_relic_store = gui_next_relic_store or GuiCreate()
local gui_next_relic_store_id = getNewId()

local gui_previous_relic_store = gui_previous_relic_store or GuiCreate()
local gui_previous_relic_store_id = getNewId()

local gui_buy_relic_store = gui_buy_relic_store or GuiCreate()
local gui_buy_relic_store_id = getNewId()

local gui_relic_pics_store = gui_relic_pics_store or GuiCreate()
local gui_relic_pics_store_id = getNewId()

local gui_relics_store = gui_relics_store or GuiCreate()
local gui_relics_store_id = getNewId()

local gui_precursor_boot = gui_precursor_boot or GuiCreate()
local gui_precursor_companion = gui_precursor_companion or GuiCreate()
local gui_precursor_death_ray = gui_precursor_death_ray or GuiCreate()
local gui_precursor_device = gui_precursor_device or GuiCreate()
local gui_precursor_effect_crystal = gui_precursor_effect_crystal or GuiCreate()
local gui_precursor_effect_stone = gui_precursor_effect_stone or GuiCreate()
local gui_precursor_elemental_gem = gui_precursor_elemental_gem or GuiCreate()
local gui_precursor_flask = gui_precursor_flask or GuiCreate()
local gui_precursor_gadget = gui_precursor_gadget or GuiCreate()
local gui_precursor_gravity_stone = gui_precursor_gravity_stone or GuiCreate()
local gui_precursor_magic_artifact = gui_precursor_magic_artifact or GuiCreate()
local gui_precursor_necklace = gui_precursor_necklace or GuiCreate()
local gui_precursor_ring = gui_precursor_ring or GuiCreate()
local gui_precursor_energy_shield = gui_precursor_energy_shield or GuiCreate()

local gui_clear_filter = gui_clear_filter or GuiCreate()
local gui_store_howto = gui_store_howto or GuiCreate()

local gui_token_relic_store = gui_token_relic_store or GuiCreate()


function filterStore(precursor_type)
	-- obtains subtable of relics according to precursor_type
	store_relics_list = getRelicsByPrecursorType(precursor_type)
	-- flags store filter
	storeFilterEnabled = true
	-- resets store index
	storeIndex = 3
end

function clearFilter()
	-- sets list back to full list
	store_relics_list = relics_list
	-- flags store filter
	storeFilterEnabled = false
	-- resets store index
	storeIndex = 3
end

function renderStoreGUI()

	-- buy price settings, for later use
	local current_buy_price = getRelicBuyPrice()
	if(entityHasSimplifiedInternalVariable(getPlayerEntity(), "relic_store_discount")) then
		current_buy_price = current_buy_price / 2
	end
	
	-- doesn't gui if player inventory is open, or if Inventory Bags inventory is currently open or if no relic stores in range
	local store_id = isPlayerInRangeofMachine("relic_store", 30)
	if(isInventoryOpen() or isInventoryBagsOpen() or (store_id == nil)) then 
		--GlobalsSetValue("ARCANERELIQUARY_RENDER_MAIN_GUI_FLAG", "1") -- machine gui can't render, main gui can render again
		return 
	end
	
	-- main gui is hidden when this gui is showing
	--GlobalsSetValue("ARCANERELIQUARY_RENDER_MAIN_GUI_FLAG", "0")
	
	-- obtains number of tokens deposited in relic store in range
	local tokens = getInternalVariableValue(store_id, "tokens", "value_int")
	
	-- precursor filter buttons - first column
	GuiStartFrame(gui_precursor_flask);
	GuiLayoutBeginVertical( gui_precursor_flask, 20, 46);
	if(GuiImageButton( gui_precursor_flask, getNewId(), -4, 0, "", getGUIIconsPath() .. "relic_precursor_flask.png" )) then
		filterStore("flask")
	end
	GuiTooltip(gui_precursor_flask, "Click to filter by flask related relics", "")
	GuiLayoutEnd( gui_precursor_flask );
	
	GuiStartFrame(gui_precursor_gadget);
	GuiLayoutBeginVertical( gui_precursor_gadget, 20, 50);
	if(GuiImageButton( gui_precursor_gadget, getNewId(), -4, 0, "", getGUIIconsPath() .. "relic_precursor_gadget.png" )) then
		filterStore("gadget")
	end
	GuiTooltip(gui_precursor_gadget, "Click to filter by gadget related relics", "")
	GuiLayoutEnd( gui_precursor_gadget );
	
	GuiStartFrame(gui_precursor_gravity_stone);
	GuiLayoutBeginVertical( gui_precursor_gravity_stone, 20, 54);
	if(GuiImageButton( gui_precursor_gravity_stone, getNewId(), -4, 0, "", getGUIIconsPath() .. "relic_precursor_gravity_stone.png" )) then
		filterStore("gravity_stone")
	end
	GuiTooltip(gui_precursor_gravity_stone, "Click to filter by gravity stone related relics", "")
	GuiLayoutEnd( gui_precursor_gravity_stone );
	
	GuiStartFrame(gui_precursor_magic_artifact);
	GuiLayoutBeginVertical( gui_precursor_magic_artifact, 20, 58);
	if(GuiImageButton( gui_precursor_magic_artifact, getNewId(), -4, 0, "", getGUIIconsPath() .. "relic_precursor_magic_artifact.png" )) then
		filterStore("magic_artifact")
	end
	GuiTooltip(gui_precursor_magic_artifact, "Click to filter by magic artifact related relics", "")
	GuiLayoutEnd( gui_precursor_magic_artifact );
	
	GuiStartFrame(gui_precursor_necklace);
	GuiLayoutBeginVertical( gui_precursor_necklace, 20, 62);
	if(GuiImageButton( gui_precursor_necklace, getNewId(), -4, 0, "", getGUIIconsPath() .. "relic_precursor_necklace.png" )) then
		filterStore("necklace")
	end
	GuiTooltip(gui_precursor_necklace, "Click to filter by necklace related relics", "")
	GuiLayoutEnd( gui_precursor_necklace );
	
	GuiStartFrame(gui_precursor_ring);
	GuiLayoutBeginVertical( gui_precursor_ring, 20, 66);
	if(GuiImageButton( gui_precursor_ring, getNewId(), -4, 0, "", getGUIIconsPath() .. "relic_precursor_ring.png" )) then
		filterStore("ring")
	end
	GuiTooltip(gui_precursor_ring, "Click to filter by ring related relics", "")
	GuiLayoutEnd( gui_precursor_ring );
	
	GuiStartFrame(gui_precursor_energy_shield);
	GuiLayoutBeginVertical( gui_precursor_energy_shield, 20, 70);
	if(GuiImageButton( gui_precursor_energy_shield, getNewId(), -4, 0, "", getGUIIconsPath() .. "relic_precursor_energy_shield.png" )) then
		filterStore("energy_shield")
	end
	GuiTooltip(gui_precursor_energy_shield, "Click to filter by energy shield related relics", "")
	GuiLayoutEnd( gui_precursor_energy_shield );
	
	GuiStartFrame(gui_store_howto);
	GuiLayoutBeginVertical( gui_store_howto, 20, 74);
	GuiImage( gui_store_howto, getNewId(), -4, 0, getGUIIconsPath() .. "howto_icon.png", 1, 1)
	GuiTooltip(gui_store_howto, how_to_text, "")
	GuiLayoutEnd( gui_store_howto );
	
	-- precursor filter buttons - second column
	GuiStartFrame(gui_precursor_boot);
	GuiLayoutBeginVertical( gui_precursor_boot, 22, 46);
	if(GuiImageButton( gui_precursor_boot, getNewId(), -4, 0, "", getGUIIconsPath() .. "relic_precursor_boot.png" )) then
		filterStore("boot")
	end
	GuiTooltip(gui_precursor_boot, "Click to filter by boot related relics", "")
	GuiLayoutEnd( gui_precursor_boot );
	
	GuiStartFrame(gui_precursor_companion);
	GuiLayoutBeginVertical( gui_precursor_companion, 22, 50);
	if(GuiImageButton( gui_precursor_companion, getNewId(), -4, 0, "", getGUIIconsPath() .. "relic_precursor_companion.png" )) then
		filterStore("companion")
	end
	GuiTooltip(gui_precursor_companion, "Click to filter by companion related relics", "")
	GuiLayoutEnd( gui_precursor_companion );
	
	GuiStartFrame(gui_precursor_death_ray);
	GuiLayoutBeginVertical( gui_precursor_death_ray, 22, 54);
	if(GuiImageButton( gui_precursor_death_ray, getNewId(), -4, 0, "", getGUIIconsPath() .. "relic_precursor_death_ray.png" )) then
		filterStore("death_ray")
	end
	GuiTooltip(gui_precursor_death_ray, "Click to filter by death ray related relics", "")
	GuiLayoutEnd( gui_precursor_death_ray );
	
	GuiStartFrame(gui_precursor_device);
	GuiLayoutBeginVertical( gui_precursor_device, 22, 58);
	if(GuiImageButton( gui_precursor_device, getNewId(), -4, 0, "", getGUIIconsPath() .. "relic_precursor_device.png" )) then
		filterStore("device")
	end
	GuiTooltip(gui_precursor_device, "Click to filter by device related relics", "")
	GuiLayoutEnd( gui_precursor_device );
	
	GuiStartFrame(gui_precursor_effect_crystal);
	GuiLayoutBeginVertical( gui_precursor_effect_crystal, 22, 62);
	if(GuiImageButton( gui_precursor_effect_crystal, getNewId(), -4, 0, "", getGUIIconsPath() .. "relic_precursor_effect_crystal.png" )) then
		filterStore("effect_crystal")
	end
	GuiTooltip(gui_precursor_effect_crystal, "Click to filter by effect crystal related relics", "")
	GuiLayoutEnd( gui_precursor_effect_crystal );
	
	GuiStartFrame(gui_precursor_effect_stone);
	GuiLayoutBeginVertical( gui_precursor_effect_stone, 22, 66);
	if(GuiImageButton( gui_precursor_effect_stone, getNewId(), -4, 0, "", getGUIIconsPath() .. "relic_precursor_effect_stone.png" )) then
		filterStore("effect_stone")
	end
	GuiTooltip(gui_precursor_effect_stone, "Click to filter by effect stone related relics", "")
	GuiLayoutEnd( gui_precursor_effect_stone );
	
	GuiStartFrame(gui_precursor_elemental_gem);
	GuiLayoutBeginVertical( gui_precursor_elemental_gem, 22, 70);
	if(GuiImageButton( gui_precursor_elemental_gem, getNewId(), -4, 0, "", getGUIIconsPath() .. "relic_precursor_elemental_gem.png" )) then
		filterStore("elemental_gem")
	end
	GuiTooltip(gui_precursor_elemental_gem, "Click to filter by elemental gem related relics", "")
	GuiLayoutEnd( gui_precursor_elemental_gem );
	
	GuiStartFrame(gui_clear_filter);
	GuiLayoutBeginVertical( gui_clear_filter, 22, 74);
	if(GuiImageButton( gui_clear_filter, getNewId(), -4, 0, "", getGUIIconsPath() .. "relic_clear_filters.png" )) then
		clearFilter()
	end
	GuiTooltip(gui_clear_filter, "Click to remove all filters", "")
	GuiLayoutEnd( gui_clear_filter );
	
	-- obtaining controls component for later use
	local cc_id = EntityGetFirstComponent(getPlayerEntity(), "ControlsComponent")
	
	-- nav up button and click action
	local left_triggered = ComponentGetValueInt( cc_id, "mButtonFrameChangeItemL" ) == GameGetFrameNum()
	GuiStartFrame(gui_previous_relic_store);
	GuiLayoutBeginVertical( gui_previous_relic_store, 25, 46);
	if(GuiImageButton( gui_previous_relic_store, getNewId(), -4, 0, "", getGUIIconsPath() .. "nav_up_icon.png" ) or left_triggered) then
		storeIndex = storeIndex - 1
		if(storeIndex <= 0) then
			storeIndex = #store_relics_list
		end
	end
	GuiTooltip(gui_previous_relic_store, "Click to navigate up", "")
	GuiLayoutEnd( gui_previous_relic_store );
	
	local previousIndex_ = storeIndex - 2
	local previousIndex = storeIndex - 1
	local nextIndex = storeIndex + 1
	local nextIndex_ = storeIndex + 2
	
	-- renders always five relics based on current store index
	local relic1 = nil
	local relic2 = nil
	local relic3 = nil
	local relic4 = nil
	local relic5 = nil
	
	if(not storeFilterEnabled) then
		relic1 = getRelicByIndex(previousIndex_)
		relic2 = getRelicByIndex(previousIndex)
		relic3 = getRelicByIndex(storeIndex) -- selected relic in list, always renders in the middle
		relic4 = getRelicByIndex(nextIndex)
		relic5 = getRelicByIndex(nextIndex_)
	else
		relic1 = store_relics_list[previousIndex_]
		relic2 = store_relics_list[previousIndex]
		relic3 = store_relics_list[storeIndex] -- selected relic in list, always renders in the middle
		relic4 = store_relics_list[nextIndex]
		relic5 = store_relics_list[nextIndex_]
	end
	
	GuiStartFrame(gui_relic_pics_store);
	GuiLayoutBeginVertical( gui_relic_pics_store, 25, 50);
	if(previousIndex_ >= 1) then -- renders previous relic only if there is one
		GuiImage( gui_relic_pics_store, getNewId(), -4, 0, getItemUIPath() .. relic1.name .. ".png", 1, 0.5)
	end
	if(previousIndex >= 1) then -- renders previous relic only if there is one
		GuiImage( gui_relic_pics_store, getNewId(), -4, 0, getItemUIPath() .. relic2.name .. ".png", 1, 0.5)
	end
	-- renders main relic pic
	GuiImage( gui_relic_pics_store, getNewId(), -4, 0, getItemUIPath() .. relic3.name .. ".png", 1, 1)
	if(nextIndex <= #store_relics_list) then -- renders next relic only if there is one
		GuiImage( gui_relic_pics_store, getNewId(), -4, 0, getItemUIPath() .. relic4.name .. ".png", 1, 0.5)
	end
	if(nextIndex_ <= #store_relics_list) then -- renders next relic only if there is one
		GuiImage( gui_relic_pics_store, getNewId(), -4, 0, getItemUIPath() .. relic5.name .. ".png", 1, 0.5)
	end
	GuiLayoutEnd( gui_relic_pics_store );
	
	-- text description for current relic
	local simplified_ui_name = relic3.ui_name
	local descriptive_ui_name = simplified_ui_name .. "\n" .. relic3.effect_desc_short
	if(relic3.is_upgradable) then
		descriptive_ui_name = descriptive_ui_name .. "\n" .. relic3.upgrade_effect_desc
	else
		descriptive_ui_name = descriptive_ui_name .. "\nCan't be upgraded."
	end
	
	-- relic3 (selected relic) description and requirement
	GuiStartFrame(gui_relics_store);
	GuiLayoutBeginVertical( gui_relics_store, 22, 50);
	GuiText( gui_relics_store, 40, 2, descriptive_ui_name)
	-- relic3 requirement if any
	if(relic3.requires_relic ~= nil) then
		GuiText( gui_relics_store, 40, 2, "Requires the following relic in order to function:")
		local required_relic = getRelic(relic3.requires_relic)
		GuiImage( gui_relics_store, getNewId(), 40, 2, getItemUIPath() .. required_relic.name .. ".png", 1, 1)
		GuiTooltip(gui_relics_store, required_relic.ui_name .. "\n" .. required_relic.effect_desc, "")
	end
	GuiLayoutEnd(gui_relics_store)
	
	-- nav down button and click action
	local right_triggered = ComponentGetValueInt( cc_id, "mButtonFrameChangeItemR" ) == GameGetFrameNum()
	GuiStartFrame(gui_next_relic_store);
	GuiLayoutBeginVertical( gui_next_relic_store, 25, 63);
	if(GuiImageButton( gui_next_relic_store, getNewId(), -4, 0, "", getGUIIconsPath() .. "nav_down_icon.png" ) or right_triggered) then
		storeIndex = storeIndex + 1
		if(storeIndex >= #store_relics_list) then
			storeIndex = 1
		end
	end
	GuiTooltip(gui_next_relic_store, "Click to navigate down", "")
	GuiLayoutEnd( gui_next_relic_store );
	
	-- buy button and click action
	GuiStartFrame(gui_buy_relic_store);
	GuiLayoutBeginVertical( gui_buy_relic_store, 25, 68);
	if(GuiImageButton( gui_buy_relic_store, getNewId(), -4, 0, "", getGUIIconsPath() .. "buy_icon.png" )) then
		-- buys relic based on which is selected in the store, relic3 in this case
		buyRelic(getRelicBuyPrice(), relic3, getRelicsBaseDir(), store_id)
	end
	GuiTooltip(gui_buy_relic_store, "Click to buy " .. relic3.ui_name .. " for " .. current_buy_price .. " gold", "")
	GuiLayoutEnd( gui_buy_relic_store );
		
	if(tokens >= 1) then
		-- token button and click action
		GuiStartFrame(gui_token_relic_store);
		GuiLayoutBeginVertical( gui_token_relic_store, 28, 68);
		if(GuiImageButton( gui_token_relic_store, getNewId(), -4, 0, "", getGUIIconsPath() .. "token_icon.png" )) then
			-- trades relic3 for 1 token
			tradeRelicforToken(relic3, getRelicsBaseDir(), store_id)
			-- deducts token from total
			setInternalVariableValue(store_id, "tokens", "value_int", tokens - 1)
		end
		GuiTooltip(gui_token_relic_store, "Click to trade " .. relic3.ui_name .. " for 1 token (" .. tokens .. " available)", "")
		GuiLayoutEnd( gui_token_relic_store );
	end
	
	

end

if gui_relics_store then 
	async_loop(
		function() 
			renderStoreGUI()
			wait(0)
		end
	)
end