dofile_once("data/scripts/lib/coroutines.lua")
dofile_once("mods/ArcaneReliquary/files/scripts/lists/utility_items_list.lua")
dofile_once("mods/ArcaneReliquary/files/scripts/utilities/arcanereliquary_utilities.lua")

-- Control variables
local main_id = 1
local store_items_list = utility_items_list
local how_to_text = 
[[
Use arrow buttons or item switch control (mouse wheel if you're on PC) to navigate
through the list of items, click buy button (golden dollar sign) to buy selected 
item.
]]

local storeIndex = 3

function getNewId()
	return main_id + 1
end

-- Utility item store gui elements settings
local gui_next_utility_store = gui_next_utility_store or GuiCreate()
local gui_next_utility_store_id = getNewId()

local gui_previous_utility_store = gui_previous_utility_store or GuiCreate()
local gui_previous_utility_store_id = getNewId()

local gui_buy_utility_store = gui_buy_utility_store or GuiCreate()
local gui_buy_utility_store_id = getNewId()

local gui_item_pics_store = gui_item_pics_store or GuiCreate()
local gui_item_pics_store_id = getNewId()

local gui_utility_items_store = gui_utility_items_store or GuiCreate()
local gui_utility_items_store_id = getNewId()

local gui_store_howto = gui_store_howto or GuiCreate()

function renderUtilityStoreGUI()

	-- buy price settings, for later use
	local relic_store_discount_flag = entityHasSimplifiedInternalVariable(getPlayerEntity(), "relic_store_discount")
	
	-- doesn't gui if player inventory is open, or if Inventory Bags inventory is currently open or if no item stores in range
	local utility_store_id = isPlayerInRangeofMachine("relic_utility_store", 30)
	if(isInventoryOpen() or isInventoryBagsOpen() or (utility_store_id == nil)) then
		--GlobalsSetValue("ARCANERELIQUARY_RENDER_MAIN_GUI_FLAG", "1") -- machine gui can't render, main gui can render again
		return 
	end
	
	-- main gui is hidden when this gui is showing
	--GlobalsSetValue("ARCANERELIQUARY_RENDER_MAIN_GUI_FLAG", "0")
	
	-- obtaining controls component for later use
	local cc_id = EntityGetFirstComponent(getPlayerEntity(), "ControlsComponent")
	
	-- how to button
	GuiStartFrame(gui_store_howto);
	GuiLayoutBeginVertical( gui_store_howto, 28, 71);
	GuiImage( gui_store_howto, getNewId(), -4, 0, getGUIIconsPath() .. "howto_icon.png", 1, 1)
	GuiTooltip(gui_store_howto, how_to_text, "")
	GuiLayoutEnd( gui_store_howto );
	
	-- nav up button and click action
	local left_triggered = ComponentGetValueInt( cc_id, "mButtonFrameChangeItemL" ) == GameGetFrameNum()
	GuiStartFrame(gui_previous_utility_store);
	GuiLayoutBeginVertical( gui_previous_utility_store, 25, 46);
	if(GuiImageButton( gui_previous_utility_store, getNewId(), -4, 0, "", getGUIIconsPath() .. "nav_up_icon.png" ) or left_triggered) then
		storeIndex = storeIndex - 1
		if(storeIndex <= 0) then
			storeIndex = #store_items_list
		end
	end
	GuiTooltip(gui_previous_utility_store, "Click to navigate up", "")
	GuiLayoutEnd( gui_previous_utility_store );
	
	local previousIndex_ = storeIndex - 2
	local previousIndex = storeIndex - 1
	local nextIndex = storeIndex + 1
	local nextIndex_ = storeIndex + 2
	
	-- renders always five relics based on current store index
	local item1 = nil
	local item2 = nil
	local item3 = nil
	local item4 = nil
	local item5 = nil
	
	if(not storeFilterEnabled) then
		item1 = getItemByIndex(previousIndex_)
		item2 = getItemByIndex(previousIndex)
		item3 = getItemByIndex(storeIndex) -- selected item in list, always renders in the middle
		item4 = getItemByIndex(nextIndex)
		item5 = getItemByIndex(nextIndex_)
	else
		item1 = store_items_list[previousIndex_]
		item2 = store_items_list[previousIndex]
		item3 = store_items_list[storeIndex] -- selected item in list, always renders in the middle
		item4 = store_items_list[nextIndex]
		item5 = store_items_list[nextIndex_]
	end
	
	GuiStartFrame(gui_item_pics_store);
	GuiLayoutBeginVertical( gui_item_pics_store, 25, 50);
	if(previousIndex_ >= 1) then -- renders previous item only if there is one
		GuiImage( gui_item_pics_store, getNewId(), -4, 4, getItemUIPath() .. item1.pic, 1, 0.5)
	end
	if(previousIndex >= 1) then -- renders previous item only if there is one
		GuiImage( gui_item_pics_store, getNewId(), -4, 4, getItemUIPath() .. item2.pic, 1, 0.5)
	end
	-- renders main item pic
	GuiImage( gui_item_pics_store, getNewId(), -4, 4, getItemUIPath() .. item3.pic, 1, 1)
	if(nextIndex <= #store_items_list) then -- renders next item only if there is one
		GuiImage( gui_item_pics_store, getNewId(), -4, 4, getItemUIPath() .. item4.pic, 1, 0.5)
	end
	if(nextIndex_ <= #store_items_list) then -- renders next item only if there is one
		GuiImage( gui_item_pics_store, getNewId(), -4, 4, getItemUIPath() .. item5.pic, 1, 0.5)
	end
	GuiLayoutEnd( gui_item_pics_store );
	
	local selected_item_price = 0
	if(relic_store_discount_flag) then
		selected_item_price = item3.price / 2
	else
		selected_item_price = item3.price
	end
	
	-- item3 (selected) description and requirement
	GuiStartFrame(gui_utility_items_store);
	GuiLayoutBeginVertical( gui_utility_items_store, 22, 50);
	GuiText( gui_utility_items_store, 40, 2, item3.ui_name .. "\n" .. item3.desc .. "\nPrice: " .. tostring(selected_item_price) .. " gold")
	GuiLayoutEnd(gui_utility_items_store)
	
	-- nav down button and click action
	local right_triggered = ComponentGetValueInt( cc_id, "mButtonFrameChangeItemR" ) == GameGetFrameNum()
	GuiStartFrame(gui_next_utility_store);
	GuiLayoutBeginVertical( gui_next_utility_store, 25, 66);
	if(GuiImageButton( gui_next_utility_store, getNewId(), -4, 0, "", getGUIIconsPath() .. "nav_down_icon.png" ) or right_triggered) then
		storeIndex = storeIndex + 1
		if(storeIndex >= #store_items_list) then
			storeIndex = 1
		end
	end
	GuiTooltip(gui_next_utility_store, "Click to navigate down", "")
	GuiLayoutEnd( gui_next_utility_store );
	
	-- buy button and click action
	GuiStartFrame(gui_buy_utility_store);
	GuiLayoutBeginVertical( gui_buy_utility_store, 25, 71);
	if(GuiImageButton( gui_buy_utility_store, getNewId(), -4, 0, "", getGUIIconsPath() .. "buy_icon.png" )) then
		-- buys item based on which is selected in the store, item3 in this case
		buyUtilityItem(selected_item_price, getUtilityItemsBaseDir() .. item3.name, item3.ui_name, utility_store_id)
	end
	GuiTooltip(gui_buy_utility_store, "Click to buy " .. item3.ui_name .. " for " .. selected_item_price .. " gold", "")
	GuiLayoutEnd( gui_buy_utility_store );
	
end

if gui_utility_items_store then 
	async_loop(
		function() 
			renderUtilityStoreGUI()
			wait(0)
		end
	)
end