dofile_once("data/scripts/lib/coroutines.lua")
dofile_once("mods/ArcaneReliquary/files/scripts/lists/relics_list.lua")
dofile_once("mods/ArcaneReliquary/files/scripts/lists/relic_precursor_materials_list.lua")
dofile_once("mods/ArcaneReliquary/files/scripts/utilities/arcanereliquary_utilities.lua")

-- Control variables
local main_id = 1
local workbenchIndex = 3
local how_to_text = 
[[
Smelt relic metal chunks or relics by throwing them into the furnace, then
pour any type of precursor material into it. A list of relics you can
construct based on the amount of materials present in this workbench
will appear as soon as there are enough materials for construction.
Select the relic you want to construct from the list, then click the
hammer button to construct and spawn it. This will cause materials
to be consumed according to construction budgets.
]]

function getNewId()
	return main_id + 1
end

-- gui elements
local gui_workbench_precursor_text = gui_workbench_precursor_text or GuiCreate()
local gui_workbench_relic_metal = gui_workbench_relic_metal or GuiCreate()
local gui_workbench_budgets = gui_workbench_budgets or GuiCreate()

local gui_next_relic = gui_next_relic or GuiCreate()
local gui_previous_relic = gui_previous_relic or GuiCreate()
local gui_relic_pics = gui_relic_pics or GuiCreate()
local gui_relics_text = gui_relics_text or GuiCreate()
local gui_construct_relic = gui_construct_relic or GuiCreate()

local gui_workbench_howto = gui_workbench_howto or GuiCreate()


function renderWorkbenchGUI()

	-- doesn't render gui if player inventory is open, or if Inventory Bags inventory is currently open or if no prize machines in range
	local relic_workbench_id = isPlayerInRangeofMachine("relic_workbench", 30)
	if(isInventoryOpen() or isInventoryBagsOpen() or (relic_workbench_id == nil)) then
		--GlobalsSetValue("ARCANERELIQUARY_RENDER_MAIN_GUI_FLAG", "1") -- machine gui can't render, main gui can render again
		return 
	end
	
	-- main gui is hidden when this gui is showing
	--GlobalsSetValue("ARCANERELIQUARY_RENDER_MAIN_GUI_FLAG", "0")
	
	-- obtaining info about construction budgets and main construction material
	local relic_material_base_construct_cost, relic_precursor_material_construct_cost = getRelicConstructCosts()
	local main_construction_material = getRelicConstructionMaterialName()
	
	-- obtaining list of precursor materials for later use
	local relic_precursor_materials_list = getRelicPrecursorMaterialsList()
	
	-- relic metal rendering
	local main_amount = getMaterialAmountOnEntityInventory(relic_workbench_id, main_construction_material)
	
	GuiStartFrame(gui_workbench_relic_metal);
	GuiLayoutBeginHorizontal( gui_workbench_relic_metal, 30, 60);
	GuiImage( gui_workbench_relic_metal, getNewId(), -4, 0, "mods/ArcaneReliquary/files/other_items_gfx/relic_metal_chunk.png", 1, 1)
	GuiText( gui_workbench_relic_metal, 5, 0, "Relic metal in this workbench: ")
	GuiText( gui_workbench_relic_metal, 5, 0, main_amount)
	GuiLayoutEnd( gui_workbench_relic_metal );
	
	-- precursor materials rendering settings
	-- initial text
	GuiStartFrame(gui_workbench_precursor_text);
	GuiLayoutBeginVertical( gui_workbench_precursor_text, 23, 65);
	GuiText( gui_workbench_precursor_text, 40, 2, "Precursor materials in this workbench:")
	GuiLayoutEnd( gui_workbench_precursor_text );
	
	-- rendering settings
	local column_1 = 30
	local column_2 = 60
	local initial_height = 70
	local column_marker = 7
	local row_count = 0
	local x = 0
	local y = initial_height
	local heigh_reset = false
	
	-- lists used to determine which relics can be constructed
	local constructable_relic_precursor_types = {}
	local constructable_relics = {}
	
	for i,item in ipairs(relic_precursor_materials_list) do
		
		if(row_count >= column_marker) then
			x = column_2
			if(not heigh_reset) then
				y = initial_height
				heigh_reset = true
			end
		else
			x = column_1
		end
		
		local gui_precursor_material = GuiCreate()
		
		local precursor_amount = getMaterialAmountOnEntityInventory(relic_workbench_id, item.precursor_material_name)
		
		if(precursor_amount >= relic_precursor_material_construct_cost and main_amount >= relic_material_base_construct_cost) then
			table.insert(constructable_relic_precursor_types, item.precursor_type)
		end
	
		GuiStartFrame(gui_precursor_material);
		GuiLayoutBeginHorizontal( gui_precursor_material, x, y);
		GuiImage( gui_precursor_material, getNewId(), -4, 0, item.icon_path, 1, 1)
		GuiText( gui_precursor_material, 10, -1, item.precursor_ui_name .. ":")
		GuiText( gui_precursor_material, 2, -1, precursor_amount)
		GuiLayoutEnd( gui_precursor_material );
		
		y = y + 3
		row_count = row_count + 1
	end
	
	-- material budgets info rendering
	GuiStartFrame(gui_workbench_budgets);
	GuiLayoutBeginHorizontal( gui_workbench_budgets, 29, 56);
	GuiText( gui_workbench_budgets, 0, 0, "Budgets per relic: Relic metal - ")
	GuiText( gui_workbench_budgets, 0, 0, relic_material_base_construct_cost)
	GuiText( gui_workbench_budgets, 0, 0, "| Precursor material - ")
	GuiText( gui_workbench_budgets, 0, 0, relic_precursor_material_construct_cost)
	GuiLayoutEnd( gui_workbench_budgets );
	
	-- how to button
	GuiStartFrame(gui_workbench_howto);
	GuiLayoutBeginVertical( gui_workbench_howto, 30, 51);
	GuiImage( gui_workbench_howto, getNewId(), -4, 0, getGUIIconsPath() .. "howto_icon.png", 1, 1)
	GuiTooltip(gui_workbench_howto, how_to_text, "")
	GuiLayoutEnd( gui_workbench_howto );
	
	
	-- constructable relics list
	-- first, builds list of all constructable relics based on amount of materials present
	for i,precursor_type in ipairs(constructable_relic_precursor_types) do
		for i,relic in ipairs(relics_list) do
			if(string.find(relic.precursor_types, precursor_type)) then
				table.insert(constructable_relics, relic)
			end
		end
	end
	
	-- stops rendering if no constructable relics
	if(#constructable_relics == 0) then return end
	
	-- controls
	-- obtaining controls component for later use
	local cc_id = EntityGetFirstComponent(getPlayerEntity(), "ControlsComponent")
	
	-- nav previous button and click action
	local left_triggered = ComponentGetValueInt( cc_id, "mButtonFrameChangeItemL" ) == GameGetFrameNum()
	GuiStartFrame(gui_previous_relic);
	GuiLayoutBeginVertical( gui_previous_relic, 59, 63);
	if(GuiImageButton( gui_previous_relic, getNewId(), -4, 0, "", getGUIIconsPath() .. "nav_previous_icon.png" ) or left_triggered) then
		workbenchIndex = workbenchIndex - 1
		if(workbenchIndex <= 0) then
			workbenchIndex = #constructable_relics
		end
	end
	GuiTooltip(gui_previous_relic, "Click to show previous constructable relic", "")
	GuiLayoutEnd( gui_previous_relic );
	
	-- nav next button and click action
	local right_triggered = ComponentGetValueInt( cc_id, "mButtonFrameChangeItemR" ) == GameGetFrameNum()
	GuiStartFrame(gui_next_relic);
	GuiLayoutBeginVertical( gui_next_relic, 70, 63);
	if(GuiImageButton( gui_next_relic, getNewId(), -4, 0, "", getGUIIconsPath() .. "nav_next_icon.png" ) or right_triggered) then
		workbenchIndex = workbenchIndex + 1
		if(workbenchIndex >= #constructable_relics) then
			workbenchIndex = 1
		end
	end
	GuiTooltip(gui_next_relic, "Click to show next constructable relic", "")
	GuiLayoutEnd( gui_next_relic );
	
	-- list of contructable relics
	-- indexes
	local previousIndex_ = workbenchIndex - 2
	local previousIndex = workbenchIndex - 1
	local nextIndex = workbenchIndex + 1
	local nextIndex_ = workbenchIndex + 2
	
	-- renders always five relics based on current index
	local relic1 = nil
	local relic2 = nil
	local relic3 = nil
	local relic4 = nil
	local relic5 = nil
	
	relic1 = constructable_relics[previousIndex_]
	relic2 = constructable_relics[previousIndex]
	relic3 = constructable_relics[workbenchIndex] -- selected relic in list, always renders in the middle
	relic4 = constructable_relics[nextIndex]
	relic5 = constructable_relics[nextIndex_]
	
	GuiStartFrame(gui_relic_pics);
	GuiLayoutBeginHorizontal( gui_relic_pics, 60, 64);
	if(previousIndex_ >= 1) then -- renders previous relic only if there is one
		GuiImage( gui_relic_pics, getNewId(), 1, 0, getItemUIPath() .. relic1.name .. ".png", 1, 0.5)
	end
	if(previousIndex >= 1) then -- renders previous relic only if there is one
		GuiImage( gui_relic_pics, getNewId(), 1, 0, getItemUIPath() .. relic2.name .. ".png", 1, 0.5)
	end
	-- renders main relic pic
	GuiImage( gui_relic_pics, getNewId(), 2, 0, getItemUIPath() .. relic3.name .. ".png", 1, 1)
	GuiTooltip(gui_relic_pics, relic3.ui_name, "")
	if(nextIndex <= #constructable_relics) then -- renders next relic only if there is one
		GuiImage( gui_relic_pics, getNewId(), 1, 0, getItemUIPath() .. relic4.name .. ".png", 1, 0.5)
	end
	if(nextIndex_ <= #constructable_relics) then -- renders next relic only if there is one
		GuiImage( gui_relic_pics, getNewId(), 1, 0, getItemUIPath() .. relic5.name .. ".png", 1, 0.5)
	end
	GuiLayoutEnd( gui_relic_pics );
	
	-- construct relic button and click action
	GuiStartFrame(gui_construct_relic);
	GuiLayoutBeginVertical( gui_construct_relic, 72, 63);
	if(GuiImageButton( gui_construct_relic, getNewId(), -4, 0, "", getGUIIconsPath() .. "construct_relic.png" )) then
		GamePrint("Relic construction started: " .. relic3.ui_name)
		-- sets workbench variables to signal it to spawn selected relic
		setInternalVariableValue(relic_workbench_id, "construction_enabled", "value_bool", true)
		setInternalVariableValue(relic_workbench_id, "relic_to_construct_name", "value_string", relic3.name)
	end
	GuiTooltip(gui_construct_relic, "Click to construct the relic you selected", "")
	GuiLayoutEnd( gui_construct_relic );
	
	-- info text
	GuiStartFrame(gui_relics_text);
	GuiLayoutBeginVertical( gui_relics_text, 59, 60);
	GuiText( gui_relics_text, 1, 0, "Relics you can construct:")
	GuiLayoutEnd( gui_relics_text );

end


async_loop(
	function() 
		renderWorkbenchGUI()
		wait(0)
	end
)
