dofile_once("data/scripts/lib/utilities.lua")
dofile_once("mods/ArcaneReliquary/files/scripts/utilities/arcanereliquary_utilities.lua")
dofile_once("mods/ArcaneReliquary/files/scripts/lists/relics_list.lua")
dofile_once("mods/ArcaneReliquary/files/scripts/lists/custom_effects_list.lua")

function item_pickup( entity_item, entity_who_picked, item_name )

	
	-- check to prevent Nemesis enemy from New Enemies Mod from stealing relics
	if(not entityHasInternalVariableValue(entity_who_picked, "arcane_reliquary_item_interact_flag")) then return end

	local internalName = EntityGetName(entity_item)
	local is_active = getInternalVariableValue(entity_item, "is_active", "value_bool")
	local x, y = EntityGetTransform( entity_item )
	
	local message = ""
	local message_secondary = ""
	local collected_relic = getRelic(internalName)
	
	-- creates new entity that will later be added as a child on the player
	local player_child_entity_id = EntityCreateNew(collected_relic.name)
	
	
	-- first of all, checks if full inventory is full, if it is, prevents collection of relic
	-- # this check is no longer necessary, relics no longer make use of built-in inventory system
	--[[
	if(isFullInventoryFull()) then
		message = "You cannot pick up this relic"
		message_secondary = "Your extended/spells inventory is full"
		
		relicPickupError(entity_item, message, message_secondary, collected_relic.name, x, y)
		return
	end
	]]
	
	-- adds flag to child item on player to indicate relic's is_active status
	if(not entityHasInternalVariableValue(player_child_entity_id, "is_active")) then
		addNewInternalVariable(player_child_entity_id, "is_active", "value_bool", is_active)
	else
		setInternalVariableValue(player_child_entity_id, "is_active", "value_bool", is_active)
	end
	
	-- then, verifies if collected relic excludes any that's already owned
	if(collected_relic.excludes_relics_tags ~= nil) then
		local excluded_relic = false
		local excluded_relic_text = ""
		local owned_relics = getOwnedRelics()
		
		
		
		if(not string.find(collected_relic.excludes_relics_tags, ",")) then -- when list contains just one tag
			for i,owned_relic_id in ipairs(owned_relics) do
				local owned_relic = getRelic(EntityGetName(owned_relic_id))
				if(owned_relic.excludes_relics_tags ~= nil and (string.find(owned_relic.excludes_relics_tags, collected_relic.excludes_relics_tags))) then
					
					--GamePrint(">>> owned relic tag: " .. owned_relic.excludes_relics_tags )
					--GamePrint(">>> collected relic tag: " .. collected_relic.excludes_relics_tags )
					
					excluded_relic_text = owned_relic.excludes_relics_desc
					excluded_relic = true
					break
				end
			end
		else -- when list contains more than one tag, in a comma separated list
			local excluded_relics_tags_list = {}
			for excluded_relics_tag in string.gmatch(collected_relic.excludes_relics_tags, "([^,]+)") do
				table.insert(excluded_relics_tags_list, excluded_relics_tag)
			end
			for i,excluded_relic_tag_each in ipairs(excluded_relics_tags_list) do
				--GamePrint(">>> checking tag: " .. excluded_relic_tag_each )
				for i,owned_relic_id in ipairs(owned_relics) do
					local owned_relic = getRelic(EntityGetName(owned_relic_id))
					if(owned_relic.excludes_relics_tags ~= nil and (string.find(owned_relic.excludes_relics_tags, excluded_relic_tag_each))) then
						excluded_relic_text = owned_relic.excludes_relics_desc
						excluded_relic = true
						break
					end
				end
			end
		end
		
		if(excluded_relic) then
			message = "You cannot collect " .. collected_relic.ui_name
			message_secondary = excluded_relic_text
			
			relicPickupError(entity_item, player_child_entity_id, message, message_secondary, collected_relic.name, x, y)
			return
		end
	end
	--[[
	if(collected_relic.excludes_relics ~= nil) then
		local excluded_relics = collected_relic.excludes_relics
		if(not string.find(excluded_relics, ",")) then -- when list contains just one relic
			if(isRelicOwned(excluded_relics) > 0) then
				message = "You cannot collect " .. collected_relic.ui_name
				message_secondary = "It is incompatible with " .. getRelic(excluded_relics).ui_name
				
				relicPickupError(entity_item, player_child_entity_id, message, message_secondary, collected_relic.name, x, y)
				return
			end
		else -- when list contains multiple relics
			local exclude_relics_list = {}
			for excluded_relic in string.gmatch(excluded_relics, "([^,]+)") do
				table.insert(exclude_relics_list, excluded_relic)
			end
			for i,excluded_relic_each in ipairs(exclude_relics_list) do
				if(isRelicOwned(excluded_relic_each) > 0) then
					message = "You cannot collect " .. collected_relic.ui_name
					message_secondary = "It is incompatible with " .. getRelic(excluded_relic_each).ui_name
					
					relicPickupError(entity_item, player_child_entity_id, message, message_secondary, collected_relic.name, x, y)
					return
				end
			end
		end
	end
	]]

	local maximum_owned_relics = getMaximumOwnedRelics()
	local total_owned_relics = countTotalOwnedRelics() + 1 -- adding one to the total is necessary due to timing issues with the pickup callback
	local times_owned = isRelicOwned(collected_relic.name) + 1 -- same here
	
	--GamePrint("### Currently owned relics: " .. total_owned_relics .. " / " .. maximum_relics)
	
	if(total_owned_relics > maximum_owned_relics) then
		message = "You cannot own more than " .. maximum_owned_relics .. " relics at the same time"
		message_secondary = "You have to drop one before colleting another"
		
		relicPickupError(entity_item, player_child_entity_id, message, message_secondary, collected_relic.name, x, y)
		return
	--elseif((not collected_relic.is_stackable) and times_owned > 1) then
	elseif(times_owned > 1) then
		--message = collected_relic.ui_name .. " is not stackable and you already own one"
		message = collected_relic.ui_name .. " is already owned"
		message_secondary = ""
		
		relicPickupError(entity_item, player_child_entity_id, message, message_secondary, collected_relic.name, x, y)
		return
	end
	
	-- checks whether relic has upgrade levels, then get it to add it to the ui name
	local relic_level = ""
	local item_level = 0
	if(entityHasInternalVariableValue(entity_item, "relic_level")) then
		item_level = getInternalVariableValue(entity_item, "relic_level", "value_int")
		relic_level = " " .. levelNumbertoRoman(item_level)
		
		-- adds level to child item on player if the required variable is not there yet
		if(not entityHasInternalVariableValue(player_child_entity_id, "relic_level")) then
			addNewInternalVariable(player_child_entity_id, "relic_level", "value_int", item_level)
		else
			setInternalVariableValue(player_child_entity_id, "relic_level", "value_int", item_level)
		end
	end
	
	-- if wisdom_medallion_effect is active, chooses a random upgrade level for the relic
	-- pre-requisites are
	-- - relic must be listed as upgradable
	-- - relic must not contain levels yet
	-- - relic must be enabled
	if(collected_relic.is_upgradable and entityHasSimplifiedInternalVariable(getPlayerEntity(), "wisdom_medallion_effect") and not entityHasInternalVariableValue(entity_item, "relic_level") and getInternalVariableValue(entity_item, "is_active", "value_bool")) then
		SetRandomSeed( GameGetFrameNum(), x + y + entity_item)
		if(Random(1,2) == 1) then
			local level = Random(1,5)
			--GamePrint("### new level applied: " .. level)
			--setRelicLevel(collected_relic, entity_item, level)
			
			-- adds level to child item on player if the required variable is not there yet
			if(not entityHasInternalVariableValue(player_child_entity_id, "relic_level")) then
				addNewInternalVariable(player_child_entity_id, "relic_level", "value_int", level)
			else
				setInternalVariableValue(player_child_entity_id, "relic_level", "value_int", level)
			end
		end
	end	
	
	-- adds variable to relic to indicate it can now be sold when dropped
	--[[
	if(not entityHasInternalVariableValue(entity_item, "can_be_sold")) then
		addNewInternalVariable(entity_item, "can_be_sold", "value_bool", true)
	end
	]]
	-- adds flag to child item on player to indicate it can be sold once dropped
	if(not entityHasInternalVariableValue(player_child_entity_id, "can_be_sold")) then
		addNewInternalVariable(player_child_entity_id, "can_be_sold", "value_bool", true)
	else
		setInternalVariableValue(player_child_entity_id, "can_be_sold", "value_bool", true)
	end
	
	message = "You got " .. collected_relic.ui_name .. relic_level
	
	GamePrint(message)
	GamePrintImportant(message, collected_relic.effect_desc, "mods/ArcaneReliquary/files/other_gfx/custom_decoration_relic.png")
	EntityLoad("mods/ArcaneReliquary/files/entity_definitions/others/relic_pickup_effect.xml", x, y)
	
	-- runs pickup function to apply any initial effects necessary according to which relic has been picked up
	if(collected_relic.onpickup_function ~= nil and is_active) then
		collected_relic.onpickup_function()
	end
	
	-- relic is tagged than added as child on the player
	EntityAddTag( player_child_entity_id, "relic" )
	EntityAddChild( entity_who_picked, player_child_entity_id )
	-- kills physical item
	EntityKill(entity_item)
end

function relicPickupError(entity_item, player_child_entity_id, main_message, secondary_message, relic_name, x, y)
	-- audiovisual feedback
	GamePrint(main_message)
	GamePrintImportant(main_message, secondary_message, "mods/ArcaneReliquary/files/other_gfx/custom_decoration_error.png" )
	EntityLoad("data/entities/particles/image_emitters/chest_effect_bad.xml", x, y)
	-- clean up
	EntityKill(entity_item)
	EntityKill(player_child_entity_id)
	-- spawns relic back
	EntityLoad(getRelicsBaseDir() .. relic_name .. ".xml", x, y)
end