dofile_once("mods/ArcaneReliquary/files/scripts/utilities/arcanereliquary_utilities.lua")

local entity_id = GetUpdatedEntityID()

if(entityHasSimplifiedInternalVariable(entity_id, "minion")) then return end -- check to prevent buffs from being applied on player spawned minions
-- this script is supposed to run just after enemy_apply_random_buffs.lua, so it assumes the creature might've received a buff
if(entityHasSimplifiedInternalVariable(entity_id, "relic_chest_buff")) then return end -- if creature has received a buff, skips everything below
if(entityHasSimplifiedInternalVariable(entity_id, "coxas_item_drop_tag")) then return end -- if creature is already carrying a special item, skips everything below

local x, y = EntityGetTransform(entity_id)

SetRandomSeed( GameGetFrameNum(), x + y + entity_id )

-- icon offsets
local icon_middle = 3

local odds = tonumber(ModSettingGet("ArcaneReliquary.special_item_drop_chance")) -- eligibility odds obtained from settings.lua file

local elligible = Random(1,odds) -- regulates how often enemies are elligible to drop upgrade/forge material containers and room spawners

if(elligible == 1) then
	
	-- checks whether No Buffs mode is enabled
	local item_type_odds = 4
	
	local item_type = Random(1,item_type_odds)
	if(item_type == 1) then -- upgrade related items
		addBuffIcon(entity_id, "item_indicator_upgrade_compound", icon_middle)
		addNewSimplifiedInternalVariable(entity_id, "coxas_item_drop_tag") -- tags enemy generically to allow other mods to detect this
		addNewSimplifiedInternalVariable(entity_id, "upgrade_item_drop_chance") -- tags enemy so that other scripts will have access to this info
		-- adds guaranteed upgrade item drop callback
		addNewLuaCallback(entity_id, "on_death", "mods/ArcaneReliquary/files/scripts/other_listeners/enemy_death_guaranteed_upgrade_item_drop.lua", 1, true, "enemy_death_guaranteed_upgrade_item_drop")
	elseif(item_type == 2) then -- relic construct related items
		addBuffIcon(entity_id, "item_indicator_precursor", icon_middle)
		addNewSimplifiedInternalVariable(entity_id, "coxas_item_drop_tag") -- tags enemy generically to allow other mods to detect this
		addNewSimplifiedInternalVariable(entity_id, "construct_item_drop_chance") -- tags enemy so that other scripts will have access to this info
		-- adds guaranteed upgrade item drop callback
		addNewLuaCallback(entity_id, "on_death", "mods/ArcaneReliquary/files/scripts/other_listeners/enemy_death_guaranteed_construct_item_drop.lua", 1, true, "enemy_death_guaranteed_construct_item_drop")
	elseif(item_type == 3) then -- relic construct related items
		addBuffIcon(entity_id, "item_indicator_room_spawner", icon_middle)
		addNewSimplifiedInternalVariable(entity_id, "coxas_item_drop_tag") -- tags enemy generically to allow other mods to detect this
		addNewSimplifiedInternalVariable(entity_id, "room_spawner_item_drop_chance") -- tags enemy so that other scripts will have access to this info
		-- adds guaranteed room spawner item drop callback
		addNewLuaCallback(entity_id, "on_death", "mods/ArcaneReliquary/files/scripts/other_listeners/enemy_death_guaranteed_room_spawner_drop.lua", 1, true, "enemy_death_guaranteed_room_spawner_drop")
	elseif(item_type == 4) then -- upgrade stone, chest keys and prize chip items
		local other_items_odds = Random(1,3)
		if(other_items_odds == 1) then
			addBuffIcon(entity_id, "item_indicator_upgrade_stone", icon_middle)
			addNewSimplifiedInternalVariable(entity_id, "coxas_item_drop_tag") -- tags enemy generically to allow other mods to detect this
			addNewSimplifiedInternalVariable(entity_id, "upgrade_stone_drop_chance") -- tags enemy so that other scripts will have access to this info
			-- adds guaranteed upgrade stone item drop callback
			addNewLuaCallback(entity_id, "on_death", "mods/ArcaneReliquary/files/scripts/other_listeners/enemy_death_guaranteed_upgrade_stone_drop.lua", 1, true, "enemy_death_guaranteed_upgrade_stone_drop")
		end
		if(other_items_odds == 2) then
			addBuffIcon(entity_id, "item_indicator_prize_chip", icon_middle)
			addNewSimplifiedInternalVariable(entity_id, "coxas_item_drop_tag") -- tags enemy generically to allow other mods to detect this
			addNewSimplifiedInternalVariable(entity_id, "prize_chip_drop_chance") -- tags enemy so that other scripts will have access to this info
			-- adds guaranteed prize chip item drop callback
			addNewLuaCallback(entity_id, "on_death", "mods/ArcaneReliquary/files/scripts/other_listeners/enemy_death_guaranteed_prize_chip_drop.lua", 1, true, "enemy_death_guaranteed_prize_chip_drop")
		end
		if(other_items_odds == 3) then
			addBuffIcon(entity_id, "item_indicator_key", icon_middle)
			addNewSimplifiedInternalVariable(entity_id, "coxas_item_drop_tag") -- tags enemy generically to allow other mods to detect this
			addNewSimplifiedInternalVariable(entity_id, "chest_key_drop_chance") -- tags enemy so that other scripts will have access to this info
			-- adds guaranteed prize chip item drop callback
			addNewLuaCallback(entity_id, "on_death", "mods/ArcaneReliquary/files/scripts/other_listeners/enemy_death_guaranteed_chest_key_drop.lua", 1, true, "enemy_death_guaranteed_chest_key_drop")
		end
	end

end

