dofile_once("mods/ArcaneReliquary/files/scripts/utilities/arcanereliquary_utilities.lua")
dofile_once("mods/ArcaneReliquary/files/scripts/lists/relics_list.lua")

-- obtains id and coordinates
local entity_id = GetUpdatedEntityID()
local x, y = EntityGetTransform(entity_id)

-- middle plate coordinates
local middle_plate_x = x
local middle_plate_y = y - 20
-- left plate coordinates
local left_plate_x = x - 25
local left_plate_y = y - 15
-- right plate coordinates
local right_plate_x = x + 25
local right_plate_y = y - 15

-- obtains names of all relics within check range
local relics_in_range_names = {}
local relics_in_range = EntityGetInRadiusWithTag(middle_plate_x, middle_plate_y, 13, "relic")

for i, relic_in_range_id in ipairs(relics_in_range) do
	local relic = getRelic(EntityGetName(relic_in_range_id))
	
	local relic_level = 0
	if(entityHasInternalVariableValue(relic_in_range_id, "relic_level")) then
		relic_level = getInternalVariableValue(relic_in_range_id, "relic_level", "value_int")
	end
	
	if(relic_level >= 2) then
		-- obtains list of precursor types associated to this relic
		local precursor_types_list = {}
		for precursor_type in string.gmatch(relic.precursor_types, "[^,]+") do
			table.insert(precursor_types_list, precursor_type)
		end
		
		-- chooses one precursor type in list
		SetRandomSeed( GameGetFrameNum(), math.random(1,50))
		local precursor_type_chosen = precursor_types_list[Random(1,#precursor_types_list)]
		
		-- obtains list of relics that match chosen precursor type
		local eligible_relics = getRelicsByPrecursorType(precursor_type_chosen)
		
		-- kills base relic
		local base_relic_x, base_relic_y = EntityGetTransform(relic_in_range_id)
		EntityKill(relic_in_range_id)
		EntityLoad("mods/ArcaneReliquary/files/entity_definitions/particle_explosions/relic_consume_explosion.xml", base_relic_x, base_relic_y)
		
		-- spawns two relics based on chosen precursor type
		GamePlaySound( "data/audio/Desktop/projectiles.snd", "player_projectiles/critical_hit/create", x, y )
		
		EntityLoad("mods/ArcaneReliquary/files/entity_definitions/particle_explosions/relic_transmute_explosion.xml", right_plate_x, right_plate_y)
		local new_relic_1_id = EntityLoad(getRelicsBaseDir() .. eligible_relics[Random(1,#eligible_relics)].name .. ".xml", right_plate_x, right_plate_y)
		setRelicLevel(getRelic(EntityGetName(new_relic_1_id)), new_relic_1_id, math.floor(relic_level / 2))
		
		EntityLoad("mods/ArcaneReliquary/files/entity_definitions/particle_explosions/relic_transmute_explosion.xml", left_plate_x, left_plate_y)
		local new_relic_2_id = EntityLoad(getRelicsBaseDir() .. eligible_relics[Random(1,#eligible_relics)].name .. ".xml", left_plate_x, left_plate_y)
		setRelicLevel(getRelic(EntityGetName(new_relic_2_id)), new_relic_2_id, math.ceil(relic_level / 2))	
	end
	
end

