dofile_once("mods/ArcaneReliquary/files/scripts/lists/relics_list.lua")
dofile_once("mods/ArcaneReliquary/files/scripts/utilities/arcanereliquary_utilities.lua")

function item_pickup( entity_item, entity_who_picked, item_name )

	-- check to prevent Nemesis enemy from New Enemies Mod from openning chests
	if(not entityHasInternalVariableValue(entity_who_picked, "arcane_reliquary_item_interact_flag")) then return end

	local x, y = EntityGetTransform( entity_item )
	local chest_name = EntityGetName(entity_item)
	SetRandomSeed( GameGetFrameNum(), x + y + entity_item )
	
	
	-- spawned relic type is determined by the chest type
	local relic = nil
	if(chest_name == "relic_chest") then -- regular relic chests spawn only basic relics
		relic = getRandomBasicRelic()
	elseif(chest_name == "relic_chest_great") then -- great relic chests spawn only combined exclusive relics
		relic = getRandomCombineExclusiveRelic()
	elseif(chest_name == "relic_chest_chaotic") then -- if chaotic relic chest was openned, spawns any relic from the pool
		relic = getRandomRelic()
	end
	
	-- loads chosen relic
	local relic_id = EntityLoad(getRelicsBaseDir() .. relic.name .. ".xml", x, y)
	
	-- if chaotic relic chest was openned and spawned relic is upgradable, applies random level to it
	if(isRelicUpgradable(relic.name) and chest_name == "relic_chest_chaotic") then
		setRelicLevel(relic, relic_id, Random(1,getRelicMaxUpgradeLevel()))
	end
	
	-- if spawned relic requires another relic, there's a 50% chance it will also spawn
	if(relic.requires_relic ~= nil) then
		if(Random(1,2) == 1) then
			local required_relic = getRelic(relic.requires_relic)
			EntityLoad(getRelicsBaseDir() .. required_relic.name .. ".xml", x, y - 5)
		end
	end

	-- settings for each compatible chest type
	local message = ""
	local text_decoration = ""
	local emitter_path = ""
	if(chest_name == "relic_chest") then
		message = "You opened a Relic Chest!"
		text_decoration = "mods/ArcaneReliquary/files/other_gfx/custom_decoration_chest.png"
		emitter_path = "mods/ArcaneReliquary/files/entity_definitions/image_emitters/chest_effect.xml"
	elseif(chest_name == "relic_chest_great") then
		message = "You opened a Great Relic Chest!"
		text_decoration = "mods/ArcaneReliquary/files/other_gfx/custom_decoration_chest_great.png"
		emitter_path = "mods/ArcaneReliquary/files/entity_definitions/image_emitters/chest_effect_great.xml"
	elseif(chest_name == "relic_chest_chaotic") then
		message = "You opened a Chaotic Relic Chest!"
		text_decoration = "mods/ArcaneReliquary/files/other_gfx/custom_decoration_chest_chaotic.png"
		emitter_path = "mods/ArcaneReliquary/files/entity_definitions/image_emitters/chest_effect_chaotic.xml"
	end
	
	-- gives user feedback according to settings above
	GamePrint(message)
	GamePrintImportant(message, "Random relic dropped", text_decoration)
	EntityLoad(emitter_path, x, y)
	EntityKill(entity_item)
	
	-- matrioshka relic effect
	if(entityHasSimplifiedInternalVariable(entity_who_picked, "matrioshka")) then
		local extra_chest_odds = Random(1,2)
		local chest_type = Random(1,3)
		if(extra_chest_odds == 1) then
			if(chest_type == 1) then
				relic_chest_name = "relic_chest"
			elseif(chest_type == 2) then
				relic_chest_name = "relic_chest_great"
			elseif(chest_type == 3) then
				relic_chest_name = "relic_chest_chaotic"
			end
			EntityLoad("mods/ArcaneReliquary/files/entity_definitions/props/" .. relic_chest_name .. ".xml", x, y)
		end
	end
end