dofile_once("mods/ArcaneReliquary/files/scripts/utilities/arcanereliquary_utilities.lua")
dofile_once("mods/ArcaneReliquary/files/scripts/lists/relics_list.lua")

-- obtains id and coordinates
local entity_id = GetUpdatedEntityID()
local x, y = EntityGetTransform(entity_id)

-- unlocks player inventories while in close vicinity
--unlockPlayerInventories(x, y, 40, 65)

-- obtains names of all relics within check range
local relics_in_range_names = {}
local relics_in_range = EntityGetInRadiusWithTag(x, y - 15, 35, "relic")

if(#relics_in_range == 0) then return end -- interrupts check if no relics are present

for i, relic_in_range_id in ipairs(relics_in_range) do
	if(not isItemDropped(relic_in_range_id)) then -- if any detected item is still on the player's inventory, interrupts check
		GamePrint("You currently have relics in your inventory, drop them or step away from the machine to allow it to work")
		return 
	end 
end

--GamePrint("### combine station check")

for i, relic_in_range_id in ipairs(relics_in_range) do
	table.insert(relics_in_range_names, EntityGetName(relic_in_range_id))
end

-- obtains a list with all combine exclusive relics
local combine_exclusive_relics = getCombineExclusiveRelics()

-- iterates through combine exclusive relics
for i, combine_exclusive_relic in ipairs(combine_exclusive_relics) do

	-- for each combine exclusive relic, obtains list of required relics
	local combine_requires_relics = combine_exclusive_relic.combine_requires_relics
	local required_relics_names = {}
	for required_relic in string.gmatch(combine_requires_relics, "([^,]+)") do
		table.insert(required_relics_names, required_relic)
	end
	
	-- iterates through required relics and relics in range in order to create a new table containing elements in common between them
	local relics_in_common = {}
	for k1,required_relic_name in pairs(required_relics_names) do
		for k2,relic_in_range_name in pairs(relics_in_range_names) do
			if required_relic_name == relic_in_range_name then
				-- checks relics_in_common before adding current relic, to prevent possible duplication bugs
				local relic_is_present = false
				for i,relic_in_common in pairs(relics_in_common) do
					if(relic_in_common == required_relic_name) then
						relic_is_present = true
						break
					end
				end
				-- if current relic is not present, then adds it to relics_in_common
				if(not relic_is_present) then
					table.insert(relics_in_common, required_relic_name)
				end
			end
		end
	end
	
	-- compares number of elements of relics in common with required relics to check whether the number of elements of both is the same
	-- if so, it means all required relics are present
	if(#relics_in_common == #required_relics_names) then
		
		--[[
		print("### combination found: " .. table.concat(required_relics_names,", "))
		print("### relics in common: " .. table.concat(relics_in_common,", "))
		print("### relics in range: " .. table.concat(relics_in_range_names,", "))
		]]
		
		local new_relic_level = 0
		local old_relic_levels = {}
		-- kills found relics
		for i, relic_in_range_id in ipairs(relics_in_range) do
			-- check to make sure only required relics are destroyed
			local can_be_killed = false
			for i, required_relic_name in ipairs(required_relics_names) do
				if(required_relic_name == EntityGetName(relic_in_range_id)) then
					can_be_killed = true
					break
				end
			end
			if(can_be_killed) then
				-- obtains levels of old relics
				if(entityHasInternalVariableValue(relic_in_range_id, "relic_level")) then
					table.insert(old_relic_levels, getInternalVariableValue(relic_in_range_id, "relic_level", "value_int"))
				end
				local relic_in_range_x, relic_in_range_y = EntityGetTransform(relic_in_range_id)
				EntityLoad("mods/ArcaneReliquary/files/entity_definitions/particle_explosions/relic_create_explosion.xml", relic_in_range_x, relic_in_range_y)
				GamePlaySound( "data/audio/Desktop/explosions.snd", "explosions/magic_grenade_small", relic_in_range_x, relic_in_range_y )
				EntityKill(relic_in_range_id)
			end
		end
		
		if(#old_relic_levels > 0 and old_relic_levels ~= nil) then
			local levels_total = 0
			for k,relic_level in pairs(old_relic_levels) do
				levels_total = levels_total + relic_level
			end
			new_relic_level = math.floor(levels_total / #old_relic_levels)
			--GamePrint("### new relic level: " .. new_relic_level)
		end
		
		-- creates new relic
		local relic_construct_entity_id = EntityLoad("mods/ArcaneReliquary/files/entity_definitions/others/relic_combine.xml", x, y - 15)
		setInternalVariableValue(relic_construct_entity_id, "relic_to_construct", "value_string", combine_exclusive_relic.name)
		setInternalVariableValue(relic_construct_entity_id, "relic_to_construct_level", "value_int", new_relic_level)
	end

	
	--[[
	local flag_array = {}
	-- iterates through required relics to check whether or not each of them is present in the list of relics detected by the station
	for i,required_relic_each in ipairs(required_relics_names) do
		-- iterates through detected relics to compare each of them with each item in the required list
		for i, relic_in_range_id in ipairs(relics_in_range) do
			if(required_relic_each == EntityGetName(relic_in_range_id))
				table.insert(flag_array, {relic_name = required_relic_each, is_present = true})
			end
		end
	end
	]]
end

