dofile_once("mods/ArcaneReliquary/files/scripts/utilities/arcanereliquary_utilities.lua")
dofile_once("mods/ArcaneReliquary/files/scripts/lists/relics_list.lua")

-- main variables
local upgrade_plate_offset_x = -14
local material_funnel_offset_x = 14
local material_funnel_offset_y = -10

-- obtains upgrade settings from relics_list.lua
local relic_material_base_upgrade_cost, relic_material_advanced_upgrade_cost = getRelicUpgradeCosts()

-- if player has material budget discount, applies it
if(entityHasSimplifiedInternalVariable(getPlayerEntity(), "upgrade_material_budget_discount")) then
	relic_material_base_upgrade_cost = relic_material_base_upgrade_cost / 2
	relic_material_advanced_upgrade_cost = relic_material_advanced_upgrade_cost / 2
end

local upgrade_material_name, relic_advanced_upgrade_material_name = getRelicUpgradeMaterialName()
local max_upgrade_level = getRelicMaxUpgradeLevel()
local basic_upgrade_level = max_upgrade_level / 2

-- obtains id and coordinates
local entity_id = GetUpdatedEntityID()
local x, y = EntityGetTransform(entity_id)

-- unlocks player inventories while in close vicinity
-- unlockPlayerInventories(x, y, 40, 65)

-- Loads unclogger field to eliminate possible obstructions on the funnel
EntityLoad("mods/ArcaneReliquary/files/entity_definitions/others/field_unclogger.xml", x + material_funnel_offset_x, y + material_funnel_offset_y)

-- obtains reference to activation indicator from forge's activation_indicator child entity
local activation_indicator_id = nil
local forge_children = EntityGetAllChildren(entity_id)
if forge_children == nil then return end
for i,child in ipairs(forge_children) do
	if(EntityGetName(child) == "activation_indicator") then
		activation_indicator_id = EntityGetFirstComponentIncludingDisabled( child, "ParticleEmitterComponent" )
	end
end
	
local x, y = EntityGetTransform(entity_id)

local amount = getMaterialAmountOnEntityInventory(entity_id, upgrade_material_name)
local amount_advanced = getMaterialAmountOnEntityInventory(entity_id, relic_advanced_upgrade_material_name)

--[[ This is no longer necessary since there are gui elements to show this info to the player
-- if the player is in range, shows amount of upgrade material currently on forge
local players_in_range = EntityGetInRadiusWithTag( x, y, 70, "player_unit" )
if(#players_in_range > 0) then
	GamePrint("Upgrade material on forge: basic - " .. amount .. " | advanced - " .. amount_advanced)
end
]]

-- turns on activation indicator if enough material for upgrades is available
if(amount >= relic_material_base_upgrade_cost or amount_advanced >= relic_material_advanced_upgrade_cost) then
	EntitySetComponentIsEnabled( entity_id, activation_indicator_id, true )
else
	EntitySetComponentIsEnabled( entity_id, activation_indicator_id, false )
end

-- relics can only be upgraded if:
-- - they are dropped on the ground
-- - they are listed as upgradable
-- - they are enabled

-- relic upgrades up to level 5
if(amount >= relic_material_base_upgrade_cost) then

	local relics_in_range = EntityGetInRadiusWithTag(x + upgrade_plate_offset_x, y, 20, "relic") -- offset so that relic can be detected at the correct place of the forge
	for i, relic_id in ipairs(relics_in_range) do
		if(isItemDropped(relic_id) and isRelicUpgradable(EntityGetName(relic_id)) and getInternalVariableValue(relic_id, "is_active", "value_bool")) then
			--GamePrint("### relic within range")
			
			-- if relic has no levels, initializes it
			if(not entityHasInternalVariableValue(relic_id, "relic_level")) then
				addNewInternalVariable(relic_id, "relic_level", "value_int", 0)
			end
			-- upgrades relic with +1 level
			local relic_level = getInternalVariableValue(relic_id, "relic_level", "value_int")
			if(relic_level < basic_upgrade_level) then
				local new_relic_level = relic_level + 1
				setRelicLevel(getRelic(EntityGetName(relic_id)), relic_id, new_relic_level)
				
				-- consumes upgrade material
				amount = amount - relic_material_base_upgrade_cost
				AddMaterialInventoryMaterial( entity_id, upgrade_material_name, amount)
				
				loadAudioVisualFeedback(relic_id)
			end
		end
	end
end

-- relic upgades from level 5 to level 10
if(amount_advanced >= relic_material_advanced_upgrade_cost) then

	local relics_in_range = EntityGetInRadiusWithTag(x + upgrade_plate_offset_x, y, 20, "relic") -- offset so that relic can be detected at the correct place of the forge
	for i, relic_id in ipairs(relics_in_range) do
		if(isItemDropped(relic_id) and isRelicUpgradable(EntityGetName(relic_id)) and getInternalVariableValue(relic_id, "is_active", "value_bool")) then
			--GamePrint("### relic within range")
			-- upgrades relic with +1 level
			local relic_level = getInternalVariableValue(relic_id, "relic_level", "value_int")
			if(relic_level >= basic_upgrade_level and relic_level < max_upgrade_level) then
				local new_relic_level = relic_level + 1
				setRelicLevel(getRelic(EntityGetName(relic_id)), relic_id, new_relic_level)
				
				-- consumes advanced upgrade material
				amount_advanced = amount_advanced - relic_material_advanced_upgrade_cost
				AddMaterialInventoryMaterial( entity_id, relic_advanced_upgrade_material_name, amount_advanced)
				
				loadAudioVisualFeedback(relic_id)
			end
		end
	end
end

function loadAudioVisualFeedback(relic_id)
	-- audio visual feedback
	local relic_x, relic_y = EntityGetTransform(relic_id)
	EntityLoad("mods/ArcaneReliquary/files/entity_definitions/others/relic_upgrade_effect.xml", relic_x, relic_y)	
end

