dofile_once("mods/ArcaneReliquary/files/scripts/utilities/arcanereliquary_utilities.lua")
dofile_once("mods/ArcaneReliquary/files/scripts/lists/relics_list.lua")
dofile_once("mods/ArcaneReliquary/files/scripts/lists/relic_precursor_materials_list.lua")

-- main variables
local craft_plate_offset_x = -14
local craft_plate_offset_y = -15
local material_funnel_offset_x = 14
local material_funnel_offset_y = -10

local relic_material_base_construct_cost, relic_precursor_material_construct_cost = getRelicConstructCosts()
local main_construction_material = getRelicConstructionMaterialName()

-- obtains id and coordinates
local entity_id = GetUpdatedEntityID()
local x, y = EntityGetTransform(entity_id)

-- unlocks player inventories while in close vicinity
-- unlockPlayerInventories(x, y, 40, 85)

-- Loads material converter to smelt relic placed on funnel
EntityLoad("mods/ArcaneReliquary/files/entity_definitions/others/field_relic_to_relic_metal.xml", x + material_funnel_offset_x, y)

-- Loads unclogger field to eliminate possible obstructions on the funnel
EntityLoad("mods/ArcaneReliquary/files/entity_definitions/others/field_unclogger.xml", x + material_funnel_offset_x, y + material_funnel_offset_y)

-- obtains amount of relic metal available in the machine
local amount = getMaterialAmountOnEntityInventory(entity_id, main_construction_material)

-- if the player is in range, shows amount of upgrade material currently on forge
--[[
local players_in_range = EntityGetInRadiusWithTag( x, y, 70, "player_unit" )
if(#players_in_range > 0) then
	GamePrint("Relic metal on workbench: " .. amount)
end
]]

-- to prevent multiple constructions to trigger simultaneously, verifies if a construction entity is already present, if so, skips everything below
if(EntityGetWithName( "relic_construct" ) ~= 0) then 
	return 
end

-- check whether or not construction can proceed
local construction_enabled = getInternalVariableValue(entity_id, "construction_enabled", "value_bool")
local relic_to_construct = nil

if(construction_enabled) then
	relic_to_construct = getRelic(getInternalVariableValue(entity_id, "relic_to_construct_name", "value_string"))
	local precursor_material_name = nil
	
	-- loops through precursor materials list to find the first material that matches the list of precursor types of the chosen relic
	-- and is found in sufficient quantity in the workbench according to the build budget for precursor materials
	local relic_precursor_materials_list = getRelicPrecursorMaterialsList()
	for i,item in ipairs(relic_precursor_materials_list) do
		if(string.find(relic_to_construct.precursor_types, item.precursor_type) and getMaterialAmountOnEntityInventory(entity_id, item.precursor_material_name) >= relic_precursor_material_construct_cost) then
			precursor_material_name = item.precursor_material_name
		end
	end
	
	-- obtains amount of precursor material
	local precursor_material_amount = getMaterialAmountOnEntityInventory(entity_id, precursor_material_name)
	
	-- checks base and precursor material amounts before spawning constructing entity
	if(amount >= relic_material_base_construct_cost and precursor_material_amount >= relic_precursor_material_construct_cost) then
	
		-- consumes materials
		amount = amount - relic_material_base_construct_cost
		AddMaterialInventoryMaterial( entity_id, main_construction_material, amount)
		
		precursor_material_amount = precursor_material_amount - relic_precursor_material_construct_cost
		AddMaterialInventoryMaterial( entity_id, precursor_material_name, precursor_material_amount)
		
		-- loads construct entity and sets relic to be constructed
		local relic_construct_entity_id = EntityLoad("mods/ArcaneReliquary/files/entity_definitions/others/relic_construct.xml", x + craft_plate_offset_x, y + craft_plate_offset_y)
		setInternalVariableValue(relic_construct_entity_id, "relic_to_construct", "value_string", relic_to_construct.name)
	end
	
	-- disables construction
	setInternalVariableValue(entity_id, "construction_enabled", "value_bool", false)
	setInternalVariableValue(entity_id, "relic_to_construct_name", "value_string", "")
end

--[[
local relic_precursor_materials_list = getRelicPrecursorMaterialsList()

for i,item in ipairs(relic_precursor_materials_list) do
	local precursor_material_name = item.precursor_material_name
	local precursor_type = item.precursor_type
	
	-- obtains amount of precursor materials
	local precursor_material_amount = getMaterialAmountOnEntityInventory(entity_id, precursor_material_name)
	--GamePrint("Relic precursor material on workbench: " .. precursor_material_amount)
	
	
	if(amount >= relic_material_base_construct_cost and precursor_material_amount >= relic_precursor_material_construct_cost) then
	
		-- consumes materials
		amount = amount - relic_material_base_construct_cost
		AddMaterialInventoryMaterial( entity_id, main_construction_material, amount)
		
		precursor_material_amount = precursor_material_amount - relic_precursor_material_construct_cost
		AddMaterialInventoryMaterial( entity_id, precursor_material_name, precursor_material_amount)
		
		-- obtains a relic according to precursor_type
		SetRandomSeed( GameGetFrameNum(), x + y + entity_id + math.random(1,50))
		local constructible_relics = getRelicsByPrecursorType(precursor_type)
		local constructed_relic = constructible_relics[Random(1,#constructible_relics)]
		
		-- loads construct entity and sets relic to be constructed
		local relic_construct_entity_id = EntityLoad("mods/ArcaneReliquary/files/entity_definitions/others/relic_construct.xml", x + craft_plate_offset_x, y + craft_plate_offset_y)
		setInternalVariableValue(relic_construct_entity_id, "relic_to_construct", "value_string", constructed_relic.name)
	end
	
end
]]

