dofile_once("mods/ArcaneReliquary/files/scripts/utilities/arcanereliquary_utilities.lua")
dofile_once("mods/ArcaneReliquary/files/scripts/lists/custom_effects_list.lua")
dofile_once("mods/ArcaneReliquary/files/scripts/lists/relics_list.lua")

local entity_id = GetUpdatedEntityID()
local x, y = EntityGetTransform(entity_id)
local ground_pound_active = getInternalVariableValue(entity_id, "ground_pound_active", "value_bool")

-- activates ground pound on down key pressed
local cc_id = EntityGetFirstComponent(entity_id, "ControlsComponent")
local down_triggered = ComponentGetValueInt( cc_id, "mButtonFrameDown" ) == GameGetFrameNum()
if(down_triggered and not isPlayerStanding() and not ground_pound_active) then
	
	-- audio feedback
	GamePlaySound( "data/audio/Desktop/projectiles.snd", "player_projectiles/bomb/create", x, y )
	
	-- adds sprite effect that lasts indefinitely until removed
	addSpriteEffect(entity_id, "mods/ArcaneReliquary/files/other_gfx/ground_pound.xml", 15, 20, -1, 0.7, "ground_pound_effect")
	
	-- applies downward velocity boost
	local vertical_down_boost = 1500
	local vel_x, vel_y = getPlayerVelocities()
	setPlayerVelocities(vel_x, vel_y + vertical_down_boost)
	
	-- sets ground pound as active
	ground_pound_active = true
	setInternalVariableValue(entity_id, "ground_pound_active", "value_bool", ground_pound_active)
end

-- if the player jumps after activating ground pound, cancells it
local jump_triggered = ComponentGetValueInt( cc_id, "mButtonFrameFly" ) == GameGetFrameNum()
if(jump_triggered and ground_pound_active) then
	ground_pound_active = false
	setInternalVariableValue(entity_id, "ground_pound_active", "value_bool", ground_pound_active)
	
	-- removes sprite effect
	removeSpriteEffect(entity_id, "ground_pound_effect")
end

-- activates explosion when ground pound is active and the player is currently standing
if(ground_pound_active and isPlayerStanding()) then
	--GamePrint("player has landed, reseting")
	if(GameGetGameEffectCount(entity_id, "BERSERK") > 0) then
		EntityLoad("mods/ArcaneReliquary/files/entity_definitions/others/ground_pound_explosion_big.xml", x, y) -- if under BERSERK effect, loads more powerful version of the ground pound explosion
	else
		EntityLoad("mods/ArcaneReliquary/files/entity_definitions/others/ground_pound_explosion.xml", x, y) -- otherwise, loads regular version
	end
	ground_pound_active = false
	setInternalVariableValue(entity_id, "ground_pound_active", "value_bool", ground_pound_active)
	
	-- removes sprite effect
	removeSpriteEffect(entity_id, "ground_pound_effect")
	
	-- applies cooldown
	local ground_pound_cooldown = 75
	removeCallback(entity_id, "ground_pound_listener")
	addNewLuaCallback(entity_id, "overtime", "mods/ArcaneReliquary/files/scripts/player/ground_pound_recover.lua", ground_pound_cooldown, true, "ground_pound_recover")
	
	-- this executes sinergy functions when specific relics are owned together with the earth core fragment relic
	for i,relic in ipairs(relics_list) do 
		if(relic.ongroundpound_function ~= nil) then
			relic.ongroundpound_function( entity_id)
		end
	end
	
	-- synergies with regular projectile launching relics
	if(entityHasSimplifiedInternalVariable(entity_id, "thunder_hat")) then
		-- Shoots electrobombs
		SetRandomSeed( GameGetFrameNum(), math.random(1,50))
		local amount = Random(8,9)
		for i=1,amount do
			local projectile_id = EntityLoad("mods/ArcaneReliquary/files/entity_definitions/custom_projectiles/electrobomb.xml", x, y - Random(10,12))
			setProjectileVelocitiesandGravity(projectile_id, Random(-350,350), Random(-100,-250))
			setProjectileShooter(projectile_id, entity_id)
		end
	elseif(entityHasSimplifiedInternalVariable(entity_id, "ballistic_boot")) then
		-- Shoots firebombs
		SetRandomSeed( GameGetFrameNum(), math.random(1,50))
		local amount = Random(8,9)
		for i=1,amount do
			local projectile_id = EntityLoad("mods/ArcaneReliquary/files/entity_definitions/custom_projectiles/firebomb_custom.xml", x, y - Random(10,12))
			setProjectileVelocitiesandGravity(projectile_id, Random(-350,350), Random(-100,-250))
			setProjectileShooter(projectile_id, entity_id)
		end
	elseif(entityHasSimplifiedInternalVariable(entity_id, "popsicle")) then
		-- Shoots freezebombs
		SetRandomSeed( GameGetFrameNum(), math.random(1,50))
		local amount = Random(8,9)
		for i=1,amount do
			local projectile_id = EntityLoad("mods/ArcaneReliquary/files/entity_definitions/custom_projectiles/freezebomb.xml", x, y - Random(10,12))
			setProjectileVelocitiesandGravity(projectile_id, Random(-350,350), Random(-100,-250))
			setProjectileShooter(projectile_id, entity_id)
		end
	elseif(entityHasSimplifiedInternalVariable(entity_id, "coagulated_blood")) then
		-- Shoots bloodbombs
		SetRandomSeed( GameGetFrameNum(), math.random(1,50))
		local amount = Random(8,9)
		for i=1,amount do
			local projectile_id = EntityLoad("mods/ArcaneReliquary/files/entity_definitions/custom_projectiles/bloodbomb.xml", x, y - Random(10,12))
			setProjectileVelocitiesandGravity(projectile_id, Random(-350,350), Random(-100,-250))
			setProjectileShooter(projectile_id, entity_id)
		end
	elseif(entityHasSimplifiedInternalVariable(entity_id, "fly_swatter")) then
		-- Shoots slapperbombs
		SetRandomSeed( GameGetFrameNum(), math.random(1,50))
		local amount = Random(8,9)
		for i=1,amount do
			local projectile_id = EntityLoad("mods/ArcaneReliquary/files/entity_definitions/custom_projectiles/slapperbomb.xml", x, y - Random(10,12))
			setProjectileVelocitiesandGravity(projectile_id, Random(-350,350), Random(-100,-250))
			setProjectileShooter(projectile_id, entity_id)
		end
	elseif(entityHasSimplifiedInternalVariable(entity_id, "mutating_bead")) then
		-- Shoots polybombs
		SetRandomSeed( GameGetFrameNum(), math.random(1,50))
		local amount = Random(8,9)
		for i=1,amount do
			local projectile_id = EntityLoad("mods/ArcaneReliquary/files/entity_definitions/custom_projectiles/polybomb.xml", x, y - Random(10,12))
			setProjectileVelocitiesandGravity(projectile_id, Random(-350,350), Random(-100,-250))
			setProjectileShooter(projectile_id, entity_id)
		end
	elseif(entityHasSimplifiedInternalVariable(entity_id, "little_cloud")) then
		-- Shoots windbombs
		SetRandomSeed( GameGetFrameNum(), math.random(1,50))
		local amount = Random(8,9)
		for i=1,amount do
			local projectile_id = EntityLoad("mods/ArcaneReliquary/files/entity_definitions/custom_projectiles/windbomb.xml", x, y - Random(10,12))
			setProjectileVelocitiesandGravity(projectile_id, Random(-350,350), Random(-100,-250))
			setProjectileShooter(projectile_id, entity_id)
		end
	end
end