dofile_once("data/scripts/lib/utilities.lua")
dofile_once("mods/ArcaneReliquary/files/scripts/utilities/arcanereliquary_utilities.lua")
dofile_once("mods/ArcaneReliquary/files/scripts/lists/custom_effects_list.lua")

local entity_id = GetUpdatedEntityID()
local pos_x, pos_y = EntityGetTransform(entity_id)

local cc_id = EntityGetFirstComponent(entity_id, "ControlsComponent")
local right_click = ComponentGetValueInt( cc_id, "mButtonFrameRightClick" ) == GameGetFrameNum()
local homing_attack_triggered = false

if(right_click) then

	-- locate nearest enemy
	local detection_range = 100
	local target_tag = "enemy"
	local enemy, enemy_x, enemy_y
	local min_dist = 9999
	for _,id in pairs(EntityGetInRadiusWithTag(pos_x, pos_y, detection_range, target_tag)) do

		local x, y = EntityGetFirstHitboxCenter( id )
		local dist = get_distance(pos_x, pos_y, x, y)
		if dist < min_dist then
			min_dist = dist
			enemy = id
			enemy_x = x
			enemy_y = y
		end
	end

	if not enemy then return end

	-- check los
	if RaytraceSurfaces(pos_x, pos_y, enemy_x, enemy_y) then return end

	-- direction
	local vel_x, vel_y = vec_sub(enemy_x, enemy_y, pos_x, pos_y)
	vel_x,vel_y = vec_normalize(vel_x, vel_y)
	
	-- scatter
	local scatter = 0.1
	local vx,vy = vec_rotate(vel_x,vel_y, rand(-scatter, scatter))

	-- calculate velocities
	local speed_gain = 1200
	vx,vy = vec_mult(vx,vy, speed_gain)
	
	-- add player velocities towards target
	--local player_vel_x, player_vel_y = getPlayerVelocities()
	setPlayerVelocities(vx, vy)
	
	homing_attack_triggered = true
end

if(homing_attack_triggered) then

	-- audio feedback
	local x, y = EntityGetTransform(entity_id)
	GamePlaySound( "data/audio/Desktop/projectiles.snd", "player_projectiles/bomb/create", x, y )
	
	-- gives 10 frames invencibility
	local game_effect_comp = GetGameEffectLoadTo( entity_id, "PROTECTION_ALL", true )
	if game_effect_comp ~= nil then
		ComponentSetValue( game_effect_comp, "frames", "10" )
	end
	
	-- adds visual effect that lasts for 10 frames
	addSpriteEffect(entity_id, "mods/ArcaneReliquary/files/other_gfx/homing_attack.xml", 15, 20, 10, 0.7, "homing_attack_effect")
	
	-- applies cooldown
	local homing_attack_cooldown = 55
	removeCallback(entity_id, "homing_attack_listener")
	addNewLuaCallback(entity_id, "overtime", "mods/ArcaneReliquary/files/scripts/player/homing_attack_recover.lua", homing_attack_cooldown, true, "homing_attack_recover")
	
	-- adds short duration damaging aura effect
	LoadGameEffectEntityTo( entity_id, getCustomEffect("effect_homing_attack").path )
end
