dofile_once("mods/ArcaneReliquary/files/scripts/utilities/arcanereliquary_utilities.lua")

local entity_id = GetUpdatedEntityID()
local laser_sight_handler_name = EntityGetName(entity_id)
local wand_id = getCurrentlyEquippedWandId()

function renderLaserSight(wand_id, entity_id)
	local spriteComponent = EntityGetFirstComponent(entity_id, "SpriteComponent")
	local lightComponent = EntityGetFirstComponent(entity_id, "LightComponent")
	if(wand_id ~= -1) then -- laser renders only if item held is a wand, otherwise hides it
		ComponentSetValue2(spriteComponent, "visible", true)
		ComponentSetValue2(lightComponent, "mAlpha", true)

		-- Has to match the width of the sprite
		local SPRITE_PIXEL_LENGTH = 40

		-- Get coordinates of start and end points
		local wand_id_x, wand_id_y, rotation = EntityGetTransform(wand_id)
		local mouse_x, mouse_y = DEBUG_GetMouseWorld()

		--wand_id_x = wand_id_x + 1
		--wand_id_y = wand_id_y - 1.75

		-- Get a visibility point, for cutting off the line. Essentially: "how far can we see".
		local did_hit, target_x, target_y = RaytracePlatforms(wand_id_x, wand_id_y, mouse_x, mouse_y)
		if (not did_hit) then
			target_x = mouse_x
			target_y = mouse_y
		end

		-- Set line length(x) and width(y)
		local length = - get_distance(wand_id_x, wand_id_y, target_x, target_y) / SPRITE_PIXEL_LENGTH
		local width = 1

		EntitySetTransform(entity_id, wand_id_x, wand_id_y, rotation, - length, width)
		
		return target_x, target_y
	else
		ComponentSetValue2(spriteComponent, "visible", false)
		ComponentSetValue2(lightComponent, "mAlpha", false)
		return -1, -1
	end
end

if(laser_sight_handler_name == "laser_sight_handler") then
	renderLaserSight(wand_id, entity_id)
end

if(laser_sight_handler_name == "laser_sight_explosive_handler") then
	
	local target_x, target_y = renderLaserSight(wand_id, entity_id)
	
	local cc_id = EntityGetFirstComponent(getPlayerEntity(), "ControlsComponent")
	local attack_triggered = ComponentGetValueInt( cc_id, "mButtonFrameFire" ) == GameGetFrameNum()
	
	local extra_effect_enabled = getInternalVariableValue(entity_id, "extra_effect_enabled", "value_bool")
	if(attack_triggered) then
		setInternalVariableValue(entity_id, "extra_effect_enabled", "value_bool", not extra_effect_enabled)
	end
	
	if(extra_effect_enabled) then
		-- switches to explosive laser sprite
		local spriteComponent = EntityGetFirstComponent(entity_id, "SpriteComponent")
		ComponentSetValue2(spriteComponent, "image_file", "mods/ArcaneReliquary/files/other_gfx/laser_sight_explosive.png")
		-- random explosion, consumes fly energy
		SetRandomSeed( GameGetFrameNum(), math.random(1,50))
		if(Random(1,5) == 1 and getFlyEnergy() >= 0.1) then
			addFlyEnergy(-0.1)
			EntityLoad("mods/ArcaneReliquary/files/entity_definitions/others/micro_explosion.xml", target_x + Random(5, -5), target_y + Random(5, -5))
		end
		
	else
		-- switches to regular laser sprite
		local spriteComponent = EntityGetFirstComponent(entity_id, "SpriteComponent")
		ComponentSetValue2(spriteComponent, "image_file", "mods/ArcaneReliquary/files/other_gfx/laser_sight.png")
	end
end

if(laser_sight_handler_name == "laser_sight_freezing_handler") then
	
	local target_x, target_y = renderLaserSight(wand_id, entity_id)
	
	local cc_id = EntityGetFirstComponent(getPlayerEntity(), "ControlsComponent")
	local attack_triggered = ComponentGetValueInt( cc_id, "mButtonFrameFire" ) == GameGetFrameNum()
	
	local extra_effect_enabled = getInternalVariableValue(entity_id, "extra_effect_enabled", "value_bool")
	if(attack_triggered) then
		setInternalVariableValue(entity_id, "extra_effect_enabled", "value_bool", not extra_effect_enabled)
	end
	
	if(extra_effect_enabled) then
		-- switches to explosive laser sprite
		local spriteComponent = EntityGetFirstComponent(entity_id, "SpriteComponent")
		ComponentSetValue2(spriteComponent, "image_file", "mods/ArcaneReliquary/files/other_gfx/laser_sight_freezing.png")
		-- random explosion, consumes fly energy
		SetRandomSeed( GameGetFrameNum(), math.random(1,50))
		if(Random(1,5) == 1 and getFlyEnergy() >= 0.1) then
			addFlyEnergy(-0.1)
			EntityLoad("mods/ArcaneReliquary/files/entity_definitions/others/effect_explosion_freeze.xml", target_x + Random(5, -5), target_y + Random(5, -5))
		end
		
	else
		-- switches to regular laser sprite
		local spriteComponent = EntityGetFirstComponent(entity_id, "SpriteComponent")
		ComponentSetValue2(spriteComponent, "image_file", "mods/ArcaneReliquary/files/other_gfx/laser_sight.png")
	end
end

if(laser_sight_handler_name == "laser_sight_electrocuting_handler") then
	
	local target_x, target_y = renderLaserSight(wand_id, entity_id)
	
	local cc_id = EntityGetFirstComponent(getPlayerEntity(), "ControlsComponent")
	local attack_triggered = ComponentGetValueInt( cc_id, "mButtonFrameFire" ) == GameGetFrameNum()
	
	local extra_effect_enabled = getInternalVariableValue(entity_id, "extra_effect_enabled", "value_bool")
	if(attack_triggered) then
		setInternalVariableValue(entity_id, "extra_effect_enabled", "value_bool", not extra_effect_enabled)
	end
	
	if(extra_effect_enabled) then
		-- switches to explosive laser sprite
		local spriteComponent = EntityGetFirstComponent(entity_id, "SpriteComponent")
		ComponentSetValue2(spriteComponent, "image_file", "mods/ArcaneReliquary/files/other_gfx/laser_sight_electrocuting.png")
		-- random explosion, consumes fly energy
		SetRandomSeed( GameGetFrameNum(), math.random(1,50))
		if(Random(1,5) == 1 and getFlyEnergy() >= 0.1) then
			addFlyEnergy(-0.1)
			EntityLoad("mods/ArcaneReliquary/files/entity_definitions/others/effect_explosion_electrocute.xml", target_x + Random(5, -5), target_y + Random(5, -5))
		end
		
	else
		-- switches to regular laser sprite
		local spriteComponent = EntityGetFirstComponent(entity_id, "SpriteComponent")
		ComponentSetValue2(spriteComponent, "image_file", "mods/ArcaneReliquary/files/other_gfx/laser_sight.png")
	end
end

if(laser_sight_handler_name == "laser_sight_slash_handler") then
	
	local target_x, target_y = renderLaserSight(wand_id, entity_id)
	
	local cc_id = EntityGetFirstComponent(getPlayerEntity(), "ControlsComponent")
	local attack_triggered = ComponentGetValueInt( cc_id, "mButtonFrameFire" ) == GameGetFrameNum()
	
	local extra_effect_enabled = getInternalVariableValue(entity_id, "extra_effect_enabled", "value_bool")
	if(attack_triggered) then
		setInternalVariableValue(entity_id, "extra_effect_enabled", "value_bool", not extra_effect_enabled)
	end
	
	if(extra_effect_enabled) then
		-- switches to explosive laser sprite
		local spriteComponent = EntityGetFirstComponent(entity_id, "SpriteComponent")
		ComponentSetValue2(spriteComponent, "image_file", "mods/ArcaneReliquary/files/other_gfx/laser_sight_slash.png")
		-- random explosion, consumes fly energy
		SetRandomSeed( GameGetFrameNum(), math.random(1,50))
		if(Random(1,5) == 1 and getFlyEnergy() >= 0.1) then
			addFlyEnergy(-0.1)
			EntityLoad("mods/ArcaneReliquary/files/entity_definitions/others/effect_explosion_slash.xml", target_x + Random(5, -5), target_y + Random(5, -5))
		end
		
	else
		-- switches to regular laser sprite
		local spriteComponent = EntityGetFirstComponent(entity_id, "SpriteComponent")
		ComponentSetValue2(spriteComponent, "image_file", "mods/ArcaneReliquary/files/other_gfx/laser_sight.png")
	end
end

if(laser_sight_handler_name == "laser_sight_slap_handler") then
	
	local target_x, target_y = renderLaserSight(wand_id, entity_id)
	
	local cc_id = EntityGetFirstComponent(getPlayerEntity(), "ControlsComponent")
	local attack_triggered = ComponentGetValueInt( cc_id, "mButtonFrameFire" ) == GameGetFrameNum()
	
	local extra_effect_enabled = getInternalVariableValue(entity_id, "extra_effect_enabled", "value_bool")
	if(attack_triggered) then
		setInternalVariableValue(entity_id, "extra_effect_enabled", "value_bool", not extra_effect_enabled)
	end
	
	if(extra_effect_enabled) then
		-- switches to explosive laser sprite
		local spriteComponent = EntityGetFirstComponent(entity_id, "SpriteComponent")
		ComponentSetValue2(spriteComponent, "image_file", "mods/ArcaneReliquary/files/other_gfx/laser_sight_slap.png")
		-- random explosion, consumes fly energy
		SetRandomSeed( GameGetFrameNum(), math.random(1,50))
		if(Random(1,5) == 1 and getFlyEnergy() >= 0.1) then
			addFlyEnergy(-0.1)
			EntityLoad("mods/ArcaneReliquary/files/entity_definitions/others/effect_explosion_slap.xml", target_x + Random(5, -5), target_y + Random(5, -5))
		end
		
	else
		-- switches to regular laser sprite
		local spriteComponent = EntityGetFirstComponent(entity_id, "SpriteComponent")
		ComponentSetValue2(spriteComponent, "image_file", "mods/ArcaneReliquary/files/other_gfx/laser_sight.png")
	end
end

if(laser_sight_handler_name == "laser_sight_bleeding_handler") then
	
	local target_x, target_y = renderLaserSight(wand_id, entity_id)
	
	local cc_id = EntityGetFirstComponent(getPlayerEntity(), "ControlsComponent")
	local attack_triggered = ComponentGetValueInt( cc_id, "mButtonFrameFire" ) == GameGetFrameNum()
	
	local extra_effect_enabled = getInternalVariableValue(entity_id, "extra_effect_enabled", "value_bool")
	if(attack_triggered) then
		setInternalVariableValue(entity_id, "extra_effect_enabled", "value_bool", not extra_effect_enabled)
	end
	
	if(extra_effect_enabled) then
		-- switches to explosive laser sprite
		local spriteComponent = EntityGetFirstComponent(entity_id, "SpriteComponent")
		ComponentSetValue2(spriteComponent, "image_file", "mods/ArcaneReliquary/files/other_gfx/laser_sight_bleeding.png")
		-- random explosion, consumes fly energy
		SetRandomSeed( GameGetFrameNum(), math.random(1,50))
		if(Random(1,5) == 1 and getFlyEnergy() >= 0.1) then
			addFlyEnergy(-0.1)
			EntityLoad("mods/ArcaneReliquary/files/entity_definitions/others/effect_explosion_bleeding.xml", target_x + Random(5, -5), target_y + Random(5, -5))
		end
		
	else
		-- switches to regular laser sprite
		local spriteComponent = EntityGetFirstComponent(entity_id, "SpriteComponent")
		ComponentSetValue2(spriteComponent, "image_file", "mods/ArcaneReliquary/files/other_gfx/laser_sight.png")
	end
end

if(laser_sight_handler_name == "laser_sight_explode_handler") then
	
	local target_x, target_y = renderLaserSight(wand_id, entity_id)
	
	local cc_id = EntityGetFirstComponent(getPlayerEntity(), "ControlsComponent")
	local attack_triggered = ComponentGetValueInt( cc_id, "mButtonFrameFire" ) == GameGetFrameNum()
	
	local extra_effect_enabled = getInternalVariableValue(entity_id, "extra_effect_enabled", "value_bool")
	if(attack_triggered) then
		setInternalVariableValue(entity_id, "extra_effect_enabled", "value_bool", not extra_effect_enabled)
	end
	
	if(extra_effect_enabled) then
		-- switches to explosive laser sprite
		local spriteComponent = EntityGetFirstComponent(entity_id, "SpriteComponent")
		ComponentSetValue2(spriteComponent, "image_file", "mods/ArcaneReliquary/files/other_gfx/laser_sight_explode.png")
		-- random explosion, consumes fly energy
		SetRandomSeed( GameGetFrameNum(), math.random(1,50))
		if(Random(1,5) == 1 and getFlyEnergy() >= 0.1) then
			addFlyEnergy(-0.1)
			EntityLoad("mods/ArcaneReliquary/files/entity_definitions/others/effect_explosion_explode.xml", target_x + Random(5, -5), target_y + Random(5, -5))
		end
		
	else
		-- switches to regular laser sprite
		local spriteComponent = EntityGetFirstComponent(entity_id, "SpriteComponent")
		ComponentSetValue2(spriteComponent, "image_file", "mods/ArcaneReliquary/files/other_gfx/laser_sight.png")
	end
end