dofile_once("mods/ArcaneReliquary/files/scripts/utilities/arcanereliquary_utilities.lua")
dofile_once("mods/ArcaneReliquary/files/scripts/lists/custom_effects_list.lua")
dofile_once("mods/ArcaneReliquary/files/scripts/lists/load_on_explosion_list.lua")

-- auxiliary functions used in this script
-- =======================================
function sideRotation(rotation) 
	local side = 0
	if((rotation >= -1.5 and rotation <= 0.0) or (rotation >= 0.0 and rotation <= 1.5)) then
		side = 1
	elseif((rotation <= -1.5 and rotation >= -3.15) or (rotation >= 1.5 and rotation <= 3.15))  then
		side = -1
	end
	return side
end

-- function for shields that allow the player to glide when pointing it up
function glide(wand_rotation)
	if(wand_rotation > -2 and wand_rotation < -1) then
		local vel_x, vel_y = getPlayerVelocities()
		setPlayerVelocities(vel_x, 15)
	end
end

-- function for shields that allow the player to fly upwards when pointing it up
function upwardMomentum(wand_rotation)
	if(wand_rotation > -2 and wand_rotation < -1) then
		local vel_x, vel_y = getPlayerVelocities()
		setPlayerVelocities(vel_x, -50)
	end
end

-- function used to adjust the orientation of the shield when the design of the shield requires it
function switchAnimationByRotation(parry_shield_id, wand_rotation)
	local sc_id = EntityGetFirstComponentIncludingDisabled(parry_shield_id, "SpriteComponent")
	local side = sideRotation(wand_rotation)
	if(side == 1) then
		ComponentSetValue2( sc_id, "rect_animation", "right_oriented")
	else
		ComponentSetValue2( sc_id, "rect_animation", "left_oriented")
	end
end

-- function to add enemy projectile destruction behavior to shield
function destroyProjectiles(x, y, detectionRadius, shield_wielder_id, wand_rotation, deal_damage_shooter, convert_damage_hp, inflict_effect_shooter)
	--local detection_y_offset = 10
	local projectiles = EntityGetInRadiusWithTag(x, y, detectionRadius, "projectile")
	for i, projectile_id in ipairs(projectiles) do
		-- conditions to prevent friendly projectiles from being redirected
		if(shield_wielder_id == getPlayerEntity() and entityHasSimplifiedInternalVariable(projectile_id, "relic_effect_projectile")) then return end
		if(string.find(EntityGetName(projectile_id), "rotating_shield_projectile")) then return end
		if(entityHasSimplifiedInternalVariable(projectile_id, "deflected")) then return end
		
		local projectile_x, projectile_y, projectile_rotation = EntityGetTransform(projectile_id)
		
		-- makeshift workaround to prevent shield from deflecting projectiles that come from behind the player
		if(sideRotation(wand_rotation) == sideRotation(projectile_rotation)) then return end
		
		if(getProjectileShooter(projectile_id) ~= shield_wielder_id) then
			-- Loads audiovisual feedback
			GamePlaySound( "data/audio/Desktop/projectiles.snd", "player_projectiles/shield/block", projectile_x, projectile_y )
			EntityLoad("mods/ArcaneReliquary/files/entity_definitions/particle_explosions/shield_deflect_explosion_red.xml", projectile_x, projectile_y)
			
			-- Damage shooter
			if(deal_damage_shooter) then
				EntityInflictDamage(getProjectileShooter(projectile_id), getProjectileBaseDamage(projectile_id), "DAMAGE_PROJECTILE", "Own projectile damage", "BLOOD_SPRAY", 0, 0)
			end
			
			-- Converts damage into hp for player
			if(convert_damage_hp) then
				addPlayerHealth(getProjectileBaseDamage(projectile_id) * 25)
			end
			
			if(inflict_effect_shooter ~= nil) then
				LoadGameEffectEntityTo( getProjectileShooter(projectile_id), inflict_effect_shooter )
			end
			
			-- kills colliding projectile
			projectileToggleExplosionOnDeath(projectile_id, false)
			EntityKill(projectile_id)
		end
		
	end
end

-- function to add enemy projectile redirection behavior to shield
function redirectProjectiles(x, y, detectionRadius, damageBoost, shield_wielder_id, wand_rotation)
	--local detection_y_offset = 10
	local projectiles = EntityGetInRadiusWithTag(x, y, detectionRadius, "projectile")
	for i, projectile_id in ipairs(projectiles) do
		-- conditions to prevent friendly projectiles from being redirected
		if(shield_wielder_id == getPlayerEntity() and entityHasSimplifiedInternalVariable(projectile_id, "relic_effect_projectile")) then return end
		if(string.find(EntityGetName(projectile_id), "rotating_shield_projectile")) then return end
		if(entityHasSimplifiedInternalVariable(projectile_id, "deflected")) then return end
		
		local projectile_x, projectile_y, projectile_rotation = EntityGetTransform(projectile_id)
		
		-- makeshift workaround to prevent shield from deflecting projectiles that come from behind the player
		if(sideRotation(wand_rotation) == sideRotation(projectile_rotation)) then return end
		
		if(getProjectileShooter(projectile_id) ~= shield_wielder_id) then
			-- Loads audiovisual feedback
			GamePlaySound( "data/audio/Desktop/projectiles.snd", "player_projectiles/shield/block", projectile_x, projectile_y )
			EntityLoad("mods/ArcaneReliquary/files/entity_definitions/particle_explosions/shield_deflect_explosion_teal.xml", projectile_x, projectile_y)
		
			-- Sets the shooter of the deflected projectile as the creature under the effect
			setProjectileShooter(projectile_id, shield_wielder_id)

			-- Flags projectile as deflected to prevent it from changing again
			addNewSimplifiedInternalVariable(projectile_id, "deflected")
			
			-- Changes projectile direction randomly
			local angle = math.rad(Random(0,359))
			local length = Random(200,500)

			local vel_x = math.cos( angle ) * length
			local vel_y = 0 - math.sin( angle ) * length
			
			setProjectileVelocitiesandGravity(projectile_id, vel_x, vel_y)
			
			if(damageBoost) then
				-- Doubles base and explosive damage
				multiplyProjectileBaseDamage(projectile_id, 2)
				multiplyProjectileExplosiveDamage(projectile_id, 2)
				multiplyProjectileExplosionRadius(projectile_id, 2)
				addBuffVisualIndicator_Projectile(projectile_id, "spark_red")
			end
		end
	end
end

-- function to add enemy projectile deflection behavior to shield
function deflectProjectiles(x, y, detectionRadius, damageBoost, shield_wielder_id, wand_rotation, effect_path, add_damage_by_type, add_load_on_explosion_entity)
	--local detection_y_offset = 10
	local projectiles = EntityGetInRadiusWithTag(x, y, detectionRadius, "projectile")
	for i, projectile_id in ipairs(projectiles) do
		-- conditions to prevent friendly projectiles from being redirected
		if(shield_wielder_id == getPlayerEntity() and entityHasSimplifiedInternalVariable(projectile_id, "relic_effect_projectile")) then return end
		if(string.find(EntityGetName(projectile_id), "rotating_shield_projectile")) then return end
		if(entityHasSimplifiedInternalVariable(projectile_id, "deflected")) then return end
		
		local projectile_x, projectile_y, projectile_rotation = EntityGetTransform(projectile_id)
		
		-- makeshift workaround to prevent shield from deflecting projectiles that come from behind the player
		if(sideRotation(wand_rotation) == sideRotation(projectile_rotation)) then return end
		
		if(getProjectileShooter(projectile_id) ~= shield_wielder_id) then
			-- Loads audiovisual feedback
			GamePlaySound( "data/audio/Desktop/projectiles.snd", "player_projectiles/shield/block", projectile_x, projectile_y )
			EntityLoad("mods/ArcaneReliquary/files/entity_definitions/particle_explosions/shield_deflect_explosion_fire.xml", projectile_x, projectile_y)
			
			local projectile_vel_x, projectile_vel_y = getProjectileVelocitiesandGravity(projectile_id)

			-- Inverts projectile velocity, turning it back to the direction it came
			setProjectileVelocitiesandGravity(projectile_id, (projectile_vel_x + 1.5) * -1, (projectile_vel_y + 1.5) * -1)
			
			if(damageBoost) then
				-- Doubles base and explosive damage
				multiplyProjectileBaseDamage(projectile_id, 2)
				multiplyProjectileExplosiveDamage(projectile_id, 2)
				multiplyProjectileExplosionRadius(projectile_id, 2)
				addBuffVisualIndicator_Projectile(projectile_id, "spark_red")
			end
			
			if(add_damage_by_type ~= nil) then
				if(add_damage_by_type == "melee") then
					attachLoadEntityOnExplosiontoProjectile(projectile_id, getLoadOnExplosion("explosion_melee").path)
					addProjectileDamageByType(projectile_id, "melee", 0.5)
				elseif(add_damage_by_type == "slice") then
					attachLoadEntityOnExplosiontoProjectile(projectile_id, getLoadOnExplosion("explosion_slash").path)
					if(not projectileHasDamageType(projectile_id, "slice")) then
						addProjectileDamageByType(projectile_id, "slice", 0.5)
					else
						addProjectileDamageByType(projectile_id, "slice", 0.2)
					end
				end
			end
			
			if(add_load_on_explosion_entity ~= nil) then
				attachLoadEntityOnExplosiontoProjectile(projectile_id, add_load_on_explosion_entity, "spark_red")
			end
			
			if(effect_path ~= nil) then
				-- Adds effect to projectile
				attachLoadEffectOntoEntitiestoProjectile(projectile_id, effect_path, "spark_red")
			end

			-- Sets the shooter of the deflected projectile as the creature under the effect
			setProjectileShooter(projectile_id, shield_wielder_id)

			-- Flags projectile as deflected to prevent it from changing again
			addNewSimplifiedInternalVariable(projectile_id, "deflected")
		end
	end
end

-- function to add enemy projectile scattering behavior to shield
function scatterProjectiles(x, y, detectionRadius, shield_wielder_id, wand_rotation, deflect_explosion_file_name, effect_path)
	--local detection_y_offset = 10
	local projectiles = EntityGetInRadiusWithTag(x, y, detectionRadius, "projectile")
	for i, projectile_id in ipairs(projectiles) do
		-- conditions to prevent friendly projectiles from being redirected
		if(shield_wielder_id == getPlayerEntity() and entityHasSimplifiedInternalVariable(projectile_id, "relic_effect_projectile")) then return end
		if(string.find(EntityGetName(projectile_id), "rotating_shield_projectile")) then return end
		if(entityHasSimplifiedInternalVariable(projectile_id, "deflected")) then return end
		
		local projectile_x, projectile_y, projectile_rotation = EntityGetTransform(projectile_id)
		
		-- makeshift workaround to prevent shield from deflecting projectiles that come from behind the player
		if(sideRotation(wand_rotation) == sideRotation(projectile_rotation)) then return end
		
		if(getProjectileShooter(projectile_id) ~= shield_wielder_id) then
			-- Loads audiovisual feedback
			GamePlaySound( "data/audio/Desktop/projectiles.snd", "player_projectiles/shield/block", projectile_x, projectile_y )
			EntityLoad("mods/ArcaneReliquary/files/entity_definitions/particle_explosions/" .. deflect_explosion_file_name .. ".xml", projectile_x, projectile_y)
			
			-- obtains settings from colliding projectile
			local projectile_definition_path = getProjectileDefinitionPath(projectile_id)
			local projectile_vel_x, projectile_vel_y = getProjectileVelocitiesandGravity(projectile_id)
			local new_default_vel_x = projectile_vel_x * -1
			local new_default_vel_y = projectile_vel_y * -1
			
			-- kills colliding projectile
			projectileToggleExplosionOnDeath(projectile_id, false)
			EntityKill(projectile_id)
			
			local scattered_projectiles_amount = Random(6,8)
			local vel_variation_threshold = 120

			for i=1,scattered_projectiles_amount do
				local new_projectile_id = EntityLoad(projectile_definition_path, projectile_x, projectile_y)
				
				if(effect_path ~= nil) then
					-- Adds effect to projectile
					attachLoadEffectOntoEntitiestoProjectile(new_projectile_id, effect_path, "spark_red")
				end
				
				-- Inverts projectile velocity, turning it back to the direction it came
				setProjectileVelocitiesandGravity(new_projectile_id, new_default_vel_x + Random(-vel_variation_threshold, vel_variation_threshold), new_default_vel_y  + Random(-vel_variation_threshold, vel_variation_threshold))

				-- Sets the shooter of the deflected projectile as the creature under the effect
				setProjectileShooter(new_projectile_id, shield_wielder_id)

				-- Flags projectile as deflected to prevent it from changing again
				addNewSimplifiedInternalVariable(new_projectile_id, "deflected")
			end
		end
	end
end

-- function to add enemy projectile convertion to other projectiles behavior to shield
function convertProjectiles(x, y, detectionRadius, shield_wielder_id, wand_rotation, custom_projectile_file_name, deflect_explosion_file_name)
	--local detection_y_offset = 10
	local projectiles = EntityGetInRadiusWithTag(x, y, detectionRadius, "projectile")
	for i, projectile_id in ipairs(projectiles) do
		-- conditions to prevent friendly projectiles from being redirected
		if(shield_wielder_id == getPlayerEntity() and entityHasSimplifiedInternalVariable(projectile_id, "relic_effect_projectile")) then return end
		if(string.find(EntityGetName(projectile_id), "rotating_shield_projectile")) then return end
		if(entityHasSimplifiedInternalVariable(projectile_id, "deflected")) then return end
		
		local projectile_x, projectile_y, projectile_rotation = EntityGetTransform(projectile_id)
		
		-- makeshift workaround to prevent shield from deflecting projectiles that come from behind the player
		if(sideRotation(wand_rotation) == sideRotation(projectile_rotation)) then return end
		
		if(getProjectileShooter(projectile_id) ~= shield_wielder_id) then
			-- Loads audiovisual feedback
			GamePlaySound( "data/audio/Desktop/projectiles.snd", "player_projectiles/shield/block", projectile_x, projectile_y )
			EntityLoad("mods/ArcaneReliquary/files/entity_definitions/particle_explosions/" .. deflect_explosion_file_name .. ".xml", projectile_x, projectile_y)
			
			-- obtains settings from colliding projectile
			local projectile_definition_path = "mods/ArcaneReliquary/files/entity_definitions/custom_projectiles/" .. custom_projectile_file_name .. ".xml"
			local projectile_vel_x, projectile_vel_y = getProjectileVelocitiesandGravity(projectile_id)
			local new_default_vel_x = projectile_vel_x * -1
			local new_default_vel_y = projectile_vel_y * -1
			
			-- kills colliding projectile
			projectileToggleExplosionOnDeath(projectile_id, false)
			EntityKill(projectile_id)
			
			local scattered_projectiles_amount = Random(6,8)
			local vel_variation_threshold = 120

			for i=1,scattered_projectiles_amount do
				local new_projectile_id = EntityLoad(projectile_definition_path, projectile_x, projectile_y)
				
				-- Inverts projectile velocity, turning it back to the direction it came
				setProjectileVelocitiesandGravity(new_projectile_id, new_default_vel_x + Random(-vel_variation_threshold, vel_variation_threshold), new_default_vel_y  + Random(-vel_variation_threshold, vel_variation_threshold))

				-- Sets the shooter of the deflected projectile as the creature under the effect
				setProjectileShooter(new_projectile_id, shield_wielder_id)

				-- Flags projectile as deflected to prevent it from changing again
				addNewSimplifiedInternalVariable(new_projectile_id, "deflected")
			end
		end
	end
end

-- function to add contact damage behavior to shield
function damageEnemies(x, y, detectionRadius, shield_wielder_id, require_player_moving, damage_explosion_path)

	local target_tag = "enemy"
	local creatures = EntityGetInRadiusWithTag(x, y, detectionRadius, target_tag)

	-- damages and knocks enemies back
	for i, creature_id in ipairs(creatures) do
		if(creature_id ~= shield_wielder_id) then -- only knocks back creatures other than the entity under this effect
			if(Random(1,2) == 1 and ((require_player_moving and isPlayerMoving()) or not require_player_moving)) then
				-- First obtains velocity of entity
				local entity_vel_x, entity_vel_y = getCreatureVelocities(shield_wielder_id)
				
				-- then applies the same velocity to creature caught
				setCreatureVelocities(creature_id, entity_vel_x, entity_vel_y)

				-- Loads explosion on creature's coordinates
				local creature_x, creature_y = EntityGetTransform(creature_id)
				EntityLoad(damage_explosion_path, creature_x, creature_y)
			end
		end	
	end
end

-- function to add enemy inflict effect on contact behavior to shield
function inflictEffectEnemies(x, y, detectionRadius, shield_wielder_id, effect_path)

	local target_tag = "enemy"
	local creatures = EntityGetInRadiusWithTag(x, y, detectionRadius, target_tag)

	-- inflicts effect
	for i, creature_id in ipairs(creatures) do
		if(creature_id ~= shield_wielder_id) then
			
			if(Random(1,2) == 1 and isPlayerMoving()) then
				local creature_x, creature_y = EntityGetTransform(creature_id)
				GamePlaySound( "data/audio/Desktop/projectiles.snd", "player_projectiles/shield/block", creature_x, creature_y )				
				LoadGameEffectEntityTo( creature_id, effect_path )
			end
			
		end	
	end
end

-- function to add enemy transmute to material on contact behavior to shield
function transmuteEnemies(x, y, detectionRadius, shield_wielder_id, materialName)
	local target_tag = "enemy"
	local creatures = EntityGetInRadiusWithTag(x, y, detectionRadius, target_tag)

	-- transmutes to material
	for i, creature_id in ipairs(creatures) do
		if(creature_id ~= shield_wielder_id) then
			
			if(Random(1,2) == 1 and isPlayerMoving()) then
				local creature_x, creature_y = EntityGetTransform(creature_id)
				GamePlaySound( "data/audio/Desktop/projectiles.snd", "player_projectiles/shield/block", creature_x, creature_y )				
				EntityConvertToMaterial( creature_id, materialName)
				EntityKill(creature_id)
			end
			
		end	
	end
end

SetRandomSeed( GameGetFrameNum(), math.random(1,50))

local parry_shield_id = GetUpdatedEntityID()
local parry_shield_handler_name = EntityGetName(parry_shield_id)
local shield_x, shield_y = EntityGetTransform(parry_shield_id)
local sc = EntityGetFirstComponent( parry_shield_id, "SpriteComponent", "sprite" )

local wand_id = getCurrentlyEquippedWandId()

if(wand_id ~= -1) then
	-- enables sprite to make it visible
	ComponentSetValue2(sc, "visible", true)
	-- gets player position
	local wielder_id = EntityGetParent(parry_shield_id)
	local wielder_x, wielder_y, wielder_rotation = EntityGetTransform(wielder_id)
	-- sets position and rotation of the shield according to the wand currently held
	local wand_x, wand_y, wand_rotation = EntityGetTransform(wand_id)
	EntitySetTransform(parry_shield_id, wielder_x, wielder_y - 6, wand_rotation)
	
	-- control variables
	local default_parry_range = 15
	local default_enemy_contact_range = 10
	
	-- parry shield variation functions
	-- --------------------------------
	-- regular parry shield behavior
	if(parry_shield_handler_name == "parry_shield_handler") then
		deflectProjectiles(shield_x, shield_y, default_parry_range, true, wielder_id, wand_rotation, nil, nil, nil)
	end
	
	-- steel parry shield behavior
	if(parry_shield_handler_name == "parry_shield_steel_handler") then
		destroyProjectiles(shield_x, shield_y, default_parry_range, wielder_id, wand_rotation, false, false, nil)
		switchAnimationByRotation(parry_shield_id, wand_rotation)
		
		-- signals relic functions this shield is active (variable should be removed when no shield is active / when no wand is being held)
		if(not entityHasSimplifiedInternalVariable(getPlayerEntity(), "parry_shield_steel_active")) then
			addNewSimplifiedInternalVariable( getPlayerEntity(), "parry_shield_steel_active" )
		end
	end
	
	-- rubber parry shield behavior
	if(parry_shield_handler_name == "parry_shield_rubber_handler") then
		deflectProjectiles(shield_x, shield_y, default_parry_range, false, wielder_id, wand_rotation, nil, nil, nil)
		switchAnimationByRotation(parry_shield_id, wand_rotation)
		
		-- signals relic functions this shield is active (variable should be removed when no shield is active / when no wand is being held)
		if(not entityHasSimplifiedInternalVariable(getPlayerEntity(), "parry_shield_rubber_active")) then
			addNewSimplifiedInternalVariable( getPlayerEntity(), "parry_shield_rubber_active" )
		end
	end
	
	-- riot parry shield behavior
	if(parry_shield_handler_name == "parry_shield_riot_handler") then
		deflectProjectiles(shield_x, shield_y, default_parry_range, false, wielder_id, wand_rotation, nil, nil, nil)
		damageEnemies(shield_x, shield_y, default_enemy_contact_range, wielder_id, true, "mods/ArcaneReliquary/files/entity_definitions/others/melee_explosion.xml")
		switchAnimationByRotation(parry_shield_id, wand_rotation)
		
		-- signals relic functions this shield is active (variable should be removed when no shield is active / when no wand is being held)
		if(not entityHasSimplifiedInternalVariable(getPlayerEntity(), "parry_shield_riot_active")) then
			addNewSimplifiedInternalVariable( getPlayerEntity(), "parry_shield_riot_active" )
		end
	end
	
	-- blast parry shield behavior
	if(parry_shield_handler_name == "parry_shield_blast_handler") then
		deflectProjectiles(shield_x, shield_y, default_parry_range, false, wielder_id, wand_rotation, nil, nil, "mods/ArcaneReliquary/files/entity_definitions/others/heavy_explosion.xml")
		switchAnimationByRotation(parry_shield_id, wand_rotation)
		
		-- signals relic functions this shield is active (variable should be removed when no shield is active / when no wand is being held)
		if(not entityHasSimplifiedInternalVariable(getPlayerEntity(), "parry_shield_blast_active")) then
			addNewSimplifiedInternalVariable( getPlayerEntity(), "parry_shield_blast_active" )
		end
	end
	
	-- ceramic parry shield behavior
	if(parry_shield_handler_name == "parry_shield_ceramic_handler") then
		deflectProjectiles(shield_x, shield_y, default_parry_range, false, wielder_id, wand_rotation, nil, nil, "mods/ArcaneReliquary/files/entity_definitions/load_on_explosion/launch_elemental_particles_fire.xml")
		switchAnimationByRotation(parry_shield_id, wand_rotation)
		
		-- signals relic functions this shield is active (variable should be removed when no shield is active / when no wand is being held)
		if(not entityHasSimplifiedInternalVariable(getPlayerEntity(), "parry_shield_ceramic_active")) then
			addNewSimplifiedInternalVariable( getPlayerEntity(), "parry_shield_ceramic_active" )
		end
	end
	
	-- lead parry shield behavior
	if(parry_shield_handler_name == "parry_shield_lead_handler") then
		deflectProjectiles(shield_x, shield_y, default_parry_range, false, wielder_id, wand_rotation, nil, nil, nil)
		switchAnimationByRotation(parry_shield_id, wand_rotation)
		
		-- signals relic functions this shield is active (variable should be removed when no shield is active / when no wand is being held)
		if(not entityHasSimplifiedInternalVariable(getPlayerEntity(), "parry_shield_lead_active")) then
			addNewSimplifiedInternalVariable( getPlayerEntity(), "parry_shield_lead_active" )
		end
	end
	
	-- plastic parry shield behavior
	if(parry_shield_handler_name == "parry_shield_plastic_handler") then
		deflectProjectiles(shield_x, shield_y, default_parry_range, false, wielder_id, wand_rotation, nil, nil, "mods/ArcaneReliquary/files/entity_definitions/load_on_explosion/launch_circular_projectiles_thunderbolts_regular.xml")
		switchAnimationByRotation(parry_shield_id, wand_rotation)
		
		-- signals relic functions this shield is active (variable should be removed when no shield is active / when no wand is being held)
		if(not entityHasSimplifiedInternalVariable(getPlayerEntity(), "parry_shield_plastic_active")) then
			addNewSimplifiedInternalVariable( getPlayerEntity(), "parry_shield_plastic_active" )
		end
	end
	
	-- spiked parry shield behavior
	if(parry_shield_handler_name == "parry_shield_spikes_handler") then
		deflectProjectiles(shield_x, shield_y, default_parry_range, false, wielder_id, wand_rotation, nil, nil, nil)
		damageEnemies(shield_x, shield_y, default_enemy_contact_range, wielder_id, true, "mods/ArcaneReliquary/files/entity_definitions/others/melee_explosion.xml")
	end
	
	-- slap parry shield behavior
	if(parry_shield_handler_name == "parry_shield_slap_handler") then
		convertProjectiles(shield_x, shield_y, default_parry_range, wielder_id, wand_rotation, "slapperbomb", "shield_deflect_explosion_green")
		inflictEffectEnemies(shield_x, shield_y, default_enemy_contact_range, wielder_id, getCustomEffect("effect_slap").path)
	end
	
	-- acid parry shield behavior
	if(parry_shield_handler_name == "parry_shield_acid_handler") then
		convertProjectiles(shield_x, shield_y, default_parry_range, wielder_id, wand_rotation, "magic_orb_acid", "shield_deflect_explosion_green")
		transmuteEnemies(shield_x, shield_y, default_enemy_contact_range, wielder_id, "acid")
	end
	
	-- explosive parry shield behavior
	if(parry_shield_handler_name == "parry_shield_explosive_handler") then
		convertProjectiles(shield_x, shield_y, default_parry_range, wielder_id, wand_rotation, "magic_orb_pink_explosive", "shield_deflect_explosion_pink")
		inflictEffectEnemies(shield_x, shield_y, default_enemy_contact_range, wielder_id, getCustomEffect("effect_exploding").path)
	end
	
	-- fire parry shield behavior
	if(parry_shield_handler_name == "parry_shield_fire_handler") then
		convertProjectiles(shield_x, shield_y, default_parry_range, wielder_id, wand_rotation, "grenade_custom", "shield_deflect_explosion_lava")
		inflictEffectEnemies(shield_x, shield_y, default_enemy_contact_range, wielder_id, "data/entities/misc/effect_on_fire.xml")
	end
	
	-- lava parry shield behavior
	if(parry_shield_handler_name == "parry_shield_lava_handler") then
		convertProjectiles(shield_x, shield_y, default_parry_range, wielder_id, wand_rotation, "magic_orb_lava", "shield_deflect_explosion_lava")
		transmuteEnemies(shield_x, shield_y, default_enemy_contact_range, wielder_id, "lava")
	end
	
	-- poison parry shield behavior
	if(parry_shield_handler_name == "parry_shield_poison_handler") then
		convertProjectiles(shield_x, shield_y, default_parry_range, wielder_id, wand_rotation, "magic_orb_poison", "shield_deflect_explosion_purple")
		transmuteEnemies(shield_x, shield_y, default_enemy_contact_range, wielder_id, "poison")
	end
	
	-- water parry shield behavior
	if(parry_shield_handler_name == "parry_shield_water_handler") then
		convertProjectiles(shield_x, shield_y, default_parry_range, wielder_id, wand_rotation, "magic_orb_water", "shield_deflect_explosion_water")
		transmuteEnemies(shield_x, shield_y, default_enemy_contact_range, wielder_id, "water")
	end
	
	-- blood parry shield behavior
	if(parry_shield_handler_name == "parry_shield_blood_handler") then
		convertProjectiles(shield_x, shield_y, default_parry_range, wielder_id, wand_rotation, "magic_orb_corrosive_blood", "shield_deflect_explosion_blood")
		transmuteEnemies(shield_x, shield_y, default_enemy_contact_range, wielder_id, "corrosive_blood")
	end
	
	-- poly parry shield behavior
	if(parry_shield_handler_name == "parry_shield_poly_handler") then
		convertProjectiles(shield_x, shield_y, default_parry_range, wielder_id, wand_rotation, "magic_orb_poly", "shield_deflect_explosion_pink")
		transmuteEnemies(shield_x, shield_y, default_enemy_contact_range, wielder_id, "magic_liquid_polymorph")
	end
	
	-- urine parry shield behavior
	if(parry_shield_handler_name == "parry_shield_urine_handler") then
		convertProjectiles(shield_x, shield_y, default_parry_range, wielder_id, wand_rotation, "magic_orb_urine", "shield_deflect_explosion_yellow")
		transmuteEnemies(shield_x, shield_y, default_enemy_contact_range, wielder_id, "urine")
	end
	
	-- explode parry shield behavior
	if(parry_shield_handler_name == "parry_shield_explode_handler") then
		convertProjectiles(shield_x, shield_y, default_parry_range, wielder_id, wand_rotation, "explodebomb", "shield_deflect_explosion_fire")
		inflictEffectEnemies(shield_x, shield_y, default_enemy_contact_range, wielder_id, getCustomEffect("effect_exploding").path)
	end
	
	-- slash parry shield behavior
	if(parry_shield_handler_name == "parry_shield_slash_handler") then
		convertProjectiles(shield_x, shield_y, default_parry_range, wielder_id, wand_rotation, "slashbomb", "shield_deflect_explosion_purple")
		inflictEffectEnemies(shield_x, shield_y, default_enemy_contact_range, wielder_id, getCustomEffect("effect_slash").path)
	end
	
	-- freeze parry shield behavior
	if(parry_shield_handler_name == "parry_shield_freeze_handler") then
		convertProjectiles(shield_x, shield_y, default_parry_range, wielder_id, wand_rotation, "freezebomb", "shield_deflect_explosion_cyan")
		inflictEffectEnemies(shield_x, shield_y, default_enemy_contact_range, wielder_id, "data/entities/misc/effect_frozen.xml")
	end
	
	-- electrocute parry shield behavior
	if(parry_shield_handler_name == "parry_shield_electrocute_handler") then
		convertProjectiles(shield_x, shield_y, default_parry_range, wielder_id, wand_rotation, "electrobomb", "shield_deflect_explosion_dark_blue")
		inflictEffectEnemies(shield_x, shield_y, default_enemy_contact_range, wielder_id, "data/entities/misc/effect_electricity.xml")
	end
	
	-- wind parry shield behavior
	if(parry_shield_handler_name == "parry_shield_wind_handler") then
		convertProjectiles(shield_x, shield_y, default_parry_range, wielder_id, wand_rotation, "windbomb", "shield_deflect_explosion_white")
		inflictEffectEnemies(shield_x, shield_y, default_enemy_contact_range, wielder_id, getCustomEffect("effect_wind_heavy").path)
	end
	
	-- bleed parry shield behavior
	if(parry_shield_handler_name == "parry_shield_bleed_handler") then
		convertProjectiles(shield_x, shield_y, default_parry_range, wielder_id, wand_rotation, "bloodbomb", "shield_deflect_explosion_red")
		inflictEffectEnemies(shield_x, shield_y, default_enemy_contact_range, wielder_id, getCustomEffect("effect_bleeding").path)
	end
	
	-- teleport parry shield behavior
	if(parry_shield_handler_name == "parry_shield_teleport_handler") then
		convertProjectiles(shield_x, shield_y, default_parry_range, wielder_id, wand_rotation, "teleportbomb", "shield_deflect_explosion_blue")
		inflictEffectEnemies(shield_x, shield_y, default_enemy_contact_range, wielder_id, "data/entities/misc/effect_teleportation.xml")
	end
	
	-- bubbletrap parry shield behavior
	if(parry_shield_handler_name == "parry_shield_bubbletrap_handler") then
		convertProjectiles(shield_x, shield_y, default_parry_range, wielder_id, wand_rotation, "bubbletrap", "shield_deflect_explosion_cyan")
		inflictEffectEnemies(shield_x, shield_y, default_enemy_contact_range, wielder_id, getCustomEffect("effect_bubbletrap").path)
	end
	
	-- green spiked parry shield behavior
	if(parry_shield_handler_name == "parry_shield_spikes_green_handler") then
		deflectProjectiles(shield_x, shield_y, default_parry_range, false, wielder_id, wand_rotation, getCustomEffect("effect_material_spit_acid").path, nil, nil)
		inflictEffectEnemies(shield_x, shield_y, default_enemy_contact_range, wielder_id, getCustomEffect("effect_material_spit_acid").path)
	end
	
	-- orange spiked parry shield behavior
	if(parry_shield_handler_name == "parry_shield_spikes_orange_handler") then
		deflectProjectiles(shield_x, shield_y, default_parry_range, false, wielder_id, wand_rotation, getCustomEffect("effect_material_spit_lava").path, nil, nil)
		inflictEffectEnemies(shield_x, shield_y, default_enemy_contact_range, wielder_id, getCustomEffect("effect_material_spit_lava").path)
	end
	
	-- red spiked parry shield behavior
	if(parry_shield_handler_name == "parry_shield_spikes_red_handler") then
		deflectProjectiles(shield_x, shield_y, default_parry_range, false, wielder_id, wand_rotation, getCustomEffect("effect_material_spit_corrosive_blood").path, nil, nil)
		inflictEffectEnemies(shield_x, shield_y, default_enemy_contact_range, wielder_id, getCustomEffect("effect_material_spit_corrosive_blood").path)
	end
	
	-- pink spiked parry shield behavior
	if(parry_shield_handler_name == "parry_shield_spikes_pink_handler") then
		deflectProjectiles(shield_x, shield_y, default_parry_range, false, wielder_id, wand_rotation, getCustomEffect("effect_material_spit_poly").path, nil, nil)
		inflictEffectEnemies(shield_x, shield_y, default_enemy_contact_range, wielder_id, getCustomEffect("effect_material_spit_poly").path)
	end
	
	-- blue spiked parry shield behavior
	if(parry_shield_handler_name == "parry_shield_spikes_blue_handler") then
		deflectProjectiles(shield_x, shield_y, default_parry_range, false, wielder_id, wand_rotation, getCustomEffect("effect_material_spit_water").path, nil, nil)
		inflictEffectEnemies(shield_x, shield_y, default_enemy_contact_range, wielder_id, getCustomEffect("effect_material_spit_water").path)
	end
	
	-- purple spiked parry shield behavior
	if(parry_shield_handler_name == "parry_shield_spikes_purple_handler") then
		deflectProjectiles(shield_x, shield_y, default_parry_range, false, wielder_id, wand_rotation, getCustomEffect("effect_material_spit_poison").path, nil, nil)
		inflictEffectEnemies(shield_x, shield_y, default_enemy_contact_range, wielder_id, getCustomEffect("effect_material_spit_poison").path)
	end
	
	-- fiery spiked parry shield behavior
	if(parry_shield_handler_name == "parry_shield_spikes_fiery_handler") then
		deflectProjectiles(shield_x, shield_y, default_parry_range, false, wielder_id, wand_rotation, getCustomEffect("effect_material_spit_gunpowder").path, nil, nil)
		inflictEffectEnemies(shield_x, shield_y, default_enemy_contact_range, wielder_id, getCustomEffect("effect_material_spit_gunpowder").path)
	end
	
	-- yellow spiked parry shield behavior
	if(parry_shield_handler_name == "parry_shield_spikes_yellow_handler") then
		deflectProjectiles(shield_x, shield_y, default_parry_range, false, wielder_id, wand_rotation, getCustomEffect("effect_material_spit_urine").path, nil, nil)
		inflictEffectEnemies(shield_x, shield_y, default_enemy_contact_range, wielder_id, getCustomEffect("effect_material_spit_urine").path)
	end
	
	-- cyan spiked parry shield behavior
	if(parry_shield_handler_name == "parry_shield_spikes_cyan_handler") then
		deflectProjectiles(shield_x, shield_y, default_parry_range, false, wielder_id, wand_rotation, getCustomEffect("effect_material_spit_freezing_liquid").path, nil, nil)
		inflictEffectEnemies(shield_x, shield_y, default_enemy_contact_range, wielder_id, getCustomEffect("effect_material_spit_freezing_liquid").path)
	end
	
	-- brickwall parry shield behavior
	if(parry_shield_handler_name == "parry_shield_brickwall_handler") then
		destroyProjectiles(shield_x, shield_y, default_parry_range, wielder_id, wand_rotation, false, false, nil)
		damageEnemies(shield_x, shield_y, default_enemy_contact_range, wielder_id, true, "mods/ArcaneReliquary/files/entity_definitions/others/melee_explosion.xml")
	end
	
	-- blades parry shield behavior
	if(parry_shield_handler_name == "parry_shield_blades_handler") then
		redirectProjectiles(shield_x, shield_y, default_parry_range, true, wielder_id, wand_rotation)
		damageEnemies(shield_x, shield_y, default_enemy_contact_range, wielder_id, false, "mods/ArcaneReliquary/files/entity_definitions/others/slice_explosion.xml")
	end
	
	-- concretewall parry shield behavior
	if(parry_shield_handler_name == "parry_shield_concretewall_handler") then
		deflectProjectiles(shield_x, shield_y, default_parry_range, false, wielder_id, wand_rotation, nil, "melee", nil)
		damageEnemies(shield_x, shield_y, default_enemy_contact_range, wielder_id, true, "mods/ArcaneReliquary/files/entity_definitions/others/melee_explosion.xml")
	end
	
	-- blades (slice) parry shield behavior
	if(parry_shield_handler_name == "parry_shield_blades_slice_handler") then
		deflectProjectiles(shield_x, shield_y, default_parry_range, false, wielder_id, wand_rotation, nil, "slice", nil)
		damageEnemies(shield_x, shield_y, default_enemy_contact_range, wielder_id, false, "mods/ArcaneReliquary/files/entity_definitions/others/slice_explosion.xml")
	end
	
	-- scatter parry shield behavior
	if(parry_shield_handler_name == "parry_shield_scatter_handler") then
		scatterProjectiles(shield_x, shield_y, default_parry_range, wielder_id, wand_rotation, "shield_deflect_explosion_red", getCustomEffect("effect_bleeding").path)
		damageEnemies(shield_x, shield_y, default_enemy_contact_range, wielder_id, false, "mods/ArcaneReliquary/files/entity_definitions/others/slice_explosion.xml")
		upwardMomentum(wand_rotation)
	end
	
	-- scatter freeze parry shield behavior
	if(parry_shield_handler_name == "parry_shield_scatter_freeze_handler") then
		scatterProjectiles(shield_x, shield_y, default_parry_range, wielder_id, wand_rotation, "shield_deflect_explosion_cyan", "data/entities/misc/effect_frozen.xml")
		damageEnemies(shield_x, shield_y, default_enemy_contact_range, wielder_id, false, "mods/ArcaneReliquary/files/entity_definitions/others/slice_explosion.xml")
		upwardMomentum(wand_rotation)
	end
	
	-- scatter electrocute parry shield behavior
	if(parry_shield_handler_name == "parry_shield_scatter_electrocute_handler") then
		scatterProjectiles(shield_x, shield_y, default_parry_range, wielder_id, wand_rotation, "shield_deflect_explosion_dark_blue", "data/entities/misc/effect_electricity.xml")
		damageEnemies(shield_x, shield_y, default_enemy_contact_range, wielder_id, false, "mods/ArcaneReliquary/files/entity_definitions/others/slice_explosion.xml")
		upwardMomentum(wand_rotation)
	end
	
	-- scatter poly parry shield behavior
	if(parry_shield_handler_name == "parry_shield_scatter_poly_handler") then
		scatterProjectiles(shield_x, shield_y, default_parry_range, wielder_id, wand_rotation, "shield_deflect_explosion_pink", "data/entities/misc/effect_polymorph.xml")
		damageEnemies(shield_x, shield_y, default_enemy_contact_range, wielder_id, false, "mods/ArcaneReliquary/files/entity_definitions/others/slice_explosion.xml")
		upwardMomentum(wand_rotation)
	end
	
	-- scatter explode parry shield behavior
	if(parry_shield_handler_name == "parry_shield_scatter_explode_handler") then
		scatterProjectiles(shield_x, shield_y, default_parry_range, wielder_id, wand_rotation, "shield_deflect_explosion_lava", getCustomEffect("effect_exploding").path)
		damageEnemies(shield_x, shield_y, default_enemy_contact_range, wielder_id, false, "mods/ArcaneReliquary/files/entity_definitions/others/slice_explosion.xml")
		upwardMomentum(wand_rotation)
	end
	
	-- scatter wind parry shield behavior
	if(parry_shield_handler_name == "parry_shield_scatter_wind_handler") then
		scatterProjectiles(shield_x, shield_y, default_parry_range, wielder_id, wand_rotation, "shield_deflect_explosion_white", getCustomEffect("effect_wind_heavy").path)
		damageEnemies(shield_x, shield_y, default_enemy_contact_range, wielder_id, false, "mods/ArcaneReliquary/files/entity_definitions/others/slice_explosion.xml")
		upwardMomentum(wand_rotation)
	end
	
	-- scatter slap parry shield behavior
	if(parry_shield_handler_name == "parry_shield_scatter_slap_handler") then
		scatterProjectiles(shield_x, shield_y, default_parry_range, wielder_id, wand_rotation, "shield_deflect_explosion_green", getCustomEffect("effect_slap").path)
		damageEnemies(shield_x, shield_y, default_enemy_contact_range, wielder_id, false, "mods/ArcaneReliquary/files/entity_definitions/others/slice_explosion.xml")
		upwardMomentum(wand_rotation)
	end
	
	-- scatter slash parry shield behavior
	if(parry_shield_handler_name == "parry_shield_scatter_slash_handler") then
		scatterProjectiles(shield_x, shield_y, default_parry_range, wielder_id, wand_rotation, "shield_deflect_explosion_purple", getCustomEffect("effect_slash").path)
		damageEnemies(shield_x, shield_y, default_enemy_contact_range, wielder_id, false, "mods/ArcaneReliquary/files/entity_definitions/others/slice_explosion.xml")
		upwardMomentum(wand_rotation)
	end
	
	-- umbrella parry shield behavior
	if(parry_shield_handler_name == "parry_shield_umbrella_handler") then
		scatterProjectiles(shield_x, shield_y, default_parry_range, wielder_id, wand_rotation, "shield_deflect_explosion_red", "data/entities/misc/effect_curse_cloud_01_temporary.xml")
		inflictEffectEnemies(shield_x, shield_y, default_enemy_contact_range, wielder_id, "data/entities/misc/effect_curse_cloud_01_temporary.xml")
		glide(wand_rotation)
	end
	
	-- umbrella green parry shield behavior
	if(parry_shield_handler_name == "parry_shield_umbrella_green_handler") then
		scatterProjectiles(shield_x, shield_y, default_parry_range, wielder_id, wand_rotation, "shield_deflect_explosion_green", getCustomEffect("effect_cursed_cloud_acid").path)
		inflictEffectEnemies(shield_x, shield_y, default_enemy_contact_range, wielder_id, getCustomEffect("effect_cursed_cloud_acid").path)
		glide(wand_rotation)
	end
	
	-- umbrella pink parry shield behavior
	if(parry_shield_handler_name == "parry_shield_umbrella_pink_handler") then
		scatterProjectiles(shield_x, shield_y, default_parry_range, wielder_id, wand_rotation, "shield_deflect_explosion_pink", getCustomEffect("effect_cursed_cloud_poly").path)
		inflictEffectEnemies(shield_x, shield_y, default_enemy_contact_range, wielder_id, getCustomEffect("effect_cursed_cloud_poly").path)
		glide(wand_rotation)
	end
	
	-- umbrella purple parry shield behavior
	if(parry_shield_handler_name == "parry_shield_umbrella_purple_handler") then
		scatterProjectiles(shield_x, shield_y, default_parry_range, wielder_id, wand_rotation, "shield_deflect_explosion_purple", getCustomEffect("effect_cursed_cloud_poison").path)
		inflictEffectEnemies(shield_x, shield_y, default_enemy_contact_range, wielder_id, getCustomEffect("effect_cursed_cloud_poison").path)
		glide(wand_rotation)
	end
	
	-- umbrella green lime parry shield behavior
	if(parry_shield_handler_name == "parry_shield_umbrella_green_lime_handler") then
		scatterProjectiles(shield_x, shield_y, default_parry_range, wielder_id, wand_rotation, "shield_deflect_explosion_green", getCustomEffect("effect_cursed_cloud_toxic").path)
		inflictEffectEnemies(shield_x, shield_y, default_enemy_contact_range, wielder_id, getCustomEffect("effect_cursed_cloud_toxic").path)
		glide(wand_rotation)
	end
	
	-- umbrella yellow parry shield behavior
	if(parry_shield_handler_name == "parry_shield_umbrella_yellow_handler") then
		scatterProjectiles(shield_x, shield_y, default_parry_range, wielder_id, wand_rotation, "shield_deflect_explosion_yellow", getCustomEffect("effect_cursed_cloud_urine").path)
		inflictEffectEnemies(shield_x, shield_y, default_enemy_contact_range, wielder_id, getCustomEffect("effect_cursed_cloud_urine").path)
		glide(wand_rotation)
	end
	
	-- umbrella orange parry shield behavior
	if(parry_shield_handler_name == "parry_shield_umbrella_orange_handler") then
		scatterProjectiles(shield_x, shield_y, default_parry_range, wielder_id, wand_rotation, "shield_deflect_explosion_lava", getCustomEffect("effect_cursed_cloud_lava").path)
		inflictEffectEnemies(shield_x, shield_y, default_enemy_contact_range, wielder_id, getCustomEffect("effect_cursed_cloud_lava").path)
		glide(wand_rotation)
	end
	
	-- umbrella pink dark parry shield behavior
	if(parry_shield_handler_name == "parry_shield_umbrella_pink_dark_handler") then
		scatterProjectiles(shield_x, shield_y, default_parry_range, wielder_id, wand_rotation, "shield_deflect_explosion_pink", getCustomEffect("effect_cursed_cloud_slime").path)
		inflictEffectEnemies(shield_x, shield_y, default_enemy_contact_range, wielder_id, getCustomEffect("effect_cursed_cloud_slime").path)
		glide(wand_rotation)
	end
	
	-- metal parry shield behavior
	if(parry_shield_handler_name == "parry_shield_metal_handler") then
		destroyProjectiles(shield_x, shield_y, default_parry_range, wielder_id, wand_rotation, true, false, nil)
		switchAnimationByRotation(parry_shield_id, wand_rotation)
	end
	
	-- metal bleed parry shield behavior
	if(parry_shield_handler_name == "parry_shield_metal_bleed_handler") then
		destroyProjectiles(shield_x, shield_y, default_parry_range, wielder_id, wand_rotation, true, false, getCustomEffect("effect_bleeding").path)
		switchAnimationByRotation(parry_shield_id, wand_rotation)
	end
	
	-- metal freeze parry shield behavior
	if(parry_shield_handler_name == "parry_shield_metal_freeze_handler") then
		destroyProjectiles(shield_x, shield_y, default_parry_range, wielder_id, wand_rotation, true, false, "data/entities/misc/effect_frozen.xml")
		switchAnimationByRotation(parry_shield_id, wand_rotation)
	end
	
	-- metal electrocute parry shield behavior
	if(parry_shield_handler_name == "parry_shield_metal_electrocute_handler") then
		destroyProjectiles(shield_x, shield_y, default_parry_range, wielder_id, wand_rotation, true, false, "data/entities/misc/effect_electricity.xml")
		switchAnimationByRotation(parry_shield_id, wand_rotation)
	end
	
	-- metal poly parry shield behavior
	if(parry_shield_handler_name == "parry_shield_metal_poly_handler") then
		destroyProjectiles(shield_x, shield_y, default_parry_range, wielder_id, wand_rotation, true, false, "data/entities/misc/effect_polymorph.xml")
		switchAnimationByRotation(parry_shield_id, wand_rotation)
	end
	
	-- metal explode parry shield behavior
	if(parry_shield_handler_name == "parry_shield_metal_explode_handler") then
		destroyProjectiles(shield_x, shield_y, default_parry_range, wielder_id, wand_rotation, true, false, getCustomEffect("effect_exploding").path)
		switchAnimationByRotation(parry_shield_id, wand_rotation)
	end
	
	-- metal slap parry shield behavior
	if(parry_shield_handler_name == "parry_shield_metal_slap_handler") then
		destroyProjectiles(shield_x, shield_y, default_parry_range, wielder_id, wand_rotation, true, false, getCustomEffect("effect_slap").path)
		switchAnimationByRotation(parry_shield_id, wand_rotation)
	end
	
	-- metal wind parry shield behavior
	if(parry_shield_handler_name == "parry_shield_metal_wind_handler") then
		destroyProjectiles(shield_x, shield_y, default_parry_range, wielder_id, wand_rotation, true, false, getCustomEffect("effect_wind_heavy").path)
		switchAnimationByRotation(parry_shield_id, wand_rotation)
	end
	
	-- metal slash parry shield behavior
	if(parry_shield_handler_name == "parry_shield_metal_slash_handler") then
		destroyProjectiles(shield_x, shield_y, default_parry_range, wielder_id, wand_rotation, true, false, getCustomEffect("effect_slash").path)
		switchAnimationByRotation(parry_shield_id, wand_rotation)
	end
	
	-- metal magnet parry shield behavior
	if(parry_shield_handler_name == "parry_shield_metal_magnet_handler") then
		destroyProjectiles(shield_x, shield_y, default_parry_range, wielder_id, wand_rotation, true, false, getCustomEffect("effect_attraction").path)
		switchAnimationByRotation(parry_shield_id, wand_rotation)
	end
	
	-- metal teleport parry shield behavior
	if(parry_shield_handler_name == "parry_shield_metal_teleport_handler") then
		destroyProjectiles(shield_x, shield_y, default_parry_range, wielder_id, wand_rotation, true, false, "data/entities/misc/effect_teleportation.xml")
		switchAnimationByRotation(parry_shield_id, wand_rotation)
	end
	
	-- medical parry shield behavior
	if(parry_shield_handler_name == "parry_shield_medical_handler") then
		destroyProjectiles(shield_x, shield_y, default_parry_range, wielder_id, wand_rotation, true, true, nil)
		switchAnimationByRotation(parry_shield_id, wand_rotation)
	end
	
	-- --------------------------------
else
	-- signals relic functions parry steel shield is no longer active
	if(entityHasSimplifiedInternalVariable(getPlayerEntity(), "parry_shield_steel_active")) then
		removeSimplifiedInternalVariable(getPlayerEntity(), "parry_shield_steel_active" )
	end
	-- signals relic functions parry rubber shield is no longer active
	if(entityHasSimplifiedInternalVariable(getPlayerEntity(), "parry_shield_rubber_active")) then
		removeSimplifiedInternalVariable(getPlayerEntity(), "parry_shield_rubber_active" )
	end
	-- signals relic functions parry riot shield is no longer active
	if(entityHasSimplifiedInternalVariable(getPlayerEntity(), "parry_shield_riot_active")) then
		removeSimplifiedInternalVariable(getPlayerEntity(), "parry_shield_riot_active" )
	end
	-- signals relic functions parry blast shield is no longer active
	if(entityHasSimplifiedInternalVariable(getPlayerEntity(), "parry_shield_blast_active")) then
		removeSimplifiedInternalVariable(getPlayerEntity(), "parry_shield_blast_active" )
	end
	-- signals relic functions parry ceramic shield is no longer active
	if(entityHasSimplifiedInternalVariable(getPlayerEntity(), "parry_shield_ceramic_active")) then
		removeSimplifiedInternalVariable(getPlayerEntity(), "parry_shield_ceramic_active" )
	end
	-- signals relic functions parry lead shield is no longer active
	if(entityHasSimplifiedInternalVariable(getPlayerEntity(), "parry_shield_lead_active")) then
		removeSimplifiedInternalVariable(getPlayerEntity(), "parry_shield_lead_active" )
	end
	-- signals relic functions parry plastic shield is no longer active
	if(entityHasSimplifiedInternalVariable(getPlayerEntity(), "parry_shield_plastic_active")) then
		removeSimplifiedInternalVariable(getPlayerEntity(), "parry_shield_plastic_active" )
	end
	-- disables sprite to make it invisible
	ComponentSetValue2(sc, "visible", false)
end
