dofile_once("data/scripts/lib/utilities.lua")
dofile_once("mods/ArcaneReliquary/files/scripts/utilities/arcanereliquary_utilities.lua")

local scatter = 0.1

local entity_id = GetUpdatedEntityID()
local pos_x, pos_y = EntityGetTransform( entity_id )

local owner_id = getInternalVariableValue(entity_id, "owner", "value_int")
local target_tag = ""
if(owner_id == getPlayerEntity()) then
	target_tag = "mortal"
else
	target_tag = "player_unit"
end

SetRandomSeed(pos_x + GameGetFrameNum(), pos_y)

local can_shoot = getInternalVariableValue(entity_id, "can_shoot", "value_bool")
if(can_shoot ~= nil and not can_shoot) then return end -- if can_shoot flag is present, but it is false, skips shooting

if(Random(1,2) == 1) then

	local projectile_definition = getInternalVariableValue(entity_id, "projectile_definition", "value_string")
	local projectile_amount = getInternalVariableValue(entity_id, "projectile_amount", "value_int")
	local detection_range = getInternalVariableValue(entity_id, "detection_range", "value_int")
	local projectile_velocity_min = getInternalVariableValue(entity_id, "vel_min", "value_int")
	local projectile_velocity_max = getInternalVariableValue(entity_id, "vel_max", "value_int")

	-- locate nearest enemy
	local enemy, enemy_x, enemy_y
	local min_dist = 9999
	for _,id in pairs(EntityGetInRadiusWithTag(pos_x, pos_y, detection_range, target_tag)) do
		local valid_target = true
		if(target_tag == "mortal") then
			valid_target = (EntityGetComponent(id, "GenomeDataComponent") ~= nil) and (EntityGetHerdRelation(owner_id, id) < 40) and (IsPlayer(owner_id) ~= IsPlayer(id))
		end
		-- is target a valid enemy
		if valid_target then
			local x, y = EntityGetFirstHitboxCenter( id )
			local dist = get_distance(pos_x, pos_y, x, y)
			if dist < min_dist then
				min_dist = dist
				enemy = id
				enemy_x = x
				enemy_y = y
			end
		end
	end

	if not enemy then return end

	-- check los
	if RaytraceSurfaces(pos_x, pos_y, enemy_x, enemy_y) then return end

	-- direction
	local vel_x, vel_y = vec_sub(enemy_x, enemy_y, pos_x, pos_y)
	vel_x,vel_y = vec_normalize(vel_x, vel_y)

	for i=1,projectile_amount do
		-- scatter
		local vx,vy = vec_rotate(vel_x,vel_y, rand(-scatter, scatter))

		-- apply velocity & shoot
		vx,vy = vec_mult(vx,vy, rand(projectile_velocity_min, projectile_velocity_max))
		local projectile_id = shoot_projectile( entity_id, projectile_definition, pos_x, pos_y, vx, vy)
		setProjectileShooter(projectile_id, owner_id) -- hacky way to prevent projectiles from hitting the player but also to prevent them from being affected by relics
		
		local rotating_projectile_definition = getInternalVariableValue(entity_id, "rotating_projectile_definition", "value_string")
		local secondary_projectile_amount = getInternalVariableValue(entity_id, "secondary_projectile_amount", "value_int")
		local radius = getInternalVariableValue(entity_id, "radius", "value_int")
		local rotation_speed = getInternalVariableValue(entity_id, "rotation_speed", "value_int")
		
		addNewInternalVariable(projectile_id, "rotating_projectile_definition", "value_string", rotating_projectile_definition)
		addNewInternalVariable(projectile_id, "rotating_projectiles_amount", "value_int", secondary_projectile_amount)
		addNewInternalVariable(projectile_id, "radius", "value_int", radius)
		addNewInternalVariable(projectile_id, "rotation_speed", "value_int", rotation_speed)
		addNewInternalVariable(projectile_id, "shooter_id", "value_int", owner_id)
		addNewLuaCallback(projectile_id, "overtime", "mods/ArcaneReliquary/files/scripts/projectiles/projectile_rotating_shield_start.lua", 1, true, "projectile_rotating_shield_start")
		
		local kill_on_shot = getInternalVariableValue(entity_id, "kill_on_shot", "value_bool")
		if(kill_on_shot ~= nil and kill_on_shot) then
			EntityKill(entity_id)
		end
	end
end