dofile_once("data/scripts/lib/utilities.lua")
dofile_once("mods/ArcaneReliquary/files/scripts/utilities/arcanereliquary_utilities.lua")

local entity_id = GetUpdatedEntityID()
local pos_x, pos_y = EntityGetTransform( entity_id )

local projectile_definition = getInternalVariableValue(entity_id, "projectile_definition", "value_string")
local detection_range = getInternalVariableValue(entity_id, "detection_range", "value_int")
local projectile_velocity = getInternalVariableValue(entity_id, "projectile_velocity", "value_int")
local target_tag = getInternalVariableValue(entity_id, "target_tag", "value_string")
local max_projectiles_shot = getInternalVariableValue(entity_id, "max_projectiles_shot", "value_int")

-- if maximum projectiles shot reaches zero, callback is removed
if(max_projectiles_shot <= 0) then
	removeCallback(entity_id, "target_and_shoot_projectile")
	return
end

local scatter = 0.1

-- locate nearest target
local target, target_x, target_y
local min_dist = 9999
for _,id in pairs(EntityGetInRadiusWithTag(pos_x, pos_y, detection_range, target_tag)) do
	local x, y = EntityGetFirstHitboxCenter( id )
	local dist = get_distance(pos_x, pos_y, x, y)
	if dist < min_dist then
		min_dist = dist
		target = id
		target_x = x
		target_y = y
	end
end

if not target then return end

local target_x, target_y = EntityGetTransform(target)

-- direction
local vel_x, vel_y = vec_sub(target_x, target_y, pos_x, pos_y)
vel_x,vel_y = vec_normalize(vel_x, vel_y)

-- scatter
SetRandomSeed(pos_x + GameGetFrameNum(), pos_y)
vel_x,vel_y = vec_rotate(vel_x,vel_y, rand(-scatter, scatter))

-- apply velocity & shoot
vel_x,vel_y = vec_mult(vel_x,vel_y, projectile_velocity)

local projectile_id = -1
if(target_tag == "enemy") then
	projectile_id = shoot_projectile( entity_id, projectile_definition, pos_x, pos_y, vel_x, vel_y)
	setProjectileShooter(projectile_id, getPlayerEntity()) -- hacky way to prevent projectiles from hitting the player but also to prevent them from being affected by relics
	addNewSimplifiedInternalVariable(projectile_id, "relic_effect_projectile")
elseif(target_tag == "player_unit") then
	projectile_id = shoot_projectile( entity_id, projectile_definition, pos_x, pos_y, vel_x, vel_y)
	setProjectileShooter(projectile_id, EntityGetClosestWithTag(  pos_x, pos_y, "enemy"))
end

-- decreases from maximum projectiles shot
setInternalVariableValue(entity_id, "max_projectiles_shot", "value_int", max_projectiles_shot - 1)