dofile_once("data/scripts/game_helpers.lua")
dofile_once("data/scripts/lib/utilities.lua")
dofile_once("data/scripts/gun/gun_enums.lua")
dofile_once("data/scripts/gun/gun_actions.lua")
dofile_once( "data/scripts/perks/perk.lua" )

function castMagicBox(material_name, x, y, entity_load_inside, material_drip_name)
	local lower_platform_id = EntityLoad("mods/ArcaneReliquary/files/entity_definitions/custom_projectiles/temporary_platform_custom.xml", x, y + 11)
	setInternalVariableValue(lower_platform_id, "wall_material_name", "value_string", material_name)
	local upper_platform_id = EntityLoad("mods/ArcaneReliquary/files/entity_definitions/custom_projectiles/temporary_platform_custom.xml", x, y - 23)
	setInternalVariableValue(upper_platform_id, "wall_material_name", "value_string", material_name)
	local left_wall_id = EntityLoad("mods/ArcaneReliquary/files/entity_definitions/custom_projectiles/temporary_wall_custom.xml", x - 18, y - 5)
	setInternalVariableValue(left_wall_id, "wall_material_name", "value_string", material_name)
	local right_wall_id = EntityLoad("mods/ArcaneReliquary/files/entity_definitions/custom_projectiles/temporary_wall_custom.xml", x + 18, y - 5)
	setInternalVariableValue(right_wall_id, "wall_material_name", "value_string", material_name)
	
	if(entity_load_inside ~= nil) then
		EntityLoad(entity_load_inside, x - 3, y - 15)
	end
	
	if(material_drip_name ~= nil) then
		local dripper_id = EntityLoad("mods/ArcaneReliquary/files/entity_definitions/custom_projectiles/temporary_dripper.xml", x, y - 20)
		setInternalVariableValue(dripper_id, "wall_material_name", "value_string", material_drip_name)
	end
end

function getBestSpawnCoordinatesNexttoMachine(machine_id)
	local x, y = EntityGetTransform( machine_id )

	local player_x, player_y = EntityGetTransform(getPlayerEntity())
	local item_spawn_offset_x = 0
	if(x >= player_x) then
		item_spawn_offset_x = 25 * -1
	else
		item_spawn_offset_x = 25
	end
	local spawn_coordinate_x = player_x + item_spawn_offset_x
	local spawn_coordinate_y = player_y - 15
	
	return spawn_coordinate_x, spawn_coordinate_y
end

function grabPrize(relic_prize, relic_base_dir, prize_machine_id, dispense_prize)
	if (dispense_prize) then
		
		local spawn_coordinate_x, spawn_coordinate_y = getBestSpawnCoordinatesNexttoMachine(prize_machine_id)
		
		-- spawns prize
		EntityLoad(relic_base_dir .. relic_prize.name .. ".xml", spawn_coordinate_x, spawn_coordinate_y)
		
		-- loads audiovisual feedback
		EntityLoad("mods/ArcaneReliquary/files/entity_definitions/others/relic_prize_machine_interact_effect.xml", spawn_coordinate_x, spawn_coordinate_y)
		
		-- shows success message
		message = relic_prize.ui_name .. " Caught!"
		GamePrint(message)
		GamePrintImportant(message, "", "mods/ArcaneReliquary/files/other_gfx/custom_decoration_claw_success.png")
	else
		-- shows fail message
		message = "You didn't catch " .. relic_prize.ui_name
		GamePrint(message)
		GamePrintImportant(message, "", "mods/ArcaneReliquary/files/other_gfx/custom_decoration_claw_fail.png")
		-- loads audio feedback
		local x, y = EntityGetTransform(prize_machine_id)
		GamePlaySound( "data/audio/Desktop/event_cues.snd", "event_cues/chest_bad/create", x, y )
	end
end

function tradeRelicforToken(relic_object, relic_base_dir, store_id)
	
	local spawn_coordinate_x, spawn_coordinate_y = getBestSpawnCoordinatesNexttoMachine(store_id)
	
	-- spawns relic
	EntityLoad(relic_base_dir .. relic_object.name .. ".xml", spawn_coordinate_x, spawn_coordinate_y)
	if(relic_object.requires_relic ~= nil) then -- if spawned relic requires another relic, spawns it together with main relic
		local required_relic = getRelic(relic_object.requires_relic)
		EntityLoad(relic_base_dir .. required_relic.name .. ".xml", spawn_coordinate_x, spawn_coordinate_y)
	end
	
	-- loads audiovisual feedback
	EntityLoad("mods/ArcaneReliquary/files/entity_definitions/others/relic_store_interact_effect.xml", spawn_coordinate_x, spawn_coordinate_y)
	
	-- shows message
	message = "You traded " .. relic_object.ui_name .. " for a relic token"
	GamePrint(message)
	GamePrintImportant(message, "", "mods/ArcaneReliquary/files/other_gfx/custom_decoration_store.png")
end

function buyUtilityItem(buy_price, item_path, item_ui_name, utility_store_id)
	-- checks if player can afford it
	if (getPlayerGold() >= buy_price) then
		local spawn_coordinate_x, spawn_coordinate_y = getBestSpawnCoordinatesNexttoMachine(utility_store_id)
		
		-- spawns bought item
		EntityLoad(item_path, spawn_coordinate_x, spawn_coordinate_y)
		
		-- loads audiovisual feedback
		EntityLoad("mods/ArcaneReliquary/files/entity_definitions/others/relic_store_interact_effect.xml", spawn_coordinate_x, spawn_coordinate_y)
		
		-- deducts gold from player
		addPlayerGold(buy_price * -1)
		
		-- shows message
		message = "You bought " .. item_ui_name .. " for " .. buy_price .. " gold"
		GamePrint(message)
		GamePrintImportant(message, "", "mods/ArcaneReliquary/files/other_gfx/custom_decoration_store.png")
		
	else
		-- shows error message when not enough gold
		message = "You need " .. buy_price .. " gold to buy " .. item_ui_name
		GamePrint(message)
		GamePrintImportant(message, "", "mods/ArcaneReliquary/files/other_gfx/custom_decoration_error.png")
		-- loads audio feedback
		local x, y = EntityGetTransform(utility_store_id)
		GamePlaySound( "data/audio/Desktop/event_cues.snd", "event_cues/chest_bad/create", x, y )
	end
end

function buyRelic(relic_buy_price, relic_object, relic_base_dir, store_id)

	if(entityHasSimplifiedInternalVariable(getPlayerEntity(), "relic_store_discount")) then
		relic_buy_price = relic_buy_price / 2
	end
	
	-- checks if player can afford it
	if (getPlayerGold() >= relic_buy_price) then
		
		local spawn_coordinate_x, spawn_coordinate_y = getBestSpawnCoordinatesNexttoMachine(store_id)
		
		-- spawns relic
		EntityLoad(relic_base_dir .. relic_object.name .. ".xml", spawn_coordinate_x, spawn_coordinate_y)
		if(relic_object.requires_relic ~= nil) then -- if spawned relic requires another relic, spawns it together with main relic
			local required_relic = getRelic(relic_object.requires_relic)
			EntityLoad(relic_base_dir .. required_relic.name .. ".xml", spawn_coordinate_x, spawn_coordinate_y)
		end
		
		-- loads audiovisual feedback
		EntityLoad("mods/ArcaneReliquary/files/entity_definitions/others/relic_store_interact_effect.xml", spawn_coordinate_x, spawn_coordinate_y)
		
		-- deducts gold from player
		addPlayerGold(relic_buy_price * -1)
		
		-- shows message
		message = "You bought " .. relic_object.ui_name .. " for " .. relic_buy_price .. " gold"
		GamePrint(message)
		GamePrintImportant(message, "", "mods/ArcaneReliquary/files/other_gfx/custom_decoration_store.png")
	else
		-- shows error message when not enough gold
		message = "You need " .. relic_buy_price .. " gold to buy a relic"
		GamePrint(message)
		GamePrintImportant(message, "", "mods/ArcaneReliquary/files/other_gfx/custom_decoration_error.png")
		-- loads audio feedback
		local x, y = EntityGetTransform(store_id)
		GamePlaySound( "data/audio/Desktop/event_cues.snd", "event_cues/chest_bad/create", x, y )
	end
end

function sellRelic(relic_sell_price, relic_level, relic_object)
	if(relic_level == nil) then relic_level = 0 end
	local sell_value = relic_sell_price + (relic_level * 50)
	addPlayerGold(sell_value)
	message = relic_object.ui_name .. " sold for " .. sell_value .. " gold"
	GamePrint(message)
	GamePrintImportant(message, "", "mods/ArcaneReliquary/files/other_gfx/custom_decoration_store.png")
	GamePlaySound( "data/audio/Desktop/event_cues.snd", "event_cues/shop_item/create", x, y )
end

function isInventoryBagsOpen()
	local InventoryBags_is_open = GlobalsGetValue("InventoryBags_is_open", "0")
	if(InventoryBags_is_open == "1") then
		return true
	else
		return false
	end
end

-- returns the first (not closest) machine entity id if there's one within radius_check
function isPlayerInRangeofMachine(machine_name, radius_check)
	local machine_in_range_id = nil
	local x, y = EntityGetTransform(getPlayerEntity())
	local machines_in_range = EntityGetInRadiusWithTag( x, y, radius_check, "item_physics" )
	for i, machine_id in ipairs(machines_in_range) do
		if(EntityGetName(machine_id) == machine_name) then
			machine_in_range_id = machine_id
			break
		end
	end
	return machine_in_range_id
end

-- function that configures a physical relic with appropriate internal variables according to its child item counterpart
-- child_item_id is the id for the child entity added to the player corresponding to the relic just dropped
-- physical_item_id is the id for the entity corresponding to the physical appearance of the relic that was just dropped
-- relic_object is a member of the list of relics table that matches the previous objects
function resetPhysicalRelicSettings(child_item_id, physical_item_id, relic_object)
	-- level
	if(entityHasInternalVariableValue(child_item_id, "relic_level")) then
		-- obtains level from child item
		local item_level = getInternalVariableValue(child_item_id, "relic_level", "value_int")
		-- sets it on physical item
		setRelicLevel(relic_object, physical_item_id, item_level)
	end
	-- is active flag
	if(entityHasInternalVariableValue(child_item_id, "is_active")) then
		local is_active = getInternalVariableValue(child_item_id, "is_active", "value_bool")
		setInternalVariableValue(physical_item_id, "is_active", "value_bool", is_active)
	end
	-- can be sold flag
	if(entityHasInternalVariableValue(child_item_id, "can_be_sold")) then
		local can_be_sold = getInternalVariableValue(child_item_id, "can_be_sold", "value_bool")
		if(not entityHasInternalVariableValue(physical_item_id, "can_be_sold")) then
			addNewInternalVariable(physical_item_id, "can_be_sold", "value_bool", true)
		else
			setInternalVariableValue(physical_item_id, "can_be_sold", "value_bool", true)
		end
	end
end

-- Checks whether or not the player has openned the inventory
function isInventoryOpen()
	local inventory = EntityGetFirstComponent( getPlayerEntity(), "InventoryGuiComponent" );
	local is_inventory_open = false;
	if inventory ~= nil then
		if ComponentGetValue( inventory, "mActive" ) == "1" then
			is_inventory_open = true;
		end
	end
	
	return is_inventory_open
end

function isItemforSale(item_id)
	if(EntityGetFirstComponent(item_id, "ItemCostComponent") ~= nil) then
		return true
	else
		return false
	end
end

function killEntitywithCustomMessage(entity_id, death_message)
	local gsc_id = EntityGetFirstComponentIncludingDisabled(entity_id, "GameStatsComponent")
	ComponentSetValue2(gsc_id, "extra_death_msg", death_message)
	EntityKill(entity_id)
end

function unlockPlayerInventories(x, y, radius, how_many_frames)
	local players_in_range = EntityGetInRadiusWithTag( x, y, radius, "player_unit" )
	if(#players_in_range > 0) then
		local game_effect_comp = GetGameEffectLoadTo( getPlayerEntity(), "EDIT_WANDS_EVERYWHERE", true )
		if game_effect_comp ~= nil then
			ComponentSetValue( game_effect_comp, "frames", tostring(how_many_frames) )
		end
	end
end

function addEntityAudioLoop(entity_id, audio_lib_name, audio_event_name, tag)
	local alc_id = EntityAddComponent2(entity_id, "AudioLoopComponent", {
		file=audio_lib_name,
		event_name=audio_event_name,
		auto_play=true
	})
	ComponentAddTag(alc_id, tag)
end

function removeAudioLoop(entity_id, tag)
	local alc_id = EntityGetFirstComponentIncludingDisabled(entity_id, "AudioLoopComponent", tag)
	EntityRemoveComponent(entity_id, alc_id)
end

function removeSpriteEffect(entity_id, sprite_effect_entity_name)
	local entity_children = EntityGetAllChildren(entity_id)
	if entity_children ~= nil then
		for i,child_id in ipairs(entity_children) do
			if(EntityGetName(child_id) == sprite_effect_entity_name) then
				EntityRemoveFromParent( child_id)
				EntityKill(child_id)
			end
		end
	end
end

function entityHasSpriteEffect(entity_id, effect_name)
	local flag = false
	local entity_children = EntityGetAllChildren(entity_id)
	if entity_children ~= nil then
		for i,child_id in ipairs(entity_children) do
			if(EntityGetName(child_id) == effect_name) then
				flag = true
			end
		end
	end
	return flag
end

function addSpriteEffect(entity_id, sprite_effect_path, sprite_offset_x, sprite_offset_y, effect_lifetime, sprite_alpha, effect_name)
	local dash_effect_id = EntityCreateNew( effect_name ) 
	EntityAddComponent2(dash_effect_id, "InheritTransformComponent", {
		only_position=true
	})
	EntityAddComponent2(dash_effect_id, "SpriteComponent", {
		image_file=sprite_effect_path,
		alpha=sprite_alpha,
		offset_x=sprite_offset_x,
		offset_y=sprite_offset_y,
		emissive=true,
		additive=true,
		update_transform=true,
		update_transform_rotation=false,
		visible=true,
		z_index=-9000,
		never_ragdollify_on_death=true,
	})
	EntityAddComponent2(dash_effect_id, "LifetimeComponent", {
		lifetime=effect_lifetime,
	})
	EntityAddChild( entity_id, dash_effect_id )
end

function getPoolofCompanions()
	local companions_pool = {"ankh", "bloody_skull", "buzzy", 
	"enchanted_skull", "flaming_skull", "friendly_slime", 
	"friendly_slime_acid", "friendly_slime_lava", "gatling_droid", 
	"overloaded_skull", "repeater_droid", "rifle_droid", 
	"rocket_droid", "security_probe", "security_probe_blue", 
	"security_probe_green", "security_probe_orange", "security_probe_purple", 
	"security_probe_teal", "security_probe_yellow", "sentient_crystal", 
	"shotgun_droid", "slapper_skull", "sniper_droid", "tiny_grenadier",
	"microrocket_droid", "interceptor_droid", "friendly_slime_poly", "friendly_slime_urine",
	"friendly_slime_blood", "friendly_slime_poison", "friendly_slime_water", "friendly_slime_frozen",
	"poly_skull", "mutating_skull", "berserk_skull", "nuclear_skull", "blizzard_skull", "fallout_skull",
	"bitter_skull", "venomous_skull", "humid_skull", "bomblay_skull", "midas_skull", "magneto_skull", 
	"explode_skull", "slash_skull", "windy_skull"}
	return companions_pool
end

function entityHasCompanion(entity_id, companion_entity_name)
	local flag = false
	local entity_children = EntityGetAllChildren(entity_id)
	if entity_children ~= nil then
		for i,child_id in ipairs(entity_children) do
			if(EntityGetName(child_id) == companion_entity_name) then
				flag = true -- if entity has companion, flags it accordingly
			end
		end
	end
	return flag
end

function detachCompanionfromEntity(entity_id, companion_entity_name)
	local entity_children = EntityGetAllChildren(entity_id)
	if entity_children ~= nil then
		for i,child_id in ipairs(entity_children) do
			if(EntityGetName(child_id) == companion_entity_name and not EntityHasTag(child_id, "relic")) then
				local x, y = EntityGetTransform(child_id)
				EntityKill(child_id)
				EntityLoad("mods/ArcaneReliquary/files/entity_definitions/others/explosion_companion.xml", x, y)
			end
		end
	end
end

function attachCompaniontoEntity(entity_id, companion_entity_name, target_tag)
	-- checks for companion existence first
	local companion_id = nil
	local spawn_companion = true
	local entity_children = EntityGetAllChildren(entity_id)
	if entity_children ~= nil then
		for i,child_id in ipairs(entity_children) do
			if(EntityGetName(child_id) == companion_entity_name and not EntityHasTag(child_id, "relic")) then
				spawn_companion = false -- if entity already has companion, flags it to not create it again
			end
		end
	end
	
	if(spawn_companion) then
		local x, y = EntityGetTransform( entity_id )
		companion_id = EntityLoad( "mods/ArcaneReliquary/files/entity_definitions/companions/" .. companion_entity_name .. ".xml", x, y )
		if(entity_id == getPlayerEntity()) then
			addNewSimplifiedInternalVariable( companion_id, "relic_companion" )
		end
		if(entityHasInternalVariableValue(companion_id, "target_tag")) then
			setInternalVariableValue(companion_id, "target_tag", "value_string", target_tag)
		end
		EntityAddChild( entity_id, companion_id )
		
		-- specific config for sentient_crystal_rainbow companion
		if(companion_entity_name == "sentient_crystal_rainbow") then
			addEntityMaterialAreaCheckerMechanic(companion_id, "companion")
		end
	end
	
	return companion_id
end

function detachMonkHandsfromEntity(entity_id, hands_entity_name)
	local entity_children = EntityGetAllChildren(entity_id)
	if entity_children ~= nil then
		for i,child_id in ipairs(entity_children) do
			if(EntityGetName(child_id) == hands_entity_name) then
				local x, y = EntityGetTransform(child_id)
				EntityKill(child_id)
				EntityLoad("mods/ArcaneReliquary/files/entity_definitions/others/explosion_companion.xml", x, y)
			end
		end
	end
end

function detachRotatingShieldfromEntity(entity_id, rotating_shield_projectile_name)
	local entity_children = EntityGetAllChildren(entity_id)
	if entity_children ~= nil then
		for i,child_id in ipairs(entity_children) do
			if(EntityGetName(child_id) == rotating_shield_projectile_name) then
				local x, y = EntityGetTransform(child_id)
				EntityKill(child_id)
			end
		end
	end
end

function entityHasRotatingShield(entity_id, rotating_shield_projectile_name)
	local flag = false
	local entity_children = EntityGetAllChildren(entity_id)
	if entity_children ~= nil then
		for i,child_id in ipairs(entity_children) do
			if(EntityGetName(child_id) == rotating_shield_projectile_name) then
				flag = true
			end
		end
	end
	return flag
end

function attachMonkHandstoEntity(entity_id, hands_entity_name, hands_fx_name, target_tag)
	-- checks for hands existence first
	local hand_1 = nil
	local hand_2 = nil
	local spawn_hands = true
	local entity_children = EntityGetAllChildren(entity_id)
	if entity_children ~= nil then
		for i,child_id in ipairs(entity_children) do
			if(EntityGetName(child_id) == hands_entity_name) then
				spawn_hands = false -- entity already has at least one hand of the given type, so flags it to not create them again
			end
		end
	end
	
	if(spawn_hands) then
		-- right hand
		local x, y = EntityGetTransform( entity_id )
		hand_1 = EntityLoad( "mods/ArcaneReliquary/files/entity_definitions/companions/" .. hands_entity_name .. ".xml", x, y )
		setInternalVariableValue(hand_1, "target_tag", "value_string", target_tag)
		if(entity_id == getPlayerEntity()) then
			addNewSimplifiedInternalVariable( hand_1, "relic_companion" )
		end
		local hand_1_fx_1 = EntityLoad( "mods/ArcaneReliquary/files/entity_definitions/companions/" .. hands_fx_name .. ".xml", x, y )
		local hand_1_fx_2 = EntityLoad( "mods/ArcaneReliquary/files/entity_definitions/companions/" .. hands_fx_name .. ".xml", x, y )
		local hand_1_fx_3 = EntityLoad( "mods/ArcaneReliquary/files/entity_definitions/companions/" .. hands_fx_name .. ".xml", x, y )
		EntityAddChild( hand_1, hand_1_fx_1 )
		EntityAddChild( hand_1, hand_1_fx_2 )
		EntityAddChild( hand_1, hand_1_fx_3 )
		EntityAddTag(hand_1_fx_1, "arm_fx")
		EntityAddTag(hand_1_fx_2, "arm_fx")
		EntityAddTag(hand_1_fx_3, "arm_fx")
		
		-- left hand
		hand_2 = EntityLoad( "mods/ArcaneReliquary/files/entity_definitions/companions/" .. hands_entity_name .. ".xml", x, y )
		setInternalVariableValue(hand_2, "target_tag", "value_string", target_tag)
		if(entity_id == getPlayerEntity()) then
			addNewSimplifiedInternalVariable( hand_2, "relic_companion" )
		end
		local hand_2_fx_1 = EntityLoad( "mods/ArcaneReliquary/files/entity_definitions/companions/" .. hands_fx_name .. ".xml", x, y )
		local hand_2_fx_2 = EntityLoad( "mods/ArcaneReliquary/files/entity_definitions/companions/" .. hands_fx_name .. ".xml", x, y )
		local hand_2_fx_3 = EntityLoad( "mods/ArcaneReliquary/files/entity_definitions/companions/" .. hands_fx_name .. ".xml", x, y )
		EntityAddChild( hand_2, hand_2_fx_1 )
		EntityAddChild( hand_2, hand_2_fx_2 )
		EntityAddChild( hand_2, hand_2_fx_3 )
		EntityAddTag(hand_2_fx_1, "arm_fx")
		EntityAddTag(hand_2_fx_2, "arm_fx")
		EntityAddTag(hand_2_fx_3, "arm_fx")
		
		local sc_id = EntityGetFirstComponentIncludingDisabled( hand_2, "SpriteComponent" )
		ComponentSetValue2(sc_id, "has_special_scale", true)
		ComponentSetValue2(sc_id, "special_scale_x", -1)
		setInternalVariableValue(hand_2, "is_right", "value_bool", false)
		
		-- adding hands as player children
		EntityAddChild( entity_id, hand_1 )
		EntityAddChild( entity_id, hand_2 )
	end
	
	return hand_1, hand_2
end

-- function that emulates item drops on death
function doArtificialDeathLogic(creature_id, gold_amount_multiplier, gold_trick_kill, relic_precursor_materials_list, precursor_items_base_dir)
	doChestDroponDeath(creature_id)
	doGoldDroponDeath(creature_id, gold_amount_multiplier, gold_trick_kill)
	doUpgradeContainerDroponDeath(creature_id)
	doChestKeyDroponDeath(creature_id)
	doPrizeChipDroponDeath(creature_id)
	doUpgradeStoneDroponDeath(creature_id)
	doPrecursorDroponDeath(creature_id, relic_precursor_materials_list, precursor_items_base_dir)
	doRoomSpawnerDroponDeath(creature_id)
end

-- this function replicates the vanilla gold drop mechanic
function doGoldDroponDeath(creature_id, amount_multiplier, trick_kill)
	if(creature_id == getPlayerEntity()) then return end -- routine is skipped if function is called with the id of the player character
	local no_gold_drop = false
	edit_component_with_tag( creature_id, "VariableStorageComponent", "no_gold_drop", function(comp,vars) no_gold_drop = true end )
	
	if no_gold_drop then
		return
	end

	local amount = 1

	edit_component( creature_id, "DamageModelComponent", function(comp,vars)
		local health = tonumber(ComponentGetValue( comp, "max_hp"))

		if health > 1.0 then
			amount = math.floor(health)
		end

	end)
	
	if GameHasFlagRun( "greed_curse" ) and ( GameHasFlagRun( "greed_curse_gone" ) == false ) then
		amount = amount * 3.0
	end

	amount = amount * amount_multiplier
	--[[if( amount > ( 15 * amount_multiplier ) ) then
		amount = 15 * amount_multiplier
	end]]--

	local money = 10 * amount
	local x, y = EntityGetTransform( creature_id )
	
	local entity_list = {}
	
	local gold_entity = "data/entities/items/pickup/goldnugget_"
	local remove_timer = false
	local drop_first_10 = true

	local comp_worldstate = EntityGetFirstComponent( GameGetWorldStateEntity(), "WorldStateComponent" )
	if( comp_worldstate ~= nil ) then
		if( ComponentGetValue2( comp_worldstate, "perk_gold_is_forever" ) ) then
			remove_timer = true
		end

		if( trick_kill ) then
			-- if worldstate perk_trick_kills_blood_money == 1
			if( ComponentGetValue2( comp_worldstate, "perk_trick_kills_blood_money" ) ) then
				gold_entity = "data/entities/items/pickup/bloodmoney_"
				drop_first_10 = false
			end
		end

		-- drop hp?
		SetRandomSeed( x + creature_id, y - creature_id )
		local hp_drop_chance = tonumber( ComponentGetValue2( comp_worldstate, "perk_hp_drop_chance" ) )
		hp_drop_chance = amount_multiplier * hp_drop_chance
		if( hp_drop_chance > 0 and Random( 1, 100 ) <= hp_drop_chance ) then
			local eid = load_gold_entity( "data/entities/items/pickup/bloodmoney_50.xml", x, y-8, remove_timer )
			money = money - 50
			
			table.insert( entity_list, {eid, 9} )
		end
	end

	--- spawn first 10 gold as small nuggets
	if( drop_first_10 ) then
		local nugget_10_count = 0
		while money >= 10 and nugget_10_count < 5 do
			local eid = load_gold_entity( gold_entity .. "10.xml", x, y-8, remove_timer )
			money = money - 10
			nugget_10_count = nugget_10_count + 1
			
			table.insert( entity_list, {eid, 6} )
		end	
	end
	
	---
	while money >= 200000 do
		local eid = load_gold_entity( gold_entity .. "200000.xml", x, y-8, remove_timer )
		money = money - 200000
		
		table.insert( entity_list, {eid, 12} )
	end	
	
	while money >= 10000 do
		local eid = load_gold_entity( gold_entity .. "10000.xml", x, y-8, remove_timer )
		money = money - 10000
		
		table.insert( entity_list, {eid, 12} )
	end	

	while money >= 1000 do
		local eid = load_gold_entity( gold_entity .. "1000.xml", x, y-8, remove_timer )
		money = money - 1000
		
		table.insert( entity_list, {eid, 20} )
	end	

	while money >= 200 do
		local eid = load_gold_entity( gold_entity .. "200.xml", x, y-8, remove_timer )
		money = money - 200
		
		table.insert( entity_list, {eid, 12} )
	end	

	while money >= 50 do
		local eid = load_gold_entity( gold_entity .. "50.xml", x, y-8, remove_timer )
		money = money - 50
		
		table.insert( entity_list, {eid, 9} )
	end	

	while money >= 10 do
		local eid = load_gold_entity( gold_entity .. "10.xml", x, y-8, remove_timer )
		money = money - 10
		
		table.insert( entity_list, {eid, 6} )
	end	
	
	if GameHasFlagRun( "exploding_gold" ) then
		for i,v in ipairs( entity_list ) do
			local eid = v[1]
			local size = tostring(v[2])
			
			EntityAddComponent( eid, "CollisionTriggerComponent", 
			{ 
				width = size,
				height = size,
				required_tag = "enemy",
				timer_for_destruction = "5",
			} )
			
			EntityAddComponent( eid, "LuaComponent", 
			{ 
				execute_every_n_frame = "-1",
				script_collision_trigger_hit = "data/scripts/perks/gold_explosion_collision_trigger.lua"
			} )
		end
	end
end

function doChestDroponDeath_NoBuff(creature_id)
	local x, y = EntityGetTransform(creature_id)
	SetRandomSeed( GameGetFrameNum(), x + y + creature_id )
	local chests_pool = {"relic_chest.xml", "relic_chest_great.xml", "relic_chest_chaotic.xml", "utility_items_chest.xml", "relic_chest_red.xml", "relic_chest_purple.xml", "relic_chest_black.xml"}
	EntityLoad("mods/ArcaneReliquary/files/entity_definitions/props/" .. chests_pool[Random(1,#chests_pool)], x, y )
end

function doChestDroponDeath(creature_id)
	-- first, checks whether or not enemy has the required variable, if not, returns to the caller without action
	if(not entityHasSimplifiedInternalVariable(creature_id, "relic_chest_buff")) then return end
	
	-- if creature has chest marker/tag on No Buffs mode, calls _NoBuff version of function
	if(entityHasSimplifiedInternalVariable(creature_id, "no_buff_chest_drop_chance")) then
		doChestDroponDeath_NoBuff(creature_id)
		return
	end
	
	
	local x, y = EntityGetTransform(creature_id)
	SetRandomSeed( GameGetFrameNum(), x + y + creature_id )
	
	local spawn_chance = 0
	
	local drop_odds = 0 -- used to calculate drop odds for regular relic chests and level 1 and 2 enemies
	local high_tier_drop_odds_multiplier = 5 -- used for great and chaotic chests dropped from level 3 and 4 enemies
	local buff_level = 0
	
	-- obtains buff level to calculate chest drop chance
	if(entityHasSimplifiedInternalVariable(creature_id, "buff_level_1")) then
		buff_level = 1
		drop_odds = 2 -- default 50% chance for level 1 buffed creatures
	elseif(entityHasSimplifiedInternalVariable(creature_id, "buff_level_2")) then
		buff_level = 2
		drop_odds = 2 -- default 50% chance for level 2 buffed creatures
	elseif(entityHasSimplifiedInternalVariable(creature_id, "buff_level_3")) then
		buff_level = 3
		drop_odds = 1 -- default 100% chance for level 3 buffed creatures
	elseif(entityHasSimplifiedInternalVariable(creature_id, "buff_level_4")) then
		buff_level = 4
		drop_odds = 1 -- default 100% chance for level 4 buffed creatures
	end
	
	local drop_message = "Relic chest dropped on level " .. buff_level .. " buffed enemy death"
	
	-- main drop odds
	spawn_chance = Random(1,drop_odds)
	
	-- 25% chance of locked chests replacing regular chests
	locked_chest_odds = Random(1,4)
	
	local relic_chest_type_spawned = ""
	
	if(spawn_chance == 1) then
		if(buff_level == 1 or buff_level == 2) then
		
			relic_chest_type_spawned = "relic_chest" -- default chest
			if(locked_chest_odds == 1) then relic_chest_type_spawned = "relic_chest_red" end -- default locked chest
			
			GamePrint(drop_message)
			
			EntityLoad("mods/ArcaneReliquary/files/entity_definitions/props/" .. relic_chest_type_spawned .. ".xml", x, y)
			if(entityHasSimplifiedInternalVariable(getPlayerEntity(), "double_chest_drop")) then -- double chest drop if double_chest_drop is active
				if(Random(1,2) == 1) then -- 50% chance another chest spawns
					EntityLoad("mods/ArcaneReliquary/files/entity_definitions/props/" .. relic_chest_type_spawned .. ".xml", x, y)
				end
			end
		elseif(buff_level == 3) then
		
			relic_chest_type_spawned = "relic_chest" -- default chest
			if(locked_chest_odds == 1) then relic_chest_type_spawned = "relic_chest_red" end -- default locked chest
			
			if(Random(1,drop_odds * high_tier_drop_odds_multiplier) == 1) then -- additional chance check for great relic chest drop
			
				relic_chest_type_spawned = "relic_chest_great"
				if(locked_chest_odds == 1) then relic_chest_type_spawned = "relic_chest_purple" end -- additional chance check for purple relic chest drop
				
			end
			GamePrint(drop_message)
			
			EntityLoad("mods/ArcaneReliquary/files/entity_definitions/props/" .. relic_chest_type_spawned .. ".xml", x, y)
			if(entityHasSimplifiedInternalVariable(getPlayerEntity(), "double_chest_drop")) then -- double chest drop if double_chest_drop is active
				if(Random(1,2) == 1) then -- 50% chance another chest spawns
					EntityLoad("mods/ArcaneReliquary/files/entity_definitions/props/" .. relic_chest_type_spawned .. ".xml", x, y)
				end
			end
		elseif(buff_level == 4) then
		
			relic_chest_type_spawned = "relic_chest_great" -- default chest
			if(locked_chest_odds == 1) then relic_chest_type_spawned = "relic_chest_purple" end -- default locked chest
			
			if(Random(1,drop_odds * high_tier_drop_odds_multiplier) == 1) then -- additional chance check for chaotic relic chest drop
			
				relic_chest_type_spawned = "relic_chest_chaotic"
				if(locked_chest_odds == 1) then relic_chest_type_spawned = "relic_chest_black" end -- additional chance check for black relic chest drop
				
			end
			GamePrint(drop_message)
			
			EntityLoad("mods/ArcaneReliquary/files/entity_definitions/props/" .. relic_chest_type_spawned .. ".xml", x, y)
			if(entityHasSimplifiedInternalVariable(getPlayerEntity(), "double_chest_drop")) then -- double chest drop if double_chest_drop is active
				if(Random(1,2) == 1) then -- 50% chance another chest spawns
					EntityLoad("mods/ArcaneReliquary/files/entity_definitions/props/" .. relic_chest_type_spawned .. ".xml", x, y)
				end
			end
		end
		
		-- utility item spawn chance
		if(Random(1,2) == 1) then
			EntityLoad("mods/ArcaneReliquary/files/entity_definitions/props/utility_items_chest.xml", x, y )
		end
	end
end

function doRoomSpawnerDroponDeath(creature_id)
	-- first, checks whether or not enemy has the required variable, if not, returns to the caller without action
	if(not entityHasSimplifiedInternalVariable(creature_id, "room_spawner_item_drop_chance")) then return end

	local x, y = EntityGetTransform(creature_id)
	
	local item_pool = {
		"mods/ArcaneReliquary/files/entity_definitions/items/compact_machine_room.xml",
		"mods/ArcaneReliquary/files/entity_definitions/items/compact_alchemy_room.xml",
		"mods/ArcaneReliquary/files/entity_definitions/items/compact_store_room.xml",
		"mods/ArcaneReliquary/files/entity_definitions/items/compact_recycling_room.xml",
		"mods/ArcaneReliquary/files/entity_definitions/items/machine_spawner.xml",
	}
	
	SetRandomSeed( GameGetFrameNum(), x + y + creature_id )
	local drop_height_offset = -10
	EntityLoad(item_pool[Random(1,#item_pool)], x, y + drop_height_offset)
end

function doChestKeyDroponDeath(creature_id)
	-- first, checks whether or not enemy has the required variable, if not, returns to the caller without action
	if(not entityHasSimplifiedInternalVariable(creature_id, "chest_key_drop_chance")) then return end

	local x, y = EntityGetTransform(creature_id)
	SetRandomSeed( GameGetFrameNum(), x + y + creature_id )
	
	local spawn_odds = 10
	
	local spawn_chance = Random(1, spawn_odds)

	local drop_height_offset = -10
	
	if(spawn_chance == 1) then
		EntityLoad("mods/ArcaneReliquary/files/entity_definitions/items/chest_key_black.xml", x, y + drop_height_offset)
	elseif(spawn_chance >= 2 and spawn_chance <= 4) then
		EntityLoad("mods/ArcaneReliquary/files/entity_definitions/items/chest_key_purple.xml", x, y + drop_height_offset)
	elseif(spawn_chance >= 5) then
		EntityLoad("mods/ArcaneReliquary/files/entity_definitions/items/chest_key_red.xml", x, y + drop_height_offset)
	end
end

function doUpgradeStoneDroponDeath(creature_id)
	-- first, checks whether or not enemy has the required variable, if not, returns to the caller without action
	if(not entityHasSimplifiedInternalVariable(creature_id, "upgrade_stone_drop_chance")) then return end

	local x, y = EntityGetTransform(creature_id)
	SetRandomSeed( GameGetFrameNum(), x + y + creature_id )
	
	local spawn_odds = 10
	
	local spawn_chance = Random(1, spawn_odds)

	local drop_height_offset = -10
	
	if(spawn_chance == 1) then
		EntityLoad("mods/ArcaneReliquary/files/entity_definitions/items/upgrade_stone_3.xml", x, y + drop_height_offset)
	elseif(spawn_chance >= 2 and spawn_chance <= 4) then
		EntityLoad("mods/ArcaneReliquary/files/entity_definitions/items/upgrade_stone_2.xml", x, y + drop_height_offset)
	elseif(spawn_chance >= 5) then
		EntityLoad("mods/ArcaneReliquary/files/entity_definitions/items/upgrade_stone.xml", x, y + drop_height_offset)
	end
end

function doPrizeChipDroponDeath(creature_id)
	-- first, checks whether or not enemy has the required variable, if not, returns to the caller without action
	if(not entityHasSimplifiedInternalVariable(creature_id, "prize_chip_drop_chance")) then return end

	local x, y = EntityGetTransform(creature_id)
	SetRandomSeed( GameGetFrameNum(), x + y + creature_id )
	
	local spawn_odds = 10
	
	local spawn_chance = Random(1, spawn_odds)

	local drop_height_offset = -10
	
	if(spawn_chance == 1) then
		EntityLoad("mods/ArcaneReliquary/files/entity_definitions/items/relic_token.xml", x, y + drop_height_offset)
	elseif(spawn_chance >= 2 and spawn_chance <= 3) then
		EntityLoad("mods/ArcaneReliquary/files/entity_definitions/items/relic_prize_chip_3.xml", x, y + drop_height_offset)
	elseif(spawn_chance >= 4 and spawn_chance <= 6) then
		EntityLoad("mods/ArcaneReliquary/files/entity_definitions/items/relic_prize_chip_2.xml", x, y + drop_height_offset)
	elseif(spawn_chance >= 7) then
		EntityLoad("mods/ArcaneReliquary/files/entity_definitions/items/relic_prize_chip.xml", x, y + drop_height_offset)
	end
end

function doUpgradeContainerDroponDeath(creature_id)
	-- first, checks whether or not enemy has the required variable, if not, returns to the caller without action
	if(not entityHasSimplifiedInternalVariable(creature_id, "upgrade_item_drop_chance")) then return end

	local x, y = EntityGetTransform(creature_id)
	SetRandomSeed( GameGetFrameNum(), x + y + creature_id )
	
	local spawn_odds = 25
	
	local spawn_chance = Random(1, spawn_odds)
	
	local advanced_container_min_chance = spawn_odds - (spawn_odds - 1)
	local advanced_container_max_chance = spawn_odds / 5
	
	local drop_height_offset = -10
	if(spawn_chance >= advanced_container_min_chance and spawn_chance <= advanced_container_max_chance) then
		EntityLoad("mods/ArcaneReliquary/files/entity_definitions/items/relic_upgrade_compound_container_advanced.xml", x, y + drop_height_offset)
	else
		EntityLoad("mods/ArcaneReliquary/files/entity_definitions/items/relic_upgrade_compound_container.xml", x, y + drop_height_offset)
	end
end

function doPrecursorDroponDeath(creature_id, relic_precursor_materials_list, precursor_items_base_dir)
	-- first, checks whether or not enemy has the required variable, if not, returns to the caller without action
	if(not entityHasSimplifiedInternalVariable(creature_id, "construct_item_drop_chance")) then return end
	
	local x, y = EntityGetTransform(creature_id)
	SetRandomSeed( GameGetFrameNum(), x + y + creature_id )
	
	local spawn_odds = 25
	
	local spawn_chance = Random(1, spawn_odds)
	
	local advanced_container_min_chance = spawn_odds - (spawn_odds - 1)
	local advanced_container_max_chance = spawn_odds / 5
	
	local drop_height_offset = -10
	if(spawn_chance >= advanced_container_min_chance and spawn_chance <= advanced_container_max_chance) then
		EntityLoad("mods/ArcaneReliquary/files/entity_definitions/items/relic_metal_chunk.xml", x, y + drop_height_offset)
	else
		local precursor_container = relic_precursor_materials_list[Random(1,#relic_precursor_materials_list)]
		EntityLoad(precursor_items_base_dir .. precursor_container.related_item, x, y + drop_height_offset)
	end
end

-- returns the name of the current animation of an entity with animations configured
function entityGetCurrentAnimationName(entity_id)
	local sc_id = EntityGetFirstComponentIncludingDisabled(entity_id, "SpriteComponent")
	local current_animation = ComponentGetValue2( sc_id, "rect_animation" )
	return current_animation
end

function addEntityLinkedLightningArcProjectile(projectile_id, linked_entity_id)
	
	EntityAddComponent2(projectile_id, "LightningComponent")
	
	local pec_id = EntityAddComponent2(projectile_id, "ArcComponent", {
		type="LIGHTNING",
		lifetime=120,
		material="",
		mArcTarget=linked_entity_id,
	})
end	

-- function to add items to player inventories
-- entity_path is the complete entity path to the item being added
-- inventory_type can be either "inventory_quick" or "inventory_full"
function addItemtoPlayerInventory(entity_path, inventory_type)

	local children = EntityGetAllChildren(getPlayerEntity())
	local inventory = nil
    if children == nil then return end
    for i,child in ipairs(children) do
		if(EntityGetName(child) == inventory_type) then
			inventory = child
		end
	end
	
	local newItem = EntityLoad(entity_path)
	EntityAddChild(inventory, newItem)
end

-- function that adds fly energy, refilling jetpack force
function addFlyEnergy(amount_to_add)
	local cdc_id = EntityGetFirstComponentIncludingDisabled(getPlayerEntity(), "CharacterDataComponent")
	local mFlyingTimeLeft = ComponentGetValue2(cdc_id, "mFlyingTimeLeft")
	ComponentSetValue2(cdc_id, "mFlyingTimeLeft", mFlyingTimeLeft + amount_to_add)
end

function getFlyEnergy()
	local cdc_id = EntityGetFirstComponentIncludingDisabled(getPlayerEntity(), "CharacterDataComponent")
	local mFlyingTimeLeft = ComponentGetValue2(cdc_id, "mFlyingTimeLeft")
	return mFlyingTimeLeft
end

-- helper function to determine whether or not wand with wand_id is a starting wand
function isStartingWand(wand_id)
	local ac_id = EntityGetFirstComponentIncludingDisabled(wand_id, "AbilityComponent")
	if(string.find(EntityGetFilename( wand_id ), "player.xml")) then
	--if(ComponentGetValue2(ac_id, "ui_name") == "Bolt staff" or ComponentGetValue2(ac_id, "ui_name") == "Bomb wand") then
		return true
	else
		return false
	end
end

-- helper function that returns whether or not a wand with wand_id has been picked by the player
function wandHasBeenPicked(wand_id)
	local ic_id = EntityGetFirstComponentIncludingDisabled(wand_id, "ItemComponent")
	local has_been_picked_by_player = ComponentGetValue2(ic_id, "has_been_picked_by_player")
	return has_been_picked_by_player
end

-- updates all ui related text of a relic item
function updateRelicUIText(relic_id, simplified_ui_name, descriptive_ui_name, short_ui_name)
	local ic_id = EntityGetFirstComponentIncludingDisabled(relic_id, "ItemComponent")
	ComponentSetValue2(ic_id, "item_name", simplified_ui_name)
	ComponentSetValue2(ic_id, "ui_description", descriptive_ui_name)

	local uiic_id = EntityGetFirstComponentIncludingDisabled(relic_id, "UIInfoComponent")
	ComponentSetValue2(uiic_id, "name", short_ui_name)
end

function toggleRelicDisabled_ChildItem(child_item_id)
	local is_active = getInternalVariableValue(child_item_id, "is_active", "value_bool")
	setInternalVariableValue(child_item_id, "is_active", "value_bool", not is_active)
end

function toggleRelicDisabled(relic_id, relic_name)
	local ic_id = EntityGetFirstComponentIncludingDisabled(relic_id, "ItemComponent")
	local uiic_id = EntityGetFirstComponentIncludingDisabled(relic_id, "UIInfoComponent")
	local preferred_inventory = ComponentGetValue2(ic_id, "preferred_inventory")
	
	local tag_name = "[DISABLED]"
	local tag_name_replace_rule = "%[DISABLED%]"
	
	-- if relic is currently active, disables it
	if(isRelicActive(relic_id)) then
		ComponentSetValue2(ic_id, "preferred_inventory", "QUICK")
		setInternalVariableValue(relic_id, "is_active", "value_bool", false)
		GamePrint(relic_name .. " disabled, collect it again to store it on your potions inventory")
		
		-- adds [DISABLED] tag to UI text
		local new_item_name = tag_name .. ComponentGetValue2(ic_id, "item_name")
		local new_ui_description = tag_name .. ComponentGetValue2(ic_id, "ui_description")
		ComponentSetValue2(ic_id, "item_name", new_item_name)
		ComponentSetValue2(ic_id, "ui_description", new_ui_description)
		
		local new_name = tag_name .. ComponentGetValue2(uiic_id, "name")
		ComponentSetValue2(uiic_id, "name", new_name)
		
	else -- otherwise, enables it
		ComponentSetValue2(ic_id, "preferred_inventory", "FULL")
		setInternalVariableValue(relic_id, "is_active", "value_bool", true)
		GamePrint(relic_name .. " enabled, collect it again to reactive its effects")
		
		-- removes [DISABLED] tag from UI text
		local new_item_name = string.gsub(ComponentGetValue2(ic_id, "item_name"), tag_name_replace_rule, "") 
		local new_ui_description = string.gsub(ComponentGetValue2(ic_id, "ui_description"), tag_name_replace_rule, "")
		ComponentSetValue2(ic_id, "item_name", new_item_name)
		ComponentSetValue2(ic_id, "ui_description", new_ui_description)
		
		local new_name = string.gsub(ComponentGetValue2(uiic_id, "name"), tag_name_replace_rule, "") 
		ComponentSetValue2(uiic_id, "name", new_name)

	end
end

function isRelicActive(relic_id)
	return getInternalVariableValue(relic_id, "is_active", "value_bool")
end

-- obtains the roman variant of a level number
function levelNumbertoRoman(level_number)
	local roman_level = ""
	local conversion_table = {
		{number = 1, roman = "I"},
		{number = 2, roman = "II"},
		{number = 3, roman = "III"},
		{number = 4, roman = "IV"},
		{number = 5, roman = "V"},
		{number = 6, roman = "VI"},
		{number = 7, roman = "VII"},
		{number = 8, roman = "VIII"},
		{number = 9, roman = "IX"},
		{number = 10, roman = "X"},
	}
	for i,element in ipairs(conversion_table) do
		if(element.number == level_number) then
			roman_level = element.roman
			break
		end
	end
	return roman_level
end

-- function to verify if a collectible item is dropped on the ground or is equipped by the player
-- returns true if dropped, false otherwise
function isItemDropped(entity_id)
	local parent = EntityGetParent(entity_id)
	if(parent == 0 or parent == nil) then
		return true -- item is dropped on the ground
	else
		return false -- item is still equipped
	end
end

-- returns the entity definition path for creature with creature_id provided it is a vanilla creature and it is not a boss
function getCreatureDefinitionPath(creature_id)
	if(isCreatureBoss(creature_id)) then	
		return nil
	end
	
	local sc_id = EntityGetFirstComponentIncludingDisabled( creature_id, "SpriteComponent" )
	local sprite_path = ComponentGetValue2(sc_id, "image_file")
	local creature_path = string.gsub(sprite_path, "data/enemies_gfx/", "data/entities/animals/")
	
	return creature_path
end

-- adds material detection mechanic on entity with entity_id, entity_type can be either 'player', 'projectile' or 'companion'
function addEntityMaterialAreaCheckerMechanic(entity_id, entity_type)
	
	if(not entityHasSimplifiedInternalVariable(entity_id, "material_detection")) then
		addNewSimplifiedInternalVariable( entity_id, "material_detection")
		
		local materials_pool = {
			CellFactory_GetType("urine"),
			CellFactory_GetType("water"),
			CellFactory_GetType("oil"),
			CellFactory_GetType("radioactive_liquid"),
			CellFactory_GetType("lava"),
			CellFactory_GetType("acid"),
			CellFactory_GetType("blood"),
			CellFactory_GetType("poison"),
			CellFactory_GetType("alcohol"),
			CellFactory_GetType("magic_liquid_berserk"),
			CellFactory_GetType("magic_liquid_teleportation"),
			CellFactory_GetType("magic_liquid_polymorph"),
			CellFactory_GetType("magic_liquid_random_polymorph"),
			CellFactory_GetType("magic_liquid_movement_faster"),
			CellFactory_GetType("magic_liquid_faster_levitation"),
			CellFactory_GetType("magic_liquid_hp_regeneration"),
			CellFactory_GetType("magic_liquid_charm"),
			CellFactory_GetType("magic_liquid_invisibility"),
			CellFactory_GetType("magic_liquid_mana_regeneration"),
			CellFactory_GetType("magic_liquid_unstable_polymorph"),
			CellFactory_GetType("magic_liquid_worm_attractor"),
			CellFactory_GetType("magic_liquid_protection_all"),
			CellFactory_GetType("slime"),	
		}
		
		for i,material_id in ipairs(materials_pool) do
			local material_detector_id = EntityLoad("mods/ArcaneReliquary/files/entity_definitions/others/generic_material_detector.xml")
			edit_component( material_detector_id, "MaterialAreaCheckerComponent", function(comp,vars)
				vars.material=material_id
			end)
			
			addNewLuaCallback(material_detector_id, "on_material_check", "mods/ArcaneReliquary/files/scripts/other_listeners/material_detector_" .. entity_type .. ".lua", 1, false, "material_detector_player")
			addNewSimplifiedInternalVariable( material_detector_id, "material_detection")
			EntityAddChild( entity_id, material_detector_id )
		end
	end
end

-- helper function to spawn a random perk card
function spawnRandomPerkCard(x, y)
	perk_spawn_random( x, y, true )
end

-- helper function that returns "enemy" if entity_id is an enemy creature, or "player" if it's the player
function creatureIs(entity_id)
	if(EntityHasTag(entity_id, "enemy")) then
		return "enemy"
	elseif(EntityHasTag(entity_id, "player_unit")) then
		return "player"
	end
end

-- sets knockback value for projectile with projectile_id
function setProjectileKnockback(projectile_id, knockback)
	local pc_id = EntityGetFirstComponentIncludingDisabled( projectile_id, "ProjectileComponent" )
	ComponentSetValue2(pc_id, "knockback_force", knockback)
	ComponentSetValue2(pc_id, "ragdoll_force_multiplier", knockback)
end

-- helper function that returns true if the player possesses perk with perk_id, or false otherwise
function playerHasPerk(perk_id)
	if(GameHasFlagRun("PERK_PICKED_" .. perk_id)) then
		return true
	else
		return false
	end
end

-- function that adds real particles emitter based on the material_name supplied
function entityAddMaterialEmitter(entity_id, material_name, tag)
	local pec_id = EntityAddComponent2(entity_id, "ParticleEmitterComponent", {
		emitted_material_name=material_name,
		x_pos_offset_min=-1,
		x_pos_offset_max=1,
		y_pos_offset_max=0,
		x_vel_min=-7,
		x_vel_max=7,
		y_vel_min=80,
		y_vel_max=180,
		count_min=1,
		count_max=2,
		lifetime_min=5,
		lifetime_max=5,
		create_real_particles=true,
		emit_cosmetic_particles=true,
		emission_interval_min_frames=0,
		emission_interval_max_frames=1,
		is_emitting=true
	})
	
	if(tag ~= nil) then
		ComponentAddTag( pec_id, tag )
	end
end

-- updates the amount of material on all potions on the player's inventory, selecting only the materials with the greatest amount on each potion
function updatePotionMaterialCount(amount_add)
	local inventory_items = EntityGetAllChildren(getPlayerInventory("inventory_quick"))
	if(inventory_items ~= nil) then
		for i,item in ipairs(inventory_items) do
			if EntityHasTag(item, "potion") then -- searches for potions only
				local materials = getMaterialsFromEntityInventory(item) -- obtains all materials stored in a potion found
				if(#materials > 0) then
					-- first, isolates material amounts to determine what amount is the greatest
					local amounts = {}
					for i,material in ipairs(materials) do
						table.insert(amounts, material.material_amount)
					end
					table.sort(amounts)
					
					-- 
					local greatest_amount = amounts[#amounts]
					
					for i,material in ipairs(materials) do
						if(material.material_amount == greatest_amount) then
							local current_amount = getMaterialAmountOnEntityInventory(item, material.material_name) -- obtains current material amount
							local new_amount = current_amount + amount_add -- updates current amount with amount to add
							if(new_amount > getMaterialSuckerSize(item)) then -- if new amount surpasses sucker size for current potion, then new amount is updated to current amount
								new_amount = current_amount
							end
							AddMaterialInventoryMaterial( item, material.material_name, new_amount)
						end
					end
					--[[
					for i,material in ipairs(materials) do
						GamePrint("### name: " .. material.material_name .. " | amount: " .. material.material_amount)
					end
					]]
				end
				
			end
		end
	end
end

-- returns a table containing all materials, and their amounts, contained on the material inventory of an entity with entity_id
function getMaterialsFromEntityInventory(entity_id)
	local materials_on_entity = {}
	local mic_id = EntityGetFirstComponentIncludingDisabled(entity_id, "MaterialInventoryComponent")
	local materials_stomach = ComponentGetValue2(mic_id, "count_per_material_type")
	for material_id,amount in pairs(materials_stomach) do
		if(amount ~= 0 and not string.find(CellFactory_GetName(material_id - 1), "fading")) then
			table.insert(materials_on_entity, {material_name = CellFactory_GetName(material_id - 1), material_amount = amount})
		end
	end
	return materials_on_entity
end

function toggleMaterialSucker(entity_id, tag, toggle)
	local components = EntityGetComponentIncludingDisabled( entity_id, "MaterialSuckerComponent" )
	for i,component in ipairs(components) do
		if(ComponentHasTag( component, tag )) then
			EntitySetComponentIsEnabled( entity_id, component, toggle)
		end
	end
end

-- obtains barrel size from material inventory of entity with entity_id
function getMaterialSuckerSize(entity_id)
	local msc_id = EntityGetFirstComponentIncludingDisabled(entity_id, "MaterialSuckerComponent")
	local barrel_size = ComponentGetValue2(msc_id, "barrel_size")
	return barrel_size
end

-- obtains the amount of a given material on an entity's material inventory
function getMaterialAmountOnEntityInventory(entity_id, material_name)
	local material_id = CellFactory_GetType(material_name) + 1
	local mic_id = EntityGetFirstComponentIncludingDisabled(entity_id, "MaterialInventoryComponent")
	local materials_stomach = ComponentGetValue2(mic_id, "count_per_material_type")
	return tonumber(materials_stomach[material_id])
end

-- obtains the amount of a given material on the player's stomach
function getMaterialAmountOnPlayerStomach(material_name)
	local material_id = CellFactory_GetType(material_name) + 1
	local mic_id = EntityGetFirstComponentIncludingDisabled(getPlayerEntity(), "MaterialInventoryComponent")
	local materials_stomach = ComponentGetValue2(mic_id, "count_per_material_type")
	return tonumber(materials_stomach[material_id])
end

-- function to check whether or not projectile with projectile_id is a magic_shield variation
function isOrbitingProjectile(projectile_id)
	local projectile_definition = getProjectileDefinitionPath(projectile_id)
	if(projectile_definition ~= nil and (string.find(projectile_definition, "magic_shield") or string.find(projectile_definition, "spiral_shot"))) then
		return true
	end
end

-- determines if creature whose id is entity_id is a boss or miniboss
function isCreatureBoss(entity_id)
	if(EntityHasTag(entity_id, "boss") or 
	   EntityHasTag(entity_id, "miniboss") or 
	   EntityHasTag(entity_id, "gate_monster") or 
	   EntityHasTag(entity_id, "sampo_or_boss") or
	   EntityHasTag(entity_id, "boss_dragon") or
	   EntityHasTag(entity_id, "boss_centipede")) then
		return true
	else
		return false
	end
end

--[[
function isCreatureBoss(entity_id)
    return (
       EntityHasTag(entity_id, "boss") or
       EntityHasTag(entity_id, "miniboss") or
       EntityHasTag(entity_id, "gate_monster") or
       EntityHasTag(entity_id, "sampo_or_boss") or
       EntityHasTag(entity_id, "boss_dragon") or
       EntityHasTag(entity_id, "boss_centipede")
    )
end
]]

-- adds config to projectile with projectile_id making it solid and allowing it to be hit by other projectiles
function makeProjectileSolid(projectile_id)
	
	EntityAddTag(projectile_id, "hittable")
	EntityAddComponent2(projectile_id, "HitboxComponent", {
		aabb_min_x=-5, 
		aabb_max_x=5, 
		aabb_min_y=-5, 
		aabb_max_y=5,
	})
	EntityAddComponent2(projectile_id, "DamageModelComponent", {
		hp=0.1,
		fire_probability_of_ignition=0,
		falling_damages=false,
		air_needed=false, 
		materials_damage=false,
		ragdoll_filenames_file="",
		blood_material="",
		create_ragdoll=false
	})
end

-- drops currently equipped item if it is a wand item
function dropCurrentlyEquippedWand()
	local inventory2component = EntityGetFirstComponent(getPlayerEntity(), "Inventory2Component")
	local item_id = nil
	if inventory2component then
		item_id = ComponentGetValue(inventory2component, "mActiveItem")
		ComponentSetValue(inventory2component, "mActiveItem", "0")
	end
	if(EntityHasTag(item_id, "wand")) then
		EntityRemoveFromParent(item_id)
		EntitySetComponentsWithTagEnabled(item_id, "enabled_in_world", true)
		EntitySetComponentsWithTagEnabled(item_id, "enabled_in_hand", true)
		EntitySetComponentsWithTagEnabled(item_id, "enabled_in_inventory", true)
	end
end

-- replaces the sprite for the explosion config of a projectile with projectile_id
function replaceProjectileExplosionSprite(projectile_id, sprite_path)
	local pc_id = EntityGetFirstComponentIncludingDisabled( projectile_id, "ProjectileComponent" )
	ComponentObjectSetValue2( pc_id, "config_explosion", "explosion_sprite", sprite_path )
end

-- converts an enemy into a random material from a fixed pool of materials
function entityConvertToRandomMaterial(entity_id)

	local materials_pool = {
		"oil",
		"blood",
		"poison",
		"water",
		"lava",
		"acid",
		"cement",
		"alcohol",
		"magic_liquid_random_polymorph",
		"magic_liquid_berserk",
		"magic_liquid_teleportation",
		"magic_liquid_hp_regeneration",
		"magic_liquid_polymorph",
		"magic_liquid_movement_faster",
		"magic_liquid_faster_levitation",
		"magic_liquid_charm",
		"magic_liquid_invisibility",
		"magic_liquid_mana_regeneration",
		"magic_liquid_unstable_polymorph",
		"magic_liquid_worm_attractor",
		"magic_liquid_protection_all",
		"radioactive_liquid",
		"radioactive_gas",
		"blood_cold_vapour",
		"blood_cold",
		"smoke",
		"sand",
		"coal",
		"diamond",
		"gunpowder_explosive",
		"slime",
		"corrosive_blood",
		"urine",
		"diamond",
		"silver",
		"liquid_fire"
		--"creepy_liquid",
	}
	
	SetRandomSeed( GameGetFrameNum(), math.random(1,50))
	
	EntityConvertToMaterial( entity_id, materials_pool[Random(1,#materials_pool)])
	EntityKill(entity_id)
end

-- activates collide_with_world flag on projectile with projectile_id, making it phase through terrain
function makeProjectilePhaseThroughTerrain(projectile_id)
	local pc_id = EntityGetFirstComponentIncludingDisabled( projectile_id, "ProjectileComponent" )
	ComponentSetValue2( pc_id, "collide_with_world", false )
end

-- disables the ability of a projectile to bounce
function projectileRemoveBouncing(projectile_id)
	local pc_id = EntityGetFirstComponentIncludingDisabled( projectile_id, "ProjectileComponent" )
	ComponentSetValue2( pc_id, "bounces_left", 0 )
	ComponentSetValue2( pc_id, "bounce_always", false )
end

-- returns true if enemy with enemy_id is resistant to damage multiplier with multiplier_name, or false otherwise
function creatureIsResistantToDamageMultiplier(enemy_id, multiplier_name)
	local flag = false
	local multiplier = getBasicDamageMultiplier(enemy_id, multiplier_name)
	if(multiplier <= 0) then
		flag = true
	end
	return flag
end

-- returns true if enemy with enemy_id can be damaged by material with material_name, or false otherwise
function creatureIsDamagedByMaterial(enemy_id, material_name)
	local flag = false
	local dmc_id = EntityGetFirstComponentIncludingDisabled(enemy_id, "DamageModelComponent")
	local materials_that_damage = ComponentGetValue2(dmc_id, "materials_that_damage")
	if(string.find(materials_that_damage, material_name)) then
		flag = true
	end
	return flag
end

function fractionProjectileVelocities(projectile_id, dividend)
	local projectile_vel_x, projectile_vel_y = getProjectileVelocitiesandGravity(projectile_id)
	setProjectileVelocitiesandGravity(projectile_id, projectile_vel_x / dividend, projectile_vel_y / dividend)
end

function invertProjectileVelocities(projectile_id)
	local projectile_vel_x, projectile_vel_y = getProjectileVelocitiesandGravity(projectile_id)
	setProjectileVelocitiesandGravity(projectile_id, projectile_vel_x * -1, projectile_vel_y * -1)
end

-- shoots a projectile based on the velocities of another projectile with base_projectile_id
-- projectile_definition_path is the path to the xml that defines the projectile that will be shot based on velocities of projectile with base_projectile_id
-- x and y are the coordinates where the shot must trigger
-- velocity_threshold is how varied the velocities of the shot projectile should be, the bigger the number, the more random it is
-- 		if omitted, new projectile will be shot with the same velocities of projectile with base_projectile_id
function shootProjectileBasedonAnotherProjectile(base_projectile_id, projectile_definition_path, x, y, velocity_threshold)
	SetRandomSeed( GameGetFrameNum(), math.random(1,50))
	local projectile_vel_x, projectile_vel_y = getProjectileVelocitiesandGravity(base_projectile_id)
	local copy_projectile = EntityLoad(projectile_definition_path, x, y)
	setProjectileShooter(copy_projectile, getPlayerEntity())
	addNewSimplifiedInternalVariable( copy_projectile, "relic_effect_projectile")
	if(velocity_threshold ~= nil) then
		setProjectileVelocitiesandGravity(copy_projectile, projectile_vel_x + Random(-velocity_threshold, velocity_threshold), projectile_vel_y + Random(-velocity_threshold, velocity_threshold))
	else
		setProjectileVelocitiesandGravity(copy_projectile, projectile_vel_x, projectile_vel_y)
	end
	
end

-- adds specific damage types to projectile with projectile_id
-- damage_by_type can be
-- 		ice
-- 		fire
-- 		electricity
--		melee
--		slice
-- amount_to_add is how much to be added to the specific damage type specified
function addProjectileDamageByType(projectile_id, damage_type, amount_to_add)
	local pc_id = EntityGetFirstComponentIncludingDisabled( projectile_id, "ProjectileComponent" )
	local damage_type_value = ComponentObjectGetValue2( pc_id, "damage_by_type", damage_type )
	ComponentObjectSetValue2( pc_id, "damage_by_type", damage_type, damage_type_value + amount_to_add )
end

-- multiplies damage type dealt by projectile with projectile_id
function multiplyProjectileDamageByType(projectile_id, damage_type, multiplier)
	local pc_id = EntityGetFirstComponentIncludingDisabled( projectile_id, "ProjectileComponent" )
	local damage_type_value = ComponentObjectGetValue2( pc_id, "damage_by_type", damage_type )
	ComponentObjectSetValue2( pc_id, "damage_by_type", damage_type, damage_type_value * multiplier )
end

-- returns whether or not projectile with projectile_id deals specified damage_type
function projectileHasDamageType(projectile_id, damage_type)
	local pc_id = EntityGetFirstComponentIncludingDisabled( projectile_id, "ProjectileComponent" )
	local damage_type_value = ComponentObjectGetValue2( pc_id, "damage_by_type", damage_type )
	if(damage_type_value ~= 0) then
		return true
	else
		return false
	end
end

-- disables the ability of a projectile with projectile_id to explode on death
function projectileToggleExplosionOnDeath(projectile_id, flag)
	local pc_id = EntityGetFirstComponentIncludingDisabled( projectile_id, "ProjectileComponent" )
	ComponentSetValue2( pc_id, "on_death_explode", flag )
end

-- attaches projectile explosion to creature with enemy_id, explosion will launch projectiles equivalent to the projectile definition of projectile_id
function attachProjectileExplosiontoCreature(enemy_id, projectile_id)
	if(not entityHasSimplifiedInternalVariable(enemy_id, "projectile_explosion")) then
		addNewSimplifiedInternalVariable(enemy_id, "projectile_explosion")
		local projectile_definition = getProjectileDefinitionPath(projectile_id)
		addNewInternalVariable(enemy_id, "projectile_definition", "value_string", projectile_definition)
		addNewLuaCallback(enemy_id, "on_death", "mods/ArcaneReliquary/files/scripts/enemies/launch_projectiles.lua", 1, false, "launch_projectiles")
	end
end
-- variation of the above function where a projectile definition is supplied instead of an id for an existent projectile
function attachProjectileExplosiontoCreature_projectileDefinition(enemy_id, projectile_definition)
	if(not entityHasSimplifiedInternalVariable(enemy_id, "projectile_explosion")) then
		addNewSimplifiedInternalVariable(enemy_id, "projectile_explosion")
		addNewInternalVariable(enemy_id, "projectile_definition", "value_string", projectile_definition)
		addNewLuaCallback(enemy_id, "on_death", "mods/ArcaneReliquary/files/scripts/enemies/launch_projectiles.lua", 1, false, "launch_projectiles")
	end
end

-- attaches projectile random shooting to creature with enemy_id, shots are equivalent to the projectile definition of projectile_id
function attachRandomProjectileShootBehaviortoCreature(enemy_id, projectile_id)
	if(not entityHasSimplifiedInternalVariable(enemy_id, "shoot_projectile_random")) then
		addNewSimplifiedInternalVariable(enemy_id, "shoot_projectile_random")
		local projectile_definition = getProjectileDefinitionPath(projectile_id)
		addNewInternalVariable(enemy_id, "projectile_definition", "value_string", projectile_definition)
		addNewLuaCallback(enemy_id, "overtime", "mods/ArcaneReliquary/files/scripts/enemies/shoot_projectile_random.lua", 20, false, "shoot_projectile_random")
	end
end
-- variation of the above function where a projectile definition is supplied instead of an id for an existent projectile
function attachRandomProjectileShootBehaviortoCreature_projectileDefinition(enemy_id, projectile_definition)
	if(not entityHasSimplifiedInternalVariable(enemy_id, "shoot_projectile_random")) then
		addNewSimplifiedInternalVariable(enemy_id, "shoot_projectile_random")
		addNewInternalVariable(enemy_id, "projectile_definition", "value_string", projectile_definition)
		addNewLuaCallback(enemy_id, "overtime", "mods/ArcaneReliquary/files/scripts/enemies/shoot_projectile_random.lua", 20, false, "shoot_projectile_random")
	end
end

-- sets shooter of a projectile with projectile_id, used to set the author of a spell after it has been cast
function setProjectileShooter(projectile_id, shooter_id)
	local pc_id = EntityGetFirstComponentIncludingDisabled( projectile_id, "ProjectileComponent" )
	ComponentSetValue2( pc_id, "mWhoShot", shooter_id)
	ComponentSetValue2( pc_id, "mShooterHerdId", getCreatureHerd(shooter_id))
	--[[
	comp_ids = EntityGetComponent( projectile_id, "ProjectileComponent" )

	if( comp_ids ~= nil ) then
		ComponentSetValue( comp_ids[1], "mWhoShot", shooter_id)
		ComponentSetValue( comp_ids[1], "mShooterHerdId", getCreatureHerd(shooter_id))
	end
	]]
end

-- adds more uses for spells on wand that have limited uses and are compatible with projectile with projectile_id
-- if the wand has multiple spells compatible with projectile_id, then all will be updated by amount_to_add
function updatedUsesRemainingCurrentWandSpells_Projectile(projectile_id, amount_to_add)
	local wand_spells = EntityGetAllChildren(getCurrentlyEquippedWandId())
    if wand_spells == nil then return end
    for i,spell in ipairs(wand_spells) do
		if(EntityHasTag(spell, "card_action")) then
			--GamePrint("### projectile action id: |" .. getActionIdFromProjectile(projectile_id) .. "| spell action id: |" .. getActionIdFromSpell(spell) .. "|")
			if(getActionIdFromProjectile(projectile_id) == getActionIdFromSpell(spell)) then
				if(spellHasLimitedUses(spell)) then
					--GamePrint("### spell has limited uses")
					if(getUsesRemainingfromSpell(spell) ~= 0) then
						--GamePrint("### +1 use")
						addUsesRemainingtoSpell(spell, amount_to_add)
					end
				end
			end
		end
	end
	
	applyInventoryFix()
end

-- adds more uses for all spells on a wand that have limited uses
function updatedUsesRemainingCurrentWandSpells(amount_to_add)
	local wand_spells = EntityGetAllChildren(getCurrentlyEquippedWandId())
    if wand_spells == nil then return end
    for i,spell in ipairs(wand_spells) do
		if(EntityHasTag(spell, "card_action")) then
			if(spellHasLimitedUses(spell)) then
				if(getUsesRemainingfromSpell(spell) ~= 0) then
					addUsesRemainingtoSpell(spell, amount_to_add)
				end
			end
		end
	end
	
	applyInventoryFix()
end

-- function to obtain the id of the action related to a projectile with projectile_id
-- it makes use of the projectile_file internal variable contained on a projectile entity file
-- and also the related_projectiles array contained on every action in the actions list
function getActionIdFromProjectile(projectile_id)
	local projectile_definition = getProjectileDefinitionPath(projectile_id)
	local action_id = nil
	for i,action in ipairs(actions) do
		if(action.related_projectiles ~= nil) then
			for i,related_projectile in ipairs(action.related_projectiles) do
				if(related_projectile == projectile_definition) then
					action_id = action.id
					break
				end
			end
		end
	end
	--GamePrint("### this projectile is related to the action " .. action_id)
	return string.gsub(action_id, "_TRIGGER", "") -- fix for spells with _TRIGGER variation
end

-- function to correct a visual bug related to the counter of spells with limited uses
function applyInventoryFix()
	-- same fix used by UNLIMITED_SPELLS vanilla perk to update inventory numbers
	local inventory2_comp = EntityGetFirstComponent( getPlayerEntity(), "Inventory2Component" )
	if( inventory2_comp ) then
		ComponentSetValue( inventory2_comp, "mActualActiveItem", "0" )
	end
end

-- returns true if spell with spell_id has limited uses, false otherwise
function spellHasLimitedUses(spell_id)
	local action_id = getActionIdFromSpell(spell_id)
	local action = getAction(action_id)
	if(action.max_uses ~= nil) then
		return true
	else
		return false
	end
end

-- obtains an action definition from the actions table
function getAction(action_id)
	local found_action = nil
	for i,action in ipairs(actions) do
		if(action.id == action_id) then
			found_action = action
			break
		end
	end
	return found_action
end


function isActionPhysicsBased(action_id)
	local is_physics_action = true
	
	-- list of physics-based spells that behave differently than regular projectiles with velocity settings
	physics_actions_list = {
		"BOMB",
		"DYNAMITE",
		"GLITTER_BOMB",
		"TNTBOX",
		"TNTBOX_BIG",
		"BOMB_CART",
		"SUMMON_ROCK",
		"SUMMON_EGG",
		"SUMMON_HOLLOW_EGG",
		"BOMB_HOLY",
		"BOMB_HOLY_GIGA",
		"PROPANE_TANK"
	}
	
	for i,physics_action_id in ipairs(physics_actions_list) do
		if(physics_action_id == action_id) then
			is_physics_action = false
			break
		end
	end
	
	return is_physics_action
end

-- obtains the action id that corresponds to projectile_definition path
function getActionIdfromProjectileDefinition(projectile_definition)
	local action_id = nil
	for i,action in ipairs(actions) do
		if (action.related_projectiles[1] == projectile_definition) then
			action_id = action.id
			break
		end
	end
	return action_id
end

-- obtains projectile definition from related_projectiles prop of an action item as in gun_actions.lua, using action_id
function getProjectileDefinitionFromActionId(action_id)
	local projectile_definition = nil
	for i,action in ipairs(actions) do
		if(action.id == action_id) then
			if(action.related_projectiles ~= nil) then
				--GamePrint("### related projectiles found:" .. action.related_projectiles[1])
				projectile_definition = action.related_projectiles[1] -- captures the definition of the projectile, if any, ignoring the second element in the array that indicates amount
			end
			break
		end
	end
	return projectile_definition
end

-- obtains action name from spell with spell_id
function getActionIdFromSpell(spell_id)
	local iac_id = EntityGetFirstComponentIncludingDisabled( spell_id, "ItemActionComponent" )
	return ComponentGetValue2(iac_id, "action_id")
end

-- returns the id of a random action from the main actions table
function getRandomActionId()
	SetRandomSeed( GameGetFrameNum(), math.random(1,50))
	return actions[Random(1,#actions)].id
end

-- returns a random action from actions table that contains projectile definitions
function getRandomProjectileAction()
	
	local elligible_actions = {}
	for i,action in ipairs(actions) do
		if(action.related_projectiles ~= nil) then
			table.insert(elligible_actions, action)
		end
	end
	
	local count = 0
	for i,element in ipairs(elligible_actions) do
		count = count + 1
	end
	
	SetRandomSeed( GameGetFrameNum(), math.random(1,50))
	
	local chosen_action = elligible_actions[Random(1,count)]
	
	--GamePrint("### Elligible actions found: " .. count)
	--GamePrint("### Chosen action: " .. chosen_action.id)
	return chosen_action
end

-- adds more uses on a spell with spell_id that has limited uses
function addUsesRemainingtoSpell(spell_id, amount_to_add)
	local ic_id = EntityGetFirstComponentIncludingDisabled( spell_id, "ItemComponent" )
	local uses_remaining = ComponentGetValue2(ic_id, "uses_remaining")
	ComponentSetValue2(ic_id, "uses_remaining", uses_remaining + amount_to_add)
end

-- get uses remaining from a spell with spell_id, to determine how many times it can be used
function getUsesRemainingfromSpell(spell_id)
	local ic_id = EntityGetFirstComponentIncludingDisabled( spell_id, "ItemComponent" )
	return ComponentGetValue2(ic_id, "uses_remaining")
end

-- sets critical damage settings of a projectile with projectile_id
function setProjectileCriticalDamageSettings(projectile_id, chance, damage_multiplier)
	local pc_id = EntityGetFirstComponentIncludingDisabled( projectile_id, "ProjectileComponent" )
	ComponentObjectSetValue2( pc_id, "damage_critical", "chance", chance )
	ComponentObjectSetValue2( pc_id, "damage_critical", "damage_multiplier", damage_multiplier )
end

-- obtains critical damage settings from projectile with projectile_id
function getProjectileCriticalDamageSettings(projectile_id)
	local pc_id = EntityGetFirstComponentIncludingDisabled( projectile_id, "ProjectileComponent" )
	local chance = ComponentObjectGetValue2( pc_id, "damage_critical", "chance" )
	local damage_multiplier = ComponentObjectGetValue2( pc_id, "damage_critical", "damage_multiplier" )
	--GamePrint("### spell critical chance: " .. chance .. " | multiplier: " .. damage_multiplier)
	return chance, damage_multiplier
end

function addNewLimitedRepeatingLuaCallback(entity_id, script_path, times_to_execute, frames_between_runs, remove_after_executed, tag)
	local lc_id = nil
	lc_id = EntityAddComponent2(entity_id, "LuaComponent", {
		script_source_file=script_path,
		execute_every_n_frame=frames_between_runs,
		remove_after_executed=remove_after_executed,
		execute_times=times_to_execute
	})
	ComponentAddTag(lc_id, tag)
end

-- adds a new callback on entity with entity id
-- callback_type is the type of callback to be added
--		possible values:
--		- overtime for regular script_source_file callbacks
--		- on_collision for script_collision_trigger_hit callbacks
--		- on_damaged for script_damage_received callbacks
--		- on_death for script_death callbacks
--		- on_projectile_shot for script_shot callbacks
--		- on_kick for script_kick callbacks
-- script_path is the full path to the script to be assigned to the callback
-- frames_between_runs is the number of frames that must pass between runs of this callback
-- remove_after_executed is a flag indicating whether or not the callback should be destroyed once executed
-- tag is the custom tag to be added to the new callback
function addNewLuaCallback(entity_id, callback_type, script_path, frames_between_runs, remove_after_executed, tag)
	local lc_id = nil
	if(callback_type == "overtime") then
		lc_id = EntityAddComponent2(entity_id, "LuaComponent", {
			script_source_file=script_path,
			execute_every_n_frame=frames_between_runs,
			remove_after_executed=remove_after_executed
		})
	elseif(callback_type == "on_collision") then
		lc_id = EntityAddComponent2(entity_id, "LuaComponent", {
			script_collision_trigger_hit=script_path,
			execute_every_n_frame=frames_between_runs,
			remove_after_executed=remove_after_executed
		})
	elseif(callback_type == "on_damaged") then
		lc_id = EntityAddComponent2(entity_id, "LuaComponent", {
			script_damage_received=script_path,
			execute_every_n_frame=frames_between_runs,
			remove_after_executed=remove_after_executed
		})
	elseif(callback_type == "on_death") then
		lc_id = EntityAddComponent2(entity_id, "LuaComponent", {
			script_death=script_path,
			execute_every_n_frame=frames_between_runs,
			remove_after_executed=remove_after_executed
		})
	elseif(callback_type == "on_projectile_shot") then
		lc_id = EntityAddComponent2(entity_id, "LuaComponent", {
			script_shot=script_path,
			execute_every_n_frame=frames_between_runs,
			remove_after_executed=remove_after_executed
		})
	elseif(callback_type == "on_kick") then
		lc_id = EntityAddComponent2(entity_id, "LuaComponent", {
			script_kick=script_path,
			execute_every_n_frame=frames_between_runs,
			remove_after_executed=remove_after_executed
		})
	elseif(callback_type == "on_wand_fired") then
		lc_id = EntityAddComponent2(entity_id, "LuaComponent", {
			script_wand_fired=script_path,
			execute_every_n_frame=frames_between_runs,
			remove_after_executed=remove_after_executed
		})
	elseif(callback_type == "on_material_check") then
		lc_id = EntityAddComponent2(entity_id, "LuaComponent", {
			script_material_area_checker_success=script_path,
			execute_every_n_frame=frames_between_runs,
			remove_after_executed=remove_after_executed
		})
	elseif(callback_type == "on_throw") then
		lc_id = EntityAddComponent2(entity_id, "LuaComponent", {
			script_throw_item=script_path,
			execute_every_n_frame=frames_between_runs,
			remove_after_executed=remove_after_executed
		})
	end
	ComponentAddTag(lc_id, tag)
end

-- adds a new callback on entity with entity id that is to run when this entity dies
function addNewLuaCallback_onRemoved(entity_id, script_path)
	local lc_id = nil
	lc_id = EntityAddComponent2(entity_id, "LuaComponent", {
		script_source_file=script_path,
		execute_every_n_frame=-1,
		execute_on_removed=true
		
	})
end

-- adds a new tag to a callback on an entity with entity_id that contains callback_tag
function callbackAddTag(entity_id, callback_tag, new_tag)
	local components = EntityGetComponent( entity_id, "LuaComponent" )
	for i,component in ipairs(components) do
		if(ComponentHasTag( component, callback_tag )) then
			ComponentAddTag(component, new_tag)
		end
	end
end

-- function to remove a lua callback present on an entity with entity_id that includes a specific tag
function removeCallback(entity_id, tag)
	local components = EntityGetComponent( entity_id, "LuaComponent" )
	for i,component in ipairs(components) do
		if(ComponentHasTag( component, tag )) then
			EntityRemoveComponent(entity_id, component)
		end
	end
end

-- function that returns true if a lua callback with specified tag is present on entity with entity_id, or false otherwise
function entityHasCallback(entity_id, tag)
	local callbackFound = false
	local components = EntityGetComponent( entity_id, "LuaComponent" )
	for i,component in ipairs(components) do
		if(ComponentHasTag( component, tag )) then
			callbackFound = true
			break
		end
	end
	return callbackFound
end

-- function to toggle a lua callback present on an entity with entity_id that includes a specific tag
function toggleCallback(entity_id, tag, toggle)
	local components = EntityGetComponent( entity_id, "LuaComponent" )
	for i,component in ipairs(components) do
		if(ComponentHasTag( component, tag )) then
			EntitySetComponentIsEnabled( entity_id, component, toggle)
		end
	end
end

-- function to toggle a collision trigger present on an entity with entity_id that includes a specific tag
function toggleCollisionTrigger(entity_id, tag, toggle)
	local components = EntityGetComponentIncludingDisabled( entity_id, "CollisionTriggerComponent" )
	for i,component in ipairs(components) do
		if(ComponentHasTag( component, tag )) then
			EntitySetComponentIsEnabled( entity_id, component, toggle)
		end
	end
end

-- attaches collision handling mechanic to entity with entity_id
-- collision_radius is a number indicating the size of the collision box
-- required_tag is a string containing the tag that should trigger the handler
-- destroy_on_triggered is a flag indicating whether or not the handler should be destroyed ince triggered
-- tag is the custom tag to be added to the handler
function attachCollisionTrigger(entity_id, collision_radius, required_tag, destroy_on_triggered, tag)
	local ctc_id = EntityAddComponent2(entity_id, "CollisionTriggerComponent", {
		radius=collision_radius,
		required_tag=required_tag,
		destroy_this_entity_when_triggered=destroy_on_triggered
	})
	ComponentAddTag(ctc_id, tag)
end

-- removes a collision trigger from an entity with entity_id that includes a specific tag
function removeCollisionTrigger(entity_id, tag)
	local components = EntityGetComponent( entity_id, "CollisionTriggerComponent" )
	for i,component in ipairs(components) do
		if(ComponentHasTag( component, tag )) then
			EntityRemoveComponent(entity_id, component)
		end
	end
end

-- checks whether or not entity with entity_id has a collision trigger with a specific tag
function entityHasCollisionTrigger(entity_id, tag)
	local collision_trigger_id = EntityGetFirstComponentIncludingDisabled( entity_id, "CollisionTriggerComponent", tag )
	if(collision_trigger_id ~= nil) then
		return true
	else
		return false
	end
end

-- get all basic damage multipliers from entity with entity_id
function getBasicDamageMultipliers(entity_id)
	local dmc_id = EntityGetFirstComponentIncludingDisabled(entity_id, "DamageModelComponent")
	
	local ice = ComponentObjectGetValue2( dmc_id, "damage_multipliers", "ice" )
	local fire = ComponentObjectGetValue2( dmc_id, "damage_multipliers", "fire" )
	local drill = ComponentObjectGetValue2( dmc_id, "damage_multipliers", "drill" )
	local slice = ComponentObjectGetValue2( dmc_id, "damage_multipliers", "slice" )
	local melee = ComponentObjectGetValue2( dmc_id, "damage_multipliers", "melee" )
	local projectile = ComponentObjectGetValue2( dmc_id, "damage_multipliers", "projectile" )
	local radioactive = ComponentObjectGetValue2( dmc_id, "damage_multipliers", "radioactive" )
	local explosion = ComponentObjectGetValue2( dmc_id, "damage_multipliers", "explosion" )
	local electricity = ComponentObjectGetValue2( dmc_id, "damage_multipliers", "electricity" )
	
	--[[
	LUA: Damage Type: ice, 1
LUA: Damage Type: electricity, 1
LUA: Damage Type: radioactive, 1
LUA: Damage Type: slice, 1
LUA: Damage Type: curse, 1
LUA: Damage Type: overeating, 1
LUA: Damage Type: projectile, 1
LUA: Damage Type: healing, 1
LUA: Damage Type: physics_hit, 1
LUA: Damage Type: explosion, 0.35
LUA: Damage Type: poison, 1
LUA: Damage Type: melee, 1
LUA: Damage Type: drill, 1
LUA: Damage Type: fire, 1
	]]
	
	return ice, fire, drill, slice, melee, projectile, radioactive, explosion, electricity
end

-- gets the value for a damage multiplier of an entity with entity_id
-- multiplier_name must be either ice, fire, drill, slice, melee, projectile, radioactive, explosion or electricity
function getBasicDamageMultiplier(entity_id, multiplier_name)
	local dmc_id = EntityGetFirstComponentIncludingDisabled(entity_id, "DamageModelComponent")
	local multiplier = ComponentObjectGetValue2( dmc_id, "damage_multipliers", multiplier_name )
	return multiplier
end

-- sets a new value for a damage multiplier of an entity with entity_id
-- multiplier_name must be either ice, fire, drill, slice, melee, projectile, radioactive, explosion or electricity
function setBasicDamageMultiplier(entity_id, multiplier_name, new_multiplier)
	local dmc_id = EntityGetFirstComponentIncludingDisabled(entity_id, "DamageModelComponent")
	
	ComponentObjectSetValue2( dmc_id, "damage_multipliers", multiplier_name, new_multiplier )
end

-- multiplies explosion radius of projectile with projectile_id based on multiplier
function multiplyProjectileExplosionRadius(projectile_id, multiplier)
	local pc_id = EntityGetFirstComponentIncludingDisabled( projectile_id, "ProjectileComponent" )
	local explosion_radius = ComponentObjectGetValue2( pc_id, "config_explosion", "explosion_radius" )
	ComponentObjectSetValue2( pc_id, "config_explosion", "explosion_radius", explosion_radius * multiplier )
end

-- multiplies explosive damage of projectile with projectile_id based on multiplier
function multiplyProjectileExplosiveDamage(projectile_id, multiplier)
	local pc_id = EntityGetFirstComponentIncludingDisabled( projectile_id, "ProjectileComponent" )
	local explosion_damage = ComponentObjectGetValue2( pc_id, "config_explosion", "damage" )
	if(explosion_damage <= 0) then
		explosion_damage = 0.5
	end
	ComponentObjectSetValue2( pc_id, "config_explosion", "damage", explosion_damage * multiplier )
end

-- adds to explosive damage of projectile with projectile_id
function addProjectileExplosiveDamage(projectile_id, amount_to_add)
	local pc_id = EntityGetFirstComponentIncludingDisabled( projectile_id, "ProjectileComponent" )
	local explosion_damage = ComponentObjectGetValue2( pc_id, "config_explosion", "damage" )
	if(explosion_damage <= 0) then
		explosion_damage = 0.5
	end
	ComponentObjectSetValue2( pc_id, "config_explosion", "damage", explosion_damage + amount_to_add )
end

-- configures projectile to create material with material_name on explosion
function addProjectileCreateCellMaterialExplosion(projectile_id, material_name)
	local pc_id = EntityGetFirstComponentIncludingDisabled( projectile_id, "ProjectileComponent" )
	ComponentObjectSetValue2( pc_id, "config_explosion", "create_cell_probability", 100 )
	ComponentObjectSetValue2( pc_id, "config_explosion", "create_cell_material", material_name )
end

-- toggles physics objects damage flag on player
function playerPhysicsObjectsDamage(flag)
	local dmc_id = EntityGetFirstComponentIncludingDisabled(getPlayerEntity(), "DamageModelComponent")
	ComponentSetValue2(dmc_id, "physics_objects_damage", flag)
end

-- toggles physics objects damage flag on entity with entity_id
function creaturePhysicsObjectsDamage(entity_id, flag)
	local dmc_id = EntityGetFirstComponentIncludingDisabled(entity_id, "DamageModelComponent")
	ComponentSetValue2(dmc_id, "physics_objects_damage", flag)
end

function shootMirroredProjectile_WandSpells(projectile_id)
	
	local projectile_vel_x, projectile_vel_y, gravity = getProjectileVelocitiesandGravity(projectile_id)
	local projectile_x, projectile_y = EntityGetTransform(projectile_id)
	local projectile_definition = getProjectileDefinitionPath(projectile_id)
	
	local mirrored_projectile_id = EntityLoad(projectile_definition, projectile_x, projectile_y)
	addNewSimplifiedInternalVariable( mirrored_projectile_id, "relic_effect_projectile")
    projectileRemoveFriendlyFire(mirrored_projectile_id)
	setProjectileShooter(mirrored_projectile_id, getPlayerEntity())
	
	local vc_id = EntityGetFirstComponentIncludingDisabled(mirrored_projectile_id, "VelocityComponent")
	ComponentSetValueVector2( vc_id, "mVelocity", projectile_vel_x * -1, projectile_vel_y * -1)
	
	-- obtains all spell definitions from current wand
	local spell_definitions = getSpellDefinitionsFromWand(getCurrentlyEquippedWandId())
	for i,spell_definition in ipairs(spell_definitions) do
		-- shoots each spell found
		shootProjectileBasedonAnotherProjectile(mirrored_projectile_id, spell_definition, projectile_x, projectile_y, 150)
	end
	
	-- kills original projectile
	projectileToggleExplosionOnDeath(mirrored_projectile_id, false)
	EntityKill(mirrored_projectile_id)
end

-- shoots a copy of projectile with projectile_id at the opposite direction
function shootMirroredProjectile(projectile_id)
	
	local projectile_vel_x, projectile_vel_y, gravity = getProjectileVelocitiesandGravity(projectile_id)
	local projectile_x, projectile_y = EntityGetTransform(projectile_id)
	local projectile_definition = getProjectileDefinitionPath(projectile_id)
	
	local mirrored_projectile_id = EntityLoad(projectile_definition, projectile_x, projectile_y)
	addNewSimplifiedInternalVariable( mirrored_projectile_id, "relic_effect_projectile")
    projectileRemoveFriendlyFire(mirrored_projectile_id)
	setProjectileShooter(mirrored_projectile_id, getPlayerEntity())
	
	local vc_id = EntityGetFirstComponentIncludingDisabled(mirrored_projectile_id, "VelocityComponent")
	ComponentSetValueVector2( vc_id, "mVelocity", projectile_vel_x * -1, projectile_vel_y * -1)
end

-- turns off friendly fire capabilities of projectile with projectile_id
function projectileRemoveFriendlyFire(projectile_id)
	local pc_id = EntityGetFirstComponentIncludingDisabled( projectile_id, "ProjectileComponent" )
	ComponentSetValue2(pc_id, "friendly_fire", false)
	ComponentSetValue2(pc_id, "collide_with_shooter_frames", 0)
end

-- attaches projectile with time stopping functionalities
function attachStopandTrackMechanictoProjectile(projectile_id)
	
	if(not isPhysicsProjectile(projectile_id)) then
		-- prevents projectile from dying at low velocity
		local pc_id = EntityGetFirstComponentIncludingDisabled( projectile_id, "ProjectileComponent" )
		ComponentSetValue2(pc_id, "die_on_low_velocity", false)
		ComponentSetValue2(pc_id, "explosion_dont_damage_shooter", true)
		-- attaches stop projectile callback
		addNewLuaCallback(projectile_id, "overtime", "mods/ArcaneReliquary/files/scripts/projectiles/projectile_stop.lua", 5, true, "projectile_stop")
		addNewLuaCallback(projectile_id, "overtime", "mods/ArcaneReliquary/files/scripts/projectiles/projectile_track_and_shoot.lua", 10, true, "projectile_track_and_shoot")
	end
end

function attachStopandTrackMechanictoProjectile_wandSpells(projectile_id)
	
	if(not isPhysicsProjectile(projectile_id)) then
		-- prevents projectile from dying at low velocity
		local pc_id = EntityGetFirstComponentIncludingDisabled( projectile_id, "ProjectileComponent" )
		ComponentSetValue2(pc_id, "die_on_low_velocity", false)
		ComponentSetValue2(pc_id, "explosion_dont_damage_shooter", true)
		-- attaches stop projectile callback
		addNewLuaCallback(projectile_id, "overtime", "mods/ArcaneReliquary/files/scripts/projectiles/projectile_stop.lua", 5, true, "projectile_stop")
		addNewLuaCallback(projectile_id, "overtime", "mods/ArcaneReliquary/files/scripts/projectiles/projectile_track_and_shoot_wand_spells.lua", 10, true, "projectile_track_and_shoot")
	end
end

-- returns true if the projectile is physics based or false otherwise
function isPhysicsProjectile(projectile_id)
	local pisc_id = EntityGetComponent(projectile_id, "PhysicsImageShapeComponent")
	if(pisc_id ~= nil) then
		return true
	else
		return false
	end
end

-- returns true if projectile with projectile_id has the hittable tag, which enables it to be hit by other projectiles
function isProjectileHittable(projectile_id)
	return EntityHasTag(projectile_id, "hittable")
end

-- returns "right" if the player is moving right, "left" if the player is  moving left, or an empty string if neither
function getPlayerHorizontalMovement()
	local vel_x, vel_y = getPlayerVelocities()
	local horizontal_movement = ""
	if(vel_x > 10) then
		horizontal_movement = "right"
	end
	if(vel_x < -10) then
		horizontal_movement = "left"
	end
	return horizontal_movement
end

-- returns "down" if the player is falling, "up" if the player is going up, or an empty string if neither
function getPlayerVerticalMovement()
	local vel_x, vel_y = getPlayerVelocities()
	local vertical_movement = ""
	if(vel_y > 0 and vel_y ~= 60) then
		vertical_movement = "down"
	end
	if(vel_y < 0) then
		vertical_movement = "up"
	end
	return vertical_movement
end

-- returns true if the player is moving, or false otherwise
function isPlayerMoving()
	local flag = false
	local x, y = EntityGetTransform( getPlayerEntity() )
	
	local movement_threshold = 0.1
	if old_position and (math.abs(x - old_position.x) > movement_threshold or math.abs(y - old_position.y) > movement_threshold) then
		flag = true
	end
	old_position = {x=x, y=y};
	
	return flag
end

-- returns true if the player is standing on solid ground, or false if in the air
function isPlayerStanding()
	local vel_x, vel_y = getPlayerVelocities()
	if(vel_y == 60) then -- default gravity value when the player is standing on something solid
		return true
	else
		return false
	end
end

-- obtains x and y velocities from a creature with creature_id
function getCreatureVelocities(creature_id)
	local cdc_id = EntityGetFirstComponentIncludingDisabled(creature_id, "CharacterDataComponent")
	local velocity_x, velocity_y = ComponentGetValue2(cdc_id, "mVelocity")
	return velocity_x, velocity_y
end

-- sets x and y velocities of a creature with creature_id
function setCreatureVelocities(creature_id, vel_x, vel_y)
	local cdc_id = EntityGetFirstComponentIncludingDisabled(creature_id, "CharacterDataComponent")
	ComponentSetValueVector2(cdc_id, "mVelocity", vel_x, vel_y)
end

-- obtains x and y velocities from the player
function getPlayerVelocities()
	local cdc_id = EntityGetFirstComponentIncludingDisabled(getPlayerEntity(), "CharacterDataComponent")
	local velocity_x, velocity_y = ComponentGetValue2(cdc_id, "mVelocity")
	return velocity_x, velocity_y
end

-- sets x and y player velocities
function setPlayerVelocities(vel_x, vel_y)
	SetRandomSeed( GameGetFrameNum(), vel_x + vel_y + getPlayerEntity() )
	local cdc_id = EntityGetFirstComponentIncludingDisabled(getPlayerEntity(), "CharacterDataComponent")
	ComponentSetValueVector2(cdc_id, "mVelocity", vel_x, vel_y)
end

-- obtains the id of the shooter of projectile with projectile_id
function getProjectileShooter(projectile_id)
	local pc_id = EntityGetFirstComponentIncludingDisabled( projectile_id, "ProjectileComponent" )
	return ComponentGetValue2(pc_id, "mWhoShot")
end

-- obtains x and y velocities from projectile with projectile_id
function getProjectileVelocitiesandGravity(projectile_id)
	local vc_id = EntityGetFirstComponentIncludingDisabled(projectile_id, "VelocityComponent")
	local velocity_x, velocity_y = ComponentGetValue2(vc_id, "mVelocity")
	local gravity = ComponentGetValue2(vc_id, "gravity_y")
	return velocity_x, velocity_y, gravity
end

-- sets x and y velocities of a projectile with projectile_id
function setProjectileVelocitiesandGravity(projectile_id, vel_x, vel_y, gravity)
	local vc_id = EntityGetFirstComponentIncludingDisabled(projectile_id, "VelocityComponent")
	ComponentSetValue2(vc_id, "mVelocity", vel_x, vel_y)
	if(gravity ~= nil) then
		ComponentSetValue2(vc_id, "gravity_y", gravity)
	end
end

-- sets mass of projectile with projectile_id
function setProjectileMass(projectile_id, mass)
	local vc_id = EntityGetFirstComponentIncludingDisabled(projectile_id, "VelocityComponent")
	ComponentSetValue2(vc_id, "mass", mass)
end

-- attaches drag mechanic to projectile with projectile_id
function attachDragCollisionHandlertoProjectile(projectile_id, script_path, hook_config, collision_radius, target_tag, destroy_on_triggered)
	
	if(hook_config) then
		-- configures basic projectile collision settings
		local pc_id = EntityGetFirstComponentIncludingDisabled(projectile_id, "ProjectileComponent")
		ComponentSetValue2( pc_id, "penetrate_entities", true )
		ComponentSetValue2( pc_id, "on_collision_die", false  )
	end
	
	-- attaches collision callback
	addNewLuaCallback(projectile_id, "on_collision", script_path, -1, false, "drag_collision_handler")
	
	-- attaches collision trigger
	attachCollisionTrigger(projectile_id, collision_radius, target_tag, destroy_on_triggered, "collision_handler")
end

-- obtains the file definition path for projectile with projectile_id
function getProjectileDefinitionPath(projectile_id)
	return getInternalVariableValue(projectile_id, "projectile_file", "value_string")
end

-- checks whether or not projectile with projectile_id contains a variable with its projectile definition path
function projectileHasProjectileDefinitionPath(projectile_id)
	if(getProjectileDefinitionPath(projectile_id) ~= nil or getProjectileDefinitionPath(projectile_id) == "") then
		return false
	else
		return true
	end
end

-- shoots circular retaliation projectiles around entity with entity_id
function shootRetaliationProjectiles(entity_id, projectile_id, multiplier)
	
	local pos_x, pos_y = EntityGetTransform( entity_id )
	SetRandomSeed( GameGetFrameNum(), pos_x + pos_y + entity_id )
	
	local how_many = Random(10,20) * multiplier
	local angle_inc = ( 2 * 3.14159 ) / how_many
	local theta = 0
	local length = Random(300,500)

	for i=1,how_many do
		local shot_projectile_id = EntityLoad( getProjectileDefinitionPath(projectile_id), pos_x, pos_y)
		addNewSimplifiedInternalVariable( shot_projectile_id, "relic_effect_projectile")

		setProjectileShooter(shot_projectile_id, getPlayerEntity())

		local vel_x = math.cos( theta ) * length
		local vel_y = math.sin( theta ) * length
		theta = theta + angle_inc

		comp_ids = EntityGetComponent( shot_projectile_id, "VelocityComponent" )

		if( comp_ids ~= nil ) then
			ComponentSetValueVector2( comp_ids[1], "mVelocity", vel_x, vel_y)
		end
		
		setProjectileVelocitiesandGravity(shot_projectile_id, vel_x, vel_y)
	end
end

-- converts blood spilled by an enemy with enemy_id to new_blood_material 
function convertCreatureBloodToMaterial(enemy_id, new_blood_material)
	edit_component( enemy_id, "DamageModelComponent", function(comp,vars)
		vars.blood_material=new_blood_material
		vars.blood_spray_material=new_blood_material
	end)
end

-- returns the material that enemy with enemy_id bleeds
function getCreatureBloodMaterial(enemy_id)
	local dmc_id = EntityGetFirstComponentIncludingDisabled(enemy_id, "DamageModelComponent")
	return ComponentGetValue(dmc_id, "blood_spray_material")
end

-- returner whether or not the player has a child entity named child_name
function playerHasChildWithName(child_name)
	local flag = false
	local player_children = EntityGetAllChildren(getPlayerEntity())
    if player_children == nil then return end
    for i,child in ipairs(player_children) do
		if(EntityGetName(child) == child_name) then
			flag = true
		end
	end
	return flag
end

-- returns the name of the herd of a creature with creature_id
function getCreatureHerdName(creature_id)
	return HerdIdToString(getCreatureHerd(creature_id))
end
-- returns what herd id a creature with creature_id is part of
function getCreatureHerd(creature_id)
	local gdc_id = EntityGetFirstComponentIncludingDisabled( creature_id, "GenomeDataComponent" )
	return ComponentGetValue2( gdc_id, "herd_id" )
end

-- returns what herd the player is current part of
function getPlayerHerd()
	local gdc_id = EntityGetFirstComponentIncludingDisabled( getPlayerEntity(), "GenomeDataComponent" )
	return ComponentGetValue2( gdc_id, "herd_id" )
end

-- makes creature with enemy_id friendly
function makeCreatureFriendly(enemy_id, emitted_material_name)
	local gdc_id = EntityGetFirstComponentIncludingDisabled(enemy_id, "GenomeDataComponent")
	if(ComponentGetValue2( gdc_id, "herd_id" ) ~= getPlayerHerd()) then
		ComponentSetValue2( gdc_id, "herd_id", getPlayerHerd() )
		if(emitted_material_name ~= nil) then
			addBuffVisualIndicator(enemy_id, emitted_material_name)
		else
			addBuffVisualIndicator(enemy_id, "plasma_fading_pink")
		end
		disableGoldReward(enemy_id)
		EntityRemoveTag( enemy_id, "enemy" )
	end
end

function attachTimeBombMechanictoEntity(entity_id)
	if(not entityHasSimplifiedInternalVariable(entity_id, "explosion_rigged")) then
		-- adds explosion on death settings
		local ec_id = EntityAddComponent2(entity_id, "ExplosionComponent", {
			trigger="ON_DEATH"
		})
		
		ComponentObjectSetValue2( ec_id, "config_explosion", "never_cache", false )
		ComponentObjectSetValue2( ec_id, "config_explosion", "camera_shake", 50.0 )
		ComponentObjectSetValue2( ec_id, "config_explosion", "explosion_radius", 60 )
		ComponentObjectSetValue2( ec_id, "config_explosion", "explosion_sprite", "mods/ArcaneReliquary/files/other_gfx/explosion_128_fiery.xml" )
		ComponentObjectSetValue2( ec_id, "config_explosion", "explosion_sprite_lifetime", 0 )
		ComponentObjectSetValue2( ec_id, "config_explosion", "hole_destroy_liquid", false  )
		ComponentObjectSetValue2( ec_id, "config_explosion", "hole_enabled", true  )
		ComponentObjectSetValue2( ec_id, "config_explosion", "ray_energy", 6000000 )
		ComponentObjectSetValue2( ec_id, "config_explosion", "explosion_sprite_additive", true )
		ComponentObjectSetValue2( ec_id, "config_explosion", "damage", 5 )
		ComponentObjectSetValue2( ec_id, "config_explosion", "max_durability_to_destroy", 11 )
		ComponentObjectSetValue2( ec_id, "config_explosion", "particle_effect", true  )
		ComponentObjectSetValue2( ec_id, "config_explosion", "damage_mortals", true )
		ComponentObjectSetValue2( ec_id, "config_explosion", "physics_explosion_power", 2.2, 3.6)
		ComponentObjectSetValue2( ec_id, "config_explosion", "physics_throw_enabled", true )
		ComponentObjectSetValue2( ec_id, "config_explosion", "shake_vegetation", true  )
		ComponentObjectSetValue2( ec_id, "config_explosion", "sparks_count_max", 15  )
		ComponentObjectSetValue2( ec_id, "config_explosion", "sparks_count_min", 12  )
		ComponentObjectSetValue2( ec_id, "config_explosion", "sparks_enabled", true  )
		ComponentObjectSetValue2( ec_id, "config_explosion", "load_this_entity", "data/entities/particles/particle_explosion/main_gunpowder_large.xml,data/entities/misc/loose_ground.xml,data/entities/misc/explosion_was_here.xml" )
		ComponentObjectSetValue2( ec_id, "config_explosion", "audio_enabled", true )
		ComponentObjectSetValue2( ec_id, "config_explosion", "audio_event_name", "explosions/tnt" )
		
		-- flags entity_id as explosion_rigged
		addNewSimplifiedInternalVariable(entity_id, "explosion_rigged")
		
		-- adds variable to control tick numbers
		addNewInternalVariable(entity_id, "ticks", "value_int", 0)
		
		-- attaches tick control that signals when the entity is about to explode
		addNewLuaCallback(entity_id, "overtime", "mods/ArcaneReliquary/files/scripts/other_listeners/time_bomb_tick_control.lua", 30, false, "time_bomb_tick_control")
		
	end
end

-- attaches material explosion on enemy with enemy_id
function attachMaterialExplosiontoCreature(enemy_id, material_name, size)
	
	if(not entityHasSimplifiedInternalVariable(enemy_id, "explosion_rigged")) then
		-- adds explosion on death settings
		local ec_id = EntityAddComponent2(enemy_id, "ExplosionComponent", {
			trigger="ON_DEATH"
		})
		
		local material_amount = 0
	
		if(size == "medium") then
			material_amount = math.random(250,350)
			ComponentObjectSetValue2( ec_id, "config_explosion", "never_cache", true )
			ComponentObjectSetValue2( ec_id, "config_explosion", "camera_shake", 5.0 )
			ComponentObjectSetValue2( ec_id, "config_explosion", "explosion_radius", 35 )
			ComponentObjectSetValue2( ec_id, "config_explosion", "explosion_sprite", "data/particles/explosion_040_poof.xml" )
			ComponentObjectSetValue2( ec_id, "config_explosion", "explosion_sprite_lifetime", 0 )
			ComponentObjectSetValue2( ec_id, "config_explosion", "hole_destroy_liquid", false  )
			ComponentObjectSetValue2( ec_id, "config_explosion", "hole_enabled", true  )
			ComponentObjectSetValue2( ec_id, "config_explosion", "ray_energy", 4000000 )
			ComponentObjectSetValue2( ec_id, "config_explosion", "explosion_sprite_additive", true )
			ComponentObjectSetValue2( ec_id, "config_explosion", "damage", 1.5 )
			ComponentObjectSetValue2( ec_id, "config_explosion", "max_durability_to_destroy", 10 )
			ComponentObjectSetValue2( ec_id, "config_explosion", "particle_effect", false  )
			ComponentObjectSetValue2( ec_id, "config_explosion", "damage_mortals", true )
			ComponentObjectSetValue2( ec_id, "config_explosion", "physics_explosion_power", 0.1, 0.25)
			ComponentObjectSetValue2( ec_id, "config_explosion", "physics_throw_enabled", true )
			ComponentObjectSetValue2( ec_id, "config_explosion", "shake_vegetation", true  )
			ComponentObjectSetValue2( ec_id, "config_explosion", "sparks_count_max", 20  )
			ComponentObjectSetValue2( ec_id, "config_explosion", "sparks_count_min", 7  )
			ComponentObjectSetValue2( ec_id, "config_explosion", "sparks_enabled", true  )
			ComponentObjectSetValue2( ec_id, "config_explosion", "light_enabled", false  )
			ComponentObjectSetValue2( ec_id, "config_explosion", "load_this_entity", "data/entities/particles/particle_explosion/main_gunpowder_tiny.xml" )
			ComponentObjectSetValue2( ec_id, "config_explosion", "audio_enabled", true )
			ComponentObjectSetValue2( ec_id, "config_explosion", "audio_event_name", "explosions/tnt" )
		elseif(size == "big") then
			material_amount = math.random(550,750)
			ComponentObjectSetValue2( ec_id, "config_explosion", "create_cell_probability", 100  )
			ComponentObjectSetValue2( ec_id, "config_explosion", "create_cell_material", "fire"  )
			ComponentObjectSetValue2( ec_id, "config_explosion", "never_cache", true  )
			ComponentObjectSetValue2( ec_id, "config_explosion", "camera_shake", 15.0  )
			ComponentObjectSetValue2( ec_id, "config_explosion", "explosion_radius", 95 )
			ComponentObjectSetValue2( ec_id, "config_explosion", "explosion_sprite", "mods/ArcaneReliquary/files/other_gfx/explosion_128_fiery.xml" )
			ComponentObjectSetValue2( ec_id, "config_explosion", "explosion_sprite_lifetime", 0  )
			ComponentObjectSetValue2( ec_id, "config_explosion", "hole_destroy_liquid", false  )
			ComponentObjectSetValue2( ec_id, "config_explosion", "hole_enabled", true  )
			ComponentObjectSetValue2( ec_id, "config_explosion", "ray_energy", 8000000 )
			ComponentObjectSetValue2( ec_id, "config_explosion", "explosion_sprite_additive", true )
			ComponentObjectSetValue2( ec_id, "config_explosion", "damage", 5 )
			ComponentObjectSetValue2( ec_id, "config_explosion", "max_durability_to_destroy", 15 )
			ComponentObjectSetValue2( ec_id, "config_explosion", "particle_effect", false  )
			ComponentObjectSetValue2( ec_id, "config_explosion", "damage_mortals", true )
			ComponentObjectSetValue2( ec_id, "config_explosion", "physics_explosion_power", 0.1, 0.25 )
			ComponentObjectSetValue2( ec_id, "config_explosion", "physics_throw_enabled", true )
			ComponentObjectSetValue2( ec_id, "config_explosion", "shake_vegetation", true  )
			ComponentObjectSetValue2( ec_id, "config_explosion", "sparks_count_max", 20  )
			ComponentObjectSetValue2( ec_id, "config_explosion", "sparks_count_min", 7  )
			ComponentObjectSetValue2( ec_id, "config_explosion", "sparks_enabled", true  )
			ComponentObjectSetValue2( ec_id, "config_explosion", "light_enabled", false  )
			ComponentObjectSetValue2( ec_id, "config_explosion", "load_this_entity", "data/entities/particles/particle_explosion/main_gunpowder_large.xml" )
			ComponentObjectSetValue2( ec_id, "config_explosion", "audio_enabled", true )
			ComponentObjectSetValue2( ec_id, "config_explosion", "audio_event_name", "explosions/box" )
		end
		
		-- removes any material inventory enemy might already have
		local materialInventories = EntityGetComponent(enemy_id, "MaterialInventoryComponent")
		if( materialInventories ~= nil ) then
			for i,materialInventory in ipairs(materialInventories) do
				EntityRemoveComponent(enemy_id, materialInventory)
			end
		end
		
		-- adds material inventory with provided material
		local mic_id = EntityAddComponent2(enemy_id, "MaterialInventoryComponent", {
			on_death_spill=true
		})
		AddMaterialInventoryMaterial( enemy_id, material_name, material_amount)
		
		addNewSimplifiedInternalVariable(enemy_id, "explosion_rigged")
	end
end

-- attaches suicide bomber logic to creature with enemy_id
function makeCreatureSuicideBomber(enemy_id, explosion_material_path, emitter_material_name)
	
	-- first makes it friendly towards the player
	makeCreatureFriendly(enemy_id, emitter_material_name)
	
	-- adds explosion on death settings
	local ec_id = EntityAddComponent2(enemy_id, "ExplosionComponent", {
		trigger="ON_DEATH"
	})
	
	ComponentObjectSetValue2( ec_id, "config_explosion", "never_cache", true )
	ComponentObjectSetValue2( ec_id, "config_explosion", "camera_shake", 5.0 )
	ComponentObjectSetValue2( ec_id, "config_explosion", "explosion_radius", 35 )
	
	ComponentObjectSetValue2( ec_id, "config_explosion", "explosion_sprite_lifetime", 0 )
	ComponentObjectSetValue2( ec_id, "config_explosion", "hole_destroy_liquid", false  )
	ComponentObjectSetValue2( ec_id, "config_explosion", "hole_enabled", true  )
	ComponentObjectSetValue2( ec_id, "config_explosion", "ray_energy", 4000000 )
	ComponentObjectSetValue2( ec_id, "config_explosion", "explosion_sprite_additive", true )
	if(explosion_material_path ~= nil) then
		ComponentObjectSetValue2( ec_id, "config_explosion", "damage", 1 )
		if(string.find(explosion_material_path, "acid")) then
			ComponentObjectSetValue2( ec_id, "config_explosion", "explosion_sprite", "mods/ArcaneReliquary/files/other_gfx/explosion_064_green.xml" )
		elseif(string.find(explosion_material_path, "poison")) then
			ComponentObjectSetValue2( ec_id, "config_explosion", "explosion_sprite", "mods/ArcaneReliquary/files/other_gfx/explosion_064_purple.xml" )
		elseif(string.find(explosion_material_path, "lava")) then
			ComponentObjectSetValue2( ec_id, "config_explosion", "explosion_sprite", "mods/ArcaneReliquary/files/other_gfx/explosion_064_orange.xml" )
		elseif(string.find(explosion_material_path, "magic_liquid_polymorph")) then
			ComponentObjectSetValue2( ec_id, "config_explosion", "explosion_sprite", "mods/ArcaneReliquary/files/other_gfx/explosion_064_pink.xml" )
		end
	else
		ComponentObjectSetValue2( ec_id, "config_explosion", "damage", 2.5 )
		ComponentObjectSetValue2( ec_id, "config_explosion", "explosion_sprite", "mods/ArcaneReliquary/files/other_gfx/explosion_064_red.xml" )
	end
	ComponentObjectSetValue2( ec_id, "config_explosion", "max_durability_to_destroy", 10 )
	ComponentObjectSetValue2( ec_id, "config_explosion", "particle_effect", false  )
	ComponentObjectSetValue2( ec_id, "config_explosion", "damage_mortals", true )
	ComponentObjectSetValue2( ec_id, "config_explosion", "physics_explosion_power", 0.1, 0.25)
	ComponentObjectSetValue2( ec_id, "config_explosion", "physics_throw_enabled", true )
	ComponentObjectSetValue2( ec_id, "config_explosion", "shake_vegetation", true  )
	ComponentObjectSetValue2( ec_id, "config_explosion", "sparks_count_max", 20  )
	ComponentObjectSetValue2( ec_id, "config_explosion", "sparks_count_min", 7  )
	ComponentObjectSetValue2( ec_id, "config_explosion", "sparks_enabled", true  )
	ComponentObjectSetValue2( ec_id, "config_explosion", "light_enabled", false  )
	if(explosion_material_path ~= nil) then
		ComponentObjectSetValue2( ec_id, "config_explosion", "load_this_entity", "data/entities/particles/particle_explosion/main_gunpowder_medium.xml" .. "," .. explosion_material_path )
	else
		ComponentObjectSetValue2( ec_id, "config_explosion", "load_this_entity", "data/entities/particles/particle_explosion/main_gunpowder_medium.xml" )
	end
	ComponentObjectSetValue2( ec_id, "config_explosion", "audio_enabled", true )
	ComponentObjectSetValue2( ec_id, "config_explosion", "audio_event_name", "explosions/tnt" )
	
	-- attaches detection field
	addNewLuaCallback(enemy_id, "overtime", "mods/ArcaneReliquary/files/scripts/other_listeners/enemy_suicide_bomber.lua", 5, false, "enemy_suicide_bomber")
	
	local emitted_material = ""
	
	if(emitter_material_name ~= nil) then
		emitted_material = emitter_material_name
	else
		emitted_material = "plasma_fading_pink"
	end
	
	-- adds particle emitter
	local pec_id = EntityAddComponent2(enemy_id, "ParticleEmitterComponent", {
		emitted_material_name=emitted_material,
		lifetime_min=0.3,
		lifetime_max=0.4,
		count_min=10,
		count_max=15,
		render_on_grid=true,
		fade_based_on_lifetime=true,
		cosmetic_force_create=false,
		airflow_force=0.5,
		airflow_time=0.5,
		airflow_scale=0.15,
		emission_interval_min_frames=1,
		emission_interval_max_frames=1,
		emit_cosmetic_particles=true,
		draw_as_long=true,
		x_vel_min=-1,
		x_vel_max=1,
		y_vel_min=-1,
		y_vel_max=1,
		is_emitting=true,
		velocity_always_away_from_center=100
	})
	ComponentSetValue2( pec_id, "gravity", 0.0, 0.0 )
	ComponentSetValue2( pec_id, "area_circle_radius", 10, 10)
end

-- attaches projectile with projectile_id with explosion config
function projectileSetExplosionConfig(projectile_id, level)
	if(not isPhysicsProjectile(projectile_id)) then
		local pc_id = EntityGetFirstComponentIncludingDisabled(projectile_id, "ProjectileComponent")
		ComponentSetValue2( pc_id, "on_death_explode", true )
		ComponentSetValue2( pc_id, "on_collision_die", true  )
		ComponentSetValue2( pc_id, "explosion_dont_damage_shooter", false  )
		
		local explosion_damage = 0.5 * (1 + level)
		local explosion_radius = 8 * (1 + level)
		
		ComponentObjectSetValue2( pc_id, "config_explosion", "damage", explosion_damage )
		ComponentObjectSetValue2( pc_id, "config_explosion", "explosion_radius", explosion_radius )
		
		ComponentObjectSetValue2( pc_id, "config_explosion", "never_cache", true )
		ComponentObjectSetValue2( pc_id, "config_explosion", "camera_shake", 0.5 )
		ComponentObjectSetValue2( pc_id, "config_explosion", "explosion_sprite_lifetime", 0  )
		ComponentObjectSetValue2( pc_id, "config_explosion", "hole_destroy_liquid", false  )
		ComponentObjectSetValue2( pc_id, "config_explosion", "hole_enabled", true  )
		ComponentObjectSetValue2( pc_id, "config_explosion", "explosion_sprite_additive", true )
		ComponentObjectSetValue2( pc_id, "config_explosion", "max_durability_to_destroy", 5 )
		ComponentObjectSetValue2( pc_id, "config_explosion", "particle_effect", false )
		ComponentObjectSetValue2( pc_id, "config_explosion", "damage_mortals", true )
		ComponentObjectSetValue2( pc_id, "config_explosion", "physics_explosion_power", 0.3, 0.5 )
		ComponentObjectSetValue2( pc_id, "config_explosion", "physics_throw_enabled", true )
		ComponentObjectSetValue2( pc_id, "config_explosion", "shake_vegetation", true  )
		ComponentObjectSetValue2( pc_id, "config_explosion", "sparks_count_max", 20  )
		ComponentObjectSetValue2( pc_id, "config_explosion", "sparks_count_min", 7  )
		ComponentObjectSetValue2( pc_id, "config_explosion", "sparks_enabled", true  )
		ComponentObjectSetValue2( pc_id, "config_explosion", "light_enabled", false  )
		ComponentObjectSetValue2( pc_id, "config_explosion", "audio_enabled", true )
		
		if(level >= 0 and level <= 3) then
			ComponentObjectSetValue2( pc_id, "config_explosion", "explosion_sprite", "data/particles/explosion_032.xml"  )
			ComponentObjectSetValue2( pc_id, "config_explosion", "ray_energy", 500000 )
			ComponentObjectSetValue2( pc_id, "config_explosion", "load_this_entity", "data/entities/particles/particle_explosion/main_gunpowder_tiny.xml" )
			ComponentObjectSetValue2( pc_id, "config_explosion", "audio_event_name", "explosions/magic_grenade_small" )
		elseif(level >= 3 and level <= 6) then
			ComponentObjectSetValue2( pc_id, "config_explosion", "explosion_sprite", "mods/ArcaneReliquary/files/other_gfx/explosion_064_fiery.xml" )
			ComponentObjectSetValue2( pc_id, "config_explosion", "ray_energy", 4000000 )
			ComponentObjectSetValue2( pc_id, "config_explosion", "load_this_entity", "data/entities/particles/particle_explosion/main_gunpowder_medium.xml" )
			ComponentObjectSetValue2( pc_id, "config_explosion", "audio_event_name", "explosions/tnt" )
		
		elseif(level >= 6 and level <= 10) then
			ComponentObjectSetValue2( pc_id, "config_explosion", "explosion_sprite", "mods/ArcaneReliquary/files/other_gfx/explosion_128_fiery.xml" )
			ComponentObjectSetValue2( pc_id, "config_explosion", "ray_energy", 8000000 )
			ComponentObjectSetValue2( pc_id, "config_explosion", "load_this_entity", "data/entities/particles/particle_explosion/main_gunpowder_large.xml" )
			ComponentObjectSetValue2( pc_id, "config_explosion", "audio_event_name", "explosions/box" )
		end
	end
end

-- configures projectile so that it is able to explode on collision
function projectileMakeExplodeonImpact(projectile_id)
	local pc_id = EntityGetFirstComponentIncludingDisabled(projectile_id, "ProjectileComponent")
	ComponentSetValue2( pc_id, "on_death_explode", true )
	ComponentSetValue2( pc_id, "on_collision_die", true )
end

-- returns whether or not a projectile with projectile_id is able to explode on impact
function projectileExplodesonImpact(projectile_id)
	local pc_id = EntityGetFirstComponentIncludingDisabled(projectile_id, "ProjectileComponent")
	local on_death_explode = ComponentGetValue2( pc_id, "on_death_explode" )
	local on_collision_die = ComponentGetValue2( pc_id, "on_collision_die" )
	
	if(on_death_explode and on_collision_die) then
		return true
	else
		return false
	end
end

-- attaches entity to load_this_entity of explosion config object of a projectile with projectile_id
function attachLoadEntityOnExplosiontoProjectile(projectile_id, load_on_explosion_path, buff_indicator_material)
	if(projectileExplodesonImpact(projectile_id)) then
		local pc_id = EntityGetFirstComponentIncludingDisabled(projectile_id, "ProjectileComponent")
		local load_this_entity = ComponentObjectGetValue2( pc_id, "config_explosion", "load_this_entity" )
		load_this_entity = load_this_entity .. "," .. load_on_explosion_path
		ComponentObjectSetValue2( pc_id, "config_explosion", "load_this_entity", load_this_entity )
		if(buff_indicator_material ~= nil and buff_indicator_material ~= "") then
			addBuffVisualIndicator_Projectile(projectile_id, buff_indicator_material)
		end
	end
end

-- attaches a status effect entity to damage_game_effect_entities property of a projectile with projectile_id
function attachLoadEffectOntoEntitiestoProjectile(projectile_id, effect_path, buff_indicator_material)
	local pc_id = EntityGetFirstComponentIncludingDisabled(projectile_id, "ProjectileComponent")
	local damage_game_effect_entities = ComponentGetValue2(pc_id, "damage_game_effect_entities")
	damage_game_effect_entities = damage_game_effect_entities .. "," .. effect_path
	ComponentSetValue2(pc_id, "damage_game_effect_entities", damage_game_effect_entities)
	if(buff_indicator_material ~= nil and buff_indicator_material ~= "") then
		addBuffVisualIndicator_Projectile(projectile_id, buff_indicator_material)
	end
end

-- multiplies current velocity of a projectile with projectile_id by multiplier provided
function multiplyProjectileVelocity(projectile_id, multiplier)
	local vc_id = EntityGetFirstComponentIncludingDisabled(projectile_id, "VelocityComponent")
	projectile_vel_x, projectile_vel_y = ComponentGetValueVector2( vc_id, "mVelocity")
	ComponentSetValueVector2( vc_id, "mVelocity", projectile_vel_x * multiplier, projectile_vel_y * multiplier)
end

function wandAddReloadTime(wand_id, amount_to_add)
	local ac_id = EntityGetFirstComponentIncludingDisabled(wand_id, "AbilityComponent")
	local reload_time = ComponentObjectGetValue2( ac_id, "gun_config", "reload_time" )
	ComponentObjectSetValue2( ac_id, "gun_config", "reload_time", reload_time + amount_to_add)
	applyInventoryFix()
end

function wandAddCastDelay(wand_id, amount_to_add)
	local ac_id = EntityGetFirstComponentIncludingDisabled(wand_id, "AbilityComponent")
	local fire_rate_wait = ComponentObjectGetValue2( ac_id, "gunaction_config", "fire_rate_wait" )
	ComponentObjectSetValue2( ac_id, "gunaction_config", "fire_rate_wait", fire_rate_wait + amount_to_add)
	applyInventoryFix()
end

function wandAddManaMax(wand_id, amount_to_add)
	local ac_id = EntityGetFirstComponentIncludingDisabled(wand_id, "AbilityComponent")
	local mana_max = ComponentGetValue2(ac_id, "mana_max")
	ComponentSetValue2(ac_id, "mana_max", mana_max + amount_to_add)
	applyInventoryFix()
end

-- returns maximum mana active wand has
function getCurrentWandManaMax()
	local mana_max = 0
	local active_wand_id = getCurrentlyEquippedWandId()
	if(active_wand_id ~= -1) then
		local ac_id = EntityGetFirstComponentIncludingDisabled(active_wand_id, "AbilityComponent")
		mana_max = ComponentGetValue2(ac_id, "mana_max")
	end
	return mana_max
end

-- returns currently active wand id, or returns -1 if it is not a wand
function getCurrentlyEquippedWandId()
	local i2c_id = EntityGetFirstComponentIncludingDisabled(getPlayerEntity(), "Inventory2Component")
	local wand_id = ComponentGetValue2( i2c_id, "mActiveItem" )
	
	if(EntityHasTag(wand_id, "wand")) then
		return wand_id
	else
		return -1
	end
end

-- returns the rotation value for the current item held by the player
function getCurrentlyHeldItemRotation()
	local i2c_id = EntityGetFirstComponentIncludingDisabled(getPlayerEntity(), "Inventory2Component")
	local active_item = ComponentGetValue2( i2c_id, "mActiveItem" )
	if(active_item ~= nil) then
		local x, y, rotation = EntityGetTransform(active_item)
		return rotation
	else
		return -1
	end
end

function getSpellDefinitionsFromWand(wand_id)
	local spell_definitions = {}
	local ac_id = EntityGetFirstComponentIncludingDisabled(wand_id, "AbilityComponent")
	local wand_children = EntityGetAllChildren(wand_id)
    if wand_children == nil then return end
    for i,child in ipairs(wand_children) do
		if(EntityHasTag(child, "card_action")) then
			local action_id = getActionIdFromSpell(child)
			local spell_definition = getProjectileDefinitionFromActionId(action_id)
			if(spell_definition ~= nil) then
				table.insert(spell_definitions, spell_definition)
			end
		end
	end
	return spell_definitions
end

-- returns how many empty and occupied slots, respectively, the wand with wand_id contains
function getWandEmptyandOccupiedSlots(wand_id)
	-- first obtains how many slots the wand has
	local ac_id = EntityGetFirstComponentIncludingDisabled(wand_id, "AbilityComponent")
	local wand_deck_size = ComponentObjectGetValue2( ac_id, "gun_config", "deck_capacity" )
	
	-- then, counts how many spells are currently attached to the wand
	local spells = 0
	local wand_children = EntityGetAllChildren(wand_id)
    if wand_children == nil then return end
    for i,child in ipairs(wand_children) do
		if(EntityHasTag(child, "card_action")) then
			spells = spells + 1
		end
	end
	
	local free_slots = wand_deck_size - spells
	
	--GamePrint("### Current wand has " .. free_slots .. " free slots and " .. spells .. " actions")
	
	return free_slots, spells
end

-- sets new material emitted by player jetpack and switches from cosmetic from real particles
function setJetpackMaterial(material_name)
	local pec_id = EntityGetFirstComponentIncludingDisabled(getPlayerEntity(), "ParticleEmitterComponent", "jetpack")
	ComponentSetValue2(pec_id, "emitted_material_name", material_name)
	ComponentSetValue2(pec_id, "create_real_particles", true)
	ComponentSetValue2(pec_id, "emit_cosmetic_particles", false)
end

-- reverts jetpack config to vanilla settings
function revertDefaultJetpackMaterial()
	local pec_id = EntityGetFirstComponentIncludingDisabled(getPlayerEntity(), "ParticleEmitterComponent", "jetpack")
	ComponentSetValue2(pec_id, "emitted_material_name", getDefaultJetpackMaterial())
	ComponentSetValue2(pec_id, "create_real_particles", false)
	ComponentSetValue2(pec_id, "emit_cosmetic_particles", true)
end

-- obtains current material emitted by the player jetpack
function getJetpackMaterial()
	local pec_id = EntityGetFirstComponentIncludingDisabled(getPlayerEntity(), "ParticleEmitterComponent", "jetpack")
	local emitted_material_name = ComponentGetValue2( pec_id, "emitted_material_name" )
	return emitted_material_name
end

-- obtains default material emitted by the player jetpack
function getDefaultJetpackMaterial()
	return getInternalVariableValue(getPlayerEntity(), "default_jetpack_material", "value_string")
end

-- stores default player values on internal variables set at init.lua
function storeDefaultPlayerValues()
	addNewInternalVariable(getPlayerEntity(), "default_jetpack_material", "value_string", getJetpackMaterial())
	
	local cd_id = EntityGetFirstComponentIncludingDisabled(getPlayerEntity(), "CharacterDataComponent")
	addNewInternalVariable(getPlayerEntity(), "fly_time_max", "value_float", ComponentGetValue2( cd_id, "fly_time_max" ))
	
	local cpc_id = EntityGetFirstComponentIncludingDisabled(getPlayerEntity(), "CharacterPlatformingComponent")
	addNewInternalVariable(getPlayerEntity(), "jump_velocity_y", "value_int", ComponentGetValue2( cpc_id, "jump_velocity_y" ))
	addNewInternalVariable(getPlayerEntity(), "jump_velocity_x", "value_int", ComponentGetValue2( cpc_id, "jump_velocity_x" ))
	
	local melee_multiplier = getBasicDamageMultiplier(getPlayerEntity(), "melee")
	addNewInternalVariable(getPlayerEntity(), "melee_multiplier", "value_float", melee_multiplier)
	
	local slice_multiplier = getBasicDamageMultiplier(getPlayerEntity(), "slice")
	addNewInternalVariable(getPlayerEntity(), "slice_multiplier", "value_float", slice_multiplier)
	
	local explosion_multiplier = getBasicDamageMultiplier(getPlayerEntity(), "explosion")
	addNewInternalVariable(getPlayerEntity(), "explosion_multiplier", "value_float", explosion_multiplier)
	
	local explosion_multiplier = getBasicDamageMultiplier(getPlayerEntity(), "radioactive")
	addNewInternalVariable(getPlayerEntity(), "radioactive_multiplier", "value_float", explosion_multiplier)
	
	local explosion_multiplier = getBasicDamageMultiplier(getPlayerEntity(), "electricity")
	addNewInternalVariable(getPlayerEntity(), "electricity_multiplier", "value_float", explosion_multiplier)
	
	local explosion_multiplier = getBasicDamageMultiplier(getPlayerEntity(), "fire")
	addNewInternalVariable(getPlayerEntity(), "fire_multiplier", "value_float", explosion_multiplier)
end

-- enables jumping mechanic and turns jetpack off
function enableJumpingandDisableJetpack()
	local cd_id = EntityGetFirstComponentIncludingDisabled(getPlayerEntity(), "CharacterDataComponent")
	local cpc_id = EntityGetFirstComponentIncludingDisabled(getPlayerEntity(), "CharacterPlatformingComponent")
	
	ComponentSetValue2(cd_id, "fly_time_max", 0)
	ComponentSetValue2(cpc_id, "jump_velocity_y", -200)
	ComponentSetValue2(cpc_id, "jump_velocity_x", 100)
end

-- reverts jetpack functionality to its normal behavior
function revertOriginalJetpackFunctionality()
	local cd_id = EntityGetFirstComponentIncludingDisabled(getPlayerEntity(), "CharacterDataComponent")
	local cpc_id = EntityGetFirstComponentIncludingDisabled(getPlayerEntity(), "CharacterPlatformingComponent")

	ComponentSetValue2(cd_id, "fly_time_max", getInternalVariableValue(getPlayerEntity(), "fly_time_max", "value_float"))
	ComponentSetValue2(cpc_id, "jump_velocity_y", getInternalVariableValue(getPlayerEntity(), "jump_velocity_y", "value_int"))
	ComponentSetValue2(cpc_id, "jump_velocity_x", getInternalVariableValue(getPlayerEntity(), "jump_velocity_x", "value_int"))
end

-- obtains the base damge of a projectile with projectile_id
function getProjectileBaseDamage(projectile_id)
	local pc_id = EntityGetFirstComponentIncludingDisabled(projectile_id, "ProjectileComponent")
	local base_damage = ComponentGetValue2( pc_id, "damage" )
	return base_damage
end

-- fractions damage of projectile with projectile_id according do damage_dividend
function fractionProjectileDamage(projectile_id, damage_dividend)
	local pc_id = EntityGetFirstComponentIncludingDisabled(projectile_id, "ProjectileComponent")
	local base_damage = ComponentGetValue2( pc_id, "damage" )
	if(base_damage ~= 0) then
		base_damage = base_damage / damage_dividend
	end
	ComponentSetValue2(pc_id, "damage", base_damage)
end

function fractionProjectileExplosiveDamage(projectile_id, damage_dividend)
	local pc_id = EntityGetFirstComponentIncludingDisabled( projectile_id, "ProjectileComponent" )
	local explosion_damage = ComponentObjectGetValue2( pc_id, "config_explosion", "damage" )
	if(explosion_damage <= 0) then
		explosion_damage = explosion_damage / damage_dividend
	end
	ComponentObjectSetValue2( pc_id, "config_explosion", "damage", explosion_damage )
end

-- adds to the base damage of a projectile with projectile_id
function addProjectileBaseDamage(projectile_id, damage_to_add)
	local pc_id = EntityGetFirstComponentIncludingDisabled(projectile_id, "ProjectileComponent")
	local base_damage = ComponentGetValue2( pc_id, "damage" )
	base_damage = base_damage + damage_to_add
	ComponentSetValue2(pc_id, "damage", base_damage)
end

-- applies damage boost to projectile with projectile_id
function multiplyProjectileBaseDamage(projectile_id, damage_multiplier)
	local pc_id = EntityGetFirstComponentIncludingDisabled(projectile_id, "ProjectileComponent")
	local base_damage = ComponentGetValue2( pc_id, "damage" )
	if(base_damage ~= 0) then
		base_damage = base_damage * damage_multiplier
	else
		base_damage = 0.1 * damage_multiplier
	end
	ComponentSetValue2(pc_id, "damage", base_damage)
end

function enemyEnableShootingBasedonProjectile(projectile_id, enemy_id)
	if(not enemyRangedAttacksEnabled(enemy_id)) then
		-- adds ranged attack
		local aaic_id = EntityGetFirstComponentIncludingDisabled(enemy_id, "AnimalAIComponent")
		ComponentSetValue2(aaic_id, "attack_ranged_enabled", true)
		ComponentSetValue2(aaic_id, "attack_ranged_entity_file", getProjectileDefinitionPath(projectile_id))
		ComponentSetValue2(aaic_id, "attack_ranged_frames_between", math.random(50,100))
	end
end

-- applies ranged attack buff on enemy with enemy_id if they have no ranged attacks
function enemyApplyRangedAttackBuff(enemy_id)

	if(not enemyRangedAttacksEnabled(enemy_id)) then
	
		-- adds ranged attack
		local aaic_id = EntityGetFirstComponentIncludingDisabled(enemy_id, "AnimalAIComponent")
		ComponentSetValue2(aaic_id, "attack_ranged_enabled", true)
		
		local ranged_attack = math.random(1,16)
		if(ranged_attack == 1) then
			ComponentSetValue2(aaic_id, "attack_ranged_entity_file", "data/entities/projectiles/deck/light_bullet.xml")
		elseif(ranged_attack == 2) then
			ComponentSetValue2(aaic_id, "attack_ranged_entity_file", "mods/ArcaneReliquary/files/entity_definitions/custom_projectiles/grenade_custom_enemy.xml")
		elseif(ranged_attack == 3) then
			ComponentSetValue2(aaic_id, "attack_ranged_entity_file", "data/entities/projectiles/deck/firebomb.xml")
		elseif(ranged_attack == 4) then
			ComponentSetValue2(aaic_id, "attack_ranged_entity_file", "mods/ArcaneReliquary/files/entity_definitions/custom_projectiles/bouncy_orb_custom.xml")
		elseif(ranged_attack == 5) then
			ComponentSetValue2(aaic_id, "attack_ranged_entity_file", "data/entities/projectiles/deck/bullet_slow.xml")
		elseif(ranged_attack == 6) then
			ComponentSetValue2(aaic_id, "attack_ranged_entity_file", "mods/ArcaneReliquary/files/entity_definitions/custom_projectiles/disc_bullet_custom.xml")
		elseif(ranged_attack == 7) then
			ComponentSetValue2(aaic_id, "attack_ranged_entity_file", "mods/ArcaneReliquary/files/entity_definitions/custom_projectiles/fireball_custom.xml")
		elseif(ranged_attack == 8) then
			ComponentSetValue2(aaic_id, "attack_ranged_entity_file", "mods/ArcaneReliquary/files/entity_definitions/custom_projectiles/bullet_heavy_custom.xml")
		elseif(ranged_attack == 9) then
			ComponentSetValue2(aaic_id, "attack_ranged_entity_file", "data/entities/projectiles/deck/glue_shot.xml")
		elseif(ranged_attack == 10) then
			ComponentSetValue2(aaic_id, "attack_ranged_entity_file", "mods/ArcaneReliquary/files/entity_definitions/custom_projectiles/rocket_custom.xml")
		elseif(ranged_attack == 11) then
			ComponentSetValue2(aaic_id, "attack_ranged_entity_file", "mods/ArcaneReliquary/files/entity_definitions/custom_projectiles/bullet_custom.xml")
		elseif(ranged_attack == 12) then
			ComponentSetValue2(aaic_id, "attack_ranged_entity_file", "data/entities/projectiles/deck/slime.xml")
		elseif(ranged_attack == 13) then
			ComponentSetValue2(aaic_id, "attack_ranged_entity_file", "data/entities/projectiles/deck/tentacle.xml")
		elseif(ranged_attack == 14) then
			ComponentSetValue2(aaic_id, "attack_ranged_entity_file", "mods/ArcaneReliquary/files/entity_definitions/custom_projectiles/rocket_player_custom.xml")
		elseif(ranged_attack == 15) then
			ComponentSetValue2(aaic_id, "attack_ranged_entity_file", "data/entities/projectiles/deck/lightning.xml")
		elseif(ranged_attack == 16) then
			ComponentSetValue2(aaic_id, "attack_ranged_entity_file", "mods/ArcaneReliquary/files/entity_definitions/custom_projectiles/iceball_custom.xml")
		end
		
		ComponentSetValue2(aaic_id, "attack_ranged_frames_between", math.random(30,100))
		
		-- disables vanilla gold rewards
		disableGoldReward(enemy_id)

	end
end

function enemyApplyMoreProjectilesBuff(enemy_id, multiplier)

	if(enemyRangedAttacksEnabled) then
		
		-- multiplies projectiles
		local aaic_id = EntityGetFirstComponentIncludingDisabled(enemy_id, "AnimalAIComponent")
		local attack_ranged_entity_count_min = ComponentGetValue2( aaic_id, "attack_ranged_entity_count_min" )
		local attack_ranged_entity_count_max = ComponentGetValue2( aaic_id, "attack_ranged_entity_count_max" )
		local new_max_projectiles = attack_ranged_entity_count_max * multiplier
		ComponentSetValue2(aaic_id, "attack_ranged_entity_count_min", new_max_projectiles)
		ComponentSetValue2(aaic_id, "attack_ranged_entity_count_max", new_max_projectiles)
		
		-- disables vanilla gold rewards
		disableGoldReward(enemy_id)
	end
end

-- applies health boost on enemy with enemy_id
function enemyApplyHealthBuff(enemy_id, health_boost_multiplier)

	--local health_boost_multiplier = 5
	
	-- buffs health
	local dmc_id = EntityGetFirstComponentIncludingDisabled(enemy_id, "DamageModelComponent")
	local max_hp = ComponentGetValue2( dmc_id, "max_hp" )
	local hp = ComponentGetValue2( dmc_id, "hp" )
	local new_hp = max_hp * health_boost_multiplier
	ComponentSetValue2(dmc_id, "max_hp", new_hp)
	ComponentSetValue2(dmc_id, "hp", new_hp)
	
	-- disables vanilla gold rewards
	disableGoldReward(enemy_id)
end

-- applies firerate boost on enemy with enemy_id if they have ranged attacks enabled
function enemyApplyFirerateBuff(enemy_id, firerate_boost_multiplier)

	if(enemyRangedAttacksEnabled(enemy_id)) then
		--local firerate_boost_multiplier = 5
		
		-- buffs firerate
		local aaic_id = EntityGetFirstComponentIncludingDisabled(enemy_id, "AnimalAIComponent")
		local attack_ranged_frames_between = ComponentGetValue2( aaic_id, "attack_ranged_frames_between" )
		ComponentSetValue2(aaic_id, "attack_ranged_frames_between", math.floor(attack_ranged_frames_between / firerate_boost_multiplier))
		
		-- disables vanilla gold rewards
		disableGoldReward(enemy_id)
	end
end

-- applies melee damage boost on enemy with enemy_id
function enemyApplyMeleeDamageBuff(enemy_id, melee_damage_multiplier)

	--local melee_damage_multiplier = 3
	
	-- buffs melee damage
	local aaic_id = EntityGetFirstComponentIncludingDisabled(enemy_id, "AnimalAIComponent")
	local attack_melee_damage_min = ComponentGetValue2( aaic_id, "attack_melee_damage_min" )
	local attack_melee_damage_max = ComponentGetValue2( aaic_id, "attack_melee_damage_max" )
	local new_melee_damage = attack_melee_damage_max * melee_damage_multiplier
	ComponentSetValue2(aaic_id, "attack_melee_damage_min", new_melee_damage)
	ComponentSetValue2(aaic_id, "attack_melee_damage_max", new_melee_damage)
	
	-- disables vanilla gold rewards
	disableGoldReward(enemy_id)
end

-- applies jumping and run speed boosts on enemy with enemy_id if they are not a flying enemy
function enemyApplyMovementBuff(enemy_id, movement_multiplier)
	
	local cpc_id = EntityGetFirstComponentIncludingDisabled(enemy_id, "CharacterPlatformingComponent")
	
	if(enemyFlightEnabled(enemy_id)) then
		-- buffs fly velocity
		local fly_velocity_x = ComponentGetValue2( cpc_id, "fly_velocity_x" )
		local fly_speed_max_up = ComponentGetValue2( cpc_id, "fly_speed_max_up" )
		local fly_speed_max_down = ComponentGetValue2( cpc_id, "fly_speed_max_down" )
		ComponentSetValue2(cpc_id, "fly_velocity_x", fly_velocity_x * movement_multiplier)
		ComponentSetValue2(cpc_id, "fly_speed_max_up", fly_speed_max_up * movement_multiplier)
		ComponentSetValue2(cpc_id, "fly_speed_max_down", fly_speed_max_down * movement_multiplier)
	else
		-- buffs movement (jumping and run speed)
		local jump_velocity_y = ComponentGetValue2( cpc_id, "jump_velocity_y" )
		local run_velocity = ComponentGetValue2( cpc_id, "run_velocity" )
		ComponentSetValue2(cpc_id, "run_velocity", run_velocity * movement_multiplier)
		ComponentSetValue2(cpc_id, "jump_velocity_y", jump_velocity_y * movement_multiplier)
	end
	
	-- disables vanilla gold rewards
	disableGoldReward(enemy_id)
end

-- applies movement speed debuff on creature with entity_id, returning changed props
function creatureApplyMovementDebuff(entity_id, debuff_threshold)
	local cpc_id = EntityGetFirstComponentIncludingDisabled(entity_id, "CharacterPlatformingComponent")
	local jump_velocity_y = ComponentGetValue2( cpc_id, "jump_velocity_y" )
	local jump_velocity_x = ComponentGetValue2( cpc_id, "jump_velocity_x" )
	local run_velocity = ComponentGetValue2( cpc_id, "run_velocity" )
	ComponentSetValue2(cpc_id, "run_velocity", run_velocity / debuff_threshold)
	ComponentSetValue2(cpc_id, "jump_velocity_y", jump_velocity_y / debuff_threshold)
	ComponentSetValue2(cpc_id, "jump_velocity_x", jump_velocity_x / debuff_threshold)
	return run_velocity, jump_velocity_y, jump_velocity_x
end

-- sets movement and jumping speed on creature with entity_id
function creatureSetMovementVelocities(entity_id, run_velocity, jump_velocity_y, jump_velocity_x)
	local cpc_id = EntityGetFirstComponentIncludingDisabled(entity_id, "CharacterPlatformingComponent")
	ComponentSetValue2(cpc_id, "run_velocity", run_velocity)
	ComponentSetValue2(cpc_id, "jump_velocity_y", jump_velocity_y)
	ComponentSetValue2(cpc_id, "jump_velocity_x", jump_velocity_x)
end

-- removes a visual buff indicator from an enemy creature
function removeBuffVisualIndicator(enemy_id, tag)
	local emitters = EntityGetComponentIncludingDisabled(enemy_id, "ParticleEmitterComponent", tag)
	for i,emitter in ipairs(emitters) do
		EntityRemoveComponent(enemy_id, emitter)
	end
end

function addBuffIcon(enemy_id, icon_name, x_offset)
	local visible_flag = true
	local emissive_flag = true
	local opacity = 0
	-- obtains settings to configure icons
	local icon_transparency = ModSettingGet("ArcaneReliquary.buff_icon_transparency")
	local icon_visibility = ModSettingGet("ArcaneReliquary.buff_icon_visibility")
	
	-- sets variables accordingly
	if(icon_transparency == "OPAQUE") then opacity = 1 end
	if(icon_transparency == "TRANSLUSCENT") then opacity = 0.2 end
	
	if(icon_visibility == "VISIBLE_FOG") then 
		visible_flag = true
		emissive_flag = true
	elseif(icon_visibility == "INVISIBLE_FOG") then 
		visible_flag = true
		emissive_flag = false
	elseif(icon_visibility == "INVISIBLE_ALL") then 
		visible_flag = false
		emissive_flag = false
	end


	EntityAddComponent2(enemy_id, "SpriteComponent", {
		image_file="mods/ArcaneReliquary/files/buff_icons/" .. icon_name .. ".png",
		offset_x=x_offset,
		offset_y=32,
		alpha=opacity,
		emissive=emissive_flag,
		additive=true,
		update_transform=true,
		update_transform_rotation=true,
		visible=visible_flag,
		z_index=-9000,
		has_special_scale=true,
		special_scale_x=0.7,
		special_scale_y=0.7,
		never_ragdollify_on_death=true,
	})
end

-- function to add a visual buff indicator on an enemy creature
function addBuffVisualIndicator(enemy_id, emitted_material, tag)
	
	local custom_style_name = "NONE"
	if(emitted_material == "fire") then
		custom_style_name = "FIRE"
	end
	
	local pec_id = EntityAddComponent2(enemy_id, "ParticleEmitterComponent", {
		emitted_material_name=emitted_material,
		custom_style=custom_style_name,
		x_pos_offset_min=-3,
		x_pos_offset_max=3,
		y_pos_offset_min=-10,
		y_pos_offset_max=0,
		x_vel_min=-8,
		x_vel_max=8,
		y_vel_min=-8,
		y_vel_max=8,
		count_min=2,
		count_max=4,
		lifetime_min=0.6,
		lifetime_max=3.5,
		airflow_force=1.0,
		airflow_time=0.401,
		airflow_scale=0.25,
		create_real_particles=false,
		emit_cosmetic_particles=true,
		render_on_grid=true,
		emission_interval_min_frames=1,
		emission_interval_max_frames=2,
		fade_based_on_lifetime=true,
		is_emitting=true
	})
	ComponentSetValue2( pec_id, "offset", 0, 0)
	ComponentSetValue2( pec_id, "gravity", 0, 0)
	
	if(tag ~= nil) then
		ComponentAddTag( pec_id, tag )
	end
end

function addProjectileSpriteParticleTrail(projectile_id, sprite_trail_path)
	local spec_id = EntityAddComponent2(projectile_id, "SpriteParticleEmitterComponent", {
		sprite_file=sprite_trail_path,
		delay=0,
		lifetime=0,
		velocity_slowdown=0,
		rotation=0,
		angular_velocity=0,
		use_velocity_as_rotation=false,
		emission_interval_min_frames=1,
		emission_interval_max_frames=3,
		count_min=1,
		count_max=1,
	})
	ComponentSetValue2( spec_id, "scale", 1, 1)
	ComponentSetValue2( spec_id, "scale_velocity", 0, 0)
	ComponentSetValue2( spec_id, "color", 1, 1, 1, 1)
	ComponentSetValue2( spec_id, "color_change", 0, 0, 0, 0)
	ComponentSetValue2( spec_id, "velocity", 0, 0)
	ComponentSetValue2( spec_id, "gravity", 0, 10)
	ComponentSetValue2( spec_id, "randomize_rotation", -3.1415, 3.1415)
	ComponentSetValue2( spec_id, "randomize_position", -2, 2, -2, 2)
end


-- function to add a visual buff indicator to a projectile
function addBuffVisualIndicator_Projectile(projectile_id, emitted_material)
	if(emitted_material == "spark" or emitted_material == "fire") then
		local pec_id = EntityAddComponent2(projectile_id, "ParticleEmitterComponent", {
			emitted_material_name=emitted_material,
			custom_style="FIRE",
			x_pos_offset_min=-3,
			x_pos_offset_max=3,
			y_pos_offset_min=-3,
			y_pos_offset_max=3,
			x_vel_min=-8,
			x_vel_max=8,
			y_vel_min=-8,
			y_vel_max=8,
			count_min=2,
			count_max=4,
			lifetime_min=0.6,
			lifetime_max=3.5,
			airflow_force=1.0,
			airflow_time=0.401,
			airflow_scale=0.25,
			create_real_particles=false,
			emit_cosmetic_particles=true,
			render_on_grid=true,
			emission_interval_min_frames=1,
			emission_interval_max_frames=2,
			fade_based_on_lifetime=true,
			is_emitting=true
		})
		ComponentSetValue2( pec_id, "offset", 0, 0)
		ComponentSetValue2( pec_id, "gravity", 0, 0)
	else
		local pec_id = EntityAddComponent2(projectile_id, "ParticleEmitterComponent", {
			emitted_material_name=emitted_material,
			x_pos_offset_min=-3,
			x_pos_offset_max=3,
			y_pos_offset_min=-3,
			y_pos_offset_max=3,
			x_vel_min=-8,
			x_vel_max=8,
			y_vel_min=-8,
			y_vel_max=8,
			count_min=2,
			count_max=4,
			lifetime_min=0.6,
			lifetime_max=3.5,
			airflow_force=1.0,
			airflow_time=0.401,
			airflow_scale=0.25,
			create_real_particles=false,
			emit_cosmetic_particles=true,
			render_on_grid=true,
			emission_interval_min_frames=1,
			emission_interval_max_frames=2,
			fade_based_on_lifetime=true,
			is_emitting=true
		})
		ComponentSetValue2( pec_id, "offset", 0, 0)
		ComponentSetValue2( pec_id, "gravity", 0, 0)
	end
end

-- toggles platforming ability on enemy with enemy_id
function enemyTogglePlatforming(enemy_id, toggle_platforming)
	local cpc_id = EntityGetFirstComponentIncludingDisabled(enemy_id, "CharacterPlatformingComponent")
	EntitySetComponentIsEnabled( enemy_id, cpc_id, toggle_platforming)
end

-- toggles flight behavior on enemy with enemy_id
function toggleEnemyFlight(enemy_id, toggle_flight)
	local aaic_id = EntityGetFirstComponentIncludingDisabled(enemy_id, "AnimalAIComponent")
	ComponentSetValue2(aaic_id, "can_fly", toggle_flight)
end

-- toggles ranged attacks on enemy with enemy_id
function toggleEnemyRangedAttacks(enemy_id, toggle_ranged_attacks)
	local aaic_id = EntityGetFirstComponentIncludingDisabled(enemy_id, "AnimalAIComponent")
	ComponentSetValue2(aaic_id, "attack_ranged_enabled", toggle_ranged_attacks)
end

-- toggles jumping ability on enemy with enemy_id
function toggleEnemyJump(enemy_id, toggle_jump)
	local pfc_id = EntityGetFirstComponentIncludingDisabled(enemy_id, "PathFindingComponent")
	ComponentSetValue2(pfc_id, "can_jump", toggle_jump)
end

-- checks whether or not enemy with enemy_id is able to jump
function enemyJumpEnabled(enemy_id)
	local pfc_id = EntityGetFirstComponentIncludingDisabled(enemy_id, "PathFindingComponent")
	local can_jump = ComponentGetValue2( pfc_id, "can_jump" )
	return can_jump
end

-- checks whether or not enemy of enemy_id has ranged attacks
function enemyRangedAttacksEnabled(enemy_id)
	local aaic_id = EntityGetFirstComponentIncludingDisabled(enemy_id, "AnimalAIComponent")
	local attack_ranged_enabled = ComponentGetValue2( aaic_id, "attack_ranged_enabled" )
	return attack_ranged_enabled
end

-- checks whether or not enemy of enemy_id is able to fly
function enemyFlightEnabled(enemy_id)
	local aaic_id = EntityGetFirstComponentIncludingDisabled(enemy_id, "AnimalAIComponent")
	local can_fly = ComponentGetValue2( aaic_id, "can_fly" )
	return can_fly
end

-- function to disable enemy gold reward
function disableGoldReward(enemy_id)
	EntityAddComponent2(enemy_id, "VariableStorageComponent", {
		_tags = "no_gold_drop", 
	})
end

function disableAlternateDamageSources(enemy_id)
	local dmc_id = EntityGetFirstComponentIncludingDisabled(enemy_id, "DamageModelComponent")
	ComponentSetValue2(dmc_id, "physics_objects_damage", false)
end

-- inventory_type can be either "inventory_quick" or "inventory_full", but in this case all relics go to the full inventory
function getPlayerInventory(inventory_type)
	local player_children = EntityGetAllChildren(getPlayerEntity())
	local inventory = nil
    if player_children == nil then return end
    for i,child in ipairs(player_children) do
		if(EntityGetName(child) == inventory_type) then
			inventory = child
			break
		end
	end
	return inventory
end

-- returns true if the full inventory is currently full
function isFullInventoryFull()
	local items_in_inventory = 0
	local inventory_items = EntityGetAllChildren(getPlayerInventory("inventory_full"))
	if(inventory_items ~= nil) then
		for i,item in ipairs(inventory_items) do
			items_in_inventory = items_in_inventory + 1
		end
	end
	--GamePrint("### Items in inventory: " .. items_in_inventory)
	if(items_in_inventory < getFullInventorySize()) then
		return false
	else
		return true
	end
end

-- returns the currently amount of slots present on player's full inventory
function getFullInventorySize()
	local i2c_id = EntityGetFirstComponentIncludingDisabled(getPlayerEntity(), "Inventory2Component")
	local inventory_x = ComponentGetValue2(i2c_id, "full_inventory_slots_x")
	local inventory_y = ComponentGetValue2(i2c_id, "full_inventory_slots_y")
	local full_inventory_size = inventory_x * inventory_y
	--GamePrint("### Full inventory size: " .. full_inventory_size)
	return full_inventory_size
end

-- returns total relics owned, regardless of name
--[[
function countTotalOwnedRelics()
	local counter = 0
	local inventory_items = EntityGetAllChildren(getPlayerInventory("inventory_full"))
	if(inventory_items ~= nil) then
		for i,item in ipairs(inventory_items) do
			if EntityHasTag(item, "relic") then
				counter = counter + 1
			end
		end
	end
	return counter
end
]]
-- # deprecated version, updated one is below, new version queries all direct player children instead of inventory children and returns a list of ids instead of a list of names
function countTotalOwnedRelics()
	local counter = 0
	local player_children = EntityGetAllChildren(getPlayerEntity())
	if(player_children ~= nil) then
		for i,item in ipairs(player_children) do
			if EntityHasTag(item, "relic") then
				counter = counter + 1
			end
		end
	end
	return counter
end

-- encloses the logic that levels up relics
-- relic must be an element of the relics_list table
-- relic_id is the id of the relic entity
-- new_relic_level is the new level the relic is going to receive
function setRelicLevel(relic, relic_id, new_relic_level)
	-- if relic has no levels, initializes it
	if(not entityHasInternalVariableValue(relic_id, "relic_level")) then
		addNewInternalVariable(relic_id, "relic_level", "value_int", 0)
	end
	setInternalVariableValue(relic_id, "relic_level", "value_int", new_relic_level)
	-- updates ui text to show new level gained
	local simplified_ui_name = relic.ui_name .. " " .. levelNumbertoRoman(new_relic_level)
	local descriptive_ui_name = simplified_ui_name .. " - " .. relic.effect_desc
	-- if relic is upgradable, adds it to main description
	if(relic.is_upgradable) then
		descriptive_ui_name = descriptive_ui_name .. "\n" .. relic.upgrade_effect_desc
	else
		descriptive_ui_name = descriptive_ui_name .. "\nThis relic can't be upgraded."
	end
	local short_ui_name = simplified_ui_name .. " - " .. relic.effect_desc_short

	updateRelicUIText(relic_id, simplified_ui_name, descriptive_ui_name, short_ui_name)
end

-- obtains a list of all owned relics containing their names
--[[
function getOwnedRelics()
	local owned_relics = {}
	local inventory_items = EntityGetAllChildren(getPlayerInventory("inventory_full"))
	if(inventory_items ~= nil) then
		for i,item in ipairs(inventory_items) do
			if EntityHasTag(item, "relic") then
				table.insert(owned_relics, EntityGetName(item))
			end
		end
	end
	return owned_relics
end
]]
-- # deprecated version, updated one is below, new version queries all direct player children instead of inventory children and returns a list of ids instead of a list of names

function getOwnedRelics()
	local owned_relics = {}
	local player_children = EntityGetAllChildren(getPlayerEntity())
	if(player_children ~= nil) then
		for i,item in ipairs(player_children) do
			if EntityHasTag(item, "relic") then
				table.insert(owned_relics, item)
			end
		end
	end
	return owned_relics
end

--[[
function getRelicLevel(relic_name)
	local level = 0
	local inventory_items = EntityGetAllChildren(getPlayerInventory("inventory_full"))
	if(inventory_items ~= nil) then
		for i,item in ipairs(inventory_items) do
			if EntityHasTag(item, "relic") then
				if(EntityGetName(item) == relic_name) then
					if(entityHasInternalVariableValue(item, "relic_level")) then
						level = getInternalVariableValue(item, "relic_level", "value_int")
					end
				end
			end
		end
	end
	return level
end
]]
-- # deprecated version, updated one is below, takes into account player children instead of inventory children

function getRelicLevel(relic_name)
	local level = 0
	local player_children = EntityGetAllChildren(getPlayerEntity())
	if(player_children ~= nil) then
		for i,item in ipairs(player_children) do
			if EntityHasTag(item, "relic") then
				if(EntityGetName(item) == relic_name) then
					if(entityHasInternalVariableValue(item, "relic_level")) then
						level = getInternalVariableValue(item, "relic_level", "value_int")
					end
				end
			end
		end
	end
	return level
end

-- returns how many times relic with relic_name is owned, or zero if none
--[[
function isRelicOwned(relic_name)
	local counter = 0
	local inventory_items = EntityGetAllChildren(getPlayerInventory("inventory_full"))
	if(inventory_items ~= nil) then
		for i,item in ipairs(inventory_items) do
			if EntityHasTag(item, "relic") then
				if(EntityGetName(item) == relic_name) then
					counter = counter + 1
				end
			end
		end
	end
	return counter
end
]]
-- # deprecated version, updated one is below, takes into account player children instead of inventory children
-- # and also what is the is active flag value before returning relic ownership

function isRelicOwned(relic_name)
	local counter = 0
	local player_children = EntityGetAllChildren(getPlayerEntity())
	if(player_children ~= nil) then
		for i,item in ipairs(player_children) do
			if EntityHasTag(item, "relic") then
				if(EntityGetName(item) == relic_name and isRelicActive(item)) then
					counter = counter + 1
				end
			end
		end
	end
	return counter
end

-- simple utility function to return the player entity
function getPlayerEntity()
	local players = EntityGetWithTag("player_unit")
	if #players == 0 then return end

	return players[1]
end

-- function to return player health
-- health values are treated internally as floats, so if the player has 5 health, the internal value will be 0.008
function getPlayerHealth()
	local damagemodels = EntityGetComponent( getPlayerEntity(), "DamageModelComponent" )
	local health = 0
	if( damagemodels ~= nil ) then
		for i,v in ipairs(damagemodels) do
		
			health = tonumber( ComponentGetValue( v, "hp" ) )
			--GamePrint(health)
			break
		end
	end
	return health
end

-- function to return player max health
function getPlayerMaxHealth()
	local damagemodels = EntityGetComponent( getPlayerEntity(), "DamageModelComponent" )
	local maxHealth = 0
	if( damagemodels ~= nil ) then
		for i,v in ipairs(damagemodels) do
		
			maxHealth = tonumber( ComponentGetValue( v, "max_hp" ) )
			
			break
		end
	end
	return maxHealth
end

function getCreatureHealth(creature_id)
	local damagemodels = EntityGetComponent( creature_id, "DamageModelComponent" )
	local health = 0
	if( damagemodels ~= nil ) then
		for i,v in ipairs(damagemodels) do
		
			health = tonumber( ComponentGetValue( v, "hp" ) )
			break
		end
	end
	return health
end

-- function that adds or subtracts health from a creature with entity_id
-- amount should be provided as a float value, e.g. 0.04 == 1 hp
function addCreatureHealth(entity_id, amount, overheal_flag)
	local damagemodels = EntityGetComponent( entity_id, "DamageModelComponent" )
	
	if( damagemodels ~= nil ) then
		for i,v in ipairs(damagemodels) do
		
			currentHealth = tonumber( ComponentGetValue( v, "hp" ) )
			currentMaxHealth = tonumber( ComponentGetValue( v, "max_hp") )
			
			local updatedHealth = currentHealth + amount
			local updatedMaxHealth = 0
			
			local updatedHealth = currentHealth + amount
			if(updatedHealth > currentMaxHealth and overheal_flag) then -- overhealing, if new health goes beyond max health, max health is updated
				updatedMaxHealth = updatedHealth
				ComponentSetValue( v, "max_hp", updatedMaxHealth)
				ComponentSetValue( v, "hp", updatedHealth)
			else
				if(updatedHealth <= 0) then
					updatedHealth = 0.04
				end
				ComponentSetValue( v, "hp", updatedHealth) -- sets health to 1 when updatedHealth is zero or below
			end
			
			break
		end
	end
end

-- function to add to player health
-- amount is how much is player health to be added to, may be positive or negative
-- amount must be provided as an integer
-- overheal_flag tells the function if amount can update maximum health when amount added goes beyond it
function addPlayerHealth(amount, overheal_flag)

	local damagemodels = EntityGetComponent( getPlayerEntity(), "DamageModelComponent" )
	
	if( damagemodels ~= nil ) then
		for i,v in ipairs(damagemodels) do
		
			currentHealth = tonumber( ComponentGetValue( v, "hp" ) )
			currentMaxHealth = tonumber( ComponentGetValue( v, "max_hp") )
			
			local trueAmount = amount * 0.04 -- how health scaling works when shown by UI
			local updatedHealth = currentHealth + trueAmount
			local updatedMaxHealth = 0
			
			local updatedHealth = currentHealth + trueAmount
			if(updatedHealth > currentMaxHealth and overheal_flag) then -- overhealing, if new health goes beyond max health, max health is updated
				updatedMaxHealth = updatedHealth
				ComponentSetValue( v, "max_hp", updatedMaxHealth)
				ComponentSetValue( v, "hp", updatedHealth)
			else
				if(updatedHealth <= 0) then
					updatedHealth = 0.04
				end
				ComponentSetValue( v, "hp", updatedHealth) -- sets health to 1 when updatedHealth is zero or below
			end
			
			break
		end
	end
end

-- function to directly set a value to player health
-- newHealth is the new health the player is going to have
-- newHealth must be provided as a float value
function setPlayerHealth(newHealth)
	local damagemodels = EntityGetComponent( getPlayerEntity(), "DamageModelComponent" )
	
	if( damagemodels ~= nil ) then
		for i,v in ipairs(damagemodels) do
			ComponentSetValue( v, "hp", newHealth)
		end
	end
end

-- function to add or subtract gold from player
-- amount can be either negative or positive
function addPlayerGold(amount)

	local player = getPlayerEntity()

	local wallet = EntityGetFirstComponent(player, "WalletComponent")
	local money = ComponentGetValueInt(wallet, "money")
	local player_money = money + amount

	edit_component(player, "WalletComponent", function(comp,vars)
		vars.money = player_money
	end)
end

-- function to get player gold
function getPlayerGold()
	
	local player = getPlayerEntity()
	
	local wallet = EntityGetFirstComponent(player, "WalletComponent")
	local money = ComponentGetValueInt(wallet, "money")

	return money
end

-- helper functions that returns the amount of internal variables currently on entity with entity_id
function getEntityInternalVariableAmount(entity_id)
	local internal_variables = EntityGetComponentIncludingDisabled( entity_id, "VariableStorageComponent" )
	if(internal_variables ~= nil) then
		return #internal_variables
	else
		return 0
	end
end

-- function to obtain a value from an internal variable contained on a variable storage component
-- entity_id is the id of the entity whose internal variables will be accessed
-- variable_name is the name of the internal variable to be accessed
-- variable_type is type indicator of the internal variable, which can be value_string, value_int or value_float
function getInternalVariableValue(entity_id, variable_name, variable_type)
	local value = nil
	local components = EntityGetComponentIncludingDisabled( entity_id, "VariableStorageComponent" )
	if ( components ~= nil ) then
		for key,comp_id in pairs(components) do 
			local var_name = ComponentGetValue2( comp_id, "name" )
			if(var_name == variable_name) then
				value = ComponentGetValue2(comp_id, variable_type)
			end
		end
	end
	return value
end

-- function to set a value of an internal variabled contained on a variable storage component of an entity with entity_id with name variable_name and type variable_type
-- entity_id is the id of the entity whose internal variables will be accessed
-- variable_name is the name of the internal variable to be accessed
-- variable_type is type indicator of the internal variable, which can be value_string, value_int or value_float
function setInternalVariableValue(entity_id, variable_name, variable_type, new_value)

	local components = EntityGetComponentIncludingDisabled( entity_id, "VariableStorageComponent" )	
	if ( components ~= nil ) then
		for key,comp_id in pairs(components) do 
			local var_name = ComponentGetValue2( comp_id, "name" )
			if( var_name == variable_name) then
				ComponentSetValue2( comp_id, variable_type, new_value )
			end
		end
	end
end

-- checks whether or not entity with entity_id contains an internal variable named variable_name
function entityHasInternalVariableValue(entity_id, variable_name)
	local flag = false
	local components = EntityGetComponentIncludingDisabled( entity_id, "VariableStorageComponent" )	
	if ( components ~= nil ) then
		for key,comp_id in pairs(components) do 
			local var_name = ComponentGetValue2( comp_id, "name" )
			if( var_name == variable_name) then
				flag = true
			end
		end
	end
	return flag
end

-- function to add new internal variables to an entity
-- entity_id is the id of the entity that will receive a new internal variable
-- variable_type is the type of variable being added, can be value_int, value_string or value_float
-- initial_value is the starting value of the variable being added, if none, must be provided as, e.g., 0, "", or 0.0
function addNewInternalVariable(entity_id, variable_name, variable_type, initial_value)
	if(variable_type == "value_int") then
		EntityAddComponent2(entity_id, "VariableStorageComponent", {
			name=variable_name,
			value_int=initial_value
		})
	elseif(variable_type == "value_string") then
		EntityAddComponent2(entity_id, "VariableStorageComponent", {
			name=variable_name,
			value_string=initial_value
		})
	elseif(variable_type == "value_float") then
		EntityAddComponent2(entity_id, "VariableStorageComponent", {
			name=variable_name,
			value_float=initial_value
		})
	elseif(variable_type == "value_bool") then
		EntityAddComponent2(entity_id, "VariableStorageComponent", {
			name=variable_name,
			value_bool=initial_value
		})
	end
end

-- function to add a new simplified internal variable to an entity
-- this internal variable approach does not need a type as it is used just as a special flag
function addNewSimplifiedInternalVariable(entity_id, variable_value)
	local ec_id = EntityAddComponent2(entity_id, "VariableStorageComponent", {
		name=variable_value
	})
end

-- function to check whether or not entity with entity_id contains a simplified internal variable with the value variable_value
function entityHasSimplifiedInternalVariable(entity_id, variable_value)
	local flag = false
	for i, component in ipairs(EntityGetComponent(entity_id, "VariableStorageComponent") or {}) do
		if ComponentGetValue2(component, "name") == variable_value then
			flag = true
		end
	end
	return flag
end

-- function to remove simplified internal variable with variable_value from entity with entity_id
function removeInternalVariable(entity_id, variable_name)
	local flag = false
	local components = EntityGetComponentIncludingDisabled( entity_id, "VariableStorageComponent" )	
	if ( components ~= nil ) then
		for key,comp_id in pairs(components) do 
			local var_name = ComponentGetValue2( comp_id, "name" )
			if( var_name == variable_name) then
				EntityRemoveComponent(entity_id, comp_id)
			end
		end
	end
	return flag
end

-- function to remove simplified internal variable with variable_value from entity with entity_id
function removeSimplifiedInternalVariable(entity_id, variable_value)
	for i, component in ipairs(EntityGetComponent(entity_id, "VariableStorageComponent") or {}) do
		if ComponentGetValue2(component, "name") == variable_value then
			EntityRemoveComponent(entity_id, component)
		end
	end
end

function removeFirstSimplifiedInternalVariable(entity_id, variable_value)
	for i, component in ipairs(EntityGetComponent(entity_id, "VariableStorageComponent") or {}) do
		if ComponentGetValue2(component, "name") == variable_value then
			EntityRemoveComponent(entity_id, component)
			break
		end
	end
end

function countSimplifiedInternalVariable(entity_id, variable_value)
	local count = 0
	for i, component in ipairs(EntityGetComponent(entity_id, "VariableStorageComponent") or {}) do
		if ComponentGetValue2(component, "name") == variable_value then
			count = count + 1
		end
	end
	return count
end

-- ================
-- UNUSED FUNCTIONS
-- ================

-- function to return whether or not projectile with projectile id is of the ACTION_TYPE_PROJECTILE action type
function projectileIsProjectileAction(projectile_id)
	local action_id = getActionIdFromProjectile(projectile_id)
	
	if(action_id == nil) then -- if projectile has no projectile definition variable, returns false automatically
		return false
	end
	
	local action = getAction(action_id)
	local action_type = action.type
	
	if(action_type == ACTION_TYPE_PROJECTILE) then
		return true
	else
		return false
	end
end

-- this function builds relics based on a static base file with all basic components already pre-configured, then adds 
-- all remaining settings to it once an entity id is available
-- it doesn't work because PhysicsImageShapeComponent seems be unable to understand when its image_file is changed after the entity is loaded
-- updating only when the object is reloaded like when dropped from the inventory
function buildRelic_static(relic_name, relic_ui_name, relic_desc, x, y)
	
	local relic_id = EntityLoad("mods/ArcaneReliquary/files/entity_definitions/relics/base_relic.xml", x, y)
	EntitySetName(relic_id, relic_name)
	
	-- adding Lua callbacks
	EntityAddComponent2(relic_id, "LuaComponent", {
		script_throw_item="mods/ArcaneReliquary/files/scripts/main_listeners/relic_drop.lua"
	})
	EntityAddComponent2(relic_id, "LuaComponent", {
		script_item_picked_up="mods/ArcaneReliquary/files/scripts/main_listeners/relic_pickup.lua"
	})
	
	-- updating components with relic info
	local simplified_ui_name = relic_ui_name
	local descriptive_ui_name = relic_ui_name .. " - " .. relic_desc
	local item_gfx_path = "mods/ArcaneReliquary/files/items_gfx/" .. relic_name .. ".png"
	local item_ui_gfx_path = "mods/ArcaneReliquary/files/item_ui_gfx/" .. relic_name .. ".png"

	local ic_id = EntityGetFirstComponentIncludingDisabled(relic_id, "ItemComponent")
	ComponentSetValue2(ic_id, "item_name", simplified_ui_name)
	ComponentSetValue2(ic_id, "ui_description", descriptive_ui_name)
	ComponentSetValue2(ic_id, "ui_sprite", item_ui_gfx_path)

	local uiic_id = EntityGetFirstComponentIncludingDisabled(relic_id, "UIInfoComponent")
	ComponentSetValue2(uiic_id, "name", descriptive_ui_name)
	
	local sc_id = EntityGetFirstComponentIncludingDisabled(relic_id, "SpriteComponent")
	ComponentSetValue2(sc_id, "image_file", item_gfx_path)
	
	local pisc_id = EntityGetFirstComponentIncludingDisabled(relic_id, "PhysicsImageShapeComponent")
	ComponentSetValue2(pisc_id, "image_file", item_gfx_path)
	
	return relic_id
end


-- this function builds a relic entity from scratch by adding and configuring all required components dynamically
-- unfortunately, it doesn't create the entity correctly like in the static file version, but it will be kept here as future reference
function buildRelic_dynamic(relic_name, relic_ui_name, relic_desc)

	local descriptive_ui_name = relic_ui_name .. " - " .. relic_desc
	
	-- creating it with required name and adding tags
	local relic_id = EntityCreateNew(relic_name)
	EntityAddTag(relic_id, "item_physics")
	EntityAddTag(relic_id, "relic")
	
	-- physics behavior
	local material_id = CellFactory_GetType("relic_material")
	--GamePrint(CellFactory_GetName(material_id))
	EntityAddComponent2(relic_id, "PhysicsImageShapeComponent", {
		body_id=1,
		centered=true,
		image_file="mods/ArcaneReliquary/files/items_gfx/" .. relic_name .. ".png",
		material=material_id,
		is_root=true,
	})
	
	EntityAddComponent2(relic_id, "PhysicsThrowableComponent", {
		max_throw_speed=210,
		throw_force_coeff=2
	})
	
	local pbc_id = EntityAddComponent2(relic_id, "PhysicsBody2Component", {
		allow_sleep=true,
		angular_damping=0, 
		fixed_rotation=false, 
		is_bullet=true, 
		linear_damping=0,
		auto_clean=false,
		destroy_body_if_entity_destroyed=false,
		kill_entity_after_initialized=false
	})
	ComponentAddTag(pbc_id, "enabled_in_world")
	
	-- Lua callbacks
	EntityAddComponent2(relic_id, "LuaComponent", {
		script_throw_item="mods/ArcaneReliquary/files/scripts/main_listeners/relic_drop.lua"
	})
	EntityAddComponent2(relic_id, "LuaComponent", {
		script_item_picked_up="mods/ArcaneReliquary/files/scripts/main_listeners/relic_pickup.lua"
	})

	-- projectile behavior
	local pc_id = EntityAddComponent2(relic_id, "ProjectileComponent", {
		lifetime=-1,
		penetrate_entities=true,
		on_death_gfx_leave_sprite=false, 
		on_lifetime_out_explode=false
	})
	ComponentAddTag(pc_id, "enabled_in_world")
	
	local vc_id = EntityAddComponent2(relic_id, "VelocityComponent")
	ComponentAddTag(vc_id, "enabled_in_world")
	
	-- damage model
	local dmc_id = EntityAddComponent2(relic_id, "DamageModelComponent", {
		air_needed=false, 
		blood_material="", 
		drop_items_on_death=false, 
		falling_damages=true, 
		fire_damage_amount=0.2,
		falling_damage_height_min=0.1,	
		fire_probability_of_ignition=0, 
		critical_damage_resistance=1,
		hp=0.5, 
		is_on_fire=false, 
		materials_create_messages=false, 
		materials_damage=true,
		materials_that_damage="acid,lava",
		materials_how_much_damage="0.001,0.001",
		ragdoll_filenames_file="",
		ragdoll_material="",
	})
	ComponentAddTag(dmc_id, "enabled_in_world")
	
	-- collectible item settings
	local sc_id = EntityAddComponent2(relic_id, "SpriteComponent", {
		offset_x=4,
		offset_y=4,
		image_file="mods/ArcaneReliquary/files/items_gfx/" .. relic_name .. ".png"
	})
	ComponentAddTag(sc_id, "enabled_in_hand")
	EntitySetComponentIsEnabled( relic_id, sc_id, false )
	
	local ic_id = EntityAddComponent2(relic_id, "ItemComponent", {
		item_name=relic_ui_name,
		max_child_items=0,
		is_pickable=true,
		is_equipable_forced=true,
		ui_sprite="mods/ArcaneReliquary/files/item_ui_gfx/" .. relic_name .. ".png",
		ui_description=descriptive_ui_name,
		preferred_inventory="FULL"
	})
	ComponentAddTag(ic_id, "enabled_in_world")
	
	local uiic_id = EntityAddComponent2(relic_id, "UIInfoComponent", {
		name=descriptive_ui_name
	})
	ComponentAddTag(uiic_id, "enabled_in_world")
	
	-- particle emitters
	local spec_id = EntityAddComponent2(relic_id, "SpriteParticleEmitterComponent", {
		sprite_file="data/particles/shine_08.xml",
		lifetime=0.2,
		--randomize_lifetime.min="0.1"
		--randomize_lifetime.max="0.8"
		emission_interval_min_frames=50,
		emission_interval_max_frames=250,
		count_min=1,
		count_max=1,
		additive=true,
		emissive=false,
		--scale.x="1.0"
		--scale.y="1.0"
		sprite_random_rotation=true,
		--randomize_scale.min_x="-0.1" 
		--randomize_scale.max_x="0.1" 
		--randomize_scale.min_y="-0.1" 
		--randomize_scale.max_y="0.1" 
		--randomize_position.min_y="-3" 
		--randomize_position.max_y="3"
		--randomize_position.min_x="-3"  
		--randomize_position.max_x="3"
		velocity_slowdown=6,
		--randomize_animation_speed_coeff.min="0.667"  
		--randomize_animation_speed_coeff.max="1.0" 
	})
	ComponentSetValue2(spec_id, "randomize_lifetime", 0.1, 0.8)
	ComponentSetValue2(spec_id, "scale", 1.0, 1.0)
	ComponentSetValue2(spec_id, "randomize_animation_speed_coeff", 0.667, 1.0)
	ComponentSetValue2(spec_id, "randomize_scale", -0.1, -0.1, 0.1, 0.1)
	ComponentSetValue2(spec_id, "randomize_position", -3, -3, 3, 3)
	
	return relic_id
end

-- calculates a random chance between base_value and max_value, based on supplied relic_level and upgrade_factor
function getRandomChance(base_value, max_value, upgrade_factor, relic_level)
	local chance_pass = false
	SetRandomSeed( GameGetFrameNum(), math.random(1,50))
	local odds = Random(base_value, max_value)
	local level_based_factor = base_value + (upgrade_factor * relic_level)
	if(odds >= base_value and odds <= level_based_factor) then
		chance_pass = true
	end
	return chance_pass
end

function addCreatureSpriteParticleEffect(creature_id, sprite_trail_path, tag)
	local spec_id = EntityAddComponent2(creature_id, "SpriteParticleEmitterComponent", {
		sprite_file=sprite_trail_path,
		delay=0,
		lifetime=0,
		velocity_slowdown=0,
		rotation=0,
		angular_velocity=0,
		use_velocity_as_rotation=false,
		emission_interval_min_frames=1,
		emission_interval_max_frames=3,
		count_min=1,
		count_max=1,
	})
	ComponentSetValue2( spec_id, "scale", 1, 1)
	ComponentSetValue2( spec_id, "scale_velocity", 0, 0)
	ComponentSetValue2( spec_id, "color", 1, 1, 1, 1)
	ComponentSetValue2( spec_id, "color_change", 0, 0, 0, 0)
	ComponentSetValue2( spec_id, "velocity", 0, 0)
	ComponentSetValue2( spec_id, "gravity", 0, 10)
	ComponentSetValue2( spec_id, "randomize_rotation", -3.1415, 3.1415)
	ComponentSetValue2( spec_id, "randomize_position", -12, 12, -12, 12)
	ComponentAddTag(spec_id, tag)
end

function removeCreatureSpriteParticleEffect(creature_id, tag)
	local spec_id = EntityGetFirstComponentIncludingDisabled(creature_id, "SpriteParticleEmitterComponent", tag)
	EntityRemoveComponent(creature_id, spec_id)
end