-- List of files that use settings registered in this file:
-- -> init.lua
-- -> scripts/props/proximity_mine_indicator.lua
-- -> scripts/traps_spawn.lua
-- -> scripts/thearmory_utilities.lua
-- -> scripts/items/enemydeath_pickupitem_drop

dofile("data/scripts/lib/mod_settings.lua") -- see this file for documentation on some of the features.

-- This file can't access other files from this or other mods in all circumstances.
-- Settings will be automatically saved.
-- Settings don't have access unsafe lua APIs.

-- Use ModSettingGet() in the game to query settings.
-- For some settings (for example those that affect world generation) you might want to retain the current value until a certain point, even
-- if the player has changed the setting while playing.
-- To make it easy to define settings like that, each setting has a "scope" (e.g. MOD_SETTING_SCOPE_NEW_GAME) that will define when the changes
-- will actually become visible via ModSettingGet(). In the case of MOD_SETTING_SCOPE_NEW_GAME the value at the start of the run will be visible
-- until the player starts a new game.
-- ModSettingSetNextValue() will set the buffered value, that will later become visible via ModSettingGet(), unless the setting scope is MOD_SETTING_SCOPE_RUNTIME.

local mod_id = "ArcaneReliquary" -- This should match the name of your mod's folder.
mod_settings_version = 1 -- This is a magic global that can be used to migrate settings to new mod versions. call mod_settings_get_version() before mod_settings_update() to get the old value. 
mod_settings = 
{
	{
		id = "_",
		ui_name = "Customize your experience in Arcane Reliquary with the config options below.\nAll settings require restarting the game to take effect.",
		not_setting = true,
	},
	{
		category_id = "enemies",
		ui_name = "Buffed Enemies",
		ui_description = "Use these settings to control aspects and behaviors of buffed enemies that carry relic chests.",
		settings = {
			{
				id = "enemy_buff_chance",
				ui_name = "Enemy Buff Frequency",
				ui_description = "How often you want enemies to become elligible \nto receive buffs and carry relic chests.",
				value_default = "5",
				values = {{"50","CLOSE TO NEVER - 2% chance"},{"20","VERY RARELY - 5% chance"},{"10","RARELY - 10% chance"}, {"5","REGULARLY - 20% chance"}, {"3","OFTEN - 30% chance"}, {"2", "VERY OFTEN - 50% chance"}},
				scope = MOD_SETTING_SCOPE_NEW_GAME,
			},
			{
				id = "max_buff_level",
				ui_name = "Maximum Buff Level",
				ui_description = "Controls what should be \nthe maximum buff level for\n creatures carrying relic chests.",
				value_default = "4",
				values = {{"1","Level 1 - few basic buffs, 50% default drop chance"}, {"2","Level 2 - more basic buffs, 50% default drop chance"}, {"3","Level 3 - basic and advanced buffs, 100% drop chance"}, {"4","Level 4 - basic and ultimate buffs, 100% drop chance"}},
				scope = MOD_SETTING_SCOPE_NEW_GAME,
			},
			{
				id = "no_buffs_mode",
				ui_name = "No Buffs Mode",
				ui_description = "By enabling this option, you turn on No Buffs Mode, \nwhere enemies no longer receive buffs, and become randomly \nelligible to drop a chest on death.",
				value_default = "DISABLED",
				values = {{"DISABLED","DISABLED"},{"ENABLED","ENABLED"}},
				scope = MOD_SETTING_SCOPE_NEW_GAME,
			},
		},
	},
	{
		category_id = "buff_icons",
		ui_name = "Buff Icons Config",
		ui_description = "Use these settings to control how you want buff icons to behave.",
		settings = {
			{
				id = "buff_icon_transparency",
				ui_name = "Buff Icon Transparency",
				ui_description = "Use this option to control the transparency \nlevel of buff icons.",
				value_default = "OPAQUE",
				values = {{"OPAQUE","Opaque icons"}, {"TRANSLUSCENT","Transluscent icons"}},
				scope = MOD_SETTING_SCOPE_NEW_GAME,
			},
			{
				id = "buff_icon_visibility",
				ui_name = "Buff Icon Visibility",
				ui_description = "Use this option to control \nthe visibility of buff icons \n(if set to invisible, overrides \nwhatever is chosen above).",
				value_default = "VISIBLE_FOG",
				values = {{"VISIBLE_FOG","Visible over fog - icons appear over fog in unrevealed areas"}, {"INVISIBLE_FOG","Visible under fog - icons are only visible where there's no fog"}, {"INVISIBLE_ALL","Invisible icons - no icons at all"}},
				scope = MOD_SETTING_SCOPE_NEW_GAME,
			},
		},
	},
	{
		category_id = "item_drops",
		ui_name = "Item Drops",
		ui_description = "Use these settings to control how enemies drop items.",
		settings = {
			{
				id = "special_item_drop_chance",
				ui_name = "Special Item Drop Frequency",
				ui_description = "How often should enemies \nbecome elligible to drop \nspecial items such as room spawners\n and upgrade/forge material containers.",
				value_default = "5",
				values = {{"50","CLOSE TO NEVER - 2% chance"},{"20","VERY RARELY - 5% chance"},{"10","RARELY - 10% chance"}, {"5","REGULARLY - 20% chance"}, {"3","OFTEN - 30% chance"}, {"2", "VERY OFTEN - 50% chance"}},
				scope = MOD_SETTING_SCOPE_NEW_GAME,
			},
		},
	},
	{
		category_id = "other_configs",
		ui_name = "Other Configs",
		ui_description = "General configurations.",
		settings = {
			{
				id = "relics_cycle_item_switch_controls",
				ui_name = "Cycle through owned relics using item switch controls",
				ui_description = "Enable this option to be \nable to cycle through your relics \nusing the controls item switch \ncontrols (mouse wheel on PC).",
				value_default = "ENABLED",
				values = {{"DISABLED","DISABLED"},{"ENABLED","ENABLED"}},
				scope = MOD_SETTING_SCOPE_NEW_GAME,
			},
		},
	},
}

-- This function is called to ensure the correct setting values are visible to the game via ModSettingGet(). your mod's settings don't work if you don't have a function like this defined in settings.lua.
-- This function is called:
--		- when entering the mod settings menu (init_scope will be MOD_SETTINGS_SCOPE_ONLY_SET_DEFAULT)
-- 		- before mod initialization when starting a new game (init_scope will be MOD_SETTING_SCOPE_NEW_GAME)
--		- when entering the game after a restart (init_scope will be MOD_SETTING_SCOPE_RESTART)
--		- at the end of an update when mod settings have been changed via ModSettingsSetNextValue() and the game is unpaused (init_scope will be MOD_SETTINGS_SCOPE_RUNTIME)
function ModSettingsUpdate( init_scope )
	local old_version = mod_settings_get_version( mod_id ) -- This can be used to migrate some settings between mod versions.
	mod_settings_update( mod_id, mod_settings, init_scope )
end

-- This function should return the number of visible setting UI elements.
-- Your mod's settings wont be visible in the mod settings menu if this function isn't defined correctly.
-- If your mod changes the displayed settings dynamically, you might need to implement custom logic.
-- The value will be used to determine whether or not to display various UI elements that link to mod settings.
-- At the moment it is fine to simply return 0 or 1 in a custom implementation, but we don't guarantee that will be the case in the future.
-- This function is called every frame when in the settings menu.
function ModSettingsGuiCount()
	return mod_settings_gui_count( mod_id, mod_settings )
end

-- This function is called to display the settings UI for this mod. Your mod's settings wont be visible in the mod settings menu if this function isn't defined correctly.
function ModSettingsGui( gui, in_main_menu )
	mod_settings_gui( mod_id, mod_settings, gui, in_main_menu )

	--example usage:
	--[[
	GuiLayoutBeginLayer( gui )

	GuiBeginAutoBox( gui )

	GuiZSet( gui, 10 )
	GuiZSetForNextWidget( gui, 11 )
	GuiText( gui, 50, 50, "Gui*AutoBox*")
	GuiImage( gui, im_id, 50, 60, "data/ui_gfx/game_over_menu/game_over.png", 1, 1, 0 )
	GuiZSetForNextWidget( gui, 13 )
	GuiImage( gui, im_id, 60, 150, "data/ui_gfx/game_over_menu/game_over.png", 1, 1, 0 )

	GuiZSetForNextWidget( gui, 12 )
	GuiEndAutoBoxNinePiece( gui )

	GuiZSetForNextWidget( gui, 11 )
	GuiImageNinePiece( gui, 12368912341, 10, 10, 80, 20 )
	GuiText( gui, 15, 15, "GuiImageNinePiece")

	GuiBeginScrollContainer( gui, 1233451, 500, 100, 100, 100 )
	GuiLayoutBeginVertical( gui, 0, 0 )
	GuiText( gui, 10, 0, "GuiScrollContainer")
	GuiImage( gui, im_id, 10, 0, "data/ui_gfx/game_over_menu/game_over.png", 1, 1, 0 )
	GuiImage( gui, im_id, 10, 0, "data/ui_gfx/game_over_menu/game_over.png", 1, 1, 0 )
	GuiImage( gui, im_id, 10, 0, "data/ui_gfx/game_over_menu/game_over.png", 1, 1, 0 )
	GuiImage( gui, im_id, 10, 0, "data/ui_gfx/game_over_menu/game_over.png", 1, 1, 0 )
	GuiLayoutEnd( gui )
	GuiEndScrollContainer( gui )

	local c,rc,hov,x,y,w,h = GuiGetPreviousWidgetInfo( gui )
	print( tostring(c) .. " " .. tostring(rc) .." " .. tostring(hov) .." " .. tostring(x) .." " .. tostring(y) .." " .. tostring(w) .." ".. tostring(h) )

	GuiLayoutEndLayer( gui )]]--
end
