-- main variables related to relics
maximum_owned_relics = 20
relic_buy_price = 500
relic_sale_price = 100
relic_sell_price = 100
relic_material_base_upgrade_cost = 150
relic_material_advanced_upgrade_cost = relic_material_base_upgrade_cost * 2
relic_max_upgrade_level = 10
relic_upgrade_material_name = "relic_upgrade_compound"
relic_advanced_upgrade_material_name = "relic_upgrade_compound_advanced"
relic_construction_main_material_name = "relic_metal"
relic_material_base_construct_cost = 100
relic_precursor_material_construct_cost = 200

-- useful strings
-- --------------
-- projectile damage types
--[[
electricity    0
radioactive    0
slice          0
curse          0
overeating     0
projectile     0
healing        0
physics_hit    0
explosion      0
poison         0
melee          0
drill          0
fire           0
ice            0
]]
-- damage message indicators
--[[
$damage_projectile
$damage_electricity
$damage_explosion
$damage_fire
$damage_melee
$damage_drill
$damage_slice
$damage_ice
$damage_healing
$damage_physicshit
$damage_radioactive
$damage_poison
$damage_water
$damage_fall
$damage_drowning
$damage_frommaterial
$damage_kick
]]
-- enum damage values
--[[
DAMAGE_MELEE
DAMAGE_PROJECTILE
DAMAGE_EXPLOSION
DAMAGE_BITE
DAMAGE_FIRE
DAMAGE_MATERIAL
DAMAGE_FALL
DAMAGE_ELECTRICITY
DAMAGE_DROWNING
DAMAGE_PHYSICS_BODY_DAMAGED
DAMAGE_DRILL
DAMAGE_SLICE
DAMAGE_ICE
DAMAGE_HEALING
DAMAGE_PHYSICS_HIT
DAMAGE_RADIOACTIVE
DAMAGE_POISON
DAMAGE_MATERIAL_WITH_FLASH
DAMAGE_OVEREATING
]]

-- enum ragdoll effects
--[[
NONE
NORMAL
BLOOD_EXPLOSION
BLOOD_SPRAY
FROZEN
CONVERT_TO_MATERIAL
CUSTOM_RAGDOLL_ENTITY
DISINTEGRATED
NO_RAGDOLL_FILE
PLAYER_RAGDOLL_CAMERA
]]

-- list containing all registered relics, these are the properties used:
-- name 					- must be the same as its entity definition internal name present at /files/entity_definitions/relics/
-- ui_name					- the name of the relic as it should appear on screen
-- categories				- a comma separated list of categories the relic pertains to
-- effect_desc				- a description of all effects the relics applies, it will be shown when a relic is collected and also when hovering on it in the extended inventory
-- upgrade_effect_desc		- a description of what changes when the relic is upgraded
-- is_upgradable			- controls whether or not the relic can be upgraded in a relic forge
-- requires_relic			- if the relic requires owning other relic in order to function, it must be put here
-- excludes_relics			- [DEPRECATED, REFER TO NEXT TWO PROPERTIES BELOW] a list containing the names of relics that can't be owned in order to own a specific relic
-- excludes_relics_desc		- a description of relics that can not be owned at the same time along with the current relic
-- excludes_relics_tags		- a tag shared among all relics of the same type that must be mutually exclusive
-- combine_exclusive		- if set to true, indicates the relic can only be obtained by combining other relics
-- combine_requires_relics	- a list containing the names of all relics required to 
-- precursor_types			- refers to the construction precursor types of the relic, which may be one of the following:
--								- boot
--								- gravity_stone
--								- necklace
--								- death_ray
--								- effect_stone
--								- effect_crystal
--								- companion
--								- elemental_gem
--								- flask
--								- ring
--								- gadget
--								- magic_artifact
--								- device
--								- energy_shield
-- trigger functions : optional code that runs contextually according to the type of listener specified, all can make use of functions present at /files/scripts/utilities/ArcaneReliquary_utilities.lua
-- ondamaged_function		- a function running whenever the player takes damage, must have the same parameters as the related callback, i.e. function( damage, message, entity_thats_responsible, is_fatal, projectile_thats_responsible ).
-- onenemydamaged_function	- a function running whenever an enemy takes damage from the player, must have the same parameters as the related callback, i.e. function( damage, message, entity_thats_responsible, is_fatal, projectile_thats_responsible ).
-- overtime_function		- a function running within the regular overtime callback, no parameters are required.
-- overtimeslow_function	- a function running within the slow overtime callback, no parameters are required.
-- onspellcast_function		- a function running whenever the player casts a spell, must have the same parameters as the related callback, i.e. function( projectile_entity_id ).
-- onenemykill_function 	- a function running when an enemy is killed by the player, must have teh same parameters as the related callback, i.e. function( damage_type_bit_field, damage_message, entity_thats_responsible, drop_items )
-- ontrickkill_function		- a function running when an enemy is killed by any source other than the player, must have teh same parameters as the related callback, i.e. function( damage_type_bit_field, damage_message, entity_thats_responsible, drop_items ) 
-- onwandfired_function 	- a function running whenever a wand is fired, must have the same parameters as the related callback, i.e. function(gun_entity_id)
-- ongoldpickup_function	- a function running when the player collects gold nuggets, no parameters are required.
-- onenemykicked_function	- a function running whenever an enemy is kicked by the player, must have the same parameters as the related callback, i.e. function(entity_who_kicked)
-- onpickup_function 		- a function running when a relic is collected, no parameters are required.
-- ondrop_function 			- a function running when a relic is dropped, no parameters are required.
-- onenemystomped_function 	- a function running when the stomp ability added by some relics is triggered, takes two parameters: entity_id, the entity stomping, and creature_id, the entity being stomped.
-- onextendedkick_function	- a function running whenever the extended kick projectile added by the Sniper Scope Relic is shot, used only for sinergies with this specified relic. Takes two parameters: entity_id, the entity launching the extended kick, and extended_kick_id, the extended kick itself.
-- ongroundpound_function	- a function running whenever the ground pound explosion added by the Earth Core Fragment happens, used only for sinergies with this specified relic. Takes two parameters: entity_id, the entity launching the extended kick, and extended_kick_id, the extended kick itself.

relics_list = {
	{
		name 					= "titanium_rose",
		ui_name					= "Titanium Rose",
		categories				= "health",
		effect_desc				= "When you take damage from any source other than harmful \nmaterials and fire, there is a 25% chance lost health will be replenished.",
		effect_desc_short		= "Replenishes your health when damaged.",
        precursor_types			= "magic_artifact",
		upgrade_effect_desc		= "For each upgrade level, hp replenish chance is increased by 5%.",
		is_upgradable 			= true,
		ondamaged_function		= function(damage, message, entity_thats_responsible, is_fatal, projectile_thats_responsible )
			if(isRelicOwned("titanium_rose") >= 1) then	
				if((not string.find(message, "damage from material")) and (not string.find(message, "fire"))) then
					SetRandomSeed( GameGetFrameNum(), math.random(1,50))
					local chance = Random(1,100)
					local odds = 25 + (5 * getRelicLevel("titanium_rose"))
					if(chance >= 1 and chance <= odds) then
						local x, y = EntityGetTransform(getPlayerEntity())
						GamePlaySound( "data/audio/Desktop/event_cues.snd", "event_cues/heart/create", x, y )
						addPlayerHealth(damage * 25, false)
					end
				end
			end
		end,
		excludes_relics_desc	= "It can't be used along with other hp replenish on damage related relics",
		excludes_relics_tags	= "hp_replenish_on_damage",
	},
	{
		name 					= "blood_gem",
		ui_name					= "Blood Gem",
		categories				= "health",
		effect_desc				= "Heals you overtime for up to 25% of your current maximum health.",
		effect_desc_short		= "Heals a small fraction of your hp.",
        precursor_types			= "elemental_gem",
		upgrade_effect_desc		= "For each upgrade level, additional hp is added on each heal tick.",
		is_upgradable 			= true,
		overtimeslow_function	= function()
			if(isRelicOwned("blood_gem") >= 1) then
				if(((getPlayerMaxHealth() / 4) > getPlayerHealth()) and getPlayerHealth() > 0.04) then -- second check is a workaround for when Saving Grace is active
					local x, y = EntityGetTransform(getPlayerEntity())
					GamePlaySound( "data/audio/Desktop/event_cues.snd", "event_cues/heart/create", x, y )
					addPlayerHealth(2 + (2 * getRelicLevel("blood_gem")), false)
				end
			end
		end,
	},
	{
		name 					= "barbed_ring",
		ui_name					= "Barbed Ring",
		categories				= "health,projectiles",
		effect_desc				= "Spells you cast get a significant damage boost, but you take 25% more damage from enemy projectiles.",
		effect_desc_short		= "Your spells get a damage boost, but you take more damage from enemy projectiles.",
        precursor_types			= "ring",
		upgrade_effect_desc		= "For each upgrade level, damage boost is slightly increased.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("barbed_ring") >= 1) then
				multiplyProjectileBaseDamage(projectile_entity_id, 3 + (2 * getRelicLevel("barbed_ring")))
			end
		end,
		ondamaged_function		= function(damage, message, entity_thats_responsible, is_fatal, projectile_thats_responsible )
			if(isRelicOwned("barbed_ring") >= 1) then
				if(message == "$damage_projectile") then
					GamePrint("Relic penalty: 25% more damage taken")
					local damage_penalty = (damage * 25 / 100)
					EntityInflictDamage( getPlayerEntity(), damage_penalty, "DAMAGE_PROJECTILE", "Barbed Ring damage penalty", "BLOOD_SPRAY", 0, 0)
				end
			end
		end,
	},
	{
		name 					= "corrupted_medalion",
		ui_name					= "Corrupted Medalion",
		categories				= "health,projectiles",
		effect_desc				= "Spells you cast have their damage reduced, but each time you kill an enemy you gain a small amount of hp.",
		effect_desc_short		= "Your spells deal reduced damage, but kills heal you.",
        precursor_types			= "magic_artifact",
		upgrade_effect_desc		= "For each upgrade level, gained hp from kills is increased.",
		is_upgradable 			= true,
		onenemykill_function	= function(damage_type_bit_field, damage_message, entity_thats_responsible, drop_items)			
			if(isRelicOwned("corrupted_medalion") >= 1) then
				addPlayerHealth(2 + (2 * getRelicLevel("corrupted_medalion")), false)
			end
		end,
		onspellcast_function		= function(projectile_entity_id)
			if(isRelicOwned("corrupted_medalion") >= 1) then
				fractionProjectileDamage(projectile_entity_id, 3)
			end
		end,
	},
	{
		name 					= "water_gem",
		ui_name					= "Water Gem",
		categories				= "materials,jetpack",
		effect_desc				= "Your jetpack now emits water.",
		effect_desc_short		= "Your jetpack now emits water.",
        precursor_types			= "elemental_gem",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			setJetpackMaterial("water")
		end,
		ondrop_function			= function()
			if(getJetpackMaterial() ~= getDefaultJetpackMaterial()) then
				revertDefaultJetpackMaterial()
			end
		end,
	},
	{
		name 					= "mystic_lens",
		ui_name					= "Mystic Lens",
		categories				= "projectiles",
		effect_desc				= "Spells you cast get a small damage bonus for each empty slot on your currently equipped wand.",
		effect_desc_short		= "Every empty slot in your wand means extra damage.",
        precursor_types			= "magic_artifact",
		upgrade_effect_desc		= "For each upgrade level, damage bonus is increased.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("mystic_lens") >= 1) then
				local current_wand_id = getCurrentlyEquippedWandId()
				if(current_wand_id ~= -1) then
					local free_slots, spells = getWandEmptyandOccupiedSlots(getCurrentlyEquippedWandId())
					if(free_slots > 0) then
						multiplyProjectileBaseDamage(projectile_entity_id, (1.5 * free_slots) + (0.4 * getRelicLevel("mystic_lens")))
						if(free_slots >= 1 and spells == 1) then
							multiplyProjectileBaseDamage(projectile_entity_id, 2.0 + (0.4 * getRelicLevel("mystic_lens")))
						end
					end
				end
			end
		end,
	},
	{
		name 					= "lightspeed_coin",
		ui_name					= "Lightspeed Coin",
		categories				= "projectiles",
		effect_desc				= "The more gold you have, the faster your spells travel when cast.",
		effect_desc_short		= "More gold = faster spells.",
        precursor_types			= "magic_artifact",
		upgrade_effect_desc		= "For each upgrade level, velocity gained is increased.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("lightspeed_coin") >= 1) then
				if(getPlayerGold() ~= 0) then
					local multiplier = math.floor(getPlayerGold() / 100)
					if(multiplier ~= 0) then
						multiplier = multiplier + 1 + (1 * getRelicLevel("lightspeed_coin"))
						GamePrint("Relic effect: " .. multiplier .. "x velocity multiplier")
						multiplyProjectileVelocity(projectile_entity_id, multiplier)
					end
				end
			end
		end,
	},
	{
		name 					= "charged_necklace",
		ui_name					= "Charged Necklace",
		categories				= "projectiles",
		effect_desc				= "Spells you cast have a 30% chance of electrocuting enemies they \ntouch, but you take 25% more damage from electricity sources or attacks.",
		effect_desc_short		= "Electrically charges your spells.",
        precursor_types			= "necklace",
		upgrade_effect_desc		= "For each upgrade level, effect chance is increased by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("charged_necklace") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 30 + (5 * getRelicLevel("charged_necklace"))
				if(chance >= 1 and chance <= odds) then
					attachLoadEffectOntoEntitiestoProjectile(projectile_entity_id, "data/entities/misc/effect_electricity.xml", "plasma_fading")
				end
			end
		end,
		ondamaged_function		= function(damage, message, entity_thats_responsible, is_fatal, projectile_thats_responsible)
			if(isRelicOwned("charged_necklace") >= 1) then
				if(message == "$damage_electricity") then
					GamePrint("Relic penalty: 25% more electricity damage taken")
					local damage_penalty = (damage * 25 / 100)
					EntityInflictDamage( getPlayerEntity(), damage_penalty, "DAMAGE_ELECTRICITY", "Thunder Necklace damage penalty", "BLOOD_SPRAY", 0, 0)
				end
			end
		end,
	},
	{
		name 					= "ice_artifact",
		ui_name					= "Ice Artifact",
		categories				= "projectiles",
		effect_desc				= "Spells you cast have a 30% chance of freezing enemies they touch, but you take double fire damage.",
		effect_desc_short		= "Freeze charges your spells.",
        precursor_types			= "magic_artifact",
		upgrade_effect_desc		= "For each upgrade level, effect chance is increased by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("ice_artifact") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 30 + (5 * getRelicLevel("ice_artifact"))
				if(chance >= 1 and chance <= odds) then
					attachLoadEffectOntoEntitiestoProjectile(projectile_entity_id, "data/entities/misc/effect_frozen.xml", "snow")
				end
			end
		end,
		onpickup_function		= function()
			local fire_multiplier = getBasicDamageMultiplier(getPlayerEntity(), "fire")
			setBasicDamageMultiplier(getPlayerEntity(), "fire", fire_multiplier * 2)
		end,
		ondrop_function			= function()
			local fire_multiplier = getBasicDamageMultiplier(getPlayerEntity(), "fire")
			setBasicDamageMultiplier(getPlayerEntity(), "fire", fire_multiplier / 2)
		end
	},
	{
		name 					= "acid_gem",
		ui_name					= "Acid Gem",
		categories				= "projectiles,materials",
		effect_desc				= "Spells you cast that explode on impact have a 30% chance of releasing acid on impact, but you \ntake 25% more damage from this material.",
		effect_desc_short		= "Your spells release acid.",
        precursor_types			= "elemental_gem",
		upgrade_effect_desc		= "For each upgrade level, effect chance is increased by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("acid_gem") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 30 + (5 * getRelicLevel("acid_gem"))
				if(chance >= 1 and chance <= odds) then
					attachLoadEntityOnExplosiontoProjectile(projectile_entity_id, getLoadOnExplosion("launch_elemental_particles_acid").path, "spark_green")
				end
			end
		end,
		ondamaged_function		= function(damage, message, entity_thats_responsible, is_fatal, projectile_thats_responsible )
			if(isRelicOwned("acid_gem") >= 1) then
				if(string.find(message, "damage from material") and string.find(message, "acid")) then
					GamePrint("Relic penalty: 25% more acid damage taken")
					local damage_penalty = (damage * 25 / 100)
					EntityInflictDamage( getPlayerEntity(), damage_penalty, "DAMAGE_MATERIAL", "Acid Gem damage penalty", "NONE", 0, 0)
				end
			end
		end,
	},
	{
		name 					= "nuclear_glass",
		ui_name					= "Nuclear Glass",
		categories				= "projectiles",
		effect_desc				= "Most spells you cast become explosive, you take 25% more damage from explosions.",
		effect_desc_short		= "Explosive spells.",
        precursor_types			= "magic_artifact",
		upgrade_effect_desc		= "For each upgrade level, explosive power and radius are increased.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("nuclear_glass") >= 1) then
				projectileSetExplosionConfig(projectile_entity_id, getRelicLevel("nuclear_glass"))
			end
		end,
		ondamaged_function		= function(damage, message, entity_thats_responsible, is_fatal, projectile_thats_responsible )
			if(isRelicOwned("nuclear_glass") >= 1) then
				if(message == "$damage_explosion") then
					GamePrint("Relic penalty: 25% more explosion damage taken")
					local damage_penalty = (damage * 25 / 100)
					EntityInflictDamage( getPlayerEntity(), damage_penalty, "DAMAGE_EXPLOSION", "Unstable Crystal damage penalty", "BLOOD_SPRAY", 0, 0)
				end
			end
		end,
		onpickup_function		= function()
			addNewSimplifiedInternalVariable(getPlayerEntity(), "nuclear_glass")
		end,
		ondrop_function			= function()
			removeSimplifiedInternalVariable(getPlayerEntity(), "nuclear_glass")
		end
	},
	{
		name 					= "sacred_ember",
		ui_name					= "Sacred Ember",
		categories				= "projectiles,status_effects",
		effect_desc				= "When you're on fire, your spells get a massive damage boost.",
		effect_desc_short		= "Damage bonus when you're on fire.",
        precursor_types			= "magic_artifact",
		upgrade_effect_desc		= "For each upgrade level, damage boost is increased.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("sacred_ember") >= 1) then
				if(GameGetGameEffectCount(getPlayerEntity(), "ON_FIRE") > 0) then
					multiplyProjectileBaseDamage(projectile_entity_id, 10 + (1 * getRelicLevel("sacred_ember")))
				end
			end
		end
	},
	{
		name 					= "cursed_ring",
		ui_name					= "Cursed Ring",
		categories				= "projectiles,health",
		effect_desc				= "You steal health from enemies you hit with your spells, but \nyou take 80% more projectile, explosion and melee damage.",
		effect_desc_short		= "Your spells steal health from enemies.",
        precursor_types			= "ring",
		upgrade_effect_desc		= "For each upgrade level, you steal more health from enemies hit.",
		is_upgradable 			= true,
		onenemydamaged_function	= function(damage, message, entity_thats_responsible, is_fatal, projectile_thats_responsible )
			if(isRelicOwned("cursed_ring") >= 1) then
				if(message == "$damage_projectile" or message == "$damage_explosion") then
					SetRandomSeed( GameGetFrameNum(), math.random(1,50))
					addPlayerHealth(((damage / Random(1,2)) + (1 * getRelicLevel("cursed_ring")) * 25), false)
				end
			end
		end,
		ondamaged_function		= function(damage, message, entity_thats_responsible, is_fatal, projectile_thats_responsible )
			if(isRelicOwned("cursed_ring") >= 1) then
				if(message == "$damage_projectile") then
					GamePrint("Relic penalty: 80% more projectile damage taken")
					local damage_penalty = (damage * 80 / 100)
					EntityInflictDamage( getPlayerEntity(), damage_penalty, "DAMAGE_PROJECTILE", "Cursed Ring damage penalty", "BLOOD_SPRAY", 0, 0)
				elseif(message == "$damage_melee") then
					GamePrint("Relic penalty: 80% more melee damage taken")
					local damage_penalty = (damage * 80 / 100)
					EntityInflictDamage( getPlayerEntity(), damage_penalty, "DAMAGE_MELEE", "Cursed Ring damage penalty", "BLOOD_SPRAY", 0, 0)
				elseif(message == "$damage_explosion") then
					GamePrint("Relic penalty: 80% more explosion damage taken")
					local damage_penalty = (damage * 80 / 100)
					EntityInflictDamage( getPlayerEntity(), damage_penalty, "DAMAGE_EXPLOSION", "Cursed Ring damage penalty", "BLOOD_SPRAY", 0, 0)
				end
			end
		end,
	},
	{
		name 					= "scorching_rose",
		ui_name					= "Scorching Rose",
		categories				= "health,materials",
		effect_desc				= "Lava no longer damages you.",
		effect_desc_short		= "Lava no longer damages you.",
        precursor_types			= "magic_artifact",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			EntitySetDamageFromMaterial( getPlayerEntity(), "lava", 0 )
		end,
		ondrop_function			= function()
			EntitySetDamageFromMaterial( getPlayerEntity(), "lava", 0.003 )
		end,
		excludes_relics			= "universal_immunity_stone,omega_shield",
		excludes_relics_desc	= "It can't be used along with other lava protection related relics",
		excludes_relics_tags	= "lava_protection",
	},
	{
		name 					= "igneous_necklace",
		ui_name					= "Igneous Necklace",
		categories				= "projectiles,materials",
		effect_desc				= "Spells you cast that explode on impact have 30% chance of releasing \nfire when they explode or hit something, but you take double damage from fire.",
		effect_desc_short		= "Your spells spit fire.",
        precursor_types			= "necklace",
		upgrade_effect_desc		= "For each upgrade level, a 5% chance is added on each spell cast.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("igneous_necklace") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 30 + (5 * getRelicLevel("igneous_necklace"))
				if(chance >= 1 and chance <= odds) then
					attachLoadEntityOnExplosiontoProjectile(projectile_entity_id, getLoadOnExplosion("launch_elemental_particles_fire").path, "fire")
				end
			end
		end,
		onpickup_function		= function()
			local fire_multiplier = getBasicDamageMultiplier(getPlayerEntity(), "fire")
			setBasicDamageMultiplier(getPlayerEntity(), "fire", fire_multiplier * 2)
		end,
		ondrop_function			= function()
			local fire_multiplier = getBasicDamageMultiplier(getPlayerEntity(), "fire")
			setBasicDamageMultiplier(getPlayerEntity(), "fire", fire_multiplier / 2)
		end
	},
	{
		name 					= "corroding_stone",
		ui_name					= "Corroding Stone",
		categories				= "health,materials",
		effect_desc				= "Acid no longer damages you.",
		effect_desc_short		= "Acid no longer damages you.",
        precursor_types			= "magic_artifact,effect_stone",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			EntitySetDamageFromMaterial( getPlayerEntity(), "acid", 0 )
		end,
		ondrop_function			= function()
			EntitySetDamageFromMaterial( getPlayerEntity(), "acid", 0.005 )
		end,
		excludes_relics			= "universal_immunity_stone,omega_shield",
		excludes_relics_desc	= "It can't be used along with other acid protection related relics",
		excludes_relics_tags	= "acid_protection",
	},
	{
		name 					= "vile_crystal",
		ui_name					= "Vile Crystal",
		categories				= "health,materials",
		effect_desc				= "Poison no longer directly damages you (but will still inflict you with the Poisoned effect).",
		effect_desc_short		= "Poison no longer damages you.",
        precursor_types			= "effect_crystal",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			EntitySetDamageFromMaterial( getPlayerEntity(), "poison", 0 )
		end,
		ondrop_function			= function()
			EntitySetDamageFromMaterial( getPlayerEntity(), "poison", 0.001 )
		end,
		excludes_relics			= "universal_immunity_stone,omega_shield",
		excludes_relics_desc	= "It can't be used along with other poison protection related relics",
		excludes_relics_tags	= "poison_protection",
	},
	{
		name 					= "mind_probe",
		ui_name					= "Mind Probe",
		categories				= "enemies",
		effect_desc				= "Your spells deal significantly less damage, but they have a 20% chance of carrying \na mind controlling payload that makes friendly enemies they hit. Buffed enemies are immune to this effect.",
		effect_desc_short		= "Make your enemies friendly.",
        precursor_types			= "magic_artifact",
		upgrade_effect_desc		= "For each upgrade level, effect chance is increased by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("mind_probe") >= 1) then
				-- bonus
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 20 + (5 * getRelicLevel("mind_probe"))
				if(chance >= 1 and chance <= odds) then
					attachLoadEffectOntoEntitiestoProjectile(projectile_entity_id, getCustomEffect("effect_make_friendly").path, "plasma_fading_pink")
				end
				-- penalty
				fractionProjectileDamage(projectile_entity_id, 3)
			end
		end,
	},
	{
		name 					= "explosive_nanite",
		ui_name					= "Explosive Nanite",
		categories				= "enemies",
		effect_desc				= "Your spells deal significantly less damage, but they have a 20% chance of carrying \nexplosive nanites that turn enemies they hit into friendly suicide bombers.  Buffed enemies are immune to this effect.",
		effect_desc_short		= "Your spells turn your enemies into suicide bombers.",
        precursor_types			= "magic_artifact",
		upgrade_effect_desc		= "For each upgrade level, effect chance is increased by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("explosive_nanite") >= 1) then
				-- bonus
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 20 + (5 * getRelicLevel("explosive_nanite"))
				if(chance >= 1 and chance <= odds) then
					attachLoadEffectOntoEntitiestoProjectile(projectile_entity_id, getCustomEffect("effect_make_suicide_bomber").path, "spark_red")
				end
				-- penalty
				fractionProjectileDamage(projectile_entity_id, 3)
			end
		end,
	},
	{
		name 					= "deflecting_medalion",
		ui_name					= "Deflecting Medalion",
		categories				= "projectiles",
		effect_desc				= "An energy shield that deflects enemy projectiles is projected \naround you, but you take 60% more explosion and melee damage.",
		effect_desc_short		= "A deflecting shield is projected around you.",
        precursor_types			= "magic_artifact,energy_shield",
		upgrade_effect_desc		= "For each upgrade level, you take 5% less explosion and melee damage.",
		is_upgradable 			= true,
		onpickup_function		= function()
			LoadGameEffectEntityTo( getPlayerEntity(), getCustomEffect("effect_deflecting_shield").path )
		end,
		ondrop_function			= function()
			local player_children = EntityGetAllChildren(getPlayerEntity())
			if player_children == nil then return end
			for i,child in ipairs(player_children) do
				if(EntityGetName(child) == "deflecting_shield") then
					EntityKill(child)
					break
				end
			end
		end,
		ondamaged_function		= function(damage, message, entity_thats_responsible, is_fatal, projectile_thats_responsible )
			if(isRelicOwned("deflecting_medalion") >= 1) then
				local damage_penalty_percentage = 60 - (5 * getRelicLevel("deflecting_medalion"))
				if(message == "$damage_explosion") then
					GamePrint("Relic penalty: " .. damage_penalty_percentage .. "% more explosion damage taken")
					local damage_penalty = (damage * damage_penalty_percentage) / 100
					EntityInflictDamage( getPlayerEntity(), damage_penalty, "DAMAGE_EXPLOSION", "Deflecting Medalion damage penalty", "BLOOD_SPRAY", 0, 0)
				elseif(message == "$damage_melee") then
					GamePrint("Relic penalty: " .. damage_penalty_percentage .. "% more melee damage taken")
					local damage_penalty = (damage * damage_penalty_percentage) / 100
					EntityInflictDamage( getPlayerEntity(), damage_penalty, "DAMAGE_MELEE", "Deflecting Medalion damage penalty", "BLOOD_SPRAY", 0, 0)
				end
			end
		end,
		--[[
		onpickup_function		= function()
			local fire_multiplier = getBasicDamageMultiplier(getPlayerEntity(), "fire")
			setBasicDamageMultiplier(getPlayerEntity(), "fire", fire_multiplier * 2)
		end,
		ondrop_function			= function()
			local fire_multiplier = getBasicDamageMultiplier(getPlayerEntity(), "fire")
			setBasicDamageMultiplier(getPlayerEntity(), "fire", fire_multiplier / 2)
		end,
		]]
		excludes_relics			= "deflecting_medalion_master",
		excludes_relics_desc	= "It can't be used along with Master Deflecting Medallion relic",
		excludes_relics_tags	= "deflecting_medalion",
	},
	{
		name 					= "golden_leaf",
		ui_name					= "Golden Leaf",
		categories				= "health",
		effect_desc				= "If you have 500 gold, it is converted into 10 health, and if you \nare at full health when gold is deducted, your maximum health will be extended by that amount.",
		effect_desc_short		= "Every 500 gold = 10 health.",
        precursor_types			= "magic_artifact",
		upgrade_effect_desc		= "For each upgrade level, 10 gold is subtracted from the base amount of 500 and bonus hp is added.",
		is_upgradable 			= true,
		overtimeslow_function	= function()
			if(isRelicOwned("golden_leaf") >= 1) then
				local base_gold = 500 - (10 * getRelicLevel("golden_leaf"))
				if(getPlayerGold() >= base_gold) then
					addPlayerGold(base_gold * -1)
					addPlayerHealth(10 * (1 * getRelicLevel("golden_leaf")), true)
				end
			end
		end,
	},
	{
		name 					= "frozen_heart",
		ui_name					= "Frozen Heart",
		categories				= "materials",
		effect_desc				= "A magic field that converts smoke to freezing gas is projected around you.",
		effect_desc_short		= "Smoke around you is converted into freezing gas.",
        precursor_types			= "magic_artifact",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			LoadGameEffectEntityTo( getPlayerEntity(), getCustomEffect("effect_field_smoke_to_freezing_gas").path )
		end,
		ondrop_function			= function()
			local player_children = EntityGetAllChildren(getPlayerEntity())
			if player_children == nil then return end
			for i,child in ipairs(player_children) do
				if(EntityGetName(child) == "field_smoke_to_freezing_gas") then
					EntityKill(child)
					break
				end
			end
		end,
	},
	{
		name 					= "ancient_mask",
		ui_name					= "Ancient Mask",
		categories				= "materials",
		effect_desc				= "Toxic gas no longer damages you, but steam is now harmful.",
		effect_desc_short		= "You're immune to toxic gas, but steam is now harmful.",
        precursor_types			= "magic_artifact",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			EntitySetDamageFromMaterial( getPlayerEntity(), "radioactive_gas", 0 ) 
			EntitySetDamageFromMaterial( getPlayerEntity(), "steam", 0.001 )
		end,
		ondrop_function			= function()
			EntitySetDamageFromMaterial( getPlayerEntity(), "radioactive_gas", 0.001 ) 
			EntitySetDamageFromMaterial( getPlayerEntity(), "steam", 0 )
		end,
		excludes_relics			= "universal_immunity_stone,omega_shield",
		excludes_relics_desc	= "It can't be used along with other toxic gas protection related relics",
		excludes_relics_tags	= "radgas_protection",
	},
	{
		name 					= "midas_necklace",
		ui_name					= "Midas Necklace",
		categories				= "enemies",
		effect_desc				= "Your spells deal significantly less damage, but they have a 10% chance of \n turning the blood of enemies they hit into gold.",
		effect_desc_short		= "Transforms enemies blood into gold.",
        precursor_types			= "necklace",
		upgrade_effect_desc		= "For each upgrade level, effect chance is increased by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("midas_necklace") >= 1) then
				-- bonus
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 10 + (5 * getRelicLevel("midas_necklace"))
				if(chance >= 1 and chance <= odds) then
					attachLoadEffectOntoEntitiestoProjectile(projectile_entity_id, getCustomEffect("effect_convert_blood_to_gold").path, "gold")
				end
				-- penalty
				fractionProjectileDamage(projectile_entity_id, 3)
			end
		end,
	},
	{
		name 					= "retaliation_ring",
		ui_name					= "Retaliation Ring",
		categories				= "projectiles",
		effect_desc				= "You take 15% more projectile damage, but when damaged by projectiles \nyou retaliate with a barrage of projectiles of the same type that hit you.",
		effect_desc_short		= "Retaliation spells are shot when hit by enemy projectiles.",
        precursor_types			= "ring",
		upgrade_effect_desc		= "For each upgrade level, you take 2% less projectile damage.",
		is_upgradable 			= true,
		ondamaged_function	= function(damage, message, entity_thats_responsible, is_fatal, projectile_thats_responsible )
			if(isRelicOwned("retaliation_ring") >= 1) then
				if(message == "$damage_projectile") then
					local retaliation_multiplier = getRelicLevel("retaliation_ring")
					if(retaliation_multiplier == 0) then
						retaliation_multiplier = 1
					end
					-- bonus
					shootRetaliationProjectiles(GetUpdatedEntityID(), projectile_thats_responsible, retaliation_multiplier )
					-- penalty
					local damage_penalty_percentage = 15 - (2 * getRelicLevel("retaliation_ring"))
					GamePrint("Relic penalty: " .. damage_penalty_percentage .. "% more projectile damage taken")
					local damage_penalty = (damage * damage_penalty_percentage)  / 100
					EntityInflictDamage( getPlayerEntity(), damage_penalty, "DAMAGE_PROJECTILE", "Retaliation Ring damage penalty", "BLOOD_SPRAY", 0, 0)
				end
			end
		end,
	},
	{
		name 					= "gold_pot",
		ui_name					= "Gold Pot",
		categories				= "materials,enemies",
		effect_desc				= "Spells you cast have a 10% chance of inducing enemies to explode \ninto gold dust when they die",
		effect_desc_short		= "Makes your enemies explode into gold dust on death.",
        precursor_types			= "magic_artifact",
		upgrade_effect_desc		= "For each upgrade level, effect chance is increased by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("gold_pot") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 10 + (5 * getRelicLevel("gold_pot"))
				if(chance >= 1 and chance <= odds) then
					attachLoadEffectOntoEntitiestoProjectile(projectile_entity_id, getCustomEffect("effect_gold_explosion").path, "gold")
				end
			end
		end,
	},
	{
		name 					= "rusty_hook",
		ui_name					= "Rusty Hook",
		categories				= "projectiles",
		effect_desc				= "Spells you cast have a 25% chance of dragging enemies away while \ncausing slice damage to them.",
		effect_desc_short		= "Drags your enemies away, causing slice damage.",
        precursor_types			= "gadget",
		upgrade_effect_desc		= "For each upgrade level, effect chance is increased by 5% and damage dealt is increased.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("rusty_hook") >= 1) then		
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 25 + (5 * getRelicLevel("rusty_hook"))
				if(chance >= 1 and chance <= odds) then
					addBuffVisualIndicator_Projectile(projectile_entity_id, "blood")
					attachDragCollisionHandlertoProjectile(projectile_entity_id, getCollisionHandler("drag_slice_damage").path, true, 20, "enemy", false)
				end
			end
		end,
		ondrop_function			= function()
			removeInternalVariable(getPlayerEntity(), "rusty_hook_damage")
		end,
		overtime_function		= function()
			local multiplier = getRelicLevel("rusty_hook")
			if(multiplier == 0) then
				multiplier = 1
			end
			local damage_threshold = 0.01 * multiplier
			if(not entityHasInternalVariableValue(getPlayerEntity(), "rusty_hook_damage")) then
				addNewInternalVariable(getPlayerEntity(), "rusty_hook_damage", "value_float", damage_threshold)
			end
			setInternalVariableValue(getPlayerEntity(), "rusty_hook_damage", "value_float", damage_threshold)
		end,
	},
	{
		name 					= "ancient_dagger",
		ui_name					= "Ancient Dagger",
		categories				= "projectiles",
		effect_desc				= "Spells you cast have a 20% chance of inducing bleeding on enemies they hit.",
		effect_desc_short		= "Your spells induce bleeding on enemies.",
        precursor_types			= "magic_artifact",
		upgrade_effect_desc		= "For each upgrade level, effect chance is increased by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("ancient_dagger") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 20 + (5 * getRelicLevel("ancient_dagger"))
				if(chance >= 1 and chance <= odds) then
					attachLoadEffectOntoEntitiestoProjectile(projectile_entity_id, getCustomEffect("effect_bleeding").path, "blood")
				end
			end
		end,
	},
	{
		name 					= "cursed_mirror",
		ui_name					= "Cursed Mirror",
		categories				= "projectiles",
		effect_desc				= "You take 50% more damage from projectiles, but if you get hit \ntheir shooters take double damage.",
		effect_desc_short		= "Enemies that hit you with projectiles also take damage.",
        precursor_types			= "magic_artifact",
		upgrade_effect_desc		= "For each upgrade level, you take 5% less projectile damage.",
		is_upgradable 			= true,
		ondamaged_function	= function(damage, message, entity_thats_responsible, is_fatal, projectile_thats_responsible )
			if(isRelicOwned("cursed_mirror") >= 1) then
				if(message == "$damage_projectile") then
					-- bonus
					EntityInflictDamage( entity_thats_responsible, damage * 2, "DAMAGE_KICK", "Cursed Mirror damage", "BLOOD_SPRAY", 0, 0)
					local entity_thats_responsible_x, entity_thats_responsible_y = EntityGetTransform(entity_thats_responsible)
					EntityLoad("mods/ArcaneReliquary/files/entity_definitions/particle_explosions/cursed_mirror_effect_explosion.xml", entity_thats_responsible_x, entity_thats_responsible_y)
					-- penalty
					local damage_penalty_percentage = 50 - (5 * getRelicLevel("cursed_mirror"))
					GamePrint("Relic penalty: " .. damage_penalty_percentage .. "% more projectile damage taken")
					local damage_penalty = (damage * damage_penalty_percentage) / 100
					EntityInflictDamage( getPlayerEntity(), damage_penalty, "DAMAGE_PROJECTILE", "Cursed Mirror damage penalty", "BLOOD_SPRAY", 0, 0)
				end
			end
		end,
	},
	{
		name 					= "wooden_monocular",
		ui_name					= "Wooden Monocular",
		categories				= "projectiles",
		effect_desc				= "Most spells you cast are now short ranged, but once cast, they will track \nnearby enemies, then shoot copies of themselves towards them.",
		effect_desc_short		= "Your spells track enemies nearby.",
        precursor_types			= "magic_artifact",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("wooden_monocular") >= 1) then
				if(not isPhysicsProjectile(projectile_entity_id) and not isOrbitingProjectile(projectile_entity_id)) then
					attachStopandTrackMechanictoProjectile(projectile_entity_id)
				end
			end
		end,
		excludes_relics			= "binoculars",
		excludes_relics_desc	= "It can't be used along with other spell track and shoot relic variations",
		excludes_relics_tags	= "track_and_shoot_effect",
	},
	{
		name 					= "stillness_ring",
		ui_name					= "Stillness Ring",
		categories				= "projectiles",
		effect_desc				= "Spells you cast while standing on solid ground, without moving, deal 4x damage.",
		effect_desc_short		= "Damage bonus when standing still.",
        precursor_types			= "ring",
		upgrade_effect_desc		= "For each upgrade level, damage bonus is increased.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("stillness_ring") >= 1) then
				if(isPlayerStanding() and (not isPlayerMoving())) then
					multiplyProjectileBaseDamage(projectile_entity_id, 4 + (1 * getRelicLevel("stillness_ring")))
				end
			end
		end,
	},
	{
		name 					= "duality_necklace",
		ui_name					= "Duality Necklace",
		categories				= "projectiles",
		effect_desc				= "Provided you are midair, for every spell you cast, a copy of it will be shot backwards.",
		effect_desc_short		= "When midair, your spells are also shot backwards.",
        precursor_types			= "necklace",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("duality_necklace") >= 1) then
				if(not isPlayerStanding()) then
					shootMirroredProjectile(projectile_entity_id)
				end
			end
		end,
	},
	{
		name 					= "metal_boot",
		ui_name					= "Metal Boot",
		categories				= "player",
		effect_desc				= "Your kicks deal 5x more damage, but you take 5x melee damage \nfrom enemies. \nIt also synergizes with Sniper Scope relic effect.",
		effect_desc_short		= "Your kicks deal 5x more damage.",
        precursor_types			= "boot",
		upgrade_effect_desc		= "For each upgrade level, bonus damage is added to kick damage multiplier.",
		is_upgradable 			= true,
		onenemydamaged_function	= function(damage, message, entity_thats_responsible, is_fatal, projectile_thats_responsible )
			if(isRelicOwned("metal_boot") >= 1) then
				if(message == "$damage_kick") then
					EntityInflictDamage( GetUpdatedEntityID(), damage * 5 + (0.4 * getRelicLevel("metal_boot")), "DAMAGE_MELEE", "Metal Boot damage", "BLOOD_SPRAY", 0, 0)
				end
			end
		end,
		ondamaged_function	= function(damage, message, entity_thats_responsible, is_fatal, projectile_thats_responsible )
			if(isRelicOwned("metal_boot") >= 1) then
				if(message == "$damage_melee") then
					GamePrint("Relic penalty: 5x more melee damage taken")
					local damage_penalty = damage * 4
					EntityInflictDamage( getPlayerEntity(), damage_penalty, "DAMAGE_MELEE", "Metal Boot damage penalty", "BLOOD_SPRAY", 0, 0)
				end
			end
		end,
		onextendedkick_function	= function( entity_id, extended_kick_id)
			if(isRelicOwned("metal_boot") >= 1) then
				multiplyProjectileDamageByType(extended_kick_id, "melee", 5)
			end
		end,
	},
	{
		name 					= "heavy_anvil",
		ui_name					= "Heavy Anvil",
		categories				= "physics,enemies",
		effect_desc				= "You are no longer immune to impact damage, but if an enemy is \ndamaged by objects, they explode instantly.",
		effect_desc_short		= "Kills with impact damage make enemies explode.",
        precursor_types			= "gadget",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onenemydamaged_function	= function(damage, message, entity_thats_responsible, is_fatal, projectile_thats_responsible )
			if(isRelicOwned("heavy_anvil") >= 1) then
				if(message == "$damage_physicshit") then
					local x, y = EntityGetTransform(GetUpdatedEntityID())
					EntityLoad("mods/ArcaneReliquary/files/entity_definitions/others/heavy_explosion.xml", x, y)
				end
			end
		end,
		onpickup_function		= function()
			playerPhysicsObjectsDamage(true)
		end,
		ondrop_function			= function()
			playerPhysicsObjectsDamage(false)
		end,
	},
	{
		name 					= "magnesium_ore",
		ui_name					= "Magnesium Ore",
		categories				= "explosions,projectiles",
		effect_desc				= "Spells you cast have their explosive power improved the more empty slots you have in your active wand.",
		effect_desc_short		= "Every empty slot in your wand means extra explosive power.",
        precursor_types			= "magic_artifact",
		upgrade_effect_desc		= "For each upgrade level, explosive power and radius are increased.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("magnesium_ore") >= 1) then
				local current_wand_id = getCurrentlyEquippedWandId()
				if(current_wand_id ~= -1) then
					local free_slots, spells = getWandEmptyandOccupiedSlots(getCurrentlyEquippedWandId())
					if(free_slots > 0) then
						addProjectileCreateCellMaterialExplosion(projectile_entity_id, "spark_blue_dark")
						multiplyProjectileExplosiveDamage(projectile_entity_id, (0.3 * free_slots) + (0.3 * getRelicLevel("magnesium_ore")))
						multiplyProjectileExplosionRadius(projectile_entity_id, (0.5 * free_slots) + (0.3 * getRelicLevel("magnesium_ore")))
					end
				end
			end
		end,
	},
	{
		name 					= "hollow_coin",
		ui_name					= "Hollow Coin",
		categories				= "projectiles",
		effect_desc				= "Every 10 individual gold nuggets you collect gives you 1x damage multiplier for your spells. \nStacks up to 10x maximum, taking projectile, melee or explosion damage resets the counter.",
		effect_desc_short		= "Gold nuggets collects without taking damage = extra spell damage.",
        precursor_types			= "magic_artifact",
		upgrade_effect_desc		= "For each upgrade level, 1 gold nugget is deducted from the 10 required for each multiplier added.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("hollow_coin") >= 1) then
				local current_gold_nuggets_collected = getInternalVariableValue(getPlayerEntity(), "gold_nuggets_collected", "value_int")
				local base_gold_nuggets = 10 - (1 * getRelicLevel("hollow_coin"))
				local multiplier = math.floor(current_gold_nuggets_collected / base_gold_nuggets)
				if(multiplier > 10) then
					multiplier = 10
				end
				if(multiplier ~= 0) then
					multiplier = multiplier + 1
					GamePrint("Relic effect: " .. multiplier .. "x damage multiplier ")
					multiplyProjectileBaseDamage(projectile_entity_id, multiplier * isRelicOwned("hollow_coin"))
				end
			end
		end,
		ondamaged_function		= function(damage, message, entity_thats_responsible, is_fatal, projectile_thats_responsible )
			if(isRelicOwned("hollow_coin") >= 1) then
				if(message == "$damage_melee" or message == "$damage_projectile" or message == "$damage_explosion") then
					GamePrint("Relic penalty: damage taken, gold nugget damage multiplier reset")
					setInternalVariableValue(getPlayerEntity(), "gold_nuggets_collected", "value_int", 0)
				end
			end
		end,
		ongoldpickup_function	= function()
			if(isRelicOwned("hollow_coin") >= 1) then
				local current_gold_nuggets_collected = getInternalVariableValue(getPlayerEntity(), "gold_nuggets_collected", "value_int")
				local new_gold_nuggets_collected = current_gold_nuggets_collected + 1
				setInternalVariableValue(getPlayerEntity(), "gold_nuggets_collected", "value_int", new_gold_nuggets_collected)
			end
		end
	},
	{
		name 					= "nuka_cola",
		ui_name					= "Nuka Cola",
		categories				= "projectiles,status_effects",
		effect_desc				= "When you're covered in toxic sludge, your spells have a \n30% chance of mutating enemies they hit.",
		effect_desc_short		= "When covered in toxic sludge, your spells have a chance to mutate enemies.",
        precursor_types			= "gadget",
		upgrade_effect_desc		= "For each upgrade level, effect chance is increased by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			
			if(isRelicOwned("nuka_cola") >= 1) then			
				if(GameGetGameEffectCount(getPlayerEntity(), "RADIOACTIVE") > 0) then
					SetRandomSeed( GameGetFrameNum(), math.random(1,50))
					local chance = Random(1,100)
					local odds = 30 + (5 * getRelicLevel("nuka_cola"))
					if(chance >= 1 and chance <= odds) then
						attachLoadEffectOntoEntitiestoProjectile(projectile_entity_id, "data/entities/misc/effect_polymorph.xml", "radioactive_gas")
					end
				end
			end
		end
	},
	{
		name 					= "nuka_cola_quantum",
		ui_name					= "Nuka Cola Quantum",
		categories				= "projectiles,enemies",
		effect_desc				= "When you're covered in toxic sludge, your spells have a \n25% chance of inducing enemies to blow off in a huge nuclear explosion when they die.",
		effect_desc_short		= "When covered in toxic sludge, your spells have a chance to induce nuclear explosions on enemies.",
        precursor_types			= "gadget",
		upgrade_effect_desc		= "For each upgrade level, effect chance is increased by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("nuka_cola_quantum") >= 1) then
				if(GameGetGameEffectCount(getPlayerEntity(), "RADIOACTIVE") > 0) then
					SetRandomSeed( GameGetFrameNum(), math.random(1,50))
					local chance = Random(1,100)
					local odds = 25 + (5 * getRelicLevel("nuka_cola_quantum"))
					if(chance >= 1 and chance <= odds) then
						attachLoadEffectOntoEntitiestoProjectile(projectile_entity_id, getCustomEffect("effect_nuclear_explosion").path, "spark_yellow")
					end
				end
			end
		end
	},
	{
		name 					= "portable_refresher",
		ui_name					= "Portable Refresher",
		categories				= "projectiles",
		effect_desc				= "When you cast a spell with limited uses, there's a 30% chance an \nextra charge will be added to all instances of the same spell in your current wand.",
		effect_desc_short		= "Sometimes your limited use spells will not consume a use when cast.",
        precursor_types			= "magic_artifact",
		upgrade_effect_desc		= "For each upgrade level, effect chance is increased by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("portable_refresher") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 30 + (5 * getRelicLevel("portable_refresher"))
				if(chance >= 1 and chance <= odds) then
					updatedUsesRemainingCurrentWandSpells_Projectile(projectile_entity_id, 1)
				end
			end
		end
	},
	{
		name 					= "sharpening_wheel",
		ui_name					= "Sharpening Wheel",
		categories				= "projectiles",
		effect_desc				= "For every 5 enemies you kill in a row, you get an extra 1% critical chance when you cast a \nspell, stacks up to 30% maximum, taking melee, explosion or projectile damage resets the counter.",
		effect_desc_short		= "Enemies killed without taking damage = extra critical chance.",
        precursor_types			= "gadget",
		upgrade_effect_desc		= "For each upgrade level, an additional 1% is included when the critical chance is received.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("sharpening_wheel") >= 1) then
				local current_kills = getInternalVariableValue(getPlayerEntity(), "enemy_kills", "value_int")
				local extra_chance = math.floor(current_kills / 5) + (1 * getRelicLevel("sharpening_wheel"))
				if(extra_chance > 30) then
					extra_chance = 30
				end
				if(extra_chance ~= 0) then
					GamePrint("Relic effect: +" .. extra_chance .. "% critical chance ")
					local chance, damage_multiplier = getProjectileCriticalDamageSettings(projectile_entity_id)
					setProjectileCriticalDamageSettings(projectile_entity_id, (chance + extra_chance), damage_multiplier)
				end
			end
		end,
		ondamaged_function		= function(damage, message, entity_thats_responsible, is_fatal, projectile_thats_responsible )
			if(isRelicOwned("sharpening_wheel") >= 1) then
				if(message == "$damage_melee" or message == "$damage_projectile" or message == "$damage_explosion") then
					GamePrint("Relic penalty: damage taken, enemy kills critical multiplier reset")
					setInternalVariableValue(getPlayerEntity(), "enemy_kills", "value_int", 0)
				end
			end
		end,
		onenemykill_function	= function()
			if(isRelicOwned("sharpening_wheel") >= 1) then
				local current_kills = getInternalVariableValue(getPlayerEntity(), "enemy_kills", "value_int")
				local new_kills = current_kills + 1
				setInternalVariableValue(getPlayerEntity(), "enemy_kills", "value_int", new_kills)
			end
		end
	},
	{
		name 					= "tesla_coil",
		ui_name					= "Tesla Coil",
		categories				= "projectiles",
		effect_desc				= "Your spells have a 25% chance of exploding into a barrage of thunderbolts when they hit an obstacle or explode.",
		effect_desc_short		= "Your spells release thunderbolts on impact.",
        precursor_types			= "gadget,device",
		upgrade_effect_desc		= "For each upgrade level, effect chance is increased by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("tesla_coil") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 25 + (5 * getRelicLevel("tesla_coil"))
				if(chance >= 1 and chance <= odds) then
					addBuffVisualIndicator_Projectile(projectile_entity_id, "spark_blue")
					attachLoadEntityOnExplosiontoProjectile(projectile_entity_id, getLoadOnExplosion("launch_circular_projectiles_thunderbolts_regular").path)
					--addNewLuaCallback(projectile_entity_id, "overtime", "mods/ArcaneReliquary/files/scripts/projectiles/shoot_thunderbolt_random.lua", 10, false, "shoot_thunderbolt_random")
				end
			end
		end,
		onpickup_function		= function()
			addNewSimplifiedInternalVariable(getPlayerEntity(), "tesla_coil")
		end,
		ondrop_function			= function()
			removeSimplifiedInternalVariable(getPlayerEntity(), "tesla_coil")
		end
	},
	{
		name 					= "kyber_crystal",
		ui_name					= "Kyber Crystal",
		categories				= "projectiles",
		effect_desc				= "Most spells you cast have a 50% chance of shooting red laser bolts at random directions once cast.",
		effect_desc_short		= "Your spells shoot laser bolts.",
        precursor_types			= "effect_crystal",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5% and when the effect triggers, laser bolts are shot more frequently.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("kyber_crystal") >= 1) then
				if((not isPhysicsProjectile(projectile_entity_id)) and (not isProjectileHittable(projectile_entity_id))) then
					SetRandomSeed( GameGetFrameNum(), math.random(1,50))
					local chance = Random(1,100)
					local odds = 50 + (5 * getRelicLevel("kyber_crystal"))
					if(chance >= 1 and chance <= odds) then
						addNewInternalVariable(projectile_entity_id, "projectile_definition", "value_string", "mods/ArcaneReliquary/files/entity_definitions/custom_projectiles/red_laser.xml")
						addNewInternalVariable(projectile_entity_id, "projectile_velocity", "value_int", 650)
						addNewInternalVariable(projectile_entity_id, "max_projectiles_shot", "value_int", 25)
						addNewLuaCallback(projectile_entity_id, "overtime", "mods/ArcaneReliquary/files/scripts/projectiles/launch_projectile_random_direction.lua", 25 - (2 * getRelicLevel("kyber_crystal")), false, "launch_projectile_random_direction")
					end
				end
			end
		end
	},
	{
		name 					= "golden_fork",
		ui_name					= "Golden Fork",
		categories				= "projectiles",
		effect_desc				= "When you cast a spell, there's a 50% chance an extra copy of it will \nbe cast for every empty slot in your active wand.",
		effect_desc_short		= "Every empty slot in your wand means extra spells.",
        precursor_types			= "magic_artifact",
		upgrade_effect_desc		= "For each upgrade level, effect chance is increased by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("golden_fork") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 50 + (5 * getRelicLevel("golden_fork"))
				if(chance >= 1 and chance <= odds) then
					local current_wand_id = getCurrentlyEquippedWandId()
					if(current_wand_id ~= -1) then
						local free_slots, spells = getWandEmptyandOccupiedSlots(getCurrentlyEquippedWandId())
						local multiplier = getRelicLevel("golden_fork")
						if(multiplier == 0) then
							multiplier = 1
						end
						for i=1,free_slots * multiplier do
							local x, y = EntityGetTransform(projectile_entity_id)
							shootProjectileBasedonAnotherProjectile(projectile_entity_id, getProjectileDefinitionPath(projectile_entity_id), x, y, 150)
						end
					end
				end
			end
		end
	},
	{
		name 					= "wicked_necklace",
		ui_name					= "Wicked Necklace",
		categories				= "projectiles,enemies",
		effect_desc				= "An energy field that damages enemies is projected \naround you, but you take triple projectile damage.",
		effect_desc_short		= "A damaging field is projected around you.",
        precursor_types			= "necklace",
		upgrade_effect_desc		= "For each upgrade level, projectile damage is decreased.",
		is_upgradable 			= true,
		onpickup_function		= function()
			LoadGameEffectEntityTo( getPlayerEntity(), getCustomEffect("effect_damaging_field").path )
		end,
		ondrop_function			= function()
			local player_children = EntityGetAllChildren(getPlayerEntity())
			if player_children == nil then return end
			for i,child in ipairs(player_children) do
				if(EntityGetName(child) == "damaging_field") then
					EntityKill(child)
					break
				end
			end
		end,
		ondamaged_function		= function(damage, message, entity_thats_responsible, is_fatal, projectile_thats_responsible )
			if(isRelicOwned("wicked_necklace") >= 1) then
				if(message == "$damage_projectile") then
					local damage_dividend = getRelicLevel("wicked_necklace")
					if(damage_dividend == 0) then
						damage_dividend = 1
					end
					EntityInflictDamage( getPlayerEntity(), damage * 2 / damage_dividend, "DAMAGE_PROJECTILE", "Wicked Necklace damage penalty", "BLOOD_SPRAY", 0, 0)
				end
			end
		end,
		excludes_relics			= "blizzard_necklace,blaze_necklace,power_surge_necklace",
		excludes_relics_desc	= "It can't be used along with Blizzard, Blaze or Power Surge Necklace relics",
		excludes_relics_tags	= "blizzard_necklace,blaze_necklace,power_surge_necklace",
		
	},
	{
		name 					= "spell_caster_nanite",
		ui_name					= "Spell Caster Nanite",
		categories				= "projectiles,enemies",
		effect_desc				= "Spells you cast that hit an enemy have a 25% chance of inducing them \nto shoot the same spell at random directions overtime.",
		effect_desc_short		= "Induces enemies into randomly shooting your spells.",
        precursor_types			= "magic_artifact",
		upgrade_effect_desc		= "For each upgrade level, effect chance is increased by 5%.",
		is_upgradable 			= true,
		onenemydamaged_function	= function(damage, message, entity_thats_responsible, is_fatal, projectile_thats_responsible )
			if(isRelicOwned("spell_caster_nanite") >= 1) then
				if(message == "$damage_projectile") then
					local enemy_id = GetUpdatedEntityID()
					SetRandomSeed( GameGetFrameNum(), math.random(1,50))
					local chance = Random(1,100)
					local odds = 25 + (5 * getRelicLevel("spell_caster_nanite"))
					if(chance >= 1 and chance <= odds) then
						attachRandomProjectileShootBehaviortoCreature(enemy_id, projectile_thats_responsible)
					end
				end
			end
		end,
	},
	{
		name 					= "explosive_mana_crystal",
		ui_name					= "Explosive Mana Crystal",
		categories				= "projectiles,enemies",
		effect_desc				= "Spells you cast that hit an enemy have a 25% chance of \ninducing them to explode launching copies of your spells when they die.",
		effect_desc_short		= "Your enemies explode releasing copies of your spells.",
        precursor_types			= "effect_crystal",
		upgrade_effect_desc		= "For each upgrade level, effect chance is increased by 5%.",
		is_upgradable 			= true,
		onenemydamaged_function	= function(damage, message, entity_thats_responsible, is_fatal, projectile_thats_responsible )
			if(isRelicOwned("explosive_mana_crystal") >= 1) then
				if(message == "$damage_projectile") then
					local enemy_id = GetUpdatedEntityID()
					SetRandomSeed( GameGetFrameNum(), math.random(1,50))
					local chance = Random(1,100)
					local odds = 25 + (5 * getRelicLevel("explosive_mana_crystal"))
					if(chance >= 1 and chance <= odds) then
						attachProjectileExplosiontoCreature(enemy_id, projectile_thats_responsible)
					end
				end
			end
		end,
	},
	{
		name 					= "pizza_slice",
		ui_name					= "Pizza Slice",
		categories				= "projectiles",
		effect_desc				= "Most spells you cast now deal slice damage, but you take twice as much slice and melee damage.",
		effect_desc_short		= "Adds slice damage to your spells.",
        precursor_types			= "magic_artifact",
		upgrade_effect_desc		= "For each upgrade level, slice damage added is increased.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("pizza_slice") >= 1) then
				attachLoadEntityOnExplosiontoProjectile(projectile_entity_id, getLoadOnExplosion("explosion_slash").path)
				if(not projectileHasDamageType(projectile_entity_id, "slice")) then
					addProjectileDamageByType(projectile_entity_id, "slice", 0.5 + (0.5 * getRelicLevel("pizza_slice")))
				else
					addProjectileDamageByType(projectile_entity_id, "slice", 0.2 + (0.2 * getRelicLevel("pizza_slice")))
				end
			end
		end,
		onpickup_function		= function()
			if(not (getBasicDamageMultiplier(getPlayerEntity(), "slice") == 0 and getBasicDamageMultiplier(getPlayerEntity(), "melee") == 0)) then -- compatiblity check for melee and slice immunity effects
				local slice_multiplier = getBasicDamageMultiplier(getPlayerEntity(), "slice")
				setBasicDamageMultiplier(getPlayerEntity(), "slice", slice_multiplier * 2)
				local melee_multiplier = getBasicDamageMultiplier(getPlayerEntity(), "melee")
				setBasicDamageMultiplier(getPlayerEntity(), "melee", melee_multiplier * 2)
			end
		end,
		ondrop_function			= function()
			if(not (getBasicDamageMultiplier(getPlayerEntity(), "slice") == 0 and getBasicDamageMultiplier(getPlayerEntity(), "melee") == 0)) then -- compatiblity check for melee and slice immunity effects
				local slice_multiplier = getBasicDamageMultiplier(getPlayerEntity(), "slice")
				setBasicDamageMultiplier(getPlayerEntity(), "slice", slice_multiplier / 2)
				local melee_multiplier = getBasicDamageMultiplier(getPlayerEntity(), "melee")
				setBasicDamageMultiplier(getPlayerEntity(), "melee", melee_multiplier / 2)
			end
		end
	},
	{
		name 					= "emerald_gauntlet",
		ui_name					= "Emerald Gauntlet",
		categories				= "projectiles",
		effect_desc				= "Most spells you cast now deal melee damage, but you take twice as much slice and melee damage.",
		effect_desc_short		= "Adds melee damage to your spells.",
        precursor_types			= "magic_artifact",
		upgrade_effect_desc		= "For each upgrade level, melee damage added is increased.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("emerald_gauntlet") >= 1) then
				if(not projectileHasDamageType(projectile_entity_id, "melee")) then
					attachLoadEntityOnExplosiontoProjectile(projectile_entity_id, getLoadOnExplosion("explosion_melee").path)
					addProjectileDamageByType(projectile_entity_id, "melee", 0.5 + (0.5 * getRelicLevel("emerald_gauntlet")))
				end
			end
		end,
		onpickup_function		= function()
			if(not (getBasicDamageMultiplier(getPlayerEntity(), "slice") == 0 and getBasicDamageMultiplier(getPlayerEntity(), "melee") == 0)) then -- compatiblity check for melee and slice immunity effects
				local slice_multiplier = getBasicDamageMultiplier(getPlayerEntity(), "slice")
				setBasicDamageMultiplier(getPlayerEntity(), "slice", slice_multiplier * 2)
				local melee_multiplier = getBasicDamageMultiplier(getPlayerEntity(), "melee")
				setBasicDamageMultiplier(getPlayerEntity(), "melee", melee_multiplier * 2)
			end
		end,
		ondrop_function			= function()
			if(not (getBasicDamageMultiplier(getPlayerEntity(), "slice") == 0 and getBasicDamageMultiplier(getPlayerEntity(), "melee") == 0)) then -- compatiblity check for melee and slice immunity effects
				local slice_multiplier = getBasicDamageMultiplier(getPlayerEntity(), "slice")
				setBasicDamageMultiplier(getPlayerEntity(), "slice", slice_multiplier / 2)
				local melee_multiplier = getBasicDamageMultiplier(getPlayerEntity(), "melee")
				setBasicDamageMultiplier(getPlayerEntity(), "melee", melee_multiplier / 2)
			end
		end
	},
	{
		name 					= "steel_shield",
		ui_name					= "Steel Shield",
		categories				= "health",
		effect_desc				= "You are now immune to slice damage.",
		effect_desc_short		= "You are immune to slice damage.",
        precursor_types			= "gadget",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			setBasicDamageMultiplier(getPlayerEntity(), "slice", 0)
		end,
		ondrop_function			= function()
			setBasicDamageMultiplier(getPlayerEntity(), "slice", getInternalVariableValue(getPlayerEntity(), "slice_multiplier", "value_float"))
		end,
		excludes_relics_desc	= "It can't be used along with Riot Shield relic or other slice proof shields",
		excludes_relics_tags	= "slice_proof_shield",
	},
	{
		name 					= "rubber_shield",
		ui_name					= "Rubber Shield",
		categories				= "health",
		effect_desc				= "You are now immune to melee damage.",
		effect_desc_short		= "You are immune to melee damage.",
        precursor_types			= "gadget",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			setBasicDamageMultiplier(getPlayerEntity(), "melee", 0)
		end,
		ondrop_function			= function()
			if(not playerHasPerk("PROTECTION_MELEE")) then
				setBasicDamageMultiplier(getPlayerEntity(), "melee", getInternalVariableValue(getPlayerEntity(), "melee_multiplier", "value_float"))
			end
		end,
		excludes_relics_desc	= "It can't be used along with Riot Shield relic or other melee proof shields",
		excludes_relics_tags	= "melee_proof_shield",
	},
	{
		name 					= "riot_shield",
		ui_name					= "Riot Shield",
		categories				= "health",
		effect_desc				= "Protects you against melee and slice damage.",
		effect_desc_short		= "Protects you against melee and slice damage.",
        precursor_types			= "gadget",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			setBasicDamageMultiplier(getPlayerEntity(), "melee", 0)
			setBasicDamageMultiplier(getPlayerEntity(), "slice", 0)
		end,
		ondrop_function			= function()
			setBasicDamageMultiplier(getPlayerEntity(), "slice", getInternalVariableValue(getPlayerEntity(), "slice_multiplier", "value_float"))
			if(not playerHasPerk("PROTECTION_MELEE")) then
				setBasicDamageMultiplier(getPlayerEntity(), "melee", getInternalVariableValue(getPlayerEntity(), "melee_multiplier", "value_float"))
			end
		end,
		excludes_relics_desc	= "It can't be used along with Rubber or Steel Shield relics or other slice / melee proof shields",
		excludes_relics_tags	= "slice_proof_shield,melee_proof_shield",
		combine_exclusive		= true,
		combine_requires_relics	= "rubber_shield,steel_shield",
	},
	{
		name 					= "poison_gem",
		ui_name					= "Poison Gem",
		categories				= "projectiles,materials",
		effect_desc				= "Spells you cast that explode on impact have 30% chance of releasing poison on impact, but you \ntake 25% more damage from this material.",
		effect_desc_short		= "Your spells release poison.",
        precursor_types			= "elemental_gem",
		upgrade_effect_desc		= "For each upgrade level, effect chance is increased by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("poison_gem") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 30 + (5 * getRelicLevel("poison_gem"))
				if(chance >= 1 and chance <= odds) then
					attachLoadEntityOnExplosiontoProjectile(projectile_entity_id, getLoadOnExplosion("launch_elemental_particles_poison").path, "spark_purple_bright")
				end
			end
		end,
		ondamaged_function		= function(damage, message, entity_thats_responsible, is_fatal, projectile_thats_responsible )
			if(isRelicOwned("poison_gem") >= 1) then
				if(string.find(message, "damage from material") and string.find(message, "Poison")) then
					GamePrint("Relic penalty: 25% more poison damage taken")
					local damage_penalty = (damage * 25 / 100)
					EntityInflictDamage( getPlayerEntity(), damage_penalty, "DAMAGE_MATERIAL", "Poison Gem damage penalty", "NONE", 0, 0)
				end
			end
		end,
	},
	{
		name 					= "fools_gold",
		ui_name					= "Fool's Gold",
		categories				= "gold,health",
		effect_desc				= "Provided you are at full health and the current wand you're holding has maximum mana \nthat is equivalent to at least twice your current health, you earn 1 gold periodically.",
		effect_desc_short		= "Earn gold overtime.",
        precursor_types			= "effect_stone,magic_artifact",
		upgrade_effect_desc		= "For each upgrade level, gold earned is increased.",
		is_upgradable 			= true,
		overtimeslow_function	= function()
			if(isRelicOwned("fools_gold") >= 1) then
				local mana_max = getCurrentWandManaMax()
				local hp_treshold = (getPlayerHealth() * 25) * 2
				local hp_full = getPlayerMaxHealth() - getPlayerHealth()
				--GamePrint("### mana_max: " .. mana_max .. " | hp threshold: " .. hp_treshold .. " | max hp - current hp: " .. hp_full)
				if((mana_max ~= 0) and (mana_max >= hp_treshold) and (hp_full == 0)) then
					addPlayerGold(1 + (1 * getRelicLevel("fools_gold")))
				end
			end
		end,
	},
	{
		name 					= "replicating_cap",
		ui_name					= "Replicating Cap",
		categories				= "projectiles,enemies",
		effect_desc				= "When you get hit by an enemy projectile, you have a 30% chance of shooting \na copy of it when you cast your spells.",
		effect_desc_short		= "You cast copies of enemy projectiles that hit you.",
        precursor_types			= "magic_artifact",
		upgrade_effect_desc		= "For each upgrade level, effect chance is increased by 5% and additional projectiles are shot.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("replicating_cap") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 30 + (5 * getRelicLevel("replicating_cap"))
				if(chance >= 1 and chance <= odds) then
					if(getRelicLevel("replicating_cap") ~= 0) then
						for i=1,getRelicLevel("replicating_cap") do
							local x, y = EntityGetTransform(projectile_entity_id)
							shootProjectileBasedonAnotherProjectile(projectile_entity_id, getInternalVariableValue(getPlayerEntity(), "enemy_projectile_definition", "value_string"), x, y, 100)
						end
					end
				end
			end
		end,
		ondamaged_function		= function(damage, message, entity_thats_responsible, is_fatal, projectile_thats_responsible )
			if(message == "$damage_projectile") then
				if(isRelicOwned("replicating_cap") >= 1) then
					setInternalVariableValue(getPlayerEntity(), "enemy_projectile_definition", "value_string", getProjectileDefinitionPath(projectile_thats_responsible))
				end
			end
		end,
	},
	{
		name 					= "seismic_crystal",
		ui_name					= "Seismic Crystal",
		categories				= "projectiles",
		effect_desc				= "Most spells you cast have a 25% chance of causing small cave-ins when they hit something solid.",
		effect_desc_short		= "Your spells cause cave-ins.",
        precursor_types			= "effect_crystal",
		upgrade_effect_desc		= "For each upgrade level, effect chance is increased by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("seismic_crystal") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 25 + (5 * getRelicLevel("seismic_crystal"))
				if(chance >= 1 and chance <= odds) then
					local level = getRelicLevel("seismic_crystal")
					local level_str = ""
					if(level >= 0 and level <= 3) then
						level_str = "1"
					elseif(level >= 3 and level <= 6) then
						level_str = "2"
					elseif(level >= 6 and level <= 10) then
						level_str = "3"
					end
					attachLoadEntityOnExplosiontoProjectile(projectile_entity_id, getLoadOnExplosion("earthquake_level_" .. level_str).path, "sand")
				end
			end
		end,
	},
	{
		name 					= "tovarish_hammer",
		ui_name					= "Tovarish Hammer",
		categories				= "projectiles",
		effect_desc				= "Spells that you cast when you're moving left deal 3x more damage.",
		effect_desc_short		= "Bonus damage when moving left.",
        precursor_types			= "gadget",
		upgrade_effect_desc		= "For each upgrade level, bonus damage is increased.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("tovarish_hammer") >= 1) then
				if(getPlayerHorizontalMovement() == "left") then
					multiplyProjectileBaseDamage(projectile_entity_id, 3 + (1 * getRelicLevel("tovarish_hammer")))
				end
			end
		end,
	},
	{
		name 					= "spike_boot",
		ui_name					= "Spike Boot",
		categories				= "player",
		effect_desc				= "When you are falling, stepping on enemies deals melee damage on them. \nIf Earth Core Fragment is also owned, an explosion is produced when \nits effect is active and an enemy is stepped on.",
		effect_desc_short		= "Stepping on enemies deals melee damage on them.",
        precursor_types			= "boot",
		upgrade_effect_desc		= "For each upgrade level, stomp damage is increased.",
		is_upgradable 			= true,
		onpickup_function		= function()
			if(not entityHasCallback(getPlayerEntity(), "stomp_listener")) then -- if no other stomp related relics added required callback, adds it
				--GamePrint(">>> stomp relic picked, stomp callback added...")
				addNewLuaCallback(getPlayerEntity(), "overtime", "mods/ArcaneReliquary/files/scripts/player/stomp_listener.lua", 2, false, "stomp_listener")
			end
			addNewSimplifiedInternalVariable(getPlayerEntity(), "relic_using_stomp_relic") -- for each stomp related relic, a new simplified variable like this is added
		end,
		ondrop_function			= function()
			removeFirstSimplifiedInternalVariable(getPlayerEntity(), "relic_using_stomp_relic") -- removes just the first variable found
			
			if(countSimplifiedInternalVariable(getPlayerEntity(), "relic_using_stomp_relic") == 0) then -- removes stomp callback only if no other relics are using it
				--GamePrint(">>> stomp relic dropped, no other related relics present, removing stomp callback...")
				removeCallback(getPlayerEntity(), "stomp_listener")
			end
		end,
		onenemystomped_function	= function(entity_id, creature_id)
			if(isRelicOwned("spike_boot") >= 1) then
						
				local multiplier = getRelicLevel("spike_boot")
				if(multiplier == 0) then multiplier = 1 end
				local damage_threshold = 0.3 * multiplier
				-- Doubles threshold if improved_stomp_relics variable is present
				if(entityHasSimplifiedInternalVariable(entity_id, "improved_stomp_relics")) then
					damage_threshold = damage_threshold * 2
				end
				
				EntityInflictDamage( creature_id, damage_threshold, "DAMAGE_MELEE", "stomped", "NORMAL", 0, 0)
			end
		end,
	},
	{
		name 					= "rocket_boot",
		ui_name					= "Rocket Boot",
		categories				= "player",
		effect_desc				= "Falling and stepping on enemies causes small explosions.",
		effect_desc_short		= "Stepping on enemies causes explosions.",
        precursor_types			= "boot",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			if(not entityHasCallback(getPlayerEntity(), "stomp_listener")) then -- if no other stomp related relics added required callback, adds it
				--GamePrint(">>> stomp relic picked, stomp callback added...")
				addNewLuaCallback(getPlayerEntity(), "overtime", "mods/ArcaneReliquary/files/scripts/player/stomp_listener.lua", 2, false, "stomp_listener")
			end
			addNewSimplifiedInternalVariable(getPlayerEntity(), "relic_using_stomp_relic") -- for each stomp related relic, a new simplified variable like this is added
		end,
		ondrop_function			= function()
			removeFirstSimplifiedInternalVariable(getPlayerEntity(), "relic_using_stomp_relic") -- removes just the first variable found
			if(countSimplifiedInternalVariable(getPlayerEntity(), "relic_using_stomp_relic") == 0) then -- removes stomp callback only if no other relics are using it
				--GamePrint(">>> stomp relic dropped, no other related relics present, removing stomp callback...")
				removeCallback(getPlayerEntity(), "stomp_listener")
			end
		end,
		onenemystomped_function	= function(entity_id, creature_id)
			if(isRelicOwned("rocket_boot") >= 1) then
				-- Loads explosion on creature coordinates
				local creature_x, creature_y = EntityGetTransform(creature_id)
				
				EntityLoad("mods/ArcaneReliquary/files/entity_definitions/others/small_explosion.xml", creature_x, creature_y)		
			end
		end,
	},
	{
		name 					= "ice_skating_boot",
		ui_name					= "Ice Skating Boot",
		categories				= "player",
		effect_desc				= "Falling and stepping on enemies freezes them.",
		effect_desc_short		= "Stepping on enemies freezes them.",
        precursor_types			= "boot",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			if(not entityHasCallback(getPlayerEntity(), "stomp_listener")) then -- if no other stomp related relics added required callback, adds it
				--GamePrint(">>> stomp relic picked, stomp callback added...")
				addNewLuaCallback(getPlayerEntity(), "overtime", "mods/ArcaneReliquary/files/scripts/player/stomp_listener.lua", 2, false, "stomp_listener")
			end
			addNewSimplifiedInternalVariable(getPlayerEntity(), "relic_using_stomp_relic") -- for each stomp related relic, a new simplified variable like this is added
		end,
		ondrop_function			= function()
			removeFirstSimplifiedInternalVariable(getPlayerEntity(), "relic_using_stomp_relic") -- removes just the first variable found
			if(countSimplifiedInternalVariable(getPlayerEntity(), "relic_using_stomp_relic") == 0) then -- removes stomp callback only if no other relics are using it
				--GamePrint(">>> stomp relic dropped, no other related relics present, removing stomp callback...")
				removeCallback(getPlayerEntity(), "stomp_listener")
			end
		end,
		onenemystomped_function	= function(entity_id, creature_id)
			if(isRelicOwned("ice_skating_boot") >= 1) then
				-- Loads audiovisual feedback
				local creature_x, creature_y = EntityGetTransform(creature_id)
				GamePlaySound( "data/audio/Desktop/projectiles.snd", "player_projectiles/shield/block", creature_x, creature_y)
				EntityLoad("mods/ArcaneReliquary/files/entity_definitions/particle_explosions/stomp_explosion_blue.xml", creature_x, creature_y)
				
				-- Applies effect on creature found
				LoadGameEffectEntityTo( creature_id, "data/entities/misc/effect_frozen.xml" )		
			end
		end,
	},
	{
		name 					= "electrified_boot",
		ui_name					= "Electrified Boot",
		categories				= "player",
		effect_desc				= "Falling and stepping on enemies electrocutes them.",
		effect_desc_short		= "Stepping on enemies electrocutes them.",
        precursor_types			= "boot",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			if(not entityHasCallback(getPlayerEntity(), "stomp_listener")) then -- if no other stomp related relics added required callback, adds it
				--GamePrint(">>> stomp relic picked, stomp callback added...")
				addNewLuaCallback(getPlayerEntity(), "overtime", "mods/ArcaneReliquary/files/scripts/player/stomp_listener.lua", 2, false, "stomp_listener")
			end
			addNewSimplifiedInternalVariable(getPlayerEntity(), "relic_using_stomp_relic") -- for each stomp related relic, a new simplified variable like this is added
		end,
		ondrop_function			= function()
			removeFirstSimplifiedInternalVariable(getPlayerEntity(), "relic_using_stomp_relic") -- removes just the first variable found
			
			if(countSimplifiedInternalVariable(getPlayerEntity(), "relic_using_stomp_relic") == 0) then -- removes stomp callback only if no other relics are using it
				--GamePrint(">>> stomp relic dropped, no other related relics present, removing stomp callback...")
				removeCallback(getPlayerEntity(), "stomp_listener")
			end
		end,
		onenemystomped_function	= function(entity_id, creature_id)
			if(isRelicOwned("electrified_boot") >= 1) then
				-- Loads audiovisual feedback
				local creature_x, creature_y = EntityGetTransform(creature_id)
				GamePlaySound( "data/audio/Desktop/projectiles.snd", "player_projectiles/shield/block", creature_x, creature_y)
				EntityLoad("mods/ArcaneReliquary/files/entity_definitions/particle_explosions/stomp_explosion_blue.xml", creature_x, creature_y)
				
				-- Applies effect on creature found
				LoadGameEffectEntityTo( creature_id, "data/entities/misc/effect_electricity.xml" )
			end
		end,
	},
	{
		name 					= "ultra_chaos_die",
		ui_name					= "Ultra Chaos Die",
		categories				= "projectiles",
		effect_desc				= "Most of your spells have a 25% chance of exploding into a barrage of \nrandom spells when they explode or hit something.",
		effect_desc_short		= "Your spells release random spells on impact.",
        precursor_types			= "magic_artifact,device",
		upgrade_effect_desc		= "For each upgrade level, effect chance is increased by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("ultra_chaos_die") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 25 + (5 * getRelicLevel("ultra_chaos_die"))
				if(chance >= 1 and chance <= odds) then
					attachLoadEntityOnExplosiontoProjectile(projectile_entity_id, getLoadOnExplosion("launch_random_projectiles").path, "material_confusion")
				end
			end
		end,
	},
	{
		name 					= "water_poison_flask",
		ui_name					= "Water Poison Flask",
		categories				= "projectiles",
		effect_desc				= "Your spells have a 50% chance of making enemies hit by them vulnerable to water, but \nif you attack a creature that is naturally vulnerable to water, you take damage equivalent to the base damage of the spell you shot.",
		effect_desc_short		= "Your spells make enemies vulnerable to water.",
        precursor_types			= "flask",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("water_poison_flask") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 50 + (5 * getRelicLevel("water_poison_flask"))
				if(chance >= 1 and chance <= odds) then
					attachLoadEffectOntoEntitiestoProjectile(projectile_entity_id, getCustomEffect("effect_make_vulnerable_to_water").path, "water")
				end
			end
		end,
		onenemydamaged_function	= function(damage, message, entity_thats_responsible, is_fatal, projectile_thats_responsible )
			if(isRelicOwned("water_poison_flask") >= 1) then
				if(message == "$damage_projectile" or message == "$damage_explosion") then
					if (creatureIsDamagedByMaterial(GetUpdatedEntityID(), "water") and not entityHasSimplifiedInternalVariable(GetUpdatedEntityID(), "water_damage")) then
						GamePrint("Relic penalty: damaged for attacking enemy naturally vulnerable to water")
						EntityInflictDamage( getPlayerEntity(), damage, "DAMAGE_PROJECTILE", "Water Poison Flask damage penalty", "NONE", 0, 0)
						--local hp = getPlayerHealth()
						--setPlayerHealth(hp / 2)
					end
				end
			end
		end,
	},
	{
		name 					= "fire_poison_flask",
		ui_name					= "Fire Poison Flask",
		categories				= "projectiles",
		effect_desc				= "Your spells have a 50% chance of making enemies a lot more vulnerable to fire damage, but \nif you attack a creature that is naturally resistant to fire, you take damage equivalent to the base damage of the spell you shot.",
		effect_desc_short		= "Your spells increase enemy vulnerability to fire.",
        precursor_types			= "flask",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("fire_poison_flask") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 50 + (5 * getRelicLevel("fire_poison_flask"))
				if(chance >= 1 and chance <= odds) then
					attachLoadEffectOntoEntitiestoProjectile(projectile_entity_id, getCustomEffect("effect_make_vulnerable_to_fire").path, "spark")
				end
			end
		end,
		onenemydamaged_function	= function(damage, message, entity_thats_responsible, is_fatal, projectile_thats_responsible )
			if(isRelicOwned("fire_poison_flask") >= 1) then
				if(message == "$damage_projectile" or message == "$damage_explosion") then
					if (creatureIsResistantToDamageMultiplier(GetUpdatedEntityID(), "fire") and not entityHasSimplifiedInternalVariable(GetUpdatedEntityID(), "more_fire_damage")) then
						GamePrint("Relic penalty: damaged for attacking fire resistant enemy")
						EntityInflictDamage( getPlayerEntity(), damage, "DAMAGE_PROJECTILE", "Fire Poison Flask damage penalty", "NONE", 0, 0)
						--local hp = getPlayerHealth()
						--setPlayerHealth(hp / 2)
					end
				end
			end
		end,
	},
	{
		name 					= "blood_poison_flask",
		ui_name					= "Blood Poison Flask",
		categories				= "projectiles",
		effect_desc				= "Your spells have a 50% chance of making enemies hit by them vulnerable to red blood.",
		effect_desc_short		= "Your spells make enemies vulnerable to red blood.",
        precursor_types			= "flask",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("blood_poison_flask") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 50 + (5 * getRelicLevel("blood_poison_flask"))
				if(chance >= 1 and chance <= odds) then
					attachLoadEffectOntoEntitiestoProjectile(projectile_entity_id, getCustomEffect("effect_make_vulnerable_to_blood").path, "blood")
				end
			end
		end,
	},
	{
		name 					= "bone",
		ui_name					= "Bone",
		categories				= "projectiles",
		effect_desc				= "Your spells have a 50% chance of making enemies hit by them a lot more vulnerable to melee damage.",
		effect_desc_short		= "Your spells make enemies vulnerable to melee damage.",
        precursor_types			= "gadget",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("bone") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 50 + (5 * getRelicLevel("bone"))
				if(chance >= 1 and chance <= odds) then
					attachLoadEffectOntoEntitiestoProjectile(projectile_entity_id, getCustomEffect("effect_make_vulnerable_to_melee").path, "spark_white")
				end
			end
		end,
	},
	{
		name 					= "pizza_cutter",
		ui_name					= "Pizza Cutter",
		categories				= "projectiles",
		effect_desc				= "Your spells have a 50% chance of making enemies hit by them a lot more vulnerable to slice damage.",
		effect_desc_short		= "Your spells make enemies vulnerable to slice damage.",
        precursor_types			= "gadget",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("pizza_cutter") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 50 + (5 * getRelicLevel("pizza_cutter"))
				if(chance >= 1 and chance <= odds) then
					attachLoadEffectOntoEntitiestoProjectile(projectile_entity_id, getCustomEffect("effect_make_vulnerable_to_slice").path, "spark_green")
				end
			end
		end,
	},
	{
		name 					= "radioactive_poison_flask",
		ui_name					= "Radioactive Poison Flask",
		categories				= "projectiles",
		effect_desc				= "Your spells have a 50% chance of making enemies hit by them vulnerable to toxic sludge.",
		effect_desc_short		= "Your spells make enemies vulnerable to toxic sludge.",
        precursor_types			= "flask",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("radioactive_poison_flask") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 50 + (5 * getRelicLevel("radioactive_poison_flask"))
				if(chance >= 1 and chance <= odds) then
					attachLoadEffectOntoEntitiestoProjectile(projectile_entity_id, getCustomEffect("effect_make_vulnerable_to_radioactive_liquid").path, "radioactive_liquid")
				end
			end
		end,
	},
	{
		name 					= "molecular_agitator",
		ui_name					= "Molecular Agitator",
		categories				= "projectiles",
		effect_desc				= "Your spells have a 50% chance of phasing through solid terrain.",
		effect_desc_short		= "Your spells phase through terrain.",
        precursor_types			= "gadget,device",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("molecular_agitator") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 50 + (5 * getRelicLevel("molecular_agitator"))
				if(chance >= 1 and chance <= odds) then
					makeProjectilePhaseThroughTerrain(projectile_entity_id)
				end
			end
		end,
	},
	{
		name 					= "phantom_essence",
		ui_name					= "Phantom Essence",
		categories				= "materials",
		effect_desc				= "A magical essence that allows you to phase through metallic and wooden \nwalls and platforms (use it carefully, you might get stuck!).",
		effect_desc_short		= "You phase through metallic and wooden walls.",
        precursor_types			= "flask",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			LoadGameEffectEntityTo( getPlayerEntity(), getCustomEffect("effect_field_phase").path )
		end,
		ondrop_function			= function()
			local player_children = EntityGetAllChildren(getPlayerEntity())
			if player_children == nil then return end
			for i,child in ipairs(player_children) do
				if(EntityGetName(child) == "field_phase") then
					EntityKill(child)
					break
				end
			end
		end,
	},
	{
		name 					= "medusas_medalion",
		ui_name					= "Medusas's Medalion",
		categories				= "projectiles",
		effect_desc				= "Most enemies have a 20% chance of turning into rock when hit by your spells.",
		effect_desc_short		= "Your spells petrify enemies.",
        precursor_types			= "magic_artifact",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("medusas_medalion") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 20 + (5 * getRelicLevel("medusas_medalion"))
				if(chance >= 1 and chance <= odds) then
					attachLoadEffectOntoEntitiestoProjectile(projectile_entity_id, getCustomEffect("effect_petrify").path, "cement")
				end
			end
		end,
	},
	{
		name 					= "transmutation_device",
		ui_name					= "Transmutation Device",
		categories				= "projectiles",
		effect_desc				= "Your spells have a 20% chance of turning enemies they hit into a random material puddle.",
		effect_desc_short		= "Your spells turn enemies into random material puddles.",
        precursor_types			= "gadget,device",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("transmutation_device") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 20 + (5 * getRelicLevel("transmutation_device"))
				if(chance >= 1 and chance <= odds) then
					attachLoadEffectOntoEntitiestoProjectile(projectile_entity_id, getCustomEffect("effect_convert_to_random_material").path, "material_confusion")
				end
			end
		end,
	},
	{
		name 					= "solar_artifact",
		ui_name					= "Solar Artifact",
		categories				= "projectiles",
		effect_desc				= "Spells you cast in an upward angle have a 30% chance of exploding and \nreleasing firebombs, provided they fly enough time and don't hit anything. \nIt also synergizes with Earth Core Fragment relic effect.",
		effect_desc_short		= "Spells cast upwards explode into firebombs.",
        precursor_types			= "magic_artifact",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("solar_artifact") >= 1) then
				if((not isPhysicsProjectile(projectile_entity_id))) then
					SetRandomSeed( GameGetFrameNum(), math.random(1,50))
					local chance = Random(1,100)
					local odds = 30 + (5 * getRelicLevel("solar_artifact"))
					if(chance >= 1 and chance <= odds) then
						local projectile_vel_x, projectile_vel_y = getProjectileVelocitiesandGravity(projectile_entity_id)
						if(projectile_vel_y < 0) then
							addNewLuaCallback(projectile_entity_id, "overtime", "mods/ArcaneReliquary/files/scripts/projectiles/explode_and_launch_grenades.lua", Random(5,8), true, "explode_and_launch_grenades")
						end
					end
				end
			end
		end,
		ongroundpound_function	= function( entity_id)
			if(isRelicOwned("solar_artifact") >= 1) then
				LoadGameEffectEntityTo( entity_id, getCustomEffect("effect_shoot_projectiles_above_firebomb").path )
			end
		end,
	},
	{
		name 					= "meteor_fragment",
		ui_name					= "Meteor Fragment",
		categories				= "projectiles",
		effect_desc				= "Spells you cast in a downward angle have a 30% chance of creating \na shockwave, that damages nearby enemies, when they hit something. \nIt also synergizes with Earth Core Fragment relic effect.",
		effect_desc_short		= "Spells cast downwards create shockwaves on the ground.",
        precursor_types			= "magic_artifact",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5% and shockwave reaches longer.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("meteor_fragment") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 30 + (5 * getRelicLevel("meteor_fragment"))
				if(chance >= 1 and chance <= odds) then
					local projectile_vel_x, projectile_vel_y = getProjectileVelocitiesandGravity(projectile_entity_id)
					if(projectile_vel_y > 0) then
						replaceProjectileExplosionSprite(projectile_entity_id, "")
						projectileRemoveBouncing(projectile_entity_id)
						if(not projectileExplodesonImpact(projectile_entity_id)) then
							projectileSetExplosionConfig(projectile_entity_id, 1)
						end
						attachLoadEntityOnExplosiontoProjectile(projectile_entity_id, getLoadOnExplosion("shockwave").path, "spark")
					end
				end
			end
		end,
		ongroundpound_function	= function( entity_id)
			if(isRelicOwned("meteor_fragment") >= 1) then
				local game_effect_comp = GetGameEffectLoadTo( entity_id, "PROTECTION_ALL", true )
				if game_effect_comp ~= nil then
					ComponentSetValue( game_effect_comp, "frames", "40" )
				end
				local x, y = EntityGetTransform(entity_id)
				EntityLoad("mods/ArcaneReliquary/files/entity_definitions/load_on_explosion/shockwave.xml", x, y)
			end
		end,
		ondrop_function			= function()
			removeInternalVariable(getPlayerEntity(), "shockwave_length_bonus")
		end,
		overtime_function		= function()
			local shockwave_length_bonus = getRelicLevel("meteor_fragment")

			if(not entityHasInternalVariableValue(getPlayerEntity(), "shockwave_length_bonus")) then
				addNewInternalVariable(getPlayerEntity(), "shockwave_length_bonus", "value_int", shockwave_length_bonus)
			end
			setInternalVariableValue(getPlayerEntity(), "shockwave_length_bonus", "value_int", shockwave_length_bonus)
		end,
	},
	{
		name 					= "corrosive_blood_essence",
		ui_name					= "Corrosive Blood Essence",
		categories				= "health",
		effect_desc				= "When you take damage, you spill acid blood, that corrodes terrain and damages enemies but not you.",
		effect_desc_short		= "You spill corrosive blood that only damages enemies.",
        precursor_types			= "flask",
		upgrade_effect_desc		= "For each upgrade level, acid blood spilled is increased.",
		is_upgradable 			= true,
		ondamaged_function		= function(damage, message, entity_thats_responsible, is_fatal, projectile_thats_responsible )
			if(isRelicOwned("corrosive_blood_essence") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local x, y = EntityGetTransform(getPlayerEntity())
				for i=1,Random(25,35) do
					local vel_variation = 200
					GameCreateParticle( "corrosive_blood", x, y, 2 + (2 * getRelicLevel("corrosive_blood_essence")), Random(-vel_variation, vel_variation), Random(-vel_variation, vel_variation), false, true)
				end
			end
		end,
	},
	{
		name 					= "imperialist_hat",
		ui_name					= "Imperialist Hat",
		categories				= "projectiles",
		effect_desc				= "Spells that you cast when you're moving right have a 20% chance of dropping gold nuggets along its travel path.",
		effect_desc_short		= "Your spells drop gold when moving right.",
        precursor_types			= "gadget",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("imperialist_hat") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 20 + (5 * getRelicLevel("imperialist_hat"))
				if(chance >= 1 and chance <= odds) then
					if(getPlayerHorizontalMovement() == "right") then
						addNewInternalVariable(projectile_entity_id, "max_nuggets_dropped", "value_int", 15)
						addNewLuaCallback(projectile_entity_id, "overtime", "mods/ArcaneReliquary/files/scripts/projectiles/drop_gold_nugget.lua", 10, false, "drop_gold_nugget")
					end
				end
			end
		end,
	},
	{
		name 					= "sulfur_crystal",
		ui_name					= "Sulfur Crystal",
		categories				= "projectiles",
		effect_desc				= "Spells that you cast have a 30% chance of shooting fireballs downwards along their travel path.",
		effect_desc_short		= "Your spells cast downward fireballs.",
        precursor_types			= "effect_crystal",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%, and when it triggers, fireballs are shot more often.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("sulfur_crystal") >= 1) then
				if(not isPhysicsProjectile(projectile_entity_id)) then
					SetRandomSeed( GameGetFrameNum(), math.random(1,50))
					local chance = Random(1,100)
					local odds = 30 + (5 * getRelicLevel("sulfur_crystal"))
					if(chance >= 1 and chance <= odds) then
						addNewInternalVariable(projectile_entity_id, "projectile_shot_down_definition", "value_string", "mods/ArcaneReliquary/files/entity_definitions/custom_projectiles/fireball_custom.xml")
						addNewInternalVariable(projectile_entity_id, "down_vel_min", "value_int", 165)
						addNewInternalVariable(projectile_entity_id, "down_vel_max", "value_int", 185)
						addNewInternalVariable(projectile_entity_id, "max_projectiles_shot", "value_int", 10)
						addNewLuaCallback(projectile_entity_id, "overtime", "mods/ArcaneReliquary/files/scripts/projectiles/launch_projectile_down.lua", 25 - (2 * getRelicLevel("sulfur_crystal")), false, "launch_projectile_down")
					end
				end
			end
		end,
	},
	{
		name 					= "magmatic_essence",
		ui_name					= "Magmatic Essence",
		categories				= "projectiles",
		effect_desc				= "When you take damage from lava, your spells get a damage bonus equivalent to \nthe damage you took. If you take damage from projectiles, explosions or other materials, the effect is reset.",
		effect_desc_short		= "Taking damage from lava gives you spell damage bonus.",
        precursor_types			= "flask",
		upgrade_effect_desc		= "For each upgrade level, damage boost gained is increased.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("magmatic_essence") >= 1) then
				local lava_damage_bonus = getInternalVariableValue(getPlayerEntity(), "lava_damage_bonus", "value_int") + (1 * getRelicLevel("magmatic_essence"))
				if(lava_damage_bonus > 0) then
					GamePrint("Relic effect: " .. lava_damage_bonus .. " lava damage bonus spell damage")
					addProjectileBaseDamage(projectile_entity_id, lava_damage_bonus)
				end
			end
		end,
		ondamaged_function		= function(damage, message, entity_thats_responsible, is_fatal, projectile_thats_responsible )
			if(isRelicOwned("magmatic_essence") >= 1) then
				if((string.find(message, "damage from material")) and (string.find(message, "lava"))) then
					local lava_damage_bonus = getInternalVariableValue(getPlayerEntity(), "lava_damage_bonus", "value_int")
					setInternalVariableValue(getPlayerEntity(), "lava_damage_bonus", "value_int", lava_damage_bonus + damage)
				elseif(message == "$damage_projectile" or message == "$damage_explosion" or ((string.find(message, "damage from material")) and (not string.find(message, "lava")))) then
					setInternalVariableValue(getPlayerEntity(), "lava_damage_bonus", "value_int", 0)
				end
			end
		end,
	},
	{
		name 					= "lava_gem",
		ui_name					= "Lava Gem",
		categories				= "projectiles,materials",
		effect_desc				= "Spells you cast that explode on impact have a 30% chance of releasing lava on impact, but you \ntake 25% more damage from this material.",
		effect_desc_short		= "Your spells release lava.",
        precursor_types			= "elemental_gem",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("lava_gem") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 30 + (5 * getRelicLevel("lava_gem"))
				if(chance >= 1 and chance <= odds) then
					attachLoadEntityOnExplosiontoProjectile(projectile_entity_id, getLoadOnExplosion("launch_elemental_particles_lava").path, "lava")
				end
			end
		end,
		ondamaged_function		= function(damage, message, entity_thats_responsible, is_fatal, projectile_thats_responsible )
			if(isRelicOwned("lava_gem") >= 1) then
				if(string.find(message, "damage from material") and string.find(message, "lava")) then
					GamePrint("Relic penalty: 25% more lava damage taken")
					local damage_penalty = (damage * 25 / 100)
					EntityInflictDamage( getPlayerEntity(), damage_penalty, "DAMAGE_MATERIAL", "Lava Gem damage penalty", "NONE", 0, 0)
				end
			end
		end,
	},
	{
		name 					= "vitality_essence",
		ui_name					= "Vitality Essence",
		categories				= "enemies,items",
		effect_desc				= "Enemies you kill have a 20% chance of dropping green healing nuggets that slightly restore your \nhp and are capable of extending your maximum health if you are currently at full health.",
		effect_desc_short		= "Killed enemies drop green healing nuggets.",
        precursor_types			= "flask",
		upgrade_effect_desc		= "For each upgrade level, drop chance increases by 5%.",
		is_upgradable 			= true,
		onenemykill_function	= function(damage_type_bit_field, damage_message, entity_thats_responsible, drop_items)
			if(isRelicOwned("vitality_essence") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 20 + (5 * getRelicLevel("vitality_essence"))
				if(chance >= 1 and chance <= odds) then
					local death_x, death_y = EntityGetTransform(GetUpdatedEntityID())
					EntityLoad("mods/ArcaneReliquary/files/entity_definitions/items/healingnugget.xml", death_x, death_y)
				end
			end
		end,
	},
	{
		name 					= "hydrant",
		ui_name					= "Hydrant",
		categories				= "materials",
		effect_desc				= "A magic field that converts fire to water is projected around you.",
		effect_desc_short		= "Fire around you is converted into water.",
        precursor_types			= "gadget",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			LoadGameEffectEntityTo( getPlayerEntity(), getCustomEffect("effect_field_fire_to_water").path )
		end,
		ondrop_function			= function()
			local player_children = EntityGetAllChildren(getPlayerEntity())
			if player_children == nil then return end
			for i,child in ipairs(player_children) do
				if(EntityGetName(child) == "field_fire_to_water") then
					EntityKill(child)
					break
				end
			end
		end,
	},
	{
		name 					= "glacial_ring",
		ui_name					= "Glacial Ring",
		categories				= "materials",
		effect_desc				= "Spells you cast have a 30% chance of creating ice spheres when they hit something.",
		effect_desc_short		= "Your spells create ice spheres.",
        precursor_types			= "ring",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("glacial_ring") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 30 + (5 * getRelicLevel("glacial_ring"))
				if(chance >= 1 and chance <= odds) then
					attachLoadEntityOnExplosiontoProjectile(projectile_entity_id, getLoadOnExplosion("ice_explosion").path, "plasma_fading")
				end
			end
		end,
	},
	
	{
		name 					= "ballistic_boot",
		ui_name					= "Ballistic Boot",
		categories				= "player,projectiles",
		effect_desc				= "When you move up, you shoot firebombs from your feet. \nIt also synergizes with Earth Core Fragment relic effect.",
		effect_desc_short		= "When you move up, firebombs are shot from your feet.",
        precursor_types			= "boot",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()		
			addNewLuaCallback(getPlayerEntity(), "overtime", "mods/ArcaneReliquary/files/scripts/player/shoot_foot_firebomb.lua", 2, false, "shoot_foot_firebomb")
		end,
		ondrop_function			= function()	
			removeCallback(getPlayerEntity(), "shoot_foot_firebomb")
		end,
		ongroundpound_function	= function( entity_id)
			if(isRelicOwned("ballistic_boot") >= 1) then
				LoadGameEffectEntityTo( entity_id, getCustomEffect("effect_shoot_projectiles_above_firebomb_small").path )
			end
		end,
	},
	{
		name 					= "aged_whiskey",
		ui_name					= "Aged Whiskey",
		categories				= "projectiles",
		effect_desc				= "When you are drunk, your spells get a massive critical damage multiplier.",
		effect_desc_short		= "When drunk, your spells get a massive critical damage multiplier.",
        precursor_types			= "gadget",
		upgrade_effect_desc		= "For each upgrade level, critical damage multiplier gained increases.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("aged_whiskey") >= 1) then
				if(GameGetGameEffectCount(getPlayerEntity(), "DRUNK") > 0) then
					local critical_damage_multiplier = 10 + (1 * getRelicLevel("aged_whiskey"))
					GamePrint("Relic effect: x" .. critical_damage_multiplier .. " critical damage multiplier ")
					local chance, damage_multiplier = getProjectileCriticalDamageSettings(projectile_entity_id)
					setProjectileCriticalDamageSettings(projectile_entity_id, chance, damage_multiplier * critical_damage_multiplier)
				end
			end
		end,
	},
	{
		name 					= "thunder_hat",
		ui_name					= "Thunder Hat",
		categories				= "player,projectiles",
		effect_desc				= "When you are falling, you shoot electrobombs from your head. \nIt also synergizes with Earth Core Fragment relic effect.",
		effect_desc_short		= "When you are falling, electrobombs are shot from your head.",
        precursor_types			= "gadget",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			addNewLuaCallback(getPlayerEntity(), "overtime", "mods/ArcaneReliquary/files/scripts/player/shoot_head_electrobombs.lua", 5, false, "shoot_head_electrobombs")
		end,
		ondrop_function			= function()		
			removeCallback(getPlayerEntity(), "shoot_head_electrobombs")
		end,
		ongroundpound_function	= function( entity_id)
			if(isRelicOwned("thunder_hat") >= 1) then
				LoadGameEffectEntityTo( entity_id, getCustomEffect("effect_shoot_projectiles_above_electrobomb").path )
			end
		end,
	},
	{
		name 					= "drilling_hat",
		ui_name					= "Drilling Hat",
		categories				= "materials",
		effect_desc				= "Your head is now capable of punching through solid materials \n(make sure to be wet when using this ability).",
		effect_desc_short		= "You drill through solid materials with your head.",
        precursor_types			= "gadget",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		overtime_function	= function()
			if(isRelicOwned("drilling_hat") >= 1) then
				LoadGameEffectEntityTo( getPlayerEntity(), getCustomEffect("effect_bonk").path )
			end
		end,
	},
	{
		name 					= "red_thunder_stone",
		ui_name					= "Red Thunder Stone",
		categories				= "projectiles",
		effect_desc				= "Spells you cast have a 50% chance of shooting red thunderbolts towards nearby enemies.",
		effect_desc_short		= "Your spells shoot thunderbolts at enemies.",
        precursor_types			= "effect_stone,magic_artifact,device",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("red_thunder_stone") >= 1) then
				if(not isPhysicsProjectile(projectile_entity_id)) then
					SetRandomSeed( GameGetFrameNum(), math.random(1,50))
					local chance = Random(1,100)
					local odds = 50 + (5 * getRelicLevel("red_thunder_stone"))
					if(chance >= 1 and chance <= odds) then
						addBuffVisualIndicator_Projectile(projectile_entity_id, "spark_red")
						addNewInternalVariable(projectile_entity_id, "projectile_definition", "value_string", "mods/ArcaneReliquary/files/entity_definitions/custom_projectiles/thunderbolt_red.xml")
						addNewInternalVariable(projectile_entity_id, "detection_range", "value_int", 100)
						addNewInternalVariable(projectile_entity_id, "projectile_velocity", "value_int", 950)
						addNewInternalVariable(projectile_entity_id, "target_tag", "value_string", "enemy")
						addNewInternalVariable(projectile_entity_id, "max_projectiles_shot", "value_int", 6)
						addNewLuaCallback(projectile_entity_id, "overtime", "mods/ArcaneReliquary/files/scripts/projectiles/target_and_shoot_projectile.lua", 10, false, "target_and_shoot_projectile")
					end
				end
			end
		end,
	},
	
	{
		name 					= "lazuli_butterfly",
		ui_name					= "Lazuli Butterfly",
		categories				= "projectiles",
		effect_desc				= "Your spells have a 50% chance of traveling chaotically once cast, with \na chance of shooting copies of themselves along their paths.",
		effect_desc_short		= "Your spells travel chaotically shooting copies of themselves.",
        precursor_types			= "magic_artifact",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("lazuli_butterfly") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 50 + (5 * getRelicLevel("lazuli_butterfly"))
				if(chance >= 1 and chance <= odds) then
					addBuffVisualIndicator_Projectile(projectile_entity_id, "spark_blue_dark")
					addNewLuaCallback(projectile_entity_id, "overtime", "mods/ArcaneReliquary/files/scripts/projectiles/truly_chaotic_projectile.lua", 1, false, "truly_chaotic_projectile")
				end
			end
		end,
	},
	{
		name 					= "blessed_shield",
		ui_name					= "Blessed Shield",
		categories				= "projectiles,player,status_effects",
		effect_desc				= "You are now protected against disarming, bleeding, freezing, electrocution, \npolymorphing, slapping and other effects applied by buffed enemies on you with their projectiles.",
		effect_desc_short		= "Buffed enemies no longer inflict effects on you with their projectiles.",
        precursor_types			= "magic_artifact",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			addNewSimplifiedInternalVariable(getPlayerEntity(), "effects_immune")
		end,
		ondrop_function			= function()
			removeSimplifiedInternalVariable(getPlayerEntity(), "effects_immune")
		end,
		excludes_relics			= "blessed_helmet",
		excludes_relics_desc	= "It can't be used along with other buffed creatures projectile effects immmunity related relics",
		excludes_relics_tags	= "buffed_creatures_effects_immunity",
	},
	{
		name 					= "freedom_ring",
		ui_name					= "Freedom Ring",
		categories				= "projectiles",
		effect_desc				= "Your spells have a 30% chance of inducing a sequence of explosions \non enemies hit by them.",
		effect_desc_short		= "Your spells induce explosion sequences on enemies.",
        precursor_types			= "ring",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("freedom_ring") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 30 + (5 * getRelicLevel("freedom_ring"))
				if(chance >= 1 and chance <= odds) then
					attachLoadEffectOntoEntitiestoProjectile(projectile_entity_id, getCustomEffect("effect_exploding").path, "spark_yellow")
				end
			end
		end,
	},
	{
		name 					= "security_probe",
		ui_name					= "Security Probe",
		categories				= "companions",
		effect_desc				= "A security probe companion that shoots laser beams joins you.",
		effect_desc_short		= "A laser beam shooting companion.",
        precursor_types			= "companion",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			attachCompaniontoEntity(getPlayerEntity(), "security_probe", "mortal")
		end,
		ondrop_function			= function()
			detachCompanionfromEntity(getPlayerEntity(), "security_probe")
		end,
		requires_relic			= "alkaline_battery",
	},
	{
		name 					= "alkaline_battery",
		ui_name					= "Alkaline Battery",
		categories				= "projectiles",
		effect_desc				= "Enhances electrically charged spells with additional electricity damage \nand is also capable of powering robotic companions.",
		effect_desc_short		= "Enhances electrically charged spells.",
        precursor_types			= "gadget",
		upgrade_effect_desc		= "For each upgrade level, electricity damage added is increased.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("alkaline_battery") >= 1) then
				if(projectileHasDamageType(projectile_entity_id, "electricity")) then
					addProjectileDamageByType(projectile_entity_id, "electricity", 0.3 + (0.3 * getRelicLevel("alkaline_battery")))
				end
			end
		end,
	},
	{
		name 					= "blaze_pearl",
		ui_name					= "Blaze Pearl",
		categories				= "projectiles",
		effect_desc				= "Enhances fire imbued spells with additional fire damage.",
		effect_desc_short		= "Enhances fire imbued spells.",
        precursor_types			= "magic_artifact",
		upgrade_effect_desc		= "For each upgrade level, fire damage added is increased.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("blaze_pearl") >= 1) then
				if(projectileHasDamageType(projectile_entity_id, "fire")) then
					addProjectileDamageByType(projectile_entity_id, "fire", 0.3 + (0.3 * getRelicLevel("blaze_pearl")))
				end
			end
		end,
	},
	{
		name 					= "flaming_skull",
		ui_name					= "Flaming Skull",
		categories				= "companions",
		effect_desc				= "A friendly flaming skull that shoots small fireballs joins you.",
		effect_desc_short		= "A fireball shooting companion.",
        precursor_types			= "companion",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			attachCompaniontoEntity(getPlayerEntity(), "flaming_skull", "mortal")
		end,
		ondrop_function			= function()
			detachCompanionfromEntity(getPlayerEntity(), "flaming_skull")
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "enchanted_skull,blaze_pearl",
		requires_relic			= "bone",
	},
	{
		name 					= "gelid_necklace",
		ui_name					= "Gelid Necklace",
		categories				= "projectiles",
		effect_desc				= "Your spells that include freeze damage receive ice damage bonus.",
		effect_desc_short		= "Enhances your freeze charged spells.",
        precursor_types			= "necklace",
		upgrade_effect_desc		= "For each upgrade level, ice damage added is increased.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("gelid_necklace") >= 1) then
				if(projectileHasDamageType(projectile_entity_id, "ice")) then
					addProjectileDamageByType(projectile_entity_id, "ice", 0.3 + (0.3 * getRelicLevel("gelid_necklace")))
				end
			end
		end,
	},
	{
		name 					= "sentient_crystal",
		ui_name					= "Sentient Crystal",
		categories				= "companions",
		effect_desc				= "A sentient crystal that shoots freezebombs joins you.",
		effect_desc_short		= "A freezebomb shooting companion.",
        precursor_types			= "companion",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			attachCompaniontoEntity(getPlayerEntity(), "sentient_crystal", "mortal")
		end,
		ondrop_function			= function()
			detachCompanionfromEntity(getPlayerEntity(), "sentient_crystal")
		end,
	},
	{
		name 					= "tiny_grenadier",
		ui_name					= "Tiny Grenadier",
		categories				= "companions",
		effect_desc				= "A small robot that shoots green grenades joins you.",
		effect_desc_short		= "A green grenade shooting companion.",
        precursor_types			= "companion",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			attachCompaniontoEntity(getPlayerEntity(), "tiny_grenadier", "mortal")
		end,
		ondrop_function			= function()
			detachCompanionfromEntity(getPlayerEntity(), "tiny_grenadier")
		end,
		requires_relic			= "alkaline_battery",
	},
	{
		name 					= "generic_poison_flask",
		ui_name					= "Generic Poison Flask",
		categories				= "materials,player,enemies",
		effect_desc				= "Damage dealt directly by harmful materials (not including status effects) is 75% more effective. \nThis is valid to your enemies, and you.",
		effect_desc_short		= "You and your enemies take a lot more material damage.",
        precursor_types			= "flask",
		upgrade_effect_desc		= "For each upgrade level, 2% more damage is added.",
		is_upgradable 			= true,
		ondamaged_function		= function(damage, message, entity_thats_responsible, is_fatal, projectile_thats_responsible )
			if(isRelicOwned("generic_poison_flask") >= 1) then
				if(string.find(message, "damage from material")) then
					local damage_penalty = (damage * 75 + (2 * getRelicLevel("generic_poison_flask"))) / 100
					GamePrint("Relic penalty: 75% more material damage taken")
					EntityInflictDamage( getPlayerEntity(), damage_penalty, "DAMAGE_MATERIAL", "Generic Poison Flask damage penalty", "NONE", 0, 0)
				end
			end
		end,
		onenemydamaged_function	= function(damage, message, entity_thats_responsible, is_fatal, projectile_thats_responsible )
			if(isRelicOwned("generic_poison_flask") >= 1) then
				if(string.find(message, "damage from material")) then
					local damage_penalty = (damage * 75 + (2 * getRelicLevel("generic_poison_flask"))) / 100
					EntityInflictDamage( GetUpdatedEntityID(), damage_penalty, "DAMAGE_MATERIAL", "Generic Poison Flask damage penalty", "NONE", 0, 0)
				end
			end
		end,
	},
	{
		name 					= "halo_flask",
		ui_name					= "Halo Flask",
		categories				= "materials,player",
		effect_desc				= "Each time you take direct damage from harmful materials (not including status effects), you earn 5 gold.",
		effect_desc_short		= "Earn gold when damaged by materials.",
        precursor_types			= "flask",
		upgrade_effect_desc		= "For each upgrade level, extra gold is added.",
		is_upgradable 			= true,
		ondamaged_function		= function(damage, message, entity_thats_responsible, is_fatal, projectile_thats_responsible )
			if(isRelicOwned("halo_flask") >= 1) then
				if(string.find(message, "damage from material")) then
					addPlayerGold(5 + (1 * getRelicLevel("halo_flask")))
				end
			end
		end,
	},
	{
		name 					= "hawk_feather",
		ui_name					= "Hawk Feather",
		categories				= "items",
		effect_desc				= "When you kill an enemy with slice damage, there is a 20% chance they might drop random eggs.",
		effect_desc_short		= "Spawn eggs on kills.",
        precursor_types			= "magic_artifact",
		upgrade_effect_desc		= "For each upgrade level, drop chance is increased by 5%.",
		is_upgradable 			= true,
		onenemykill_function	= function(damage_type_bit_field, damage_message, entity_thats_responsible, drop_items)
			if(isRelicOwned("hawk_feather") >= 1) then
				if(damage_message == "$damage_slice") then
					SetRandomSeed( GameGetFrameNum(), math.random(1,50))
					local chance = Random(1,100)
					local odds = 20 + (5 * getRelicLevel("hawk_feather"))
					if(chance >= 1 and chance <= odds) then
						local x, y = EntityGetTransform(GetUpdatedEntityID())
						local amount = Random(1,3)
						for i=0,amount do
							local egg_chance = Random(1,8)
							if(egg_chance == 1) then
								EntityLoad("data/entities/items/pickup/egg_worm.xml", x, y)
							elseif(egg_chance == 2) then
								EntityLoad("data/entities/items/pickup/egg_spiders.xml", x, y)
							elseif(egg_chance == 3) then
								EntityLoad("data/entities/items/pickup/egg_slime.xml", x, y)
							elseif(egg_chance == 4) then
								EntityLoad("data/entities/items/pickup/egg_red.xml", x, y)
							elseif(egg_chance == 5) then
								EntityLoad("data/entities/items/pickup/egg_purple.xml", x, y)
							elseif(egg_chance == 6) then
								EntityLoad("data/entities/items/pickup/egg_monster.xml", x, y)
							elseif(egg_chance == 7) then
								EntityLoad("data/entities/items/pickup/egg_hollow.xml", x, y)
							elseif(egg_chance == 8) then
								EntityLoad("data/entities/items/pickup/egg_fire.xml", x, y)
							end
						end
					end
				end
			end
		end,
	},
	{
		name 					= "friendly_slime",
		ui_name					= "Friendly Slime",
		categories				= "companions",
		effect_desc				= "A friendly slime that shoots slime bolts joins you.",
		effect_desc_short		= "A slime bolt shooting companion.",
        precursor_types			= "companion",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			attachCompaniontoEntity(getPlayerEntity(), "friendly_slime", "mortal")
		end,
		ondrop_function			= function()
			detachCompanionfromEntity(getPlayerEntity(), "friendly_slime")
		end,
		requires_relic			= "ball_of_goo",
	},
	{
		name 					= "ball_of_goo",
		ui_name					= "Ball of Goo",
		categories				= "projectiles",
		effect_desc				= "Your spells have a 25% chance of exploding into a barrage of slime bolts when they hit an obstacle or explode.",
		effect_desc_short		= "Your spells release slime bolts on impact.",
        precursor_types			= "magic_artifact",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("ball_of_goo") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 25 + (5 * getRelicLevel("ball_of_goo"))
				if(chance >= 1 and chance <= odds) then
					addBuffVisualIndicator_Projectile(projectile_entity_id, "radioactive_liquid_fading")
					attachLoadEntityOnExplosiontoProjectile(projectile_entity_id, getLoadOnExplosion("launch_circular_projectiles_slimebolts").path)
				end
			end
		end
	},
	{
		name 					= "overloaded_skull",
		ui_name					= "Overloaded Skull",
		categories				= "companions",
		effect_desc				= "A friendly skull that shoots electrocuting bombs joins you.",
		effect_desc_short		= "An electrocuting bombs shooting companion.",
        precursor_types			= "companion",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			attachCompaniontoEntity(getPlayerEntity(), "overloaded_skull", "mortal")
		end,
		ondrop_function			= function()
			detachCompanionfromEntity(getPlayerEntity(), "overloaded_skull")
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "enchanted_skull,charged_necklace",
		requires_relic			= "bone",
	},
	{
		name 					= "shotgun_droid",
		ui_name					= "Shotgun Droid",
		categories				= "companions",
		effect_desc				= "A small robot that shoots shotgun pellets joins you.",
		effect_desc_short		= "A shotgun pellet shooting companion.",
        precursor_types			= "companion",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			attachCompaniontoEntity(getPlayerEntity(), "shotgun_droid", "mortal")
		end,
		ondrop_function			= function()
			detachCompanionfromEntity(getPlayerEntity(), "shotgun_droid")
		end,
		requires_relic			= "alkaline_battery",
	},
	{
		name 					= "rifle_droid",
		ui_name					= "Rifle Droid",
		categories				= "companions",
		effect_desc				= "A small robot that shoots machine gun rounds joins you.",
		effect_desc_short		= "A rifle bullet shooting companion.",
        precursor_types			= "companion",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			attachCompaniontoEntity(getPlayerEntity(), "rifle_droid", "mortal")
		end,
		ondrop_function			= function()
			detachCompanionfromEntity(getPlayerEntity(), "rifle_droid")
		end,
		requires_relic			= "alkaline_battery",
	},
	{
		name 					= "hopper_crown",
		ui_name					= "Hopper Crown",
		categories				= "projectiles",
		effect_desc				= "Spells you cast have a 50% chance of forcing enemies up when they get \nhit (does not work on flying creatures).",
		effect_desc_short		= "Your spells force enemies up.",
        precursor_types			= "magic_artifact",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("hopper_crown") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 50 + (5 * getRelicLevel("hopper_crown"))
				if(chance >= 1 and chance <= odds) then
					attachLoadEffectOntoEntitiestoProjectile(projectile_entity_id, getCustomEffect("effect_force_up").path, "material_confusion")
				end
			end
		end,
	},
	{
		name 					= "slapper_stool",
		ui_name					= "Slapper Stool",
		categories				= "projectiles",
		effect_desc				= "Spells you cast have a 30% chance of applying the slap effect on enemies hit by them, causing \nthem to take damage overtime and be forced at random directions.",
		effect_desc_short		= "Your spells induce slapping on enemies.",
        precursor_types			= "magic_artifact",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("slapper_stool") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 30 + (5 * getRelicLevel("slapper_stool"))
				if(chance >= 1 and chance <= odds) then
					attachLoadEffectOntoEntitiestoProjectile(projectile_entity_id, getCustomEffect("effect_slap").path, "material_confusion")
				end
			end
		end,
	},
	{
		name 					= "olive_oil",
		ui_name					= "Olive Oil",
		categories				= "projectiles",
		effect_desc				= "When you're covered in oil, your spells get an additional 90% chance of inflicting critical \ndamage, but each time you cast a spell there is a 30% chance your wand will slip from your hands.",
		effect_desc_short		= "Massive critical damage when oiled.",
        precursor_types			= "gadget",
		upgrade_effect_desc		= "For each upgrade level, wand slip chance is decreased by 2%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("olive_oil") >= 1) then
				if(GameGetGameEffectCount(getPlayerEntity(), "OILED") > 0) then
					-- bonus
					local chance, damage_multiplier = getProjectileCriticalDamageSettings(projectile_entity_id)
					setProjectileCriticalDamageSettings(projectile_entity_id, (chance + 90), damage_multiplier)
					-- penalty
					SetRandomSeed( GameGetFrameNum(), math.random(1,50))
					local chance = Random(1,100)
					local odds = 30 - (2 * getRelicLevel("olive_oil"))
					if(chance >= 1 and chance <= odds) then
						GamePrint("Relic penalty: current wand slipped from your hands")
						dropCurrentlyEquippedWand()
					end
				end
			end
		end,
	},
	{
		name 					= "magnet",
		ui_name					= "Magnet",
		categories				= "projectiles",
		effect_desc				= "Spells you cast have 30% chance of magnetizing enemies they hit, causing \nthem to attract objects and projectiles in the vicinity.",
		effect_desc_short		= "Your spells magnetize enemies.",
        precursor_types			= "gadget",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("magnet") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 30 + (5 * getRelicLevel("magnet"))
				if(chance >= 1 and chance <= odds) then
					addBuffVisualIndicator_Projectile(projectile_entity_id, "spark_blue")
					attachLoadEffectOntoEntitiestoProjectile(projectile_entity_id, getCustomEffect("effect_attraction").path, "spark_red")
				end
			end
		end,
	},
	{
		name 					= "ferrofluid_flask",
		ui_name					= "Ferrofluid Flask",
		categories				= "projectiles",
		effect_desc				= "Each time you cast a spell, there's a 50% chance it will be magnetized, causing \nit to attract objects and projectiles other than your own in the vicinity.",
		effect_desc_short		= "Magnetized spells.",
        precursor_types			= "flask",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("ferrofluid_flask") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 50 + (5 * getRelicLevel("ferrofluid_flask"))
				if(chance >= 1 and chance <= odds) then
					addBuffVisualIndicator_Projectile(projectile_entity_id, "cement")
					addNewLuaCallback(projectile_entity_id, "overtime", "mods/ArcaneReliquary/files/scripts/projectiles/attract_objects_and_projectiles.lua", 2, false, "attract_objects_and_projectiles")
				end
			end
		end,
	},
	{
		name 					= "oil_gem",
		ui_name					= "Oil Gem",
		categories				= "projectiles,materials",
		effect_desc				= "Spells you cast that explode on impact have a 20% chance of releasing oil on impact.",
		effect_desc_short		= "Your spells release oil.",
        precursor_types			= "elemental_gem",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("oil_gem") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 20 + (5 * getRelicLevel("oil_gem"))
				if(chance >= 1 and chance <= odds) then
					attachLoadEntityOnExplosiontoProjectile(projectile_entity_id, getLoadOnExplosion("launch_elemental_particles_oil").path, "oil")
				end
			end
		end,
	},
	{
		name 					= "berserkium_gem",
		ui_name					= "Berserkium Gem",
		categories				= "projectiles,materials",
		effect_desc				= "Spells you cast that explode on impact have a 20% chance of releasing berserkium on impact.",
		effect_desc_short		= "Your spells release berserkium.",
        precursor_types			= "elemental_gem",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("berserkium_gem") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 20 + (5 * getRelicLevel("berserkium_gem"))
				if(chance >= 1 and chance <= odds) then
					attachLoadEntityOnExplosiontoProjectile(projectile_entity_id, getLoadOnExplosion("launch_elemental_particles_berserkium").path, "magic_liquid_berserk")
				end
			end
		end,
	},
	{
		name 					= "alcohol_gem",
		ui_name					= "Whiskey Gem",
		categories				= "projectiles,materials",
		effect_desc				= "Spells you cast that explode on impact have a 20% chance of releasing whiskey on impact.",
		effect_desc_short		= "Your spells release whiskey.",
        precursor_types			= "elemental_gem",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("alcohol_gem") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 20 + (5 * getRelicLevel("alcohol_gem"))
				if(chance >= 1 and chance <= odds) then
					attachLoadEntityOnExplosiontoProjectile(projectile_entity_id, getLoadOnExplosion("launch_elemental_particles_alcohol").path, "alcohol")
				end
			end
		end,
	},
	{
		name 					= "urine_gem",
		ui_name					= "Urine Gem",
		categories				= "projectiles,materials",
		effect_desc				= "Spells you cast that explode on impact have a 20% chance of releasing urine on impact.",
		effect_desc_short		= "Your spells release urine.",
        precursor_types			= "elemental_gem",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("urine_gem") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 20 + (5 * getRelicLevel("urine_gem"))
				if(chance >= 1 and chance <= odds) then
					attachLoadEntityOnExplosiontoProjectile(projectile_entity_id, getLoadOnExplosion("launch_elemental_particles_urine").path, "urine")
				end
			end
		end,
	},
	{
		name 					= "freezing_liquid_gem",
		ui_name					= "Freezing Liquid Gem",
		categories				= "projectiles,materials",
		effect_desc				= "Spells you cast that explode on impact have a 20% chance of releasing freezing liquid on impact.",
		effect_desc_short		= "Your spells release freezing liquid.",
        precursor_types			= "elemental_gem",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("freezing_liquid_gem") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 20 + (5 * getRelicLevel("freezing_liquid_gem"))
				if(chance >= 1 and chance <= odds) then
					attachLoadEntityOnExplosiontoProjectile(projectile_entity_id, getLoadOnExplosion("launch_elemental_particles_freezing_liquid").path, "blood_cold_vapour")
				end
			end
		end,
	},
	{
		name 					= "light_bulb",
		ui_name					= "Light Bulb",
		categories				= "health,materials",
		effect_desc				= "Freezing gas no longer damages you.",
		effect_desc_short		= "Freezing gas no longer damages you.",
        precursor_types			= "gadget",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			EntitySetDamageFromMaterial( getPlayerEntity(), "blood_cold", 0 )
			EntitySetDamageFromMaterial( getPlayerEntity(), "blood_cold_vapour", 0 )
		end,
		ondrop_function			= function()
			EntitySetDamageFromMaterial( getPlayerEntity(), "blood_cold_vapour", 0.0006 )
			EntitySetDamageFromMaterial( getPlayerEntity(), "blood_cold", 0.0009 )
		end,
		excludes_relics			= "universal_immunity_stone,omega_shield",
		excludes_relics_desc	= "It can't be used along with other freezing gas / liquid protection related relics",
		excludes_relics_tags	= "freezegas_protection",
	},
	{
		name 					= "test_tube",
		ui_name					= "Test Tube",
		categories				= "materials,player",
		effect_desc				= "Enemies you kill have a 25% chance of dropping a random potion.",
		effect_desc_short		= "Random potion drops on kills.",
        precursor_types			= "gadget",
		upgrade_effect_desc		= "For each upgrade level, drop chance increases by 5%.",
		is_upgradable 			= true,
		onenemykill_function	= function(damage_type_bit_field, damage_message, entity_thats_responsible, drop_items)
			if(isRelicOwned("test_tube") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 25 + (5 * getRelicLevel("test_tube"))
				if(chance >= 1 and chance <= odds) then
					local x, y = EntityGetTransform(GetUpdatedEntityID())
					EntityLoad("data/entities/items/pickup/potion_random_material.xml", x, y)
				end
			end
		end,
	},
	{
		name 					= "corrupted_mask",
		ui_name					= "Corrupted Mask",
		categories				= "materials",
		effect_desc				= "Toxic sludge near you is automatically converted into water, but any red blood around you is converted into poison.",
		effect_desc_short		= "Toxic sluge near you becomes water, blood near you becomes poison.",
        precursor_types			= "magic_artifact",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			LoadGameEffectEntityTo( getPlayerEntity(), getCustomEffect("effect_field_radioactive_liquid_to_water").path )
			LoadGameEffectEntityTo( getPlayerEntity(), getCustomEffect("effect_field_blood_to_poison").path )
		end,
		ondrop_function			= function()
			local player_children = EntityGetAllChildren(getPlayerEntity())
			if player_children == nil then return end
			for i,child in ipairs(player_children) do
				if(EntityGetName(child) == "field_blood_to_poison" or EntityGetName(child) == "field_radioactive_liquid_to_water" ) then
					EntityKill(child)
				end
			end
		end,
	},
	{
		name 					= "gas_mask",
		ui_name					= "Gas Mask",
		categories				= "materials",
		effect_desc				= "Most gases around you are filtered into harmless steam.",
		effect_desc_short		= "Filters gases around you.",
        precursor_types			= "gadget",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			LoadGameEffectEntityTo( getPlayerEntity(), getCustomEffect("effect_field_gases_to_steam").path )
		end,
		ondrop_function			= function()
			local player_children = EntityGetAllChildren(getPlayerEntity())
			if player_children == nil then return end
			for i,child in ipairs(player_children) do
				if(EntityGetName(child) == "field_gases_to_steam") then
					EntityKill(child)
					break
				end
			end
		end,
	},
	{
		name 					= "blessed_essence",
		ui_name					= "Blessed Essence",
		categories				= "health",
		effect_desc				= "Provided you have more gold than hp, and your current wand has maximum mana that is \nequivalent to at least twice your current health, you have a 20% chance of being healed by 1 each time you cast a spell.",
		effect_desc_short		= "Heals you on spell casts.",
        precursor_types			= "flask",
		upgrade_effect_desc		= "For each upgrade level, heal chance increases by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("blessed_essence") >= 1) then
				local player_gold = getPlayerGold()
				local hp = getPlayerHealth() * 25
				local hp_treshold = (getPlayerHealth() * 25) * 2
				local mana_max = getCurrentWandManaMax()
				
				if((player_gold > hp) and (mana_max >= hp_treshold)) then
					SetRandomSeed( GameGetFrameNum(), math.random(1,50))
					local chance = Random(1,100)
					local odds = 20 + (5 * getRelicLevel("blessed_essence"))
					if(chance >= 1 and chance <= odds) then
						GamePrint("Relic effect: +1 hp on spell cast")
						addPlayerHealth(1, false)
					end
				end
			end
		end,
	},
	{
		name 					= "stillness_poison_flask",
		ui_name					= "Stillness Poison Flask",
		categories				= "projectiles,player",
		effect_desc				= "If you stop moving, you start losing hp, but when you are moving, your spells deal extra damage.",
		effect_desc_short		= "Moving boosts spell damage, but standing still damages.",
        precursor_types			= "flask",
		upgrade_effect_desc		= "For each upgrade level, damage bonus is increased",
		is_upgradable 			= true,
		overtime_function	= function()
			if(isRelicOwned("stillness_poison_flask") >= 1) then
				if(isPlayerStanding() and (not isPlayerMoving())) then
					local hp_lost = 1
					GamePrint("Relic penalty: -" .. hp_lost .. " hp for standing still")
					addPlayerHealth(hp_lost * -1)
				end
			end
		end,
		onspellcast_function	= function(projectile_entity_id)
			if(isPlayerMoving()) then
				if(isRelicOwned("stillness_poison_flask") >= 1) then
					multiplyProjectileBaseDamage(projectile_entity_id, 2 + (1 * getRelicLevel("stillness_poison_flask")))
				end
			end
		end,
	},
	{
		name 					= "shotgun_shell",
		ui_name					= "Shotgun Shell",
		categories				= "projectiles,materials",
		effect_desc				= "Spells you cast that explode on impact have a 30% chance of releasing gunpowder on impact.",
		effect_desc_short		= "Your spells release gunpowder.",
        precursor_types			= "gadget",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("shotgun_shell") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 30 + (5 * getRelicLevel("shotgun_shell"))
				if(chance >= 1 and chance <= odds) then
					attachLoadEntityOnExplosiontoProjectile(projectile_entity_id, getLoadOnExplosion("launch_elemental_particles_gunpowder").path, "gunpowder")
				end
			end
		end,
	},
	{
		name 					= "clay_baked_doll",
		ui_name					= "Clay Baked Doll",
		categories				= "health,status_effects",
		effect_desc				= "You are now immune to all polymorphine variations, but you take triple melee damage.",
		effect_desc_short		= "You're immune to polymorphine, but you take triple melee damage.",
        precursor_types			= "magic_artifact",
		upgrade_effect_desc		= "For each upgrade level, melee damage drawback is slightly decreased.",
		is_upgradable 			= true,
		ondamaged_function	= function(damage, message, entity_thats_responsible, is_fatal, projectile_thats_responsible )
			if(isRelicOwned("clay_baked_doll") >= 1) then
				if(message == "$damage_melee") then
					GamePrint("Relic penalty: triple melee damage taken")
					local damage_penalty = damage * 2 - (0.3 * getRelicLevel("clay_baked_doll"))
					EntityInflictDamage( getPlayerEntity(), damage_penalty, "DAMAGE_MELEE", "Clay Baked Doll damage penalty", "BLOOD_SPRAY", 0, 0)
				end
			end
		end,
		onpickup_function		= function()
			local game_effect_comp_id, game_effect_entity_id = GetGameEffectLoadTo( getPlayerEntity(), "PROTECTION_POLYMORPH", true )
			if game_effect_comp_id ~= nil then
				ComponentSetValue2( game_effect_comp_id, "frames", -1)
				EntityAddTag(game_effect_entity_id, "clay_baked_doll_effect")
			end

		end,
		ondrop_function			= function()
			local player_children = EntityGetAllChildren(getPlayerEntity())
			if player_children == nil then return end
			for i,child in ipairs(player_children) do
				if(EntityHasTag(child, "clay_baked_doll_effect")) then
					EntityKill(child)
					break
				end
			end
		end,
		excludes_relics			= "wooden_helmet,blessed_helmet",
		excludes_relics_desc	= "It can't be used along with other polymorph immunity related relics",
		excludes_relics_tags	= "polymorph_immunity",
	},
	{
		name 					= "poly_gem",
		ui_name					= "Poly Gem",
		categories				= "projectiles,materials",
		effect_desc				= "Spells you cast that explode on impact have a 20% chance of releasing polymorphine on impact.",
		effect_desc_short		= "Your spells release polymorphine.",
        precursor_types			= "elemental_gem",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("poly_gem") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 20 + (5 * getRelicLevel("poly_gem"))
				if(chance >= 1 and chance <= odds) then
					attachLoadEntityOnExplosiontoProjectile(projectile_entity_id, getLoadOnExplosion("launch_elemental_particles_poly").path, "magic_liquid_polymorph")
				end
			end
		end,
	},
	{
		name 					= "miner_hat",
		ui_name					= "Miner Hat",
		categories				= "projectiles",
		effect_desc				= "Enemies you kill have a 20% chance of spawning a minecart.",
		effect_desc_short		= "Killed enemies spawn minecart.",
        precursor_types			= "gadget",
		upgrade_effect_desc		= "For each upgrade level, minecart spawn chance increases by 5%.",
		is_upgradable 			= true,
		onenemykill_function	= function(damage_type_bit_field, damage_message, entity_thats_responsible, drop_items)
			if(isRelicOwned("miner_hat") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 20 + (5 * getRelicLevel("miner_hat"))
				if(chance >= 1 and chance <= odds) then
					local x, y = EntityGetTransform(GetUpdatedEntityID())
					EntityLoad("data/entities/props/physics_minecart.xml", x, y)
				end
			end
		end,
	},
	{
		name 					= "vaders_hilt",
		ui_name					= "Vader's Hilt ",
		categories				= "projectiles",
		effect_desc				= "Your spells have a 30% chance of casting incendiary lightsaber blades that are capable of deflecting enemy projectiles.",
		effect_desc_short		= "Your spells emit incendiary lightsaber blades.",
        precursor_types			= "gadget,device",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("vaders_hilt") >= 1 and isRelicOwned("dormant_kyber_crystal") >= 1) then
				if(not isPhysicsProjectile(projectile_entity_id)) then
					SetRandomSeed( GameGetFrameNum(), math.random(1,50))
					local chance = Random(1,100)
					local odds = 30 + (5 * getRelicLevel("vaders_hilt"))
					if(chance >= 1 and chance <= odds) then
						addNewSimplifiedInternalVariable(projectile_entity_id, "deflected")
						addNewLuaCallback(projectile_entity_id, "overtime", "mods/ArcaneReliquary/files/scripts/projectiles/launch_circular_lightsaberblade_projectiles.lua", 5, false, "launch_circular_lightsaberblade_projectiles")
					end
				end
			end
		end,
		requires_relic			= "dormant_kyber_crystal",
	},
	{
		name 					= "kenobis_hilt",
		ui_name					= "Kenobi's Hilt ",
		categories				= "projectiles",
		effect_desc				= "Your spells have a 30% chance of casting freezing lightsaber blades that are capable of deflecting enemy projectiles.",
		effect_desc_short		= "Your spells emit freezing lightsaber blades.",
        precursor_types			= "gadget,device",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("kenobis_hilt") >= 1 and isRelicOwned("dormant_kyber_crystal") >= 1) then
				if(not isPhysicsProjectile(projectile_entity_id)) then
					SetRandomSeed( GameGetFrameNum(), math.random(1,50))
					local chance = Random(1,100)
					local odds = 30 + (5 * getRelicLevel("kenobis_hilt"))
					if(chance >= 1 and chance <= odds) then
						addNewSimplifiedInternalVariable(projectile_entity_id, "deflected")
						addNewLuaCallback(projectile_entity_id, "overtime", "mods/ArcaneReliquary/files/scripts/projectiles/launch_circular_lightsaberblade_projectiles_blue.lua", 5, false, "launch_circular_lightsaberblade_projectiles_blue")
					end
				end
			end
		end,
		requires_relic			= "dormant_kyber_crystal",
	},
	{
		name 					= "windus_hilt",
		ui_name					= "Windu's Hilt ",
		categories				= "projectiles",
		effect_desc				= "Your spells have a 30% chance of casting mutating purple lightsaber blades that are capable of deflecting enemy projectiles.",
		effect_desc_short		= "Your spells emit mutating lightsaber blades.",
        precursor_types			= "gadget,device",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("windus_hilt") >= 1 and isRelicOwned("dormant_kyber_crystal") >= 1) then
				if(not isPhysicsProjectile(projectile_entity_id)) then
					SetRandomSeed( GameGetFrameNum(), math.random(1,50))
					local chance = Random(1,100)
					local odds = 30 + (5 * getRelicLevel("windus_hilt"))
					if(chance >= 1 and chance <= odds) then
						addNewSimplifiedInternalVariable(projectile_entity_id, "deflected")
						addNewLuaCallback(projectile_entity_id, "overtime", "mods/ArcaneReliquary/files/scripts/projectiles/launch_circular_lightsaberblade_projectiles_purple.lua", 5, false, "launch_circular_lightsaberblade_projectiles_purple")
					end
				end
			end
		end,
		requires_relic			= "dormant_kyber_crystal",
	},
	{
		name 					= "lukes_hilt",
		ui_name					= "Luke's Hilt ",
		categories				= "projectiles",
		effect_desc				= "Your spells have a 30% chance of casting electrocuting lightsaber blades that are capable of deflecting enemy projectiles.",
		effect_desc_short		= "Your spells emit electrocuting lightsaber blades.",
        precursor_types			= "gadget,device",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("lukes_hilt") >= 1 and isRelicOwned("dormant_kyber_crystal") >= 1) then
				if(not isPhysicsProjectile(projectile_entity_id)) then
					SetRandomSeed( GameGetFrameNum(), math.random(1,50))
					local chance = Random(1,100)
					local odds = 30 + (5 * getRelicLevel("lukes_hilt"))
					if(chance >= 1 and chance <= odds) then
						addNewSimplifiedInternalVariable(projectile_entity_id, "deflected")
						addNewLuaCallback(projectile_entity_id, "overtime", "mods/ArcaneReliquary/files/scripts/projectiles/launch_circular_lightsaberblade_projectiles_blue_dark.lua", 5, false, "launch_circular_lightsaberblade_projectiles_blue_dark")
					end
				end
			end
		end,
		requires_relic			= "dormant_kyber_crystal",
	},
	{
		name 					= "dormant_kyber_crystal",
		ui_name					= "Dormant Kyber Crystal",
		categories				= "projectiles",
		effect_desc				= "Your spells gain a slight damage bonus, and if you own any lightsaber \nhilts, their effects will now activate.",
		effect_desc_short		= "Slight bonus damage, activates lightsaber hilts.",
        precursor_types			= "effect_crystal",
		upgrade_effect_desc		= "For each upgrade level, damage bonus increases.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("dormant_kyber_crystal") >= 1) then
				multiplyProjectileBaseDamage(projectile_entity_id, 1.5 + (1 * getRelicLevel("dormant_kyber_crystal")))
			end
		end,
	},
	{
		name 					= "fragmentation_grenade",
		ui_name					= "Fragmentation Grenade",
		categories				= "projectiles",
		effect_desc				= "Your spells have a 25% chance of exploding into a barrage of invisible shrapnel that penetrates walls.",
		effect_desc_short		= "Your spells release invisible shrapnel.",
        precursor_types			= "gadget",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("fragmentation_grenade") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 25 + (5 * getRelicLevel("fragmentation_grenade"))
				if(chance >= 1 and chance <= odds) then
					addBuffVisualIndicator_Projectile(projectile_entity_id, "acid")
					attachLoadEntityOnExplosiontoProjectile(projectile_entity_id, getLoadOnExplosion("launch_circular_projectiles_invisible_shrapnel").path)
				end
			end
		end
	},
	{
		name 					= "syringe",
		ui_name					= "Syringe",
		categories				= "projectiles",
		effect_desc				= "Provided they travel enough time, your spells have a 30% chance of injecting water wherever they hit.",
		effect_desc_short		= "Your spells inject water on hit.",
        precursor_types			= "gadget",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("syringe") >= 1) then
				if(not isPhysicsProjectile(projectile_entity_id)) then
					SetRandomSeed( GameGetFrameNum(), math.random(1,50))
					local chance = Random(1,100)
					local odds = 30 + (5 * getRelicLevel("syringe"))
					if(chance >= 1 and chance <= odds) then
						projectileRemoveBouncing(projectile_entity_id)
						
						addNewInternalVariable(projectile_entity_id, "projectile_pos_x", "value_int", 0)
						addNewInternalVariable(projectile_entity_id, "projectile_pos_y", "value_int", 0)
						addNewInternalVariable(projectile_entity_id, "projectile_vel_x", "value_int", 0)
						addNewInternalVariable(projectile_entity_id, "projectile_vel_y", "value_int", 0)
						
						addBuffVisualIndicator_Projectile(projectile_entity_id, "water")
						addNewInternalVariable(projectile_entity_id, "projectile_definition", "value_string", "mods/ArcaneReliquary/files/entity_definitions/custom_projectiles/invisible_shrapnel_projectile_water.xml")
							
						addNewLuaCallback(projectile_entity_id, "overtime", "mods/ArcaneReliquary/files/scripts/projectiles/store_projectile_vel_pos.lua", 1, false, "store_projectile_vel_pos")
						addNewLuaCallback_onRemoved(projectile_entity_id, "mods/ArcaneReliquary/files/scripts/projectiles/launch_directional_shrapnel_projectile.lua")
					end
				end
			end
		end
	},
	{
		name 					= "prism",
		ui_name					= "Prism",
		categories				= "projectiles",
		effect_desc				= "Your spells have a 30% chance of breaking down into lasers of various colors and effects.",
		effect_desc_short		= "Your spells break down into colored lasers.",
        precursor_types			= "magic_artifact",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("prism") >= 1) then
				if(not isPhysicsProjectile(projectile_entity_id)) then
					SetRandomSeed( GameGetFrameNum(), math.random(1,50))
					local chance = Random(1,100)
					local odds = 30 + (5 * getRelicLevel("prism"))
					if(chance >= 1 and chance <= odds) then
						addBuffVisualIndicator_Projectile(projectile_entity_id, "material_rainbow")
						addNewLuaCallback(projectile_entity_id, "overtime", "mods/ArcaneReliquary/files/scripts/projectiles/launch_prism_laser_projectiles.lua", 4, true, "launch_prism_laser_projectiles")
					end
				end
			end
		end
	},
	{
		name 					= "frozen_leaf",
		ui_name					= "Frozen Leaf",
		categories				= "projectiles,materials",
		effect_desc				= "Enemy projectiles have a chance to turn into ice balls when they get near you.",
		effect_desc_short		= "Enemy projectiles turn into ice balls near you.",
        precursor_types			= "magic_artifact",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			LoadGameEffectEntityTo( getPlayerEntity(), getCustomEffect("effect_field_projectiles_to_ice").path )
		end,
		ondrop_function			= function()
			local player_children = EntityGetAllChildren(getPlayerEntity())
			if player_children == nil then return end
			for i,child in ipairs(player_children) do
				if(EntityGetName(child) == "field_projectiles_to_ice") then
					EntityKill(child)
					break
				end
			end
		end,
	},
	{
		name 					= "piercing_crystal",
		ui_name					= "Piercing Crystal",
		categories				= "projectiles",
		effect_desc				= "Your spells will no longer be deflected by buffed enemies energy shields.",
		effect_desc_short		= "Your spells bypass buffed enemies energy shields.",
        precursor_types			= "effect_crystal",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("piercing_crystal") >= 1) then
				addNewSimplifiedInternalVariable(projectile_entity_id, "shield_bypass")
			end
		end
	},
	{
		name 					= "time_bomb",
		ui_name					= "Time Bomb",
		categories				= "player,kick",
		effect_desc				= "Enemies and some objects (such as barrels, explosive boxes \nand others) you kick become rigged to explode after six ticks. \nIt also synergizes with Sniper Scope relic effect.",
		effect_desc_short		= "Kicking objects and enemies turn them into time bombs.",
        precursor_types			= "gadget",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onenemykicked_function	= function(entity_who_kicked )
			if(isRelicOwned("time_bomb") >= 1) then
				if(not isCreatureBoss(GetUpdatedEntityID())) then
					attachTimeBombMechanictoEntity(GetUpdatedEntityID())
				end
			end
		end,
		onextendedkick_function	= function( entity_id, extended_kick_id)
			if(isRelicOwned("time_bomb") >= 1) then
				attachLoadEffectOntoEntitiestoProjectile(extended_kick_id, getCustomEffect("effect_time_bomb").path, "spark_red")
			end
		end,
	},
	{
		name 					= "frying_pan",
		ui_name					= "Frying Pan",
		categories				= "projectiles",
		effect_desc				= "Your spells destroy enemy projectiles. \nIt also synergizes with Sniper Scope relic effect.",
		effect_desc_short		= "Your spells destroy enemy projectiles.",
        precursor_types			= "gadget",
		upgrade_effect_desc		= "For each upgrade level, collision radius is increased.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("frying_pan") >= 1) then
				if(not isOrbitingProjectile(projectile_entity_id)) then
					addNewSimplifiedInternalVariable( projectile_entity_id, "relic_effect_projectile")
					attachDragCollisionHandlertoProjectile(projectile_entity_id, getCollisionHandler("destroy_enemy_projectiles").path, false, 10 + (1 * getRelicLevel("frying_pan")), "projectile", false)
				end
			end
		end,
		onextendedkick_function	= function( entity_id, extended_kick_id)
			if(isRelicOwned("frying_pan") >= 1) then
				addBuffVisualIndicator_Projectile(extended_kick_id, "plasma_fading_green")
				attachDragCollisionHandlertoProjectile(extended_kick_id, getCollisionHandler("destroy_enemy_projectiles").path, false, 20, "projectile", false)
			end
		end,
	},
	{
		name 					= "hazardous_wand",
		ui_name					= "Hazardous Wand",
		categories				= "projectiles",
		effect_desc				= "When you cast a spell, you have a 30% chance of summoning a red \nspectral wand that shoots a powerful explosive laser.",
		effect_desc_short		= "Your spells summon a spectral wand that shoots red explosive lasers.",
        precursor_types			= "magic_artifact",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("hazardous_wand") >= 1) then
				if(not projectileHasProjectileDefinitionPath(projectile_entity_id)) then
					SetRandomSeed( GameGetFrameNum(), math.random(1,50))
					local chance = Random(1,100)
					local odds = 30 + (5 * getRelicLevel("hazardous_wand"))
					if(chance >= 1 and chance <= odds) then
						local x, y = EntityGetTransform(projectile_entity_id)
						local laser_shooter_id = EntityLoad("mods/ArcaneReliquary/files/entity_definitions/custom_projectiles/megalaser_custom.xml", x, y)
						addNewSimplifiedInternalVariable( laser_shooter_id, "relic_effect_projectile")
						local projectile_vel_x, projectile_vel_y = getProjectileVelocitiesandGravity(projectile_entity_id)
						setProjectileVelocitiesandGravity(laser_shooter_id, projectile_vel_x, projectile_vel_y)
					end
				end
			end
		end
	},
	{
		name 					= "matrioshka",
		ui_name					= "Matrioshka",
		categories				= "player,relics",
		effect_desc				= "Whenever you open a relic chest, there's a chance another chest will spawn.",
		effect_desc_short		= "Whenever you open a relic chest, there's a chance another chest will spawn.",
        precursor_types			= "magic_artifact",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			addNewSimplifiedInternalVariable(getPlayerEntity(), "matrioshka")
		end,
		ondrop_function			= function()
			removeSimplifiedInternalVariable(getPlayerEntity(), "matrioshka")
		end
	},
	{
		name 					= "liquid_soap",
		ui_name					= "Liquid Soap",
		categories				= "companions",
		effect_desc				= "A friendly bubble that shoots bubbletraps joins you.",
		effect_desc_short		= "A bubbletrap shooting companion joins you.",
        precursor_types			= "companion",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			attachCompaniontoEntity(getPlayerEntity(), "little_bubble", "mortal")
		end,
		ondrop_function			= function()
			detachCompanionfromEntity(getPlayerEntity(), "little_bubble")
		end,
	},
	{
		name 					= "soap",
		ui_name					= "Soap",
		categories				= "projectiles",
		effect_desc				= "Enemies you damage with spells have a 25% chance of being trapped \non explosive bubbletraps.",
		effect_desc_short		= "Your spells trap enemies on explosive bubbles.",
        precursor_types			= "gadget",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onenemydamaged_function	= function(damage, message, entity_thats_responsible, is_fatal, projectile_thats_responsible )
			if(isRelicOwned("soap") >= 1) then
				if(message == "$damage_projectile") then
					SetRandomSeed( GameGetFrameNum(), math.random(1,50))
					local chance = Random(1,100)
					local odds = 25 + (5 * getRelicLevel("soap"))
					if(chance >= 1 and chance <= odds) then
					
						LoadGameEffectEntityTo( GetUpdatedEntityID(), getCustomEffect("effect_bubbletrap").path )
						--[[
						local x, y = EntityGetTransform(GetUpdatedEntityID())
						local bubble_id = EntityLoad("mods/ArcaneReliquary/files/entity_definitions/custom_projectiles/bubbletrap.xml", x, y)
						--setProjectileShooter(bubble_id, getPlayerEntity())
						addNewSimplifiedInternalVariable(bubble_id, "deflected")
						setProjectileVelocitiesandGravity(bubble_id, Random(-160, 160), Random(-160, 160))
						]]
					end	
				end
			end
		end,
	},
	{
		name 					= "purified_water",
		ui_name					= "Purified Water",
		categories				= "player,enemies",
		effect_desc				= "When you're wet, scoring kills has a 25% chance of healing you in small \namounts, and if you're at full health when that happens, your maximum hp will be upgraded.",
		effect_desc_short		= "Kills heal when wet.",
        precursor_types			= "gadget",
		upgrade_effect_desc		= "For each upgrade level, heal chance increases by 5%.",
		is_upgradable 			= true,
		onenemykill_function	= function(damage_type_bit_field, damage_message, entity_thats_responsible, drop_items)
			if(isRelicOwned("purified_water") >= 1) then
				if(GameGetGameEffectCount(getPlayerEntity(), "WET") > 0) then
					SetRandomSeed( GameGetFrameNum(), math.random(1,50))
					local chance = Random(1,100)
					local odds = 25 + (5 * getRelicLevel("purified_water"))
					if(chance >= 1 and chance <= odds) then
						addPlayerHealth(Random(1,2), true)
						local x, y = EntityGetTransform(getPlayerEntity())
						GamePlaySound( "data/audio/Desktop/event_cues.snd", "event_cues/heart/create", x, y )
					end
				end
			end
		end,
	},
	{
		name 					= "particle_displacer",
		ui_name					= "Particle Displacer",
		categories				= "projectiles",
		effect_desc				= "When you hit an enemy with your spells, there's a 25% chance you'll \nswitch places with them.",
		effect_desc_short		= "Your spells make you switch places with enemies.",
        precursor_types			= "gadget,device",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onenemydamaged_function	= function(damage, message, entity_thats_responsible, is_fatal, projectile_thats_responsible )
			if(isRelicOwned("particle_displacer") >= 1) then
				if(message == "$damage_projectile") then
					SetRandomSeed( GameGetFrameNum(), math.random(1,50))
					local chance = Random(1,100)
					local odds = 25 + (5 * getRelicLevel("particle_displacer"))
					if(chance >= 1 and chance <= odds) then
						local enemy_x, enemy_y = EntityGetTransform(GetUpdatedEntityID())
						local player_x, player_y = EntityGetTransform(getPlayerEntity())
						
						EntitySetTransform(GetUpdatedEntityID(), player_x, player_y)
						EntitySetTransform(getPlayerEntity(), enemy_x, enemy_y)
					end
				end
			end
		end,
	},
	{
		name 					= "amnesia_medalion",
		ui_name					= "Amnesia Medalion",
		categories				= "projectiles",
		effect_desc				= "Enemies who are able to shoot hit by your spells have a 25% chance of \nlosing their ability to shoot for a few seconds.",
		effect_desc_short		= "Enemies who shoot hit by your spells become unable to shoot for a few seconds.",
        precursor_types			= "magic_artifact",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onenemydamaged_function	= function(damage, message, entity_thats_responsible, is_fatal, projectile_thats_responsible )
			if(isRelicOwned("amnesia_medalion") >= 1) then
				if(message == "$damage_projectile") then
					SetRandomSeed( GameGetFrameNum(), math.random(1,50))
					local chance = Random(1,100)
					local odds = 25 + (5 * getRelicLevel("amnesia_medalion"))
					if(chance >= 1 and chance <= odds) then
						LoadGameEffectEntityTo( GetUpdatedEntityID(), getCustomEffect("effect_disarm_debuff").path )
					end
				end
			end
		end,
	},
	{
		name 					= "gravitation_ring",
		ui_name					= "Gravitation Ring",
		categories				= "projectiles",
		effect_desc				= "Flying enemies hit by your spells have a 25% chance of losing their \nability to fly for a few seconds.",
		effect_desc_short		= "Flying enemies hit by your spells become unable to fly for a few seconds.",
        precursor_types			= "ring",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onenemydamaged_function	= function(damage, message, entity_thats_responsible, is_fatal, projectile_thats_responsible )
			if(isRelicOwned("gravitation_ring") >= 1) then
				if(message == "$damage_projectile") then
					SetRandomSeed( GameGetFrameNum(), math.random(1,50))
					local chance = Random(1,100)
					local odds = 25 + (5 * getRelicLevel("gravitation_ring"))
					if(chance >= 1 and chance <= odds) then
						LoadGameEffectEntityTo( GetUpdatedEntityID(), getCustomEffect("effect_no_flight_debuff").path )
					end
				end
			end
		end,
	},
	{
		name 					= "paralysing_crystal",
		ui_name					= "Paralysing Crystal",
		categories				= "projectiles",
		effect_desc				= "Ground enemies you hit with your spells have a 25% chance of being \nparalysed for a few seconds. \nIt also synergizes with Sniper Scope relic effect.",
		effect_desc_short		= "Ground enemies hit by your spells are paralysed for a few seconds.",
        precursor_types			= "effect_crystal",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("paralysing_crystal") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 25 + (5 * getRelicLevel("paralysing_crystal"))
				if(chance >= 1 and chance <= odds) then
					attachLoadEffectOntoEntitiestoProjectile(projectile_entity_id, getCustomEffect("effect_paralyse").path, "plasma_fading_green")
				end
			end
		end,
		onextendedkick_function	= function( entity_id, extended_kick_id)
			if(isRelicOwned("paralysing_crystal") >= 1) then
				attachLoadEffectOntoEntitiestoProjectile(extended_kick_id, getCustomEffect("effect_paralyse").path, "acid_gas")
			end
		end,
	},
	{
		name 					= "pinwheel",
		ui_name					= "Pinwheel",
		categories				= "projectiles",
		effect_desc				= "Your spells have a 30% chance of generating random winds that \ncarry enemies away when they are hit (more effective on ground creatures than on flying ones).",
		effect_desc_short		= "Your spells generate winds that carries enemies away.",
        precursor_types			= "magic_artifact",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("pinwheel") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 30 + (5 * getRelicLevel("pinwheel"))
				if(chance >= 1 and chance <= odds) then
					attachLoadEffectOntoEntitiestoProjectile(projectile_entity_id, getCustomEffect("effect_wind").path, "spark_white")
				end
			end
		end
	},
	{
		name 					= "pinwheel_colored",
		ui_name					= "Colored Pinwheel",
		categories				= "projectiles",
		effect_desc				= "Your spells have a 30% chance of generating random winds that \ncarry enemies away as well as inflict impact damage on them when they are \nhit (more effective on ground creatures than on flying ones).",
		effect_desc_short		= "Your spells generate damaging winds that carries enemies away.",
        precursor_types			= "magic_artifact",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("pinwheel_colored") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 30 + (5 * getRelicLevel("pinwheel_colored"))
				if(chance >= 1 and chance <= odds) then
					attachLoadEffectOntoEntitiestoProjectile(projectile_entity_id, getCustomEffect("effect_wind_heavy").path, "spark_white")
				end
			end
		end
	},
	{
		name 					= "bloody_talon",
		ui_name					= "Bloody Talon",
		categories				= "player",
		effect_desc				= "When you consume at least 100 unities of regular, non-toxic, meat, you are healed by 5.",
		effect_desc_short		= "Consuming meat heals you.",
        precursor_types			= "magic_artifact",
		upgrade_effect_desc		= "For each upgrade level, healed hp is increased.",
		is_upgradable 			= true,
		overtimeslow_function	= function()
			if(isRelicOwned("bloody_talon") >= 1) then
				local meat_consumed = getMaterialAmountOnPlayerStomach("meat")
				if(meat_consumed > 100) then
					meat_consumed = meat_consumed - 100
					AddMaterialInventoryMaterial( GetUpdatedEntityID(), "meat", meat_consumed)
					
					local x, y = EntityGetTransform(GetUpdatedEntityID())
					GamePlaySound( "data/audio/Desktop/event_cues.snd", "event_cues/heart/create", x, y )
					local healed_hp = 5 + (1 * getRelicLevel("bloody_talon"))
					addPlayerHealth(healed_hp, false)
					GamePrint("Relic effect: healed by " .. healed_hp .. " for consuming 100 meat.")
				end
			end
		end,
	},
	{
		name 					= "ankle_pads",
		ui_name					= "Ankle Pads",
		categories				= "player,projectiles",
		effect_desc				= "Your spell damage is heavily reduced, but your kicks deal 25x more damage. \nIt also synergizes with Sniper Scope relic effect.",
		effect_desc_short		= "Reduced spell damage, 25x kick damage multiplier.",
        precursor_types			= "boot",
		upgrade_effect_desc		= "For each upgrade level, spell damage reduction is decreased.",
		is_upgradable 			= true,
		onenemydamaged_function	= function(damage, message, entity_thats_responsible, is_fatal, projectile_thats_responsible )
			if(isRelicOwned("ankle_pads") >= 1) then
				if(message == "$damage_kick") then
					GamePrint("Relic effect: 25x kick damage")
					EntityInflictDamage( GetUpdatedEntityID(), damage * 24, "DAMAGE_MELEE", "Metal Boot damage", "BLOOD_SPRAY", 0, 0)
				end
			end
		end,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("ankle_pads") >= 1) then
				fractionProjectileDamage(projectile_entity_id, 15 - (1 * getRelicLevel("ankle_pads")))
			end
		end,
		onextendedkick_function	= function( entity_id, extended_kick_id)
			if(isRelicOwned("ankle_pads") >= 1) then
				multiplyProjectileDamageByType(extended_kick_id, "melee", 25)
			end
		end,
		excludes_relics_desc	= "It can't be used along with other kick damage boost related relics",
		excludes_relics_tags	= "kick_damage_boost",
	},
	{
		name 					= "mob_charm",
		ui_name					= "Mob Charm",
		categories				= "projectiles",
		effect_desc				= "Buffed creatures take additional damage from your spells, the \nbigger their buff level, the more damage you deal.",
		effect_desc_short		= "Bonus spell damage dealt to buffed enemies.",
        precursor_types			= "magic_artifact",
		upgrade_effect_desc		= "For each upgrade level, damage bonus is increased.",
		is_upgradable 			= true,
		onenemydamaged_function	= function(damage, message, entity_thats_responsible, is_fatal, projectile_thats_responsible )
			if(isRelicOwned("mob_charm") >= 1) then
				if(message == "$damage_projectile") then
					local creature_id = GetUpdatedEntityID()
					if(entityHasSimplifiedInternalVariable(creature_id, "buff_level_1")) then
						--GamePrint("Relic effect: 2x damage dealt to level 1 buffed enemy")
						EntityInflictDamage( creature_id, damage + (1 * getRelicLevel("mob_charm")), "DAMAGE_PROJECTILE", "Mob Charm damage penalty", "NORMAL", 0, 0)
					elseif(entityHasSimplifiedInternalVariable(creature_id, "buff_level_2")) then
						--GamePrint("Relic effect: 3x damage dealt to level 2 buffed enemy")
						EntityInflictDamage( creature_id, (damage * 2) + (1 * getRelicLevel("mob_charm")), "DAMAGE_PROJECTILE", "Mob Charm damage penalty", "NORMAL", 0, 0)
					elseif(entityHasSimplifiedInternalVariable(creature_id, "buff_level_3")) then
						--GamePrint("Relic effect: 4x damage dealt to level 3 buffed enemy")
						EntityInflictDamage( creature_id, (damage * 3) + (1 * getRelicLevel("mob_charm")), "DAMAGE_PROJECTILE", "Mob Charm damage penalty", "NORMAL", 0, 0)
					elseif(entityHasSimplifiedInternalVariable(creature_id, "buff_level_4")) then
						--GamePrint("Relic effect: 5x damage dealt to level 3 buffed enemy")
						EntityInflictDamage( creature_id, (damage * 4) + (1 * getRelicLevel("mob_charm")), "DAMAGE_PROJECTILE", "Mob Charm damage penalty", "NORMAL", 0, 0)
					end
					
					-- additional damage that covers no buffs mode enemies as well
					if(entityHasSimplifiedInternalVariable(creature_id, "relic_chest_buff")) then
						EntityInflictDamage( creature_id, damage + (1 * getRelicLevel("mob_charm")), "DAMAGE_PROJECTILE", "Mob Charm damage penalty", "NORMAL", 0, 0)
					end
				end
			end
		end,
	},
	{
		name 					= "blessed_dynamite",
		ui_name					= "Blessed Dynamite",
		categories				= "enemies,health",
		effect_desc				= "You have a 30% chance of being healed by a small amount when \nan enemy is killed by explosions caused by your spells.",
		effect_desc_short		= "Killing enemies with explosive spells heals you.",
        precursor_types			= "magic_artifact",
		upgrade_effect_desc		= "For each upgrade level, heal chance increases by 5% and amount healed is also increased.",
		is_upgradable 			= true,
		onenemykill_function	= function(damage_type_bit_field, damage_message, entity_thats_responsible, drop_items)
			if(isRelicOwned("blessed_dynamite") >= 1) then
				if(damage_message == "$damage_explosion") then
					SetRandomSeed( GameGetFrameNum(), math.random(1,50))
					local chance = Random(1,100)
					local odds = 30 + (5 * getRelicLevel("blessed_dynamite"))
					if(chance >= 1 and chance <= odds) then
						local amount_healed = Random(1,2)
						GamePrint("Relic effect: healed by " .. amount_healed .. " for killing enemy with explosion.")
						addPlayerHealth(amount_healed + (1 * getRelicLevel("blessed_dynamite")), false)
						local x, y = EntityGetTransform(getPlayerEntity())
						GamePlaySound( "data/audio/Desktop/event_cues.snd", "event_cues/heart/create", x, y )
					end
				end
			end
		end,
	},
	{
		name 					= "relic_charm",
		ui_name					= "Relic Charm",
		categories				= "player,relics",
		effect_desc				= "Every time a buffed enemy drops a chest on death, there's a \n50% chance it will drop another one",
		effect_desc_short		= "Doubles the amount of relic chest drops from buffed enemies.",
        precursor_types			= "magic_artifact",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			addNewSimplifiedInternalVariable(getPlayerEntity(), "double_chest_drop")
		end,
		ondrop_function			= function()
			removeSimplifiedInternalVariable(getPlayerEntity(), "double_chest_drop")
		end
	},
	{
		name 					= "voodoo_doll",
		ui_name					= "Voodoo Doll",
		categories				= "projectiles",
		effect_desc				= "Enemies hit by your spells have a 30% chance to be temporarily \nunder the control of your cursor.",
		effect_desc_short		= "Your spells make enemies follow your cursor.",
        precursor_types			= "magic_artifact",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onenemydamaged_function	= function(damage, message, entity_thats_responsible, is_fatal, projectile_thats_responsible )
			if(isRelicOwned("voodoo_doll") >= 1) then
				if(message == "$damage_projectile") then
					SetRandomSeed( GameGetFrameNum(), math.random(1,50))
					local chance = Random(1,100)
					local odds = 30 + (5 * getRelicLevel("voodoo_doll"))
					if(chance >= 1 and chance <= odds) then
						if(not entityHasSimplifiedInternalVariable(GetUpdatedEntityID(), "voodoo_doll_effect")) then
							addBuffVisualIndicator(GetUpdatedEntityID(), "spark_yellow", "voodoo_doll_effect")
							addNewSimplifiedInternalVariable(GetUpdatedEntityID(), "voodoo_doll_effect")
							addNewLuaCallback(GetUpdatedEntityID(), "overtime", "mods/ArcaneReliquary/files/scripts/enemies/follow_mouse_cursor.lua", 1, false, "follow_mouse_cursor")
							addNewLuaCallback(GetUpdatedEntityID(), "overtime", "mods/ArcaneReliquary/files/scripts/enemies/remove_follow_mouse_cursor.lua", 150, true, "remove_follow_mouse_cursor")
						end
					end
				end
			end
		end,
	},
	{
		name 					= "teddy_bear",
		ui_name					= "Teddy Bear",
		categories				= "projectiles",
		effect_desc				= "Your spells have a 25% chance of applying the Charm effect on enemies they hit.",
		effect_desc_short		= "Your spells charm enemies.",
        precursor_types			= "gadget,magic_artifact",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("teddy_bear") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 25 + (5 * getRelicLevel("teddy_bear"))
				if(chance >= 1 and chance <= odds) then
					attachLoadEffectOntoEntitiestoProjectile(projectile_entity_id, "data/entities/misc/effect_charm.xml", "magic_liquid_berserk")
				end
			end
		end
	},
	{
		name 					= "cursed_bomb",
		ui_name					= "Cursed Bomb",
		categories				= "player",
		effect_desc				= "When you take projectile, melee or explosion damage, there's a 50% chance all enemy \ncreatures (except the one that attacked you) in the vicinity will be transformed into bombs.",
		effect_desc_short		= "Taking damage transforms enemies around into bombs.",
        precursor_types			= "magic_artifact",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		ondamaged_function		= function(damage, message, entity_thats_responsible, is_fatal, projectile_thats_responsible )
			if(isRelicOwned("cursed_bomb") >= 1) then
				if(message == "$damage_projectile" or message == "$damage_explosion" or message == "$damage_melee") then
					SetRandomSeed( GameGetFrameNum(), math.random(1,50))
					local chance = Random(1,100)
					local odds = 50 + (5 * getRelicLevel("cursed_bomb"))
					if(chance >= 1 and chance <= odds) then
						local player_x, player_y = EntityGetTransform(GetUpdatedEntityID())
						local enemies = EntityGetInRadiusWithTag( player_x, player_y, 200, "enemy" )
						for i, enemy_id in ipairs(enemies) do
							if(enemy_id ~= entity_thats_responsible) then
								LoadGameEffectEntityTo( enemy_id, getCustomEffect("effect_convert_to_bomb").path )
							end
						end
					end
				end
			end
		end,
	},
	{
		name 					= "disco_ball",
		ui_name					= "Disco Ball",
		categories				= "projectiles",
		effect_desc				= "Enemy projectiles have a chance to turn into glitter bombs when they get near you.",
		effect_desc_short		= "Enemy projectiles turn into glitter bombs near you.",
        precursor_types			= "gadget",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			LoadGameEffectEntityTo( getPlayerEntity(), getCustomEffect("effect_field_projectiles_to_glitter_bomb").path )
		end,
		ondrop_function			= function()
			local player_children = EntityGetAllChildren(getPlayerEntity())
			if player_children == nil then return end
			for i,child in ipairs(player_children) do
				if(EntityGetName(child) == "field_projectiles_to_glitter_bomb") then
					EntityKill(child)
					break
				end
			end
		end,
	},
	{
		name 					= "empty_bottle",
		ui_name					= "Empty Bottle",
		categories				= "player",
		effect_desc				= "Every 200 unities of water you drink are converted into +2 critical \ndamage multiplier, stacking up to a maximum of +30. Taking damage from projectiles, explosions or melee attacks resets it.",
		effect_desc_short		= "Drinking water gives you critical damage bonus.",
        precursor_types			= "gadget",
		upgrade_effect_desc		= "For each upgrade level, 5 is subtracted from the require 200 water.",
		is_upgradable 			= true,
		overtimeslow_function	= function()
			if(isRelicOwned("empty_bottle") >= 1) then
				local water_drunk = getMaterialAmountOnPlayerStomach("water")
				local base_water = 200 - (5 * getRelicLevel("empty_bottle"))
				if(water_drunk > base_water) then
					GamePrint("Relic effect: +2 critical damage multiplier for drinking " .. base_water .. " water.")
					water_drunk = water_drunk - base_water
					AddMaterialInventoryMaterial( GetUpdatedEntityID(), "water", water_drunk)
					
					local x, y = EntityGetTransform(GetUpdatedEntityID())
					GamePlaySound( "data/audio/Desktop/event_cues.snd", "event_cues/heart/create", x, y )
					local water_damage_multiplier = getInternalVariableValue(getPlayerEntity(), "water_damage_multiplier", "value_int")
					setInternalVariableValue(getPlayerEntity(), "water_damage_multiplier", "value_int", water_damage_multiplier + 2)
				end
			end
		end,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("empty_bottle") >= 1) then
				local critical_damage_multiplier = getInternalVariableValue(getPlayerEntity(), "water_damage_multiplier", "value_int")
				if (critical_damage_multiplier > 30) then
					critical_damage_multiplier = 30
				end
				if(critical_damage_multiplier > 0) then
					GamePrint("Relic effect: x" .. critical_damage_multiplier .. " water critical damage multiplier ")
					local chance, damage_multiplier = getProjectileCriticalDamageSettings(projectile_entity_id)
					setProjectileCriticalDamageSettings(projectile_entity_id, chance, damage_multiplier * critical_damage_multiplier)
				end
			end
		end,
		ondamaged_function		= function(damage, message, entity_thats_responsible, is_fatal, projectile_thats_responsible )
			if(isRelicOwned("empty_bottle") >= 1) then
				if(message == "$damage_melee" or message == "$damage_projectile" or message == "$damage_explosion") then
					GamePrint("Relic penalty: damage taken, water drunk critical damage multiplier reset")
					setInternalVariableValue(getPlayerEntity(), "water_damage_multiplier", "value_int", 0)
				end
			end
		end,
	},
	{
		name 					= "jerry_can",
		ui_name					= "Jerry Can",
		categories				= "projectiles,enemies,materials",
		effect_desc				= "Hitting enemies with your spells \nhas a 50% chance of inducing \nthem to emit a trail of persistent fire.",
		effect_desc_short		= "Enemies hit by your spells leave a fire trail.",
        precursor_types			= "gadget",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("jerry_can") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 50 + (5 * getRelicLevel("jerry_can"))
				if(chance >= 1 and chance <= odds) then
					attachLoadEffectOntoEntitiestoProjectile(projectile_entity_id, getCustomEffect("effect_fire_trail").path, "lava")
				end
			end
		end,
	},
	{
		name 					= "old_gear",
		ui_name					= "Old Gear",
		categories				= "projectiles",
		effect_desc				= "Spells you cast have a 25% chance of dragging enemies away while \ncausing melee damage to them.",
		effect_desc_short		= "Drags your enemies away, causing melee damage.",
        precursor_types			= "gadget",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("old_gear") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 25 + (5 * getRelicLevel("old_gear"))
				if(chance >= 1 and chance <= odds) then
					addBuffVisualIndicator_Projectile(projectile_entity_id, "blood")
					attachDragCollisionHandlertoProjectile(projectile_entity_id, getCollisionHandler("drag_melee_damage").path, true, 20, "enemy", false)
				end
			end
		end,
		ondrop_function			= function()
			removeInternalVariable(getPlayerEntity(), "old_gear_damage")
		end,
		overtime_function		= function()
			local multiplier = getRelicLevel("old_gear")
			if(multiplier == 0) then
				multiplier = 1
			end
			local damage_threshold = 0.01 * multiplier
			if(not entityHasInternalVariableValue(getPlayerEntity(), "old_gear_damage")) then
				addNewInternalVariable(getPlayerEntity(), "old_gear_damage", "value_float", damage_threshold)
			end
			setInternalVariableValue(getPlayerEntity(), "old_gear_damage", "value_float", damage_threshold)
		end
	},
	{
		name 					= "uranium_ingot",
		ui_name					= "Uranium Ingot",
		categories				= "player",
		effect_desc				= "Every 350 unities of toxic sludge you drink is converted into 15 gold. \nIt also synergizes with Earth Core Fragment relic effect.",
		effect_desc_short		= "Drinking toxic sludge gives you gold.",
        precursor_types			= "magic_artifact",
		upgrade_effect_desc		= "For each upgrade level, 5 is subtracted from the required 350 toxic sludge.",
		is_upgradable 			= true,
		overtimeslow_function	= function()
			if(isRelicOwned("uranium_ingot") >= 1) then
				local radioactive_liquid_drunk = getMaterialAmountOnPlayerStomach("radioactive_liquid")
				local base_radioactive_liquid = 350 - (5 * getRelicLevel("uranium_ingot"))
				if(radioactive_liquid_drunk > base_radioactive_liquid) then
					GamePrint("Relic effect: +15 gold for drinking " .. base_radioactive_liquid .. " toxic sludge.")
					radioactive_liquid_drunk = radioactive_liquid_drunk - base_radioactive_liquid
					AddMaterialInventoryMaterial( GetUpdatedEntityID(), "radioactive_liquid", radioactive_liquid_drunk)
					
					local x, y = EntityGetTransform(GetUpdatedEntityID())
					GamePlaySound( "data/audio/Desktop/event_cues.snd", "event_cues/heart/create", x, y )
					addPlayerGold(15)
				end
			end
		end,
		ongroundpound_function	= function( entity_id)
			if(isRelicOwned("uranium_ingot") >= 1) then
				LoadGameEffectEntityTo( entity_id, getCustomEffect("effect_shoot_projectiles_above_fireball_radioactive").path )
			end
		end,
	},
	{
		name 					= "oil_can",
		ui_name					= "Oil Can",
		categories				= "projectiles,enemies,materials",
		effect_desc				= "Your spells have a 30% chance of inducing enemies they \nhit to produce random explosions that release persistent fire.",
		effect_desc_short		= "Enemies hit by your spells leave a fire trail.",
        precursor_types			= "gadget",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("oil_can") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 30 + (5 * getRelicLevel("oil_can"))
				if(chance >= 1 and chance <= odds) then
					attachLoadEffectOntoEntitiestoProjectile(projectile_entity_id, getCustomEffect("effect_exploding_random_fire").path, "liquid_fire")
				end
			end
		end,
	},
	{
		name 					= "jester_box",
		ui_name					= "Jester Box",
		categories				= "player,projectiles,enemies",
		effect_desc				= "Kills have a 50% chance of yielding huge additional gold chunks and \nyour spells have a 50% chance of dealing 8x more damage, but your hp oscilates constantly.",
		effect_desc_short		= "Kills yield extra gold, your spells deal more damage, your up oscilates constantly.",
        precursor_types			= "gadget,device",
		upgrade_effect_desc		= "For each upgrade level, extra gold drop and improved damage chances increase by 5%.",
		is_upgradable 			= true,
		onpickup_function		= function()
			addNewLuaCallback(getPlayerEntity(), "overtime", "mods/ArcaneReliquary/files/scripts/player/hp_oscilation.lua", 2, false, "hp_oscilation")
		end,
		ondrop_function			= function()
			removeCallback(getPlayerEntity(), "hp_oscilation")
		end,
		onenemykill_function	= function(damage_type_bit_field, damage_message, entity_thats_responsible, drop_items)
			if(isRelicOwned("jester_box") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,2)
				if(chance == 1) then
					local death_x, death_y = EntityGetTransform(GetUpdatedEntityID())
					local chance = Random(1,100)
					local odds = 50 + (5 * getRelicLevel("jester_box"))
					if(chance >= 1 and chance <= odds) then
						EntityLoad("data/entities/items/pickup/goldnugget_50.xml", death_x, death_y)
					end
				end
			end
		end,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("jester_box") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 50 + (5 * getRelicLevel("jester_box"))
				if(chance >= 1 and chance <= odds) then
					multiplyProjectileBaseDamage(projectile_entity_id, 8)
				end
			end
		end,
	},
	{
		name 					= "greased_ring",
		ui_name					= "Greased Ring",
		categories				= "enemies,materials",
		effect_desc				= "When you hit an enemy covered in oil with your spells \nthere's a 30% chance a circle of oil will be cast around them.",
		effect_desc_short		= "Your spells create circle of oil when they hit oiled enemies.",
        precursor_types			= "ring",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onenemydamaged_function	= function(damage, message, entity_thats_responsible, is_fatal, projectile_thats_responsible )
			if(isRelicOwned("greased_ring") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 30 + (5 * getRelicLevel("greased_ring"))
				if(chance >= 1 and chance <= odds) then
					if(GameGetGameEffectCount(GetUpdatedEntityID(), "OILED") > 0) then
						if(message == "$damage_projectile") then
							local x, y = EntityGetTransform(GetUpdatedEntityID())
							EntityLoad("mods/ArcaneReliquary/files/entity_definitions/others/circle_oil.xml", x, y)
						end
					end
				end
			end
		end,
	},
	{
		name 					= "molten_ring",
		ui_name					= "Molten Ring",
		categories				= "enemies,materials",
		effect_desc				= "When you hit an enemy on fire with your spells \nthere's a 30% chance a circle of lava will be cast around them.",
		effect_desc_short		= "Your spells create circle of lava when they hit enemies on fire.",
        precursor_types			= "ring",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onenemydamaged_function	= function(damage, message, entity_thats_responsible, is_fatal, projectile_thats_responsible )
			if(isRelicOwned("molten_ring") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 30 + (5 * getRelicLevel("molten_ring"))
				if(chance >= 1 and chance <= odds) then
					if(GameGetGameEffectCount(GetUpdatedEntityID(), "ON_FIRE") > 0) then
						if(message == "$damage_projectile") then
							local x, y = EntityGetTransform(GetUpdatedEntityID())
							EntityLoad("mods/ArcaneReliquary/files/entity_definitions/others/circle_lava.xml", x, y)
						end
					end
				end
			end
		end,
	},
	{
		name 					= "sour_necklace",
		ui_name					= "Sour Necklace",
		categories				= "enemies,materials",
		effect_desc				= "When you hit an enemy covered in toxic sludge with your spells \nthere's a 30% chance a circle of acid will be cast around them.",
		effect_desc_short		= "Your spells create circle of acid when they hit enemies covered in toxic sludge.",
        precursor_types			= "necklace",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onenemydamaged_function	= function(damage, message, entity_thats_responsible, is_fatal, projectile_thats_responsible )
			if(isRelicOwned("sour_necklace") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 30 + (5 * getRelicLevel("sour_necklace"))
				if(chance >= 1 and chance <= odds) then
					if(GameGetGameEffectCount(GetUpdatedEntityID(), "RADIOACTIVE") > 0) then
						if(message == "$damage_projectile") then
							local x, y = EntityGetTransform(GetUpdatedEntityID())
							EntityLoad("mods/ArcaneReliquary/files/entity_definitions/others/circle_acid.xml", x, y)
						end
					end
				end
			end
		end,
	},
	{
		name 					= "toxic_gem",
		ui_name					= "Toxic Gem",
		categories				= "projectiles,materials",
		effect_desc				= "Spells you cast that explode on impact have a 20% chance of releasing toxic sludge on impact.",
		effect_desc_short		= "Your spells release toxic sludge.",
        precursor_types			= "elemental_gem",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("toxic_gem") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 20 + (5 * getRelicLevel("toxic_gem"))
				if(chance >= 1 and chance <= odds) then
					attachLoadEntityOnExplosiontoProjectile(projectile_entity_id, getLoadOnExplosion("launch_elemental_particles_radioactive_liquid").path, "radioactive_liquid")
				end
			end
		end,
	},
	{
		name 					= "corrosive_blood_gem",
		ui_name					= "Corrosive Blood Gem",
		categories				= "projectiles,materials",
		effect_desc				= "Spells you cast that explode on impact have a 20% chance of releasing corrosive blood on impact.",
		effect_desc_short		= "Your spells release corrosive blood.",
        precursor_types			= "elemental_gem",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("corrosive_blood_gem") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 20 + (5 * getRelicLevel("corrosive_blood_gem"))
				if(chance >= 1 and chance <= odds) then
					attachLoadEntityOnExplosiontoProjectile(projectile_entity_id, getLoadOnExplosion("launch_elemental_particles_corrosive_blood").path, "blood")
				end
			end
		end,
	},
	{
		name 					= "corrupted_necklace",
		ui_name					= "Corrupted Necklace",
		categories				= "enemies,materials",
		effect_desc				= "When you hit a wet enemy with your spells \nthere's a 30% chance a circle of poison will be cast around them.",
		effect_desc_short		= "Your spells create circle of poison when they hit wet enemies.",
        precursor_types			= "necklace",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onenemydamaged_function	= function(damage, message, entity_thats_responsible, is_fatal, projectile_thats_responsible )
			if(isRelicOwned("corrupted_necklace") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 30 + (5 * getRelicLevel("corrupted_necklace"))
				if(chance >= 1 and chance <= odds) then
					if(GameGetGameEffectCount(GetUpdatedEntityID(), "WET") > 0) then
						if(message == "$damage_projectile") then
							local x, y = EntityGetTransform(GetUpdatedEntityID())
							EntityLoad("mods/ArcaneReliquary/files/entity_definitions/others/circle_poison.xml", x, y)
						end
					end
				end
			end
		end,
	},
	{
		name 					= "stone_hand_explosive",
		ui_name					= "Stone Hand with Red Band",
		categories				= "companions",
		effect_desc				= "A pair of stone hands that shoot explosive magic stars \naids you. Requires an Elemental Band.",
		effect_desc_short		= "A pair of stone hands that shoot explosive magic stars aids you.",
        precursor_types			= "companion",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			
			attachMonkHandstoEntity(getPlayerEntity(), "monk_hand_explosive", "monk_arm_fx_explosive", "mortal")
		end,
		ondrop_function			= function()
			
			detachMonkHandsfromEntity(getPlayerEntity(), "monk_hand_explosive")
		end,
		requires_relic			= "elemental_band",
		combine_exclusive		= true,
		combine_requires_relics	= "stone_hand,explosive_mana_crystal",
	},
	{
		name 					= "stone_hand_acid",
		ui_name					= "Stone Hand with Green Band",
		categories				= "companions",
		effect_desc				= "A pair of stone hands that shoot acid magic stars \naids you. Requires an Elemental Band.",
		effect_desc_short		= "A pair of stone hands that shoot acid magic stars aids you.",
        precursor_types			= "companion",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			
			attachMonkHandstoEntity(getPlayerEntity(), "monk_hand_acid", "monk_arm_fx_acid", "mortal")
		end,
		ondrop_function			= function()
			
			detachMonkHandsfromEntity(getPlayerEntity(), "monk_hand_acid")
		end,
		requires_relic			= "elemental_band",
		combine_exclusive		= true,
		combine_requires_relics	= "stone_hand,acid_gem",
	},
	{
		name 					= "stone_hand_poly",
		ui_name					= "Stone Hand with Pink Band",
		categories				= "companions",
		effect_desc				= "A pair of stone hands that shoot mutating magic stars \naids you. Requires an Elemental Band.",
		effect_desc_short		= "A pair of stone hands that shoot mutating magic stars aids you.",
        precursor_types			= "companion",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			
			attachMonkHandstoEntity(getPlayerEntity(), "monk_hand_poly", "monk_arm_fx_poly", "mortal")
		end,
		ondrop_function			= function()
			
			detachMonkHandsfromEntity(getPlayerEntity(), "monk_hand_poly")
		end,
		requires_relic			= "elemental_band",
		combine_exclusive		= true,
		combine_requires_relics	= "stone_hand,poly_gem",
	},
	{
		name 					= "stone_hand_water",
		ui_name					= "Stone Hand with Blue Band",
		categories				= "companions",
		effect_desc				= "A pair of stone hands that shoot water magic stars \naids you. Requires an Elemental Band.",
		effect_desc_short		= "A pair of stone hands that shoot water magic stars aids you.",
        precursor_types			= "companion",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			
			attachMonkHandstoEntity(getPlayerEntity(), "monk_hand_water", "monk_arm_fx_water", "mortal")
		end,
		ondrop_function			= function()
			
			detachMonkHandsfromEntity(getPlayerEntity(), "monk_hand_water")
		end,
		requires_relic			= "elemental_band",
		combine_exclusive		= true,
		combine_requires_relics	= "stone_hand,water_gem",
	},
	{
		name 					= "stone_hand_poison",
		ui_name					= "Stone Hand with Purple Band",
		categories				= "companions",
		effect_desc				= "A pair of stone hands that shoot poison magic stars \naids you. Requires an Elemental Band.",
		effect_desc_short		= "A pair of stone hands that shoot poison magic stars aids you.",
        precursor_types			= "companion",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			
			attachMonkHandstoEntity(getPlayerEntity(), "monk_hand_poison", "monk_arm_fx_poison", "mortal")
		end,
		ondrop_function			= function()
			
			detachMonkHandsfromEntity(getPlayerEntity(), "monk_hand_poison")
		end,
		requires_relic			= "elemental_band",
		combine_exclusive		= true,
		combine_requires_relics	= "stone_hand,poison_gem",
	},
	{
		name 					= "stone_hand_urine",
		ui_name					= "Stone Hand with Yellow Band",
		categories				= "companions",
		effect_desc				= "A pair of stone hands that shoot urine magic stars \naids you. Requires an Elemental Band.",
		effect_desc_short		= "A pair of stone hands that shoot urine magic stars aids you.",
        precursor_types			= "companion",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			
			attachMonkHandstoEntity(getPlayerEntity(), "monk_hand_urine", "monk_arm_fx_urine", "mortal")
		end,
		ondrop_function			= function()
			
			detachMonkHandsfromEntity(getPlayerEntity(), "monk_hand_urine")
		end,
		requires_relic			= "elemental_band",
		combine_exclusive		= true,
		combine_requires_relics	= "stone_hand,urine_gem",
	},
	{
		name 					= "stone_hand_lava",
		ui_name					= "Stone Hand with Orange Band",
		categories				= "companions",
		effect_desc				= "A pair of stone hands that shoot lava magic stars \naids you. Requires an Elemental Band.",
		effect_desc_short		= "A pair of stone hands that shoot lava magic stars aids you.",
        precursor_types			= "companion",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			
			attachMonkHandstoEntity(getPlayerEntity(), "monk_hand_lava", "monk_arm_fx_lava", "mortal")
		end,
		ondrop_function			= function()
			
			detachMonkHandsfromEntity(getPlayerEntity(), "monk_hand_lava")
		end,
		requires_relic			= "elemental_band",
		combine_exclusive		= true,
		combine_requires_relics	= "stone_hand,lava_gem",
	},
	{
		name 					= "stone_hand_power_orb",
		ui_name					= "Stone Hand with Power Crystal",
		categories				= "companions",
		effect_desc				= "A pair of stone hands that shoot power orbs \naids you.",
		effect_desc_short		= "A pair of stone hands that shoot power orbs aids you.",
        precursor_types			= "companion",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
		
			attachMonkHandstoEntity(getPlayerEntity(), "monk_hand_power_orb", "monk_arm_fx_power_orb", "mortal")
		end,
		ondrop_function			= function()
			
			detachMonkHandsfromEntity(getPlayerEntity(), "monk_hand_power_orb")
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "stone_hand,power_stone",
	},
	{
		name 					= "stone_hand_power_orb_nuclear",
		ui_name					= "Stone Hand with Nuclear Power Crystal",
		categories				= "companions",
		effect_desc				= "A pair of stone hands that shoot nuclear power orbs \naids you.",
		effect_desc_short		= "A pair of stone hands that shoot nuclear power orbs aids you.",
        precursor_types			= "companion",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			
			attachMonkHandstoEntity(getPlayerEntity(), "monk_hand_power_orb_nuclear", "monk_arm_fx_power_orb_nuclear", "mortal")
		end,
		ondrop_function			= function()
			
			detachMonkHandsfromEntity(getPlayerEntity(), "monk_hand_power_orb_nuclear")
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "stone_hand_power_orb,stone_hand_nuclear",
	},
	{
		name 					= "stone_hand_melee",
		ui_name					= "Melee Stone Hand",
		categories				= "companions",
		effect_desc				= "A pair of stone hands that launch themselves towards enemies aids you.",
		effect_desc_short		= "A pair of stone hands that launch themselves towards enemies aids you.",
        precursor_types			= "companion",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
		
			attachMonkHandstoEntity(getPlayerEntity(), "monk_hand_melee", "monk_arm_fx_melee", "mortal")
		end,
		ondrop_function			= function()
			
			detachMonkHandsfromEntity(getPlayerEntity(), "monk_hand_melee")
		end,
		overtime_function		= function()
			if(isRelicOwned("stone_hand_melee") >= 1) then
				attachMonkHandstoEntity(getPlayerEntity(), "monk_hand_melee", "monk_arm_fx_melee", "mortal")
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "emerald_gauntlet,stone_hand",
	},
	{
		name 					= "stone_hand_overheated",
		ui_name					= "Overheated Stone Hand",
		categories				= "companions",
		effect_desc				= "A pair of overheated stone hands that launch themselves towards enemies aids you.",
		effect_desc_short		= "A pair of overheated stone hands that launch themselves towards enemies aids you.",
        precursor_types			= "companion",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			
			attachMonkHandstoEntity(getPlayerEntity(), "monk_hand_overheated", "monk_arm_fx_overheated", "mortal")
		end,
		ondrop_function			= function()
			
			detachMonkHandsfromEntity(getPlayerEntity(), "monk_hand_overheated")
		end,
		overtime_function		= function()
			if(isRelicOwned("stone_hand_overheated") >= 1) then
				attachMonkHandstoEntity(getPlayerEntity(), "monk_hand_overheated", "monk_arm_fx_overheated", "mortal")
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "stone_hand_melee,blaze_pearl",
	},
	{
		name 					= "stone_hand_frozen",
		ui_name					= "Frozen Stone Hand",
		categories				= "companions",
		effect_desc				= "A pair of frozen stone hands that launch themselves towards enemies aids you.",
		effect_desc_short		= "A pair of frozen stone hands that launch themselves towards enemies aids you.",
        precursor_types			= "companion",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			
			attachMonkHandstoEntity(getPlayerEntity(), "monk_hand_frozen", "monk_arm_fx_frozen", "mortal")
		end,
		ondrop_function			= function()
			
			detachMonkHandsfromEntity(getPlayerEntity(), "monk_hand_frozen")
		end,
		overtime_function		= function()
			if(isRelicOwned("stone_hand_frozen") >= 1) then
				attachMonkHandstoEntity(getPlayerEntity(), "monk_hand_frozen", "monk_arm_fx_frozen", "mortal")
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "stone_hand_melee,ice_artifact",
	},
	{
		name 					= "stone_hand_charged",
		ui_name					= "Charged Stone Hand",
		categories				= "companions",
		effect_desc				= "A pair of charged stone hands that launch themselves towards enemies aids you.",
		effect_desc_short		= "A pair of charged stone hands that launch themselves towards enemies aids you.",
        precursor_types			= "companion",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			
			attachMonkHandstoEntity(getPlayerEntity(), "monk_hand_charged", "monk_arm_fx", "mortal")
		end,
		ondrop_function			= function()
			
			detachMonkHandsfromEntity(getPlayerEntity(), "monk_hand_charged")
		end,
		overtime_function		= function()
			if(isRelicOwned("stone_hand_charged") >= 1) then
				attachMonkHandstoEntity(getPlayerEntity(), "monk_hand_charged", "monk_arm_fx", "mortal")
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "stone_hand_melee,charged_necklace",
	},
	{
		name 					= "stone_hand_nuclear",
		ui_name					= "Nuclear Stone Hand",
		categories				= "companions",
		effect_desc				= "A pair of nuclear stone hands that launch themselves towards enemies aids you.",
		effect_desc_short		= "A pair of nuclear stone hands that launch themselves towards enemies aids you.",
        precursor_types			= "companion",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			
			attachMonkHandstoEntity(getPlayerEntity(), "monk_hand_nuclear", "monk_arm_fx_nuclear", "mortal")
		end,
		ondrop_function			= function()
			
			detachMonkHandsfromEntity(getPlayerEntity(), "monk_hand_nuclear")
		end,
		overtime_function		= function()
			if(isRelicOwned("stone_hand_nuclear") >= 1) then
				attachMonkHandstoEntity(getPlayerEntity(), "monk_hand_nuclear", "monk_arm_fx_nuclear", "mortal")
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "stone_hand_overheated,uranium_ingot",
	},
	{
		name 					= "stone_hand_spectral",
		ui_name					= "Spectral Stone Hand",
		categories				= "companions",
		effect_desc				= "A pair of spectral stone hands that launch themselves towards enemies aids you.",
		effect_desc_short		= "A pair of spectral stone hands that launch themselves towards enemies aids you.",
        precursor_types			= "companion",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			
			attachMonkHandstoEntity(getPlayerEntity(), "monk_hand_spectral", "monk_arm_fx_spectral", "mortal")
		end,
		ondrop_function			= function()
			
			detachMonkHandsfromEntity(getPlayerEntity(), "monk_hand_spectral")
		end,
		overtime_function		= function()
			if(isRelicOwned("stone_hand_spectral") >= 1) then
				attachMonkHandstoEntity(getPlayerEntity(), "monk_hand_spectral", "monk_arm_fx_spectral", "mortal")
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "stone_hand_melee,molecular_agitator",
	},
	{
		name 					= "stone_hand",
		ui_name					= "Stone Hand",
		categories				= "companions",
		effect_desc				= "A pair of stone hands that shoot magic orbs aids you.",
		effect_desc_short		= "A pair of stone hands that shoot magic orbs aids you.",
        precursor_types			= "companion",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			
			attachMonkHandstoEntity(getPlayerEntity(), "monk_hand", "monk_arm_fx", "mortal")
		end,
		ondrop_function			= function()
		
			detachMonkHandsfromEntity(getPlayerEntity(), "monk_hand")
		end,
	},
	{
		name 					= "elemental_band",
		ui_name					= "Elemental Band",
		categories				= "projectiles",
		effect_desc				= "For every relic you possess related to \nmaterials, your spells gain a small damage boost.",
		effect_desc_short		= "Owning material-related relics gives you spell damage bonus.",
        precursor_types			= "magic_artifact",
		upgrade_effect_desc		= "For each upgrade level, damage boost increases.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("elemental_band") >= 1) then
				local bonus_damage = 0
				local damage_threshold = 0.5 + (0.5 * getRelicLevel("elemental_band"))
				local owned_relics = getOwnedRelics()
				for i,owned_relic in ipairs(owned_relics) do
					if(string.find(getRelic(EntityGetName(owned_relic)).categories, "materials")) then
						--GamePrint("###" .. getRelic(owned_relic).name .. " is of materials category")
						bonus_damage = bonus_damage + damage_threshold
					end
				end
				--GamePrint("###" .. bonus_damage)
				addProjectileBaseDamage(projectile_entity_id, bonus_damage)
			end
		end,
	},
	{
		name 					= "gravity_stone_red",
		ui_name					= "Red Gravity Stone",
		categories				= "player",
		effect_desc				= "Red energy spheres capable of destroying enemy projectiles orbit around you.",
		effect_desc_short		= "Red energy spheres capable of destroying enemy projectiles orbit around you.",
        precursor_types			= "gravity_stone",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		ondrop_function			= function()
			detachRotatingShieldfromEntity(getPlayerEntity(), "rotating_shield_projectile")
		end,
		overtime_function		= function()
			if(isRelicOwned("gravity_stone_red") >= 1) then
				if(not entityHasRotatingShield(getPlayerEntity(), "rotating_shield_projectile")) then -- this makes sure the orbiting projectiles are loaded again if the player is polymorphed
					local start_id = EntityLoad("mods/ArcaneReliquary/files/entity_definitions/custom_projectiles/rotating_shield_start.xml")
					setInternalVariableValue(start_id, "host_entity_id", "value_int", getPlayerEntity())
					setInternalVariableValue(start_id, "rotating_projectile_definition", "value_string", "mods/ArcaneReliquary/files/entity_definitions/custom_projectiles/rotating_shield_part.xml")
					setInternalVariableValue(start_id, "rotating_projectiles_amount", "value_int", 2)
					setInternalVariableValue(start_id, "radius", "value_int", 20)
					setInternalVariableValue(start_id, "rotation_speed", "value_int", 10)
				end
			end
		end,
	},
	{
		name 					= "gravity_stone_teal",
		ui_name					= "Teal Gravity Stone",
		categories				= "player",
		effect_desc				= "Teal energy spheres capable of redirecting enemy projectiles orbit around you.",
		effect_desc_short		= "Teal energy spheres capable of redirecting enemy projectiles orbit around you.",
        precursor_types			= "gravity_stone",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		ondrop_function			= function()
			detachRotatingShieldfromEntity(getPlayerEntity(), "rotating_shield_projectile_change_direction")
		end,
		overtime_function		= function()
			if(isRelicOwned("gravity_stone_teal") >= 1) then
				if(not entityHasRotatingShield(getPlayerEntity(), "rotating_shield_projectile_change_direction")) then -- this makes sure the orbiting projectiles are loaded again if the player is polymorphed
					local start_id = EntityLoad("mods/ArcaneReliquary/files/entity_definitions/custom_projectiles/rotating_shield_start.xml")
					setInternalVariableValue(start_id, "host_entity_id", "value_int", getPlayerEntity())
					setInternalVariableValue(start_id, "rotating_projectile_definition", "value_string", "mods/ArcaneReliquary/files/entity_definitions/custom_projectiles/rotating_shield_change_direction_part.xml")
					setInternalVariableValue(start_id, "rotating_projectiles_amount", "value_int", 4)
					setInternalVariableValue(start_id, "radius", "value_int", 25)
					setInternalVariableValue(start_id, "rotation_speed", "value_int", 5)
				end
			end
		end,
	},
	{
		name 					= "gravity_stone_yellow",
		ui_name					= "Yellow Gravity Stone",
		categories				= "player",
		effect_desc				= "A yellow energy sphere capable or dragging enemies, dealing slice damage, orbits around you.",
		effect_desc_short		= "A yellow energy sphere capable or dragging enemies orbits around you.",
        precursor_types			= "gravity_stone",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		ondrop_function			= function()
			detachRotatingShieldfromEntity(getPlayerEntity(), "rotating_shield_projectile_drag")
		end,
		overtime_function		= function()
			if(isRelicOwned("gravity_stone_yellow") >= 1) then
				if(not entityHasRotatingShield(getPlayerEntity(), "rotating_shield_projectile_drag")) then -- this makes sure the orbiting projectiles are loaded again if the player is polymorphed
					local start_id = EntityLoad("mods/ArcaneReliquary/files/entity_definitions/custom_projectiles/rotating_shield_start.xml")
					setInternalVariableValue(start_id, "host_entity_id", "value_int", getPlayerEntity())
					setInternalVariableValue(start_id, "rotating_projectile_definition", "value_string", "mods/ArcaneReliquary/files/entity_definitions/custom_projectiles/rotating_shield_drag_part.xml")
					setInternalVariableValue(start_id, "rotating_projectiles_amount", "value_int", 1)
					setInternalVariableValue(start_id, "radius", "value_int", 28)
					setInternalVariableValue(start_id, "rotation_speed", "value_int", 10)
				end
			end
		end,
	},
	{
		name 					= "gravity_stone_blue",
		ui_name					= "Blue Gravity Stone",
		categories				= "player",
		effect_desc				= "Blue energy spheres that target enemies and shoot explosive orbs orbit around you.",
		effect_desc_short		= "Blue energy spheres that target enemies and shoot explosive orbs orbit around you.",
        precursor_types			= "gravity_stone",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		ondrop_function			= function()
			detachRotatingShieldfromEntity(getPlayerEntity(), "rotating_shield_projectile_target")
		end,
		overtime_function		= function()
			if(isRelicOwned("gravity_stone_blue") >= 1) then
				if(not entityHasRotatingShield(getPlayerEntity(), "rotating_shield_projectile_target")) then -- this makes sure the orbiting projectiles are loaded again if the player is polymorphed
					local start_id = EntityLoad("mods/ArcaneReliquary/files/entity_definitions/custom_projectiles/rotating_shield_start.xml")
					setInternalVariableValue(start_id, "host_entity_id", "value_int", getPlayerEntity())
					setInternalVariableValue(start_id, "rotating_projectile_definition", "value_string", "mods/ArcaneReliquary/files/entity_definitions/custom_projectiles/rotating_shield_target_part.xml")
					setInternalVariableValue(start_id, "rotating_projectiles_amount", "value_int", 3)
					setInternalVariableValue(start_id, "radius", "value_int", 18)
					setInternalVariableValue(start_id, "rotation_speed", "value_int", 2)
				end
			end
		end,
	},
	{
		name 					= "stone_rat",
		ui_name					= "Stone Rat",
		categories				= "projectiles,minions",
		effect_desc				= "Your spells have a 30% chance of being transformed into friendly bomb rats.",
		effect_desc_short		= "Your spells summon friendly bomb rats.",
        precursor_types			= "magic_artifact",
		upgrade_effect_desc		= "For each upgrade level, spawn chance increases by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("stone_rat") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 30 + (5 * getRelicLevel("stone_rat"))
				if(chance >= 1 and chance <= odds) then
					addNewLuaCallback(projectile_entity_id, "overtime", "mods/ArcaneReliquary/files/scripts/projectiles/destroy_and_spawn_bomb_rat.lua", 5, false, "destroy_and_spawn_bomb_rat")
				end
			end
		end,
	},
	{
		name 					= "stone_bat",
		ui_name					= "Stone Bat",
		categories				= "projectiles,minions",
		effect_desc				= "Your spells have a 30% chance of being transformed into friendly bomb bats.",
		effect_desc_short		= "Your spells summon friendly bomb bats.",
        precursor_types			= "magic_artifact",
		upgrade_effect_desc		= "For each upgrade level, spawn chance increases by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("stone_bat") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 30 + (5 * getRelicLevel("stone_bat"))
				if(chance >= 1 and chance <= odds) then
					addNewLuaCallback(projectile_entity_id, "overtime", "mods/ArcaneReliquary/files/scripts/projectiles/destroy_and_spawn_bomb_bat.lua", 5, false, "destroy_and_spawn_bomb_bat")
				end
			end
		end,
	},
	{
		name 					= "scarab_shell",
		ui_name					= "Scarab Shell",
		categories				= "companions",
		effect_desc				= "A magic ankh follows you, eventually shooting barrages of magic \norbs if you take damage from enemy projectiles or explosions.",
		effect_desc_short		= "A magic ankh follows you, retaliating eventually when you're damaged.",
        precursor_types			= "companion",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			attachCompaniontoEntity(getPlayerEntity(), "ankh", "mortal")
		end,
		ondrop_function			= function()
			detachCompanionfromEntity(getPlayerEntity(), "ankh")
		end,
		ondamaged_function		= function(damage, message, entity_thats_responsible, is_fatal, projectile_thats_responsible )
			if(isRelicOwned("scarab_shell") >= 1) then
				if(message == "$damage_projectile" or message == "$damage_explosion") then
					SetRandomSeed( GameGetFrameNum(), math.random(1,50))
					local chance = Random(1,2)
					if(chance == 1) then
						local player_children = EntityGetAllChildren(getPlayerEntity())
						if player_children == nil then return end
						for i,child_id in ipairs(player_children) do
							if(EntityGetName(child_id) == "ankh") then
								local companion_x, companion_y = EntityGetTransform(child_id)
								EntityLoad("mods/ArcaneReliquary/files/entity_definitions/load_on_explosion/launch_circular_projectiles_magic_orbs_yellow.xml", companion_x, companion_y)
							end
						end
					end
				end
			end
		end,
	},
	{
		name 					= "question_block",
		ui_name					= "Question Block",
		categories				= "player",
		effect_desc				= "When you drill through terrain using the Drilling Hat, there's a \nchance gold nuggets will drop, but each time this happens you take minimal damage.",
		effect_desc_short		= "If you posses the Drilling Hat, you might drop gold when drilling through terrain.",
        precursor_types			= "magic_artifact",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			addNewSimplifiedInternalVariable(getPlayerEntity(), "drilling_hat_drop_gold")
		end,
		ondrop_function			= function()
			removeSimplifiedInternalVariable(getPlayerEntity(), "drilling_hat_drop_gold")
		end,
		requires_relic			= "drilling_hat",
	},
	{
		name 					= "spring",
		ui_name					= "Spring",
		categories				= "player,kick",
		effect_desc				= "Ground enemies you kick are sent upwards, and \nif they hit the ceiling, they explode. \nIt also synergizes with Sniper Scope relic effect.",
		effect_desc_short		= "Ground enemies you kick are sent upwards.",
        precursor_types			= "boot,gadget",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onenemykicked_function	= function (entity_who_kicked)
			if(isRelicOwned("spring") >= 1) then
				LoadGameEffectEntityTo( GetUpdatedEntityID(), getCustomEffect("effect_force_up_bonk").path )
			end
		end,
		onextendedkick_function	= function( entity_id, extended_kick_id)
			if(isRelicOwned("spring") >= 1) then
				attachLoadEffectOntoEntitiestoProjectile(extended_kick_id, getCustomEffect("effect_force_up_bonk").path, "spark_white")
			end
		end,
	},
	{
		name 					= "tick",
		ui_name					= "Tick",
		categories				= "player,kick,status_effects",
		effect_desc				= "Enemies you kick bleed for a long time. \nIt also synergizes with Sniper Scope relic effect.",
		effect_desc_short		= "Enemies you kick bleed for a long time.",
        precursor_types			= "boot",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onenemykicked_function	= function (entity_who_kicked)
			if(isRelicOwned("tick") >= 1) then
				LoadGameEffectEntityTo( GetUpdatedEntityID(), getCustomEffect("effect_bleeding_longer").path )
			end
		end,
		onextendedkick_function	= function( entity_id, extended_kick_id)
			if(isRelicOwned("tick") >= 1) then
				attachLoadEffectOntoEntitiestoProjectile(extended_kick_id, getCustomEffect("effect_bleeding_longer").path, "blood")
			end
		end,
	},
	{
		name 					= "toxic_gland",
		ui_name					= "Toxic Gland",
		categories				= "player,kick,materials",
		effect_desc				= "Enemies you kick emit a trail of acid. \nIt also synergizes with Sniper Scope relic effect.",
		effect_desc_short		= "Enemies you kick emit a trail of acid.",
        precursor_types			= "boot",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onenemykicked_function	= function (entity_who_kicked)
			if(isRelicOwned("toxic_gland") >= 1) then
				LoadGameEffectEntityTo( GetUpdatedEntityID(), getCustomEffect("effect_acid_trail").path )
			end
		end,
		onextendedkick_function	= function( entity_id, extended_kick_id)
			if(isRelicOwned("toxic_gland") >= 1) then
				attachLoadEffectOntoEntitiestoProjectile(extended_kick_id, getCustomEffect("effect_acid_trail").path, "acid")
			end
		end,
	},
	{
		name 					= "anticoagulant_flask",
		ui_name					= "Anticoagulant Flask",
		categories				= "projectiles,enemies,materials",
		effect_desc				= "Each time you cast a spell, there's a 50% chance \nyou'll bleed your current blood type without taking any damage.",
		effect_desc_short		= "Casting spells induce harmlessly bleeding your current blood type.",
        precursor_types			= "flask",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("anticoagulant_flask") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 50 + (5 * getRelicLevel("anticoagulant_flask"))
				if(chance >= 1 and chance <= odds) then
					LoadGameEffectEntityTo( GetUpdatedEntityID(), getCustomEffect("effect_current_blood_trail").path )
				end
			end
		end,
	},
	{
		name 					= "midas_essence",
		ui_name					= "Midas Essence",
		categories				= "player,materials",
		effect_desc				= "Each time you take damage from enemy projectiles or melee \nattacks, there's a 10% chance you'll emit a gold dust trail.",
		effect_desc_short		= "Taking projectile or melee damage induces you to emit a trail of gold.",
        precursor_types			= "flask",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		ondamaged_function		= function(damage, message, entity_thats_responsible, is_fatal, projectile_thats_responsible )
			if(isRelicOwned("midas_essence") >= 1) then
				if(message == "$damage_projectile" or message == "$damage_melee") then
					SetRandomSeed( GameGetFrameNum(), math.random(1,50))
					local chance = Random(1,100)
					local odds = 10 + (5 * getRelicLevel("midas_essence"))
					if(chance >= 1 and chance <= odds) then
						LoadGameEffectEntityTo( GetUpdatedEntityID(), getCustomEffect("effect_gold_trail").path )
					end
				end
			end
		end,
	},
	{
		name 					= "freezing_liquid_flask",
		ui_name					= "Freezing Liquid Flask",
		categories				= "player,kick,materials",
		effect_desc				= "Enemies you kick emit a trail of freezing liquid. \nIt also synergizes with Sniper Scope relic effect.",
		effect_desc_short		= "Enemies you kick emit a trail of freezing liquid.",
        precursor_types			= "flask,boot",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onenemykicked_function	= function (entity_who_kicked)
			if(isRelicOwned("freezing_liquid_flask") >= 1) then
				LoadGameEffectEntityTo( GetUpdatedEntityID(), getCustomEffect("effect_freezing_liquid_trail").path )
			end
		end,
		onextendedkick_function	= function( entity_id, extended_kick_id)
			if(isRelicOwned("freezing_liquid_flask") >= 1) then
				attachLoadEffectOntoEntitiestoProjectile(extended_kick_id, getCustomEffect("effect_freezing_liquid_trail").path, "diamond")
			end
		end,
	},
	{
		name 					= "hermes_sandal",
		ui_name					= "Hermes Sandal",
		categories				= "player",
		effect_desc				= "Each time you take damage from enemy projectiles, explosions or melee \nattacks, there's a 30% chance you'll move faster for a while (a blue aura around you will indicate this effect).",
		effect_desc_short		= "Taking projectile, explosion or melee damage has a chance of making you temporarily faster.",
        precursor_types			= "boot",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		ondamaged_function		= function(damage, message, entity_thats_responsible, is_fatal, projectile_thats_responsible )
			if(isRelicOwned("hermes_sandal") >= 1) then
				if(message == "$damage_projectile" or message == "$damage_melee" or message == "$damage_explosion") then
					SetRandomSeed( GameGetFrameNum(), math.random(1,50))
					local chance = Random(1,100)
					local odds = 30 + (5 * getRelicLevel("hermes_sandal"))
					if(chance >= 1 and chance <= odds) then
						LoadGameEffectEntityTo( GetUpdatedEntityID(), getCustomEffect("effect_movement_faster_short").path )
					end
				end
			end
		end,
	},
	{
		name 					= "hourglass",
		ui_name					= "Hourglass",
		categories				= "player,kick",
		effect_desc				= "Enemies you kick are teleported away. \nIt also synergizes with Sniper Scope relic effect.",
		effect_desc_short		= "Enemies you kick are teleported away.",
        precursor_types			= "boot",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onenemykicked_function	= function (entity_who_kicked)
			if(isRelicOwned("hourglass") >= 1) then
				LoadGameEffectEntityTo( GetUpdatedEntityID(), "data/entities/misc/effect_teleportation.xml" )
			end
		end,
		onextendedkick_function	= function( entity_id, extended_kick_id)
			if(isRelicOwned("hourglass") >= 1) then
				attachLoadEffectOntoEntitiestoProjectile(extended_kick_id, "data/entities/misc/effect_teleportation.xml", "magic_liquid_teleportation")
			end
		end,
	},
	{
		name 					= "displacing_stone",
		ui_name					= "Displacing Stone",
		categories				= "player",
		effect_desc				= "Each time you take damage from enemy projectiles or melee \nattacks, there's a 30% chance you'll be teleported away.",
		effect_desc_short		= "Taking projectile or melee damage has a chance of teleporting you away.",
        precursor_types			= "effect_stone",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		ondamaged_function		= function(damage, message, entity_thats_responsible, is_fatal, projectile_thats_responsible )
			if(isRelicOwned("displacing_stone") >= 1) then
				if(message == "$damage_projectile" or message == "$damage_melee") then
					SetRandomSeed( GameGetFrameNum(), math.random(1,50))
					local chance = Random(1,100)
					local odds = 30 + (5 * getRelicLevel("displacing_stone"))
					if(chance >= 1 and chance <= odds) then
						LoadGameEffectEntityTo( GetUpdatedEntityID(), "data/entities/misc/effect_teleportation.xml" )
					end
				end
			end
		end,
	},
	{
		name 					= "sniper_scope",
		ui_name					= "Sniper Scope",
		categories				= "player,kick",
		effect_desc				= "Provided you're currently holding a wand or item, pressing the kick button will \nunleash an energy wave that deals melee damage, and if you posses any \nkick-enhancing perks or relics, it will synergize with them.",
		effect_desc_short		= "Long range kicks.",
        precursor_types			= "boot",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			addNewLuaCallback(getPlayerEntity(), "overtime", "mods/ArcaneReliquary/files/scripts/player/extended_kick_listener.lua", 1, false, "extended_kick_listener")
		end,
		ondrop_function			= function()
			removeCallback(getPlayerEntity(), "extended_kick_listener")
		end
	},
	{
		name 					= "obsidian",
		ui_name					= "Obsidian",
		categories				= "player,kick,materials",
		effect_desc				= "Enemies you kick emit a trail of lava. \nIt also synergizes with Sniper Scope relic effect.",
		effect_desc_short		= "Enemies you kick emit a trail of lava.",
        precursor_types			= "boot",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onenemykicked_function	= function (entity_who_kicked)
			if(isRelicOwned("obsidian") >= 1) then
				LoadGameEffectEntityTo( GetUpdatedEntityID(), getCustomEffect("effect_lava_trail").path )
			end
		end,
		onextendedkick_function	= function( entity_id, extended_kick_id)
			if(isRelicOwned("obsidian") >= 1) then
				attachLoadEffectOntoEntitiestoProjectile(extended_kick_id, getCustomEffect("effect_lava_trail").path, "lava")
			end
		end,
	},
	{
		name 					= "vile_stone",
		ui_name					= "Vile Stone",
		categories				= "player,kick,materials",
		effect_desc				= "Enemies you kick emit a trail of poison. \nIt also synergizes with Sniper Scope relic effect.",
		effect_desc_short		= "Enemies you kick emit a trail of poison.",
        precursor_types			= "boot,magic_artifact,effect_stone",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onenemykicked_function	= function (entity_who_kicked)
			if(isRelicOwned("vile_stone") >= 1) then
				LoadGameEffectEntityTo( GetUpdatedEntityID(), getCustomEffect("effect_poison_trail").path )
			end
		end,
		onextendedkick_function	= function( entity_id, extended_kick_id)
			if(isRelicOwned("vile_stone") >= 1) then
				attachLoadEffectOntoEntitiestoProjectile(extended_kick_id, getCustomEffect("effect_poison_trail").path, "spark_purple_bright")
			end
		end,
	},
	{
		name 					= "canned_beans",
		ui_name					= "Canned Beans",
		categories				= "player,kick,materials",
		effect_desc				= "Enemies you kick emit a trail of toxic gas. \nIt also synergizes with Sniper Scope relic effect.",
		effect_desc_short		= "Enemies you kick emit a trail of toxic gas.",
        precursor_types			= "boot",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onenemykicked_function	= function (entity_who_kicked)
			if(isRelicOwned("canned_beans") >= 1) then
				LoadGameEffectEntityTo( GetUpdatedEntityID(), getCustomEffect("effect_radioactive_gas_trail").path )
			end
		end,
		onextendedkick_function	= function( entity_id, extended_kick_id)
			if(isRelicOwned("canned_beans") >= 1) then
				attachLoadEffectOntoEntitiestoProjectile(extended_kick_id, getCustomEffect("effect_radioactive_gas_trail").path, "radioactive_gas")
			end
		end,
	},
	{
		name 					= "toy_turret",
		ui_name					= "Toy Turret",
		categories				= "enemies,items,minions",
		effect_desc				= "Enemies you kill have a 20% chance of dropping briefcases on death. \nThese briefcases can be thrown on the ground to deploy friendly turrets that aid you in combat.",
		effect_desc_short		= "Killed enemies drop briefcases that deploy friendly turrets.",
        precursor_types			= "gadget",
		upgrade_effect_desc		= "For each upgrade level, spawn chance increases by 5%.",
		is_upgradable 			= true,
		onenemykill_function	= function(damage_type_bit_field, damage_message, entity_thats_responsible, drop_items)
			if(isRelicOwned("toy_turret") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 20 + (5 * getRelicLevel("toy_turret"))
				if(chance >= 1 and chance <= odds) then
					local death_x, death_y = EntityGetTransform(GetUpdatedEntityID())
					EntityLoad("mods/ArcaneReliquary/files/entity_definitions/items/briefcase.xml", death_x, death_y)
				end
			end
		end,
	},
	{
		name 					= "toy_turret_laser",
		ui_name					= "Toy Laser Turret",
		categories				= "enemies,items,minions",
		effect_desc				= "Enemies you kill have a 20% chance of dropping briefcases on death. \nThese briefcases can be thrown on the ground to deploy friendly laser turrets that aid you in combat.",
		effect_desc_short		= "Killed enemies drop briefcases that deploy friendly laser turrets.",
        precursor_types			= "gadget",
		upgrade_effect_desc		= "For each upgrade level, spawn chance increases by 5%.",
		is_upgradable 			= true,
		onenemykill_function	= function(damage_type_bit_field, damage_message, entity_thats_responsible, drop_items)
			if(isRelicOwned("toy_turret_laser") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 20 + (5 * getRelicLevel("toy_turret_laser"))
				if(chance >= 1 and chance <= odds) then
					local death_x, death_y = EntityGetTransform(GetUpdatedEntityID())
					EntityLoad("mods/ArcaneReliquary/files/entity_definitions/items/briefcase_laser.xml", death_x, death_y)
				end
			end
		end,
	},
	{
		name 					= "toy_turret_rocket",
		ui_name					= "Toy Rocket Turret",
		categories				= "enemies,items,minions",
		effect_desc				= "Enemies you kill have a 20% chance of dropping briefcases on death. \nThese briefcases can be thrown on the ground to deploy friendly rocket turrets that aid you in combat.",
		effect_desc_short		= "Killed enemies drop briefcases that deploy friendly rocket turrets.",
        precursor_types			= "gadget",
		upgrade_effect_desc		= "For each upgrade level, spawn chance increases by 5%.",
		is_upgradable 			= true,
		onenemykill_function	= function(damage_type_bit_field, damage_message, entity_thats_responsible, drop_items)
			if(isRelicOwned("toy_turret_rocket") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 20 + (5 * getRelicLevel("toy_turret_rocket"))
				if(chance >= 1 and chance <= odds) then
					local death_x, death_y = EntityGetTransform(GetUpdatedEntityID())
					EntityLoad("mods/ArcaneReliquary/files/entity_definitions/items/briefcase_rocket.xml", death_x, death_y)
				end
			end
		end,
	},
	{
		name 					= "mint_leaf",
		ui_name					= "Mint Leaf",
		categories				= "player,kick,status_effects",
		effect_desc				= "Enemies you kick are inflicted with the slap effect. \nIt also synergizes with Sniper Scope relic effect.",
		effect_desc_short		= "Enemies you kick are inflicted with the slap effect.",
        precursor_types			= "boot",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onenemykicked_function	= function (entity_who_kicked)
			if(isRelicOwned("mint_leaf") >= 1) then
				LoadGameEffectEntityTo( GetUpdatedEntityID(), getCustomEffect("effect_slap").path )
			end
		end,
		onextendedkick_function	= function( entity_id, extended_kick_id)
			if(isRelicOwned("mint_leaf") >= 1) then
				attachLoadEffectOntoEntitiestoProjectile(extended_kick_id, getCustomEffect("effect_slap").path, "plasma_fading_green")
			end
		end,
	},
	{
		name 					= "nitro_container",
		ui_name					= "Nitro Container",
		categories				= "player,kick,status_effects",
		effect_desc				= "Enemies you kick are inflicted with the exploding effect. \nIt also synergizes with Sniper Scope relic effect.",
		effect_desc_short		= "Enemies you kick are inflicted with the exploding effect.",
        precursor_types			= "boot",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onenemykicked_function	= function (entity_who_kicked)
			if(isRelicOwned("nitro_container") >= 1) then
				LoadGameEffectEntityTo( GetUpdatedEntityID(), getCustomEffect("effect_exploding").path )
			end
		end,
		onextendedkick_function	= function( entity_id, extended_kick_id)
			if(isRelicOwned("nitro_container") >= 1) then
				attachLoadEffectOntoEntitiestoProjectile(extended_kick_id, getCustomEffect("effect_exploding").path, "spark_yellow")
			end
		end,
	},
	{
		name 					= "electrical_plug",
		ui_name					= "Electrical Plug",
		categories				= "player,kick,status_effects",
		effect_desc				= "Enemies you kick are electrocuted. \nIt also synergizes with Sniper Scope relic effect.",
		effect_desc_short		= "Enemies you kick are electrocuted.",
        precursor_types			= "boot",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onenemykicked_function	= function (entity_who_kicked)
			if(isRelicOwned("electrical_plug") >= 1) then
				LoadGameEffectEntityTo( GetUpdatedEntityID(), "data/entities/misc/effect_electricity.xml" )
			end
		end,
		onextendedkick_function	= function( entity_id, extended_kick_id)
			if(isRelicOwned("electrical_plug") >= 1) then
				attachLoadEffectOntoEntitiestoProjectile(extended_kick_id, "data/entities/misc/effect_electricity.xml", "spark_blue_dark")
			end
		end,
	},
	{
		name 					= "freezing_gas_container",
		ui_name					= "Freezing Gas Container",
		categories				= "player,kick,status_effects",
		effect_desc				= "Enemies you kick have a 50% chance of being frozen. \nIt also synergizes with Sniper Scope relic effect.",
		effect_desc_short		= "Enemies you kick have a 50% chance of being frozen.",
        precursor_types			= "boot",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onenemykicked_function	= function (entity_who_kicked)
			if(isRelicOwned("freezing_gas_container") >= 1) then
				LoadGameEffectEntityTo( GetUpdatedEntityID(), getCustomEffect("effect_frozen_random").path )
			end
		end,
		onextendedkick_function	= function( entity_id, extended_kick_id)
			if(isRelicOwned("freezing_gas_container") >= 1) then
				attachLoadEffectOntoEntitiestoProjectile(extended_kick_id, getCustomEffect("effect_frozen_random").path, "plasma_fading")
			end
		end,
	},
	{
		name 					= "frozen_helmet",
		ui_name					= "Frozen Helmet",
		categories				= "player,status_effects",
		effect_desc				= "You are now immune to freezing from any source \n(You'll still be turned blue if inflicted with freezing, but your movement will not be restricted anymore).",
		effect_desc_short		= "You are now immune to freezing from any source.",
        precursor_types			= "magic_artifact",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			LoadGameEffectEntityTo( getPlayerEntity(), getCustomEffect("effect_freeze_protection").path )
		end,
		ondrop_function			= function()
			local player_children = EntityGetAllChildren(getPlayerEntity())
			if player_children == nil then return end
			for i,child in ipairs(player_children) do
				if(EntityGetName(child) == "freeze_protection") then
					EntityKill(child)
					break
				end
			end
		end,
		excludes_relics			= "wooden_helmet,blessed_helmet",
		excludes_relics_desc	= "It can't be used along with other freeze immunity related relics",
		excludes_relics_tags	= "freeze_immunity",
	},
	{
		name 					= "plasma_boot",
		ui_name					= "Plasma Boot",
		categories				= "player",
		effect_desc				= "Falling and stepping on enemies makes them shoot plasma orbs sideways.",
		effect_desc_short		= "Stepping on enemies makes them shoot plasma orbs sideways.",
        precursor_types			= "boot",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			if(not entityHasCallback(getPlayerEntity(), "stomp_listener")) then -- if no other stomp related relics added required callback, adds it
				--GamePrint(">>> stomp relic picked, stomp callback added...")
				addNewLuaCallback(getPlayerEntity(), "overtime", "mods/ArcaneReliquary/files/scripts/player/stomp_listener.lua", 2, false, "stomp_listener")
			end
			addNewSimplifiedInternalVariable(getPlayerEntity(), "relic_using_stomp_relic") -- for each stomp related relic, a new simplified variable like this is added
		end,
		ondrop_function			= function()
			removeFirstSimplifiedInternalVariable(getPlayerEntity(), "relic_using_stomp_relic") -- removes just the first variable found
			
			if(countSimplifiedInternalVariable(getPlayerEntity(), "relic_using_stomp_relic") == 0) then -- removes stomp callback only if no other relics are using it
				--GamePrint(">>> stomp relic dropped, no other related relics present, removing stomp callback...")
				removeCallback(getPlayerEntity(), "stomp_listener")
			end
		end,
		onenemystomped_function	= function(entity_id, creature_id)
			if(isRelicOwned("plasma_boot") >= 1) then
				-- Shoots side orbs
				local creature_x, creature_y = EntityGetTransform(creature_id)
				local orb_right_id = EntityLoad("mods/ArcaneReliquary/files/entity_definitions/custom_projectiles/blue_laser_orb.xml", creature_x + 10, creature_y - 5)
				setProjectileVelocitiesandGravity(orb_right_id, 140, 0)
				setProjectileShooter(orb_right_id, entity_id)
				local orb_left_id = EntityLoad("mods/ArcaneReliquary/files/entity_definitions/custom_projectiles/blue_laser_orb.xml", creature_x - 10, creature_y - 5)
				setProjectileVelocitiesandGravity(orb_left_id, -140, 0)
				setProjectileShooter(orb_left_id, entity_id)
			end
		end,
	},
	{
		name 					= "volcanic_boot",
		ui_name					= "Volcanic Boot",
		categories				= "player",
		effect_desc				= "Falling and stepping on enemies causes you to shoot fireballs from your feet for a few seconds.",
		effect_desc_short		= "Stepping on enemies causes you to shoot fireballs from your feet.",
        precursor_types			= "boot",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			if(not entityHasCallback(getPlayerEntity(), "stomp_listener")) then -- if no other stomp related relics added required callback, adds it
				--GamePrint(">>> stomp relic picked, stomp callback added...")
				addNewLuaCallback(getPlayerEntity(), "overtime", "mods/ArcaneReliquary/files/scripts/player/stomp_listener.lua", 2, false, "stomp_listener")
			end
			addNewSimplifiedInternalVariable(getPlayerEntity(), "relic_using_stomp_relic") -- for each stomp related relic, a new simplified variable like this is added
		end,
		ondrop_function			= function()
			removeFirstSimplifiedInternalVariable(getPlayerEntity(), "relic_using_stomp_relic") -- removes just the first variable found
			
			if(countSimplifiedInternalVariable(getPlayerEntity(), "relic_using_stomp_relic") == 0) then -- removes stomp callback only if no other relics are using it
				--GamePrint(">>> stomp relic dropped, no other related relics present, removing stomp callback...")
				removeCallback(getPlayerEntity(), "stomp_listener")
			end
		end,
		onenemystomped_function	= function(entity_id, creature_id)
			if(isRelicOwned("volcanic_boot") >= 1) then
				-- Applies effect on creature found
				LoadGameEffectEntityTo( entity_id, getCustomEffect("effect_shoot_projectile_downwards_fireball").path )
			end
		end,
	},
	{
		name 					= "thunder_boot",
		ui_name					= "Thunder Boot",
		categories				= "player",
		effect_desc				= "Falling and stepping on enemies causes you to shoot thunderbolts from your feet for a few seconds.",
		effect_desc_short		= "Stepping on enemies causes you to shoot thunderbolts from your feet.",
        precursor_types			= "boot",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			if(not entityHasCallback(getPlayerEntity(), "stomp_listener")) then -- if no other stomp related relics added required callback, adds it
				--GamePrint(">>> stomp relic picked, stomp callback added...")
				addNewLuaCallback(getPlayerEntity(), "overtime", "mods/ArcaneReliquary/files/scripts/player/stomp_listener.lua", 2, false, "stomp_listener")
			end
			addNewSimplifiedInternalVariable(getPlayerEntity(), "relic_using_stomp_relic") -- for each stomp related relic, a new simplified variable like this is added
		end,
		ondrop_function			= function()
			removeFirstSimplifiedInternalVariable(getPlayerEntity(), "relic_using_stomp_relic") -- removes just the first variable found
			
			if(countSimplifiedInternalVariable(getPlayerEntity(), "relic_using_stomp_relic") == 0) then -- removes stomp callback only if no other relics are using it
				--GamePrint(">>> stomp relic dropped, no other related relics present, removing stomp callback...")
				removeCallback(getPlayerEntity(), "stomp_listener")
			end
		end,
		onenemystomped_function	= function(entity_id, creature_id)
			if(isRelicOwned("thunder_boot") >= 1) then
				-- Applies effect on creature found
				LoadGameEffectEntityTo( entity_id, getCustomEffect("effect_shoot_projectile_downwards_thunderbolt").path )
			end
		end,
	},
	{
		name 					= "drill_boot",
		ui_name					= "Drill Boot",
		categories				= "player",
		effect_desc				= "Falling and stepping on enemies causes them to be buried on the ground.",
		effect_desc_short		= "Stepping on enemies causes them to be buried on the ground.",
        precursor_types			= "boot",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			if(not entityHasCallback(getPlayerEntity(), "stomp_listener")) then -- if no other stomp related relics added required callback, adds it
				--GamePrint(">>> stomp relic picked, stomp callback added...")
				addNewLuaCallback(getPlayerEntity(), "overtime", "mods/ArcaneReliquary/files/scripts/player/stomp_listener.lua", 2, false, "stomp_listener")
			end
			addNewSimplifiedInternalVariable(getPlayerEntity(), "relic_using_stomp_relic") -- for each stomp related relic, a new simplified variable like this is added
		end,
		ondrop_function			= function()
			removeFirstSimplifiedInternalVariable(getPlayerEntity(), "relic_using_stomp_relic") -- removes just the first variable found
			
			if(countSimplifiedInternalVariable(getPlayerEntity(), "relic_using_stomp_relic") == 0) then -- removes stomp callback only if no other relics are using it
				--GamePrint(">>> stomp relic dropped, no other related relics present, removing stomp callback...")
				removeCallback(getPlayerEntity(), "stomp_listener")
			end
		end,
		onenemystomped_function	= function(entity_id, creature_id)
			if(isRelicOwned("drill_boot") >= 1) then
				-- Applies effect on creature found
				LoadGameEffectEntityTo( creature_id, getCustomEffect("effect_bury").path )
			end
		end,
	},
	{
		name 					= "compass",
		ui_name					= "Compass",
		categories				= "projectiles",
		effect_desc				= "Your spells are now capable of changing the direction of enemy projectiles. \nIt also synergizes with Sniper Scope relic effect.",
		effect_desc_short		= "Your spells are now capable of changing the direction of enemy projectiles.",
        precursor_types			= "gadget",
		upgrade_effect_desc		= "For each upgrade level, collision radius is increased.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("compass") >= 1) then
				if(not isOrbitingProjectile(projectile_entity_id)) then
					addNewSimplifiedInternalVariable( projectile_entity_id, "relic_effect_projectile")
					attachDragCollisionHandlertoProjectile(projectile_entity_id, getCollisionHandler("redirect_enemy_projectiles").path, false, 10 + (1 * getRelicLevel("compass")), "projectile", false)
				end
			end
		end,
		onextendedkick_function	= function( entity_id, extended_kick_id)
			if(isRelicOwned("compass") >= 1) then
				addBuffVisualIndicator_Projectile(extended_kick_id, "spark_teal")
				attachDragCollisionHandlertoProjectile(extended_kick_id, getCollisionHandler("redirect_enemy_projectiles").path, false, 20, "projectile", false)
			end
		end,
	},
	{
		name 					= "mechanical_heart",
		ui_name					= "Mechanical Heart",
		categories				= "player",
		effect_desc				= "A pulsating energy field emanates from you. Enemy projectiles are \nintercepted by it and you a are healed by small amounts when this happens, but \nif you get hit, you receive triple damage.",
		effect_desc_short		= "A pulsating energy field that converts enemy projectiles into hp emanates from you.",
        precursor_types			= "gadget,device,energy_shield",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			LoadGameEffectEntityTo( getPlayerEntity(), getCustomEffect("effect_pulsating_shield").path )
		end,
		ondrop_function			= function()
			local player_children = EntityGetAllChildren(getPlayerEntity())
			if player_children == nil then return end
			for i,child in ipairs(player_children) do
				if(EntityGetName(child) == "pulsating_shield") then
					EntityKill(child)
					break
				end
			end
		end,
		ondamaged_function		= function(damage, message, entity_thats_responsible, is_fatal, projectile_thats_responsible )
			if(isRelicOwned("mechanical_heart") >= 1) then
				if(message == "$damage_projectile") then
					GamePrint("Relic penalty: triple damage taken")
					local damage_penalty = damage * 2
					EntityInflictDamage( getPlayerEntity(), damage_penalty, "DAMAGE_PROJECTILE", "Mechanical Heart damage penalty", "BLOOD_SPRAY", 0, 0)
				end
			end
		end,
	},
	{
		name 					= "life_draining_stone",
		ui_name					= "Life Draining Stone",
		categories				= "player",
		effect_desc				= "A pulsating energy field emanates from you. Enemies caught within \nit have their life essence transfered to you, but you take triple melee damage.",
		effect_desc_short		= "A pulsating energy field that drains the life essence of enemies emanates from you.",
        precursor_types			= "effect_stone,energy_shield",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			LoadGameEffectEntityTo( getPlayerEntity(), getCustomEffect("effect_pulsating_shield_life_drain").path )
		end,
		ondrop_function			= function()
			local player_children = EntityGetAllChildren(getPlayerEntity())
			if player_children == nil then return end
			for i,child in ipairs(player_children) do
				if(EntityGetName(child) == "pulsating_shield_life_drain") then
					EntityKill(child)
					break
				end
			end
		end,
		ondamaged_function		= function(damage, message, entity_thats_responsible, is_fatal, projectile_thats_responsible )
			if(isRelicOwned("life_draining_stone") >= 1) then
				if(message == "$damage_melee") then
					GamePrint("Relic penalty: triple melee damage taken")
					local damage_penalty = damage * 2
					EntityInflictDamage( getPlayerEntity(), damage_penalty, "DAMAGE_MELEE", "Life Draining Stone damage penalty", "BLOOD_SPRAY", 0, 0)
				end
			end
		end,
	},
	{
		name 					= "ruler",
		ui_name					= "Ruler",
		categories				= "player,kick",
		effect_desc				= "Doubles the range of the extended kick ability added by the Sniper Scope relic.",
		effect_desc_short		= "Doubles the range of the extended kick ability added by the Sniper Scope relic.",
        precursor_types			= "gadget",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			addNewSimplifiedInternalVariable(getPlayerEntity(), "double_extended_kick_range")
		end,
		ondrop_function			= function()
			removeSimplifiedInternalVariable(getPlayerEntity(), "double_extended_kick_range")
		end
	},
	{
		name 					= "high_density_stone",
		ui_name					= "High Density Stone",
		categories				= "projectiles",
		effect_desc				= "Your spells have a 25% chance of creating black holes when they explode or hit something.",
		effect_desc_short		= "Your spells create black holes.",
        precursor_types			= "effect_stone",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("high_density_stone") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 25 + (5 * getRelicLevel("high_density_stone"))
				if(chance >= 1 and chance <= odds) then
					attachLoadEntityOnExplosiontoProjectile(projectile_entity_id, getLoadOnExplosion("blackhole_custom").path, "spark_purple_bright")
				end
			end
		end,
	},
	{
		name 					= "proximity_crystal",
		ui_name					= "Proximity Crystal",
		categories				= "projectiles,enemies",
		effect_desc				= "Every enemy hit by your spells has a 25% chance of being transformed into an Unstable Crystal.",
		effect_desc_short		= "Your spells transform enemies into Unstable Crystals.",
        precursor_types			= "effect_crystal",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onenemydamaged_function	= function(damage, message, entity_thats_responsible, is_fatal, projectile_thats_responsible )
			if(isRelicOwned("proximity_crystal") >= 1) then
				if(message == "$damage_projectile") then
					SetRandomSeed( GameGetFrameNum(), math.random(1,50))
					local chance = Random(1,100)
					local odds = 25 + (5 * getRelicLevel("proximity_crystal"))
					if(chance >= 1 and chance <= odds) then
						LoadGameEffectEntityTo( GetUpdatedEntityID(), getCustomEffect("effect_convert_to_mine").path )
					end
				end
			end
		end,
	},
	{
		name 					= "regenerating_stone",
		ui_name					= "Regenerating Stone",
		categories				= "projectiles,enemies",
		effect_desc				= "Provided you're on less than 10% of your total health, there's \na 20% chance enemies will spawn a healing field when killed by you.",
		effect_desc_short		= "Killed enemies spawn healing field.",
        precursor_types			= "effect_stone",
		upgrade_effect_desc		= "For each upgrade level, healing field spawn chance increases by 5%.",
		is_upgradable 			= true,
		onenemykill_function	= function(damage_type_bit_field, damage_message, entity_thats_responsible, drop_items)
			if(isRelicOwned("regenerating_stone") >= 1) then
				local current_hp = getPlayerHealth()
				local max_hp = getPlayerMaxHealth()
				if(current_hp < max_hp / 10 ) then
					SetRandomSeed( GameGetFrameNum(), math.random(1,50))
					local chance = Random(1,100)
					local odds = 20 + (5 * getRelicLevel("regenerating_stone"))
					if(chance >= 1 and chance <= odds) then
						local enemy_id = GetUpdatedEntityID()
						local x, y = EntityGetTransform(enemy_id)
						EntityLoad("data/entities/projectiles/deck/regeneration_field.xml", x, y)
					end
				end
			end
		end,
	},
	{
		name 					= "shockwave_boot",
		ui_name					= "Shockwave Boot",
		categories				= "player",
		effect_desc				= "Falling and stepping on enemies creates shockwaves around them.",
		effect_desc_short		= "Stepping on enemies creates shockwaves around them.",
        precursor_types			= "boot",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			if(not entityHasCallback(getPlayerEntity(), "stomp_listener")) then -- if no other stomp related relics added required callback, adds it
				--GamePrint(">>> stomp relic picked, stomp callback added...")
				addNewLuaCallback(getPlayerEntity(), "overtime", "mods/ArcaneReliquary/files/scripts/player/stomp_listener.lua", 2, false, "stomp_listener")
			end
			addNewSimplifiedInternalVariable(getPlayerEntity(), "relic_using_stomp_relic") -- for each stomp related relic, a new simplified variable like this is added
		end,
		ondrop_function			= function()
			removeFirstSimplifiedInternalVariable(getPlayerEntity(), "relic_using_stomp_relic") -- removes just the first variable found
			
			if(countSimplifiedInternalVariable(getPlayerEntity(), "relic_using_stomp_relic") == 0) then -- removes stomp callback only if no other relics are using it
				--GamePrint(">>> stomp relic dropped, no other related relics present, removing stomp callback...")
				removeCallback(getPlayerEntity(), "stomp_listener")
			end
		end,
		onenemystomped_function	= function(entity_id, creature_id)
			if(isRelicOwned("shockwave_boot") >= 1) then
				-- Loads shockwave at creature's coordinates
				local creature_x, creature_y = EntityGetTransform(creature_id)
				EntityLoad(getLoadOnExplosion("shockwave").path, creature_x, creature_y)
			end
		end,
	},
	{
		name 					= "contained_orb_bloody",
		ui_name					= "Contained Bloody Orb",
		categories				= "player,materials",
		effect_desc				= "Falling and stepping on enemies makes them shoot corrosive blood orbs. \nIt also synergizes with Sniper Scope and Earth Core Fragment relic effects.",
		effect_desc_short		= "Stepping on enemies makes them shoot corrosive blood orbs.",
        precursor_types			= "magic_artifact",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			if(not entityHasCallback(getPlayerEntity(), "stomp_listener")) then -- if no other stomp related relics added required callback, adds it
				--GamePrint(">>> stomp relic picked, stomp callback added...")
				addNewLuaCallback(getPlayerEntity(), "overtime", "mods/ArcaneReliquary/files/scripts/player/stomp_listener.lua", 2, false, "stomp_listener")
			end
			addNewSimplifiedInternalVariable(getPlayerEntity(), "relic_using_stomp_relic") -- for each stomp related relic, a new simplified variable like this is added
		end,
		ondrop_function			= function()
			removeFirstSimplifiedInternalVariable(getPlayerEntity(), "relic_using_stomp_relic") -- removes just the first variable found
			
			if(countSimplifiedInternalVariable(getPlayerEntity(), "relic_using_stomp_relic") == 0) then -- removes stomp callback only if no other relics are using it
				--GamePrint(">>> stomp relic dropped, no other related relics present, removing stomp callback...")
				removeCallback(getPlayerEntity(), "stomp_listener")
			end
		end,
		onenemystomped_function	= function(entity_id, creature_id)
			if(isRelicOwned("contained_orb_bloody") >= 1) then
				-- Shoots corrosive blood orbs
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local creature_x, creature_y = EntityGetTransform(creature_id)
				local amount = Random(4,6)
				for i=1,amount do
					local orb_id = EntityLoad("mods/ArcaneReliquary/files/entity_definitions/custom_projectiles/corrosive_blood_orb.xml", creature_x, creature_y - Random(10,12))
					setProjectileVelocitiesandGravity(orb_id, Random(-300,300), Random(-150,-250))
					setProjectileShooter(orb_id, entity_id)
				end
			end
		end,
		onextendedkick_function	= function( entity_id, extended_kick_id)
			if(isRelicOwned("contained_orb_bloody") >= 1) then
				attachLoadEffectOntoEntitiestoProjectile(extended_kick_id, getCustomEffect("effect_shoot_projectiles_above_bloody_orb").path, "blood")
			end
		end,
		ongroundpound_function	= function( entity_id)
			if(isRelicOwned("contained_orb_bloody") >= 1) then
				LoadGameEffectEntityTo( entity_id, getCustomEffect("effect_shoot_projectiles_above_bloody_orb").path )
			end
		end,
	},
	{
		name 					= "contained_orb_scorching",
		ui_name					= "Contained Scorching Orb",
		categories				= "player,materials",
		effect_desc				= "Falling and stepping on enemies makes them shoot lava orbs. \nIt also synergizes with Sniper Scope and Earth Core Fragment relic effects.",
		effect_desc_short		= "Stepping on enemies makes them shoot lava orbs.",
        precursor_types			= "magic_artifact",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			if(not entityHasCallback(getPlayerEntity(), "stomp_listener")) then -- if no other stomp related relics added required callback, adds it
				--GamePrint(">>> stomp relic picked, stomp callback added...")
				addNewLuaCallback(getPlayerEntity(), "overtime", "mods/ArcaneReliquary/files/scripts/player/stomp_listener.lua", 2, false, "stomp_listener")
			end
			addNewSimplifiedInternalVariable(getPlayerEntity(), "relic_using_stomp_relic") -- for each stomp related relic, a new simplified variable like this is added
		end,
		ondrop_function			= function()
			removeFirstSimplifiedInternalVariable(getPlayerEntity(), "relic_using_stomp_relic") -- removes just the first variable found
			
			if(countSimplifiedInternalVariable(getPlayerEntity(), "relic_using_stomp_relic") == 0) then -- removes stomp callback only if no other relics are using it
				--GamePrint(">>> stomp relic dropped, no other related relics present, removing stomp callback...")
				removeCallback(getPlayerEntity(), "stomp_listener")
			end
		end,
		onenemystomped_function	= function(entity_id, creature_id)
			if(isRelicOwned("contained_orb_scorching") >= 1) then
				-- Shoots lava orbs
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local creature_x, creature_y = EntityGetTransform(creature_id)
				local amount = Random(4,6)
				for i=1,amount do
					local orb_id = EntityLoad("mods/ArcaneReliquary/files/entity_definitions/custom_projectiles/lava_orb.xml", creature_x, creature_y - Random(10,12))
					setProjectileVelocitiesandGravity(orb_id, Random(-300,300), Random(-150,-250))
					setProjectileShooter(orb_id, entity_id)
				end
			end
		end,
		onextendedkick_function	= function( entity_id, extended_kick_id)
			if(isRelicOwned("contained_orb_scorching") >= 1) then
				attachLoadEffectOntoEntitiestoProjectile(extended_kick_id, getCustomEffect("effect_shoot_projectiles_above_lava_orb").path, "lava")
			end
		end,
		ongroundpound_function	= function( entity_id)
			if(isRelicOwned("contained_orb_scorching") >= 1) then
				LoadGameEffectEntityTo( entity_id, getCustomEffect("effect_shoot_projectiles_above_lava_orb").path )
			end
		end,
	},
	{
		name 					= "contained_orb_corrosive",
		ui_name					= "Contained Corrosive Orb",
		categories				= "projectiles,materials",
		effect_desc				= "Enemies you kill with your spells have a 50% chance of releasing acid orbs. \nIt also synergizes with Sniper Scope and Earth Core Fragment relic effects.",
		effect_desc_short		= "Enemies killed by your spells release acid orbs.",
        precursor_types			= "magic_artifact",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onenemykill_function	= function(damage_type_bit_field, damage_message, entity_thats_responsible, drop_items)
			if(isRelicOwned("contained_orb_corrosive") >= 1) then
				if(damage_message == "$damage_projectile") then
					SetRandomSeed( GameGetFrameNum(), math.random(1,50))
					local chance = Random(1,100)
					local odds = 50 + (5 * getRelicLevel("contained_orb_corrosive"))
					if(chance >= 1 and chance <= odds) then
						local enemy_id = GetUpdatedEntityID()
						local x, y = EntityGetTransform(enemy_id)
						local amount = Random(4,6)
						for i=1,amount do
							local orb_id = EntityLoad("mods/ArcaneReliquary/files/entity_definitions/custom_projectiles/acid_orb.xml", x, y - Random(10,12))
							setProjectileVelocitiesandGravity(orb_id, Random(-300,300), Random(-150,-250))
							setProjectileShooter(orb_id, getPlayerEntity())
						end
					end
				end
			end
		end,
		onextendedkick_function	= function( entity_id, extended_kick_id)
			if(isRelicOwned("contained_orb_corrosive") >= 1) then
				attachLoadEffectOntoEntitiestoProjectile(extended_kick_id, getCustomEffect("effect_shoot_projectiles_above_acid_orb").path, "acid_gas")
			end
		end,
		ongroundpound_function	= function( entity_id)
			if(isRelicOwned("contained_orb_corrosive") >= 1) then
				LoadGameEffectEntityTo( entity_id, getCustomEffect("effect_shoot_projectiles_above_acid_orb").path )
			end
		end,
	},
	{
		name 					= "contained_orb_explosive",
		ui_name					= "Contained Explosive Orb",
		categories				= "player,kick",
		effect_desc				= "Enemies you kick have a 50% chance of releasing explosive orbs. \nIt also synergizes with Sniper Scope and Earth Core Fragment relic effects.",
		effect_desc_short		= "Kicking enemies makes them shoot explosive orbs.",
        precursor_types			= "boot,magic_artifact",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onenemykicked_function	= function (entity_who_kicked)
			if(isRelicOwned("contained_orb_explosive") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 50 + (5 * getRelicLevel("contained_orb_explosive"))
				if(chance >= 1 and chance <= odds) then
					local enemy_id = GetUpdatedEntityID()
					local x, y = EntityGetTransform(enemy_id)
					local amount = Random(4,6)
					for i=1,amount do
						local orb_id = EntityLoad("mods/ArcaneReliquary/files/entity_definitions/custom_projectiles/explosive_orb.xml", x, y - Random(10,12))
						setProjectileVelocitiesandGravity(orb_id, Random(-300,300), Random(-150,-250))
						setProjectileShooter(orb_id, getPlayerEntity())
					end
				end
			end
		end,
		onextendedkick_function	= function( entity_id, extended_kick_id)
			if(isRelicOwned("contained_orb_explosive") >= 1) then
				attachLoadEffectOntoEntitiestoProjectile(extended_kick_id, getCustomEffect("effect_shoot_projectiles_above_explosive_orb").path, "spark_blue_dark")
			end
		end,
		ongroundpound_function	= function( entity_id)
			if(isRelicOwned("contained_orb_explosive") >= 1) then
				LoadGameEffectEntityTo( entity_id, getCustomEffect("effect_shoot_projectiles_above_explosive_orb").path )
			end
		end,
	},
	{
		name 					= "contained_orb_wet",
		ui_name					= "Contained Wet Orb",
		categories				= "player,enemies,materials",
		effect_desc				= "Enemies killed with explosions from your spells release water orbs. \nIt also synergizes with Sniper Scope and Earth Core Fragment relic effects.",
		effect_desc_short		= "Enemies killed with explosions release water orbs.",
        precursor_types			= "magic_artifact",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onenemykill_function	= function(damage_type_bit_field, damage_message, entity_thats_responsible, drop_items)
			if(isRelicOwned("contained_orb_wet") >= 1) then
				if(damage_message == "$damage_explosion") then
					local enemy_id = GetUpdatedEntityID()
					local x, y = EntityGetTransform(enemy_id)
					local amount = Random(4,6)
					for i=1,amount do
						local orb_id = EntityLoad("mods/ArcaneReliquary/files/entity_definitions/custom_projectiles/water_orb.xml", x, y - Random(10,12))
						setProjectileVelocitiesandGravity(orb_id, Random(-300,300), Random(-150,-250))
						setProjectileShooter(orb_id, getPlayerEntity())
					end
				end
			end
		end,
		onextendedkick_function	= function( entity_id, extended_kick_id)
			if(isRelicOwned("contained_orb_wet") >= 1) then
				attachLoadEffectOntoEntitiestoProjectile(extended_kick_id, getCustomEffect("effect_shoot_projectiles_above_water_orb").path, "water")
			end
		end,
		ongroundpound_function	= function( entity_id)
			if(isRelicOwned("contained_orb_wet") >= 1) then
				LoadGameEffectEntityTo( entity_id, getCustomEffect("effect_shoot_projectiles_above_water_orb").path )
			end
		end,
	},
	{
		name 					= "contained_orb_vile",
		ui_name					= "Contained Vile Orb",
		categories				= "projectiles,materials",
		effect_desc				= "Enemies you hit with your spells have a 50% chance of releasing poison orbs. \nIt also synergizes with Sniper Scope and Earth Core Fragment relic effects.",
		effect_desc_short		= "Your spells make enemies shoot poison orbs.",
        precursor_types			= "magic_artifact",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onenemydamaged_function	= function(damage_type_bit_field, damage_message, entity_thats_responsible, drop_items)
			if(isRelicOwned("contained_orb_vile") >= 1) then
				if(damage_message == "$damage_projectile") then
					SetRandomSeed( GameGetFrameNum(), math.random(1,50))
					local chance = Random(1,100)
					local odds = 50 + (5 * getRelicLevel("contained_orb_vile"))
					if(chance >= 1 and chance <= odds) then
						local enemy_id = GetUpdatedEntityID()
						local x, y = EntityGetTransform(enemy_id)
						local amount = Random(4,6)
						for i=1,amount do
							local orb_id = EntityLoad("mods/ArcaneReliquary/files/entity_definitions/custom_projectiles/poison_orb.xml", x, y - Random(10,12))
							setProjectileVelocitiesandGravity(orb_id, Random(-300,300), Random(-150,-250))
							setProjectileShooter(orb_id, getPlayerEntity())
						end
					end
				end
			end
		end,
		onextendedkick_function	= function( entity_id, extended_kick_id)
			if(isRelicOwned("contained_orb_vile") >= 1) then
				attachLoadEffectOntoEntitiestoProjectile(extended_kick_id, getCustomEffect("effect_shoot_projectiles_above_poison_orb").path, "poison")
			end
		end,
		ongroundpound_function	= function( entity_id)
			if(isRelicOwned("contained_orb_vile") >= 1) then
				LoadGameEffectEntityTo( entity_id, getCustomEffect("effect_shoot_projectiles_above_poison_orb").path )
			end
		end,
	},
	{
		name 					= "contained_orb_uric",
		ui_name					= "Contained Uric Orb",
		categories				= "projectiles,materials",
		effect_desc				= "Enemies you kill with your spells have a 50% chance of releasing urine orbs. \nIt also synergizes with Sniper Scope and Earth Core Fragment relic effects.",
		effect_desc_short		= "Enemies killed by your spells release urine orbs.",
        precursor_types			= "magic_artifact",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onenemykill_function	= function(damage_type_bit_field, damage_message, entity_thats_responsible, drop_items)
			if(isRelicOwned("contained_orb_uric") >= 1) then
				if(damage_message == "$damage_projectile") then
					SetRandomSeed( GameGetFrameNum(), math.random(1,50))
					local chance = Random(1,100)
					local odds = 50 + (5 * getRelicLevel("contained_orb_uric"))
					if(chance >= 1 and chance <= odds) then
						local enemy_id = GetUpdatedEntityID()
						local x, y = EntityGetTransform(enemy_id)
						local amount = Random(4,6)
						for i=1,amount do
							local orb_id = EntityLoad("mods/ArcaneReliquary/files/entity_definitions/custom_projectiles/urine_orb.xml", x, y - Random(10,12))
							setProjectileVelocitiesandGravity(orb_id, Random(-300,300), Random(-150,-250))
							setProjectileShooter(orb_id, getPlayerEntity())
						end
					end
				end
			end
		end,
		onextendedkick_function	= function( entity_id, extended_kick_id)
			if(isRelicOwned("contained_orb_uric") >= 1) then
				attachLoadEffectOntoEntitiestoProjectile(extended_kick_id, getCustomEffect("effect_shoot_projectiles_above_urine_orb").path, "urine")
			end
		end,
		ongroundpound_function	= function( entity_id)
			if(isRelicOwned("contained_orb_uric") >= 1) then
				LoadGameEffectEntityTo( entity_id, getCustomEffect("effect_shoot_projectiles_above_urine_orb").path )
			end
		end,
	},
	{
		name 					= "contained_orb_radioactive",
		ui_name					= "Contained Radioactive Orb",
		categories				= "player,kick",
		effect_desc				= "Enemies you kick have a 50% chance of releasing toxic gas orbs. \nIt also synergizes with Sniper Scope and Earth Core Fragment relic effects.",
		effect_desc_short		= "Kicking enemies makes them shoot toxic gas orbs.",
        precursor_types			= "boot,magic_artifact",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onenemykicked_function	= function (entity_who_kicked)
			if(isRelicOwned("contained_orb_radioactive") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 50 + (5 * getRelicLevel("contained_orb_radioactive"))
				if(chance >= 1 and chance <= odds) then
					local enemy_id = GetUpdatedEntityID()
					local x, y = EntityGetTransform(enemy_id)
					local amount = Random(4,6)
					for i=1,amount do
						local orb_id = EntityLoad("mods/ArcaneReliquary/files/entity_definitions/custom_projectiles/radioactive_gas_orb.xml", x, y - Random(10,12))
						setProjectileVelocitiesandGravity(orb_id, Random(-300,300), Random(-150,-250))
						setProjectileShooter(orb_id, getPlayerEntity())
					end
				end
			end
		end,
		onextendedkick_function	= function( entity_id, extended_kick_id)
			if(isRelicOwned("contained_orb_radioactive") >= 1) then
				attachLoadEffectOntoEntitiestoProjectile(extended_kick_id, getCustomEffect("effect_shoot_projectiles_above_radioactive_gas_orb").path, "radioactive_gas")
			end
		end,
		ongroundpound_function	= function( entity_id)
			if(isRelicOwned("contained_orb_radioactive") >= 1) then
				LoadGameEffectEntityTo( entity_id, getCustomEffect("effect_shoot_projectiles_above_radioactive_gas_orb").path )
			end
		end,
	},
	{
		name 					= "contained_orb_frozen",
		ui_name					= "Contained Frozen Orb",
		categories				= "projectiles,materials",
		effect_desc				= "Enemies you hit with your spells have a 50% chance of releasing freezing gas orbs. \nIt also synergizes with Sniper Scope and Earth Core Fragment relic effects.",
		effect_desc_short		= "Your spells make enemies shoot freezing gas orbs.",
        precursor_types			= "magic_artifact",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onenemydamaged_function	= function(damage_type_bit_field, damage_message, entity_thats_responsible, drop_items)
			if(isRelicOwned("contained_orb_frozen") >= 1) then
				if(damage_message == "$damage_projectile") then
					SetRandomSeed( GameGetFrameNum(), math.random(1,50))
					local chance = Random(1,100)
					local odds = 50 + (5 * getRelicLevel("contained_orb_frozen"))
					if(chance >= 1 and chance <= odds) then
						local enemy_id = GetUpdatedEntityID()
						local x, y = EntityGetTransform(enemy_id)
						local amount = Random(4,6)
						for i=1,amount do
							local orb_id = EntityLoad("mods/ArcaneReliquary/files/entity_definitions/custom_projectiles/freezing_gas_orb.xml", x, y - Random(10,12))
							setProjectileVelocitiesandGravity(orb_id, Random(-300,300), Random(-150,-250))
							setProjectileShooter(orb_id, getPlayerEntity())
						end
					end
				end
			end
		end,
		onextendedkick_function	= function( entity_id, extended_kick_id)
			if(isRelicOwned("contained_orb_frozen") >= 1) then
				attachLoadEffectOntoEntitiestoProjectile(extended_kick_id, getCustomEffect("effect_shoot_projectiles_above_freezing_gas_orb").path, "diamond")
			end
		end,
		ongroundpound_function	= function( entity_id)
			if(isRelicOwned("contained_orb_frozen") >= 1) then
				LoadGameEffectEntityTo( entity_id, getCustomEffect("effect_shoot_projectiles_above_freezing_gas_orb").path )
			end
		end,
	},
	{
		name 					= "contained_orb_mutating",
		ui_name					= "Contained Mutating Orb",
		categories				= "projectiles,materials",
		effect_desc				= "Enemies you hit with your spells have a 50% chance of releasing polymorphine orbs. \nIt also synergizes with Sniper Scope and Earth Core Fragment relic effects.",
		effect_desc_short		= "Your spells make enemies shoot polymorphine orbs.",
        precursor_types			= "magic_artifact",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onenemydamaged_function	= function(damage_type_bit_field, damage_message, entity_thats_responsible, drop_items)
			if(isRelicOwned("contained_orb_mutating") >= 1) then
				if(damage_message == "$damage_projectile") then
					SetRandomSeed( GameGetFrameNum(), math.random(1,50))
					local chance = Random(1,100)
					local odds = 50 + (5 * getRelicLevel("contained_orb_mutating"))
					if(chance >= 1 and chance <= odds) then
						local enemy_id = GetUpdatedEntityID()
						local x, y = EntityGetTransform(enemy_id)
						local amount = Random(4,6)
						for i=1,amount do
							local orb_id = EntityLoad("mods/ArcaneReliquary/files/entity_definitions/custom_projectiles/polymorphine_orb.xml", x, y - Random(10,12))
							setProjectileVelocitiesandGravity(orb_id, Random(-300,300), Random(-150,-250))
							setProjectileShooter(orb_id, getPlayerEntity())
						end
					end
				end
			end
		end,
		onextendedkick_function	= function( entity_id, extended_kick_id)
			if(isRelicOwned("contained_orb_mutating") >= 1) then
				attachLoadEffectOntoEntitiestoProjectile(extended_kick_id, getCustomEffect("effect_shoot_projectiles_above_poly_orb").path, "plasma_fading_pink")
			end
		end,
		ongroundpound_function	= function( entity_id)
			if(isRelicOwned("contained_orb_mutating") >= 1) then
				LoadGameEffectEntityTo( entity_id, getCustomEffect("effect_shoot_projectiles_above_poly_orb").path )
			end
		end,
	},
	{
		name 					= "death_ray_stone",
		ui_name					= "Death Ray Stone",
		categories				= "player,kick",
		effect_desc				= "When you land a kick on an enemy, there's a 50% chance a death ray will be summoned. \nIt also synergizes with Sniper Scope relic effect.",
		effect_desc_short		= "Kicking enemies has a chance of summoning a death ray.",
        precursor_types			= "death_ray,effect_stone",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onenemykicked_function	= function (entity_who_kicked)
			if(isRelicOwned("death_ray_stone") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 50 + (5 * getRelicLevel("death_ray_stone"))
				if(chance >= 1 and chance <= odds) then
					local enemy_id = GetUpdatedEntityID()
					local x, y = EntityGetTransform(enemy_id)
					GamePlaySound( "data/audio/Desktop/misc.bank", "misc/beam_from_sky_kick", x, y )
					EntityLoad( "mods/ArcaneReliquary/files/entity_definitions/others/death_ray.xml", x, y )
				end
			end
		end,
		onextendedkick_function	= function( entity_id, extended_kick_id)
			if(isRelicOwned("death_ray_stone") >= 1) then
				attachLoadEffectOntoEntitiestoProjectile(extended_kick_id, getCustomEffect("effect_summon_deathray").path, "plasma_fading_pink")
			end
		end,
		excludes_relics			= "death_ray_stone_nuclear",
		excludes_relics_desc	= "It can't be used along with other death ray summon on kick related relics",
		excludes_relics_tags	= "deathray_summon_on_kick",
	},
	{
		name 					= "thunderous_wand",
		ui_name					= "Thunderous Wand",
		categories				= "projectiles",
		effect_desc				= "When you cast a spell, you have a 30% chance of summoning a blue \nspectral wand that shoots a powerful thunder laser.",
		effect_desc_short		= "Your spells summon a spectral wand that shoots blue thunder lasers.",
        precursor_types			= "magic_artifact",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("thunderous_wand") >= 1) then
				if(not projectileHasProjectileDefinitionPath(projectile_entity_id)) then
					SetRandomSeed( GameGetFrameNum(), math.random(1,50))
					local chance = Random(1,100)
					local odds = 30 + (5 * getRelicLevel("thunderous_wand"))
					if(chance >= 1 and chance <= odds) then
						local x, y = EntityGetTransform(projectile_entity_id)
						local laser_shooter_id = EntityLoad("mods/ArcaneReliquary/files/entity_definitions/custom_projectiles/megalaser_thunderbolts.xml", x, y)
						addNewSimplifiedInternalVariable( laser_shooter_id, "relic_effect_projectile")
						local projectile_vel_x, projectile_vel_y = getProjectileVelocitiesandGravity(projectile_entity_id)
						setProjectileVelocitiesandGravity(laser_shooter_id, projectile_vel_x, projectile_vel_y)
					end
				end
			end
		end
	},
	{
		name 					= "pungent_wand",
		ui_name					= "Pungent Wand",
		categories				= "projectiles,materials",
		effect_desc				= "When you cast a spell, you have a 30% chance of summoning a green \nspectral wand that shoots a powerful acid laser.",
		effect_desc_short		= "Your spells summon a spectral wand that shoots green acid lasers.",
        precursor_types			= "magic_artifact",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("pungent_wand") >= 1) then
				if(not projectileHasProjectileDefinitionPath(projectile_entity_id)) then
					SetRandomSeed( GameGetFrameNum(), math.random(1,50))
					local chance = Random(1,100)
					local odds = 30 + (5 * getRelicLevel("pungent_wand"))
					if(chance >= 1 and chance <= odds) then
						local x, y = EntityGetTransform(projectile_entity_id)
						local laser_shooter_id = EntityLoad("mods/ArcaneReliquary/files/entity_definitions/custom_projectiles/megalaser_acid.xml", x, y)
						addNewSimplifiedInternalVariable( laser_shooter_id, "relic_effect_projectile")
						local projectile_vel_x, projectile_vel_y = getProjectileVelocitiesandGravity(projectile_entity_id)
						setProjectileVelocitiesandGravity(laser_shooter_id, projectile_vel_x, projectile_vel_y)
					end
				end
			end
		end
	},
	{
		name 					= "very_hazardous_wand",
		ui_name					= "Very Hazardous Wand",
		categories				= "projectiles",
		effect_desc				= "When you cast a spell, you have a 30% chance of summoning an orange \nspectral wand that shoots a powerful super explosive laser.",
		effect_desc_short		= "Your spells summon a spectral wand that shoots orange super explosive lasers.",
        precursor_types			= "magic_artifact",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("very_hazardous_wand") >= 1) then
				if(not projectileHasProjectileDefinitionPath(projectile_entity_id)) then
					SetRandomSeed( GameGetFrameNum(), math.random(1,50))
					local chance = Random(1,100)
					local odds = 30 + (5 * getRelicLevel("very_hazardous_wand"))
					if(chance >= 1 and chance <= odds) then
						local x, y = EntityGetTransform(projectile_entity_id)
						local laser_shooter_id = EntityLoad("mods/ArcaneReliquary/files/entity_definitions/custom_projectiles/megalaser_super_explosive.xml", x, y)
						addNewSimplifiedInternalVariable( laser_shooter_id, "relic_effect_projectile")
						local projectile_vel_x, projectile_vel_y = getProjectileVelocitiesandGravity(projectile_entity_id)
						setProjectileVelocitiesandGravity(laser_shooter_id, projectile_vel_x, projectile_vel_y)
					end
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "hazardous_wand,explosive_mana_crystal",
	},
	{
		name 					= "gravity_stone_corrosive_blood",
		ui_name					= "Corrosive Blood Gravity Stone",
		categories				= "player,materials",
		effect_desc				= "A red energy sphere that drips corrosive blood orbits around you.",
		effect_desc_short		= "A red energy sphere that drips corrosive blood orbits around you.",
        precursor_types			= "gravity_stone",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		ondrop_function			= function()
			detachRotatingShieldfromEntity(getPlayerEntity(), "rotating_shield_projectile_drip")
		end,
		overtime_function		= function()
			if(isRelicOwned("gravity_stone_corrosive_blood") >= 1) then
				if(not entityHasRotatingShield(getPlayerEntity(), "rotating_shield_projectile_drip")) then -- this makes sure the orbiting projectiles are loaded again if the player is polymorphed
					local start_id = EntityLoad("mods/ArcaneReliquary/files/entity_definitions/custom_projectiles/rotating_shield_start.xml")
					setInternalVariableValue(start_id, "host_entity_id", "value_int", getPlayerEntity())
					setInternalVariableValue(start_id, "rotating_projectile_definition", "value_string", "mods/ArcaneReliquary/files/entity_definitions/custom_projectiles/rotating_shield_drip_part.xml")
					setInternalVariableValue(start_id, "rotating_projectiles_amount", "value_int", 1)
					setInternalVariableValue(start_id, "radius", "value_int", 32)
					setInternalVariableValue(start_id, "rotation_speed", "value_int", 2)
				end
			end
		end,
	},
	{
		name 					= "gravity_stone_pink",
		ui_name					= "Pink Gravity Stone",
		categories				= "player",
		effect_desc				= "Pink energy spheres that mutate enemies that \nhit them with their projectiles orbit around you.",
		effect_desc_short		= "Pink energy spheres that mutate enemies orbit around you.",
        precursor_types			= "gravity_stone",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		ondrop_function			= function()
			detachRotatingShieldfromEntity(getPlayerEntity(), "rotating_shield_projectile_mutate")
		end,
		overtime_function		= function()
			if(isRelicOwned("gravity_stone_pink") >= 1) then
				if(not entityHasRotatingShield(getPlayerEntity(), "rotating_shield_projectile_mutate")) then -- this makes sure the orbiting projectiles are loaded again if the player is polymorphed
					local start_id = EntityLoad("mods/ArcaneReliquary/files/entity_definitions/custom_projectiles/rotating_shield_start.xml")
					setInternalVariableValue(start_id, "host_entity_id", "value_int", getPlayerEntity())
					setInternalVariableValue(start_id, "rotating_projectile_definition", "value_string", "mods/ArcaneReliquary/files/entity_definitions/custom_projectiles/rotating_shield_mutate_part.xml")
					setInternalVariableValue(start_id, "rotating_projectiles_amount", "value_int", 2)
					setInternalVariableValue(start_id, "radius", "value_int", 28)
					setInternalVariableValue(start_id, "rotation_speed", "value_int", 2)
				end
			end
		end,
	},
	{
		name 					= "gravity_stone_water",
		ui_name					= "Water Gravity Stone",
		categories				= "player,materials",
		effect_desc				= "A blue energy sphere that drips water orbits around you.",
		effect_desc_short		= "A blue energy sphere that drips water orbits around you.",
        precursor_types			= "gravity_stone",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		ondrop_function			= function()
			detachRotatingShieldfromEntity(getPlayerEntity(), "rotating_shield_projectile_drip_water")
		end,
		overtime_function		= function()
			if(isRelicOwned("gravity_stone_water") >= 1) then
				if(not entityHasRotatingShield(getPlayerEntity(), "rotating_shield_projectile_drip_water")) then -- this makes sure the orbiting projectiles are loaded again if the player is polymorphed
					local start_id = EntityLoad("mods/ArcaneReliquary/files/entity_definitions/custom_projectiles/rotating_shield_start.xml")
					setInternalVariableValue(start_id, "host_entity_id", "value_int", getPlayerEntity())
					setInternalVariableValue(start_id, "rotating_projectile_definition", "value_string", "mods/ArcaneReliquary/files/entity_definitions/custom_projectiles/rotating_shield_drip_water_part.xml")
					setInternalVariableValue(start_id, "rotating_projectiles_amount", "value_int", 1)
					setInternalVariableValue(start_id, "radius", "value_int", 32)
					setInternalVariableValue(start_id, "rotation_speed", "value_int", 2)
				end
			end
		end,
	},
	{
		name 					= "gravity_stone_orange",
		ui_name					= "Orange Gravity Stone",
		categories				= "player,materials",
		effect_desc				= "Orange energy spheres that convert enemy projectiles \ninto lava orbs and shoot them back orbit around you.",
		effect_desc_short		= "Orange energy spheres that convert enemy projectiles into lava orbs orbit around you.",
        precursor_types			= "gravity_stone",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		ondrop_function			= function()
			detachRotatingShieldfromEntity(getPlayerEntity(), "rotating_shield_projectile_lava_shot")
		end,
		overtime_function		= function()
			if(isRelicOwned("gravity_stone_orange") >= 1) then
				if(not entityHasRotatingShield(getPlayerEntity(), "rotating_shield_projectile_lava_shot")) then -- this makes sure the orbiting projectiles are loaded again if the player is polymorphed
					local start_id = EntityLoad("mods/ArcaneReliquary/files/entity_definitions/custom_projectiles/rotating_shield_start.xml")
					setInternalVariableValue(start_id, "host_entity_id", "value_int", getPlayerEntity())
					setInternalVariableValue(start_id, "rotating_projectile_definition", "value_string", "mods/ArcaneReliquary/files/entity_definitions/custom_projectiles/rotating_shield_lava_shot_part.xml")
					setInternalVariableValue(start_id, "rotating_projectiles_amount", "value_int", 2)
			setInternalVariableValue(start_id, "radius", "value_int", 28)
			setInternalVariableValue(start_id, "rotation_speed", "value_int", 5)
				end
			end
		end,
	},
	{
		name 					= "sentient_crystal_poisonous",
		ui_name					= "Sentient Poisonous Crystal",
		categories				= "companions,materials",
		effect_desc				= "A sentient crystal that shoots poison orbs if \nenemy projectiles touch them joins you.",
		effect_desc_short		= "A poison orb shooting companion.",
        precursor_types			= "companion",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			attachCompaniontoEntity(getPlayerEntity(), "sentient_crystal_poisonous", "mortal")
		end,
		ondrop_function			= function()
			detachCompanionfromEntity(getPlayerEntity(), "sentient_crystal_poisonous")
		end,
	},
	{
		name 					= "gravity_stone_purple",
		ui_name					= "Purple Gravity Stone",
		categories				= "player,materials",
		effect_desc				= "Purple energy spheres that convert enemy projectiles \ninto poison orbs and shoot them back orbit around you.",
		effect_desc_short		= "Purple energy spheres that convert enemy projectiles into poison orbs orbit around you.",
        precursor_types			= "gravity_stone",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		ondrop_function			= function()
			detachRotatingShieldfromEntity(getPlayerEntity(), "rotating_shield_projectile_poison_shot")
		end,
		overtime_function		= function()
			if(isRelicOwned("gravity_stone_purple") >= 1) then
				if(not entityHasRotatingShield(getPlayerEntity(), "rotating_shield_projectile_poison_shot")) then -- this makes sure the orbiting projectiles are loaded again if the player is polymorphed
					local start_id = EntityLoad("mods/ArcaneReliquary/files/entity_definitions/custom_projectiles/rotating_shield_start.xml")
					setInternalVariableValue(start_id, "host_entity_id", "value_int", getPlayerEntity())
					setInternalVariableValue(start_id, "rotating_projectile_definition", "value_string", "mods/ArcaneReliquary/files/entity_definitions/custom_projectiles/rotating_shield_poison_shot_part.xml")
					setInternalVariableValue(start_id, "rotating_projectiles_amount", "value_int", 2)
					setInternalVariableValue(start_id, "radius", "value_int", 23)
					setInternalVariableValue(start_id, "rotation_speed", "value_int", 5)
				end
			end
		end,
	},
	{
		name 					= "gravity_stone_green",
		ui_name					= "Green Gravity Stone",
		categories				= "player,materials",
		effect_desc				= "Green energy spheres that convert enemy projectiles \ninto acid orbs and shoot them back orbit around you.",
		effect_desc_short		= "Green energy spheres that convert enemy projectiles into acid orbs orbit around you.",
        precursor_types			= "gravity_stone",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		ondrop_function			= function()
			detachRotatingShieldfromEntity(getPlayerEntity(), "rotating_shield_projectile_acid_shot")
		end,
		overtime_function		= function()
			if(isRelicOwned("gravity_stone_green") >= 1) then
				if(not entityHasRotatingShield(getPlayerEntity(), "rotating_shield_projectile_acid_shot")) then -- this makes sure the orbiting projectiles are loaded again if the player is polymorphed
					local start_id = EntityLoad("mods/ArcaneReliquary/files/entity_definitions/custom_projectiles/rotating_shield_start.xml")
					setInternalVariableValue(start_id, "host_entity_id", "value_int", getPlayerEntity())
					setInternalVariableValue(start_id, "rotating_projectile_definition", "value_string", "mods/ArcaneReliquary/files/entity_definitions/custom_projectiles/rotating_shield_acid_shot_part.xml")
					setInternalVariableValue(start_id, "rotating_projectiles_amount", "value_int", 2)
					setInternalVariableValue(start_id, "radius", "value_int", 25)
					setInternalVariableValue(start_id, "rotation_speed", "value_int", 5)
				end
			end
		end,
	},
	{
		name 					= "sawblade",
		ui_name					= "Sawblade",
		categories				= "player",
		effect_desc				= "Sawblades that deal slice damage to enemies they \ntouch and destroy any projectile (including your own) orbit around you.",
		effect_desc_short		= "Sawblades that deal slice damage orbit around you.",
        precursor_types			= "gadget",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		ondrop_function			= function()
			detachRotatingShieldfromEntity(getPlayerEntity(), "rotating_shield_projectile_disk_bullet")
		end,
		overtime_function		= function()
			if(isRelicOwned("sawblade") >= 1) then
				if(not entityHasRotatingShield(getPlayerEntity(), "rotating_shield_projectile_disk_bullet")) then -- this makes sure the orbiting projectiles are loaded again if the player is polymorphed
					local start_id = EntityLoad("mods/ArcaneReliquary/files/entity_definitions/custom_projectiles/rotating_shield_start.xml")
					setInternalVariableValue(start_id, "host_entity_id", "value_int", getPlayerEntity())
					setInternalVariableValue(start_id, "rotating_projectile_definition", "value_string", "mods/ArcaneReliquary/files/entity_definitions/custom_projectiles/rotating_disc_bullet_big.xml")
					setInternalVariableValue(start_id, "rotating_projectiles_amount", "value_int", 3)
					setInternalVariableValue(start_id, "radius", "value_int", 25)
					setInternalVariableValue(start_id, "rotation_speed", "value_int", 5)
				end
			end
		end,
	},
	{
		name 					= "targeting_device",
		ui_name					= "Targeting Device",
		categories				= "projectiles",
		effect_desc				= "Spells you cast have a 50% chance of shooting rockets towards nearby enemies.",
		effect_desc_short		= "Your spells shoot rockets at enemies.",
        precursor_types			= "gadget,device",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("targeting_device") >= 1) then
				if(not isPhysicsProjectile(projectile_entity_id)) then
					SetRandomSeed( GameGetFrameNum(), math.random(1,50))
					local chance = Random(1,100)
					local odds = 50 + (5 * getRelicLevel("targeting_device"))
					if(chance >= 1 and chance <= odds) then
						addBuffVisualIndicator_Projectile(projectile_entity_id, "spark_white")
						addNewInternalVariable(projectile_entity_id, "projectile_definition", "value_string", "mods/ArcaneReliquary/files/entity_definitions/custom_projectiles/rocket_tank_custom.xml")
						addNewInternalVariable(projectile_entity_id, "detection_range", "value_int", 100)
						addNewInternalVariable(projectile_entity_id, "projectile_velocity", "value_int", 350)
						addNewInternalVariable(projectile_entity_id, "target_tag", "value_string", "enemy")
						addNewInternalVariable(projectile_entity_id, "max_projectiles_shot", "value_int", 15)
						addNewLuaCallback(projectile_entity_id, "overtime", "mods/ArcaneReliquary/files/scripts/projectiles/target_and_shoot_projectile.lua", 10, false, "target_and_shoot_projectile")
					end
				end
			end
		end,
	},
	{
		name 					= "pickaxe",
		ui_name					= "Pickaxe",
		categories				= "projectiles",
		effect_desc				= "Spells you cast have a 30% chance of shooting unstable crystals downwards along their travel path.",
		effect_desc_short		= "Your spells cast unstable crystals downwards.",
        precursor_types			= "gadget",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("pickaxe") >= 1) then
				if(not isPhysicsProjectile(projectile_entity_id)) then
					SetRandomSeed( GameGetFrameNum(), math.random(1,50))
					local chance = Random(1,100)
					local odds = 30 + (5 * getRelicLevel("pickaxe"))
					if(chance >= 1 and chance <= odds) then	
						addNewInternalVariable(projectile_entity_id, "projectile_shot_down_definition", "value_string", "mods/ArcaneReliquary/files/entity_definitions/custom_projectiles/mine_custom.xml")
						addNewInternalVariable(projectile_entity_id, "down_vel_min", "value_int", 50)
						addNewInternalVariable(projectile_entity_id, "down_vel_max", "value_int", 80)
						addNewInternalVariable(projectile_entity_id, "max_projectiles_shot", "value_int", 10)
						addNewLuaCallback(projectile_entity_id, "overtime", "mods/ArcaneReliquary/files/scripts/projectiles/launch_projectile_down.lua", 10, false, "launch_projectile_down")
					end
				end
			end
		end,
	},
	{
		name 					= "charged_mana_crystal",
		ui_name					= "Charged Mana Crystal",
		categories				= "projectiles",
		effect_desc				= "Spells you cast have a 50% chance of spawning \nrotating orbs that electrocute enemies they touch.",
		effect_desc_short		= "Your spells spawn electrocuting rotating orbs.",
        precursor_types			= "effect_crystal",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("charged_mana_crystal") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 50 + (5 * getRelicLevel("charged_mana_crystal"))
				if(chance >= 1 and chance <= odds) then		
					addNewInternalVariable(projectile_entity_id, "rotating_projectile_definition", "value_string", "mods/ArcaneReliquary/files/entity_definitions/custom_projectiles/rotating_shield_electric_part.xml")
					addNewInternalVariable(projectile_entity_id, "rotating_projectiles_amount", "value_int", Random(2,4))
					addNewInternalVariable(projectile_entity_id, "radius", "value_int", 15)
					addNewInternalVariable(projectile_entity_id, "rotation_speed", "value_int", Random(10,20))
					addNewInternalVariable(projectile_entity_id, "shooter_id", "value_int", getPlayerEntity())
					addNewLuaCallback(projectile_entity_id, "overtime", "mods/ArcaneReliquary/files/scripts/projectiles/projectile_rotating_shield_start.lua", 1, true, "projectile_rotating_shield_start")
				end
			end
		end,
	},
	{
		name 					= "frigid_mana_crystal",
		ui_name					= "Frigid Mana Crystal",
		categories				= "projectiles",
		effect_desc				= "Spells you cast have a 50% chance of spawning \nrotating orbs that freeze enemies they touch.",
		effect_desc_short		= "Your spells spawn freezing rotating orbs.",
        precursor_types			= "effect_crystal",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("frigid_mana_crystal") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 50 + (5 * getRelicLevel("frigid_mana_crystal"))
				if(chance >= 1 and chance <= odds) then		
					addNewInternalVariable(projectile_entity_id, "rotating_projectile_definition", "value_string", "mods/ArcaneReliquary/files/entity_definitions/custom_projectiles/rotating_shield_freezing_part.xml")
					addNewInternalVariable(projectile_entity_id, "rotating_projectiles_amount", "value_int", Random(2,4))
					addNewInternalVariable(projectile_entity_id, "radius", "value_int", 15)
					addNewInternalVariable(projectile_entity_id, "rotation_speed", "value_int", Random(10,20))
					addNewInternalVariable(projectile_entity_id, "shooter_id", "value_int", getPlayerEntity())
					addNewLuaCallback(projectile_entity_id, "overtime", "mods/ArcaneReliquary/files/scripts/projectiles/projectile_rotating_shield_start.lua", 1, true, "projectile_rotating_shield_start")
				end
			end
		end,
	},
	{
		name 					= "overheated_mana_crystal",
		ui_name					= "Overheated Mana Crystal",
		categories				= "projectiles",
		effect_desc				= "Spells you cast have a 50% chance of spawning \nrotating orbs that set on fire enemies they touch.",
		effect_desc_short		= "Your spells spawn burning rotating orbs.",
        precursor_types			= "effect_crystal",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("overheated_mana_crystal") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 50 + (5 * getRelicLevel("overheated_mana_crystal"))
				if(chance >= 1 and chance <= odds) then		
					addNewInternalVariable(projectile_entity_id, "rotating_projectile_definition", "value_string", "mods/ArcaneReliquary/files/entity_definitions/custom_projectiles/rotating_shield_burning_part.xml")
					addNewInternalVariable(projectile_entity_id, "rotating_projectiles_amount", "value_int", Random(2,4))
					addNewInternalVariable(projectile_entity_id, "radius", "value_int", 15)
					addNewInternalVariable(projectile_entity_id, "rotation_speed", "value_int", Random(10,20))
					addNewInternalVariable(projectile_entity_id, "shooter_id", "value_int", getPlayerEntity())
					addNewLuaCallback(projectile_entity_id, "overtime", "mods/ArcaneReliquary/files/scripts/projectiles/projectile_rotating_shield_start.lua", 1, true, "projectile_rotating_shield_start")
				end
			end
		end,
	},
	{
		name 					= "mutating_mana_crystal",
		ui_name					= "Mutating Mana Crystal",
		categories				= "projectiles",
		effect_desc				= "Spells you cast have a 50% chance of spawning \nrotating orbs that mutate enemies they touch.",
		effect_desc_short		= "Your spells spawn mutating rotating orbs.",
        precursor_types			= "effect_crystal",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("mutating_mana_crystal") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 50 + (5 * getRelicLevel("mutating_mana_crystal"))
				if(chance >= 1 and chance <= odds) then		
					addNewInternalVariable(projectile_entity_id, "rotating_projectile_definition", "value_string", "mods/ArcaneReliquary/files/entity_definitions/custom_projectiles/rotating_shield_mutating_part.xml")
					addNewInternalVariable(projectile_entity_id, "rotating_projectiles_amount", "value_int", Random(2,4))
					addNewInternalVariable(projectile_entity_id, "radius", "value_int", 15)
					addNewInternalVariable(projectile_entity_id, "rotation_speed", "value_int", Random(10,20))
					addNewInternalVariable(projectile_entity_id, "shooter_id", "value_int", getPlayerEntity())
					addNewLuaCallback(projectile_entity_id, "overtime", "mods/ArcaneReliquary/files/scripts/projectiles/projectile_rotating_shield_start.lua", 1, true, "projectile_rotating_shield_start")
				end
			end
		end,
	},
	{
		name 					= "evil_mask",
		ui_name					= "Evil Mask",
		categories				= "player",
		effect_desc				= "When you consume at least 200 unities of meat of an innocent creature, you are \nhealed by 25, but each time this happens, there's a 50% chance a death ray is summoned at your location.",
		effect_desc_short		= "Eating meat of an innocent creature heals you, but there's a chance a death ray is summoned on you.",
        precursor_types			= "death_ray",
		upgrade_effect_desc		= "For each upgrade level, death ray chance is reduced by 5%.",
		is_upgradable 			= true,
		overtimeslow_function	= function()
			if(isRelicOwned("evil_mask") >= 1) then
				local meat_consumed = getMaterialAmountOnPlayerStomach("meat_helpless")
				if(meat_consumed > 200) then
					GamePrint("Relic effect: healed by 25 for consuming 200 meat of an innocent creature.")
					meat_consumed = meat_consumed - 200
					AddMaterialInventoryMaterial( GetUpdatedEntityID(), "meat_helpless", meat_consumed)
					
					SetRandomSeed( GameGetFrameNum(), math.random(1,50))
					local chance = Random(1,100)
					local odds = 50 - (5 * getRelicLevel("evil_mask"))
					if(chance >= 1 and chance <= odds) then	
						local player_id = GetUpdatedEntityID()
						local x, y = EntityGetTransform(player_id)
						GamePlaySound( "data/audio/Desktop/misc.bank", "misc/beam_from_sky_kick", x, y )
						EntityLoad( "mods/ArcaneReliquary/files/entity_definitions/others/death_ray_fast.xml", x, y )
					end
					
					local x, y = EntityGetTransform(GetUpdatedEntityID())
					GamePlaySound( "data/audio/Desktop/event_cues.snd", "event_cues/heart/create", x, y )
					addPlayerHealth(25, false)
				end
			end
		end,
	},
	{
		name 					= "pearl_necklace_corroding",
		ui_name					= "Corroding Pearl Necklace",
		categories				= "enemies,items,materials",
		effect_desc				= "Enemies you kill have a 20% chance of dropping acid death ray \npearls that once thrown summon an acid death ray.",
		effect_desc_short		= "Killed enemies drop acid pearls that summon acid death rays once thrown.",
        precursor_types			= "death_ray",
		upgrade_effect_desc		= "For each upgrade level, drop chance increases by 5%.",
		is_upgradable 			= true,
		onenemykill_function	= function(damage_type_bit_field, damage_message, entity_thats_responsible, drop_items)
			if(isRelicOwned("pearl_necklace_corroding") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 20 + (5 * getRelicLevel("pearl_necklace_corroding"))
				if(chance >= 1 and chance <= odds) then	
					local death_x, death_y = EntityGetTransform(GetUpdatedEntityID())
					EntityLoad("mods/ArcaneReliquary/files/entity_definitions/items/death_ray_pearl_acid.xml", death_x, death_y)
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "blaze_pearl,acid_gem,death_ray_stone",
	},
	{
		name 					= "pearl_necklace_frozen",
		ui_name					= "Frozen Pearl Necklace",
		categories				= "enemies,items,materials",
		effect_desc				= "Enemies you kill have a 20% chance of dropping ice death ray \npearls that once thrown summon an ice death ray.",
		effect_desc_short		= "Killed enemies drop ice pearls that summon ice death rays once thrown.",
        precursor_types			= "death_ray",
		upgrade_effect_desc		= "For each upgrade level, drop chance increases by 5%.",
		is_upgradable 			= true,
		onenemykill_function	= function(damage_type_bit_field, damage_message, entity_thats_responsible, drop_items)
			if(isRelicOwned("pearl_necklace_frozen") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 20 + (5 * getRelicLevel("pearl_necklace_frozen"))
				if(chance >= 1 and chance <= odds) then	
					local death_x, death_y = EntityGetTransform(GetUpdatedEntityID())
					EntityLoad("mods/ArcaneReliquary/files/entity_definitions/items/death_ray_pearl_ice.xml", death_x, death_y)
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "blaze_pearl,ice_artifact,death_ray_stone",
	},
	{
		name 					= "pearl_necklace_humid",
		ui_name					= "Humid Pearl Necklace",
		categories				= "enemies,items,materials",
		effect_desc				= "Enemies you kill have a 20% chance of dropping water death ray \npearls that once thrown summon a water death ray.",
		effect_desc_short		= "Killed enemies drop water pearls that summon water death rays once thrown.",
        precursor_types			= "death_ray",
		upgrade_effect_desc		= "For each upgrade level, drop chance increases by 5%.",
		is_upgradable 			= true,
		onenemykill_function	= function(damage_type_bit_field, damage_message, entity_thats_responsible, drop_items)
			if(isRelicOwned("pearl_necklace_humid") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 20 + (5 * getRelicLevel("pearl_necklace_humid"))
				if(chance >= 1 and chance <= odds) then	
					local death_x, death_y = EntityGetTransform(GetUpdatedEntityID())
					EntityLoad("mods/ArcaneReliquary/files/entity_definitions/items/death_ray_pearl_water.xml", death_x, death_y)
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "blaze_pearl,water_gem,death_ray_stone",
	},
	{
		name 					= "pearl_necklace_igneous",
		ui_name					= "Igneous Pearl Necklace",
		categories				= "enemies,items,materials",
		effect_desc				= "Enemies you kill have a 20% chance of dropping fire death ray \npearls that once thrown summon a lava death ray.",
		effect_desc_short		= "Killed enemies drop fire pearls that summon lava death rays once thrown.",
        precursor_types			= "death_ray",
		upgrade_effect_desc		= "For each upgrade level, drop chance increases by 5%.",
		is_upgradable 			= true,
		onenemykill_function	= function(damage_type_bit_field, damage_message, entity_thats_responsible, drop_items)
			if(isRelicOwned("pearl_necklace_igneous") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 20 + (5 * getRelicLevel("pearl_necklace_igneous"))
				if(chance >= 1 and chance <= odds) then	
					local death_x, death_y = EntityGetTransform(GetUpdatedEntityID())
					EntityLoad("mods/ArcaneReliquary/files/entity_definitions/items/death_ray_pearl_fire.xml", death_x, death_y)
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "blaze_pearl,lava_gem,death_ray_stone",
	},
	{
		name 					= "pearl_necklace_midas",
		ui_name					= "Midas Pearl Necklace",
		categories				= "enemies,items,materials",
		effect_desc				= "Enemies you kill have a 10% chance of dropping gold death ray \npearls that once thrown summon a gold death ray.",
		effect_desc_short		= "Killed enemies drop gold pearls that summon gold death rays once thrown.",
        precursor_types			= "death_ray",
		upgrade_effect_desc		= "For each upgrade level, drop chance increases by 5%.",
		is_upgradable 			= true,
		onenemykill_function	= function(damage_type_bit_field, damage_message, entity_thats_responsible, drop_items)
			if(isRelicOwned("pearl_necklace_midas") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 10 + (5 * getRelicLevel("pearl_necklace_midas"))
				if(chance >= 1 and chance <= odds) then	
					local death_x, death_y = EntityGetTransform(GetUpdatedEntityID())
					EntityLoad("mods/ArcaneReliquary/files/entity_definitions/items/death_ray_pearl_gold.xml", death_x, death_y)
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "blaze_pearl,midas_essence,death_ray_stone",
	},
	{
		name 					= "pearl_necklace_urinous",
		ui_name					= "Urinous Pearl Necklace",
		categories				= "enemies,items,materials",
		effect_desc				= "Enemies you kill have a 20% chance of dropping urine death ray \npearls that once thrown summon an urine death ray.",
		effect_desc_short		= "Killed enemies drop urine pearls that summon urine death rays once thrown.",
        precursor_types			= "death_ray",
		upgrade_effect_desc		= "For each upgrade level, drop chance increases by 5%.",
		is_upgradable 			= true,
		onenemykill_function	= function(damage_type_bit_field, damage_message, entity_thats_responsible, drop_items)
			if(isRelicOwned("pearl_necklace_urinous") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 20 + (5 * getRelicLevel("pearl_necklace_urinous"))
				if(chance >= 1 and chance <= odds) then	
					local death_x, death_y = EntityGetTransform(GetUpdatedEntityID())
					EntityLoad("mods/ArcaneReliquary/files/entity_definitions/items/death_ray_pearl_urine.xml", death_x, death_y)
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "blaze_pearl,urine_gem,death_ray_stone",
	},
	{
		name 					= "pearl_necklace_vile",
		ui_name					= "Vile Pearl Necklace",
		categories				= "enemies,items,materials",
		effect_desc				= "Enemies you kill have a 20% chance of dropping poison death ray \npearls that once thrown summon a poison death ray.",
		effect_desc_short		= "Killed enemies drop poison pearls that summon poison death rays once thrown.",
        precursor_types			= "death_ray",
		upgrade_effect_desc		= "For each upgrade level, drop chance increases by 5%.",
		is_upgradable 			= true,
		onenemykill_function	= function(damage_type_bit_field, damage_message, entity_thats_responsible, drop_items)
			if(isRelicOwned("pearl_necklace_vile") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 20 + (5 * getRelicLevel("pearl_necklace_vile"))
				if(chance >= 1 and chance <= odds) then	
					local death_x, death_y = EntityGetTransform(GetUpdatedEntityID())
					EntityLoad("mods/ArcaneReliquary/files/entity_definitions/items/death_ray_pearl_poison.xml", death_x, death_y)
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "blaze_pearl,poison_gem,death_ray_stone",
	},
	{
		name 					= "spell_charm",
		ui_name					= "Spell Charm",
		categories				= "enemies,items",
		effect_desc				= "When you kill a buffed enemy with spells, there's \na 10% chance a random collectible spell card will be dropped.",
		effect_desc_short		= "Buffed enemies killed with your spells might drop random spell cards.",
        precursor_types			= "magic_artifact",
		upgrade_effect_desc		= "For each upgrade level, drop chance increases by 5%.",
		is_upgradable 			= true,
		onenemykill_function	= function(damage_type_bit_field, damage_message, entity_thats_responsible, drop_items)
			if(isRelicOwned("spell_charm") >= 1) then
				if(entityHasSimplifiedInternalVariable(GetUpdatedEntityID(), "relic_chest_buff")) then
					if(damage_message == "$damage_projectile" or damage_message == "$damage_explosion") then
						SetRandomSeed( GameGetFrameNum(), math.random(1,50))
						local chance = Random(1,100)
						local odds = 10 + (5 * getRelicLevel("spell_charm"))
						if(chance >= 1 and chance <= odds) then	
							local death_x, death_y = EntityGetTransform(GetUpdatedEntityID())
							CreateItemActionEntity( getRandomActionId(), death_x, death_y)
						end
					end
				end
			end
		end,
	},
	{
		name 					= "holy_brick",
		ui_name					= "Holy Brick",
		categories				= "enemies,items",
		effect_desc				= "When you kill a buffed enemy with spells, there's \na 10% chance a random perk card will be dropped.",
		effect_desc_short		= "Buffed enemies killed with your spells might drop random perk cards.",
        precursor_types			= "magic_artifact",
		upgrade_effect_desc		= "For each upgrade level, drop chance increases by 5%.",
		is_upgradable 			= true,
		onenemykill_function	= function(damage_type_bit_field, damage_message, entity_thats_responsible, drop_items)
			if(isRelicOwned("holy_brick") >= 1) then
				if(entityHasSimplifiedInternalVariable(GetUpdatedEntityID(), "relic_chest_buff")) then
					if(damage_message == "$damage_projectile" or damage_message == "$damage_explosion") then
						SetRandomSeed( GameGetFrameNum(), math.random(1,50))
						local chance = Random(1,100)
						local odds = 10 + (5 * getRelicLevel("holy_brick"))
						if(chance >= 1 and chance <= odds) then	
							local death_x, death_y = EntityGetTransform(GetUpdatedEntityID())
							spawnRandomPerkCard(death_x, death_y - 10)
						end
					end
				end
			end
		end,
	},
	{
		name 					= "mysterious_plant",
		ui_name					= "Mysterious Plant",
		categories				= "enemies,items",
		effect_desc				= "When you kill an enemy with spells, there's a 50% chance a special seed will be dropped by them. \nThese seeds create portals that shoot various spells at random directions.",
		effect_desc_short		= "Killed enemies drop special seeds.",
        precursor_types			= "magic_artifact",
		upgrade_effect_desc		= "For each upgrade level, drop chance increases by 5%.",
		is_upgradable 			= true,
		onenemykill_function	= function(damage_type_bit_field, damage_message, entity_thats_responsible, drop_items)
			if(isRelicOwned("mysterious_plant") >= 1) then
				if(damage_message == "$damage_projectile") then
					SetRandomSeed( GameGetFrameNum(), math.random(1,50))
					local chance = Random(1,100)
					local odds = 50 + (5 * getRelicLevel("mysterious_plant"))
					if(chance >= 1 and chance <= odds) then	
						seed_pool = {"seed_purple_slime", "seed_toxic", "seed_blood", "seed_water", "seed_acid", "seed_charged", "seed_explosive", "seed_fire", "seed_ice", "seed_laser", "seed_lava", "seed_poison", "seed_poly", "seed_slime", "seed_thunder", "seed_urine"}
						local death_x, death_y = EntityGetTransform(GetUpdatedEntityID())
						EntityLoad("mods/ArcaneReliquary/files/entity_definitions/items/" .. seed_pool[Random(1,#seed_pool)] .. ".xml", death_x, death_y - 10)
					end
				end
			end
		end,
	},
	{
		name 					= "stone_of_wealth_and_vigour",
		ui_name					= "Stone of Wealth and Vigour",
		categories				= "enemies,items",
		effect_desc				= "When you kill an enemy with spells, there's a 25% chance a blessed briefcase will be dropped. \nWhen you throw it in the air, it creates a portal that launches gold and healing nuggets.",
		effect_desc_short		= "Killed enemies drop blessed briefcases.",
        precursor_types			= "effect_stone",
		upgrade_effect_desc		= "For each upgrade level, drop chance increases by 5%.",
		is_upgradable 			= true,
		onenemykill_function	= function(damage_type_bit_field, damage_message, entity_thats_responsible, drop_items)
			if(isRelicOwned("stone_of_wealth_and_vigour") >= 1) then
				if(damage_message == "$damage_projectile") then
					SetRandomSeed( GameGetFrameNum(), math.random(1,50))
					local chance = Random(1,100)
					local odds = 25 + (5 * getRelicLevel("stone_of_wealth_and_vigour"))
					if(chance >= 1 and chance <= odds) then	
						local death_x, death_y = EntityGetTransform(GetUpdatedEntityID())
						EntityLoad("mods/ArcaneReliquary/files/entity_definitions/items/blessed_briefcase.xml", death_x, death_y - 10)
					end
				end
			end
		end,
	},
	{
		name 					= "rainbow_gem",
		ui_name					= "Rainbow Gem",
		categories				= "companions,materials",
		effect_desc				= "A rainbow sentient crystal follows your mouse \ncursor, dripping materials it comes in contact with.",
		effect_desc_short		= "A rainbow sentient crystal follows your mouse cursor, dripping materials it comes in contact with.",
        precursor_types			= "elemental_gem",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			attachCompaniontoEntity(getPlayerEntity(), "sentient_crystal_rainbow", "mortal")
		end,
		ondrop_function			= function()
			detachCompanionfromEntity(getPlayerEntity(), "sentient_crystal_rainbow")
		end,
		--[[
		onpickup_function		= function()
			if(EntityGetWithName( "sentient_crystal_rainbow" ) == nil or EntityGetWithName( "sentient_crystal_rainbow" ) == 0) then
				local x, y = DEBUG_GetMouseWorld()
				local companion_id = EntityLoad( "mods/ArcaneReliquary/files/entity_definitions/companions/sentient_crystal_rainbow.xml", x, y )
				addEntityMaterialAreaCheckerMechanic(companion_id, "companion")
				addNewSimplifiedInternalVariable( companion_id, "relic_companion" )
				EntityAddChild( getPlayerEntity(), companion_id )
			end
		end,
		ondrop_function			= function()
			local companion_id = EntityGetWithName( "sentient_crystal_rainbow" )
			if(companion_id ~= nil or companion_id ~= 0) then
				local x, y = EntityGetTransform(companion_id)
				EntityKill(companion_id)
				EntityLoad("mods/ArcaneReliquary/files/entity_definitions/others/explosion_companion.xml", x, y)
			end
		end,
		]]
	},
	{
		name 					= "companion_charm",
		ui_name					= "Companion Charm",
		categories				= "projectiles",
		effect_desc				= "For every relic you possess related to \ncompanions, your spells gain a small critical chance boost.",
		effect_desc_short		= "Owning companion-related relics gives you critical chance bonus.",
        precursor_types			= "magic_artifact",
		upgrade_effect_desc		= "For each upgrade level, critical chance boost is increased.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("companion_charm") >= 1) then
				local bonus_critical_chance_threshold = 2 + (1 * getRelicLevel("companion_charm"))
				local owned_relics = getOwnedRelics()
				for i,owned_relic in ipairs(owned_relics) do
					if(string.find(getRelic(EntityGetName(owned_relic)).categories, "companions")) then
						local chance, damage_multiplier = getProjectileCriticalDamageSettings(projectile_entity_id)
						setProjectileCriticalDamageSettings(projectile_entity_id, chance + bonus_critical_chance_threshold, damage_multiplier)
					end
				end
			end
		end,
	},
	{
		name 					= "pearl_necklace_rainbow",
		ui_name					= "Rainbow Pearl Necklace",
		categories				= "projectiles",
		effect_desc				= "A rainbow orb orbits around you, dripping materials your spells detect. \nTo change its dripping material, cast a spell on a material puddle.",
		effect_desc_short		= "A rainbow orb orbits around, dripping materials detected by your spells.",
        precursor_types			= "gravity_stone",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("pearl_necklace_rainbow") >= 1) then
				addEntityMaterialAreaCheckerMechanic(projectile_entity_id, "projectile")
			end
		end,
		ondrop_function			= function()
			detachRotatingShieldfromEntity(getPlayerEntity(), "rotating_shield_projectile_drip_materials")
		end,
		overtime_function		= function()
			if(isRelicOwned("pearl_necklace_rainbow") >= 1) then
				if(not entityHasRotatingShield(getPlayerEntity(), "rotating_shield_projectile_drip_materials")) then -- this makes sure the orbiting projectiles are loaded again if the player is polymorphed
					local start_id = EntityLoad("mods/ArcaneReliquary/files/entity_definitions/custom_projectiles/rotating_shield_start.xml")
					setInternalVariableValue(start_id, "host_entity_id", "value_int", getPlayerEntity())
					setInternalVariableValue(start_id, "rotating_projectile_definition", "value_string", "mods/ArcaneReliquary/files/entity_definitions/custom_projectiles/rotating_shield_drip_materials_part.xml")
					setInternalVariableValue(start_id, "rotating_projectiles_amount", "value_int", 1)
					setInternalVariableValue(start_id, "radius", "value_int", 34)
					setInternalVariableValue(start_id, "rotation_speed", "value_int", 2)
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "blaze_pearl,rainbow_gem",
	},
	{
		name 					= "rainbow_jetpack",
		ui_name					= "Rainbow Jetpack",
		categories				= "player,jetpack,materials",
		effect_desc				= "Your jetpack emits materials you immerse yourself in. \nThe Water Gem relic cancels this effect.",
		effect_desc_short		= "Your jetpack emits materials you immerse yourself in. The Water Gem relic cancels this effect.",
        precursor_types			= "gadget,device",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			addEntityMaterialAreaCheckerMechanic(getPlayerEntity(), "player")
		end,
		ondrop_function			= function()
			local player_children = EntityGetAllChildren(getPlayerEntity())
			if player_children == nil then return end
			for i,child in ipairs(player_children) do
				if(entityHasSimplifiedInternalVariable(child, "material_detection")) then
					EntityKill(child)
				end
			end
			
			if(getJetpackMaterial() ~= getDefaultJetpackMaterial()) then -- reverts back to default material first
				revertDefaultJetpackMaterial()
				if(isRelicOwned("water_gem") >= 1) then -- if Water Gem relic is owned, reverts back to water only
					setJetpackMaterial("water")
				end
			end
			
		end,
		overtime_function		= function()
			if(isRelicOwned("water_gem") < 1) then -- Water Gem cancels this effect
				if(isRelicOwned("rainbow_jetpack") >= 1) then 
					local last_material_detected_player = getInternalVariableValue(getPlayerEntity(), "last_material_detected_player", "value_string")
					setJetpackMaterial(last_material_detected_player)
				end
			end
		end,
	},
	{
		name 					= "ember_stone",
		ui_name					= "Ember Stone",
		categories				= "projectiles",
		effect_desc				= "Your spells have a 30% chance of exploding into a \nbarrage of fireballs when they explode or hit something. \nIt also synergizes with Earth Core Fragment relic effect.",
		effect_desc_short		= "Your spells explode into fireballs on impact.",
        precursor_types			= "effect_stone",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("ember_stone") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 30 + (5 * getRelicLevel("ember_stone"))
				if(chance >= 1 and chance <= odds) then	
					addBuffVisualIndicator_Projectile(projectile_entity_id, "liquid_fire")
					attachLoadEntityOnExplosiontoProjectile(projectile_entity_id, getLoadOnExplosion("launch_circular_projectiles_fireballs").path)
				end
			end
		end,
		ongroundpound_function	= function( entity_id)
			if(isRelicOwned("ember_stone") >= 1) then
				LoadGameEffectEntityTo( entity_id, getCustomEffect("effect_shoot_projectiles_above_fireball").path )
			end
		end,
	},
	{
		name 					= "frost_artifact",
		ui_name					= "Frost Artifact",
		categories				= "projectiles",
		effect_desc				= "Your spells have a 30% chance of exploding into a \nbarrage of iceballs when they explode or hit something. \nIt also synergizes with Earth Core Fragment relic effect.",
		effect_desc_short		= "Your spells explode into iceballs on impact.",
        precursor_types			= "effect_stone,magic_artifact",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("frost_artifact") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 30 + (5 * getRelicLevel("frost_artifact"))
				if(chance >= 1 and chance <= odds) then	
					addBuffVisualIndicator_Projectile(projectile_entity_id, "spark_white")
					attachLoadEntityOnExplosiontoProjectile(projectile_entity_id, getLoadOnExplosion("launch_circular_projectiles_iceballs").path)
				end
			end
		end,
		ongroundpound_function	= function( entity_id)
			if(isRelicOwned("frost_artifact") >= 1) then
				LoadGameEffectEntityTo( entity_id, getCustomEffect("effect_shoot_projectiles_above_iceball").path )
			end
		end,
	},
	{
		name 					= "gravity_stone_grey",
		ui_name					= "Grey Gravity Stone",
		categories				= "player",
		effect_desc				= "Grey energy spheres orbit around you. If they touch an \nenemy, they explode and deal damage to them. New spheres appear when the last one explodes.\nYou take double melee, explosion and projectile and damage.",
		effect_desc_short		= "Grey energy spheres orbit around you. If they touch an enemy, they take a random amount of damage.",
        precursor_types			= "gravity_stone",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		ondrop_function			= function()
			detachRotatingShieldfromEntity(getPlayerEntity(), "rotating_shield_projectile_damage")
		end,
		overtime_function		= function()
			if(isRelicOwned("gravity_stone_grey") >= 1) then
				if(not entityHasRotatingShield(getPlayerEntity(), "rotating_shield_projectile_damage")) then -- this makes sure the orbiting projectiles are loaded again if the player is polymorphed
					local start_id = EntityLoad("mods/ArcaneReliquary/files/entity_definitions/custom_projectiles/rotating_shield_start.xml")
					setInternalVariableValue(start_id, "host_entity_id", "value_int", getPlayerEntity())
					setInternalVariableValue(start_id, "rotating_projectile_definition", "value_string", "mods/ArcaneReliquary/files/entity_definitions/custom_projectiles/rotating_shield_damage_part.xml")
					setInternalVariableValue(start_id, "rotating_projectiles_amount", "value_int", 8)
					setInternalVariableValue(start_id, "radius", "value_int", 20)
					setInternalVariableValue(start_id, "rotation_speed", "value_int", 5)
				end
			end
		end,
		ondamaged_function		= function(damage, message, entity_thats_responsible, is_fatal, projectile_thats_responsible )
			if(isRelicOwned("gravity_stone_grey") >= 1) then
				if(message == "$damage_projectile") then
					EntityInflictDamage( getPlayerEntity(), damage, "DAMAGE_PROJECTILE", "Grey Gravity Stone damage penalty", "NONE", 0, 0)
				elseif(message == "$damage_melee") then
					EntityInflictDamage( getPlayerEntity(), damage, "DAMAGE_MELEE", "Grey Gravity Stone damage penalty", "NONE", 0, 0)
				elseif(message == "$damage_explosion") then
					EntityInflictDamage( getPlayerEntity(), damage, "DAMAGE_EXPLOSION", "Grey Gravity Stone damage penalty", "NONE", 0, 0)
				end
			end
		end,
	},
	{
		name 					= "slot_machine",
		ui_name					= "Slot Machine",
		categories				= "player",
		effect_desc				= "Fortune orbs rotate around you. If an enemy comes close, it \nis captured by them and when 5 are captured, a random prize is spawned.",
		effect_desc_short		= "Fortune orbs that capture enemies and convert them into prizes orbit around you.",
        precursor_types			= "gravity_stone,device",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		ondrop_function			= function()
			detachRotatingShieldfromEntity(getPlayerEntity(), "rotating_shield_projectile_fortune")
		end,
		overtimeslow_function	= function()
			
			if(isRelicOwned("slot_machine") >= 1) then
				
				local prize_wheel_count = getInternalVariableValue(getPlayerEntity(), "prize_wheel_count", "value_int")
				
				if(prize_wheel_count == 5) then
					
					local prize_pool_items = {
						"data/entities/items/pickup/chest_random.xml",
						"data/entities/items/pickup/spell_refresh.xml",
						"data/entities/items/pickup/heart_fullhp.xml",
						"data/entities/items/pickup/beamstone.xml",
						"data/entities/items/pickup/broken_wand.xml",
						"data/entities/items/pickup/moon.xml",
						"data/entities/items/pickup/potion_random_material.xml",
						"data/entities/items/pickup/jar_of_urine.xml",
						"data/entities/items/pickup/evil_eye.xml",
						"data/entities/items/pickup/brimstone.xml",
						"data/entities/items/pickup/thunderstone.xml",
						"data/entities/items/pickup/powder_stash.xml",
						"data/entities/items/pickup/goldnugget_1000.xml",
						"mods/ArcaneReliquary/files/entity_definitions/props/relic_chest.xml",
						"mods/ArcaneReliquary/files/entity_definitions/props/relic_chest_great.xml",
						"mods/ArcaneReliquary/files/entity_definitions/props/relic_chest_chaotic.xml"
					}
					
					local x, y = EntityGetTransform(getPlayerEntity())
					SetRandomSeed( GameGetFrameNum(), math.random(1,50))
					local chance = Random(1,3)
					if(chance == 1) then
						EntityLoad(prize_pool_items[Random(1,#prize_pool_items)], x, y - 15)
					elseif(chance == 2) then
						CreateItemActionEntity( getRandomActionId(), x, y - 15)
					elseif(chance == 3) then
						spawnRandomPerkCard(x, y - 15)
					end
					
					EntityLoad("mods/ArcaneReliquary/files/entity_definitions/particle_explosions/prize_wheel_explosion.xml", x, y - 15)
					
					while(EntityGetWithName("rotating_shield_projectile_fortune") ~= 0) do
						local projectile_id = EntityGetWithName("rotating_shield_projectile_fortune")
						EntityKill(projectile_id)
					end
					
					
				end
			end
		end,
		overtime_function		= function()
			if(isRelicOwned("slot_machine") >= 1) then
				if(not entityHasRotatingShield(getPlayerEntity(), "rotating_shield_projectile_fortune")) then -- this makes sure the orbiting projectiles are loaded again if the player is polymorphed
					local start_id = EntityLoad("mods/ArcaneReliquary/files/entity_definitions/custom_projectiles/rotating_shield_start.xml")
					setInternalVariableValue(start_id, "host_entity_id", "value_int", getPlayerEntity())
					setInternalVariableValue(start_id, "rotating_projectile_definition", "value_string", "mods/ArcaneReliquary/files/entity_definitions/custom_projectiles/rotating_shield_fortune_part.xml")
					setInternalVariableValue(start_id, "rotating_projectiles_amount", "value_int", 5)
					setInternalVariableValue(start_id, "radius", "value_int", 35)
					setInternalVariableValue(start_id, "rotation_speed", "value_int", 2)
				end
			end
		end,
	},
	{
		name 					= "sawblade_bloody",
		ui_name					= "Bloody Sawblade",
		categories				= "projectiles",
		effect_desc				= "Spells you cast have a 50% chance of spawning rotating sawblades \nthat destroy enemy projectiles and deal slice damage to enemies they touch.",
		effect_desc_short		= "Your spells spawn rotating sawblades.",
        precursor_types			= "gadget",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("sawblade_bloody") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 50 + (5 * getRelicLevel("frost_artifact"))
				if(chance >= 1 and chance <= odds) then		
					addNewInternalVariable(projectile_entity_id, "rotating_projectile_definition", "value_string", "mods/ArcaneReliquary/files/entity_definitions/custom_projectiles/rotating_disc_bullet_big_projectile_owned.xml")
					addNewInternalVariable(projectile_entity_id, "rotating_projectiles_amount", "value_int", Random(2,4))
					addNewInternalVariable(projectile_entity_id, "radius", "value_int", Random(15,20))
					addNewInternalVariable(projectile_entity_id, "rotation_speed", "value_int", Random(10,20))
					addNewInternalVariable(projectile_entity_id, "shooter_id", "value_int", getPlayerEntity())
					addNewLuaCallback(projectile_entity_id, "overtime", "mods/ArcaneReliquary/files/scripts/projectiles/projectile_rotating_shield_start.lua", 1, true, "projectile_rotating_shield_start")
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "sawblade,pizza_slice",
	},
	{
		name 					= "hiisi_statue",
		ui_name					= "Hiisi Statue",
		categories				= "enemies,items",
		effect_desc				= "Enemies you kill have a 30% chance of dropping mob balls on death. \nThese mob balls can be thrown on enemies to capture them, and then thrown again to spawn the captured enemy as a friendly mob.",
		effect_desc_short		= "Killed enemies drop mob balls, with which you can capture and spawn friendly mobs.",
        precursor_types			= "magic_artifact",
		upgrade_effect_desc		= "For each upgrade level, drop chance increases by 5%.",
		is_upgradable 			= true,
		onenemykill_function	= function(damage_type_bit_field, damage_message, entity_thats_responsible, drop_items)
			if(isRelicOwned("hiisi_statue") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 30 + (5 * getRelicLevel("hiisi_statue"))
				if(chance >= 1 and chance <= odds) then	
					local death_x, death_y = EntityGetTransform(GetUpdatedEntityID())
					EntityLoad("mods/ArcaneReliquary/files/entity_definitions/items/mob_ball.xml", death_x, death_y)
				end
			end
		end,
	},
	{
		name 					= "hiisi_statue_golden",
		ui_name					= "Golden Hiisi Statue",
		categories				= "enemies",
		effect_desc				= "Enemies damaged by explosions caused by your spells \nhave a 10% chance of being instantly turned into gold.",
		effect_desc_short		= "Enemies damaged with explosions from your spells turn into gold.",
        precursor_types			= "magic_artifact",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onenemydamaged_function	= function(damage, message, entity_thats_responsible, is_fatal, projectile_thats_responsible )
			if(isRelicOwned("hiisi_statue_golden") >= 1) then
				if(message == "$damage_explosion") then
					local enemy_id = GetUpdatedEntityID()
					SetRandomSeed( GameGetFrameNum(), math.random(1,50))
					local chance = Random(1,100)
					local odds = 10 + (5 * getRelicLevel("hiisi_statue_golden"))
					if(chance >= 1 and chance <= odds) then
						LoadGameEffectEntityTo( enemy_id, getCustomEffect("effect_convert_to_gold").path )
					end
				end
			end
		end,
	},
	{
		name 					= "credit_card",
		ui_name					= "Credit Card",
		categories				= "player",
		effect_desc				= "You receive a 50% discount on relic stores.",
		effect_desc_short		= "You receive a 50% discount on relic stores.",
        precursor_types			= "gadget",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			addNewSimplifiedInternalVariable(getPlayerEntity(), "relic_store_discount")
		end,
		ondrop_function			= function()
			removeSimplifiedInternalVariable(getPlayerEntity(), "relic_store_discount")
		end
	},
	{
		name 					= "chemistry_kit",
		ui_name					= "Chemistry Kit",
		categories				= "player",
		effect_desc				= "Material budget required to upgrade relics on relic forges is now halved.",
		effect_desc_short		= "Halves the material budget required to upgrade relics on relic forges.",
        precursor_types			= "gadget,device",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			addNewSimplifiedInternalVariable(getPlayerEntity(), "upgrade_material_budget_discount")
		end,
		ondrop_function			= function()
			removeSimplifiedInternalVariable(getPlayerEntity(), "upgrade_material_budget_discount")
		end
	},
	{
		name 					= "gravitation_necklace",
		ui_name					= "Gravitation Necklace",
		categories				= "player,jetpack",
		effect_desc				= "You lose the ability to fly with your jetpack, but you gain the ability \nto wall climb when very close to walls (to use this ability press jump when moving towards a wall while midair) and perform double jumps. \nIf you own any relics related to stomp damage, their effects are also improved.",
		effect_desc_short		= "You lose your jetpack, but can now climb walls and perform double jumps.",
        precursor_types			= "necklace",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			addNewLuaCallback(getPlayerEntity(), "overtime", "mods/ArcaneReliquary/files/scripts/player/walljump_listener.lua", 1, false, "walljump_listener")
			enableJumpingandDisableJetpack()
			addNewSimplifiedInternalVariable(getPlayerEntity(), "improved_stomp_relics")
		end,
		ondrop_function			= function()
			removeCallback(getPlayerEntity(), "walljump_listener")
			revertOriginalJetpackFunctionality()
			removeSimplifiedInternalVariable(getPlayerEntity(), "improved_stomp_relics")
		end
	},
	{
		name 					= "jade_key",
		ui_name					= "Jade Key",
		categories				= "player,inventory",
		effect_desc				= "When in close vicinity to an untouched wand, if they are \ncurrently locked, your inventories become accessible.",
		effect_desc_short		= "When close to untouched wands, your inventories become accessible.",
        precursor_types			= "magic_artifact",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		overtime_function		= function()
			if(isRelicOwned("jade_key") >= 1) then
				local x, y = EntityGetTransform(getPlayerEntity())
				local wands_in_range = EntityGetInRadiusWithTag( x, y, 25, "wand" )
				if(#wands_in_range > 0) then
					for i,wand in ipairs(wands_in_range) do
						if(not wandHasBeenPicked(wand) and isItemDropped(wand) and not isStartingWand(wand)) then
							--GamePrint("### untouched wand found with id: " .. wand)
							local game_effect_comp = GetGameEffectLoadTo( getPlayerEntity(), "EDIT_WANDS_EVERYWHERE", true )
							if game_effect_comp ~= nil then
								ComponentSetValue( game_effect_comp, "frames", "55" )
							end
							break
						end
					end
				end
			end
		end,
	},
	{
		name 					= "glass_bomb",
		ui_name					= "Glass Bomb",
		categories				= "player,projectiles",
		effect_desc				= "When in close vicinity to explosive objects, your spells \ndeal 4x damage, but taking projectile or melee damage under the same circumstance has a \nchance of producing small explosions.",
		effect_desc_short		= "Casting spells while close to explosive objects gives your spells a 4x damage multiplier.",
        precursor_types			= "magic_artifact",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("glass_bomb") >= 1) then
				local x, y = EntityGetTransform(getPlayerEntity())
				local objects_in_range = EntityGetInRadiusWithTag( x, y, 25, "prop_physics" ) -- first obtains objects tagged as physics props
				if(#objects_in_range > 0) then
					for i,object in ipairs(objects_in_range) do
						if(EntityHasTag(object, "mortal")) then -- narrows down to explosive physics objects
							multiplyProjectileBaseDamage(projectile_entity_id, 4)
						end		
					end
				end
			end
		end,
		ondamaged_function		= function(damage, message, entity_thats_responsible, is_fatal, projectile_thats_responsible )
			if(isRelicOwned("glass_bomb") >= 1) then
				if(message == "$damage_projectile" or message == "$damage_melee") then
					local x, y = EntityGetTransform(getPlayerEntity())
					local objects_in_range = EntityGetInRadiusWithTag( x, y, 30, "prop_physics" ) -- first obtains objects tagged as physics props
					if(#objects_in_range > 0) then
						for i,object in ipairs(objects_in_range) do
							if(EntityHasTag(object, "mortal")) then -- narrows down to explosive physics objects
								SetRandomSeed( GameGetFrameNum(), math.random(1,50))
								local chance = Random(1,3)
								if(chance == 1) then
									EntityLoad("mods/ArcaneReliquary/files/entity_definitions/others/small_explosion.xml", x + Random(-3, 3), y + Random(-3, 3))
								end
							end
						end
					end
				end
			end
		end,
	},
	{
		name 					= "midas_mirror",
		ui_name					= "Midas Mirror",
		categories				= "player,enemies",
		effect_desc				= "When in close vicinity to enemies, you start to spontaneously lose hp, but \neach time this happens, there's a 50% chance a gold dust explosion will happen.",
		effect_desc_short		= "When close to enemies, you lose hp but gold explosions might happen closeby.",
        precursor_types			= "magic_artifact",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		overtime_function		= function()
			if(isRelicOwned("midas_mirror") >= 1) then
				local x, y = EntityGetTransform(getPlayerEntity())
				local enemies_in_range = EntityGetInRadiusWithTag( x, y, 50, "enemy" )
				if(#enemies_in_range > 0) then
					for i,enemy in ipairs(enemies_in_range) do
						SetRandomSeed( GameGetFrameNum(), math.random(1,50))
						addPlayerHealth(Random(4,8) * -1, false)
						local chance = Random(1,100)
						local odds = 50
						if(chance >= 1 and chance <= odds) then
							local enemy_x, enemy_y = EntityGetTransform(enemy)
							EntityLoad("mods/ArcaneReliquary/files/entity_definitions/load_on_explosion/launch_elemental_particles_gold.xml", enemy_x, enemy_y)
						end
					end
				end
			end
		end,
	},
	{
		name 					= "super_serum",
		ui_name					= "Super Serum",
		categories				= "player,enemies",
		effect_desc				= "Taking melee damage has a 20% chance of making you immune to all damage types for a few seconds.",
		effect_desc_short		= "Taking melee damage has a chance of making you immune to all types of damage for a few seconds.",
        precursor_types			= "magic_artifact",
		upgrade_effect_desc		= "For each upgrade level, effect chance is increased by 5%.",
		is_upgradable 			= true,
		ondamaged_function		= function(damage, message, entity_thats_responsible, is_fatal, projectile_thats_responsible )
			if(isRelicOwned("super_serum") >= 1) then
				if(message == "$damage_melee") then
					SetRandomSeed( GameGetFrameNum(), math.random(1,50))
					local chance = Random(1,100)
					local odds = 20 + (5 * getRelicLevel("super_serum"))
					if(chance >= 1 and chance <= odds) then
						local game_effect_comp = GetGameEffectLoadTo( getPlayerEntity(), "PROTECTION_ALL", true )
						if game_effect_comp ~= nil then
							ComponentSetValue( game_effect_comp, "frames", "105" )
						end
					end
				end
			end
		end,
	},
	{
		name 					= "unsighted_essence",
		ui_name					= "Unsighted Essence",
		categories				= "player,enemies",
		effect_desc				= "Taking damage from enemy projectiles or explosions has a 20% chance of inducing you to release invisiblium.",
		effect_desc_short		= "Taking damage from enemy projectiles or explosions has a chance of inducing you to release invisiblium.",
        precursor_types			= "flask",
		upgrade_effect_desc		= "For each upgrade level, effect chance is increased by 5%.",
		is_upgradable 			= true,
		ondamaged_function		= function(damage, message, entity_thats_responsible, is_fatal, projectile_thats_responsible )
			if(isRelicOwned("unsighted_essence") >= 1) then
				if(message == "$damage_projectile" or message == "$damage_explosion") then
					SetRandomSeed( GameGetFrameNum(), math.random(1,50))
					local chance = Random(1,100)
					local odds = 20 + (5 * getRelicLevel("unsighted_essence"))
					if(chance >= 1 and chance <= odds) then
						local x, y = EntityGetTransform(getPlayerEntity())
						EntityLoad("mods/ArcaneReliquary/files/entity_definitions/load_on_explosion/launch_elemental_particles_invisiblium.xml", x, y)
						
						--[[
						local game_effect_comp = GetGameEffectLoadTo( getPlayerEntity(), "INVISIBILITY", true )
						if game_effect_comp ~= nil then
							ComponentSetValue( game_effect_comp, "frames", "105" )
						end
						]]
						
					end
				end
			end
		end,
	},
	{
		name 					= "golden_heart",
		ui_name					= "Golden Heart",
		categories				= "player,health",
		effect_desc				= "When you take damage from any source, lost hp is replenished and gold is deducted \ninstead, but if you have no gold, you lose hp equivalent to triple the damage taken.",
		effect_desc_short		= "Gold is deducted instead of hp when you take damage.",
        precursor_types			= "magic_artifact",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		ondamaged_function		= function(damage, message, entity_thats_responsible, is_fatal, projectile_thats_responsible )
			if(isRelicOwned("golden_heart") >= 1) then
				local damage_deduction = damage * 25
				if(getPlayerGold() > math.floor(damage_deduction)) then
					addPlayerGold(damage_deduction * -1)
					addPlayerHealth(damage_deduction, false)
				else
					addPlayerHealth((damage_deduction * -1) * 3, false)
				end
			end
		end,
	},
	{
		name 					= "transmuting_stone",
		ui_name					= "Transmuting Stone",
		categories				= "projectiles",
		effect_desc				= "Your spells transmute enemy projectiles into random materials.",
		effect_desc_short		= "Your spells transmute enemy projectiles into random materials.",
        precursor_types			= "effect_stone",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("transmuting_stone") >= 1) then
				if(not isOrbitingProjectile(projectile_entity_id)) then
					addNewSimplifiedInternalVariable( projectile_entity_id, "relic_effect_projectile")
					attachDragCollisionHandlertoProjectile(projectile_entity_id, getCollisionHandler("convert_enemy_projectiles_to_materials").path, false, 20, "projectile", false)
				end
			end
		end,
		onpickup_function		= function()
			addNewSimplifiedInternalVariable(getPlayerEntity(), "transmute_enemy_projectiles")
		end,
		ondrop_function			= function()
			removeSimplifiedInternalVariable(getPlayerEntity(), "transmute_enemy_projectiles")
		end
	},
	{
		name 					= "silver_bullet",
		ui_name					= "Silver Bullet",
		categories				= "player,minions",
		effect_desc				= "When you take melee or projectile damage, there's a 25% chance a friendly \nwolf will be summoned to aid you in combat.",
		effect_desc_short		= "Taking melee or projectile damage has a chance of summoning a friendly wolf.",
        precursor_types			= "gadget,magic_artifact",
		upgrade_effect_desc		= "For each upgrade level, summon chance increases by 5%.",
		is_upgradable 			= true,
		ondamaged_function		= function(damage, message, entity_thats_responsible, is_fatal, projectile_thats_responsible )
			if(isRelicOwned("silver_bullet") >= 1) then
				if(message == "$damage_projectile" or message == "$damage_melee") then
					SetRandomSeed( GameGetFrameNum(), math.random(1,50))
					local chance = Random(1,100)
					local odds = 25 + (5 * getRelicLevel("silver_bullet"))
					if(chance >= 1 and chance <= odds) then
						local x, y = EntityGetTransform(getPlayerEntity())
						local minion_spawn_x = x + Random(-5, 5)
						local minion_spawn_y = y + Random(-5, 5)
						EntityLoad("mods/ArcaneReliquary/files/entity_definitions/particle_explosions/wolf_spawn_explosion.xml", minion_spawn_x, minion_spawn_y)
						EntityLoad("mods/ArcaneReliquary/files/entity_definitions/minions/wolf.xml", minion_spawn_x, minion_spawn_y)
					end
				end
			end
		end,
	},
	{
		name 					= "alchemical_stone",
		ui_name					= "Alchemical Stone",
		categories				= "player",
		effect_desc				= "Scoring kills fills by 30 your potion flasks with the material they contain the most, but \nyou take 10x more damage from harmful materials.",
		effect_desc_short		= "Scoring kills fills by 30 your potion flasks with the material they contain the most.",
        precursor_types			= "effect_stone",
		upgrade_effect_desc		= "For each upgrade level, amount filled increases by 5.",
		is_upgradable 			= true,
		onenemykill_function	= function(damage_type_bit_field, damage_message, entity_thats_responsible, drop_items)
			if(isRelicOwned("alchemical_stone") >= 1) then
				updatePotionMaterialCount(30 + (5 * getRelicLevel(alchemical_stone)))
			end
		end,
		ondamaged_function		= function(damage, message, entity_thats_responsible, is_fatal, projectile_thats_responsible )
			if(isRelicOwned("alchemical_stone") >= 1) then
				if(string.find(message, "damage from material")) then
					GamePrint("Relic penalty: 10x more damage from material")
					EntityInflictDamage( getPlayerEntity(), damage * 9, "DAMAGE_MATERIAL", "Alchemical Stone damage penalty", "NONE", 0, 0)
				end
			end
		end,
	},
	{
		name 					= "grand_master_mask_fragment_bottom",
		ui_name					= "Grand Master Mask Fragment (Bottom)",
		categories				= "player,wands",
		effect_desc				= "Taking damage from any source except fire and materials has a 50% chance of \npermanently decreasing the cast delay of your active wand.",
		effect_desc_short		= "Taking damage permanently decreases cast delay of your active wand.",
        precursor_types			= "magic_artifact",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		ondamaged_function		= function(damage, message, entity_thats_responsible, is_fatal, projectile_thats_responsible )
			if(isRelicOwned("grand_master_mask_fragment_bottom") >= 1) then
				if(not string.find(message, "damage from material") and not message == "$damage_fire") then
					SetRandomSeed( GameGetFrameNum(), math.random(1,50))
					local chance = Random(1,100)
					local odds = 50
					if(chance >= 1 and chance <= odds) then
						wandAddCastDelay(getCurrentlyEquippedWandId(), -1)
					end
				end
			end
		end,
		excludes_relics			= "grand_master_mask_full",
		excludes_relics_desc	= "It can't be used along with full Grand Master Mask relic",
		excludes_relics_tags	= "grandmaster_mask_bottom",
	},
	{
		name 					= "grand_master_mask_fragment_left",
		ui_name					= "Grand Master Mask Fragment (Left)",
		categories				= "player,wands",
		effect_desc				= "Taking damage from any source except fire and materials has a 50% chance of \npermanently decreasing the reload time of your active wand.",
		effect_desc_short		= "Taking damage permanently decreases reload time of your active wand.",
        precursor_types			= "magic_artifact",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		ondamaged_function		= function(damage, message, entity_thats_responsible, is_fatal, projectile_thats_responsible )
			if(isRelicOwned("grand_master_mask_fragment_left") >= 1) then
				if(not string.find(message, "damage from material") and not message == "$damage_fire") then
					SetRandomSeed( GameGetFrameNum(), math.random(1,50))
					local chance = Random(1,100)
					local odds = 50
					if(chance >= 1 and chance <= odds) then
						wandAddReloadTime(getCurrentlyEquippedWandId(), -1)
					end
				end
			end
		end,
		excludes_relics			= "grand_master_mask_full",
		excludes_relics_desc	= "It can't be used along with full Grand Master Mask relic",
		excludes_relics_tags	= "grandmaster_mask_left",
	},
	{
		name 					= "grand_master_mask_fragment_right",
		ui_name					= "Grand Master Mask Fragment (right)",
		categories				= "player,wands",
		effect_desc				= "Taking damage from any source except fire and materials has a 50% chance of \npermanently increasing the maximum mana capacity of your active wand.",
		effect_desc_short		= "Taking damage permanently increases maximum mana of your active wand.",
        precursor_types			= "magic_artifact",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		ondamaged_function		= function(damage, message, entity_thats_responsible, is_fatal, projectile_thats_responsible )
			if(isRelicOwned("grand_master_mask_fragment_left") >= 1) then
				if(not string.find(message, "damage from material") and not message == "$damage_fire") then
					SetRandomSeed( GameGetFrameNum(), math.random(1,50))
					local chance = Random(1,100)
					local odds = 50
					if(chance >= 1 and chance <= odds) then
						wandAddManaMax(getCurrentlyEquippedWandId(), 5)
					end
				end
			end
		end,
		excludes_relics			= "grand_master_mask_full",
		excludes_relics_desc	= "It can't be used along with full Grand Master Mask relic",
		excludes_relics_tags	= "grandmaster_mask_right",
	},
	{
		name 					= "grand_master_mask_full",
		ui_name					= "Grand Master Mask (Full)",
		categories				= "player,wands",
		effect_desc				= "Taking damage from any source except fire and materials has a 50% chance of \npermanently increasing the maximum mana capacity and decreasing the reload time and cast delay \nof your active wand.",
		effect_desc_short		= "Taking damage permanently increases maximum mana of your active wand.",
        precursor_types			= "magic_artifact",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		ondamaged_function		= function(damage, message, entity_thats_responsible, is_fatal, projectile_thats_responsible )
			if(isRelicOwned("grand_master_mask_fragment_left") >= 1) then
				if(not string.find(message, "damage from material") and not message == "$damage_fire") then
					SetRandomSeed( GameGetFrameNum(), math.random(1,50))
					local chance = Random(1,100)
					local odds = 50
					if(chance >= 1 and chance <= odds) then
						wandAddManaMax(getCurrentlyEquippedWandId(), 5)
						wandAddReloadTime(getCurrentlyEquippedWandId(), -1)
						wandAddCastDelay(getCurrentlyEquippedWandId(), -1)
					end
				end
			end
		end,
		excludes_relics			= "grand_master_mask_fragment_bottom,grand_master_mask_fragment_left,grand_master_mask_fragment_right",
		excludes_relics_desc	= "It can't be used along with Grand Master Mask fragment relics",
		excludes_relics_tags	= "grandmaster_mask_right,grandmaster_mask_left,grandmaster_mask_bottom",
		combine_exclusive		= true,
		combine_requires_relics	= "grand_master_mask_fragment_bottom,grand_master_mask_fragment_left,grand_master_mask_fragment_right",
	},
	{
		name 					= "deflecting_medalion_charged",
		ui_name					= "Deflecting Medalion (Charged)",
		categories				= "projectiles,shields",
		effect_desc				= "An energy shield that deflects and electrically charges enemy projectiles is projected \naround you, but you take 60% more explosion and melee damage.",
		effect_desc_short		= "An energy shield tha deflects and electrically charges enemy projectiles is projected around you.",
        precursor_types			= "magic_artifact,energy_shield",
		upgrade_effect_desc		= "For each upgrade level, you take 5% less explosion and melee damage.",
		is_upgradable 			= true,
		onpickup_function		= function()
			LoadGameEffectEntityTo( getPlayerEntity(), getCustomEffect("effect_deflecting_electric_shield").path )
		end,
		ondrop_function			= function()
			local player_children = EntityGetAllChildren(getPlayerEntity())
			if player_children == nil then return end
			for i,child in ipairs(player_children) do
				if(EntityGetName(child) == "deflecting_shield_electric") then
					EntityKill(child)
					break
				end
			end
		end,
		ondamaged_function		= function(damage, message, entity_thats_responsible, is_fatal, projectile_thats_responsible )
			if(isRelicOwned("deflecting_medalion_charged") >= 1) then
				local damage_penalty_percentage = 60 - (5 * getRelicLevel("deflecting_medalion_charged"))
				if(message == "$damage_explosion") then
					GamePrint("Relic penalty: " .. damage_penalty_percentage .. "% more explosion damage taken")
					local damage_penalty = (damage * damage_penalty_percentage) / 100
					EntityInflictDamage( getPlayerEntity(), damage_penalty, "DAMAGE_EXPLOSION", "Deflecting Medalion damage penalty", "BLOOD_SPRAY", 0, 0)
				elseif(message == "$damage_melee") then
					GamePrint("Relic penalty: " .. damage_penalty_percentage .. "% more melee damage taken")
					local damage_penalty = (damage * damage_penalty_percentage) / 100
					EntityInflictDamage( getPlayerEntity(), damage_penalty, "DAMAGE_MELEE", "Deflecting Medalion damage penalty", "BLOOD_SPRAY", 0, 0)
				end
			end
		end,
		--[[
		onpickup_function		= function()
			local fire_multiplier = getBasicDamageMultiplier(getPlayerEntity(), "fire")
			setBasicDamageMultiplier(getPlayerEntity(), "fire", fire_multiplier * 2)
		end,
		ondrop_function			= function()
			local fire_multiplier = getBasicDamageMultiplier(getPlayerEntity(), "fire")
			setBasicDamageMultiplier(getPlayerEntity(), "fire", fire_multiplier / 2)
		end,
		]]
		excludes_relics			= "deflecting_medalion_master",
		excludes_relics_desc	= "It can't be used along with Master Deflecting Medallion relic",
		excludes_relics_tags	= "deflecting_medalion_charged",
		combine_exclusive		= true,
		combine_requires_relics	= "deflecting_medalion,charged_necklace",
	},
	{
		name 					= "deflecting_medalion_molten",
		ui_name					= "Deflecting Medalion (Molten)",
		categories				= "projectiles,shields",
		effect_desc				= "An energy shield that converts enemy projectiles into firebombs and shoots \nthem back is projected around you, but you take 60% more explosion and melee damage.",
		effect_desc_short		= "You gain an energy shield that converts enemy projectiles into firebombs.",
        precursor_types			= "magic_artifact,energy_shield",
		upgrade_effect_desc		= "For each upgrade level, you take 5% less explosion and melee damage.",
		is_upgradable 			= true,
		onpickup_function		= function()
			LoadGameEffectEntityTo( getPlayerEntity(), getCustomEffect("effect_firebomb_shield").path )
		end,
		ondrop_function			= function()
			local player_children = EntityGetAllChildren(getPlayerEntity())
			if player_children == nil then return end
			for i,child in ipairs(player_children) do
				if(EntityGetName(child) == "deflecting_shield_firebomb") then
					EntityKill(child)
					break
				end
			end
		end,
		ondamaged_function		= function(damage, message, entity_thats_responsible, is_fatal, projectile_thats_responsible )
			if(isRelicOwned("deflecting_medalion_molten") >= 1) then
				local damage_penalty_percentage = 60 - (5 * getRelicLevel("deflecting_medalion_molten"))
				if(message == "$damage_explosion") then
					GamePrint("Relic penalty: " .. damage_penalty_percentage .. "% more explosion damage taken")
					local damage_penalty = (damage * damage_penalty_percentage) / 100
					EntityInflictDamage( getPlayerEntity(), damage_penalty, "DAMAGE_EXPLOSION", "Deflecting Medalion damage penalty", "BLOOD_SPRAY", 0, 0)
				elseif(message == "$damage_melee") then
					GamePrint("Relic penalty: " .. damage_penalty_percentage .. "% more melee damage taken")
					local damage_penalty = (damage * damage_penalty_percentage) / 100
					EntityInflictDamage( getPlayerEntity(), damage_penalty, "DAMAGE_MELEE", "Deflecting Medalion damage penalty", "BLOOD_SPRAY", 0, 0)
				end
			end
		end,
		--[[
		onpickup_function		= function()
			local fire_multiplier = getBasicDamageMultiplier(getPlayerEntity(), "fire")
			setBasicDamageMultiplier(getPlayerEntity(), "fire", fire_multiplier * 2)
		end,
		ondrop_function			= function()
			local fire_multiplier = getBasicDamageMultiplier(getPlayerEntity(), "fire")
			setBasicDamageMultiplier(getPlayerEntity(), "fire", fire_multiplier / 2)
		end,
		]]
		excludes_relics			= "deflecting_medalion_master",
		excludes_relics_desc	= "It can't be used along with Master Deflecting Medallion relic",
		excludes_relics_tags	= "deflecting_medalion_molten",
		combine_exclusive		= true,
		combine_requires_relics	= "deflecting_medalion,sacred_ember",
	},
	{
		name 					= "deflecting_medalion_frost",
		ui_name					= "Deflecting Medalion (Frost)",
		categories				= "projectiles,shields",
		effect_desc				= "An energy shield that deflects and freeze charges enemy projectiles \nis projected around you, but you take 60% more explosion and melee damage.",
		effect_desc_short		= "You gain an energy shield that deflects and freeze charges enemy projectiles.",
        precursor_types			= "magic_artifact,energy_shield",
		upgrade_effect_desc		= "For each upgrade level, you take 5% less explosion and melee damage.",
		is_upgradable 			= true,
		onpickup_function		= function()
			LoadGameEffectEntityTo( getPlayerEntity(), getCustomEffect("effect_deflecting_freezing_shield").path )
		end,
		ondrop_function			= function()
			local player_children = EntityGetAllChildren(getPlayerEntity())
			if player_children == nil then return end
			for i,child in ipairs(player_children) do
				if(EntityGetName(child) == "deflecting_shield_freezing") then
					EntityKill(child)
					break
				end
			end
		end,
		ondamaged_function		= function(damage, message, entity_thats_responsible, is_fatal, projectile_thats_responsible )
			if(isRelicOwned("deflecting_medalion_frost") >= 1) then
				local damage_penalty_percentage = 60 - (5 * getRelicLevel("deflecting_medalion_frost"))
				if(message == "$damage_explosion") then
					GamePrint("Relic penalty: " .. damage_penalty_percentage .. "% more explosion damage taken")
					local damage_penalty = (damage * damage_penalty_percentage) / 100
					EntityInflictDamage( getPlayerEntity(), damage_penalty, "DAMAGE_EXPLOSION", "Deflecting Medalion damage penalty", "BLOOD_SPRAY", 0, 0)
				elseif(message == "$damage_melee") then
					GamePrint("Relic penalty: " .. damage_penalty_percentage .. "% more melee damage taken")
					local damage_penalty = (damage * damage_penalty_percentage) / 100
					EntityInflictDamage( getPlayerEntity(), damage_penalty, "DAMAGE_MELEE", "Deflecting Medalion damage penalty", "BLOOD_SPRAY", 0, 0)
				end
			end
		end,
		--[[
		onpickup_function		= function()
			local fire_multiplier = getBasicDamageMultiplier(getPlayerEntity(), "fire")
			setBasicDamageMultiplier(getPlayerEntity(), "fire", fire_multiplier * 2)
		end,
		ondrop_function			= function()
			local fire_multiplier = getBasicDamageMultiplier(getPlayerEntity(), "fire")
			setBasicDamageMultiplier(getPlayerEntity(), "fire", fire_multiplier / 2)
		end,
		]]
		excludes_relics			= "deflecting_medalion_master",
		excludes_relics_desc	= "It can't be used along with Master Deflecting Medallion relic",
		excludes_relics_tags	= "deflecting_medalion_frost",
		combine_exclusive		= true,
		combine_requires_relics	= "deflecting_medalion,ice_artifact",
	},
	{
		name 					= "deflecting_medalion_transmuting",
		ui_name					= "Deflecting Medalion (Transmuting)",
		categories				= "projectiles,shields",
		effect_desc				= "An energy shield that transmutes enemy projectiles into random materials \nis projected around you, but you take 60% more explosion and melee damage.",
		effect_desc_short		= "You gain an energy shield that transmutes enemy projectiles into random materials.",
        precursor_types			= "magic_artifact,energy_shield",
		upgrade_effect_desc		= "For each upgrade level, you take 5% less explosion and melee damage.",
		is_upgradable 			= true,
		onpickup_function		= function()
			LoadGameEffectEntityTo( getPlayerEntity(), getCustomEffect("effect_transmuting_shield").path )
		end,
		ondrop_function			= function()
			local player_children = EntityGetAllChildren(getPlayerEntity())
			if player_children == nil then return end
			for i,child in ipairs(player_children) do
				if(EntityGetName(child) == "transmuting_shield") then
					EntityKill(child)
					break
				end
			end
		end,
		ondamaged_function		= function(damage, message, entity_thats_responsible, is_fatal, projectile_thats_responsible )
			if(isRelicOwned("deflecting_medalion_transmuting") >= 1) then
				local damage_penalty_percentage = 60 - (5 * getRelicLevel("deflecting_medalion_transmuting"))
				if(message == "$damage_explosion") then
					GamePrint("Relic penalty: " .. damage_penalty_percentage .. "% more explosion damage taken")
					local damage_penalty = (damage * damage_penalty_percentage) / 100
					EntityInflictDamage( getPlayerEntity(), damage_penalty, "DAMAGE_EXPLOSION", "Deflecting Medalion damage penalty", "BLOOD_SPRAY", 0, 0)
				elseif(message == "$damage_melee") then
					GamePrint("Relic penalty: " .. damage_penalty_percentage .. "% more melee damage taken")
					local damage_penalty = (damage * damage_penalty_percentage) / 100
					EntityInflictDamage( getPlayerEntity(), damage_penalty, "DAMAGE_MELEE", "Deflecting Medalion damage penalty", "BLOOD_SPRAY", 0, 0)
				end
			end
		end,
		--[[
		onpickup_function		= function()
			local fire_multiplier = getBasicDamageMultiplier(getPlayerEntity(), "fire")
			setBasicDamageMultiplier(getPlayerEntity(), "fire", fire_multiplier * 2)
		end,
		ondrop_function			= function()
			local fire_multiplier = getBasicDamageMultiplier(getPlayerEntity(), "fire")
			setBasicDamageMultiplier(getPlayerEntity(), "fire", fire_multiplier / 2)
		end,
		]]
		excludes_relics			= "deflecting_medalion_master",
		excludes_relics_desc	= "It can't be used along with Master Deflecting Medallion relic",
		excludes_relics_tags	= "deflecting_medalion_transmuting",
		combine_exclusive		= true,
		combine_requires_relics	= "deflecting_medalion,transmuting_stone",
	},
	{
		name 					= "explosive_nanite_lava",
		ui_name					= "Explosive Nanite (Lava)",
		categories				= "enemies",
		effect_desc				= "Your spells deal significantly less damage, but they have a 20% chance of carrying \nexplosive nanites that turn enemies they hit into friendly suicide bombers that release lava \nwhen exploding.  Buffed enemies are immune to this effect.",
		effect_desc_short		= "Your spells turn your enemies into lava suicide bombers.",
        precursor_types			= "magic_artifact",
		upgrade_effect_desc		= "For each upgrade level, effect chance is increased by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("explosive_nanite_lava") >= 1) then
				-- bonus
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 20 + (5 * getRelicLevel("explosive_nanite_lava"))
				if(chance >= 1 and chance <= odds) then
					attachLoadEffectOntoEntitiestoProjectile(projectile_entity_id, getCustomEffect("effect_make_suicide_bomber_lava").path, "lava")
				end
				-- penalty
				fractionProjectileDamage(projectile_entity_id, 3)
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "explosive_nanite,lava_gem",
	},
	{
		name 					= "explosive_nanite_poly",
		ui_name					= "Explosive Nanite (Polymorphine)",
		categories				= "enemies",
		effect_desc				= "Your spells deal significantly less damage, but they have a 20% chance of carrying \nexplosive nanites that turn enemies they hit into friendly suicide bombers that release polymorphine \nwhen exploding.  Buffed enemies are immune to this effect.",
		effect_desc_short		= "Your spells turn your enemies into polymorphine suicide bombers.",
        precursor_types			= "magic_artifact",
		upgrade_effect_desc		= "For each upgrade level, effect chance is increased by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("explosive_nanite_poly") >= 1) then
				-- bonus
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 20 + (5 * getRelicLevel("explosive_nanite_poly"))
				if(chance >= 1 and chance <= odds) then
					attachLoadEffectOntoEntitiestoProjectile(projectile_entity_id, getCustomEffect("effect_make_suicide_bomber_poly").path, "magic_liquid_polymorph")
				end
				-- penalty
				fractionProjectileDamage(projectile_entity_id, 3)
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "explosive_nanite,poly_gem",
	},
	{
		name 					= "explosive_nanite_acid",
		ui_name					= "Explosive Nanite (Acid)",
		categories				= "enemies",
		effect_desc				= "Your spells deal significantly less damage, but they have a 20% chance of carrying \nexplosive nanites that turn enemies they hit into friendly suicide bombers that release acid \nwhen exploding.  Buffed enemies are immune to this effect.",
		effect_desc_short		= "Your spells turn your enemies into acid suicide bombers.",
        precursor_types			= "magic_artifact",
		upgrade_effect_desc		= "For each upgrade level, effect chance is increased by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("explosive_nanite_acid") >= 1) then
				-- bonus
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 20 + (5 * getRelicLevel("explosive_nanite_acid"))
				if(chance >= 1 and chance <= odds) then
					attachLoadEffectOntoEntitiestoProjectile(projectile_entity_id, getCustomEffect("effect_make_suicide_bomber_acid").path, "acid")
				end
				-- penalty
				fractionProjectileDamage(projectile_entity_id, 3)
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "explosive_nanite,acid_gem",
	},
	{
		name 					= "explosive_nanite_poison",
		ui_name					= "Explosive Nanite (Poison)",
		categories				= "enemies",
		effect_desc				= "Your spells deal significantly less damage, but they have a 20% chance of carrying \nexplosive nanites that turn enemies they hit into friendly suicide bombers that release poison \nwhen exploding.  Buffed enemies are immune to this effect.",
		effect_desc_short		= "Your spells turn your enemies into acid suicide bombers.",
        precursor_types			= "magic_artifact",
		upgrade_effect_desc		= "For each upgrade level, effect chance is increased by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("explosive_nanite_poison") >= 1) then
				-- bonus
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 20 + (5 * getRelicLevel("explosive_nanite_poison"))
				if(chance >= 1 and chance <= odds) then
					attachLoadEffectOntoEntitiestoProjectile(projectile_entity_id, getCustomEffect("effect_make_suicide_bomber_poison").path, "poison")
				end
				-- penalty
				fractionProjectileDamage(projectile_entity_id, 3)
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "explosive_nanite,poison_gem",
	},
	{
		name 					= "water_pot",
		ui_name					= "Water Pot",
		categories				= "materials,enemies",
		effect_desc				= "Spells you cast have a 10% chance of inducing enemies to explode \ninto water when they die.",
		effect_desc_short		= "Makes your enemies explode into water on death.",
        precursor_types			= "magic_artifact",
		upgrade_effect_desc		= "For each upgrade level, effect chance is increased by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("water_pot") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 10 + (5 * getRelicLevel("water_pot"))
				if(chance >= 1 and chance <= odds) then
					attachLoadEffectOntoEntitiestoProjectile(projectile_entity_id, getCustomEffect("effect_water_explosion").path, "water")
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "gold_pot,water_gem",
	},
	{
		name 					= "poly_pot",
		ui_name					= "Poly Pot",
		categories				= "materials,enemies",
		effect_desc				= "Spells you cast have a 10% chance of inducing enemies to explode \ninto polymorphine when they die.",
		effect_desc_short		= "Makes your enemies explode into polymorphine on death.",
        precursor_types			= "magic_artifact",
		upgrade_effect_desc		= "For each upgrade level, effect chance is increased by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("poly_pot") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 10 + (5 * getRelicLevel("poly_pot"))
				if(chance >= 1 and chance <= odds) then
					attachLoadEffectOntoEntitiestoProjectile(projectile_entity_id, getCustomEffect("effect_poly_explosion").path, "magic_liquid_polymorph")
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "gold_pot,poly_gem",
	},
	{
		name 					= "urine_pot",
		ui_name					= "Urine Pot",
		categories				= "materials,enemies",
		effect_desc				= "Spells you cast have a 10% chance of inducing enemies to explode \ninto urine when they die.",
		effect_desc_short		= "Makes your enemies explode into urine on death.",
        precursor_types			= "magic_artifact",
		upgrade_effect_desc		= "For each upgrade level, effect chance is increased by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("urine_pot") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 10 + (5 * getRelicLevel("urine_pot"))
				if(chance >= 1 and chance <= odds) then
					attachLoadEffectOntoEntitiestoProjectile(projectile_entity_id, getCustomEffect("effect_urine_explosion").path, "urine")
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "gold_pot,urine_gem",
	},
	{
		name 					= "blood_pot",
		ui_name					= "Blood Pot",
		categories				= "materials,enemies",
		effect_desc				= "Spells you cast have a 10% chance of inducing enemies to explode \ninto blood when they die.",
		effect_desc_short		= "Makes your enemies explode into blood on death.",
        precursor_types			= "magic_artifact",
		upgrade_effect_desc		= "For each upgrade level, effect chance is increased by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("blood_pot") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 10 + (5 * getRelicLevel("blood_pot"))
				if(chance >= 1 and chance <= odds) then
					attachLoadEffectOntoEntitiestoProjectile(projectile_entity_id, getCustomEffect("effect_blood_explosion").path, "blood")
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "gold_pot,blood_gem",
	},
	{
		name 					= "acid_pot",
		ui_name					= "Acid Pot",
		categories				= "materials,enemies",
		effect_desc				= "Spells you cast have a 10% chance of inducing enemies to explode \ninto acid when they die.",
		effect_desc_short		= "Makes your enemies explode into acid on death.",
        precursor_types			= "magic_artifact",
		upgrade_effect_desc		= "For each upgrade level, effect chance is increased by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("acid_pot") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 10 + (5 * getRelicLevel("acid_pot"))
				if(chance >= 1 and chance <= odds) then
					attachLoadEffectOntoEntitiestoProjectile(projectile_entity_id, getCustomEffect("effect_acid_explosion").path, "acid")
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "gold_pot,acid_gem",
	},
	{
		name 					= "lava_pot",
		ui_name					= "Lava Pot",
		categories				= "materials,enemies",
		effect_desc				= "Spells you cast have a 10% chance of inducing enemies to explode \ninto lava when they die.",
		effect_desc_short		= "Makes your enemies explode into lava on death.",
        precursor_types			= "magic_artifact",
		upgrade_effect_desc		= "For each upgrade level, effect chance is increased by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("lava_pot") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 10 + (5 * getRelicLevel("lava_pot"))
				if(chance >= 1 and chance <= odds) then
					attachLoadEffectOntoEntitiestoProjectile(projectile_entity_id, getCustomEffect("effect_lava_explosion").path, "lava")
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "gold_pot,lava_gem",
	},
	{
		name 					= "poison_pot",
		ui_name					= "Poison Pot",
		categories				= "materials,enemies",
		effect_desc				= "Spells you cast have a 10% chance of inducing enemies to explode \ninto poison when they die.",
		effect_desc_short		= "Makes your enemies explode into poison on death.",
        precursor_types			= "magic_artifact",
		upgrade_effect_desc		= "For each upgrade level, effect chance is increased by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("poison_pot") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 10 + (5 * getRelicLevel("poison_pot"))
				if(chance >= 1 and chance <= odds) then
					attachLoadEffectOntoEntitiestoProjectile(projectile_entity_id, getCustomEffect("effect_poison_explosion").path, "poison")
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "gold_pot,poison_gem",
	},
	{
		name 					= "power_surge_necklace",
		ui_name					= "Power Surge Necklace",
		categories				= "projectiles,enemies",
		effect_desc				= "An energy field that electrocutes enemies is projected \naround you, but you take triple projectile damage.",
		effect_desc_short		= "An electrocuting field is projected around you.",
        precursor_types			= "necklace",
		upgrade_effect_desc		= "For each upgrade level, projectile damage is decreased.",
		is_upgradable 			= true,
		onpickup_function		= function()
			LoadGameEffectEntityTo( getPlayerEntity(), getCustomEffect("effect_electrocuting_field").path )
		end,
		ondrop_function			= function()
			local player_children = EntityGetAllChildren(getPlayerEntity())
			if player_children == nil then return end
			for i,child in ipairs(player_children) do
				if(EntityGetName(child) == "electrocuting_field") then
					EntityKill(child)
					break
				end
			end
		end,
		ondamaged_function		= function(damage, message, entity_thats_responsible, is_fatal, projectile_thats_responsible )
			if(isRelicOwned("power_surge_necklace") >= 1) then
				if(message == "$damage_projectile") then
					local damage_dividend = getRelicLevel("power_surge_necklace")
					if(damage_dividend == 0) then
						damage_dividend = 1
					end
					EntityInflictDamage( getPlayerEntity(), damage * 2 / damage_dividend, "DAMAGE_PROJECTILE", "Power Surge Necklace damage penalty", "BLOOD_SPRAY", 0, 0)
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "wicked_necklace,electrical_plug",
		excludes_relics_desc	= "It can't be used along with Wicked Necklace relic",
		excludes_relics_tags	= "power_surge_necklace",
	},
	{
		name 					= "blizzard_necklace",
		ui_name					= "Blizzard Necklace",
		categories				= "projectiles,enemies",
		effect_desc				= "An energy field that freezes enemies is projected \naround you, but you take triple projectile damage.",
		effect_desc_short		= "A freezing field is projected around you.",
        precursor_types			= "necklace",
		upgrade_effect_desc		= "For each upgrade level, projectile damage is decreased.",
		is_upgradable 			= true,
		onpickup_function		= function()
			LoadGameEffectEntityTo( getPlayerEntity(), getCustomEffect("effect_freezing_field").path )
		end,
		ondrop_function			= function()
			local player_children = EntityGetAllChildren(getPlayerEntity())
			if player_children == nil then return end
			for i,child in ipairs(player_children) do
				if(EntityGetName(child) == "freezing_field") then
					EntityKill(child)
					break
				end
			end
		end,
		ondamaged_function		= function(damage, message, entity_thats_responsible, is_fatal, projectile_thats_responsible )
			if(isRelicOwned("blizzard_necklace") >= 1) then
				if(message == "$damage_projectile") then
					local damage_dividend = getRelicLevel("blizzard_necklace")
					if(damage_dividend == 0) then
						damage_dividend = 1
					end
					EntityInflictDamage( getPlayerEntity(), damage * 2 / damage_dividend, "DAMAGE_PROJECTILE", "Blizzard Necklace damage penalty", "BLOOD_SPRAY", 0, 0)
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "wicked_necklace,frost_artifact",
		excludes_relics_desc	= "It can't be used along with Wicked Necklace relic",
		excludes_relics_tags	= "blizzard_necklace",
	},
	{
		name 					= "blaze_necklace",
		ui_name					= "Blaze Necklace",
		categories				= "projectiles,enemies",
		effect_desc				= "An energy field that burns enemies is projected \naround you, but you take triple projectile damage.",
		effect_desc_short		= "A burning field is projected around you.",
        precursor_types			= "necklace",
		upgrade_effect_desc		= "For each upgrade level, projectile damage is decreased.",
		is_upgradable 			= true,
		onpickup_function		= function()
			LoadGameEffectEntityTo( getPlayerEntity(), getCustomEffect("effect_burning_field").path )
		end,
		ondrop_function			= function()
			local player_children = EntityGetAllChildren(getPlayerEntity())
			if player_children == nil then return end
			for i,child in ipairs(player_children) do
				if(EntityGetName(child) == "burning_field") then
					EntityKill(child)
					break
				end
			end
		end,
		ondamaged_function		= function(damage, message, entity_thats_responsible, is_fatal, projectile_thats_responsible )
			if(isRelicOwned("blaze_necklace") >= 1) then
				if(message == "$damage_projectile") then
					local damage_dividend = getRelicLevel("blaze_necklace")
					if(damage_dividend == 0) then
						damage_dividend = 1
					end
					EntityInflictDamage( getPlayerEntity(), damage * 2 / damage_dividend, "DAMAGE_PROJECTILE", "Blaze Necklace damage penalty", "BLOOD_SPRAY", 0, 0)
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "wicked_necklace,ember_stone",
		excludes_relics_desc	= "It can't be used along with Wicked Necklace relic",
		excludes_relics_tags	= "blaze_necklace",
	},
	{
		name 					= "scorpions_sting",
		ui_name					= "Scorpion's Sting",
		categories				= "projectiles",
		effect_desc				= "Spells you cast have a 25% chance of pulling enemies towards you while \ncausing melee damage to them.",
		effect_desc_short		= "Pulls enemies towards you, causing melee damage.",
        precursor_types			= "gadget",
		upgrade_effect_desc		= "For each upgrade level, effect chance is increased by 5% and damage dealt is increased.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("scorpions_sting") >= 1) then		
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 25 + (5 * getRelicLevel("scorpions_sting"))
				if(chance >= 1 and chance <= odds) then
					addBuffVisualIndicator_Projectile(projectile_entity_id, "blood")
					attachDragCollisionHandlertoProjectile(projectile_entity_id, getCollisionHandler("pull_melee_damage").path, true, 20, "enemy", false)
				end
			end
		end,
		ondrop_function			= function()
			removeInternalVariable(getPlayerEntity(), "scorpions_sting_damage")
		end,
		overtime_function		= function()
			local multiplier = getRelicLevel("scorpions_sting")
			if(multiplier == 0) then
				multiplier = 1
			end
			local damage_threshold = 0.01 * multiplier
			if(not entityHasInternalVariableValue(getPlayerEntity(), "scorpions_sting_damage")) then
				addNewInternalVariable(getPlayerEntity(), "scorpions_sting_damage", "value_float", damage_threshold)
			end
			setInternalVariableValue(getPlayerEntity(), "scorpions_sting_damage", "value_float", damage_threshold)
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "rusty_hook,cursed_mirror",
	},
	{
		name 					= "freedom_ring_acid",
		ui_name					= "Freedom Ring (Acid)",
		categories				= "projectiles",
		effect_desc				= "Your spells have a 30% chance of inducing a sequence of acid explosions \non enemies hit by them.",
		effect_desc_short		= "Your spells induce acid explosion sequences on enemies.",
        precursor_types			= "ring",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("freedom_ring_acid") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 30 + (5 * getRelicLevel("freedom_ring_acid"))
				if(chance >= 1 and chance <= odds) then
					attachLoadEffectOntoEntitiestoProjectile(projectile_entity_id, getCustomEffect("effect_exploding_acid").path, "acid")
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "freedom_ring,acid_gem",
	},
	{
		name 					= "freedom_ring_lava",
		ui_name					= "Freedom Ring (Lava)",
		categories				= "projectiles",
		effect_desc				= "Your spells have a 30% chance of inducing a sequence of lava explosions \non enemies hit by them.",
		effect_desc_short		= "Your spells induce lava explosion sequences on enemies.",
        precursor_types			= "ring",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("freedom_ring_lava") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 30 + (5 * getRelicLevel("freedom_ring_lava"))
				if(chance >= 1 and chance <= odds) then
					attachLoadEffectOntoEntitiestoProjectile(projectile_entity_id, getCustomEffect("effect_exploding_lava").path, "lava")
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "freedom_ring,lava_gem",
	},
	{
		name 					= "freedom_ring_urine",
		ui_name					= "Freedom Ring (Urine)",
		categories				= "projectiles",
		effect_desc				= "Your spells have a 30% chance of inducing a sequence of urine explosions \non enemies hit by them.",
		effect_desc_short		= "Your spells induce urine explosion sequences on enemies.",
        precursor_types			= "ring",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("freedom_ring_urine") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 30 + (5 * getRelicLevel("freedom_ring_urine"))
				if(chance >= 1 and chance <= odds) then
					attachLoadEffectOntoEntitiestoProjectile(projectile_entity_id, getCustomEffect("effect_exploding_urine").path, "urine")
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "freedom_ring,urine_gem",
	},
	{
		name 					= "freedom_ring_water",
		ui_name					= "Freedom Ring (Water)",
		categories				= "projectiles",
		effect_desc				= "Your spells have a 30% chance of inducing a sequence of water explosions \non enemies hit by them.",
		effect_desc_short		= "Your spells induce water explosion sequences on enemies.",
        precursor_types			= "ring",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("freedom_ring_water") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 30 + (5 * getRelicLevel("freedom_ring_water"))
				if(chance >= 1 and chance <= odds) then
					attachLoadEffectOntoEntitiestoProjectile(projectile_entity_id, getCustomEffect("effect_exploding_water").path, "water")
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "freedom_ring,water_gem",
	},
	{
		name 					= "freedom_ring_blood",
		ui_name					= "Freedom Ring (Blood)",
		categories				= "projectiles",
		effect_desc				= "Your spells have a 30% chance of inducing a sequence of blood explosions \non enemies hit by them.",
		effect_desc_short		= "Your spells induce blood explosion sequences on enemies.",
        precursor_types			= "ring",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("freedom_ring_blood") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 30 + (5 * getRelicLevel("freedom_ring_blood"))
				if(chance >= 1 and chance <= odds) then
					attachLoadEffectOntoEntitiestoProjectile(projectile_entity_id, getCustomEffect("effect_exploding_blood").path, "blood")
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "freedom_ring,blood_gem",
	},
	{
		name 					= "freedom_ring_poison",
		ui_name					= "Freedom Ring (Poison)",
		categories				= "projectiles",
		effect_desc				= "Your spells have a 30% chance of inducing a sequence of poison explosions \non enemies hit by them.",
		effect_desc_short		= "Your spells induce poison explosion sequences on enemies.",
        precursor_types			= "ring",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("freedom_ring_poison") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 30 + (5 * getRelicLevel("freedom_ring_poison"))
				if(chance >= 1 and chance <= odds) then
					attachLoadEffectOntoEntitiestoProjectile(projectile_entity_id, getCustomEffect("effect_exploding_poison").path, "poison")
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "freedom_ring,poison_gem",
	},
	{
		name 					= "freedom_ring_fire",
		ui_name					= "Freedom Ring (Fire)",
		categories				= "projectiles",
		effect_desc				= "Your spells have a 30% chance of inducing a sequence of fire explosions \non enemies hit by them.",
		effect_desc_short		= "Your spells induce fire explosion sequences on enemies.",
        precursor_types			= "ring",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("freedom_ring_fire") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 30 + (5 * getRelicLevel("freedom_ring_fire"))
				if(chance >= 1 and chance <= odds) then
					attachLoadEffectOntoEntitiestoProjectile(projectile_entity_id, getCustomEffect("effect_exploding_fire").path, "fire")
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "freedom_ring,sacred_ember",
	},
	{
		name 					= "freedom_ring_radioactive_liquid",
		ui_name					= "Freedom Ring (Toxic Sludge)",
		categories				= "projectiles",
		effect_desc				= "Your spells have a 30% chance of inducing a sequence of toxic sludge explosions \non enemies hit by them.",
		effect_desc_short		= "Your spells induce toxic sludge explosion sequences on enemies.",
        precursor_types			= "ring",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("freedom_ring_radioactive_liquid") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 30 + (5 * getRelicLevel("freedom_ring_radioactive_liquid"))
				if(chance >= 1 and chance <= odds) then
					attachLoadEffectOntoEntitiestoProjectile(projectile_entity_id, getCustomEffect("effect_exploding_radioactive_liquid").path, "radioactive_liquid_fading")
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "freedom_ring,toxic_gem",
	},
	{
		name 					= "freedom_ring_oil",
		ui_name					= "Freedom Ring (Oil)",
		categories				= "projectiles",
		effect_desc				= "Your spells have a 30% chance of inducing a sequence of oil explosions \non enemies hit by them.",
		effect_desc_short		= "Your spells induce oil explosion sequences on enemies.",
        precursor_types			= "ring",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("freedom_ring_oil") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 30 + (5 * getRelicLevel("freedom_ring_oil"))
				if(chance >= 1 and chance <= odds) then
					attachLoadEffectOntoEntitiestoProjectile(projectile_entity_id, getCustomEffect("effect_exploding_oil").path, "oil")
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "freedom_ring,oil_gem",
	},
	{
		name 					= "meteor_fragment_molten",
		ui_name					= "Molten Meteor Fragment",
		categories				= "projectiles",
		effect_desc				= "Spells you cast in a downward angle have a 30% chance of creating \na fire shockwave, that damages nearby enemies, when they hit something. \nIt also synergizes with Earth Core Fragment relic effect.",
		effect_desc_short		= "Spells cast downwards create fire shockwaves on the ground.",
        precursor_types			= "magic_artifact",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5% and shockwave reaches longer.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("meteor_fragment_molten") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 30 + (5 * getRelicLevel("meteor_fragment_molten"))
				if(chance >= 1 and chance <= odds) then
					local projectile_vel_x, projectile_vel_y = getProjectileVelocitiesandGravity(projectile_entity_id)
					if(projectile_vel_y > 0) then
						replaceProjectileExplosionSprite(projectile_entity_id, "")
						projectileRemoveBouncing(projectile_entity_id)
						if(not projectileExplodesonImpact(projectile_entity_id)) then
							projectileSetExplosionConfig(projectile_entity_id, 1)
						end
						attachLoadEntityOnExplosiontoProjectile(projectile_entity_id, getLoadOnExplosion("shockwave_fire").path, "spark")
					end
				end
			end
		end,
		ongroundpound_function	= function( entity_id)
			if(isRelicOwned("meteor_fragment_molten") >= 1) then
				local game_effect_comp = GetGameEffectLoadTo( entity_id, "PROTECTION_ALL", true )
				if game_effect_comp ~= nil then
					ComponentSetValue( game_effect_comp, "frames", "40" )
				end
				local x, y = EntityGetTransform(entity_id)
				EntityLoad("mods/ArcaneReliquary/files/entity_definitions/load_on_explosion/shockwave_fire.xml", x, y)
			end
		end,
		ondrop_function			= function()
			removeInternalVariable(getPlayerEntity(), "fire_shockwave_length_bonus")
		end,
		overtime_function		= function()
			local fire_shockwave_length_bonus = getRelicLevel("meteor_fragment_molten")

			if(not entityHasInternalVariableValue(getPlayerEntity(), "fire_shockwave_length_bonus")) then
				addNewInternalVariable(getPlayerEntity(), "fire_shockwave_length_bonus", "value_int", fire_shockwave_length_bonus)
			end
			setInternalVariableValue(getPlayerEntity(), "fire_shockwave_length_bonus", "value_int", fire_shockwave_length_bonus)
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "meteor_fragment,blaze_pearl",
	},
	{
		name 					= "meteor_fragment_frozen",
		ui_name					= "Frozen Meteor Fragment",
		categories				= "projectiles",
		effect_desc				= "Spells you cast in a downward angle have a 30% chance of creating \nan ice shockwave, that damages nearby enemies, when they hit something. \nIt also synergizes with Earth Core Fragment relic effect.",
		effect_desc_short		= "Spells cast downwards create ice shockwaves on the ground.",
        precursor_types			= "magic_artifact",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5% and shockwave reaches longer.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("meteor_fragment_frozen") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 30 + (5 * getRelicLevel("meteor_fragment_frozen"))
				if(chance >= 1 and chance <= odds) then
					local projectile_vel_x, projectile_vel_y = getProjectileVelocitiesandGravity(projectile_entity_id)
					if(projectile_vel_y > 0) then
						replaceProjectileExplosionSprite(projectile_entity_id, "")
						projectileRemoveBouncing(projectile_entity_id)
						if(not projectileExplodesonImpact(projectile_entity_id)) then
							projectileSetExplosionConfig(projectile_entity_id, 1)
						end
						attachLoadEntityOnExplosiontoProjectile(projectile_entity_id, getLoadOnExplosion("shockwave_ice").path, "spark")
					end
				end
			end
		end,
		ongroundpound_function	= function( entity_id)
			if(isRelicOwned("meteor_fragment_frozen") >= 1) then
				local game_effect_comp = GetGameEffectLoadTo( entity_id, "PROTECTION_ALL", true )
				if game_effect_comp ~= nil then
					ComponentSetValue( game_effect_comp, "frames", "40" )
				end
				local x, y = EntityGetTransform(entity_id)
				EntityLoad("mods/ArcaneReliquary/files/entity_definitions/load_on_explosion/shockwave_ice.xml", x, y)
			end
		end,
		ondrop_function			= function()
			removeInternalVariable(getPlayerEntity(), "ice_shockwave_length_bonus")
		end,
		overtime_function		= function()
			local ice_shockwave_length_bonus = getRelicLevel("meteor_fragment_frozen")

			if(not entityHasInternalVariableValue(getPlayerEntity(), "ice_shockwave_length_bonus")) then
				addNewInternalVariable(getPlayerEntity(), "ice_shockwave_length_bonus", "value_int", ice_shockwave_length_bonus)
			end
			setInternalVariableValue(getPlayerEntity(), "ice_shockwave_length_bonus", "value_int", ice_shockwave_length_bonus)
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "meteor_fragment,ice_artifact",
	},
	{
		name 					= "meteor_fragment_aluminum",
		ui_name					= "Aluminum Meteor Fragment",
		categories				= "projectiles",
		effect_desc				= "Spells you cast in a downward angle have a 30% chance of creating \nan electric shockwave, that damages nearby enemies, when they hit something. \nIt also synergizes with Earth Core Fragment relic effect.",
		effect_desc_short		= "Spells cast downwards create electric shockwaves on the ground.",
        precursor_types			= "magic_artifact",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5% and shockwave reaches longer.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("meteor_fragment_aluminum") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 30 + (5 * getRelicLevel("meteor_fragment_aluminum"))
				if(chance >= 1 and chance <= odds) then
					local projectile_vel_x, projectile_vel_y = getProjectileVelocitiesandGravity(projectile_entity_id)
					if(projectile_vel_y > 0) then
						replaceProjectileExplosionSprite(projectile_entity_id, "")
						projectileRemoveBouncing(projectile_entity_id)
						if(not projectileExplodesonImpact(projectile_entity_id)) then
							projectileSetExplosionConfig(projectile_entity_id, 1)
						end
						attachLoadEntityOnExplosiontoProjectile(projectile_entity_id, getLoadOnExplosion("shockwave_electric").path, "spark")
					end
				end
			end
		end,
		ongroundpound_function	= function( entity_id)
			if(isRelicOwned("meteor_fragment_aluminum") >= 1) then
				local game_effect_comp = GetGameEffectLoadTo( entity_id, "PROTECTION_ALL", true )
				if game_effect_comp ~= nil then
					ComponentSetValue( game_effect_comp, "frames", "40" )
				end
				local x, y = EntityGetTransform(entity_id)
				EntityLoad("mods/ArcaneReliquary/files/entity_definitions/load_on_explosion/shockwave_electric.xml", x, y)
			end
		end,
		ondrop_function			= function()
			removeInternalVariable(getPlayerEntity(), "electric_shockwave_length_bonus")
		end,
		overtime_function		= function()
			local electric_shockwave_length_bonus = getRelicLevel("meteor_fragment_aluminum")

			if(not entityHasInternalVariableValue(getPlayerEntity(), "electric_shockwave_length_bonus")) then
				addNewInternalVariable(getPlayerEntity(), "electric_shockwave_length_bonus", "value_int", electric_shockwave_length_bonus)
			end
			setInternalVariableValue(getPlayerEntity(), "electric_shockwave_length_bonus", "value_int", electric_shockwave_length_bonus)
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "meteor_fragment,alkaline_battery",
	},
	{
		name 					= "meteor_fragment_silver",
		ui_name					= "Silver Meteor Fragment",
		categories				= "projectiles",
		effect_desc				= "Spells you cast in a downward angle have a 30% chance of creating \na thunderbolt shockwave, that damages nearby enemies, when they hit something. \nIt also synergizes with Earth Core Fragment relic effect.",
		effect_desc_short		= "Spells cast downwards create thunderbolt shockwaves on the ground.",
        precursor_types			= "magic_artifact",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5% and shockwave reaches longer.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("meteor_fragment_silver") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 30 + (5 * getRelicLevel("meteor_fragment_silver"))
				if(chance >= 1 and chance <= odds) then
					local projectile_vel_x, projectile_vel_y = getProjectileVelocitiesandGravity(projectile_entity_id)
					if(projectile_vel_y > 0) then
						replaceProjectileExplosionSprite(projectile_entity_id, "")
						projectileRemoveBouncing(projectile_entity_id)
						if(not projectileExplodesonImpact(projectile_entity_id)) then
							projectileSetExplosionConfig(projectile_entity_id, 1)
						end
						attachLoadEntityOnExplosiontoProjectile(projectile_entity_id, getLoadOnExplosion("shockwave_thunderbolts").path, "spark")
					end
				end
			end
		end,
		ongroundpound_function	= function( entity_id)
			if(isRelicOwned("meteor_fragment_silver") >= 1) then
				local game_effect_comp = GetGameEffectLoadTo( entity_id, "PROTECTION_ALL", true )
				if game_effect_comp ~= nil then
					ComponentSetValue( game_effect_comp, "frames", "40" )
				end
				local x, y = EntityGetTransform(entity_id)
				EntityLoad("mods/ArcaneReliquary/files/entity_definitions/load_on_explosion/shockwave_thunderbolts.xml", x, y)
			end
		end,
		ondrop_function			= function()
			removeInternalVariable(getPlayerEntity(), "thunderbolt_shockwave_length_bonus")
		end,
		overtime_function		= function()
			local thunderbolt_shockwave_length_bonus = getRelicLevel("meteor_fragment_silver")

			if(not entityHasInternalVariableValue(getPlayerEntity(), "thunderbolt_shockwave_length_bonus")) then
				addNewInternalVariable(getPlayerEntity(), "thunderbolt_shockwave_length_bonus", "value_int", thunderbolt_shockwave_length_bonus)
			end
			setInternalVariableValue(getPlayerEntity(), "thunderbolt_shockwave_length_bonus", "value_int", thunderbolt_shockwave_length_bonus)
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "meteor_fragment,tesla_coil",
	},
	{
		name 					= "balloon",
		ui_name					= "Balloon",
		categories				= "player",
		effect_desc				= "Scoring kills while midair replenishes jetpack energy.",
		effect_desc_short		= "Scoring kills while midair replenishes jetpack energy.",
        precursor_types			= "magic_artifact,gadget",
		upgrade_effect_desc		= "For each upgrade level, jetpack energy gained is increased.",
		is_upgradable 			= true,
		onenemykill_function	= function(damage_type_bit_field, damage_message, entity_thats_responsible, drop_items)
			if(isRelicOwned("balloon") >= 1) then
				if(not isPlayerStanding()) then
					local fly_energy_threshold = 0.3 + (0.1 * getRelicLevel("balloon"))
					addFlyEnergy(fly_energy_threshold)
				end
			end
		end,
		excludes_relics			= "balloon_golden,balloon_diamond",
		excludes_relics_desc	= "It can't be used along with other Balloon relic variations",
		excludes_relics_tags	= "balloon_relic",
	},
	{
		name 					= "balloon_golden",
		ui_name					= "Golden Balloon",
		categories				= "player",
		effect_desc				= "Scoring kills while midair entirely refills your jetpack energy.",
		effect_desc_short		= "Scoring kills while midair entirely refills your jetpack energy.",
        precursor_types			= "magic_artifact,gadget",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onenemykill_function	= function(damage_type_bit_field, damage_message, entity_thats_responsible, drop_items)
			if(isRelicOwned("balloon_golden") >= 1) then
				if(not isPlayerStanding()) then
					addFlyEnergy(99)
				end
			end
		end,
		excludes_relics			= "balloon,balloon_diamond",
		excludes_relics_desc	= "It can't be used along with other Balloon relic variations",
		excludes_relics_tags	= "balloon_relic",
		combine_exclusive		= true,
		combine_requires_relics	= "balloon,golden_heart",
	},
	{
		name 					= "balloon_diamond",
		ui_name					= "Diamond Balloon",
		categories				= "player",
		effect_desc				= "Damaging enemies with your spells while midair has a 50% chance of entirely refilling your jetpack energy.",
		effect_desc_short		= "Damaging enemies while midair has a chance of refilling your jetpack energy.",
        precursor_types			= "magic_artifact,gadget",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onenemydamaged_function	= function(damage, message, entity_thats_responsible, is_fatal, projectile_thats_responsible )
			if(isRelicOwned("balloon_diamond") >= 1) then
				if(not isPlayerStanding()) then
					SetRandomSeed( GameGetFrameNum(), math.random(1,50))
					local chance = Random(1,100)
					local odds = 50
					if(chance >= 1 and chance <= odds) then
						addFlyEnergy(99)
					end
				end
			end
		end,
		excludes_relics			= "balloon,balloon_golden",
		excludes_relics_desc	= "It can't be used along with other Balloon relic variations",
		excludes_relics_tags	= "balloon_relic",
		combine_exclusive		= true,
		combine_requires_relics	= "balloon_golden,duality_necklace",
	},
	{
		name 					= "kyber_crystal_yellow",
		ui_name					= "Yellow Kyber Crystal",
		categories				= "projectiles",
		effect_desc				= "Most spells you cast have a 50% chance of shooting explosive yellow laser bolts at random directions once cast.",
		effect_desc_short		= "Your spells shoot explosive laser bolts.",
        precursor_types			= "effect_crystal",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5% and when the effect triggers, laser bolts are shot more frequently.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("kyber_crystal_yellow") >= 1) then
				if((not isPhysicsProjectile(projectile_entity_id)) and (not isProjectileHittable(projectile_entity_id))) then
					SetRandomSeed( GameGetFrameNum(), math.random(1,50))
					local chance = Random(1,100)
					local odds = 50 + (5 * getRelicLevel("kyber_crystal_yellow"))
					if(chance >= 1 and chance <= odds) then
						addNewInternalVariable(projectile_entity_id, "projectile_definition", "value_string", "mods/ArcaneReliquary/files/entity_definitions/custom_projectiles/yellow_laser.xml")
						addNewInternalVariable(projectile_entity_id, "projectile_velocity", "value_int", 650)
						addNewInternalVariable(projectile_entity_id, "max_projectiles_shot", "value_int", 25)
						addNewLuaCallback(projectile_entity_id, "overtime", "mods/ArcaneReliquary/files/scripts/projectiles/launch_projectile_random_direction.lua", 25 - (2 * getRelicLevel("kyber_crystal_yellow")), false, "launch_projectile_random_direction")
					end
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "kyber_crystal,explosive_mana_crystal",
	},
	{
		name 					= "kyber_crystal_orange",
		ui_name					= "Orange Kyber Crystal",
		categories				= "projectiles",
		effect_desc				= "Most spells you cast have a 50% chance of shooting incendiary orange laser bolts at random directions once cast.",
		effect_desc_short		= "Your spells shoot incendiary laser bolts.",
        precursor_types			= "effect_crystal",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5% and when the effect triggers, laser bolts are shot more frequently.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("kyber_crystal_orange") >= 1) then
				if((not isPhysicsProjectile(projectile_entity_id)) and (not isProjectileHittable(projectile_entity_id))) then
					SetRandomSeed( GameGetFrameNum(), math.random(1,50))
					local chance = Random(1,100)
					local odds = 50 + (5 * getRelicLevel("kyber_crystal_orange"))
					if(chance >= 1 and chance <= odds) then
						addNewInternalVariable(projectile_entity_id, "projectile_definition", "value_string", "mods/ArcaneReliquary/files/entity_definitions/custom_projectiles/orange_laser.xml")
						addNewInternalVariable(projectile_entity_id, "projectile_velocity", "value_int", 650)
						addNewInternalVariable(projectile_entity_id, "max_projectiles_shot", "value_int", 25)
						addNewLuaCallback(projectile_entity_id, "overtime", "mods/ArcaneReliquary/files/scripts/projectiles/launch_projectile_random_direction.lua", 25 - (2 * getRelicLevel("kyber_crystal_orange")), false, "launch_projectile_random_direction")
					end
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "kyber_crystal,blaze_pearl",
	},
	{
		name 					= "kyber_crystal_blue",
		ui_name					= "Blue Kyber Crystal",
		categories				= "projectiles",
		effect_desc				= "Most spells you cast have a 50% chance of shooting electrocuting blue laser bolts at random directions once cast.",
		effect_desc_short		= "Your spells shoot electrocuting laser bolts.",
        precursor_types			= "effect_crystal",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5% and when the effect triggers, laser bolts are shot more frequently.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("kyber_crystal_blue") >= 1) then
				if((not isPhysicsProjectile(projectile_entity_id)) and (not isProjectileHittable(projectile_entity_id))) then
					SetRandomSeed( GameGetFrameNum(), math.random(1,50))
					local chance = Random(1,100)
					local odds = 50 + (5 * getRelicLevel("kyber_crystal_blue"))
					if(chance >= 1 and chance <= odds) then
						addNewInternalVariable(projectile_entity_id, "projectile_definition", "value_string", "mods/ArcaneReliquary/files/entity_definitions/custom_projectiles/blue_laser.xml")
						addNewInternalVariable(projectile_entity_id, "projectile_velocity", "value_int", 650)
						addNewInternalVariable(projectile_entity_id, "max_projectiles_shot", "value_int", 25)
						addNewLuaCallback(projectile_entity_id, "overtime", "mods/ArcaneReliquary/files/scripts/projectiles/launch_projectile_random_direction.lua", 25 - (2 * getRelicLevel("kyber_crystal_blue")), false, "launch_projectile_random_direction")
					end
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "kyber_crystal,alkaline_battery",
	},
	{
		name 					= "kyber_crystal_teal",
		ui_name					= "Blue Kyber Crystal",
		categories				= "projectiles",
		effect_desc				= "Most spells you cast have a 50% chance of shooting freezing teal laser bolts at random directions once cast.",
		effect_desc_short		= "Your spells shoot freezing laser bolts.",
        precursor_types			= "effect_crystal",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5% and when the effect triggers, laser bolts are shot more frequently.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("kyber_crystal_teal") >= 1) then
				if((not isPhysicsProjectile(projectile_entity_id)) and (not isProjectileHittable(projectile_entity_id))) then
					SetRandomSeed( GameGetFrameNum(), math.random(1,50))
					local chance = Random(1,100)
					local odds = 50 + (5 * getRelicLevel("kyber_crystal_teal"))
					if(chance >= 1 and chance <= odds) then
						addNewInternalVariable(projectile_entity_id, "projectile_definition", "value_string", "mods/ArcaneReliquary/files/entity_definitions/custom_projectiles/teal_laser.xml")
						addNewInternalVariable(projectile_entity_id, "projectile_velocity", "value_int", 650)
						addNewInternalVariable(projectile_entity_id, "max_projectiles_shot", "value_int", 25)
						addNewLuaCallback(projectile_entity_id, "overtime", "mods/ArcaneReliquary/files/scripts/projectiles/launch_projectile_random_direction.lua", 25 - (2 * getRelicLevel("kyber_crystal_teal")), false, "launch_projectile_random_direction")
					end
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "kyber_crystal,ice_artifact",
	},
	{
		name 					= "kyber_crystal_green",
		ui_name					= "Green Kyber Crystal",
		categories				= "projectiles",
		effect_desc				= "Most spells you cast have a 50% chance of shooting corroding green laser bolts at random directions once cast.",
		effect_desc_short		= "Your spells shoot corroding laser bolts.",
        precursor_types			= "effect_crystal",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5% and when the effect triggers, laser bolts are shot more frequently.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("kyber_crystal_green") >= 1) then
				if((not isPhysicsProjectile(projectile_entity_id)) and (not isProjectileHittable(projectile_entity_id))) then
					SetRandomSeed( GameGetFrameNum(), math.random(1,50))
					local chance = Random(1,100)
					local odds = 50 + (5 * getRelicLevel("kyber_crystal_green"))
					if(chance >= 1 and chance <= odds) then
						addNewInternalVariable(projectile_entity_id, "projectile_definition", "value_string", "mods/ArcaneReliquary/files/entity_definitions/custom_projectiles/green_laser.xml")
						addNewInternalVariable(projectile_entity_id, "projectile_velocity", "value_int", 650)
						addNewInternalVariable(projectile_entity_id, "max_projectiles_shot", "value_int", 25)
						addNewLuaCallback(projectile_entity_id, "overtime", "mods/ArcaneReliquary/files/scripts/projectiles/launch_projectile_random_direction.lua", 25 - (2 * getRelicLevel("kyber_crystal_green")), false, "launch_projectile_random_direction")
					end
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "kyber_crystal,acid_gem",
	},
	{
		name 					= "kyber_crystal_purple",
		ui_name					= "Purple Kyber Crystal",
		categories				= "projectiles",
		effect_desc				= "Most spells you cast have a 50% chance of shooting mutating purple laser bolts at random directions once cast.",
		effect_desc_short		= "Your spells shoot mutating laser bolts.",
        precursor_types			= "effect_crystal",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5% and when the effect triggers, laser bolts are shot more frequently.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("kyber_crystal_purple") >= 1) then
				if((not isPhysicsProjectile(projectile_entity_id)) and (not isProjectileHittable(projectile_entity_id))) then
					SetRandomSeed( GameGetFrameNum(), math.random(1,50))
					local chance = Random(1,100)
					local odds = 50 + (5 * getRelicLevel("kyber_crystal_purple"))
					if(chance >= 1 and chance <= odds) then
						addNewInternalVariable(projectile_entity_id, "projectile_definition", "value_string", "mods/ArcaneReliquary/files/entity_definitions/custom_projectiles/purple_laser.xml")
						addNewInternalVariable(projectile_entity_id, "projectile_velocity", "value_int", 650)
						addNewInternalVariable(projectile_entity_id, "max_projectiles_shot", "value_int", 25)
						addNewLuaCallback(projectile_entity_id, "overtime", "mods/ArcaneReliquary/files/scripts/projectiles/launch_projectile_random_direction.lua", 25 - (2 * getRelicLevel("kyber_crystal_purple")), false, "launch_projectile_random_direction")
					end
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "kyber_crystal,poly_gem",
	},
	{
		name 					= "syringe_acid",
		ui_name					= "Syringe Filled with Acid",
		categories				= "projectiles",
		effect_desc				= "Provided they travel enough time, your spells have a 30% chance of injecting acid wherever it hits.",
		effect_desc_short		= "Your spells inject acid on hit.",
        precursor_types			= "gadget",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("syringe_acid") >= 1) then
				if(not isPhysicsProjectile(projectile_entity_id)) then
					SetRandomSeed( GameGetFrameNum(), math.random(1,50))
					local chance = Random(1,100)
					local odds = 30 + (5 * getRelicLevel("syringe_acid"))
					if(chance >= 1 and chance <= odds) then
						projectileRemoveBouncing(projectile_entity_id)
						
						addNewInternalVariable(projectile_entity_id, "projectile_pos_x", "value_int", 0)
						addNewInternalVariable(projectile_entity_id, "projectile_pos_y", "value_int", 0)
						addNewInternalVariable(projectile_entity_id, "projectile_vel_x", "value_int", 0)
						addNewInternalVariable(projectile_entity_id, "projectile_vel_y", "value_int", 0)
						
						addBuffVisualIndicator_Projectile(projectile_entity_id, "acid")
						addNewInternalVariable(projectile_entity_id, "projectile_definition", "value_string", "mods/ArcaneReliquary/files/entity_definitions/custom_projectiles/invisible_shrapnel_projectile_acid.xml")
							
						addNewLuaCallback(projectile_entity_id, "overtime", "mods/ArcaneReliquary/files/scripts/projectiles/store_projectile_vel_pos.lua", 1, false, "store_projectile_vel_pos")
						addNewLuaCallback_onRemoved(projectile_entity_id, "mods/ArcaneReliquary/files/scripts/projectiles/launch_directional_shrapnel_projectile.lua")
					end
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "syringe,acid_gem",
	},
	{
		name 					= "syringe_poison",
		ui_name					= "Syringe Filled with Poison",
		categories				= "projectiles",
		effect_desc				= "Provided they travel enough time, your spells have a 30% chance of injecting poison wherever it hits.",
		effect_desc_short		= "Your spells inject poison on hit.",
        precursor_types			= "gadget",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("syringe_poison") >= 1) then
				if(not isPhysicsProjectile(projectile_entity_id)) then
					SetRandomSeed( GameGetFrameNum(), math.random(1,50))
					local chance = Random(1,100)
					local odds = 30 + (5 * getRelicLevel("syringe_poison"))
					if(chance >= 1 and chance <= odds) then
						projectileRemoveBouncing(projectile_entity_id)
						
						addNewInternalVariable(projectile_entity_id, "projectile_pos_x", "value_int", 0)
						addNewInternalVariable(projectile_entity_id, "projectile_pos_y", "value_int", 0)
						addNewInternalVariable(projectile_entity_id, "projectile_vel_x", "value_int", 0)
						addNewInternalVariable(projectile_entity_id, "projectile_vel_y", "value_int", 0)
						
						addBuffVisualIndicator_Projectile(projectile_entity_id, "poison")
						addNewInternalVariable(projectile_entity_id, "projectile_definition", "value_string", "mods/ArcaneReliquary/files/entity_definitions/custom_projectiles/invisible_shrapnel_projectile_poison.xml")
							
						addNewLuaCallback(projectile_entity_id, "overtime", "mods/ArcaneReliquary/files/scripts/projectiles/store_projectile_vel_pos.lua", 1, false, "store_projectile_vel_pos")
						addNewLuaCallback_onRemoved(projectile_entity_id, "mods/ArcaneReliquary/files/scripts/projectiles/launch_directional_shrapnel_projectile.lua")
					end
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "syringe,poison_gem",
	},
	{
		name 					= "syringe_lava",
		ui_name					= "Syringe Filled with Lava",
		categories				= "projectiles",
		effect_desc				= "Provided they travel enough time, your spells have a 30% chance of injecting lava wherever it hits.",
		effect_desc_short		= "Your spells inject lava on hit.",
        precursor_types			= "gadget",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("syringe_lava") >= 1) then
				if(not isPhysicsProjectile(projectile_entity_id)) then
					SetRandomSeed( GameGetFrameNum(), math.random(1,50))
					local chance = Random(1,100)
					local odds = 30 + (5 * getRelicLevel("syringe_lava"))
					if(chance >= 1 and chance <= odds) then
						projectileRemoveBouncing(projectile_entity_id)
						
						addNewInternalVariable(projectile_entity_id, "projectile_pos_x", "value_int", 0)
						addNewInternalVariable(projectile_entity_id, "projectile_pos_y", "value_int", 0)
						addNewInternalVariable(projectile_entity_id, "projectile_vel_x", "value_int", 0)
						addNewInternalVariable(projectile_entity_id, "projectile_vel_y", "value_int", 0)
						
						addBuffVisualIndicator_Projectile(projectile_entity_id, "lava")
						addNewInternalVariable(projectile_entity_id, "projectile_definition", "value_string", "mods/ArcaneReliquary/files/entity_definitions/custom_projectiles/invisible_shrapnel_projectile_lava.xml")
							
						addNewLuaCallback(projectile_entity_id, "overtime", "mods/ArcaneReliquary/files/scripts/projectiles/store_projectile_vel_pos.lua", 1, false, "store_projectile_vel_pos")
						addNewLuaCallback_onRemoved(projectile_entity_id, "mods/ArcaneReliquary/files/scripts/projectiles/launch_directional_shrapnel_projectile.lua")
					end
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "syringe,lava_gem",
	},
	{
		name 					= "syringe_poly",
		ui_name					= "Syringe Filled with Polymorphine",
		categories				= "projectiles",
		effect_desc				= "Provided they travel enough time, your spells have a 30% chance of injecting polymorphine wherever it hits.",
		effect_desc_short		= "Your spells inject polymorphine on hit.",
        precursor_types			= "gadget",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("syringe_poly") >= 1) then
				if(not isPhysicsProjectile(projectile_entity_id)) then
					SetRandomSeed( GameGetFrameNum(), math.random(1,50))
					local chance = Random(1,100)
					local odds = 30 + (5 * getRelicLevel("syringe_poly"))
					if(chance >= 1 and chance <= odds) then
						projectileRemoveBouncing(projectile_entity_id)
						
						addNewInternalVariable(projectile_entity_id, "projectile_pos_x", "value_int", 0)
						addNewInternalVariable(projectile_entity_id, "projectile_pos_y", "value_int", 0)
						addNewInternalVariable(projectile_entity_id, "projectile_vel_x", "value_int", 0)
						addNewInternalVariable(projectile_entity_id, "projectile_vel_y", "value_int", 0)
						
						addBuffVisualIndicator_Projectile(projectile_entity_id, "magic_liquid_polymorph")
						addNewInternalVariable(projectile_entity_id, "projectile_definition", "value_string", "mods/ArcaneReliquary/files/entity_definitions/custom_projectiles/invisible_shrapnel_projectile_poly.xml")
							
						addNewLuaCallback(projectile_entity_id, "overtime", "mods/ArcaneReliquary/files/scripts/projectiles/store_projectile_vel_pos.lua", 1, false, "store_projectile_vel_pos")
						addNewLuaCallback_onRemoved(projectile_entity_id, "mods/ArcaneReliquary/files/scripts/projectiles/launch_directional_shrapnel_projectile.lua")
					end
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "syringe,poly_gem",
	},
	{
		name 					= "acid_grenade",
		ui_name					= "Acid Grenade",
		categories				= "projectiles",
		effect_desc				= "Your spells have a 25% chance of exploding into a barrage of acid shrapnel that penetrates walls.",
		effect_desc_short		= "Your spells release acid shrapnel.",
        precursor_types			= "gadget",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("acid_grenade") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 25 + (5 * getRelicLevel("acid_grenade"))
				if(chance >= 1 and chance <= odds) then
					addBuffVisualIndicator_Projectile(projectile_entity_id, "acid")
					attachLoadEntityOnExplosiontoProjectile(projectile_entity_id, getLoadOnExplosion("launch_circular_projectiles_invisible_shrapnel_acid").path)
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "fragmentation_grenade,acid_gem",
	},
	{
		name 					= "urine_grenade",
		ui_name					= "Urine Grenade",
		categories				= "projectiles",
		effect_desc				= "Your spells have a 25% chance of exploding into a barrage of urine shrapnel that penetrates walls.",
		effect_desc_short		= "Your spells release urine shrapnel.",
        precursor_types			= "gadget",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("urine_grenade") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 25 + (5 * getRelicLevel("urine_grenade"))
				if(chance >= 1 and chance <= odds) then
					addBuffVisualIndicator_Projectile(projectile_entity_id, "urine")
					attachLoadEntityOnExplosiontoProjectile(projectile_entity_id, getLoadOnExplosion("launch_circular_projectiles_invisible_shrapnel_urine").path)
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "fragmentation_grenade,urine_gem",
	},
	{
		name 					= "poly_grenade",
		ui_name					= "Poly Grenade",
		categories				= "projectiles",
		effect_desc				= "Your spells have a 25% chance of exploding into a barrage of polymorphine shrapnel that penetrates walls.",
		effect_desc_short		= "Your spells release polymorphine shrapnel.",
        precursor_types			= "gadget",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("poly_grenade") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 25 + (5 * getRelicLevel("poly_grenade"))
				if(chance >= 1 and chance <= odds) then
					addBuffVisualIndicator_Projectile(projectile_entity_id, "magic_liquid_polymorph")
					attachLoadEntityOnExplosiontoProjectile(projectile_entity_id, getLoadOnExplosion("launch_circular_projectiles_invisible_shrapnel_poly").path)
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "fragmentation_grenade,poly_gem",
	},
	{
		name 					= "water_grenade",
		ui_name					= "Water Grenade",
		categories				= "projectiles",
		effect_desc				= "Your spells have a 25% chance of exploding into a barrage of water shrapnel that penetrates walls.",
		effect_desc_short		= "Your spells release water shrapnel.",
        precursor_types			= "gadget",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("water_grenade") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 25 + (5 * getRelicLevel("water_grenade"))
				if(chance >= 1 and chance <= odds) then
					addBuffVisualIndicator_Projectile(projectile_entity_id, "water")
					attachLoadEntityOnExplosiontoProjectile(projectile_entity_id, getLoadOnExplosion("launch_circular_projectiles_invisible_shrapnel_water").path)
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "fragmentation_grenade,water_gem",
	},
	{
		name 					= "lava_grenade",
		ui_name					= "Lava Grenade",
		categories				= "projectiles",
		effect_desc				= "Your spells have a 25% chance of exploding into a barrage of lava shrapnel that penetrates walls.",
		effect_desc_short		= "Your spells release lava shrapnel.",
        precursor_types			= "gadget",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("lava_grenade") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 25 + (5 * getRelicLevel("lava_grenade"))
				if(chance >= 1 and chance <= odds) then
					addBuffVisualIndicator_Projectile(projectile_entity_id, "lava")
					attachLoadEntityOnExplosiontoProjectile(projectile_entity_id, getLoadOnExplosion("launch_circular_projectiles_invisible_shrapnel_lava").path)
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "fragmentation_grenade,lava_gem",
	},
	{
		name 					= "poison_grenade",
		ui_name					= "Poison Grenade",
		categories				= "projectiles",
		effect_desc				= "Your spells have a 25% chance of exploding into a barrage of poison shrapnel that penetrates walls.",
		effect_desc_short		= "Your spells release poison shrapnel.",
        precursor_types			= "gadget",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("poison_grenade") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 25 + (5 * getRelicLevel("poison_grenade"))
				if(chance >= 1 and chance <= odds) then
					addBuffVisualIndicator_Projectile(projectile_entity_id, "poison")
					attachLoadEntityOnExplosiontoProjectile(projectile_entity_id, getLoadOnExplosion("launch_circular_projectiles_invisible_shrapnel_poison").path)
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "fragmentation_grenade,poison_gem",
	},
	{
		name 					= "universal_immunity_stone",
		ui_name					= "Universal Immunity Stone",
		categories				= "health,materials",
		effect_desc				= "You are now immune to direct damage from lava, acid, poison, toxic gas and freezing gas.",
		effect_desc_short		= "You are now immune to direct damage from most damaging materials.",
        precursor_types			= "magic_artifact,effect_stone",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			EntitySetDamageFromMaterial( getPlayerEntity(), "acid", 0 )
			EntitySetDamageFromMaterial( getPlayerEntity(), "lava", 0)
			EntitySetDamageFromMaterial( getPlayerEntity(), "poison", 0)
			EntitySetDamageFromMaterial( getPlayerEntity(), "radioactive_gas", 0)
			EntitySetDamageFromMaterial( getPlayerEntity(), "blood_cold_vapour", 0)
			EntitySetDamageFromMaterial( getPlayerEntity(), "blood_cold", 0)
		end,
		ondrop_function			= function()
			EntitySetDamageFromMaterial( getPlayerEntity(), "acid", 0.005 )
			EntitySetDamageFromMaterial( getPlayerEntity(), "lava", 0.003 )
			EntitySetDamageFromMaterial( getPlayerEntity(), "poison", 0.001 )
			EntitySetDamageFromMaterial( getPlayerEntity(), "radioactive_gas", 0.001 )
			EntitySetDamageFromMaterial( getPlayerEntity(), "blood_cold_vapour", 0.0006 )
			EntitySetDamageFromMaterial( getPlayerEntity(), "blood_cold", 0.0009 )
		end,
		excludes_relics			= "scorching_rose,vile_crystal,corroding_stone,ancient_mask,light_bulb,omega_shield",
		excludes_relics_desc	= "It can't be used along with other harmful material protection related relics",
		excludes_relics_tags	= "lava_protection,acid_protection,poison_protection,radgas_protection,freezegas_protection",
		combine_exclusive		= true,
		combine_requires_relics	= "scorching_rose,vile_crystal,corroding_stone,ancient_mask,light_bulb",
	},
	{
		name 					= "wooden_helmet",
		ui_name					= "Wooden Helmet",
		categories				= "player,status_effects",
		effect_desc				= "You can no longer be frozen or polymorphed.",
		effect_desc_short		= "You can no longer be frozen or polymorphed.",
        precursor_types			= "magic_artifact",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			-- freeze protection
			LoadGameEffectEntityTo( getPlayerEntity(), getCustomEffect("effect_freeze_protection").path )
			-- poly protection
			local game_effect_comp_id, game_effect_entity_id = GetGameEffectLoadTo( getPlayerEntity(), "PROTECTION_POLYMORPH", true )
			if game_effect_comp_id ~= nil then
				ComponentSetValue2( game_effect_comp_id, "frames", -1)
				EntityAddTag(game_effect_entity_id, "clay_baked_doll_effect")
			end
		end,
		ondrop_function			= function()
			-- remove freeze and poly protection
			local player_children = EntityGetAllChildren(getPlayerEntity())
			if player_children == nil then return end
			for i,child in ipairs(player_children) do
				if(EntityGetName(child) == "freeze_protection" or EntityHasTag(child, "clay_baked_doll_effect")) then
					EntityKill(child)
				end
			end
		end,
		excludes_relics			= "frozen_helmet,clay_baked_doll,blessed_helmet",
		excludes_relics_desc	= "It can't be used along with other freeze or polymorph immunity related relics",
		excludes_relics_tags	= "freeze_immunitypolymorph_immunity",
		combine_exclusive		= true,
		combine_requires_relics	= "frozen_helmet,clay_baked_doll",
	},
	{
		name 					= "blessed_helmet",
		ui_name					= "Blessed Helmet",
		categories				= "player,status_effects",
		effect_desc				= "You are protected against freezing, polymorphing and effects inflicted by buffed enemies with their projectiles.",
		effect_desc_short		= "You can no longer be frozen, polymorphed or inflicted with effects by buffed enemies.",
        precursor_types			= "magic_artifact",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			-- freeze protection
			LoadGameEffectEntityTo( getPlayerEntity(), getCustomEffect("effect_freeze_protection").path )
			-- poly protection
			local game_effect_comp_id, game_effect_entity_id = GetGameEffectLoadTo( getPlayerEntity(), "PROTECTION_POLYMORPH", true )
			if game_effect_comp_id ~= nil then
				ComponentSetValue2( game_effect_comp_id, "frames", -1)
				EntityAddTag(game_effect_entity_id, "clay_baked_doll_effect")
			end
			-- immunity to status effects
			addNewSimplifiedInternalVariable(getPlayerEntity(), "effects_immune")
		end,
		ondrop_function			= function()
			-- remove freeze and poly protection
			local player_children = EntityGetAllChildren(getPlayerEntity())
			if player_children == nil then return end
			for i,child in ipairs(player_children) do
				if(EntityGetName(child) == "freeze_protection" or EntityHasTag(child, "clay_baked_doll_effect")) then
					EntityKill(child)
				end
			end
			-- remove immunity to status effects
			removeSimplifiedInternalVariable(getPlayerEntity(), "effects_immune")
		end,
		excludes_relics			= "frozen_helmet,clay_baked_doll,wooden_helmet,blessed_shield",
		excludes_relics_desc	= "It can't be used along with other buffed creature projectile effects, freeze or polymorph immunity related relics",
		excludes_relics_tags	= "buffed_creatures_effects_immunity,freeze_immunity,polymorph_immunity",
		combine_exclusive		= true,
		combine_requires_relics	= "wooden_helmet,blessed_shield",
	},
	{
		name 					= "omega_shield",
		ui_name					= "Omega Shield",
		categories				= "player,status_effects,materials",
		effect_desc				= "You are immune to freezing, polymorphing, effects inflicted by buffed enemies and damaging liquids and gases.",
		effect_desc_short		= "You are immune to freezing, polymorphing, effects inflicted by buffed enemies and damaging liquids and gases.",
        precursor_types			= "magic_artifact",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			-- freeze protection
			LoadGameEffectEntityTo( getPlayerEntity(), getCustomEffect("effect_freeze_protection").path )
			-- poly protection
			local game_effect_comp_id, game_effect_entity_id = GetGameEffectLoadTo( getPlayerEntity(), "PROTECTION_POLYMORPH", true )
			if game_effect_comp_id ~= nil then
				ComponentSetValue2( game_effect_comp_id, "frames", -1)
				EntityAddTag(game_effect_entity_id, "clay_baked_doll_effect")
			end
			-- immunity to status effects
			addNewSimplifiedInternalVariable(getPlayerEntity(), "effects_immune")
			-- immunity to damaging materials
			EntitySetDamageFromMaterial( getPlayerEntity(), "acid", 0 )
			EntitySetDamageFromMaterial( getPlayerEntity(), "lava", 0)
			EntitySetDamageFromMaterial( getPlayerEntity(), "poison", 0)
			EntitySetDamageFromMaterial( getPlayerEntity(), "radioactive_gas", 0)
			EntitySetDamageFromMaterial( getPlayerEntity(), "blood_cold_vapour", 0)
			EntitySetDamageFromMaterial( getPlayerEntity(), "blood_cold", 0)
		end,
		ondrop_function			= function()
			-- remove freeze and poly protection
			local player_children = EntityGetAllChildren(getPlayerEntity())
			if player_children == nil then return end
			for i,child in ipairs(player_children) do
				if(EntityGetName(child) == "freeze_protection" or EntityHasTag(child, "clay_baked_doll_effect")) then
					EntityKill(child)
				end
			end
			-- remove immunity to status effects
			removeSimplifiedInternalVariable(getPlayerEntity(), "effects_immune")
			-- remove immunity to materials
			EntitySetDamageFromMaterial( getPlayerEntity(), "acid", 0.005 )
			EntitySetDamageFromMaterial( getPlayerEntity(), "lava", 0.003 )
			EntitySetDamageFromMaterial( getPlayerEntity(), "poison", 0.001 )
			EntitySetDamageFromMaterial( getPlayerEntity(), "radioactive_gas", 0.001 )
			EntitySetDamageFromMaterial( getPlayerEntity(), "blood_cold_vapour", 0.0006 )
			EntitySetDamageFromMaterial( getPlayerEntity(), "blood_cold", 0.0009 )
		end,
		excludes_relics			= "frozen_helmet,clay_baked_doll,wooden_helmet,blessed_shield,blessed_helmet,universal_immunity_stone,scorching_rose,vile_crystal,corroding_stone,ancient_mask,light_bulb",
		excludes_relics_desc	= "It can't be used along with other material protection and effect immunity related relics",
		excludes_relics_tags	= "lava_protection,acid_protection,poison_protection,radgas_protection,freezegas_protection,freeze_immunity,polymorph_immunity,buffed_creatures_effects_immunity",
		combine_exclusive		= true,
		combine_requires_relics	= "blessed_helmet,universal_immunity_stone",
	},
	{
		name 					= "overloaded_necklace",
		ui_name					= "Overloaded Necklace",
		categories				= "projectiles",
		effect_desc				= "Enemies killed by your spells leave a thunderball that explodes shortly afterwards.",
		effect_desc_short		= "Enemies killed by your spells leave a thunderball that explodes shortly afterwards.",
        precursor_types			= "necklace",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onenemykill_function	= function(damage_type_bit_field, damage_message, entity_thats_responsible, drop_items)
			if(isRelicOwned("overloaded_necklace") >= 1) then
				local death_x, death_y = EntityGetTransform(GetUpdatedEntityID())
				EntityLoad("mods/ArcaneReliquary/files/entity_definitions/custom_projectiles/thunderball_custom.xml", death_x, death_y - 15)
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "charged_necklace,tesla_coil",
	},
	{
		name 					= "mystic_thunder_necklace",
		ui_name					= "Mystic Thunder Crystal",
		categories				= "projectiles",
		effect_desc				= "Spells that you cast have a 40% chance of leaving red thunderballs along its travel path.",
		effect_desc_short		= "Your spells leave red thunderballs as they travel.",
        precursor_types			= "necklace",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%, and when it triggers, red thunderballs are left more often.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("mystic_thunder_necklace") >= 1) then
				if(not isPhysicsProjectile(projectile_entity_id)) then
					SetRandomSeed( GameGetFrameNum(), math.random(1,50))
					local chance = Random(1,100)
					local odds = 40 + (5 * getRelicLevel("mystic_thunder_necklace"))
					if(chance >= 1 and chance <= odds) then
						addNewInternalVariable(projectile_entity_id, "projectile_shot_down_definition", "value_string", "mods/ArcaneReliquary/files/entity_definitions/custom_projectiles/thunderball_red.xml")
						addNewInternalVariable(projectile_entity_id, "down_vel_min", "value_int", 0)
						addNewInternalVariable(projectile_entity_id, "down_vel_max", "value_int", 0)
						addNewInternalVariable(projectile_entity_id, "max_projectiles_shot", "value_int", 10)
						addNewLuaCallback(projectile_entity_id, "overtime", "mods/ArcaneReliquary/files/scripts/projectiles/launch_projectile_down.lua", 25 - (2 * getRelicLevel("mystic_thunder_necklace")), false, "launch_projectile_down")
					end
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "overloaded_necklace,red_thunder_stone",
	},
	{
		name 					= "little_chest",
		ui_name					= "Little Chest",
		categories				= "companions",
		effect_desc				= "A little chest joins you. Store up to 10 relics on it by throwing them at it, and \nmake it drop them by shooting it.",
		effect_desc_short		= "A little chest capable or carrying relics joins you.",
        precursor_types			= "companion",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			attachCompaniontoEntity(getPlayerEntity(), "little_chest", "mortal")
		end,
		ondrop_function			= function()	
			-- before killing companion, drops all relics it has stored
			local player_children = EntityGetAllChildren(getPlayerEntity())
			if player_children == nil then return end
			for i,child_id in ipairs(player_children) do
				if(EntityGetName(child_id) == "little_chest") then
					local x, y = EntityGetTransform(child_id)

					SetRandomSeed( GameGetFrameNum(), math.random(1,50))
					local internal_variables = EntityGetComponentIncludingDisabled( child_id, "VariableStorageComponent" )
					if ( internal_variables ~= nil ) then
						for key,internal_variable in pairs(internal_variables) do
							if(string.find(ComponentGetValue2( internal_variable, "name" ), "stored_relic" )) then
								EntityLoad(getRelicsBaseDir() .. ComponentGetValue2( internal_variable, "value_string" ) .. ".xml", x + Random(-5, 5), y + Random(12,14))
								EntityRemoveComponent(child_id, internal_variable)
							end
						end
					end
				end
			end
			-- kills companion
			detachCompanionfromEntity(getPlayerEntity(), "little_chest")
		end,
	},
	{
		name 					= "wisdom_medallion",
		ui_name					= "Wisdom Medallion",
		categories				= "player",
		effect_desc				= "New relics collected have a 50% chance to be instantly upgraded to a random level up to level 5. \nIt has no effect on relics that already have levels.",
		effect_desc_short		= "New relics collected have a 50% chance to be instantly upgraded to a random level up to level 5.",
        precursor_types			= "magic_artifact",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			addNewSimplifiedInternalVariable(getPlayerEntity(), "wisdom_medallion_effect")
		end,
		ondrop_function			= function()
			removeSimplifiedInternalVariable(getPlayerEntity(), "wisdom_medallion_effect")
		end
	},
	{
		name 					= "nuclear_shroom",
		ui_name					= "Nuclear Shroom",
		categories				= "enemies",
		effect_desc				= "Enemies you kill have a 5% chance of summoning a fungi death ray where they die.",
		effect_desc_short		= "Killed enemies summon fungi death rays.",
        precursor_types			= "magic_artifact,death_ray",
		upgrade_effect_desc		= "For each upgrade level, spawn chance increases by 5%.",
		is_upgradable 			= true,
		onenemykill_function	= function(damage_type_bit_field, damage_message, entity_thats_responsible, drop_items)
			if(isRelicOwned("nuclear_shroom") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 5 + (5 * getRelicLevel("nuclear_shroom"))
				if(chance >= 1 and chance <= odds) then
					local death_x, death_y = EntityGetTransform(GetUpdatedEntityID())
					EntityLoad("mods/ArcaneReliquary/files/entity_definitions/others/death_ray_fungi.xml", death_x, death_y)
				end
			end
		end,
	},
	{
		name 					= "fungi_gem",
		ui_name					= "Fungi Gem",
		categories				= "projectiles,materials",
		effect_desc				= "Spells you cast that explode on impact have a 20% chance of releasing fungi on impact.",
		effect_desc_short		= "Your spells release fungi.",
        precursor_types			= "elemental_gem",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("fungi_gem") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 20 + (5 * getRelicLevel("fungi_gem"))
				if(chance >= 1 and chance <= odds) then
					attachLoadEntityOnExplosiontoProjectile(projectile_entity_id, getLoadOnExplosion("launch_elemental_particles_fungi").path, "fungi")
				end
			end
		end,
	},
	{
		name 					= "death_ray_stone_nuclear",
		ui_name					= "Nuclear Death Ray Stone",
		categories				= "player,kick",
		effect_desc				= "When you land a kick on an enemy, there's a 50% chance a nuclear \ndeath ray that explodes in a huge radius will be summoned. \nIt also synergizes with Sniper Scope relic effect.",
		effect_desc_short		= "Kicking enemies has a chance of summoning a nuclear death ray.",
        precursor_types			= "death_ray,effect_stone",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onenemykicked_function	= function (entity_who_kicked)
			if(isRelicOwned("death_ray_stone_nuclear") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 50 + (5 * getRelicLevel("death_ray_stone_nuclear"))
				if(chance >= 1 and chance <= odds) then
					local enemy_id = GetUpdatedEntityID()
					local x, y = EntityGetTransform(enemy_id)
					GamePlaySound( "data/audio/Desktop/misc.bank", "misc/beam_from_sky_kick", x, y )
					EntityLoad( "mods/ArcaneReliquary/files/entity_definitions/others/death_ray_nuclear.xml", x, y )
				end
			end
		end,
		onextendedkick_function	= function( entity_id, extended_kick_id)
			if(isRelicOwned("death_ray_stone_nuclear") >= 1) then
				attachLoadEffectOntoEntitiestoProjectile(extended_kick_id, getCustomEffect("effect_summon_deathray_nuclear").path, "radioactive_gas")
			end
		end,
		excludes_relics			= "death_ray_stone",
		excludes_relics_desc	= "It can't be used along with other death ray summon on kick related relics",
		excludes_relics_tags	= "deathray_summon_on_kick",
		combine_exclusive		= true,
		combine_requires_relics	= "death_ray_stone,uranium_ingot",
	},
	{
		name 					= "friendly_slime_lava",
		ui_name					= "Friendly Lava Slime",
		categories				= "companions",
		effect_desc				= "A friendly lava slime that shoots lava bolts joins you.",
		effect_desc_short		= "A lava bolt shooting companion.",
        precursor_types			= "companion",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			attachCompaniontoEntity(getPlayerEntity(), "friendly_slime_lava", "mortal")
		end,
		ondrop_function			= function()
			detachCompanionfromEntity(getPlayerEntity(), "friendly_slime_lava")
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "friendly_slime,ember_stone",
		requires_relic			= "ball_of_goo",
	},
	{
		name 					= "friendly_slime_acid",
		ui_name					= "Friendly Acid Slime",
		categories				= "companions",
		effect_desc				= "A friendly acid slime that shoots acid bolts joins you.",
		effect_desc_short		= "A acid bolt shooting companion.",
        precursor_types			= "companion",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			attachCompaniontoEntity(getPlayerEntity(), "friendly_slime_acid", "mortal")
		end,
		ondrop_function			= function()
			detachCompanionfromEntity(getPlayerEntity(), "friendly_slime_acid")
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "friendly_slime,toxic_gland",
		requires_relic			= "ball_of_goo",
	},
	{
		name 					= "rocket_droid",
		ui_name					= "Rocket Droid",
		categories				= "companions",
		effect_desc				= "A small robot that shoots rockets joins you.",
		effect_desc_short		= "A rocket shooting companion.",
        precursor_types			= "companion",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			attachCompaniontoEntity(getPlayerEntity(), "rocket_droid", "mortal")
		end,
		ondrop_function			= function()
			detachCompanionfromEntity(getPlayerEntity(), "rocket_droid")
		end,
		requires_relic			= "alkaline_battery",
		combine_exclusive		= true,
		combine_requires_relics	= "rifle_droid,shotgun_droid",
	},
	{
		name 					= "repeater_droid",
		ui_name					= "Repeater Droid",
		categories				= "companions",
		effect_desc				= "A companion that shoots red laser orbs in quick succession joins you.",
		effect_desc_short		= "A red laser orb shooting companion.",
        precursor_types			= "companion",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			attachCompaniontoEntity(getPlayerEntity(), "repeater_droid", "mortal")
		end,
		ondrop_function			= function()
			detachCompanionfromEntity(getPlayerEntity(), "repeater_droid")
		end,
		requires_relic			= "alkaline_battery",
		combine_exclusive		= true,
		combine_requires_relics	= "security_probe,tiny_grenadier",
	},
	{
		name 					= "power_stone",
		ui_name					= "Power Stone",
		categories				= "projectiles",
		effect_desc				= "Your spells receive bonus damage and explosive power equivalent to 10% of the \nmana capacity of your currently equipped wand.",
		effect_desc_short		= "Your spells gain bonus damage and explosive power based your wand's mana.",
        precursor_types			= "effect_stone",
		upgrade_effect_desc		= "For each upgrade level, damage bonus threshold increases by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("power_stone") >= 1) then
				local mana_max = getCurrentWandManaMax()
				local bonus_damage = ((mana_max * 10 + (5 * getRelicLevel("power_stone"))) / 100) / 25 -- scaling it down so it gets closer to regular damage from vanilla spells
				--GamePrint("### bonus damage: " .. bonus_damage)
				addProjectileBaseDamage(projectile_entity_id, bonus_damage)
				addProjectileExplosiveDamage(projectile_entity_id, bonus_damage)
			end
		end,
	},
	{
		name 					= "tenacity_stone",
		ui_name					= "Tenacity Stone",
		categories				= "enemies,health",
		effect_desc				= "When you kill an enemy with your spells, there's a 25% chance you'll be healed \nby 5% of the mana capacity of your currently equipped wand.",
		effect_desc_short		= "Kills heal based on a portion of your wand's mana.",
        precursor_types			= "effect_stone",
		upgrade_effect_desc		= "For each upgrade level, healed amount and effect chance increase by 5%.",
		is_upgradable 			= true,
		onenemykill_function	= function(damage_type_bit_field, damage_message, entity_thats_responsible, drop_items)
			if(isRelicOwned("tenacity_stone") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 25 + (5 * getRelicLevel("tenacity_stone"))
				if(chance >= 1 and chance <= odds) then
					local mana_max = getCurrentWandManaMax()
					local heal_bonus = (mana_max * 5 + (5 * getRelicLevel("tenacity_stone"))) / 100
					addPlayerHealth(heal_bonus)
					local x, y = EntityGetTransform(getPlayerEntity())
					GamePlaySound( "data/audio/Desktop/event_cues.snd", "event_cues/heart/create", x, y )
				end
			end
		end,
	},
	{
		name 					= "hastiness_stone",
		ui_name					= "Hastiness Stone",
		categories				= "projectiles",
		effect_desc				= "Your spells get a speed multiplier equivalent to the quotient between number of \nslots and spells in your wand.",
		effect_desc_short		= "Your spells get speed bonus based on empty slots in your wand.",
        precursor_types			= "effect_stone",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("hastiness_stone") >= 1) then
				local free_slots, spells = getWandEmptyandOccupiedSlots(getCurrentlyEquippedWandId())
				if(free_slots ~= 0) then
					local multiplier = free_slots / spells
					if(multiplier > 1) then
						multiplyProjectileVelocity(projectile_entity_id, multiplier)
					end
				end
			end
		end,
	},
	{
		name 					= "decelerating_crystal",
		ui_name					= "Decelerating Crystal",
		categories				= "player",
		effect_desc				= "A pulsating energy field emanates from you. Enemies touched by it have their movement \nspeed severely reduced, but if you get hit by enemy projectiles the same effect is applied on you.",
		effect_desc_short		= "A pulsating energy field that slows enemies down emanates from you.",
        precursor_types			= "gadget,effect_crystal,energy_shield",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			LoadGameEffectEntityTo( getPlayerEntity(), getCustomEffect("effect_pulsating_shield_decelerate").path )
		end,
		ondrop_function			= function()
			local player_children = EntityGetAllChildren(getPlayerEntity())
			if player_children == nil then return end
			for i,child in ipairs(player_children) do
				if(EntityGetName(child) == "pulsating_shield_decelerate") then
					EntityKill(child)
					break
				end
			end
		end,
		ondamaged_function		= function(damage, message, entity_thats_responsible, is_fatal, projectile_thats_responsible )
			if(isRelicOwned("decelerating_crystal") >= 1) then
				if(message == "$damage_projectile" and not entityHasSimplifiedInternalVariable(GetUpdatedEntityID(), "movement_speed_debuff")) then
					LoadGameEffectEntityTo( getPlayerEntity(), getCustomEffect("effect_movement_speed_debuff").path )
				end
			end
		end,
	},
	{
		name 					= "radioactive_waste",
		ui_name					= "Radioactive Waste",
		categories				= "player",
		effect_desc				= "A pulsating energy field emanates from you. Enemies touched by it are instantly mutated, but \nif you get hit by enemy projectile, there's a 10% chance you'll be mutated instead.",
		effect_desc_short		= "A pulsating energy field that mutates enemies emanates from you.",
        precursor_types			= "gadget,energy_shield",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			LoadGameEffectEntityTo( getPlayerEntity(), getCustomEffect("effect_pulsating_shield_mutate").path )
		end,
		ondrop_function			= function()
			local player_children = EntityGetAllChildren(getPlayerEntity())
			if player_children == nil then return end
			for i,child in ipairs(player_children) do
				if(EntityGetName(child) == "pulsating_shield_mutate") then
					EntityKill(child)
					break
				end
			end
		end,
		ondamaged_function		= function(damage, message, entity_thats_responsible, is_fatal, projectile_thats_responsible )
			if(isRelicOwned("radioactive_waste") >= 1) then
				if(message == "$damage_projectile") then
					SetRandomSeed( GameGetFrameNum(), math.random(1,50))
					local chance = Random(1,100)
					local odds = 10
					if(chance >= 1 and chance <= odds) then
						LoadGameEffectEntityTo( getPlayerEntity(), "data/entities/misc/effect_polymorph.xml" )
					end
				end
			end
		end,
	},
	{
		name 					= "disarming_stone",
		ui_name					= "Disarming Stone",
		categories				= "player",
		effect_desc				= "A pulsating energy field emanates from you. Shooter enemies touched by \nit temporarily lose their ability to shoot.",
		effect_desc_short		= "A pulsating energy field that makes enemies unable to shoot emanates from you.",
        precursor_types			= "effect_stone,energy_shield",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			LoadGameEffectEntityTo( getPlayerEntity(), getCustomEffect("effect_pulsating_shield_disarm").path )
		end,
		ondrop_function			= function()
			local player_children = EntityGetAllChildren(getPlayerEntity())
			if player_children == nil then return end
			for i,child in ipairs(player_children) do
				if(EntityGetName(child) == "pulsating_shield_disarm") then
					EntityKill(child)
					break
				end
			end
		end,
	},
	{
		name 					= "gravity_crystal",
		ui_name					= "Gravity Crystal",
		categories				= "player",
		effect_desc				= "A pulsating energy field emanates from you. Flying enemies touched by \nit temporarily lose their ability to fly.",
		effect_desc_short		= "A pulsating energy field that makes enemies unable to fly emanates from you.",
        precursor_types			= "effect_crystal,energy_shield",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			LoadGameEffectEntityTo( getPlayerEntity(), getCustomEffect("effect_pulsating_shield_disable_flight").path )
		end,
		ondrop_function			= function()
			local player_children = EntityGetAllChildren(getPlayerEntity())
			if player_children == nil then return end
			for i,child in ipairs(player_children) do
				if(EntityGetName(child) == "pulsating_shield_disable_flight") then
					EntityKill(child)
					break
				end
			end
		end,
	},
	{
		name 					= "humid_crystal",
		ui_name					= "Humid Crystal",
		categories				= "player,materials",
		effect_desc				= "A pulsating energy field emanates from you. Enemy projectiles intercepted \nby it are transmuted into water, but if you get hit by them, poison is created instead.",
		effect_desc_short		= "A pulsating energy field that converts enemy projectiles into water emanates from you.",
        precursor_types			= "effect_crystal,energy_shield",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			LoadGameEffectEntityTo( getPlayerEntity(), getCustomEffect("effect_pulsating_shield_projectiles_to_water").path )
		end,
		ondrop_function			= function()
			local player_children = EntityGetAllChildren(getPlayerEntity())
			if player_children == nil then return end
			for i,child in ipairs(player_children) do
				if(EntityGetName(child) == "pulsating_shield_projectiles_to_water") then
					EntityKill(child)
					break
				end
			end
		end,
		ondamaged_function		= function(damage, message, entity_thats_responsible, is_fatal, projectile_thats_responsible )
			if(isRelicOwned("humid_crystal") >= 1) then
				if(message == "$damage_projectile") then
					local player_x, player_y = EntityGetTransform(GetUpdatedEntityID())
					SetRandomSeed( GameGetFrameNum(), math.random(1,50))
					EntityLoad(getLoadOnExplosion("launch_elemental_particles_poison").path, player_x + Random(-2, 2), player_y + Random(-2, 2))
				end
			end
		end,
	},
	{
		name 					= "frenzy_crystal",
		ui_name					= "Frenzy Crystal",
		categories				= "player,materials",
		effect_desc				= "A pulsating energy field emanates from you. Enemy projectiles intercepted \nby it are transmuted into berserkium, but if you get hit by them, toxic sludge is created instead.",
		effect_desc_short		= "A pulsating energy field that converts enemy projectiles into berserkium emanates from you.",
        precursor_types			= "effect_crystal,energy_shield",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			LoadGameEffectEntityTo( getPlayerEntity(), getCustomEffect("effect_pulsating_shield_projectiles_to_berserkium").path )
		end,
		ondrop_function			= function()
			local player_children = EntityGetAllChildren(getPlayerEntity())
			if player_children == nil then return end
			for i,child in ipairs(player_children) do
				if(EntityGetName(child) == "pulsating_shield_projectiles_to_berserkium") then
					EntityKill(child)
					break
				end
			end
		end,
		ondamaged_function		= function(damage, message, entity_thats_responsible, is_fatal, projectile_thats_responsible )
			if(isRelicOwned("frenzy_crystal") >= 1) then
				if(message == "$damage_projectile") then
					local player_x, player_y = EntityGetTransform(GetUpdatedEntityID())
					SetRandomSeed( GameGetFrameNum(), math.random(1,50))
					EntityLoad(getLoadOnExplosion("launch_elemental_particles_radioactive_liquid").path, player_x + Random(-2, 2), player_y + Random(-2, 2))
				end
			end
		end,
	},
	{
		name 					= "bloody_stone",
		ui_name					= "Bloody Stone",
		categories				= "player,materials",
		effect_desc				= "A pulsating energy field emanates from you. Enemy projectiles intercepted \nby it are transmuted into red blood, but if you get hit by them, acid is created instead.",
		effect_desc_short		= "A pulsating energy field that converts enemy projectiles into red blood emanates from you.",
        precursor_types			= "effect_stone,energy_shield",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			LoadGameEffectEntityTo( getPlayerEntity(), getCustomEffect("effect_pulsating_shield_projectiles_to_blood").path )
		end,
		ondrop_function			= function()
			local player_children = EntityGetAllChildren(getPlayerEntity())
			if player_children == nil then return end
			for i,child in ipairs(player_children) do
				if(EntityGetName(child) == "pulsating_shield_projectiles_to_blood") then
					EntityKill(child)
					break
				end
			end
		end,
		ondamaged_function		= function(damage, message, entity_thats_responsible, is_fatal, projectile_thats_responsible )
			if(isRelicOwned("bloody_stone") >= 1) then
				if(message == "$damage_projectile") then
					local player_x, player_y = EntityGetTransform(GetUpdatedEntityID())
					SetRandomSeed( GameGetFrameNum(), math.random(1,50))
					EntityLoad(getLoadOnExplosion("launch_elemental_particles_acid").path, player_x + Random(-2, 2), player_y + Random(-2, 2))
				end
			end
		end,
	},
	{
		name 					= "ethanolic_stone",
		ui_name					= "Ethanolic Stone",
		categories				= "player,materials",
		effect_desc				= "A pulsating energy field emanates from you. Enemy projectiles intercepted \nby it are transmuted into whiskey, but if you get hit by them, lava is created instead.",
		effect_desc_short		= "A pulsating energy field that converts enemy projectiles into whiskey emanates from you.",
        precursor_types			= "effect_stone,energy_shield",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			LoadGameEffectEntityTo( getPlayerEntity(), getCustomEffect("effect_pulsating_shield_projectiles_to_alcohol").path )
		end,
		ondrop_function			= function()
			local player_children = EntityGetAllChildren(getPlayerEntity())
			if player_children == nil then return end
			for i,child in ipairs(player_children) do
				if(EntityGetName(child) == "pulsating_shield_projectiles_to_alcohol") then
					EntityKill(child)
					break
				end
			end
		end,
		ondamaged_function		= function(damage, message, entity_thats_responsible, is_fatal, projectile_thats_responsible )
			if(isRelicOwned("ethanolic_stone") >= 1) then
				if(message == "$damage_projectile") then
					local player_x, player_y = EntityGetTransform(GetUpdatedEntityID())
					SetRandomSeed( GameGetFrameNum(), math.random(1,50))
					EntityLoad(getLoadOnExplosion("launch_elemental_particles_lava").path, player_x + Random(-2, 2), player_y + Random(-2, 2))
				end
			end
		end,
	},
	{
		name 					= "frost_crystal",
		ui_name					= "Frost Crystal",
		categories				= "player,materials",
		effect_desc				= "A pulsating energy field emanates from you. Enemy projectiles intercepted \nby it are transmuted into freezing gas, but if you get hit by them, slime is created instead.",
		effect_desc_short		= "A pulsating energy field that converts enemy projectiles into freezing gas emanates from you.",
        precursor_types			= "effect_crystal,energy_shield",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			LoadGameEffectEntityTo( getPlayerEntity(), getCustomEffect("effect_pulsating_shield_projectiles_to_blood_cold_vapour").path )
		end,
		ondrop_function			= function()
			local player_children = EntityGetAllChildren(getPlayerEntity())
			if player_children == nil then return end
			for i,child in ipairs(player_children) do
				if(EntityGetName(child) == "pulsating_shield_projectiles_to_blood_cold_vapour") then
					EntityKill(child)
					break
				end
			end
		end,
		ondamaged_function		= function(damage, message, entity_thats_responsible, is_fatal, projectile_thats_responsible )
			if(isRelicOwned("frost_crystal") >= 1) then
				if(message == "$damage_projectile") then
					local player_x, player_y = EntityGetTransform(GetUpdatedEntityID())
					SetRandomSeed( GameGetFrameNum(), math.random(1,50))
					EntityLoad(getLoadOnExplosion("launch_elemental_particles_slime").path, player_x + Random(-2, 2), player_y + Random(-2, 2))
				end
			end
		end,
	},
	{
		name 					= "toxic_stone",
		ui_name					= "Toxic Stone",
		categories				= "player,materials",
		effect_desc				= "A pulsating energy field emanates from you. Enemy projectiles intercepted \nby it are transmuted into toxic gas, but if you get hit by them, freezing liquid is created instead.",
		effect_desc_short		= "A pulsating energy field that converts enemy projectiles into toxic gas emanates from you.",
        precursor_types			= "effect_stone,energy_shield",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			LoadGameEffectEntityTo( getPlayerEntity(), getCustomEffect("effect_pulsating_shield_projectiles_to_radioactive_gas").path )
		end,
		ondrop_function			= function()
			local player_children = EntityGetAllChildren(getPlayerEntity())
			if player_children == nil then return end
			for i,child in ipairs(player_children) do
				if(EntityGetName(child) == "pulsating_shield_projectiles_to_radioactive_gas") then
					EntityKill(child)
					break
				end
			end
		end,
		ondamaged_function		= function(damage, message, entity_thats_responsible, is_fatal, projectile_thats_responsible )
			if(isRelicOwned("toxic_stone") >= 1) then
				if(message == "$damage_projectile") then
					local player_x, player_y = EntityGetTransform(GetUpdatedEntityID())
					SetRandomSeed( GameGetFrameNum(), math.random(1,50))
					EntityLoad(getLoadOnExplosion("launch_elemental_particles_freezing_liquid").path, player_x + Random(-2, 2), player_y + Random(-2, 2))
				end
			end
		end,
	},
	{
		name 					= "greased_stone",
		ui_name					= "Greased Stone",
		categories				= "player,materials",
		effect_desc				= "A pulsating energy field emanates from you. Enemy projectiles intercepted \nby it are transmuted into oil, but if you get hit by them, urine is created instead.",
		effect_desc_short		= "A pulsating energy field that converts enemy projectiles into oil emanates from you.",
        precursor_types			= "effect_stone,energy_shield",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			LoadGameEffectEntityTo( getPlayerEntity(), getCustomEffect("effect_pulsating_shield_projectiles_to_oil").path )
		end,
		ondrop_function			= function()
			local player_children = EntityGetAllChildren(getPlayerEntity())
			if player_children == nil then return end
			for i,child in ipairs(player_children) do
				if(EntityGetName(child) == "pulsating_shield_projectiles_to_oil") then
					EntityKill(child)
					break
				end
			end
		end,
		ondamaged_function		= function(damage, message, entity_thats_responsible, is_fatal, projectile_thats_responsible )
			if(isRelicOwned("greased_stone") >= 1) then
				if(message == "$damage_projectile") then
					local player_x, player_y = EntityGetTransform(GetUpdatedEntityID())
					SetRandomSeed( GameGetFrameNum(), math.random(1,50))
					EntityLoad(getLoadOnExplosion("launch_elemental_particles_urine").path, player_x + Random(-2, 2), player_y + Random(-2, 2))
				end
			end
		end,
	},
	{
		name 					= "urinous_crystal",
		ui_name					= "Urinous Crystal",
		categories				= "player,materials",
		effect_desc				= "A pulsating energy field emanates from you. Enemy projectiles intercepted \nby it are transmuted into urine, but if you get hit by them, polymorphine is created instead.",
		effect_desc_short		= "A pulsating energy field that converts enemy projectiles into urine emanates from you.",
        precursor_types			= "effect_crystal,energy_shield",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			LoadGameEffectEntityTo( getPlayerEntity(), getCustomEffect("effect_pulsating_shield_projectiles_to_urine").path )
		end,
		ondrop_function			= function()
			local player_children = EntityGetAllChildren(getPlayerEntity())
			if player_children == nil then return end
			for i,child in ipairs(player_children) do
				if(EntityGetName(child) == "pulsating_shield_projectiles_to_urine") then
					EntityKill(child)
					break
				end
			end
		end,
		ondamaged_function		= function(damage, message, entity_thats_responsible, is_fatal, projectile_thats_responsible )
			if(isRelicOwned("urinous_crystal") >= 1) then
				if(message == "$damage_projectile") then
					local player_x, player_y = EntityGetTransform(GetUpdatedEntityID())
					SetRandomSeed( GameGetFrameNum(), math.random(1,50))
					EntityLoad(getLoadOnExplosion("launch_elemental_particles_poly").path, player_x + Random(-2, 2), player_y + Random(-2, 2))
				end
			end
		end,
	},
	{
		name 					= "concentrated_mana_crystal",
		ui_name					= "Concentrated Mana Crystal",
		categories				= "player,materials",
		effect_desc				= "A pulsating energy field emanates from you. Enemy projectiles intercepted \nby it are transmuted into concentrated mana, but if you get hit by them, teleportatium is created instead.",
		effect_desc_short		= "A pulsating energy field that converts enemy projectiles into concentrated mana emanates from you.",
        precursor_types			= "effect_crystal,energy_shield",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			LoadGameEffectEntityTo( getPlayerEntity(), getCustomEffect("effect_pulsating_shield_projectiles_to_concentrated_mana").path )
		end,
		ondrop_function			= function()
			local player_children = EntityGetAllChildren(getPlayerEntity())
			if player_children == nil then return end
			for i,child in ipairs(player_children) do
				if(EntityGetName(child) == "pulsating_shield_projectiles_to_concentrated_mana") then
					EntityKill(child)
					break
				end
			end
		end,
		ondamaged_function		= function(damage, message, entity_thats_responsible, is_fatal, projectile_thats_responsible )
			if(isRelicOwned("concentrated_mana_crystal") >= 1) then
				if(message == "$damage_projectile") then
					local player_x, player_y = EntityGetTransform(GetUpdatedEntityID())
					SetRandomSeed( GameGetFrameNum(), math.random(1,50))
					EntityLoad(getLoadOnExplosion("launch_elemental_particles_teleportatium").path, player_x + Random(-2, 2), player_y + Random(-2, 2))
				end
			end
		end,
	},
	{
		name 					= "invulnerability_crystal",
		ui_name					= "Invulnerability Crystal",
		categories				= "player,materials",
		effect_desc				= "A pulsating energy field emanates from you. Enemy projectiles intercepted \nby it are transmuted into ambrosia.",
		effect_desc_short		= "A pulsating energy field that converts enemy projectiles into ambrosia emanates from you.",
        precursor_types			= "effect_crystal,energy_shield",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			LoadGameEffectEntityTo( getPlayerEntity(), getCustomEffect("effect_pulsating_shield_projectiles_to_ambrosia").path )
		end,
		ondrop_function			= function()
			local player_children = EntityGetAllChildren(getPlayerEntity())
			if player_children == nil then return end
			for i,child in ipairs(player_children) do
				if(EntityGetName(child) == "pulsating_shield_projectiles_to_ambrosia") then
					EntityKill(child)
					break
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "concentrated_mana_crystal,humid_crystal,urinous_crystal,frost_crystal,frenzy_crystal",
	},
	{
		name 					= "midas_stone",
		ui_name					= "Midas Stone",
		categories				= "player,materials",
		effect_desc				= "A pulsating energy field emanates from you. Enemy projectiles intercepted \nby it are transmuted into gold.",
		effect_desc_short		= "A pulsating energy field that converts enemy projectiles into gold emanates from you.",
        precursor_types			= "effect_stone,energy_shield",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			LoadGameEffectEntityTo( getPlayerEntity(), getCustomEffect("effect_pulsating_shield_projectiles_to_gold").path )
		end,
		ondrop_function			= function()
			local player_children = EntityGetAllChildren(getPlayerEntity())
			if player_children == nil then return end
			for i,child in ipairs(player_children) do
				if(EntityGetName(child) == "pulsating_shield_projectiles_to_gold") then
					EntityKill(child)
					break
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "bloody_stone,ethanolic_stone,greased_stone,toxic_stone",
	},
	{
		name 					= "necklace_of_strength",
		ui_name					= "Necklace of Strength",
		categories				= "player,materials",
		effect_desc				= "A pulsating energy field emanates from you. Enemy projectiles intercepted \nby it are transmuted into healthium.",
		effect_desc_short		= "A pulsating energy field that converts enemy projectiles into healthium emanates from you.",
        precursor_types			= "necklace,energy_shield",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			LoadGameEffectEntityTo( getPlayerEntity(), getCustomEffect("effect_pulsating_shield_projectiles_to_healthium").path )
		end,
		ondrop_function			= function()
			local player_children = EntityGetAllChildren(getPlayerEntity())
			if player_children == nil then return end
			for i,child in ipairs(player_children) do
				if(EntityGetName(child) == "pulsating_shield_projectiles_to_healthium") then
					EntityKill(child)
					break
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "midas_stone,invulnerability_crystal",
	},
	{
		name 					= "gatling_droid",
		ui_name					= "Gatling Droid",
		categories				= "companions",
		effect_desc				= "A companion equipped with a gatling gun joins you.",
		effect_desc_short		= "A companion equipped with a gatling gun.",
        precursor_types			= "companion",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			attachCompaniontoEntity(getPlayerEntity(), "gatling_droid", "mortal")
		end,
		ondrop_function			= function()
			detachCompanionfromEntity(getPlayerEntity(), "gatling_droid")
		end,
		requires_relic			= "alkaline_battery",
		combine_exclusive		= true,
		combine_requires_relics	= "rifle_droid,hermes_sandal",
	},
	{
		name 					= "sniper_droid",
		ui_name					= "Sniper Droid",
		categories				= "companions",
		effect_desc				= "A companion equipped with a sniper rifle joins you.",
		effect_desc_short		= "A companion equipped with a sniper rifle.",
        precursor_types			= "companion",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			attachCompaniontoEntity(getPlayerEntity(), "sniper_droid", "mortal")
		end,
		ondrop_function			= function()
			detachCompanionfromEntity(getPlayerEntity(), "sniper_droid")
		end,
		requires_relic			= "alkaline_battery",
		combine_exclusive		= true,
		combine_requires_relics	= "rifle_droid,sniper_scope",
	},
	{
		name 					= "buzzy",
		ui_name					= "Buzzy",
		categories				= "companions",
		effect_desc				= "A companion equipped with a sawblade launcher joins you.",
		effect_desc_short		= "A companion equipped with a sawblade launcher.",
        precursor_types			= "companion",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			attachCompaniontoEntity(getPlayerEntity(), "buzzy", "mortal")
		end,
		ondrop_function			= function()
			detachCompanionfromEntity(getPlayerEntity(), "buzzy")
		end,
		requires_relic			= "alkaline_battery",
		combine_exclusive		= true,
		combine_requires_relics	= "rifle_droid,sawblade",
	},
	{
		name 					= "bloody_skull",
		ui_name					= "Bloody Skull",
		categories				= "companions",
		effect_desc				= "A friendly skull that shoots bloodbombs that induce bleeding on enemies joins you.",
		effect_desc_short		= "A friendly skull that shoots bloodbombs that induce bleeding on enemies.",
        precursor_types			= "companion",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			attachCompaniontoEntity(getPlayerEntity(), "bloody_skull", "mortal")
		end,
		ondrop_function			= function()
			detachCompanionfromEntity(getPlayerEntity(), "bloody_skull")
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "enchanted_skull,ancient_dagger",
		requires_relic			= "bone",
	},
	{
		name 					= "enchanted_skull",
		ui_name					= "Enchanted Skull",
		categories				= "companions",
		effect_desc				= "A friendly skull that shoots bones that deal melee damage joins you.",
		effect_desc_short		= "A friendly skull that shoots bones that deal melee damage.",
        precursor_types			= "companion",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			attachCompaniontoEntity(getPlayerEntity(), "enchanted_skull", "mortal")
			--addNewSimplifiedInternalVariable( getPlayerEntity(), "companion_shield")
		end,
		ondrop_function			= function()
			detachCompanionfromEntity(getPlayerEntity(), "enchanted_skull")
		end,
		requires_relic			= "bone",
	},
	{
		name 					= "slapper_skull",
		ui_name					= "Slapper Skull",
		categories				= "companions",
		effect_desc				= "A friendly skull that shoots slapping bombs joins you.",
		effect_desc_short		= "A friendly skull that shoots slapping bombs.",
        precursor_types			= "companion",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			attachCompaniontoEntity(getPlayerEntity(), "slapper_skull", "mortal")
		end,
		ondrop_function			= function()
			detachCompanionfromEntity(getPlayerEntity(), "slapper_skull")
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "enchanted_skull,slapper_stool",
		requires_relic			= "bone",
	},
	{
		name 					= "security_probe_yellow",
		ui_name					= "Security Probe with Yellow Lens",
		categories				= "companions",
		effect_desc				= "A security probe companion that shoots explosive laser beams joins you.",
		effect_desc_short		= "An explosive laser beam shooting companion.",
        precursor_types			= "companion",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			attachCompaniontoEntity(getPlayerEntity(), "security_probe_yellow", "mortal")
		end,
		ondrop_function			= function()
			detachCompanionfromEntity(getPlayerEntity(), "security_probe_yellow")
		end,
		requires_relic			= "alkaline_battery",
		combine_exclusive		= true,
		combine_requires_relics	= "security_probe,nuclear_glass",
	},
	{
		name 					= "security_probe_orange",
		ui_name					= "Security Probe with Orange Lens",
		categories				= "companions",
		effect_desc				= "A security probe companion that shoots incendiary laser beams joins you.",
		effect_desc_short		= "An incendiary laser beam shooting companion.",
        precursor_types			= "companion",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			attachCompaniontoEntity(getPlayerEntity(), "security_probe_orange", "mortal")
		end,
		ondrop_function			= function()
			detachCompanionfromEntity(getPlayerEntity(), "security_probe_orange")
		end,
		requires_relic			= "alkaline_battery",
		combine_exclusive		= true,
		combine_requires_relics	= "security_probe,blaze_pearl",
	},
	{
		name 					= "security_probe_green",
		ui_name					= "Security Probe with Green Lens",
		categories				= "companions",
		effect_desc				= "A security probe companion that shoots corroding laser beams joins you.",
		effect_desc_short		= "A corroding laser beam shooting companion.",
        precursor_types			= "companion",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			attachCompaniontoEntity(getPlayerEntity(), "security_probe_green", "mortal")
		end,
		ondrop_function			= function()
			detachCompanionfromEntity(getPlayerEntity(), "security_probe_green")
		end,
		requires_relic			= "alkaline_battery",
		combine_exclusive		= true,
		combine_requires_relics	= "security_probe,acid_gem",
	},
	{
		name 					= "security_probe_teal",
		ui_name					= "Security Probe with Teal Lens",
		categories				= "companions",
		effect_desc				= "A security probe companion that shoots freezing laser beams joins you.",
		effect_desc_short		= "A freezing laser beam shooting companion.",
        precursor_types			= "companion",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			attachCompaniontoEntity(getPlayerEntity(), "security_probe_teal", "mortal")
		end,
		ondrop_function			= function()
			detachCompanionfromEntity(getPlayerEntity(), "security_probe_teal")
		end,
		requires_relic			= "alkaline_battery",
		combine_exclusive		= true,
		combine_requires_relics	= "security_probe,ice_artifact",
	},
	{
		name 					= "security_probe_blue",
		ui_name					= "Security Probe with Blue Lens",
		categories				= "companions",
		effect_desc				= "A security probe companion that shoots electrocuting laser beams joins you.",
		effect_desc_short		= "An electrocuting laser beam shooting companion.",
        precursor_types			= "companion",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			attachCompaniontoEntity(getPlayerEntity(), "security_probe_blue", "mortal")
		end,
		ondrop_function			= function()
			detachCompanionfromEntity(getPlayerEntity(), "security_probe_blue")
		end,
		requires_relic			= "alkaline_battery",
		combine_exclusive		= true,
		combine_requires_relics	= "security_probe,charged_necklace",
	},
	{
		name 					= "security_probe_purple",
		ui_name					= "Security Probe with Purple Lens",
		categories				= "companions",
		effect_desc				= "A security probe companion that shoots mutating laser beams joins you.",
		effect_desc_short		= "A mutating laser beam shooting companion.",
        precursor_types			= "companion",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			attachCompaniontoEntity(getPlayerEntity(), "security_probe_purple", "mortal")
		end,
		ondrop_function			= function()
			detachCompanionfromEntity(getPlayerEntity(), "security_probe_purple")
		end,
		requires_relic			= "alkaline_battery",
		combine_exclusive		= true,
		combine_requires_relics	= "security_probe,poly_gem",
	},
	{
		name 					= "deceasing_stone",
		ui_name					= "Deceasing Stone",
		categories				= "projectiles",
		effect_desc				= "Spells you cast have a 5% chance of instantly killing creatures they hit.",
		effect_desc_short		= "Your spells have a chance to instantly kill enemies hit.",
        precursor_types			= "effect_stone",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 2%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("deceasing_stone") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 5 + (2 * getRelicLevel("deceasing_stone"))
				if(chance >= 1 and chance <= odds) then
					attachLoadEffectOntoEntitiestoProjectile(projectile_entity_id, getCustomEffect("effect_instakill").path, "spark_purple_bright")
				end
			end
		end,
	},
	{
		name 					= "deflecting_medalion_thunder",
		ui_name					= "Deflecting Medalion of Thunders",
		categories				= "player",
		effect_desc				= "An energy shield is projected around, deflecting and \ncharging enemy projectile with the power of thunders and \nalso electrocuting enemies that get caught by it.",
		effect_desc_short		= "A thunderous energy shield is projected around you.",
        precursor_types			= "magic_artifact,energy_shield",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			LoadGameEffectEntityTo( getPlayerEntity(), getCustomEffect("effect_deflecting_thunder_shield").path )
		end,
		ondrop_function			= function()
			local player_children = EntityGetAllChildren(getPlayerEntity())
			if player_children == nil then return end
			for i,child in ipairs(player_children) do
				if(EntityGetName(child) == "deflecting_shield_thunder") then
					EntityKill(child)
					break
				end
			end
		end,
		excludes_relics			= "deflecting_medalion_master",
		excludes_relics_desc	= "It can't be used along with Master Deflecting Medallion relic",
		excludes_relics_tags	= "deflecting_medalion_thunder",
		combine_exclusive		= true,
		combine_requires_relics	= "deflecting_medalion_charged,tesla_coil",
	},
	{
		name 					= "deflecting_medalion_thunder_red",
		ui_name					= "Deflecting Medalion of Red Thunders",
		categories				= "player",
		effect_desc				= "An energy shield is projected around, deflecting and \ncharging enemy projectile with the power of red homing thunders and \nalso electrocuting enemies that get caught by it.",
		effect_desc_short		= "A red thunderous energy shield is projected around you.",
        precursor_types			= "magic_artifact,energy_shield",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			LoadGameEffectEntityTo( getPlayerEntity(), getCustomEffect("effect_deflecting_thunder_red_shield").path )
		end,
		ondrop_function			= function()
			local player_children = EntityGetAllChildren(getPlayerEntity())
			if player_children == nil then return end
			for i,child in ipairs(player_children) do
				if(EntityGetName(child) == "deflecting_shield_thunder_red") then
					EntityKill(child)
					break
				end
			end
		end,
		excludes_relics			= "deflecting_medalion_master",
		excludes_relics_desc	= "It can't be used along with Master Deflecting Medallion relic",
		excludes_relics_tags	= "deflecting_medalion_thunder_red",
		combine_exclusive		= true,
		combine_requires_relics	= "deflecting_medalion_charged,red_thunder_stone",
	},
	{
		name 					= "deflecting_medalion_flame",
		ui_name					= "Deflecting Medalion of Flames",
		categories				= "player",
		effect_desc				= "An energy shield is projected around you, converting enemy \nprojectiles into explosive fireballs and shooting them back, and \nalso setting on fire enemies caught by it.",
		effect_desc_short		= "A flame-imbued energy shield is projected around you.",
        precursor_types			= "magic_artifact,energy_shield",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			LoadGameEffectEntityTo( getPlayerEntity(), getCustomEffect("effect_fireball_shield").path )
		end,
		ondrop_function			= function()
			local player_children = EntityGetAllChildren(getPlayerEntity())
			if player_children == nil then return end
			for i,child in ipairs(player_children) do
				if(EntityGetName(child) == "deflecting_shield_fireball") then
					EntityKill(child)
					break
				end
			end
		end,
		excludes_relics			= "deflecting_medalion_master",
		excludes_relics_desc	= "It can't be used along with Master Deflecting Medallion relic",
		excludes_relics_tags	= "deflecting_medalion_flame",
		combine_exclusive		= true,
		combine_requires_relics	= "deflecting_medalion_molten,ember_stone",
	},
	{
		name 					= "deflecting_medalion_radioactive",
		ui_name					= "Deflecting Medalion of Radiation",
		categories				= "player",
		effect_desc				= "An energy shield is projected around you, converting enemy \nprojectiles into radioactive fireballs and shooting them back, and \nalso causing radiation damage on enemies caught by it.",
		effect_desc_short		= "A radiation-imbued energy shield is projected around you.",
        precursor_types			= "magic_artifact,energy_shield",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			LoadGameEffectEntityTo( getPlayerEntity(), getCustomEffect("effect_fireball_radioactive_shield").path )
		end,
		ondrop_function			= function()
			local player_children = EntityGetAllChildren(getPlayerEntity())
			if player_children == nil then return end
			for i,child in ipairs(player_children) do
				if(EntityGetName(child) == "deflecting_shield_fireball_radioactive") then
					EntityKill(child)
					break
				end
			end
		end,
		excludes_relics			= "deflecting_medalion_master",
		excludes_relics_desc	= "It can't be used along with Master Deflecting Medallion relic",
		excludes_relics_tags	= "deflecting_medalion_radioactive",
		combine_exclusive		= true,
		combine_requires_relics	= "deflecting_medalion_molten,uranium_ingot",
	},
	{
		name 					= "deflecting_medalion_cold",
		ui_name					= "Deflecting Medalion of Cold",
		categories				= "player",
		effect_desc				= "An energy shield is projected around you, converting enemy \nprojectiles into iceballs and shooting them back, and \nalso freezing enemies caught by it.",
		effect_desc_short		= "A frost energy shield is projected around you.",
        precursor_types			= "magic_artifact,energy_shield",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			LoadGameEffectEntityTo( getPlayerEntity(), getCustomEffect("effect_iceball_shield").path )
		end,
		ondrop_function			= function()
			local player_children = EntityGetAllChildren(getPlayerEntity())
			if player_children == nil then return end
			for i,child in ipairs(player_children) do
				if(EntityGetName(child) == "deflecting_shield_iceball") then
					EntityKill(child)
					break
				end
			end
		end,
		excludes_relics			= "deflecting_medalion_master",
		excludes_relics_desc	= "It can't be used along with Master Deflecting Medallion relic",
		excludes_relics_tags	= "deflecting_medalion_cold",
		combine_exclusive		= true,
		combine_requires_relics	= "deflecting_medalion_frost,frost_artifact",
	},
	{
		name 					= "deflecting_medalion_metamorph",
		ui_name					= "Deflecting Medalion of Metamorphosis",
		categories				= "player",
		effect_desc				= "An energy shield is projected around you, deflecting and \nimbueing enemy projectiles with polymorphine, causing them to mutate creatures they \nhit back, and also instantly mutating them if they touch the shield.",
		effect_desc_short		= "A metamorphing energy shield is projected around you.",
        precursor_types			= "magic_artifact,energy_shield",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			LoadGameEffectEntityTo( getPlayerEntity(), getCustomEffect("effect_deflecting_mutating_shield").path )
		end,
		ondrop_function			= function()
			local player_children = EntityGetAllChildren(getPlayerEntity())
			if player_children == nil then return end
			for i,child in ipairs(player_children) do
				if(EntityGetName(child) == "deflecting_shield_mutating") then
					EntityKill(child)
					break
				end
			end
		end,
		excludes_relics			= "deflecting_medalion_master",
		excludes_relics_desc	= "It can't be used along with Master Deflecting Medallion relic",
		excludes_relics_tags	= "deflecting_medalion_metamorph",
		combine_exclusive		= true,
		combine_requires_relics	= "deflecting_medalion,poly_gem",
	},
	{
		name 					= "gravity_scepter_blue",
		ui_name					= "Blue Gravity Scepter",
		categories				= "player",
		effect_desc				= "Blue energy spheres that target enemies and shoot thunder orbs orbit around you.",
		effect_desc_short		= "Blue energy spheres that target enemies and shoot thunder orbs orbit around you.",
        precursor_types			= "gravity_stone,magic_artifact",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		ondrop_function			= function()
			detachRotatingShieldfromEntity(getPlayerEntity(), "rotating_shield_projectile_target_thunder")
		end,
		overtime_function		= function()
			if(isRelicOwned("gravity_scepter_blue") >= 1) then
				if(not entityHasRotatingShield(getPlayerEntity(), "rotating_shield_projectile_target_thunder")) then -- this makes sure the orbiting projectiles are loaded again if the player is polymorphed
					local start_id = EntityLoad("mods/ArcaneReliquary/files/entity_definitions/custom_projectiles/rotating_shield_start.xml")
					setInternalVariableValue(start_id, "host_entity_id", "value_int", getPlayerEntity())
					setInternalVariableValue(start_id, "rotating_projectile_definition", "value_string", "mods/ArcaneReliquary/files/entity_definitions/custom_projectiles/rotating_shield_target_thunder_part.xml")
					setInternalVariableValue(start_id, "rotating_projectiles_amount", "value_int", 4)
					setInternalVariableValue(start_id, "radius", "value_int", 22)
					setInternalVariableValue(start_id, "rotation_speed", "value_int", 2)
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "gravity_stone_blue,tesla_coil",
	},
	{
		name 					= "gravity_scepter_yellow",
		ui_name					= "Yellow Gravity Scepter",
		categories				= "player",
		effect_desc				= "Yellow energy spheres that target enemies and shoot explosive orbs in \nretaliation if you take damage orbit around you.",
		effect_desc_short		= "Yellow energy spheres that target enemies and shoot explosive orbs orbit around you.",
        precursor_types			= "gravity_stone,magic_artifact",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		ondrop_function			= function()
			detachRotatingShieldfromEntity(getPlayerEntity(), "rotating_shield_projectile_target_explosive")
		end,
		overtime_function		= function()
			if(isRelicOwned("gravity_scepter_yellow") >= 1) then
				if(not entityHasRotatingShield(getPlayerEntity(), "rotating_shield_projectile_target_explosive")) then -- this makes sure the orbiting projectiles are loaded again if the player is polymorphed
					local start_id = EntityLoad("mods/ArcaneReliquary/files/entity_definitions/custom_projectiles/rotating_shield_start.xml")
					setInternalVariableValue(start_id, "host_entity_id", "value_int", getPlayerEntity())
					setInternalVariableValue(start_id, "rotating_projectile_definition", "value_string", "mods/ArcaneReliquary/files/entity_definitions/custom_projectiles/rotating_shield_target_explosive_part.xml")
					setInternalVariableValue(start_id, "rotating_projectiles_amount", "value_int", 4)
					setInternalVariableValue(start_id, "radius", "value_int", 22)
					setInternalVariableValue(start_id, "rotation_speed", "value_int", 2)
				end
			end
		end,
		ondamaged_function		= function(damage, message, entity_thats_responsible, is_fatal, projectile_thats_responsible )
			if(isRelicOwned("gravity_scepter_yellow") >= 1) then
				if(message == "$damage_projectile" or message == "$damage_melee" or message == "$damage_explosion") then
					local entity_children = EntityGetAllChildren(GetUpdatedEntityID())
					if entity_children ~= nil then
						for i,child_id in ipairs(entity_children) do
							if(EntityGetName(child_id) == "rotating_shield_projectile_target_explosive") then
								setInternalVariableValue(child_id, "can_shoot", "value_bool", true)
							end
						end
					end
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "gravity_stone_blue,scarab_shell",
	},
	{
		name 					= "gravity_scepter_orange",
		ui_name					= "Orange Gravity Scepter",
		categories				= "player",
		effect_desc				= "Orange energy spheres that target enemies and shoot solar orbs orbit around you.",
		effect_desc_short		= "Orange energy spheres that target enemies and shoot solar orbs orbit around you.",
        precursor_types			= "gravity_stone,magic_artifact",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		ondrop_function			= function()
			detachRotatingShieldfromEntity(getPlayerEntity(), "rotating_shield_projectile_target_solar")
		end,
		overtime_function		= function()
			if(isRelicOwned("gravity_scepter_orange") >= 1) then
				if(not entityHasRotatingShield(getPlayerEntity(), "rotating_shield_projectile_target_solar")) then -- this makes sure the orbiting projectiles are loaded again if the player is polymorphed
					local start_id = EntityLoad("mods/ArcaneReliquary/files/entity_definitions/custom_projectiles/rotating_shield_start.xml")
					setInternalVariableValue(start_id, "host_entity_id", "value_int", getPlayerEntity())
					setInternalVariableValue(start_id, "rotating_projectile_definition", "value_string", "mods/ArcaneReliquary/files/entity_definitions/custom_projectiles/rotating_shield_target_solar_part.xml")
					setInternalVariableValue(start_id, "rotating_projectiles_amount", "value_int", 4)
					setInternalVariableValue(start_id, "radius", "value_int", 22)
					setInternalVariableValue(start_id, "rotation_speed", "value_int", 2)
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "gravity_stone_blue,ember_stone",
	},
	{
		name 					= "gravity_scepter_cyan",
		ui_name					= "Cyan Gravity Scepter",
		categories				= "player",
		effect_desc				= "Cyan energy spheres that target enemies and shoot blizzard orbs orbit around you.",
		effect_desc_short		= "Cyan energy spheres that target enemies and shoot blizzard orbs orbit around you.",
        precursor_types			= "gravity_stone,magic_artifact",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		ondrop_function			= function()
			detachRotatingShieldfromEntity(getPlayerEntity(), "rotating_shield_projectile_target_blizzard")
		end,
		overtime_function		= function()
			if(isRelicOwned("gravity_scepter_cyan") >= 1) then
				if(not entityHasRotatingShield(getPlayerEntity(), "rotating_shield_projectile_target_blizzard")) then -- this makes sure the orbiting projectiles are loaded again if the player is polymorphed
					local start_id = EntityLoad("mods/ArcaneReliquary/files/entity_definitions/custom_projectiles/rotating_shield_start.xml")
					setInternalVariableValue(start_id, "host_entity_id", "value_int", getPlayerEntity())
					setInternalVariableValue(start_id, "rotating_projectile_definition", "value_string", "mods/ArcaneReliquary/files/entity_definitions/custom_projectiles/rotating_shield_target_blizzard_part.xml")
					setInternalVariableValue(start_id, "rotating_projectiles_amount", "value_int", 4)
					setInternalVariableValue(start_id, "radius", "value_int", 22)
					setInternalVariableValue(start_id, "rotation_speed", "value_int", 2)
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "gravity_stone_blue,frost_artifact",
	},
	{
		name 					= "gravity_scepter_red",
		ui_name					= "Red Gravity Scepter",
		categories				= "player",
		effect_desc				= "Red energy spheres that target enemies and shoot corrosive blood orbs orbit around you.",
		effect_desc_short		= "Red energy spheres that target enemies and shoot corrosive blood orbs orbit around you.",
        precursor_types			= "gravity_stone,magic_artifact",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		ondrop_function			= function()
			detachRotatingShieldfromEntity(getPlayerEntity(), "rotating_shield_projectile_target_blood")
		end,
		overtime_function		= function()
			if(isRelicOwned("gravity_scepter_red") >= 1) then
				if(not entityHasRotatingShield(getPlayerEntity(), "rotating_shield_projectile_target_blood")) then -- this makes sure the orbiting projectiles are loaded again if the player is polymorphed
					local start_id = EntityLoad("mods/ArcaneReliquary/files/entity_definitions/custom_projectiles/rotating_shield_start.xml")
					setInternalVariableValue(start_id, "host_entity_id", "value_int", getPlayerEntity())
					setInternalVariableValue(start_id, "rotating_projectile_definition", "value_string", "mods/ArcaneReliquary/files/entity_definitions/custom_projectiles/rotating_shield_target_blood_part.xml")
					setInternalVariableValue(start_id, "rotating_projectiles_amount", "value_int", 4)
					setInternalVariableValue(start_id, "radius", "value_int", 22)
					setInternalVariableValue(start_id, "rotation_speed", "value_int", 2)
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "gravity_stone_blue,corrosive_blood_gem",
	},
	{
		name 					= "gravity_scepter_green",
		ui_name					= "Green Gravity Scepter",
		categories				= "player",
		effect_desc				= "Green energy spheres that target enemies and shoot acid orbs orbit around you.",
		effect_desc_short		= "Green energy spheres that target enemies and shoot acid orbs orbit around you.",
        precursor_types			= "gravity_stone,magic_artifact",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		ondrop_function			= function()
			detachRotatingShieldfromEntity(getPlayerEntity(), "rotating_shield_projectile_target_acid")
		end,
		overtime_function		= function()
			if(isRelicOwned("gravity_scepter_green") >= 1) then
				if(not entityHasRotatingShield(getPlayerEntity(), "rotating_shield_projectile_target_acid")) then -- this makes sure the orbiting projectiles are loaded again if the player is polymorphed
					local start_id = EntityLoad("mods/ArcaneReliquary/files/entity_definitions/custom_projectiles/rotating_shield_start.xml")
					setInternalVariableValue(start_id, "host_entity_id", "value_int", getPlayerEntity())
					setInternalVariableValue(start_id, "rotating_projectile_definition", "value_string", "mods/ArcaneReliquary/files/entity_definitions/custom_projectiles/rotating_shield_target_acid_part.xml")
					setInternalVariableValue(start_id, "rotating_projectiles_amount", "value_int", 4)
					setInternalVariableValue(start_id, "radius", "value_int", 22)
					setInternalVariableValue(start_id, "rotation_speed", "value_int", 2)
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "gravity_stone_blue,acid_gem",
	},
	{
		name 					= "gravity_scepter_purple",
		ui_name					= "Purple Gravity Scepter",
		categories				= "player",
		effect_desc				= "Purple energy spheres that target enemies and shoot nuclear orbs in \nretaliation if you take damage orbit around you.",
		effect_desc_short		= "Purple energy spheres that target enemies and shoot nuclear orbs orbit around you.",
        precursor_types			= "gravity_stone,magic_artifact",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		ondrop_function			= function()
			detachRotatingShieldfromEntity(getPlayerEntity(), "rotating_shield_projectile_target_nuclear")
		end,
		overtime_function		= function()
			if(isRelicOwned("gravity_scepter_purple") >= 1) then
				if(not entityHasRotatingShield(getPlayerEntity(), "rotating_shield_projectile_target_nuclear")) then -- this makes sure the orbiting projectiles are loaded again if the player is polymorphed
					local start_id = EntityLoad("mods/ArcaneReliquary/files/entity_definitions/custom_projectiles/rotating_shield_start.xml")
					setInternalVariableValue(start_id, "host_entity_id", "value_int", getPlayerEntity())
					setInternalVariableValue(start_id, "rotating_projectile_definition", "value_string", "mods/ArcaneReliquary/files/entity_definitions/custom_projectiles/rotating_shield_target_nuclear_part.xml")
					setInternalVariableValue(start_id, "rotating_projectiles_amount", "value_int", 4)
					setInternalVariableValue(start_id, "radius", "value_int", 22)
					setInternalVariableValue(start_id, "rotation_speed", "value_int", 2)
				end
			end
		end,
		ondamaged_function		= function(damage, message, entity_thats_responsible, is_fatal, projectile_thats_responsible )
			if(isRelicOwned("gravity_scepter_purple") >= 1) then
				if(message == "$damage_projectile" or message == "$damage_melee" or message == "$damage_explosion") then
					local entity_children = EntityGetAllChildren(GetUpdatedEntityID())
					if entity_children ~= nil then
						for i,child_id in ipairs(entity_children) do
							if(EntityGetName(child_id) == "rotating_shield_projectile_target_nuclear") then
								setInternalVariableValue(child_id, "can_shoot", "value_bool", true)
							end
						end
					end
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "gravity_stone_blue,poison_gem",
	},
	{
		name 					= "gravity_scepter_oceanic",
		ui_name					= "Oceanic Gravity Scepter",
		categories				= "player",
		effect_desc				= "Cyan energy spheres that target enemies and shoot water orbs orbit around you.",
		effect_desc_short		= "Cyan energy spheres that target enemies and shoot water orbs orbit around you.",
        precursor_types			= "gravity_stone,magic_artifact",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		ondrop_function			= function()
			detachRotatingShieldfromEntity(getPlayerEntity(), "rotating_shield_projectile_target_water")
		end,
		overtime_function		= function()
			if(isRelicOwned("gravity_scepter_oceanic") >= 1) then
				if(not entityHasRotatingShield(getPlayerEntity(), "rotating_shield_projectile_target_water")) then -- this makes sure the orbiting projectiles are loaded again if the player is polymorphed
					local start_id = EntityLoad("mods/ArcaneReliquary/files/entity_definitions/custom_projectiles/rotating_shield_start.xml")
					setInternalVariableValue(start_id, "host_entity_id", "value_int", getPlayerEntity())
					setInternalVariableValue(start_id, "rotating_projectile_definition", "value_string", "mods/ArcaneReliquary/files/entity_definitions/custom_projectiles/rotating_shield_target_water_part.xml")
					setInternalVariableValue(start_id, "rotating_projectiles_amount", "value_int", 4)
					setInternalVariableValue(start_id, "radius", "value_int", 22)
					setInternalVariableValue(start_id, "rotation_speed", "value_int", 2)
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "gravity_stone_blue,water_gem",
	},
	{
		name 					= "gravity_scepter_lime",
		ui_name					= "Lime Gravity Scepter",
		categories				= "player",
		effect_desc				= "Lime energy spheres that target enemies and shoot radioactive orbs orbit around you.",
		effect_desc_short		= "Lime energy spheres that target enemies and shoot radioactive orbs orbit around you.",
        precursor_types			= "gravity_stone,magic_artifact",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		ondrop_function			= function()
			detachRotatingShieldfromEntity(getPlayerEntity(), "rotating_shield_projectile_target_radioactive")
		end,
		overtime_function		= function()
			if(isRelicOwned("gravity_scepter_lime") >= 1) then
				if(not entityHasRotatingShield(getPlayerEntity(), "rotating_shield_projectile_target_radioactive")) then -- this makes sure the orbiting projectiles are loaded again if the player is polymorphed
					local start_id = EntityLoad("mods/ArcaneReliquary/files/entity_definitions/custom_projectiles/rotating_shield_start.xml")
					setInternalVariableValue(start_id, "host_entity_id", "value_int", getPlayerEntity())
					setInternalVariableValue(start_id, "rotating_projectile_definition", "value_string", "mods/ArcaneReliquary/files/entity_definitions/custom_projectiles/rotating_shield_target_radioactive_part.xml")
					setInternalVariableValue(start_id, "rotating_projectiles_amount", "value_int", 4)
					setInternalVariableValue(start_id, "radius", "value_int", 22)
					setInternalVariableValue(start_id, "rotation_speed", "value_int", 2)
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "gravity_stone_blue,uranium_ingot",
	},
	{
		name 					= "gravity_scepter_white",
		ui_name					= "White Gravity Scepter",
		categories				= "player",
		effect_desc				= "White energy spheres that target enemies and shoot melee orbs orbit around you.",
		effect_desc_short		= "White energy spheres that target enemies and shoot melee orbs orbit around you.",
        precursor_types			= "gravity_stone,magic_artifact",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		ondrop_function			= function()
			detachRotatingShieldfromEntity(getPlayerEntity(), "rotating_shield_projectile_target_melee")
		end,
		overtime_function		= function()
			if(isRelicOwned("gravity_scepter_white") >= 1) then
				if(not entityHasRotatingShield(getPlayerEntity(), "rotating_shield_projectile_target_melee")) then -- this makes sure the orbiting projectiles are loaded again if the player is polymorphed
					local start_id = EntityLoad("mods/ArcaneReliquary/files/entity_definitions/custom_projectiles/rotating_shield_start.xml")
					setInternalVariableValue(start_id, "host_entity_id", "value_int", getPlayerEntity())
					setInternalVariableValue(start_id, "rotating_projectile_definition", "value_string", "mods/ArcaneReliquary/files/entity_definitions/custom_projectiles/rotating_shield_target_melee_part.xml")
					setInternalVariableValue(start_id, "rotating_projectiles_amount", "value_int", 4)
					setInternalVariableValue(start_id, "radius", "value_int", 22)
					setInternalVariableValue(start_id, "rotation_speed", "value_int", 2)
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "gravity_stone_blue,emerald_gauntlet",
	},
	{
		name 					= "stone_hand_white_star",
		ui_name					= "Stone Hand with White Band",
		categories				= "companions",
		effect_desc				= "A pair of stone hands that shoot white magic stars \naids you. Requires an Elemental Band.",
		effect_desc_short		= "A pair of stone hands that shoot white magic stars aids you.",
        precursor_types			= "companion",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			
			attachMonkHandstoEntity(getPlayerEntity(), "monk_hand_white_star", "monk_arm_fx_white_star", "mortal")
		end,
		ondrop_function			= function()
			
			detachMonkHandsfromEntity(getPlayerEntity(), "monk_hand_white_star")
		end,
		requires_relic			= "elemental_band",
	},
	{
		name 					= "mystic_mandala_defense",
		ui_name					= "Mystic Mandala (Defense)",
		categories				= "player",
		effect_desc				= "Green energy spheres that deflect enemy projectiles \nand multiply their base damage orbit around you.",
		effect_desc_short		= "Green energy spheres that deflect enemy projectiles orbit around you.",
        precursor_types			= "gravity_stone,magic_artifact",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		ondrop_function			= function()
			detachRotatingShieldfromEntity(getPlayerEntity(), "rotating_shield_projectile_deflect")
		end,
		overtime_function		= function()
			if(isRelicOwned("mystic_mandala_defense") >= 1) then
				if(not entityHasRotatingShield(getPlayerEntity(), "rotating_shield_projectile_deflect")) then -- this makes sure the orbiting projectiles are loaded again if the player is polymorphed
					local start_id = EntityLoad("mods/ArcaneReliquary/files/entity_definitions/custom_projectiles/rotating_shield_start.xml")
					setInternalVariableValue(start_id, "host_entity_id", "value_int", getPlayerEntity())
					setInternalVariableValue(start_id, "rotating_projectile_definition", "value_string", "mods/ArcaneReliquary/files/entity_definitions/custom_projectiles/rotating_shield_deflect_part.xml")
					setInternalVariableValue(start_id, "rotating_projectiles_amount", "value_int", 4)
					setInternalVariableValue(start_id, "radius", "value_int", 22)
					setInternalVariableValue(start_id, "rotation_speed", "value_int", 2)
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "gravity_stone_teal,gravity_stone_red,gravity_stone_grey",
	},
	{
		name 					= "mystic_mandala_elements",
		ui_name					= "Mystic Mandala (Elements)",
		categories				= "player",
		effect_desc				= "Elemental energy spheres that target enemies and shoot elemental \norbs orbit around you.",
		effect_desc_short		= "Elemental energy spheres that shoot elemental orbs orbit around you.",
        precursor_types			= "gravity_stone,magic_artifact",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		ondrop_function			= function()
			detachRotatingShieldfromEntity(getPlayerEntity(), "rotating_shield_projectile_target_elemental_acid")
			detachRotatingShieldfromEntity(getPlayerEntity(), "rotating_shield_projectile_target_elemental_lava")
			detachRotatingShieldfromEntity(getPlayerEntity(), "rotating_shield_projectile_target_elemental_poison")
			detachRotatingShieldfromEntity(getPlayerEntity(), "rotating_shield_projectile_target_elemental_poly")
			detachRotatingShieldfromEntity(getPlayerEntity(), "rotating_shield_projectile_target_elemental_blood")
			detachRotatingShieldfromEntity(getPlayerEntity(), "rotating_shield_projectile_target_elemental_water")
			detachRotatingShieldfromEntity(getPlayerEntity(), "rotating_shield_projectile_target_elemental_urine")
			detachRotatingShieldfromEntity(getPlayerEntity(), "rotating_shield_projectile_target_elemental_freezing_liquid")
		end,
		overtime_function		= function()
			if(isRelicOwned("mystic_mandala_elements") >= 1) then
				if(not entityHasRotatingShield(getPlayerEntity(), "rotating_shield_projectile_target_elemental_acid") and
					not entityHasRotatingShield(getPlayerEntity(), "rotating_shield_projectile_target_elemental_lava") and
					not entityHasRotatingShield(getPlayerEntity(), "rotating_shield_projectile_target_elemental_poison") and
					not entityHasRotatingShield(getPlayerEntity(), "rotating_shield_projectile_target_elemental_poly") and
					not entityHasRotatingShield(getPlayerEntity(), "rotating_shield_projectile_target_elemental_blood") and
					not entityHasRotatingShield(getPlayerEntity(), "rotating_shield_projectile_target_elemental_freezing_liquid") and
					not entityHasRotatingShield(getPlayerEntity(), "rotating_shield_projectile_target_elemental_urine") and
					not entityHasRotatingShield(getPlayerEntity(), "rotating_shield_projectile_target_elemental_water")) then -- this makes sure the orbiting projectiles are loaded again if the player is polymorphed
					local start_id = EntityLoad("mods/ArcaneReliquary/files/entity_definitions/custom_projectiles/rotating_shield_start_various.xml")
					setInternalVariableValue(start_id, "host_entity_id", "value_int", getPlayerEntity())
					setInternalVariableValue(start_id, "rotating_projectiles_list", "value_string", "rotating_shield_target_elemental_urine_part,rotating_shield_target_elemental_freezing_liquid_part,rotating_shield_target_elemental_water_part,rotating_shield_target_elemental_acid_part,rotating_shield_target_elemental_lava_part,rotating_shield_target_elemental_poison_part,rotating_shield_target_elemental_blood_part,rotating_shield_target_elemental_poly_part")
					setInternalVariableValue(start_id, "radius", "value_int", 27)
					setInternalVariableValue(start_id, "rotation_speed", "value_int", 5)
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "gravity_stone_water,gravity_stone_corrosive_blood,gravity_stone_green,gravity_stone_purple,gravity_stone_orange",
	},
	{
		name 					= "gravity_scepter_pink",
		ui_name					= "Pink Gravity Scepter",
		categories				= "player",
		effect_desc				= "Pink energy spheres that target enemies and shoot polymorphine orbs orbit around you.",
		effect_desc_short		= "Pink energy spheres that target enemies and shoot polymorphine orbs orbit around you.",
        precursor_types			= "gravity_stone,magic_artifact",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		ondrop_function			= function()
			detachRotatingShieldfromEntity(getPlayerEntity(), "rotating_shield_projectile_target_poly")
		end,
		overtime_function		= function()
			if(isRelicOwned("gravity_scepter_pink") >= 1) then
				if(not entityHasRotatingShield(getPlayerEntity(), "rotating_shield_projectile_target_poly")) then -- this makes sure the orbiting projectiles are loaded again if the player is polymorphed
					local start_id = EntityLoad("mods/ArcaneReliquary/files/entity_definitions/custom_projectiles/rotating_shield_start.xml")
					setInternalVariableValue(start_id, "host_entity_id", "value_int", getPlayerEntity())
					setInternalVariableValue(start_id, "rotating_projectile_definition", "value_string", "mods/ArcaneReliquary/files/entity_definitions/custom_projectiles/rotating_shield_target_poly_part.xml")
					setInternalVariableValue(start_id, "rotating_projectiles_amount", "value_int", 4)
					setInternalVariableValue(start_id, "radius", "value_int", 22)
					setInternalVariableValue(start_id, "rotation_speed", "value_int", 2)
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "gravity_stone_blue,poly_gem",
	},
	
	{
		name 					= "mystic_mandala_transmutation",
		ui_name					= "Mystic Mandala (Transmutation)",
		categories				= "player",
		effect_desc				= "Yellow energy spheres capable of dragging enemies and \neventually transmutating them to random materials orbit around you.",
		effect_desc_short		= "Yellow energy spheres that drag enemies and transmute them to random materials orbit around you.",
        precursor_types			= "gravity_stone,magic_artifact",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		ondrop_function			= function()
			detachRotatingShieldfromEntity(getPlayerEntity(), "rotating_shield_projectile_drag_transmute")
		end,
		overtime_function		= function()
			if(isRelicOwned("mystic_mandala_transmutation") >= 1) then
				if(not entityHasRotatingShield(getPlayerEntity(), "rotating_shield_projectile_drag_transmute")) then -- this makes sure the orbiting projectiles are loaded again if the player is polymorphed
					local start_id = EntityLoad("mods/ArcaneReliquary/files/entity_definitions/custom_projectiles/rotating_shield_start.xml")
					setInternalVariableValue(start_id, "host_entity_id", "value_int", getPlayerEntity())
					setInternalVariableValue(start_id, "rotating_projectile_definition", "value_string", "mods/ArcaneReliquary/files/entity_definitions/custom_projectiles/rotating_shield_drag_transmute_part.xml")
					setInternalVariableValue(start_id, "rotating_projectiles_amount", "value_int", 2)
					setInternalVariableValue(start_id, "radius", "value_int", 32)
					setInternalVariableValue(start_id, "rotation_speed", "value_int", 10)
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "mystic_mandala_elements,gravity_stone_yellow",
	},
	{
		name 					= "mystic_mandala_full",
		ui_name					= "Mystic Mandala (Full)",
		categories				= "player",
		effect_desc				= "Green energy spheres that target enemies and shoot powerful \ndragging orbs that explode on impact and can eventually transmute enemies into random materials orbit around you.",
		effect_desc_short		= "Green energy spheres that target enemies and shoot powerful elemental orbs orbit around you.",
        precursor_types			= "gravity_stone,magic_artifact",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		ondrop_function			= function()
			detachRotatingShieldfromEntity(getPlayerEntity(), "rotating_shield_projectile_target_drag_explosive_elemental")
		end,
		overtime_function		= function()
			if(isRelicOwned("mystic_mandala_full") >= 1) then
				if(not entityHasRotatingShield(getPlayerEntity(), "rotating_shield_projectile_target_drag_explosive_elemental")) then -- this makes sure the orbiting projectiles are loaded again if the player is polymorphed
					local start_id = EntityLoad("mods/ArcaneReliquary/files/entity_definitions/custom_projectiles/rotating_shield_start.xml")
					setInternalVariableValue(start_id, "host_entity_id", "value_int", getPlayerEntity())
					setInternalVariableValue(start_id, "rotating_projectile_definition", "value_string", "mods/ArcaneReliquary/files/entity_definitions/custom_projectiles/rotating_shield_target_drag_explosive_elemental_part.xml")
					setInternalVariableValue(start_id, "rotating_projectiles_amount", "value_int", 2)
					setInternalVariableValue(start_id, "radius", "value_int", 22)
					setInternalVariableValue(start_id, "rotation_speed", "value_int", 2)
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "mystic_mandala_elements,mystic_mandala_defense,mystic_mandala_transmutation",
	},
	{
		name 					= "microrocket_droid",
		ui_name					= "Microrocket Droid",
		categories				= "companions",
		effect_desc				= "A small robot that shoots homing microrockets joins you.",
		effect_desc_short		= "A microrocket shooting companion.",
        precursor_types			= "companion",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			attachCompaniontoEntity(getPlayerEntity(), "microrocket_droid", "mortal")
		end,
		ondrop_function			= function()
			detachCompanionfromEntity(getPlayerEntity(), "microrocket_droid")
		end,
		requires_relic			= "alkaline_battery",
		combine_exclusive		= true,
		combine_requires_relics	= "rocket_droid,targeting_device",
	},
	{
		name 					= "interceptor_droid",
		ui_name					= "Interceptor Droid",
		categories				= "companions",
		effect_desc				= "A small robot that shoots intercepting rockets that target enemy projectiles joins you.",
		effect_desc_short		= "An intercepting rocket shooting companion.",
        precursor_types			= "companion",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			attachCompaniontoEntity(getPlayerEntity(), "interceptor_droid", "projectile")
		end,
		ondrop_function			= function()
			detachCompanionfromEntity(getPlayerEntity(), "interceptor_droid")
		end,
		requires_relic			= "alkaline_battery",
	},
	{
		name 					= "ancient_root",
		ui_name					= "Ancient Root",
		categories				= "projectiles",
		effect_desc				= "Your spells destroy and scatter enemy projectiles on the opposite angle from where they came.",
		effect_desc_short		= "Your spells destroy and scatter enemy projectiles.",
        precursor_types			= "gadget",
		upgrade_effect_desc		= "For each upgrade level, collision radius is increased.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("ancient_root") >= 1) then
				if(not isOrbitingProjectile(projectile_entity_id)) then
					addNewSimplifiedInternalVariable( projectile_entity_id, "relic_effect_projectile")
					attachDragCollisionHandlertoProjectile(projectile_entity_id, getCollisionHandler("root_effect").path, false, 10 + (1 * getRelicLevel("ancient_root")), "projectile", false)
				end
			end
		end,
		onpickup_function		= function()
			addNewSimplifiedInternalVariable(getPlayerEntity(), "root_effect")
		end,
		ondrop_function			= function()
			removeSimplifiedInternalVariable(getPlayerEntity(), "root_effect")
		end,
	},
	{
		name 					= "eye_of_vigour",
		ui_name					= "Eye of Vigour",
		categories				= "projectiles",
		effect_desc				= "Your spells destroy enemy projectiles and convert their base damage into health for your character.",
		effect_desc_short		= "Your spells destroy and scatter enemy projectiles.",
        precursor_types			= "gadget",
		upgrade_effect_desc		= "For each upgrade level, collision radius is increased.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("eye_of_vigour") >= 1) then
				if(not isOrbitingProjectile(projectile_entity_id)) then
					addNewSimplifiedInternalVariable( projectile_entity_id, "relic_effect_projectile")
					attachDragCollisionHandlertoProjectile(projectile_entity_id, getCollisionHandler("eye_of_vigour_effect").path, false, 10 + (1 * getRelicLevel("eye_of_vigour")), "projectile", false)
				end
			end
		end,
		onpickup_function		= function()
			addNewSimplifiedInternalVariable(getPlayerEntity(), "eye_of_vigour_effect")
		end,
		ondrop_function			= function()
			removeSimplifiedInternalVariable(getPlayerEntity(), "eye_of_vigour_effect")
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "ancient_root,compass,frying_pan,blood_gem",
	},
	{
		name 					= "deflecting_medalion_master",
		ui_name					= "Master Deflecting Medalion",
		categories				= "player",
		effect_desc				= "An energy shield that deflects enemy projectiles and causes enemies hit by \nthem to be instantly converted to gold. When a projectile is deflected, you are also healed for the \nbase damage of the projetile.",
		effect_desc_short		= "The ultimate deflecting shield, heals you and makes you rich real quick.",
        precursor_types			= "magic_artifact,energy_shield",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			LoadGameEffectEntityTo( getPlayerEntity(), getCustomEffect("effect_deflecting_master_shield").path )
		end,
		ondrop_function			= function()
			local player_children = EntityGetAllChildren(getPlayerEntity())
			if player_children == nil then return end
			for i,child in ipairs(player_children) do
				if(EntityGetName(child) == "deflecting_shield_master") then
					EntityKill(child)
					break
				end
			end
		end,
		excludes_relics			= "deflecting_medalion,deflecting_medalion_charged,deflecting_medalion_molten,deflecting_medalion_frost,deflecting_medalion_transmuting,deflecting_medalion_radioactive,deflecting_medalion_thunder_red,deflecting_medalion_thunder,deflecting_medalion_flame,deflecting_medalion_cold,deflecting_medalion_metamorph",
		excludes_relics_desc	= "It can't be used along with other basic Deflecting Medallion relics",
		excludes_relics_tags	= "deflecting_medalion,deflecting_medalion_charged,deflecting_medalion_molten,deflecting_medalion_frost,deflecting_medalion_transmuting,deflecting_medalion_radioactive,deflecting_medalion_thunder_red,deflecting_medalion_thunder,deflecting_medalion_flame,deflecting_medalion_cold,deflecting_medalion_metamorph",
		combine_exclusive		= true,
		combine_requires_relics	= "deflecting_medalion_metamorph,deflecting_medalion_cold,deflecting_medalion_radioactive,deflecting_medalion_flame,deflecting_medalion_thunder_red,deflecting_medalion_thunder",
	},
	{
		name 					= "wendigos_medalion",
		ui_name					= "Wendigo's Medalion",
		categories				= "projectiles",
		effect_desc				= "Most enemies have a 20% chance of being instantly frozen solid when hit by your spells.",
		effect_desc_short		= "Your spells freeze enemies solid.",
        precursor_types			= "magic_artifact",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("wendigos_medalion") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 20 + (5 * getRelicLevel("wendigos_medalion"))
				if(chance >= 1 and chance <= odds) then
					attachLoadEffectOntoEntitiestoProjectile(projectile_entity_id, getCustomEffect("effect_freeze_solid").path, "diamond")
				end
			end
		end,
	},
	{
		name 					= "stone_hand_blood",
		ui_name					= "Bloody Stone Hand",
		categories				= "companions",
		effect_desc				= "A pair of stone hands that shoot bleeding magic stars \naids you. Requires an Elemental Band.",
		effect_desc_short		= "A pair of stone hands that shoot bleeding magic stars aids you.",
        precursor_types			= "companion",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			
			attachMonkHandstoEntity(getPlayerEntity(), "monk_hand_blood", "monk_arm_fx_blood", "mortal")
		end,
		ondrop_function			= function()
			
			detachMonkHandsfromEntity(getPlayerEntity(), "monk_hand_blood")
		end,
		requires_relic			= "elemental_band",
		combine_exclusive		= true,
		combine_requires_relics	= "stone_hand_white_star,ancient_dagger",
	},
	{
		name 					= "stone_hand_slap",
		ui_name					= "Slappy Stone Hand",
		categories				= "companions",
		effect_desc				= "A pair of stone hands that shoot slapping magic stars \naids you. Requires an Elemental Band.",
		effect_desc_short		= "A pair of stone hands that shoot slapping magic stars aids you.",
        precursor_types			= "companion",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			
			attachMonkHandstoEntity(getPlayerEntity(), "monk_hand_slap", "monk_arm_fx_slap", "mortal")
		end,
		ondrop_function			= function()
			
			detachMonkHandsfromEntity(getPlayerEntity(), "monk_hand_slap")
		end,
		requires_relic			= "elemental_band",
		combine_exclusive		= true,
		combine_requires_relics	= "stone_hand_white_star,slapper_stool",
	},
	{
		name 					= "stone_hand_wind",
		ui_name					= "Wind Stone Hand",
		categories				= "companions",
		effect_desc				= "A pair of stone hands that shoot wind magic stars \naids you. Requires an Elemental Band.",
		effect_desc_short		= "A pair of stone hands that shoot wind magic stars aids you.",
        precursor_types			= "companion",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			
			attachMonkHandstoEntity(getPlayerEntity(), "monk_hand_wind", "monk_arm_fx_wind", "mortal")
		end,
		ondrop_function			= function()
			
			detachMonkHandsfromEntity(getPlayerEntity(), "monk_hand_wind")
		end,
		requires_relic			= "elemental_band",
		combine_exclusive		= true,
		combine_requires_relics	= "stone_hand_white_star,pinwheel_colored",
	},
	{
		name 					= "stone_hand_high_explosive",
		ui_name					= "High Explosive Stone Hand",
		categories				= "companions",
		effect_desc				= "A pair of stone hands that shoot high explosive magic stars \naids you. Requires an Elemental Band.",
		effect_desc_short		= "A pair of stone hands that shoot high explosive magic stars aids you.",
        precursor_types			= "companion",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			
			attachMonkHandstoEntity(getPlayerEntity(), "monk_hand_high_explosive", "monk_arm_fx_high_explosive", "mortal")
		end,
		ondrop_function			= function()
			
			detachMonkHandsfromEntity(getPlayerEntity(), "monk_hand_high_explosive")
		end,
		requires_relic			= "elemental_band",
		combine_exclusive		= true,
		combine_requires_relics	= "stone_hand_white_star,explosive_mana_crystal",
	},
	{
		name 					= "kraken_limb",
		ui_name					= "Kraken Limb",
		categories				= "projectiles",
		effect_desc				= "Spells you cast have a 50% chance of launching tentacles \nthat deal melee damage towards nearby enemies.",
		effect_desc_short		= "Your spells launch tentacles at enemies.",
        precursor_types			= "magic_artifact",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("kraken_limb") >= 1) then
				if(not isPhysicsProjectile(projectile_entity_id)) then
					SetRandomSeed( GameGetFrameNum(), math.random(1,50))
					local chance = Random(1,100)
					local odds = 50 + (5 * getRelicLevel("kraken_limb"))
					if(chance >= 1 and chance <= odds) then
						addBuffVisualIndicator_Projectile(projectile_entity_id, "spark_purple_bright")
						addNewInternalVariable(projectile_entity_id, "projectile_definition", "value_string", "mods/ArcaneReliquary/files/entity_definitions/custom_projectiles/tentacle_custom.xml")
						addNewInternalVariable(projectile_entity_id, "detection_range", "value_int", 100)
						addNewInternalVariable(projectile_entity_id, "projectile_velocity", "value_int", 350)
						addNewInternalVariable(projectile_entity_id, "target_tag", "value_string", "enemy")
						addNewInternalVariable(projectile_entity_id, "max_projectiles_shot", "value_int", 15)
						addNewLuaCallback(projectile_entity_id, "overtime", "mods/ArcaneReliquary/files/scripts/projectiles/target_and_shoot_projectile.lua", 10, false, "target_and_shoot_projectile")
					end
				end
			end
		end,
	},
	{
		name 					= "kraken_eye",
		ui_name					= "Kraken Eye",
		categories				= "player",
		effect_desc				= "When you're wet, your body spontaneously launches tentacles, that \ndeal melee damage, towards enemies should they come too close.",
		effect_desc_short		= "You launch tentacles at enemies if they get close when you're wet.",
        precursor_types			= "magic_artifact",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			LoadGameEffectEntityTo( getPlayerEntity(), getCustomEffect("effect_target_shoot_tentacle").path )
		end,
		ondrop_function			= function()
			local player_children = EntityGetAllChildren(getPlayerEntity())
			if player_children == nil then return end
			for i,child in ipairs(player_children) do
				if(EntityGetName(child) == "target_shoot_tentacle") then
					EntityKill(child)
					break
				end
			end
		end,
	},
	{
		name 					= "kraken_eye_blood",
		ui_name					= "Kraken Eye (Blood)",
		categories				= "player",
		effect_desc				= "When you're convered in red blood, your body spontaneously launches \ntentacles, that deal bleeding damage, towards enemies should they come too close.",
		effect_desc_short		= "You launch bleeding tentacles at enemies if they get close when you're covered in blood.",
        precursor_types			= "magic_artifact",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			LoadGameEffectEntityTo( getPlayerEntity(), getCustomEffect("effect_target_shoot_tentacle_blood").path )
		end,
		ondrop_function			= function()
			local player_children = EntityGetAllChildren(getPlayerEntity())
			if player_children == nil then return end
			for i,child in ipairs(player_children) do
				if(EntityGetName(child) == "target_shoot_tentacle_blood") then
					EntityKill(child)
					break
				end
			end
		end,
	},
	{
		name 					= "defense_probe",
		ui_name					= "Defense Probe",
		categories				= "player",
		effect_desc				= "A defense probe orbits around you, shooting intercepting laser bolts \nand destroying enemy projectiles in the vicinity.",
		effect_desc_short		= "A defense probe that targets and destroys enemy projectiles orbits around you.",
        precursor_types			= "gravity_stone",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		ondrop_function			= function()
			detachRotatingShieldfromEntity(getPlayerEntity(), "rotating_shield_projectile_defense_probe")
		end,
		overtime_function		= function()
			if(isRelicOwned("defense_probe") >= 1) then
				if(not entityHasRotatingShield(getPlayerEntity(), "rotating_shield_projectile_defense_probe")) then -- this makes sure the orbiting projectiles are loaded again if the player is polymorphed
					local start_id = EntityLoad("mods/ArcaneReliquary/files/entity_definitions/custom_projectiles/rotating_shield_start.xml")
					setInternalVariableValue(start_id, "host_entity_id", "value_int", getPlayerEntity())
					setInternalVariableValue(start_id, "rotating_projectile_definition", "value_string", "mods/ArcaneReliquary/files/entity_definitions/custom_projectiles/rotating_defense_probe.xml")
					setInternalVariableValue(start_id, "rotating_projectiles_amount", "value_int", 1)
					setInternalVariableValue(start_id, "radius", "value_int", 18)
					setInternalVariableValue(start_id, "rotation_speed", "value_int", 2)
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "security_probe,frying_pan",
	},
	{
		name 					= "meteor_fragment_nuclear",
		ui_name					= "Nuclear Meteor Fragment",
		categories				= "projectiles",
		effect_desc				= "Spells you cast in a downward angle have a 30% chance of creating \na nuclear shockwave, that damages nearby enemies, when they hit something. \nIt also synergizes with Earth Core Fragment relic effect.",
		effect_desc_short		= "Spells cast downwards create nuclear shockwaves on the ground.",
        precursor_types			= "magic_artifact",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5% and shockwave reaches longer.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("meteor_fragment_nuclear") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 30 + (5 * getRelicLevel("meteor_fragment_nuclear"))
				if(chance >= 1 and chance <= odds) then
					local projectile_vel_x, projectile_vel_y = getProjectileVelocitiesandGravity(projectile_entity_id)
					if(projectile_vel_y > 0) then
						replaceProjectileExplosionSprite(projectile_entity_id, "")
						projectileRemoveBouncing(projectile_entity_id)
						if(not projectileExplodesonImpact(projectile_entity_id)) then
							projectileSetExplosionConfig(projectile_entity_id, 1)
						end
						attachLoadEntityOnExplosiontoProjectile(projectile_entity_id, getLoadOnExplosion("shockwave_nuclear").path, "spark")
					end
				end
			end
		end,
		ongroundpound_function	= function( entity_id)
			if(isRelicOwned("meteor_fragment_nuclear") >= 1) then
				local game_effect_comp = GetGameEffectLoadTo( entity_id, "PROTECTION_ALL", true )
				if game_effect_comp ~= nil then
					ComponentSetValue( game_effect_comp, "frames", "40" )
				end
				local x, y = EntityGetTransform(entity_id)
				EntityLoad("mods/ArcaneReliquary/files/entity_definitions/load_on_explosion/shockwave_nuclear.xml", x, y)
			end
		end,
		ondrop_function			= function()
			removeInternalVariable(getPlayerEntity(), "nuclear_shockwave_length_bonus")
		end,
		overtime_function		= function()
			local nuclear_shockwave_length_bonus = getRelicLevel("meteor_fragment_nuclear")

			if(not entityHasInternalVariableValue(getPlayerEntity(), "nuclear_shockwave_length_bonus")) then
				addNewInternalVariable(getPlayerEntity(), "nuclear_shockwave_length_bonus", "value_int", nuclear_shockwave_length_bonus)
			end
			setInternalVariableValue(getPlayerEntity(), "nuclear_shockwave_length_bonus", "value_int", nuclear_shockwave_length_bonus)
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "meteor_fragment_aluminum,meteor_fragment_frozen,meteor_fragment_molten,meteor_fragment_silver",
	},
	{
		name 					= "kraky",
		ui_name					= "Kraky",
		categories				= "companions",
		effect_desc				= "Kraky, a friendly little kraken that attacks enemies with \ntentacle shots joins you. Requires a Kraken Eye.",
		effect_desc_short		= "A tentacle shooting companion.",
        precursor_types			= "companion",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			attachCompaniontoEntity(getPlayerEntity(), "kraky", "mortal")
		end,
		ondrop_function			= function()
			detachCompanionfromEntity(getPlayerEntity(), "kraky")
		end,
		requires_relic			= "kraken_eye",
	},
	{
		name 					= "kraky_blood",
		ui_name					= "Bloody Kraky",
		categories				= "companions",
		effect_desc				= "Bloody Kraky, a friendly little kraken that attacks enemies with \nbleeding tentacle shots joins you. Requires a Kraken Eye.",
		effect_desc_short		= "A bleeding tentacle shooting companion.",
        precursor_types			= "companion",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			attachCompaniontoEntity(getPlayerEntity(), "kraky_blood", "mortal")
		end,
		ondrop_function			= function()
			detachCompanionfromEntity(getPlayerEntity(), "kraky_blood")
		end,
		requires_relic			= "kraken_eye",
	},
	{
		name 					= "kraken_limb_blood",
		ui_name					= "Bloody Kraken Limb",
		categories				= "projectiles",
		effect_desc				= "Spells you cast have a 50% chance of launching tentacles \nthat deal bleeding damage towards nearby enemies.",
		effect_desc_short		= "Your spells launch bleeding tentacles at enemies.",
        precursor_types			= "magic_artifact",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("kraken_limb_blood") >= 1) then
				if(not isPhysicsProjectile(projectile_entity_id)) then
					SetRandomSeed( GameGetFrameNum(), math.random(1,50))
					local chance = Random(1,100)
					local odds = 50 + (5 * getRelicLevel("kraken_limb_blood"))
					if(chance >= 1 and chance <= odds) then
						addBuffVisualIndicator_Projectile(projectile_entity_id, "blood")
						addNewInternalVariable(projectile_entity_id, "projectile_definition", "value_string", "mods/ArcaneReliquary/files/entity_definitions/custom_projectiles/tentacle_blood.xml")
						addNewInternalVariable(projectile_entity_id, "detection_range", "value_int", 100)
						addNewInternalVariable(projectile_entity_id, "projectile_velocity", "value_int", 350)
						addNewInternalVariable(projectile_entity_id, "target_tag", "value_string", "enemy")
						addNewInternalVariable(projectile_entity_id, "max_projectiles_shot", "value_int", 15)
						addNewLuaCallback(projectile_entity_id, "overtime", "mods/ArcaneReliquary/files/scripts/projectiles/target_and_shoot_projectile.lua", 10, false, "target_and_shoot_projectile")
					end
				end
			end
		end,
	},
	{
		name 					= "shuriken",
		ui_name					= "Shuriken",
		categories				= "projectiles,status_effects",
		effect_desc				= "Your spells have a 30% chance of inducing the \nslash effect on enemies hit by them.",
		effect_desc_short		= "Your spells induce the slash effect on enemies.",
        precursor_types			= "magic_artifact,gadget",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("shuriken") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 30 + (5 * getRelicLevel("shuriken"))
				if(chance >= 1 and chance <= odds) then
					attachLoadEffectOntoEntitiestoProjectile(projectile_entity_id, getCustomEffect("effect_slash").path, "spark_white")
				end
			end
		end,
		onpickup_function		= function()
			addNewSimplifiedInternalVariable(getPlayerEntity(), "shuriken")
		end,
		ondrop_function			= function()
			removeSimplifiedInternalVariable(getPlayerEntity(), "shuriken")
		end
	},
	{
		name 					= "poisonous_sting",
		ui_name					= "Poisonous Sting",
		categories				= "player,status_effects",
		effect_desc				= "When you take projectile or melee damage, the creature \nresponsible for it is inflicted with the acid spit effect.",
		effect_desc_short		= "Taking projectile or melee damage inflicts the acid spit effect on the creature that causes it.",
        precursor_types			= "magic_artifact",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		ondamaged_function		= function(damage, message, entity_thats_responsible, is_fatal, projectile_thats_responsible )
			if(isRelicOwned("poisonous_sting") >= 1) then
				if(message == "$damage_projectile" or message == "$damage_melee") then
					LoadGameEffectEntityTo( entity_thats_responsible, getCustomEffect("effect_material_spit_acid").path )
				end
			end
		end,
		onpickup_function		= function()
			addNewSimplifiedInternalVariable(getPlayerEntity(), "poisonous_sting")
		end,
		ondrop_function			= function()
			removeSimplifiedInternalVariable(getPlayerEntity(), "poisonous_sting")
		end
	},
	{
		name 					= "magmatic_artifact",
		ui_name					= "Magmatic Artifact",
		categories				= "projectiles,status_effects",
		effect_desc				= "Your spells have a 30% chance of inducing the \nlava spit effect on enemies hit by them.",
		effect_desc_short		= "Your spells induce the lava spit effect on enemies.",
        precursor_types			= "magic_artifact,effect_stone",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("magmatic_artifact") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 30 + (5 * getRelicLevel("magmatic_artifact"))
				if(chance >= 1 and chance <= odds) then
					attachLoadEffectOntoEntitiestoProjectile(projectile_entity_id, getCustomEffect("effect_material_spit_lava").path, "lava")
				end
			end
		end,
		onpickup_function		= function()
			addNewSimplifiedInternalVariable(getPlayerEntity(), "magmatic_artifact")
		end,
		ondrop_function			= function()
			removeSimplifiedInternalVariable(getPlayerEntity(), "magmatic_artifact")
		end
	},
	{
		name 					= "vile_artifact",
		ui_name					= "Vile Artifact",
		categories				= "player",
		effect_desc				= "When you are falling, stepping on enemies inflicts \nthem with the poison spit effect.",
		effect_desc_short		= "Stepping on enemies inflicts them with the poison spit effect.",
        precursor_types			= "magic_artifact,effect_stone",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			if(not entityHasCallback(getPlayerEntity(), "stomp_listener")) then -- if no other stomp related relics added required callback, adds it
				--GamePrint(">>> stomp relic picked, stomp callback added...")
				addNewLuaCallback(getPlayerEntity(), "overtime", "mods/ArcaneReliquary/files/scripts/player/stomp_listener.lua", 2, false, "stomp_listener")
			end
			addNewSimplifiedInternalVariable(getPlayerEntity(), "relic_using_stomp_relic") -- for each stomp related relic, a new simplified variable like this is added
		end,
		ondrop_function			= function()
			removeFirstSimplifiedInternalVariable(getPlayerEntity(), "relic_using_stomp_relic") -- removes just the first variable found
			if(countSimplifiedInternalVariable(getPlayerEntity(), "relic_using_stomp_relic") == 0) then -- removes stomp callback only if no other relics are using it
				--GamePrint(">>> stomp relic dropped, no other related relics present, removing stomp callback...")
				removeCallback(getPlayerEntity(), "stomp_listener")
			end
		end,
		onenemystomped_function	= function(entity_id, creature_id)
			if(isRelicOwned("vile_artifact") >= 1) then
				-- Loads audiovisual feedback
				local creature_x, creature_y = EntityGetTransform(creature_id)
				GamePlaySound( "data/audio/Desktop/projectiles.snd", "player_projectiles/shield/block", creature_x, creature_y)
				EntityLoad("mods/ArcaneReliquary/files/entity_definitions/particle_explosions/stomp_explosion_purple.xml", creature_x, creature_y)
				
				-- Applies effect on creature found
				LoadGameEffectEntityTo( creature_id, getCustomEffect("effect_material_spit_poison").path )		
			end
		end,
	},
	{
		name 					= "glacial_artifact",
		ui_name					= "Glacial Artifact",
		categories				= "player,kick,status_effects",
		effect_desc				= "Enemies you kick are inflicted with the freezing liquid spit effect. \nIt also synergizes with Sniper Scope relic effect.",
		effect_desc_short		= "Enemies you kick are inflicted with the freezing liquid spit effect.",
        precursor_types			= "boot,effect_stone",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onenemykicked_function	= function (entity_who_kicked)
			if(isRelicOwned("glacial_artifact") >= 1) then
				LoadGameEffectEntityTo( GetUpdatedEntityID(), getCustomEffect("effect_material_spit_freezing_liquid").path )
			end
		end,
		onextendedkick_function	= function( entity_id, extended_kick_id)
			if(isRelicOwned("glacial_artifact") >= 1) then
				attachLoadEffectOntoEntitiestoProjectile(extended_kick_id, getCustomEffect("effect_material_spit_freezing_liquid").path, "plasma_fading")
			end
		end,
	},
	{
		name 					= "pyrophobic_stone",
		ui_name					= "Pyrophobic Stone",
		categories				= "player,status_effects",
		effect_desc				= "When you take fire damage, there's a 10% chance \nyou'll be inflicted with the water spit effect.",
		effect_desc_short		= "Taking fire damage has a chance of inflicting the water spit effect on you.",
        precursor_types			= "magic_artifact,effect_stone",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		ondamaged_function		= function(damage, message, entity_thats_responsible, is_fatal, projectile_thats_responsible )
			if(isRelicOwned("pyrophobic_stone") >= 1) then
				if(message == "$damage_fire") then
					SetRandomSeed( GameGetFrameNum(), math.random(1,50))
					local chance = Random(1,100)
					local odds = 10 + (5 * getRelicLevel("pyrophobic_stone"))
					if(chance >= 1 and chance <= odds) then
						LoadGameEffectEntityTo( getPlayerEntity(), getCustomEffect("effect_material_spit_water").path )
					end
				end
			end
		end,
	},
	{
		name 					= "vicious_stone",
		ui_name					= "Vicious Stone",
		categories				= "player,status_effects",
		effect_desc				= "When you take explosion, projectile or melee damage, there's a 20% chance \nyou'll be inflicted with the berserkium spit effect.",
		effect_desc_short		= "Taking fire explosion, projectile or melee damage has a chance of inflicting the berserkium spit effect on you.",
        precursor_types			= "magic_artifact,effect_stone",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		ondamaged_function		= function(damage, message, entity_thats_responsible, is_fatal, projectile_thats_responsible )
			if(isRelicOwned("vicious_stone") >= 1) then
				if(message == "$damage_projectile" or message == "$damage_melee" or message == "$damage_explosion") then
					SetRandomSeed( GameGetFrameNum(), math.random(1,50))
					local chance = Random(1,100)
					local odds = 20 + (5 * getRelicLevel("vicious_stone"))
					if(chance >= 1 and chance <= odds) then
						LoadGameEffectEntityTo( getPlayerEntity(), getCustomEffect("effect_material_spit_berserkium").path )
					end
				end
			end
		end,
	},
	{
		name 					= "obsidian_boot",
		ui_name					= "Obsidian Boot",
		categories				= "health,materials",
		effect_desc				= "You are now immune to solid damaging materials like cursed and toxic rock.",
		effect_desc_short		= "You are now immune to solid damaging materials like cursed and toxic rock.",
        precursor_types			= "magic_artifact",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			EntitySetDamageFromMaterial( getPlayerEntity(), "rock_static_radioactive", 0 )
			EntitySetDamageFromMaterial( getPlayerEntity(), "rock_static_poison", 0 )
			EntitySetDamageFromMaterial( getPlayerEntity(), "ice_radioactive_static", 0 )
			EntitySetDamageFromMaterial( getPlayerEntity(), "ice_radioactive_glass", 0 )
			EntitySetDamageFromMaterial( getPlayerEntity(), "ice_acid_static", 0 )
			EntitySetDamageFromMaterial( getPlayerEntity(), "ice_acid_glass", 0 )
			EntitySetDamageFromMaterial( getPlayerEntity(), "rock_static_cursed", 0 )
			EntitySetDamageFromMaterial( getPlayerEntity(), "rock_static_cursed_green", 0 )
		end,
		ondrop_function			= function()
			EntitySetDamageFromMaterial( getPlayerEntity(), "rock_static_radioactive", 0.001 )
			EntitySetDamageFromMaterial( getPlayerEntity(), "rock_static_poison", 0.001 )
			EntitySetDamageFromMaterial( getPlayerEntity(), "ice_radioactive_static", 0.001 )
			EntitySetDamageFromMaterial( getPlayerEntity(), "ice_radioactive_glass", 0.001 )
			EntitySetDamageFromMaterial( getPlayerEntity(), "ice_acid_static", 0.001 )
			EntitySetDamageFromMaterial( getPlayerEntity(), "ice_acid_glass", 0.001 )
			EntitySetDamageFromMaterial( getPlayerEntity(), "rock_static_cursed", 0.005 )
			EntitySetDamageFromMaterial( getPlayerEntity(), "rock_static_cursed_green", 0.004 )
		end,
	},
	{
		name 					= "tanto",
		ui_name					= "Tanto",
		categories				= "player",
		effect_desc				= "You gain the ability to dash attack, to use it hit the 'E' key \nwhile moving left or right. This ability also synergizes \nwith other relic effects.",
		effect_desc_short		= "You gain the ability to dash attack.",
        precursor_types			= "magic_artifact,gadget",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()		
			addNewLuaCallback(getPlayerEntity(), "overtime", "mods/ArcaneReliquary/files/scripts/player/dash_attack_listener.lua", 1, false, "dash_attack_listener")
		end,
		ondrop_function			= function()
			removeCallback(getPlayerEntity(), "dash_attack_listener")
		end
	},
	{
		name 					= "shuriken_bloody",
		ui_name					= "Bloody Shuriken",
		categories				= "projectiles,status_effects",
		effect_desc				= "Your spells have a 30% chance of inducing the \nbleed slash effect on enemies hit by them, causing great slice damage and spilling their blood type.",
		effect_desc_short		= "Your spells induce the bleed slash effect on enemies.",
        precursor_types			= "magic_artifact,gadget",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("shuriken_bloody") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 30 + (5 * getRelicLevel("shuriken_bloody"))
				if(chance >= 1 and chance <= odds) then
					attachLoadEffectOntoEntitiestoProjectile(projectile_entity_id, getCustomEffect("effect_slash_bleed").path, "blood")
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "shuriken,ancient_dagger",
	},
	{
		name 					= "shuriken_toxic",
		ui_name					= "Toxic Shuriken",
		categories				= "projectiles,status_effects",
		effect_desc				= "Your spells have a 30% chance of inducing the \nacid slash effect on enemies hit by them.",
		effect_desc_short		= "Your spells induce the acid slash effect on enemies.",
        precursor_types			= "magic_artifact,gadget",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("shuriken_toxic") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 30 + (5 * getRelicLevel("shuriken_toxic"))
				if(chance >= 1 and chance <= odds) then
					attachLoadEffectOntoEntitiestoProjectile(projectile_entity_id, getCustomEffect("effect_slash_acid").path, "acid")
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "shuriken,poisonous_sting",
	},
	{
		name 					= "shuriken_scorching",
		ui_name					= "Scorching Shuriken",
		categories				= "projectiles,status_effects",
		effect_desc				= "Your spells have a 30% chance of inducing the \nlava slash effect on enemies hit by them.",
		effect_desc_short		= "Your spells induce the lava slash effect on enemies.",
        precursor_types			= "magic_artifact,gadget",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("shuriken_scorching") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 30 + (5 * getRelicLevel("shuriken_scorching"))
				if(chance >= 1 and chance <= odds) then
					attachLoadEffectOntoEntitiestoProjectile(projectile_entity_id, getCustomEffect("effect_slash_lava").path, "lava")
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "shuriken,magmatic_artifact",
	},
	{
		name 					= "shuriken_venomous",
		ui_name					= "Venomous Shuriken",
		categories				= "projectiles,status_effects",
		effect_desc				= "Your spells have a 30% chance of inducing the \npoison slash effect on enemies hit by them.",
		effect_desc_short		= "Your spells induce the poison slash effect on enemies.",
        precursor_types			= "magic_artifact,gadget",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("shuriken_venomous") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 30 + (5 * getRelicLevel("shuriken_venomous"))
				if(chance >= 1 and chance <= odds) then
					attachLoadEffectOntoEntitiestoProjectile(projectile_entity_id, getCustomEffect("effect_slash_poison").path, "poison")
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "shuriken,vile_artifact",
	},
	{
		name 					= "uric_stone",
		ui_name					= "Uric Stone",
		categories				= "projectiles,status_effects",
		effect_desc				= "Your spells have a 30% chance of inducing the \nurine spit effect on enemies hit by them.",
		effect_desc_short		= "Your spells induce the urine spit effect on enemies.",
        precursor_types			= "magic_artifact,effect_stone",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("uric_stone") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 30 + (5 * getRelicLevel("uric_stone"))
				if(chance >= 1 and chance <= odds) then
					attachLoadEffectOntoEntitiestoProjectile(projectile_entity_id, getCustomEffect("effect_material_spit_urine").path, "urine")
				end
			end
		end,
	},
	{
		name 					= "shuriken_urinous",
		ui_name					= "Urinous Shuriken",
		categories				= "projectiles,status_effects",
		effect_desc				= "Your spells have a 30% chance of inducing the \nurine slash effect on enemies hit by them.",
		effect_desc_short		= "Your spells induce the urine slash effect on enemies.",
        precursor_types			= "magic_artifact,gadget",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("shuriken_urinous") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 30 + (5 * getRelicLevel("shuriken_urinous"))
				if(chance >= 1 and chance <= odds) then
					attachLoadEffectOntoEntitiestoProjectile(projectile_entity_id, getCustomEffect("effect_slash_urine").path, "urine")
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "shuriken,uric_stone",
	},
	{
		name 					= "firecrackers",
		ui_name					= "Firecrackers",
		categories				= "projectiles,status_effects",
		effect_desc				= "Your spells have a 50% chance of inducing the \ngunpowder spit effect on enemies hit by them.",
		effect_desc_short		= "Your spells induce the gunpowder spit effect on enemies.",
        precursor_types			= "magic_artifact,effect_stone",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("firecrackers") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 50 + (5 * getRelicLevel("firecrackers"))
				if(chance >= 1 and chance <= odds) then
					attachLoadEffectOntoEntitiestoProjectile(projectile_entity_id, getCustomEffect("effect_material_spit_gunpowder").path, "gunpowder")
				end
			end
		end,
		onpickup_function		= function()
			addNewSimplifiedInternalVariable(getPlayerEntity(), "firecrackers")
		end,
		ondrop_function			= function()
			removeSimplifiedInternalVariable(getPlayerEntity(), "firecrackers")
		end
	},
	{
		name 					= "fireworks",
		ui_name					= "Fireworks",
		categories				= "projectiles",
		effect_desc				= "When you hit an enemy with your spells, there's a 50% chance a fireworks rocket will \nbe attached to them, sending them upwards and exploding.",
		effect_desc_short		= "Enemies hit by your spells are attached with fireworks.",
        precursor_types			= "gadget",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onenemydamaged_function	= function(damage, message, entity_thats_responsible, is_fatal, projectile_thats_responsible )
			if(isRelicOwned("fireworks") >= 1) then
				if(message == "$damage_projectile" and not is_fatal) then
					SetRandomSeed( GameGetFrameNum(), math.random(1,50))
					local chance = Random(1,100)
					local odds = 50 + (5 * getRelicLevel("fireworks"))
					if(chance >= 1 and chance <= odds) then
						local enemy_x, enemy_y = EntityGetTransform(GetUpdatedEntityID())
						EntityLoad("mods/ArcaneReliquary/files/entity_definitions/custom_projectiles/fireworks.xml", enemy_x, enemy_y)
					end
				end
			end
		end,
	},
	{
		name 					= "shuriken_wet",
		ui_name					= "Wet Shuriken",
		categories				= "projectiles,status_effects",
		effect_desc				= "Your spells have a 30% chance of inducing the \nwater slash effect on enemies hit by them.",
		effect_desc_short		= "Your spells induce the water slash effect on enemies.",
        precursor_types			= "magic_artifact,gadget",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)

			if(isRelicOwned("shuriken_wet") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 30 + (5 * getRelicLevel("shuriken_wet"))
				if(chance >= 1 and chance <= odds) then
					attachLoadEffectOntoEntitiestoProjectile(projectile_entity_id, getCustomEffect("effect_slash_water").path, "water")
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "shuriken,pyrophobic_stone",
	},
	{
		name 					= "nuclear_wand",
		ui_name					= "Nuclear Wand",
		categories				= "projectiles",
		effect_desc				= "When you cast a spell, you have a 30% chance of summoning a green \nspectral wand that shoots a nuclear lasers that release acid thunderballs.",
		effect_desc_short		= "Your spells summon a spectral wand that shoots nuclear lasers that release acid thunderballs.",
        precursor_types			= "magic_artifact",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("nuclear_wand") >= 1) then
				if(not projectileHasProjectileDefinitionPath(projectile_entity_id)) then
					SetRandomSeed( GameGetFrameNum(), math.random(1,50))
					local chance = Random(1,100)
					local odds = 30 + (5 * getRelicLevel("nuclear_wand"))
					if(chance >= 1 and chance <= odds) then
						local x, y = EntityGetTransform(projectile_entity_id)
						local laser_shooter_id = EntityLoad("mods/ArcaneReliquary/files/entity_definitions/custom_projectiles/megalaser_nuclear.xml", x, y)
						addNewSimplifiedInternalVariable( laser_shooter_id, "relic_effect_projectile")
						local projectile_vel_x, projectile_vel_y = getProjectileVelocitiesandGravity(projectile_entity_id)
						setProjectileVelocitiesandGravity(laser_shooter_id, projectile_vel_x, projectile_vel_y)
					end
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "very_hazardous_wand,thunderous_wand,pungent_wand",
	},
	{
		name 					= "cursed_necklace",
		ui_name					= "Cursed Necklace",
		categories				= "projectiles,status_effects",
		effect_desc				= "Your spells have a 30% chance of inducing enemies they hit to \nsummon spectral wands that target and shoot laser beams on them.",
		effect_desc_short		= "Your spells induce enemies to summon spectral wands that target and shoot laser beams on them.",
        precursor_types			= "magic_artifact,necklace",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("cursed_necklace") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 30 + (5 * getRelicLevel("cursed_necklace"))
				if(chance >= 1 and chance <= odds) then
					attachLoadEffectOntoEntitiestoProjectile(projectile_entity_id, getCustomEffect("effect_spectral_wand_target").path, "spark_purple_bright")
				end
			end
		end,
	},
	{
		name 					= "bomb_charm",
		ui_name					= "Bomb Charm",
		categories				= "projectiles,status_effects",
		effect_desc				= "Your spells have a 30% chance of inducing enemies they hit to \nspawn bombs periodically.",
		effect_desc_short		= "Your spells induce enemies to spawn bombs periodically.",
        precursor_types			= "gadget",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("bomb_charm") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 30 + (5 * getRelicLevel("bomb_charm"))
				if(chance >= 1 and chance <= odds) then
					attachLoadEffectOntoEntitiestoProjectile(projectile_entity_id, getCustomEffect("effect_spawn_object_bomb").path, "spark_white")
				end
			end
		end,
	},
	{
		name 					= "glitter_bomb_charm",
		ui_name					= "Glitter Bomb Charm",
		categories				= "projectiles,status_effects",
		effect_desc				= "Your spells have a 30% chance of inducing enemies they hit to \nspawn glitter bombs periodically.",
		effect_desc_short		= "Your spells induce enemies to spawn glitter bombs periodically.",
        precursor_types			= "gadget",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("glitter_bomb_charm") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 30 + (5 * getRelicLevel("glitter_bomb_charm"))
				if(chance >= 1 and chance <= odds) then
					attachLoadEffectOntoEntitiestoProjectile(projectile_entity_id, getCustomEffect("effect_spawn_object_glitter_bomb").path, "plasma_fading_pink")
				end
			end
		end,
	},
	{
		name 					= "tnt_charm",
		ui_name					= "TNT Charm",
		categories				= "projectiles,status_effects",
		effect_desc				= "Your spells have a 30% chance of inducing enemies they hit to \nspawn tnt periodically.",
		effect_desc_short		= "Your spells induce enemies to spawn tnt periodically.",
        precursor_types			= "gadget",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("tnt_charm") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 30 + (5 * getRelicLevel("tnt_charm"))
				if(chance >= 1 and chance <= odds) then
					attachLoadEffectOntoEntitiestoProjectile(projectile_entity_id, getCustomEffect("effect_spawn_object_tnt").path, "spark_red")
				end
			end
		end,
	},
	{
		name 					= "explosive_box_charm",
		ui_name					= "Explosive Box Charm",
		categories				= "projectiles,status_effects",
		effect_desc				= "Your spells have a 30% chance of inducing enemies they hit to \nspawn explosive boxes periodically.",
		effect_desc_short		= "Your spells induce enemies to spawn explosive boxes periodically.",
        precursor_types			= "gadget",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("explosive_box_charm") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 30 + (5 * getRelicLevel("explosive_box_charm"))
				if(chance >= 1 and chance <= odds) then
					attachLoadEffectOntoEntitiestoProjectile(projectile_entity_id, getCustomEffect("effect_spawn_object_explosive_box").path, "mushroom")
				end
			end
		end,
	},
	{
		name 					= "tnt_box_charm",
		ui_name					= "TNT Box Charm",
		categories				= "projectiles,status_effects",
		effect_desc				= "Your spells have a 30% chance of inducing enemies they hit to \nspawn tnt boxes periodically.",
		effect_desc_short		= "Your spells induce enemies to spawn tnt boxes periodically.",
        precursor_types			= "gadget",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("tnt_box_charm") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 30 + (5 * getRelicLevel("tnt_box_charm"))
				if(chance >= 1 and chance <= odds) then
					attachLoadEffectOntoEntitiestoProjectile(projectile_entity_id, getCustomEffect("effect_spawn_object_tnt_box").path, "spark_red")
				end
			end
		end,
	},
	{
		name 					= "oil_barrel_charm",
		ui_name					= "Oil Barrel Charm",
		categories				= "projectiles,status_effects",
		effect_desc				= "Your spells have a 30% chance of inducing enemies they hit to \nspawn oil barrels periodically.",
		effect_desc_short		= "Your spells induce enemies to spawn oil barrels periodically.",
        precursor_types			= "gadget",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("oil_barrel_charm") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 30 + (5 * getRelicLevel("oil_barrel_charm"))
				if(chance >= 1 and chance <= odds) then
					attachLoadEffectOntoEntitiestoProjectile(projectile_entity_id, getCustomEffect("effect_spawn_object_oil_barrel").path, "oil")
				end
			end
		end,
	},
	{
		name 					= "toxic_barrel_charm",
		ui_name					= "Toxic Barrel Charm",
		categories				= "projectiles,status_effects",
		effect_desc				= "Your spells have a 30% chance of inducing enemies they hit to \nspawn toxic barrels periodically.",
		effect_desc_short		= "Your spells induce enemies to spawn toxic barrels periodically.",
        precursor_types			= "gadget",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("toxic_barrel_charm") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 30 + (5 * getRelicLevel("toxic_barrel_charm"))
				if(chance >= 1 and chance <= odds) then
					attachLoadEffectOntoEntitiestoProjectile(projectile_entity_id, getCustomEffect("effect_spawn_object_toxic_barrel").path, "radioactive_gas")
				end
			end
		end,
	},
	{
		name 					= "seamine_charm",
		ui_name					= "Sea Mine Charm",
		categories				= "projectiles,enemies",
		effect_desc				= "Every enemy hit by your spells has a 25% chance of being transformed into an Sea Mine.",
		effect_desc_short		= "Your spells transform enemies into Sea Mines.",
        precursor_types			= "gadget",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onenemydamaged_function	= function(damage, message, entity_thats_responsible, is_fatal, projectile_thats_responsible )
			if(isRelicOwned("seamine_charm") >= 1) then
				if(message == "$damage_projectile") then
					SetRandomSeed( GameGetFrameNum(), math.random(1,50))
					local chance = Random(1,100)
					local odds = 25 + (5 * getRelicLevel("seamine_charm"))
					if(chance >= 1 and chance <= odds) then
						LoadGameEffectEntityTo( GetUpdatedEntityID(), getCustomEffect("effect_convert_to_seamine").path )
					end
				end
			end
		end,
	},
	{
		name 					= "acid_tank_charm",
		ui_name					= "Acid Tank Charm",
		categories				= "projectiles,enemies",
		effect_desc				= "Every enemy hit by your spells has a 25% chance of being transformed into an Acid Container.",
		effect_desc_short		= "Your spells transform enemies into Acid Container.",
        precursor_types			= "gadget",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onenemydamaged_function	= function(damage, message, entity_thats_responsible, is_fatal, projectile_thats_responsible )
			if(isRelicOwned("acid_tank_charm") >= 1) then
				if(message == "$damage_projectile") then
					SetRandomSeed( GameGetFrameNum(), math.random(1,50))
					local chance = Random(1,100)
					local odds = 25 + (5 * getRelicLevel("acid_tank_charm"))
					if(chance >= 1 and chance <= odds) then
						LoadGameEffectEntityTo( GetUpdatedEntityID(), getCustomEffect("effect_convert_to_acid_container").path )
					end
				end
			end
		end,
	},
	{
		name 					= "withering_necklace",
		ui_name					= "Withering Necklace",
		categories				= "projectiles,status_effects",
		effect_desc				= "Your spells have a 30% chance of inducing enemies they hit to \nsummon spectral wands that target and shoot freezing laser beams on them.",
		effect_desc_short		= "Your spells induce enemies to summon spectral wands that target and shoot freezing laser beams on them.",
        precursor_types			= "magic_artifact,necklace",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("withering_necklace") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 30 + (5 * getRelicLevel("withering_necklace"))
				if(chance >= 1 and chance <= odds) then
					attachLoadEffectOntoEntitiestoProjectile(projectile_entity_id, getCustomEffect("effect_spectral_wand_target_freeze").path, "spark_blue")
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "cursed_necklace,ice_artifact",
	},
	{
		name 					= "scorching_necklace",
		ui_name					= "Scorching Necklace",
		categories				= "projectiles,status_effects",
		effect_desc				= "Your spells have a 30% chance of inducing enemies they hit to \nsummon spectral wands that target and shoot lava beams on them.",
		effect_desc_short		= "Your spells induce enemies to summon spectral wands that target and shoot lava beams on them.",
        precursor_types			= "magic_artifact,necklace",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("scorching_necklace") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 30 + (5 * getRelicLevel("scorching_necklace"))
				if(chance >= 1 and chance <= odds) then
					attachLoadEffectOntoEntitiestoProjectile(projectile_entity_id, getCustomEffect("effect_spectral_wand_target_lava").path, "lava")
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "cursed_necklace,lava_gem",
	},
	{
		name 					= "venomous_necklace",
		ui_name					= "Venomous Necklace",
		categories				= "projectiles,status_effects",
		effect_desc				= "Your spells have a 30% chance of inducing enemies they hit to \nsummon spectral wands that target and shoot poison beams on them.",
		effect_desc_short		= "Your spells induce enemies to summon spectral wands that target and shoot poison beams on them.",
        precursor_types			= "magic_artifact,necklace",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("venomous_necklace") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 30 + (5 * getRelicLevel("venomous_necklace"))
				if(chance >= 1 and chance <= odds) then
					attachLoadEffectOntoEntitiestoProjectile(projectile_entity_id, getCustomEffect("effect_spectral_wand_target_poison").path, "poison")
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "cursed_necklace,poison_gem",
	},
	{
		name 					= "sharp_necklace",
		ui_name					= "Sharp Necklace",
		categories				= "projectiles,status_effects",
		effect_desc				= "Your spells have a 30% chance of inducing enemies they hit to \nsummon spectral wands that target and shoot slicing beams on them.",
		effect_desc_short		= "Your spells induce enemies to summon spectral wands that target and shoot slicing beams on them.",
        precursor_types			= "magic_artifact,necklace",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("sharp_necklace") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 30 + (5 * getRelicLevel("sharp_necklace"))
				if(chance >= 1 and chance <= odds) then
					attachLoadEffectOntoEntitiestoProjectile(projectile_entity_id, getCustomEffect("effect_spectral_wand_target_slice").path, "spark_white")
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "cursed_necklace,pizza_slice",
	},
	{
		name 					= "diamond_ring",
		ui_name					= "Diamond Ring",
		categories				= "projectiles,status_effects",
		effect_desc				= "Your spells have a 30% chance of creating a mystic target on \ncreatures you hit, making them summon magic orbs towards themselves.",
		effect_desc_short		= "Your spells create mystic targets on creatures, summoning magic orbs towards them.",
        precursor_types			= "magic_artifact,ring",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("diamond_ring") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 30 + (5 * getRelicLevel("diamond_ring"))
				if(chance >= 1 and chance <= odds) then
					attachLoadEffectOntoEntitiestoProjectile(projectile_entity_id, getCustomEffect("effect_projectile_target").path, "spark_blue_dark")
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "elemental_band,gravity_stone_blue",
	},
	{
		name 					= "earth_core_fragment",
		ui_name					= "Earth Core Fragment",
		categories				= "player",
		effect_desc				= "You gain the ability to ground pound, to use it hit the 'Down' key \nwhile midair, and cancel it by flying or jumping. This ability also synergizes \nwith other relic effects.",
		effect_desc_short		= "You gain the ability to ground pound.",
        precursor_types			= "magic_artifact",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()		
			addNewLuaCallback(getPlayerEntity(), "overtime", "mods/ArcaneReliquary/files/scripts/player/ground_pound_listener.lua", 1, false, "ground_pound_listener")
			
			addNewSimplifiedInternalVariable(getPlayerEntity(), "earth_core_fragment")
		end,
		ondrop_function			= function()
			removeCallback(getPlayerEntity(), "ground_pound_listener")
			
			removeSimplifiedInternalVariable(getPlayerEntity(), "earth_core_fragment")
		end
	},
	{
		name 					= "boxing_glove",
		ui_name					= "Boxing Glove",
		categories				= "player",
		effect_desc				= "You gain the ability to uppercut enemies, to use it hit 'Up' or 'Space' key \nwhile moving right or left, on the ground or midair. This ability also synergizes \nwith other relic effects.",
		effect_desc_short		= "You gain the ability to uppercut enemies.",
        precursor_types			= "gadget",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()		
			addNewLuaCallback(getPlayerEntity(), "overtime", "mods/ArcaneReliquary/files/scripts/player/uppercut_listener.lua", 1, false, "uppercut_listener")
		end,
		ondrop_function			= function()
			removeCallback(getPlayerEntity(), "uppercut_listener")
		end
	},
	{
		name 					= "porcupine_stone",
		ui_name					= "Porcupine Stone",
		categories				= "player",
		effect_desc				= "You gain the homing attack ability, to use it hit the right mouse \nbutton while in close vicinity to an enemy.",
		effect_desc_short		= "You gain the homing attack ability.",
        precursor_types			= "effect_stone",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()		
			addNewLuaCallback(getPlayerEntity(), "overtime", "mods/ArcaneReliquary/files/scripts/player/homing_attack_listener.lua", 1, false, "homing_attack_listener")
		end,
		ondrop_function			= function()
			removeCallback(getPlayerEntity(), "homing_attack_listener")
		end
	},
	{
		name 					= "neodymium_bomb",
		ui_name					= "Neodymium Bomb",
		categories				= "player",
		effect_desc				= "You gain immunity to explosions and your spells gain a x5 explosive power \nmultiplier, but you are permanently inflicted with the attraction effect, causing you to attract nearby projectiles not shot by you.",
		effect_desc_short		= "You gain explosion immunity and your spells get bonus explosive power, but you attract enemy projectiles.",
        precursor_types			= "magic_artifact,gadget",
		upgrade_effect_desc		= "For each upgrade level, explosive power multiplier increases by 2.",
		is_upgradable 			= true,
		onpickup_function		= function()
			
			-- adds attraction effect
			LoadGameEffectEntityTo( getPlayerEntity(), getCustomEffect("effect_attraction_permanent").path )
			-- adds explosion protection effect
			local game_effect_comp_id, game_effect_entity_id = GetGameEffectLoadTo( getPlayerEntity(), "PROTECTION_EXPLOSION", true )
			if game_effect_comp_id ~= nil then
				ComponentSetValue2( game_effect_comp_id, "frames", -1)
				addNewSimplifiedInternalVariable(game_effect_entity_id, "protection_explosion_neodymium_bomb")
			end
		end,
		ondrop_function			= function()
			
			-- removes attraction effect
			local player_children = EntityGetAllChildren(getPlayerEntity())
			if player_children == nil then return end
			for i,child in ipairs(player_children) do
				if(EntityGetName(child) == "attraction_permanent") then
					EntityKill(child)
					break
				end
			end
			
			-- removes explosion protection effect
			local player_children = EntityGetAllChildren(getPlayerEntity())
			if player_children == nil then return end
			for i,child in ipairs(player_children) do
				if(entityHasSimplifiedInternalVariable(child, "protection_explosion_neodymium_bomb")) then
					EntityKill(child)
					break
				end
			end
		end,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("neodymium_bomb") >= 1) then
				multiplyProjectileExplosiveDamage(projectile_entity_id, 5 + (2 * getRelicLevel("neodymium_bomb")))
			end
		end,
		excludes_relics_desc	= "It can't be used along with other relics that provide explosions resistance",
		excludes_relics_tags	= "explosion_resistance",
	},
	{
		name 					= "nuclear_boot",
		ui_name					= "Nuclear Boot",
		categories				= "player",
		effect_desc				= "Falling and stepping on enemies causes you to shoot radioactive fireballs from your feet for a few seconds.",
		effect_desc_short		= "Stepping on enemies causes you to shoot radioactive fireballs from your feet.",
        precursor_types			= "boot",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			if(not entityHasCallback(getPlayerEntity(), "stomp_listener")) then -- if no other stomp related relics added required callback, adds it
				--GamePrint(">>> stomp relic picked, stomp callback added...")
				addNewLuaCallback(getPlayerEntity(), "overtime", "mods/ArcaneReliquary/files/scripts/player/stomp_listener.lua", 2, false, "stomp_listener")
			end
			addNewSimplifiedInternalVariable(getPlayerEntity(), "relic_using_stomp_relic") -- for each stomp related relic, a new simplified variable like this is added
		end,
		ondrop_function			= function()
			removeFirstSimplifiedInternalVariable(getPlayerEntity(), "relic_using_stomp_relic") -- removes just the first variable found
			
			if(countSimplifiedInternalVariable(getPlayerEntity(), "relic_using_stomp_relic") == 0) then -- removes stomp callback only if no other relics are using it
				--GamePrint(">>> stomp relic dropped, no other related relics present, removing stomp callback...")
				removeCallback(getPlayerEntity(), "stomp_listener")
			end
		end,
		onenemystomped_function	= function(entity_id, creature_id)
			if(isRelicOwned("nuclear_boot") >= 1) then
				-- Applies effect on creature found
				LoadGameEffectEntityTo( entity_id, getCustomEffect("effect_shoot_projectile_downwards_fireball_radioactive").path )
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "volcanic_boot,uranium_ingot",
	},
	{
		name 					= "popsicle",
		ui_name					= "Popsicle",
		categories				= "player,projectiles",
		effect_desc				= "When you move left or right, freezebombs are shot from your back. \nIt also synergizes with Earth Core Fragment relic effect.",
		effect_desc_short		= "When you move left or right, freezebombs are shot from your back.",
        precursor_types			= "gadget",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			addNewLuaCallback(getPlayerEntity(), "overtime", "mods/ArcaneReliquary/files/scripts/player/shoot_sideways_freezebomb.lua", 5, false, "shoot_sideways_freezebomb")
		end,
		ondrop_function			= function()		
			removeCallback(getPlayerEntity(), "shoot_sideways_freezebomb")
		end,
		ongroundpound_function	= function( entity_id)
			if(isRelicOwned("popsicle") >= 1) then
				LoadGameEffectEntityTo( entity_id, getCustomEffect("effect_shoot_projectiles_above_freezebomb").path )
			end
		end,
	},
	{
		name 					= "coagulated_blood",
		ui_name					= "Coagulated Blood",
		categories				= "player",
		effect_desc				= "When you take projectile or melee damage, you have a 50% chance of releasing a barrage of bloodbombs. \nIt also synergizes with Earth Core Fragment relic effect.",
		effect_desc_short		= "You release bloodbombs when taking projectile or melee damage.",
        precursor_types			= "gadget",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		ondamaged_function		= function(damage, message, entity_thats_responsible, is_fatal, projectile_thats_responsible )
			if(isRelicOwned("coagulated_blood") >= 1) then	
				if(message == "$damage_projectile" or message == "$damage_melee") then
					SetRandomSeed( GameGetFrameNum(), math.random(1,50))
					local chance = Random(1,100)
					local odds = 50 + (5 * getRelicLevel("coagulated_blood"))
					if(chance >= 1 and chance <= odds) then
						local player_x, player_y = EntityGetTransform(GetUpdatedEntityID())
						EntityLoad("mods/ArcaneReliquary/files/entity_definitions/load_on_explosion/launch_circular_projectiles_bloodbombs.xml", player_x, player_y)
					end
				end
			end
		end,
		ongroundpound_function	= function( entity_id)
			if(isRelicOwned("coagulated_blood") >= 1) then
				LoadGameEffectEntityTo( entity_id, getCustomEffect("effect_shoot_projectiles_above_bloodbomb").path )
			end
		end,
	},
	{
		name 					= "fly_swatter",
		ui_name					= "Fly Swatter",
		categories				= "projectiles",
		effect_desc				= "Your spells have a 50% chance of exploding into a \nbarrage of slapperbombs when they explode or hit something. \nIt also synergizes with Earth Core Fragment relic effect.",
		effect_desc_short		= "Your spells explode into slapperbombs on impact.",
        precursor_types			= "effect_stone",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("fly_swatter") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 50 + (5 * getRelicLevel("fly_swatter"))
				if(chance >= 1 and chance <= odds) then	
					addBuffVisualIndicator_Projectile(projectile_entity_id, "plasma_fading_green")
					attachLoadEntityOnExplosiontoProjectile(projectile_entity_id, getLoadOnExplosion("launch_circular_projectiles_slapperbombs").path)
				end
			end
		end,
		ongroundpound_function	= function( entity_id)
			if(isRelicOwned("fly_swatter") >= 1) then
				LoadGameEffectEntityTo( entity_id, getCustomEffect("effect_shoot_projectiles_above_slapperbomb").path )
			end
		end,
	},
	{
		name 					= "bombs_in_a_jar",
		ui_name					= "Bombs in a Jar",
		categories				= "player",
		effect_desc				= "You spontaneously shoot various effect bombs at random directions.",
		effect_desc_short		= "You spontaneously shoot various effect bombs at random directions.",
        precursor_types			= "magic_artifact,gadget",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			
			LoadGameEffectEntityTo( getPlayerEntity(), getCustomEffect("effect_launch_various_bombs").path )

		end,
		ondrop_function			= function()

			local player_children = EntityGetAllChildren(getPlayerEntity())
			if player_children == nil then return end
			for i,child in ipairs(player_children) do
				if(EntityGetName(child) == "launch_various_bombs") then
					EntityKill(child)
					break
				end
			end

		end,
		combine_exclusive		= true,
		combine_requires_relics	= "ballistic_boot,popsicle,coagulated_blood,fly_swatter,thunder_hat,mutating_bead,little_cloud",
	},
	{
		name 					= "mutating_bead",
		ui_name					= "Mutating Bead",
		categories				= "projectiles",
		effect_desc				= "Spells that you cast have a 30% chance of shooting barrages of polybombs along their travel path. \nIt also synergizes with Earth Core Fragment relic effect.",
		effect_desc_short		= "Your spells cast barrages of polybombs.",
        precursor_types			= "effect_crystal",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%, and when it triggers, polybombs are shot more often.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("mutating_bead") >= 1) then
				if(not isPhysicsProjectile(projectile_entity_id)) then
					SetRandomSeed( GameGetFrameNum(), math.random(1,50))
					local chance = Random(1,100)
					local odds = 30 + (5 * getRelicLevel("mutating_bead"))
					if(chance >= 1 and chance <= odds) then
						addNewInternalVariable(projectile_entity_id, "projectile_shot_down_definition", "value_string", "mods/ArcaneReliquary/files/entity_definitions/load_on_explosion/launch_circular_projectiles_polybombs.xml")
						addNewInternalVariable(projectile_entity_id, "down_vel_min", "value_int", 0)
						addNewInternalVariable(projectile_entity_id, "down_vel_max", "value_int", 0)
						addNewInternalVariable(projectile_entity_id, "max_projectiles_shot", "value_int", 10)
						addNewLuaCallback(projectile_entity_id, "overtime", "mods/ArcaneReliquary/files/scripts/projectiles/launch_projectile_down.lua", 25 - (2 * getRelicLevel("mutating_bead")), false, "launch_projectile_down")
					end
				end
			end
		end,
		ongroundpound_function	= function( entity_id)
			if(isRelicOwned("mutating_bead") >= 1) then
				LoadGameEffectEntityTo( entity_id, getCustomEffect("effect_shoot_projectiles_above_polybomb").path )
			end
		end,
	},
	{
		name 					= "little_cloud",
		ui_name					= "Little Cloud",
		categories				= "player",
		effect_desc				= "Falling and stepping on enemies causes you to shoot windbombs from your feet for a few seconds. \nIt also synergizes with Earth Core Fragment relic effect.",
		effect_desc_short		= "Stepping on enemies causes you to shoot windbombs from your feet.",
        precursor_types			= "boot",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			if(not entityHasCallback(getPlayerEntity(), "stomp_listener")) then -- if no other stomp related relics added required callback, adds it
				--GamePrint(">>> stomp relic picked, stomp callback added...")
				addNewLuaCallback(getPlayerEntity(), "overtime", "mods/ArcaneReliquary/files/scripts/player/stomp_listener.lua", 2, false, "stomp_listener")
			end
			addNewSimplifiedInternalVariable(getPlayerEntity(), "relic_using_stomp_relic") -- for each stomp related relic, a new simplified variable like this is added
		end,
		ondrop_function			= function()
			removeFirstSimplifiedInternalVariable(getPlayerEntity(), "relic_using_stomp_relic") -- removes just the first variable found
			
			if(countSimplifiedInternalVariable(getPlayerEntity(), "relic_using_stomp_relic") == 0) then -- removes stomp callback only if no other relics are using it
				--GamePrint(">>> stomp relic dropped, no other related relics present, removing stomp callback...")
				removeCallback(getPlayerEntity(), "stomp_listener")
			end
		end,
		onenemystomped_function	= function(entity_id, creature_id)
			if(isRelicOwned("little_cloud") >= 1) then
				-- Applies effect on creature found
				LoadGameEffectEntityTo( entity_id, getCustomEffect("effect_shoot_projectile_downwards_windbomb").path )
			end
		end,
		ongroundpound_function	= function( entity_id)
			if(isRelicOwned("little_cloud") >= 1) then
				LoadGameEffectEntityTo( entity_id, getCustomEffect("effect_shoot_projectiles_above_windbomb").path )
			end
		end,
	},
	{
		name 					= "sawblade_small",
		ui_name					= "Small Sawblade",
		categories				= "player",
		effect_desc				= "Small sawblades that propel themselves towards \nenemies and redirect enemy projectiles that collide with them orbit around you.",
		effect_desc_short		= "Small sawblades that propel themselves towards enemies orbit around you.",
        precursor_types			= "gadget",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		ondrop_function			= function()
			detachRotatingShieldfromEntity(getPlayerEntity(), "rotating_shield_projectile_disk_bullet_medium")
		end,
		overtime_function		= function()
			if(isRelicOwned("sawblade_small") >= 1) then
				if(not entityHasRotatingShield(getPlayerEntity(), "rotating_shield_projectile_disk_bullet_medium")) then -- this makes sure the orbiting projectiles are loaded again if the player is polymorphed
					local start_id = EntityLoad("mods/ArcaneReliquary/files/entity_definitions/custom_projectiles/rotating_shield_start.xml")
					setInternalVariableValue(start_id, "host_entity_id", "value_int", getPlayerEntity())
					setInternalVariableValue(start_id, "rotating_projectile_definition", "value_string", "mods/ArcaneReliquary/files/entity_definitions/custom_projectiles/rotating_disc_bullet_medium.xml")
					setInternalVariableValue(start_id, "rotating_projectiles_amount", "value_int", 4)
					setInternalVariableValue(start_id, "radius", "value_int", 32)
					setInternalVariableValue(start_id, "rotation_speed", "value_int", 2)
				end
			end
		end,
		
	},
	{
		name 					= "rocket_toy",
		ui_name					= "Rocket Toy",
		categories				= "player",
		effect_desc				= "When you take projectile damage, there's a 50% chance you'll \nsummon homing microrockets and if you cast a spell, there's a 20% chance the same will happen.",
		effect_desc_short		= "You summon homing microrockets when damaged or casting spells.",
        precursor_types			= "gadget",
		upgrade_effect_desc		= "For each upgrade level, summon chance by taking damage or casting spells increases by 5%.",
		is_upgradable 			= true,
		ondamaged_function		= function(damage, message, entity_thats_responsible, is_fatal, projectile_thats_responsible )
			if(isRelicOwned("rocket_toy") >= 1) then
				if(message == "$damage_projectile") then
					SetRandomSeed( GameGetFrameNum(), math.random(1,50))
					local chance = Random(1,100)
					local odds = 50 + (5 * getRelicLevel("rocket_toy"))
					if(chance >= 1 and chance <= odds) then
						local projectile_amount = Random(1,5)
						for i=1,projectile_amount do
							local player_x, player_y = EntityGetTransform(getPlayerEntity())
							local projectile_id = EntityLoad("mods/ArcaneReliquary/files/entity_definitions/custom_projectiles/rocket_tiny_roll_custom.xml", player_x, player_y)
							setProjectileVelocitiesandGravity(projectile_id, Random(-20,20), Random(-130,-160))
							setProjectileShooter(projectile_id, getPlayerEntity())
						end
					end
				end
			end
		end,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("rocket_toy") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 20 + (5 * getRelicLevel("rocket_toy"))
				if(chance >= 1 and chance <= odds) then
					local projectile_amount = Random(1,2)
					for i=1,projectile_amount do
						local player_x, player_y = EntityGetTransform(getPlayerEntity())
						local projectile_id = EntityLoad("mods/ArcaneReliquary/files/entity_definitions/custom_projectiles/rocket_tiny_roll_custom.xml", player_x, player_y)
						setProjectileVelocitiesandGravity(projectile_id, Random(-20,20), Random(-130,-160))
					end
				end
			end
		end,
	},
	{
		name 					= "elemental_charge_artifact_conducting",
		ui_name					= "Conducting Artifact",
		categories				= "player",
		effect_desc				= "Upon taking electricity damage, you become electrically charged, and the \nfirst spell you cast after it happens will cause you to unleash a barrage of thunderballs.",
		effect_desc_short		= "Casting a spell after taking electricity damage unleashes a barrage of thunderballs.",
        precursor_types			= "magic_artifact,effect_stone",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		ondamaged_function		= function(damage, message, entity_thats_responsible, is_fatal, projectile_thats_responsible )
			if(isRelicOwned("elemental_charge_artifact_conducting") >= 1) then
				if(message == "$damage_electricity") then
					if(not entityHasSimplifiedInternalVariable(GetUpdatedEntityID(), "thunderball_barrage")) then
						addNewSimplifiedInternalVariable(GetUpdatedEntityID(), "thunderball_barrage")
						addSpriteEffect(GetUpdatedEntityID(), "mods/ArcaneReliquary/files/other_gfx/thunder_charge.xml", 15, 20, -1, 0.7, "thunder_charge_effect")
						addEntityAudioLoop(GetUpdatedEntityID(), "data/audio/Desktop/animals.bank", "animals/thundermage/movement_loop", "thunder_charge_effect")
					end
				end
			end
		end,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("elemental_charge_artifact_conducting") >= 1) then
				if(entityHasSimplifiedInternalVariable(GetUpdatedEntityID(), "thunderball_barrage")) then
					
					-- loads thunderball barrage and removes tag and sprite effect
					local player_x, player_y = EntityGetTransform(getPlayerEntity())
					local projectile_id = EntityLoad(getLoadOnExplosion("launch_circular_projectiles_thunderballs").path, player_x, player_y)
					removeSimplifiedInternalVariable(GetUpdatedEntityID(), "thunderball_barrage")
					removeSpriteEffect(GetUpdatedEntityID(), "thunder_charge_effect")
					removeAudioLoop(GetUpdatedEntityID(), "thunder_charge_effect")
					
					-- gives temporary immunity to electricity and explosions
					local game_effect_comp = GetGameEffectLoadTo( getPlayerEntity(), "PROTECTION_ELECTRICITY", true )
					if game_effect_comp ~= nil then
						ComponentSetValue( game_effect_comp, "frames", "90" )
					end

					local game_effect_comp = GetGameEffectLoadTo( getPlayerEntity(), "PROTECTION_EXPLOSION", true )
					if game_effect_comp ~= nil then
						ComponentSetValue( game_effect_comp, "frames", "90" )
					end
				end
			end
		end,
		ondrop_function			= function()
			-- removes charge effect
			removeSimplifiedInternalVariable(getPlayerEntity(), "thunderball_barrage")
			removeSpriteEffect(getPlayerEntity(), "thunder_charge_effect")
			removeAudioLoop(getPlayerEntity(), "thunder_charge_effect")
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "absorbing_crystal,overloaded_necklace",
	},
	{
		name 					= "elemental_charge_artifact_infernal",
		ui_name					= "Infernal Artifact",
		categories				= "player",
		effect_desc				= "Upon taking fire damage, you become fire charged, and the \nfirst spell you cast after it happens will cause you to unleash a barrage of fireballs.",
		effect_desc_short		= "Casting a spell after taking fire damage unleashes a barrage of fireballs.",
        precursor_types			= "magic_artifact,effect_stone",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		ondamaged_function		= function(damage, message, entity_thats_responsible, is_fatal, projectile_thats_responsible )
			if(isRelicOwned("elemental_charge_artifact_infernal") >= 1) then
				if(message == "$damage_fire") then
					if(not entityHasSimplifiedInternalVariable(GetUpdatedEntityID(), "fireball_barrage")) then
						addNewSimplifiedInternalVariable(GetUpdatedEntityID(), "fireball_barrage")
						addSpriteEffect(GetUpdatedEntityID(), "mods/ArcaneReliquary/files/other_gfx/fire_charge.xml", 17, 22, -1, 0.7, "fire_charge_effect")
						addEntityAudioLoop(GetUpdatedEntityID(), "data/audio/Desktop/animals.bank", "animals/firemage/burning_loop", "fire_charge_effect")
					end
				end
			end
		end,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("elemental_charge_artifact_infernal") >= 1) then
				if(entityHasSimplifiedInternalVariable(GetUpdatedEntityID(), "fireball_barrage")) then
					
					-- loads fireballs barrage and removes tag and sprite effect
					local player_x, player_y = EntityGetTransform(getPlayerEntity())
					local projectile_id = EntityLoad(getLoadOnExplosion("launch_circular_projectiles_fireballs").path, player_x, player_y)
					removeSimplifiedInternalVariable(GetUpdatedEntityID(), "fireball_barrage")
					removeSpriteEffect(GetUpdatedEntityID(), "fire_charge_effect")
					removeAudioLoop(GetUpdatedEntityID(), "fire_charge_effect")
					
					-- gives temporary immunity to fire and explosions
					local game_effect_comp = GetGameEffectLoadTo( getPlayerEntity(), "PROTECTION_FIRE", true )
					if game_effect_comp ~= nil then
						ComponentSetValue( game_effect_comp, "frames", "90" )
					end
					
					local game_effect_comp = GetGameEffectLoadTo( getPlayerEntity(), "PROTECTION_EXPLOSION", true )
					if game_effect_comp ~= nil then
						ComponentSetValue( game_effect_comp, "frames", "90" )
					end
				end
			end
		end,
		ondrop_function			= function()
			-- removes charge effect
			removeSimplifiedInternalVariable(getPlayerEntity(), "fireball_barrage")
			removeSpriteEffect(getPlayerEntity(), "fire_charge_effect")
			removeAudioLoop(getPlayerEntity(), "fire_charge_effect")
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "absorbing_crystal,ember_stone",
	},
	{
		name 					= "elemental_charge_artifact_blizzard",
		ui_name					= "Blizzard Artifact",
		categories				= "player",
		effect_desc				= "Upon taking ice damage, or taking direct damage from freezing gas or freezing liquid, you \nbecome ice charged, and the first spell you cast after it happens will cause you to unleash a barrage of iceballs.",
		effect_desc_short		= "Casting a spell after taking ice damage unleashes a barrage of iceballs.",
        precursor_types			= "magic_artifact,effect_stone",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		ondamaged_function		= function(damage, message, entity_thats_responsible, is_fatal, projectile_thats_responsible )
			if(isRelicOwned("elemental_charge_artifact_blizzard") >= 1) then
				if(message == "$damage_ice" or (string.find(message, "damage from material") and string.find(message, "freezing"))) then
					if(not entityHasSimplifiedInternalVariable(GetUpdatedEntityID(), "iceball_barrage")) then
						addNewSimplifiedInternalVariable(GetUpdatedEntityID(), "iceball_barrage")
						addSpriteEffect(GetUpdatedEntityID(), "mods/ArcaneReliquary/files/other_gfx/ice_charge.xml", 17, 22, -1, 0.7, "ice_charge_effect")
						addEntityAudioLoop(GetUpdatedEntityID(), "data/audio/Desktop/animals.bank", "animals/thundermage/movement_loop", "ice_charge_effect")
					end
				end
			end
		end,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("elemental_charge_artifact_blizzard") >= 1) then
				if(entityHasSimplifiedInternalVariable(GetUpdatedEntityID(), "iceball_barrage")) then
					
					-- loads iceballs barrage and removes tag and sprite effect
					local player_x, player_y = EntityGetTransform(getPlayerEntity())
					local projectile_id = EntityLoad(getLoadOnExplosion("launch_circular_projectiles_iceballs").path, player_x, player_y)
					removeSimplifiedInternalVariable(GetUpdatedEntityID(), "iceball_barrage")
					removeSpriteEffect(GetUpdatedEntityID(), "ice_charge_effect")
					removeAudioLoop(GetUpdatedEntityID(), "ice_charge_effect")
					
					-- gives temporary immunity to explosions
					local game_effect_comp = GetGameEffectLoadTo( getPlayerEntity(), "PROTECTION_EXPLOSION", true )
					if game_effect_comp ~= nil then
						ComponentSetValue( game_effect_comp, "frames", "90" )
					end
				end
			end
		end,
		ondrop_function			= function()
			-- removes charge effect
			removeSimplifiedInternalVariable(getPlayerEntity(), "iceball_barrage")
			removeSpriteEffect(getPlayerEntity(), "ice_charge_effect")
			removeAudioLoop(getPlayerEntity(), "ice_charge_effect")
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "absorbing_crystal,frost_artifact",
	},
	{
		name 					= "elemental_charge_artifact_radioactive",
		ui_name					= "Radioactive Artifact",
		categories				= "player,materials",
		effect_desc				= "Upon taking radiation damage, or taking direct damage from radioactive \nmaterials, you become radioactively charged, and the first spell you cast after it happens will cause you to unleash a barrage of radioactive fireballs.",
		effect_desc_short		= "Casting a spell after taking radiation damage unleashes a barrage of radioactive fireballs.",
        precursor_types			= "magic_artifact,effect_stone",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		ondamaged_function		= function(damage, message, entity_thats_responsible, is_fatal, projectile_thats_responsible )
			if(isRelicOwned("elemental_charge_artifact_radioactive") >= 1) then
				if(message == "$damage_radioactive" or (string.find(message, "damage from material") and string.find(message, "radioactive"))) then
					if(not entityHasSimplifiedInternalVariable(GetUpdatedEntityID(), "radioactive_fireball_barrage")) then
						addNewSimplifiedInternalVariable(GetUpdatedEntityID(), "radioactive_fireball_barrage")
						addSpriteEffect(GetUpdatedEntityID(), "mods/ArcaneReliquary/files/other_gfx/radioactive_charge.xml", 17, 22, -1, 0.7, "radioactive_charge_effect")
						addEntityAudioLoop(GetUpdatedEntityID(), "data/audio/Desktop/animals.bank", "animals/firemage/burning_loop", "radioactive_charge_effect")
					end
				end
			end
		end,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("elemental_charge_artifact_radioactive") >= 1) then
				if(entityHasSimplifiedInternalVariable(GetUpdatedEntityID(), "radioactive_fireball_barrage")) then
					
					-- loads fireballs barrage and removes tag and sprite effect
					local player_x, player_y = EntityGetTransform(getPlayerEntity())
					local projectile_id = EntityLoad(getLoadOnExplosion("launch_circular_projectiles_fireballs_radioactive").path, player_x, player_y)
					removeSimplifiedInternalVariable(GetUpdatedEntityID(), "radioactive_fireball_barrage")
					removeSpriteEffect(GetUpdatedEntityID(), "radioactive_charge_effect")
					removeAudioLoop(GetUpdatedEntityID(), "radioactive_charge_effect")
					
					-- gives temporary immunity to radiation and explosions
					local game_effect_comp = GetGameEffectLoadTo( getPlayerEntity(), "PROTECTION_RADIOACTIVITY", true )
					if game_effect_comp ~= nil then
						ComponentSetValue( game_effect_comp, "frames", "90" )
					end
					
					local game_effect_comp = GetGameEffectLoadTo( getPlayerEntity(), "PROTECTION_EXPLOSION", true )
					if game_effect_comp ~= nil then
						ComponentSetValue( game_effect_comp, "frames", "90" )
					end
				end
			end
		end,
		ondrop_function			= function()
			-- removes charge effect
			removeSimplifiedInternalVariable(getPlayerEntity(), "radioactive_fireball_barrage")
			removeSpriteEffect(getPlayerEntity(), "radioactive_charge_effect")
			removeAudioLoop(getPlayerEntity(), "radioactive_charge_effect")
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "elemental_charge_artifact_infernal,uranium_ingot",
	},
	{
		name 					= "elemental_charge_artifact_pungent",
		ui_name					= "Pungent Artifact",
		categories				= "player,materials",
		effect_desc				= "Upon taking direct damage from acid, you \nbecome acid charged, and the first spell you cast after it happens will cause you to unleash a barrage of acid orbs.",
		effect_desc_short		= "Casting a spell after taking damage from acid unleashes a barrage of acid orbs.",
        precursor_types			= "magic_artifact,effect_stone",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		ondamaged_function		= function(damage, message, entity_thats_responsible, is_fatal, projectile_thats_responsible )
			if(isRelicOwned("elemental_charge_artifact_pungent") >= 1) then
				if(string.find(message, "damage from material") and string.find(message, "acid")) then
					if(not entityHasSimplifiedInternalVariable(GetUpdatedEntityID(), "magic_orb_acid_barrage")) then
						addNewSimplifiedInternalVariable(GetUpdatedEntityID(), "magic_orb_acid_barrage")
						addSpriteEffect(GetUpdatedEntityID(), "mods/ArcaneReliquary/files/other_gfx/material_charge_acid.xml", 17, 22, -1, 0.7, "acid_charge_effect")
						addEntityAudioLoop(GetUpdatedEntityID(), "data/audio/Desktop/animals.bank", "animals/slimy_small/movement_loop", "acid_charge_effect")
					end
				end
			end
		end,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("elemental_charge_artifact_pungent") >= 1) then
				if(entityHasSimplifiedInternalVariable(GetUpdatedEntityID(), "magic_orb_acid_barrage")) then
					
					-- loads iceballs barrage and removes tag and sprite effect
					local player_x, player_y = EntityGetTransform(getPlayerEntity())
					local projectile_id = EntityLoad(getLoadOnExplosion("launch_circular_projectiles_magic_orbs_acid").path, player_x, player_y)
					removeSimplifiedInternalVariable(GetUpdatedEntityID(), "magic_orb_acid_barrage")
					removeSpriteEffect(GetUpdatedEntityID(), "acid_charge_effect")
					removeAudioLoop(GetUpdatedEntityID(), "acid_charge_effect")
				end
			end
		end,
		ondrop_function			= function()
			-- removes charge effect
			removeSimplifiedInternalVariable(getPlayerEntity(), "magic_orb_acid_barrage")
			removeSpriteEffect(getPlayerEntity(), "acid_charge_effect")
			removeAudioLoop(getPlayerEntity(), "acid_charge_effect")
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "absorbing_crystal,toxic_gland",
	},
	{
		name 					= "elemental_charge_artifact_vile",
		ui_name					= "Vile Artifact",
		categories				= "player,materials",
		effect_desc				= "Upon taking direct damage from poison, you \nbecome poison charged, and the first spell you cast after it happens will cause you to unleash a barrage of poison orbs.",
		effect_desc_short		= "Casting a spell after taking damage from poison unleashes a barrage of poison orbs.",
        precursor_types			= "magic_artifact,effect_stone",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		ondamaged_function		= function(damage, message, entity_thats_responsible, is_fatal, projectile_thats_responsible )
			if(isRelicOwned("elemental_charge_artifact_vile") >= 1) then
				if(string.find(message, "damage from material") and string.find(message, "Poison")) then
					if(not entityHasSimplifiedInternalVariable(GetUpdatedEntityID(), "magic_orb_poison_barrage")) then
						addNewSimplifiedInternalVariable(GetUpdatedEntityID(), "magic_orb_poison_barrage")
						addSpriteEffect(GetUpdatedEntityID(), "mods/ArcaneReliquary/files/other_gfx/material_charge_poison.xml", 17, 22, -1, 0.7, "poison_charge_effect")
						addEntityAudioLoop(GetUpdatedEntityID(), "data/audio/Desktop/animals.bank", "animals/slimy_small/movement_loop", "poison_charge_effect")
					end
				end
			end
		end,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("elemental_charge_artifact_vile") >= 1) then
				if(entityHasSimplifiedInternalVariable(GetUpdatedEntityID(), "magic_orb_poison_barrage")) then
					
					-- loads iceballs barrage and removes tag and sprite effect
					local player_x, player_y = EntityGetTransform(getPlayerEntity())
					local projectile_id = EntityLoad(getLoadOnExplosion("launch_circular_projectiles_magic_orbs_poison").path, player_x, player_y)
					removeSimplifiedInternalVariable(GetUpdatedEntityID(), "magic_orb_poison_barrage")
					removeSpriteEffect(GetUpdatedEntityID(), "poison_charge_effect")
					removeAudioLoop(GetUpdatedEntityID(), "poison_charge_effect")
				end
			end
		end,
		ondrop_function			= function()
			-- removes charge effect
			removeSimplifiedInternalVariable(getPlayerEntity(), "magic_orb_poison_barrage")
			removeSpriteEffect(getPlayerEntity(), "poison_charge_effect")
			removeAudioLoop(getPlayerEntity(), "poison_charge_effect")
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "absorbing_crystal,vile_stone",
	},
	{
		name 					= "elemental_charge_artifact_igneous",
		ui_name					= "Igneous Artifact",
		categories				= "player,materials",
		effect_desc				= "Upon taking direct damage from lava, you \nbecome lava charged, and the first spell you cast after it happens will cause you to unleash a barrage of lava orbs.",
		effect_desc_short		= "Casting a spell after taking damage from lava unleashes a barrage of lava orbs.",
        precursor_types			= "magic_artifact,effect_stone",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		ondamaged_function		= function(damage, message, entity_thats_responsible, is_fatal, projectile_thats_responsible )
			if(isRelicOwned("elemental_charge_artifact_igneous") >= 1) then
				if(string.find(message, "damage from material") and string.find(message, "lava")) then
					if(not entityHasSimplifiedInternalVariable(GetUpdatedEntityID(), "magic_orb_lava_barrage")) then
						addNewSimplifiedInternalVariable(GetUpdatedEntityID(), "magic_orb_lava_barrage")
						addSpriteEffect(GetUpdatedEntityID(), "mods/ArcaneReliquary/files/other_gfx/material_charge_lava.xml", 17, 22, -1, 0.7, "lava_charge_effect")
						addEntityAudioLoop(GetUpdatedEntityID(), "data/audio/Desktop/animals.bank", "animals/slimy_small/movement_loop", "lava_charge_effect")
					end
				end
			end
		end,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("elemental_charge_artifact_igneous") >= 1) then
				if(entityHasSimplifiedInternalVariable(GetUpdatedEntityID(), "magic_orb_lava_barrage")) then
					
					-- loads iceballs barrage and removes tag and sprite effect
					local player_x, player_y = EntityGetTransform(getPlayerEntity())
					local projectile_id = EntityLoad(getLoadOnExplosion("launch_circular_projectiles_magic_orbs_lava").path, player_x, player_y)
					removeSimplifiedInternalVariable(GetUpdatedEntityID(), "magic_orb_lava_barrage")
					removeSpriteEffect(GetUpdatedEntityID(), "lava_charge_effect")
					removeAudioLoop(GetUpdatedEntityID(), "lava_charge_effect")
				end
			end
		end,
		ondrop_function			= function()
			-- removes charge effect
			removeSimplifiedInternalVariable(getPlayerEntity(), "magic_orb_lava_barrage")
			removeSpriteEffect(getPlayerEntity(), "lava_charge_effect")
			removeAudioLoop(getPlayerEntity(), "lava_charge_effect")
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "absorbing_crystal,obsidian",
	},
	{
		name 					= "diamond_ring_green",
		ui_name					= "Green Diamond Ring",
		categories				= "projectiles,status_effects",
		effect_desc				= "Your spells have a 30% chance of creating a mystic target on \ncreatures you hit, making them summon acid orbs towards themselves.",
		effect_desc_short		= "Your spells create mystic targets on creatures, summoning acid orbs towards them.",
        precursor_types			= "magic_artifact,ring",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("diamond_ring_green") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 30 + (5 * getRelicLevel("diamond_ring_green"))
				if(chance >= 1 and chance <= odds) then
					attachLoadEffectOntoEntitiestoProjectile(projectile_entity_id, getCustomEffect("effect_projectile_target_orb_acid").path, "spark_green")
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "elemental_band,acid_gem",
	},
	{
		name 					= "diamond_ring_orange",
		ui_name					= "Orange Diamond Ring",
		categories				= "projectiles,status_effects",
		effect_desc				= "Your spells have a 30% chance of creating a mystic target on \ncreatures you hit, making them summon lava orbs towards themselves.",
		effect_desc_short		= "Your spells create mystic targets on creatures, summoning lava orbs towards them.",
        precursor_types			= "magic_artifact,ring",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("diamond_ring_orange") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 30 + (5 * getRelicLevel("diamond_ring_orange"))
				if(chance >= 1 and chance <= odds) then
					attachLoadEffectOntoEntitiestoProjectile(projectile_entity_id, getCustomEffect("effect_projectile_target_orb_lava").path, "lava")
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "elemental_band,lava_gem",
	},
	{
		name 					= "diamond_ring_purple",
		ui_name					= "Purple Diamond Ring",
		categories				= "projectiles,status_effects",
		effect_desc				= "Your spells have a 30% chance of creating a mystic target on \ncreatures you hit, making them summon poison orbs towards themselves.",
		effect_desc_short		= "Your spells create mystic targets on creatures, summoning poison orbs towards them.",
        precursor_types			= "magic_artifact,ring",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("diamond_ring_purple") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 30 + (5 * getRelicLevel("diamond_ring_purple"))
				if(chance >= 1 and chance <= odds) then
					attachLoadEffectOntoEntitiestoProjectile(projectile_entity_id, getCustomEffect("effect_projectile_target_orb_poison").path, "spark_purple_bright")
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "elemental_band,poison_gem",
	},
	{
		name 					= "diamond_ring_red",
		ui_name					= "Red Diamond Ring",
		categories				= "projectiles,status_effects",
		effect_desc				= "Your spells have a 30% chance of creating a mystic target on \ncreatures you hit, making them summon explosive magic stars towards themselves.",
		effect_desc_short		= "Your spells create mystic targets on creatures, summoning explosive magic stars towards them.",
        precursor_types			= "magic_artifact,ring",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("diamond_ring_red") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 30 + (5 * getRelicLevel("diamond_ring_red"))
				if(chance >= 1 and chance <= odds) then
					attachLoadEffectOntoEntitiestoProjectile(projectile_entity_id, getCustomEffect("effect_projectile_target_magic_star_explosive").path, "spark_red")
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "elemental_band,stone_hand_explosive",
	},
	{
		name 					= "diamond_ring_yellow",
		ui_name					= "Yellow Diamond Ring",
		categories				= "projectiles,status_effects",
		effect_desc				= "Your spells have a 30% chance of creating a mystic target on \ncreatures you hit, making them summon urine orbs towards themselves.",
		effect_desc_short		= "Your spells create mystic targets on creatures, summoning urine orbs towards them.",
        precursor_types			= "magic_artifact,ring",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("diamond_ring_yellow") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 30 + (5 * getRelicLevel("diamond_ring_yellow"))
				if(chance >= 1 and chance <= odds) then
					attachLoadEffectOntoEntitiestoProjectile(projectile_entity_id, getCustomEffect("effect_projectile_target_orb_urine").path, "spark_yellow")
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "elemental_band,urine_gem",
	},
	{
		name 					= "diamond_ring_water_blue",
		ui_name					= "Water Blue Diamond Ring",
		categories				= "projectiles,status_effects",
		effect_desc				= "Your spells have a 30% chance of creating a mystic target on \ncreatures you hit, making them summon water orbs towards themselves.",
		effect_desc_short		= "Your spells create mystic targets on creatures, summoning water orbs towards them.",
        precursor_types			= "magic_artifact,ring",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("diamond_ring_water_blue") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 30 + (5 * getRelicLevel("diamond_ring_water_blue"))
				if(chance >= 1 and chance <= odds) then
					attachLoadEffectOntoEntitiestoProjectile(projectile_entity_id, getCustomEffect("effect_projectile_target_orb_water").path, "spark_blue")
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "elemental_band,water_gem",
	},
	{
		name 					= "diamond_ring_blood_red",
		ui_name					= "Blood Red Diamond Ring",
		categories				= "projectiles,status_effects",
		effect_desc				= "Your spells have a 30% chance of creating a mystic target on \ncreatures you hit, making them summon corrosive blood orbs towards themselves.",
		effect_desc_short		= "Your spells create mystic targets on creatures, summoning corrosive blood orbs towards them.",
        precursor_types			= "magic_artifact,ring",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("diamond_ring_blood_red") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 30 + (5 * getRelicLevel("diamond_ring_blood_red"))
				if(chance >= 1 and chance <= odds) then
					attachLoadEffectOntoEntitiestoProjectile(projectile_entity_id, getCustomEffect("effect_projectile_target_orb_corrosive_blood").path, "blood")
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "elemental_band,corrosive_blood_gem",
	},
	{
		name 					= "diamond_ring_pink",
		ui_name					= "Pink Diamond Ring",
		categories				= "projectiles,status_effects",
		effect_desc				= "Your spells have a 30% chance of creating a mystic target on \ncreatures you hit, making them summon polymorphine orbs towards themselves.",
		effect_desc_short		= "Your spells create mystic targets on creatures, summoning polymorphine orbs towards them.",
        precursor_types			= "magic_artifact,ring",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("diamond_ring_pink") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 30 + (5 * getRelicLevel("diamond_ring_pink"))
				if(chance >= 1 and chance <= odds) then
					attachLoadEffectOntoEntitiestoProjectile(projectile_entity_id, getCustomEffect("effect_projectile_target_orb_poly").path, "plasma_fading_pink")
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "elemental_band,poly_gem",
	},
	{
		name 					= "diamond_ring_wine_red",
		ui_name					= "Wine Red Diamond Ring",
		categories				= "projectiles,status_effects",
		effect_desc				= "Your spells have a 30% chance of creating a mystic target on \ncreatures you hit, making them summon bloodbombs towards themselves.",
		effect_desc_short		= "Your spells create mystic targets on creatures, summoning bloodbombs towards them.",
        precursor_types			= "magic_artifact,ring",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("diamond_ring_wine_red") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 30 + (5 * getRelicLevel("diamond_ring_wine_red"))
				if(chance >= 1 and chance <= odds) then
					attachLoadEffectOntoEntitiestoProjectile(projectile_entity_id, getCustomEffect("effect_projectile_target_bloodbomb").path, "blood")
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "elemental_band,coagulated_blood",
	},
	{
		name 					= "diamond_ring_lime_green",
		ui_name					= "Lime Green Diamond Ring",
		categories				= "projectiles,status_effects",
		effect_desc				= "Your spells have a 30% chance of creating a mystic target on \ncreatures you hit, making them summon slapperbombs towards themselves.",
		effect_desc_short		= "Your spells create mystic targets on creatures, summoning slapperbombs towards them.",
        precursor_types			= "magic_artifact,ring",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("diamond_ring_lime_green") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 30 + (5 * getRelicLevel("diamond_ring_lime_green"))
				if(chance >= 1 and chance <= odds) then
					attachLoadEffectOntoEntitiestoProjectile(projectile_entity_id, getCustomEffect("effect_projectile_target_slapperbomb").path, "plasma_fading_green")
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "elemental_band,fly_swatter",
	},
	{
		name 					= "diamond_ring_fire_orange",
		ui_name					= "Fire Orange Diamond Ring",
		categories				= "projectiles,status_effects",
		effect_desc				= "Your spells have a 30% chance of creating a mystic target on \ncreatures you hit, making them summon firebombs towards themselves.",
		effect_desc_short		= "Your spells create mystic targets on creatures, summoning firebombs towards them.",
        precursor_types			= "magic_artifact,ring",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("diamond_ring_fire_orange") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 30 + (5 * getRelicLevel("diamond_ring_fire_orange"))
				if(chance >= 1 and chance <= odds) then
					attachLoadEffectOntoEntitiestoProjectile(projectile_entity_id, getCustomEffect("effect_projectile_target_firebomb").path, "fire")
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "elemental_band,solar_artifact",
	},
	{
		name 					= "diamond_ring_cyan",
		ui_name					= "Cyan Diamond Ring",
		categories				= "projectiles,status_effects",
		effect_desc				= "Your spells have a 30% chance of creating a mystic target on \ncreatures you hit, making them summon freezebombs towards themselves.",
		effect_desc_short		= "Your spells create mystic targets on creatures, summoning freezebombs towards them.",
        precursor_types			= "magic_artifact,ring",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("diamond_ring_cyan") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 30 + (5 * getRelicLevel("diamond_ring_cyan"))
				if(chance >= 1 and chance <= odds) then
					attachLoadEffectOntoEntitiestoProjectile(projectile_entity_id, getCustomEffect("effect_projectile_target_freezebomb").path, "spark_blue")
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "elemental_band,popsicle",
	},
	{
		name 					= "diamond_ring_dark_blue",
		ui_name					= "Dark Blue Diamond Ring",
		categories				= "projectiles,status_effects",
		effect_desc				= "Your spells have a 30% chance of creating a mystic target on \ncreatures you hit, making them summon electrobombs towards themselves.",
		effect_desc_short		= "Your spells create mystic targets on creatures, summoning electrobombs towards them.",
        precursor_types			= "magic_artifact,ring",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("diamond_ring_dark_blue") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 30 + (5 * getRelicLevel("diamond_ring_dark_blue"))
				if(chance >= 1 and chance <= odds) then
					attachLoadEffectOntoEntitiestoProjectile(projectile_entity_id, getCustomEffect("effect_projectile_target_electrobomb").path, "spark_blue_dark")
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "elemental_band,thunder_hat",
	},
	{
		name 					= "corroding_necklace",
		ui_name					= "Corroding Necklace",
		categories				= "projectiles,status_effects",
		effect_desc				= "Your spells have a 30% chance of inducing enemies they hit to \nsummon spectral wands that target and shoot acid beams on them.",
		effect_desc_short		= "Your spells induce enemies to summon spectral wands that target and shoot acid beams on them.",
        precursor_types			= "magic_artifact,necklace",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("corroding_necklace") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 30 + (5 * getRelicLevel("corroding_necklace"))
				if(chance >= 1 and chance <= odds) then
					attachLoadEffectOntoEntitiestoProjectile(projectile_entity_id, getCustomEffect("effect_spectral_wand_target_acid").path, "acid")
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "cursed_necklace,acid_gem",
	},
	{
		name 					= "venomous_wand",
		ui_name					= "Venomous Wand",
		categories				= "projectiles,materials",
		effect_desc				= "When you cast a spell, you have a 30% chance of summoning a purple \nspectral wand that shoots a powerful poison laser.",
		effect_desc_short		= "Your spells summon a spectral wand that shoots purple poison lasers.",
        precursor_types			= "magic_artifact",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("venomous_wand") >= 1) then
				if(not projectileHasProjectileDefinitionPath(projectile_entity_id)) then
					SetRandomSeed( GameGetFrameNum(), math.random(1,50))
					local chance = Random(1,100)
					local odds = 30 + (5 * getRelicLevel("venomous_wand"))
					if(chance >= 1 and chance <= odds) then
						local x, y = EntityGetTransform(projectile_entity_id)
						local laser_shooter_id = EntityLoad("mods/ArcaneReliquary/files/entity_definitions/custom_projectiles/megalaser_poison.xml", x, y)
						addNewSimplifiedInternalVariable( laser_shooter_id, "relic_effect_projectile")
						local projectile_vel_x, projectile_vel_y = getProjectileVelocitiesandGravity(projectile_entity_id)
						setProjectileVelocitiesandGravity(laser_shooter_id, projectile_vel_x, projectile_vel_y)
					end
				end
			end
		end
	},
	{
		name 					= "magmatic_wand",
		ui_name					= "Magmatic Wand",
		categories				= "projectiles,materials",
		effect_desc				= "When you cast a spell, you have a 30% chance of summoning an orange \nspectral wand that shoots a powerful lava laser.",
		effect_desc_short		= "Your spells summon a spectral wand that shoots orange lava lasers.",
        precursor_types			= "magic_artifact",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("magmatic_wand") >= 1) then
				if(not projectileHasProjectileDefinitionPath(projectile_entity_id)) then
					SetRandomSeed( GameGetFrameNum(), math.random(1,50))
					local chance = Random(1,100)
					local odds = 30 + (5 * getRelicLevel("magmatic_wand"))
					if(chance >= 1 and chance <= odds) then
						local x, y = EntityGetTransform(projectile_entity_id)
						local laser_shooter_id = EntityLoad("mods/ArcaneReliquary/files/entity_definitions/custom_projectiles/megalaser_lava.xml", x, y)
						addNewSimplifiedInternalVariable( laser_shooter_id, "relic_effect_projectile")
						local projectile_vel_x, projectile_vel_y = getProjectileVelocitiesandGravity(projectile_entity_id)
						setProjectileVelocitiesandGravity(laser_shooter_id, projectile_vel_x, projectile_vel_y)
					end
				end
			end
		end
	},
	{
		name 					= "sharp_wand",
		ui_name					= "Sharp Wand",
		categories				= "projectiles,materials",
		effect_desc				= "When you cast a spell, you have a 30% chance of summoning a white \nspectral wand that shoots a powerful slice laser.",
		effect_desc_short		= "Your spells summon a spectral wand that shoots white slice lasers.",
        precursor_types			= "magic_artifact",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("sharp_wand") >= 1) then
				if(not projectileHasProjectileDefinitionPath(projectile_entity_id)) then
					SetRandomSeed( GameGetFrameNum(), math.random(1,50))
					local chance = Random(1,100)
					local odds = 30 + (5 * getRelicLevel("sharp_wand"))
					if(chance >= 1 and chance <= odds) then
						local x, y = EntityGetTransform(projectile_entity_id)
						local laser_shooter_id = EntityLoad("mods/ArcaneReliquary/files/entity_definitions/custom_projectiles/megalaser_slice.xml", x, y)
						addNewSimplifiedInternalVariable( laser_shooter_id, "relic_effect_projectile")
						local projectile_vel_x, projectile_vel_y = getProjectileVelocitiesandGravity(projectile_entity_id)
						setProjectileVelocitiesandGravity(laser_shooter_id, projectile_vel_x, projectile_vel_y)
					end
				end
			end
		end
	},
	{
		name 					= "elemental_charge_artifact_wet",
		ui_name					= "Wet Artifact",
		categories				= "player,materials",
		effect_desc				= "Upon getting wet, you \nbecome water charged, and the first spell you cast after it happens will cause you to unleash a barrage of water orbs.",
		effect_desc_short		= "Casting a spell after getting wet unleashes a barrage of water orbs.",
        precursor_types			= "magic_artifact,effect_stone",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		overtimeslow_function	= function()
			if(isRelicOwned("elemental_charge_artifact_wet") >= 1) then
				if(GameGetGameEffectCount(GetUpdatedEntityID(), "WET") > 0) then
					if(not entityHasSimplifiedInternalVariable(GetUpdatedEntityID(), "magic_orb_water_barrage")) then
						addNewSimplifiedInternalVariable(GetUpdatedEntityID(), "magic_orb_water_barrage")
						addSpriteEffect(GetUpdatedEntityID(), "mods/ArcaneReliquary/files/other_gfx/material_charge_water.xml", 17, 22, -1, 0.7, "water_charge_effect")
						addEntityAudioLoop(GetUpdatedEntityID(), "data/audio/Desktop/animals.bank", "animals/slimy_small/movement_loop", "water_charge_effect")
					end
				end
			end
		end,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("elemental_charge_artifact_wet") >= 1) then
				if(entityHasSimplifiedInternalVariable(GetUpdatedEntityID(), "magic_orb_water_barrage")) then
					
					-- loads iceballs barrage and removes tag and sprite effect
					local player_x, player_y = EntityGetTransform(getPlayerEntity())
					local projectile_id = EntityLoad(getLoadOnExplosion("launch_circular_projectiles_magic_orbs_water").path, player_x, player_y)
					removeSimplifiedInternalVariable(GetUpdatedEntityID(), "magic_orb_water_barrage")
					removeSpriteEffect(GetUpdatedEntityID(), "water_charge_effect")
					removeAudioLoop(GetUpdatedEntityID(), "water_charge_effect")
				end
			end
		end,
		ondrop_function			= function()
			-- removes charge effect
			removeSimplifiedInternalVariable(getPlayerEntity(), "magic_orb_water_barrage")
			removeSpriteEffect(getPlayerEntity(), "water_charge_effect")
			removeAudioLoop(getPlayerEntity(), "water_charge_effect")
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "absorbing_crystal,water_gem",
	},
	{
		name 					= "elemental_charge_artifact_uric",
		ui_name					= "Uric Artifact",
		categories				= "player,materials",
		effect_desc				= "Upon getting covered in urine, you \nbecome urine charged, and the first spell you cast after it happens will cause you to unleash a barrage of urine orbs.",
		effect_desc_short		= "Casting a spell after getting covered in urine unleashes a barrage of urine orbs.",
        precursor_types			= "magic_artifact,effect_stone",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		overtimeslow_function	= function()
			if(isRelicOwned("elemental_charge_artifact_uric") >= 1) then
				if(GameGetGameEffectCount(GetUpdatedEntityID(), "JARATE") > 0) then
					if(not entityHasSimplifiedInternalVariable(GetUpdatedEntityID(), "magic_orb_urine_barrage")) then
						addNewSimplifiedInternalVariable(GetUpdatedEntityID(), "magic_orb_urine_barrage")
						addSpriteEffect(GetUpdatedEntityID(), "mods/ArcaneReliquary/files/other_gfx/material_charge_urine.xml", 17, 22, -1, 0.7, "urine_charge_effect")
						addEntityAudioLoop(GetUpdatedEntityID(), "data/audio/Desktop/animals.bank", "animals/slimy_small/movement_loop", "urine_charge_effect")
					end
				end
			end
		end,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("elemental_charge_artifact_uric") >= 1) then
				if(entityHasSimplifiedInternalVariable(GetUpdatedEntityID(), "magic_orb_urine_barrage")) then
					
					-- loads iceballs barrage and removes tag and sprite effect
					local player_x, player_y = EntityGetTransform(getPlayerEntity())
					local projectile_id = EntityLoad(getLoadOnExplosion("launch_circular_projectiles_magic_orbs_urine").path, player_x, player_y)
					removeSimplifiedInternalVariable(GetUpdatedEntityID(), "magic_orb_urine_barrage")
					removeSpriteEffect(GetUpdatedEntityID(), "urine_charge_effect")
					removeAudioLoop(GetUpdatedEntityID(), "urine_charge_effect")
				end
			end
		end,
		ondrop_function			= function()
			-- removes charge effect
			removeSimplifiedInternalVariable(getPlayerEntity(), "magic_orb_urine_barrage")
			removeSpriteEffect(getPlayerEntity(), "urine_charge_effect")
			removeAudioLoop(getPlayerEntity(), "urine_charge_effect")
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "absorbing_crystal,urine_gem",
	},
	{
		name 					= "elemental_charge_artifact_bloody",
		ui_name					= "Bloody Artifact",
		categories				= "player,materials",
		effect_desc				= "Upon getting covered in blood, you \nbecome blood charged, and the first spell you cast after it happens will cause you to unleash a barrage of corrosive blood orbs.",
		effect_desc_short		= "Casting a spell after getting covered in blood unleashes a barrage of corrosive blood orbs.",
        precursor_types			= "magic_artifact,effect_stone",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		overtimeslow_function	= function()
			if(isRelicOwned("elemental_charge_artifact_bloody") >= 1) then
				if(GameGetGameEffectCount(GetUpdatedEntityID(), "BLOODY") > 0) then
					if(not entityHasSimplifiedInternalVariable(GetUpdatedEntityID(), "magic_orb_corrosive_blood_barrage")) then
						addNewSimplifiedInternalVariable(GetUpdatedEntityID(), "magic_orb_corrosive_blood_barrage")
						addSpriteEffect(GetUpdatedEntityID(), "mods/ArcaneReliquary/files/other_gfx/material_charge_blood.xml", 17, 22, -1, 0.7, "blood_charge_effect")
						addEntityAudioLoop(GetUpdatedEntityID(), "data/audio/Desktop/animals.bank", "animals/slimy_small/movement_loop", "blood_charge_effect")
					end
				end
			end
		end,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("elemental_charge_artifact_bloody") >= 1) then
				if(entityHasSimplifiedInternalVariable(GetUpdatedEntityID(), "magic_orb_corrosive_blood_barrage")) then
					
					-- loads iceballs barrage and removes tag and sprite effect
					local player_x, player_y = EntityGetTransform(getPlayerEntity())
					local projectile_id = EntityLoad(getLoadOnExplosion("launch_circular_projectiles_magic_orbs_corrosive_blood").path, player_x, player_y)
					removeSimplifiedInternalVariable(GetUpdatedEntityID(), "magic_orb_corrosive_blood_barrage")
					removeSpriteEffect(GetUpdatedEntityID(), "blood_charge_effect")
					removeAudioLoop(GetUpdatedEntityID(), "blood_charge_effect")
				end
			end
		end,
		ondrop_function			= function()
			-- removes charge effect
			removeSimplifiedInternalVariable(getPlayerEntity(), "magic_orb_corrosive_blood_barrage")
			removeSpriteEffect(getPlayerEntity(), "blood_charge_effect")
			removeAudioLoop(getPlayerEntity(), "blood_charge_effect")
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "absorbing_crystal,corrosive_blood_gem",
	},
	{
		name 					= "hallowed_bomb",
		ui_name					= "Hallowed Bomb",
		categories				= "player",
		effect_desc				= "Enemies you kick are transformed into holy bombs. \nIt also synergizes with Sniper Scope relic effect.",
		effect_desc_short		= "Enemies you kick are transformed into holy bombs.",
        precursor_types			= "gadget",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onenemykicked_function	= function(entity_who_kicked )
			if(isRelicOwned("hallowed_bomb") >= 1) then
				LoadGameEffectEntityTo( GetUpdatedEntityID(), getCustomEffect("effect_convert_to_holy_bomb").path )
			end
		end,
		onextendedkick_function	= function( entity_id, extended_kick_id)
			if(isRelicOwned("hallowed_bomb") >= 1) then
				attachLoadEffectOntoEntitiestoProjectile(extended_kick_id, getCustomEffect("effect_convert_to_holy_bomb").path, "spark_yellow")
			end
		end,
	},
	{
		name 					= "apocalypse_bomb",
		ui_name					= "Apocalypse Bomb",
		categories				= "player",
		effect_desc				= "When you take projectile, melee or explosion damage, there's a 50% chance all enemy \ncreatures (except the one that attacked you) in the vicinity will be transformed into holy bombs.",
		effect_desc_short		= "Taking damage transforms enemies around into holy bombs.",
        precursor_types			= "magic_artifact",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		ondamaged_function		= function(damage, message, entity_thats_responsible, is_fatal, projectile_thats_responsible )
			if(isRelicOwned("apocalypse_bomb") >= 1) then
				if(message == "$damage_projectile" or message == "$damage_explosion" or message == "$damage_melee") then
					SetRandomSeed( GameGetFrameNum(), math.random(1,50))
					local chance = Random(1,100)
					local odds = 50 + (5 * getRelicLevel("apocalypse_bomb"))
					if(chance >= 1 and chance <= odds) then
						local player_x, player_y = EntityGetTransform(GetUpdatedEntityID())
						local enemies = EntityGetInRadiusWithTag( player_x, player_y, 200, "enemy" )
						for i, enemy_id in ipairs(enemies) do
							if(enemy_id ~= entity_thats_responsible) then
								LoadGameEffectEntityTo( enemy_id, getCustomEffect("effect_convert_to_holy_bomb").path )
							end
						end
					end
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "cursed_bomb,hallowed_bomb",
	},
	{
		name 					= "plutonium_ingot",
		ui_name					= "Plutonium Ingot",
		categories				= "projectiles,minions",
		effect_desc				= "Your spells have a 10% chance of being transformed into nukes mid-flight.",
		effect_desc_short		= "Your spells have a chance to be turned into nukes.",
        precursor_types			= "magic_artifact",
		upgrade_effect_desc		= "For each upgrade level, spawn chance increases by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("plutonium_ingot") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 10 + (5 * getRelicLevel("plutonium_ingot"))
				if(chance >= 1 and chance <= odds) then
					addNewLuaCallback(projectile_entity_id, "overtime", "mods/ArcaneReliquary/files/scripts/projectiles/destroy_and_spawn_nuke.lua", 5, false, "destroy_and_spawn_nuke")
				end
			end
		end,
	},
	{
		name 					= "poly_flask",
		ui_name					= "Poly Flask",
		categories				= "player",
		effect_desc				= "When you take projectile, melee or explosion damage, there's a 50% chance all enemy \ncreatures in the vicinity (including you) will be mutated into sheep.",
		effect_desc_short		= "Taking damage mutates enemies (and you) into sheep.",
        precursor_types			= "flask",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		ondamaged_function		= function(damage, message, entity_thats_responsible, is_fatal, projectile_thats_responsible )
			if(isRelicOwned("poly_flask") >= 1) then
				if(message == "$damage_projectile" or message == "$damage_explosion" or message == "$damage_melee") then
					SetRandomSeed( GameGetFrameNum(), math.random(1,50))
					local chance = Random(1,100)
					local odds = 50 + (5 * getRelicLevel("poly_flask"))
					if(chance >= 1 and chance <= odds) then
						-- mutates enemies around
						local player_x, player_y = EntityGetTransform(GetUpdatedEntityID())
						local enemies = EntityGetInRadiusWithTag( player_x, player_y, 200, "enemy" )
						for i, enemy_id in ipairs(enemies) do
							LoadGameEffectEntityTo( enemy_id, "data/entities/misc/effect_polymorph.xml" )
						end
						
						-- randomly mutates player
						if(Random(1,4) == 1) then
							LoadGameEffectEntityTo( GetUpdatedEntityID(), "data/entities/misc/effect_polymorph.xml" )
						end						
					end
				end
			end
		end,
	},
	{
		name 					= "chaotic_poly_flask",
		ui_name					= "Chaotic Poly Flask",
		categories				= "player",
		effect_desc				= "When you take projectile, melee or explosion damage, there's a 50% chance all enemy \ncreatures in the vicinity (including you) will be mutated into random creatures.",
		effect_desc_short		= "Taking damage mutates enemies (and you) into random creatures.",
        precursor_types			= "flask",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		ondamaged_function		= function(damage, message, entity_thats_responsible, is_fatal, projectile_thats_responsible )
			if(isRelicOwned("chaotic_poly_flask") >= 1) then
				if(message == "$damage_projectile" or message == "$damage_explosion" or message == "$damage_melee") then
					SetRandomSeed( GameGetFrameNum(), math.random(1,50))
					local chance = Random(1,100)
					local odds = 50 + (5 * getRelicLevel("chaotic_poly_flask"))
					if(chance >= 1 and chance <= odds) then
						-- mutates enemies around
						local player_x, player_y = EntityGetTransform(GetUpdatedEntityID())
						local enemies = EntityGetInRadiusWithTag( player_x, player_y, 200, "enemy" )
						for i, enemy_id in ipairs(enemies) do
							LoadGameEffectEntityTo( enemy_id, "data/entities/misc/effect_polymorph_random.xml" )
						end
						
						-- randomly mutates player
						if(Random(1,2) == 1) then
							LoadGameEffectEntityTo( GetUpdatedEntityID(), "data/entities/misc/effect_polymorph_random.xml" )
						end						
					end
				end
			end
		end,
	},
	{
		name 					= "leaking_nuclear_core",
		ui_name					= "Leaking Nuclear Core",
		categories				= "player",
		effect_desc				= "You gain immunity to radiation damage, but radioactive explosions happen around you randomly.",
		effect_desc_short		= "You gain immunity to radiation damage, but radioactive explosions happen around you randomly.",
        precursor_types			= "flask",
		upgrade_effect_desc		= "At level 10, big nuclear explosions happen instead.",
		is_upgradable 			= true,
		overtimeslow_function	= function()
			if(isRelicOwned("leaking_nuclear_core") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				
				if(Random(1,2) == 1) then
					local player_x, player_y = EntityGetTransform(GetUpdatedEntityID())
					if(getRelicLevel("leaking_nuclear_core") < 10) then
						EntityLoad("mods/ArcaneReliquary/files/entity_definitions/others/radioactive_explosion.xml", player_x + Random(-50,50), player_y + Random(-50,50))
					else
						EntityLoad("mods/ArcaneReliquary/files/entity_definitions/others/nuclear_explosion.xml", player_x + Random(-50,50), player_y + Random(-50,50))
					end
				end
			end
		end,
		onpickup_function		= function()
			
			-- adds radiation protection effect
			local game_effect_comp_id, game_effect_entity_id = GetGameEffectLoadTo( getPlayerEntity(), "PROTECTION_RADIOACTIVITY", true )
			if game_effect_comp_id ~= nil then
				ComponentSetValue2( game_effect_comp_id, "frames", -1)
				addNewSimplifiedInternalVariable(game_effect_entity_id, "protection_radioactivity_effect")
			end
		end,
		ondrop_function			= function()
			
			-- removes radiation protection effect
			local player_children = EntityGetAllChildren(getPlayerEntity())
			if player_children == nil then return end
			for i,child in ipairs(player_children) do
				if(entityHasSimplifiedInternalVariable(child, "protection_radioactivity_effect")) then
					EntityKill(child)
					break
				end
			end
		end,
	},
	{
		name 					= "blackhole_droid",
		ui_name					= "Blackhole Droid",
		categories				= "companions",
		effect_desc				= "A companion equipped with dark matter projector that creates black holes on impact.",
		effect_desc_short		= "A companion equipped with dark matter projector that creates black holes on impact.",
        precursor_types			= "companion",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			attachCompaniontoEntity(getPlayerEntity(), "blackhole_droid", "mortal")
		end,
		ondrop_function			= function()
			detachCompanionfromEntity(getPlayerEntity(), "blackhole_droid")
		end,
		requires_relic			= "alkaline_battery",
		combine_exclusive		= true,
		combine_requires_relics	= "rifle_droid,high_density_stone",
	},
	{
		name 					= "barrys_crest",
		ui_name					= "Barry's Crest",
		categories				= "player",
		effect_desc				= "When you kill an enemy with your spells, there's a 50% chance you'll gain a temporary movement speed boost.",
		effect_desc_short		= "Killing enemies gives you temporary movement speed boost.",
        precursor_types			= "magic_artifact,gadget",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onenemykill_function	= function(damage_type_bit_field, damage_message, entity_thats_responsible, drop_items)
			if(isRelicOwned("barrys_crest") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 50 + (5 * getRelicLevel("barrys_crest"))
				if(chance >= 1 and chance <= odds) then
					LoadGameEffectEntityTo( getPlayerEntity(), getCustomEffect("effect_movement_faster_short").path )
				end
			end
		end,
		excludes_relics_desc	= "It can't be used along with other movement speed boost related relics",
		excludes_relics_tags	= "movement_speed_boost",
	},
	{
		name 					= "kryptonite_chunk",
		ui_name					= "Kryptonite Chunk",
		categories				= "player",
		effect_desc				= "When you kill an enemy with your spells, there's a 50% chance you'll gain a temporary levitation speed boost.",
		effect_desc_short		= "Killing enemies gives you temporary levitation speed boost.",
        precursor_types			= "effect_stone,magic_artifact",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onenemykill_function	= function(damage_type_bit_field, damage_message, entity_thats_responsible, drop_items)
			if(isRelicOwned("kryptonite_chunk") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 50 + (5 * getRelicLevel("kryptonite_chunk"))
				if(chance >= 1 and chance <= odds) then
					LoadGameEffectEntityTo( getPlayerEntity(), getCustomEffect("effect_levitation_faster_short").path )
				end
			end
		end,
		excludes_relics_desc	= "It can't be used along with other fly speed boost related relics",
		excludes_relics_tags	= "fly_speed_boost",
	},
	{
		name 					= "cosmic_stone",
		ui_name					= "Cosmic Stone",
		categories				= "player",
		effect_desc				= "Your spells are destroyed once cast, but your kicks now deal x100 damage, you move and fly \nfaster, and 50% of hp lost when taking damage from all sources is replenished.",
		effect_desc_short		= "Your spells get destroyed once cast, but you kick harder, move and fly faster and absorb part of damage taken.",
        precursor_types			= "effect_stone,magic_artifact",
		upgrade_effect_desc		= "For each upgrade level, 2% more hp is replenished on damaged and x5 multiplier is added to kick damage bonus.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("cosmic_stone") >= 1) then
				projectileToggleExplosionOnDeath(projectile_entity_id, false)
				EntityKill(projectile_entity_id)
			end
		end,
		onenemydamaged_function	= function(damage, message, entity_thats_responsible, is_fatal, projectile_thats_responsible )
			if(isRelicOwned("cosmic_stone") >= 1) then
				if(message == "$damage_kick") then
					--GamePrint("Relic effect: 100x kick damage")
					EntityInflictDamage( GetUpdatedEntityID(), damage * 99 + (5 * getRelicLevel("cosmic_stone")), "DAMAGE_MELEE", "Cosmic Stone damage", "BLOOD_SPRAY", 0, 0)
				end
			end
		end,
		ondamaged_function		= function(damage, message, entity_thats_responsible, is_fatal, projectile_thats_responsible )
			if(isRelicOwned("cosmic_stone") >= 1) then	
				--GamePrint("Relic effect: damage partially absorbed")
				local hp_replenished = damage * (50 + 2 * getRelicLevel("cosmic_stone")) / 100
				addPlayerHealth(hp_replenished, false)
			end
		end,
		onpickup_function		= function()
			-- adds movement and levitation boost effect
			LoadGameEffectEntityTo( getPlayerEntity(), getCustomEffect("effect_movement_and_levitation_boost").path )
		end,
		ondrop_function			= function()
			
			-- removes movement and levitation boost effect
			local player_children = EntityGetAllChildren(getPlayerEntity())
			if player_children == nil then return end
			for i,child in ipairs(player_children) do
				if(EntityGetName(child) == "movement_and_levitation_boost") then
					EntityKill(child)
					break
				end
			end
		end,
		excludes_relics			= "ankle_pads,kryptonite_chunk,barrys_crest,titanium_rose",
		excludes_relics_desc	= "It can't be used along with other kick damage, fly speed or movement speed boost or hp replenish on damage related relics",
		excludes_relics_tags	= "kick_damage_boost,movement_speed_boost,fly_speed_boost,hp_replenish_on_damage",
		combine_exclusive		= true,
		combine_requires_relics	= "ankle_pads,kryptonite_chunk,barrys_crest,titanium_rose",
	},
	{
		name 					= "polymorphic_stone",
		ui_name					= "Polymorphic Stone",
		categories				= "player,kick",
		effect_desc				= "Enemies you kick are polymorphed. \nIt also synergizes with Sniper Scope relic effect.",
		effect_desc_short		= "Enemies you kick are polymorphed.",
        precursor_types			= "boot,magic_artifact,effect_stone",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onenemykicked_function	= function (entity_who_kicked)
			if(isRelicOwned("polymorphic_stone") >= 1) then
				LoadGameEffectEntityTo( GetUpdatedEntityID(), "data/entities/misc/effect_polymorph.xml" )
			end
		end,
		onextendedkick_function	= function( entity_id, extended_kick_id)
			if(isRelicOwned("polymorphic_stone") >= 1) then
				attachLoadEffectOntoEntitiestoProjectile(extended_kick_id, "data/entities/misc/effect_polymorph.xml", "plasma_fading_pink")
			end
		end,
	},
	{
		name 					= "skull_necklace_golden",
		ui_name					= "Golden Skull Necklace",
		categories				= "enemies",
		effect_desc				= "Enemies killed accidentally have a 25% chance of leaving a gold pool on their place of death.",
		effect_desc_short		= "Enemies killed accidentally leave gold pools.",
        precursor_types			= "necklace",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		ontrickkill_function	= function(damage_type_bit_field, damage_message, entity_thats_responsible, drop_items)
			if(isRelicOwned("skull_necklace_golden") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 25 + (5 * getRelicLevel("skull_necklace_golden"))
				if(chance >= 1 and chance <= odds) then
					local enemy_x, enemy_y = EntityGetTransform(GetUpdatedEntityID())
					EntityLoad(getLoadOnExplosion("launch_elemental_particles_gold").path, enemy_x, enemy_y)
				end
			end
		end,
	},
	{
		name 					= "skull_necklace_radioactive",
		ui_name					= "Radioactive Skull Necklace",
		categories				= "enemies",
		effect_desc				= "Enemies killed accidentally have a 50% chance of creating a nuclear explosion.",
		effect_desc_short		= "Enemies killed accidentally create nuclear explosions.",
        precursor_types			= "necklace",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		ontrickkill_function	= function(damage_type_bit_field, damage_message, entity_thats_responsible, drop_items)
			if(isRelicOwned("skull_necklace_radioactive") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 50 + (5 * getRelicLevel("skull_necklace_radioactive"))
				if(chance >= 1 and chance <= odds) then
					local enemy_x, enemy_y = EntityGetTransform(GetUpdatedEntityID())
					EntityLoad("mods/ArcaneReliquary/files/entity_definitions/others/nuclear_explosion.xml", enemy_x, enemy_y)
				end
			end
		end,
	},
	{
		name 					= "skull_necklace_vicious",
		ui_name					= "Vicious Skull Necklace",
		categories				= "enemies",
		effect_desc				= "Enemies killed accidentally add to a counter that increases the base \ndamage of your spells, taking damage from any source except materials and fire resets the counter.",
		effect_desc_short		= "Enemies killed accidentally boost the base damage of your spells.",
        precursor_types			= "necklace",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		ontrickkill_function	= function(damage_type_bit_field, damage_message, entity_thats_responsible, drop_items)
			if(isRelicOwned("skull_necklace_vicious") >= 1) then
				local trick_kill_damage_counter = getInternalVariableValue(getPlayerEntity(), "trick_kill_damage_counter", "value_int")
				setInternalVariableValue(getPlayerEntity(), "trick_kill_damage_counter", "value_int", trick_kill_damage_counter + 1)
			end
		end,
		ondamaged_function		= function(damage, message, entity_thats_responsible, is_fatal, projectile_thats_responsible )
			if(isRelicOwned("skull_necklace_vicious") >= 1) then	
				if((not string.find(message, "damage from material")) and (not string.find(message, "fire"))) then
					setInternalVariableValue(getPlayerEntity(), "trick_kill_damage_counter", "value_int", 0)
				end
			end
		end,
		onspellcast_function	= function(projectile_entity_id)				
			if(isRelicOwned("skull_necklace_vicious") >= 1) then
				local trick_kill_damage_counter = getInternalVariableValue(getPlayerEntity(), "trick_kill_damage_counter", "value_int")
				if(trick_kill_damage_counter ~= 0) then
					local bonus_damage = trick_kill_damage_counter / 3
					if(bonus_damage > 5) then
						bonus_damage = 5
					end
					addProjectileBaseDamage(projectile_entity_id, bonus_damage)
				end
			end
		end,
	},
	{
		name 					= "skull_necklace_scorching",
		ui_name					= "Scorching Skull Necklace",
		categories				= "enemies",
		effect_desc				= "Enemies killed accidentally have a 50% chance of leaving a fire pool on their place of death.",
		effect_desc_short		= "Enemies killed accidentally leave fire pools.",
        precursor_types			= "necklace",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		ontrickkill_function	= function(damage_type_bit_field, damage_message, entity_thats_responsible, drop_items)
			if(isRelicOwned("skull_necklace_scorching") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 50 + (5 * getRelicLevel("skull_necklace_scorching"))
				if(chance >= 1 and chance <= odds) then
					local enemy_x, enemy_y = EntityGetTransform(GetUpdatedEntityID())
					EntityLoad(getLoadOnExplosion("launch_elemental_particles_liquid_fire").path, enemy_x, enemy_y)
				end
			end
		end,
	},
	{
		name 					= "skull_necklace_wet",
		ui_name					= "Wet Skull Necklace",
		categories				= "enemies",
		effect_desc				= "Enemies killed accidentally have a 50% chance of leaving a water pool on their place of death.",
		effect_desc_short		= "Enemies killed accidentally leave water pools.",
        precursor_types			= "necklace",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		ontrickkill_function	= function(damage_type_bit_field, damage_message, entity_thats_responsible, drop_items)
			if(isRelicOwned("skull_necklace_wet") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 50 + (5 * getRelicLevel("skull_necklace_wet"))
				if(chance >= 1 and chance <= odds) then
					local enemy_x, enemy_y = EntityGetTransform(GetUpdatedEntityID())
					EntityLoad(getLoadOnExplosion("launch_elemental_particles_water").path, enemy_x, enemy_y)
				end
			end
		end,
	},
	{
		name 					= "skull_necklace_molten",
		ui_name					= "Molten Skull Necklace",
		categories				= "enemies",
		effect_desc				= "Enemies killed accidentally have a 50% chance of leaving a lava pool on their place of death.",
		effect_desc_short		= "Enemies killed accidentally leave lava pools.",
        precursor_types			= "necklace",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		ontrickkill_function	= function(damage_type_bit_field, damage_message, entity_thats_responsible, drop_items)
			if(isRelicOwned("skull_necklace_molten") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 50 + (5 * getRelicLevel("skull_necklace_molten"))
				if(chance >= 1 and chance <= odds) then
					local enemy_x, enemy_y = EntityGetTransform(GetUpdatedEntityID())
					EntityLoad(getLoadOnExplosion("launch_elemental_particles_lava").path, enemy_x, enemy_y)
				end
			end
		end,
	},
	{
		name 					= "skull_necklace_mutating",
		ui_name					= "Mutating Skull Necklace",
		categories				= "enemies",
		effect_desc				= "Enemies killed accidentally have a 50% chance of leaving a polymorphine pool on their place of death.",
		effect_desc_short		= "Enemies killed accidentally leave polymorphine pools.",
        precursor_types			= "necklace",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		ontrickkill_function	= function(damage_type_bit_field, damage_message, entity_thats_responsible, drop_items)
			if(isRelicOwned("skull_necklace_mutating") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 50 + (5 * getRelicLevel("skull_necklace_mutating"))
				if(chance >= 1 and chance <= odds) then
					local enemy_x, enemy_y = EntityGetTransform(GetUpdatedEntityID())
					EntityLoad(getLoadOnExplosion("launch_elemental_particles_poly").path, enemy_x, enemy_y)
				end
			end
		end,
	},
	{
		name 					= "skull_necklace_corroding",
		ui_name					= "Corroding Skull Necklace",
		categories				= "enemies",
		effect_desc				= "Enemies killed accidentally have a 50% chance of leaving an acid pool on their place of death.",
		effect_desc_short		= "Enemies killed accidentally leave acid pools.",
        precursor_types			= "necklace",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		ontrickkill_function	= function(damage_type_bit_field, damage_message, entity_thats_responsible, drop_items)
			if(isRelicOwned("skull_necklace_corroding") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 50 + (5 * getRelicLevel("skull_necklace_corroding"))
				if(chance >= 1 and chance <= odds) then
					local enemy_x, enemy_y = EntityGetTransform(GetUpdatedEntityID())
					EntityLoad(getLoadOnExplosion("launch_elemental_particles_acid").path, enemy_x, enemy_y)
				end
			end
		end,
	},
	{
		name 					= "skull_necklace_venomous",
		ui_name					= "Venomous Skull Necklace",
		categories				= "enemies",
		effect_desc				= "Enemies killed accidentally have a 50% chance of leaving a poison pool on their place of death.",
		effect_desc_short		= "Enemies killed accidentally leave poison pools.",
        precursor_types			= "necklace",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		ontrickkill_function	= function(damage_type_bit_field, damage_message, entity_thats_responsible, drop_items)
			if(isRelicOwned("skull_necklace_venomous") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 50 + (5 * getRelicLevel("skull_necklace_venomous"))
				if(chance >= 1 and chance <= odds) then
					local enemy_x, enemy_y = EntityGetTransform(GetUpdatedEntityID())
					EntityLoad(getLoadOnExplosion("launch_elemental_particles_poison").path, enemy_x, enemy_y)
				end
			end
		end,
	},
	{
		name 					= "skull_necklace_uric",
		ui_name					= "Uric Skull Necklace",
		categories				= "enemies",
		effect_desc				= "Enemies killed accidentally have a 50% chance of leaving an urine pool on their place of death.",
		effect_desc_short		= "Enemies killed accidentally leave urine pools.",
        precursor_types			= "necklace",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		ontrickkill_function	= function(damage_type_bit_field, damage_message, entity_thats_responsible, drop_items)
			if(isRelicOwned("skull_necklace_uric") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 50 + (5 * getRelicLevel("skull_necklace_uric"))
				if(chance >= 1 and chance <= odds) then
					local enemy_x, enemy_y = EntityGetTransform(GetUpdatedEntityID())
					EntityLoad(getLoadOnExplosion("launch_elemental_particles_urine").path, enemy_x, enemy_y)
				end
			end
		end,
	},
	{
		name 					= "ruby_butterfly",
		ui_name					= "Ruby Butterfly",
		categories				= "player",
		effect_desc				= "When you take projectile, melee or explosion damage, there's a 50% chance all enemy \ncreatures in the vicinity will be transformed into random projectiles.",
		effect_desc_short		= "Taking damage transforms enemies around into random projectiles.",
        precursor_types			= "magic_artifact",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		ondamaged_function		= function(damage, message, entity_thats_responsible, is_fatal, projectile_thats_responsible )
			if(isRelicOwned("ruby_butterfly") >= 1) then
				if(message == "$damage_projectile" or message == "$damage_explosion" or message == "$damage_melee") then
					SetRandomSeed( GameGetFrameNum(), math.random(1,50))
					local chance = Random(1,100)
					local odds = 50 + (5 * getRelicLevel("ruby_butterfly"))
					if(chance >= 1 and chance <= odds) then
						local player_x, player_y = EntityGetTransform(GetUpdatedEntityID())
						local enemies = EntityGetInRadiusWithTag( player_x, player_y, 200, "enemy" )
						for i, enemy_id in ipairs(enemies) do
							if(enemy_id ~= entity_thats_responsible) then
								LoadGameEffectEntityTo( enemy_id, getCustomEffect("effect_convert_to_random_projectile").path )
							end
						end
					end
				end
			end
		end,
	},
	{
		name 					= "stable_nuclear_core",
		ui_name					= "Stable Nuclear Core",
		categories				= "player",
		effect_desc				= "When you take projectile, melee or explosion damage, there's a 50% chance \nradioactive explosions will be triggered on nearby creatures.",
		effect_desc_short		= "Taking damage transforms triggers nuclear explosions on nearby enemies.",
        precursor_types			= "magic_artifact",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5% and at level 10, big nuclear explosions are triggered instead.",
		is_upgradable 			= true,
		ondamaged_function		= function(damage, message, entity_thats_responsible, is_fatal, projectile_thats_responsible )
			if(isRelicOwned("stable_nuclear_core") >= 1) then
				if(message == "$damage_projectile" or message == "$damage_explosion" or message == "$damage_melee") then
					SetRandomSeed( GameGetFrameNum(), math.random(1,50))
					local chance = Random(1,100)
					local odds = 50 + (5 * getRelicLevel("stable_nuclear_core"))
					if(chance >= 1 and chance <= odds) then
						local player_x, player_y = EntityGetTransform(GetUpdatedEntityID())
						local enemies = EntityGetInRadiusWithTag( player_x, player_y, 200, "enemy" )
						for i, enemy_id in ipairs(enemies) do
							if(Random(1,2) == 1) then
								local enemy_x, enemy_y = EntityGetTransform(enemy_id)
								if(getRelicLevel("stable_nuclear_core") < 10) then
									EntityLoad("mods/ArcaneReliquary/files/entity_definitions/others/radioactive_explosion.xml", enemy_x + Random(-50,50), enemy_y + Random(-50,50))
								else
									EntityLoad("mods/ArcaneReliquary/files/entity_definitions/others/nuclear_explosion.xml", enemy_x + Random(-50,50), enemy_y + Random(-50,50))
								end
							end
						end
					end
				end
			end
		end,
	},
	{
		name 					= "toy_turret_nuke",
		ui_name					= "Toy Nuke Turret",
		categories				= "enemies,items,minions",
		effect_desc				= "Enemies you kill have a 20% chance of dropping briefcases on death. \nThese briefcases can be thrown on the ground to deploy friendly nuke turrets that aid you in combat.",
		effect_desc_short		= "Killed enemies drop briefcases that deploy friendly nuke turrets.",
        precursor_types			= "gadget",
		upgrade_effect_desc		= "For each upgrade level, spawn chance increases by 5%.",
		is_upgradable 			= true,
		onenemykill_function	= function(damage_type_bit_field, damage_message, entity_thats_responsible, drop_items)
			if(isRelicOwned("toy_turret_nuke") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 20 + (5 * getRelicLevel("toy_turret_nuke"))
				if(chance >= 1 and chance <= odds) then
					local death_x, death_y = EntityGetTransform(GetUpdatedEntityID())
					EntityLoad("mods/ArcaneReliquary/files/entity_definitions/items/briefcase_nuke.xml", death_x, death_y)
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "toy_turret,toy_turret_laser,toy_turret_rocket",
	},
	
	{
		name 					= "toy_submarine",
		ui_name					= "Toy Submarine",
		categories				= "companions",
		effect_desc				= "Whenever you take explosion damage, a floating proximity mine that follows \nyour mouse cursor appears, allowing you to use it against enemies.",
		effect_desc_short		= "After taking explosion damage, a floating proximity mine appears and follows your mouse cursor",
        precursor_types			= "companion",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		requires_relic			= "alkaline_battery",
		ondamaged_function		= function(damage, message, entity_thats_responsible, is_fatal, projectile_thats_responsible )
			if(isRelicOwned("toy_submarine") >= 1) then
				if(message == "$damage_explosion") then
					attachCompaniontoEntity(getPlayerEntity(), "floating_mine", "mortal")
				end
			end
		end,
		ondrop_function			= function()
			detachCompanionfromEntity(getPlayerEntity(), "floating_mine")
		end,
	},
	{
		name 					= "chain_on_a_block",
		ui_name					= "Chain on a Block",
		categories				= "companions",
		effect_desc				= "A companion that bites enemies, causing slice damage, follows your mouse cursor.",
		effect_desc_short		= "A biting companion follows your mouse cursor.",
        precursor_types			= "companion",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			attachCompaniontoEntity(getPlayerEntity(), "chomp", "mortal")
		end,
		ondrop_function			= function()
			detachCompanionfromEntity(getPlayerEntity(), "chomp")
		end,
	},
	{
		name 					= "mystic_shield",
		ui_name					= "Mystic Shield",
		categories				= "companions",
		effect_desc				= "A mystic shield that deflects enemy projectiles appears and follows your mouse cursor.",
		effect_desc_short		= "A mystic shield that deflects enemy projectiles appears and follows your mouse cursor.",
        precursor_types			= "companion",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			attachCompaniontoEntity(getPlayerEntity(), "mystic_shield", "mortal")
		end,
		ondrop_function			= function()
			detachCompanionfromEntity(getPlayerEntity(), "mystic_shield")
		end,
	},
	{
		name 					= "absorbing_crystal",
		ui_name					= "Absorbing Crystal",
		categories				= "companions",
		effect_desc				= "A crystal that absorbs enemy projectiles and uses their energy to cast explosive orbs at enemies follows your mouse cursor.",
		effect_desc_short		= "A crystal that absorbs enemy projectiles and uses their energy to cast explosive orbs at enemies follows your mouse cursor.",
        precursor_types			= "companion",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			attachCompaniontoEntity(getPlayerEntity(), "absorbing_crystal", "mortal")
		end,
		ondrop_function			= function()
			detachCompanionfromEntity(getPlayerEntity(), "absorbing_crystal")
		end,
	},
	{
		name 					= "samurai_helmet",
		ui_name					= "Samurai Helmet",
		categories				= "player",
		effect_desc				= "A mystic katana rotates around you, causing slice damage to any enemy it touches.",
		effect_desc_short		= "A mystic katana rotates around you, causing slice damage to any enemy it touches.",
        precursor_types			= "gravity_stone",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		ondrop_function			= function()
			detachRotatingShieldfromEntity(getPlayerEntity(), "rotating_shield_projectile_katana")
		end,
		overtime_function		= function()
			if(isRelicOwned("samurai_helmet") >= 1) then
				if(not entityHasRotatingShield(getPlayerEntity(), "rotating_shield_projectile_katana")) then -- this makes sure the orbiting projectiles are loaded again if the player is polymorphed
					local start_id = EntityLoad("mods/ArcaneReliquary/files/entity_definitions/custom_projectiles/rotating_shield_start.xml")
					setInternalVariableValue(start_id, "host_entity_id", "value_int", getPlayerEntity())
					setInternalVariableValue(start_id, "rotating_projectile_definition", "value_string", "mods/ArcaneReliquary/files/entity_definitions/custom_projectiles/rotating_katana.xml")
					setInternalVariableValue(start_id, "rotating_projectiles_amount", "value_int", 1)
					setInternalVariableValue(start_id, "radius", "value_int", 40)
					setInternalVariableValue(start_id, "rotation_speed", "value_int", 6)
				end
			end
		end,
	},
	{
		name 					= "spikeball",
		ui_name					= "Spikeball",
		categories				= "player",
		effect_desc				= "A heavy spikeball rotates around you, causing impact damage to any enemy it touches.",
		effect_desc_short		= "A heavy spikeball rotates around you, causing impact damage to any enemy it touches.",
        precursor_types			= "gravity_stone",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		ondrop_function			= function()
			detachRotatingShieldfromEntity(getPlayerEntity(), "rotating_shield_projectile_spikeball")
		end,
		overtime_function		= function()
			if(isRelicOwned("spikeball") >= 1) then
				if(not entityHasRotatingShield(getPlayerEntity(), "rotating_shield_projectile_spikeball")) then -- this makes sure the orbiting projectiles are loaded again if the player is polymorphed
					local start_id = EntityLoad("mods/ArcaneReliquary/files/entity_definitions/custom_projectiles/rotating_shield_start.xml")
					setInternalVariableValue(start_id, "host_entity_id", "value_int", getPlayerEntity())
					setInternalVariableValue(start_id, "rotating_projectile_definition", "value_string", "mods/ArcaneReliquary/files/entity_definitions/custom_projectiles/rotating_spikeball.xml")
					setInternalVariableValue(start_id, "rotating_projectiles_amount", "value_int", 1)
					setInternalVariableValue(start_id, "radius", "value_int", 40)
					setInternalVariableValue(start_id, "rotation_speed", "value_int", 6)
				end
			end
		end,
	},
	{
		name 					= "toy_cannon",
		ui_name					= "Toy Cannon",
		categories				= "player",
		effect_desc				= "Cannonballs orbit around you, and propel themselves towards enemies if they get close.",
		effect_desc_short		= "Cannonballs orbit around you, and propel themselves towards enemies if they get close.",
        precursor_types			= "gravity_stone",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		ondrop_function			= function()
			detachRotatingShieldfromEntity(getPlayerEntity(), "rotating_shield_projectile_cannonball")
		end,
		overtime_function		= function()
			if(isRelicOwned("toy_cannon") >= 1) then
				if(not entityHasRotatingShield(getPlayerEntity(), "rotating_shield_projectile_cannonball")) then -- this makes sure the orbiting projectiles are loaded again if the player is polymorphed
					local start_id = EntityLoad("mods/ArcaneReliquary/files/entity_definitions/custom_projectiles/rotating_shield_start.xml")
					setInternalVariableValue(start_id, "host_entity_id", "value_int", getPlayerEntity())
					setInternalVariableValue(start_id, "rotating_projectile_definition", "value_string", "mods/ArcaneReliquary/files/entity_definitions/custom_projectiles/rotating_cannonball.xml")
					setInternalVariableValue(start_id, "rotating_projectiles_amount", "value_int", 4)
					setInternalVariableValue(start_id, "radius", "value_int", 15)
					setInternalVariableValue(start_id, "rotation_speed", "value_int", 3)
				end
			end
		end,
	},
	{
		name 					= "shuriken_frozen",
		ui_name					= "Frozen Shuriken",
		categories				= "projectiles,status_effects",
		effect_desc				= "Your spells have a 30% chance of inducing the \nfreezing gas slash effect on enemies hit by them.",
		effect_desc_short		= "Your spells induce the freezing gas slash effect on enemies.",
        precursor_types			= "magic_artifact,gadget",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("shuriken_frozen") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 30 + (5 * getRelicLevel("shuriken_frozen"))
				if(chance >= 1 and chance <= odds) then
					attachLoadEffectOntoEntitiestoProjectile(projectile_entity_id, getCustomEffect("effect_slash_freezing_gas").path, "blood_cold_vapour")
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "shuriken,freezing_gas_container",
	},
	{
		name 					= "defense_probe_improved",
		ui_name					= "Improved Defense Probe",
		categories				= "player",
		effect_desc				= "A defense probe orbits around you, targeting and intercepting enemy \nprojectiles and enemies as well.",
		effect_desc_short		= "A defense probe that targets enemies and their projectiles orbits around you.",
        precursor_types			= "gravity_stone",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		ondrop_function			= function()
			detachRotatingShieldfromEntity(getPlayerEntity(), "rotating_shield_projectile_defense_probe_improved")
		end,
		overtime_function		= function()
			if(isRelicOwned("defense_probe_improved") >= 1) then
				if(not entityHasRotatingShield(getPlayerEntity(), "rotating_shield_projectile_defense_probe_improved")) then -- this makes sure the orbiting projectiles are loaded again if the player is polymorphed
					local start_id = EntityLoad("mods/ArcaneReliquary/files/entity_definitions/custom_projectiles/rotating_shield_start.xml")
					setInternalVariableValue(start_id, "host_entity_id", "value_int", getPlayerEntity())
					setInternalVariableValue(start_id, "rotating_projectile_definition", "value_string", "mods/ArcaneReliquary/files/entity_definitions/custom_projectiles/rotating_defense_probe_improved.xml")
					setInternalVariableValue(start_id, "rotating_projectiles_amount", "value_int", 1)
					setInternalVariableValue(start_id, "radius", "value_int", 24)
					setInternalVariableValue(start_id, "rotation_speed", "value_int", 3)
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "defense_probe,targeting_device",
	},
	{
		name 					= "defense_probe_barrage",
		ui_name					= "Barrage Defense Probe",
		categories				= "player",
		effect_desc				= "A defense probe orbits around you, releasing a barrage of intercepting \norbs when it detects enemy projectiles nearby.",
		effect_desc_short		= "A defense probe that releases intercepting orbs when enemy projectiles are detected.",
        precursor_types			= "gravity_stone",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		ondrop_function			= function()
			detachRotatingShieldfromEntity(getPlayerEntity(), "rotating_shield_projectile_defense_probe_barrage")
		end,
		overtime_function		= function()
			if(isRelicOwned("defense_probe_barrage") >= 1) then
				if(not entityHasRotatingShield(getPlayerEntity(), "rotating_shield_projectile_defense_probe_barrage")) then -- this makes sure the orbiting projectiles are loaded again if the player is polymorphed
					local start_id = EntityLoad("mods/ArcaneReliquary/files/entity_definitions/custom_projectiles/rotating_shield_start.xml")
					setInternalVariableValue(start_id, "host_entity_id", "value_int", getPlayerEntity())
					setInternalVariableValue(start_id, "rotating_projectile_definition", "value_string", "mods/ArcaneReliquary/files/entity_definitions/custom_projectiles/rotating_defense_probe_barrage.xml")
					setInternalVariableValue(start_id, "rotating_projectiles_amount", "value_int", 1)
					setInternalVariableValue(start_id, "radius", "value_int", 34)
					setInternalVariableValue(start_id, "rotation_speed", "value_int", 3)
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "defense_probe,scarab_shell",
	},
	{
		name 					= "defense_probe_deflecting",
		ui_name					= "Deflecting Defense Probe",
		categories				= "player",
		effect_desc				= "A defense probe orbits around you, destroying and scattering enemy \nprojectiles it touches.",
		effect_desc_short		= "A defense probe that destroys and scatters enemy projectiles it touches.",
        precursor_types			= "gravity_stone",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		ondrop_function			= function()
			detachRotatingShieldfromEntity(getPlayerEntity(), "rotating_shield_projectile_defense_probe_deflecting")
		end,
		overtime_function		= function()
			if(isRelicOwned("defense_probe_deflecting") >= 1) then
				if(not entityHasRotatingShield(getPlayerEntity(), "rotating_shield_projectile_defense_probe_deflecting")) then -- this makes sure the orbiting projectiles are loaded again if the player is polymorphed
					local start_id = EntityLoad("mods/ArcaneReliquary/files/entity_definitions/custom_projectiles/rotating_shield_start.xml")
					setInternalVariableValue(start_id, "host_entity_id", "value_int", getPlayerEntity())
					setInternalVariableValue(start_id, "rotating_projectile_definition", "value_string", "mods/ArcaneReliquary/files/entity_definitions/custom_projectiles/rotating_defense_probe_deflecting.xml")
					setInternalVariableValue(start_id, "rotating_projectiles_amount", "value_int", 1)
					setInternalVariableValue(start_id, "radius", "value_int", 22)
					setInternalVariableValue(start_id, "rotation_speed", "value_int", 5)
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "defense_probe,ancient_root",
	},
	{
		name 					= "defense_probe_thunder",
		ui_name					= "Thunder Defense Probe",
		categories				= "player",
		effect_desc				= "A defense probe orbits around you, shooting intercepting laser bolts that \nconvert enemy projectiles into thunderbolts.",
		effect_desc_short		= "A defense probe that targets and destroys enemy projectiles, converting them into thunderbolts.",
        precursor_types			= "gravity_stone",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		ondrop_function			= function()
			detachRotatingShieldfromEntity(getPlayerEntity(), "rotating_shield_projectile_defense_probe_thunder")
		end,
		overtime_function		= function()
			if(isRelicOwned("defense_probe_thunder") >= 1) then
				if(not entityHasRotatingShield(getPlayerEntity(), "rotating_shield_projectile_defense_probe_thunder")) then -- this makes sure the orbiting projectiles are loaded again if the player is polymorphed
					local start_id = EntityLoad("mods/ArcaneReliquary/files/entity_definitions/custom_projectiles/rotating_shield_start.xml")
					setInternalVariableValue(start_id, "host_entity_id", "value_int", getPlayerEntity())
					setInternalVariableValue(start_id, "rotating_projectile_definition", "value_string", "mods/ArcaneReliquary/files/entity_definitions/custom_projectiles/rotating_defense_probe_thunder.xml")
					setInternalVariableValue(start_id, "rotating_projectiles_amount", "value_int", 1)
					setInternalVariableValue(start_id, "radius", "value_int", 18)
					setInternalVariableValue(start_id, "rotation_speed", "value_int", 2)
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "defense_probe,tesla_coil",
	},
	{
		name 					= "defense_probe_energy_store",
		ui_name					= "Energy Storing Defense Probe",
		categories				= "player",
		effect_desc				= "A defense probe orbits around you, absorbing enemy projectiles and charging \na power orb cannon that targets nearby enemies.",
		effect_desc_short		= "A defense probe that absorbs enemy projectiles and charges a power orb cannon that targets enemies.",
        precursor_types			= "gravity_stone",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		ondrop_function			= function()
			detachRotatingShieldfromEntity(getPlayerEntity(), "rotating_shield_projectile_defense_probe_energy_store")
		end,
		overtime_function		= function()
			if(isRelicOwned("defense_probe_energy_store") >= 1) then
				if(not entityHasRotatingShield(getPlayerEntity(), "rotating_shield_projectile_defense_probe_energy_store")) then -- this makes sure the orbiting projectiles are loaded again if the player is polymorphed
					local start_id = EntityLoad("mods/ArcaneReliquary/files/entity_definitions/custom_projectiles/rotating_shield_start.xml")
					setInternalVariableValue(start_id, "host_entity_id", "value_int", getPlayerEntity())
					setInternalVariableValue(start_id, "rotating_projectile_definition", "value_string", "mods/ArcaneReliquary/files/entity_definitions/custom_projectiles/rotating_defense_probe_energy_store.xml")
					setInternalVariableValue(start_id, "rotating_projectiles_amount", "value_int", 1)
					setInternalVariableValue(start_id, "radius", "value_int", 18)
					setInternalVariableValue(start_id, "rotation_speed", "value_int", 2)
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "defense_probe,absorbing_crystal",
	},
	{
		name 					= "defense_probe_slash",
		ui_name					= "Slash Defense Probe",
		categories				= "player",
		effect_desc				= "A defense probe orbits around you, converting enemy projectiles it touches into a barrage of slash lasers.",
		effect_desc_short		= "A defense probe that converts enemy projectiles it touches into slash lasers.",
        precursor_types			= "gravity_stone",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		ondrop_function			= function()
			detachRotatingShieldfromEntity(getPlayerEntity(), "rotating_shield_projectile_defense_probe_slash")
		end,
		overtime_function		= function()
			if(isRelicOwned("defense_probe_slash") >= 1) then
				if(not entityHasRotatingShield(getPlayerEntity(), "rotating_shield_projectile_defense_probe_slash")) then -- this makes sure the orbiting projectiles are loaded again if the player is polymorphed
					local start_id = EntityLoad("mods/ArcaneReliquary/files/entity_definitions/custom_projectiles/rotating_shield_start.xml")
					setInternalVariableValue(start_id, "host_entity_id", "value_int", getPlayerEntity())
					setInternalVariableValue(start_id, "rotating_projectile_definition", "value_string", "mods/ArcaneReliquary/files/entity_definitions/custom_projectiles/rotating_defense_probe_slash.xml")
					setInternalVariableValue(start_id, "rotating_projectiles_amount", "value_int", 1)
					setInternalVariableValue(start_id, "radius", "value_int", 18)
					setInternalVariableValue(start_id, "rotation_speed", "value_int", 2)
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "defense_probe,laser_pointer_white",
	},
	{
		name 					= "lens_red",
		ui_name					= "Red Lens",
		categories				= "projectiles",
		effect_desc				= "Spells you cast have a 40% chance of inducing enemies to shoot laser bolts at random directions.",
		effect_desc_short		= "Spells you cast have a chance to induce enemies to randomly shoot laser bolts.",
        precursor_types			= "gadget",
		upgrade_effect_desc		= "For each upgrade level, effect chance is increased by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("lens_red") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 40 + (5 * getRelicLevel("lens_red"))
				if(chance >= 1 and chance <= odds) then
					attachLoadEffectOntoEntitiestoProjectile(projectile_entity_id, getCustomEffect("effect_shoot_projectile_random_laser").path, "spark_red")
				end
			end
		end,
	},
	{
		name 					= "copper_wiring",
		ui_name					= "Copper Wiring",
		categories				= "projectiles",
		effect_desc				= "Spells you cast have a 40% chance of inducing enemies to shoot thunderbolts at random directions.",
		effect_desc_short		= "Spells you cast have a chance to induce enemies to randomly shoot thunderbolts.",
        precursor_types			= "gadget",
		upgrade_effect_desc		= "For each upgrade level, effect chance is increased by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("copper_wiring") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 40 + (5 * getRelicLevel("copper_wiring"))
				if(chance >= 1 and chance <= odds) then
					attachLoadEffectOntoEntitiestoProjectile(projectile_entity_id, getCustomEffect("effect_shoot_projectile_random_thunderbolt").path, "spark_blue")
				end
			end
		end,
	},
	{
		name 					= "shuriken_laser",
		ui_name					= "Laser Shuriken",
		categories				= "projectiles,status_effects",
		effect_desc				= "Your spells have a 30% chance of inducing the \nlaser bolt slash effect on enemies hit by them.",
		effect_desc_short		= "Your spells induce the laser bolt slash effect on enemies.",
        precursor_types			= "magic_artifact,gadget",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("shuriken_laser") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 30 + (5 * getRelicLevel("shuriken_laser"))
				if(chance >= 1 and chance <= odds) then
					attachLoadEffectOntoEntitiestoProjectile(projectile_entity_id, getCustomEffect("effect_slash_projectile_laser").path, "spark_red")
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "shuriken,kyber_crystal",
	},
	{
		name 					= "shuriken_thunder",
		ui_name					= "Thunder Shuriken",
		categories				= "projectiles,status_effects",
		effect_desc				= "Your spells have a 30% chance of inducing the \nthunderbolt slash effect on enemies hit by them.",
		effect_desc_short		= "Your spells induce the thunderbolt slash effect on enemies.",
        precursor_types			= "magic_artifact,gadget",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("shuriken_thunder") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 30 + (5 * getRelicLevel("shuriken_thunder"))
				if(chance >= 1 and chance <= odds) then
					attachLoadEffectOntoEntitiestoProjectile(projectile_entity_id, getCustomEffect("effect_slash_projectile_thunderbolt").path, "spark_blue")
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "shuriken,tesla_coil",
	},
	{
		name 					= "tuning_fork",
		ui_name					= "Tuning Fork",
		categories				= "projectiles,status_effects",
		effect_desc				= "Your spells have a 30% chance of inducing \nshockwaves on enemies.",
		effect_desc_short		= "Your spells induce shockwaves on enemies.",
        precursor_types			= "magic_artifact,gadget",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("tuning_fork") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 30 + (5 * getRelicLevel("tuning_fork"))
				if(chance >= 1 and chance <= odds) then
					attachLoadEffectOntoEntitiestoProjectile(projectile_entity_id, getCustomEffect("effect_shockwave").path, "spark_white")
				end
			end
		end,
	},
	{
		name 					= "tuning_fork_molten",
		ui_name					= "Molten Tuning Fork",
		categories				= "projectiles,status_effects",
		effect_desc				= "Your spells have a 30% chance of inducing \nfire shockwaves on enemies.",
		effect_desc_short		= "Your spells induce fire shockwaves on enemies.",
        precursor_types			= "magic_artifact,gadget",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("tuning_fork_molten") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 30 + (5 * getRelicLevel("tuning_fork_molten"))
				if(chance >= 1 and chance <= odds) then
					attachLoadEffectOntoEntitiestoProjectile(projectile_entity_id, getCustomEffect("effect_shockwave_fire").path, "lava")
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "tuning_fork,blaze_pearl",
	},
	{
		name 					= "tuning_fork_frozen",
		ui_name					= "Frozen Tuning Fork",
		categories				= "projectiles,status_effects",
		effect_desc				= "Your spells have a 30% chance of inducing \nice shockwaves on enemies.",
		effect_desc_short		= "Your spells induce ice shockwaves on enemies.",
        precursor_types			= "magic_artifact,gadget",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("tuning_fork_frozen") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 30 + (5 * getRelicLevel("tuning_fork_frozen"))
				if(chance >= 1 and chance <= odds) then
					attachLoadEffectOntoEntitiestoProjectile(projectile_entity_id, getCustomEffect("effect_shockwave_ice").path, "blood_cold_vapour")
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "tuning_fork,ice_artifact",
	},
	{
		name 					= "tuning_fork_charged",
		ui_name					= "Charged Tuning Fork",
		categories				= "projectiles,status_effects",
		effect_desc				= "Your spells have a 30% chance of inducing \nelectric shockwaves on enemies.",
		effect_desc_short		= "Your spells induce electric shockwaves on enemies.",
        precursor_types			= "magic_artifact,gadget",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("tuning_fork_charged") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 30 + (5 * getRelicLevel("tuning_fork_charged"))
				if(chance >= 1 and chance <= odds) then
					attachLoadEffectOntoEntitiestoProjectile(projectile_entity_id, getCustomEffect("effect_shockwave_electric").path, "spark_blue_dark")
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "tuning_fork,electrical_plug",
	},
	{
		name 					= "tuning_fork_overloaded",
		ui_name					= "Overloaded Tuning Fork",
		categories				= "projectiles,status_effects",
		effect_desc				= "Your spells have a 30% chance of inducing \nthunderbolt shockwaves on enemies.",
		effect_desc_short		= "Your spells induce thunderbolt shockwaves on enemies.",
        precursor_types			= "magic_artifact,gadget",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("tuning_fork_overloaded") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 30 + (5 * getRelicLevel("tuning_fork_overloaded"))
				if(chance >= 1 and chance <= odds) then
					attachLoadEffectOntoEntitiestoProjectile(projectile_entity_id, getCustomEffect("effect_shockwave_thunderbolts").path, "spark_blue")
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "tuning_fork,tesla_coil",
	},
	{
		name 					= "tuning_fork_nuclear",
		ui_name					= "Nuclear Tuning Fork",
		categories				= "projectiles,status_effects",
		effect_desc				= "Your spells have a 30% chance of inducing \nnuclear shockwaves on enemies.",
		effect_desc_short		= "Your spells induce nuclear shockwaves on enemies.",
        precursor_types			= "magic_artifact,gadget",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("tuning_fork_nuclear") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 30 + (5 * getRelicLevel("tuning_fork_nuclear"))
				if(chance >= 1 and chance <= odds) then
					attachLoadEffectOntoEntitiestoProjectile(projectile_entity_id, getCustomEffect("effect_shockwave_nuclear").path, "radioactive_gas")
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "tuning_fork,uranium_ingot",
	},
	{
		name 					= "shuriken_radioactive",
		ui_name					= "Radioactive Shuriken",
		categories				= "projectiles,status_effects",
		effect_desc				= "Your spells have a 30% chance of inducing the \nslime bolt slash effect on enemies hit by them.",
		effect_desc_short		= "Your spells induce the slime bolt slash effect on enemies.",
        precursor_types			= "magic_artifact,gadget",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("shuriken_radioactive") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 30 + (5 * getRelicLevel("shuriken_radioactive"))
				if(chance >= 1 and chance <= odds) then
					attachLoadEffectOntoEntitiestoProjectile(projectile_entity_id, getCustomEffect("effect_slash_projectile_slimebolt").path, "radioactive_gas")
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "shuriken,ball_of_goo",
	},
	{
		name 					= "shuriken_sour",
		ui_name					= "Sour Shuriken",
		categories				= "projectiles,status_effects",
		effect_desc				= "Your spells have a 30% chance of inducing the \nacid bolt slash effect on enemies hit by them.",
		effect_desc_short		= "Your spells induce the acid bolt slash effect on enemies.",
        precursor_types			= "magic_artifact,gadget",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("shuriken_sour") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 30 + (5 * getRelicLevel("shuriken_sour"))
				if(chance >= 1 and chance <= odds) then
					attachLoadEffectOntoEntitiestoProjectile(projectile_entity_id, getCustomEffect("effect_slash_projectile_acidbolt").path, "acid")
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "shuriken,acid_gem",
	},
	{
		name 					= "shuriken_igneous",
		ui_name					= "Igneous Shuriken",
		categories				= "projectiles,status_effects",
		effect_desc				= "Your spells have a 30% chance of inducing the \nlava bolt slash effect on enemies hit by them.",
		effect_desc_short		= "Your spells induce the lava bolt slash effect on enemies.",
        precursor_types			= "magic_artifact,gadget",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("shuriken_igneous") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 30 + (5 * getRelicLevel("shuriken_igneous"))
				if(chance >= 1 and chance <= odds) then
					attachLoadEffectOntoEntitiestoProjectile(projectile_entity_id, getCustomEffect("effect_slash_projectile_lavabolt").path, "lava")
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "shuriken,lava_gem",
	},
	{
		name 					= "ball_of_goo_pungent",
		ui_name					= "Ball of Pungent Goo",
		categories				= "projectiles",
		effect_desc				= "Your spells have a 25% chance of exploding into a barrage of acid bolts when they hit an obstacle or explode.",
		effect_desc_short		= "Your spells release acid bolts on impact.",
        precursor_types			= "magic_artifact",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("ball_of_goo_pungent") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 25 + (5 * getRelicLevel("ball_of_goo_pungent"))
				if(chance >= 1 and chance <= odds) then
					addBuffVisualIndicator_Projectile(projectile_entity_id, "acid")
					attachLoadEntityOnExplosiontoProjectile(projectile_entity_id, getLoadOnExplosion("launch_circular_projectiles_acidbolts").path)
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "ball_of_goo,acid_gem",
	},
	{
		name 					= "ball_of_goo_molten",
		ui_name					= "Ball of Molten Goo",
		categories				= "projectiles",
		effect_desc				= "Your spells have a 25% chance of exploding into a barrage of lava bolts when they hit an obstacle or explode.",
		effect_desc_short		= "Your spells release lava bolts on impact.",
        precursor_types			= "magic_artifact",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("ball_of_goo_molten") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 25 + (5 * getRelicLevel("ball_of_goo_molten"))
				if(chance >= 1 and chance <= odds) then
					addBuffVisualIndicator_Projectile(projectile_entity_id, "lava")
					attachLoadEntityOnExplosiontoProjectile(projectile_entity_id, getLoadOnExplosion("launch_circular_projectiles_lavabolts").path)
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "ball_of_goo,lava_gem",
	},
	{
		name 					= "ball_of_goo_mutating",
		ui_name					= "Ball of Mutating Goo",
		categories				= "projectiles",
		effect_desc				= "Your spells have a 25% chance of exploding into a barrage of polymorphine bolts when they hit an obstacle or explode.",
		effect_desc_short		= "Your spells release polymorphine bolts on impact.",
        precursor_types			= "magic_artifact",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("ball_of_goo_mutating") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 25 + (5 * getRelicLevel("ball_of_goo_mutating"))
				if(chance >= 1 and chance <= odds) then
					addBuffVisualIndicator_Projectile(projectile_entity_id, "plasma_fading_pink")
					attachLoadEntityOnExplosiontoProjectile(projectile_entity_id, getLoadOnExplosion("launch_circular_projectiles_polybolts").path)
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "ball_of_goo,poly_gem",
	},
	{
		name 					= "ball_of_goo_urinous",
		ui_name					= "Ball of Urinous Goo",
		categories				= "projectiles",
		effect_desc				= "Your spells have a 25% chance of exploding into a barrage of urine bolts when they hit an obstacle or explode.",
		effect_desc_short		= "Your spells release urine bolts on impact.",
        precursor_types			= "magic_artifact",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("ball_of_goo_urinous") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 25 + (5 * getRelicLevel("ball_of_goo_urinous"))
				if(chance >= 1 and chance <= odds) then
					addBuffVisualIndicator_Projectile(projectile_entity_id, "spark_yellow")
					attachLoadEntityOnExplosiontoProjectile(projectile_entity_id, getLoadOnExplosion("launch_circular_projectiles_urinebolts").path)
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "ball_of_goo,urine_gem",
	},
	{
		name 					= "ball_of_goo_bloody",
		ui_name					= "Ball of Bloody Goo",
		categories				= "projectiles",
		effect_desc				= "Your spells have a 25% chance of exploding into a barrage of corrosive blood bolts when they hit an obstacle or explode.",
		effect_desc_short		= "Your spells release corrosive blood bolts on impact.",
        precursor_types			= "magic_artifact",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("ball_of_goo_bloody") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 25 + (5 * getRelicLevel("ball_of_goo_bloody"))
				if(chance >= 1 and chance <= odds) then
					addBuffVisualIndicator_Projectile(projectile_entity_id, "blood")
					attachLoadEntityOnExplosiontoProjectile(projectile_entity_id, getLoadOnExplosion("launch_circular_projectiles_corrosivebloodbolts").path)
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "ball_of_goo,corrosive_blood_gem",
	},
	{
		name 					= "friendly_slime_poly",
		ui_name					= "Friendly Polymorphine Slime",
		categories				= "companions",
		effect_desc				= "A friendly polymorphine slime that shoots polymorphine bolts joins you.",
		effect_desc_short		= "A polymorphine bolt shooting companion.",
        precursor_types			= "companion",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			attachCompaniontoEntity(getPlayerEntity(), "friendly_slime_poly", "mortal")
		end,
		ondrop_function			= function()
			detachCompanionfromEntity(getPlayerEntity(), "friendly_slime_poly")
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "friendly_slime,poly_gem",
		requires_relic			= "ball_of_goo",
	},
	{
		name 					= "friendly_slime_urine",
		ui_name					= "Friendly Urine Slime",
		categories				= "companions",
		effect_desc				= "A friendly urine slime that shoots urine bolts joins you.",
		effect_desc_short		= "A urine bolt shooting companion.",
        precursor_types			= "companion",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			attachCompaniontoEntity(getPlayerEntity(), "friendly_slime_urine", "mortal")
		end,
		ondrop_function			= function()
			detachCompanionfromEntity(getPlayerEntity(), "friendly_slime_urine")
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "friendly_slime,urine_gem",
		requires_relic			= "ball_of_goo",
	},
	{
		name 					= "friendly_slime_blood",
		ui_name					= "Friendly Blood Slime",
		categories				= "companions",
		effect_desc				= "A friendly blood slime that shoots corrosive blood bolts joins you.",
		effect_desc_short		= "A corrosive blood bolt shooting companion.",
        precursor_types			= "companion",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			attachCompaniontoEntity(getPlayerEntity(), "friendly_slime_blood", "mortal")
		end,
		ondrop_function			= function()
			detachCompanionfromEntity(getPlayerEntity(), "friendly_slime_blood")
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "friendly_slime,corrosive_blood_gem",
		requires_relic			= "ball_of_goo",
	},
	{
		name 					= "shuriken_mutating",
		ui_name					= "Mutating Shuriken",
		categories				= "projectiles,status_effects",
		effect_desc				= "Your spells have a 30% chance of inducing the \npolymorphine bolt slash effect on enemies hit by them.",
		effect_desc_short		= "Your spells induce the polymorphine bolt slash effect on enemies.",
        precursor_types			= "magic_artifact,gadget",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("shuriken_mutating") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 30 + (5 * getRelicLevel("shuriken_mutating"))
				if(chance >= 1 and chance <= odds) then
					attachLoadEffectOntoEntitiestoProjectile(projectile_entity_id, getCustomEffect("effect_slash_projectile_polybolt").path, "plasma_fading_pink")
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "shuriken,poly_gem",
	},
	{
		name 					= "shuriken_urine",
		ui_name					= "Urine Shuriken",
		categories				= "projectiles,status_effects",
		effect_desc				= "Your spells have a 30% chance of inducing the \nurine bolt slash effect on enemies hit by them.",
		effect_desc_short		= "Your spells induce the urine bolt slash effect on enemies.",
        precursor_types			= "magic_artifact,gadget",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("shuriken_urine") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 30 + (5 * getRelicLevel("shuriken_urine"))
				if(chance >= 1 and chance <= odds) then
					attachLoadEffectOntoEntitiestoProjectile(projectile_entity_id, getCustomEffect("effect_slash_projectile_urinebolt").path, "urine")
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "shuriken,urine_gem",
	},
	{
		name 					= "shuriken_sanguine",
		ui_name					= "Sanguine Shuriken",
		categories				= "projectiles,status_effects",
		effect_desc				= "Your spells have a 30% chance of inducing the \ncorrosive blood bolt slash effect on enemies hit by them.",
		effect_desc_short		= "Your spells induce the corrosive blood bolt slash effect on enemies.",
        precursor_types			= "magic_artifact,gadget",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("shuriken_sanguine") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 30 + (5 * getRelicLevel("shuriken_sanguine"))
				if(chance >= 1 and chance <= odds) then
					attachLoadEffectOntoEntitiestoProjectile(projectile_entity_id, getCustomEffect("effect_slash_projectile_corrosivebloodbolt").path, "blood")
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "shuriken,corrosive_blood_gem",
	},
	{
		name 					= "jar_of_goo",
		ui_name					= "Jar of Goo",
		categories				= "projectiles",
		effect_desc				= "Spells you cast have a 40% chance of inducing enemies to shoot slime bolts at random directions.",
		effect_desc_short		= "Spells you cast have a chance to induce enemies to randomly shoot slime bolts.",
        precursor_types			= "gadget",
		upgrade_effect_desc		= "For each upgrade level, effect chance is increased by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("jar_of_goo") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 40 + (5 * getRelicLevel("jar_of_goo"))
				if(chance >= 1 and chance <= odds) then
					attachLoadEffectOntoEntitiestoProjectile(projectile_entity_id, getCustomEffect("effect_shoot_projectile_random_slimebolt").path, "radioactive_liquid_fading")
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "empty_bottle,ball_of_goo",
	},
	{
		name 					= "jar_of_goo_pungent",
		ui_name					= "Jar of Pungent Goo",
		categories				= "projectiles",
		effect_desc				= "Spells you cast have a 40% chance of inducing enemies to shoot acid bolts at random directions.",
		effect_desc_short		= "Spells you cast have a chance to induce enemies to randomly shoot acid bolts.",
        precursor_types			= "gadget",
		upgrade_effect_desc		= "For each upgrade level, effect chance is increased by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("jar_of_goo_pungent") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 40 + (5 * getRelicLevel("jar_of_goo_pungent"))
				if(chance >= 1 and chance <= odds) then
					attachLoadEffectOntoEntitiestoProjectile(projectile_entity_id, getCustomEffect("effect_shoot_projectile_random_acidbolt").path, "plasma_fading_green")
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "empty_bottle,ball_of_goo_pungent",
	},
	{
		name 					= "jar_of_goo_molten",
		ui_name					= "Jar of Molten Goo",
		categories				= "projectiles",
		effect_desc				= "Spells you cast have a 40% chance of inducing enemies to shoot lava bolts at random directions.",
		effect_desc_short		= "Spells you cast have a chance to induce enemies to randomly shoot lava bolts.",
        precursor_types			= "gadget",
		upgrade_effect_desc		= "For each upgrade level, effect chance is increased by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("jar_of_goo_molten") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 40 + (5 * getRelicLevel("jar_of_goo_molten"))
				if(chance >= 1 and chance <= odds) then
					attachLoadEffectOntoEntitiestoProjectile(projectile_entity_id, getCustomEffect("effect_shoot_projectile_random_lavabolt").path, "lava")
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "empty_bottle,ball_of_goo_molten",
	},
	{
		name 					= "jar_of_goo_urinous",
		ui_name					= "Jar of Urinous Goo",
		categories				= "projectiles",
		effect_desc				= "Spells you cast have a 40% chance of inducing enemies to shoot urine bolts at random directions.",
		effect_desc_short		= "Spells you cast have a chance to induce enemies to randomly shoot urine bolts.",
        precursor_types			= "gadget",
		upgrade_effect_desc		= "For each upgrade level, effect chance is increased by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("jar_of_goo_urinous") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 40 + (5 * getRelicLevel("jar_of_goo_urinous"))
				if(chance >= 1 and chance <= odds) then
					attachLoadEffectOntoEntitiestoProjectile(projectile_entity_id, getCustomEffect("effect_shoot_projectile_random_urinebolt").path, "urine")
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "empty_bottle,ball_of_goo_urinous",
	},
	{
		name 					= "jar_of_goo_bloody",
		ui_name					= "Jar of Bloody Goo",
		categories				= "projectiles",
		effect_desc				= "Spells you cast have a 40% chance of inducing enemies to shoot corrosive blood bolts at random directions.",
		effect_desc_short		= "Spells you cast have a chance to induce enemies to randomly shoot corrosive blood bolts.",
        precursor_types			= "gadget",
		upgrade_effect_desc		= "For each upgrade level, effect chance is increased by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("jar_of_goo_bloody") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 40 + (5 * getRelicLevel("jar_of_goo_bloody"))
				if(chance >= 1 and chance <= odds) then
					attachLoadEffectOntoEntitiestoProjectile(projectile_entity_id, getCustomEffect("effect_shoot_projectile_random_corrosivebloodbolt").path, "blood")
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "empty_bottle,ball_of_goo_bloody",
	},
	{
		name 					= "jar_of_goo_mutating",
		ui_name					= "Jar of Mutating Goo",
		categories				= "projectiles",
		effect_desc				= "Spells you cast have a 40% chance of inducing enemies to shoot polymorphine bolts at random directions.",
		effect_desc_short		= "Spells you cast have a chance to induce enemies to randomly shoot polymorphine bolts.",
        precursor_types			= "gadget",
		upgrade_effect_desc		= "For each upgrade level, effect chance is increased by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("jar_of_goo_mutating") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 40 + (5 * getRelicLevel("jar_of_goo_mutating"))
				if(chance >= 1 and chance <= odds) then
					attachLoadEffectOntoEntitiestoProjectile(projectile_entity_id, getCustomEffect("effect_shoot_projectile_random_polybolt").path, "plasma_fading_pink")
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "empty_bottle,ball_of_goo_mutating",
	},
	{
		name 					= "lens_yellow",
		ui_name					= "Yellow Lens",
		categories				= "projectiles",
		effect_desc				= "Spells you cast have a 40% chance of inducing enemies to shoot yellow explosive laser bolts at random directions.",
		effect_desc_short		= "Spells you cast have a chance to induce enemies to randomly shoot yellow explosive laser bolts.",
        precursor_types			= "gadget",
		upgrade_effect_desc		= "For each upgrade level, effect chance is increased by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("lens_yellow") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 40 + (5 * getRelicLevel("lens_yellow"))
				if(chance >= 1 and chance <= odds) then
					attachLoadEffectOntoEntitiestoProjectile(projectile_entity_id, getCustomEffect("effect_shoot_projectile_random_laser_yellow").path, "spark_yellow")
				end
			end
		end,
	},
	{
		name 					= "lens_blue",
		ui_name					= "Blue Lens",
		categories				= "projectiles",
		effect_desc				= "Spells you cast have a 40% chance of inducing enemies to shoot blue electrocuting laser bolts at random directions.",
		effect_desc_short		= "Spells you cast have a chance to induce enemies to randomly shoot blue electrocuting laser bolts.",
        precursor_types			= "gadget",
		upgrade_effect_desc		= "For each upgrade level, effect chance is increased by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("lens_blue") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 40 + (5 * getRelicLevel("lens_blue"))
				if(chance >= 1 and chance <= odds) then
					attachLoadEffectOntoEntitiestoProjectile(projectile_entity_id, getCustomEffect("effect_shoot_projectile_random_laser_blue").path, "spark_blue_dark")
				end
			end
		end,
	},
	{
		name 					= "lens_orange",
		ui_name					= "Orange Lens",
		categories				= "projectiles",
		effect_desc				= "Spells you cast have a 40% chance of inducing enemies to shoot orange incendiary laser bolts at random directions.",
		effect_desc_short		= "Spells you cast have a chance to induce enemies to randomly shoot orange incendiary laser bolts.",
        precursor_types			= "gadget",
		upgrade_effect_desc		= "For each upgrade level, effect chance is increased by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("lens_orange") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 40 + (5 * getRelicLevel("lens_orange"))
				if(chance >= 1 and chance <= odds) then
					attachLoadEffectOntoEntitiestoProjectile(projectile_entity_id, getCustomEffect("effect_shoot_projectile_random_laser_orange").path, "lava")
				end
			end
		end,
	},
	{
		name 					= "lens_teal",
		ui_name					= "Teal Lens",
		categories				= "projectiles",
		effect_desc				= "Spells you cast have a 40% chance of inducing enemies to shoot teal freezing laser bolts at random directions.",
		effect_desc_short		= "Spells you cast have a chance to induce enemies to randomly shoot teal freezing laser bolts.",
        precursor_types			= "gadget",
		upgrade_effect_desc		= "For each upgrade level, effect chance is increased by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("lens_teal") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 40 + (5 * getRelicLevel("lens_teal"))
				if(chance >= 1 and chance <= odds) then
					attachLoadEffectOntoEntitiestoProjectile(projectile_entity_id, getCustomEffect("effect_shoot_projectile_random_laser_teal").path, "spark_blue")
				end
			end
		end,
	},
	{
		name 					= "lens_green",
		ui_name					= "Green Lens",
		categories				= "projectiles",
		effect_desc				= "Spells you cast have a 40% chance of inducing enemies to shoot green acid laser bolts at random directions.",
		effect_desc_short		= "Spells you cast have a chance to induce enemies to randomly shoot green acid laser bolts.",
        precursor_types			= "gadget",
		upgrade_effect_desc		= "For each upgrade level, effect chance is increased by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("lens_green") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 40 + (5 * getRelicLevel("lens_green"))
				if(chance >= 1 and chance <= odds) then
					attachLoadEffectOntoEntitiestoProjectile(projectile_entity_id, getCustomEffect("effect_shoot_projectile_random_laser_green").path, "plasma_fading_green")
				end
			end
		end,
	},
	{
		name 					= "lens_purple",
		ui_name					= "Purple Lens",
		categories				= "projectiles",
		effect_desc				= "Spells you cast have a 40% chance of inducing enemies to shoot purple mutating laser bolts at random directions.",
		effect_desc_short		= "Spells you cast have a chance to induce enemies to randomly shoot purple mutating laser bolts.",
        precursor_types			= "gadget",
		upgrade_effect_desc		= "For each upgrade level, effect chance is increased by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("lens_purple") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 40 + (5 * getRelicLevel("lens_purple"))
				if(chance >= 1 and chance <= odds) then
					attachLoadEffectOntoEntitiestoProjectile(projectile_entity_id, getCustomEffect("effect_shoot_projectile_random_laser_purple").path, "plasma_fading_pink")
				end
			end
		end,
	},
	{
		name 					= "lens_white",
		ui_name					= "White Lens",
		categories				= "projectiles",
		effect_desc				= "Spells you cast have a 40% chance of inducing enemies to shoot white slash laser bolts at random directions.",
		effect_desc_short		= "Spells you cast have a chance to induce enemies to randomly shoot white slash laser bolts.",
        precursor_types			= "gadget",
		upgrade_effect_desc		= "For each upgrade level, effect chance is increased by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("lens_white") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 40 + (5 * getRelicLevel("lens_white"))
				if(chance >= 1 and chance <= odds) then
					attachLoadEffectOntoEntitiestoProjectile(projectile_entity_id, getCustomEffect("effect_shoot_projectile_random_laser_slash").path, "spark_white")
				end
			end
		end,
	},
	{
		name 					= "defense_probe_firstaid",
		ui_name					= "First Aid Defense Probe",
		categories				= "player",
		effect_desc				= "A defense probe orbits around you, targeting and intercepting enemy \nprojectiles, converting their base damage into health for your character.",
		effect_desc_short		= "A defense probe that targets enemy projectiles and converts them into health.",
        precursor_types			= "gravity_stone",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		ondrop_function			= function()
			detachRotatingShieldfromEntity(getPlayerEntity(), "rotating_shield_projectile_defense_probe_firstaid")
		end,
		overtime_function		= function()
			if(isRelicOwned("defense_probe_firstaid") >= 1) then
				if(not entityHasRotatingShield(getPlayerEntity(), "rotating_shield_projectile_defense_probe_firstaid")) then -- this makes sure the orbiting projectiles are loaded again if the player is polymorphed
					local start_id = EntityLoad("mods/ArcaneReliquary/files/entity_definitions/custom_projectiles/rotating_shield_start.xml")
					setInternalVariableValue(start_id, "host_entity_id", "value_int", getPlayerEntity())
					setInternalVariableValue(start_id, "rotating_projectile_definition", "value_string", "mods/ArcaneReliquary/files/entity_definitions/custom_projectiles/rotating_defense_probe_firstaid.xml")
					setInternalVariableValue(start_id, "rotating_projectiles_amount", "value_int", 1)
					setInternalVariableValue(start_id, "radius", "value_int", 24)
					setInternalVariableValue(start_id, "rotation_speed", "value_int", 3)
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "defense_probe,blood_gem",
	},
	{
		name 					= "laser_pointer",
		ui_name					= "Laser Pointer",
		categories				= "projectiles",
		effect_desc				= "Your spells have a 25% chance of exploding into a barrage of laser bolts when they hit an obstacle or explode.",
		effect_desc_short		= "Your spells release laser bolts on impact.",
        precursor_types			= "magic_artifact",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("laser_pointer") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 25 + (5 * getRelicLevel("laser_pointer"))
				if(chance >= 1 and chance <= odds) then
					addBuffVisualIndicator_Projectile(projectile_entity_id, "spark_red")
					attachLoadEntityOnExplosiontoProjectile(projectile_entity_id, getLoadOnExplosion("launch_circular_projectiles_lasers").path)
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "kyber_crystal,lens_red",
	},
	{
		name 					= "laser_pointer_white",
		ui_name					= "White Laser Pointer",
		categories				= "projectiles",
		effect_desc				= "Your spells have a 25% chance of exploding into a barrage of slash laser bolts when they hit an obstacle or explode.",
		effect_desc_short		= "Your spells release slash laser bolts on impact.",
        precursor_types			= "magic_artifact",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("laser_pointer_white") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 25 + (5 * getRelicLevel("laser_pointer_white"))
				if(chance >= 1 and chance <= odds) then
					addBuffVisualIndicator_Projectile(projectile_entity_id, "spark_white")
					attachLoadEntityOnExplosiontoProjectile(projectile_entity_id, getLoadOnExplosion("launch_circular_projectiles_lasers_slash").path)
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "kyber_crystal,lens_white",
	},
	{
		name 					= "laser_pointer_yellow",
		ui_name					= "Yellow Laser Pointer",
		categories				= "projectiles",
		effect_desc				= "Your spells have a 25% chance of exploding into a barrage of explosive laser bolts when they hit an obstacle or explode.",
		effect_desc_short		= "Your spells release explosive laser bolts on impact.",
        precursor_types			= "magic_artifact",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("laser_pointer_yellow") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 25 + (5 * getRelicLevel("laser_pointer_yellow"))
				if(chance >= 1 and chance <= odds) then
					addBuffVisualIndicator_Projectile(projectile_entity_id, "spark_yellow")
					attachLoadEntityOnExplosiontoProjectile(projectile_entity_id, getLoadOnExplosion("launch_circular_projectiles_lasers_yellow").path)
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "kyber_crystal,lens_yellow",
	},
	{
		name 					= "laser_pointer_orange",
		ui_name					= "Orange Laser Pointer",
		categories				= "projectiles",
		effect_desc				= "Your spells have a 25% chance of exploding into a barrage of incendiary laser bolts when they hit an obstacle or explode.",
		effect_desc_short		= "Your spells release incendiary laser bolts on impact.",
        precursor_types			= "magic_artifact",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("laser_pointer_orange") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 25 + (5 * getRelicLevel("laser_pointer_orange"))
				if(chance >= 1 and chance <= odds) then
					addBuffVisualIndicator_Projectile(projectile_entity_id, "lava")
					attachLoadEntityOnExplosiontoProjectile(projectile_entity_id, getLoadOnExplosion("launch_circular_projectiles_lasers_orange").path)
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "kyber_crystal,lens_orange",
	},
	{
		name 					= "laser_pointer_blue",
		ui_name					= "Blue Laser Pointer",
		categories				= "projectiles",
		effect_desc				= "Your spells have a 25% chance of exploding into a barrage of electrocuting laser bolts when they hit an obstacle or explode.",
		effect_desc_short		= "Your spells release electrocuting laser bolts on impact.",
        precursor_types			= "magic_artifact",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("laser_pointer_blue") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 25 + (5 * getRelicLevel("laser_pointer_blue"))
				if(chance >= 1 and chance <= odds) then
					addBuffVisualIndicator_Projectile(projectile_entity_id, "spark_blue_dark")
					attachLoadEntityOnExplosiontoProjectile(projectile_entity_id, getLoadOnExplosion("launch_circular_projectiles_lasers_blue").path)
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "kyber_crystal,lens_blue",
	},
	{
		name 					= "laser_pointer_teal",
		ui_name					= "Teal Laser Pointer",
		categories				= "projectiles",
		effect_desc				= "Your spells have a 25% chance of exploding into a barrage of freezing laser bolts when they hit an obstacle or explode.",
		effect_desc_short		= "Your spells release freezing laser bolts on impact.",
        precursor_types			= "magic_artifact",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("laser_pointer_teal") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 25 + (5 * getRelicLevel("laser_pointer_teal"))
				if(chance >= 1 and chance <= odds) then
					addBuffVisualIndicator_Projectile(projectile_entity_id, "spark_blue")
					attachLoadEntityOnExplosiontoProjectile(projectile_entity_id, getLoadOnExplosion("launch_circular_projectiles_lasers_teal").path)
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "kyber_crystal,lens_teal",
	},
	{
		name 					= "laser_pointer_green",
		ui_name					= "Green Laser Pointer",
		categories				= "projectiles",
		effect_desc				= "Your spells have a 25% chance of exploding into a barrage of acid laser bolts when they hit an obstacle or explode.",
		effect_desc_short		= "Your spells release acid laser bolts on impact.",
        precursor_types			= "magic_artifact",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("laser_pointer_green") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 25 + (5 * getRelicLevel("laser_pointer_green"))
				if(chance >= 1 and chance <= odds) then
					addBuffVisualIndicator_Projectile(projectile_entity_id, "plasma_fading_green")
					attachLoadEntityOnExplosiontoProjectile(projectile_entity_id, getLoadOnExplosion("launch_circular_projectiles_lasers_green").path)
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "kyber_crystal,lens_green",
	},
	{
		name 					= "laser_pointer_purple",
		ui_name					= "Purple Laser Pointer",
		categories				= "projectiles",
		effect_desc				= "Your spells have a 25% chance of exploding into a barrage of mutating laser bolts when they hit an obstacle or explode.",
		effect_desc_short		= "Your spells release mutating laser bolts on impact.",
        precursor_types			= "magic_artifact",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("laser_pointer_purple") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 25 + (5 * getRelicLevel("laser_pointer_purple"))
				if(chance >= 1 and chance <= odds) then
					addBuffVisualIndicator_Projectile(projectile_entity_id, "plasma_fading_pink")
					attachLoadEntityOnExplosiontoProjectile(projectile_entity_id, getLoadOnExplosion("launch_circular_projectiles_lasers_purple").path)
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "kyber_crystal,lens_purple",
	},
	{
		name 					= "mystic_crystal_withering",
		ui_name					= "Mystic Crystal of Withering",
		categories				= "enemies",
		effect_desc				= "When you kill an enemy with your spells, there's a 50% chance a \nportal that shoots acid orbs will spawn on their place of death.",
		effect_desc_short		= "Enemies you kill spawn portals that shoot acid orbs.",
        precursor_types			= "magic_artifact,effect_crystal",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onenemykill_function	= function(damage_type_bit_field, damage_message, entity_thats_responsible, drop_items)
			if(isRelicOwned("mystic_crystal_withering") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 50 + (5 * getRelicLevel("mystic_crystal_withering"))
				if(chance >= 1 and chance <= odds) then
					local enemy_x, enemy_y = EntityGetTransform(GetUpdatedEntityID())
					EntityLoad("mods/ArcaneReliquary/files/entity_definitions/others/portal_shooter_acid.xml", enemy_x, enemy_y)
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "mysterious_plant,acid_gem",
		
	},
	{
		name 					= "mystic_crystal_thunders",
		ui_name					= "Mystic Crystal of Thunders",
		categories				= "enemies",
		effect_desc				= "When you kill an enemy with your spells, there's a 50% chance a \nportal that shoots thunderbolts will spawn on their place of death.",
		effect_desc_short		= "Enemies you kill spawn portals that shoot thunderbolts.",
        precursor_types			= "magic_artifact,effect_crystal",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onenemykill_function	= function(damage_type_bit_field, damage_message, entity_thats_responsible, drop_items)
			if(isRelicOwned("mystic_crystal_thunders") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 50 + (5 * getRelicLevel("mystic_crystal_thunders"))
				if(chance >= 1 and chance <= odds) then
					local enemy_x, enemy_y = EntityGetTransform(GetUpdatedEntityID())
					EntityLoad("mods/ArcaneReliquary/files/entity_definitions/others/portal_shooter_thunderbolts.xml", enemy_x, enemy_y)
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "mysterious_plant,tesla_coil",
	},
	{
		name 					= "mystic_crystal_ashes",
		ui_name					= "Mystic Crystal of Ashes",
		categories				= "enemies",
		effect_desc				= "When you kill an enemy with your spells, there's a 50% chance a \nportal that shoots lava orbs will spawn on their place of death.",
		effect_desc_short		= "Enemies you kill spawn portals that shoot lava orbs.",
        precursor_types			= "magic_artifact,effect_crystal",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onenemykill_function	= function(damage_type_bit_field, damage_message, entity_thats_responsible, drop_items)
			if(isRelicOwned("mystic_crystal_ashes") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 50 + (5 * getRelicLevel("mystic_crystal_ashes"))
				if(chance >= 1 and chance <= odds) then
					local enemy_x, enemy_y = EntityGetTransform(GetUpdatedEntityID())
					EntityLoad("mods/ArcaneReliquary/files/entity_definitions/others/portal_shooter_lava.xml", enemy_x, enemy_y)
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "mysterious_plant,lava_gem",
	},
	{
		name 					= "mystic_crystal_pestilence",
		ui_name					= "Mystic Crystal of Pestilence",
		categories				= "enemies",
		effect_desc				= "When you kill an enemy with your spells, there's a 50% chance a \nportal that shoots poison orbs will spawn on their place of death.",
		effect_desc_short		= "Enemies you kill spawn portals that shoot poison orbs.",
        precursor_types			= "magic_artifact,effect_crystal",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onenemykill_function	= function(damage_type_bit_field, damage_message, entity_thats_responsible, drop_items)
			if(isRelicOwned("mystic_crystal_pestilence") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 50 + (5 * getRelicLevel("mystic_crystal_pestilence"))
				if(chance >= 1 and chance <= odds) then
					local enemy_x, enemy_y = EntityGetTransform(GetUpdatedEntityID())
					EntityLoad("mods/ArcaneReliquary/files/entity_definitions/others/portal_shooter_poison.xml", enemy_x, enemy_y)
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "mysterious_plant,poison_gem",
	},
	{
		name 					= "mystic_crystal_bloodiness",
		ui_name					= "Mystic Crystal of Bloodiness",
		categories				= "enemies",
		effect_desc				= "When you kill an enemy with your spells, there's a 50% chance a \nportal that shoots corrosive blood orbs will spawn on their place of death.",
		effect_desc_short		= "Enemies you kill spawn portals that shoot corrosive blood orbs.",
        precursor_types			= "magic_artifact,effect_crystal",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onenemykill_function	= function(damage_type_bit_field, damage_message, entity_thats_responsible, drop_items)
			if(isRelicOwned("mystic_crystal_bloodiness") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 50 + (5 * getRelicLevel("mystic_crystal_bloodiness"))
				if(chance >= 1 and chance <= odds) then
					local enemy_x, enemy_y = EntityGetTransform(GetUpdatedEntityID())
					EntityLoad("mods/ArcaneReliquary/files/entity_definitions/others/portal_shooter_blood.xml", enemy_x, enemy_y)
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "mysterious_plant,corrosive_blood_gem",
	},
	{
		name 					= "mystic_crystal_wetness",
		ui_name					= "Mystic Crystal of Wetness",
		categories				= "enemies",
		effect_desc				= "When you kill an enemy with your spells, there's a 50% chance a \nportal that shoots water orbs will spawn on their place of death.",
		effect_desc_short		= "Enemies you kill spawn portals that shoot water orbs.",
        precursor_types			= "magic_artifact,effect_crystal",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onenemykill_function	= function(damage_type_bit_field, damage_message, entity_thats_responsible, drop_items)
			if(isRelicOwned("mystic_crystal_wetness") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 50 + (5 * getRelicLevel("mystic_crystal_wetness"))
				if(chance >= 1 and chance <= odds) then
					local enemy_x, enemy_y = EntityGetTransform(GetUpdatedEntityID())
					EntityLoad("mods/ArcaneReliquary/files/entity_definitions/others/portal_shooter_water.xml", enemy_x, enemy_y)
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "mysterious_plant,water_gem",
	},
	{
		name 					= "mystic_crystal_emiction",
		ui_name					= "Mystic Crystal of Emiction",
		categories				= "enemies",
		effect_desc				= "When you kill an enemy with your spells, there's a 50% chance a \nportal that shoots urine orbs will spawn on their place of death.",
		effect_desc_short		= "Enemies you kill spawn portals that shoot urine orbs.",
        precursor_types			= "magic_artifact,effect_crystal",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onenemykill_function	= function(damage_type_bit_field, damage_message, entity_thats_responsible, drop_items)
			if(isRelicOwned("mystic_crystal_emiction") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 50 + (5 * getRelicLevel("mystic_crystal_emiction"))
				if(chance >= 1 and chance <= odds) then
					local enemy_x, enemy_y = EntityGetTransform(GetUpdatedEntityID())
					EntityLoad("mods/ArcaneReliquary/files/entity_definitions/others/portal_shooter_urine.xml", enemy_x, enemy_y)
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "mysterious_plant,urine_gem",
	},
	{
		name 					= "mystic_crystal_metamorphosis",
		ui_name					= "Mystic Crystal of Metamorphosis",
		categories				= "enemies",
		effect_desc				= "When you kill an enemy with your spells, there's a 50% chance a \nportal that shoots polymorphine orbs will spawn on their place of death.",
		effect_desc_short		= "Enemies you kill spawn portals that shoot polymorphine orbs.",
        precursor_types			= "magic_artifact,effect_crystal",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onenemykill_function	= function(damage_type_bit_field, damage_message, entity_thats_responsible, drop_items)
			if(isRelicOwned("mystic_crystal_metamorphosis") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 50 + (5 * getRelicLevel("mystic_crystal_metamorphosis"))
				if(chance >= 1 and chance <= odds) then
					local enemy_x, enemy_y = EntityGetTransform(GetUpdatedEntityID())
					EntityLoad("mods/ArcaneReliquary/files/entity_definitions/others/portal_shooter_poly.xml", enemy_x, enemy_y)
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "mysterious_plant,poly_gem",
	},
	{
		name 					= "mystic_crystal_slapping",
		ui_name					= "Mystic Crystal of Slapping",
		categories				= "enemies",
		effect_desc				= "When you kill an enemy with your spells, there's a 50% chance a \nportal that shoots slapperbombs will spawn on their place of death.",
		effect_desc_short		= "Enemies you kill spawn portals that shoot slapperbombs.",
        precursor_types			= "magic_artifact,effect_crystal",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onenemykill_function	= function(damage_type_bit_field, damage_message, entity_thats_responsible, drop_items)
			if(isRelicOwned("mystic_crystal_slapping") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 50 + (5 * getRelicLevel("mystic_crystal_slapping"))
				if(chance >= 1 and chance <= odds) then
					local enemy_x, enemy_y = EntityGetTransform(GetUpdatedEntityID())
					EntityLoad("mods/ArcaneReliquary/files/entity_definitions/others/portal_shooter_slap.xml", enemy_x, enemy_y)
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "mysterious_plant,fly_swatter",
	},
	{
		name 					= "mystic_crystal_chills",
		ui_name					= "Mystic Crystal of Chills",
		categories				= "enemies",
		effect_desc				= "When you kill an enemy with your spells, there's a 50% chance a \nportal that shoots freezebombs will spawn on their place of death.",
		effect_desc_short		= "Enemies you kill spawn portals that shoot freezebombs.",
        precursor_types			= "magic_artifact,effect_crystal",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onenemykill_function	= function(damage_type_bit_field, damage_message, entity_thats_responsible, drop_items)
			if(isRelicOwned("mystic_crystal_chills") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 50 + (5 * getRelicLevel("mystic_crystal_chills"))
				if(chance >= 1 and chance <= odds) then
					local enemy_x, enemy_y = EntityGetTransform(GetUpdatedEntityID())
					EntityLoad("mods/ArcaneReliquary/files/entity_definitions/others/portal_shooter_ice.xml", enemy_x, enemy_y)
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "mysterious_plant,popsicle",
	},
	{
		name 					= "mystic_crystal_entropy",
		ui_name					= "Mystic Crystal of Entropy",
		categories				= "enemies",
		effect_desc				= "When you kill an enemy with your spells, there's a 50% chance a \nportal that shoots electrobombs will spawn on their place of death.",
		effect_desc_short		= "Enemies you kill spawn portals that shoot electrobombs.",
        precursor_types			= "magic_artifact,effect_crystal",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onenemykill_function	= function(damage_type_bit_field, damage_message, entity_thats_responsible, drop_items)
			if(isRelicOwned("mystic_crystal_entropy") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 50 + (5 * getRelicLevel("mystic_crystal_entropy"))
				if(chance >= 1 and chance <= odds) then
					local enemy_x, enemy_y = EntityGetTransform(GetUpdatedEntityID())
					EntityLoad("mods/ArcaneReliquary/files/entity_definitions/others/portal_shooter_electricity.xml", enemy_x, enemy_y)
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "mysterious_plant,charged_necklace",
	},
	{
		name 					= "mystic_crystal_scorching",
		ui_name					= "Mystic Crystal of Scorching",
		categories				= "enemies",
		effect_desc				= "When you kill an enemy with your spells, there's a 50% chance a \nportal that shoots fireballs will spawn on their place of death.",
		effect_desc_short		= "Enemies you kill spawn portals that shoot fireballs.",
        precursor_types			= "magic_artifact,effect_crystal",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onenemykill_function	= function(damage_type_bit_field, damage_message, entity_thats_responsible, drop_items)
			if(isRelicOwned("mystic_crystal_scorching") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 50 + (5 * getRelicLevel("mystic_crystal_scorching"))
				if(chance >= 1 and chance <= odds) then
					local enemy_x, enemy_y = EntityGetTransform(GetUpdatedEntityID())
					EntityLoad("mods/ArcaneReliquary/files/entity_definitions/others/portal_shooter_fireballs.xml", enemy_x, enemy_y)
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "mysterious_plant,sacred_ember",
	},
	{
		name 					= "mystic_crystal_radiation",
		ui_name					= "Mystic Crystal of Radiation",
		categories				= "enemies",
		effect_desc				= "When you kill an enemy with your spells, there's a 50% chance a \nportal that shoots radioactive fireballs will spawn on their place of death.",
		effect_desc_short		= "Enemies you kill spawn portals that shoot radioactive fireballs.",
        precursor_types			= "magic_artifact,effect_crystal",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onenemykill_function	= function(damage_type_bit_field, damage_message, entity_thats_responsible, drop_items)
			if(isRelicOwned("mystic_crystal_radiation") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 50 + (5 * getRelicLevel("mystic_crystal_radiation"))
				if(chance >= 1 and chance <= odds) then
					local enemy_x, enemy_y = EntityGetTransform(GetUpdatedEntityID())
					EntityLoad("mods/ArcaneReliquary/files/entity_definitions/others/portal_shooter_fireballs_nuclear.xml", enemy_x, enemy_y)
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "mysterious_plant,uranium_ingot",
	},
	{
		name 					= "ball_of_goo_venomous",
		ui_name					= "Ball of Venomous Goo",
		categories				= "projectiles",
		effect_desc				= "Your spells have a 25% chance of exploding into a barrage of poison bolts when they hit an obstacle or explode.",
		effect_desc_short		= "Your spells release poison bolts on impact.",
        precursor_types			= "magic_artifact",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("ball_of_goo_venomous") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 25 + (5 * getRelicLevel("ball_of_goo_venomous"))
				if(chance >= 1 and chance <= odds) then
					addBuffVisualIndicator_Projectile(projectile_entity_id, "poison")
					attachLoadEntityOnExplosiontoProjectile(projectile_entity_id, getLoadOnExplosion("launch_circular_projectiles_poisonbolts").path)
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "ball_of_goo,poison_gem",
	},
	{
		name 					= "friendly_slime_poison",
		ui_name					= "Friendly Poison Slime",
		categories				= "companions",
		effect_desc				= "A friendly poison slime that shoots poison bolts joins you.",
		effect_desc_short		= "A poison bolt shooting companion.",
        precursor_types			= "companion",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			attachCompaniontoEntity(getPlayerEntity(), "friendly_slime_poison", "mortal")
		end,
		ondrop_function			= function()
			detachCompanionfromEntity(getPlayerEntity(), "friendly_slime_poison")
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "friendly_slime,poison_gem",
		requires_relic			= "ball_of_goo",
	},
	{
		name 					= "jar_of_goo_venomous",
		ui_name					= "Jar of Venomous Goo",
		categories				= "projectiles",
		effect_desc				= "Spells you cast have a 40% chance of inducing enemies to shoot poison bolts at random directions.",
		effect_desc_short		= "Spells you cast have a chance to induce enemies to randomly shoot poison bolts.",
        precursor_types			= "gadget",
		upgrade_effect_desc		= "For each upgrade level, effect chance is increased by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("jar_of_goo_venomous") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 40 + (5 * getRelicLevel("jar_of_goo_venomous"))
				if(chance >= 1 and chance <= odds) then
					attachLoadEffectOntoEntitiestoProjectile(projectile_entity_id, getCustomEffect("effect_shoot_projectile_random_poisonbolt").path, "spark_purple_bright")
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "empty_bottle,ball_of_goo_venomous",
	},
	{
		name 					= "shuriken_poisonous",
		ui_name					= "Poisonous Shuriken",
		categories				= "projectiles,status_effects",
		effect_desc				= "Your spells have a 30% chance of inducing the \npoison bolt slash effect on enemies hit by them.",
		effect_desc_short		= "Your spells induce the poison bolt slash effect on enemies.",
        precursor_types			= "magic_artifact,gadget",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("shuriken_poisonous") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 30 + (5 * getRelicLevel("shuriken_poisonous"))
				if(chance >= 1 and chance <= odds) then
					attachLoadEffectOntoEntitiestoProjectile(projectile_entity_id, getCustomEffect("effect_slash_projectile_poisonbolt").path, "spark_purple_bright")
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "shuriken,corrosive_blood_gem",
	},
	{
		name 					= "sawn_log",
		ui_name					= "Sawn Log",
		categories				= "player",
		effect_desc				= "A mystic chainsaw rotates around you, dragging and damaging enemies it touches.",
		effect_desc_short		= "A mystic chainsaw rotates around you, dragging and damaging enemies it touches.",
        precursor_types			= "gravity_stone",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		ondrop_function			= function()
			detachRotatingShieldfromEntity(getPlayerEntity(), "rotating_shield_projectile_chainsaw")
		end,
		overtime_function		= function()
			if(isRelicOwned("sawn_log") >= 1) then
				if(not entityHasRotatingShield(getPlayerEntity(), "rotating_shield_projectile_chainsaw")) then -- this makes sure the orbiting projectiles are loaded again if the player is polymorphed
					local start_id = EntityLoad("mods/ArcaneReliquary/files/entity_definitions/custom_projectiles/rotating_shield_start.xml")
					setInternalVariableValue(start_id, "host_entity_id", "value_int", getPlayerEntity())
					setInternalVariableValue(start_id, "rotating_projectile_definition", "value_string", "mods/ArcaneReliquary/files/entity_definitions/custom_projectiles/rotating_chainsaw.xml")
					setInternalVariableValue(start_id, "rotating_projectiles_amount", "value_int", 1)
					setInternalVariableValue(start_id, "radius", "value_int", 40)
					setInternalVariableValue(start_id, "rotation_speed", "value_int", 6)
				end
			end
		end,
	},
	{
		name 					= "damage_absorption_artifact",
		ui_name					= "Damage Absorption Artifact",
		categories				= "player",
		effect_desc				= "You absorb damage taken directly from enemy projectiles (not counting explosive damage), and \nall damage absorbed is converted into bonus damage for your next spell cast. \nWhen it happens you also recover health equivalent to 95% of all damage taken previously.",
		effect_desc_short		= "Damage taken from enemy projectiles is converted into bonus damage for your next spell cast.",
        precursor_types			= "magic_artifact",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		ondamaged_function	= function(damage, message, entity_thats_responsible, is_fatal, projectile_thats_responsible )
			if(isRelicOwned("damage_absorption_artifact") >= 1) then
				if(message == "$damage_projectile") then
					-- adds to absorbed damage counter
					local damage_absorbed = getInternalVariableValue(getPlayerEntity(), "damage_absorbed", "value_float")
					setInternalVariableValue(getPlayerEntity(), "damage_absorbed", "value_float", damage_absorbed + damage)
				end
			end
		end,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("damage_absorption_artifact") >= 1) then
				-- adds to base damage based on amount absorbed
				local damage_absorbed = getInternalVariableValue(getPlayerEntity(), "damage_absorbed", "value_float")
				if(damage_absorbed > 0) then

					addProjectileBaseDamage(projectile_entity_id, damage_absorbed)
					-- adds damage portion back as health
					addPlayerHealth((damage_absorbed * 25) * 95 / 100, false)
					
					-- visual feedback on projectile
					addBuffVisualIndicator_Projectile(projectile_entity_id, "spark_yellow")
					
					-- clears damage absorbed
					setInternalVariableValue(getPlayerEntity(), "damage_absorbed", "value_float", 0)
				end
			end
		end,
	},
	{
		name 					= "damage_absorption_artifact_melee",
		ui_name					= "Melee Damage Absorption Artifact",
		categories				= "player",
		effect_desc				= "You absorb damage taken from melee attacks, and all damage absorbed is converted into bonus \ndamage for your next kick. When it happens you also recover health equivalent \nto 95% of all damage taken previously.",
		effect_desc_short		= "Damage taken from melee attacks is converted into bonus damage for your next kick.",
        precursor_types			= "magic_artifact",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		ondamaged_function	= function(damage, message, entity_thats_responsible, is_fatal, projectile_thats_responsible )
			if(isRelicOwned("damage_absorption_artifact_melee") >= 1) then
				if(message == "$damage_melee") then
					-- adds to absorbed damage counter
					local damage_absorbed_melee = getInternalVariableValue(getPlayerEntity(), "damage_absorbed_melee", "value_float")
					setInternalVariableValue(getPlayerEntity(), "damage_absorbed_melee", "value_float", damage_absorbed_melee + damage)
				end
			end
		end,
		onenemydamaged_function	= function(damage, message, entity_thats_responsible, is_fatal, projectile_thats_responsible )
			if(isRelicOwned("damage_absorption_artifact_melee") >= 1) then
				if(message == "$damage_kick") then
					-- inflicts extra melee damage when a kick lands
					local damage_absorbed_melee = getInternalVariableValue(getPlayerEntity(), "damage_absorbed_melee", "value_float")
					EntityInflictDamage( GetUpdatedEntityID(), damage_absorbed_melee, "DAMAGE_MELEE", "Melee Damage Absorption damage", "BLOOD_SPRAY", 0, 0)
					-- adds damage portion back as health
					addPlayerHealth((damage_absorbed_melee * 25) * 95 / 100, false)
					-- clears melee damage absorbed
					setInternalVariableValue(getPlayerEntity(), "damage_absorbed_melee", "value_float", 0)
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "damage_absorption_artifact,emerald_gauntlet",
	},
	{
		name 					= "damage_absorption_artifact_explosions",
		ui_name					= "Explosion Damage Absorption Artifact",
		categories				= "player",
		effect_desc				= "You absorb damage taken from explosions, and \nall damage absorbed is converted into bonus explosive radius and damage for your next spell cast. \nWhen it happens you also recover health equivalent to 95% of all damage taken previously.",
		effect_desc_short		= "Damage taken from explosions is converted into bonus explosive radius and damage for your next spell cast.",
        precursor_types			= "magic_artifact",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		ondamaged_function	= function(damage, message, entity_thats_responsible, is_fatal, projectile_thats_responsible )
			if(isRelicOwned("damage_absorption_artifact_explosions") >= 1) then
				if(message == "$damage_explosion") then
					-- adds to absorbed damage counter
					local damage_absorbed_explosion = getInternalVariableValue(getPlayerEntity(), "damage_absorbed_explosion", "value_float")
					setInternalVariableValue(getPlayerEntity(), "damage_absorbed_explosion", "value_float", damage_absorbed_explosion + damage)
				end
			end
		end,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("damage_absorption_artifact_explosions") >= 1) then
				-- adds to base damage based on amount absorbed
				local damage_absorbed_explosion = getInternalVariableValue(getPlayerEntity(), "damage_absorbed_explosion", "value_float")
				if(damage_absorbed_explosion > 0) then
					
					-- adds bonus explosive damage and multiplies explosion radius
					addProjectileCreateCellMaterialExplosion(projectile_entity_id, "spark_red")
					if(damage_absorbed_explosion > 1) then
						multiplyProjectileExplosionRadius(projectile_entity_id, damage_absorbed_explosion)
					end
					addProjectileExplosiveDamage(projectile_entity_id, damage_absorbed_explosion)
					
					-- visual feedback on projectile
					addBuffVisualIndicator_Projectile(projectile_entity_id, "spark_red")
					
					-- adds damage portion back as health
					addPlayerHealth((damage_absorbed_explosion * 25) * 95 / 100, false)
					
					-- clears damage absorbed
					setInternalVariableValue(getPlayerEntity(), "damage_absorbed_explosion", "value_float", 0)
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "damage_absorption_artifact,explosive_mana_crystal",
	},
	
	{
		name 					= "damage_absorption_artifact_fire",
		ui_name					= "Fire Damage Absorption Artifact",
		categories				= "player",
		effect_desc				= "You absorb damage taken directly from fire, and \nall damage absorbed is converted into bonus damage for your next spell cast. \nWhen it happens you also recover health equivalent to 95% of all damage taken previously.",
		effect_desc_short		= "Damage taken from fire is converted into bonus damage for your next spell cast.",
        precursor_types			= "magic_artifact",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		ondamaged_function	= function(damage, message, entity_thats_responsible, is_fatal, projectile_thats_responsible )
			if(isRelicOwned("damage_absorption_artifact_fire") >= 1) then
				if(message == "$damage_fire") then
					-- adds to absorbed damage counter
					local damage_absorbed_fire = getInternalVariableValue(getPlayerEntity(), "damage_absorbed_fire", "value_float")
					setInternalVariableValue(getPlayerEntity(), "damage_absorbed_fire", "value_float", damage_absorbed_fire + damage)
				end
			end
		end,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("damage_absorption_artifact_fire") >= 1) then
				-- adds to base damage based on amount absorbed
				local damage_absorbed_fire = getInternalVariableValue(getPlayerEntity(), "damage_absorbed_fire", "value_float")
				if(damage_absorbed_fire > 0) then
				
					addProjectileBaseDamage(projectile_entity_id, damage_absorbed_fire)
					-- adds damage portion back as health
					addPlayerHealth((damage_absorbed_fire * 25) * 95 / 100, false)
					
					-- visual feedback on projectile
					addBuffVisualIndicator_Projectile(projectile_entity_id, "fire")
					
					-- clears damage absorbed
					setInternalVariableValue(getPlayerEntity(), "damage_absorbed_fire", "value_float", 0)
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "damage_absorption_artifact,sacred_ember",
	},
	
	{
		name 					= "organic_heart",
		ui_name					= "Organic Heart",
		categories				= "player",
		effect_desc				= "A pulsating energy field emanates from you. Enemy projectiles \nare deflected by it and imbued with the bleeding effect. \nEnemies touched by it are also inflicted with this effect.",
		effect_desc_short		= "A pulsating energy field that deflects and imbues enemy projectiles with the bleeding effect.",
        precursor_types			= "magic_artifact,energy_shield",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			LoadGameEffectEntityTo( getPlayerEntity(), getCustomEffect("effect_pulsating_shield_projectile_deflect_bleed").path )
		end,
		ondrop_function			= function()
			local player_children = EntityGetAllChildren(getPlayerEntity())
			if player_children == nil then return end
			for i,child in ipairs(player_children) do
				if(EntityGetName(child) == "pulsating_shield_projectiles_deflect_bleed") then
					EntityKill(child)
					break
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "mechanical_heart,ancient_dagger",
	},
	{
		name 					= "organic_heart_slap",
		ui_name					= "Lush Heart",
		categories				= "player",
		effect_desc				= "A pulsating energy field emanates from you. Enemy projectiles \nare deflected by it and imbued with the slap effect. \nEnemies touched by it are also inflicted with this effect.",
		effect_desc_short		= "A pulsating energy field that deflects and imbues enemy projectiles with the slap effect.",
        precursor_types			= "magic_artifact,energy_shield",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			LoadGameEffectEntityTo( getPlayerEntity(), getCustomEffect("effect_pulsating_shield_projectile_deflect_slap").path )
		end,
		ondrop_function			= function()
			local player_children = EntityGetAllChildren(getPlayerEntity())
			if player_children == nil then return end
			for i,child in ipairs(player_children) do
				if(EntityGetName(child) == "pulsating_shield_projectiles_deflect_slap") then
					EntityKill(child)
					break
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "mechanical_heart,fly_swatter",
	},
	{
		name 					= "organic_heart_wind",
		ui_name					= "Marble Heart",
		categories				= "player",
		effect_desc				= "A pulsating energy field emanates from you. Enemy projectiles \nare deflected by it and imbued with the wind effect. \nEnemies touched by it are also inflicted with this effect.",
		effect_desc_short		= "A pulsating energy field that deflects and imbues enemy projectiles with the wind effect.",
        precursor_types			= "magic_artifact,energy_shield",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			LoadGameEffectEntityTo( getPlayerEntity(), getCustomEffect("effect_pulsating_shield_projectile_deflect_wind").path )
		end,
		ondrop_function			= function()
			local player_children = EntityGetAllChildren(getPlayerEntity())
			if player_children == nil then return end
			for i,child in ipairs(player_children) do
				if(EntityGetName(child) == "pulsating_shield_projectiles_deflect_wind") then
					EntityKill(child)
					break
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "mechanical_heart,little_cloud",
	},
	{
		name 					= "organic_heart_petrify",
		ui_name					= "Granite Heart",
		categories				= "player",
		effect_desc				= "A pulsating energy field emanates from you. Enemy projectiles \nare deflected by it and imbued with the petrify effect. \nEnemies touched by it are also inflicted with this effect.",
		effect_desc_short		= "A pulsating energy field that deflects and imbues enemy projectiles with the petrify effect.",
        precursor_types			= "magic_artifact,energy_shield",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			LoadGameEffectEntityTo( getPlayerEntity(), getCustomEffect("effect_pulsating_shield_projectile_deflect_petrify").path )
		end,
		ondrop_function			= function()
			local player_children = EntityGetAllChildren(getPlayerEntity())
			if player_children == nil then return end
			for i,child in ipairs(player_children) do
				if(EntityGetName(child) == "pulsating_shield_projectiles_deflect_petrify") then
					EntityKill(child)
					break
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "mechanical_heart,medusas_medalion",
	},
	{
		name 					= "organic_heart_exploding",
		ui_name					= "Igneous Heart",
		categories				= "player",
		effect_desc				= "A pulsating energy field emanates from you. Enemy projectiles \nare deflected by it and imbued with the exploding effect. \nEnemies touched by it are also inflicted with this effect.",
		effect_desc_short		= "A pulsating energy field that deflects and imbues enemy projectiles with the exploding effect.",
        precursor_types			= "magic_artifact,energy_shield",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			LoadGameEffectEntityTo( getPlayerEntity(), getCustomEffect("effect_pulsating_shield_projectile_deflect_exploding").path )
		end,
		ondrop_function			= function()
			local player_children = EntityGetAllChildren(getPlayerEntity())
			if player_children == nil then return end
			for i,child in ipairs(player_children) do
				if(EntityGetName(child) == "pulsating_shield_projectiles_deflect_exploding") then
					EntityKill(child)
					break
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "mechanical_heart,freedom_ring",
	},
	{
		name 					= "organic_heart_instakill",
		ui_name					= "Venomous Heart",
		categories				= "player",
		effect_desc				= "A pulsating energy field emanates from you. Enemy projectiles \nare deflected by it and imbued with the instant kill effect. \nEnemies touched by it are also inflicted with this effect.",
		effect_desc_short		= "A pulsating energy field that deflects and imbues enemy projectiles with the instant kill effect.",
        precursor_types			= "magic_artifact,energy_shield",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			LoadGameEffectEntityTo( getPlayerEntity(), getCustomEffect("effect_pulsating_shield_projectile_deflect_instakill").path )
		end,
		ondrop_function			= function()
			local player_children = EntityGetAllChildren(getPlayerEntity())
			if player_children == nil then return end
			for i,child in ipairs(player_children) do
				if(EntityGetName(child) == "pulsating_shield_projectiles_deflect_instakill") then
					EntityKill(child)
					break
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "mechanical_heart,deceasing_stone",
	},
	{
		name 					= "organic_heart_thunderbolts",
		ui_name					= "Overloaded Heart",
		categories				= "player",
		effect_desc				= "A pulsating energy field emanates from you. Enemy projectiles \nare deflected by it and imbued with the thunderbolts effect. \nEnemies touched by it are also inflicted with this effect.",
		effect_desc_short		= "A pulsating energy field that deflects and imbues enemy projectiles with the thunderbolts effect.",
        precursor_types			= "magic_artifact,energy_shield",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			LoadGameEffectEntityTo( getPlayerEntity(), getCustomEffect("effect_pulsating_shield_projectile_deflect_thunderbolts").path )
		end,
		ondrop_function			= function()
			local player_children = EntityGetAllChildren(getPlayerEntity())
			if player_children == nil then return end
			for i,child in ipairs(player_children) do
				if(EntityGetName(child) == "pulsating_shield_projectiles_deflect_thunderbolts") then
					EntityKill(child)
					break
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "mechanical_heart,tesla_coil",
	},
	{
		name 					= "organic_heart_freeze_solid",
		ui_name					= "Blizzard Heart",
		categories				= "player",
		effect_desc				= "A pulsating energy field emanates from you. Enemy projectiles \nare deflected by it and imbued with the freeze solid effect. \nEnemies touched by it are also inflicted with this effect.",
		effect_desc_short		= "A pulsating energy field that deflects and imbues enemy projectiles with the freeze solid effect.",
        precursor_types			= "magic_artifact,energy_shield",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			LoadGameEffectEntityTo( getPlayerEntity(), getCustomEffect("effect_pulsating_shield_projectile_deflect_freeze_solid").path )
		end,
		ondrop_function			= function()
			local player_children = EntityGetAllChildren(getPlayerEntity())
			if player_children == nil then return end
			for i,child in ipairs(player_children) do
				if(EntityGetName(child) == "pulsating_shield_projectiles_deflect_freeze_solid") then
					EntityKill(child)
					break
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "mechanical_heart,wendigos_medalion",
	},
	{
		name 					= "organic_heart_timebomb",
		ui_name					= "Rigged Heart",
		categories				= "player",
		effect_desc				= "A pulsating energy field emanates from you. Enemy projectiles \nare deflected by it and imbued with the time bomb effect. \nEnemies touched by it are also inflicted with this effect.",
		effect_desc_short		= "A pulsating energy field that deflects and imbues enemy projectiles with the time bomb effect.",
        precursor_types			= "magic_artifact,energy_shield",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			LoadGameEffectEntityTo( getPlayerEntity(), getCustomEffect("effect_pulsating_shield_projectile_deflect_timebomb").path )
		end,
		ondrop_function			= function()
			local player_children = EntityGetAllChildren(getPlayerEntity())
			if player_children == nil then return end
			for i,child in ipairs(player_children) do
				if(EntityGetName(child) == "pulsating_shield_projectiles_deflect_timebomb") then
					EntityKill(child)
					break
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "mechanical_heart,time_bomb",
	},
	{
		name 					= "organic_heart_slash",
		ui_name					= "Rose Heart",
		categories				= "player",
		effect_desc				= "A pulsating energy field emanates from you. Enemy projectiles \nare deflected by it and imbued with the slash effect. \nEnemies touched by it are also inflicted with this effect.",
		effect_desc_short		= "A pulsating energy field that deflects and imbues enemy projectiles with the slash effect.",
        precursor_types			= "magic_artifact,energy_shield",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			LoadGameEffectEntityTo( getPlayerEntity(), getCustomEffect("effect_pulsating_shield_projectile_deflect_slash").path )
		end,
		ondrop_function			= function()
			local player_children = EntityGetAllChildren(getPlayerEntity())
			if player_children == nil then return end
			for i,child in ipairs(player_children) do
				if(EntityGetName(child) == "pulsating_shield_projectiles_deflect_slash") then
					EntityKill(child)
					break
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "mechanical_heart,shuriken",
	},
	{
		name 					= "security_probe_white",
		ui_name					= "Security Probe with White Lens",
		categories				= "companions",
		effect_desc				= "A security probe companion that shoots slash laser beams joins you.",
		effect_desc_short		= "A slash laser beam shooting companion.",
        precursor_types			= "companion",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			attachCompaniontoEntity(getPlayerEntity(), "security_probe_white", "mortal")
		end,
		ondrop_function			= function()
			detachCompanionfromEntity(getPlayerEntity(), "security_probe_white")
		end,
		requires_relic			= "alkaline_battery",
		combine_exclusive		= true,
		combine_requires_relics	= "security_probe,shuriken",
	},
	{
		name 					= "kill_rewards_hiisi",
		ui_name					= "Hiisi Spine",
		categories				= "health,enemies",
		effect_desc				= "For every hiisi you kill, 2 gold are instantly rewarded.",
		effect_desc_short		= "For every hiisi you kill, 2 gold are instantly rewarded.",
        precursor_types			= "magic_artifact",
		upgrade_effect_desc		= "For each upgrade level, gold gained from kills is increased.",
		is_upgradable 			= true,
		onenemykill_function	= function(damage_type_bit_field, damage_message, entity_thats_responsible, drop_items)			
			if(isRelicOwned("kill_rewards_hiisi") >= 1) then
				
				if(getCreatureHerdName(GetUpdatedEntityID()) == "orcs") then
					local gold_earned = 2 + (2 * getRelicLevel("kill_rewards_hiisi"))
					GamePrint(gold_earned .. " gold earned for hiisi kill")
					
					-- audio feedback
					local x, y = EntityGetTransform(getPlayerEntity())
					GamePlaySound( "data/audio/Desktop/event_cues.snd", "event_cues/goldnugget/create", x, y )
					
					-- adds gold on kill
					addPlayerGold(gold_earned)
				end
				
			end
		end,
	},
	{
		name 					= "kill_rewards_slimes",
		ui_name					= "Slime Skull",
		categories				= "gold,enemies",
		effect_desc				= "For every slime you kill, 2 health are instantly rewarded.",
		effect_desc_short		= "For every slime you kill, 2 health are instantly rewarded.",
        precursor_types			= "magic_artifact",
		upgrade_effect_desc		= "For each upgrade level, health gained from kills is increased.",
		is_upgradable 			= true,
		onenemykill_function	= function(damage_type_bit_field, damage_message, entity_thats_responsible, drop_items)			
			if(isRelicOwned("kill_rewards_slimes") >= 1) then
				
				if(getCreatureHerdName(GetUpdatedEntityID()) == "slimes") then
					local hp_earned = 2 + (2 * getRelicLevel("kill_rewards_slimes"))
					GamePrint(hp_earned .. " hp earned for slime kill")
					
					-- audio feedback
					local x, y = EntityGetTransform(getPlayerEntity())
					GamePlaySound( "data/audio/Desktop/event_cues.snd", "event_cues/heart/create", x, y )
					
					-- adds hp on kill
					addPlayerHealth(hp_earned, false)
				end
				
			end
		end,
	},
	{
		name 					= "kill_rewards_mages",
		ui_name					= "Mage on Ambar",
		categories				= "health,gold,enemies",
		effect_desc				= "For every mage you kill, 5 health and 5 gold are instantly rewarded.",
		effect_desc_short		= "For every mage you kill, 5 health and 5 gold are instantly rewarded.",
        precursor_types			= "magic_artifact",
		upgrade_effect_desc		= "For each upgrade level, health and gold gained from kills is increased.",
		is_upgradable 			= true,
		onenemykill_function	= function(damage_type_bit_field, damage_message, entity_thats_responsible, drop_items)			
			if(isRelicOwned("kill_rewards_mages") >= 1) then
				
				if(getCreatureHerdName(GetUpdatedEntityID()) == "mage") then
				
					local hp_earned = 5 + (5 * getRelicLevel("kill_rewards_mages"))
					GamePrint(hp_earned .. " hp earned for mage kill")
					-- audio feedback
					local x, y = EntityGetTransform(getPlayerEntity())
					GamePlaySound( "data/audio/Desktop/event_cues.snd", "event_cues/heart/create", x, y )
					-- adds hp on kill
					addPlayerHealth(hp_earned, false)
					
					local gold_earned = 5 + (5 * getRelicLevel("kill_rewards_mages"))
					GamePrint(gold_earned .. " gold earned for mage kill")
					-- audio feedback
					local x, y = EntityGetTransform(getPlayerEntity())
					GamePlaySound( "data/audio/Desktop/event_cues.snd", "event_cues/goldnugget/create", x, y )
					-- adds gold on kill
					addPlayerGold(gold_earned)
				end
				
			end
		end,
	},
	{
		name 					= "kill_rewards_zombies_insects",
		ui_name					= "Pest Control",
		categories				= "spells,enemies",
		effect_desc				= "For every zombie, insect or worm you kill, there's a 5% chance a random spell card will be dropped.",
		effect_desc_short		= "For every zombie, insect or worm you kill, there's a chance they will drop a random spell card.",
        precursor_types			= "magic_artifact",
		upgrade_effect_desc		= "For each upgrade level, spell card drop chance increases by 5%.",
		is_upgradable 			= true,
		onenemykill_function	= function(damage_type_bit_field, damage_message, entity_thats_responsible, drop_items)			
			if(isRelicOwned("kill_rewards_zombies_insects") >= 1) then
				
				local creature_herd_name = getCreatureHerdName(GetUpdatedEntityID())
				if (creature_herd_name == "zombie" or creature_herd_name == "ant" or creature_herd_name == "fly" or creature_herd_name == "worm" or creature_herd_name == "spider") then
				
					SetRandomSeed( GameGetFrameNum(), math.random(1,50))
					local chance = Random(1,100)
					local odds = 5 + (5 * getRelicLevel("kill_rewards_zombies_insects"))
					if(chance >= 1 and chance <= odds) then
						GamePrint("Spell card dropped on pest kill")
						local death_x, death_y = EntityGetTransform(GetUpdatedEntityID())
						CreateItemActionEntity( getRandomActionId(), death_x, death_y)
					end
				end
				
			end
		end,
	},
	{
		name 					= "kill_rewards_mages_perk",
		ui_name					= "Golden Medal",
		categories				= "spells,enemies",
		effect_desc				= "For every mage you kill, there's a 5% chance a random perk card will be dropped.",
		effect_desc_short		= "For every mage you kill, there's a chance they will drop a random spell card.",
        precursor_types			= "magic_artifact",
		upgrade_effect_desc		= "For each upgrade level, spell card drop chance increases by 5%.",
		is_upgradable 			= true,
		onenemykill_function	= function(damage_type_bit_field, damage_message, entity_thats_responsible, drop_items)			
			if(isRelicOwned("kill_rewards_mages_perk") >= 1) then
				
				local creature_herd_name = getCreatureHerdName(GetUpdatedEntityID())
				if (creature_herd_name == "mage") then
				
					SetRandomSeed( GameGetFrameNum(), math.random(1,50))
					local chance = Random(1,100)
					local odds = 5 + (5 * getRelicLevel("kill_rewards_mages_perk"))
					if(chance >= 1 and chance <= odds) then
						GamePrint("Perk card dropped on mage kill")
						local death_x, death_y = EntityGetTransform(GetUpdatedEntityID())
						spawnRandomPerkCard(death_x, death_y - 10)
					end
				end
				
			end
		end,
	},
	{
		name 					= "match_box",
		ui_name					= "Match Box",
		categories				= "projectiles,status_effects",
		effect_desc				= "Spells you cast have a 40% chance of inducing enemies to shoot firebombs at random directions.",
		effect_desc_short		= "Spells you cast have a chance to induce enemies to randomly shoot firebombs.",
        precursor_types			= "gadget",
		upgrade_effect_desc		= "For each upgrade level, effect chance is increased by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("match_box") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 40 + (5 * getRelicLevel("match_box"))
				if(chance >= 1 and chance <= odds) then
					attachLoadEffectOntoEntitiestoProjectile(projectile_entity_id, getCustomEffect("effect_shoot_projectile_random_firebomb").path, "fire")
				end
			end
		end,
	},
	{
		name 					= "energy_sphere",
		ui_name					= "Energy Sphere",
		categories				= "projectiles,status_effects",
		effect_desc				= "Spells you cast have a 40% chance of inducing enemies to shoot fireballs at random directions.",
		effect_desc_short		= "Spells you cast have a chance to induce enemies to randomly shoot fireballs.",
        precursor_types			= "gadget",
		upgrade_effect_desc		= "For each upgrade level, effect chance is increased by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("energy_sphere") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 40 + (5 * getRelicLevel("energy_sphere"))
				if(chance >= 1 and chance <= odds) then
					attachLoadEffectOntoEntitiestoProjectile(projectile_entity_id, getCustomEffect("effect_shoot_projectile_random_fireball").path, "fire")
				end
			end
		end,
	},
	{
		name 					= "energy_sphere_frozen",
		ui_name					= "Frozen Energy Sphere",
		categories				= "projectiles,status_effects",
		effect_desc				= "Spells you cast have a 40% chance of inducing enemies to shoot iceballs at random directions.",
		effect_desc_short		= "Spells you cast have a chance to induce enemies to randomly shoot iceballs.",
        precursor_types			= "gadget",
		upgrade_effect_desc		= "For each upgrade level, effect chance is increased by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("energy_sphere_frozen") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 40 + (5 * getRelicLevel("energy_sphere_frozen"))
				if(chance >= 1 and chance <= odds) then
					attachLoadEffectOntoEntitiestoProjectile(projectile_entity_id, getCustomEffect("effect_shoot_projectile_random_iceball").path, "diamond")
				end
			end
		end,
	},
	{
		name 					= "energy_sphere_nuclear",
		ui_name					= "Nuclear Energy Sphere",
		categories				= "projectiles,status_effects",
		effect_desc				= "Spells you cast have a 40% chance of inducing enemies to shoot radioactive fireballs at random directions.",
		effect_desc_short		= "Spells you cast have a chance to induce enemies to randomly shoot radioactive fireballs.",
        precursor_types			= "gadget",
		upgrade_effect_desc		= "For each upgrade level, effect chance is increased by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("energy_sphere_nuclear") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 40 + (5 * getRelicLevel("energy_sphere_nuclear"))
				if(chance >= 1 and chance <= odds) then
					attachLoadEffectOntoEntitiestoProjectile(projectile_entity_id, getCustomEffect("effect_shoot_projectile_random_fireball_nuclear").path, "plasma_fading_green")
				end
			end
		end,
	},
	{
		name 					= "mystic_shield_scatter",
		ui_name					= "Scattering Mystic Shield",
		categories				= "companions",
		effect_desc				= "A mystic shield that destroys and scatters enemy projectiles appears and follows your mouse cursor.",
		effect_desc_short		= "A mystic shield that destroys and scatters enemy projectiles appears and follows your mouse cursor.",
        precursor_types			= "companion",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			attachCompaniontoEntity(getPlayerEntity(), "mystic_shield_scatter", "mortal")
		end,
		ondrop_function			= function()
			detachCompanionfromEntity(getPlayerEntity(), "mystic_shield_scatter")
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "mystic_shield,ancient_root",
	},
	{
		name 					= "mystic_shield_lava",
		ui_name					= "Lava Mystic Shield",
		categories				= "companions",
		effect_desc				= "A mystic shield that destroys enemy projectiles and shoots lava orbs back appears and follows your mouse cursor.",
		effect_desc_short		= "A mystic shield that destroys enemy projectiles and shoots lava orbs back appears and follows your mouse cursor.",
        precursor_types			= "companion",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			attachCompaniontoEntity(getPlayerEntity(), "mystic_shield_lava", "mortal")
		end,
		ondrop_function			= function()
			detachCompanionfromEntity(getPlayerEntity(), "mystic_shield_lava")
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "mystic_shield,lava_gem",
	},
	{
		name 					= "mystic_shield_acid",
		ui_name					= "Acid Mystic Shield",
		categories				= "companions",
		effect_desc				= "A mystic shield that destroys enemy projectiles and shoots acid orbs back appears and follows your mouse cursor.",
		effect_desc_short		= "A mystic shield that destroys enemy projectiles and shoots acid orbs back appears and follows your mouse cursor.",
        precursor_types			= "companion",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			attachCompaniontoEntity(getPlayerEntity(), "mystic_shield_acid", "mortal")
		end,
		ondrop_function			= function()
			detachCompanionfromEntity(getPlayerEntity(), "mystic_shield_acid")
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "mystic_shield,acid_gem",
	},
	{
		name 					= "mystic_shield_poison",
		ui_name					= "Poison Mystic Shield",
		categories				= "companions",
		effect_desc				= "A mystic shield that destroys enemy projectiles and shoots poison orbs back appears and follows your mouse cursor.",
		effect_desc_short		= "A mystic shield that destroys enemy projectiles and shoots poison orbs back appears and follows your mouse cursor.",
        precursor_types			= "companion",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			attachCompaniontoEntity(getPlayerEntity(), "mystic_shield_poison", "mortal")
		end,
		ondrop_function			= function()
			detachCompanionfromEntity(getPlayerEntity(), "mystic_shield_poison")
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "mystic_shield,poison_gem",
	},
	{
		name 					= "meteor_fragment_granite",
		ui_name					= "Granite Meteor Fragment",
		categories				= "projectiles",
		effect_desc				= "Spells you cast in a downward angle have a 30% chance of creating \nan earth shockwave, that deal impact damage to enemies, when they hit something. \nIt also synergizes with Earth Core Fragment relic effect.",
		effect_desc_short		= "Spells cast downwards create earth shockwaves on the ground.",
        precursor_types			= "magic_artifact",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5% and shockwave reaches longer.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("meteor_fragment_granite") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 30 + (5 * getRelicLevel("meteor_fragment_granite"))
				if(chance >= 1 and chance <= odds) then
					local projectile_vel_x, projectile_vel_y = getProjectileVelocitiesandGravity(projectile_entity_id)
					if(projectile_vel_y > 0) then
						replaceProjectileExplosionSprite(projectile_entity_id, "")
						projectileRemoveBouncing(projectile_entity_id)
						if(not projectileExplodesonImpact(projectile_entity_id)) then
							projectileSetExplosionConfig(projectile_entity_id, 1)
						end
						attachLoadEntityOnExplosiontoProjectile(projectile_entity_id, getLoadOnExplosion("shockwave_earth").path, "spark")
					end
				end
			end
		end,
		ongroundpound_function	= function( entity_id)
			if(isRelicOwned("meteor_fragment_granite") >= 1) then
				local game_effect_comp = GetGameEffectLoadTo( entity_id, "PROTECTION_ALL", true )
				if game_effect_comp ~= nil then
					ComponentSetValue( game_effect_comp, "frames", "40" )
				end
				local x, y = EntityGetTransform(entity_id)
				EntityLoad("mods/ArcaneReliquary/files/entity_definitions/load_on_explosion/shockwave_earth.xml", x, y)
			end
		end,
		ondrop_function			= function()
			removeInternalVariable(getPlayerEntity(), "earth_shockwave_length_bonus")
		end,
		overtime_function		= function()
			local earth_shockwave_length_bonus = getRelicLevel("meteor_fragment_granite")

			if(not entityHasInternalVariableValue(getPlayerEntity(), "earth_shockwave_length_bonus")) then
				addNewInternalVariable(getPlayerEntity(), "earth_shockwave_length_bonus", "value_int", earth_shockwave_length_bonus)
			end
			setInternalVariableValue(getPlayerEntity(), "earth_shockwave_length_bonus", "value_int", earth_shockwave_length_bonus)
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "meteor_fragment,seismic_crystal",
	},
	{
		name 					= "meteor_fragment_iron",
		ui_name					= "Iron Meteor Fragment",
		categories				= "projectiles",
		effect_desc				= "Spells you cast in a downward angle have a 30% chance of creating \nan iron spikes shockwave, that deal slice damage to enemies, when they hit something. \nIt also synergizes with Earth Core Fragment relic effect.",
		effect_desc_short		= "Spells cast downwards create iron spikes shockwaves on the ground.",
        precursor_types			= "magic_artifact",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5% and shockwave reaches longer.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("meteor_fragment_iron") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 30 + (5 * getRelicLevel("meteor_fragment_iron"))
				if(chance >= 1 and chance <= odds) then
					local projectile_vel_x, projectile_vel_y = getProjectileVelocitiesandGravity(projectile_entity_id)
					if(projectile_vel_y > 0) then
						replaceProjectileExplosionSprite(projectile_entity_id, "")
						projectileRemoveBouncing(projectile_entity_id)
						if(not projectileExplodesonImpact(projectile_entity_id)) then
							projectileSetExplosionConfig(projectile_entity_id, 1)
						end
						attachLoadEntityOnExplosiontoProjectile(projectile_entity_id, getLoadOnExplosion("shockwave_iron").path, "spark")
					end
				end
			end
		end,
		ongroundpound_function	= function( entity_id)
			if(isRelicOwned("meteor_fragment_iron") >= 1) then
				local game_effect_comp = GetGameEffectLoadTo( entity_id, "PROTECTION_ALL", true )
				if game_effect_comp ~= nil then
					ComponentSetValue( game_effect_comp, "frames", "40" )
				end
				local x, y = EntityGetTransform(entity_id)
				EntityLoad("mods/ArcaneReliquary/files/entity_definitions/load_on_explosion/shockwave_iron.xml", x, y)
			end
		end,
		ondrop_function			= function()
			removeInternalVariable(getPlayerEntity(), "iron_shockwave_length_bonus")
		end,
		overtime_function		= function()
			local iron_shockwave_length_bonus = getRelicLevel("meteor_fragment_iron")

			if(not entityHasInternalVariableValue(getPlayerEntity(), "iron_shockwave_length_bonus")) then
				addNewInternalVariable(getPlayerEntity(), "iron_shockwave_length_bonus", "value_int", iron_shockwave_length_bonus)
			end
			setInternalVariableValue(getPlayerEntity(), "iron_shockwave_length_bonus", "value_int", iron_shockwave_length_bonus)
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "meteor_fragment,pickaxe",
	},
	{
		name 					= "meteor_fragment_crystal",
		ui_name					= "Crystal Meteor Fragment",
		categories				= "projectiles",
		effect_desc				= "Spells you cast in a downward angle have a 30% chance of creating \na crystal shockwave, that deal radiation damage to enemies, when they hit something. \nIt also synergizes with Earth Core Fragment relic effect.",
		effect_desc_short		= "Spells cast downwards create crystal shockwaves on the ground.",
        precursor_types			= "magic_artifact",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5% and shockwave reaches longer.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("meteor_fragment_crystal") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 30 + (5 * getRelicLevel("meteor_fragment_crystal"))
				if(chance >= 1 and chance <= odds) then
					local projectile_vel_x, projectile_vel_y = getProjectileVelocitiesandGravity(projectile_entity_id)
					if(projectile_vel_y > 0) then
						replaceProjectileExplosionSprite(projectile_entity_id, "")
						projectileRemoveBouncing(projectile_entity_id)
						if(not projectileExplodesonImpact(projectile_entity_id)) then
							projectileSetExplosionConfig(projectile_entity_id, 1)
						end
						attachLoadEntityOnExplosiontoProjectile(projectile_entity_id, getLoadOnExplosion("shockwave_crystal").path, "spark")
					end
				end
			end
		end,
		ongroundpound_function	= function( entity_id)
			if(isRelicOwned("meteor_fragment_crystal") >= 1) then
				local game_effect_comp = GetGameEffectLoadTo( entity_id, "PROTECTION_ALL", true )
				if game_effect_comp ~= nil then
					ComponentSetValue( game_effect_comp, "frames", "40" )
				end
				local x, y = EntityGetTransform(entity_id)
				EntityLoad("mods/ArcaneReliquary/files/entity_definitions/load_on_explosion/shockwave_crystal.xml", x, y)
			end
		end,
		ondrop_function			= function()
			removeInternalVariable(getPlayerEntity(), "crystal_shockwave_length_bonus")
		end,
		overtime_function		= function()
			local crystal_shockwave_length_bonus = getRelicLevel("meteor_fragment_crystal")

			if(not entityHasInternalVariableValue(getPlayerEntity(), "crystal_shockwave_length_bonus")) then
				addNewInternalVariable(getPlayerEntity(), "crystal_shockwave_length_bonus", "value_int", crystal_shockwave_length_bonus)
			end
			setInternalVariableValue(getPlayerEntity(), "crystal_shockwave_length_bonus", "value_int", crystal_shockwave_length_bonus)
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "meteor_fragment,proximity_crystal",
	},
	{
		name 					= "mortar",
		ui_name					= "Mortar",
		categories				= "projectiles",
		effect_desc				= "Spells you cast in a downward angle have a 30% chance of creating \na firebomb shockwave, when they hit something. \nIt also synergizes with Earth Core Fragment relic effect.",
		effect_desc_short		= "Spells cast downwards create firebomb shockwaves on the ground.",
        precursor_types			= "gadget",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5% and shockwave reaches longer.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("mortar") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 30 + (5 * getRelicLevel("mortar"))
				if(chance >= 1 and chance <= odds) then
					local projectile_vel_x, projectile_vel_y = getProjectileVelocitiesandGravity(projectile_entity_id)
					if(projectile_vel_y > 0) then
						replaceProjectileExplosionSprite(projectile_entity_id, "")
						projectileRemoveBouncing(projectile_entity_id)
						if(not projectileExplodesonImpact(projectile_entity_id)) then
							projectileSetExplosionConfig(projectile_entity_id, 1)
						end
						attachLoadEntityOnExplosiontoProjectile(projectile_entity_id, getLoadOnExplosion("shockwave_firebombs").path, "spark")
					end
				end
			end
		end,
		ongroundpound_function	= function( entity_id)
			if(isRelicOwned("mortar") >= 1) then
				local game_effect_comp = GetGameEffectLoadTo( entity_id, "PROTECTION_ALL", true )
				if game_effect_comp ~= nil then
					ComponentSetValue( game_effect_comp, "frames", "40" )
				end
				local x, y = EntityGetTransform(entity_id)
				EntityLoad("mods/ArcaneReliquary/files/entity_definitions/load_on_explosion/shockwave_firebombs.xml", x, y)
			end
		end,
		ondrop_function			= function()
			removeInternalVariable(getPlayerEntity(), "firebomb_shockwave_length_bonus")
		end,
		overtime_function		= function()
			local firebomb_shockwave_length_bonus = getRelicLevel("mortar")

			if(not entityHasInternalVariableValue(getPlayerEntity(), "firebomb_shockwave_length_bonus")) then
				addNewInternalVariable(getPlayerEntity(), "firebomb_shockwave_length_bonus", "value_int", firebomb_shockwave_length_bonus)
			end
			setInternalVariableValue(getPlayerEntity(), "firebomb_shockwave_length_bonus", "value_int", firebomb_shockwave_length_bonus)
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "meteor_fragment,solar_artifact",
	},
	{
		name 					= "mortar_laser",
		ui_name					= "Mortar with Energy Cell",
		categories				= "projectiles",
		effect_desc				= "Spells you cast in a downward angle have a 30% chance of creating \na laser shockwave, when they hit something. \nIt also synergizes with Earth Core Fragment relic effect.",
		effect_desc_short		= "Spells cast downwards create laser shockwaves on the ground.",
        precursor_types			= "gadget",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5% and shockwave reaches longer.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("mortar_laser") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 30 + (5 * getRelicLevel("mortar_laser"))
				if(chance >= 1 and chance <= odds) then
					local projectile_vel_x, projectile_vel_y = getProjectileVelocitiesandGravity(projectile_entity_id)
					if(projectile_vel_y > 0) then
						replaceProjectileExplosionSprite(projectile_entity_id, "")
						projectileRemoveBouncing(projectile_entity_id)
						if(not projectileExplodesonImpact(projectile_entity_id)) then
							projectileSetExplosionConfig(projectile_entity_id, 1)
						end
						attachLoadEntityOnExplosiontoProjectile(projectile_entity_id, getLoadOnExplosion("shockwave_lasers").path, "spark")
					end
				end
			end
		end,
		ongroundpound_function	= function( entity_id)
			if(isRelicOwned("mortar_laser") >= 1) then
				local game_effect_comp = GetGameEffectLoadTo( entity_id, "PROTECTION_ALL", true )
				if game_effect_comp ~= nil then
					ComponentSetValue( game_effect_comp, "frames", "40" )
				end
				local x, y = EntityGetTransform(entity_id)
				EntityLoad("mods/ArcaneReliquary/files/entity_definitions/load_on_explosion/shockwave_lasers.xml", x, y)
			end
		end,
		ondrop_function			= function()
			removeInternalVariable(getPlayerEntity(), "laser_shockwave_length_bonus")
		end,
		overtime_function		= function()
			local laser_shockwave_length_bonus = getRelicLevel("mortar_laser")

			if(not entityHasInternalVariableValue(getPlayerEntity(), "laser_shockwave_length_bonus")) then
				addNewInternalVariable(getPlayerEntity(), "laser_shockwave_length_bonus", "value_int", laser_shockwave_length_bonus)
			end
			setInternalVariableValue(getPlayerEntity(), "laser_shockwave_length_bonus", "value_int", laser_shockwave_length_bonus)
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "mortar,laser_pointer",
	},
	{
		name 					= "mortar_slime",
		ui_name					= "Mortar with Slime",
		categories				= "projectiles",
		effect_desc				= "Spells you cast in a downward angle have a 30% chance of creating \na slime bolt shockwave, when they hit something. \nIt also synergizes with Earth Core Fragment relic effect.",
		effect_desc_short		= "Spells cast downwards create slime bolt shockwaves on the ground.",
        precursor_types			= "gadget",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5% and shockwave reaches longer.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("mortar_slime") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 30 + (5 * getRelicLevel("mortar_slime"))
				if(chance >= 1 and chance <= odds) then
					local projectile_vel_x, projectile_vel_y = getProjectileVelocitiesandGravity(projectile_entity_id)
					if(projectile_vel_y > 0) then
						replaceProjectileExplosionSprite(projectile_entity_id, "")
						projectileRemoveBouncing(projectile_entity_id)
						if(not projectileExplodesonImpact(projectile_entity_id)) then
							projectileSetExplosionConfig(projectile_entity_id, 1)
						end
						attachLoadEntityOnExplosiontoProjectile(projectile_entity_id, getLoadOnExplosion("shockwave_slimebolts").path, "spark")
					end
				end
			end
		end,
		ongroundpound_function	= function( entity_id)
			if(isRelicOwned("mortar_slime") >= 1) then
				local game_effect_comp = GetGameEffectLoadTo( entity_id, "PROTECTION_ALL", true )
				if game_effect_comp ~= nil then
					ComponentSetValue( game_effect_comp, "frames", "40" )
				end
				local x, y = EntityGetTransform(entity_id)
				EntityLoad("mods/ArcaneReliquary/files/entity_definitions/load_on_explosion/shockwave_slimebolts.xml", x, y)
			end
		end,
		ondrop_function			= function()
			removeInternalVariable(getPlayerEntity(), "slimebolt_shockwave_length_bonus")
		end,
		overtime_function		= function()
			local slimebolt_shockwave_length_bonus = getRelicLevel("mortar_slime")

			if(not entityHasInternalVariableValue(getPlayerEntity(), "slimebolt_shockwave_length_bonus")) then
				addNewInternalVariable(getPlayerEntity(), "slimebolt_shockwave_length_bonus", "value_int", slimebolt_shockwave_length_bonus)
			end
			setInternalVariableValue(getPlayerEntity(), "slimebolt_shockwave_length_bonus", "value_int", slimebolt_shockwave_length_bonus)
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "mortar,ball_of_goo",
	},
	{
		name 					= "mortar_acid",
		ui_name					= "Mortar with Acid",
		categories				= "projectiles",
		effect_desc				= "Spells you cast in a downward angle have a 30% chance of creating \nan acid bolt shockwave, when they hit something. \nIt also synergizes with Earth Core Fragment relic effect.",
		effect_desc_short		= "Spells cast downwards create acid bolt shockwaves on the ground.",
        precursor_types			= "gadget",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5% and shockwave reaches longer.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("mortar_acid") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 30 + (5 * getRelicLevel("mortar_acid"))
				if(chance >= 1 and chance <= odds) then
					local projectile_vel_x, projectile_vel_y = getProjectileVelocitiesandGravity(projectile_entity_id)
					if(projectile_vel_y > 0) then
						replaceProjectileExplosionSprite(projectile_entity_id, "")
						projectileRemoveBouncing(projectile_entity_id)
						if(not projectileExplodesonImpact(projectile_entity_id)) then
							projectileSetExplosionConfig(projectile_entity_id, 1)
						end
						attachLoadEntityOnExplosiontoProjectile(projectile_entity_id, getLoadOnExplosion("shockwave_acidbolts").path, "spark")
					end
				end
			end
		end,
		ongroundpound_function	= function( entity_id)
			if(isRelicOwned("mortar_acid") >= 1) then
				local game_effect_comp = GetGameEffectLoadTo( entity_id, "PROTECTION_ALL", true )
				if game_effect_comp ~= nil then
					ComponentSetValue( game_effect_comp, "frames", "40" )
				end
				local x, y = EntityGetTransform(entity_id)
				EntityLoad("mods/ArcaneReliquary/files/entity_definitions/load_on_explosion/shockwave_acidbolts.xml", x, y)
			end
		end,
		ondrop_function			= function()
			removeInternalVariable(getPlayerEntity(), "acidbolt_shockwave_length_bonus")
		end,
		overtime_function		= function()
			local acidbolt_shockwave_length_bonus = getRelicLevel("mortar_acid")

			if(not entityHasInternalVariableValue(getPlayerEntity(), "acidbolt_shockwave_length_bonus")) then
				addNewInternalVariable(getPlayerEntity(), "acidbolt_shockwave_length_bonus", "value_int", acidbolt_shockwave_length_bonus)
			end
			setInternalVariableValue(getPlayerEntity(), "acidbolt_shockwave_length_bonus", "value_int", acidbolt_shockwave_length_bonus)
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "mortar,ball_of_goo_pungent",
	},
	{
		name 					= "mortar_lava",
		ui_name					= "Mortar with Lava",
		categories				= "projectiles",
		effect_desc				= "Spells you cast in a downward angle have a 30% chance of creating \nan lava bolt shockwave, when they hit something. \nIt also synergizes with Earth Core Fragment relic effect.",
		effect_desc_short		= "Spells cast downwards create lava bolt shockwaves on the ground.",
        precursor_types			= "gadget",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5% and shockwave reaches longer.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("mortar_lava") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 30 + (5 * getRelicLevel("mortar_lava"))
				if(chance >= 1 and chance <= odds) then
					local projectile_vel_x, projectile_vel_y = getProjectileVelocitiesandGravity(projectile_entity_id)
					if(projectile_vel_y > 0) then
						replaceProjectileExplosionSprite(projectile_entity_id, "")
						projectileRemoveBouncing(projectile_entity_id)
						if(not projectileExplodesonImpact(projectile_entity_id)) then
							projectileSetExplosionConfig(projectile_entity_id, 1)
						end
						attachLoadEntityOnExplosiontoProjectile(projectile_entity_id, getLoadOnExplosion("shockwave_lavabolts").path, "spark")
					end
				end
			end
		end,
		ongroundpound_function	= function( entity_id)
			if(isRelicOwned("mortar_lava") >= 1) then
				local game_effect_comp = GetGameEffectLoadTo( entity_id, "PROTECTION_ALL", true )
				if game_effect_comp ~= nil then
					ComponentSetValue( game_effect_comp, "frames", "40" )
				end
				local x, y = EntityGetTransform(entity_id)
				EntityLoad("mods/ArcaneReliquary/files/entity_definitions/load_on_explosion/shockwave_lavabolts.xml", x, y)
			end
		end,
		ondrop_function			= function()
			removeInternalVariable(getPlayerEntity(), "lavabolt_shockwave_length_bonus")
		end,
		overtime_function		= function()
			local lavabolt_shockwave_length_bonus = getRelicLevel("mortar_lava")

			if(not entityHasInternalVariableValue(getPlayerEntity(), "lavabolt_shockwave_length_bonus")) then
				addNewInternalVariable(getPlayerEntity(), "lavabolt_shockwave_length_bonus", "value_int", lavabolt_shockwave_length_bonus)
			end
			setInternalVariableValue(getPlayerEntity(), "lavabolt_shockwave_length_bonus", "value_int", lavabolt_shockwave_length_bonus)
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "mortar,ball_of_goo_molten",
	},
	{
		name 					= "mortar_blood",
		ui_name					= "Mortar with Corrosive Blood",
		categories				= "projectiles",
		effect_desc				= "Spells you cast in a downward angle have a 30% chance of creating \na corrosive blood bolt shockwave, when they hit something. \nIt also synergizes with Earth Core Fragment relic effect.",
		effect_desc_short		= "Spells cast downwards create corrosive blood bolt shockwaves on the ground.",
        precursor_types			= "gadget",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5% and shockwave reaches longer.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("mortar_blood") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 30 + (5 * getRelicLevel("mortar_blood"))
				if(chance >= 1 and chance <= odds) then
					local projectile_vel_x, projectile_vel_y = getProjectileVelocitiesandGravity(projectile_entity_id)
					if(projectile_vel_y > 0) then
						replaceProjectileExplosionSprite(projectile_entity_id, "")
						projectileRemoveBouncing(projectile_entity_id)
						if(not projectileExplodesonImpact(projectile_entity_id)) then
							projectileSetExplosionConfig(projectile_entity_id, 1)
						end
						attachLoadEntityOnExplosiontoProjectile(projectile_entity_id, getLoadOnExplosion("shockwave_corrosivebloodbolts").path, "spark")
					end
				end
			end
		end,
		ongroundpound_function	= function( entity_id)
			if(isRelicOwned("mortar_blood") >= 1) then
				local game_effect_comp = GetGameEffectLoadTo( entity_id, "PROTECTION_ALL", true )
				if game_effect_comp ~= nil then
					ComponentSetValue( game_effect_comp, "frames", "40" )
				end
				local x, y = EntityGetTransform(entity_id)
				EntityLoad("mods/ArcaneReliquary/files/entity_definitions/load_on_explosion/shockwave_corrosivebloodbolts.xml", x, y)
			end
		end,
		ondrop_function			= function()
			removeInternalVariable(getPlayerEntity(), "corrosivebloodbolt_shockwave_length_bonus")
		end,
		overtime_function		= function()
			local corrosivebloodbolt_shockwave_length_bonus = getRelicLevel("mortar_blood")

			if(not entityHasInternalVariableValue(getPlayerEntity(), "corrosivebloodbolt_shockwave_length_bonus")) then
				addNewInternalVariable(getPlayerEntity(), "corrosivebloodbolt_shockwave_length_bonus", "value_int", corrosivebloodbolt_shockwave_length_bonus)
			end
			setInternalVariableValue(getPlayerEntity(), "corrosivebloodbolt_shockwave_length_bonus", "value_int", corrosivebloodbolt_shockwave_length_bonus)
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "mortar,ball_of_goo_bloody",
	},
	{
		name 					= "mortar_urine",
		ui_name					= "Mortar with Urine",
		categories				= "projectiles",
		effect_desc				= "Spells you cast in a downward angle have a 30% chance of creating \na urine bolt shockwave, when they hit something. \nIt also synergizes with Earth Core Fragment relic effect.",
		effect_desc_short		= "Spells cast downwards create urine bolt shockwaves on the ground.",
        precursor_types			= "gadget",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5% and shockwave reaches longer.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("mortar_urine") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 30 + (5 * getRelicLevel("mortar_urine"))
				if(chance >= 1 and chance <= odds) then
					local projectile_vel_x, projectile_vel_y = getProjectileVelocitiesandGravity(projectile_entity_id)
					if(projectile_vel_y > 0) then
						replaceProjectileExplosionSprite(projectile_entity_id, "")
						projectileRemoveBouncing(projectile_entity_id)
						if(not projectileExplodesonImpact(projectile_entity_id)) then
							projectileSetExplosionConfig(projectile_entity_id, 1)
						end
						attachLoadEntityOnExplosiontoProjectile(projectile_entity_id, getLoadOnExplosion("shockwave_urinebolts").path, "spark")
					end
				end
			end
		end,
		ongroundpound_function	= function( entity_id)
			if(isRelicOwned("mortar_urine") >= 1) then
				local game_effect_comp = GetGameEffectLoadTo( entity_id, "PROTECTION_ALL", true )
				if game_effect_comp ~= nil then
					ComponentSetValue( game_effect_comp, "frames", "40" )
				end
				local x, y = EntityGetTransform(entity_id)
				EntityLoad("mods/ArcaneReliquary/files/entity_definitions/load_on_explosion/shockwave_urinebolts.xml", x, y)
			end
		end,
		ondrop_function			= function()
			removeInternalVariable(getPlayerEntity(), "urinebolt_shockwave_length_bonus")
		end,
		overtime_function		= function()
			local urinebolt_shockwave_length_bonus = getRelicLevel("mortar_urine")

			if(not entityHasInternalVariableValue(getPlayerEntity(), "urinebolt_shockwave_length_bonus")) then
				addNewInternalVariable(getPlayerEntity(), "urinebolt_shockwave_length_bonus", "value_int", urinebolt_shockwave_length_bonus)
			end
			setInternalVariableValue(getPlayerEntity(), "urinebolt_shockwave_length_bonus", "value_int", urinebolt_shockwave_length_bonus)
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "mortar,ball_of_goo_urinous",
	},
	{
		name 					= "mortar_poly",
		ui_name					= "Mortar with Polymorphine",
		categories				= "projectiles",
		effect_desc				= "Spells you cast in a downward angle have a 30% chance of creating \na polymorphine bolt shockwave, when they hit something. \nIt also synergizes with Earth Core Fragment relic effect.",
		effect_desc_short		= "Spells cast downwards create polymorphine bolt shockwaves on the ground.",
        precursor_types			= "gadget",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5% and shockwave reaches longer.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("mortar_poly") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 30 + (5 * getRelicLevel("mortar_poly"))
				if(chance >= 1 and chance <= odds) then
					local projectile_vel_x, projectile_vel_y = getProjectileVelocitiesandGravity(projectile_entity_id)
					if(projectile_vel_y > 0) then
						replaceProjectileExplosionSprite(projectile_entity_id, "")
						projectileRemoveBouncing(projectile_entity_id)
						if(not projectileExplodesonImpact(projectile_entity_id)) then
							projectileSetExplosionConfig(projectile_entity_id, 1)
						end
						attachLoadEntityOnExplosiontoProjectile(projectile_entity_id, getLoadOnExplosion("shockwave_polybolts").path, "spark")
					end
				end
			end
		end,
		ongroundpound_function	= function( entity_id)
			if(isRelicOwned("mortar_poly") >= 1) then
				local game_effect_comp = GetGameEffectLoadTo( entity_id, "PROTECTION_ALL", true )
				if game_effect_comp ~= nil then
					ComponentSetValue( game_effect_comp, "frames", "40" )
				end
				local x, y = EntityGetTransform(entity_id)
				EntityLoad("mods/ArcaneReliquary/files/entity_definitions/load_on_explosion/shockwave_polybolts.xml", x, y)
			end
		end,
		ondrop_function			= function()
			removeInternalVariable(getPlayerEntity(), "polybolt_shockwave_length_bonus")
		end,
		overtime_function		= function()
			local polybolt_shockwave_length_bonus = getRelicLevel("mortar_poly")

			if(not entityHasInternalVariableValue(getPlayerEntity(), "polybolt_shockwave_length_bonus")) then
				addNewInternalVariable(getPlayerEntity(), "polybolt_shockwave_length_bonus", "value_int", polybolt_shockwave_length_bonus)
			end
			setInternalVariableValue(getPlayerEntity(), "polybolt_shockwave_length_bonus", "value_int", polybolt_shockwave_length_bonus)
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "mortar,ball_of_goo_mutating",
	},
	{
		name 					= "mortar_poison",
		ui_name					= "Mortar with Poison",
		categories				= "projectiles",
		effect_desc				= "Spells you cast in a downward angle have a 30% chance of creating \na poison bolt shockwave, when they hit something. \nIt also synergizes with Earth Core Fragment relic effect.",
		effect_desc_short		= "Spells cast downwards create poison bolt shockwaves on the ground.",
        precursor_types			= "gadget",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5% and shockwave reaches longer.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("mortar_poison") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 30 + (5 * getRelicLevel("mortar_poison"))
				if(chance >= 1 and chance <= odds) then
					local projectile_vel_x, projectile_vel_y = getProjectileVelocitiesandGravity(projectile_entity_id)
					if(projectile_vel_y > 0) then
						replaceProjectileExplosionSprite(projectile_entity_id, "")
						projectileRemoveBouncing(projectile_entity_id)
						if(not projectileExplodesonImpact(projectile_entity_id)) then
							projectileSetExplosionConfig(projectile_entity_id, 1)
						end
						attachLoadEntityOnExplosiontoProjectile(projectile_entity_id, getLoadOnExplosion("shockwave_poisonbolts").path, "spark")
					end
				end
			end
		end,
		ongroundpound_function	= function( entity_id)
			if(isRelicOwned("mortar_poison") >= 1) then
				local game_effect_comp = GetGameEffectLoadTo( entity_id, "PROTECTION_ALL", true )
				if game_effect_comp ~= nil then
					ComponentSetValue( game_effect_comp, "frames", "40" )
				end
				local x, y = EntityGetTransform(entity_id)
				EntityLoad("mods/ArcaneReliquary/files/entity_definitions/load_on_explosion/shockwave_poisonbolts.xml", x, y)
			end
		end,
		ondrop_function			= function()
			removeInternalVariable(getPlayerEntity(), "poisonbolt_shockwave_length_bonus")
		end,
		overtime_function		= function()
			local poisonbolt_shockwave_length_bonus = getRelicLevel("mortar_poison")

			if(not entityHasInternalVariableValue(getPlayerEntity(), "poisonbolt_shockwave_length_bonus")) then
				addNewInternalVariable(getPlayerEntity(), "poisonbolt_shockwave_length_bonus", "value_int", poisonbolt_shockwave_length_bonus)
			end
			setInternalVariableValue(getPlayerEntity(), "poisonbolt_shockwave_length_bonus", "value_int", poisonbolt_shockwave_length_bonus)
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "mortar,ball_of_goo_venomous",
	},
	{
		name 					= "mortar_water",
		ui_name					= "Mortar with Water",
		categories				= "projectiles",
		effect_desc				= "Spells you cast in a downward angle have a 30% chance of creating \na water bolt shockwave, when they hit something. \nIt also synergizes with Earth Core Fragment relic effect.",
		effect_desc_short		= "Spells cast downwards create water bolt shockwaves on the ground.",
        precursor_types			= "gadget",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5% and shockwave reaches longer.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("mortar_water") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 30 + (5 * getRelicLevel("mortar_water"))
				if(chance >= 1 and chance <= odds) then
					local projectile_vel_x, projectile_vel_y = getProjectileVelocitiesandGravity(projectile_entity_id)
					if(projectile_vel_y > 0) then
						replaceProjectileExplosionSprite(projectile_entity_id, "")
						projectileRemoveBouncing(projectile_entity_id)
						if(not projectileExplodesonImpact(projectile_entity_id)) then
							projectileSetExplosionConfig(projectile_entity_id, 1)
						end
						attachLoadEntityOnExplosiontoProjectile(projectile_entity_id, getLoadOnExplosion("shockwave_waterbolts").path, "spark")
					end
				end
			end
		end,
		ongroundpound_function	= function( entity_id)
			if(isRelicOwned("mortar_water") >= 1) then
				local game_effect_comp = GetGameEffectLoadTo( entity_id, "PROTECTION_ALL", true )
				if game_effect_comp ~= nil then
					ComponentSetValue( game_effect_comp, "frames", "40" )
				end
				local x, y = EntityGetTransform(entity_id)
				EntityLoad("mods/ArcaneReliquary/files/entity_definitions/load_on_explosion/shockwave_waterbolts.xml", x, y)
			end
		end,
		ondrop_function			= function()
			removeInternalVariable(getPlayerEntity(), "waterbolt_shockwave_length_bonus")
		end,
		overtime_function		= function()
			local waterbolt_shockwave_length_bonus = getRelicLevel("mortar_water")

			if(not entityHasInternalVariableValue(getPlayerEntity(), "waterbolt_shockwave_length_bonus")) then
				addNewInternalVariable(getPlayerEntity(), "waterbolt_shockwave_length_bonus", "value_int", waterbolt_shockwave_length_bonus)
			end
			setInternalVariableValue(getPlayerEntity(), "waterbolt_shockwave_length_bonus", "value_int", waterbolt_shockwave_length_bonus)
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "mortar,ball_of_goo_humid",
	},
	{
		name 					= "mortar_ice",
		ui_name					= "Mortar with Freezing Liquid",
		categories				= "projectiles",
		effect_desc				= "Spells you cast in a downward angle have a 30% chance of creating \na freezing liquid bolt shockwave, when they hit something. \nIt also synergizes with Earth Core Fragment relic effect.",
		effect_desc_short		= "Spells cast downwards create freezing liquic bolt shockwaves on the ground.",
        precursor_types			= "gadget",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5% and shockwave reaches longer.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("mortar_ice") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 30 + (5 * getRelicLevel("mortar_ice"))
				if(chance >= 1 and chance <= odds) then
					local projectile_vel_x, projectile_vel_y = getProjectileVelocitiesandGravity(projectile_entity_id)
					if(projectile_vel_y > 0) then
						replaceProjectileExplosionSprite(projectile_entity_id, "")
						projectileRemoveBouncing(projectile_entity_id)
						if(not projectileExplodesonImpact(projectile_entity_id)) then
							projectileSetExplosionConfig(projectile_entity_id, 1)
						end
						attachLoadEntityOnExplosiontoProjectile(projectile_entity_id, getLoadOnExplosion("shockwave_frozenbolts").path, "spark")
					end
				end
			end
		end,
		ongroundpound_function	= function( entity_id)
			if(isRelicOwned("mortar_ice") >= 1) then
				local game_effect_comp = GetGameEffectLoadTo( entity_id, "PROTECTION_ALL", true )
				if game_effect_comp ~= nil then
					ComponentSetValue( game_effect_comp, "frames", "40" )
				end
				local x, y = EntityGetTransform(entity_id)
				EntityLoad("mods/ArcaneReliquary/files/entity_definitions/load_on_explosion/shockwave_frozenbolts.xml", x, y)
			end
		end,
		ondrop_function			= function()
			removeInternalVariable(getPlayerEntity(), "frozenbolt_shockwave_length_bonus")
		end,
		overtime_function		= function()
			local frozenbolt_shockwave_length_bonus = getRelicLevel("mortar_ice")

			if(not entityHasInternalVariableValue(getPlayerEntity(), "frozenbolt_shockwave_length_bonus")) then
				addNewInternalVariable(getPlayerEntity(), "frozenbolt_shockwave_length_bonus", "value_int", frozenbolt_shockwave_length_bonus)
			end
			setInternalVariableValue(getPlayerEntity(), "frozenbolt_shockwave_length_bonus", "value_int", frozenbolt_shockwave_length_bonus)
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "mortar,ball_of_goo_frozen",
	},
	{
		name 					= "mortar_sulfur",
		ui_name					= "Mortar with Sulfur",
		categories				= "projectiles",
		effect_desc				= "Spells you cast in a downward angle have a 30% chance of creating \na fireball shockwave, when they hit something. \nIt also synergizes with Earth Core Fragment relic effect.",
		effect_desc_short		= "Spells cast downwards create fireball shockwaves on the ground.",
        precursor_types			= "gadget",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5% and shockwave reaches longer.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("mortar_sulfur") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 30 + (5 * getRelicLevel("mortar_sulfur"))
				if(chance >= 1 and chance <= odds) then
					local projectile_vel_x, projectile_vel_y = getProjectileVelocitiesandGravity(projectile_entity_id)
					if(projectile_vel_y > 0) then
						replaceProjectileExplosionSprite(projectile_entity_id, "")
						projectileRemoveBouncing(projectile_entity_id)
						if(not projectileExplodesonImpact(projectile_entity_id)) then
							projectileSetExplosionConfig(projectile_entity_id, 1)
						end
						attachLoadEntityOnExplosiontoProjectile(projectile_entity_id, getLoadOnExplosion("shockwave_fireballs").path, "spark")
					end
				end
			end
		end,
		ongroundpound_function	= function( entity_id)
			if(isRelicOwned("mortar_sulfur") >= 1) then
				local game_effect_comp = GetGameEffectLoadTo( entity_id, "PROTECTION_ALL", true )
				if game_effect_comp ~= nil then
					ComponentSetValue( game_effect_comp, "frames", "40" )
				end
				local x, y = EntityGetTransform(entity_id)
				EntityLoad("mods/ArcaneReliquary/files/entity_definitions/load_on_explosion/shockwave_fireballs.xml", x, y)
			end
		end,
		ondrop_function			= function()
			removeInternalVariable(getPlayerEntity(), "fireball_shockwave_length_bonus")
		end,
		overtime_function		= function()
			local fireball_shockwave_length_bonus = getRelicLevel("mortar_sulfur")

			if(not entityHasInternalVariableValue(getPlayerEntity(), "fireball_shockwave_length_bonus")) then
				addNewInternalVariable(getPlayerEntity(), "fireball_shockwave_length_bonus", "value_int", fireball_shockwave_length_bonus)
			end
			setInternalVariableValue(getPlayerEntity(), "fireball_shockwave_length_bonus", "value_int", fireball_shockwave_length_bonus)
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "mortar,sulfur_crystal",
	},
	{
		name 					= "mortar_uranium",
		ui_name					= "Mortar with Uranium",
		categories				= "projectiles",
		effect_desc				= "Spells you cast in a downward angle have a 30% chance of creating \na radioactive fireball shockwave, when they hit something. \nIt also synergizes with Earth Core Fragment relic effect.",
		effect_desc_short		= "Spells cast downwards create radioactive fireball shockwaves on the ground.",
        precursor_types			= "gadget",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5% and shockwave reaches longer.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("mortar_uranium") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 30 + (5 * getRelicLevel("mortar_uranium"))
				if(chance >= 1 and chance <= odds) then
					local projectile_vel_x, projectile_vel_y = getProjectileVelocitiesandGravity(projectile_entity_id)
					if(projectile_vel_y > 0) then
						replaceProjectileExplosionSprite(projectile_entity_id, "")
						projectileRemoveBouncing(projectile_entity_id)
						if(not projectileExplodesonImpact(projectile_entity_id)) then
							projectileSetExplosionConfig(projectile_entity_id, 1)
						end
						attachLoadEntityOnExplosiontoProjectile(projectile_entity_id, getLoadOnExplosion("shockwave_fireballsradioactive").path, "spark")
					end
				end
			end
		end,
		ongroundpound_function	= function( entity_id)
			if(isRelicOwned("mortar_uranium") >= 1) then
				local game_effect_comp = GetGameEffectLoadTo( entity_id, "PROTECTION_ALL", true )
				if game_effect_comp ~= nil then
					ComponentSetValue( game_effect_comp, "frames", "40" )
				end
				local x, y = EntityGetTransform(entity_id)
				EntityLoad("mods/ArcaneReliquary/files/entity_definitions/load_on_explosion/shockwave_fireballsradioactive.xml", x, y)
			end
		end,
		ondrop_function			= function()
			removeInternalVariable(getPlayerEntity(), "fireballradioactive_shockwave_length_bonus")
		end,
		overtime_function		= function()
			local fireballradioactive_shockwave_length_bonus = getRelicLevel("mortar_uranium")

			if(not entityHasInternalVariableValue(getPlayerEntity(), "fireballradioactive_shockwave_length_bonus")) then
				addNewInternalVariable(getPlayerEntity(), "fireballradioactive_shockwave_length_bonus", "value_int", fireballradioactive_shockwave_length_bonus)
			end
			setInternalVariableValue(getPlayerEntity(), "fireballradioactive_shockwave_length_bonus", "value_int", fireballradioactive_shockwave_length_bonus)
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "mortar,uranium_ingot",
	},
	{
		name 					= "ball_of_goo_humid",
		ui_name					= "Ball of Humid Goo",
		categories				= "projectiles",
		effect_desc				= "Your spells have a 25% chance of exploding into a barrage of water bolts when they hit an obstacle or explode.",
		effect_desc_short		= "Your spells release water bolts on impact.",
        precursor_types			= "magic_artifact",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("ball_of_goo_humid") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 25 + (5 * getRelicLevel("ball_of_goo_humid"))
				if(chance >= 1 and chance <= odds) then
					addBuffVisualIndicator_Projectile(projectile_entity_id, "water")
					attachLoadEntityOnExplosiontoProjectile(projectile_entity_id, getLoadOnExplosion("launch_circular_projectiles_waterbolts").path)
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "ball_of_goo,water_gem",
	},
	{
		name 					= "ball_of_goo_frozen",
		ui_name					= "Ball of Frozen Goo",
		categories				= "projectiles",
		effect_desc				= "Your spells have a 25% chance of exploding into a barrage of freezing liquid bolts when they hit an obstacle or explode.",
		effect_desc_short		= "Your spells release freezing liquid bolts on impact.",
        precursor_types			= "magic_artifact",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("ball_of_goo_frozen") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 25 + (5 * getRelicLevel("ball_of_goo_frozen"))
				if(chance >= 1 and chance <= odds) then
					addBuffVisualIndicator_Projectile(projectile_entity_id, "diamond")
					attachLoadEntityOnExplosiontoProjectile(projectile_entity_id, getLoadOnExplosion("launch_circular_projectiles_frozenbolts").path)
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "ball_of_goo,freezing_liquid_gem",
	},
	{
		name 					= "friendly_slime_water",
		ui_name					= "Friendly Water Slime",
		categories				= "companions",
		effect_desc				= "A friendly water slime that shoots water bolts joins you.",
		effect_desc_short		= "A water bolt shooting companion.",
        precursor_types			= "companion",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			attachCompaniontoEntity(getPlayerEntity(), "friendly_slime_water", "mortal")
		end,
		ondrop_function			= function()
			detachCompanionfromEntity(getPlayerEntity(), "friendly_slime_water")
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "friendly_slime,water_gem",
		requires_relic			= "ball_of_goo",
	},
	{
		name 					= "friendly_slime_frozen",
		ui_name					= "Friendly Frozen Slime",
		categories				= "companions",
		effect_desc				= "A friendly frozen slime that shoots freezing liquid bolts joins you.",
		effect_desc_short		= "A freezing liquid bolt shooting companion.",
        precursor_types			= "companion",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			attachCompaniontoEntity(getPlayerEntity(), "friendly_slime_frozen", "mortal")
		end,
		ondrop_function			= function()
			detachCompanionfromEntity(getPlayerEntity(), "friendly_slime_frozen")
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "friendly_slime,freezing_liquid_gem",
		requires_relic			= "ball_of_goo",
	},
	{
		name 					= "slapping_mana_crystal",
		ui_name					= "Slapping Mana Crystal",
		categories				= "projectiles",
		effect_desc				= "Spells you cast have a 50% chance of spawning \nrotating orbs that inflict the slap effect on enemies they touch.",
		effect_desc_short		= "Your spells spawn slapping rotating orbs.",
        precursor_types			= "effect_crystal",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("slapping_mana_crystal") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 50 + (5 * getRelicLevel("slapping_mana_crystal"))
				if(chance >= 1 and chance <= odds) then		
					addNewInternalVariable(projectile_entity_id, "rotating_projectile_definition", "value_string", "mods/ArcaneReliquary/files/entity_definitions/custom_projectiles/rotating_shield_slapping_part.xml")
					addNewInternalVariable(projectile_entity_id, "rotating_projectiles_amount", "value_int", Random(2,4))
					addNewInternalVariable(projectile_entity_id, "radius", "value_int", 15)
					addNewInternalVariable(projectile_entity_id, "rotation_speed", "value_int", Random(10,20))
					addNewInternalVariable(projectile_entity_id, "shooter_id", "value_int", getPlayerEntity())
					addNewLuaCallback(projectile_entity_id, "overtime", "mods/ArcaneReliquary/files/scripts/projectiles/projectile_rotating_shield_start.lua", 1, true, "projectile_rotating_shield_start")
				end
			end
		end,
	},
	{
		name 					= "bloody_mana_crystal",
		ui_name					= "Bloody Mana Crystal",
		categories				= "projectiles",
		effect_desc				= "Spells you cast have a 50% chance of spawning \nrotating orbs that inflict the bleeding effect on enemies they touch.",
		effect_desc_short		= "Your spells spawn bleeding rotating orbs.",
        precursor_types			= "effect_crystal",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("bloody_mana_crystal") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 50 + (5 * getRelicLevel("bloody_mana_crystal"))
				if(chance >= 1 and chance <= odds) then		
					addNewInternalVariable(projectile_entity_id, "rotating_projectile_definition", "value_string", "mods/ArcaneReliquary/files/entity_definitions/custom_projectiles/rotating_shield_bleeding_part.xml")
					addNewInternalVariable(projectile_entity_id, "rotating_projectiles_amount", "value_int", Random(2,4))
					addNewInternalVariable(projectile_entity_id, "radius", "value_int", 15)
					addNewInternalVariable(projectile_entity_id, "rotation_speed", "value_int", Random(10,20))
					addNewInternalVariable(projectile_entity_id, "shooter_id", "value_int", getPlayerEntity())
					addNewLuaCallback(projectile_entity_id, "overtime", "mods/ArcaneReliquary/files/scripts/projectiles/projectile_rotating_shield_start.lua", 1, true, "projectile_rotating_shield_start")
				end
			end
		end,
	},
	{
		name 					= "windy_mana_crystal",
		ui_name					= "Windy Mana Crystal",
		categories				= "projectiles",
		effect_desc				= "Spells you cast have a 50% chance of spawning \nrotating orbs that inflict the wind effect on enemies they touch.",
		effect_desc_short		= "Your spells spawn wind rotating orbs.",
        precursor_types			= "effect_crystal",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("windy_mana_crystal") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 50 + (5 * getRelicLevel("windy_mana_crystal"))
				if(chance >= 1 and chance <= odds) then		
					addNewInternalVariable(projectile_entity_id, "rotating_projectile_definition", "value_string", "mods/ArcaneReliquary/files/entity_definitions/custom_projectiles/rotating_shield_wind_part.xml")
					addNewInternalVariable(projectile_entity_id, "rotating_projectiles_amount", "value_int", Random(2,4))
					addNewInternalVariable(projectile_entity_id, "radius", "value_int", 15)
					addNewInternalVariable(projectile_entity_id, "rotation_speed", "value_int", Random(10,20))
					addNewInternalVariable(projectile_entity_id, "shooter_id", "value_int", getPlayerEntity())
					addNewLuaCallback(projectile_entity_id, "overtime", "mods/ArcaneReliquary/files/scripts/projectiles/projectile_rotating_shield_start.lua", 1, true, "projectile_rotating_shield_start")
				end
			end
		end,
	},
	{
		name 					= "slash_mana_crystal",
		ui_name					= "Slash Mana Crystal",
		categories				= "projectiles",
		effect_desc				= "Spells you cast have a 50% chance of spawning \nrotating orbs that inflict the slash effect on enemies they touch.",
		effect_desc_short		= "Your spells spawn slash rotating orbs.",
        precursor_types			= "effect_crystal",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("slash_mana_crystal") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 50 + (5 * getRelicLevel("slash_mana_crystal"))
				if(chance >= 1 and chance <= odds) then		
					addNewInternalVariable(projectile_entity_id, "rotating_projectile_definition", "value_string", "mods/ArcaneReliquary/files/entity_definitions/custom_projectiles/rotating_shield_slash_part.xml")
					addNewInternalVariable(projectile_entity_id, "rotating_projectiles_amount", "value_int", Random(2,4))
					addNewInternalVariable(projectile_entity_id, "radius", "value_int", 15)
					addNewInternalVariable(projectile_entity_id, "rotation_speed", "value_int", Random(10,20))
					addNewInternalVariable(projectile_entity_id, "shooter_id", "value_int", getPlayerEntity())
					addNewLuaCallback(projectile_entity_id, "overtime", "mods/ArcaneReliquary/files/scripts/projectiles/projectile_rotating_shield_start.lua", 1, true, "projectile_rotating_shield_start")
				end
			end
		end,
	},
	{
		name 					= "weakness_mana_crystal",
		ui_name					= "Weakness Mana Crystal",
		categories				= "projectiles",
		effect_desc				= "Spells you cast have a 50% chance of spawning \nrotating orbs that inflict the paralyse effect on enemies they touch.",
		effect_desc_short		= "Your spells spawn paralyse rotating orbs.",
        precursor_types			= "effect_crystal",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("weakness_mana_crystal") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 50 + (5 * getRelicLevel("weakness_mana_crystal"))
				if(chance >= 1 and chance <= odds) then		
					addNewInternalVariable(projectile_entity_id, "rotating_projectile_definition", "value_string", "mods/ArcaneReliquary/files/entity_definitions/custom_projectiles/rotating_shield_paralyse_part.xml")
					addNewInternalVariable(projectile_entity_id, "rotating_projectiles_amount", "value_int", Random(2,4))
					addNewInternalVariable(projectile_entity_id, "radius", "value_int", 15)
					addNewInternalVariable(projectile_entity_id, "rotation_speed", "value_int", Random(10,20))
					addNewInternalVariable(projectile_entity_id, "shooter_id", "value_int", getPlayerEntity())
					addNewLuaCallback(projectile_entity_id, "overtime", "mods/ArcaneReliquary/files/scripts/projectiles/projectile_rotating_shield_start.lua", 1, true, "projectile_rotating_shield_start")
				end
			end
		end,
	},
	{
		name 					= "magnetic_mana_crystal",
		ui_name					= "Magnetic Mana Crystal",
		categories				= "projectiles",
		effect_desc				= "Spells you cast have a 50% chance of spawning \nrotating orbs that inflict the attraction effect on enemies they touch.",
		effect_desc_short		= "Your spells spawn attraction rotating orbs.",
        precursor_types			= "effect_crystal",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("magnetic_mana_crystal") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 50 + (5 * getRelicLevel("magnetic_mana_crystal"))
				if(chance >= 1 and chance <= odds) then		
					addNewInternalVariable(projectile_entity_id, "rotating_projectile_definition", "value_string", "mods/ArcaneReliquary/files/entity_definitions/custom_projectiles/rotating_shield_magnetic_part.xml")
					addNewInternalVariable(projectile_entity_id, "rotating_projectiles_amount", "value_int", Random(2,4))
					addNewInternalVariable(projectile_entity_id, "radius", "value_int", 15)
					addNewInternalVariable(projectile_entity_id, "rotation_speed", "value_int", Random(10,20))
					addNewInternalVariable(projectile_entity_id, "shooter_id", "value_int", getPlayerEntity())
					addNewLuaCallback(projectile_entity_id, "overtime", "mods/ArcaneReliquary/files/scripts/projectiles/projectile_rotating_shield_start.lua", 1, true, "projectile_rotating_shield_start")
				end
			end
		end,
	},
	{
		name 					= "gravity_stone_electric",
		ui_name					= "Electric Gravity Stone",
		categories				= "player",
		effect_desc				= "Energy orbs rotate around you, electrocuting enemies they touch.",
		effect_desc_short		= "Energy orbs rotate around you, electrocuting enemies they touch.",
        precursor_types			= "gravity_stone",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		ondrop_function			= function()
			detachRotatingShieldfromEntity(getPlayerEntity(), "rotating_shield_electric_pulsate_part")
		end,
		overtime_function		= function()
			if(isRelicOwned("gravity_stone_electric") >= 1) then
				if(not entityHasRotatingShield(getPlayerEntity(), "rotating_shield_electric_pulsate_part")) then -- this makes sure the orbiting projectiles are loaded again if the player is polymorphed
					local start_id = EntityLoad("mods/ArcaneReliquary/files/entity_definitions/custom_projectiles/rotating_shield_start.xml")
					setInternalVariableValue(start_id, "host_entity_id", "value_int", getPlayerEntity())
					setInternalVariableValue(start_id, "rotating_projectile_definition", "value_string", "mods/ArcaneReliquary/files/entity_definitions/custom_projectiles/rotating_shield_electric_pulsate_part.xml")
					setInternalVariableValue(start_id, "rotating_projectiles_amount", "value_int", 3)
					setInternalVariableValue(start_id, "radius", "value_int", 40)
					setInternalVariableValue(start_id, "rotation_speed", "value_int", 4)
				end
			end
		end,
	},
	{
		name 					= "unstable_mana_crystal",
		ui_name					= "Unstable Mana Crystal",
		categories				= "projectiles",
		effect_desc				= "Spells you cast have a 50% chance of spawning \nrotating orbs that inflict the exploding effect on enemies they touch.",
		effect_desc_short		= "Your spells spawn exploding rotating orbs.",
        precursor_types			= "effect_crystal",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("unstable_mana_crystal") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 50 + (5 * getRelicLevel("unstable_mana_crystal"))
				if(chance >= 1 and chance <= odds) then		
					addNewInternalVariable(projectile_entity_id, "rotating_projectile_definition", "value_string", "mods/ArcaneReliquary/files/entity_definitions/custom_projectiles/rotating_shield_exploding_part.xml")
					addNewInternalVariable(projectile_entity_id, "rotating_projectiles_amount", "value_int", Random(2,4))
					addNewInternalVariable(projectile_entity_id, "radius", "value_int", 15)
					addNewInternalVariable(projectile_entity_id, "rotation_speed", "value_int", Random(10,20))
					addNewInternalVariable(projectile_entity_id, "shooter_id", "value_int", getPlayerEntity())
					addNewLuaCallback(projectile_entity_id, "overtime", "mods/ArcaneReliquary/files/scripts/projectiles/projectile_rotating_shield_start.lua", 1, true, "projectile_rotating_shield_start")
				end
			end
		end,
	},
	{
		name 					= "gravity_stone_burning",
		ui_name					= "Burning Gravity Stone",
		categories				= "player",
		effect_desc				= "Energy orbs rotate around you, setting on fire enemies they touch.",
		effect_desc_short		= "Energy orbs rotate around you, setting on fire enemies they touch.",
        precursor_types			= "gravity_stone",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		ondrop_function			= function()
			detachRotatingShieldfromEntity(getPlayerEntity(), "rotating_shield_burning_pulsate_part")
		end,
		overtime_function		= function()
			if(isRelicOwned("gravity_stone_burning") >= 1) then
				if(not entityHasRotatingShield(getPlayerEntity(), "rotating_shield_burning_pulsate_part")) then -- this makes sure the orbiting projectiles are loaded again if the player is polymorphed
					local start_id = EntityLoad("mods/ArcaneReliquary/files/entity_definitions/custom_projectiles/rotating_shield_start.xml")
					setInternalVariableValue(start_id, "host_entity_id", "value_int", getPlayerEntity())
					setInternalVariableValue(start_id, "rotating_projectile_definition", "value_string", "mods/ArcaneReliquary/files/entity_definitions/custom_projectiles/rotating_shield_burning_pulsate_part.xml")
					setInternalVariableValue(start_id, "rotating_projectiles_amount", "value_int", 4)
					setInternalVariableValue(start_id, "radius", "value_int", 40)
					setInternalVariableValue(start_id, "rotation_speed", "value_int", 4)
				end
			end
		end,
	},
	{
		name 					= "gravity_stone_frozen",
		ui_name					= "Frozen Gravity Stone",
		categories				= "player",
		effect_desc				= "Energy orbs rotate around you, freezing enemies they touch.",
		effect_desc_short		= "Energy orbs rotate around you, freezing enemies they touch.",
        precursor_types			= "gravity_stone",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		ondrop_function			= function()
			detachRotatingShieldfromEntity(getPlayerEntity(), "rotating_shield_freezing_pulsate_part")
		end,
		overtime_function		= function()
			if(isRelicOwned("gravity_stone_frozen") >= 1) then
				if(not entityHasRotatingShield(getPlayerEntity(), "rotating_shield_freezing_pulsate_part")) then -- this makes sure the orbiting projectiles are loaded again if the player is polymorphed
					local start_id = EntityLoad("mods/ArcaneReliquary/files/entity_definitions/custom_projectiles/rotating_shield_start.xml")
					setInternalVariableValue(start_id, "host_entity_id", "value_int", getPlayerEntity())
					setInternalVariableValue(start_id, "rotating_projectile_definition", "value_string", "mods/ArcaneReliquary/files/entity_definitions/custom_projectiles/rotating_shield_freezing_pulsate_part.xml")
					setInternalVariableValue(start_id, "rotating_projectiles_amount", "value_int", 3)
					setInternalVariableValue(start_id, "radius", "value_int", 40)
					setInternalVariableValue(start_id, "rotation_speed", "value_int", 4)
				end
			end
		end,
	},
	{
		name 					= "gravity_stone_mutating",
		ui_name					= "Mutating Gravity Stone",
		categories				= "player",
		effect_desc				= "Energy orbs rotate around you, polymorphing enemies they touch.",
		effect_desc_short		= "Energy orbs rotate around you, polymorphing enemies they touch.",
        precursor_types			= "gravity_stone",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		ondrop_function			= function()
			detachRotatingShieldfromEntity(getPlayerEntity(), "rotating_shield_mutating_pulsate_part")
		end,
		overtime_function		= function()
			if(isRelicOwned("gravity_stone_mutating") >= 1) then
				if(not entityHasRotatingShield(getPlayerEntity(), "rotating_shield_mutating_pulsate_part")) then -- this makes sure the orbiting projectiles are loaded again if the player is polymorphed
					local start_id = EntityLoad("mods/ArcaneReliquary/files/entity_definitions/custom_projectiles/rotating_shield_start.xml")
					setInternalVariableValue(start_id, "host_entity_id", "value_int", getPlayerEntity())
					setInternalVariableValue(start_id, "rotating_projectile_definition", "value_string", "mods/ArcaneReliquary/files/entity_definitions/custom_projectiles/rotating_shield_mutating_pulsate_part.xml")
					setInternalVariableValue(start_id, "rotating_projectiles_amount", "value_int", 2)
					setInternalVariableValue(start_id, "radius", "value_int", 45)
					setInternalVariableValue(start_id, "rotation_speed", "value_int", 5)
				end
			end
		end,
	},
	{
		name 					= "gravity_stone_deceasing",
		ui_name					= "Deceasing Gravity Stone",
		categories				= "player",
		effect_desc				= "An energy orb rotates around you, instantly killing enemies they touch.",
		effect_desc_short		= "An energy orb rotates around you, instantly killing enemies they touch.",
        precursor_types			= "gravity_stone",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		ondrop_function			= function()
			detachRotatingShieldfromEntity(getPlayerEntity(), "rotating_shield_instakill_pulsate_part")
		end,
		overtime_function		= function()
			if(isRelicOwned("gravity_stone_deceasing") >= 1) then
				if(not entityHasRotatingShield(getPlayerEntity(), "rotating_shield_instakill_pulsate_part")) then -- this makes sure the orbiting projectiles are loaded again if the player is polymorphed
					local start_id = EntityLoad("mods/ArcaneReliquary/files/entity_definitions/custom_projectiles/rotating_shield_start.xml")
					setInternalVariableValue(start_id, "host_entity_id", "value_int", getPlayerEntity())
					setInternalVariableValue(start_id, "rotating_projectile_definition", "value_string", "mods/ArcaneReliquary/files/entity_definitions/custom_projectiles/rotating_shield_instakill_pulsate_part.xml")
					setInternalVariableValue(start_id, "rotating_projectiles_amount", "value_int", 1)
					setInternalVariableValue(start_id, "radius", "value_int", 50)
					setInternalVariableValue(start_id, "rotation_speed", "value_int", 6)
				end
			end
		end,
	},
	{
		name 					= "gravity_stone_petrifying",
		ui_name					= "Petrifying Gravity Stone",
		categories				= "player",
		effect_desc				= "An energy orb rotates around you, petrifying enemies they touch.",
		effect_desc_short		= "An energy orb rotates around you, petrifying enemies they touch.",
        precursor_types			= "gravity_stone",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		ondrop_function			= function()
			detachRotatingShieldfromEntity(getPlayerEntity(), "rotating_shield_petrify_pulsate_part")
		end,
		overtime_function		= function()
			if(isRelicOwned("gravity_stone_petrifying") >= 1) then
				if(not entityHasRotatingShield(getPlayerEntity(), "rotating_shield_petrify_pulsate_part")) then -- this makes sure the orbiting projectiles are loaded again if the player is polymorphed
					local start_id = EntityLoad("mods/ArcaneReliquary/files/entity_definitions/custom_projectiles/rotating_shield_start.xml")
					setInternalVariableValue(start_id, "host_entity_id", "value_int", getPlayerEntity())
					setInternalVariableValue(start_id, "rotating_projectile_definition", "value_string", "mods/ArcaneReliquary/files/entity_definitions/custom_projectiles/rotating_shield_petrify_pulsate_part.xml")
					setInternalVariableValue(start_id, "rotating_projectiles_amount", "value_int", 1)
					setInternalVariableValue(start_id, "radius", "value_int", 50)
					setInternalVariableValue(start_id, "rotation_speed", "value_int", 6)
				end
			end
		end,
	},
	{
		name 					= "gravity_stone_slapping",
		ui_name					= "Slapping Gravity Stone",
		categories				= "player",
		effect_desc				= "Energy orbs rotate around you, inflicting the slap effect on enemies they touch.",
		effect_desc_short		= "Energy orbs rotate around you, inflicting the slap effect on enemies they touch.",
        precursor_types			= "gravity_stone",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		ondrop_function			= function()
			detachRotatingShieldfromEntity(getPlayerEntity(), "rotating_shield_slapping_pulsate_part")
		end,
		overtime_function		= function()
			if(isRelicOwned("gravity_stone_slapping") >= 1) then
				if(not entityHasRotatingShield(getPlayerEntity(), "rotating_shield_slapping_pulsate_part")) then -- this makes sure the orbiting projectiles are loaded again if the player is polymorphed
					local start_id = EntityLoad("mods/ArcaneReliquary/files/entity_definitions/custom_projectiles/rotating_shield_start.xml")
					setInternalVariableValue(start_id, "host_entity_id", "value_int", getPlayerEntity())
					setInternalVariableValue(start_id, "rotating_projectile_definition", "value_string", "mods/ArcaneReliquary/files/entity_definitions/custom_projectiles/rotating_shield_slapping_pulsate_part.xml")
					setInternalVariableValue(start_id, "rotating_projectiles_amount", "value_int", 3)
					setInternalVariableValue(start_id, "radius", "value_int", 40)
					setInternalVariableValue(start_id, "rotation_speed", "value_int", 4)
				end
			end
		end,
	},
	{
		name 					= "gravity_stone_bloody",
		ui_name					= "Bloody Gravity Stone",
		categories				= "player",
		effect_desc				= "Energy orbs rotate around you, inflicting the bleed effect on enemies they touch.",
		effect_desc_short		= "Energy orbs rotate around you, inflicting the bleed effect on enemies they touch.",
        precursor_types			= "gravity_stone",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		ondrop_function			= function()
			detachRotatingShieldfromEntity(getPlayerEntity(), "rotating_shield_bleeding_pulsate_part")
		end,
		overtime_function		= function()
			if(isRelicOwned("gravity_stone_bloody") >= 1) then
				if(not entityHasRotatingShield(getPlayerEntity(), "rotating_shield_bleeding_pulsate_part")) then -- this makes sure the orbiting projectiles are loaded again if the player is polymorphed
					local start_id = EntityLoad("mods/ArcaneReliquary/files/entity_definitions/custom_projectiles/rotating_shield_start.xml")
					setInternalVariableValue(start_id, "host_entity_id", "value_int", getPlayerEntity())
					setInternalVariableValue(start_id, "rotating_projectile_definition", "value_string", "mods/ArcaneReliquary/files/entity_definitions/custom_projectiles/rotating_shield_bleeding_pulsate_part.xml")
					setInternalVariableValue(start_id, "rotating_projectiles_amount", "value_int", 3)
					setInternalVariableValue(start_id, "radius", "value_int", 40)
					setInternalVariableValue(start_id, "rotation_speed", "value_int", 4)
				end
			end
		end,
	},
	{
		name 					= "gravity_stone_windy",
		ui_name					= "Windy Gravity Stone",
		categories				= "player",
		effect_desc				= "Energy orbs rotate around you, inflicting the wind effect on enemies they touch.",
		effect_desc_short		= "Energy orbs rotate around you, inflicting the wind effect on enemies they touch.",
        precursor_types			= "gravity_stone",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		ondrop_function			= function()
			detachRotatingShieldfromEntity(getPlayerEntity(), "rotating_shield_wind_pulsate_part")
		end,
		overtime_function		= function()
			if(isRelicOwned("gravity_stone_windy") >= 1) then
				if(not entityHasRotatingShield(getPlayerEntity(), "rotating_shield_wind_pulsate_part")) then -- this makes sure the orbiting projectiles are loaded again if the player is polymorphed
					local start_id = EntityLoad("mods/ArcaneReliquary/files/entity_definitions/custom_projectiles/rotating_shield_start.xml")
					setInternalVariableValue(start_id, "host_entity_id", "value_int", getPlayerEntity())
					setInternalVariableValue(start_id, "rotating_projectile_definition", "value_string", "mods/ArcaneReliquary/files/entity_definitions/custom_projectiles/rotating_shield_wind_pulsate_part.xml")
					setInternalVariableValue(start_id, "rotating_projectiles_amount", "value_int", 3)
					setInternalVariableValue(start_id, "radius", "value_int", 40)
					setInternalVariableValue(start_id, "rotation_speed", "value_int", 4)
				end
			end
		end,
	},
	{
		name 					= "gravity_stone_slash",
		ui_name					= "Slash Gravity Stone",
		categories				= "player",
		effect_desc				= "Energy orbs rotate around you, inflicting the slash effect on enemies they touch.",
		effect_desc_short		= "Energy orbs rotate around you, inflicting the slash effect on enemies they touch.",
        precursor_types			= "gravity_stone",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		ondrop_function			= function()
			detachRotatingShieldfromEntity(getPlayerEntity(), "rotating_shield_slash_pulsate_part")
		end,
		overtime_function		= function()
			if(isRelicOwned("gravity_stone_slash") >= 1) then
				if(not entityHasRotatingShield(getPlayerEntity(), "rotating_shield_slash_pulsate_part")) then -- this makes sure the orbiting projectiles are loaded again if the player is polymorphed
					local start_id = EntityLoad("mods/ArcaneReliquary/files/entity_definitions/custom_projectiles/rotating_shield_start.xml")
					setInternalVariableValue(start_id, "host_entity_id", "value_int", getPlayerEntity())
					setInternalVariableValue(start_id, "rotating_projectile_definition", "value_string", "mods/ArcaneReliquary/files/entity_definitions/custom_projectiles/rotating_shield_slash_pulsate_part.xml")
					setInternalVariableValue(start_id, "rotating_projectiles_amount", "value_int", 2)
					setInternalVariableValue(start_id, "radius", "value_int", 45)
					setInternalVariableValue(start_id, "rotation_speed", "value_int", 5)
				end
			end
		end,
	},
	{
		name 					= "gravity_stone_unstable",
		ui_name					= "Unstable Gravity Stone",
		categories				= "player",
		effect_desc				= "Energy orbs rotate around you, inflicting the exploding effect on enemies they touch.",
		effect_desc_short		= "Energy orbs rotate around you, inflicting the exploding effect on enemies they touch.",
        precursor_types			= "gravity_stone",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		ondrop_function			= function()
			detachRotatingShieldfromEntity(getPlayerEntity(), "rotating_shield_exploding_pulsate_part")
		end,
		overtime_function		= function()
			if(isRelicOwned("gravity_stone_unstable") >= 1) then
				if(not entityHasRotatingShield(getPlayerEntity(), "rotating_shield_exploding_pulsate_part")) then -- this makes sure the orbiting projectiles are loaded again if the player is polymorphed
					local start_id = EntityLoad("mods/ArcaneReliquary/files/entity_definitions/custom_projectiles/rotating_shield_start.xml")
					setInternalVariableValue(start_id, "host_entity_id", "value_int", getPlayerEntity())
					setInternalVariableValue(start_id, "rotating_projectile_definition", "value_string", "mods/ArcaneReliquary/files/entity_definitions/custom_projectiles/rotating_shield_exploding_pulsate_part.xml")
					setInternalVariableValue(start_id, "rotating_projectiles_amount", "value_int", 2)
					setInternalVariableValue(start_id, "radius", "value_int", 45)
					setInternalVariableValue(start_id, "rotation_speed", "value_int", 5)
				end
			end
		end,
	},
	{
		name 					= "gravity_stone_magnetic",
		ui_name					= "Magnetic Gravity Stone",
		categories				= "player",
		effect_desc				= "Energy orbs rotate around you, inflicting the attraction effect on enemies they touch.",
		effect_desc_short		= "Energy orbs rotate around you, inflicting the attraction effect on enemies they touch.",
        precursor_types			= "gravity_stone",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		ondrop_function			= function()
			detachRotatingShieldfromEntity(getPlayerEntity(), "rotating_shield_magnetic_pulsate_part")
		end,
		overtime_function		= function()
			if(isRelicOwned("gravity_stone_magnetic") >= 1) then
				if(not entityHasRotatingShield(getPlayerEntity(), "rotating_shield_magnetic_pulsate_part")) then -- this makes sure the orbiting projectiles are loaded again if the player is polymorphed
					local start_id = EntityLoad("mods/ArcaneReliquary/files/entity_definitions/custom_projectiles/rotating_shield_start.xml")
					setInternalVariableValue(start_id, "host_entity_id", "value_int", getPlayerEntity())
					setInternalVariableValue(start_id, "rotating_projectile_definition", "value_string", "mods/ArcaneReliquary/files/entity_definitions/custom_projectiles/rotating_shield_magnetic_pulsate_part.xml")
					setInternalVariableValue(start_id, "rotating_projectiles_amount", "value_int", 3)
					setInternalVariableValue(start_id, "radius", "value_int", 40)
					setInternalVariableValue(start_id, "rotation_speed", "value_int", 4)
				end
			end
		end,
	},
	{
		name 					= "gravity_stone_explosive",
		ui_name					= "Explosive Gravity Stone",
		categories				= "player",
		effect_desc				= "Energy orbs rotate around you, exploding when they touch an enemy.",
		effect_desc_short		= "Energy orbs rotate around you, exploding when they touch an enemy.",
        precursor_types			= "gravity_stone",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		ondrop_function			= function()
			detachRotatingShieldfromEntity(getPlayerEntity(), "rotating_shield_explode_pulsate_part")
		end,
		overtime_function		= function()
			if(isRelicOwned("gravity_stone_explosive") >= 1) then
				if(not entityHasRotatingShield(getPlayerEntity(), "rotating_shield_explode_pulsate_part")) then -- this makes sure the orbiting projectiles are loaded again if the player is polymorphed
					local start_id = EntityLoad("mods/ArcaneReliquary/files/entity_definitions/custom_projectiles/rotating_shield_start.xml")
					setInternalVariableValue(start_id, "host_entity_id", "value_int", getPlayerEntity())
					setInternalVariableValue(start_id, "rotating_projectile_definition", "value_string", "mods/ArcaneReliquary/files/entity_definitions/custom_projectiles/rotating_shield_explode_pulsate_part.xml")
					setInternalVariableValue(start_id, "rotating_projectiles_amount", "value_int", 12)
					setInternalVariableValue(start_id, "radius", "value_int", 55)
					setInternalVariableValue(start_id, "rotation_speed", "value_int", 6)
				end
			end
		end,
	},
	{
		name 					= "gravity_stone_deflecting",
		ui_name					= "Deflecting Gravity Stone",
		categories				= "player",
		effect_desc				= "Energy orbs rotate around you, deflecting enemy projectiles and heavily increasing their base damage and explosion power.",
		effect_desc_short		= "Energy orbs rotate around you, deflecting enemy projectiles and heavily increasing their base damage and explosion power.",
        precursor_types			= "gravity_stone",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		ondrop_function			= function()
			detachRotatingShieldfromEntity(getPlayerEntity(), "rotating_shield_deflect_pulsate_part")
		end,
		overtime_function		= function()
			if(isRelicOwned("gravity_stone_deflecting") >= 1) then
				if(not entityHasRotatingShield(getPlayerEntity(), "rotating_shield_deflect_pulsate_part")) then -- this makes sure the orbiting projectiles are loaded again if the player is polymorphed
					local start_id = EntityLoad("mods/ArcaneReliquary/files/entity_definitions/custom_projectiles/rotating_shield_start.xml")
					setInternalVariableValue(start_id, "host_entity_id", "value_int", getPlayerEntity())
					setInternalVariableValue(start_id, "rotating_projectile_definition", "value_string", "mods/ArcaneReliquary/files/entity_definitions/custom_projectiles/rotating_shield_deflect_pulsate_part.xml")
					setInternalVariableValue(start_id, "rotating_projectiles_amount", "value_int", 10)
					setInternalVariableValue(start_id, "radius", "value_int", 55)
					setInternalVariableValue(start_id, "rotation_speed", "value_int", 6)
				end
			end
		end,
	},
	{
		name 					= "gravity_stone_desintegrating",
		ui_name					= "Desintegrating Gravity Stone",
		categories				= "player",
		effect_desc				= "Energy orbs rotate around you, converting enemy projectiles into explosive red orbs.",
		effect_desc_short		= "Energy orbs rotate around you, converting enemy projectiles into explosive red orbs.",
        precursor_types			= "gravity_stone",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		ondrop_function			= function()
			detachRotatingShieldfromEntity(getPlayerEntity(), "rotating_shield_destroy_projectiles_pulsate_part")
		end,
		overtime_function		= function()
			if(isRelicOwned("gravity_stone_desintegrating") >= 1) then
				if(not entityHasRotatingShield(getPlayerEntity(), "rotating_shield_destroy_projectiles_pulsate_part")) then -- this makes sure the orbiting projectiles are loaded again if the player is polymorphed
					local start_id = EntityLoad("mods/ArcaneReliquary/files/entity_definitions/custom_projectiles/rotating_shield_start.xml")
					setInternalVariableValue(start_id, "host_entity_id", "value_int", getPlayerEntity())
					setInternalVariableValue(start_id, "rotating_projectile_definition", "value_string", "mods/ArcaneReliquary/files/entity_definitions/custom_projectiles/rotating_shield_destroy_projectiles_pulsate_part.xml")
					setInternalVariableValue(start_id, "rotating_projectiles_amount", "value_int", 10)
					setInternalVariableValue(start_id, "radius", "value_int", 55)
					setInternalVariableValue(start_id, "rotation_speed", "value_int", 6)
				end
			end
		end,
	},
	{
		name 					= "sawblade_spectral",
		ui_name					= "Spectral Sawblade",
		categories				= "player",
		effect_desc				= "Spectral sawblades rotate around you, converting enemy projectiles into small sawblades and shooting them back.",
		effect_desc_short		= "Spectral sawblades rotate around you, converting enemy projectiles into small sawblades.",
        precursor_types			= "gravity_stone,gadget",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		ondrop_function			= function()
			detachRotatingShieldfromEntity(getPlayerEntity(), "rotating_disc_bullet_medium_spectral")
		end,
		overtime_function		= function()
			if(isRelicOwned("sawblade_spectral") >= 1) then
				if(not entityHasRotatingShield(getPlayerEntity(), "rotating_disc_bullet_medium_spectral")) then -- this makes sure the orbiting projectiles are loaded again if the player is polymorphed
					local start_id = EntityLoad("mods/ArcaneReliquary/files/entity_definitions/custom_projectiles/rotating_shield_start.xml")
					setInternalVariableValue(start_id, "host_entity_id", "value_int", getPlayerEntity())
					setInternalVariableValue(start_id, "rotating_projectile_definition", "value_string", "mods/ArcaneReliquary/files/entity_definitions/custom_projectiles/rotating_disc_bullet_medium_spectral.xml")
					setInternalVariableValue(start_id, "rotating_projectiles_amount", "value_int", 10)
					setInternalVariableValue(start_id, "radius", "value_int", 55)
					setInternalVariableValue(start_id, "rotation_speed", "value_int", 3)
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "sawblade_small,molecular_agitator",
	},
	{
		name 					= "hamis_egg",
		ui_name					= "Hamis Egg",
		categories				= "minions",
		effect_desc				= "Enemies you kill have a 50% chance of spawning packs of friendly hamis.",
		effect_desc_short		= "Enemies you kill have a 50% chance of spawning packs of friendly hamis.",
        precursor_types			= "magic_artifact",
		upgrade_effect_desc		= "For each upgrade level, spawn chance increased by 5%.",
		is_upgradable 			= true,
		onenemykill_function	= function(damage_type_bit_field, damage_message, entity_thats_responsible, drop_items)
			if(isRelicOwned("hamis_egg") >= 1) then
				if(EntityHasTag(GetUpdatedEntityID(), "enemy")) then -- check to prevent killed minions from spawning more minions
					SetRandomSeed( GameGetFrameNum(), math.random(1,50))
					local chance = Random(1,100)
					local odds = 50 + (5 * getRelicLevel("hamis_egg"))
					if(chance >= 1 and chance <= odds) then	
						local death_x, death_y = EntityGetTransform(GetUpdatedEntityID())
						local amount = Random (1,3)
						for i=0,amount do
							EntityLoad("mods/ArcaneReliquary/files/entity_definitions/minions/spider_friendly.xml", death_x, death_y)
						end
					end
				end
			end
		end,
	},
	{
		name 					= "hamis_egg_explosive",
		ui_name					= "Hamis Egg (Explosive)",
		categories				= "minions",
		effect_desc				= "Enemies you kill have a 50% chance of spawning packs of friendly explosive hamis.",
		effect_desc_short		= "Enemies you kill have a 50% chance of spawning packs of friendly explosive hamis.",
        precursor_types			= "magic_artifact",
		upgrade_effect_desc		= "For each upgrade level, spawn chance increased by 5%.",
		is_upgradable 			= true,
		onenemykill_function	= function(damage_type_bit_field, damage_message, entity_thats_responsible, drop_items)
			if(isRelicOwned("hamis_egg_explosive") >= 1) then
				if(EntityHasTag(GetUpdatedEntityID(), "enemy")) then -- check to prevent killed minions from spawning more minions
					SetRandomSeed( GameGetFrameNum(), math.random(1,50))
					local chance = Random(1,100)
					local odds = 50 + (5 * getRelicLevel("hamis_egg_explosive"))
					if(chance >= 1 and chance <= odds) then	
						local death_x, death_y = EntityGetTransform(GetUpdatedEntityID())
						local amount = Random (1,3)
						for i=0,amount do
							EntityLoad("mods/ArcaneReliquary/files/entity_definitions/minions/spider_explosive.xml", death_x, death_y)
						end
					end
				end
			end
		end,
	},
	{
		name 					= "hamis_egg_acid",
		ui_name					= "Hamis Egg (Acid)",
		categories				= "minions",
		effect_desc				= "Enemies you kill have a 50% chance of spawning packs of friendly acid hamis.",
		effect_desc_short		= "Enemies you kill have a 50% chance of spawning packs of friendly acid hamis.",
        precursor_types			= "magic_artifact",
		upgrade_effect_desc		= "For each upgrade level, spawn chance increased by 5%.",
		is_upgradable 			= true,
		onenemykill_function	= function(damage_type_bit_field, damage_message, entity_thats_responsible, drop_items)
			if(isRelicOwned("hamis_egg_acid") >= 1) then
				if(EntityHasTag(GetUpdatedEntityID(), "enemy")) then -- check to prevent killed minions from spawning more minions
					SetRandomSeed( GameGetFrameNum(), math.random(1,50))
					local chance = Random(1,100)
					local odds = 50 + (5 * getRelicLevel("hamis_egg_acid"))
					if(chance >= 1 and chance <= odds) then	
						local death_x, death_y = EntityGetTransform(GetUpdatedEntityID())
						local amount = Random (1,3)
						for i=0,amount do
							EntityLoad("mods/ArcaneReliquary/files/entity_definitions/minions/spider_acid.xml", death_x, death_y)
						end
					end
				end
			end
		end,
	},
	{
		name 					= "hamis_egg_lava",
		ui_name					= "Hamis Egg (Lava)",
		categories				= "minions",
		effect_desc				= "Enemies you kill have a 50% chance of spawning packs of friendly lava hamis.",
		effect_desc_short		= "Enemies you kill have a 50% chance of spawning packs of friendly lava hamis.",
        precursor_types			= "magic_artifact",
		upgrade_effect_desc		= "For each upgrade level, spawn chance increased by 5%.",
		is_upgradable 			= true,
		onenemykill_function	= function(damage_type_bit_field, damage_message, entity_thats_responsible, drop_items)
			if(isRelicOwned("hamis_egg_lava") >= 1) then
				if(EntityHasTag(GetUpdatedEntityID(), "enemy")) then -- check to prevent killed minions from spawning more minions
					SetRandomSeed( GameGetFrameNum(), math.random(1,50))
					local chance = Random(1,100)
					local odds = 50 + (5 * getRelicLevel("hamis_egg_lava"))
					if(chance >= 1 and chance <= odds) then	
						local death_x, death_y = EntityGetTransform(GetUpdatedEntityID())
						local amount = Random (1,3)
						for i=0,amount do
							EntityLoad("mods/ArcaneReliquary/files/entity_definitions/minions/spider_lava.xml", death_x, death_y)
						end
					end
				end
			end
		end,
	},
	{
		name 					= "hamis_egg_poly",
		ui_name					= "Hamis Egg (Poly)",
		categories				= "minions",
		effect_desc				= "Enemies you kill have a 50% chance of spawning packs of friendly polymorphine hamis.",
		effect_desc_short		= "Enemies you kill have a 50% chance of spawning packs of friendly polymorphine hamis.",
        precursor_types			= "magic_artifact",
		upgrade_effect_desc		= "For each upgrade level, spawn chance increased by 5%.",
		is_upgradable 			= true,
		onenemykill_function	= function(damage_type_bit_field, damage_message, entity_thats_responsible, drop_items)
			if(isRelicOwned("hamis_egg_poly") >= 1) then
				if(EntityHasTag(GetUpdatedEntityID(), "enemy")) then -- check to prevent killed minions from spawning more minions
					SetRandomSeed( GameGetFrameNum(), math.random(1,50))
					local chance = Random(1,100)
					local odds = 50 + (5 * getRelicLevel("hamis_egg_poly"))
					if(chance >= 1 and chance <= odds) then	
						local death_x, death_y = EntityGetTransform(GetUpdatedEntityID())
						local amount = Random (1,3)
						for i=0,amount do
							EntityLoad("mods/ArcaneReliquary/files/entity_definitions/minions/spider_poly.xml", death_x, death_y)
						end
					end
				end
			end
		end,
	},
	{
		name 					= "firebug_egg",
		ui_name					= "Firefly Egg",
		categories				= "minions",
		effect_desc				= "Enemies you kill have a 50% chance of spawning packs of friendly fireflies.",
		effect_desc_short		= "Enemies you kill have a 50% chance of spawning packs of friendly fireflies.",
        precursor_types			= "magic_artifact",
		upgrade_effect_desc		= "For each upgrade level, spawn chance increased by 5%.",
		is_upgradable 			= true,
		onenemykill_function	= function(damage_type_bit_field, damage_message, entity_thats_responsible, drop_items)
			if(isRelicOwned("firebug_egg") >= 1) then
				if(EntityHasTag(GetUpdatedEntityID(), "enemy")) then -- check to prevent killed minions from spawning more minions
					SetRandomSeed( GameGetFrameNum(), math.random(1,50))
					local chance = Random(1,100)
					local odds = 50 + (5 * getRelicLevel("firebug_egg"))
					if(chance >= 1 and chance <= odds) then	
						local death_x, death_y = EntityGetTransform(GetUpdatedEntityID())
						local amount = Random (1,3)
						for i=0,amount do
							EntityLoad("mods/ArcaneReliquary/files/entity_definitions/minions/firebug_friendly.xml", death_x, death_y)
						end
					end
				end
			end
		end,
	},
	{
		name 					= "firebug_egg_frozen",
		ui_name					= "Frostfly Egg",
		categories				= "minions",
		effect_desc				= "Enemies you kill have a 50% chance of spawning packs of friendly frostflies.",
		effect_desc_short		= "Enemies you kill have a 50% chance of spawning packs of friendly frostflies.",
        precursor_types			= "magic_artifact",
		upgrade_effect_desc		= "For each upgrade level, spawn chance increased by 5%.",
		is_upgradable 			= true,
		onenemykill_function	= function(damage_type_bit_field, damage_message, entity_thats_responsible, drop_items)
			if(isRelicOwned("firebug_egg_frozen") >= 1) then
				if(EntityHasTag(GetUpdatedEntityID(), "enemy")) then -- check to prevent killed minions from spawning more minions
					SetRandomSeed( GameGetFrameNum(), math.random(1,50))
					local chance = Random(1,100)
					local odds = 50 + (5 * getRelicLevel("firebug_egg_frozen"))
					if(chance >= 1 and chance <= odds) then	
						local death_x, death_y = EntityGetTransform(GetUpdatedEntityID())
						local amount = Random (1,3)
						for i=0,amount do
							EntityLoad("mods/ArcaneReliquary/files/entity_definitions/minions/firebug_frozen.xml", death_x, death_y)
						end
					end
				end
			end
		end,
	},
	{
		name 					= "ball_of_goo_alcoholic",
		ui_name					= "Ball of Alcoholic Goo",
		categories				= "projectiles",
		effect_desc				= "Your spells have a 25% chance of exploding into a barrage of whiskey bolts when they hit an obstacle or explode.",
		effect_desc_short		= "Your spells release whiskey bolts on impact.",
        precursor_types			= "magic_artifact",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("ball_of_goo_alcoholic") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 25 + (5 * getRelicLevel("ball_of_goo_alcoholic"))
				if(chance >= 1 and chance <= odds) then
					addBuffVisualIndicator_Projectile(projectile_entity_id, "alcohol")
					attachLoadEntityOnExplosiontoProjectile(projectile_entity_id, getLoadOnExplosion("launch_circular_projectiles_alcoholbolts").path)
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "ball_of_goo,alcohol_gem",
	},
	{
		name 					= "friendly_slime_alcohol",
		ui_name					= "Friendly Whiskey Slime",
		categories				= "companions",
		effect_desc				= "A friendly frozen slime that shoots whiskey bolts joins you.",
		effect_desc_short		= "A whiskey bolt shooting companion.",
        precursor_types			= "companion",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			attachCompaniontoEntity(getPlayerEntity(), "friendly_slime_alcohol", "mortal")
		end,
		ondrop_function			= function()
			detachCompanionfromEntity(getPlayerEntity(), "friendly_slime_alcohol")
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "friendly_slime,alcohol_gem",
		requires_relic			= "ball_of_goo",
	},
	{
		name 					= "mortar_alcohol",
		ui_name					= "Mortar with Whiskey",
		categories				= "projectiles",
		effect_desc				= "Spells you cast in a downward angle have a 30% chance of creating \na whiskey bolt shockwave, when they hit something. \nIt also synergizes with Earth Core Fragment relic effect.",
		effect_desc_short		= "Spells cast downwards create whiskey bolt shockwaves on the ground.",
        precursor_types			= "gadget",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5% and shockwave reaches longer.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("mortar_alcohol") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 30 + (5 * getRelicLevel("mortar_alcohol"))
				if(chance >= 1 and chance <= odds) then
					local projectile_vel_x, projectile_vel_y = getProjectileVelocitiesandGravity(projectile_entity_id)
					if(projectile_vel_y > 0) then
						replaceProjectileExplosionSprite(projectile_entity_id, "")
						projectileRemoveBouncing(projectile_entity_id)
						if(not projectileExplodesonImpact(projectile_entity_id)) then
							projectileSetExplosionConfig(projectile_entity_id, 1)
						end
						attachLoadEntityOnExplosiontoProjectile(projectile_entity_id, getLoadOnExplosion("shockwave_alcoholbolts").path, "spark")
					end
				end
			end
		end,
		ongroundpound_function	= function( entity_id)
			if(isRelicOwned("mortar_alcohol") >= 1) then
				local game_effect_comp = GetGameEffectLoadTo( entity_id, "PROTECTION_ALL", true )
				if game_effect_comp ~= nil then
					ComponentSetValue( game_effect_comp, "frames", "40" )
				end
				local x, y = EntityGetTransform(entity_id)
				EntityLoad("mods/ArcaneReliquary/files/entity_definitions/load_on_explosion/shockwave_alcoholbolts.xml", x, y)
			end
		end,
		ondrop_function			= function()
			removeInternalVariable(getPlayerEntity(), "alcoholbolt_shockwave_length_bonus")
		end,
		overtime_function		= function()
			local alcoholbolt_shockwave_length_bonus = getRelicLevel("mortar_alcohol")

			if(not entityHasInternalVariableValue(getPlayerEntity(), "alcoholbolt_shockwave_length_bonus")) then
				addNewInternalVariable(getPlayerEntity(), "alcoholbolt_shockwave_length_bonus", "value_int", alcoholbolt_shockwave_length_bonus)
			end
			setInternalVariableValue(getPlayerEntity(), "alcoholbolt_shockwave_length_bonus", "value_int", alcoholbolt_shockwave_length_bonus)
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "mortar,ball_of_goo_alcoholic",
	},
	{
		name 					= "jar_of_goo_alcoholic",
		ui_name					= "Jar of Alcoholic Goo",
		categories				= "projectiles",
		effect_desc				= "Spells you cast have a 40% chance of inducing enemies to shoot whiskey bolts at random directions.",
		effect_desc_short		= "Spells you cast have a chance to induce enemies to randomly shoot whiskey bolts.",
        precursor_types			= "gadget",
		upgrade_effect_desc		= "For each upgrade level, effect chance is increased by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("jar_of_goo_alcoholic") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 40 + (5 * getRelicLevel("jar_of_goo_alcoholic"))
				if(chance >= 1 and chance <= odds) then
					attachLoadEffectOntoEntitiestoProjectile(projectile_entity_id, getCustomEffect("effect_shoot_projectile_random_alcoholbolt").path, "alcohol")
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "empty_bottle,ball_of_goo_alcoholic",
	},
	{
		name 					= "shuriken_alcoholic",
		ui_name					= "Shuriken Soaked with Whiskey",
		categories				= "projectiles,status_effects",
		effect_desc				= "Your spells have a 30% chance of inducing the \nwhiskey bolt slash effect on enemies hit by them.",
		effect_desc_short		= "Your spells induce the whiskey bolt slash effect on enemies.",
        precursor_types			= "magic_artifact,gadget",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("shuriken_alcoholic") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 30 + (5 * getRelicLevel("shuriken_alcoholic"))
				if(chance >= 1 and chance <= odds) then
					attachLoadEffectOntoEntitiestoProjectile(projectile_entity_id, getCustomEffect("effect_slash_projectile_alcoholbolt").path, "alcohol")
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "shuriken,ball_of_goo_alcoholic",
	},
	{
		name 					= "skull_scepter",
		ui_name					= "Skull Scepter",
		categories				= "player",
		effect_desc				= "Taking damage from enemy attacks kills you instantly, and ambrosia around you is turned into poison, \nbut you earn 2 gold periodically and when this happens there's a 5% chance a random spell card will be spawned.",
		effect_desc_short		= "Enemy attacks kill you instantly, ambrosia around you turns into poison, but you gain gold and spell cards periodically.",
        precursor_types			= "magic_artifact",
		upgrade_effect_desc		= "For each upgrade level, earned gold increases by 5 and spell card drop chance increases by 5%.",
		is_upgradable 			= true,
		overtimeslow_function	= function()
			if(isRelicOwned("skull_scepter") >= 1) then
				-- gold reward
				local gold_amount = 2 + (5 * getRelicLevel("skull_scepter"))
				addPlayerGold(gold_amount)
				GamePrint("Skull Scepter effect - " .. gold_amount .. " gold added")
				-- spell card drop chance
				local spell_card_spawn_chance = Random(1,100)
				local spell_card_spawn_odds = 5 + (5 * getRelicLevel("skull_scepter"))
				if(spell_card_spawn_chance >= 1 and spell_card_spawn_chance <= spell_card_spawn_odds) then
					GamePrint("Skull Scepter effect - random spell card dropped")
					local player_x, player_y = EntityGetTransform(getPlayerEntity())
					CreateItemActionEntity( getRandomActionId(), player_x, player_y)
				end
			end
		end,
		ondamaged_function		= function(damage, message, entity_thats_responsible, is_fatal, projectile_thats_responsible )
			if(isRelicOwned("skull_scepter") >= 1) then
				if(EntityHasTag(entity_thats_responsible, "enemy")) then
					-- death on enemy attack
					local player_x, player_y = EntityGetTransform(getPlayerEntity())
					
					killEntitywithCustomMessage(getPlayerEntity(), "Skull Scepter effect - instant death from enemy attack")
					
					EntityLoad("mods/ArcaneReliquary/files/entity_definitions/particle_explosions/instakill_explosion.xml", player_x, player_y)
				end
			end
		end,
		onpickup_function		= function()
			LoadGameEffectEntityTo( getPlayerEntity(), getCustomEffect("effect_field_ambrosia_to_poison").path )
		end,
		ondrop_function			= function()
			local player_children = EntityGetAllChildren(getPlayerEntity())
			if player_children == nil then return end
			for i,child in ipairs(player_children) do
				if(EntityGetName(child) == "field_ambrosia_to_poison") then
					EntityKill(child)
					break
				end
			end
		end,
	},
	{
		name 					= "ambrosia_gem",
		ui_name					= "Ambrosia Gem",
		categories				= "projectiles,materials",
		effect_desc				= "Spells you cast that explode on impact have a 10% chance of releasing ambrosia on impact.",
		effect_desc_short		= "Your spells release ambrosia.",
        precursor_types			= "elemental_gem",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("ambrosia_gem") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 20 + (5 * getRelicLevel("ambrosia_gem"))
				if(chance >= 1 and chance <= odds) then
					attachLoadEntityOnExplosiontoProjectile(projectile_entity_id, getLoadOnExplosion("launch_elemental_particles_ambrosia").path, "magic_liquid_protection_all")
				end
			end
		end,
	},
	{
		name 					= "skull_scepter_wet",
		ui_name					= "Wet Skull Scepter",
		categories				= "player",
		effect_desc				= "Getting wet kills you instantly, but when you pick up gold nuggets, you are healed by 10 \n and if you are at max health, there is a chance your maximum hp will be upgraded by this amount.",
		effect_desc_short		= "Getting wet kills you instantly, but getting gold nuggets heals you by 10.",
        precursor_types			= "magic_artifact",
		upgrade_effect_desc		= "For each upgrade level, earned hp increases by 5.",
		is_upgradable 			= true,
		overtimeslow_function	= function()
			if(isRelicOwned("skull_scepter_wet") >= 1) then
				
				-- death if wet
				if(GameGetGameEffectCount(getPlayerEntity(), "WET") > 0) then
					
					local player_x, player_y = EntityGetTransform(getPlayerEntity())
					
					killEntitywithCustomMessage(getPlayerEntity(), "Wet Skull Scepter effect - instant death for being wet")

					EntityLoad("mods/ArcaneReliquary/files/entity_definitions/particle_explosions/instakill_explosion.xml", player_x, player_y)
				end
			end
		end,
		ongoldpickup_function	= function()
			if(isRelicOwned("skull_scepter_wet") >= 1) then
				-- hp reward
				local hp_added = 10 + (5 * getRelicLevel("skull_scepter_wet"))
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local overheal = false
				-- 50% chance of setting overheal flag
				if(Random(1,2) == 1) then overheal = true end
				addPlayerHealth(hp_added, overheal)
				GamePrint("Wet Skull Scepter effect - " .. hp_added .. " hp added")
			end
		end
	},
	{
		name 					= "skull_scepter_fiery",
		ui_name					= "Fiery Skull Scepter",
		categories				= "player",
		effect_desc				= "Damage from fire kills you instantly, but your spells \ndeal 10x more damage and enemies killed by your spells drop healing nuggets.",
		effect_desc_short		= "Damage from fire kills you instantly, but your spells deal 10x more damage and enemies drop healing nuggets on death.",
        precursor_types			= "magic_artifact",
		upgrade_effect_desc		= "For each upgrade level, spell damage multiplier increases by 2.",
		is_upgradable 			= true,
		ondamaged_function	= function(damage, message, entity_thats_responsible, is_fatal, projectile_thats_responsible )
			if(isRelicOwned("skull_scepter_fiery") >= 1) then
				if(message == "$damage_fire") then
					-- death if damage by fire
					local player_x, player_y = EntityGetTransform(getPlayerEntity())
					
					killEntitywithCustomMessage(getPlayerEntity(), "Fiery Skull Scepter effect - instant death for taking damage from fire")

					EntityLoad("mods/ArcaneReliquary/files/entity_definitions/particle_explosions/instakill_explosion.xml", player_x, player_y)
				end
			end
		end,
		onenemykill_function	= function(damage_type_bit_field, damage_message, entity_thats_responsible, drop_items)
			if(isRelicOwned("skull_scepter_fiery") >= 1) then
				local death_x, death_y = EntityGetTransform(GetUpdatedEntityID())
				EntityLoad("mods/ArcaneReliquary/files/entity_definitions/items/healingnugget.xml", death_x, death_y)
			end
		end,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("skull_scepter_fiery") >= 1) then
				multiplyProjectileBaseDamage(projectile_entity_id, 10 + (2 * getRelicLevel("skull_scepter_fiery")))
			end
		end,
	},
	{
		name 					= "skull_scepter_radioactive",
		ui_name					= "Radioactive Skull Scepter",
		categories				= "player",
		effect_desc				= "Getting covered with toxic sludge kills you instantly, but every \ntime you kill an enemy with your spells, there's a 50% chance radioactive explosions might \nhappen around enemies in the vicinity.",
		effect_desc_short		= "Getting covered with toxic sludge kills you instantly, but your kills cause radioactive explosions on nearby enemies.",
        precursor_types			= "magic_artifact",
		upgrade_effect_desc		= "For each upgrade level, radioactive explosion chance on nearby enemies increases by 5%.",
		is_upgradable 			= true,
		overtimeslow_function	= function()
			if(isRelicOwned("skull_scepter_radioactive") >= 1) then
				
				-- death if covered with toxic sludge
				if(GameGetGameEffectCount(getPlayerEntity(), "RADIOACTIVE") > 0) then
					
					local player_x, player_y = EntityGetTransform(getPlayerEntity())
					
					killEntitywithCustomMessage(getPlayerEntity(), "Radioactive Skull Scepter effect - instant death for being covered in toxic sludge")

					EntityLoad("mods/ArcaneReliquary/files/entity_definitions/particle_explosions/instakill_explosion.xml", player_x, player_y)
				end
			end
		end,
		onenemykill_function	= function(damage_type_bit_field, damage_message, entity_thats_responsible, drop_items)
			if(isRelicOwned("skull_scepter_radioactive") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 50 + (5 * getRelicLevel("skull_scepter_radioactive"))
				if(chance >= 1 and chance <= odds) then
					local player_x, player_y = EntityGetTransform(getPlayerEntity())
					local enemies = EntityGetInRadiusWithTag( player_x, player_y, 300, "enemy" )
					for i, enemy_id in ipairs(enemies) do
						if(enemy_id ~= GetUpdatedEntityID()) then
							local enemy_x, enemy_y = EntityGetTransform(enemy_id)
							EntityLoad("mods/ArcaneReliquary/files/entity_definitions/others/radioactive_explosion.xml", enemy_x + Random(-10,10), enemy_y + Random(-10,10))
						end
					end
				end
			end
		end,
	},
	{
		name 					= "skull_scepter_bloody",
		ui_name					= "Bloody Skull Scepter",
		categories				= "player",
		effect_desc				= "Getting covered in red blood periodically reduces your hp, but every \nenemy you kill has a 40% chance of spawning a portal that spits gold and healing nuggets.",
		effect_desc_short		= "Being covered in red blood reduces your hp, but kills create portals that spit gold and healing nuggets",
        precursor_types			= "magic_artifact",
		upgrade_effect_desc		= "For each upgrade level, portal spawning chance increases by 5%.",
		is_upgradable 			= true,
		overtimeslow_function	= function()
			if(isRelicOwned("skull_scepter_bloody") >= 1) then
				-- hp reduced if covered in red blood
				if(GameGetGameEffectCount(getPlayerEntity(), "BLOODY") > 0) then
					GamePrint("Bloody Skull Scepter effect - hp reduced by 5 for being covered in red blood")
					addPlayerHealth(-5)
				end
			end
		end,
		onenemykill_function	= function(damage_type_bit_field, damage_message, entity_thats_responsible, drop_items)
			if(isRelicOwned("skull_scepter_bloody") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 40 + (5 * getRelicLevel("skull_scepter_bloody"))
				if(chance >= 1 and chance <= odds) then
					local death_x, death_y = EntityGetTransform(GetUpdatedEntityID())
					EntityLoad("mods/ArcaneReliquary/files/entity_definitions/others/portal_wealth_and_vigour.xml", death_x, death_y)
				end
			end
		end,
	},
	{
		name 					= "skull_scepter_unstable",
		ui_name					= "Unstable Skull Scepter",
		categories				= "player",
		effect_desc				= "Getting caught by explosions kills you instantly, but every enemy \nyou kill has a 20% chance of dropping a random perk card.",
		effect_desc_short		= "Getting caught by explosions kills you instantly, but kills have a chance of dropping perk cards.",
        precursor_types			= "magic_artifact",
		upgrade_effect_desc		= "For each upgrade level, perk card spawning chance increases by 5%.",
		is_upgradable 			= true,
		ondamaged_function	= function(damage, message, entity_thats_responsible, is_fatal, projectile_thats_responsible )
			if(isRelicOwned("skull_scepter_unstable") >= 1) then
				if(message == "$damage_explosion") then
					-- death if damage by explosions
					local player_x, player_y = EntityGetTransform(getPlayerEntity())
					
					killEntitywithCustomMessage(getPlayerEntity(), "Unstable Skull Scepter effect - instant death for taking damage from explosion")

					EntityLoad("mods/ArcaneReliquary/files/entity_definitions/particle_explosions/instakill_explosion.xml", player_x, player_y)
				end
			end
		end,
		onenemykill_function	= function(damage_type_bit_field, damage_message, entity_thats_responsible, drop_items)
			if(isRelicOwned("skull_scepter_unstable") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 20 + (5 * getRelicLevel("skull_scepter_unstable"))
				if(chance >= 1 and chance <= odds) then	
					local death_x, death_y = EntityGetTransform(GetUpdatedEntityID())
					spawnRandomPerkCard(death_x, death_y - 10)
				end
			end
		end,
	},
	{
		name 					= "skull_scepter_corroding",
		ui_name					= "Corroding Skull Scepter",
		categories				= "player",
		effect_desc				= "Taking direct damage from acid rewards you with 15 gold.",
		effect_desc_short		= "Taking direct damage from acid rewards you with 15 gold.",
        precursor_types			= "magic_artifact",
		upgrade_effect_desc		= "For each upgrade level, earned gold is increased by 10.",
		is_upgradable 			= true,
		ondamaged_function	= function(damage, message, entity_thats_responsible, is_fatal, projectile_thats_responsible )
			if(isRelicOwned("skull_scepter_corroding") >= 1) then
				if(string.find(message, "damage from material") and string.find(message, "acid")) then
					local gold_amount = 15 + (10 * getRelicLevel("skull_scepter_corroding"))
					addPlayerGold(gold_amount)
					GamePrint("Corroding Skull Scepter effect - " .. gold_amount.. " added for taking damage from acid")
				end
			end
		end,
	},
	{
		name 					= "skull_scepter_venomous",
		ui_name					= "Venomous Skull Scepter",
		categories				= "player",
		effect_desc				= "Taking direct damage from poison has a 25% chance of dropping random spell cards.",
		effect_desc_short		= "Taking direct damage from poison has a 25% chance of dropping random spell cards.",
        precursor_types			= "magic_artifact",
		upgrade_effect_desc		= "For each upgrade level, spell card drop chance increases by 5%.",
		is_upgradable 			= true,
		ondamaged_function	= function(damage, message, entity_thats_responsible, is_fatal, projectile_thats_responsible )
			if(isRelicOwned("skull_scepter_venomous") >= 1) then
				if(string.find(message, "damage from material") and string.find(message, "Poison")) then
					-- spell card drop chance
					local spell_card_spawn_chance = Random(1,100)
					local spell_card_spawn_odds = 25 + (5 * getRelicLevel("skull_scepter_venomous"))
					if(spell_card_spawn_chance >= 1 and spell_card_spawn_chance <= spell_card_spawn_odds) then
						GamePrint("Venomous Skull Scepter effect - random spell card dropped")
						local player_x, player_y = EntityGetTransform(getPlayerEntity())
						CreateItemActionEntity( getRandomActionId(), player_x, player_y)
					end
				end
			end
		end,
	},
	{
		name 					= "skull_scepter_uric",
		ui_name					= "Uric Skull Scepter",
		categories				= "player",
		effect_desc				= "Being covered in urine has a 5% chance of dropping random perk cards.",
		effect_desc_short		= "Being covered in urine has a 5% chance of dropping random perk cards.",
        precursor_types			= "magic_artifact",
		upgrade_effect_desc		= "For each upgrade level, perk card drop chance increases by 5%.",
		is_upgradable 			= true,
		overtimeslow_function	= function()
			if(isRelicOwned("skull_scepter_uric") >= 1) then
				if(GameGetGameEffectCount(getPlayerEntity(), "JARATE") > 0) then
					SetRandomSeed( GameGetFrameNum(), math.random(1,50))
					local chance = Random(1,100)
					local odds = 5 + (5 * getRelicLevel("skull_scepter_uric"))
					if(chance >= 1 and chance <= odds) then	
						local player_x, player_y = EntityGetTransform(getPlayerEntity())
						spawnRandomPerkCard(player_x, player_y - 10)
					end
				end
			end
		end,
	},
	{
		name 					= "damage_absorption_artifact_acid",
		ui_name					= "Acid Damage Absorption Artifact",
		categories				= "player",
		effect_desc				= "You absorb damage taken directly from acid, and it is converted into extra duplicates of the next spell you cast. \nWhen it happens you also recover health equivalent to 95% of all damage taken previously.",
		effect_desc_short		= "Damage from acid is converted into extra duplicates for your next spell cast.",
        precursor_types			= "magic_artifact",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		ondamaged_function	= function(damage, message, entity_thats_responsible, is_fatal, projectile_thats_responsible )
			if(isRelicOwned("damage_absorption_artifact_acid") >= 1) then
				if(string.find(message, "damage from material") and string.find(message, "acid")) then
					-- adds to absorbed damage counter
					local damage_absorbed_acid = getInternalVariableValue(getPlayerEntity(), "damage_absorbed_acid", "value_float")
					setInternalVariableValue(getPlayerEntity(), "damage_absorbed_acid", "value_float", damage_absorbed_acid + damage)
				end
			end
		end,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("damage_absorption_artifact_acid") >= 1) then
				local damage_absorbed_acid = getInternalVariableValue(getPlayerEntity(), "damage_absorbed_acid", "value_float")
				if(damage_absorbed_acid > 0) then
					local extra_projectiles = math.floor(damage_absorbed_acid * 25)
					for i=1,extra_projectiles do
						local x, y = EntityGetTransform(projectile_entity_id)
						shootProjectileBasedonAnotherProjectile(projectile_entity_id, getProjectileDefinitionPath(projectile_entity_id), x, y, 150)
						
						-- adds damage portion back as health
						addPlayerHealth((damage_absorbed_acid * 25) * 95 / 100, false)
						
						-- clears damage absorbed
						setInternalVariableValue(getPlayerEntity(), "damage_absorbed_acid", "value_float", 0)
					end
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "damage_absorption_artifact,acid_gem",
	},
	{
		name 					= "damage_absorption_artifact_poison",
		ui_name					= "Poison Damage Absorption Artifact",
		categories				= "player",
		effect_desc				= "You absorb damage taken directly from poison, and it is converted \ninto extra gold nuggets dropped by the next enemy you kill with your spells. \nWhen it happens you also recover health equivalent to 95% of all damage taken previously.",
		effect_desc_short		= "Damage from poison is converted into extra gold nuggets dropped by the next enemy you kill.",
        precursor_types			= "magic_artifact",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		ondamaged_function	= function(damage, message, entity_thats_responsible, is_fatal, projectile_thats_responsible )
			if(isRelicOwned("damage_absorption_artifact_poison") >= 1) then
				if(string.find(message, "damage from material") and string.find(message, "Poison")) then
					-- adds to absorbed damage counter
					local damage_absorbed_poison = getInternalVariableValue(getPlayerEntity(), "damage_absorbed_poison", "value_float")
					setInternalVariableValue(getPlayerEntity(), "damage_absorbed_poison", "value_float", damage_absorbed_poison + damage)
				end
			end
		end,
		onenemykill_function	= function(damage_type_bit_field, damage_message, entity_thats_responsible, drop_items)
			if(isRelicOwned("damage_absorption_artifact_poison") >= 1) then
				local damage_absorbed_poison = getInternalVariableValue(getPlayerEntity(), "damage_absorbed_poison", "value_float")
				if(damage_absorbed_poison > 0) then
					local gold_nuggets = math.floor(damage_absorbed_poison * 25)
					local death_x, death_y = EntityGetTransform(GetUpdatedEntityID())
					for i=1,gold_nuggets do
						EntityLoad("data/entities/items/pickup/goldnugget.xml", death_x, death_y)
					end
					-- adds damage portion back as health
					addPlayerHealth((damage_absorbed_poison * 25) * 95 / 100, false)
					
					-- clears damage absorbed
					setInternalVariableValue(getPlayerEntity(), "damage_absorbed_poison", "value_float", 0)
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "damage_absorption_artifact,poison_gem",
	},
	{
		name 					= "programmable_droid",
		ui_name					= "Programmable Droid",
		categories				= "companions",
		effect_desc				= "A small robot that shoot spells you give it. \nTo program it, drop a spell card on it (when it despawns or you give it another spell, the current \none will be dropped and can be collected again).",
		effect_desc_short		= "A robot that shoots spells you give it.",
        precursor_types			= "companion",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			attachCompaniontoEntity(getPlayerEntity(), "programmable_droid", "mortal")
		end,
		ondrop_function			= function()
			detachCompanionfromEntity(getPlayerEntity(), "programmable_droid")
		end,
		requires_relic			= "alkaline_battery",
	},
	{
		name 					= "programmable_droid_shotgunner",
		ui_name					= "Programmable Shotgunner Droid",
		categories				= "companions",
		effect_desc				= "A small robot that shoot spells you give it in a volley of projectiles. \nTo program it, drop a spell card on it (when it despawns or you give it another spell, the current \none will be dropped and can be collected again).",
		effect_desc_short		= "A robot that shoots spells you give it.",
        precursor_types			= "companion",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			attachCompaniontoEntity(getPlayerEntity(), "programmable_droid_shotgunner", "mortal")
		end,
		ondrop_function			= function()
			detachCompanionfromEntity(getPlayerEntity(), "programmable_droid_shotgunner")
		end,
		requires_relic			= "alkaline_battery",
	},
	{
		name 					= "programmable_droid_absorb",
		ui_name					= "Programmable Absorbing Droid",
		categories				= "companions",
		effect_desc				= "A small robot you control with your mouse \ncursor, that copies and shoots enemy spells it absorbs.",
		effect_desc_short		= "A robot that absorbs and shoots enemy spells.",
        precursor_types			= "companion",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			attachCompaniontoEntity(getPlayerEntity(), "programmable_droid_absorb", "mortal")
		end,
		ondrop_function			= function()
			detachCompanionfromEntity(getPlayerEntity(), "programmable_droid_absorb")
		end,
		requires_relic			= "alkaline_battery",
	},
	{
		name 					= "programmable_droid_absorb_shotgunner",
		ui_name					= "Programmable Absorbing Shotgunner Droid",
		categories				= "companions",
		effect_desc				= "A small robot you control with your mouse \ncursor, that copies and shoots enemy spells it absorbs in a volley of projectiles.",
		effect_desc_short		= "A robot that absorbs and shoots enemy spells.",
        precursor_types			= "companion",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			attachCompaniontoEntity(getPlayerEntity(), "programmable_droid_absorb_shotgunner", "mortal")
		end,
		ondrop_function			= function()
			detachCompanionfromEntity(getPlayerEntity(), "programmable_droid_absorb_shotgunner")
		end,
		requires_relic			= "alkaline_battery",
	},
	{
		name 					= "horseshoe",
		ui_name					= "Horseshoe",
		categories				= "projectiles",
		effect_desc				= "Extra projectiles are added when you cast a spell for \nas much alive enemies that happen to be close by.",
		effect_desc_short		= "Extra projectiles are added on your spell casts based on alive enemies around.",
        precursor_types			= "magic_artifact",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("horseshoe") >= 1) then
				
				local player_x, player_y = EntityGetTransform(getPlayerEntity())
				local enemies = EntityGetInRadiusWithTag( player_x, player_y, 200, "enemy" )
				
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				
				for i=1,#enemies do
					local x, y = EntityGetTransform(projectile_entity_id)
					shootProjectileBasedonAnotherProjectile(projectile_entity_id, getProjectileDefinitionPath(projectile_entity_id), x, y, Random(150,200))
				end
			end
		end,
	},
	{
		name 					= "horseshoe_lucky",
		ui_name					= "Lucky Horseshoe",
		categories				= "player",
		effect_desc				= "When you kill an enemy, there's a 15% chance all other enemy \ncreatures in the vicinity will be transformed into gold dust.",
		effect_desc_short		= "Killing an enemy turns other enemies nearby into gold dust.",
        precursor_types			= "magic_artifact",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onenemykill_function	= function(damage_type_bit_field, damage_message, entity_thats_responsible, drop_items)	
			if(isRelicOwned("horseshoe_lucky") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 15 + (5 * getRelicLevel("horseshoe_lucky"))
				if(chance >= 1 and chance <= odds) then
					local player_x, player_y = EntityGetTransform(GetUpdatedEntityID())
					local enemies = EntityGetInRadiusWithTag( player_x, player_y, 200, "enemy" )
					for i, enemy_id in ipairs(enemies) do
						if(enemy_id ~= GetUpdatedEntityID()) then
							LoadGameEffectEntityTo( enemy_id, getCustomEffect("effect_convert_to_gold").path )
						end
					end
				end
			end
		end,
	},
	{
		name 					= "horseshoe_frozen",
		ui_name					= "Frozen Horseshoe",
		categories				= "player",
		effect_desc				= "When you kill an enemy, there's a 25% chance an ice sphere will \nbe conjured on all enemies in the vicinity.",
		effect_desc_short		= "Killing an enemy conjurs ice spheres on nearby enemies.",
        precursor_types			= "magic_artifact",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onenemykill_function	= function(damage_type_bit_field, damage_message, entity_thats_responsible, drop_items)	
			if(isRelicOwned("horseshoe_frozen") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 15 + (5 * getRelicLevel("horseshoe_frozen"))
				if(chance >= 1 and chance <= odds) then
					local player_x, player_y = EntityGetTransform(GetUpdatedEntityID())
					local enemies = EntityGetInRadiusWithTag( player_x, player_y, 200, "enemy" )
					for i, enemy_id in ipairs(enemies) do
						if(enemy_id ~= GetUpdatedEntityID()) then
							local enemy_x, enemy_y = EntityGetTransform(enemy_id)
							EntityLoad(getLoadOnExplosion("ice_explosion").path, enemy_x, enemy_y)
						end
					end
				end
			end
		end,
	},
	{
		name 					= "horseshoe_crystal",
		ui_name					= "Crystallized Horseshoe",
		categories				= "player",
		effect_desc				= "When you kill an enemy, there's a 15% chance all other enemy \ncreatures in the vicinity will be converted into explosive crystals.",
		effect_desc_short		= "Killing an enemy turns other enemies nearby into explosive crystals.",
        precursor_types			= "magic_artifact",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onenemykill_function	= function(damage_type_bit_field, damage_message, entity_thats_responsible, drop_items)	
			if(isRelicOwned("horseshoe_crystal") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 15 + (5 * getRelicLevel("horseshoe_crystal"))
				if(chance >= 1 and chance <= odds) then
					local player_x, player_y = EntityGetTransform(GetUpdatedEntityID())
					local enemies = EntityGetInRadiusWithTag( player_x, player_y, 200, "enemy" )
					for i, enemy_id in ipairs(enemies) do
						if(enemy_id ~= GetUpdatedEntityID()) then
							LoadGameEffectEntityTo( enemy_id, getCustomEffect("effect_convert_to_mine").path )
						end
					end
				end
			end
		end,
	},
	{
		name 					= "horseshoe_cursed",
		ui_name					= "Cursed Horseshoe",
		categories				= "player",
		effect_desc				= "When you kill an enemy, there's a 5% chance all other enemy \ncreatures in the vicinity will be instantly killed with their hp transfered to \nyou, but each time you damage an enemy with your spells, you get damaged for half the damage you caused them.",
		effect_desc_short		= "Killing an enemy turns other enemies nearby into explosive crystals.",
        precursor_types			= "magic_artifact",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5% and damage penalty decreaes by 5%.",
		is_upgradable 			= true,
		onenemykill_function	= function(damage_type_bit_field, damage_message, entity_thats_responsible, drop_items)	
			if(isRelicOwned("horseshoe_cursed") >= 1) then
				-- effect on nearby enemies
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 5 + (5 * getRelicLevel("horseshoe_cursed"))
				if(chance >= 1 and chance <= odds) then
					local player_x, player_y = EntityGetTransform(GetUpdatedEntityID())
					local enemies = EntityGetInRadiusWithTag( player_x, player_y, 200, "enemy" )
					for i, enemy_id in ipairs(enemies) do
						if(enemy_id ~= GetUpdatedEntityID()) then
							local added_hp = getCreatureHealth(enemy_id)
							addPlayerHealth(added_hp * 25, false)
							LoadGameEffectEntityTo( enemy_id, getCustomEffect("effect_instakill").path )
						end
					end
				end
			end
		end,
		onenemydamaged_function	= function(damage, message, entity_thats_responsible, is_fatal, projectile_thats_responsible )
			if(isRelicOwned("horseshoe_cursed") >= 1) then
				-- penalty
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 50 - (5 * getRelicLevel("horseshoe_cursed"))
				if(chance >= 1 and chance <= odds) then
					local damage_penalty = (damage / 2)
					EntityInflictDamage( getPlayerEntity(), damage_penalty, "DAMAGE_PROJECTILE", "Cursed Horseshoe damage penalty", "BLOOD_SPRAY", 0, 0)
				end
			end
		end,
	},
	{
		name 					= "skull_scepter_cursed",
		ui_name					= "Cursed Skull Scepter",
		categories				= "player",
		effect_desc				= "When you take melee, projectile or explosion damage, if it is caused \nby an enemy, they are instantly killed, but if you take damage from any other source, you die instantly.",
		effect_desc_short		= "Enemies die instantly when they attack you, but you instantly die when damaged by any means.",
        precursor_types			= "magic_artifact",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		ondamaged_function		= function(damage, message, entity_thats_responsible, is_fatal, projectile_thats_responsible )
			if(isRelicOwned("skull_scepter_cursed") >= 1) then
				if((message == "$damage_projectile" or message == "$damage_explosion" or message == "$damage_melee") and EntityHasTag(entity_thats_responsible, "enemy")) then
					LoadGameEffectEntityTo( entity_thats_responsible, getCustomEffect("effect_instakill").path )
				else
					killEntitywithCustomMessage(GetUpdatedEntityID(), "Cursed Skull Scepter effect - instant death from damage sources other than enemies")
				end
			end
		end,
	},
	{
		name 					= "gem_scepter_blue",
		ui_name					= "Blue Gem Scepter",
		categories				= "player",
		effect_desc				= "You convert regular relic chests to great chests, and great chests to \nchaotic chests by kicking them. This happens only once per chest.",
		effect_desc_short		= "You upgrade chests by kicking them once.",
        precursor_types			= "magic_artifact,boot",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			addNewSimplifiedInternalVariable(getPlayerEntity(), "chest_upgrade_on_kick")
		end,
		ondrop_function			= function()
			removeSimplifiedInternalVariable(getPlayerEntity(), "chest_upgrade_on_kick")
		end
	},
	{
		name 					= "gem_scepter_red",
		ui_name					= "Red Gem Scepter",
		categories				= "player",
		effect_desc				= "Enemies you kick have a 30% chance of being instantly transformed into a relic. \nIt also synergizes with Sniper Scope relic effect.",
		effect_desc_short		= "Enemies you kick are transformed into relics.",
        precursor_types			= "magic_artifact,boot",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onenemykicked_function	= function(entity_who_kicked )
			if(isRelicOwned("gem_scepter_red") >= 1) then
				local chance = Random(1,100)
				local odds = 30 + (5 * getRelicLevel("gem_scepter_red"))
				if(chance >= 1 and chance <= odds) then
					LoadGameEffectEntityTo( GetUpdatedEntityID(), getCustomEffect("effect_convert_to_relic").path )
				end	
			end
		end,
		onextendedkick_function	= function( entity_id, extended_kick_id)
			if(isRelicOwned("gem_scepter_red") >= 1) then
				attachLoadEffectOntoEntitiestoProjectile(extended_kick_id, getCustomEffect("effect_convert_to_relic").path, "plasma_fading_pink")
			end
		end,
	},
	{
		name 					= "gem_scepter_pink",
		ui_name					= "Pink Gem Scepter",
		categories				= "projectiles",
		effect_desc				= "Cast a spell on a relic to transform it into another relic once.",
		effect_desc_short		= "Cast a spell on a relic to transform it into another relic once.",
        precursor_types			= "magic_artifact",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("gem_scepter_pink") >= 1) then
				attachCollisionTrigger(projectile_entity_id, 30, "relic", false, "relic_shuffle_trigger")
				addNewLuaCallback(projectile_entity_id, "on_collision", getCollisionHandler("relic_shuffle").path, -1, false, "relic_shuffle_handler")
			end
		end,
	},
	{
		name 					= "gem_scepter_green",
		ui_name					= "Green Gem Scepter",
		categories				= "projectiles",
		effect_desc				= "Your spells have a 5% chance of instantly converting enemies they hit into random relics.",
		effect_desc_short		= "Your spells convert enemies into relics.",
        precursor_types			= "magic_artifact",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("gem_scepter_green") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 5 + (5 * getRelicLevel("gem_scepter_green"))
				if(chance >= 1 and chance <= odds) then
					attachLoadEffectOntoEntitiestoProjectile(projectile_entity_id, getCustomEffect("effect_convert_to_relic").path, "plasma_fading_pink")
				end
			end
		end,
	},
	{
		name 					= "sunflower",
		ui_name					= "Sunflower",
		categories				= "player",
		effect_desc				= "You receive the charm effect for as long as you are in possession \nof this relic, and enemies hit by your spells have a 25% chance of being instantly charmed.",
		effect_desc_short		= "You get charmed, and enemies hit by your spells have a chance of being instantly charmed.",
        precursor_types			= "magic_artifact",
		upgrade_effect_desc		= "For each upgrade level, charm effect chance on enemies increases by 5%.",
		is_upgradable 			= true,
		onenemydamaged_function	= function(damage, message, entity_thats_responsible, is_fatal, projectile_thats_responsible )
			if(isRelicOwned("sunflower") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 25 + (5 * getRelicLevel("sunflower"))
				if(chance >= 1 and chance <= odds) then
					LoadGameEffectEntityTo( GetUpdatedEntityID(), "data/entities/misc/effect_charm_short.xml" )
				end
			end
		end,
		onpickup_function		= function()

			-- adds charm effect
			local game_effect_comp_id, game_effect_entity_id = GetGameEffectLoadTo( getPlayerEntity(), "CHARM", true )
			if game_effect_comp_id ~= nil then
				ComponentSetValue2( game_effect_comp_id, "frames", -1)
				addNewSimplifiedInternalVariable(game_effect_entity_id, "sunflower_charm_effect")
			end
		end,
		ondrop_function			= function()
			
			-- removes charm effect
			local player_children = EntityGetAllChildren(getPlayerEntity())
			if player_children == nil then return end
			for i,child in ipairs(player_children) do
				if(entityHasSimplifiedInternalVariable(child, "sunflower_charm_effect")) then
					EntityKill(child)
					break
				end
			end
		end,
	},
	{
		name 					= "toy_bomber",
		ui_name					= "Toy Bomber",
		categories				= "projectiles,status_effects",
		effect_desc				= "Spells you cast have a 40% chance of inducing enemies to shoot rockets at random directions.",
		effect_desc_short		= "Spells you cast have a chance to induce enemies to randomly shoot rockets.",
        precursor_types			= "gadget",
		upgrade_effect_desc		= "For each upgrade level, effect chance is increased by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("toy_bomber") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 40 + (5 * getRelicLevel("toy_bomber"))
				if(chance >= 1 and chance <= odds) then
					attachLoadEffectOntoEntitiestoProjectile(projectile_entity_id, getCustomEffect("effect_shoot_projectile_random_rocket").path, "smoke")
				end
			end
		end,
	},
	{
		name 					= "crimson_stone",
		ui_name					= "Crimson Stone",
		categories				= "player,kick",
		effect_desc				= "Whenever you kick an enemy, you receive 1 hp.",
		effect_desc_short		= "Whenever you kick an enemy, you receive 1 hp.",
        precursor_types			= "effect_stone,boot,magic_artifact",
		upgrade_effect_desc		= "For each upgrade level, earned hp is increased by 1.",
		is_upgradable 			= true,
		onenemykicked_function	= function(entity_who_kicked )
			if(isRelicOwned("crimson_stone") >= 1) then
				local x, y = EntityGetTransform(getPlayerEntity())
				GamePlaySound( "data/audio/Desktop/event_cues.snd", "event_cues/heart/create", x, y )
				local added_hp = 1 + (1 * getRelicLevel("crimson_stone"))
				addPlayerHealth(added_hp, false)
			end
		end,
	},
	{
		name 					= "pushback_artifact",
		ui_name					= "Pushback Artifact",
		categories				= "projectiles,player",
		effect_desc				= "Whenever you cast a spell, you are pushed in the opposite direction of your spell, but its base damage is doubled.",
		effect_desc_short		= "You are pushed back when you cast a spell but their base damage is doubled.",
        precursor_types			= "effect_stone,magic_artifact",
		upgrade_effect_desc		= "For each upgrade level, base damage multiplier is increased.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("pushback_artifact") >= 1) then
				-- pushback
				local projectile_vel_x, projectile_vel_y = getProjectileVelocitiesandGravity(projectile_entity_id)
				--local pushback = 1 + (1 * getRelicLevel("pushback_artifact"))
				--setPlayerVelocities((projectile_vel_x * pushback) * -1, (projectile_vel_y * pushback) * -1)
				setPlayerVelocities(projectile_vel_x * -1, projectile_vel_y * -1)
				-- base damage boost
				multiplyProjectileBaseDamage(projectile_entity_id, 2 + (1 * getRelicLevel("pushback_artifact")))
			end
		end,
	},
	{
		name 					= "pushforward_artifact",
		ui_name					= "Forward Push Artifact",
		categories				= "projectiles,player",
		effect_desc				= "Whenever you cast a spell, you are pushed in the same direction of your spell, but its critical chance is doubled.",
		effect_desc_short		= "You are pushed forward when you cast a spell but their critical chance is doubled.",
        precursor_types			= "effect_stone,magic_artifact",
		upgrade_effect_desc		= "For each upgrade level, critical chance multiplier is increased.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("pushforward_artifact") >= 1) then
				-- push forward
				local projectile_vel_x, projectile_vel_y = getProjectileVelocitiesandGravity(projectile_entity_id)
				--local pushforward = 1 + (1 * getRelicLevel("pushforward_artifact"))
				--setPlayerVelocities((projectile_vel_x * pushforward), (projectile_vel_y * pushforward))
				setPlayerVelocities(projectile_vel_x, projectile_vel_y)
				-- critical chance boost
				local chance, damage_multiplier = getProjectileCriticalDamageSettings(projectile_entity_id)
				setProjectileCriticalDamageSettings(projectile_entity_id, chance * (2 + (1 * getRelicLevel("pushforward_artifact"))), damage_multiplier)
			end
		end,
	},
	{
		name 					= "gambling_bomb",
		ui_name					= "Gambling Bomb",
		categories				= "player",
		effect_desc				= "Whenever you kill an enemy with your spells, there's a 50% chance you'll \nbe rewarded with either 50 gold or 50 hp, but when this happens, a bomb spawns at your feet.",
		effect_desc_short		= "Kills reward either 50 gold or 50 hp, but bombs spawn when this happens.",
        precursor_types			= "gadget",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onenemykill_function	= function(damage_type_bit_field, damage_message, entity_thats_responsible, drop_items)	
			if(isRelicOwned("gambling_bomb") >= 1) then
				
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 50
				local reward_triggered = false
				if(chance >= 1 and chance <= odds) then
					local x, y = EntityGetTransform(getPlayerEntity())
					if(Random(1,2) == 1) then
						-- hp reward
						GamePlaySound( "data/audio/Desktop/event_cues.snd", "event_cues/heart/create", x, y )
						addPlayerHealth(50, false)
						reward_triggered = true
					else
						-- gold reward
						GamePlaySound( "data/audio/Desktop/event_cues.snd", "event_cues/goldnugget/create", x, y )
						addPlayerGold(50)
						reward_triggered = true
					end
					
					if(reward_triggered) then
						EntityLoad("data/entities/projectiles/bomb.xml", x, y)
					end
				end
			end
		end,
	},
	{
		name 					= "elemental_proximity_artifact_fire",
		ui_name					= "Elemental Proximity Artifact (Fire)",
		categories				= "player,enemies",
		effect_desc				= "For each enemy on fire in the vicinity, your spells get a base damage boost.",
		effect_desc_short		= "Your spells get damage boost for every enemy on fire in the vicinity.",
        precursor_types			= "magic_artifact",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("elemental_proximity_artifact_fire") >= 1) then
				local multiplier = 1
				local player_x, player_y = EntityGetTransform(getPlayerEntity())
				local enemies = EntityGetInRadiusWithTag( player_x, player_y, 150, "enemy" )
				for i, enemy_id in ipairs(enemies) do
					if(GameGetGameEffectCount(enemy_id, "ON_FIRE") > 0) then
						multiplier = multiplier + 1
					end
				end
				
				multiplyProjectileBaseDamage(projectile_entity_id, multiplier)
			end
		end,
	},
	{
		name 					= "elemental_proximity_artifact_water",
		ui_name					= "Elemental Proximity Artifact (Water)",
		categories				= "player,enemies",
		effect_desc				= "Spells that you cast when wet enemies are around cause a heavy water explosion where they hit.",
		effect_desc_short		= "Spells that you cast when wet enemies are around cause a heavy water explosion where they hit.",
        precursor_types			= "magic_artifact",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("elemental_proximity_artifact_water") >= 1) then
				local count = 0
				local player_x, player_y = EntityGetTransform(getPlayerEntity())
				local enemies = EntityGetInRadiusWithTag( player_x, player_y, 150, "enemy" )
				for i, enemy_id in ipairs(enemies) do
					if(GameGetGameEffectCount(enemy_id, "WET") > 0) then
						count = count + 1
					end
				end
				
				if(count > 0) then
					attachLoadEntityOnExplosiontoProjectile(projectile_entity_id, "mods/ArcaneReliquary/files/entity_definitions/others/heavy_explosion_water.xml")
				end
			end
		end,
	},
	{
		name 					= "elemental_proximity_artifact_blood",
		ui_name					= "Elemental Proximity Artifact (Blood)",
		categories				= "player,enemies",
		effect_desc				= "Whenever you kill an enemy while other enemies around are covered in red blood, you are healed by 5.",
		effect_desc_short		= "You are healed by 5 on kills when enemies covered in blood are around.",
        precursor_types			= "magic_artifact",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onenemykill_function	= function(damage_type_bit_field, damage_message, entity_thats_responsible, drop_items)		
			if(isRelicOwned("elemental_proximity_artifact_blood") >= 1) then
				local count = 0
				local player_x, player_y = EntityGetTransform(getPlayerEntity())
				local enemies = EntityGetInRadiusWithTag( player_x, player_y, 150, "enemy" )
				for i, enemy_id in ipairs(enemies) do
					if(GameGetGameEffectCount(enemy_id, "BLOODY") > 0) then
						count = count + 1
					end
				end
				
				if(count > 0) then
					local x, y = EntityGetTransform(getPlayerEntity())
					GamePlaySound( "data/audio/Desktop/event_cues.snd", "event_cues/heart/create", x, y )
					addPlayerHealth(5, false)
				end
			end
		end,
	},
	{
		name 					= "elemental_proximity_artifact_radioactive",
		ui_name					= "Elemental Proximity Artifact (Radioactive)",
		categories				= "player,enemies",
		effect_desc				= "Spells that you cast when enemies covered in toxic sludge are around \ninflict enemies they hit with the time bomb effect, rigging them to explode in a few seconds.",
		effect_desc_short		= "Spells that you cast when enemies covered in toxic sludge are around are inflicted with the time bomb effect.",
        precursor_types			= "magic_artifact",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("elemental_proximity_artifact_radioactive") >= 1) then
				local count = 0
				local player_x, player_y = EntityGetTransform(getPlayerEntity())
				local enemies = EntityGetInRadiusWithTag( player_x, player_y, 150, "enemy" )
				for i, enemy_id in ipairs(enemies) do
					if(GameGetGameEffectCount(enemy_id, "RADIOACTIVE") > 0) then
						count = count + 1
					end
				end
				
				if(count > 0) then
					attachLoadEffectOntoEntitiestoProjectile(projectile_entity_id, getCustomEffect("effect_time_bomb").path, "urine")
				end
			end
		end,
	},
	{
		name 					= "star_fragment_red",
		ui_name					= "Red Star Fragment",
		categories				= "player",
		effect_desc				= "Magic stars rotate around you, converting enemy creatures and projectiles into barrages of firebombs.",
		effect_desc_short		= "Magic stars rotate around you, converting enemy creatures and projectiles into barrages of firebombs.",
        precursor_types			= "gravity_stone",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		ondrop_function			= function()
			detachRotatingShieldfromEntity(getPlayerEntity(), "rotating_shield_projectile_star_red")
		end,
		overtime_function		= function()
			if(isRelicOwned("star_fragment_red") >= 1) then
				if(not entityHasRotatingShield(getPlayerEntity(), "rotating_shield_projectile_star_red")) then -- this makes sure the orbiting projectiles are loaded again if the player is polymorphed
					local start_id = EntityLoad("mods/ArcaneReliquary/files/entity_definitions/custom_projectiles/rotating_shield_start.xml")
					setInternalVariableValue(start_id, "host_entity_id", "value_int", getPlayerEntity())
					setInternalVariableValue(start_id, "rotating_projectile_definition", "value_string", "mods/ArcaneReliquary/files/entity_definitions/custom_projectiles/rotating_star_red.xml")
					setInternalVariableValue(start_id, "rotating_projectiles_amount", "value_int", 3)
					setInternalVariableValue(start_id, "radius", "value_int", 50)
					setInternalVariableValue(start_id, "rotation_speed", "value_int", 2)
				end
			end
		end,
	},
	{
		name 					= "mystic_shield_medical",
		ui_name					= "Medical Mystic Shield",
		categories				= "companions",
		effect_desc				= "A mystic shield that converts enemy projectiles into healing nuggets appears and follows your mouse cursor.",
		effect_desc_short		= "A mystic shield that converts enemy projectiles into healing nuggets appears and follows your mouse cursor.",
        precursor_types			= "companion",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			attachCompaniontoEntity(getPlayerEntity(), "mystic_shield_medical", "mortal")
		end,
		ondrop_function			= function()
			detachCompanionfromEntity(getPlayerEntity(), "mystic_shield_medical")
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "mystic_shield,blood_gem",
	},
	{
		name 					= "star_fragment_cyan",
		ui_name					= "Cyan Star Fragment",
		categories				= "player",
		effect_desc				= "Magic stars rotate around you, converting enemy creatures and projectiles into barrages of freezebombs.",
		effect_desc_short		= "Magic stars rotate around you, converting enemy creatures and projectiles into barrages of freezebombs.",
        precursor_types			= "gravity_stone",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		ondrop_function			= function()
			detachRotatingShieldfromEntity(getPlayerEntity(), "rotating_shield_projectile_star_cyan")
		end,
		overtime_function		= function()
			if(isRelicOwned("star_fragment_cyan") >= 1) then
				if(not entityHasRotatingShield(getPlayerEntity(), "rotating_shield_projectile_star_cyan")) then -- this makes sure the orbiting projectiles are loaded again if the player is polymorphed
					local start_id = EntityLoad("mods/ArcaneReliquary/files/entity_definitions/custom_projectiles/rotating_shield_start.xml")
					setInternalVariableValue(start_id, "host_entity_id", "value_int", getPlayerEntity())
					setInternalVariableValue(start_id, "rotating_projectile_definition", "value_string", "mods/ArcaneReliquary/files/entity_definitions/custom_projectiles/rotating_star_cyan.xml")
					setInternalVariableValue(start_id, "rotating_projectiles_amount", "value_int", 3)
					setInternalVariableValue(start_id, "radius", "value_int", 50)
					setInternalVariableValue(start_id, "rotation_speed", "value_int", 2)
				end
			end
		end,
	},
	{
		name 					= "star_fragment_dark_red",
		ui_name					= "Dark Red Star Fragment",
		categories				= "player",
		effect_desc				= "Magic stars rotate around you, converting enemy creatures and projectiles into barrages of bleeding bombs.",
		effect_desc_short		= "Magic stars rotate around you, converting enemy creatures and projectiles into barrages of bleeding bombs.",
        precursor_types			= "gravity_stone",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		ondrop_function			= function()
			detachRotatingShieldfromEntity(getPlayerEntity(), "rotating_shield_projectile_star_blood")
		end,
		overtime_function		= function()
			if(isRelicOwned("star_fragment_dark_red") >= 1) then
				if(not entityHasRotatingShield(getPlayerEntity(), "rotating_shield_projectile_star_blood")) then -- this makes sure the orbiting projectiles are loaded again if the player is polymorphed
					local start_id = EntityLoad("mods/ArcaneReliquary/files/entity_definitions/custom_projectiles/rotating_shield_start.xml")
					setInternalVariableValue(start_id, "host_entity_id", "value_int", getPlayerEntity())
					setInternalVariableValue(start_id, "rotating_projectile_definition", "value_string", "mods/ArcaneReliquary/files/entity_definitions/custom_projectiles/rotating_star_blood.xml")
					setInternalVariableValue(start_id, "rotating_projectiles_amount", "value_int", 3)
					setInternalVariableValue(start_id, "radius", "value_int", 50)
					setInternalVariableValue(start_id, "rotation_speed", "value_int", 2)
				end
			end
		end,
	},
	{
		name 					= "star_fragment_blue",
		ui_name					= "Blue Star Fragment",
		categories				= "player",
		effect_desc				= "Magic stars rotate around you, converting enemy creatures and projectiles into barrages of electrobombs.",
		effect_desc_short		= "Magic stars rotate around you, converting enemy creatures and projectiles into barrages of electrobombs.",
        precursor_types			= "gravity_stone",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		ondrop_function			= function()
			detachRotatingShieldfromEntity(getPlayerEntity(), "rotating_shield_projectile_star_blue")
		end,
		overtime_function		= function()
			if(isRelicOwned("star_fragment_blue") >= 1) then
				if(not entityHasRotatingShield(getPlayerEntity(), "rotating_shield_projectile_star_blue")) then -- this makes sure the orbiting projectiles are loaded again if the player is polymorphed
					local start_id = EntityLoad("mods/ArcaneReliquary/files/entity_definitions/custom_projectiles/rotating_shield_start.xml")
					setInternalVariableValue(start_id, "host_entity_id", "value_int", getPlayerEntity())
					setInternalVariableValue(start_id, "rotating_projectile_definition", "value_string", "mods/ArcaneReliquary/files/entity_definitions/custom_projectiles/rotating_star_blue.xml")
					setInternalVariableValue(start_id, "rotating_projectiles_amount", "value_int", 3)
					setInternalVariableValue(start_id, "radius", "value_int", 50)
					setInternalVariableValue(start_id, "rotation_speed", "value_int", 2)
				end
			end
		end,
	},
	{
		name 					= "star_fragment_green",
		ui_name					= "Green Star Fragment",
		categories				= "player",
		effect_desc				= "Magic stars rotate around you, converting enemy creatures and projectiles into barrages of slapperbombs.",
		effect_desc_short		= "Magic stars rotate around you, converting enemy creatures and projectiles into barrages of slapperbombs.",
        precursor_types			= "gravity_stone",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		ondrop_function			= function()
			detachRotatingShieldfromEntity(getPlayerEntity(), "rotating_shield_projectile_star_green")
		end,
		overtime_function		= function()
			if(isRelicOwned("star_fragment_green") >= 1) then
				if(not entityHasRotatingShield(getPlayerEntity(), "rotating_shield_projectile_star_green")) then -- this makes sure the orbiting projectiles are loaded again if the player is polymorphed
					local start_id = EntityLoad("mods/ArcaneReliquary/files/entity_definitions/custom_projectiles/rotating_shield_start.xml")
					setInternalVariableValue(start_id, "host_entity_id", "value_int", getPlayerEntity())
					setInternalVariableValue(start_id, "rotating_projectile_definition", "value_string", "mods/ArcaneReliquary/files/entity_definitions/custom_projectiles/rotating_star_green.xml")
					setInternalVariableValue(start_id, "rotating_projectiles_amount", "value_int", 3)
					setInternalVariableValue(start_id, "radius", "value_int", 50)
					setInternalVariableValue(start_id, "rotation_speed", "value_int", 2)
				end
			end
		end,
	},
	{
		name 					= "fun_serum",
		ui_name					= "Fun Serum",
		categories				= "projectiles",
		effect_desc				= "Your spells have a 30% chance of inducing enemies into launching various effect bombs once hit by them.",
		effect_desc_short		= "Your spells have a 30% chance of inducing enemies into launching various effect bombs once hit by them.",
        precursor_types			= "magic_artifact",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("fun_serum") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 30 + (5 * getRelicLevel("fun_serum"))
				if(chance >= 1 and chance <= odds) then
					attachLoadEffectOntoEntitiestoProjectile(projectile_entity_id, getCustomEffect("effect_launch_various_bombs_finite").path, "material_rainbow")
				end
			end
		end
	},
	{
		name 					= "star_fragment_pink",
		ui_name					= "Pink Star Fragment",
		categories				= "player",
		effect_desc				= "Magic stars rotate around you, converting enemy creatures and projectiles into barrages of polybombs.",
		effect_desc_short		= "Magic stars rotate around you, converting enemy creatures and projectiles into barrages of polybombs.",
        precursor_types			= "gravity_stone",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		ondrop_function			= function()
			detachRotatingShieldfromEntity(getPlayerEntity(), "rotating_shield_projectile_star_pink")
		end,
		overtime_function		= function()
			if(isRelicOwned("star_fragment_pink") >= 1) then
				if(not entityHasRotatingShield(getPlayerEntity(), "rotating_shield_projectile_star_pink")) then -- this makes sure the orbiting projectiles are loaded again if the player is polymorphed
					local start_id = EntityLoad("mods/ArcaneReliquary/files/entity_definitions/custom_projectiles/rotating_shield_start.xml")
					setInternalVariableValue(start_id, "host_entity_id", "value_int", getPlayerEntity())
					setInternalVariableValue(start_id, "rotating_projectile_definition", "value_string", "mods/ArcaneReliquary/files/entity_definitions/custom_projectiles/rotating_star_pink.xml")
					setInternalVariableValue(start_id, "rotating_projectiles_amount", "value_int", 3)
					setInternalVariableValue(start_id, "radius", "value_int", 50)
					setInternalVariableValue(start_id, "rotation_speed", "value_int", 2)
				end
			end
		end,
	},
	{
		name 					= "star_fragment_white",
		ui_name					= "White Star Fragment",
		categories				= "player",
		effect_desc				= "Magic stars rotate around you, converting enemy creatures and projectiles into barrages of windbombs.",
		effect_desc_short		= "Magic stars rotate around you, converting enemy creatures and projectiles into barrages of windbombs.",
        precursor_types			= "gravity_stone",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		ondrop_function			= function()
			detachRotatingShieldfromEntity(getPlayerEntity(), "rotating_shield_projectile_star_white")
		end,
		overtime_function		= function()
			if(isRelicOwned("star_fragment_white") >= 1) then
				if(not entityHasRotatingShield(getPlayerEntity(), "rotating_shield_projectile_star_white")) then -- this makes sure the orbiting projectiles are loaded again if the player is polymorphed
					local start_id = EntityLoad("mods/ArcaneReliquary/files/entity_definitions/custom_projectiles/rotating_shield_start.xml")
					setInternalVariableValue(start_id, "host_entity_id", "value_int", getPlayerEntity())
					setInternalVariableValue(start_id, "rotating_projectile_definition", "value_string", "mods/ArcaneReliquary/files/entity_definitions/custom_projectiles/rotating_star_white.xml")
					setInternalVariableValue(start_id, "rotating_projectiles_amount", "value_int", 3)
					setInternalVariableValue(start_id, "radius", "value_int", 50)
					setInternalVariableValue(start_id, "rotation_speed", "value_int", 2)
				end
			end
		end,
	},
	{
		name 					= "star_fragment_purple",
		ui_name					= "Purple Star Fragment",
		categories				= "player",
		effect_desc				= "Magic stars rotate around you, converting enemy creatures and projectiles into barrages of slashbombs.",
		effect_desc_short		= "Magic stars rotate around you, converting enemy creatures and projectiles into barrages of slashbombs.",
        precursor_types			= "gravity_stone",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		ondrop_function			= function()
			detachRotatingShieldfromEntity(getPlayerEntity(), "rotating_shield_projectile_star_purple")
		end,
		overtime_function		= function()
			if(isRelicOwned("star_fragment_purple") >= 1) then
				if(not entityHasRotatingShield(getPlayerEntity(), "rotating_shield_projectile_star_purple")) then -- this makes sure the orbiting projectiles are loaded again if the player is polymorphed
					local start_id = EntityLoad("mods/ArcaneReliquary/files/entity_definitions/custom_projectiles/rotating_shield_start.xml")
					setInternalVariableValue(start_id, "host_entity_id", "value_int", getPlayerEntity())
					setInternalVariableValue(start_id, "rotating_projectile_definition", "value_string", "mods/ArcaneReliquary/files/entity_definitions/custom_projectiles/rotating_star_purple.xml")
					setInternalVariableValue(start_id, "rotating_projectiles_amount", "value_int", 3)
					setInternalVariableValue(start_id, "radius", "value_int", 50)
					setInternalVariableValue(start_id, "rotation_speed", "value_int", 2)
				end
			end
		end,
	},
	{
		name 					= "cursed_dagger",
		ui_name					= "Cursed Dagger",
		categories				= "player",
		effect_desc				= "You release a barrage of slashbombs once damaged by enemies. \nIt also synergizes with Earth Core Fragment relic effect.",
		effect_desc_short		= "You release a barrage of slashbombs once damaged by enemies.",
        precursor_types			= "magic_artifact",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		ondamaged_function		= function(damage, message, entity_thats_responsible, is_fatal, projectile_thats_responsible )
			if(isRelicOwned("cursed_dagger") >= 1) then	
				if(EntityHasTag(entity_thats_responsible, "enemy")) then
					local player_x, player_y = EntityGetTransform(GetUpdatedEntityID())
					EntityLoad("mods/ArcaneReliquary/files/entity_definitions/load_on_explosion/launch_circular_projectiles_slashbombs.xml", player_x, player_y)
				end
			end
		end,
		ongroundpound_function	= function( entity_id)
			if(isRelicOwned("cursed_dagger") >= 1) then
				LoadGameEffectEntityTo( entity_id, getCustomEffect("effect_shoot_projectiles_above_slashbomb").path )
			end
		end,
		
	},
	{
		name 					= "star_fragment_orange",
		ui_name					= "Orange Star Fragment",
		categories				= "player",
		effect_desc				= "Magic stars rotate around you, converting enemy creatures and projectiles into barrages of explodebombs.",
		effect_desc_short		= "Magic stars rotate around you, converting enemy creatures and projectiles into barrages of explodebombs.",
        precursor_types			= "gravity_stone",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		ondrop_function			= function()
			detachRotatingShieldfromEntity(getPlayerEntity(), "rotating_shield_projectile_star_orange")
		end,
		overtime_function		= function()
			if(isRelicOwned("star_fragment_orange") >= 1) then
				if(not entityHasRotatingShield(getPlayerEntity(), "rotating_shield_projectile_star_orange")) then -- this makes sure the orbiting projectiles are loaded again if the player is polymorphed
					local start_id = EntityLoad("mods/ArcaneReliquary/files/entity_definitions/custom_projectiles/rotating_shield_start.xml")
					setInternalVariableValue(start_id, "host_entity_id", "value_int", getPlayerEntity())
					setInternalVariableValue(start_id, "rotating_projectile_definition", "value_string", "mods/ArcaneReliquary/files/entity_definitions/custom_projectiles/rotating_star_orange.xml")
					setInternalVariableValue(start_id, "rotating_projectiles_amount", "value_int", 3)
					setInternalVariableValue(start_id, "radius", "value_int", 50)
					setInternalVariableValue(start_id, "rotation_speed", "value_int", 2)
				end
			end
		end,
	},
	{
		name 					= "exploding_scepter",
		ui_name					= "Exploding Scepter",
		categories				= "projectiles",
		effect_desc				= "Your spells have a 25% chance of exploding into a barrage of explodebombs when they hit an obstacle or explode. \nIt also synergizes with Earth Core Fragment relic effect.",
		effect_desc_short		= "Your spells release explodebombs on impact.",
        precursor_types			= "magic_artifact",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("exploding_scepter") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 25 + (5 * getRelicLevel("exploding_scepter"))
				if(chance >= 1 and chance <= odds) then
					addBuffVisualIndicator_Projectile(projectile_entity_id, "lava")
					attachLoadEntityOnExplosiontoProjectile(projectile_entity_id, getLoadOnExplosion("launch_circular_projectiles_explodebombs").path)
				end
			end
		end,
		ongroundpound_function	= function( entity_id)
			if(isRelicOwned("exploding_scepter") >= 1) then
				LoadGameEffectEntityTo( entity_id, getCustomEffect("effect_shoot_projectiles_above_explodebomb").path )
			end
		end,
	},
	{
		name 					= "toy_submarine_nuclear",
		ui_name					= "Toy Nuclear Submarine",
		categories				= "player",
		effect_desc				= "Upon taking explosion damage, two nuclear bombs rotate around you, exploding when very close to an enemy.",
		effect_desc_short		= "Upon taking explosion damage, two nuclear bombs rotate around you, exploding when very close to an enemy.",
        precursor_types			= "gravity_stone",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		ondrop_function			= function()
			detachRotatingShieldfromEntity(getPlayerEntity(), "rotating_nuclear_bomb")
		end,
		ondamaged_function		= function(damage, message, entity_thats_responsible, is_fatal, projectile_thats_responsible )
			if(isRelicOwned("toy_submarine_nuclear") >= 1) then
				if(message == "$damage_explosion") then
					if(not entityHasRotatingShield(getPlayerEntity(), "rotating_nuclear_bomb")) then -- this makes sure the orbiting projectiles are loaded again if the player is polymorphed
						local start_id = EntityLoad("mods/ArcaneReliquary/files/entity_definitions/custom_projectiles/rotating_shield_start.xml")
						setInternalVariableValue(start_id, "host_entity_id", "value_int", getPlayerEntity())
						setInternalVariableValue(start_id, "rotating_projectile_definition", "value_string", "mods/ArcaneReliquary/files/entity_definitions/custom_projectiles/rotating_nuclear_bomb.xml")
						setInternalVariableValue(start_id, "rotating_projectiles_amount", "value_int", 2)
						setInternalVariableValue(start_id, "radius", "value_int", 70)
						setInternalVariableValue(start_id, "rotation_speed", "value_int", 2)
					end
				end
			end
		end,
	},
	{
		name 					= "star_fragment_gray",
		ui_name					= "Gray Star Fragment",
		categories				= "player",
		effect_desc				= "Magic stars rotate around you, converting enemy creatures and projectiles into barrages of magnetobombs.",
		effect_desc_short		= "Magic stars rotate around you, converting enemy creatures and projectiles into barrages of magnetobombs.",
        precursor_types			= "gravity_stone",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		ondrop_function			= function()
			detachRotatingShieldfromEntity(getPlayerEntity(), "rotating_shield_projectile_star_gray")
		end,
		overtime_function		= function()
			if(isRelicOwned("star_fragment_gray") >= 1) then
				if(not entityHasRotatingShield(getPlayerEntity(), "rotating_shield_projectile_star_gray")) then -- this makes sure the orbiting projectiles are loaded again if the player is polymorphed
					local start_id = EntityLoad("mods/ArcaneReliquary/files/entity_definitions/custom_projectiles/rotating_shield_start.xml")
					setInternalVariableValue(start_id, "host_entity_id", "value_int", getPlayerEntity())
					setInternalVariableValue(start_id, "rotating_projectile_definition", "value_string", "mods/ArcaneReliquary/files/entity_definitions/custom_projectiles/rotating_star_gray.xml")
					setInternalVariableValue(start_id, "rotating_projectiles_amount", "value_int", 3)
					setInternalVariableValue(start_id, "radius", "value_int", 50)
					setInternalVariableValue(start_id, "rotation_speed", "value_int", 2)
				end
			end
		end,
	},
	{
		name 					= "star_fragment_black",
		ui_name					= "Black Star Fragment",
		categories				= "player",
		effect_desc				= "Magic stars rotate around you, converting enemy creatures and projectiles into barrages of bomb laying bombs.",
		effect_desc_short		= "Magic stars rotate around you, converting enemy creatures and projectiles into barrages of bomb laying bombs.",
        precursor_types			= "gravity_stone",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		ondrop_function			= function()
			detachRotatingShieldfromEntity(getPlayerEntity(), "rotating_shield_projectile_star_black")
		end,
		overtime_function		= function()
			if(isRelicOwned("star_fragment_black") >= 1) then
				if(not entityHasRotatingShield(getPlayerEntity(), "rotating_shield_projectile_star_black")) then -- this makes sure the orbiting projectiles are loaded again if the player is polymorphed
					local start_id = EntityLoad("mods/ArcaneReliquary/files/entity_definitions/custom_projectiles/rotating_shield_start.xml")
					setInternalVariableValue(start_id, "host_entity_id", "value_int", getPlayerEntity())
					setInternalVariableValue(start_id, "rotating_projectile_definition", "value_string", "mods/ArcaneReliquary/files/entity_definitions/custom_projectiles/rotating_star_black.xml")
					setInternalVariableValue(start_id, "rotating_projectiles_amount", "value_int", 3)
					setInternalVariableValue(start_id, "radius", "value_int", 50)
					setInternalVariableValue(start_id, "rotation_speed", "value_int", 2)
				end
			end
		end,
	},
	{
		name 					= "windy_skull",
		ui_name					= "Windy Skull",
		categories				= "companions",
		effect_desc				= "A friendly skull that shoots windbombs joins you.",
		effect_desc_short		= "A windbomb shooting companion.",
        precursor_types			= "companion",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			attachCompaniontoEntity(getPlayerEntity(), "windy_skull", "mortal")
		end,
		ondrop_function			= function()
			detachCompanionfromEntity(getPlayerEntity(), "windy_skull")
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "enchanted_skull,little_cloud",
		requires_relic			= "bone",
	},
	{
		name 					= "slash_skull",
		ui_name					= "Slash Skull",
		categories				= "companions",
		effect_desc				= "A friendly skull that shoots slashbombs joins you.",
		effect_desc_short		= "A slashbomb shooting companion.",
        precursor_types			= "companion",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			attachCompaniontoEntity(getPlayerEntity(), "slash_skull", "mortal")
		end,
		ondrop_function			= function()
			detachCompanionfromEntity(getPlayerEntity(), "slash_skull")
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "enchanted_skull,shuriken",
		requires_relic			= "bone",
	},
	{
		name 					= "explode_skull",
		ui_name					= "Explode Skull",
		categories				= "companions",
		effect_desc				= "A friendly skull that shoots explodebombs joins you.",
		effect_desc_short		= "A explodebomb shooting companion.",
        precursor_types			= "companion",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			attachCompaniontoEntity(getPlayerEntity(), "explode_skull", "mortal")
		end,
		ondrop_function			= function()
			detachCompanionfromEntity(getPlayerEntity(), "explode_skull")
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "enchanted_skull,freedom_ring",
		requires_relic			= "bone",
	},
	{
		name 					= "poly_skull",
		ui_name					= "Poly Skull",
		categories				= "companions",
		effect_desc				= "A friendly skull that shoots polybombs joins you.",
		effect_desc_short		= "A polybomb shooting companion.",
        precursor_types			= "companion",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			attachCompaniontoEntity(getPlayerEntity(), "poly_skull", "mortal")
		end,
		ondrop_function			= function()
			detachCompanionfromEntity(getPlayerEntity(), "poly_skull")
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "enchanted_skull,poly_gem",
		requires_relic			= "bone",
	},
	{
		name 					= "magneto_skull",
		ui_name					= "Magneto Skull",
		categories				= "companions",
		effect_desc				= "A friendly skull that shoots magnetobombs joins you.",
		effect_desc_short		= "A magnetobomb shooting companion.",
        precursor_types			= "companion",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			attachCompaniontoEntity(getPlayerEntity(), "magneto_skull", "mortal")
		end,
		ondrop_function			= function()
			detachCompanionfromEntity(getPlayerEntity(), "magneto_skull")
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "enchanted_skull,magnet",
		requires_relic			= "bone",
	},
	{
		name 					= "bomblay_skull",
		ui_name					= "Bomb Lay Skull",
		categories				= "companions",
		effect_desc				= "A friendly skull that shoots bomb laying bombs joins you.",
		effect_desc_short		= "A bomb laying bomb shooting companion.",
        precursor_types			= "companion",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			attachCompaniontoEntity(getPlayerEntity(), "bomblay_skull", "mortal")
		end,
		ondrop_function			= function()
			detachCompanionfromEntity(getPlayerEntity(), "bomblay_skull")
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "enchanted_skull,bomb_charm",
		requires_relic			= "bone",
	},
	{
		name 					= "diamond_ring_purple_dark",
		ui_name					= "Dark Purple Diamond Ring",
		categories				= "projectiles,status_effects",
		effect_desc				= "Your spells have a 30% chance of creating a mystic target on \ncreatures you hit, making them summon slashbombs towards themselves.",
		effect_desc_short		= "Your spells create mystic targets on creatures, summoning slashbombs towards them.",
        precursor_types			= "magic_artifact,ring",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("diamond_ring_purple_dark") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 30 + (5 * getRelicLevel("diamond_ring_purple_dark"))
				if(chance >= 1 and chance <= odds) then
					attachLoadEffectOntoEntitiestoProjectile(projectile_entity_id, getCustomEffect("effect_projectile_target_slashbomb").path, "spark_purple_bright")
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "elemental_band,cursed_dagger",
	},
	{
		name 					= "diamond_ring_white",
		ui_name					= "White Diamond Ring",
		categories				= "projectiles,status_effects",
		effect_desc				= "Your spells have a 30% chance of creating a mystic target on \ncreatures you hit, making them summon windbombs towards themselves.",
		effect_desc_short		= "Your spells create mystic targets on creatures, summoning windbombs towards them.",
        precursor_types			= "magic_artifact,ring",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("diamond_ring_white") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 30 + (5 * getRelicLevel("diamond_ring_white"))
				if(chance >= 1 and chance <= odds) then
					attachLoadEffectOntoEntitiestoProjectile(projectile_entity_id, getCustomEffect("effect_projectile_target_windbomb").path, "spark_white")
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "elemental_band,pinwheel",
	},
	{
		name 					= "diamond_ring_pink_light",
		ui_name					= "Light Pink Diamond Ring",
		categories				= "projectiles,status_effects",
		effect_desc				= "Your spells have a 30% chance of creating a mystic target on \ncreatures you hit, making them summon spark bolts towards themselves.",
		effect_desc_short		= "Your spells create mystic targets on creatures, summoning spark bolts towards them.",
        precursor_types			= "magic_artifact,ring",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("diamond_ring_pink_light") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 30 + (5 * getRelicLevel("diamond_ring_pink_light"))
				if(chance >= 1 and chance <= odds) then
					attachLoadEffectOntoEntitiestoProjectile(projectile_entity_id, getCustomEffect("effect_projectile_target_sparkbolt").path, "plasma_fading_pink")
				end
			end
		end,
	},
	{
		name 					= "diamond_ring_blue_light",
		ui_name					= "Light Blue Diamond Ring",
		categories				= "projectiles,status_effects",
		effect_desc				= "Your spells have a 30% chance of creating a mystic target on \ncreatures you hit, making them summon energy orbs towards themselves.",
		effect_desc_short		= "Your spells create mystic targets on creatures, summoning energy orbs towards them.",
        precursor_types			= "magic_artifact,ring",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("diamond_ring_blue_light") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 30 + (5 * getRelicLevel("diamond_ring_blue_light"))
				if(chance >= 1 and chance <= odds) then
					attachLoadEffectOntoEntitiestoProjectile(projectile_entity_id, getCustomEffect("effect_projectile_target_energyorb").path, "plasma_fading")
				end
			end
		end,
	},
	{
		name 					= "diamond_ring_silver",
		ui_name					= "Silver Diamond Ring",
		categories				= "projectiles,status_effects",
		effect_desc				= "Your spells have a 30% chance of creating a mystic target on \ncreatures you hit, making them summon rockets towards themselves.",
		effect_desc_short		= "Your spells create mystic targets on creatures, summoning rockets towards them.",
        precursor_types			= "magic_artifact,ring",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("diamond_ring_silver") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 30 + (5 * getRelicLevel("diamond_ring_silver"))
				if(chance >= 1 and chance <= odds) then
					attachLoadEffectOntoEntitiestoProjectile(projectile_entity_id, getCustomEffect("effect_projectile_target_rocket").path, "spark_white")
				end
			end
		end,
	},
	{
		name 					= "star_fragment_gold",
		ui_name					= "Gold Star Fragment",
		categories				= "player",
		effect_desc				= "Magic stars rotate around you, converting enemy creatures and projectiles into barrages of midasbombs.",
		effect_desc_short		= "Magic stars rotate around you, converting enemy creatures and projectiles into barrages of midasbombs.",
        precursor_types			= "gravity_stone",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		ondrop_function			= function()
			detachRotatingShieldfromEntity(getPlayerEntity(), "rotating_shield_projectile_star_gold")
		end,
		overtime_function		= function()
			if(isRelicOwned("star_fragment_gold") >= 1) then
				if(not entityHasRotatingShield(getPlayerEntity(), "rotating_shield_projectile_star_gold")) then -- this makes sure the orbiting projectiles are loaded again if the player is polymorphed
					local start_id = EntityLoad("mods/ArcaneReliquary/files/entity_definitions/custom_projectiles/rotating_shield_start.xml")
					setInternalVariableValue(start_id, "host_entity_id", "value_int", getPlayerEntity())
					setInternalVariableValue(start_id, "rotating_projectile_definition", "value_string", "mods/ArcaneReliquary/files/entity_definitions/custom_projectiles/rotating_star_gold.xml")
					setInternalVariableValue(start_id, "rotating_projectiles_amount", "value_int", 3)
					setInternalVariableValue(start_id, "radius", "value_int", 50)
					setInternalVariableValue(start_id, "rotation_speed", "value_int", 2)
				end
			end
		end,
	},
	{
		name 					= "midas_skull",
		ui_name					= "Midas Skull",
		categories				= "companions",
		effect_desc				= "A friendly skull that shoots midasbombs joins you.",
		effect_desc_short		= "A midasbomb shooting companion.",
        precursor_types			= "companion",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			attachCompaniontoEntity(getPlayerEntity(), "midas_skull", "mortal")
		end,
		ondrop_function			= function()
			detachCompanionfromEntity(getPlayerEntity(), "midas_skull")
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "enchanted_skull,midas_stone",
		requires_relic			= "bone",
	},
	{
		name 					= "scarab_shell_golden",
		ui_name					= "Golden Scarab Shell",
		categories				= "projectiles",
		effect_desc				= "Most spells you cast have a 50% chance of shooting midasbombs at random directions once cast. \nIt also synergizes with Earth Core Fragment relic effect.",
		effect_desc_short		= "Your spells shoot midasbombs.",
        precursor_types			= "magic_artifact",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5% and when the effect triggers, midasbombs are shot more frequently.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("scarab_shell_golden") >= 1) then
				if((not isPhysicsProjectile(projectile_entity_id)) and (not isProjectileHittable(projectile_entity_id))) then
					SetRandomSeed( GameGetFrameNum(), math.random(1,50))
					local chance = Random(1,100)
					local odds = 50 + (5 * getRelicLevel("scarab_shell_golden"))
					if(chance >= 1 and chance <= odds) then
						addNewInternalVariable(projectile_entity_id, "projectile_definition", "value_string", "mods/ArcaneReliquary/files/entity_definitions/custom_projectiles/midasbomb.xml")
						addNewInternalVariable(projectile_entity_id, "projectile_velocity", "value_int", 140)
						addNewInternalVariable(projectile_entity_id, "max_projectiles_shot", "value_int", 25)
						addNewLuaCallback(projectile_entity_id, "overtime", "mods/ArcaneReliquary/files/scripts/projectiles/launch_projectile_random_direction.lua", 15 - (2 * getRelicLevel("scarab_shell_golden")), false, "launch_projectile_random_direction")
					end
				end
			end
		end,
		ongroundpound_function	= function( entity_id)
			if(isRelicOwned("scarab_shell_golden") >= 1) then
				LoadGameEffectEntityTo( entity_id, getCustomEffect("effect_shoot_projectiles_above_midasbomb").path )
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "scarab_shell,midas_essence",
	},
	{
		name 					= "scarab_shell_lazuli",
		ui_name					= "Lazuli Scarab Shell",
		categories				= "enemies,player",
		effect_desc				= "Upon taking damage from an enemy, other enemies in the vicinity are electrocuted and thunderballs are summoned on their place.",
		effect_desc_short		= "Being damaged by an enemy electrocutes enemies around and summons thunderballs on them.",
        precursor_types			= "magic_artifact",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		ondamaged_function		= function(damage, message, entity_thats_responsible, is_fatal, projectile_thats_responsible )
			if(isRelicOwned("scarab_shell_lazuli") >= 1) then
				if(EntityHasTag(entity_thats_responsible, "enemy")) then
					local player_x, player_y = EntityGetTransform(GetUpdatedEntityID())
					local enemies = EntityGetInRadiusWithTag( player_x, player_y, 200, "enemy" )
					for i, enemy_id in ipairs(enemies) do
						if(enemy_id ~= entity_thats_responsible) then
							GetGameEffectLoadTo( enemy_id, "ELECTROCUTION", true )
							local enemy_x, enemy_y = EntityGetTransform(enemy_id)
							EntityLoad("mods/ArcaneReliquary/files/entity_definitions/custom_projectiles/thunderball_custom.xml", enemy_x, enemy_y - 15)
						end
					end
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "scarab_shell,overloaded_necklace",
	},
	{
		name 					= "scarab_shell_crimson",
		ui_name					= "Crimson Scarab Shell",
		categories				= "enemies,player",
		effect_desc				= "Enemies that damage you have a 50% chance of being inflicted with the heavy bleeding effect.",
		effect_desc_short		= "Enemies that damage you are inflicted with the heavy bleeding effect.",
        precursor_types			= "magic_artifact",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		ondamaged_function		= function(damage, message, entity_thats_responsible, is_fatal, projectile_thats_responsible )
			if(isRelicOwned("scarab_shell_crimson") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 50 + (5 * getRelicLevel("scarab_shell_crimson"))
				if(chance >= 1 and chance <= odds) then
					LoadGameEffectEntityTo( entity_thats_responsible, getCustomEffect("effect_bleeding_heavy").path )
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "scarab_shell,ancient_dagger",
	},
	{
		name 					= "scarab_shell_amethyst",
		ui_name					= "Amethyst Scarab Shell",
		categories				= "enemies,player",
		effect_desc				= "Enemies that damage you with projectiles have a 50% chance of being inflicted with the slash effect.",
		effect_desc_short		= "Enemies that damage you with projectiles are inflicted with the slash effect.",
        precursor_types			= "magic_artifact",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		ondamaged_function		= function(damage, message, entity_thats_responsible, is_fatal, projectile_thats_responsible )
			if(isRelicOwned("scarab_shell_amethyst") >= 1) then
				if(message == "$damage_projectile") then
					SetRandomSeed( GameGetFrameNum(), math.random(1,50))
					local chance = Random(1,100)
					local odds = 50 + (5 * getRelicLevel("scarab_shell_amethyst"))
					if(chance >= 1 and chance <= odds) then
						LoadGameEffectEntityTo( entity_thats_responsible, getCustomEffect("effect_slash").path )
					end
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "scarab_shell,shuriken",
	},
	{
		name 					= "scarab_shell_scorching",
		ui_name					= "Scorching Scarab Shell",
		categories				= "enemies,player",
		effect_desc				= "Upon taking damage from an enemy, other enemies in the vicinity are inflicted with the firebomb shockwave effect.",
		effect_desc_short		= "Being damaged by an enemy inflicts other enemies with the firebomb shockwave effect.",
        precursor_types			= "magic_artifact",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		ondamaged_function		= function(damage, message, entity_thats_responsible, is_fatal, projectile_thats_responsible )
			if(isRelicOwned("scarab_shell_scorching") >= 1) then
				if(EntityHasTag(entity_thats_responsible, "enemy")) then
					local player_x, player_y = EntityGetTransform(GetUpdatedEntityID())
					local enemies = EntityGetInRadiusWithTag( player_x, player_y, 200, "enemy" )
					for i, enemy_id in ipairs(enemies) do
						if(enemy_id ~= entity_thats_responsible) then
							LoadGameEffectEntityTo( enemy_id, getCustomEffect("effect_shockwave_firebombs").path )
						end
					end
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "scarab_shell,sacred_ember",
	},
	{
		name 					= "tuning_fork_sulfuric",
		ui_name					= "Sulfuric Tuning Fork",
		categories				= "projectiles,status_effects",
		effect_desc				= "Your spells have a 30% chance of inducing \nfirebombs shockwaves on enemies.",
		effect_desc_short		= "Your spells induce firebombs shockwaves on enemies.",
        precursor_types			= "magic_artifact,gadget",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("tuning_fork_sulfuric") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 30 + (5 * getRelicLevel("tuning_fork_sulfuric"))
				if(chance >= 1 and chance <= odds) then
					attachLoadEffectOntoEntitiestoProjectile(projectile_entity_id, getCustomEffect("effect_shockwave_firebombs").path, "lava")
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "tuning_fork,sulfur_crystal",
	},
	{
		name 					= "scarab_shell_rose",
		ui_name					= "Rose Scarab Shell",
		categories				= "enemies,player",
		effect_desc				= "Your spells have a 40% chance of mutating enemies they hit into random creatures.",
		effect_desc_short		= "Your spells have a chance to mutate enemies they hit into random creatures.",
        precursor_types			= "magic_artifact",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("scarab_shell_rose") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 40 + (5 * getRelicLevel("scarab_shell_rose"))
				if(chance >= 1 and chance <= odds) then
					attachLoadEffectOntoEntitiestoProjectile(projectile_entity_id, "data/entities/misc/effect_polymorph_random.xml", "plasma_fading_pink")
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "scarab_shell,chaotic_poly_flask",
	},
	{
		name 					= "berserk_skull",
		ui_name					= "Berserk Skull",
		categories				= "companions",
		effect_desc				= "A friendly skull that shoots explosive fireballs joins you.",
		effect_desc_short		= "An explosive fireball shooting companion.",
        precursor_types			= "companion",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			attachCompaniontoEntity(getPlayerEntity(), "berserk_skull", "mortal")
		end,
		ondrop_function			= function()
			detachCompanionfromEntity(getPlayerEntity(), "berserk_skull")
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "flaming_skull,ember_stone",
		requires_relic			= "bone",
	},
	{
		name 					= "nuclear_skull",
		ui_name					= "Nuclear Skull",
		categories				= "companions",
		effect_desc				= "A friendly skull that shoots radioactive fireballs joins you.",
		effect_desc_short		= "A radioactive fireball shooting companion.",
        precursor_types			= "companion",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			attachCompaniontoEntity(getPlayerEntity(), "nuclear_skull", "mortal")
		end,
		ondrop_function			= function()
			detachCompanionfromEntity(getPlayerEntity(), "nuclear_skull")
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "flaming_skull,uranium_ingot",
		requires_relic			= "bone",
	},
	{
		name 					= "energy_crystal_blue",
		ui_name					= "Blue Energy Crystal",
		categories				= "player",
		effect_desc				= "A pulsating energy field emanates from you, it destroys enemy projectiles and \nshoots electrobombs back and also electrocutes enemies it touches. \nIf you take damage, you are electrocuted.",
		effect_desc_short		= "An energy field emanates from you, destroying enemy projectiles and shooting electrobombs back.",
        precursor_types			= "magic_artifact,energy_shield",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			LoadGameEffectEntityTo( getPlayerEntity(), getCustomEffect("effect_pulsating_shield_projectile_scatter_electrobomb").path )
		end,
		ondrop_function			= function()
			local player_children = EntityGetAllChildren(getPlayerEntity())
			if player_children == nil then return end
			for i,child in ipairs(player_children) do
				if(EntityGetName(child) == "pulsating_shield_projectiles_scatter_electrobomb") then
					EntityKill(child)
					break
				end
			end
		end,
		ondamaged_function		= function(damage, message, entity_thats_responsible, is_fatal, projectile_thats_responsible )
			if(isRelicOwned("energy_crystal_blue") >= 1) then
				if(EntityHasTag(entity_thats_responsible, "enemy")) then
					LoadGameEffectEntityTo( GetUpdatedEntityID(), "data/entities/misc/effect_electricity.xml" )
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "mechanical_heart,ancient_root,charged_necklace",
	},
	{
		name 					= "blizzard_skull",
		ui_name					= "Blizzard Skull",
		categories				= "companions",
		effect_desc				= "A friendly skull that shoots iceballs joins you.",
		effect_desc_short		= "An iceball shooting companion.",
        precursor_types			= "companion",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			attachCompaniontoEntity(getPlayerEntity(), "blizzard_skull", "mortal")
		end,
		ondrop_function			= function()
			detachCompanionfromEntity(getPlayerEntity(), "blizzard_skull")
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "flaming_skull,frost_artifact",
		requires_relic			= "bone",
	},
	{
		name 					= "energy_crystal_red",
		ui_name					= "Red Energy Crystal",
		categories				= "player",
		effect_desc				= "A pulsating energy field emanates from you, it destroys enemy projectiles and \nshoots bloodbombs back and also inflicts bleeding on enemies it touches. \nIf you take damage, you are inflicted with bleeding.",
		effect_desc_short		= "An energy field emanates from you, destroying enemy projectiles and shooting bloodbombs back.",
        precursor_types			= "magic_artifact,energy_shield",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			LoadGameEffectEntityTo( getPlayerEntity(), getCustomEffect("effect_pulsating_shield_projectile_scatter_bloodbomb").path )
		end,
		ondrop_function			= function()
			local player_children = EntityGetAllChildren(getPlayerEntity())
			if player_children == nil then return end
			for i,child in ipairs(player_children) do
				if(EntityGetName(child) == "pulsating_shield_projectiles_scatter_bloodbomb") then
					EntityKill(child)
					break
				end
			end
		end,
		ondamaged_function		= function(damage, message, entity_thats_responsible, is_fatal, projectile_thats_responsible )
			if(isRelicOwned("energy_crystal_red") >= 1) then
				if(EntityHasTag(entity_thats_responsible, "enemy")) then
					LoadGameEffectEntityTo( GetUpdatedEntityID(), getCustomEffect("effect_bleeding").path )
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "mechanical_heart,ancient_root,ancient_dagger",
	},
	{
		name 					= "fallout_skull",
		ui_name					= "Fallout Skull",
		categories				= "companions",
		effect_desc				= "A friendly skull that shoots fallout fireballs joins you.",
		effect_desc_short		= "A fallout fireball shooting companion.",
        precursor_types			= "companion",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			attachCompaniontoEntity(getPlayerEntity(), "fallout_skull", "mortal")
		end,
		ondrop_function			= function()
			detachCompanionfromEntity(getPlayerEntity(), "fallout_skull")
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "flaming_skull,leaking_nuclear_core",
		requires_relic			= "bone",
	},
	{
		name 					= "bitter_skull",
		ui_name					= "Bitter Skull",
		categories				= "companions",
		effect_desc				= "A friendly skull that shoots acid fireballs joins you.",
		effect_desc_short		= "An acid fireball shooting companion.",
        precursor_types			= "companion",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			attachCompaniontoEntity(getPlayerEntity(), "bitter_skull", "mortal")
		end,
		ondrop_function			= function()
			detachCompanionfromEntity(getPlayerEntity(), "bitter_skull")
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "flaming_skull,toxic_gland",
		requires_relic			= "bone",
	},
	{
		name 					= "venomous_skull",
		ui_name					= "Venomous Skull",
		categories				= "companions",
		effect_desc				= "A friendly skull that shoots poison fireballs joins you.",
		effect_desc_short		= "A poison fireball shooting companion.",
        precursor_types			= "companion",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			attachCompaniontoEntity(getPlayerEntity(), "venomous_skull", "mortal")
		end,
		ondrop_function			= function()
			detachCompanionfromEntity(getPlayerEntity(), "venomous_skull")
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "flaming_skull,vile_stone",
		requires_relic			= "bone",
	},
	{
		name 					= "vault_boy_figurine",
		ui_name					= "Vault Boy Figurine",
		categories				= "projectiles",
		effect_desc				= "Your spells have a 30% chance of exploding into a \nbarrage of fallout fireballs when they explode or hit something. \nIt also synergizes with Earth Core Fragment relic effect.",
		effect_desc_short		= "Your spells explode into fallout fireballs on impact.",
        precursor_types			= "gadget",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("vault_boy_figurine") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 30 + (5 * getRelicLevel("vault_boy_figurine"))
				if(chance >= 1 and chance <= odds) then	
					addBuffVisualIndicator_Projectile(projectile_entity_id, "lava")
					attachLoadEntityOnExplosiontoProjectile(projectile_entity_id, getLoadOnExplosion("launch_circular_projectiles_fireballs_fallout").path)
				end
			end
		end,
		ongroundpound_function	= function( entity_id)
			if(isRelicOwned("vault_boy_figurine") >= 1) then
				LoadGameEffectEntityTo( entity_id, getCustomEffect("effect_shoot_projectiles_above_fireball_fallout").path )
			end
		end,
	},
	{
		name 					= "bitter_crystal",
		ui_name					= "Bitter Crystal",
		categories				= "projectiles",
		effect_desc				= "Spells that you cast have a 30% chance of shooting acid fireballs downwards along their travel path.",
		effect_desc_short		= "Your spells cast downward acid fireballs.",
        precursor_types			= "effect_crystal",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%, and when it triggers, fireballs are shot more often.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("bitter_crystal") >= 1) then
				if(not isPhysicsProjectile(projectile_entity_id)) then
					SetRandomSeed( GameGetFrameNum(), math.random(1,50))
					local chance = Random(1,100)
					local odds = 30 + (5 * getRelicLevel("bitter_crystal"))
					if(chance >= 1 and chance <= odds) then
						addNewInternalVariable(projectile_entity_id, "projectile_shot_down_definition", "value_string", "mods/ArcaneReliquary/files/entity_definitions/custom_projectiles/fireball_acid.xml")
						addNewInternalVariable(projectile_entity_id, "down_vel_min", "value_int", 165)
						addNewInternalVariable(projectile_entity_id, "down_vel_max", "value_int", 185)
						addNewInternalVariable(projectile_entity_id, "max_projectiles_shot", "value_int", 10)
						addNewLuaCallback(projectile_entity_id, "overtime", "mods/ArcaneReliquary/files/scripts/projectiles/launch_projectile_down.lua", 25 - (2 * getRelicLevel("bitter_crystal")), false, "launch_projectile_down")
					end
				end
			end
		end,
	},
	{
		name 					= "venomous_crystal",
		ui_name					= "Venomous Crystal",
		categories				= "projectiles",
		effect_desc				= "Spells that you cast have a 30% chance of shooting poison fireballs downwards along their travel path.",
		effect_desc_short		= "Your spells cast downward poison fireballs.",
        precursor_types			= "effect_crystal",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%, and when it triggers, fireballs are shot more often.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("venomous_crystal") >= 1) then
				if(not isPhysicsProjectile(projectile_entity_id)) then
					SetRandomSeed( GameGetFrameNum(), math.random(1,50))
					local chance = Random(1,100)
					local odds = 30 + (5 * getRelicLevel("venomous_crystal"))
					if(chance >= 1 and chance <= odds) then
						addNewInternalVariable(projectile_entity_id, "projectile_shot_down_definition", "value_string", "mods/ArcaneReliquary/files/entity_definitions/custom_projectiles/fireball_poison.xml")
						addNewInternalVariable(projectile_entity_id, "down_vel_min", "value_int", 165)
						addNewInternalVariable(projectile_entity_id, "down_vel_max", "value_int", 185)
						addNewInternalVariable(projectile_entity_id, "max_projectiles_shot", "value_int", 10)
						addNewLuaCallback(projectile_entity_id, "overtime", "mods/ArcaneReliquary/files/scripts/projectiles/launch_projectile_down.lua", 25 - (2 * getRelicLevel("venomous_crystal")), false, "launch_projectile_down")
					end
				end
			end
		end,
	},
	{
		name 					= "humid_skull",
		ui_name					= "Humid Skull",
		categories				= "companions",
		effect_desc				= "A friendly skull that shoots water fireballs joins you.",
		effect_desc_short		= "A water fireball shooting companion.",
        precursor_types			= "companion",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			attachCompaniontoEntity(getPlayerEntity(), "humid_skull", "mortal")
		end,
		ondrop_function			= function()
			detachCompanionfromEntity(getPlayerEntity(), "humid_skull")
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "flaming_skull,water_gem",
		requires_relic			= "bone",
	},
	{
		name 					= "aquatic_crystal",
		ui_name					= "Aquatic Crystal",
		categories				= "projectiles",
		effect_desc				= "Spells that you cast have a 30% chance of shooting water fireballs downwards along their travel path.",
		effect_desc_short		= "Your spells cast downward water fireballs.",
        precursor_types			= "effect_crystal",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%, and when it triggers, fireballs are shot more often.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("aquatic_crystal") >= 1) then
				if(not isPhysicsProjectile(projectile_entity_id)) then
					SetRandomSeed( GameGetFrameNum(), math.random(1,50))
					local chance = Random(1,100)
					local odds = 30 + (5 * getRelicLevel("aquatic_crystal"))
					if(chance >= 1 and chance <= odds) then
						addNewInternalVariable(projectile_entity_id, "projectile_shot_down_definition", "value_string", "mods/ArcaneReliquary/files/entity_definitions/custom_projectiles/fireball_water.xml")
						addNewInternalVariable(projectile_entity_id, "down_vel_min", "value_int", 165)
						addNewInternalVariable(projectile_entity_id, "down_vel_max", "value_int", 185)
						addNewInternalVariable(projectile_entity_id, "max_projectiles_shot", "value_int", 10)
						addNewLuaCallback(projectile_entity_id, "overtime", "mods/ArcaneReliquary/files/scripts/projectiles/launch_projectile_down.lua", 25 - (2 * getRelicLevel("aquatic_crystal")), false, "launch_projectile_down")
					end
				end
			end
		end,
	},
	{
		name 					= "energy_crystal_blue_light",
		ui_name					= "Light Blue Energy Crystal",
		categories				= "player",
		effect_desc				= "A pulsating energy field emanates from you, it destroys enemy projectiles and \nshoots freezebombs back and also freezes enemies it touches. \nIf you take damage, you are frozen.",
		effect_desc_short		= "An energy field emanates from you, destroying enemy projectiles and shooting freezebombs back.",
        precursor_types			= "magic_artifact,energy_shield",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			LoadGameEffectEntityTo( getPlayerEntity(), getCustomEffect("effect_pulsating_shield_projectile_scatter_freezebomb").path )
		end,
		ondrop_function			= function()
			local player_children = EntityGetAllChildren(getPlayerEntity())
			if player_children == nil then return end
			for i,child in ipairs(player_children) do
				if(EntityGetName(child) == "pulsating_shield_projectiles_scatter_freezebomb") then
					EntityKill(child)
					break
				end
			end
		end,
		ondamaged_function		= function(damage, message, entity_thats_responsible, is_fatal, projectile_thats_responsible )
			if(isRelicOwned("energy_crystal_blue_light") >= 1) then
				if(EntityHasTag(entity_thats_responsible, "enemy")) then
					LoadGameEffectEntityTo( GetUpdatedEntityID(), "data/entities/misc/effect_frozen.xml" )
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "mechanical_heart,ancient_root,popsicle",
	},
	{
		name 					= "energy_crystal_green",
		ui_name					= "Green Energy Crystal",
		categories				= "player",
		effect_desc				= "A pulsating energy field emanates from you, it destroys enemy projectiles and \nshoots slapperbombs back and also inflicts the slap effect on enemies it touches. \nIf you take damage, you are inflicted with the slap effect.",
		effect_desc_short		= "An energy field emanates from you, destroying enemy projectiles and shooting slapperbombs back.",
        precursor_types			= "magic_artifact,energy_shield",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			LoadGameEffectEntityTo( getPlayerEntity(), getCustomEffect("effect_pulsating_shield_projectile_scatter_slapperbomb").path )
		end,
		ondrop_function			= function()
			local player_children = EntityGetAllChildren(getPlayerEntity())
			if player_children == nil then return end
			for i,child in ipairs(player_children) do
				if(EntityGetName(child) == "pulsating_shield_projectiles_scatter_slapperbomb") then
					EntityKill(child)
					break
				end
			end
		end,
		ondamaged_function		= function(damage, message, entity_thats_responsible, is_fatal, projectile_thats_responsible )
			if(isRelicOwned("energy_crystal_green") >= 1) then
				if(EntityHasTag(entity_thats_responsible, "enemy")) then
					LoadGameEffectEntityTo( GetUpdatedEntityID(), getCustomEffect("effect_slap").path )
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "mechanical_heart,ancient_root,fly_swatter",
	},
	{
		name 					= "energy_crystal_white",
		ui_name					= "White Energy Crystal",
		categories				= "player",
		effect_desc				= "A pulsating energy field emanates from you, it destroys enemy projectiles and \nshoots windbombs back and also inflicts the heavy wind effect on enemies it touches. \nIf you take damage, you are inflicted with the heavy wind effect.",
		effect_desc_short		= "An energy field emanates from you, destroying enemy projectiles and shooting windbombs back.",
        precursor_types			= "magic_artifact,energy_shield",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			LoadGameEffectEntityTo( getPlayerEntity(), getCustomEffect("effect_pulsating_shield_projectile_scatter_windbomb").path )
		end,
		ondrop_function			= function()
			local player_children = EntityGetAllChildren(getPlayerEntity())
			if player_children == nil then return end
			for i,child in ipairs(player_children) do
				if(EntityGetName(child) == "pulsating_shield_projectiles_scatter_windbomb") then
					EntityKill(child)
					break
				end
			end
		end,
		ondamaged_function		= function(damage, message, entity_thats_responsible, is_fatal, projectile_thats_responsible )
			if(isRelicOwned("energy_crystal_white") >= 1) then
				if(EntityHasTag(entity_thats_responsible, "enemy")) then
					LoadGameEffectEntityTo( GetUpdatedEntityID(), getCustomEffect("effect_wind_heavy").path )
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "mechanical_heart,ancient_root,little_cloud",
	},
	{
		name 					= "energy_crystal_orange",
		ui_name					= "Orange Energy Crystal",
		categories				= "player",
		effect_desc				= "A pulsating energy field emanates from you, it destroys enemy projectiles and \nshoots explodebombs back and also inflicts the exploding effect on enemies it touches. \nIf you take damage, you are inflicted with the exploding effect.",
		effect_desc_short		= "An energy field emanates from you, destroying enemy projectiles and shooting explodebombs back.",
        precursor_types			= "magic_artifact,energy_shield",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			LoadGameEffectEntityTo( getPlayerEntity(), getCustomEffect("effect_pulsating_shield_projectile_scatter_explodebomb").path )
		end,
		ondrop_function			= function()
			local player_children = EntityGetAllChildren(getPlayerEntity())
			if player_children == nil then return end
			for i,child in ipairs(player_children) do
				if(EntityGetName(child) == "pulsating_shield_projectiles_scatter_explodebomb") then
					EntityKill(child)
					break
				end
			end
		end,
		ondamaged_function		= function(damage, message, entity_thats_responsible, is_fatal, projectile_thats_responsible )
			if(isRelicOwned("energy_crystal_orange") >= 1) then
				if(EntityHasTag(entity_thats_responsible, "enemy")) then
					LoadGameEffectEntityTo( GetUpdatedEntityID(), getCustomEffect("effect_exploding").path )
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "mechanical_heart,ancient_root,freedom_ring",
	},
	{
		name 					= "energy_crystal_purple",
		ui_name					= "Purple Energy Crystal",
		categories				= "player",
		effect_desc				= "A pulsating energy field emanates from you, it destroys enemy projectiles and \nshoots slashbombs back and also inflicts the slash effect on enemies it touches. \nIf you take damage, you are inflicted with the slash effect.",
		effect_desc_short		= "An energy field emanates from you, destroying enemy projectiles and shooting slashbombs back.",
        precursor_types			= "magic_artifact,energy_shield",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			LoadGameEffectEntityTo( getPlayerEntity(), getCustomEffect("effect_pulsating_shield_projectile_scatter_slashbomb").path )
		end,
		ondrop_function			= function()
			local player_children = EntityGetAllChildren(getPlayerEntity())
			if player_children == nil then return end
			for i,child in ipairs(player_children) do
				if(EntityGetName(child) == "pulsating_shield_projectiles_scatter_slashbomb") then
					EntityKill(child)
					break
				end
			end
		end,
		ondamaged_function		= function(damage, message, entity_thats_responsible, is_fatal, projectile_thats_responsible )
			if(isRelicOwned("energy_crystal_purple") >= 1) then
				if(EntityHasTag(entity_thats_responsible, "enemy")) then
					LoadGameEffectEntityTo( GetUpdatedEntityID(), getCustomEffect("effect_slash").path )
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "mechanical_heart,ancient_root,shuriken",
	},
	{
		name 					= "energy_crystal_pink",
		ui_name					= "Pink Energy Crystal",
		categories				= "player",
		effect_desc				= "A pulsating energy field emanates from you, it destroys enemy projectiles and \nshoots polybombs back and also mutates enemies it touches. \nIf you take damage, you are mutated.",
		effect_desc_short		= "An energy field emanates from you, destroying enemy projectiles and shooting polybombs back.",
        precursor_types			= "magic_artifact,energy_shield",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			LoadGameEffectEntityTo( getPlayerEntity(), getCustomEffect("effect_pulsating_shield_projectile_scatter_polybomb").path )
		end,
		ondrop_function			= function()
			local player_children = EntityGetAllChildren(getPlayerEntity())
			if player_children == nil then return end
			for i,child in ipairs(player_children) do
				if(EntityGetName(child) == "pulsating_shield_projectiles_scatter_polybomb") then
					EntityKill(child)
					break
				end
			end
		end,
		ondamaged_function		= function(damage, message, entity_thats_responsible, is_fatal, projectile_thats_responsible )
			if(isRelicOwned("energy_crystal_pink") >= 1) then
				if(EntityHasTag(entity_thats_responsible, "enemy")) then
					LoadGameEffectEntityTo( GetUpdatedEntityID(), "data/entities/misc/effect_polymorph.xml" )
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "mechanical_heart,ancient_root,mutating_bead",
	},
	{
		name 					= "mutating_skull",
		ui_name					= "Mutating Skull",
		categories				= "companions",
		effect_desc				= "A friendly skull that shoots poly fireballs joins you.",
		effect_desc_short		= "A poly fireball shooting companion.",
        precursor_types			= "companion",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			attachCompaniontoEntity(getPlayerEntity(), "mutating_skull", "mortal")
		end,
		ondrop_function			= function()
			detachCompanionfromEntity(getPlayerEntity(), "mutating_skull")
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "flaming_skull,poly_gem",
		requires_relic			= "bone",
	},
	{
		name 					= "energy_crystal_flame_orange",
		ui_name					= "Flame Orange Energy Crystal",
		categories				= "player",
		effect_desc				= "A pulsating energy field emanates from you, it destroys enemy projectiles and \nshoots fireballs back and also burns enemies it touches. \nIf you take damage, you catch fire.",
		effect_desc_short		= "An energy field emanates from you, destroying enemy projectiles and shooting fireballs back.",
        precursor_types			= "magic_artifact,energy_shield",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			LoadGameEffectEntityTo( getPlayerEntity(), getCustomEffect("effect_pulsating_shield_projectile_scatter_fireball").path )
		end,
		ondrop_function			= function()
			local player_children = EntityGetAllChildren(getPlayerEntity())
			if player_children == nil then return end
			for i,child in ipairs(player_children) do
				if(EntityGetName(child) == "pulsating_shield_projectiles_scatter_fireball") then
					EntityKill(child)
					break
				end
			end
		end,
		ondamaged_function		= function(damage, message, entity_thats_responsible, is_fatal, projectile_thats_responsible )
			if(isRelicOwned("energy_crystal_flame_orange") >= 1) then
				if(EntityHasTag(entity_thats_responsible, "enemy")) then
					LoadGameEffectEntityTo( GetUpdatedEntityID(), "data/entities/misc/effect_on_fire.xml" )
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "mechanical_heart,ancient_root,ember_stone",
	},
	{
		name 					= "energy_crystal_radioactive_green",
		ui_name					= "Radioactive Green Energy Crystal",
		categories				= "player",
		effect_desc				= "A pulsating energy field emanates from you, it destroys enemy projectiles and \nshoots radioactive fireballs back and also burns enemies it touches. \nIf you take damage, you catch fire.",
		effect_desc_short		= "An energy field emanates from you, destroying enemy projectiles and shooting radioactive fireballs back.",
        precursor_types			= "magic_artifact,energy_shield",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			LoadGameEffectEntityTo( getPlayerEntity(), getCustomEffect("effect_pulsating_shield_projectile_scatter_fireball_radioactive").path )
		end,
		ondrop_function			= function()
			local player_children = EntityGetAllChildren(getPlayerEntity())
			if player_children == nil then return end
			for i,child in ipairs(player_children) do
				if(EntityGetName(child) == "pulsating_shield_projectiles_scatter_fireball_radioactive") then
					EntityKill(child)
					break
				end
			end
		end,
		ondamaged_function		= function(damage, message, entity_thats_responsible, is_fatal, projectile_thats_responsible )
			if(isRelicOwned("energy_crystal_radioactive_green") >= 1) then
				if(EntityHasTag(entity_thats_responsible, "enemy")) then
					LoadGameEffectEntityTo( GetUpdatedEntityID(), "data/entities/misc/effect_on_fire.xml" )
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "mechanical_heart,ancient_root,uranium_ingot",
	},
	{
		name 					= "energy_crystal_cyan",
		ui_name					= "Cyan Energy Crystal",
		categories				= "player",
		effect_desc				= "A pulsating energy field emanates from you, it destroys enemy projectiles and \nshoots iceballs back and also freezes enemies it touches. \nIf you take damage, you are frozen.",
		effect_desc_short		= "An energy field emanates from you, destroying enemy projectiles and shooting iceballs back.",
        precursor_types			= "magic_artifact,energy_shield",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			LoadGameEffectEntityTo( getPlayerEntity(), getCustomEffect("effect_pulsating_shield_projectile_scatter_iceball").path )
		end,
		ondrop_function			= function()
			local player_children = EntityGetAllChildren(getPlayerEntity())
			if player_children == nil then return end
			for i,child in ipairs(player_children) do
				if(EntityGetName(child) == "pulsating_shield_projectiles_scatter_iceball") then
					EntityKill(child)
					break
				end
			end
		end,
		ondamaged_function		= function(damage, message, entity_thats_responsible, is_fatal, projectile_thats_responsible )
			if(isRelicOwned("energy_crystal_cyan") >= 1) then
				if(EntityHasTag(entity_thats_responsible, "enemy")) then
					LoadGameEffectEntityTo( GetUpdatedEntityID(), "data/entities/misc/effect_frozen.xml" )
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "mechanical_heart,ancient_root,ice_artifact",
	},
	{
		name 					= "energy_crystal_fallout_red",
		ui_name					= "Fallout Red Energy Crystal",
		categories				= "player",
		effect_desc				= "A pulsating energy field emanates from you, it destroys enemy projectiles and \nshoots fallout fireballs back and also burns enemies it touches. \nIf you take damage, you catch fire.",
		effect_desc_short		= "An energy field emanates from you, destroying enemy projectiles and shooting fallout fireballs back.",
        precursor_types			= "magic_artifact,energy_shield",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			LoadGameEffectEntityTo( getPlayerEntity(), getCustomEffect("effect_pulsating_shield_projectile_scatter_fireball_fallout").path )
		end,
		ondrop_function			= function()
			local player_children = EntityGetAllChildren(getPlayerEntity())
			if player_children == nil then return end
			for i,child in ipairs(player_children) do
				if(EntityGetName(child) == "pulsating_shield_projectiles_scatter_fireball_fallout") then
					EntityKill(child)
					break
				end
			end
		end,
		ondamaged_function		= function(damage, message, entity_thats_responsible, is_fatal, projectile_thats_responsible )
			if(isRelicOwned("energy_crystal_fallout_red") >= 1) then
				if(EntityHasTag(entity_thats_responsible, "enemy")) then
					LoadGameEffectEntityTo( GetUpdatedEntityID(), "data/entities/misc/effect_on_fire.xml" )
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "mechanical_heart,ancient_root,plutonium_ingot",
	},
	{
		name 					= "anomalous_crystal",
		ui_name					= "Anomalous Crystal",
		categories				= "projectiles",
		effect_desc				= "Spells that you cast have a 30% chance of shooting poly fireballs downwards along their travel path.",
		effect_desc_short		= "Your spells cast downward poly fireballs.",
        precursor_types			= "effect_crystal",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%, and when it triggers, fireballs are shot more often.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("anomalous_crystal") >= 1) then
				if(not isPhysicsProjectile(projectile_entity_id)) then
					SetRandomSeed( GameGetFrameNum(), math.random(1,50))
					local chance = Random(1,100)
					local odds = 30 + (5 * getRelicLevel("anomalous_crystal"))
					if(chance >= 1 and chance <= odds) then
						addNewInternalVariable(projectile_entity_id, "projectile_shot_down_definition", "value_string", "mods/ArcaneReliquary/files/entity_definitions/custom_projectiles/fireball_poly.xml")
						addNewInternalVariable(projectile_entity_id, "down_vel_min", "value_int", 165)
						addNewInternalVariable(projectile_entity_id, "down_vel_max", "value_int", 185)
						addNewInternalVariable(projectile_entity_id, "max_projectiles_shot", "value_int", 10)
						addNewLuaCallback(projectile_entity_id, "overtime", "mods/ArcaneReliquary/files/scripts/projectiles/launch_projectile_down.lua", 25 - (2 * getRelicLevel("anomalous_crystal")), false, "launch_projectile_down")
					end
				end
			end
		end,
	},
	{
		name 					= "energy_crystal_pink_dark",
		ui_name					= "Dark Pink Energy Crystal",
		categories				= "player",
		effect_desc				= "A pulsating energy field emanates from you, it destroys enemy projectiles and \nshoots poly fireballs back and also mutates enemies it touches. \nIf you take damage, you are mutated.",
		effect_desc_short		= "An energy field emanates from you, destroying enemy projectiles and shooting poly fireballs back.",
        precursor_types			= "magic_artifact,energy_shield",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			LoadGameEffectEntityTo( getPlayerEntity(), getCustomEffect("effect_pulsating_shield_projectile_scatter_fireball_poly").path )
		end,
		ondrop_function			= function()
			local player_children = EntityGetAllChildren(getPlayerEntity())
			if player_children == nil then return end
			for i,child in ipairs(player_children) do
				if(EntityGetName(child) == "pulsating_shield_projectiles_scatter_fireball_poly") then
					EntityKill(child)
					break
				end
			end
		end,
		ondamaged_function		= function(damage, message, entity_thats_responsible, is_fatal, projectile_thats_responsible )
			if(isRelicOwned("energy_crystal_pink_dark") >= 1) then
				if(EntityHasTag(entity_thats_responsible, "enemy")) then
					LoadGameEffectEntityTo( GetUpdatedEntityID(), "data/entities/misc/effect_polymorph.xml" )
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "mechanical_heart,ancient_root,anomalous_crystal",
	},
	{
		name 					= "energy_crystal_water_blue",
		ui_name					= "Water Blue Energy Crystal",
		categories				= "player",
		effect_desc				= "A pulsating energy field emanates from you, it destroys enemy projectiles and \nshoots water fireballs back and also burns enemies it touches. \nIf you take damage, you catch fire.",
		effect_desc_short		= "An energy field emanates from you, destroying enemy projectiles and shooting water fireballs back.",
        precursor_types			= "magic_artifact,energy_shield",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			LoadGameEffectEntityTo( getPlayerEntity(), getCustomEffect("effect_pulsating_shield_projectile_scatter_fireball_water").path )
		end,
		ondrop_function			= function()
			local player_children = EntityGetAllChildren(getPlayerEntity())
			if player_children == nil then return end
			for i,child in ipairs(player_children) do
				if(EntityGetName(child) == "pulsating_shield_projectiles_scatter_fireball_water") then
					EntityKill(child)
					break
				end
			end
		end,
		ondamaged_function		= function(damage, message, entity_thats_responsible, is_fatal, projectile_thats_responsible )
			if(isRelicOwned("energy_crystal_water_blue") >= 1) then
				if(EntityHasTag(entity_thats_responsible, "enemy")) then
					LoadGameEffectEntityTo( GetUpdatedEntityID(), "data/entities/misc/effect_on_fire.xml" )
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "mechanical_heart,ancient_root,aquatic_crystal",
	},
	{
		name 					= "energy_crystal_acid_green",
		ui_name					= "Acid Green Energy Crystal",
		categories				= "player",
		effect_desc				= "A pulsating energy field emanates from you, it destroys enemy projectiles and \nshoots acid fireballs back and also burns enemies it touches. \nIf you take damage, you catch fire.",
		effect_desc_short		= "An energy field emanates from you, destroying enemy projectiles and shooting acid fireballs back.",
        precursor_types			= "magic_artifact,energy_shield",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			LoadGameEffectEntityTo( getPlayerEntity(), getCustomEffect("effect_pulsating_shield_projectile_scatter_fireball_acid").path )
		end,
		ondrop_function			= function()
			local player_children = EntityGetAllChildren(getPlayerEntity())
			if player_children == nil then return end
			for i,child in ipairs(player_children) do
				if(EntityGetName(child) == "pulsating_shield_projectiles_scatter_fireball_acid") then
					EntityKill(child)
					break
				end
			end
		end,
		ondamaged_function		= function(damage, message, entity_thats_responsible, is_fatal, projectile_thats_responsible )
			if(isRelicOwned("energy_crystal_acid_green") >= 1) then
				if(EntityHasTag(entity_thats_responsible, "enemy")) then
					LoadGameEffectEntityTo( GetUpdatedEntityID(), "data/entities/misc/effect_on_fire.xml" )
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "mechanical_heart,ancient_root,bitter_crystal",
	},
	{
		name 					= "energy_crystal_venom_purple",
		ui_name					= "Venom Purple Energy Crystal",
		categories				= "player",
		effect_desc				= "A pulsating energy field emanates from you, it destroys enemy projectiles and \nshoots poison fireballs back and also burns enemies it touches. \nIf you take damage, you catch fire.",
		effect_desc_short		= "An energy field emanates from you, destroying enemy projectiles and shooting poison fireballs back.",
        precursor_types			= "magic_artifact,energy_shield",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			LoadGameEffectEntityTo( getPlayerEntity(), getCustomEffect("effect_pulsating_shield_projectile_scatter_fireball_poison").path )
		end,
		ondrop_function			= function()
			local player_children = EntityGetAllChildren(getPlayerEntity())
			if player_children == nil then return end
			for i,child in ipairs(player_children) do
				if(EntityGetName(child) == "pulsating_shield_projectiles_scatter_fireball_poison") then
					EntityKill(child)
					break
				end
			end
		end,
		ondamaged_function		= function(damage, message, entity_thats_responsible, is_fatal, projectile_thats_responsible )
			if(isRelicOwned("energy_crystal_venom_purple") >= 1) then
				if(EntityHasTag(entity_thats_responsible, "enemy")) then
					LoadGameEffectEntityTo( GetUpdatedEntityID(), "data/entities/misc/effect_on_fire.xml" )
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "mechanical_heart,ancient_root,venomous_crystal",
	},
	{
		name 					= "bronze_rat",
		ui_name					= "Bronze Rat",
		categories				= "projectiles,minions",
		effect_desc				= "Your spells have a 25% chance of being transformed into friendly nuclear bomb rats.",
		effect_desc_short		= "Your spells summon friendly nuclear bomb rats.",
        precursor_types			= "magic_artifact",
		upgrade_effect_desc		= "For each upgrade level, spawn chance increases by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("bronze_rat") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 25 + (5 * getRelicLevel("bronze_rat"))
				if(chance >= 1 and chance <= odds) then
					addNewLuaCallback(projectile_entity_id, "overtime", "mods/ArcaneReliquary/files/scripts/projectiles/destroy_and_spawn_nuclear_rat.lua", 5, false, "destroy_and_spawn_nuclear_rat")
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "stone_rat,leaking_nuclear_core",
	},
	{
		name 					= "bronze_bat",
		ui_name					= "Bronze Bat",
		categories				= "projectiles,minions",
		effect_desc				= "Your spells have a 25% chance of being transformed into friendly nuclear bomb bats.",
		effect_desc_short		= "Your spells summon friendly nuclear bomb bats.",
        precursor_types			= "magic_artifact",
		upgrade_effect_desc		= "For each upgrade level, spawn chance increases by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("bronze_bat") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 25 + (5 * getRelicLevel("bronze_bat"))
				if(chance >= 1 and chance <= odds) then
					addNewLuaCallback(projectile_entity_id, "overtime", "mods/ArcaneReliquary/files/scripts/projectiles/destroy_and_spawn_nuclear_bat.lua", 5, false, "destroy_and_spawn_nuclear_bat")
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "stone_bat,leaking_nuclear_core",
	},
	{
		name 					= "scarab_shell_obsidian",
		ui_name					= "Obsidian Scarab Shell",
		categories				= "enemies,player",
		effect_desc				= "Upon taking damage from an enemy, black holes are summoned on other enemies nearby.",
		effect_desc_short		= "Upon taking damage from an enemy, black holes are summoned on other enemies nearby.",
        precursor_types			= "magic_artifact",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		ondamaged_function		= function(damage, message, entity_thats_responsible, is_fatal, projectile_thats_responsible )
			if(isRelicOwned("scarab_shell_obsidian") >= 1) then
				if(EntityHasTag(entity_thats_responsible, "enemy")) then
					local player_x, player_y = EntityGetTransform(GetUpdatedEntityID())
					local enemies = EntityGetInRadiusWithTag( player_x, player_y, 200, "enemy" )
					for i, enemy_id in ipairs(enemies) do
						if(enemy_id ~= entity_thats_responsible) then
							local enemy_x, enemy_y = EntityGetTransform(enemy_id)
							EntityLoad("mods/ArcaneReliquary/files/entity_definitions/load_on_explosion/blackhole_custom.xml", enemy_x, enemy_y - 15)
						end
					end
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "scarab_shell,high_density_stone",
	},
	{
		name 					= "unicorn_toy",
		ui_name					= "Unicorn Toy",
		categories				= "projectiles",
		effect_desc				= "Your spells have a 30% chance of breaking down into various effect bombs.",
		effect_desc_short		= "Your spells break down into effect bombs.",
        precursor_types			= "magic_artifact",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("unicorn_toy") >= 1) then
				if(not isPhysicsProjectile(projectile_entity_id)) then
					SetRandomSeed( GameGetFrameNum(), math.random(1,50))
					local chance = Random(1,100)
					local odds = 30 + (5 * getRelicLevel("unicorn_toy"))
					if(chance >= 1 and chance <= odds) then
						addBuffVisualIndicator_Projectile(projectile_entity_id, "material_rainbow")
						addNewLuaCallback(projectile_entity_id, "overtime", "mods/ArcaneReliquary/files/scripts/projectiles/launch_effect_bomb_projectiles.lua", 4, true, "launch_effect_bomb_projectiles")
					end
				end
			end
		end
	},
	{
		name 					= "spaceship_toy",
		ui_name					= "Spaceship Toy",
		categories				= "projectiles",
		effect_desc				= "Spells you cast have a 25% chance of inducing enemies to shoot blackhole bombs at random directions.",
		effect_desc_short		= "Spells you cast have a chance to induce enemies to randomly shoot blackhole bombs.",
        precursor_types			= "gadget",
		upgrade_effect_desc		= "For each upgrade level, effect chance is increased by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("spaceship_toy") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 25 + (5 * getRelicLevel("spaceship_toy"))
				if(chance >= 1 and chance <= odds) then
					attachLoadEffectOntoEntitiestoProjectile(projectile_entity_id, getCustomEffect("effect_shoot_projectile_random_blackholebomb").path, "spark_purple_bright")
				end
			end
		end,
	},
	{
		name 					= "projectile_transmutation_device_battery",
		ui_name					= "Projectile Transmutation Device Battery",
		categories				= "projectiles",
		effect_desc				= "Spells you cast have a 5% chance of transforming enemies they hit into various objects.",
		effect_desc_short		= "Spells you cast have a chance of transforming enemies they hit into various objects.",
        precursor_types			= "gadget",
		upgrade_effect_desc		= "For each upgrade level, effect chance is increased by 5%.",
		is_upgradable 			= true,
		onenemydamaged_function	= function(damage, message, entity_thats_responsible, is_fatal, projectile_thats_responsible )
			if(isRelicOwned("projectile_transmutation_device_battery") >= 1) then
				if(EntityHasTag(entity_thats_responsible, "player_unit")) then
					SetRandomSeed( GameGetFrameNum(), math.random(1,50))
					local chance = Random(1,100)
					local odds = 25 + (5 * getRelicLevel("projectile_transmutation_device_battery"))
					if(chance >= 1 and chance <= odds) then
						local objects_pool = {
							"data/entities/props/physics_barrel_oil.xml",
							"data/entities/props/physics_box_explosive.xml",
							"data/entities/props/physics_barrel_radioactive.xml",
							"data/entities/props/physics_propane_tank.xml",
							"data/entities/props/physics_barrel_water.xml",
							"data/entities/props/physics_minecart.xml",
							"data/entities/props/physics_skateboard.xml",
							"data/entities/props/physics_brewing_stand.xml",
							"data/entities/props/physics_crate.xml",
						}
						local entity_id = GetUpdatedEntityID()
						local enemy_x, enemy_y = EntityGetTransform(entity_id)
						killEntitywithCustomMessage(entity_id, "Transformed into object")
						doArtificialDeathLogic(entity_id, 1, false, getRelicPrecursorMaterialsList(), getPrecursorItemsBaseDir())
						
						EntityLoad(objects_pool[Random(1,#objects_pool)] , enemy_x, enemy_y)
					end
				end
			end
		end,
	},
	{
		name 					= "projectile_transmutation_device_module_propane_tank",
		ui_name					= "Projectile Transmutation Device Propane Tank Module",
		categories				= "player",
		effect_desc				= "A pulsating energy field is projected around you, converting enemies and their projectiles into propane tanks.",
		effect_desc_short		= "A pulsating energy field is projected around you, converting enemies and their projectiles into propane tanks.",
        precursor_types			= "gadget,energy_shield",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			LoadGameEffectEntityTo( getPlayerEntity(), getCustomEffect("effect_pulsating_shield_projectile_transmute_object_propane_tank").path )
		end,
		ondrop_function			= function()
			local player_children = EntityGetAllChildren(getPlayerEntity())
			if player_children == nil then return end
			for i,child in ipairs(player_children) do
				if(EntityGetName(child) == "pulsating_shield_projectile_transmute_object_propane_tank") then
					EntityKill(child)
					break
				end
			end
		end,
	},
	{
		name 					= "projectile_transmutation_device_module_oil_barrel",
		ui_name					= "Projectile Transmutation Device Oil Barrel Module",
		categories				= "player",
		effect_desc				= "A pulsating energy field is projected around you, converting enemies and their projectiles into oil barrels.",
		effect_desc_short		= "A pulsating energy field is projected around you, converting enemies and their projectiles into oil barrels.",
        precursor_types			= "gadget,energy_shield",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			LoadGameEffectEntityTo( getPlayerEntity(), getCustomEffect("effect_pulsating_shield_projectile_transmute_object_oil_barrel").path )
		end,
		ondrop_function			= function()
			local player_children = EntityGetAllChildren(getPlayerEntity())
			if player_children == nil then return end
			for i,child in ipairs(player_children) do
				if(EntityGetName(child) == "pulsating_shield_projectile_transmute_object_oil_barrel") then
					EntityKill(child)
					break
				end
			end
		end,
	},
	{
		name 					= "projectile_transmutation_device_module_tnt_box",
		ui_name					= "Projectile Transmutation Device Explosive Box Module",
		categories				= "player",
		effect_desc				= "A pulsating energy field is projected around you, converting enemies and their projectiles into explosive boxes.",
		effect_desc_short		= "A pulsating energy field is projected around you, converting enemies and their projectiles into explosive boxes.",
        precursor_types			= "gadget,energy_shield",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			LoadGameEffectEntityTo( getPlayerEntity(), getCustomEffect("effect_pulsating_shield_projectile_transmute_object_tnt_box").path )
		end,
		ondrop_function			= function()
			local player_children = EntityGetAllChildren(getPlayerEntity())
			if player_children == nil then return end
			for i,child in ipairs(player_children) do
				if(EntityGetName(child) == "pulsating_shield_projectile_transmute_object_tnt_box") then
					EntityKill(child)
					break
				end
			end
		end,
	},
	{
		name 					= "projectile_transmutation_device_module_water_barrel",
		ui_name					= "Projectile Transmutation Device Water Barrel Module",
		categories				= "player",
		effect_desc				= "A pulsating energy field is projected around you, converting enemies and their projectiles into water barrels.",
		effect_desc_short		= "A pulsating energy field is projected around you, converting enemies and their projectiles into water barrels.",
        precursor_types			= "gadget,energy_shield",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			LoadGameEffectEntityTo( getPlayerEntity(), getCustomEffect("effect_pulsating_shield_projectile_transmute_object_water_barrel").path )
		end,
		ondrop_function			= function()
			local player_children = EntityGetAllChildren(getPlayerEntity())
			if player_children == nil then return end
			for i,child in ipairs(player_children) do
				if(EntityGetName(child) == "pulsating_shield_projectile_transmute_object_water_barrel") then
					EntityKill(child)
					break
				end
			end
		end,
	},
	{
		name 					= "projectile_transmutation_device_module_radioactive_barrel",
		ui_name					= "Projectile Transmutation Device Toxic Barrel Module",
		categories				= "player",
		effect_desc				= "A pulsating energy field is projected around you, converting enemies and their projectiles into toxic barrels.",
		effect_desc_short		= "A pulsating energy field is projected around you, converting enemies and their projectiles into toxic barrels.",
        precursor_types			= "gadget,energy_shield",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			LoadGameEffectEntityTo( getPlayerEntity(), getCustomEffect("effect_pulsating_shield_projectile_transmute_object_radioactive_barrel").path )
		end,
		ondrop_function			= function()
			local player_children = EntityGetAllChildren(getPlayerEntity())
			if player_children == nil then return end
			for i,child in ipairs(player_children) do
				if(EntityGetName(child) == "pulsating_shield_projectile_transmute_object_radioactive_barrel") then
					EntityKill(child)
					break
				end
			end
		end,
	},
	{
		name 					= "mininuke",
		ui_name					= "Mininuke",
		categories				= "projectiles",
		effect_desc				= "Your spells have a 30% chance of exploding into a \nbarrage of radioactive fireballs when they explode or hit something.",
		effect_desc_short		= "Your spells explode into radioactive fireballs on impact.",
        precursor_types			= "gadget",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("mininuke") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 30 + (5 * getRelicLevel("mininuke"))
				if(chance >= 1 and chance <= odds) then	
					addBuffVisualIndicator_Projectile(projectile_entity_id, "radioactive_gas")
					attachLoadEntityOnExplosiontoProjectile(projectile_entity_id, getLoadOnExplosion("launch_circular_projectiles_fireballs_radioactive").path)
				end
			end
		end,
		onpickup_function		= function()
			addNewSimplifiedInternalVariable(getPlayerEntity(), "mininuke")
		end,
		ondrop_function			= function()
			removeSimplifiedInternalVariable(getPlayerEntity(), "mininuke")
		end,
	},
	{
		name 					= "thunder_cloud",
		ui_name					= "Thunder Cloud",
		categories				= "projectiles",
		effect_desc				= "Your spells have a 30% chance of exploding into a \nbarrage of thunderballs when they explode or hit something.",
		effect_desc_short		= "Your spells explode into thunderballs on impact.",
        precursor_types			= "gadget",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("thunder_cloud") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 30 + (5 * getRelicLevel("thunder_cloud"))
				if(chance >= 1 and chance <= odds) then	
					addBuffVisualIndicator_Projectile(projectile_entity_id, "plasma_fading")
					attachLoadEntityOnExplosiontoProjectile(projectile_entity_id, getLoadOnExplosion("launch_circular_projectiles_thunderballs").path)
				end
			end
		end,
		onpickup_function		= function()
			addNewSimplifiedInternalVariable(getPlayerEntity(), "thunder_cloud")
		end,
		ondrop_function			= function()
			removeSimplifiedInternalVariable(getPlayerEntity(), "thunder_cloud")
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "little_cloud,tesla_coil",
	},
	{
		name 					= "thunder_cloud_red",
		ui_name					= "Red Thunder Cloud",
		categories				= "projectiles",
		effect_desc				= "Your spells have a 30% chance of exploding into a \nbarrage of red thunderballs when they explode or hit something.",
		effect_desc_short		= "Your spells explode into red thunderballs on impact.",
        precursor_types			= "gadget",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("thunder_cloud_red") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 30 + (5 * getRelicLevel("thunder_cloud_red"))
				if(chance >= 1 and chance <= odds) then	
					addBuffVisualIndicator_Projectile(projectile_entity_id, "spark_red")
					attachLoadEntityOnExplosiontoProjectile(projectile_entity_id, getLoadOnExplosion("launch_circular_projectiles_thunderballs_red").path)
				end
			end
		end,
		onpickup_function		= function()
			addNewSimplifiedInternalVariable(getPlayerEntity(), "thunder_cloud_red")
		end,
		ondrop_function			= function()
			removeSimplifiedInternalVariable(getPlayerEntity(), "thunder_cloud_red")
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "thunder_cloud,mystic_thunder_necklace",
	},
	{
		name 					= "energy_crystal_thunder_blue",
		ui_name					= "Thunder Blue Energy Crystal",
		categories				= "player",
		effect_desc				= "A pulsating energy field emanates from you, it destroys enemy projectiles and \nshoots thunderballs back and also electrocutes enemies it touches. \nIf you take damage, you are electrocuted.",
		effect_desc_short		= "An energy field emanates from you, destroying enemy projectiles and shooting thunderballs back.",
        precursor_types			= "magic_artifact,energy_shield",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			LoadGameEffectEntityTo( getPlayerEntity(), getCustomEffect("effect_pulsating_shield_projectile_scatter_thunderball").path )
		end,
		ondrop_function			= function()
			local player_children = EntityGetAllChildren(getPlayerEntity())
			if player_children == nil then return end
			for i,child in ipairs(player_children) do
				if(EntityGetName(child) == "pulsating_shield_projectiles_scatter_thunderball") then
					EntityKill(child)
					break
				end
			end
		end,
		ondamaged_function		= function(damage, message, entity_thats_responsible, is_fatal, projectile_thats_responsible )
			if(isRelicOwned("energy_crystal_thunder_blue") >= 1) then
				if(EntityHasTag(entity_thats_responsible, "enemy")) then
					LoadGameEffectEntityTo( GetUpdatedEntityID(), "data/entities/misc/effect_electricity.xml" )
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "mechanical_heart,ancient_root,tesla_coil",
	},
	{
		name 					= "energy_crystal_thunder_red",
		ui_name					= "Thunder Red Energy Crystal",
		categories				= "player",
		effect_desc				= "A pulsating energy field emanates from you, it destroys enemy projectiles and \nshoots red thunderballs back and also electrocutes enemies it touches. \nIf you take damage, you are electrocuted.",
		effect_desc_short		= "An energy field emanates from you, destroying enemy projectiles and shooting red thunderballs back.",
        precursor_types			= "magic_artifact,energy_shield",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			LoadGameEffectEntityTo( getPlayerEntity(), getCustomEffect("effect_pulsating_shield_projectile_scatter_thunderball_red").path )
		end,
		ondrop_function			= function()
			local player_children = EntityGetAllChildren(getPlayerEntity())
			if player_children == nil then return end
			for i,child in ipairs(player_children) do
				if(EntityGetName(child) == "pulsating_shield_projectiles_scatter_thunderball_red") then
					EntityKill(child)
					break
				end
			end
		end,
		ondamaged_function		= function(damage, message, entity_thats_responsible, is_fatal, projectile_thats_responsible )
			if(isRelicOwned("energy_crystal_thunder_red") >= 1) then
				if(EntityHasTag(entity_thats_responsible, "enemy")) then
					LoadGameEffectEntityTo( GetUpdatedEntityID(), "data/entities/misc/effect_electricity.xml" )
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "mechanical_heart,ancient_root,mystic_thunder_necklace",
	},
	{
		name 					= "energy_crystal_yellow",
		ui_name					= "Yellow Energy Crystal",
		categories				= "player",
		effect_desc				= "A pulsating energy field emanates from you, it destroys enemy projectiles and \nshoots firebombs back and also burns enemies it touches. \nIf you take damage, you catch fire.",
		effect_desc_short		= "An energy field emanates from you, destroying enemy projectiles and shooting firebombs back.",
        precursor_types			= "magic_artifact,energy_shield",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			LoadGameEffectEntityTo( getPlayerEntity(), getCustomEffect("effect_pulsating_shield_projectile_scatter_firebomb").path )
		end,
		ondrop_function			= function()
			local player_children = EntityGetAllChildren(getPlayerEntity())
			if player_children == nil then return end
			for i,child in ipairs(player_children) do
				if(EntityGetName(child) == "pulsating_shield_projectiles_scatter_firebomb") then
					EntityKill(child)
					break
				end
			end
		end,
		ondamaged_function		= function(damage, message, entity_thats_responsible, is_fatal, projectile_thats_responsible )
			if(isRelicOwned("energy_crystal_yellow") >= 1) then
				if(EntityHasTag(entity_thats_responsible, "enemy")) then
					LoadGameEffectEntityTo( GetUpdatedEntityID(), "data/entities/misc/effect_on_fire.xml" )
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "mechanical_heart,ancient_root,solar_artifact",
	},
	{
		name 					= "energy_crystal_light_red",
		ui_name					= "Light Red Energy Crystal",
		categories				= "player",
		effect_desc				= "A pulsating energy field emanates from you, it destroys enemy projectiles and \nshoots laser beams back and also induces enemies it touches into shooting laser beams randomly. \nIf you take damage, you are induced into shooting laser beams randomly.",
		effect_desc_short		= "An energy field emanates from you, destroying enemy projectiles and shooting laser beams back.",
        precursor_types			= "magic_artifact,energy_shield",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			LoadGameEffectEntityTo( getPlayerEntity(), getCustomEffect("effect_pulsating_shield_projectile_scatter_laser").path )
		end,
		ondrop_function			= function()
			local player_children = EntityGetAllChildren(getPlayerEntity())
			if player_children == nil then return end
			for i,child in ipairs(player_children) do
				if(EntityGetName(child) == "pulsating_shield_projectiles_scatter_laser") then
					EntityKill(child)
					break
				end
			end
		end,
		ondamaged_function		= function(damage, message, entity_thats_responsible, is_fatal, projectile_thats_responsible )
			if(isRelicOwned("energy_crystal_light_red") >= 1) then
				if(EntityHasTag(entity_thats_responsible, "enemy")) then
					LoadGameEffectEntityTo( GetUpdatedEntityID(), getCustomEffect("effect_shoot_projectile_random_laser").path )
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "mechanical_heart,ancient_root,laser_pointer",
	},
	{
		name 					= "energy_crystal_gray",
		ui_name					= "Gray Energy Crystal",
		categories				= "player",
		effect_desc				= "A pulsating energy field emanates from you, it destroys enemy projectiles and \nshoots rockets back and also induces enemies it touches into shooting rockets randomly. \nIf you take damage, you are induced into shooting rockets randomly.",
		effect_desc_short		= "An energy field emanates from you, destroying enemy projectiles and shooting rockets back.",
        precursor_types			= "magic_artifact,energy_shield",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			LoadGameEffectEntityTo( getPlayerEntity(), getCustomEffect("effect_pulsating_shield_projectile_scatter_rocket").path )
		end,
		ondrop_function			= function()
			local player_children = EntityGetAllChildren(getPlayerEntity())
			if player_children == nil then return end
			for i,child in ipairs(player_children) do
				if(EntityGetName(child) == "pulsating_shield_projectiles_scatter_rocket") then
					EntityKill(child)
					break
				end
			end
		end,
		ondamaged_function		= function(damage, message, entity_thats_responsible, is_fatal, projectile_thats_responsible )
			if(isRelicOwned("energy_crystal_gray") >= 1) then
				if(EntityHasTag(entity_thats_responsible, "enemy")) then
					LoadGameEffectEntityTo( GetUpdatedEntityID(), getCustomEffect("effect_shoot_projectile_random_rocket").path )
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "mechanical_heart,ancient_root,toy_bomber",
	},
	{
		name 					= "energy_crystal_black",
		ui_name					= "Black Energy Crystal",
		categories				= "player",
		effect_desc				= "A pulsating energy field emanates from you, it destroys enemy projectiles and \nshoots bomb laying bombs back and also induces enemies it touches into spawning bombs. \nIf you take damage, you are induced into spawning bombs.",
		effect_desc_short		= "An energy field emanates from you, destroying enemy projectiles and shooting bomb laying bombs back.",
        precursor_types			= "magic_artifact,energy_shield",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			LoadGameEffectEntityTo( getPlayerEntity(), getCustomEffect("effect_pulsating_shield_projectile_scatter_bomblayingbomb").path )
		end,
		ondrop_function			= function()
			local player_children = EntityGetAllChildren(getPlayerEntity())
			if player_children == nil then return end
			for i,child in ipairs(player_children) do
				if(EntityGetName(child) == "pulsating_shield_projectiles_scatter_bomblayingbomb") then
					EntityKill(child)
					break
				end
			end
		end,
		ondamaged_function		= function(damage, message, entity_thats_responsible, is_fatal, projectile_thats_responsible )
			if(isRelicOwned("energy_crystal_black") >= 1) then
				if(EntityHasTag(entity_thats_responsible, "enemy")) then
					LoadGameEffectEntityTo( GetUpdatedEntityID(), getCustomEffect("effect_spawn_object_bomb").path )
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "mechanical_heart,ancient_root,bomb_charm",
	},
	{
		name 					= "energy_crystal_light_gray",
		ui_name					= "Light Gray Energy Crystal",
		categories				= "player",
		effect_desc				= "A pulsating energy field emanates from you, it destroys enemy projectiles and \nshoots magnetobombs back and also inflicts enemies it touches with the attraction effect. \nIf you take damage, you are inflicted with the attraction effect.",
		effect_desc_short		= "An energy field emanates from you, destroying enemy projectiles and shooting magnetobombs back.",
        precursor_types			= "magic_artifact,energy_shield",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			LoadGameEffectEntityTo( getPlayerEntity(), getCustomEffect("effect_pulsating_shield_projectile_scatter_magnetobomb").path )
		end,
		ondrop_function			= function()
			local player_children = EntityGetAllChildren(getPlayerEntity())
			if player_children == nil then return end
			for i,child in ipairs(player_children) do
				if(EntityGetName(child) == "pulsating_shield_projectiles_scatter_magnetobomb") then
					EntityKill(child)
					break
				end
			end
		end,
		ondamaged_function		= function(damage, message, entity_thats_responsible, is_fatal, projectile_thats_responsible )
			if(isRelicOwned("energy_crystal_light_gray") >= 1) then
				if(EntityHasTag(entity_thats_responsible, "enemy")) then
					LoadGameEffectEntityTo( GetUpdatedEntityID(), getCustomEffect("effect_attraction").path )
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "mechanical_heart,ancient_root,magnet",
	},
	{
		name 					= "petrifying_scepter",
		ui_name					= "Petrifying Scepter",
		categories				= "projectiles",
		effect_desc				= "Spells that you cast have a 30% chance of shooting petrifying bombs downwards along their travel path.",
		effect_desc_short		= "Your spells cast downward petrifying bombs.",
        precursor_types			= "magic_artifact",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%, and when it triggers, fireballs are shot more often.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("petrifying_scepter") >= 1) then
				if(not isPhysicsProjectile(projectile_entity_id)) then
					SetRandomSeed( GameGetFrameNum(), math.random(1,50))
					local chance = Random(1,100)
					local odds = 30 + (5 * getRelicLevel("petrifying_scepter"))
					if(chance >= 1 and chance <= odds) then
						addNewInternalVariable(projectile_entity_id, "projectile_shot_down_definition", "value_string", "mods/ArcaneReliquary/files/entity_definitions/custom_projectiles/petrifybomb.xml")
						addNewInternalVariable(projectile_entity_id, "down_vel_min", "value_int", 250)
						addNewInternalVariable(projectile_entity_id, "down_vel_max", "value_int", 280)
						addNewInternalVariable(projectile_entity_id, "max_projectiles_shot", "value_int", 15)
						addNewLuaCallback(projectile_entity_id, "overtime", "mods/ArcaneReliquary/files/scripts/projectiles/launch_projectile_down.lua", 22 - (2 * getRelicLevel("petrifying_scepter")), false, "launch_projectile_down")
					end
				end
			end
		end,
	},
	{
		name 					= "grim_scythe",
		ui_name					= "Grim Scythe",
		categories				= "projectiles",
		effect_desc				= "Once damaged by enemies, you have a 30% chance of shooting instakill bombs at random directions.",
		effect_desc_short		= "Once damaged by enemies, you release instakill bombs randomly.",
        precursor_types			= "magic_artifact",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		ondamaged_function		= function(damage, message, entity_thats_responsible, is_fatal, projectile_thats_responsible )
			if(isRelicOwned("grim_scythe") >= 1) then
				if(EntityHasTag(entity_thats_responsible, "enemy")) then
					SetRandomSeed( GameGetFrameNum(), math.random(1,50))
					local chance = Random(1,100)
					local odds = 30 + (5 * getRelicLevel("grim_scythe"))
					if(chance >= 1 and chance <= odds) then
						LoadGameEffectEntityTo( GetUpdatedEntityID(), getCustomEffect("effect_shoot_projectile_random_instakillbomb").path )
					end
				end
			end
		end,
	},
	{
		name 					= "frost_dagger",
		ui_name					= "Frost Dagger",
		categories				= "projectiles",
		effect_desc				= "Your spells have a 30% chance of exploding into a barrage of freeze solid bombs when they hit something.",
		effect_desc_short		= "Your spells explode into freeze solid bombs when they hit something.",
        precursor_types			= "magic_artifact",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("frost_dagger") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 30 + (5 * getRelicLevel("frost_dagger"))
				if(chance >= 1 and chance <= odds) then	
					addBuffVisualIndicator_Projectile(projectile_entity_id, "plasma_fading")
					attachLoadEntityOnExplosiontoProjectile(projectile_entity_id, getLoadOnExplosion("launch_circular_projectiles_freezesolidbombs").path)
				end
			end
		end,
	},
	{
		name 					= "triad_of_death",
		ui_name					= "Triad of Death",
		categories				= "projectiles",
		effect_desc				= "Your spells have a 30% chance of breaking down into instakill, freeze solid and petrify bombs.",
		effect_desc_short		= "Your spells break down into instant killing bombs.",
        precursor_types			= "magic_artifact",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("triad_of_death") >= 1) then
				if(not isPhysicsProjectile(projectile_entity_id)) then
					SetRandomSeed( GameGetFrameNum(), math.random(1,50))
					local chance = Random(1,100)
					local odds = 30 + (5 * getRelicLevel("triad_of_death"))
					if(chance >= 1 and chance <= odds) then
						addBuffVisualIndicator_Projectile(projectile_entity_id, "plasma_fading_green")
						addNewLuaCallback(projectile_entity_id, "overtime", "mods/ArcaneReliquary/files/scripts/projectiles/launch_death_bomb_projectiles.lua", 4, true, "launch_death_bomb_projectiles")
					end
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "petrifying_scepter,grim_scythe,frost_dagger",
	},
	{
		name 					= "detergent",
		ui_name					= "Detergent",
		categories				= "player",
		effect_desc				= "When you take damage from enemies, there's a 30% chance you'll release barrage of bubbletraps.",
		effect_desc_short		= "You release a barrage of bubbletraps once damaged by enemies.",
        precursor_types			= "gadget",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= false,
		ondamaged_function		= function(damage, message, entity_thats_responsible, is_fatal, projectile_thats_responsible )
			if(isRelicOwned("detergent") >= 1) then	
				if(EntityHasTag(entity_thats_responsible, "enemy")) then
					SetRandomSeed( GameGetFrameNum(), math.random(1,50))
					local chance = Random(1,100)
					local odds = 30 + (5 * getRelicLevel("detergent"))
					if(chance >= 1 and chance <= odds) then
						local player_x, player_y = EntityGetTransform(GetUpdatedEntityID())
						EntityLoad("mods/ArcaneReliquary/files/entity_definitions/load_on_explosion/launch_circular_projectiles_bubbletraps.xml", player_x, player_y)
					end
				end
			end
		end,
	},
	{
		name 					= "energy_crystal_light_pink",
		ui_name					= "Light Pink Energy Crystal",
		categories				= "player",
		effect_desc				= "A pulsating energy field emanates from you, it destroys enemy projectiles and \nshoots bubbletraps back and also freezes enemies it touches. \nIf you take damage, you are frozen.",
		effect_desc_short		= "An energy field emanates from you, destroying enemy projectiles and shooting bubbletraps back.",
        precursor_types			= "magic_artifact,energy_shield",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			LoadGameEffectEntityTo( getPlayerEntity(), getCustomEffect("effect_pulsating_shield_projectile_scatter_bubbletrap").path )
		end,
		ondrop_function			= function()
			local player_children = EntityGetAllChildren(getPlayerEntity())
			if player_children == nil then return end
			for i,child in ipairs(player_children) do
				if(EntityGetName(child) == "pulsating_shield_projectiles_scatter_bubbletrap") then
					EntityKill(child)
					break
				end
			end
		end,
		ondamaged_function		= function(damage, message, entity_thats_responsible, is_fatal, projectile_thats_responsible )
			if(isRelicOwned("energy_crystal_light_pink") >= 1) then
				if(EntityHasTag(entity_thats_responsible, "enemy")) then
					LoadGameEffectEntityTo( GetUpdatedEntityID(), "data/entities/misc/effect_frozen.xml" )
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "mechanical_heart,ancient_root,detergent",
	},
	{
		name 					= "bubble_blower",
		ui_name					= "Bubble Blower",
		categories				= "projectiles",
		effect_desc				= "Most spells you cast have a 40% chance of shooting bubbletraps at random directions once cast. \nIt also synergizes with Earth Core Fragment relic effect.",
		effect_desc_short		= "Your spells shoot bubbletraps.",
        precursor_types			= "gadget",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5% and when the effect triggers, laser bolts are shot more frequently.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("bubble_blower") >= 1) then
				if((not isPhysicsProjectile(projectile_entity_id)) and (not isProjectileHittable(projectile_entity_id))) then
					SetRandomSeed( GameGetFrameNum(), math.random(1,50))
					local chance = Random(1,100)
					local odds = 40 + (5 * getRelicLevel("bubble_blower"))
					if(chance >= 1 and chance <= odds) then
						addNewInternalVariable(projectile_entity_id, "projectile_definition", "value_string", "mods/ArcaneReliquary/files/entity_definitions/custom_projectiles/bubbletrap.xml")
						addNewInternalVariable(projectile_entity_id, "projectile_velocity", "value_int", 160)
						addNewInternalVariable(projectile_entity_id, "max_projectiles_shot", "value_int", 15)
						addNewLuaCallback(projectile_entity_id, "overtime", "mods/ArcaneReliquary/files/scripts/projectiles/launch_projectile_random_direction.lua", 25 - (2 * getRelicLevel("bubble_blower")), false, "launch_projectile_random_direction")
					end
				end
			end
		end,
		ongroundpound_function	= function( entity_id)
			if(isRelicOwned("bubble_blower") >= 1) then
				LoadGameEffectEntityTo( entity_id, getCustomEffect("effect_shoot_projectiles_above_bubbletrap").path )
			end
		end,
	},
	{
		name 					= "dolphin_toy",
		ui_name					= "Dolphin Toy",
		categories				= "projectiles,status_effects",
		effect_desc				= "When you take damage from enemy projectiles, there's a 30% chance the shooter will shoot bubbletraps at random directions.",
		effect_desc_short		= "When you take damage from enemy projectiles, the shooter shoots bubbletraps at random directions.",
        precursor_types			= "gadget",
		upgrade_effect_desc		= "For each upgrade level, effect chance is increased by 5%.",
		is_upgradable 			= true,
		ondamaged_function		= function(damage, message, entity_thats_responsible, is_fatal, projectile_thats_responsible )
			if(isRelicOwned("dolphin_toy") >= 1) then
				if(message == "$damage_projectile") then
					SetRandomSeed( GameGetFrameNum(), math.random(1,50))
					local chance = Random(1,100)
					local odds = 30 + (5 * getRelicLevel("dolphin_toy"))
					if(chance >= 1 and chance <= odds) then
						LoadGameEffectEntityTo( entity_thats_responsible, getCustomEffect("effect_shoot_projectile_random_bubbletrap").path )
					end
				end
			end
		end,
	},
	{
		name 					= "feather_band",
		ui_name					= "Feather Band",
		categories				= "projectiles",
		effect_desc				= "When you cast a spell, spells from your current wand are cast together at once.",
		effect_desc_short		= "Casting a spell causes spells of your wand to be cast at once.",
        precursor_types			= "gadget",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("feather_band") >= 1) then
				
				-- prevent effect from happening if only one spell available
				--local free_slots, spells = getWandEmptyandOccupiedSlots(getCurrentlyEquippedWandId())
				--if(spells == 1) then return end
				
				local x, y = EntityGetTransform(projectile_entity_id)
				
				-- obtains all spell definitions from current wand
				local spell_definitions = getSpellDefinitionsFromWand(getCurrentlyEquippedWandId())
				for i,spell_definition in ipairs(spell_definitions) do
					-- shoots each spell found
					shootProjectileBasedonAnotherProjectile(projectile_entity_id, spell_definition, x, y, 100)
				end
				
				-- kills original projectile
				--projectileToggleExplosionOnDeath(projectile_entity_id, false)
				--EntityKill(projectile_entity_id)
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "hawk_feather,elemental_band",
	},
	{
		name 					= "binoculars",
		ui_name					= "Binoculars",
		categories				= "projectiles",
		effect_desc				= "Most spells you cast are now short ranged, but once cast, they will track \nnearby enemies, then shoot copies of themselves or other spells in your equipped wand towards them.",
		effect_desc_short		= "Your spells track enemies nearby, shooting spells from your equipped wand towards them.",
        precursor_types			= "magic_artifact",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("binoculars") >= 1) then
				if(not isPhysicsProjectile(projectile_entity_id) and not isOrbitingProjectile(projectile_entity_id)) then
					attachStopandTrackMechanictoProjectile_wandSpells(projectile_entity_id)
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "wooden_monocular,feather_band",
		excludes_relics			= "wooden_monocular",
		excludes_relics_desc	= "It can't be used along with other spell track and shoot relic variations",
		excludes_relics_tags	= "track_and_shoot_effect",
	},
	{
		name 					= "wolf_head_necklace",
		ui_name					= "Wolf Head Necklace",
		categories				= "player,materials,items",
		effect_desc				= "The energy of the Small Moon item causes you to move faster, deal more damage \nwith your spells and also be slowly healed when near it, but direct contact with silver causes heavy damage. \n(The Small Moon item must be either near you, or in your potions inventory for these effects to trigger)",
		effect_desc_short		= "The Small Moon item buffs and heals you, but silver damages you.",
        precursor_types			= "magic_artifact,necklace",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			EntitySetDamageFromMaterial( getPlayerEntity(), "silver", 0.01 )
		end,
		ondrop_function			= function()
			EntitySetDamageFromMaterial( getPlayerEntity(), "silver", 0 )
		end,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("wolf_head_necklace") >= 1) then
				local player_x, player_y = EntityGetTransform(getPlayerEntity())
				local moon_items = EntityGetInRadiusWithTag( player_x, player_y, 50, "moon_energy" )
				for i, moon_id in ipairs(moon_items) do
					addBuffVisualIndicator_Projectile(projectile_entity_id, "spark_red")
					multiplyProjectileBaseDamage(projectile_entity_id, 3)
				end
			end
		end,
		overtime_function		= function()
			if(isRelicOwned("wolf_head_necklace") >= 1) then
				local player_x, player_y = EntityGetTransform(getPlayerEntity())
				local moon_items = EntityGetInRadiusWithTag( player_x, player_y, 50, "moon_energy" )
				for i, moon_id in ipairs(moon_items) do
					if(not entityHasSimplifiedInternalVariable(getPlayerEntity(), "faster_movement")) then
						LoadGameEffectEntityTo( getPlayerEntity(), getCustomEffect("effect_movement_faster_short").path )
					end
				end
			end
		end,
		overtimeslow_function	= function()
			if(isRelicOwned("wolf_head_necklace") >= 1) then
				local player_x, player_y = EntityGetTransform(getPlayerEntity())
				local moon_items = EntityGetInRadiusWithTag( player_x, player_y, 50, "moon_energy" )
				for i, moon_id in ipairs(moon_items) do
					addPlayerHealth(1, false)
				end
			end
		end,
	},
	{
		name 					= "wand_necklace",
		ui_name					= "Wand Necklace",
		categories				= "player,items",
		effect_desc				= "Causes the Broken Wand item to shoot more powerful spells when thrown.",
		effect_desc_short		= "Causes the Broken Wand item to shoot more powerful spells when thrown.",
        precursor_types			= "magic_artifact,necklace",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			addNewSimplifiedInternalVariable(getPlayerEntity(), "wand_necklace")
		end,
		ondrop_function			= function()
			removeSimplifiedInternalVariable(getPlayerEntity(), "wand_necklace")
		end
	},
	{
		name 					= "storm_necklace",
		ui_name					= "Necklace of Storms",
		categories				= "player,items",
		effect_desc				= "The power of the Thunderstone item induces enemies nearby to be \nelectrocuted, and if you take damage, enemies in the vicinity, except the one \nwho attacked you, are inflicted with the thunderbolts effect. \n(The Thunderstone item must be either near you, or in your potions inventory for these effects to trigger)",
		effect_desc_short		= "The Thunderstone item electrocutes enemies and causes them to shoot thunderbolts.",
        precursor_types			= "magic_artifact,necklace",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		overtime_function		= function()
			if(isRelicOwned("storm_necklace") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local player_x, player_y = EntityGetTransform(getPlayerEntity())
				local thunderstone_items = EntityGetInRadiusWithTag( player_x, player_y, 50, "thunderstone" )
				for i, thunderstone_id in ipairs(thunderstone_items) do
					local enemies = EntityGetInRadiusWithTag( player_x, player_y, 200, "enemy" )
					for i, enemy_id in ipairs(enemies) do
						local chance = Random(1,2)
						if(chance == 1) then
							LoadGameEffectEntityTo( enemy_id, "data/entities/misc/effect_electricity.xml" )
						end
					end
				end
			end
		end,
		ondamaged_function		= function(damage, message, entity_thats_responsible, is_fatal, projectile_thats_responsible )
			if(isRelicOwned("storm_necklace") >= 1) then
				if(EntityHasTag(entity_thats_responsible, "enemy")) then
					local player_x, player_y = EntityGetTransform(getPlayerEntity())
					local thunderstone_items = EntityGetInRadiusWithTag( player_x, player_y, 50, "thunderstone" )
					for i, thunderstone_id in ipairs(thunderstone_items) do
						local enemies = EntityGetInRadiusWithTag( player_x, player_y, 200, "enemy" )
						for i, enemy_id in ipairs(enemies) do
							if(enemy_id ~= entity_thats_responsible) then
								LoadGameEffectEntityTo( enemy_id, getCustomEffect("effect_shoot_projectile_random_thunderbolt").path )
							end
						end
					end
				end
			end
		end
	},
	{
		name 					= "ember_necklace",
		ui_name					= "Necklace of Embers",
		categories				= "player,items",
		effect_desc				= "The power of the Brimstone item causes enemies that attack you, as well as others in the \nvicinity, to be inflicted with a mystic target that shoots firebombs on them. \n(The Brimstone item must be either near you, or in your potions inventory for these effects to trigger)",
		effect_desc_short		= "The Thunderstone item electrocutes enemies and causes them to shoot thunderbolts.",
        precursor_types			= "magic_artifact,necklace",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		ondamaged_function		= function(damage, message, entity_thats_responsible, is_fatal, projectile_thats_responsible )
			if(isRelicOwned("ember_necklace") >= 1) then
				if(EntityHasTag(entity_thats_responsible, "enemy")) then
					local player_x, player_y = EntityGetTransform(getPlayerEntity())
					local thunderstone_items = EntityGetInRadiusWithTag( player_x, player_y, 50, "brimstone" )
					for i, thunderstone_id in ipairs(thunderstone_items) do
						local enemies = EntityGetInRadiusWithTag( player_x, player_y, 200, "enemy" )
						for i, enemy_id in ipairs(enemies) do
							LoadGameEffectEntityTo( enemy_id, getCustomEffect("effect_projectile_target_firebomb").path )
						end
					end
				end
			end
		end
	},
	{
		name 					= "wealth_necklace",
		ui_name					= "Necklace of Wealth",
		categories				= "player,items",
		effect_desc				= "Causes the Shiny Orb item to no longer randomly explode when you kick it.",
		effect_desc_short		= "Causes the Shiny Orb item to no longer randomly explode when you kick it.",
        precursor_types			= "magic_artifact,necklace",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			addNewSimplifiedInternalVariable(getPlayerEntity(), "wealth_necklace")
		end,
		ondrop_function			= function()
			removeSimplifiedInternalVariable(getPlayerEntity(), "wealth_necklace")
		end
	},
	{
		name 					= "duality_medalion",
		ui_name					= "Duality Medalion",
		categories				= "projectiles",
		effect_desc				= "Provided you are midair, for every spell you cast, spells of your currently active wand are shot from your back.",
		effect_desc_short		= "When midair, casting a spell shoots other spells of your active wand backwards.",
        precursor_types			= "necklace",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("duality_medalion") >= 1) then
				if(not isPlayerStanding()) then
					shootMirroredProjectile_WandSpells(projectile_entity_id)
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "feather_band,duality_necklace",
	},
	{
		name 					= "unusual_mana_crystal",
		ui_name					= "Unusual Mana Crystal",
		categories				= "projectiles",
		effect_desc				= "Most spells you cast have a 30% chance of shooting all spells of your active wand.",
		effect_desc_short		= "Your spells shoot spells from your active wand.",
        precursor_types			= "effect_crystal",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("unusual_mana_crystal") >= 1) then
				if((not isPhysicsProjectile(projectile_entity_id)) and (not isProjectileHittable(projectile_entity_id))) then
					SetRandomSeed( GameGetFrameNum(), math.random(1,50))
					local chance = Random(1,100)
					local odds = 30 + (5 * getRelicLevel("unusual_mana_crystal"))
					if(chance >= 1 and chance <= odds) then
						addNewInternalVariable(projectile_entity_id, "projectile_velocity", "value_int", Random(400,600))
						addNewInternalVariable(projectile_entity_id, "max_projectiles_shot", "value_int", 25)
						addNewLuaCallback(projectile_entity_id, "overtime", "mods/ArcaneReliquary/files/scripts/projectiles/launch_projectile_random_direction_wand_spells.lua", 5, false, "launch_projectile_random_direction_wand_spells")
					end
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "feather_band,explosive_mana_crystal",
	},
	{
		name 					= "lazuli_butterfly_necklace",
		ui_name					= "Lazuli Butterfly Necklace",
		categories				= "projectiles",
		effect_desc				= "Your spells have a 30% chance of exploding into a \nbarrage of spells of your active wand.",
		effect_desc_short		= "Your spells explode into a barrage of spells of your active wand on impact.",
        precursor_types			= "necklace",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("lazuli_butterfly_necklace") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 30 + (5 * getRelicLevel("lazuli_butterfly_necklace"))
				if(chance >= 1 and chance <= odds) then	
					addBuffVisualIndicator_Projectile(projectile_entity_id, "spark_blue")
					attachLoadEntityOnExplosiontoProjectile(projectile_entity_id, getLoadOnExplosion("launch_circular_projectiles_wand_spells").path)
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "feather_band,lazuli_butterfly",
	},
	{
		name 					= "ruby_butterfly_necklace",
		ui_name					= "Ruby Butterfly Necklace",
		categories				= "projectiles",
		effect_desc				= "When damaged by enemies, you release a barrage of spells of your active wand.",
		effect_desc_short		= "When damaged by enemies, you release a barrage of spells of your active wand.",
        precursor_types			= "necklace",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		ondamaged_function		= function(damage, message, entity_thats_responsible, is_fatal, projectile_thats_responsible )
			if(isRelicOwned("ruby_butterfly_necklace") >= 1) then	
				if(EntityHasTag(entity_thats_responsible, "enemy")) then
					local player_x, player_y = EntityGetTransform(GetUpdatedEntityID())
					EntityLoad("mods/ArcaneReliquary/files/entity_definitions/load_on_explosion/launch_circular_projectiles_wand_spells.xml", player_x, player_y)
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "feather_band,ruby_butterfly",
	},
	{
		name 					= "butterfly_ring",
		ui_name					= "Butterfly Ring",
		categories				= "projectiles",
		effect_desc				= "Provided your active wand has at least 6 empty slots, each spell cast causes all spells of your \nactive wand to be cast at random directions, with how many casts depending on the \namount of empty slots in your active wand.",
		effect_desc_short		= "Casting spells causes spells of your active wand to be cast at random directions based on its empty slots.",
        precursor_types			= "ring",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("butterfly_ring") >= 1) then
				local free_slots, spells = getWandEmptyandOccupiedSlots(getCurrentlyEquippedWandId())
				if(free_slots >= 6) then
					local player_x, player_y = EntityGetTransform(GetUpdatedEntityID())
					EntityLoad("mods/ArcaneReliquary/files/entity_definitions/load_on_explosion/launch_circular_projectiles_wand_spells_empty_slots.xml", player_x, player_y)
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "ruby_butterfly_necklace,lazuli_butterfly_necklace,golden_fork",
	},
	{
		name 					= "skull_ring_red",
		ui_name					= "Cursed Ruby Skull Ring",
		categories				= "player",
		effect_desc				= "Cursed skulls rotate around you. They target and shoot enemies with laser beams \n and might also eventually propel themselves towards them exploding on impact. \nCollecting gold has a 50% chance of inflicting you with the laser beam shooting effect.",
		effect_desc_short		= "Cursed skulls rotate around you and shoot enemies with laser beams.",
        precursor_types			= "gravity_stone,ring",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		ondrop_function			= function()
			detachRotatingShieldfromEntity(getPlayerEntity(), "rotating_shield_projectile_skull")		
		end,
		overtime_function		= function()
			if(isRelicOwned("skull_ring_red") >= 1) then
				if(not entityHasRotatingShield(getPlayerEntity(), "rotating_shield_projectile_skull")) then -- this makes sure the orbiting projectiles are loaded again if the player is polymorphed
					
					local start_id = EntityLoad("mods/ArcaneReliquary/files/entity_definitions/custom_projectiles/rotating_shield_start.xml")
					setInternalVariableValue(start_id, "host_entity_id", "value_int", getPlayerEntity())
					setInternalVariableValue(start_id, "rotating_projectile_definition", "value_string", "mods/ArcaneReliquary/files/entity_definitions/custom_projectiles/rotating_skull.xml")
					setInternalVariableValue(start_id, "rotating_projectiles_amount", "value_int", 4)
					setInternalVariableValue(start_id, "radius", "value_int", 30)
					setInternalVariableValue(start_id, "rotation_speed", "value_int", 2)
				end
			end
		end,
		ongoldpickup_function	= function()
			if(isRelicOwned("skull_ring_red") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				if(Random(1,2) == 1) then
					LoadGameEffectEntityTo(getPlayerEntity(), getCustomEffect("effect_shoot_projectile_random_laser").path)
				end
			end
		end		
	},
	{
		name 					= "skull_ring_fire",
		ui_name					= "Cursed Fire Skull Ring",
		categories				= "player",
		effect_desc				= "Cursed skulls rotate around you. They target and shoot enemies with incendiary laser beams \n and might also eventually propel themselves towards them exploding on impact. \nCollecting gold has a 50% chance of inflicting you with the incendiary laser beam shooting effect.",
		effect_desc_short		= "Cursed skulls rotate around you and shoot enemies with incendiary laser beams.",
        precursor_types			= "gravity_stone,ring",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		ondrop_function			= function()
			detachRotatingShieldfromEntity(getPlayerEntity(), "rotating_shield_projectile_skull_fire")		
		end,
		overtime_function		= function()
			if(isRelicOwned("skull_ring_fire") >= 1) then
				if(not entityHasRotatingShield(getPlayerEntity(), "rotating_shield_projectile_skull_fire")) then -- this makes sure the orbiting projectiles are loaded again if the player is polymorphed
					
					local start_id = EntityLoad("mods/ArcaneReliquary/files/entity_definitions/custom_projectiles/rotating_shield_start.xml")
					setInternalVariableValue(start_id, "host_entity_id", "value_int", getPlayerEntity())
					setInternalVariableValue(start_id, "rotating_projectile_definition", "value_string", "mods/ArcaneReliquary/files/entity_definitions/custom_projectiles/rotating_skull_fire.xml")
					setInternalVariableValue(start_id, "rotating_projectiles_amount", "value_int", 4)
					setInternalVariableValue(start_id, "radius", "value_int", 30)
					setInternalVariableValue(start_id, "rotation_speed", "value_int", 2)
				end
			end
		end,
		ongoldpickup_function	= function()
			if(isRelicOwned("skull_ring_fire") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				if(Random(1,2) == 1) then
					LoadGameEffectEntityTo(getPlayerEntity(), getCustomEffect("effect_shoot_projectile_random_laser_orange").path)
				end
			end
		end		
	},
	{
		name 					= "skull_ring_ice",
		ui_name					= "Cursed Ice Skull Ring",
		categories				= "player",
		effect_desc				= "Cursed skulls rotate around you. They target and shoot enemies with freezing laser beams \n and might also eventually propel themselves towards them exploding on impact. \nCollecting gold has a 50% chance of inflicting you with the freezing laser beam shooting effect.",
		effect_desc_short		= "Cursed skulls rotate around you and shoot enemies with freezing laser beams.",
        precursor_types			= "gravity_stone,ring",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		ondrop_function			= function()
			detachRotatingShieldfromEntity(getPlayerEntity(), "rotating_shield_projectile_skull_ice")		
		end,
		overtime_function		= function()
			if(isRelicOwned("skull_ring_ice") >= 1) then
				if(not entityHasRotatingShield(getPlayerEntity(), "rotating_shield_projectile_skull_ice")) then -- this makes sure the orbiting projectiles are loaded again if the player is polymorphed
					
					local start_id = EntityLoad("mods/ArcaneReliquary/files/entity_definitions/custom_projectiles/rotating_shield_start.xml")
					setInternalVariableValue(start_id, "host_entity_id", "value_int", getPlayerEntity())
					setInternalVariableValue(start_id, "rotating_projectile_definition", "value_string", "mods/ArcaneReliquary/files/entity_definitions/custom_projectiles/rotating_skull_ice.xml")
					setInternalVariableValue(start_id, "rotating_projectiles_amount", "value_int", 4)
					setInternalVariableValue(start_id, "radius", "value_int", 30)
					setInternalVariableValue(start_id, "rotation_speed", "value_int", 2)
				end
			end
		end,
		ongoldpickup_function	= function()
			if(isRelicOwned("skull_ring_ice") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				if(Random(1,2) == 1) then
					LoadGameEffectEntityTo(getPlayerEntity(), getCustomEffect("effect_shoot_projectile_random_laser_teal").path)
				end
			end
		end		
	},
	{
		name 					= "skull_ring_marble",
		ui_name					= "Cursed Marble Skull Ring",
		categories				= "player",
		effect_desc				= "Cursed skulls rotate around you. They target and shoot enemies with slash laser beams \n and might also eventually propel themselves towards them exploding on impact. \nCollecting gold has a 50% chance of inflicting you with the slash laser beam shooting effect.",
		effect_desc_short		= "Cursed skulls rotate around you and shoot enemies with slash laser beams.",
        precursor_types			= "gravity_stone,ring",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		ondrop_function			= function()
			detachRotatingShieldfromEntity(getPlayerEntity(), "rotating_shield_projectile_skull_marble")		
		end,
		overtime_function		= function()
			if(isRelicOwned("skull_ring_marble") >= 1) then
				if(not entityHasRotatingShield(getPlayerEntity(), "rotating_shield_projectile_skull_marble")) then -- this makes sure the orbiting projectiles are loaded again if the player is polymorphed
					
					local start_id = EntityLoad("mods/ArcaneReliquary/files/entity_definitions/custom_projectiles/rotating_shield_start.xml")
					setInternalVariableValue(start_id, "host_entity_id", "value_int", getPlayerEntity())
					setInternalVariableValue(start_id, "rotating_projectile_definition", "value_string", "mods/ArcaneReliquary/files/entity_definitions/custom_projectiles/rotating_skull_marble.xml")
					setInternalVariableValue(start_id, "rotating_projectiles_amount", "value_int", 4)
					setInternalVariableValue(start_id, "radius", "value_int", 30)
					setInternalVariableValue(start_id, "rotation_speed", "value_int", 2)
				end
			end
		end,
		ongoldpickup_function	= function()
			if(isRelicOwned("skull_ring_marble") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				if(Random(1,2) == 1) then
					LoadGameEffectEntityTo(getPlayerEntity(), getCustomEffect("effect_shoot_projectile_random_laser_slash").path)
				end
			end
		end		
	},
	{
		name 					= "skull_ring_sulfur",
		ui_name					= "Cursed Sulfuric Skull Ring",
		categories				= "player",
		effect_desc				= "Cursed skulls rotate around you. They target and shoot enemies with explosive laser beams \n and might also eventually propel themselves towards them exploding on impact. \nCollecting gold has a 50% chance of inflicting you with the explosive laser beam shooting effect.",
		effect_desc_short		= "Cursed skulls rotate around you and shoot enemies with explosive laser beams.",
        precursor_types			= "gravity_stone,ring",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		ondrop_function			= function()
			detachRotatingShieldfromEntity(getPlayerEntity(), "rotating_shield_projectile_skull_sulfur")		
		end,
		overtime_function		= function()
			if(isRelicOwned("skull_ring_sulfur") >= 1) then
				if(not entityHasRotatingShield(getPlayerEntity(), "rotating_shield_projectile_skull_sulfur")) then -- this makes sure the orbiting projectiles are loaded again if the player is polymorphed
					
					local start_id = EntityLoad("mods/ArcaneReliquary/files/entity_definitions/custom_projectiles/rotating_shield_start.xml")
					setInternalVariableValue(start_id, "host_entity_id", "value_int", getPlayerEntity())
					setInternalVariableValue(start_id, "rotating_projectile_definition", "value_string", "mods/ArcaneReliquary/files/entity_definitions/custom_projectiles/rotating_skull_sulfur.xml")
					setInternalVariableValue(start_id, "rotating_projectiles_amount", "value_int", 4)
					setInternalVariableValue(start_id, "radius", "value_int", 30)
					setInternalVariableValue(start_id, "rotation_speed", "value_int", 2)
				end
			end
		end,
		ongoldpickup_function	= function()
			if(isRelicOwned("skull_ring_sulfur") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				if(Random(1,2) == 1) then
					LoadGameEffectEntityTo(getPlayerEntity(), getCustomEffect("effect_shoot_projectile_random_laser_yellow").path)
				end
			end
		end		
	},
	{
		name 					= "skull_ring_lazuli",
		ui_name					= "Cursed Lazuli Skull Ring",
		categories				= "player",
		effect_desc				= "Cursed skulls rotate around you. They target and shoot enemies with electrocuting laser beams \n and might also eventually propel themselves towards them exploding on impact. \nCollecting gold has a 50% chance of inflicting you with the electrocuting laser beam shooting effect.",
		effect_desc_short		= "Cursed skulls rotate around you and shoot enemies with electrocuting laser beams.",
        precursor_types			= "gravity_stone,ring",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		ondrop_function			= function()
			detachRotatingShieldfromEntity(getPlayerEntity(), "rotating_shield_projectile_skull_lazuli")		
		end,
		overtime_function		= function()
			if(isRelicOwned("skull_ring_lazuli") >= 1) then
				if(not entityHasRotatingShield(getPlayerEntity(), "rotating_shield_projectile_skull_lazuli")) then -- this makes sure the orbiting projectiles are loaded again if the player is polymorphed
					
					local start_id = EntityLoad("mods/ArcaneReliquary/files/entity_definitions/custom_projectiles/rotating_shield_start.xml")
					setInternalVariableValue(start_id, "host_entity_id", "value_int", getPlayerEntity())
					setInternalVariableValue(start_id, "rotating_projectile_definition", "value_string", "mods/ArcaneReliquary/files/entity_definitions/custom_projectiles/rotating_skull_lazuli.xml")
					setInternalVariableValue(start_id, "rotating_projectiles_amount", "value_int", 4)
					setInternalVariableValue(start_id, "radius", "value_int", 30)
					setInternalVariableValue(start_id, "rotation_speed", "value_int", 2)
				end
			end
		end,
		ongoldpickup_function	= function()
			if(isRelicOwned("skull_ring_lazuli") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				if(Random(1,2) == 1) then
					LoadGameEffectEntityTo(getPlayerEntity(), getCustomEffect("effect_shoot_projectile_random_laser_blue").path)
				end
			end
		end		
	},
	{
		name 					= "stone_hand_fallout",
		ui_name					= "Fallout Stone Hand",
		categories				= "companions",
		effect_desc				= "A pair of fallout stone hands that launch themselves towards enemies aids you.",
		effect_desc_short		= "A pair of fallout stone hands that launch themselves towards enemies aids you.",
        precursor_types			= "companion",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			
			attachMonkHandstoEntity(getPlayerEntity(), "monk_hand_fallout", "monk_arm_fx_fallout", "mortal")
		end,
		ondrop_function			= function()
			
			detachMonkHandsfromEntity(getPlayerEntity(), "monk_hand_fallout")
		end,
		overtime_function		= function()
			if(isRelicOwned("stone_hand_fallout") >= 1) then
				attachMonkHandstoEntity(getPlayerEntity(), "monk_hand_fallout", "monk_arm_fx_fallout", "mortal")
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "stone_hand_nuclear,plutonium_ingot",
	},
	{
		name 					= "stone_hand_pungent",
		ui_name					= "Pungent Stone Hand",
		categories				= "companions",
		effect_desc				= "A pair of acid stone hands that launch themselves towards enemies aids you.",
		effect_desc_short		= "A pair of acid stone hands that launch themselves towards enemies aids you.",
        precursor_types			= "companion",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			
			attachMonkHandstoEntity(getPlayerEntity(), "monk_hand_pungent", "monk_arm_fx_pungent", "mortal")
		end,
		ondrop_function			= function()
			
			detachMonkHandsfromEntity(getPlayerEntity(), "monk_hand_pungent")
		end,
		overtime_function		= function()
			if(isRelicOwned("stone_hand_pungent") >= 1) then
				attachMonkHandstoEntity(getPlayerEntity(), "monk_hand_pungent", "monk_arm_fx_pungent", "mortal")
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "stone_hand_melee,acid_gem",
	},
	{
		name 					= "stone_hand_mutating",
		ui_name					= "Mutating Stone Hand",
		categories				= "companions",
		effect_desc				= "A pair of polymorphine stone hands that launch themselves towards enemies aids you.",
		effect_desc_short		= "A pair of polymorphine stone hands that launch themselves towards enemies aids you.",
        precursor_types			= "companion",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			
			attachMonkHandstoEntity(getPlayerEntity(), "monk_hand_mutating", "monk_arm_fx_mutating", "mortal")
		end,
		ondrop_function			= function()
			
			detachMonkHandsfromEntity(getPlayerEntity(), "monk_hand_mutating")
		end,
		overtime_function		= function()
			if(isRelicOwned("stone_hand_mutating") >= 1) then
				attachMonkHandstoEntity(getPlayerEntity(), "monk_hand_mutating", "monk_arm_fx_mutating", "mortal")
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "stone_hand_melee,poly_gem",
	},
	{
		name 					= "stone_hand_venomous",
		ui_name					= "Venomous Stone Hand",
		categories				= "companions",
		effect_desc				= "A pair of poison stone hands that launch themselves towards enemies aids you.",
		effect_desc_short		= "A pair of poison stone hands that launch themselves towards enemies aids you.",
        precursor_types			= "companion",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			
			attachMonkHandstoEntity(getPlayerEntity(), "monk_hand_venomous", "monk_arm_fx_venomous", "mortal")
		end,
		ondrop_function			= function()
			
			detachMonkHandsfromEntity(getPlayerEntity(), "monk_hand_venomous")
		end,
		overtime_function		= function()
			if(isRelicOwned("stone_hand_venomous") >= 1) then
				attachMonkHandstoEntity(getPlayerEntity(), "monk_hand_venomous", "monk_arm_fx_venomous", "mortal")
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "stone_hand_melee,poison_gem",
	},
	{
		name 					= "stone_hand_humid",
		ui_name					= "Humid Stone Hand",
		categories				= "companions",
		effect_desc				= "A pair of water stone hands that launch themselves towards enemies aids you.",
		effect_desc_short		= "A pair of water stone hands that launch themselves towards enemies aids you.",
        precursor_types			= "companion",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			
			attachMonkHandstoEntity(getPlayerEntity(), "monk_hand_humid", "monk_arm_fx_humid", "mortal")
		end,
		ondrop_function			= function()
			
			detachMonkHandsfromEntity(getPlayerEntity(), "monk_hand_humid")
		end,
		overtime_function		= function()
			if(isRelicOwned("stone_hand_humid") >= 1) then
				attachMonkHandstoEntity(getPlayerEntity(), "monk_hand_humid", "monk_arm_fx_humid", "mortal")
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "stone_hand_melee,water_gem",
	},
	{
		name 					= "stone_hand_uric",
		ui_name					= "Uric Stone Hand",
		categories				= "companions",
		effect_desc				= "A pair of urine stone hands that launch themselves towards enemies aids you.",
		effect_desc_short		= "A pair of urine stone hands that launch themselves towards enemies aids you.",
        precursor_types			= "companion",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			
			attachMonkHandstoEntity(getPlayerEntity(), "monk_hand_uric", "monk_arm_fx_uric", "mortal")
		end,
		ondrop_function			= function()
			
			detachMonkHandsfromEntity(getPlayerEntity(), "monk_hand_uric")
		end,
		overtime_function		= function()
			if(isRelicOwned("stone_hand_uric") >= 1) then
				attachMonkHandstoEntity(getPlayerEntity(), "monk_hand_uric", "monk_arm_fx_uric", "mortal")
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "stone_hand_melee,water_gem",
	},
	{
		name 					= "uric_skull",
		ui_name					= "Uric Skull",
		categories				= "companions",
		effect_desc				= "A friendly skull that shoots urine fireballs joins you.",
		effect_desc_short		= "An urine fireball shooting companion.",
        precursor_types			= "companion",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			attachCompaniontoEntity(getPlayerEntity(), "uric_skull", "mortal")
		end,
		ondrop_function			= function()
			detachCompanionfromEntity(getPlayerEntity(), "uric_skull")
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "flaming_skull,urine_gem",
		requires_relic			= "bone",
	},
	{
		name 					= "energy_crystal_urine_yellow",
		ui_name					= "Urine Yellow Energy Crystal",
		categories				= "player",
		effect_desc				= "A pulsating energy field emanates from you, it destroys enemy projectiles and \nshoots urine fireballs back and also burns enemies it touches. \nIf you take damage, you catch fire.",
		effect_desc_short		= "An energy field emanates from you, destroying enemy projectiles and shooting urine fireballs back.",
        precursor_types			= "magic_artifact,energy_shield",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			LoadGameEffectEntityTo( getPlayerEntity(), getCustomEffect("effect_pulsating_shield_projectile_scatter_fireball_urine").path )
		end,
		ondrop_function			= function()
			local player_children = EntityGetAllChildren(getPlayerEntity())
			if player_children == nil then return end
			for i,child in ipairs(player_children) do
				if(EntityGetName(child) == "pulsating_shield_projectiles_scatter_fireball_urine") then
					EntityKill(child)
					break
				end
			end
		end,
		ondamaged_function		= function(damage, message, entity_thats_responsible, is_fatal, projectile_thats_responsible )
			if(isRelicOwned("energy_crystal_urine_yellow") >= 1) then
				if(EntityHasTag(entity_thats_responsible, "enemy")) then
					LoadGameEffectEntityTo( GetUpdatedEntityID(), "data/entities/misc/effect_on_fire.xml" )
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "mechanical_heart,ancient_root,urine_gem",
	},
	{
		name 					= "uric_crystal",
		ui_name					= "Uric Crystal",
		categories				= "projectiles",
		effect_desc				= "Spells that you cast have a 30% chance of shooting urine fireballs downwards along their travel path.",
		effect_desc_short		= "Your spells cast downward urine fireballs.",
        precursor_types			= "effect_crystal",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%, and when it triggers, fireballs are shot more often.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("uric_crystal") >= 1) then
				if(not isPhysicsProjectile(projectile_entity_id)) then
					SetRandomSeed( GameGetFrameNum(), math.random(1,50))
					local chance = Random(1,100)
					local odds = 30 + (5 * getRelicLevel("uric_crystal"))
					if(chance >= 1 and chance <= odds) then
						addNewInternalVariable(projectile_entity_id, "projectile_shot_down_definition", "value_string", "mods/ArcaneReliquary/files/entity_definitions/custom_projectiles/fireball_urine.xml")
						addNewInternalVariable(projectile_entity_id, "down_vel_min", "value_int", 165)
						addNewInternalVariable(projectile_entity_id, "down_vel_max", "value_int", 185)
						addNewInternalVariable(projectile_entity_id, "max_projectiles_shot", "value_int", 10)
						addNewLuaCallback(projectile_entity_id, "overtime", "mods/ArcaneReliquary/files/scripts/projectiles/launch_projectile_down.lua", 25 - (2 * getRelicLevel("uric_crystal")), false, "launch_projectile_down")
					end
				end
			end
		end,
	},
	{
		name 					= "stone_hand_slash",
		ui_name					= "Slash Stone Hand",
		categories				= "companions",
		effect_desc				= "A pair of stone hands that shoot slash magic stars \naids you. Requires an Elemental Band.",
		effect_desc_short		= "A pair of stone hands that shoot slash magic stars aids you.",
        precursor_types			= "companion",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			
			attachMonkHandstoEntity(getPlayerEntity(), "monk_hand_slash", "monk_arm_fx_slash", "mortal")
		end,
		ondrop_function			= function()
			
			detachMonkHandsfromEntity(getPlayerEntity(), "monk_hand_slash")
		end,
		requires_relic			= "elemental_band",
		combine_exclusive		= true,
		combine_requires_relics	= "stone_hand_white_star,pizza_slice",
	},
	{
		name 					= "stone_hand_exploding",
		ui_name					= "Exploding Stone Hand",
		categories				= "companions",
		effect_desc				= "A pair of stone hands that shoot exploding magic stars \naids you. Requires an Elemental Band.",
		effect_desc_short		= "A pair of stone hands that shoot exploding magic stars aids you.",
        precursor_types			= "companion",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			
			attachMonkHandstoEntity(getPlayerEntity(), "monk_hand_exploding", "monk_arm_fx_exploding", "mortal")
		end,
		ondrop_function			= function()
			
			detachMonkHandsfromEntity(getPlayerEntity(), "monk_hand_exploding")
		end,
		requires_relic			= "elemental_band",
		combine_exclusive		= true,
		combine_requires_relics	= "stone_hand_white_star,freedom_ring",
	},
	{
		name 					= "skull_ring_jade",
		ui_name					= "Cursed Jade Skull Ring",
		categories				= "player",
		effect_desc				= "Cursed skulls rotate around you. They target and shoot enemies with acid laser beams \n and might also eventually propel themselves towards them exploding on impact. \nCollecting gold has a 50% chance of inflicting you with the acid laser beam shooting effect.",
		effect_desc_short		= "Cursed skulls rotate around you and shoot enemies with acid laser beams.",
        precursor_types			= "gravity_stone,ring",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		ondrop_function			= function()
			detachRotatingShieldfromEntity(getPlayerEntity(), "rotating_shield_projectile_skull_jade")		
		end,
		overtime_function		= function()
			if(isRelicOwned("skull_ring_jade") >= 1) then
				if(not entityHasRotatingShield(getPlayerEntity(), "rotating_shield_projectile_skull_jade")) then -- this makes sure the orbiting projectiles are loaded again if the player is polymorphed
					
					local start_id = EntityLoad("mods/ArcaneReliquary/files/entity_definitions/custom_projectiles/rotating_shield_start.xml")
					setInternalVariableValue(start_id, "host_entity_id", "value_int", getPlayerEntity())
					setInternalVariableValue(start_id, "rotating_projectile_definition", "value_string", "mods/ArcaneReliquary/files/entity_definitions/custom_projectiles/rotating_skull_jade.xml")
					setInternalVariableValue(start_id, "rotating_projectiles_amount", "value_int", 4)
					setInternalVariableValue(start_id, "radius", "value_int", 30)
					setInternalVariableValue(start_id, "rotation_speed", "value_int", 2)
				end
			end
		end,
		ongoldpickup_function	= function()
			if(isRelicOwned("skull_ring_jade") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				if(Random(1,2) == 1) then
					LoadGameEffectEntityTo(getPlayerEntity(), getCustomEffect("effect_shoot_projectile_random_laser_green").path)
				end
			end
		end		
	},
	{
		name 					= "deflecting_crystal",
		ui_name					= "Deflecting Crystal (Green)",
		categories				= "player",
		effect_desc				= "Whenever you take damage from any source except fire and \nmaterials, a temporary energy shield with large radius is projected around you. \nThis shield deflects and scatters enemy projectiles, imbuing them with the \nslap effect and also inflicts this effect on enemies that touch it.",
		effect_desc_short		= "A temporary shield is projected around you upon taking damage, causing the slap effect on enemies.",
        precursor_types			= "magic_artifact,energy_shield",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		ondamaged_function		= function(damage, message, entity_thats_responsible, is_fatal, projectile_thats_responsible )
			if(isRelicOwned("deflecting_crystal") >= 1) then
				if(not(string.find(message, "damage from material") or (message == "$damage_fire") or (message == "$damage_radioactive"))) then
					local player_children = EntityGetAllChildren(getPlayerEntity())
					if player_children == nil then return end
					local effect_found = false
					for i,child in ipairs(player_children) do
						if(EntityGetName(child) == "temporary_shield_slap") then
							effect_found = true
							break
						end
					end
					if(not effect_found) then
						LoadGameEffectEntityTo( getPlayerEntity(), getCustomEffect("effect_temporary_shield_slap").path )
					end
				end
			end
		end,
		ondrop_function			= function()
			local player_children = EntityGetAllChildren(getPlayerEntity())
			if player_children == nil then return end
			for i,child in ipairs(player_children) do
				if(EntityGetName(child) == "temporary_shield_slap") then
					EntityKill(child)
					break
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "deflecting_medalion,fly_swatter,ancient_root",
	},
	{
		name 					= "deflecting_crystal_cyan",
		ui_name					= "Deflecting Crystal (Cyan)",
		categories				= "player",
		effect_desc				= "Whenever you take damage from any source except fire and \nmaterials, a temporary energy shield with large radius is projected around you. \nThis shield deflects and scatters enemy projectiles, imbuing them with the \nfreeze effect and also inflicts this effect on enemies that touch it.",
		effect_desc_short		= "A temporary shield is projected around you upon taking damage, causing the freeze effect on enemies.",
        precursor_types			= "magic_artifact,energy_shield",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		ondamaged_function		= function(damage, message, entity_thats_responsible, is_fatal, projectile_thats_responsible )
			if(isRelicOwned("deflecting_crystal_cyan") >= 1) then
				if(not(string.find(message, "damage from material") or (message == "$damage_fire") or (message == "$damage_radioactive"))) then
					local player_children = EntityGetAllChildren(getPlayerEntity())
					if player_children == nil then return end
					local effect_found = false
					for i,child in ipairs(player_children) do
						if(EntityGetName(child) == "temporary_shield_freeze") then
							effect_found = true
							break
						end
					end
					if(not effect_found) then
						LoadGameEffectEntityTo( getPlayerEntity(), getCustomEffect("effect_temporary_shield_freeze").path )
					end
				end
			end
		end,
		ondrop_function			= function()
			local player_children = EntityGetAllChildren(getPlayerEntity())
			if player_children == nil then return end
			for i,child in ipairs(player_children) do
				if(EntityGetName(child) == "temporary_shield_freeze") then
					EntityKill(child)
					break
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "deflecting_medalion,ice_artifact,ancient_root",
	},
	{
		name 					= "deflecting_crystal_fiery",
		ui_name					= "Deflecting Crystal (Fiery)",
		categories				= "player",
		effect_desc				= "Whenever you take damage from any source except fire and \nmaterials, a temporary energy shield with large radius is projected around you. \nThis shield deflects and scatters enemy projectiles, imbuing them with the \nexploding effect and also inflicts this effect on enemies that touch it.",
		effect_desc_short		= "A temporary shield is projected around you upon taking damage, causing the exploding effect on enemies.",
        precursor_types			= "magic_artifact,energy_shield",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		ondamaged_function		= function(damage, message, entity_thats_responsible, is_fatal, projectile_thats_responsible )
			if(isRelicOwned("deflecting_crystal_fiery") >= 1) then
				if(not(string.find(message, "damage from material") or (message == "$damage_fire") or (message == "$damage_radioactive"))) then
					local player_children = EntityGetAllChildren(getPlayerEntity())
					if player_children == nil then return end
					local effect_found = false
					for i,child in ipairs(player_children) do
						if(EntityGetName(child) == "temporary_shield_explode") then
							effect_found = true
							break
						end
					end
					if(not effect_found) then
						LoadGameEffectEntityTo( getPlayerEntity(), getCustomEffect("effect_temporary_shield_explode").path )
					end
				end
			end
		end,
		ondrop_function			= function()
			local player_children = EntityGetAllChildren(getPlayerEntity())
			if player_children == nil then return end
			for i,child in ipairs(player_children) do
				if(EntityGetName(child) == "temporary_shield_explode") then
					EntityKill(child)
					break
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "deflecting_medalion,freedom_ring,ancient_root",
	},
	{
		name 					= "deflecting_crystal_bloody",
		ui_name					= "Deflecting Crystal (Bloody)",
		categories				= "player",
		effect_desc				= "Whenever you take damage from any source except fire and \nmaterials, a temporary energy shield with large radius is projected around you. \nThis shield deflects and scatters enemy projectiles, imbuing them with the \nbleeding effect and also inflicts this effect on enemies that touch it.",
		effect_desc_short		= "A temporary shield is projected around you upon taking damage, causing the bleeding effect on enemies.",
        precursor_types			= "magic_artifact,energy_shield",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		ondamaged_function		= function(damage, message, entity_thats_responsible, is_fatal, projectile_thats_responsible )
			if(isRelicOwned("deflecting_crystal_bloody") >= 1) then
				if(not(string.find(message, "damage from material") or (message == "$damage_fire") or (message == "$damage_radioactive"))) then
					local player_children = EntityGetAllChildren(getPlayerEntity())
					if player_children == nil then return end
					local effect_found = false
					for i,child in ipairs(player_children) do
						if(EntityGetName(child) == "temporary_shield_bleed") then
							effect_found = true
							break
						end
					end
					if(not effect_found) then
						LoadGameEffectEntityTo( getPlayerEntity(), getCustomEffect("effect_temporary_shield_bleed").path )
					end
				end
			end
		end,
		ondrop_function			= function()
			local player_children = EntityGetAllChildren(getPlayerEntity())
			if player_children == nil then return end
			for i,child in ipairs(player_children) do
				if(EntityGetName(child) == "temporary_shield_bleed") then
					EntityKill(child)
					break
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "deflecting_medalion,ancient_dagger,ancient_root",
	},
	{
		name 					= "deflecting_crystal_white",
		ui_name					= "Deflecting Crystal (White)",
		categories				= "player",
		effect_desc				= "Whenever you take damage from any source except fire and \nmaterials, a temporary energy shield with large radius is projected around you. \nThis shield deflects and scatters enemy projectiles, imbuing them with the \nwind effect and also inflicts this effect on enemies that touch it.",
		effect_desc_short		= "A temporary shield is projected around you upon taking damage, causing the wind effect on enemies.",
        precursor_types			= "magic_artifact,energy_shield",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		ondamaged_function		= function(damage, message, entity_thats_responsible, is_fatal, projectile_thats_responsible )
			if(isRelicOwned("deflecting_crystal_white") >= 1) then
				if(not(string.find(message, "damage from material") or (message == "$damage_fire") or (message == "$damage_radioactive"))) then
					local player_children = EntityGetAllChildren(getPlayerEntity())
					if player_children == nil then return end
					local effect_found = false
					for i,child in ipairs(player_children) do
						if(EntityGetName(child) == "temporary_shield_wind") then
							effect_found = true
							break
						end
					end
					if(not effect_found) then
						LoadGameEffectEntityTo( getPlayerEntity(), getCustomEffect("effect_temporary_shield_wind").path )
					end
				end
			end
		end,
		ondrop_function			= function()
			local player_children = EntityGetAllChildren(getPlayerEntity())
			if player_children == nil then return end
			for i,child in ipairs(player_children) do
				if(EntityGetName(child) == "temporary_shield_wind") then
					EntityKill(child)
					break
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "deflecting_medalion,pinwheel,ancient_root",
	},
	{
		name 					= "deflecting_crystal_purple",
		ui_name					= "Deflecting Crystal (Purple)",
		categories				= "player",
		effect_desc				= "Whenever you take damage from any source except fire and \nmaterials, a temporary energy shield with large radius is projected around you. \nThis shield deflects and scatters enemy projectiles, imbuing them with the \nslash effect and also inflicts this effect on enemies that touch it.",
		effect_desc_short		= "A temporary shield is projected around you upon taking damage, causing the slash effect on enemies.",
        precursor_types			= "magic_artifact,energy_shield",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		ondamaged_function		= function(damage, message, entity_thats_responsible, is_fatal, projectile_thats_responsible )
			if(isRelicOwned("deflecting_crystal_purple") >= 1) then
				if(not(string.find(message, "damage from material") or (message == "$damage_fire") or (message == "$damage_radioactive"))) then
					local player_children = EntityGetAllChildren(getPlayerEntity())
					if player_children == nil then return end
					local effect_found = false
					for i,child in ipairs(player_children) do
						if(EntityGetName(child) == "temporary_shield_slash") then
							effect_found = true
							break
						end
					end
					if(not effect_found) then
						LoadGameEffectEntityTo( getPlayerEntity(), getCustomEffect("effect_temporary_shield_slash").path )
					end
				end
			end
		end,
		ondrop_function			= function()
			local player_children = EntityGetAllChildren(getPlayerEntity())
			if player_children == nil then return end
			for i,child in ipairs(player_children) do
				if(EntityGetName(child) == "temporary_shield_slash") then
					EntityKill(child)
					break
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "deflecting_medalion,shuriken,ancient_root",
	},
	{
		name 					= "deflecting_crystal_brown",
		ui_name					= "Deflecting Crystal (Brown)",
		categories				= "player",
		effect_desc				= "Whenever you take damage from any source except fire and \nmaterials, a temporary energy shield with large radius is projected around you. \nThis shield deflects and scatters enemy projectiles, imbuing them with the \nshockwave effect and also inflicts this effect on enemies that touch it.",
		effect_desc_short		= "A temporary shield is projected around you upon taking damage, causing the shockwave effect on enemies.",
        precursor_types			= "magic_artifact,energy_shield",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		ondamaged_function		= function(damage, message, entity_thats_responsible, is_fatal, projectile_thats_responsible )
			if(isRelicOwned("deflecting_crystal_brown") >= 1) then
				if(not(string.find(message, "damage from material") or (message == "$damage_fire") or (message == "$damage_radioactive"))) then
					local player_children = EntityGetAllChildren(getPlayerEntity())
					if player_children == nil then return end
					local effect_found = false
					for i,child in ipairs(player_children) do
						if(EntityGetName(child) == "temporary_shield_shockwave") then
							effect_found = true
							break
						end
					end
					if(not effect_found) then
						LoadGameEffectEntityTo( getPlayerEntity(), getCustomEffect("effect_temporary_shield_shockwave").path )
					end
				end
			end
		end,
		ondrop_function			= function()
			local player_children = EntityGetAllChildren(getPlayerEntity())
			if player_children == nil then return end
			for i,child in ipairs(player_children) do
				if(EntityGetName(child) == "temporary_shield_shockwave") then
					EntityKill(child)
					break
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "deflecting_medalion,tuning_fork,ancient_root",
	},
	{
		name 					= "deflecting_crystal_pink",
		ui_name					= "Deflecting Crystal (Pink)",
		categories				= "player",
		effect_desc				= "Whenever you take damage from any source except fire and \nmaterials, a temporary energy shield with large radius is projected around you. \nThis shield deflects and scatters enemy projectiles, imbuing them with the \npolymorph effect and also inflicts this effect on enemies that touch it.",
		effect_desc_short		= "A temporary shield is projected around you upon taking damage, causing the polymorph effect on enemies.",
        precursor_types			= "magic_artifact,energy_shield",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		ondamaged_function		= function(damage, message, entity_thats_responsible, is_fatal, projectile_thats_responsible )
			if(isRelicOwned("deflecting_crystal_pink") >= 1) then
				if(not(string.find(message, "damage from material") or (message == "$damage_fire") or (message == "$damage_radioactive"))) then
					local player_children = EntityGetAllChildren(getPlayerEntity())
					if player_children == nil then return end
					local effect_found = false
					for i,child in ipairs(player_children) do
						if(EntityGetName(child) == "temporary_shield_poly") then
							effect_found = true
							break
						end
					end
					if(not effect_found) then
						LoadGameEffectEntityTo( getPlayerEntity(), getCustomEffect("effect_temporary_shield_poly").path )
					end
				end
			end
		end,
		ondrop_function			= function()
			local player_children = EntityGetAllChildren(getPlayerEntity())
			if player_children == nil then return end
			for i,child in ipairs(player_children) do
				if(EntityGetName(child) == "temporary_shield_poly") then
					EntityKill(child)
					break
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "deflecting_medalion,mutating_bead,ancient_root",
	},
	{
		name 					= "deflecting_crystal_blue",
		ui_name					= "Deflecting Crystal (Blue)",
		categories				= "player",
		effect_desc				= "Whenever you take damage from any source except fire and \nmaterials, a temporary energy shield with large radius is projected around you. \nThis shield deflects and scatters enemy projectiles, imbuing them with the \nelectrocute effect and also inflicts this effect on enemies that touch it.",
		effect_desc_short		= "A temporary shield is projected around you upon taking damage, causing the electrocute effect on enemies.",
        precursor_types			= "magic_artifact,energy_shield",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		ondamaged_function		= function(damage, message, entity_thats_responsible, is_fatal, projectile_thats_responsible )
			if(isRelicOwned("deflecting_crystal_blue") >= 1) then
				if(not(string.find(message, "damage from material") or (message == "$damage_fire") or (message == "$damage_radioactive"))) then
					local player_children = EntityGetAllChildren(getPlayerEntity())
					if player_children == nil then return end
					local effect_found = false
					for i,child in ipairs(player_children) do
						if(EntityGetName(child) == "temporary_shield_electrocute") then
							effect_found = true
							break
						end
					end
					if(not effect_found) then
						LoadGameEffectEntityTo( getPlayerEntity(), getCustomEffect("effect_temporary_shield_electrocute").path )
					end
				end
			end
		end,
		ondrop_function			= function()
			local player_children = EntityGetAllChildren(getPlayerEntity())
			if player_children == nil then return end
			for i,child in ipairs(player_children) do
				if(EntityGetName(child) == "temporary_shield_electrocute") then
					EntityKill(child)
					break
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "deflecting_medalion,charged_necklace,ancient_root",
	},
	{
		name 					= "deflecting_crystal_yellow",
		ui_name					= "Deflecting Crystal (Yellow)",
		categories				= "player",
		effect_desc				= "Whenever you take damage from any source except fire and \nmaterials, a temporary energy shield with large radius is projected around you. \nThis shield deflects and scatters enemy projectiles, imbuing them with the \ntime bomb effect and also inflicts this effect on enemies that touch it.",
		effect_desc_short		= "A temporary shield is projected around you upon taking damage, causing the time bomb effect on enemies.",
        precursor_types			= "magic_artifact,energy_shield",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		ondamaged_function		= function(damage, message, entity_thats_responsible, is_fatal, projectile_thats_responsible )
			if(isRelicOwned("deflecting_crystal_yellow") >= 1) then
				if(not(string.find(message, "damage from material") or (message == "$damage_fire") or (message == "$damage_radioactive"))) then
					local player_children = EntityGetAllChildren(getPlayerEntity())
					if player_children == nil then return end
					local effect_found = false
					for i,child in ipairs(player_children) do
						if(EntityGetName(child) == "temporary_shield_timebomb") then
							effect_found = true
							break
						end
					end
					if(not effect_found) then
						LoadGameEffectEntityTo( getPlayerEntity(), getCustomEffect("effect_temporary_shield_timebomb").path )
					end
				end
			end
		end,
		ondrop_function			= function()
			local player_children = EntityGetAllChildren(getPlayerEntity())
			if player_children == nil then return end
			for i,child in ipairs(player_children) do
				if(EntityGetName(child) == "temporary_shield_timebomb") then
					EntityKill(child)
					break
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "deflecting_medalion,time_bomb,ancient_root",
	},
	{
		name 					= "diamond_ring_pink_dark",
		ui_name					= "Dark Pink Diamond Ring",
		categories				= "projectiles,status_effects",
		effect_desc				= "Your spells have a 30% chance of creating a mystic target on \ncreatures you hit, making them summon explosive orbs towards themselves.",
		effect_desc_short		= "Your spells create mystic targets on creatures, summoning explosive orbs towards them.",
        precursor_types			= "magic_artifact,ring",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("diamond_ring_pink_dark") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 30 + (5 * getRelicLevel("diamond_ring_pink_dark"))
				if(chance >= 1 and chance <= odds) then
					attachLoadEffectOntoEntitiestoProjectile(projectile_entity_id, getCustomEffect("effect_projectile_target_explosiveorb").path, "plasma_fading_pink")
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "elemental_band,absorbing_crystal",
	},
	{
		name 					= "diamond_ring_inferno_orange",
		ui_name					= "Inferno Orange Diamond Ring",
		categories				= "projectiles,status_effects",
		effect_desc				= "Your spells have a 30% chance of creating a mystic target on \ncreatures you hit, making them summon heavy firebombs towards themselves.",
		effect_desc_short		= "Your spells create mystic targets on creatures, summoning heavy firebombs towards them.",
        precursor_types			= "magic_artifact,ring",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("diamond_ring_inferno_orange") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 30 + (5 * getRelicLevel("diamond_ring_inferno_orange"))
				if(chance >= 1 and chance <= odds) then
					attachLoadEffectOntoEntitiestoProjectile(projectile_entity_id, getCustomEffect("effect_projectile_target_grenade").path, "lava")
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "elemental_band,match_box",
	},
	{
		name 					= "diamond_ring_dark_green",
		ui_name					= "Dark Green Diamond Ring",
		categories				= "projectiles,status_effects",
		effect_desc				= "Your spells have a 30% chance of creating a mystic target on \ncreatures you hit, making them summon acid fireballs towards themselves.",
		effect_desc_short		= "Your spells create mystic targets on creatures, summoning acid fireballs towards them.",
        precursor_types			= "magic_artifact,ring",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("diamond_ring_dark_green") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 30 + (5 * getRelicLevel("diamond_ring_dark_green"))
				if(chance >= 1 and chance <= odds) then
					attachLoadEffectOntoEntitiestoProjectile(projectile_entity_id, getCustomEffect("effect_projectile_target_fireball_acid").path, "acid")
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "elemental_band,bitter_crystal",
	},
	{
		name 					= "diamond_ring_dark_orange",
		ui_name					= "Dark Orange Diamond Ring",
		categories				= "projectiles,status_effects",
		effect_desc				= "Your spells have a 30% chance of creating a mystic target on \ncreatures you hit, making them summon explodebombs towards themselves.",
		effect_desc_short		= "Your spells create mystic targets on creatures, summoning explodebombs towards them.",
        precursor_types			= "magic_artifact,ring",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("diamond_ring_dark_orange") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 30 + (5 * getRelicLevel("diamond_ring_dark_orange"))
				if(chance >= 1 and chance <= odds) then
					attachLoadEffectOntoEntitiestoProjectile(projectile_entity_id, getCustomEffect("effect_projectile_target_explodebomb").path, "fire")
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "elemental_band,exploding_scepter",
	},
	{
		name 					= "diamond_ring_scorching_yellow",
		ui_name					= "Scorching Yellow Diamond Ring",
		categories				= "projectiles,status_effects",
		effect_desc				= "Your spells have a 30% chance of creating a mystic target on \ncreatures you hit, making them summon fireballs towards themselves.",
		effect_desc_short		= "Your spells create mystic targets on creatures, summoning fireballs towards them.",
        precursor_types			= "magic_artifact,ring",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("diamond_ring_scorching_yellow") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 30 + (5 * getRelicLevel("diamond_ring_scorching_yellow"))
				if(chance >= 1 and chance <= odds) then
					attachLoadEffectOntoEntitiestoProjectile(projectile_entity_id, getCustomEffect("effect_projectile_target_fireball").path, "spark_yellow")
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "elemental_band,ember_stone",
	},
	{
		name 					= "diamond_ring_hemoglobin_red",
		ui_name					= "Hemoglobin Red Diamond Ring",
		categories				= "projectiles,status_effects",
		effect_desc				= "Your spells have a 30% chance of creating a mystic target on \ncreatures you hit, making them summon blood fireballs towards themselves.",
		effect_desc_short		= "Your spells create mystic targets on creatures, summoning blood fireballs towards them.",
        precursor_types			= "magic_artifact,ring",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("diamond_ring_hemoglobin_red") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 30 + (5 * getRelicLevel("diamond_ring_hemoglobin_red"))
				if(chance >= 1 and chance <= odds) then
					attachLoadEffectOntoEntitiestoProjectile(projectile_entity_id, getCustomEffect("effect_projectile_target_fireball_blood").path, "blood")
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "elemental_band,blood_gem",
	},
	{
		name 					= "diamond_ring_fireworks_red",
		ui_name					= "Fireworks Red Diamond Ring",
		categories				= "projectiles,status_effects",
		effect_desc				= "Your spells have a 30% chance of creating a mystic target on \ncreatures you hit, making them summon gunpowder fireballs towards themselves.",
		effect_desc_short		= "Your spells create mystic targets on creatures, summoning gunpowder fireballs towards them.",
        precursor_types			= "magic_artifact,ring",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("diamond_ring_fireworks_red") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 30 + (5 * getRelicLevel("diamond_ring_fireworks_red"))
				if(chance >= 1 and chance <= odds) then
					attachLoadEffectOntoEntitiestoProjectile(projectile_entity_id, getCustomEffect("effect_projectile_target_fireball_gunpowder").path, "gunpowder_unstable")
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "elemental_band,firecrackers",
	},
	{
		name 					= "diamond_ring_lazuli_blue",
		ui_name					= "Lazuli Blue Diamond Ring",
		categories				= "projectiles,status_effects",
		effect_desc				= "Your spells have a 30% chance of creating a mystic target on \ncreatures you hit, making them summon water fireballs towards themselves.",
		effect_desc_short		= "Your spells create mystic targets on creatures, summoning water fireballs towards them.",
        precursor_types			= "magic_artifact,ring",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("diamond_ring_lazuli_blue") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 30 + (5 * getRelicLevel("diamond_ring_lazuli_blue"))
				if(chance >= 1 and chance <= odds) then
					attachLoadEffectOntoEntitiestoProjectile(projectile_entity_id, getCustomEffect("effect_projectile_target_fireball_water").path, "spark_blue_dark")
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "elemental_band,aquatic_crystal",
	},
	{
		name 					= "diamond_ring_purple_poison",
		ui_name					= "Poison Purple Diamond Ring",
		categories				= "projectiles,status_effects",
		effect_desc				= "Your spells have a 30% chance of creating a mystic target on \ncreatures you hit, making them summon poison fireballs towards themselves.",
		effect_desc_short		= "Your spells create mystic targets on creatures, summoning poison fireballs towards them.",
        precursor_types			= "magic_artifact,ring",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("diamond_ring_purple_poison") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 30 + (5 * getRelicLevel("diamond_ring_purple_poison"))
				if(chance >= 1 and chance <= odds) then
					attachLoadEffectOntoEntitiestoProjectile(projectile_entity_id, getCustomEffect("effect_projectile_target_fireball_poison").path, "spark_purple_bright")
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "elemental_band,venomous_crystal",
	},
	{
		name 					= "blood_cell",
		ui_name					= "Blood Cell",
		categories				= "projectiles",
		effect_desc				= "Your spells have a 20% chance of exploding into a \nbarrage of blood fireballs when they explode or hit something. \nIt also synergizes with Earth Core Fragment relic effect.",
		effect_desc_short		= "Your spells explode into blood fireballs on impact.",
        precursor_types			= "magic_artifact",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("blood_cell") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 20 + (5 * getRelicLevel("blood_cell"))
				if(chance >= 1 and chance <= odds) then	
					addBuffVisualIndicator_Projectile(projectile_entity_id, "blood")
					attachLoadEntityOnExplosiontoProjectile(projectile_entity_id, getLoadOnExplosion("launch_circular_projectiles_fireballs_blood").path)
				end
			end
		end,
		ongroundpound_function	= function( entity_id)
			if(isRelicOwned("blood_cell") >= 1) then
				LoadGameEffectEntityTo( entity_id, getCustomEffect("effect_shoot_projectiles_above_fireball_blood").path )
			end
		end,
	},
	{
		name 					= "water_pipe",
		ui_name					= "Water Pipe",
		categories				= "projectiles",
		effect_desc				= "Your spells have a 20% chance of exploding into a \nbarrage of water fireballs when they explode or hit something. \nIt also synergizes with Earth Core Fragment relic effect.",
		effect_desc_short		= "Your spells explode into water fireballs on impact.",
        precursor_types			= "gadget",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("water_pipe") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 20 + (5 * getRelicLevel("water_pipe"))
				if(chance >= 1 and chance <= odds) then	
					addBuffVisualIndicator_Projectile(projectile_entity_id, "spark_blue")
					attachLoadEntityOnExplosiontoProjectile(projectile_entity_id, getLoadOnExplosion("launch_circular_projectiles_fireballs_water").path)
				end
			end
		end,
		ongroundpound_function	= function( entity_id)
			if(isRelicOwned("water_pipe") >= 1) then
				LoadGameEffectEntityTo( entity_id, getCustomEffect("effect_shoot_projectiles_above_fireball_water").path )
			end
		end,
	},
	{
		name 					= "rhodonite_necklace",
		ui_name					= "Rhodonite Necklace",
		categories				= "projectiles",
		effect_desc				= "Your spells have a 20% chance of exploding into a \nbarrage of explosive orbs when they explode or hit something. \nIt also synergizes with Earth Core Fragment relic effect.",
		effect_desc_short		= "Your spells explode into explosive orbs on impact.",
        precursor_types			= "magic_artifact,necklace",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("rhodonite_necklace") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 20 + (5 * getRelicLevel("rhodonite_necklace"))
				if(chance >= 1 and chance <= odds) then	
					addBuffVisualIndicator_Projectile(projectile_entity_id, "plasma_fading_pink")
					attachLoadEntityOnExplosiontoProjectile(projectile_entity_id, getLoadOnExplosion("launch_circular_projectiles_explosiveorbs").path)
				end
			end
		end,
		ongroundpound_function	= function( entity_id)
			if(isRelicOwned("rhodonite_necklace") >= 1) then
				LoadGameEffectEntityTo( entity_id, getCustomEffect("effect_shoot_projectiles_above_explosive_orb_pink").path )
			end
		end,
	},
	{
		name 					= "malachite_necklace",
		ui_name					= "Malachite Necklace",
		categories				= "projectiles",
		effect_desc				= "Your spells have a 20% chance of exploding into a \nbarrage of acid fireballs when they explode or hit something. \nIt also synergizes with Earth Core Fragment relic effect.",
		effect_desc_short		= "Your spells explode into acid fireballs on impact.",
        precursor_types			= "magic_artifact,necklace",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("malachite_necklace") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 20 + (5 * getRelicLevel("malachite_necklace"))
				if(chance >= 1 and chance <= odds) then	
					addBuffVisualIndicator_Projectile(projectile_entity_id, "plasma_fading_green")
					attachLoadEntityOnExplosiontoProjectile(projectile_entity_id, getLoadOnExplosion("launch_circular_projectiles_fireballs_acid").path)
				end
			end
		end,
		ongroundpound_function	= function( entity_id)
			if(isRelicOwned("malachite_necklace") >= 1) then
				LoadGameEffectEntityTo( entity_id, getCustomEffect("effect_shoot_projectiles_above_fireball_acid").path )
			end
		end,
	},
	{
		name 					= "meteorite_necklace",
		ui_name					= "Meteorite Necklace",
		categories				= "projectiles",
		effect_desc				= "Your spells have a 20% chance of exploding into a \nbarrage of gunpowder fireballs when they explode or hit something. \nIt also synergizes with Earth Core Fragment relic effect.",
		effect_desc_short		= "Your spells explode into gunpowder fireballs on impact.",
        precursor_types			= "magic_artifact,necklace",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("meteorite_necklace") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 20 + (5 * getRelicLevel("meteorite_necklace"))
				if(chance >= 1 and chance <= odds) then	
					addBuffVisualIndicator_Projectile(projectile_entity_id, "gunpowder_unstable")
					attachLoadEntityOnExplosiontoProjectile(projectile_entity_id, getLoadOnExplosion("launch_circular_projectiles_fireballs_gunpowder").path)
				end
			end
		end,
		ongroundpound_function	= function( entity_id)
			if(isRelicOwned("meteorite_necklace") >= 1) then
				LoadGameEffectEntityTo( entity_id, getCustomEffect("effect_shoot_projectiles_above_fireball_gunpowder").path )
			end
		end,
	},
	{
		name 					= "sapphire_necklace",
		ui_name					= "Sapphire Necklace",
		categories				= "projectiles",
		effect_desc				= "Your spells have a 20% chance of exploding into a \nbarrage of poison fireballs when they explode or hit something. \nIt also synergizes with Earth Core Fragment relic effect.",
		effect_desc_short		= "Your spells explode into poison fireballs on impact.",
        precursor_types			= "magic_artifact,necklace",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("sapphire_necklace") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 20 + (5 * getRelicLevel("sapphire_necklace"))
				if(chance >= 1 and chance <= odds) then	
					addBuffVisualIndicator_Projectile(projectile_entity_id, "spark_purple_bright")
					attachLoadEntityOnExplosiontoProjectile(projectile_entity_id, getLoadOnExplosion("launch_circular_projectiles_fireballs_poison").path)
				end
			end
		end,
		ongroundpound_function	= function( entity_id)
			if(isRelicOwned("sapphire_necklace") >= 1) then
				LoadGameEffectEntityTo( entity_id, getCustomEffect("effect_shoot_projectiles_above_fireball_poison").path )
			end
		end,
	},
	{
		name 					= "artifact_of_patience",
		ui_name					= "Artifact of Patience",
		categories				= "projectiles",
		effect_desc				= "When you shoot your spells very pausedly, they gain \na 15x base damage multiplier, but if you shoot too fast, you are \ninflicted with the slap, bleed, freeze or electrocute effects.",
		effect_desc_short		= "Shooting pausedly gives your spells great base damage boost.",
        precursor_types			= "magic_artifact",
		upgrade_effect_desc		= "For each upgrade level, spell damage multiplier increases by 2.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("artifact_of_patience") >= 1) then
				
				-- if shoot frame is not set yet, sets it then returns
				local shoot_frame = getInternalVariableValue(getPlayerEntity(), "shoot_frame", "value_int")
				if(shoot_frame == 0) then
					setInternalVariableValue(getPlayerEntity(), "shoot_frame", "value_int", GameGetFrameNum())
					return
				else
					local current_frame_num = GameGetFrameNum()
					if(current_frame_num - shoot_frame >= 100) then -- paused shot, gives bonus
						local paused_shot_multiplier = 15 + (2 * getRelicLevel("artifact_of_patience"))
						GamePrint("Relic effect: " .. paused_shot_multiplier .. " base damage multiplier for shooting pausedly")
						addBuffVisualIndicator_Projectile(projectile_entity_id, "plasma_fading_pink")
						multiplyProjectileBaseDamage(projectile_entity_id, paused_shot_multiplier)
					else -- shooting too fast, applies penalty
						local possible_effects = {
							{ name = "Bleeding", entity_name = getCustomEffect("effect_bleeding").path },
							{ name = "Slap", entity_name = getCustomEffect("effect_slap").path },
							{ name = "Freeze", entity_name = "data/entities/misc/effect_frozen.xml" },
							{ name = "Electrocute", entity_name = "data/entities/misc/effect_electricity.xml" }
						}
						local inflicted_effect = possible_effects[Random(1,#possible_effects)]
						GamePrint("Relic penalty: " .. inflicted_effect.name .. " effect inflicted for shooting too fast")
						LoadGameEffectEntityTo( GetUpdatedEntityID(), inflicted_effect.entity_name )
					end
					setInternalVariableValue(getPlayerEntity(), "shoot_frame", "value_int", GameGetFrameNum())
				end
			end
		end,
		excludes_relics			= "crest_of_tenderness",
		excludes_relics_desc	= "It can't be used along with Crest of Tenderness relic",
		excludes_relics_tags	= "artifact_of_patience",
	},
	{
		name 					= "artifact_of_restraint",
		ui_name					= "Artifact of Restraint",
		categories				= "projectiles",
		effect_desc				= "When you shoot your spells very pausedly, additional \nprojectiles are shot together based on the amount of spell slots your current wand has, but \nif you shoot too fast, your spells are destroyed immediatelly once cast.",
		effect_desc_short		= "Shooting pausedly adds more spells to your cast.",
        precursor_types			= "magic_artifact",
		upgrade_effect_desc		= "For each upgrade level, more projectiles are added per cast.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("artifact_of_restraint") >= 1) then
				
				-- if shoot frame is not set yet, sets it then returns
				local shoot_frame = getInternalVariableValue(getPlayerEntity(), "shoot_frame", "value_int")
				if(shoot_frame == 0) then
					setInternalVariableValue(getPlayerEntity(), "shoot_frame", "value_int", GameGetFrameNum())
					return
				else
					local current_frame_num = GameGetFrameNum()
					if(current_frame_num - shoot_frame >= 100) then -- paused shot, gives bonus
						GamePrint("Relic effect: more spells added on cast for shooting pausedly")
						
						local current_wand_id = getCurrentlyEquippedWandId()
						if(current_wand_id ~= -1) then
							local free_slots, spells = getWandEmptyandOccupiedSlots(getCurrentlyEquippedWandId())
							local multiplier = getRelicLevel("artifact_of_restraint")
							if(multiplier == 0) then
								multiplier = 1
							end
							for i=1,(spells + free_slots) * multiplier do
								local x, y = EntityGetTransform(projectile_entity_id)
								shootProjectileBasedonAnotherProjectile(projectile_entity_id, getProjectileDefinitionPath(projectile_entity_id), x, y, 150)
							end
						end
						
					else -- shooting too fast, applies penalty
						projectileToggleExplosionOnDeath(projectile_entity_id, false)
						EntityKill(projectile_entity_id)
						GamePrint("Relic penalty: spell destroyed for shooting too fast")
						
					end
					setInternalVariableValue(getPlayerEntity(), "shoot_frame", "value_int", GameGetFrameNum())
				end
			end
		end,
		excludes_relics			= "crest_of_tenderness",
		excludes_relics_desc	= "It can't be used along with Crest of Tenderness relic",
		excludes_relics_tags	= "artifact_of_restraint",
	},
	{
		name 					= "artifact_of_discipline",
		ui_name					= "Artifact of Discipline",
		categories				= "projectiles",
		effect_desc				= "When you shoot your spells very pausedly, a temporary large radius energy \nshield is projected around you. This shield deflects and heavily increase the base damage \nof enemy projectiles, and also damages enemies that come in contact with it. \nIf you shoot too fast, you are mutated into sheep.",
		effect_desc_short		= "Shooting pausedly projects a deflecting shield around you.",
        precursor_types			= "magic_artifact",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("artifact_of_discipline") >= 1) then
				
				-- if shoot frame is not set yet, sets it then returns
				local shoot_frame = getInternalVariableValue(getPlayerEntity(), "shoot_frame", "value_int")
				if(shoot_frame == 0) then
					setInternalVariableValue(getPlayerEntity(), "shoot_frame", "value_int", GameGetFrameNum())
					return
				else
					local current_frame_num = GameGetFrameNum()
					if(current_frame_num - shoot_frame >= 100) then -- paused shot, gives bonus
					
						GamePrint("Relic effect: deflecting shield projected for shooting pausedly")
						local player_children = EntityGetAllChildren(getPlayerEntity())
						if player_children == nil then return end
						local effect_found = false
						for i,child in ipairs(player_children) do
							if(EntityGetName(child) == "temporary_shield_damage") then
								effect_found = true
								break
							end
						end
						if(not effect_found) then
							LoadGameEffectEntityTo( getPlayerEntity(), getCustomEffect("effect_temporary_shield_damage").path )
						end
							
					else -- shooting too fast, applies penalty
						
						GamePrint("Relic penalty: mutated into sheep for shooting too fast")
						LoadGameEffectEntityTo( GetUpdatedEntityID(), "data/entities/misc/effect_polymorph.xml" )
						
					end
					setInternalVariableValue(getPlayerEntity(), "shoot_frame", "value_int", GameGetFrameNum())
				end
			end
		end,
		ondrop_function			= function()
			local player_children = EntityGetAllChildren(getPlayerEntity())
			if player_children == nil then return end
			for i,child in ipairs(player_children) do
				if(EntityGetName(child) == "temporary_shield_damage") then
					EntityKill(child)
					break
				end
			end
		end,
		excludes_relics			= "crest_of_tenderness",
		excludes_relics_desc	= "It can't be used along with Crest of Tenderness relic",
		excludes_relics_tags	= "artifact_of_discipline",
	},
	{
		name 					= "artifact_of_serenity",
		ui_name					= "Artifact of Serenity",
		categories				= "projectiles",
		effect_desc				= "When you shoot your spells very pausedly, 5 gold is instantly earned, but \nif you shoot too fast, you get damaged for the base damage of the \nspell you just shot.",
		effect_desc_short		= "Shooting pausedly earns you gold.",
        precursor_types			= "magic_artifact",
		upgrade_effect_desc		= "For each upgrade level, amount of gold earned is increased by 2.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("artifact_of_serenity") >= 1) then
				
				-- if shoot frame is not set yet, sets it then returns
				local shoot_frame = getInternalVariableValue(getPlayerEntity(), "shoot_frame", "value_int")
				if(shoot_frame == 0) then
					setInternalVariableValue(getPlayerEntity(), "shoot_frame", "value_int", GameGetFrameNum())
					return
				else
					local current_frame_num = GameGetFrameNum()
					if(current_frame_num - shoot_frame >= 100) then -- paused shot, gives bonus
						local gold_earned = 5 + (2 * getRelicLevel("artifact_of_serenity"))
						GamePrint("Relic effect: " .. gold_earned .. " gold earned for shooting pausedly")
						addPlayerGold(gold_earned)
					else -- shooting too fast, applies penalty
						EntityInflictDamage( getPlayerEntity(), getProjectileBaseDamage(projectile_entity_id), "DAMAGE_PROJECTILE", "Artifact of Serenity penalty", "BLOOD_SPRAY", 0, 0)
						GamePrint("Relic penalty: damage inflicted for shooting too fast")
					end
					setInternalVariableValue(getPlayerEntity(), "shoot_frame", "value_int", GameGetFrameNum())
				end
			end
		end,
		excludes_relics			= "crest_of_tenderness",
		excludes_relics_desc	= "It can't be used along with Crest of Tenderness relic",
		excludes_relics_tags	= "artifact_of_serenity",
	},
	{
		name 					= "artifact_of_fortitude",
		ui_name					= "Artifact of Fortitude",
		categories				= "projectiles",
		effect_desc				= "When you shoot your spells very pausedly, as long as there are enemies \nalive around you, you are inflicted with the healing \neffect, but if you shoot too fast, you are inflicted with the \nexploding effect.",
		effect_desc_short		= "Shooting pausedly gives your spells great base damage boost.",
        precursor_types			= "magic_artifact",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("artifact_of_fortitude") >= 1) then
				
				-- if shoot frame is not set yet, sets it then returns
				local shoot_frame = getInternalVariableValue(getPlayerEntity(), "shoot_frame", "value_int")
				if(shoot_frame == 0) then
					setInternalVariableValue(getPlayerEntity(), "shoot_frame", "value_int", GameGetFrameNum())
					return
				else
					local current_frame_num = GameGetFrameNum()
					if(current_frame_num - shoot_frame >= 100) then -- paused shot, gives bonus
						local player_x, player_y = EntityGetTransform(GetUpdatedEntityID())
						local enemies = EntityGetInRadiusWithTag( player_x, player_y, 200, "enemy" )
						if(#enemies >= 1 ) then
							GamePrint("Relic effect: Healing Effect inflicted for shooting pausedly")
							LoadGameEffectEntityTo( GetUpdatedEntityID(), "data/entities/misc/effect_regeneration.xml" )
						end
					else -- shooting too fast, applies penalty

						GamePrint("Relic penalty: Exploding effect inflicted for shooting too fast")
						LoadGameEffectEntityTo( GetUpdatedEntityID(), getCustomEffect("effect_exploding").path )
					end
					setInternalVariableValue(getPlayerEntity(), "shoot_frame", "value_int", GameGetFrameNum())
				end
			end
		end,
		excludes_relics			= "crest_of_tenderness",
		excludes_relics_desc	= "It can't be used along with Crest of Tenderness relic",
		excludes_relics_tags	= "artifact_of_fortitude",
	},
	{
		name 					= "crest_of_tenderness",
		ui_name					= "Crest of Tenderness",
		categories				= "projectiles",
		effect_desc				= "When you shoot your spells very pausedly, your spells get base damage \nboost, additional shots are added based on the amount of slots \nin your current wand, you receive gold, the healing effect is applied \non you and a large radius deflecting shield is projected around \nyou, but if you shoot too fast there's a 50% you are inflicted with the \nbleeding, exploding, slap, wind, freeze or electrocute effects.",
		effect_desc_short		= "Shooting pausedly gives you various buffs.",
        precursor_types			= "magic_artifact",
		upgrade_effect_desc		= "For each upgrade level, buffs applied are improved.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("crest_of_tenderness") >= 1) then
				
				-- if shoot frame is not set yet, sets it then returns
				local shoot_frame = getInternalVariableValue(getPlayerEntity(), "shoot_frame", "value_int")
				if(shoot_frame == 0) then
					setInternalVariableValue(getPlayerEntity(), "shoot_frame", "value_int", GameGetFrameNum())
					return
				else
					local current_frame_num = GameGetFrameNum()
					if(current_frame_num - shoot_frame >= 100) then -- paused shot, gives bonus
						
						GamePrint("Relic effect: buffs activated for shooting pausedly")
						
						-- multi-shots
						local paused_shot_multiplier = 15 + (2 * getRelicLevel("crest_of_tenderness"))
						local current_wand_id = getCurrentlyEquippedWandId()
						if(current_wand_id ~= -1) then
							local free_slots, spells = getWandEmptyandOccupiedSlots(getCurrentlyEquippedWandId())
							local multiplier = getRelicLevel("crest_of_tenderness")
							if(multiplier == 0) then
								multiplier = 1
							end
							for i=1,(spells + free_slots) * multiplier do
								local x, y = EntityGetTransform(projectile_entity_id)
								shootProjectileBasedonAnotherProjectile(projectile_entity_id, getProjectileDefinitionPath(projectile_entity_id), x, y, 150)
							end
						end
						
						-- base damage boost
						addBuffVisualIndicator_Projectile(projectile_entity_id, "plasma_fading_pink")
						multiplyProjectileBaseDamage(projectile_entity_id, paused_shot_multiplier)
						
						-- energy shield
						GamePrint("Relic effect: deflecting shield projected for shooting pausedly")
						local player_children = EntityGetAllChildren(getPlayerEntity())
						if player_children == nil then return end
						local effect_found = false
						for i,child in ipairs(player_children) do
							if(EntityGetName(child) == "temporary_shield_damage") then
								effect_found = true
								break
							end
						end
						if(not effect_found) then
							LoadGameEffectEntityTo( getPlayerEntity(), getCustomEffect("effect_temporary_shield_damage").path )
						end
						
						-- gold bonus
						local gold_earned = 5 + (2 * getRelicLevel("crest_of_tenderness"))
						addPlayerGold(gold_earned)
						
						-- healing effect
						LoadGameEffectEntityTo( GetUpdatedEntityID(), "data/entities/misc/effect_regeneration.xml" )
						
					else -- shooting too fast, applies penalty
						if(Random(1,2) == 1) then
							local possible_effects = {
								{ name = "Bleeding", entity_name = getCustomEffect("effect_bleeding").path },
								{ name = "Slap", entity_name = getCustomEffect("effect_slap").path },
								{ name = "Exploding", entity_name = getCustomEffect("effect_exploding").path },
								{ name = "Wind", entity_name = getCustomEffect("effect_wind").path },
								{ name = "Freeze", entity_name = "data/entities/misc/effect_frozen.xml" },
								{ name = "Electrocute", entity_name = "data/entities/misc/effect_electricity.xml" }
							}
							local inflicted_effect = possible_effects[Random(1,#possible_effects)]
							GamePrint("Relic penalty: " .. inflicted_effect.name .. " effect inflicted for shooting too fast")
							LoadGameEffectEntityTo( GetUpdatedEntityID(), inflicted_effect.entity_name )
						end
					end
					setInternalVariableValue(getPlayerEntity(), "shoot_frame", "value_int", GameGetFrameNum())
				end
			end
		end,
		ondrop_function			= function()
			local player_children = EntityGetAllChildren(getPlayerEntity())
			if player_children == nil then return end
			for i,child in ipairs(player_children) do
				if(EntityGetName(child) == "temporary_shield_damage") then
					EntityKill(child)
					break
				end
			end
		end,
		excludes_relics			= "artifact_of_discipline,artifact_of_fortitude,artifact_of_patience,artifact_of_restraint,artifact_of_serenity",
		excludes_relics_desc	= "It can't be used along with basic paused spell cast bonus related relics",
		excludes_relics_tags	= "artifact_of_discipline,artifact_of_fortitude,artifact_of_patience,artifact_of_restraint,artifact_of_serenity",
		combine_exclusive		= true,
		combine_requires_relics	= "artifact_of_discipline,artifact_of_fortitude,artifact_of_patience,artifact_of_restraint,artifact_of_serenity",
	},
	{
		name 					= "deflecting_crystal_rainbow",
		ui_name					= "Deflecting Crystal (Rainbow)",
		categories				= "player",
		effect_desc				= "A pulsating energy shield emanates from you. This shield deflects \nenemy projectiles and shoots back spells from your wand together when \nit happens. The deflected projectile will also explode into a \nbarrage of spells of your wand. Enemies touched by the \nshield shoot random spells from your wand at random \ndirections and also release a barrage of your spells on death.",
		effect_desc_short		= "A pulsating shield that causes various effects related to the spells of your wand.",
        precursor_types			= "magic_artifact,energy_shield",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			LoadGameEffectEntityTo( getPlayerEntity(), getCustomEffect("effect_pulsating_shield_rainbow").path )
		end,
		ondrop_function			= function()
			local player_children = EntityGetAllChildren(getPlayerEntity())
			if player_children == nil then return end
			for i,child in ipairs(player_children) do
				if(EntityGetName(child) == "pulsating_shield_rainbow") then
					EntityKill(child)
					break
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "explosive_mana_crystal,spell_caster_nanite,butterfly_ring",
	},
	{
		name 					= "companion_necklace_guardianship",
		ui_name					= "Necklace of Guardianship",
		categories				= "companions",
		effect_desc				= "Companions following you that are not naturally capable of \ndeflecting enemy projectiles gain a deflecting shield.",
		effect_desc_short		= "Companions following you gain a deflecting shield.",
        precursor_types			= "companion",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			addNewSimplifiedInternalVariable( getPlayerEntity(), "companion_shield")
		end,
		ondrop_function			= function()
			removeSimplifiedInternalVariable( getPlayerEntity(), "companion_shield")
		end,
	},
	{
		name 					= "companion_necklace_impairment",
		ui_name					= "Necklace of Impairment",
		categories				= "companions",
		effect_desc				= "Companions following you that are not naturally capable of \ndealing contact damage to enemies gain the ability to inflict damage on enemies they touch.",
		effect_desc_short		= "Companions following you gain the ability to contact damage enemies.",
        precursor_types			= "companion",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			addNewSimplifiedInternalVariable( getPlayerEntity(), "companion_contact_damage")
		end,
		ondrop_function			= function()
			removeSimplifiedInternalVariable( getPlayerEntity(), "companion_contact_damage")
		end,
	},
	{
		name 					= "companion_necklace_electrocution",
		ui_name					= "Necklace of Electrocution",
		categories				= "companions",
		effect_desc				= "Companions following you gain the ability to electrocute enemies they touch.",
		effect_desc_short		= "Companions following you gain the ability to electrocute enemies they touch.",
        precursor_types			= "companion",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			addNewSimplifiedInternalVariable( getPlayerEntity(), "companion_contact_electrocution")
		end,
		ondrop_function			= function()
			removeSimplifiedInternalVariable( getPlayerEntity(), "companion_contact_electrocution")
		end,
	},
	{
		name 					= "companion_necklace_chilling",
		ui_name					= "Necklace of Chilling",
		categories				= "companions",
		effect_desc				= "Companions following you gain the ability to freeze enemies they touch.",
		effect_desc_short		= "Companions following you gain the ability to freeze enemies they touch.",
        precursor_types			= "companion",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			addNewSimplifiedInternalVariable( getPlayerEntity(), "companion_contact_freeze")
		end,
		ondrop_function			= function()
			removeSimplifiedInternalVariable( getPlayerEntity(), "companion_contact_freeze")
		end,
	},
	{
		name 					= "companion_necklace_deviation",
		ui_name					= "Necklace of Deviation",
		categories				= "companions",
		effect_desc				= "Companions following you gain the ability to mutate enemies they touch.",
		effect_desc_short		= "Companions following you gain the ability to mutate enemies they touch.",
        precursor_types			= "companion",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			addNewSimplifiedInternalVariable( getPlayerEntity(), "companion_contact_poly")
		end,
		ondrop_function			= function()
			removeSimplifiedInternalVariable( getPlayerEntity(), "companion_contact_poly")
		end,
	},
	{
		name 					= "companion_necklace_guidance",
		ui_name					= "Necklace of Guidance",
		categories				= "companions",
		effect_desc				= "Companions following you that are not able to shoot gain \nthe ability to double the damage of your spells and redirect them towards nearby enemies.",
		effect_desc_short		= "Non-shooter companions following you gain the ability to redirect your spells towards nearby enemies.",
        precursor_types			= "companion",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			addNewSimplifiedInternalVariable( getPlayerEntity(), "companion_projectile_guide")
		end,
		ondrop_function			= function()
			removeSimplifiedInternalVariable( getPlayerEntity(), "companion_projectile_guide")
		end,
	},
	{
		name 					= "companion_necklace_detonation",
		ui_name					= "Necklace of Detonation",
		categories				= "companions",
		effect_desc				= "Companions following you gain the ability to inflict the exploding effect on enemies they touch.",
		effect_desc_short		= "Companions following you inflict the exploding effect on enemies they touch.",
        precursor_types			= "companion",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			addNewSimplifiedInternalVariable( getPlayerEntity(), "companion_contact_exploding")
		end,
		ondrop_function			= function()
			removeSimplifiedInternalVariable( getPlayerEntity(), "companion_contact_exploding")
		end,
	},
	{
		name 					= "companion_necklace_draining",
		ui_name					= "Necklace of Draining",
		categories				= "companions",
		effect_desc				= "Companions following you gain the ability to inflict the bleeding effect on enemies they touch.",
		effect_desc_short		= "Companions following you inflict the bleeding effect on enemies they touch.",
        precursor_types			= "companion",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			addNewSimplifiedInternalVariable( getPlayerEntity(), "companion_contact_bleeding")
		end,
		ondrop_function			= function()
			removeSimplifiedInternalVariable( getPlayerEntity(), "companion_contact_bleeding")
		end,
	},
	{
		name 					= "companion_necklace_laceration",
		ui_name					= "Necklace of Laceration",
		categories				= "companions",
		effect_desc				= "Companions following you gain the ability to inflict the slash effect on enemies they touch.",
		effect_desc_short		= "Companions following you inflict the slash effect on enemies they touch.",
        precursor_types			= "companion",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			addNewSimplifiedInternalVariable( getPlayerEntity(), "companion_contact_slash")
		end,
		ondrop_function			= function()
			removeSimplifiedInternalVariable( getPlayerEntity(), "companion_contact_slash")
		end,
	},
	{
		name 					= "companion_necklace_ventilation",
		ui_name					= "Necklace of Ventilation",
		categories				= "companions",
		effect_desc				= "Companions following you gain the ability to inflict the wind effect on enemies they touch.",
		effect_desc_short		= "Companions following you inflict the wind effect on enemies they touch.",
        precursor_types			= "companion",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			addNewSimplifiedInternalVariable( getPlayerEntity(), "companion_contact_wind")
		end,
		ondrop_function			= function()
			removeSimplifiedInternalVariable( getPlayerEntity(), "companion_contact_wind")
		end,
	},
	{
		name 					= "companion_necklace_smacking",
		ui_name					= "Necklace of Smacking",
		categories				= "companions",
		effect_desc				= "Companions following you gain the ability to inflict the slap effect on enemies they touch.",
		effect_desc_short		= "Companions following you inflict the slap effect on enemies they touch.",
        precursor_types			= "companion",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			addNewSimplifiedInternalVariable( getPlayerEntity(), "companion_contact_slap")
		end,
		ondrop_function			= function()
			removeSimplifiedInternalVariable( getPlayerEntity(), "companion_contact_slap")
		end,
	},
	{
		name 					= "companion_necklace_countdown",
		ui_name					= "Necklace of Countdown",
		categories				= "companions",
		effect_desc				= "Companions following you gain the ability to inflict the time bomb effect on enemies they touch.",
		effect_desc_short		= "Companions following you inflict the time bomb effect on enemies they touch.",
        precursor_types			= "companion",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			addNewSimplifiedInternalVariable( getPlayerEntity(), "companion_contact_time_bomb")
		end,
		ondrop_function			= function()
			removeSimplifiedInternalVariable( getPlayerEntity(), "companion_contact_time_bomb")
		end,
	},
	{
		name 					= "companion_necklace_attraction",
		ui_name					= "Necklace of Attraction",
		categories				= "companions",
		effect_desc				= "Companions following you gain the ability to inflict the attraction effect on enemies they touch.",
		effect_desc_short		= "Companions following you inflict the attraction effect on enemies they touch.",
        precursor_types			= "companion",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			addNewSimplifiedInternalVariable( getPlayerEntity(), "companion_contact_attraction")
		end,
		ondrop_function			= function()
			removeSimplifiedInternalVariable( getPlayerEntity(), "companion_contact_attraction")
		end,
	},
	{
		name 					= "elemental_buddy_seed_water",
		ui_name					= "Water Elemental Buddy Seed",
		categories				= "companions,materials",
		effect_desc				= "An elemental buddy capable of spitting water on enemies joins you.",
		effect_desc_short		= "An elemental buddy capable of spitting water on enemies joins you.",
        precursor_types			= "companion",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			attachCompaniontoEntity(getPlayerEntity(), "elemental_buddy_water", "mortal")
		end,
		ondrop_function			= function()
			detachCompanionfromEntity(getPlayerEntity(), "elemental_buddy_water")
		end,
		requires_relic			= "elemental_pot",
	},
	{
		name 					= "elemental_buddy_seed_lava",
		ui_name					= "Lava Elemental Buddy Seed",
		categories				= "companions,materials",
		effect_desc				= "An elemental buddy capable of spitting lava on enemies joins you.",
		effect_desc_short		= "An elemental buddy capable of spitting lava on enemies joins you.",
        precursor_types			= "companion",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			attachCompaniontoEntity(getPlayerEntity(), "elemental_buddy_lava", "mortal")
		end,
		ondrop_function			= function()
			detachCompanionfromEntity(getPlayerEntity(), "elemental_buddy_lava")
		end,
		requires_relic			= "elemental_pot",
	},
	{
		name 					= "elemental_buddy_seed_acid",
		ui_name					= "Acid Elemental Buddy Seed",
		categories				= "companions,materials",
		effect_desc				= "An elemental buddy capable of spitting acid on enemies joins you.",
		effect_desc_short		= "An elemental buddy capable of spitting acid on enemies joins you.",
        precursor_types			= "companion",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			attachCompaniontoEntity(getPlayerEntity(), "elemental_buddy_acid", "mortal")
		end,
		ondrop_function			= function()
			detachCompanionfromEntity(getPlayerEntity(), "elemental_buddy_acid")
		end,
		requires_relic			= "elemental_pot",
	},
	{
		name 					= "elemental_buddy_seed_poison",
		ui_name					= "Poison Elemental Buddy Seed",
		categories				= "companions,materials",
		effect_desc				= "An elemental buddy capable of spitting poison on enemies joins you.",
		effect_desc_short		= "An elemental buddy capable of spitting poison on enemies joins you.",
        precursor_types			= "companion",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			attachCompaniontoEntity(getPlayerEntity(), "elemental_buddy_poison", "mortal")
		end,
		ondrop_function			= function()
			detachCompanionfromEntity(getPlayerEntity(), "elemental_buddy_poison")
		end,
		requires_relic			= "elemental_pot",
	},
	{
		name 					= "elemental_buddy_seed_poly",
		ui_name					= "Polymorphine Elemental Buddy Seed",
		categories				= "companions,materials",
		effect_desc				= "An elemental buddy capable of spitting polymorphine on enemies joins you.",
		effect_desc_short		= "An elemental buddy capable of spitting polymorphine on enemies joins you.",
        precursor_types			= "companion",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			attachCompaniontoEntity(getPlayerEntity(), "elemental_buddy_poly", "mortal")
		end,
		ondrop_function			= function()
			detachCompanionfromEntity(getPlayerEntity(), "elemental_buddy_poly")
		end,
		requires_relic			= "elemental_pot",
	},
	{
		name 					= "elemental_buddy_seed_toxic_sludge",
		ui_name					= "Toxic Sludge Elemental Buddy Seed",
		categories				= "companions,materials",
		effect_desc				= "An elemental buddy capable of spitting toxic sludge on enemies joins you.",
		effect_desc_short		= "An elemental buddy capable of spitting toxic sludge on enemies joins you.",
        precursor_types			= "companion",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			attachCompaniontoEntity(getPlayerEntity(), "elemental_buddy_toxic_sludge", "mortal")
		end,
		ondrop_function			= function()
			detachCompanionfromEntity(getPlayerEntity(), "elemental_buddy_toxic_sludge")
		end,
		requires_relic			= "elemental_pot",
	},
	{
		name 					= "elemental_buddy_seed_gunpowder",
		ui_name					= "Gunpowder Elemental Buddy Seed",
		categories				= "companions,materials",
		effect_desc				= "An elemental buddy capable of spitting gunpowder on enemies joins you.",
		effect_desc_short		= "An elemental buddy capable of spitting gunpowder on enemies joins you.",
        precursor_types			= "companion",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			attachCompaniontoEntity(getPlayerEntity(), "elemental_buddy_gunpowder", "mortal")
		end,
		ondrop_function			= function()
			detachCompanionfromEntity(getPlayerEntity(), "elemental_buddy_gunpowder")
		end,
		requires_relic			= "elemental_pot",
	},
	{
		name 					= "elemental_buddy_seed_freezing_liquid",
		ui_name					= "Freezing Liquid Elemental Buddy Seed",
		categories				= "companions,materials",
		effect_desc				= "An elemental buddy capable of spitting freezing liquid on enemies joins you.",
		effect_desc_short		= "An elemental buddy capable of spitting freezing liquid on enemies joins you.",
        precursor_types			= "companion",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			attachCompaniontoEntity(getPlayerEntity(), "elemental_buddy_freezing_liquid", "mortal")
		end,
		ondrop_function			= function()
			detachCompanionfromEntity(getPlayerEntity(), "elemental_buddy_freezing_liquid")
		end,
		requires_relic			= "elemental_pot",
	},
	{
		name 					= "elemental_buddy_seed_urine",
		ui_name					= "Urine Elemental Buddy Seed",
		categories				= "companions,materials",
		effect_desc				= "An elemental buddy capable of spitting urine on enemies joins you.",
		effect_desc_short		= "An elemental buddy capable of spitting urine on enemies joins you.",
        precursor_types			= "companion",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			attachCompaniontoEntity(getPlayerEntity(), "elemental_buddy_urine", "mortal")
		end,
		ondrop_function			= function()
			detachCompanionfromEntity(getPlayerEntity(), "elemental_buddy_urine")
		end,
		requires_relic			= "elemental_pot",
	},
	{
		name 					= "elemental_buddy_seed_corrosive_blood",
		ui_name					= "Corrosive Blood Elemental Buddy Seed",
		categories				= "companions,materials",
		effect_desc				= "An elemental buddy capable of spitting corrosive blood on enemies joins you.",
		effect_desc_short		= "An elemental buddy capable of spitting corrosive blood on enemies joins you.",
        precursor_types			= "companion",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			attachCompaniontoEntity(getPlayerEntity(), "elemental_buddy_corrosive_blood", "mortal")
		end,
		ondrop_function			= function()
			detachCompanionfromEntity(getPlayerEntity(), "elemental_buddy_corrosive_blood")
		end,
		requires_relic			= "elemental_pot",
	},
	{
		name 					= "elemental_pot",
		ui_name					= "Elemental Pot",
		categories				= "player",
		effect_desc				= "Owning any relic related to materials gives you a 25% chance to be \ninflicted with the healing effect every time you take direct damage from harmful materials.",
		effect_desc_short		= "Owning material-related heals you if you take damage from harmful materials.",
        precursor_types			= "magic_artifact",
		upgrade_effect_desc		= "For each upgrade level, healing effect chance when damaged by materials increases by 5%",
		is_upgradable 			= true,
		ondamaged_function		= function(damage, message, entity_thats_responsible, is_fatal, projectile_thats_responsible )
			if(isRelicOwned("elemental_pot") >= 1) then
				if(string.find(message, "damage from material")) then
					-- first verifies if player possesses any relic under the materials category
					local related_relics_owned = false
					local owned_relics = getOwnedRelics()
					for i,owned_relic in ipairs(owned_relics) do
						if(string.find(getRelic(EntityGetName(owned_relic)).categories, "materials")) then
							related_relics_owned = true
							break
						end
					end
					
					-- if so, proceeds to relic effect
					if(related_relics_owned) then
						SetRandomSeed( GameGetFrameNum(), math.random(1,50))
						local chance = Random(1,100)
						local odds = 25 + (5 * getRelicLevel("elemental_pot"))
						if(chance >= 1 and chance <= odds) then
							LoadGameEffectEntityTo( GetUpdatedEntityID(), "data/entities/misc/effect_regeneration.xml" )
						end
					end
				end
			end
		end,
	},
	{
		name 					= "feather_lava",
		ui_name					= "Lava Feather",
		categories				= "companions,materials",
		effect_desc				= "Lava Sparry, an elemental bird that lays lava orbs on enemies directly below it joins you.",
		effect_desc_short		= "Lava Sparry, an elemental bird that lays lava orbs on enemies directly below it joins you.",
        precursor_types			= "companion",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			attachCompaniontoEntity(getPlayerEntity(), "sparry_lava", "mortal")
		end,
		ondrop_function			= function()
			detachCompanionfromEntity(getPlayerEntity(), "sparry_lava")
		end,
		requires_relic			= "bag_of_magical_seeds",
	},
	{
		name 					= "feather_water",
		ui_name					= "Water Feather",
		categories				= "companions,materials",
		effect_desc				= "Water Sparry, an elemental bird that lays water orbs on enemies directly below it joins you.",
		effect_desc_short		= "Water Sparry, an elemental bird that lays water orbs on enemies directly below it joins you.",
        precursor_types			= "companion",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			attachCompaniontoEntity(getPlayerEntity(), "sparry_water", "mortal")
		end,
		ondrop_function			= function()
			detachCompanionfromEntity(getPlayerEntity(), "sparry_water")
		end,
		requires_relic			= "bag_of_magical_seeds",
	},
	{
		name 					= "feather_acid",
		ui_name					= "Acid Feather",
		categories				= "companions,materials",
		effect_desc				= "Acid Sparry, an elemental bird that lays acid orbs on enemies directly below it joins you.",
		effect_desc_short		= "Acid Sparry, an elemental bird that lays acid orbs on enemies directly below it joins you.",
        precursor_types			= "companion",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			attachCompaniontoEntity(getPlayerEntity(), "sparry_acid", "mortal")
		end,
		ondrop_function			= function()
			detachCompanionfromEntity(getPlayerEntity(), "sparry_acid")
		end,
		requires_relic			= "bag_of_magical_seeds",
	},
	{
		name 					= "feather_poison",
		ui_name					= "Poison Feather",
		categories				= "companions,materials",
		effect_desc				= "Poison Sparry, an elemental bird that lays poison orbs on enemies directly below it joins you.",
		effect_desc_short		= "Poison Sparry, an elemental bird that lays poison orbs on enemies directly below it joins you.",
        precursor_types			= "companion",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			attachCompaniontoEntity(getPlayerEntity(), "sparry_poison", "mortal")
		end,
		ondrop_function			= function()
			detachCompanionfromEntity(getPlayerEntity(), "sparry_poison")
		end,
		requires_relic			= "bag_of_magical_seeds",
	},
	{
		name 					= "feather_poly",
		ui_name					= "Poly Feather",
		categories				= "companions,materials",
		effect_desc				= "Poly Sparry, an elemental bird that lays polymorphine orbs on enemies directly below it joins you.",
		effect_desc_short		= "Poly Sparry, an elemental bird that lays polymorphine orbs on enemies directly below it joins you.",
        precursor_types			= "companion",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			attachCompaniontoEntity(getPlayerEntity(), "sparry_poly", "mortal")
		end,
		ondrop_function			= function()
			detachCompanionfromEntity(getPlayerEntity(), "sparry_poly")
		end,
		requires_relic			= "bag_of_magical_seeds",
	},
	{
		name 					= "feather_urine",
		ui_name					= "Urine Feather",
		categories				= "companions,materials",
		effect_desc				= "Urine Sparry, an elemental bird that lays urine orbs on enemies directly below it joins you.",
		effect_desc_short		= "Urine Sparry, an elemental bird that lays urine orbs on enemies directly below it joins you.",
        precursor_types			= "companion",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			attachCompaniontoEntity(getPlayerEntity(), "sparry_urine", "mortal")
		end,
		ondrop_function			= function()
			detachCompanionfromEntity(getPlayerEntity(), "sparry_urine")
		end,
		requires_relic			= "bag_of_magical_seeds",
	},
	{
		name 					= "feather_frost",
		ui_name					= "Frost Feather",
		categories				= "companions,materials",
		effect_desc				= "Frost Sparry, an elemental bird that lays freezing liquid orbs on enemies directly below it joins you.",
		effect_desc_short		= "Frost Sparry, an elemental bird that lays freezing liquid orbs on enemies directly below it joins you.",
        precursor_types			= "companion",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			attachCompaniontoEntity(getPlayerEntity(), "sparry_freezing_liquid", "mortal")
		end,
		ondrop_function			= function()
			detachCompanionfromEntity(getPlayerEntity(), "sparry_freezing_liquid")
		end,
		requires_relic			= "bag_of_magical_seeds",
	},
	{
		name 					= "feather_gunpowder",
		ui_name					= "Gunpowder Feather",
		categories				= "companions,materials",
		effect_desc				= "Gunpowder Sparry, an elemental bird that lays gunpowder orbs on enemies directly below it joins you.",
		effect_desc_short		= "Gunpowder Sparry, an elemental bird that lays gunpowder orbs on enemies directly below it joins you.",
        precursor_types			= "companion",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			attachCompaniontoEntity(getPlayerEntity(), "sparry_gunpowder", "mortal")
		end,
		ondrop_function			= function()
			detachCompanionfromEntity(getPlayerEntity(), "sparry_gunpowder")
		end,
		requires_relic			= "bag_of_magical_seeds",
	},
	{
		name 					= "feather_toxic",
		ui_name					= "Toxic Feather",
		categories				= "companions,materials",
		effect_desc				= "Toxic Sparry, an elemental bird that lays toxic sludge orbs on enemies directly below it joins you.",
		effect_desc_short		= "Toxic Sparry, an elemental bird that lays toxic sludge orbs on enemies directly below it joins you.",
        precursor_types			= "companion",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			attachCompaniontoEntity(getPlayerEntity(), "sparry_toxic_sludge", "mortal")
		end,
		ondrop_function			= function()
			detachCompanionfromEntity(getPlayerEntity(), "sparry_toxic_sludge")
		end,
		requires_relic			= "bag_of_magical_seeds",
	},
	{
		name 					= "feather_bloody",
		ui_name					= "Bloody Feather",
		categories				= "companions,materials",
		effect_desc				= "Bloody Sparry, an elemental bird that lays corrosive blood orbs on enemies directly below it joins you.",
		effect_desc_short		= "Bloody Sparry, an elemental bird that lays corrosive blood orbs on enemies directly below it joins you.",
        precursor_types			= "companion",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			attachCompaniontoEntity(getPlayerEntity(), "sparry_corrosive_blood", "mortal")
		end,
		ondrop_function			= function()
			detachCompanionfromEntity(getPlayerEntity(), "sparry_corrosive_blood")
		end,
		requires_relic			= "bag_of_magical_seeds",
	},
	{
		name 					= "feather_explosive",
		ui_name					= "Explosive Feather",
		categories				= "companions",
		effect_desc				= "Explosive Sparry, a bird that lays explosive orbs on enemies directly below it joins you.",
		effect_desc_short		= "Explosive Sparry, a bird that lays explosive orbs on enemies directly below it joins you.",
        precursor_types			= "companion",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			attachCompaniontoEntity(getPlayerEntity(), "sparry_explosive", "mortal")
		end,
		ondrop_function			= function()
			detachCompanionfromEntity(getPlayerEntity(), "sparry_explosive")
		end,
		requires_relic			= "bag_of_magical_seeds",
	},
	{
		name 					= "feather_pirate",
		ui_name					= "Pirate Feather",
		categories				= "companions",
		effect_desc				= "Pirate Sparry, a bird that lays cannonballs on enemies directly below it joins you.",
		effect_desc_short		= "Pirate Sparry, a bird that lays cannonballs on enemies directly below it joins you.",
        precursor_types			= "companion",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			attachCompaniontoEntity(getPlayerEntity(), "sparry_pirate", "mortal")
		end,
		ondrop_function			= function()
			detachCompanionfromEntity(getPlayerEntity(), "sparry_pirate")
		end,
		requires_relic			= "bag_of_magical_seeds",
	},
	{
		name 					= "elemental_buddy_seed_alcohol",
		ui_name					= "Whiskey Elemental Buddy Seed",
		categories				= "companions,materials",
		effect_desc				= "An elemental buddy capable of spitting whiskey on enemies joins you.",
		effect_desc_short		= "An elemental buddy capable of spitting whiskey on enemies joins you.",
        precursor_types			= "companion",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			attachCompaniontoEntity(getPlayerEntity(), "elemental_buddy_alcohol", "mortal")
		end,
		ondrop_function			= function()
			detachCompanionfromEntity(getPlayerEntity(), "elemental_buddy_alcohol")
		end,
		requires_relic			= "elemental_pot",
	},
	{
		name 					= "elemental_buddy_seed_teleportatium",
		ui_name					= "Teleportatium Elemental Buddy Seed",
		categories				= "companions,materials",
		effect_desc				= "An elemental buddy capable of spitting teleportatium on enemies joins you.",
		effect_desc_short		= "An elemental buddy capable of spitting teleportatium on enemies joins you.",
        precursor_types			= "companion",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			attachCompaniontoEntity(getPlayerEntity(), "elemental_buddy_teleportatium", "mortal")
		end,
		ondrop_function			= function()
			detachCompanionfromEntity(getPlayerEntity(), "elemental_buddy_teleportatium")
		end,
		requires_relic			= "elemental_pot",
	},
	{
		name 					= "teleportatium_gem",
		ui_name					= "Teleportatium Gem",
		categories				= "projectiles,materials",
		effect_desc				= "Spells you cast that explode on impact have a 20% chance of releasing teleportatium on impact.",
		effect_desc_short		= "Your spells release teleportatium.",
        precursor_types			= "elemental_gem",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("teleportatium_gem") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 20 + (5 * getRelicLevel("teleportatium_gem"))
				if(chance >= 1 and chance <= odds) then
					attachLoadEntityOnExplosiontoProjectile(projectile_entity_id, getLoadOnExplosion("launch_elemental_particles_teleportatium").path, "magic_liquid_teleportation")
				end
			end
		end,
	},
	{
		name 					= "elemental_buddy_seed_oil",
		ui_name					= "Oil Elemental Buddy Seed",
		categories				= "companions,materials",
		effect_desc				= "An elemental buddy capable of spitting oil on enemies joins you.",
		effect_desc_short		= "An elemental buddy capable of spitting oil on enemies joins you.",
        precursor_types			= "companion",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			attachCompaniontoEntity(getPlayerEntity(), "elemental_buddy_oil", "mortal")
		end,
		ondrop_function			= function()
			detachCompanionfromEntity(getPlayerEntity(), "elemental_buddy_oil")
		end,
		requires_relic			= "elemental_pot",
	},
	{
		name 					= "elemental_buddy_seed_fire",
		ui_name					= "Fire Elemental Buddy Seed",
		categories				= "companions,materials",
		effect_desc				= "An elemental buddy capable of spitting fire on enemies joins you.",
		effect_desc_short		= "An elemental buddy capable of spitting fire on enemies joins you.",
        precursor_types			= "companion",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			attachCompaniontoEntity(getPlayerEntity(), "elemental_buddy_fire", "mortal")
		end,
		ondrop_function			= function()
			detachCompanionfromEntity(getPlayerEntity(), "elemental_buddy_fire")
		end,
		requires_relic			= "elemental_pot",
	},
	{
		name 					= "invisiblium_gem",
		ui_name					= "Invisiblium Gem",
		categories				= "projectiles,materials",
		effect_desc				= "Spells you cast that explode on impact have a 20% chance of releasing invisiblium on impact.",
		effect_desc_short		= "Your spells release invisiblium.",
        precursor_types			= "elemental_gem",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("invisiblium_gem") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 20 + (5 * getRelicLevel("invisiblium_gem"))
				if(chance >= 1 and chance <= odds) then
					attachLoadEntityOnExplosiontoProjectile(projectile_entity_id, getLoadOnExplosion("launch_elemental_particles_invisiblium").path, "magic_liquid_invisibility")
				end
			end
		end,
	},
	{
		name 					= "healthium_gem",
		ui_name					= "Healthium Gem",
		categories				= "projectiles,materials",
		effect_desc				= "Spells you cast that explode on impact have a 5% chance of releasing healthium on impact.",
		effect_desc_short		= "Your spells release healthium.",
        precursor_types			= "elemental_gem",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 2%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("healthium_gem") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 5 + (2 * getRelicLevel("healthium_gem"))
				if(chance >= 1 and chance <= odds) then
					attachLoadEntityOnExplosiontoProjectile(projectile_entity_id, getLoadOnExplosion("launch_elemental_particles_healthium").path, "magic_liquid_hp_regeneration")
				end
			end
		end,
	},
	{
		name 					= "gold_gem",
		ui_name					= "Gold Gem",
		categories				= "projectiles,materials",
		effect_desc				= "Spells you cast that explode on impact have a 5% chance of releasing gold dust on impact.",
		effect_desc_short		= "Your spells release gold dust.",
        precursor_types			= "elemental_gem",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 2%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("gold_gem") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 5 + (2 * getRelicLevel("gold_gem"))
				if(chance >= 1 and chance <= odds) then
					attachLoadEntityOnExplosiontoProjectile(projectile_entity_id, getLoadOnExplosion("launch_elemental_particles_gold").path, "gold")
				end
			end
		end,
	},
	{
		name 					= "silver_gem",
		ui_name					= "Silver Gem",
		categories				= "projectiles,materials",
		effect_desc				= "Spells you cast that explode on impact have a 20% chance of releasing silver on impact.",
		effect_desc_short		= "Your spells release silver.",
        precursor_types			= "elemental_gem",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("silver_gem") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 20 + (5 * getRelicLevel("silver_gem"))
				if(chance >= 1 and chance <= odds) then
					attachLoadEntityOnExplosiontoProjectile(projectile_entity_id, getLoadOnExplosion("launch_elemental_particles_silver").path, "silver")
				end
			end
		end,
	},
	{
		name 					= "diamond_gem",
		ui_name					= "Diamond Gem",
		categories				= "projectiles,materials",
		effect_desc				= "Spells you cast that explode on impact have a 20% chance of releasing diamond on impact.",
		effect_desc_short		= "Your spells release diamond.",
        precursor_types			= "elemental_gem",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("diamond_gem") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 20 + (5 * getRelicLevel("diamond_gem"))
				if(chance >= 1 and chance <= odds) then
					attachLoadEntityOnExplosiontoProjectile(projectile_entity_id, getLoadOnExplosion("launch_elemental_particles_diamond").path, "diamond")
				end
			end
		end,
	},
	{
		name 					= "elemental_buddy_seed_invisiblium",
		ui_name					= "Invisiblium Elemental Buddy Seed",
		categories				= "companions,materials",
		effect_desc				= "An elemental buddy capable of spitting invisiblium on enemies joins you.",
		effect_desc_short		= "An elemental buddy capable of spitting invisiblium on enemies joins you.",
        precursor_types			= "companion",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			attachCompaniontoEntity(getPlayerEntity(), "elemental_buddy_invisiblium", "mortal")
		end,
		ondrop_function			= function()
			detachCompanionfromEntity(getPlayerEntity(), "elemental_buddy_invisiblium")
		end,
		requires_relic			= "elemental_pot",
	},
	{
		name 					= "elemental_buddy_seed_berserkium",
		ui_name					= "Berserkium Elemental Buddy Seed",
		categories				= "companions,materials",
		effect_desc				= "An elemental buddy capable of spitting berserkium on enemies joins you.",
		effect_desc_short		= "An elemental buddy capable of spitting berserkium on enemies joins you.",
        precursor_types			= "companion",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			attachCompaniontoEntity(getPlayerEntity(), "elemental_buddy_berserkium", "mortal")
		end,
		ondrop_function			= function()
			detachCompanionfromEntity(getPlayerEntity(), "elemental_buddy_berserkium")
		end,
		requires_relic			= "elemental_pot",
	},
	{
		name 					= "elemental_buddy_seed_slime",
		ui_name					= "Slime Elemental Buddy Seed",
		categories				= "companions,materials",
		effect_desc				= "An elemental buddy capable of spitting slime on enemies joins you.",
		effect_desc_short		= "An elemental buddy capable of spitting slime on enemies joins you.",
        precursor_types			= "companion",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			attachCompaniontoEntity(getPlayerEntity(), "elemental_buddy_slime", "mortal")
		end,
		ondrop_function			= function()
			detachCompanionfromEntity(getPlayerEntity(), "elemental_buddy_slime")
		end,
		requires_relic			= "elemental_pot",
	},
	{
		name 					= "slime_gem",
		ui_name					= "Slime Gem",
		categories				= "projectiles,materials",
		effect_desc				= "Spells you cast that explode on impact have a 20% chance of releasing slime on impact.",
		effect_desc_short		= "Your spells release slime.",
        precursor_types			= "elemental_gem",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("slime_gem") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 20 + (5 * getRelicLevel("slime_gem"))
				if(chance >= 1 and chance <= odds) then
					attachLoadEntityOnExplosiontoProjectile(projectile_entity_id, getLoadOnExplosion("launch_elemental_particles_slime").path, "slime")
				end
			end
		end,
	},
	{
		name 					= "elemental_buddy_seed_fungi",
		ui_name					= "Fungi Elemental Buddy Seed",
		categories				= "companions,materials",
		effect_desc				= "An elemental buddy capable of spitting fungi on enemies joins you.",
		effect_desc_short		= "An elemental buddy capable of spitting fungi on enemies joins you.",
        precursor_types			= "companion",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			attachCompaniontoEntity(getPlayerEntity(), "elemental_buddy_fungi", "mortal")
		end,
		ondrop_function			= function()
			detachCompanionfromEntity(getPlayerEntity(), "elemental_buddy_fungi")
		end,
		requires_relic			= "elemental_pot",
	},
	{
		name 					= "feather_thunder",
		ui_name					= "Thunder Feather",
		categories				= "companions",
		effect_desc				= "Thunder Sparry, a bird that lays thunder orbs on enemies directly below it joins you.",
		effect_desc_short		= "Thunder Sparry, a bird that lays thunder orbs  on enemies directly below it joins you.",
        precursor_types			= "companion",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			attachCompaniontoEntity(getPlayerEntity(), "sparry_thunder", "mortal")
		end,
		ondrop_function			= function()
			detachCompanionfromEntity(getPlayerEntity(), "sparry_thunder")
		end,
		requires_relic			= "bag_of_magical_seeds",
	},
	{
		name 					= "feather_alcohol",
		ui_name					= "Whiskey Feather",
		categories				= "companions,materials",
		effect_desc				= "Whiskey Sparry, an elemental bird that lays whiskey orbs on enemies directly below it joins you.",
		effect_desc_short		= "Whiskey Sparry, an elemental bird that lays whiskey orbs on enemies directly below it joins you.",
        precursor_types			= "companion",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			attachCompaniontoEntity(getPlayerEntity(), "sparry_alcohol", "mortal")
		end,
		ondrop_function			= function()
			detachCompanionfromEntity(getPlayerEntity(), "sparry_alcohol")
		end,
		requires_relic			= "bag_of_magical_seeds",
	},
	{
		name 					= "feather_teleportatium",
		ui_name					= "Teleportatium Feather",
		categories				= "companions,materials",
		effect_desc				= "Teleportatium Sparry, an elemental bird that lays teleportatium orbs on enemies directly below it joins you.",
		effect_desc_short		= "Teleportatium Sparry, an elemental bird that lays teleportatium orbs on enemies directly below it joins you.",
        precursor_types			= "companion",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			attachCompaniontoEntity(getPlayerEntity(), "sparry_teleportatium", "mortal")
		end,
		ondrop_function			= function()
			detachCompanionfromEntity(getPlayerEntity(), "sparry_teleportatium")
		end,
		requires_relic			= "bag_of_magical_seeds",
	},
	{
		name 					= "feather_oil",
		ui_name					= "Oil Feather",
		categories				= "companions,materials",
		effect_desc				= "Oil Sparry, an elemental bird that lays oil orbs on enemies directly below it joins you.",
		effect_desc_short		= "Oil Sparry, an elemental bird that lays oil orbs on enemies directly below it joins you.",
        precursor_types			= "companion",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			attachCompaniontoEntity(getPlayerEntity(), "sparry_oil", "mortal")
		end,
		ondrop_function			= function()
			detachCompanionfromEntity(getPlayerEntity(), "sparry_oil")
		end,
		requires_relic			= "bag_of_magical_seeds",
	},
	{
		name 					= "feather_fire",
		ui_name					= "Fire Feather",
		categories				= "companions,materials",
		effect_desc				= "Fire Sparry, an elemental bird that lays fire orbs on enemies directly below it joins you.",
		effect_desc_short		= "Fire Sparry, an elemental bird that lays fire orbs on enemies directly below it joins you.",
        precursor_types			= "companion",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			attachCompaniontoEntity(getPlayerEntity(), "sparry_fire", "mortal")
		end,
		ondrop_function			= function()
			detachCompanionfromEntity(getPlayerEntity(), "sparry_fire")
		end,
		requires_relic			= "bag_of_magical_seeds",
	},
	{
		name 					= "feather_invisiblium",
		ui_name					= "Invisiblium Feather",
		categories				= "companions,materials",
		effect_desc				= "Invisiblium Sparry, an elemental bird that lays invisiblium orbs on enemies directly below it joins you.",
		effect_desc_short		= "Invisiblium Sparry, an elemental bird that lays invisiblium orbs on enemies directly below it joins you.",
        precursor_types			= "companion",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			attachCompaniontoEntity(getPlayerEntity(), "sparry_invisiblium", "mortal")
		end,
		ondrop_function			= function()
			detachCompanionfromEntity(getPlayerEntity(), "sparry_invisiblium")
		end,
		requires_relic			= "bag_of_magical_seeds",
	},
	{
		name 					= "feather_berserkium",
		ui_name					= "Berserkium Feather",
		categories				= "companions,materials",
		effect_desc				= "Berserkium Sparry, an elemental bird that lays berserkium orbs on enemies directly below it joins you.",
		effect_desc_short		= "Berserkium Sparry, an elemental bird that lays berserkium orbs on enemies directly below it joins you.",
        precursor_types			= "companion",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			attachCompaniontoEntity(getPlayerEntity(), "sparry_berserkium", "mortal")
		end,
		ondrop_function			= function()
			detachCompanionfromEntity(getPlayerEntity(), "sparry_berserkium")
		end,
		requires_relic			= "bag_of_magical_seeds",
	},
	{
		name 					= "feather_slime",
		ui_name					= "Slime Feather",
		categories				= "companions,materials",
		effect_desc				= "Slime Sparry, an elemental bird that lays slime orbs on enemies directly below it joins you.",
		effect_desc_short		= "Slime Sparry, an elemental bird that lays slime orbs on enemies directly below it joins you.",
        precursor_types			= "companion",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			attachCompaniontoEntity(getPlayerEntity(), "sparry_slime", "mortal")
		end,
		ondrop_function			= function()
			detachCompanionfromEntity(getPlayerEntity(), "sparry_slime")
		end,
		requires_relic			= "bag_of_magical_seeds",
	},
	{
		name 					= "feather_fungi",
		ui_name					= "Fungi Feather",
		categories				= "companions,materials",
		effect_desc				= "Fungi Sparry, an elemental bird that lays fungi orbs on enemies directly below it joins you.",
		effect_desc_short		= "Fungi Sparry, an elemental bird that lays fungi orbs on enemies directly below it joins you.",
        precursor_types			= "companion",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			attachCompaniontoEntity(getPlayerEntity(), "sparry_fungi", "mortal")
		end,
		ondrop_function			= function()
			detachCompanionfromEntity(getPlayerEntity(), "sparry_fungi")
		end,
		requires_relic			= "bag_of_magical_seeds",
	},
	{
		name 					= "pheromone_gem",
		ui_name					= "Pheromone Gem",
		categories				= "projectiles,materials",
		effect_desc				= "Spells you cast that explode on impact have a 20% chance of releasing pheromone on impact.",
		effect_desc_short		= "Your spells release pheromone.",
        precursor_types			= "elemental_gem",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("pheromone_gem") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 20 + (5 * getRelicLevel("pheromone_gem"))
				if(chance >= 1 and chance <= odds) then
					attachLoadEntityOnExplosiontoProjectile(projectile_entity_id, getLoadOnExplosion("launch_elemental_particles_pheromone").path, "magic_liquid_charm")
				end
			end
		end,
	},
	{
		name 					= "feather_pheromone",
		ui_name					= "Pheromone Feather",
		categories				= "companions,materials",
		effect_desc				= "Pheromone Sparry, an elemental bird that lays pheromone orbs on enemies directly below it joins you.",
		effect_desc_short		= "Pheromone Sparry, an elemental bird that lays pheromone orbs on enemies directly below it joins you.",
        precursor_types			= "companion",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			attachCompaniontoEntity(getPlayerEntity(), "sparry_pheromone", "mortal")
		end,
		ondrop_function			= function()
			detachCompanionfromEntity(getPlayerEntity(), "sparry_pheromone")
		end,
		requires_relic			= "bag_of_magical_seeds",
	},
	{
		name 					= "elemental_buddy_seed_pheromone",
		ui_name					= "Pheromone Elemental Buddy Seed",
		categories				= "companions,materials",
		effect_desc				= "An elemental buddy capable of spitting pheromone on enemies joins you.",
		effect_desc_short		= "An elemental buddy capable of spitting pheromone on enemies joins you.",
        precursor_types			= "companion",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			attachCompaniontoEntity(getPlayerEntity(), "elemental_buddy_pheromone", "mortal")
		end,
		ondrop_function			= function()
			detachCompanionfromEntity(getPlayerEntity(), "elemental_buddy_pheromone")
		end,
		requires_relic			= "elemental_pot",
	},
	{
		name 					= "pearl_necklace_bloody",
		ui_name					= "Bloody Pearl Necklace",
		categories				= "enemies,items,materials",
		effect_desc				= "Enemies you kill have a 20% chance of dropping blood death ray \npearls that once thrown summon a blood death ray.",
		effect_desc_short		= "Killed enemies drop blood pearls that summon blood death rays once thrown.",
        precursor_types			= "death_ray",
		upgrade_effect_desc		= "For each upgrade level, drop chance increases by 5%.",
		is_upgradable 			= true,
		onenemykill_function	= function(damage_type_bit_field, damage_message, entity_thats_responsible, drop_items)
			if(isRelicOwned("pearl_necklace_bloody") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 20 + (5 * getRelicLevel("pearl_necklace_bloody"))
				if(chance >= 1 and chance <= odds) then	
					local death_x, death_y = EntityGetTransform(GetUpdatedEntityID())
					EntityLoad("mods/ArcaneReliquary/files/entity_definitions/items/death_ray_pearl_blood.xml", death_x, death_y)
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "blaze_pearl,scarlet_gem,death_ray_stone",
	},
	{
		name 					= "pearl_necklace_mutating",
		ui_name					= "Mutating Pearl Necklace",
		categories				= "enemies,items,materials",
		effect_desc				= "Enemies you kill have a 20% chance of dropping polymorphine death ray \npearls that once thrown summon a polymorphine death ray.",
		effect_desc_short		= "Killed enemies drop polymorphine pearls that summon polymorphine death rays once thrown.",
        precursor_types			= "death_ray",
		upgrade_effect_desc		= "For each upgrade level, drop chance increases by 5%.",
		is_upgradable 			= true,
		onenemykill_function	= function(damage_type_bit_field, damage_message, entity_thats_responsible, drop_items)
			if(isRelicOwned("pearl_necklace_mutating") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 20 + (5 * getRelicLevel("pearl_necklace_mutating"))
				if(chance >= 1 and chance <= odds) then	
					local death_x, death_y = EntityGetTransform(GetUpdatedEntityID())
					EntityLoad("mods/ArcaneReliquary/files/entity_definitions/items/death_ray_pearl_poly.xml", death_x, death_y)
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "blaze_pearl,poly_gem,death_ray_stone",
	},
	{
		name 					= "parry_shield",
		ui_name					= "Parry Shield",
		categories				= "player",
		effect_desc				= "You hold a shield that deflects enemy projectiles, doubling their base damage and explosive power.",
		effect_desc_short		= "You hold a shield that deflects enemy projectiles, doubling their base damage and explosive power.",
        precursor_types			= "gadget,magic_artifact",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			EntityAddChild(getPlayerEntity(), EntityLoad("mods/ArcaneReliquary/files/entity_definitions/others/parry_shield_handler.xml"))
		end,
		ondrop_function			= function()
			local entity_children = EntityGetAllChildren(getPlayerEntity())
			if entity_children ~= nil then
				for i,child_id in ipairs(entity_children) do
					if(EntityGetName(child_id) == "parry_shield_handler" and not EntityHasTag(child_id, "relic")) then
						local x, y = EntityGetTransform(child_id)
						EntityKill(child_id)
					end
				end
			end
		end,
		excludes_relics_desc	= "It can't be used along with other Parry Shield Relic variations",
		excludes_relics_tags	= "parry_shield_variation",
	},
	{
		name 					= "parry_shield_spikes",
		ui_name					= "Parry Shield with Spikes",
		categories				= "player",
		effect_desc				= "You hold a shield that deflects enemy projectiles and, as long as you're moving, damages enemies that touch it.",
		effect_desc_short		= "You hold a shield that deflects enemy projectiles and, as long as you're moving, damages enemies that touch it.",
        precursor_types			= "gadget,magic_artifact",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			EntityAddChild(getPlayerEntity(), EntityLoad("mods/ArcaneReliquary/files/entity_definitions/others/parry_shield_spikes_handler.xml"))
		end,
		ondrop_function			= function()
			local entity_children = EntityGetAllChildren(getPlayerEntity())
			if entity_children ~= nil then
				for i,child_id in ipairs(entity_children) do
					if(EntityGetName(child_id) == "parry_shield_spikes_handler" and not EntityHasTag(child_id, "relic")) then
						local x, y = EntityGetTransform(child_id)
						EntityKill(child_id)
					end
				end
			end
		end,
		excludes_relics_desc	= "It can't be used along with other Parry Shield Relic variations",
		excludes_relics_tags	= "parry_shield_variation",
	},
	{
		name 					= "parry_shield_slap",
		ui_name					= "Parry Shield with Slap Effect",
		categories				= "player",
		effect_desc				= "You hold a shield that converts enemy projectiles into slapperbombs \nand inflicts the slap effect on enemies that touch it.",
		effect_desc_short		= "You hold a shield that converts enemy projectiles into slapperbombs and inflicts the slap effect on enemies.",
        precursor_types			= "gadget,magic_artifact",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			EntityAddChild(getPlayerEntity(), EntityLoad("mods/ArcaneReliquary/files/entity_definitions/others/parry_shield_slap_handler.xml"))
		end,
		ondrop_function			= function()
			local entity_children = EntityGetAllChildren(getPlayerEntity())
			if entity_children ~= nil then
				for i,child_id in ipairs(entity_children) do
					if(EntityGetName(child_id) == "parry_shield_slap_handler" and not EntityHasTag(child_id, "relic")) then
						local x, y = EntityGetTransform(child_id)
						EntityKill(child_id)
					end
				end
			end
		end,
		excludes_relics_desc	= "It can't be used along with other Parry Shield Relic variations",
		excludes_relics_tags	= "parry_shield_variation",
		combine_exclusive		= true,
		combine_requires_relics	= "parry_shield,fly_swatter",
	},
	{
		name 					= "parry_shield_acid",
		ui_name					= "Acid Parry Shield",
		categories				= "player",
		effect_desc				= "You hold a shield that converts enemy projectiles into acid orbs \nand transmutes enemies that touch it into acid.",
		effect_desc_short		= "You hold a shield that converts enemy projectiles into acid orbs and transmutes enemies that touch it into acid.",
        precursor_types			= "gadget,magic_artifact",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			EntityAddChild(getPlayerEntity(), EntityLoad("mods/ArcaneReliquary/files/entity_definitions/others/parry_shield_acid_handler.xml"))
		end,
		ondrop_function			= function()
			local entity_children = EntityGetAllChildren(getPlayerEntity())
			if entity_children ~= nil then
				for i,child_id in ipairs(entity_children) do
					if(EntityGetName(child_id) == "parry_shield_acid_handler" and not EntityHasTag(child_id, "relic")) then
						local x, y = EntityGetTransform(child_id)
						EntityKill(child_id)
					end
				end
			end
		end,
		excludes_relics_desc	= "It can't be used along with other Parry Shield Relic variations",
		excludes_relics_tags	= "parry_shield_variation",
		combine_exclusive		= true,
		combine_requires_relics	= "parry_shield,acid_gem",
	},
	{
		name 					= "parry_shield_explosive",
		ui_name					= "Explosive Parry Shield",
		categories				= "player",
		effect_desc				= "You hold a shield that converts enemy projectiles into explosive orbs \nand inflicts the exploding effect on enemies that touch it.",
		effect_desc_short		= "You hold a shield that converts enemy projectiles into explosive orbs and inflicts the slap effect on enemies that touch it.",
        precursor_types			= "gadget,magic_artifact",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			EntityAddChild(getPlayerEntity(), EntityLoad("mods/ArcaneReliquary/files/entity_definitions/others/parry_shield_explosive_handler.xml"))
		end,
		ondrop_function			= function()
			local entity_children = EntityGetAllChildren(getPlayerEntity())
			if entity_children ~= nil then
				for i,child_id in ipairs(entity_children) do
					if(EntityGetName(child_id) == "parry_shield_explosive_handler" and not EntityHasTag(child_id, "relic")) then
						local x, y = EntityGetTransform(child_id)
						EntityKill(child_id)
					end
				end
			end
		end,
		excludes_relics_desc	= "It can't be used along with other Parry Shield Relic variations",
		excludes_relics_tags	= "parry_shield_variation",
		combine_exclusive		= true,
		combine_requires_relics	= "parry_shield,rhodonite_necklace",
	},
	{
		name 					= "parry_shield_fire",
		ui_name					= "Fire Parry Shield",
		categories				= "player",
		effect_desc				= "You hold a shield that converts enemy projectiles into firebombs \nand incinerates enemies that touch it.",
		effect_desc_short		= "You hold a shield that converts enemy projectiles into firebombs and incinerates enemies that touch it.",
        precursor_types			= "gadget,magic_artifact",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			EntityAddChild(getPlayerEntity(), EntityLoad("mods/ArcaneReliquary/files/entity_definitions/others/parry_shield_fire_handler.xml"))
		end,
		ondrop_function			= function()
			local entity_children = EntityGetAllChildren(getPlayerEntity())
			if entity_children ~= nil then
				for i,child_id in ipairs(entity_children) do
					if(EntityGetName(child_id) == "parry_shield_fire_handler" and not EntityHasTag(child_id, "relic")) then
						local x, y = EntityGetTransform(child_id)
						EntityKill(child_id)
					end
				end
			end
		end,
		excludes_relics_desc	= "It can't be used along with other Parry Shield Relic variations",
		excludes_relics_tags	= "parry_shield_variation",
		combine_exclusive		= true,
		combine_requires_relics	= "parry_shield,solar_artifact",
	},
	{
		name 					= "parry_shield_lava",
		ui_name					= "Lava Parry Shield",
		categories				= "player",
		effect_desc				= "You hold a shield that converts enemy projectiles into lava orbs \nand transmutes enemies that touch it into lava.",
		effect_desc_short		= "You hold a shield that converts enemy projectiles into lava orbs and transmutes enemies that touch it into lava.",
        precursor_types			= "gadget,magic_artifact",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			EntityAddChild(getPlayerEntity(), EntityLoad("mods/ArcaneReliquary/files/entity_definitions/others/parry_shield_lava_handler.xml"))
		end,
		ondrop_function			= function()
			local entity_children = EntityGetAllChildren(getPlayerEntity())
			if entity_children ~= nil then
				for i,child_id in ipairs(entity_children) do
					if(EntityGetName(child_id) == "parry_shield_lava_handler" and not EntityHasTag(child_id, "relic")) then
						local x, y = EntityGetTransform(child_id)
						EntityKill(child_id)
					end
				end
			end
		end,
		excludes_relics_desc	= "It can't be used along with other Parry Shield Relic variations",
		excludes_relics_tags	= "parry_shield_variation",
		combine_exclusive		= true,
		combine_requires_relics	= "parry_shield,lava_gem",
	},
	{
		name 					= "parry_shield_poison",
		ui_name					= "Poison Parry Shield",
		categories				= "player",
		effect_desc				= "You hold a shield that converts enemy projectiles into poison orbs \nand transmutes enemies that touch it into poison.",
		effect_desc_short		= "You hold a shield that converts enemy projectiles into poison orbs and transmutes enemies that touch it into poison.",
        precursor_types			= "gadget,magic_artifact",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			EntityAddChild(getPlayerEntity(), EntityLoad("mods/ArcaneReliquary/files/entity_definitions/others/parry_shield_poison_handler.xml"))
		end,
		ondrop_function			= function()
			local entity_children = EntityGetAllChildren(getPlayerEntity())
			if entity_children ~= nil then
				for i,child_id in ipairs(entity_children) do
					if(EntityGetName(child_id) == "parry_shield_poison_handler" and not EntityHasTag(child_id, "relic")) then
						local x, y = EntityGetTransform(child_id)
						EntityKill(child_id)
					end
				end
			end
		end,
		excludes_relics_desc	= "It can't be used along with other Parry Shield Relic variations",
		excludes_relics_tags	= "parry_shield_variation",
		combine_exclusive		= true,
		combine_requires_relics	= "parry_shield,poison_gem",
	},
	{
		name 					= "parry_shield_water",
		ui_name					= "Water Parry Shield",
		categories				= "player",
		effect_desc				= "You hold a shield that converts enemy projectiles into water orbs \nand transmutes enemies that touch it into water.",
		effect_desc_short		= "You hold a shield that converts enemy projectiles into water orbs and transmutes enemies that touch it into water.",
        precursor_types			= "gadget,magic_artifact",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			EntityAddChild(getPlayerEntity(), EntityLoad("mods/ArcaneReliquary/files/entity_definitions/others/parry_shield_water_handler.xml"))
		end,
		ondrop_function			= function()
			local entity_children = EntityGetAllChildren(getPlayerEntity())
			if entity_children ~= nil then
				for i,child_id in ipairs(entity_children) do
					if(EntityGetName(child_id) == "parry_shield_water_handler" and not EntityHasTag(child_id, "relic")) then
						local x, y = EntityGetTransform(child_id)
						EntityKill(child_id)
					end
				end
			end
		end,
		excludes_relics_desc	= "It can't be used along with other Parry Shield Relic variations",
		excludes_relics_tags	= "parry_shield_variation",
		combine_exclusive		= true,
		combine_requires_relics	= "parry_shield,humid_gem",
	},
	{
		name 					= "parry_shield_blood",
		ui_name					= "Blood Parry Shield",
		categories				= "player",
		effect_desc				= "You hold a shield that converts enemy projectiles into corrosive blood orbs \nand transmutes enemies that touch it into corrosive blood.",
		effect_desc_short		= "You hold a shield that converts enemy projectiles into corrosive blood orbs and transmutes enemies that touch it into corrosive blood.",
        precursor_types			= "gadget,magic_artifact",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			EntityAddChild(getPlayerEntity(), EntityLoad("mods/ArcaneReliquary/files/entity_definitions/others/parry_shield_blood_handler.xml"))
		end,
		ondrop_function			= function()
			local entity_children = EntityGetAllChildren(getPlayerEntity())
			if entity_children ~= nil then
				for i,child_id in ipairs(entity_children) do
					if(EntityGetName(child_id) == "parry_shield_blood_handler" and not EntityHasTag(child_id, "relic")) then
						local x, y = EntityGetTransform(child_id)
						EntityKill(child_id)
					end
				end
			end
		end,
		excludes_relics_desc	= "It can't be used along with other Parry Shield Relic variations",
		excludes_relics_tags	= "parry_shield_variation",
		combine_exclusive		= true,
		combine_requires_relics	= "parry_shield,corrosive_blood_gem",
	},
	{
		name 					= "parry_shield_poly",
		ui_name					= "Polymorphine Parry Shield",
		categories				= "player",
		effect_desc				= "You hold a shield that converts enemy projectiles into polymorphine orbs \nand transmutes enemies that touch it into polymorphine.",
		effect_desc_short		= "You hold a shield that converts enemy projectiles into polymorphine orbs and transmutes enemies that touch it into polymorphine.",
        precursor_types			= "gadget,magic_artifact",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			EntityAddChild(getPlayerEntity(), EntityLoad("mods/ArcaneReliquary/files/entity_definitions/others/parry_shield_poly_handler.xml"))
		end,
		ondrop_function			= function()
			local entity_children = EntityGetAllChildren(getPlayerEntity())
			if entity_children ~= nil then
				for i,child_id in ipairs(entity_children) do
					if(EntityGetName(child_id) == "parry_shield_poly_handler" and not EntityHasTag(child_id, "relic")) then
						local x, y = EntityGetTransform(child_id)
						EntityKill(child_id)
					end
				end
			end
		end,
		excludes_relics_desc	= "It can't be used along with other Parry Shield Relic variations",
		excludes_relics_tags	= "parry_shield_variation",
		combine_exclusive		= true,
		combine_requires_relics	= "parry_shield,poly_gem",
	},
	{
		name 					= "parry_shield_urine",
		ui_name					= "Urine Parry Shield",
		categories				= "player",
		effect_desc				= "You hold a shield that converts enemy projectiles into urine orbs \nand transmutes enemies that touch it into urine.",
		effect_desc_short		= "You hold a shield that converts enemy projectiles into urine orbs and transmutes enemies that touch it into urine.",
        precursor_types			= "gadget,magic_artifact",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			EntityAddChild(getPlayerEntity(), EntityLoad("mods/ArcaneReliquary/files/entity_definitions/others/parry_shield_urine_handler.xml"))
		end,
		ondrop_function			= function()
			local entity_children = EntityGetAllChildren(getPlayerEntity())
			if entity_children ~= nil then
				for i,child_id in ipairs(entity_children) do
					if(EntityGetName(child_id) == "parry_shield_urine_handler" and not EntityHasTag(child_id, "relic")) then
						local x, y = EntityGetTransform(child_id)
						EntityKill(child_id)
					end
				end
			end
		end,
		excludes_relics_desc	= "It can't be used along with other Parry Shield Relic variations",
		excludes_relics_tags	= "parry_shield_variation",
		combine_exclusive		= true,
		combine_requires_relics	= "parry_shield,urine_gem",
	},
	{
		name 					= "parry_shield_explode",
		ui_name					= "Parry Shield with Exploding Effect",
		categories				= "player",
		effect_desc				= "You hold a shield that converts enemy projectiles into explodebombs \nand inflicts the exploding effect on enemies that touch it.",
		effect_desc_short		= "You hold a shield that converts enemy projectiles into explodebombs and inflicts the exploding effect on enemies.",
        precursor_types			= "gadget,magic_artifact",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			EntityAddChild(getPlayerEntity(), EntityLoad("mods/ArcaneReliquary/files/entity_definitions/others/parry_shield_explode_handler.xml"))
		end,
		ondrop_function			= function()
			local entity_children = EntityGetAllChildren(getPlayerEntity())
			if entity_children ~= nil then
				for i,child_id in ipairs(entity_children) do
					if(EntityGetName(child_id) == "parry_shield_explode_handler" and not EntityHasTag(child_id, "relic")) then
						local x, y = EntityGetTransform(child_id)
						EntityKill(child_id)
					end
				end
			end
		end,
		excludes_relics_desc	= "It can't be used along with other Parry Shield Relic variations",
		excludes_relics_tags	= "parry_shield_variation",
		combine_exclusive		= true,
		combine_requires_relics	= "parry_shield,exploding_scepter",
	},
	{
		name 					= "parry_shield_slash",
		ui_name					= "Parry Shield with Slash Effect",
		categories				= "player",
		effect_desc				= "You hold a shield that converts enemy projectiles into slashbombs \nand inflicts the slash effect on enemies that touch it.",
		effect_desc_short		= "You hold a shield that converts enemy projectiles into slashbombs and inflicts the slash effect on enemies.",
        precursor_types			= "gadget,magic_artifact",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			EntityAddChild(getPlayerEntity(), EntityLoad("mods/ArcaneReliquary/files/entity_definitions/others/parry_shield_slash_handler.xml"))
		end,
		ondrop_function			= function()
			local entity_children = EntityGetAllChildren(getPlayerEntity())
			if entity_children ~= nil then
				for i,child_id in ipairs(entity_children) do
					if(EntityGetName(child_id) == "parry_shield_slash_handler" and not EntityHasTag(child_id, "relic")) then
						local x, y = EntityGetTransform(child_id)
						EntityKill(child_id)
					end
				end
			end
		end,
		excludes_relics_desc	= "It can't be used along with other Parry Shield Relic variations",
		excludes_relics_tags	= "parry_shield_variation",
		combine_exclusive		= true,
		combine_requires_relics	= "parry_shield,cursed_dagger",
	},
	{
		name 					= "parry_shield_freeze",
		ui_name					= "Parry Shield with Freeze Effect",
		categories				= "player",
		effect_desc				= "You hold a shield that converts enemy projectiles into freezebombs \nand freezes enemies that touch it.",
		effect_desc_short		= "You hold a shield that converts enemy projectiles into freezebombs and freezes enemies.",
        precursor_types			= "gadget,magic_artifact",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			EntityAddChild(getPlayerEntity(), EntityLoad("mods/ArcaneReliquary/files/entity_definitions/others/parry_shield_freeze_handler.xml"))
		end,
		ondrop_function			= function()
			local entity_children = EntityGetAllChildren(getPlayerEntity())
			if entity_children ~= nil then
				for i,child_id in ipairs(entity_children) do
					if(EntityGetName(child_id) == "parry_shield_freeze_handler" and not EntityHasTag(child_id, "relic")) then
						local x, y = EntityGetTransform(child_id)
						EntityKill(child_id)
					end
				end
			end
		end,
		excludes_relics_desc	= "It can't be used along with other Parry Shield Relic variations",
		excludes_relics_tags	= "parry_shield_variation",
		combine_exclusive		= true,
		combine_requires_relics	= "parry_shield,ice_artifact",
	},
	{
		name 					= "parry_shield_electrocute",
		ui_name					= "Parry Shield with Electrocute Effect",
		categories				= "player",
		effect_desc				= "You hold a shield that converts enemy projectiles into electrobombs \nand electrocutes enemies that touch it.",
		effect_desc_short		= "You hold a shield that converts enemy projectiles into electrobombs and electrocutes enemies.",
        precursor_types			= "gadget,magic_artifact",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			EntityAddChild(getPlayerEntity(), EntityLoad("mods/ArcaneReliquary/files/entity_definitions/others/parry_shield_electrocute_handler.xml"))
		end,
		ondrop_function			= function()
			local entity_children = EntityGetAllChildren(getPlayerEntity())
			if entity_children ~= nil then
				for i,child_id in ipairs(entity_children) do
					if(EntityGetName(child_id) == "parry_shield_electrocute_handler" and not EntityHasTag(child_id, "relic")) then
						local x, y = EntityGetTransform(child_id)
						EntityKill(child_id)
					end
				end
			end
		end,
		excludes_relics_desc	= "It can't be used along with other Parry Shield Relic variations",
		excludes_relics_tags	= "parry_shield_variation",
		combine_exclusive		= true,
		combine_requires_relics	= "parry_shield,charged_necklace",
	},
	{
		name 					= "parry_shield_wind",
		ui_name					= "Parry Shield with Wind Effect",
		categories				= "player",
		effect_desc				= "You hold a shield that converts enemy projectiles into windbombs \nand inflicts the wind effect on enemies that touch it.",
		effect_desc_short		= "You hold a shield that converts enemy projectiles into windbombs and inflicts the wind effect on enemies.",
        precursor_types			= "gadget,magic_artifact",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			EntityAddChild(getPlayerEntity(), EntityLoad("mods/ArcaneReliquary/files/entity_definitions/others/parry_shield_wind_handler.xml"))
		end,
		ondrop_function			= function()
			local entity_children = EntityGetAllChildren(getPlayerEntity())
			if entity_children ~= nil then
				for i,child_id in ipairs(entity_children) do
					if(EntityGetName(child_id) == "parry_shield_wind_handler" and not EntityHasTag(child_id, "relic")) then
						local x, y = EntityGetTransform(child_id)
						EntityKill(child_id)
					end
				end
			end
		end,
		excludes_relics_desc	= "It can't be used along with other Parry Shield Relic variations",
		excludes_relics_tags	= "parry_shield_variation",
		combine_exclusive		= true,
		combine_requires_relics	= "parry_shield,little_cloud",
	},
	{
		name 					= "parry_shield_spikes_green",
		ui_name					= "Parry Shield with Green Spikes",
		categories				= "player",
		effect_desc				= "You hold a shield that deflects enemy projectiles and imbues them with \nthe acid spit effect and also inflicts this effect on enemies that touch it.",
		effect_desc_short		= "You hold a shield that deflects enemy projectiles and imbues them with the acid spit effect and causes this effect on enemies.",
        precursor_types			= "gadget,magic_artifact",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			EntityAddChild(getPlayerEntity(), EntityLoad("mods/ArcaneReliquary/files/entity_definitions/others/parry_shield_spikes_green_handler.xml"))
		end,
		ondrop_function			= function()
			local entity_children = EntityGetAllChildren(getPlayerEntity())
			if entity_children ~= nil then
				for i,child_id in ipairs(entity_children) do
					if(EntityGetName(child_id) == "parry_shield_spikes_green_handler" and not EntityHasTag(child_id, "relic")) then
						local x, y = EntityGetTransform(child_id)
						EntityKill(child_id)
					end
				end
			end
		end,
		excludes_relics_desc	= "It can't be used along with other Parry Shield Relic variations",
		excludes_relics_tags	= "parry_shield_variation",
		combine_exclusive		= true,
		combine_requires_relics	= "parry_shield_spikes,poisonous_sting",
	},
	{
		name 					= "parry_shield_spikes_orange",
		ui_name					= "Parry Shield with Orange Spikes",
		categories				= "player",
		effect_desc				= "You hold a shield that deflects enemy projectiles and imbues them with \nthe lava spit effect and also inflicts this effect on enemies that touch it.",
		effect_desc_short		= "You hold a shield that deflects enemy projectiles and imbues them with the lava spit effect and causes this effect on enemies.",
        precursor_types			= "gadget,magic_artifact",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			EntityAddChild(getPlayerEntity(), EntityLoad("mods/ArcaneReliquary/files/entity_definitions/others/parry_shield_spikes_orange_handler.xml"))
		end,
		ondrop_function			= function()
			local entity_children = EntityGetAllChildren(getPlayerEntity())
			if entity_children ~= nil then
				for i,child_id in ipairs(entity_children) do
					if(EntityGetName(child_id) == "parry_shield_spikes_orange_handler" and not EntityHasTag(child_id, "relic")) then
						local x, y = EntityGetTransform(child_id)
						EntityKill(child_id)
					end
				end
			end
		end,
		excludes_relics_desc	= "It can't be used along with other Parry Shield Relic variations",
		excludes_relics_tags	= "parry_shield_variation",
		combine_exclusive		= true,
		combine_requires_relics	= "parry_shield_spikes,magmatic_artifact",
	},
	{
		name 					= "parry_shield_spikes_red",
		ui_name					= "Parry Shield with Red Spikes",
		categories				= "player",
		effect_desc				= "You hold a shield that deflects enemy projectiles and imbues them with \nthe corrosive blood spit effect and also inflicts this effect on enemies that touch it.",
		effect_desc_short		= "You hold a shield that deflects enemy projectiles and imbues them with the corrosive blood spit effect and causes this effect on enemies.",
        precursor_types			= "gadget,magic_artifact",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			EntityAddChild(getPlayerEntity(), EntityLoad("mods/ArcaneReliquary/files/entity_definitions/others/parry_shield_spikes_red_handler.xml"))
		end,
		ondrop_function			= function()
			local entity_children = EntityGetAllChildren(getPlayerEntity())
			if entity_children ~= nil then
				for i,child_id in ipairs(entity_children) do
					if(EntityGetName(child_id) == "parry_shield_spikes_red_handler" and not EntityHasTag(child_id, "relic")) then
						local x, y = EntityGetTransform(child_id)
						EntityKill(child_id)
					end
				end
			end
		end,
		excludes_relics_desc	= "It can't be used along with other Parry Shield Relic variations",
		excludes_relics_tags	= "parry_shield_variation",
		combine_exclusive		= true,
		combine_requires_relics	= "parry_shield_spikes,corrosive_blood_essence",
	},
	{
		name 					= "parry_shield_spikes_pink",
		ui_name					= "Parry Shield with Pink Spikes",
		categories				= "player",
		effect_desc				= "You hold a shield that deflects enemy projectiles and imbues them with \nthe polymorphine spit effect and also inflicts this effect on enemies that touch it.",
		effect_desc_short		= "You hold a shield that deflects enemy projectiles and imbues them with the polymorphine spit effect and causes this effect on enemies.",
        precursor_types			= "gadget,magic_artifact",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			EntityAddChild(getPlayerEntity(), EntityLoad("mods/ArcaneReliquary/files/entity_definitions/others/parry_shield_spikes_pink_handler.xml"))
		end,
		ondrop_function			= function()
			local entity_children = EntityGetAllChildren(getPlayerEntity())
			if entity_children ~= nil then
				for i,child_id in ipairs(entity_children) do
					if(EntityGetName(child_id) == "parry_shield_spikes_pink_handler" and not EntityHasTag(child_id, "relic")) then
						local x, y = EntityGetTransform(child_id)
						EntityKill(child_id)
					end
				end
			end
		end,
		excludes_relics_desc	= "It can't be used along with other Parry Shield Relic variations",
		excludes_relics_tags	= "parry_shield_variation",
		combine_exclusive		= true,
		combine_requires_relics	= "parry_shield_spikes,poly_flask",
	},
	{
		name 					= "parry_shield_spikes_blue",
		ui_name					= "Parry Shield with Blue Spikes",
		categories				= "player",
		effect_desc				= "You hold a shield that deflects enemy projectiles and imbues them with \nthe water spit effect and also inflicts this effect on enemies that touch it.",
		effect_desc_short		= "You hold a shield that deflects enemy projectiles and imbues them with the water spit effect and causes this effect on enemies.",
        precursor_types			= "gadget,magic_artifact",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			EntityAddChild(getPlayerEntity(), EntityLoad("mods/ArcaneReliquary/files/entity_definitions/others/parry_shield_spikes_blue_handler.xml"))
		end,
		ondrop_function			= function()
			local entity_children = EntityGetAllChildren(getPlayerEntity())
			if entity_children ~= nil then
				for i,child_id in ipairs(entity_children) do
					if(EntityGetName(child_id) == "parry_shield_spikes_blue_handler" and not EntityHasTag(child_id, "relic")) then
						local x, y = EntityGetTransform(child_id)
						EntityKill(child_id)
					end
				end
			end
		end,
		excludes_relics_desc	= "It can't be used along with other Parry Shield Relic variations",
		excludes_relics_tags	= "parry_shield_variation",
		combine_exclusive		= true,
		combine_requires_relics	= "parry_shield_spikes,pyrophobic_stone",
	},
	{
		name 					= "parry_shield_spikes_purple",
		ui_name					= "Parry Shield with Purple Spikes",
		categories				= "player",
		effect_desc				= "You hold a shield that deflects enemy projectiles and imbues them with \nthe poison spit effect and also inflicts this effect on enemies that touch it.",
		effect_desc_short		= "You hold a shield that deflects enemy projectiles and imbues them with the poison spit effect and causes this effect on enemies.",
        precursor_types			= "gadget,magic_artifact",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			EntityAddChild(getPlayerEntity(), EntityLoad("mods/ArcaneReliquary/files/entity_definitions/others/parry_shield_spikes_purple_handler.xml"))
		end,
		ondrop_function			= function()
			local entity_children = EntityGetAllChildren(getPlayerEntity())
			if entity_children ~= nil then
				for i,child_id in ipairs(entity_children) do
					if(EntityGetName(child_id) == "parry_shield_spikes_purple_handler" and not EntityHasTag(child_id, "relic")) then
						local x, y = EntityGetTransform(child_id)
						EntityKill(child_id)
					end
				end
			end
		end,
		excludes_relics_desc	= "It can't be used along with other Parry Shield Relic variations",
		excludes_relics_tags	= "parry_shield_variation",
		combine_exclusive		= true,
		combine_requires_relics	= "parry_shield_spikes,vile_artifact",
	},
	{
		name 					= "parry_shield_spikes_fiery",
		ui_name					= "Parry Shield with Fiery Spikes",
		categories				= "player",
		effect_desc				= "You hold a shield that deflects enemy projectiles and imbues them with \nthe gunpowder spit effect and also inflicts this effect on enemies that touch it.",
		effect_desc_short		= "You hold a shield that deflects enemy projectiles and imbues them with the gunpowder spit effect and causes this effect on enemies.",
        precursor_types			= "gadget,magic_artifact",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			EntityAddChild(getPlayerEntity(), EntityLoad("mods/ArcaneReliquary/files/entity_definitions/others/parry_shield_spikes_fiery_handler.xml"))
		end,
		ondrop_function			= function()
			local entity_children = EntityGetAllChildren(getPlayerEntity())
			if entity_children ~= nil then
				for i,child_id in ipairs(entity_children) do
					if(EntityGetName(child_id) == "parry_shield_spikes_fiery_handler" and not EntityHasTag(child_id, "relic")) then
						local x, y = EntityGetTransform(child_id)
						EntityKill(child_id)
					end
				end
			end
		end,
		excludes_relics_desc	= "It can't be used along with other Parry Shield Relic variations",
		excludes_relics_tags	= "parry_shield_variation",
		combine_exclusive		= true,
		combine_requires_relics	= "parry_shield_spikes,firecrackers",
	},
	{
		name 					= "parry_shield_spikes_yellow",
		ui_name					= "Parry Shield with Yellow Spikes",
		categories				= "player",
		effect_desc				= "You hold a shield that deflects enemy projectiles and imbues them with \nthe urine spit effect and also inflicts this effect on enemies that touch it.",
		effect_desc_short		= "You hold a shield that deflects enemy projectiles and imbues them with the urine spit effect and causes this effect on enemies.",
        precursor_types			= "gadget,magic_artifact",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			EntityAddChild(getPlayerEntity(), EntityLoad("mods/ArcaneReliquary/files/entity_definitions/others/parry_shield_spikes_yellow_handler.xml"))
		end,
		ondrop_function			= function()
			local entity_children = EntityGetAllChildren(getPlayerEntity())
			if entity_children ~= nil then
				for i,child_id in ipairs(entity_children) do
					if(EntityGetName(child_id) == "parry_shield_spikes_yellow_handler" and not EntityHasTag(child_id, "relic")) then
						local x, y = EntityGetTransform(child_id)
						EntityKill(child_id)
					end
				end
			end
		end,
		excludes_relics_desc	= "It can't be used along with other Parry Shield Relic variations",
		excludes_relics_tags	= "parry_shield_variation",
		combine_exclusive		= true,
		combine_requires_relics	= "parry_shield_spikes,uric_stone",
	},
	{
		name 					= "parry_shield_spikes_cyan",
		ui_name					= "Parry Shield with Cyan Spikes",
		categories				= "player",
		effect_desc				= "You hold a shield that deflects enemy projectiles and imbues them with \nthe freezing liquid spit effect and also inflicts this effect on enemies that touch it.",
		effect_desc_short		= "You hold a shield that deflects enemy projectiles and imbues them with the freezing liquid spit effect and causes this effect on enemies.",
        precursor_types			= "gadget,magic_artifact",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			EntityAddChild(getPlayerEntity(), EntityLoad("mods/ArcaneReliquary/files/entity_definitions/others/parry_shield_spikes_cyan_handler.xml"))
		end,
		ondrop_function			= function()
			local entity_children = EntityGetAllChildren(getPlayerEntity())
			if entity_children ~= nil then
				for i,child_id in ipairs(entity_children) do
					if(EntityGetName(child_id) == "parry_shield_spikes_cyan_handler" and not EntityHasTag(child_id, "relic")) then
						local x, y = EntityGetTransform(child_id)
						EntityKill(child_id)
					end
				end
			end
		end,
		excludes_relics_desc	= "It can't be used along with other Parry Shield Relic variations",
		excludes_relics_tags	= "parry_shield_variation",
		combine_exclusive		= true,
		combine_requires_relics	= "parry_shield_spikes,glacial_artifact",
	},
	{
		name 					= "laser_sight",
		ui_name					= "Laser Sight",
		categories				= "player",
		effect_desc				= "When holding wands you emit a harmless laser beam that stops on obstacles and your cursor.",
		effect_desc_short		= "When holding wands you emit a harmless laser beam that stops on obstacles and your cursor.",
        precursor_types			= "gadget",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			EntityAddChild(getPlayerEntity(), EntityLoad("mods/ArcaneReliquary/files/entity_definitions/others/laser_sight_handler.xml"))
		end,
		ondrop_function			= function()
			local entity_children = EntityGetAllChildren(getPlayerEntity())
			if entity_children ~= nil then
				for i,child_id in ipairs(entity_children) do
					if(EntityGetName(child_id) == "laser_sight_handler" and not EntityHasTag(child_id, "relic")) then
						local x, y = EntityGetTransform(child_id)
						EntityKill(child_id)
					end
				end
			end
		end,
		excludes_relics_desc	= "It can't be used along with other Laser Sight Relic variations",
		excludes_relics_tags	= "laser_sight_variation",
	},
	{
		name 					= "laser_sight_explosive",
		ui_name					= "Explosive Laser Sight",
		categories				= "player",
		effect_desc				= "When holding wands you emit a laser beam that stops on obstacles and your \ncursor, causing small explosions that punch through terrain where it stops, consuming jetpack energy when it happens. \nUse attack button to toggle between it and the harmless laser beam.",
		effect_desc_short		= "When holding wands you emit a laser beam that stops and causes small explosions on obstacles and your cursor.",
        precursor_types			= "gadget",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			EntityAddChild(getPlayerEntity(), EntityLoad("mods/ArcaneReliquary/files/entity_definitions/others/laser_sight_explosive_handler.xml"))
		end,
		ondrop_function			= function()
			local entity_children = EntityGetAllChildren(getPlayerEntity())
			if entity_children ~= nil then
				for i,child_id in ipairs(entity_children) do
					if(EntityGetName(child_id) == "laser_sight_explosive_handler" and not EntityHasTag(child_id, "relic")) then
						local x, y = EntityGetTransform(child_id)
						EntityKill(child_id)
					end
				end
			end
		end,
		excludes_relics_desc	= "It can't be used along with other Laser Sight Relic variations",
		excludes_relics_tags	= "laser_sight_variation",
		combine_exclusive		= true,
		combine_requires_relics	= "laser_sight,nuclear_glass",
	},
	{
		name 					= "laser_sight_freezing",
		ui_name					= "Freezing Laser Sight",
		categories				= "player",
		effect_desc				= "When holding wands you emit a laser beam that stops on obstacles and your \ncursor, causing small freezing explosions where it stops, consuming jetpack energy when it happens. \nUse attack button to toggle between it and the harmless laser beam.",
		effect_desc_short		= "When holding wands you emit a laser beam that stops and causes small freezing explosions on obstacles and your cursor.",
        precursor_types			= "gadget",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			EntityAddChild(getPlayerEntity(), EntityLoad("mods/ArcaneReliquary/files/entity_definitions/others/laser_sight_freezing_handler.xml"))
		end,
		ondrop_function			= function()
			local entity_children = EntityGetAllChildren(getPlayerEntity())
			if entity_children ~= nil then
				for i,child_id in ipairs(entity_children) do
					if(EntityGetName(child_id) == "laser_sight_freezing_handler" and not EntityHasTag(child_id, "relic")) then
						local x, y = EntityGetTransform(child_id)
						EntityKill(child_id)
					end
				end
			end
		end,
		excludes_relics_desc	= "It can't be used along with other Laser Sight Relic variations",
		excludes_relics_tags	= "laser_sight_variation",
		combine_exclusive		= true,
		combine_requires_relics	= "laser_sight,ice_artifact",
	},
	{
		name 					= "laser_sight_electrocuting",
		ui_name					= "Electrocuting Laser Sight",
		categories				= "player",
		effect_desc				= "When holding wands you emit a laser beam that stops on obstacles and your \ncursor, causing small electrocuting explosions where it stops, consuming jetpack energy when it happens. \nUse attack button to toggle between it and the harmless laser beam.",
		effect_desc_short		= "When holding wands you emit a laser beam that stops and causes small electrocuting explosions on obstacles and your cursor.",
        precursor_types			= "gadget",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			EntityAddChild(getPlayerEntity(), EntityLoad("mods/ArcaneReliquary/files/entity_definitions/others/laser_sight_electrocuting_handler.xml"))
		end,
		ondrop_function			= function()
			local entity_children = EntityGetAllChildren(getPlayerEntity())
			if entity_children ~= nil then
				for i,child_id in ipairs(entity_children) do
					if(EntityGetName(child_id) == "laser_sight_electrocuting_handler" and not EntityHasTag(child_id, "relic")) then
						local x, y = EntityGetTransform(child_id)
						EntityKill(child_id)
					end
				end
			end
		end,
		excludes_relics_desc	= "It can't be used along with other Laser Sight Relic variations",
		excludes_relics_tags	= "laser_sight_variation",
		combine_exclusive		= true,
		combine_requires_relics	= "laser_sight,charged_necklace",
	},
	{
		name 					= "laser_sight_slash",
		ui_name					= "Slash Laser Sight",
		categories				= "player",
		effect_desc				= "When holding wands you emit a laser beam that stops on obstacles and your \ncursor, causing small explosions that inflict the slash effect where it stops, consuming jetpack energy when it happens. \nUse attack button to toggle between it and the harmless laser beam.",
		effect_desc_short		= "When holding wands you emit a laser beam that stops and causes small slash explosions on obstacles and your cursor.",
        precursor_types			= "gadget",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			EntityAddChild(getPlayerEntity(), EntityLoad("mods/ArcaneReliquary/files/entity_definitions/others/laser_sight_slash_handler.xml"))
		end,
		ondrop_function			= function()
			local entity_children = EntityGetAllChildren(getPlayerEntity())
			if entity_children ~= nil then
				for i,child_id in ipairs(entity_children) do
					if(EntityGetName(child_id) == "laser_sight_slash_handler" and not EntityHasTag(child_id, "relic")) then
						local x, y = EntityGetTransform(child_id)
						EntityKill(child_id)
					end
				end
			end
		end,
		excludes_relics_desc	= "It can't be used along with other Laser Sight Relic variations",
		excludes_relics_tags	= "laser_sight_variation",
		combine_exclusive		= true,
		combine_requires_relics	= "laser_sight,shuriken",
	},
	{
		name 					= "laser_sight_slap",
		ui_name					= "Slap Laser Sight",
		categories				= "player",
		effect_desc				= "When holding wands you emit a laser beam that stops on obstacles and your \ncursor, causing small explosions that inflict the slap effect where it stops, consuming jetpack energy when it happens. \nUse attack button to toggle between it and the harmless laser beam.",
		effect_desc_short		= "When holding wands you emit a laser beam that stops and causes small slap explosions on obstacles and your cursor.",
        precursor_types			= "gadget",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			EntityAddChild(getPlayerEntity(), EntityLoad("mods/ArcaneReliquary/files/entity_definitions/others/laser_sight_slap_handler.xml"))
		end,
		ondrop_function			= function()
			local entity_children = EntityGetAllChildren(getPlayerEntity())
			if entity_children ~= nil then
				for i,child_id in ipairs(entity_children) do
					if(EntityGetName(child_id) == "laser_sight_slap_handler" and not EntityHasTag(child_id, "relic")) then
						local x, y = EntityGetTransform(child_id)
						EntityKill(child_id)
					end
				end
			end
		end,
		excludes_relics_desc	= "It can't be used along with other Laser Sight Relic variations",
		excludes_relics_tags	= "laser_sight_variation",
		combine_exclusive		= true,
		combine_requires_relics	= "laser_sight,fly_swatter",
	},
	{
		name 					= "laser_sight_bleeding",
		ui_name					= "Bleeding Laser Sight",
		categories				= "player",
		effect_desc				= "When holding wands you emit a laser beam that stops on obstacles and your \ncursor, causing small explosions that inflict the bleeding effect where it stops, consuming jetpack energy when it happens. \nUse attack button to toggle between it and the harmless laser beam.",
		effect_desc_short		= "When holding wands you emit a laser beam that stops and causes small bleeding explosions on obstacles and your cursor.",
        precursor_types			= "gadget",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			EntityAddChild(getPlayerEntity(), EntityLoad("mods/ArcaneReliquary/files/entity_definitions/others/laser_sight_bleeding_handler.xml"))
		end,
		ondrop_function			= function()
			local entity_children = EntityGetAllChildren(getPlayerEntity())
			if entity_children ~= nil then
				for i,child_id in ipairs(entity_children) do
					if(EntityGetName(child_id) == "laser_sight_bleeding_handler" and not EntityHasTag(child_id, "relic")) then
						local x, y = EntityGetTransform(child_id)
						EntityKill(child_id)
					end
				end
			end
		end,
		excludes_relics_desc	= "It can't be used along with other Laser Sight Relic variations",
		excludes_relics_tags	= "laser_sight_variation",
		combine_exclusive		= true,
		combine_requires_relics	= "laser_sight,ancient_dagger",
	},
	{
		name 					= "laser_sight_explode",
		ui_name					= "Explode Laser Sight",
		categories				= "player",
		effect_desc				= "When holding wands you emit a laser beam that stops on obstacles and your \ncursor, causing small explosions that inflict the exploding effect where it stops, consuming jetpack energy when it happens. \nUse attack button to toggle between it and the harmless laser beam.",
		effect_desc_short		= "When holding wands you emit a laser beam that stops and causes small exploding explosions on obstacles and your cursor.",
        precursor_types			= "gadget",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			EntityAddChild(getPlayerEntity(), EntityLoad("mods/ArcaneReliquary/files/entity_definitions/others/laser_sight_explode_handler.xml"))
		end,
		ondrop_function			= function()
			local entity_children = EntityGetAllChildren(getPlayerEntity())
			if entity_children ~= nil then
				for i,child_id in ipairs(entity_children) do
					if(EntityGetName(child_id) == "laser_sight_explode_handler" and not EntityHasTag(child_id, "relic")) then
						local x, y = EntityGetTransform(child_id)
						EntityKill(child_id)
					end
				end
			end
		end,
		excludes_relics_desc	= "It can't be used along with other Laser Sight Relic variations",
		excludes_relics_tags	= "laser_sight_variation",
		combine_exclusive		= true,
		combine_requires_relics	= "laser_sight,exploding_scepter",
	},
	{
		name 					= "elemental_sphere",
		ui_name					= "Elemental Sphere",
		categories				= "projectiles,status_effects,materials",
		effect_desc				= "Your spells have a 50% chance of inducing enemies hit by them to spit random materials.",
		effect_desc_short		= "Your spells induce enemies to spit random materials.",
        precursor_types			= "magic_artifact,effect_stone",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("elemental_sphere") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 50 + (5 * getRelicLevel("elemental_sphere"))
				if(chance >= 1 and chance <= odds) then
					local material_spit_effects_pool = {
						getCustomEffect("effect_material_spit_acid").path,
						getCustomEffect("effect_material_spit_lava").path,
						getCustomEffect("effect_material_spit_poison").path,
						getCustomEffect("effect_material_spit_poly").path,
						getCustomEffect("effect_material_spit_freezing_liquid").path,
						getCustomEffect("effect_material_spit_water").path,
						getCustomEffect("effect_material_spit_urine").path,
						getCustomEffect("effect_material_spit_gunpowder").path,
						getCustomEffect("effect_material_spit_berserkium").path,
						getCustomEffect("effect_material_spit_corrosive_blood").path,
						getCustomEffect("effect_material_spit_blood").path,
						getCustomEffect("effect_material_spit_gold").path,
						getCustomEffect("effect_material_spit_ambrosia").path,
						getCustomEffect("effect_material_spit_invisiblium").path,
						getCustomEffect("effect_material_spit_healthium").path,
						getCustomEffect("effect_material_spit_teleportatium").path,
						getCustomEffect("effect_material_spit_alcohol").path,
						getCustomEffect("effect_material_spit_oil").path,
						getCustomEffect("effect_material_spit_pheromone").path,
						getCustomEffect("effect_material_spit_concentrated_mana").path,
						getCustomEffect("effect_material_spit_slime").path,
					}
					attachLoadEffectOntoEntitiestoProjectile(projectile_entity_id, material_spit_effects_pool[Random(1,#material_spit_effects_pool)], "plasma_fading_pink")
				end
			end
		end,
	},
	{
		name 					= "concentrated_mana_gem",
		ui_name					= "Concentrated Mana Gem",
		categories				= "projectiles,materials",
		effect_desc				= "Spells you cast that explode on impact have a 20% chance of releasing concentrated mana on impact.",
		effect_desc_short		= "Your spells release concentrated mana.",
        precursor_types			= "elemental_gem",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("concentrated_mana_gem") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 20 + (5 * getRelicLevel("concentrated_mana_gem"))
				if(chance >= 1 and chance <= odds) then
					attachLoadEntityOnExplosiontoProjectile(projectile_entity_id, getLoadOnExplosion("launch_elemental_particles_concentrated_mana").path, "magic_liquid_mana_regeneration")
				end
			end
		end,
	},
	{
		name 					= "scarlet_gem",
		ui_name					= "Scarlet Gem",
		categories				= "projectiles,materials",
		effect_desc				= "Spells you cast that explode on impact have a 30% chance of releasing red blood on impact.",
		effect_desc_short		= "Your spells release red blood.",
        precursor_types			= "elemental_gem",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("scarlet_gem") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 30 + (5 * getRelicLevel("scarlet_gem"))
				if(chance >= 1 and chance <= odds) then
					attachLoadEntityOnExplosiontoProjectile(projectile_entity_id, getLoadOnExplosion("launch_elemental_particles_blood").path, "spark_red")
				end
			end
		end,
	},
	{
		name 					= "elemental_ring",
		ui_name					= "Elemental Ring",
		categories				= "player,status_effects,materials",
		effect_desc				= "Upon taking damage from melee attacks or enemy projectiles you have a 50% chance of spitting random materials.",
		effect_desc_short		= "Being damaged by enemies makes you spit random materials.",
        precursor_types			= "magic_artifact,ring",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		ondamaged_function		= function(damage, message, entity_thats_responsible, is_fatal, projectile_thats_responsible )
			if(isRelicOwned("elemental_ring") >= 1) then	
				if(EntityHasTag(entity_thats_responsible, "enemy")) then
					SetRandomSeed( GameGetFrameNum(), math.random(1,50))
					local chance = Random(1,100)
					local odds = 50 + (5 * getRelicLevel("elemental_ring"))
					if(chance >= 1 and chance <= odds) then
						local material_spit_effects_pool = {
							getCustomEffect("effect_material_spit_acid").path,
							getCustomEffect("effect_material_spit_lava").path,
							getCustomEffect("effect_material_spit_poison").path,
							getCustomEffect("effect_material_spit_poly").path,
							getCustomEffect("effect_material_spit_freezing_liquid").path,
							getCustomEffect("effect_material_spit_water").path,
							getCustomEffect("effect_material_spit_urine").path,
							getCustomEffect("effect_material_spit_gunpowder").path,
							getCustomEffect("effect_material_spit_berserkium").path,
							getCustomEffect("effect_material_spit_corrosive_blood").path,
							getCustomEffect("effect_material_spit_blood").path,
							getCustomEffect("effect_material_spit_gold").path,
							getCustomEffect("effect_material_spit_ambrosia").path,
							getCustomEffect("effect_material_spit_invisiblium").path,
							getCustomEffect("effect_material_spit_healthium").path,
							getCustomEffect("effect_material_spit_teleportatium").path,
							getCustomEffect("effect_material_spit_alcohol").path,
							getCustomEffect("effect_material_spit_oil").path,
							getCustomEffect("effect_material_spit_pheromone").path,
							getCustomEffect("effect_material_spit_concentrated_mana").path,
							getCustomEffect("effect_material_spit_slime").path,
						}
						LoadGameEffectEntityTo( GetUpdatedEntityID(), material_spit_effects_pool[Random(1,#material_spit_effects_pool)] )
					end
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "elemental_sphere,elemental_band",
	},
	{
		name 					= "elemental_necklace",
		ui_name					= "Elemental Necklace",
		categories				= "enemies,status_effects,materials",
		effect_desc				= "Enemies that cause damage to you have a 50% chance of spitting random materials.",
		effect_desc_short		= "Enemies that cause damage to you spit random materials.",
        precursor_types			= "magic_artifact,necklace",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		ondamaged_function		= function(damage, message, entity_thats_responsible, is_fatal, projectile_thats_responsible )
			if(isRelicOwned("elemental_necklace") >= 1) then	
				if(EntityHasTag(entity_thats_responsible, "enemy")) then
					SetRandomSeed( GameGetFrameNum(), math.random(1,50))
					local chance = Random(1,100)
					local odds = 50 + (5 * getRelicLevel("elemental_necklace"))
					if(chance >= 1 and chance <= odds) then
						local material_spit_effects_pool = {
							getCustomEffect("effect_material_spit_acid").path,
							getCustomEffect("effect_material_spit_lava").path,
							getCustomEffect("effect_material_spit_poison").path,
							getCustomEffect("effect_material_spit_poly").path,
							getCustomEffect("effect_material_spit_freezing_liquid").path,
							getCustomEffect("effect_material_spit_water").path,
							getCustomEffect("effect_material_spit_urine").path,
							getCustomEffect("effect_material_spit_gunpowder").path,
							getCustomEffect("effect_material_spit_berserkium").path,
							getCustomEffect("effect_material_spit_corrosive_blood").path,
							getCustomEffect("effect_material_spit_blood").path,
							getCustomEffect("effect_material_spit_gold").path,
							getCustomEffect("effect_material_spit_ambrosia").path,
							getCustomEffect("effect_material_spit_invisiblium").path,
							getCustomEffect("effect_material_spit_healthium").path,
							getCustomEffect("effect_material_spit_teleportatium").path,
							getCustomEffect("effect_material_spit_alcohol").path,
							getCustomEffect("effect_material_spit_oil").path,
							getCustomEffect("effect_material_spit_pheromone").path,
							getCustomEffect("effect_material_spit_concentrated_mana").path,
							getCustomEffect("effect_material_spit_slime").path,
						}
						LoadGameEffectEntityTo( entity_thats_responsible, material_spit_effects_pool[Random(1,#material_spit_effects_pool)] )
					end
				end
			end
		end,
	},
	{
		name 					= "elemental_bracelet",
		ui_name					= "Elemental Bracelet",
		categories				= "player,enemies,status_effects,materials",
		effect_desc				= "Upon taking damage from enemy attacks, there's a 50% chance all enemies nearby, except the one that attacked you, will spit random materials.",
		effect_desc_short		= "Taking damage from enemy attacks makes nearby enemies spit random materials.",
        precursor_types			= "magic_artifact",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		ondamaged_function		= function(damage, message, entity_thats_responsible, is_fatal, projectile_thats_responsible )
			if(isRelicOwned("elemental_bracelet") >= 1) then	
				if(EntityHasTag(entity_thats_responsible, "enemy")) then
					SetRandomSeed( GameGetFrameNum(), math.random(1,50))
					local chance = Random(1,100)
					local odds = 50 + (5 * getRelicLevel("elemental_bracelet"))
					if(chance >= 1 and chance <= odds) then
						local material_spit_effects_pool = {
							getCustomEffect("effect_material_spit_acid").path,
							getCustomEffect("effect_material_spit_lava").path,
							getCustomEffect("effect_material_spit_poison").path,
							getCustomEffect("effect_material_spit_poly").path,
							getCustomEffect("effect_material_spit_freezing_liquid").path,
							getCustomEffect("effect_material_spit_water").path,
							getCustomEffect("effect_material_spit_urine").path,
							getCustomEffect("effect_material_spit_gunpowder").path,
							getCustomEffect("effect_material_spit_berserkium").path,
							getCustomEffect("effect_material_spit_corrosive_blood").path,
							getCustomEffect("effect_material_spit_blood").path,
							getCustomEffect("effect_material_spit_gold").path,
							getCustomEffect("effect_material_spit_ambrosia").path,
							getCustomEffect("effect_material_spit_invisiblium").path,
							getCustomEffect("effect_material_spit_healthium").path,
							getCustomEffect("effect_material_spit_teleportatium").path,
							getCustomEffect("effect_material_spit_alcohol").path,
							getCustomEffect("effect_material_spit_oil").path,
							getCustomEffect("effect_material_spit_pheromone").path,
							getCustomEffect("effect_material_spit_concentrated_mana").path,
							getCustomEffect("effect_material_spit_slime").path,
						}
						
						local player_x, player_y = EntityGetTransform(GetUpdatedEntityID())
						local enemies = EntityGetInRadiusWithTag( player_x, player_y, 200, "enemy" )
						for i, enemy_id in ipairs(enemies) do
							if(enemy_id ~= entity_thats_responsible) then
								LoadGameEffectEntityTo( enemy_id, material_spit_effects_pool[Random(1,#material_spit_effects_pool)] )
							end
						end						
					end
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "elemental_ring,elemental_necklace",
	},
	{
		name 					= "elemental_sphere_chaotic",
		ui_name					= "Chaotic Elemental Sphere",
		categories				= "enemies,materials",
		effect_desc				= "Enemies that cause damage to you have a 30% chance of being transmuted into random materials.",
		effect_desc_short		= "Enemies that cause damage to you are transmuted into random materials.",
        precursor_types			= "magic_artifact,effect_stone",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		ondamaged_function		= function(damage, message, entity_thats_responsible, is_fatal, projectile_thats_responsible )
			if(isRelicOwned("elemental_sphere_chaotic") >= 1) then	
				if(EntityHasTag(entity_thats_responsible, "enemy")) then
					SetRandomSeed( GameGetFrameNum(), math.random(1,50))
					local chance = Random(1,100)
					local odds = 30 + (5 * getRelicLevel("elemental_sphere_chaotic"))
					if(chance >= 1 and chance <= odds) then
						LoadGameEffectEntityTo( entity_thats_responsible, getCustomEffect("effect_convert_to_random_material").path )
					end
				end
			end
		end,
	},
	{
		name 					= "elemental_ring_chaotic",
		ui_name					= "Chaotic Elemental Ring",
		categories				= "enemies,materials",
		effect_desc				= "When you kill an enemy with your spells there's a 30% all enemies in the vicinity will be transmuted into random materials.",
		effect_desc_short		= "Killing enemies transmutes other enemies nearby into random materials.",
        precursor_types			= "magic_artifact,ring",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onenemykill_function	= function(damage_type_bit_field, damage_message, entity_thats_responsible, drop_items)			
			if(isRelicOwned("elemental_ring_chaotic") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 30 + (5 * getRelicLevel("elemental_ring_chaotic"))
				if(chance >= 1 and chance <= odds) then
					local player_x, player_y = EntityGetTransform(getPlayerEntity())
					local enemies = EntityGetInRadiusWithTag( player_x, player_y, 200, "enemy" )
					for i, enemy_id in ipairs(enemies) do
						LoadGameEffectEntityTo( enemy_id, getCustomEffect("effect_convert_to_random_material").path)
					end	
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "elemental_sphere_chaotic,elemental_band",
	},
	{
		name 					= "elemental_necklace_chaotic",
		ui_name					= "Chaotic Elemental Necklace",
		categories				= "player,kick,materials",
		effect_desc				= "Enemies you kick are immediatelly transmuted into random materials. \nIt also synergizes with Sniper Scope relic effect.",
		effect_desc_short		= "Enemies you kick are immediatelly transmuted into random materials.",
        precursor_types			= "magic_artifact,necklace",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onenemykicked_function	= function (entity_who_kicked)
			if(isRelicOwned("elemental_necklace_chaotic") >= 1) then
				LoadGameEffectEntityTo( GetUpdatedEntityID(), getCustomEffect("effect_convert_to_random_material").path)
			end
		end,
		onextendedkick_function	= function( entity_id, extended_kick_id)
			if(isRelicOwned("elemental_necklace_chaotic") >= 1) then
				attachLoadEffectOntoEntitiestoProjectile(extended_kick_id, getCustomEffect("effect_convert_to_random_material").path, "material_rainbow")
			end
		end,
	},
	{
		name 					= "elemental_bracelet_chaotic",
		ui_name					= "Chaotic Elemental Bracelet",
		categories				= "player,enemies,materials",
		effect_desc				= "Enemies that get very close to you might spontaneously be transmuted into random materials.",
		effect_desc_short		= "Enemies that get very close to you might spontaneously be transmuted into random materials.",
        precursor_types			= "magic_artifact",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		overtime_function		= function()
			if(isRelicOwned("elemental_bracelet_chaotic") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				if(Random(1,2) == 1) then
					local player_x, player_y = EntityGetTransform(getPlayerEntity())
					local enemies = EntityGetInRadiusWithTag( player_x, player_y, 40, "enemy" )
					for i, enemy_id in ipairs(enemies) do
						LoadGameEffectEntityTo( enemy_id, getCustomEffect("effect_convert_to_random_material").path)
					end	
				end
			end
		end,
	},
	{
		name 					= "buckshot",
		ui_name					= "Buckshot",
		categories				= "projectiles",
		effect_desc				= "Provided they travel enough time, your spells have a 30% chance of shooting \ninvisible shrapnel projectiles that penetrate walls when they hit something.",
		effect_desc_short		= "Your spells shoot invisible shrapnel projectiles when they hit something.",
        precursor_types			= "gadget",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("buckshot") >= 1) then
				if(not isPhysicsProjectile(projectile_entity_id)) then
					SetRandomSeed( GameGetFrameNum(), math.random(1,50))
					local chance = Random(1,100)
					local odds = 30 + (5 * getRelicLevel("buckshot"))
					if(chance >= 1 and chance <= odds) then
						projectileRemoveBouncing(projectile_entity_id)
						
						addNewInternalVariable(projectile_entity_id, "projectile_pos_x", "value_int", 0)
						addNewInternalVariable(projectile_entity_id, "projectile_pos_y", "value_int", 0)
						addNewInternalVariable(projectile_entity_id, "projectile_vel_x", "value_int", 0)
						addNewInternalVariable(projectile_entity_id, "projectile_vel_y", "value_int", 0)
						
						addBuffVisualIndicator_Projectile(projectile_entity_id, "spark")
						addNewInternalVariable(projectile_entity_id, "projectile_definition", "value_string", "mods/ArcaneReliquary/files/entity_definitions/custom_projectiles/invisible_shrapnel_projectile.xml")
							
						addNewLuaCallback(projectile_entity_id, "overtime", "mods/ArcaneReliquary/files/scripts/projectiles/store_projectile_vel_pos.lua", 1, false, "store_projectile_vel_pos")
						addNewLuaCallback_onRemoved(projectile_entity_id, "mods/ArcaneReliquary/files/scripts/projectiles/launch_directional_shrapnel_projectile_multiple.lua")
					end
				end
			end
		end
	},
	{
		name 					= "buckshot_magnesium",
		ui_name					= "Magnesium Buckshot",
		categories				= "projectiles",
		effect_desc				= "Provided they travel enough time, your spells have a 30% chance of shooting \ninvisible shrapnel projectiles that penetrate walls and explode when they hit something.",
		effect_desc_short		= "Your spells shoot invisible explosive shrapnel projectiles when they hit something.",
        precursor_types			= "gadget",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("buckshot_magnesium") >= 1) then
				if(not isPhysicsProjectile(projectile_entity_id)) then
					SetRandomSeed( GameGetFrameNum(), math.random(1,50))
					local chance = Random(1,100)
					local odds = 30 + (5 * getRelicLevel("buckshot_magnesium"))
					if(chance >= 1 and chance <= odds) then
						projectileRemoveBouncing(projectile_entity_id)
						
						addNewInternalVariable(projectile_entity_id, "projectile_pos_x", "value_int", 0)
						addNewInternalVariable(projectile_entity_id, "projectile_pos_y", "value_int", 0)
						addNewInternalVariable(projectile_entity_id, "projectile_vel_x", "value_int", 0)
						addNewInternalVariable(projectile_entity_id, "projectile_vel_y", "value_int", 0)
						
						addBuffVisualIndicator_Projectile(projectile_entity_id, "spark")
						addNewInternalVariable(projectile_entity_id, "projectile_definition", "value_string", "mods/ArcaneReliquary/files/entity_definitions/custom_projectiles/invisible_shrapnel_projectile_explosive.xml")
							
						addNewLuaCallback(projectile_entity_id, "overtime", "mods/ArcaneReliquary/files/scripts/projectiles/store_projectile_vel_pos.lua", 1, false, "store_projectile_vel_pos")
						addNewLuaCallback_onRemoved(projectile_entity_id, "mods/ArcaneReliquary/files/scripts/projectiles/launch_directional_shrapnel_projectile_multiple.lua")
					end
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "buckshot,magnesium_ore",
	},
	{
		name 					= "buckshot_white_phosphorus",
		ui_name					= "White Phosphorus Buckshot",
		categories				= "projectiles",
		effect_desc				= "Provided they travel enough time, your spells have a 30% chance of shooting \ninvisible incendiary shrapnel projectiles that penetrate walls when they hit something.",
		effect_desc_short		= "Your spells shoot invisible incendiary shrapnel projectiles when they hit something.",
        precursor_types			= "gadget",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("buckshot_white_phosphorus") >= 1) then
				if(not isPhysicsProjectile(projectile_entity_id)) then
					SetRandomSeed( GameGetFrameNum(), math.random(1,50))
					local chance = Random(1,100)
					local odds = 30 + (5 * getRelicLevel("buckshot_white_phosphorus"))
					if(chance >= 1 and chance <= odds) then
						projectileRemoveBouncing(projectile_entity_id)
						
						addNewInternalVariable(projectile_entity_id, "projectile_pos_x", "value_int", 0)
						addNewInternalVariable(projectile_entity_id, "projectile_pos_y", "value_int", 0)
						addNewInternalVariable(projectile_entity_id, "projectile_vel_x", "value_int", 0)
						addNewInternalVariable(projectile_entity_id, "projectile_vel_y", "value_int", 0)
						
						addBuffVisualIndicator_Projectile(projectile_entity_id, "spark")
						addNewInternalVariable(projectile_entity_id, "projectile_definition", "value_string", "mods/ArcaneReliquary/files/entity_definitions/custom_projectiles/invisible_shrapnel_projectile_incendiary.xml")
							
						addNewLuaCallback(projectile_entity_id, "overtime", "mods/ArcaneReliquary/files/scripts/projectiles/store_projectile_vel_pos.lua", 1, false, "store_projectile_vel_pos")
						addNewLuaCallback_onRemoved(projectile_entity_id, "mods/ArcaneReliquary/files/scripts/projectiles/launch_directional_shrapnel_projectile_multiple.lua")
					end
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "buckshot,sacred_ember",
	},
	{
		name 					= "buckshot_cryo",
		ui_name					= "Cryo Buckshot",
		categories				= "projectiles",
		effect_desc				= "Provided they travel enough time, your spells have a 30% chance of shooting \ninvisible ice shrapnel projectiles that penetrate walls when they hit something.",
		effect_desc_short		= "Your spells shoot invisible ice shrapnel projectiles when they hit something.",
        precursor_types			= "gadget",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("buckshot_cryo") >= 1) then
				if(not isPhysicsProjectile(projectile_entity_id)) then
					SetRandomSeed( GameGetFrameNum(), math.random(1,50))
					local chance = Random(1,100)
					local odds = 30 + (5 * getRelicLevel("buckshot_cryo"))
					if(chance >= 1 and chance <= odds) then
						projectileRemoveBouncing(projectile_entity_id)
						
						addNewInternalVariable(projectile_entity_id, "projectile_pos_x", "value_int", 0)
						addNewInternalVariable(projectile_entity_id, "projectile_pos_y", "value_int", 0)
						addNewInternalVariable(projectile_entity_id, "projectile_vel_x", "value_int", 0)
						addNewInternalVariable(projectile_entity_id, "projectile_vel_y", "value_int", 0)
						
						addBuffVisualIndicator_Projectile(projectile_entity_id, "spark")
						addNewInternalVariable(projectile_entity_id, "projectile_definition", "value_string", "mods/ArcaneReliquary/files/entity_definitions/custom_projectiles/invisible_shrapnel_projectile_ice.xml")
							
						addNewLuaCallback(projectile_entity_id, "overtime", "mods/ArcaneReliquary/files/scripts/projectiles/store_projectile_vel_pos.lua", 1, false, "store_projectile_vel_pos")
						addNewLuaCallback_onRemoved(projectile_entity_id, "mods/ArcaneReliquary/files/scripts/projectiles/launch_directional_shrapnel_projectile_multiple.lua")
					end
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "buckshot,glacial_ring",
	},
	{
		name 					= "buckshot_overheated",
		ui_name					= "Overheated Buckshot",
		categories				= "projectiles",
		effect_desc				= "Provided they travel enough time, your spells have a 30% chance of shooting \ninvisible gunpowder shrapnel projectiles that penetrate walls when they hit something.",
		effect_desc_short		= "Your spells shoot invisible gunpowder shrapnel projectiles when they hit something.",
        precursor_types			= "gadget",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("buckshot_overheated") >= 1) then
				if(not isPhysicsProjectile(projectile_entity_id)) then
					SetRandomSeed( GameGetFrameNum(), math.random(1,50))
					local chance = Random(1,100)
					local odds = 30 + (5 * getRelicLevel("buckshot_overheated"))
					if(chance >= 1 and chance <= odds) then
						projectileRemoveBouncing(projectile_entity_id)
						
						addNewInternalVariable(projectile_entity_id, "projectile_pos_x", "value_int", 0)
						addNewInternalVariable(projectile_entity_id, "projectile_pos_y", "value_int", 0)
						addNewInternalVariable(projectile_entity_id, "projectile_vel_x", "value_int", 0)
						addNewInternalVariable(projectile_entity_id, "projectile_vel_y", "value_int", 0)
						
						addBuffVisualIndicator_Projectile(projectile_entity_id, "spark")
						addNewInternalVariable(projectile_entity_id, "projectile_definition", "value_string", "mods/ArcaneReliquary/files/entity_definitions/custom_projectiles/invisible_shrapnel_projectile_gunpowder.xml")
							
						addNewLuaCallback(projectile_entity_id, "overtime", "mods/ArcaneReliquary/files/scripts/projectiles/store_projectile_vel_pos.lua", 1, false, "store_projectile_vel_pos")
						addNewLuaCallback_onRemoved(projectile_entity_id, "mods/ArcaneReliquary/files/scripts/projectiles/launch_directional_shrapnel_projectile_multiple.lua")
					end
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "buckshot,fireworks",
	},
	{
		name 					= "bag_of_magical_seeds",
		ui_name					= "Bag of Magical Seeds",
		categories				= "player",
		effect_desc				= "When you collect gold nuggets, there's 20% chance special seeds will spawn near you. \nThese seeds create portals that shoot various spells at random directions.",
		effect_desc_short		= "Special seeds spawn near you when collecting gold.",
        precursor_types			= "magic_artifact",
		upgrade_effect_desc		= "For each upgrade level, seed spawn chance increases by 5%.",
		is_upgradable 			= true,
		ongoldpickup_function	= function()
			if(isRelicOwned("bag_of_magical_seeds") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 20 + (5 * getRelicLevel("bag_of_magical_seeds"))
				if(chance >= 1 and chance <= odds) then
				
					local player_x, player_y = EntityGetTransform(getPlayerEntity())
					seed_pool = {"seed_purple_slime", "seed_toxic", "seed_blood", "seed_water", "seed_acid", "seed_charged", "seed_explosive", "seed_fire", "seed_ice", "seed_laser", "seed_lava", "seed_poison", "seed_poly", "seed_slime", "seed_thunder", "seed_urine"}
					
					local spawn_pattern = Random(1,2)
					if(spawn_pattern == 1) then
						EntityLoad("mods/ArcaneReliquary/files/entity_definitions/items/" .. seed_pool[Random(1,#seed_pool)] .. ".xml", player_x, player_y - 10)
					elseif(spawn_pattern == 2) then
						EntityLoad("mods/ArcaneReliquary/files/entity_definitions/items/" .. seed_pool[Random(1,#seed_pool)] .. ".xml", player_x - 10, player_y - 10)
						EntityLoad("mods/ArcaneReliquary/files/entity_definitions/items/" .. seed_pool[Random(1,#seed_pool)] .. ".xml", player_x + 10, player_y - 10)
					end
				end
			end
		end
	},
	{
		name 					= "bag_of_magical_pearls",
		ui_name					= "Bag of Magical Pearls",
		categories				= "player",
		effect_desc				= "When you take damage from enemy attacks, there's 20% chance random death ray pearls will spawn near you.",
		effect_desc_short		= "Death ray pearls spawn near you upon taking damage from enemy attacks.",
        precursor_types			= "magic_artifact",
		upgrade_effect_desc		= "For each upgrade level, seed spawn chance increases by 5%.",
		is_upgradable 			= true,
		ondamaged_function		= function(damage, message, entity_thats_responsible, is_fatal, projectile_thats_responsible )
			if(isRelicOwned("bag_of_magical_pearls") >= 1) then	
				if(EntityHasTag(entity_thats_responsible, "enemy")) then
					SetRandomSeed( GameGetFrameNum(), math.random(1,50))
					local chance = Random(1,100)
					local odds = 20 + (5 * getRelicLevel("bag_of_magical_pearls"))
					if(chance >= 1 and chance <= odds) then
					
						local player_x, player_y = EntityGetTransform(getPlayerEntity())
						pearl_pool = {"death_ray_pearl_acid", "death_ray_pearl_fire", "death_ray_pearl_blood", "death_ray_pearl_gold", "death_ray_pearl_ice", "death_ray_pearl_poison", "death_ray_pearl_poly", "death_ray_pearl_urine", "death_ray_pearl_water"}
						
						local spawn_pattern = Random(1,2)
						if(spawn_pattern == 1) then
							EntityLoad("mods/ArcaneReliquary/files/entity_definitions/items/" .. pearl_pool[Random(1,#pearl_pool)] .. ".xml", player_x, player_y - 10)
						elseif(spawn_pattern == 2) then
							EntityLoad("mods/ArcaneReliquary/files/entity_definitions/items/" .. pearl_pool[Random(1,#pearl_pool)] .. ".xml", player_x - 10, player_y - 10)
							EntityLoad("mods/ArcaneReliquary/files/entity_definitions/items/" .. pearl_pool[Random(1,#pearl_pool)] .. ".xml", player_x + 10, player_y - 10)
						end
					end
				end
			end
		end
	},
	{
		name 					= "armored_helmet",
		ui_name					= "Armored Helmet",
		categories				= "player,kick",
		effect_desc				= "Succesfully landing a kick on an enemy grants you immunity to all damage types for a few seconds.",
		effect_desc_short		= "Succesfully landing a kick on an enemy grants you immunity to all damage types for a few seconds.",
        precursor_types			= "magic_artifact",
		upgrade_effect_desc		= "For each upgrade level, effect chance is increased by 5%.",
		is_upgradable 			= true,
		onenemykicked_function	= function(entity_who_kicked )
			if(isRelicOwned("armored_helmet") >= 1) then
				local game_effect_comp = GetGameEffectLoadTo( getPlayerEntity(), "PROTECTION_ALL", true )
				if game_effect_comp ~= nil then
					ComponentSetValue( game_effect_comp, "frames", "105" )
				end
			end
		end,
		onpickup_function		= function()
			addNewSimplifiedInternalVariable(getPlayerEntity(), "armored_helmet_effect")
		end,
		ondrop_function			= function()
			removeSimplifiedInternalVariable(getPlayerEntity(), "armored_helmet_effect")
		end
	},
	{
		name 					= "toy_brick",
		ui_name					= "Red Toy Brick",
		categories				= "projectiles,player",
		effect_desc				= "Walking on solid ground deals damage, but while in the air your spells deal extra damage.",
		effect_desc_short		= "Walking on solid ground deals damage, but while in the air your spells deal extra damage.",
        precursor_types			= "gadget",
		upgrade_effect_desc		= "For each upgrade level, damage bonus is increased",
		is_upgradable 			= true,
		overtime_function	= function()
			if(isRelicOwned("toy_brick") >= 1) then
				if(isPlayerStanding() and isPlayerMoving()) then
					local hp_lost = 1
					GamePrint("Relic penalty: -" .. hp_lost .. " hp for walking on solid ground")
					addPlayerHealth(hp_lost * -1)
				end
			end
		end,
		onspellcast_function	= function(projectile_entity_id)
			if(not isPlayerStanding()) then
				if(isRelicOwned("toy_brick") >= 1) then
					multiplyProjectileBaseDamage(projectile_entity_id, 2 + (1 * getRelicLevel("toy_brick")))
				end
			end
		end,
	},
	{
		name 					= "toy_brick_blue",
		ui_name					= "Blue Toy Brick",
		categories				= "projectiles,player",
		effect_desc				= "Walking on solid ground eventually causes small earthquakes on the soil directly below your feet.",
		effect_desc_short		= "Walking on solid ground eventually causes small earthquakes on the soil directly below your feet.",
        precursor_types			= "gadget",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		overtime_function	= function()
			if(isRelicOwned("toy_brick_blue") >= 1) then
				if(isPlayerStanding() and isPlayerMoving()) then
					local player_x, player_y = EntityGetTransform(getPlayerEntity())
					EntityLoad("mods/ArcaneReliquary/files/entity_definitions/load_on_explosion/earthquake_small.xml", player_x, player_y + 10)
				end
			end
		end,
	},
	{
		name 					= "toy_brick_yellow",
		ui_name					= "Yellow Toy Brick",
		categories				= "projectiles,player",
		effect_desc				= "Walking on solid ground eventually causes you to shoot invisible shrapnel projectiles from your feet.",
		effect_desc_short		= "Walking on solid ground eventually causes you to shoot invisible shrapnel projectiles from your feet.",
        precursor_types			= "gadget",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		overtime_function	= function()
			if(isRelicOwned("toy_brick_yellow") >= 1) then
				if(isPlayerStanding() and isPlayerMoving()) then
					LoadGameEffectEntityTo( getPlayerEntity(), getCustomEffect("effect_shoot_projectile_downwards_invisible_shrapnel").path )
				end
			end
		end,
	},
	{
		name 					= "toy_brick_green",
		ui_name					= "Green Toy Brick",
		categories				= "projectiles,player",
		effect_desc				= "Walking on solid ground eventually causes small pockets of acid to appear bellow your feet.",
		effect_desc_short		= "Walking on solid ground eventually causes small pockets of acid to appear bellow your feet.",
        precursor_types			= "gadget",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		overtime_function	= function()
			if(isRelicOwned("toy_brick_green") >= 1) then
				if(isPlayerStanding() and isPlayerMoving()) then
					local player_x, player_y = EntityGetTransform(getPlayerEntity())
					EntityLoad("mods/ArcaneReliquary/files/entity_definitions/others/acid_explosion.xml", player_x, player_y + 10)
				end
			end
		end,
	},
	{
		name 					= "toy_brick_pink",
		ui_name					= "Pink Toy Brick",
		categories				= "projectiles,player",
		effect_desc				= "Casting spells while standing on solid ground causes them to be destroyed, but while \nmidair, extra copies of them are cast together based on the total number of slots of your current wand.",
		effect_desc_short		= "Your spells are destroyed if you cast them while standing on solid ground, but extra copies are cast when midair.",
        precursor_types			= "gadget",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("toy_brick_pink") >= 1) then
				if(isPlayerStanding() and (not isPlayerMoving())) then
					projectileToggleExplosionOnDeath(projectile_entity_id, false)
					EntityKill(projectile_entity_id)
				else
					local free_slots, spells = getWandEmptyandOccupiedSlots(getCurrentlyEquippedWandId())
					for i=1,(spells + free_slots) do
						local x, y = EntityGetTransform(projectile_entity_id)
						shootProjectileBasedonAnotherProjectile(projectile_entity_id, getProjectileDefinitionPath(projectile_entity_id), x, y, 150)
					end
				end
			end
		end,
	},
	{
		name 					= "toy_brick_orange",
		ui_name					= "Orange Toy Brick",
		categories				= "projectiles,player",
		effect_desc				= "Dealing damage to enemies while standing on solid ground, provided damage dealt is \nnot fatal, causes you to lose 1 hp, but doing so while midair heals you by 1.",
		effect_desc_short		= "Damaging enemies while standing on solid ground damages you, but doing so while midair heals you.",
        precursor_types			= "gadget",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onenemydamaged_function	= function(damage, message, entity_thats_responsible, is_fatal, projectile_thats_responsible )
			if(isRelicOwned("toy_brick_orange") >= 1) then
				local hp = 1
				if(isPlayerStanding() and (not isPlayerMoving())) then
					GamePrint("Relic penalty: -" .. hp .. " hp lost for dealing damage to enemies while on solid ground")
					addPlayerHealth(hp * -1)
				else
					addPlayerHealth(hp)
				end
			end
		end,
	},
	{
		name 					= "joystick",
		ui_name					= "Joystick with Red Tipped Antenna",
		categories				= "companions",
		effect_desc				= "You control a flying saucer that shoots red laser beams on enemies directly below it.",
		effect_desc_short		= "You control a flying saucer that shoots red laser beams on enemies directly below it.",
        precursor_types			= "companion",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			attachCompaniontoEntity(getPlayerEntity(), "flying_saucer", "mortal")
		end,
		ondrop_function			= function()
			detachCompanionfromEntity(getPlayerEntity(), "flying_saucer")
		end,
		requires_relic			= "alkaline_battery",
	},
	{
		name 					= "parrot_egg",
		ui_name					= "Green Dotted Parrot Egg",
		categories				= "companions",
		effect_desc				= "Commando Parrot, a bird carrying a shotgun, capable of firing at enemies directly in front or behind it, joins you.",
		effect_desc_short		= "Commando Parrot, a bird carrying a shotgun, capable of firing at enemies directly in front or behind it, joins you.",
        precursor_types			= "companion",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			attachCompaniontoEntity(getPlayerEntity(), "commando_parrot", "mortal")
		end,
		ondrop_function			= function()
			detachCompanionfromEntity(getPlayerEntity(), "commando_parrot")
		end,
		requires_relic			= "bag_of_magical_seeds",
	},
	{
		name 					= "parrot_egg_red",
		ui_name					= "Red Dotted Parrot Egg",
		categories				= "companions",
		effect_desc				= "Red Commando Parrot, a bird carrying an SMG, capable of firing at enemies directly in front or behind it, joins you.",
		effect_desc_short		= "Red Commando Parrot, a bird carrying an SMG, capable of firing at enemies directly in front or behind it, joins you.",
        precursor_types			= "companion",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			attachCompaniontoEntity(getPlayerEntity(), "commando_parrot_smg", "mortal")
		end,
		ondrop_function			= function()
			detachCompanionfromEntity(getPlayerEntity(), "commando_parrot_smg")
		end,
		requires_relic			= "bag_of_magical_seeds",
	},
	{
		name 					= "joystick_yellow",
		ui_name					= "Joystick with Yellow Tipped Antenna",
		categories				= "companions",
		effect_desc				= "You control a flying saucer that shoots explosive laser beams on enemies directly below it.",
		effect_desc_short		= "You control a flying saucer that shoots explosive laser beams on enemies directly below it.",
        precursor_types			= "companion",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			attachCompaniontoEntity(getPlayerEntity(), "flying_saucer_yellow", "mortal")
		end,
		ondrop_function			= function()
			detachCompanionfromEntity(getPlayerEntity(), "flying_saucer_yellow")
		end,
		requires_relic			= "alkaline_battery",
	},
	{
		name 					= "joystick_orange",
		ui_name					= "Joystick with Orange Tipped Antenna",
		categories				= "companions",
		effect_desc				= "You control a flying saucer that shoots incendiary laser beams on enemies directly below it.",
		effect_desc_short		= "You control a flying saucer that shoots incendiary laser beams on enemies directly below it.",
        precursor_types			= "companion",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			attachCompaniontoEntity(getPlayerEntity(), "flying_saucer_orange", "mortal")
		end,
		ondrop_function			= function()
			detachCompanionfromEntity(getPlayerEntity(), "flying_saucer_orange")
		end,
		requires_relic			= "alkaline_battery",
	},
	{
		name 					= "joystick_green",
		ui_name					= "Joystick with Green Tipped Antenna",
		categories				= "companions",
		effect_desc				= "You control a flying saucer that shoots corroding laser beams on enemies directly below it.",
		effect_desc_short		= "You control a flying saucer that shoots corroding laser beams on enemies directly below it.",
        precursor_types			= "companion",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			attachCompaniontoEntity(getPlayerEntity(), "flying_saucer_green", "mortal")
		end,
		ondrop_function			= function()
			detachCompanionfromEntity(getPlayerEntity(), "flying_saucer_green")
		end,
		requires_relic			= "alkaline_battery",
	},
	{
		name 					= "joystick_cyan",
		ui_name					= "Joystick with Cyan Tipped Antenna",
		categories				= "companions",
		effect_desc				= "You control a flying saucer that shoots freezing laser beams on enemies directly below it.",
		effect_desc_short		= "You control a flying saucer that shoots freezing laser beams on enemies directly below it.",
        precursor_types			= "companion",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			attachCompaniontoEntity(getPlayerEntity(), "flying_saucer_cyan", "mortal")
		end,
		ondrop_function			= function()
			detachCompanionfromEntity(getPlayerEntity(), "flying_saucer_cyan")
		end,
		requires_relic			= "alkaline_battery",
	},
	{
		name 					= "joystick_blue",
		ui_name					= "Joystick with Blue Tipped Antenna",
		categories				= "companions",
		effect_desc				= "You control a flying saucer that shoots electrocuting laser beams on enemies directly below it.",
		effect_desc_short		= "You control a flying saucer that shoots electrocuting laser beams on enemies directly below it.",
        precursor_types			= "companion",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			attachCompaniontoEntity(getPlayerEntity(), "flying_saucer_blue", "mortal")
		end,
		ondrop_function			= function()
			detachCompanionfromEntity(getPlayerEntity(), "flying_saucer_blue")
		end,
		requires_relic			= "alkaline_battery",
	},
	{
		name 					= "joystick_pink",
		ui_name					= "Joystick with Pink Tipped Antenna",
		categories				= "companions",
		effect_desc				= "You control a flying saucer that shoots mutating laser beams on enemies directly below it.",
		effect_desc_short		= "You control a flying saucer that shoots mutating laser beams on enemies directly below it.",
        precursor_types			= "companion",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			attachCompaniontoEntity(getPlayerEntity(), "flying_saucer_pink", "mortal")
		end,
		ondrop_function			= function()
			detachCompanionfromEntity(getPlayerEntity(), "flying_saucer_pink")
		end,
		requires_relic			= "alkaline_battery",
	},
	{
		name 					= "joystick_white",
		ui_name					= "Joystick with White Tipped Antenna",
		categories				= "companions",
		effect_desc				= "You control a flying saucer that shoots slash laser beams on enemies directly below it.",
		effect_desc_short		= "You control a flying saucer that shoots slash laser beams on enemies directly below it.",
        precursor_types			= "companion",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			attachCompaniontoEntity(getPlayerEntity(), "flying_saucer_white", "mortal")
		end,
		ondrop_function			= function()
			detachCompanionfromEntity(getPlayerEntity(), "flying_saucer_white")
		end,
		requires_relic			= "alkaline_battery",
	},
	{
		name 					= "parrot_egg_blue",
		ui_name					= "Blue Dotted Parrot Egg",
		categories				= "companions",
		effect_desc				= "Blue Commando Parrot, a bird carrying a rocket launcher, capable of firing at enemies directly in front or behind it, joins you.",
		effect_desc_short		= "Blue Commando Parrot, a bird carrying a rocket launcher, capable of firing at enemies directly in front or behind it, joins you.",
        precursor_types			= "companion",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			attachCompaniontoEntity(getPlayerEntity(), "commando_parrot_rpg", "mortal")
		end,
		ondrop_function			= function()
			detachCompanionfromEntity(getPlayerEntity(), "commando_parrot_rpg")
		end,
		requires_relic			= "bag_of_magical_seeds",
	},
	{
		name 					= "parrot_egg_purple",
		ui_name					= "Purple Dotted Parrot Egg",
		categories				= "companions",
		effect_desc				= "Purple Commando Parrot, a bird carrying a grenade launcher, capable of firing at enemies directly in front or behind it, joins you.",
		effect_desc_short		= "Purple Commando Parrot, a bird carrying a grenade launcher, capable of firing at enemies directly in front or behind it, joins you.",
        precursor_types			= "companion",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			attachCompaniontoEntity(getPlayerEntity(), "commando_parrot_grenades", "mortal")
		end,
		ondrop_function			= function()
			detachCompanionfromEntity(getPlayerEntity(), "commando_parrot_grenades")
		end,
		requires_relic			= "bag_of_magical_seeds",
	},
	{
		name 					= "parrot_egg_cyan",
		ui_name					= "Cyan Dotted Parrot Egg",
		categories				= "companions",
		effect_desc				= "Cyan Commando Parrot, a bird carrying a cannon, capable of firing cannonballs at enemies directly in front or behind it, joins you.",
		effect_desc_short		= "Cyan Commando Parrot, a bird carrying a cannon, capable of firing cannonballs at enemies directly in front or behind it, joins you.",
        precursor_types			= "companion",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			attachCompaniontoEntity(getPlayerEntity(), "commando_parrot_cannon", "mortal")
		end,
		ondrop_function			= function()
			detachCompanionfromEntity(getPlayerEntity(), "commando_parrot_cannon")
		end,
		requires_relic			= "bag_of_magical_seeds",
	},
	{
		name 					= "parrot_egg_pink",
		ui_name					= "Pink Dotted Parrot Egg",
		categories				= "companions",
		effect_desc				= "Pink Commando Parrot, a bird carrying a wand, capable of firing spells from your current wand at enemies directly in front or behind it, joins you.",
		effect_desc_short		= "Pink Commando Parrot, a bird carrying a wand, capable of firing spells from your current wand at enemies directly in front or behind it, joins you.",
        precursor_types			= "companion",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			attachCompaniontoEntity(getPlayerEntity(), "commando_parrot_wand", "mortal")
		end,
		ondrop_function			= function()
			detachCompanionfromEntity(getPlayerEntity(), "commando_parrot_wand")
		end,
		requires_relic			= "bag_of_magical_seeds",
	},
	{
		name 					= "mystic_pentagram",
		ui_name					= "Mystic Pentagram",
		categories				= "projectiles",
		effect_desc				= "Each time you cast a spell, there's a 50% chance a dense barrage of spells of the same type will be cast from you.",
		effect_desc_short		= "Each time you cast a spell, there's a 50% chance a dense barrage of spells of the same type will be cast from you.",
        precursor_types			= "magic_artifact",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("mystic_pentagram") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 50 + (5 * getRelicLevel("mystic_pentagram"))
				if(chance >= 1 and chance <= odds) then
					
					local projectile_definition = getProjectileDefinitionPath(projectile_entity_id)
					
					if(projectile_definition ~= nil) then
						if(not entityHasInternalVariableValue(getPlayerEntity(), "last_spell_cast_definition")) then
							addNewInternalVariable(getPlayerEntity(), "last_spell_cast_definition", "value_string", getProjectileDefinitionPath(projectile_entity_id))
						else
							setInternalVariableValue(getPlayerEntity(), "last_spell_cast_definition", "value_string", getProjectileDefinitionPath(projectile_entity_id))
						end
					else
						removeInternalVariable(getPlayerEntity(), "last_spell_cast_definition")
					end
					
					local player_x, player_y = EntityGetTransform(getPlayerEntity())
					EntityLoad("mods/ArcaneReliquary/files/entity_definitions/load_on_explosion/launch_circular_projectiles_last_spell_cast.xml", player_x, player_y)
				end
			end
		end,
	},
	{
		name 					= "tailed_star",
		ui_name					= "Tailed Star",
		categories				= "companions",
		effect_desc				= "Pegasus, a magical creature that possess two different attacks that inflict the wind effect on enemies, joins you.",
		effect_desc_short		= "Pegasus, a magical creature that possess two different attacks that inflict the wind effect on enemies, joins you.",
        precursor_types			= "companion",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			attachCompaniontoEntity(getPlayerEntity(), "pegasus", "mortal")
		end,
		ondrop_function			= function()
			detachCompanionfromEntity(getPlayerEntity(), "pegasus")
		end,
		excludes_relics_desc	= "It can't be used along with other Pegasus summoning relics",
		excludes_relics_tags	= "pegasus_summon",
		requires_relic			= "little_cloud",
	},
	{
		name 					= "tailed_star_fire",
		ui_name					= "Tailed Fire Star",
		categories				= "companions",
		effect_desc				= "Fire Pegasus, a magical creature that possess two different incendiary attacks, joins you. \nYou can also mount it by pressing the interaction key while close to it, press it again \nto dismount.",
		effect_desc_short		= "Fire Pegasus, a magical creature that possess two different incendiary attacks, joins you.",
        precursor_types			= "companion",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			attachCompaniontoEntity(getPlayerEntity(), "pegasus_fire", "mortal")
		end,
		ondrop_function			= function()
			detachCompanionfromEntity(getPlayerEntity(), "pegasus_fire")
		end,
		excludes_relics_desc	= "It can't be used along with other Pegasus summoning relics",
		excludes_relics_tags	= "pegasus_summon",
		requires_relic			= "little_cloud",
	},
	{
		name 					= "tailed_star_ice",
		ui_name					= "Tailed Ice Star",
		categories				= "companions",
		effect_desc				= "Ice Pegasus, a magical creature that possess two different freezing attacks, joins you. \nYou can also mount it by pressing the interaction key while close to it, press it again \nto dismount.",
		effect_desc_short		= "Ice Pegasus, a magical creature that possess two different freezing attacks, joins you.",
        precursor_types			= "companion",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			attachCompaniontoEntity(getPlayerEntity(), "pegasus_ice", "mortal")
		end,
		ondrop_function			= function()
			detachCompanionfromEntity(getPlayerEntity(), "pegasus_ice")
		end,
		excludes_relics_desc	= "It can't be used along with other Pegasus summoning relics",
		excludes_relics_tags	= "pegasus_summon",
		requires_relic			= "little_cloud",
	},
	{
		name 					= "tailed_star_electro",
		ui_name					= "Tailed Electric Star",
		categories				= "companions",
		effect_desc				= "Electric Pegasus, a magical creature that possess two different electrocuting attacks, joins you. \nYou can also mount it by pressing the interaction key while close to it, press it again \nto dismount.",
		effect_desc_short		= "Electric Pegasus, a magical creature that possess two different electrocuting attacks, joins you.",
        precursor_types			= "companion",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			attachCompaniontoEntity(getPlayerEntity(), "pegasus_electro", "mortal")
		end,
		ondrop_function			= function()
			detachCompanionfromEntity(getPlayerEntity(), "pegasus_electro")
		end,
		excludes_relics_desc	= "It can't be used along with other Pegasus summoning relics",
		excludes_relics_tags	= "pegasus_summon",
		requires_relic			= "little_cloud",
	},
	{
		name 					= "tailed_star_mutate",
		ui_name					= "Tailed Mutating Star",
		categories				= "companions",
		effect_desc				= "Mutating Pegasus, a magical creature that possess two different mutating attacks, joins you. \nYou can also mount it by pressing the interaction key while close to it, press it again \nto dismount.",
		effect_desc_short		= "Mutating Pegasus, a magical creature that possess two different mutating attacks, joins you.",
        precursor_types			= "companion",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			attachCompaniontoEntity(getPlayerEntity(), "pegasus_mutate", "mortal")
		end,
		ondrop_function			= function()
			detachCompanionfromEntity(getPlayerEntity(), "pegasus_mutate")
		end,
		excludes_relics_desc	= "It can't be used along with other Pegasus summoning relics",
		excludes_relics_tags	= "pegasus_summon",
		requires_relic			= "little_cloud",
	},
	{
		name 					= "tailed_star_slap",
		ui_name					= "Tailed Slap Star",
		categories				= "companions",
		effect_desc				= "Slap Pegasus, a magical creature that possess two different attacks that inflict the slap effect on enemies, joins you. \nYou can also mount it by pressing the interaction key while close to it, press it again \nto dismount.",
		effect_desc_short		= "Slap Pegasus, a magical creature that possess two different attacks that inflict the slap effect on enemies, joins you.",
        precursor_types			= "companion",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			attachCompaniontoEntity(getPlayerEntity(), "pegasus_slap", "mortal")
		end,
		ondrop_function			= function()
			detachCompanionfromEntity(getPlayerEntity(), "pegasus_slap")
		end,
		excludes_relics_desc	= "It can't be used along with other Pegasus summoning relics",
		excludes_relics_tags	= "pegasus_summon",
		requires_relic			= "little_cloud",
	},
	{
		name 					= "tailed_star_slash",
		ui_name					= "Tailed Slash Star",
		categories				= "companions",
		effect_desc				= "Slash Pegasus, a magical creature that possess two different attacks that inflict the slash effect on enemies, joins you. \nYou can also mount it by pressing the interaction key while close to it, press it again \nto dismount.",
		effect_desc_short		= "Slash Pegasus, a magical creature that possess two different attacks that inflict the slash effect on enemies, joins you.",
        precursor_types			= "companion",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			attachCompaniontoEntity(getPlayerEntity(), "pegasus_slash", "mortal")
		end,
		ondrop_function			= function()
			detachCompanionfromEntity(getPlayerEntity(), "pegasus_slash")
		end,
		excludes_relics_desc	= "It can't be used along with other Pegasus summoning relics",
		excludes_relics_tags	= "pegasus_summon",
		requires_relic			= "little_cloud",
	},
	{
		name 					= "tailed_star_explode",
		ui_name					= "Tailed Exploding Star",
		categories				= "companions",
		effect_desc				= "Exploding Pegasus, a magical creature that possess two different attacks that inflict the exploding effect on enemies, joins you. \nYou can also mount it by pressing the interaction key while close to it, press it again \nto dismount.",
		effect_desc_short		= "Exploding Pegasus, a magical creature that possess two different attacks that inflict the exploding effect on enemies, joins you.",
        precursor_types			= "companion",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			attachCompaniontoEntity(getPlayerEntity(), "pegasus_explode", "mortal")
		end,
		ondrop_function			= function()
			detachCompanionfromEntity(getPlayerEntity(), "pegasus_explode")
		end,
		excludes_relics_desc	= "It can't be used along with other Pegasus summoning relics",
		excludes_relics_tags	= "pegasus_summon",
		requires_relic			= "little_cloud",
	},
	{
		name 					= "tailed_star_bleed",
		ui_name					= "Tailed Bleeding Star",
		categories				= "companions",
		effect_desc				= "Bleeding Pegasus, a magical creature that possess two different attacks that inflict the bleeding effect on enemies, joins you. \nYou can also mount it by pressing the interaction key while close to it, press it again \nto dismount.",
		effect_desc_short		= "Bleeding Pegasus, a magical creature that possess two different attacks that inflict the bleeding effect on enemies, joins you.",
        precursor_types			= "companion",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			attachCompaniontoEntity(getPlayerEntity(), "pegasus_bleed", "mortal")
		end,
		ondrop_function			= function()
			detachCompanionfromEntity(getPlayerEntity(), "pegasus_bleed")
		end,
		excludes_relics_desc	= "It can't be used along with other Pegasus summoning relics",
		excludes_relics_tags	= "pegasus_summon",
		requires_relic			= "little_cloud",
	},
	{
		name 					= "tailed_star_magnet",
		ui_name					= "Tailed Magnetic Star",
		categories				= "companions",
		effect_desc				= "Magnetic Pegasus, a magical creature that possess two different attacks that inflict the attraction effect on enemies, joins you. \nYou can also mount it by pressing the interaction key while close to it, press it again \nto dismount.",
		effect_desc_short		= "Magnetic Pegasus, a magical creature that possess two different attacks that inflict the attraction effect on enemies, joins you.",
        precursor_types			= "companion",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			attachCompaniontoEntity(getPlayerEntity(), "pegasus_magnet", "mortal")
		end,
		ondrop_function			= function()
			detachCompanionfromEntity(getPlayerEntity(), "pegasus_magnet")
		end,
		excludes_relics_desc	= "It can't be used along with other Pegasus summoning relics",
		excludes_relics_tags	= "pegasus_summon",
		requires_relic			= "little_cloud",
	},
	{
		name 					= "gravity_core",
		ui_name					= "Gravity Core",
		categories				= "projectiles",
		effect_desc				= "For every relic you possess related to \ngravity stones, your spells gain a small damage boost.",
		effect_desc_short		= "Owning gravity stone-related relics gives you spell damage bonus.",
        precursor_types			= "magic_artifact",
		upgrade_effect_desc		= "For each upgrade level, damage boost increases.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("gravity_core") >= 1) then
				local bonus_damage = 0
				local damage_threshold = 0.5 + (0.5 * getRelicLevel("gravity_core"))
				local owned_relics = getOwnedRelics()
				for i,owned_relic in ipairs(owned_relics) do
					if(string.find(getRelic(EntityGetName(owned_relic)).precursor_types, "gravity_stone")) then
						--GamePrint(">>> " .. getRelic(EntityGetName(owned_relic)).name .. " pertains to gravity_stone precursor type")
						bonus_damage = bonus_damage + damage_threshold
					end
				end
				--GamePrint(">>> " .. bonus_damage)
				addProjectileBaseDamage(projectile_entity_id, bonus_damage)
			end
		end,
	},
	{
		name 					= "elemental_crystal_thunder",
		ui_name					= "Thunder Elemental Crystal",
		categories				= "companions",
		effect_desc				= "You are joined by Thunder Gravity Elemental, a creature that shoots thunder orbs at enemies.",
		effect_desc_short		= "You are joined by Thunder Gravity Elemental, a creature that shoots thunder orbs at enemies.",
        precursor_types			= "companion",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			attachCompaniontoEntity(getPlayerEntity(), "gravity_elemental_thunder", "mortal")
		end,
		ondrop_function			= function()
			detachCompanionfromEntity(getPlayerEntity(), "gravity_elemental_thunder")
		end,
		requires_relic			= "gravity_core",
	},
	{
		name 					= "elemental_crystal_explosive",
		ui_name					= "Explosive Elemental Crystal",
		categories				= "companions",
		effect_desc				= "You are joined by Explosive Gravity Elemental, a creature that shoots explosive orbs at enemies.",
		effect_desc_short		= "You are joined by Explosive Gravity Elemental, a creature that shoots explosive orbs at enemies.",
        precursor_types			= "companion",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			attachCompaniontoEntity(getPlayerEntity(), "gravity_elemental_explosive", "mortal")
		end,
		ondrop_function			= function()
			detachCompanionfromEntity(getPlayerEntity(), "gravity_elemental_explosive")
		end,
		requires_relic			= "gravity_core",
	},
	{
		name 					= "elemental_crystal_lava",
		ui_name					= "Lava Elemental Crystal",
		categories				= "companions",
		effect_desc				= "You are joined by Lava Gravity Elemental, a creature that shoots lava orbs at enemies.",
		effect_desc_short		= "You are joined by Lava Gravity Elemental, a creature that shoots lava orbs at enemies.",
        precursor_types			= "companion",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			attachCompaniontoEntity(getPlayerEntity(), "gravity_elemental_lava", "mortal")
		end,
		ondrop_function			= function()
			detachCompanionfromEntity(getPlayerEntity(), "gravity_elemental_lava")
		end,
		requires_relic			= "gravity_core",
	},
	{
		name 					= "elemental_crystal_frost",
		ui_name					= "Frost Elemental Crystal",
		categories				= "companions",
		effect_desc				= "You are joined by Frost Gravity Elemental, a creature that shoots freezing liquid orbs at enemies.",
		effect_desc_short		= "You are joined by Frost Gravity Elemental, a creature that shoots freezing liquid orbs at enemies.",
        precursor_types			= "companion",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			attachCompaniontoEntity(getPlayerEntity(), "gravity_elemental_frost", "mortal")
		end,
		ondrop_function			= function()
			detachCompanionfromEntity(getPlayerEntity(), "gravity_elemental_frost")
		end,
		requires_relic			= "gravity_core",
	},
	{
		name 					= "elemental_crystal_acid",
		ui_name					= "Acid Elemental Crystal",
		categories				= "companions",
		effect_desc				= "You are joined by Acid Gravity Elemental, a creature that shoots acid orbs at enemies.",
		effect_desc_short		= "You are joined by Acid Gravity Elemental, a creature that shoots acid orbs at enemies.",
        precursor_types			= "companion",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			attachCompaniontoEntity(getPlayerEntity(), "gravity_elemental_acid", "mortal")
		end,
		ondrop_function			= function()
			detachCompanionfromEntity(getPlayerEntity(), "gravity_elemental_acid")
		end,
		requires_relic			= "gravity_core",
	},
	{
		name 					= "elemental_crystal_poison",
		ui_name					= "Poison Elemental Crystal",
		categories				= "companions",
		effect_desc				= "You are joined by Poison Gravity Elemental, a creature that shoots poison orbs at enemies.",
		effect_desc_short		= "You are joined by Poison Gravity Elemental, a creature that shoots poison orbs at enemies.",
        precursor_types			= "companion",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			attachCompaniontoEntity(getPlayerEntity(), "gravity_elemental_poison", "mortal")
		end,
		ondrop_function			= function()
			detachCompanionfromEntity(getPlayerEntity(), "gravity_elemental_poison")
		end,
		requires_relic			= "gravity_core",
	},
	{
		name 					= "elemental_crystal_poly",
		ui_name					= "Poly Elemental Crystal",
		categories				= "companions",
		effect_desc				= "You are joined by Poly Gravity Elemental, a creature that shoots polymorphine orbs at enemies.",
		effect_desc_short		= "You are joined by Poly Gravity Elemental, a creature that shoots polymorphine orbs at enemies.",
        precursor_types			= "companion",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			attachCompaniontoEntity(getPlayerEntity(), "gravity_elemental_poly", "mortal")
		end,
		ondrop_function			= function()
			detachCompanionfromEntity(getPlayerEntity(), "gravity_elemental_poly")
		end,
		requires_relic			= "gravity_core",
	},
	{
		name 					= "elemental_crystal_water",
		ui_name					= "Water Elemental Crystal",
		categories				= "companions",
		effect_desc				= "You are joined by Water Gravity Elemental, a creature that shoots water orbs at enemies.",
		effect_desc_short		= "You are joined by Water Gravity Elemental, a creature that shoots water orbs at enemies.",
        precursor_types			= "companion",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			attachCompaniontoEntity(getPlayerEntity(), "gravity_elemental_water", "mortal")
		end,
		ondrop_function			= function()
			detachCompanionfromEntity(getPlayerEntity(), "gravity_elemental_water")
		end,
		requires_relic			= "gravity_core",
	},
	{
		name 					= "elemental_crystal_blood",
		ui_name					= "Blood Elemental Crystal",
		categories				= "companions",
		effect_desc				= "You are joined by Blood Gravity Elemental, a creature that shoots corrosive blood orbs at enemies.",
		effect_desc_short		= "You are joined by Blood Gravity Elemental, a creature that shoots corrosive blood orbs at enemies.",
        precursor_types			= "companion",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			attachCompaniontoEntity(getPlayerEntity(), "gravity_elemental_blood", "mortal")
		end,
		ondrop_function			= function()
			detachCompanionfromEntity(getPlayerEntity(), "gravity_elemental_blood")
		end,
		requires_relic			= "gravity_core",
	},
	{
		name 					= "elemental_crystal_urine",
		ui_name					= "Urine Elemental Crystal",
		categories				= "companions",
		effect_desc				= "You are joined by Urine Gravity Elemental, a creature that shoots urine orbs at enemies.",
		effect_desc_short		= "You are joined by Urine Gravity Elemental, a creature that shoots urine orbs at enemies.",
        precursor_types			= "companion",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			attachCompaniontoEntity(getPlayerEntity(), "gravity_elemental_urine", "mortal")
		end,
		ondrop_function			= function()
			detachCompanionfromEntity(getPlayerEntity(), "gravity_elemental_urine")
		end,
		requires_relic			= "gravity_core",
	},
	{
		name 					= "turtle_shell",
		ui_name					= "Turtle Shell",
		categories				= "companions",
		effect_desc				= "You are joined by the Clown Ship. It drops bomb rats on enemies it detects below it. \nIf you take damage from enemy attacks, it enters angry mode and drops nuclear rats instead. \nYou can also mount it by pressing the interaction key while close to it, press it again \nto dismount. While flying on it, guide its movement by pointing to empty areas, otherwise it will stop.",
		effect_desc_short		= "You are joined by the Clown Ship.",
        precursor_types			= "companion",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			attachCompaniontoEntity(getPlayerEntity(), "clownship", "mortal")
		end,
		ondrop_function			= function()
			detachCompanionfromEntity(getPlayerEntity(), "clownship")
		end,
		ondamaged_function		= function(damage, message, entity_thats_responsible, is_fatal, projectile_thats_responsible )
			if(isRelicOwned("turtle_shell") >= 1) then	
				if(EntityHasTag(entity_thats_responsible, "enemy")) then
					if(not entityHasSimplifiedInternalVariable(GetUpdatedEntityID(), "turtle_shell_damage_flag")) then
						addNewSimplifiedInternalVariable( GetUpdatedEntityID(), "turtle_shell_damage_flag" )
					end
				end
			end
		end,
	},
	{
		name 					= "turtle_shell_blue",
		ui_name					= "Blue Turtle Shell",
		categories				= "companions",
		effect_desc				= "You are joined by the Blue Clown Ship. It drops spikeballs on enemies it detects below it. \nIf you take damage from enemy attacks, it enters angry mode and drops overheated spikeballs instead. \nYou can also mount it by pressing the interaction key while close to it, press it again \nto dismount. While flying on it, guide its movement by pointing to empty areas, otherwise it will stop.",
		effect_desc_short		= "You are joined by the Blue Clown Ship.",
        precursor_types			= "companion",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			attachCompaniontoEntity(getPlayerEntity(), "clownship_blue", "mortal")
		end,
		ondrop_function			= function()
			detachCompanionfromEntity(getPlayerEntity(), "clownship_blue")
		end,
		ondamaged_function		= function(damage, message, entity_thats_responsible, is_fatal, projectile_thats_responsible )
			if(isRelicOwned("turtle_shell_blue") >= 1) then	
				if(EntityHasTag(entity_thats_responsible, "enemy")) then
					if(not entityHasSimplifiedInternalVariable(GetUpdatedEntityID(), "turtle_shell_blue_damage_flag")) then
						addNewSimplifiedInternalVariable( GetUpdatedEntityID(), "turtle_shell_blue_damage_flag" )
					end
				end
			end
		end,
	},
	{
		name 					= "gory_skull",
		ui_name					= "Gory Skull",
		categories				= "companions",
		effect_desc				= "A friendly skull that shoots blood fireballs joins you.",
		effect_desc_short		= "A blood fireball shooting companion.",
        precursor_types			= "companion",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			attachCompaniontoEntity(getPlayerEntity(), "gory_skull", "mortal")
		end,
		ondrop_function			= function()
			detachCompanionfromEntity(getPlayerEntity(), "gory_skull")
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "flaming_skull,blood_cell",
		requires_relic			= "bone",
	},
	{
		name 					= "pyrotechnic_skull",
		ui_name					= "Pyrotechnic Skull",
		categories				= "companions",
		effect_desc				= "A friendly skull that shoots gunpowder fireballs joins you.",
		effect_desc_short		= "A gunpowder fireball shooting companion.",
        precursor_types			= "companion",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			attachCompaniontoEntity(getPlayerEntity(), "pyrotechnic_skull", "mortal")
		end,
		ondrop_function			= function()
			detachCompanionfromEntity(getPlayerEntity(), "pyrotechnic_skull")
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "flaming_skull,firecrackers",
		requires_relic			= "bone",
	},
	{
		name 					= "joystick_wind",
		ui_name					= "Joystick with Gray Tipped Antenna",
		categories				= "companions",
		effect_desc				= "You control a flying saucer that shoots wind laser beams on enemies directly below it.",
		effect_desc_short		= "You control a flying saucer that shoots wind laser beams on enemies directly below it.",
        precursor_types			= "companion",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			attachCompaniontoEntity(getPlayerEntity(), "flying_saucer_wind", "mortal")
		end,
		ondrop_function			= function()
			detachCompanionfromEntity(getPlayerEntity(), "flying_saucer_wind")
		end,
		requires_relic			= "alkaline_battery",
	},
	{
		name 					= "joystick_slap",
		ui_name					= "Joystick with Dark Green Tipped Antenna",
		categories				= "companions",
		effect_desc				= "You control a flying saucer that shoots slap laser beams on enemies directly below it.",
		effect_desc_short		= "You control a flying saucer that shoots slap laser beams on enemies directly below it.",
        precursor_types			= "companion",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			attachCompaniontoEntity(getPlayerEntity(), "flying_saucer_slap", "mortal")
		end,
		ondrop_function			= function()
			detachCompanionfromEntity(getPlayerEntity(), "flying_saucer_slap")
		end,
		requires_relic			= "alkaline_battery",
	},
	{
		name 					= "joystick_slash",
		ui_name					= "Joystick with Purple Tipped Antenna",
		categories				= "companions",
		effect_desc				= "You control a flying saucer that shoots slash laser beams on enemies directly below it.",
		effect_desc_short		= "You control a flying saucer that shoots slash laser beams on enemies directly below it.",
        precursor_types			= "companion",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			attachCompaniontoEntity(getPlayerEntity(), "flying_saucer_slash", "mortal")
		end,
		ondrop_function			= function()
			detachCompanionfromEntity(getPlayerEntity(), "flying_saucer_slash")
		end,
		requires_relic			= "alkaline_battery",
	},
	{
		name 					= "joystick_explode",
		ui_name					= "Joystick with Fiery Orange Tipped Antenna",
		categories				= "companions",
		effect_desc				= "You control a flying saucer that shoots exploding laser beams on enemies directly below it.",
		effect_desc_short		= "You control a flying saucer that shoots exploding laser beams on enemies directly below it.",
        precursor_types			= "companion",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			attachCompaniontoEntity(getPlayerEntity(), "flying_saucer_explode", "mortal")
		end,
		ondrop_function			= function()
			detachCompanionfromEntity(getPlayerEntity(), "flying_saucer_explode")
		end,
		requires_relic			= "alkaline_battery",
	},
	{
		name 					= "joystick_bleed",
		ui_name					= "Joystick with Blood Red Tipped Antenna",
		categories				= "companions",
		effect_desc				= "You control a flying saucer that shoots bleeding laser beams on enemies directly below it.",
		effect_desc_short		= "You control a flying saucer that shoots bleeding laser beams on enemies directly below it.",
        precursor_types			= "companion",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			attachCompaniontoEntity(getPlayerEntity(), "flying_saucer_bleed", "mortal")
		end,
		ondrop_function			= function()
			detachCompanionfromEntity(getPlayerEntity(), "flying_saucer_bleed")
		end,
		requires_relic			= "alkaline_battery",
	},
	{
		name 					= "joystick_magnet",
		ui_name					= "Joystick with Blue/Red Tipped Antenna",
		categories				= "companions",
		effect_desc				= "You control a flying saucer that shoots attracting laser beams on enemies directly below it.",
		effect_desc_short		= "You control a flying saucer that shoots attracting laser beams on enemies directly below it.",
        precursor_types			= "companion",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			attachCompaniontoEntity(getPlayerEntity(), "flying_saucer_magnet", "mortal")
		end,
		ondrop_function			= function()
			detachCompanionfromEntity(getPlayerEntity(), "flying_saucer_magnet")
		end,
		requires_relic			= "alkaline_battery",
	},
	{
		name 					= "repeater_droid_wind",
		ui_name					= "Wind Repeater Droid",
		categories				= "companions",
		effect_desc				= "A companion that shoots wind laser beams in quick succession joins you.",
		effect_desc_short		= "A wind laser beam shooting companion.",
        precursor_types			= "companion",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			attachCompaniontoEntity(getPlayerEntity(), "repeater_droid_wind", "mortal")
		end,
		ondrop_function			= function()
			detachCompanionfromEntity(getPlayerEntity(), "repeater_droid_wind")
		end,
		requires_relic			= "alkaline_battery",
		combine_exclusive		= true,
		combine_requires_relics	= "security_probe,little_cloud",
	},
	{
		name 					= "repeater_droid_slap",
		ui_name					= "Slap Repeater Droid",
		categories				= "companions",
		effect_desc				= "A companion that shoots slap laser beams in quick succession joins you.",
		effect_desc_short		= "A slap laser beam shooting companion.",
        precursor_types			= "companion",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			attachCompaniontoEntity(getPlayerEntity(), "repeater_droid_slap", "mortal")
		end,
		ondrop_function			= function()
			detachCompanionfromEntity(getPlayerEntity(), "repeater_droid_slap")
		end,
		requires_relic			= "alkaline_battery",
		combine_exclusive		= true,
		combine_requires_relics	= "security_probe,fly_swatter",
	},
	{
		name 					= "repeater_droid_slash",
		ui_name					= "Slash Repeater Droid",
		categories				= "companions",
		effect_desc				= "A companion that shoots slash laser beams in quick succession joins you.",
		effect_desc_short		= "A slash laser beam shooting companion.",
        precursor_types			= "companion",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			attachCompaniontoEntity(getPlayerEntity(), "repeater_droid_slash", "mortal")
		end,
		ondrop_function			= function()
			detachCompanionfromEntity(getPlayerEntity(), "repeater_droid_slash")
		end,
		requires_relic			= "alkaline_battery",
		combine_exclusive		= true,
		combine_requires_relics	= "security_probe,cursed_dagger",
	},
	{
		name 					= "repeater_droid_magnet",
		ui_name					= "Magnetic Repeater Droid",
		categories				= "companions",
		effect_desc				= "A companion that shoots attracting laser beams in quick succession joins you.",
		effect_desc_short		= "A attracting laser beam shooting companion.",
        precursor_types			= "companion",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			attachCompaniontoEntity(getPlayerEntity(), "repeater_droid_magnet", "mortal")
		end,
		ondrop_function			= function()
			detachCompanionfromEntity(getPlayerEntity(), "repeater_droid_magnet")
		end,
		requires_relic			= "alkaline_battery",
		combine_exclusive		= true,
		combine_requires_relics	= "security_probe,magnet",
	},
	{
		name 					= "repeater_droid_bleed",
		ui_name					= "Bleed Repeater Droid",
		categories				= "companions",
		effect_desc				= "A companion that shoots bleeding laser beams in quick succession joins you.",
		effect_desc_short		= "A bleeding laser beam shooting companion.",
        precursor_types			= "companion",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			attachCompaniontoEntity(getPlayerEntity(), "repeater_droid_bleed", "mortal")
		end,
		ondrop_function			= function()
			detachCompanionfromEntity(getPlayerEntity(), "repeater_droid_bleed")
		end,
		requires_relic			= "alkaline_battery",
		combine_exclusive		= true,
		combine_requires_relics	= "security_probe,ancient_dagger",
	},
	{
		name 					= "repeater_droid_explode",
		ui_name					= "Explode Repeater Droid",
		categories				= "companions",
		effect_desc				= "A companion that shoots exploding laser beams in quick succession joins you.",
		effect_desc_short		= "A exploding laser beam shooting companion.",
        precursor_types			= "companion",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			attachCompaniontoEntity(getPlayerEntity(), "repeater_droid_explode", "mortal")
		end,
		ondrop_function			= function()
			detachCompanionfromEntity(getPlayerEntity(), "repeater_droid_explode")
		end,
		requires_relic			= "alkaline_battery",
		combine_exclusive		= true,
		combine_requires_relics	= "security_probe,freedom_ring",
	},
	{
		name 					= "freedom_ring_freezing_liquid",
		ui_name					= "Freedom Ring (Freezing Liquid)",
		categories				= "projectiles",
		effect_desc				= "Your spells have a 30% chance of inducing a sequence of freezing liquid explosions \non enemies hit by them.",
		effect_desc_short		= "Your spells induce freezing liquid explosion sequences on enemies.",
        precursor_types			= "ring",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("freedom_ring_freezing_liquid") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 30 + (5 * getRelicLevel("freedom_ring_freezing_liquid"))
				if(chance >= 1 and chance <= odds) then
					attachLoadEffectOntoEntitiestoProjectile(projectile_entity_id, getCustomEffect("effect_exploding_freezing_liquid").path, "blood_cold_vapour")
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "freedom_ring,freezing_liquid_gem",
	},
	{
		name 					= "neodymium_magnet",
		ui_name					= "Neodymium Magnet",
		categories				= "player",
		effect_desc				= "When you take projectile or melee damage, you have a 50% chance of releasing a barrage of magnetobombs. \nIt also synergizes with Earth Core Fragment relic effect.",
		effect_desc_short		= "You release magnetobombs when taking projectile or melee damage.",
        precursor_types			= "gadget",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		ondamaged_function		= function(damage, message, entity_thats_responsible, is_fatal, projectile_thats_responsible )
			if(isRelicOwned("neodymium_magnet") >= 1) then	
				if(message == "$damage_projectile" or message == "$damage_melee") then
					SetRandomSeed( GameGetFrameNum(), math.random(1,50))
					local chance = Random(1,100)
					local odds = 50 + (5 * getRelicLevel("neodymium_magnet"))
					if(chance >= 1 and chance <= odds) then
						local player_x, player_y = EntityGetTransform(GetUpdatedEntityID())
						EntityLoad("mods/ArcaneReliquary/files/entity_definitions/load_on_explosion/launch_circular_projectiles_magnetobombs.xml", player_x, player_y)
					end
				end
			end
		end,
		ongroundpound_function	= function( entity_id)
			if(isRelicOwned("neodymium_magnet") >= 1) then
				LoadGameEffectEntityTo( entity_id, getCustomEffect("effect_shoot_projectiles_above_magnetobomb").path )
			end
		end,
	},
	{
		name 					= "festive_bomb",
		ui_name					= "Festive Bomb",
		categories				= "player",
		effect_desc				= "When you cast a spell there's a 25% chance you'll release a barrage of bomb laying bombs. \nIt also synergizes with Earth Core Fragment relic effect.",
		effect_desc_short		= "You release bomb laying bombs when casting spells.",
        precursor_types			= "gadget",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("festive_bomb") >= 1) then	
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 25 + (5 * getRelicLevel("festive_bomb"))
				if(chance >= 1 and chance <= odds) then
					local player_x, player_y = EntityGetTransform(GetUpdatedEntityID())
					EntityLoad("mods/ArcaneReliquary/files/entity_definitions/load_on_explosion/launch_circular_projectiles_bomblayingbombs.xml", player_x, player_y)
				end
			end
		end,
		ongroundpound_function	= function( entity_id)
			if(isRelicOwned("festive_bomb") >= 1) then
				LoadGameEffectEntityTo( entity_id, getCustomEffect("effect_shoot_projectiles_above_bomblayingbomb").path )
			end
		end,
	},
	{
		name 					= "diamond_ring_transluscid",
		ui_name					= "Transluscid Diamond Ring",
		categories				= "projectiles,status_effects",
		effect_desc				= "Your spells have a 30% chance of creating a mystic target on \ncreatures you hit, making them summon thunderbolts towards themselves.",
		effect_desc_short		= "Your spells create mystic targets on creatures, summoning thunderbolts towards them.",
        precursor_types			= "magic_artifact,ring",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("diamond_ring_transluscid") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 30 + (5 * getRelicLevel("diamond_ring_transluscid"))
				if(chance >= 1 and chance <= odds) then
					attachLoadEffectOntoEntitiestoProjectile(projectile_entity_id, getCustomEffect("effect_projectile_target_thunderbolt").path, "spark_blue_dark")
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "elemental_band,copper_wiring",
	},
	{
		name 					= "diamond_ring_ice_blue",
		ui_name					= "Ice Blue Diamond Ring",
		categories				= "projectiles,status_effects",
		effect_desc				= "Your spells have a 30% chance of creating a mystic target on \ncreatures you hit, making them summon iceballs towards themselves.",
		effect_desc_short		= "Your spells create mystic targets on creatures, summoning iceballs towards them.",
        precursor_types			= "magic_artifact,ring",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("diamond_ring_ice_blue") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 30 + (5 * getRelicLevel("diamond_ring_ice_blue"))
				if(chance >= 1 and chance <= odds) then
					attachLoadEffectOntoEntitiestoProjectile(projectile_entity_id, getCustomEffect("effect_projectile_target_iceball").path, "plasma_fading")
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "elemental_band,copper_wiring",
	},
	{
		name 					= "enchanted_pot",
		ui_name					= "Enchanted Pot",
		categories				= "companions",
		effect_desc				= "An enchanted pot appears. Throw a Toxic, Water, Lava, Poly, Purple Slime or \nBlood Seed on it to cultivate a special plant that is able to consume a \nspecific material in order to grow, and once it is fully grown, a \nrandom reward is dispensed. \nYou can use it multiple times cultivating different plants for different rewards.",
		effect_desc_short		= "An enchanted pot appears. Throw magic seeds on it to grow plants that give you rewards.",
        precursor_types			= "companion",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			attachCompaniontoEntity(getPlayerEntity(), "enchanted_pot", "mortal")
		end,
		ondrop_function			= function()
			detachCompanionfromEntity(getPlayerEntity(), "enchanted_pot")
		end,
		requires_relic			= "bag_of_magical_seeds",
	},
	{
		name 					= "humid_gem",
		ui_name					= "Humid Gem",
		categories				= "projectiles,materials",
		effect_desc				= "Spells you cast that explode on impact have a 20% chance of releasing water on impact.",
		effect_desc_short		= "Your spells release water.",
        precursor_types			= "elemental_gem",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("humid_gem") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 20 + (5 * getRelicLevel("humid_gem"))
				if(chance >= 1 and chance <= odds) then
					attachLoadEntityOnExplosiontoProjectile(projectile_entity_id, getLoadOnExplosion("launch_elemental_particles_water").path, "water")
				end
			end
		end,
	},
	{
		name 					= "sideshoot_scepter_red_sphere",
		ui_name					= "Red Sphere Scepter",
		categories				= "projectiles,status_effects",
		effect_desc				= "Spells you cast have a 30% chance of inducing enemies they hit to shoot red laser orbs sideways.",
		effect_desc_short		= "Your spells induce enemies to shoot red laser orbs sideways.",
        precursor_types			= "magic_artifact",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("sideshoot_scepter_red_sphere") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 30 + (5 * getRelicLevel("sideshoot_scepter_red_sphere"))
				if(chance >= 1 and chance <= odds) then
					attachLoadEffectOntoEntitiestoProjectile(projectile_entity_id, getCustomEffect("effect_shoot_projectiles_sideways_red_laser_orb").path, "spark_red")
				end
			end
		end,
	},
	{
		name 					= "sideshoot_scepter_blue_sphere",
		ui_name					= "Blue Sphere Scepter",
		categories				= "projectiles,status_effects",
		effect_desc				= "Spells you cast have a 30% chance of inducing enemies they hit to shoot blue laser orbs sideways.",
		effect_desc_short		= "Your spells induce enemies to shoot blue laser orbs sideways.",
        precursor_types			= "magic_artifact",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("sideshoot_scepter_blue_sphere") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 30 + (5 * getRelicLevel("sideshoot_scepter_blue_sphere"))
				if(chance >= 1 and chance <= odds) then
					attachLoadEffectOntoEntitiestoProjectile(projectile_entity_id, getCustomEffect("effect_shoot_projectiles_sideways_blue_laser_orb").path, "spark_blue_dark")
				end
			end
		end,
	},
	{
		name 					= "sideshoot_scepter_fireball",
		ui_name					= "Fireball Scepter",
		categories				= "projectiles,status_effects",
		effect_desc				= "Spells you cast have a 30% chance of inducing enemies they hit to shoot firebombs sideways.",
		effect_desc_short		= "Your spells induce enemies to shoot firebombs sideways.",
        precursor_types			= "magic_artifact",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("sideshoot_scepter_fireball") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 30 + (5 * getRelicLevel("sideshoot_scepter_fireball"))
				if(chance >= 1 and chance <= odds) then
					attachLoadEffectOntoEntitiestoProjectile(projectile_entity_id, getCustomEffect("effect_shoot_projectiles_sideways_firebomb").path, "lava")
				end
			end
		end,
	},
	{
		name 					= "sideshoot_scepter_disc_bullet",
		ui_name					= "Disc Projectile Scepter",
		categories				= "projectiles,status_effects",
		effect_desc				= "Spells you cast have a 30% chance of inducing enemies they hit to shoot disc projectiles sideways.",
		effect_desc_short		= "Your spells induce enemies to shoot disc projectiles sideways.",
        precursor_types			= "magic_artifact",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("sideshoot_scepter_disc_bullet") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 30 + (5 * getRelicLevel("sideshoot_scepter_disc_bullet"))
				if(chance >= 1 and chance <= odds) then
					attachLoadEffectOntoEntitiestoProjectile(projectile_entity_id, getCustomEffect("effect_shoot_projectiles_sideways_disc_bullet").path, "spark_white")
				end
			end
		end,
	},
	{
		name 					= "sideshoot_scepter_slime",
		ui_name					= "Slime Scepter",
		categories				= "projectiles,status_effects",
		effect_desc				= "Spells you cast have a 30% chance of inducing enemies they hit to shoot slime bolts sideways.",
		effect_desc_short		= "Your spells induce enemies to shoot slime bolts sideways.",
        precursor_types			= "magic_artifact",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("sideshoot_scepter_slime") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 30 + (5 * getRelicLevel("sideshoot_scepter_slime"))
				if(chance >= 1 and chance <= odds) then
					attachLoadEffectOntoEntitiestoProjectile(projectile_entity_id, getCustomEffect("effect_shoot_projectiles_sideways_slime").path, "radioactive_gas")
				end
			end
		end,
	},
	{
		name 					= "sideshoot_scepter_thunderbolt",
		ui_name					= "Thunderbolt Scepter",
		categories				= "projectiles,status_effects",
		effect_desc				= "Spells you cast have a 30% chance of inducing enemies they hit to shoot thunderbolts sideways.",
		effect_desc_short		= "Your spells induce enemies to shoot thunderbolts sideways.",
        precursor_types			= "magic_artifact",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("sideshoot_scepter_thunderbolt") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 30 + (5 * getRelicLevel("sideshoot_scepter_thunderbolt"))
				if(chance >= 1 and chance <= odds) then
					attachLoadEffectOntoEntitiestoProjectile(projectile_entity_id, getCustomEffect("effect_shoot_projectiles_sideways_thunderbolt").path, "spark_blue_dark")
				end
			end
		end,
	},
	{
		name 					= "amethyst_necklace",
		ui_name					= "Amethyst Necklace",
		categories				= "projectiles",
		effect_desc				= "Your spells have a 20% chance of exploding into a \nbarrage of polymorphine fireballs when they explode or hit something. \nIt also synergizes with Earth Core Fragment relic effect.",
		effect_desc_short		= "Your spells explode into polymorphine fireballs on impact.",
        precursor_types			= "magic_artifact,necklace",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("amethyst_necklace") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 20 + (5 * getRelicLevel("amethyst_necklace"))
				if(chance >= 1 and chance <= odds) then	
					addBuffVisualIndicator_Projectile(projectile_entity_id, "plasma_fading_pink")
					attachLoadEntityOnExplosiontoProjectile(projectile_entity_id, getLoadOnExplosion("launch_circular_projectiles_fireballs_poly").path)
				end
			end
		end,
		ongroundpound_function	= function( entity_id)
			if(isRelicOwned("amethyst_necklace") >= 1) then
				LoadGameEffectEntityTo( entity_id, getCustomEffect("effect_shoot_projectiles_above_fireball_poly").path )
			end
		end,
	},
	{
		name 					= "fools_gold_necklace",
		ui_name					= "Fool's Gold Necklace",
		effect_desc				= "Your spells have a 20% chance of exploding into a \nbarrage of urine fireballs when they explode or hit something. \nIt also synergizes with Earth Core Fragment relic effect.",
		categories				= "projectiles",
		effect_desc_short		= "Your spells explode into urine fireballs on impact.",
        precursor_types			= "magic_artifact,necklace",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("fools_gold_necklace") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 20 + (5 * getRelicLevel("fools_gold_necklace"))
				if(chance >= 1 and chance <= odds) then	
					addBuffVisualIndicator_Projectile(projectile_entity_id, "spark_yellow")
					attachLoadEntityOnExplosiontoProjectile(projectile_entity_id, getLoadOnExplosion("launch_circular_projectiles_fireballs_urine").path)
				end
			end
		end,
		ongroundpound_function	= function( entity_id)
			if(isRelicOwned("fools_gold_necklace") >= 1) then
				LoadGameEffectEntityTo( entity_id, getCustomEffect("effect_shoot_projectiles_above_fireball_urine").path )
			end
		end,
	},
	{
		name 					= "wizard_hat",
		ui_name					= "Wizard Hat",
		categories				= "player",
		effect_desc				= "When you take projectile or melee damage, you have a 30% chance of releasing a barrage of teleportbombs. \nIt also synergizes with Earth Core Fragment relic effect.",
		effect_desc_short		= "You release teleportbombs when taking projectile or melee damage.",
        precursor_types			= "gadget",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		ondamaged_function		= function(damage, message, entity_thats_responsible, is_fatal, projectile_thats_responsible )
			if(isRelicOwned("wizard_hat") >= 1) then	
				if(message == "$damage_projectile" or message == "$damage_melee") then
					SetRandomSeed( GameGetFrameNum(), math.random(1,50))
					local chance = Random(1,100)
					local odds = 30 + (5 * getRelicLevel("wizard_hat"))
					if(chance >= 1 and chance <= odds) then
						local player_x, player_y = EntityGetTransform(GetUpdatedEntityID())
						EntityLoad("mods/ArcaneReliquary/files/entity_definitions/load_on_explosion/launch_circular_projectiles_teleportbombs.xml", player_x, player_y)
					end
				end
			end
		end,
		ongroundpound_function	= function( entity_id)
			if(isRelicOwned("wizard_hat") >= 1) then
				LoadGameEffectEntityTo( entity_id, getCustomEffect("effect_shoot_projectiles_above_teleportbomb").path )
			end
		end,
	},
	{
		name 					= "wizard_skull",
		ui_name					= "Wizard Skull",
		categories				= "companions",
		effect_desc				= "A friendly skull that shoots teleportbombs joins you.",
		effect_desc_short		= "A teleportbomb shooting companion.",
        precursor_types			= "companion",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			attachCompaniontoEntity(getPlayerEntity(), "wizard_skull", "mortal")
		end,
		ondrop_function			= function()
			detachCompanionfromEntity(getPlayerEntity(), "wizard_skull")
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "enchanted_skull,wizard_hat",
		requires_relic			= "bone",
	},
	{
		name 					= "magic_bubble",
		ui_name					= "Magic Bubble",
		categories				= "projectiles",
		effect_desc				= "Your spells have a 30% chance of exploding into a \nbarrage of bubbletrapbombs when they explode or hit something. \nIt also synergizes with Earth Core Fragment relic effect.",
		effect_desc_short		= "Your spells explode into bubbletrapbombs on impact.",
        precursor_types			= "effect_stone",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("magic_bubble") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 30 + (5 * getRelicLevel("magic_bubble"))
				if(chance >= 1 and chance <= odds) then	
					addBuffVisualIndicator_Projectile(projectile_entity_id, "plasma_fading")
					attachLoadEntityOnExplosiontoProjectile(projectile_entity_id, getLoadOnExplosion("launch_circular_projectiles_bubbletrapbombs").path)
				end
			end
		end,
		ongroundpound_function	= function( entity_id)
			if(isRelicOwned("magic_bubble") >= 1) then
				LoadGameEffectEntityTo( entity_id, getCustomEffect("effect_shoot_projectiles_above_bubbletrapbomb").path )
			end
		end,
	},
	{
		name 					= "soapy_skull",
		ui_name					= "Soapy Skull",
		categories				= "companions",
		effect_desc				= "A friendly skull that shoots bubbletrapbombs joins you.",
		effect_desc_short		= "A bubbletrapbomb shooting companion.",
        precursor_types			= "companion",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			attachCompaniontoEntity(getPlayerEntity(), "soapy_skull", "mortal")
		end,
		ondrop_function			= function()
			detachCompanionfromEntity(getPlayerEntity(), "soapy_skull")
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "enchanted_skull,magic_bubble",
		requires_relic			= "bone",
	},
	{
		name 					= "sentient_crystal_magmatic",
		ui_name					= "Sentient Magmatic Crystal",
		categories				= "companions,materials",
		effect_desc				= "A sentient crystal that shoots lava orbs if \nenemy projectiles touch them joins you.",
		effect_desc_short		= "A lava orb shooting companion.",
        precursor_types			= "companion",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			attachCompaniontoEntity(getPlayerEntity(), "sentient_crystal_magmatic", "mortal")
		end,
		ondrop_function			= function()
			detachCompanionfromEntity(getPlayerEntity(), "sentient_crystal_magmatic")
		end,
	},
	{
		name 					= "rainbow_skull",
		ui_name					= "Rainbow Skull",
		categories				= "companions",
		effect_desc				= "A friendly skull that shoots material spit bombs joins you.",
		effect_desc_short		= "A material spit bombs shooting companion.",
        precursor_types			= "companion",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			attachCompaniontoEntity(getPlayerEntity(), "rainbow_skull", "mortal")
		end,
		ondrop_function			= function()
			detachCompanionfromEntity(getPlayerEntity(), "rainbow_skull")
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "enchanted_skull,rainbow_sphere",
		requires_relic			= "bone",
	},
	{
		name 					= "rainbow_sphere",
		ui_name					= "Rainbow Sphere",
		categories				= "projectiles",
		effect_desc				= "Your spells have a 30% chance of exploding into a \nbarrage of material spit bombs when they explode or hit something. \nIt also synergizes with Earth Core Fragment relic effect.",
		effect_desc_short		= "Your spells explode into material spit bombs on impact.",
        precursor_types			= "effect_stone",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("rainbow_sphere") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 30 + (5 * getRelicLevel("rainbow_sphere"))
				if(chance >= 1 and chance <= odds) then	
					addBuffVisualIndicator_Projectile(projectile_entity_id, "material_rainbow")
					attachLoadEntityOnExplosiontoProjectile(projectile_entity_id, getLoadOnExplosion("launch_circular_projectiles_materialspitbombs").path)
				end
			end
		end,
		ongroundpound_function	= function( entity_id)
			if(isRelicOwned("rainbow_sphere") >= 1) then
				LoadGameEffectEntityTo( entity_id, getCustomEffect("effect_shoot_projectiles_above_materialspitbomb").path )
			end
		end,
	},
	{
		name 					= "sentient_crystal_bitter",
		ui_name					= "Sentient Bitter Crystal",
		categories				= "companions,materials",
		effect_desc				= "A sentient crystal that shoots acid orbs if \nenemy projectiles touch them joins you.",
		effect_desc_short		= "A acid orb shooting companion.",
        precursor_types			= "companion",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			attachCompaniontoEntity(getPlayerEntity(), "sentient_crystal_bitter", "mortal")
		end,
		ondrop_function			= function()
			detachCompanionfromEntity(getPlayerEntity(), "sentient_crystal_bitter")
		end,
	},
	{
		name 					= "sentient_crystal_bloody",
		ui_name					= "Sentient Bloody Crystal",
		categories				= "companions,materials",
		effect_desc				= "A sentient crystal that shoots corrosive blood orbs if \nenemy projectiles touch them joins you.",
		effect_desc_short		= "A corrosive blood orb shooting companion.",
        precursor_types			= "companion",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			attachCompaniontoEntity(getPlayerEntity(), "sentient_crystal_bloody", "mortal")
		end,
		ondrop_function			= function()
			detachCompanionfromEntity(getPlayerEntity(), "sentient_crystal_bloody")
		end,
	},
	{
		name 					= "sentient_crystal_humid",
		ui_name					= "Sentient Humid Crystal",
		categories				= "companions,materials",
		effect_desc				= "A sentient crystal that shoots water orbs if \nenemy projectiles touch them joins you.",
		effect_desc_short		= "A water orb shooting companion.",
        precursor_types			= "companion",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			attachCompaniontoEntity(getPlayerEntity(), "sentient_crystal_humid", "mortal")
		end,
		ondrop_function			= function()
			detachCompanionfromEntity(getPlayerEntity(), "sentient_crystal_humid")
		end,
	},
	{
		name 					= "sentient_crystal_toxic",
		ui_name					= "Sentient Toxic Crystal",
		categories				= "companions,materials",
		effect_desc				= "A sentient crystal that shoots toxic sludge orbs if \nenemy projectiles touch them joins you.",
		effect_desc_short		= "A toxic sludge orb shooting companion.",
        precursor_types			= "companion",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			attachCompaniontoEntity(getPlayerEntity(), "sentient_crystal_toxic", "mortal")
		end,
		ondrop_function			= function()
			detachCompanionfromEntity(getPlayerEntity(), "sentient_crystal_toxic")
		end,
	},
	{
		name 					= "sentient_crystal_cold",
		ui_name					= "Sentient Cold Crystal",
		categories				= "companions,materials",
		effect_desc				= "A sentient crystal that shoots freezing liquid orbs if \nenemy projectiles touch them joins you.",
		effect_desc_short		= "A freezing liquid orb shooting companion.",
        precursor_types			= "companion",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			attachCompaniontoEntity(getPlayerEntity(), "sentient_crystal_cold", "mortal")
		end,
		ondrop_function			= function()
			detachCompanionfromEntity(getPlayerEntity(), "sentient_crystal_cold")
		end,
	},
	{
		name 					= "sentient_crystal_uric",
		ui_name					= "Sentient Uric Crystal",
		categories				= "companions,materials",
		effect_desc				= "A sentient crystal that shoots urine orbs if \nenemy projectiles touch them joins you.",
		effect_desc_short		= "A urine orb shooting companion.",
        precursor_types			= "companion",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			attachCompaniontoEntity(getPlayerEntity(), "sentient_crystal_uric", "mortal")
		end,
		ondrop_function			= function()
			detachCompanionfromEntity(getPlayerEntity(), "sentient_crystal_uric")
		end,
	},
	{
		name 					= "sentient_crystal_greased",
		ui_name					= "Sentient Greased Crystal",
		categories				= "companions,materials",
		effect_desc				= "A sentient crystal that shoots oil orbs if \nenemy projectiles touch them joins you.",
		effect_desc_short		= "A oil orb shooting companion.",
        precursor_types			= "companion",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			attachCompaniontoEntity(getPlayerEntity(), "sentient_crystal_greased", "mortal")
		end,
		ondrop_function			= function()
			detachCompanionfromEntity(getPlayerEntity(), "sentient_crystal_greased")
		end,
	},
	{
		name 					= "sentient_crystal_volatile",
		ui_name					= "Sentient Volatile Crystal",
		categories				= "companions,materials",
		effect_desc				= "A sentient crystal that shoots toxic gas orbs if \nenemy projectiles touch them joins you.",
		effect_desc_short		= "A toxic gas orb shooting companion.",
        precursor_types			= "companion",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			attachCompaniontoEntity(getPlayerEntity(), "sentient_crystal_volatile", "mortal")
		end,
		ondrop_function			= function()
			detachCompanionfromEntity(getPlayerEntity(), "sentient_crystal_volatile")
		end,
	},
	{
		name 					= "sentient_crystal_ethanolic",
		ui_name					= "Sentient Ethanolic Crystal",
		categories				= "companions,materials",
		effect_desc				= "A sentient crystal that shoots whiskey orbs if \nenemy projectiles touch them joins you.",
		effect_desc_short		= "A whiskey orb shooting companion.",
        precursor_types			= "companion",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			attachCompaniontoEntity(getPlayerEntity(), "sentient_crystal_ethanolic", "mortal")
		end,
		ondrop_function			= function()
			detachCompanionfromEntity(getPlayerEntity(), "sentient_crystal_ethanolic")
		end,
	},
	{
		name 					= "sentient_crystal_mutating",
		ui_name					= "Sentient Mutating Crystal",
		categories				= "companions,materials",
		effect_desc				= "A sentient crystal that shoots polymorphine orbs if \nenemy projectiles touch them joins you.",
		effect_desc_short		= "A polymorphine orb shooting companion.",
        precursor_types			= "companion",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			attachCompaniontoEntity(getPlayerEntity(), "sentient_crystal_mutating", "mortal")
		end,
		ondrop_function			= function()
			detachCompanionfromEntity(getPlayerEntity(), "sentient_crystal_mutating")
		end,
	},
	{
		name 					= "sentient_crystal_slimy",
		ui_name					= "Sentient Slimy Crystal",
		categories				= "companions,materials",
		effect_desc				= "A sentient crystal that shoots slime orbs if \nenemy projectiles touch them joins you.",
		effect_desc_short		= "A slime orb shooting companion.",
        precursor_types			= "companion",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			attachCompaniontoEntity(getPlayerEntity(), "sentient_crystal_slimy", "mortal")
		end,
		ondrop_function			= function()
			detachCompanionfromEntity(getPlayerEntity(), "sentient_crystal_slimy")
		end,
	},
	{
		name 					= "mystic_shield_blood",
		ui_name					= "Blood Mystic Shield",
		categories				= "companions",
		effect_desc				= "A mystic shield that destroys enemy projectiles and shoots corrosive blood orbs back appears and follows your mouse cursor.",
		effect_desc_short		= "A mystic shield that destroys enemy projectiles and shoots corrosive blood orbs back appears and follows your mouse cursor.",
        precursor_types			= "companion",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			attachCompaniontoEntity(getPlayerEntity(), "mystic_shield_blood", "mortal")
		end,
		ondrop_function			= function()
			detachCompanionfromEntity(getPlayerEntity(), "mystic_shield_blood")
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "mystic_shield,corrosive_blood_gem",
	},
	{
		name 					= "mystic_shield_freezing_liquid",
		ui_name					= "Freezing Liquid Mystic Shield",
		categories				= "companions",
		effect_desc				= "A mystic shield that destroys enemy projectiles and shoots freezing liquid orbs back appears and follows your mouse cursor.",
		effect_desc_short		= "A mystic shield that destroys enemy projectiles and shoots freezing liquid orbs back appears and follows your mouse cursor.",
        precursor_types			= "companion",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			attachCompaniontoEntity(getPlayerEntity(), "mystic_shield_freezing_liquid", "mortal")
		end,
		ondrop_function			= function()
			detachCompanionfromEntity(getPlayerEntity(), "mystic_shield_freezing_liquid")
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "mystic_shield,freezing_liquid_gem",
	},
	{
		name 					= "mystic_shield_alcohol",
		ui_name					= "Whiskey Mystic Shield",
		categories				= "companions",
		effect_desc				= "A mystic shield that destroys enemy projectiles and shoots whiskey orbs back appears and follows your mouse cursor.",
		effect_desc_short		= "A mystic shield that destroys enemy projectiles and shoots whiskey orbs back appears and follows your mouse cursor.",
        precursor_types			= "companion",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			attachCompaniontoEntity(getPlayerEntity(), "mystic_shield_alcohol", "mortal")
		end,
		ondrop_function			= function()
			detachCompanionfromEntity(getPlayerEntity(), "mystic_shield_alcohol")
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "mystic_shield,alcohol_gem",
	},
	{
		name 					= "mystic_shield_oil",
		ui_name					= "Oil Mystic Shield",
		categories				= "companions",
		effect_desc				= "A mystic shield that destroys enemy projectiles and shoots oil orbs back appears and follows your mouse cursor.",
		effect_desc_short		= "A mystic shield that destroys enemy projectiles and shoots oil orbs back appears and follows your mouse cursor.",
        precursor_types			= "companion",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			attachCompaniontoEntity(getPlayerEntity(), "mystic_shield_oil", "mortal")
		end,
		ondrop_function			= function()
			detachCompanionfromEntity(getPlayerEntity(), "mystic_shield_oil")
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "mystic_shield,oil_gem",
	},
	{
		name 					= "mystic_shield_water",
		ui_name					= "Water Mystic Shield",
		categories				= "companions",
		effect_desc				= "A mystic shield that destroys enemy projectiles and shoots water orbs back appears and follows your mouse cursor.",
		effect_desc_short		= "A mystic shield that destroys enemy projectiles and shoots water orbs back appears and follows your mouse cursor.",
        precursor_types			= "companion",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			attachCompaniontoEntity(getPlayerEntity(), "mystic_shield_water", "mortal")
		end,
		ondrop_function			= function()
			detachCompanionfromEntity(getPlayerEntity(), "mystic_shield_water")
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "mystic_shield,humid_gem",
	},
	{
		name 					= "mystic_shield_poly",
		ui_name					= "Polymorphine Mystic Shield",
		categories				= "companions",
		effect_desc				= "A mystic shield that destroys enemy projectiles and shoots polymorphine orbs back appears and follows your mouse cursor.",
		effect_desc_short		= "A mystic shield that destroys enemy projectiles and shoots polymorphine orbs back appears and follows your mouse cursor.",
        precursor_types			= "companion",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			attachCompaniontoEntity(getPlayerEntity(), "mystic_shield_poly", "mortal")
		end,
		ondrop_function			= function()
			detachCompanionfromEntity(getPlayerEntity(), "mystic_shield_poly")
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "mystic_shield,poly_gem",
	},
	{
		name 					= "mystic_shield_explosive",
		ui_name					= "Explosive Mystic Shield",
		categories				= "companions",
		effect_desc				= "A mystic shield that destroys enemy projectiles and shoots pink explosive orbs back appears and follows your mouse cursor.",
		effect_desc_short		= "A mystic shield that destroys enemy projectiles and shoots pink explosive orbs back appears and follows your mouse cursor.",
        precursor_types			= "companion",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			attachCompaniontoEntity(getPlayerEntity(), "mystic_shield_explosive", "mortal")
		end,
		ondrop_function			= function()
			detachCompanionfromEntity(getPlayerEntity(), "mystic_shield_explosive")
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "mystic_shield,rhodonite_necklace",
	},
	{
		name 					= "mystic_shield_slime",
		ui_name					= "Slime Mystic Shield",
		categories				= "companions",
		effect_desc				= "A mystic shield that destroys enemy projectiles and shoots slime orbs back appears and follows your mouse cursor.",
		effect_desc_short		= "A mystic shield that destroys enemy projectiles and shoots slime orbs back appears and follows your mouse cursor.",
        precursor_types			= "companion",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			attachCompaniontoEntity(getPlayerEntity(), "mystic_shield_slime", "mortal")
		end,
		ondrop_function			= function()
			detachCompanionfromEntity(getPlayerEntity(), "mystic_shield_slime")
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "mystic_shield,slime_gem",
	},
	{
		name 					= "mystic_shield_radioactive_liquid",
		ui_name					= "Toxic Sludge Mystic Shield",
		categories				= "companions",
		effect_desc				= "A mystic shield that destroys enemy projectiles and shoots toxic sludge orbs back appears and follows your mouse cursor.",
		effect_desc_short		= "A mystic shield that destroys enemy projectiles and shoots toxic sludge orbs back appears and follows your mouse cursor.",
        precursor_types			= "companion",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			attachCompaniontoEntity(getPlayerEntity(), "mystic_shield_radioactive_liquid", "mortal")
		end,
		ondrop_function			= function()
			detachCompanionfromEntity(getPlayerEntity(), "mystic_shield_radioactive_liquid")
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "mystic_shield,toxic_gem",
	},
	{
		name 					= "mystic_shield_radioactive_gas",
		ui_name					= "Toxic Gas Mystic Shield",
		categories				= "companions",
		effect_desc				= "A mystic shield that destroys enemy projectiles and shoots toxic gas orbs back appears and follows your mouse cursor.",
		effect_desc_short		= "A mystic shield that destroys enemy projectiles and shoots toxic gas orbs back appears and follows your mouse cursor.",
        precursor_types			= "companion",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			attachCompaniontoEntity(getPlayerEntity(), "mystic_shield_radioactive_gas", "mortal")
		end,
		ondrop_function			= function()
			detachCompanionfromEntity(getPlayerEntity(), "mystic_shield_radioactive_gas")
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "mystic_shield,canned_beans",
	},
	{
		name 					= "mystic_shield_urine",
		ui_name					= "Urine Mystic Shield",
		categories				= "companions",
		effect_desc				= "A mystic shield that destroys enemy projectiles and shoots urine orbs back appears and follows your mouse cursor.",
		effect_desc_short		= "A mystic shield that destroys enemy projectiles and shoots urine orbs back appears and follows your mouse cursor.",
        precursor_types			= "companion",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			attachCompaniontoEntity(getPlayerEntity(), "mystic_shield_urine", "mortal")
		end,
		ondrop_function			= function()
			detachCompanionfromEntity(getPlayerEntity(), "mystic_shield_urine")
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "mystic_shield,urine_gem",
	},
	{
		name 					= "parry_shield_bleed",
		ui_name					= "Parry Shield with Bleed Effect",
		categories				= "player",
		effect_desc				= "You hold a shield that converts enemy projectiles into bloodbombs \nand inflicts the bleeding effect on enemies that touch it.",
		effect_desc_short		= "You hold a shield that converts enemy projectiles into bloodbombs and inflicts the bleeding effect on enemies.",
        precursor_types			= "gadget,magic_artifact",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			EntityAddChild(getPlayerEntity(), EntityLoad("mods/ArcaneReliquary/files/entity_definitions/others/parry_shield_bleed_handler.xml"))
		end,
		ondrop_function			= function()
			local entity_children = EntityGetAllChildren(getPlayerEntity())
			if entity_children ~= nil then
				for i,child_id in ipairs(entity_children) do
					if(EntityGetName(child_id) == "parry_shield_bleed_handler" and not EntityHasTag(child_id, "relic")) then
						local x, y = EntityGetTransform(child_id)
						EntityKill(child_id)
					end
				end
			end
		end,
		excludes_relics_desc	= "It can't be used along with other Parry Shield Relic variations",
		excludes_relics_tags	= "parry_shield_variation",
		combine_exclusive		= true,
		combine_requires_relics	= "parry_shield,coagulated_blood",
	},
	{
		name 					= "parry_shield_teleport",
		ui_name					= "Parry Shield with Teleport Effect",
		categories				= "player",
		effect_desc				= "You hold a shield that converts enemy projectiles into teleportbombs \nand inflicts the teleport effect on enemies that touch it.",
		effect_desc_short		= "You hold a shield that converts enemy projectiles into teleportbombs and inflicts the teleport effect on enemies.",
        precursor_types			= "gadget,magic_artifact",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			EntityAddChild(getPlayerEntity(), EntityLoad("mods/ArcaneReliquary/files/entity_definitions/others/parry_shield_teleport_handler.xml"))
		end,
		ondrop_function			= function()
			local entity_children = EntityGetAllChildren(getPlayerEntity())
			if entity_children ~= nil then
				for i,child_id in ipairs(entity_children) do
					if(EntityGetName(child_id) == "parry_shield_teleport_handler" and not EntityHasTag(child_id, "relic")) then
						local x, y = EntityGetTransform(child_id)
						EntityKill(child_id)
					end
				end
			end
		end,
		excludes_relics_desc	= "It can't be used along with other Parry Shield Relic variations",
		excludes_relics_tags	= "parry_shield_variation",
		combine_exclusive		= true,
		combine_requires_relics	= "parry_shield,wizard_hat",
	},
	{
		name 					= "parry_shield_bubbletrap",
		ui_name					= "Parry Shield with Bubbletrap Effect",
		categories				= "player",
		effect_desc				= "You hold a shield that converts enemy projectiles into bubbletraps \nand inflicts the bubbletrap effect on enemies that touch it.",
		effect_desc_short		= "You hold a shield that converts enemy projectiles into bubbletraps and inflicts the bubbletrap effect on enemies.",
        precursor_types			= "gadget,magic_artifact",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			EntityAddChild(getPlayerEntity(), EntityLoad("mods/ArcaneReliquary/files/entity_definitions/others/parry_shield_bubbletrap_handler.xml"))
		end,
		ondrop_function			= function()
			local entity_children = EntityGetAllChildren(getPlayerEntity())
			if entity_children ~= nil then
				for i,child_id in ipairs(entity_children) do
					if(EntityGetName(child_id) == "parry_shield_bubbletrap_handler" and not EntityHasTag(child_id, "relic")) then
						local x, y = EntityGetTransform(child_id)
						EntityKill(child_id)
					end
				end
			end
		end,
		excludes_relics_desc	= "It can't be used along with other Parry Shield Relic variations",
		excludes_relics_tags	= "parry_shield_variation",
		combine_exclusive		= true,
		combine_requires_relics	= "parry_shield,magic_bubble",
	},
	{
		name 					= "parry_shield_brickwall",
		ui_name					= "Brickwall Parry Shield",
		categories				= "player",
		effect_desc				= "You hold a shield that destroys enemy projectiles and \ncauses melee damage to enemies that touch it.",
		effect_desc_short		= "You hold a shield that destroys enemy projectiles and causes melee damage to enemies that touch it.",
        precursor_types			= "gadget,magic_artifact",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			EntityAddChild(getPlayerEntity(), EntityLoad("mods/ArcaneReliquary/files/entity_definitions/others/parry_shield_brickwall_handler.xml"))
		end,
		ondrop_function			= function()
			local entity_children = EntityGetAllChildren(getPlayerEntity())
			if entity_children ~= nil then
				for i,child_id in ipairs(entity_children) do
					if(EntityGetName(child_id) == "parry_shield_brickwall_handler" and not EntityHasTag(child_id, "relic")) then
						local x, y = EntityGetTransform(child_id)
						EntityKill(child_id)
					end
				end
			end
		end,
		excludes_relics_desc	= "It can't be used along with other Parry Shield Relic variations",
		excludes_relics_tags	= "parry_shield_variation",
	},
	{
		name 					= "parry_shield_blades",
		ui_name					= "Bladed Parry Shield",
		categories				= "player",
		effect_desc				= "You hold a shield that redirects enemy projectiles and \ncauses slice damage to enemies that touch it.",
		effect_desc_short		= "You hold a shield that redirects enemy projectiles and causes slice damage to enemies that touch it.",
        precursor_types			= "gadget,magic_artifact",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			EntityAddChild(getPlayerEntity(), EntityLoad("mods/ArcaneReliquary/files/entity_definitions/others/parry_shield_blades_handler.xml"))
		end,
		ondrop_function			= function()
			local entity_children = EntityGetAllChildren(getPlayerEntity())
			if entity_children ~= nil then
				for i,child_id in ipairs(entity_children) do
					if(EntityGetName(child_id) == "parry_shield_blades_handler" and not EntityHasTag(child_id, "relic")) then
						local x, y = EntityGetTransform(child_id)
						EntityKill(child_id)
					end
				end
			end
		end,
		excludes_relics_desc	= "It can't be used along with other Parry Shield Relic variations",
		excludes_relics_tags	= "parry_shield_variation",
	},
	{
		name 					= "parry_shield_concretewall",
		ui_name					= "Concrete Wall Parry Shield",
		categories				= "player",
		effect_desc				= "You hold a shield that deflects enemy projectiles \nadding melee damage to them, and causes melee damage to enemies that touch it.",
		effect_desc_short		= "You hold a shield that deflects enemy projectiles adding melee damage to them and causes melee damage to enemies that touch it.",
        precursor_types			= "gadget,magic_artifact",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			EntityAddChild(getPlayerEntity(), EntityLoad("mods/ArcaneReliquary/files/entity_definitions/others/parry_shield_concretewall_handler.xml"))
		end,
		ondrop_function			= function()
			local entity_children = EntityGetAllChildren(getPlayerEntity())
			if entity_children ~= nil then
				for i,child_id in ipairs(entity_children) do
					if(EntityGetName(child_id) == "parry_shield_concretewall_handler" and not EntityHasTag(child_id, "relic")) then
						local x, y = EntityGetTransform(child_id)
						EntityKill(child_id)
					end
				end
			end
		end,
		excludes_relics_desc	= "It can't be used along with other Parry Shield Relic variations",
		excludes_relics_tags	= "parry_shield_variation",
		combine_exclusive		= true,
		combine_requires_relics	= "parry_shield_brickwall,emerald_gauntlet",
	},
	{
		name 					= "parry_shield_blades_slice",
		ui_name					= "Bladed Parry Shield with Slice Damage",
		categories				= "player",
		effect_desc				= "You hold a shield that deflects enemy projectiles \nadding slice damage to them, and causes slice damage to enemies that touch it.",
		effect_desc_short		= "You hold a shield that deflects enemy projectiles adding slice damage to them and causes slice damage to enemies that touch it.",
        precursor_types			= "gadget,magic_artifact",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			EntityAddChild(getPlayerEntity(), EntityLoad("mods/ArcaneReliquary/files/entity_definitions/others/parry_shield_blades_slice_handler.xml"))
		end,
		ondrop_function			= function()
			local entity_children = EntityGetAllChildren(getPlayerEntity())
			if entity_children ~= nil then
				for i,child_id in ipairs(entity_children) do
					if(EntityGetName(child_id) == "parry_shield_blades_slice_handler" and not EntityHasTag(child_id, "relic")) then
						local x, y = EntityGetTransform(child_id)
						EntityKill(child_id)
					end
				end
			end
		end,
		excludes_relics_desc	= "It can't be used along with other Parry Shield Relic variations",
		excludes_relics_tags	= "parry_shield_variation",
		combine_exclusive		= true,
		combine_requires_relics	= "parry_shield_blades,pizza_slice",
	},
	{
		name 					= "parry_shield_scatter",
		ui_name					= "Copter Parry Shield",
		categories				= "player",
		effect_desc				= "You hold a shield that scatters enemy projectiles, embuing them with the \nbleeding effect, and causes slice damage to enemies that touch it. \nIf you point this shield up, you gain a small upwards momentum.",
		effect_desc_short		= "You hold a shield that scatters enemy projectiles, embuing them with the bleeding effect, and causes slice damage to enemies that touch it.",
        precursor_types			= "gadget,magic_artifact",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			EntityAddChild(getPlayerEntity(), EntityLoad("mods/ArcaneReliquary/files/entity_definitions/others/parry_shield_scatter_handler.xml"))
		end,
		ondrop_function			= function()
			local entity_children = EntityGetAllChildren(getPlayerEntity())
			if entity_children ~= nil then
				for i,child_id in ipairs(entity_children) do
					if(EntityGetName(child_id) == "parry_shield_scatter_handler" and not EntityHasTag(child_id, "relic")) then
						local x, y = EntityGetTransform(child_id)
						EntityKill(child_id)
					end
				end
			end
		end,
		excludes_relics_desc	= "It can't be used along with other Parry Shield Relic variations",
		excludes_relics_tags	= "parry_shield_variation",
		combine_exclusive		= true,
		combine_requires_relics	= "parry_shield_blades,ancient_root,ancient_dagger",
	},
	{
		name 					= "parry_shield_umbrella",
		ui_name					= "Umbrella",
		categories				= "player",
		effect_desc				= "You hold an umbrella that scatters enemy projectiles, embuing them with the \nthunder cloud effect and also inflicting this effect on enemies that touch it. \nIf you point it up while mid-air, you are able to glide.",
		effect_desc_short		= "You hold an umbrella that scatters enemy projectiles, embuing them with the thunder cloud effect.",
        precursor_types			= "gadget,magic_artifact",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			EntityAddChild(getPlayerEntity(), EntityLoad("mods/ArcaneReliquary/files/entity_definitions/others/parry_shield_umbrella_handler.xml"))
		end,
		ondrop_function			= function()
			local entity_children = EntityGetAllChildren(getPlayerEntity())
			if entity_children ~= nil then
				for i,child_id in ipairs(entity_children) do
					if(EntityGetName(child_id) == "parry_shield_umbrella_handler" and not EntityHasTag(child_id, "relic")) then
						local x, y = EntityGetTransform(child_id)
						EntityKill(child_id)
					end
				end
			end
		end,
		excludes_relics_desc	= "It can't be used along with other Parry Shield Relic variations",
		excludes_relics_tags	= "parry_shield_variation",
		combine_exclusive		= true,
		combine_requires_relics	= "parry_shield,thunder_cloud",
	},
	{
		name 					= "parry_shield_metal",
		ui_name					= "Metal Parry Shield",
		categories				= "player",
		effect_desc				= "You hold a shield that destroys enemy projectiles and causes automatic \ndamage to their shooter equivalent to the base damage of their projectile.",
		effect_desc_short		= "You hold a shield that destroys enemy projectiles and damages their shooter automatically.",
        precursor_types			= "gadget,magic_artifact",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			EntityAddChild(getPlayerEntity(), EntityLoad("mods/ArcaneReliquary/files/entity_definitions/others/parry_shield_metal_handler.xml"))
		end,
		ondrop_function			= function()
			local entity_children = EntityGetAllChildren(getPlayerEntity())
			if entity_children ~= nil then
				for i,child_id in ipairs(entity_children) do
					if(EntityGetName(child_id) == "parry_shield_metal_handler" and not EntityHasTag(child_id, "relic")) then
						local x, y = EntityGetTransform(child_id)
						EntityKill(child_id)
					end
				end
			end
		end,
		excludes_relics_desc	= "It can't be used along with other Parry Shield Relic variations",
		excludes_relics_tags	= "parry_shield_variation",
		combine_exclusive		= true,
		combine_requires_relics	= "parry_shield,cursed_mirror",
	},
	{
		name 					= "parry_shield_medical",
		ui_name					= "Medical Parry Shield",
		categories				= "player",
		effect_desc				= "You hold a shield that destroys enemy projectiles and converts their base \ndamage into hp for your character.",
		effect_desc_short		= "You hold a shield that destroys enemy projectiles and converts their base damage into hp for your character.",
        precursor_types			= "gadget,magic_artifact",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			EntityAddChild(getPlayerEntity(), EntityLoad("mods/ArcaneReliquary/files/entity_definitions/others/parry_shield_medical_handler.xml"))
		end,
		ondrop_function			= function()
			local entity_children = EntityGetAllChildren(getPlayerEntity())
			if entity_children ~= nil then
				for i,child_id in ipairs(entity_children) do
					if(EntityGetName(child_id) == "parry_shield_medical_handler" and not EntityHasTag(child_id, "relic")) then
						local x, y = EntityGetTransform(child_id)
						EntityKill(child_id)
					end
				end
			end
		end,
		excludes_relics_desc	= "It can't be used along with other Parry Shield Relic variations",
		excludes_relics_tags	= "parry_shield_variation",
		combine_exclusive		= true,
		combine_requires_relics	= "parry_shield_metal,blood_gem",
	},
	{
		name 					= "stone_hand_ornament",
		ui_name					= "Stone Hand Ornament",
		categories				= "projectiles,status_effects",
		effect_desc				= "Your spells have a 30% chance of creating a mystic target on \ncreatures you hit, making them summon stone punch hands towards themselves.",
		effect_desc_short		= "Your spells create mystic targets on creatures, summoning stone punch hands towards them.",
        precursor_types			= "magic_artifact,ring",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("stone_hand_ornament") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 30 + (5 * getRelicLevel("stone_hand_ornament"))
				if(chance >= 1 and chance <= odds) then
					attachLoadEffectOntoEntitiestoProjectile(projectile_entity_id, getCustomEffect("effect_projectile_target_punch_hand").path, "spark_white")
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "stone_hand_melee,diamond_ring",
	},
	{
		name 					= "parry_shield_scatter_freeze",
		ui_name					= "Copter Parry Shield (Freeze)",
		categories				= "player",
		effect_desc				= "You hold a shield that scatters enemy projectiles, embuing them with the \nfreeze effect, and causes slice damage to enemies that touch it. \nIf you point this shield up, you gain a small upwards momentum.",
		effect_desc_short		= "You hold a shield that scatters enemy projectiles, embuing them with the freeze effect, and causes slice damage to enemies that touch it.",
        precursor_types			= "gadget,magic_artifact",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			EntityAddChild(getPlayerEntity(), EntityLoad("mods/ArcaneReliquary/files/entity_definitions/others/parry_shield_scatter_freeze_handler.xml"))
		end,
		ondrop_function			= function()
			local entity_children = EntityGetAllChildren(getPlayerEntity())
			if entity_children ~= nil then
				for i,child_id in ipairs(entity_children) do
					if(EntityGetName(child_id) == "parry_shield_scatter_freeze_handler" and not EntityHasTag(child_id, "relic")) then
						local x, y = EntityGetTransform(child_id)
						EntityKill(child_id)
					end
				end
			end
		end,
		excludes_relics_desc	= "It can't be used along with other Parry Shield Relic variations",
		excludes_relics_tags	= "parry_shield_variation",
		combine_exclusive		= true,
		combine_requires_relics	= "parry_shield_blades,ancient_root,ice_artifact",
	},
	{
		name 					= "parry_shield_scatter_electrocute",
		ui_name					= "Copter Parry Shield (Electrocute)",
		categories				= "player",
		effect_desc				= "You hold a shield that scatters enemy projectiles, embuing them with the \nelectrocute effect, and causes slice damage to enemies that touch it. \nIf you point this shield up, you gain a small upwards momentum.",
		effect_desc_short		= "You hold a shield that scatters enemy projectiles, embuing them with the electrocute effect, and causes slice damage to enemies that touch it.",
        precursor_types			= "gadget,magic_artifact",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			EntityAddChild(getPlayerEntity(), EntityLoad("mods/ArcaneReliquary/files/entity_definitions/others/parry_shield_scatter_electrocute_handler.xml"))
		end,
		ondrop_function			= function()
			local entity_children = EntityGetAllChildren(getPlayerEntity())
			if entity_children ~= nil then
				for i,child_id in ipairs(entity_children) do
					if(EntityGetName(child_id) == "parry_shield_scatter_electrocute_handler" and not EntityHasTag(child_id, "relic")) then
						local x, y = EntityGetTransform(child_id)
						EntityKill(child_id)
					end
				end
			end
		end,
		excludes_relics_desc	= "It can't be used along with other Parry Shield Relic variations",
		excludes_relics_tags	= "parry_shield_variation",
		combine_exclusive		= true,
		combine_requires_relics	= "parry_shield_blades,ancient_root,charged_necklace",
	},
	{
		name 					= "parry_shield_scatter_poly",
		ui_name					= "Copter Parry Shield (Poly)",
		categories				= "player",
		effect_desc				= "You hold a shield that scatters enemy projectiles, embuing them with the \npolymorph effect, and causes slice damage to enemies that touch it. \nIf you point this shield up, you gain a small upwards momentum.",
		effect_desc_short		= "You hold a shield that scatters enemy projectiles, embuing them with the polymorph effect, and causes slice damage to enemies that touch it.",
        precursor_types			= "gadget,magic_artifact",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			EntityAddChild(getPlayerEntity(), EntityLoad("mods/ArcaneReliquary/files/entity_definitions/others/parry_shield_scatter_poly_handler.xml"))
		end,
		ondrop_function			= function()
			local entity_children = EntityGetAllChildren(getPlayerEntity())
			if entity_children ~= nil then
				for i,child_id in ipairs(entity_children) do
					if(EntityGetName(child_id) == "parry_shield_scatter_poly_handler" and not EntityHasTag(child_id, "relic")) then
						local x, y = EntityGetTransform(child_id)
						EntityKill(child_id)
					end
				end
			end
		end,
		excludes_relics_desc	= "It can't be used along with other Parry Shield Relic variations",
		excludes_relics_tags	= "parry_shield_variation",
		combine_exclusive		= true,
		combine_requires_relics	= "parry_shield_blades,ancient_root,poly_flask",
	},
	{
		name 					= "parry_shield_scatter_explode",
		ui_name					= "Copter Parry Shield (Explode)",
		categories				= "player",
		effect_desc				= "You hold a shield that scatters enemy projectiles, embuing them with the \nexploding effect, and causes slice damage to enemies that touch it. \nIf you point this shield up, you gain a small upwards momentum.",
		effect_desc_short		= "You hold a shield that scatters enemy projectiles, embuing them with the exploding effect, and causes slice damage to enemies that touch it.",
        precursor_types			= "gadget,magic_artifact",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			EntityAddChild(getPlayerEntity(), EntityLoad("mods/ArcaneReliquary/files/entity_definitions/others/parry_shield_scatter_explode_handler.xml"))
		end,
		ondrop_function			= function()
			local entity_children = EntityGetAllChildren(getPlayerEntity())
			if entity_children ~= nil then
				for i,child_id in ipairs(entity_children) do
					if(EntityGetName(child_id) == "parry_shield_scatter_explode_handler" and not EntityHasTag(child_id, "relic")) then
						local x, y = EntityGetTransform(child_id)
						EntityKill(child_id)
					end
				end
			end
		end,
		excludes_relics_desc	= "It can't be used along with other Parry Shield Relic variations",
		excludes_relics_tags	= "parry_shield_variation",
		combine_exclusive		= true,
		combine_requires_relics	= "parry_shield_blades,ancient_root,exploding_scepter",
	},
	{
		name 					= "parry_shield_scatter_wind",
		ui_name					= "Copter Parry Shield (Wind)",
		categories				= "player",
		effect_desc				= "You hold a shield that scatters enemy projectiles, embuing them with the \nheavy wind effect, and causes slice damage to enemies that touch it. \nIf you point this shield up, you gain a small upwards momentum.",
		effect_desc_short		= "You hold a shield that scatters enemy projectiles, embuing them with the heavy wind effect, and causes slice damage to enemies that touch it.",
        precursor_types			= "gadget,magic_artifact",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			EntityAddChild(getPlayerEntity(), EntityLoad("mods/ArcaneReliquary/files/entity_definitions/others/parry_shield_scatter_wind_handler.xml"))
		end,
		ondrop_function			= function()
			local entity_children = EntityGetAllChildren(getPlayerEntity())
			if entity_children ~= nil then
				for i,child_id in ipairs(entity_children) do
					if(EntityGetName(child_id) == "parry_shield_scatter_wind_handler" and not EntityHasTag(child_id, "relic")) then
						local x, y = EntityGetTransform(child_id)
						EntityKill(child_id)
					end
				end
			end
		end,
		excludes_relics_desc	= "It can't be used along with other Parry Shield Relic variations",
		excludes_relics_tags	= "parry_shield_variation",
		combine_exclusive		= true,
		combine_requires_relics	= "parry_shield_blades,ancient_root,little_cloud",
	},
	{
		name 					= "parry_shield_scatter_slap",
		ui_name					= "Copter Parry Shield (Slap)",
		categories				= "player",
		effect_desc				= "You hold a shield that scatters enemy projectiles, embuing them with the \nslap effect, and causes slice damage to enemies that touch it. \nIf you point this shield up, you gain a small upwards momentum.",
		effect_desc_short		= "You hold a shield that scatters enemy projectiles, embuing them with the slap effect, and causes slice damage to enemies that touch it.",
        precursor_types			= "gadget,magic_artifact",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			EntityAddChild(getPlayerEntity(), EntityLoad("mods/ArcaneReliquary/files/entity_definitions/others/parry_shield_scatter_slap_handler.xml"))
		end,
		ondrop_function			= function()
			local entity_children = EntityGetAllChildren(getPlayerEntity())
			if entity_children ~= nil then
				for i,child_id in ipairs(entity_children) do
					if(EntityGetName(child_id) == "parry_shield_scatter_slap_handler" and not EntityHasTag(child_id, "relic")) then
						local x, y = EntityGetTransform(child_id)
						EntityKill(child_id)
					end
				end
			end
		end,
		excludes_relics_desc	= "It can't be used along with other Parry Shield Relic variations",
		excludes_relics_tags	= "parry_shield_variation",
		combine_exclusive		= true,
		combine_requires_relics	= "parry_shield_blades,ancient_root,fly_swatter",
	},
	{
		name 					= "parry_shield_scatter_slash",
		ui_name					= "Copter Parry Shield (Slash)",
		categories				= "player",
		effect_desc				= "You hold a shield that scatters enemy projectiles, embuing them with the \nslash effect, and causes slice damage to enemies that touch it. \nIf you point this shield up, you gain a small upwards momentum.",
		effect_desc_short		= "You hold a shield that scatters enemy projectiles, embuing them with the slash effect, and causes slice damage to enemies that touch it.",
        precursor_types			= "gadget,magic_artifact",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			EntityAddChild(getPlayerEntity(), EntityLoad("mods/ArcaneReliquary/files/entity_definitions/others/parry_shield_scatter_slash_handler.xml"))
		end,
		ondrop_function			= function()
			local entity_children = EntityGetAllChildren(getPlayerEntity())
			if entity_children ~= nil then
				for i,child_id in ipairs(entity_children) do
					if(EntityGetName(child_id) == "parry_shield_scatter_slash_handler" and not EntityHasTag(child_id, "relic")) then
						local x, y = EntityGetTransform(child_id)
						EntityKill(child_id)
					end
				end
			end
		end,
		excludes_relics_desc	= "It can't be used along with other Parry Shield Relic variations",
		excludes_relics_tags	= "parry_shield_variation",
		combine_exclusive		= true,
		combine_requires_relics	= "parry_shield_blades,ancient_root,cursed_dagger",
	},
	{
		name 					= "parry_shield_umbrella_green",
		ui_name					= "Green Umbrella",
		categories				= "player",
		effect_desc				= "You hold an umbrella that scatters enemy projectiles, embuing them with the \nacid cloud effect and also inflicting this effect on enemies that touch it. \nIf you point it up while mid-air, you are able to glide.",
		effect_desc_short		= "You hold an umbrella that scatters enemy projectiles, embuing them with the acid cloud effect.",
        precursor_types			= "gadget,magic_artifact",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			EntityAddChild(getPlayerEntity(), EntityLoad("mods/ArcaneReliquary/files/entity_definitions/others/parry_shield_umbrella_green_handler.xml"))
		end,
		ondrop_function			= function()
			local entity_children = EntityGetAllChildren(getPlayerEntity())
			if entity_children ~= nil then
				for i,child_id in ipairs(entity_children) do
					if(EntityGetName(child_id) == "parry_shield_umbrella_green_handler" and not EntityHasTag(child_id, "relic")) then
						local x, y = EntityGetTransform(child_id)
						EntityKill(child_id)
					end
				end
			end
		end,
		excludes_relics_desc	= "It can't be used along with other Parry Shield Relic variations",
		excludes_relics_tags	= "parry_shield_variation",
		combine_exclusive		= true,
		combine_requires_relics	= "parry_shield,little_cloud,corroding_stone",
	},
	{
		name 					= "parry_shield_umbrella_pink",
		ui_name					= "Pink Umbrella",
		categories				= "player",
		effect_desc				= "You hold an umbrella that scatters enemy projectiles, embuing them with the \npolymorphine cloud effect and also inflicting this effect on enemies that touch it. \nIf you point it up while mid-air, you are able to glide.",
		effect_desc_short		= "You hold an umbrella that scatters enemy projectiles, embuing them with the polymorphine cloud effect.",
        precursor_types			= "gadget,magic_artifact",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			EntityAddChild(getPlayerEntity(), EntityLoad("mods/ArcaneReliquary/files/entity_definitions/others/parry_shield_umbrella_pink_handler.xml"))
		end,
		ondrop_function			= function()
			local entity_children = EntityGetAllChildren(getPlayerEntity())
			if entity_children ~= nil then
				for i,child_id in ipairs(entity_children) do
					if(EntityGetName(child_id) == "parry_shield_umbrella_pink_handler" and not EntityHasTag(child_id, "relic")) then
						local x, y = EntityGetTransform(child_id)
						EntityKill(child_id)
					end
				end
			end
		end,
		excludes_relics_desc	= "It can't be used along with other Parry Shield Relic variations",
		excludes_relics_tags	= "parry_shield_variation",
		combine_exclusive		= true,
		combine_requires_relics	= "parry_shield,little_cloud,polymorphic_stone",
	},
	{
		name 					= "parry_shield_umbrella_purple",
		ui_name					= "Purple Umbrella",
		categories				= "player",
		effect_desc				= "You hold an umbrella that scatters enemy projectiles, embuing them with the \npoison cloud effect and also inflicting this effect on enemies that touch it. \nIf you point it up while mid-air, you are able to glide.",
		effect_desc_short		= "You hold an umbrella that scatters enemy projectiles, embuing them with the poison cloud effect.",
        precursor_types			= "gadget,magic_artifact",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			EntityAddChild(getPlayerEntity(), EntityLoad("mods/ArcaneReliquary/files/entity_definitions/others/parry_shield_umbrella_purple_handler.xml"))
		end,
		ondrop_function			= function()
			local entity_children = EntityGetAllChildren(getPlayerEntity())
			if entity_children ~= nil then
				for i,child_id in ipairs(entity_children) do
					if(EntityGetName(child_id) == "parry_shield_umbrella_purple_handler" and not EntityHasTag(child_id, "relic")) then
						local x, y = EntityGetTransform(child_id)
						EntityKill(child_id)
					end
				end
			end
		end,
		excludes_relics_desc	= "It can't be used along with other Parry Shield Relic variations",
		excludes_relics_tags	= "parry_shield_variation",
		combine_exclusive		= true,
		combine_requires_relics	= "parry_shield,little_cloud,vile_stone",
	},
	{
		name 					= "parry_shield_umbrella_green_lime",
		ui_name					= "Lime Green Umbrella",
		categories				= "player",
		effect_desc				= "You hold an umbrella that scatters enemy projectiles, embuing them with the \ntoxic cloud effect and also inflicting this effect on enemies that touch it. \nIf you point it up while mid-air, you are able to glide.",
		effect_desc_short		= "You hold an umbrella that scatters enemy projectiles, embuing them with the toxic cloud effect.",
        precursor_types			= "gadget,magic_artifact",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			EntityAddChild(getPlayerEntity(), EntityLoad("mods/ArcaneReliquary/files/entity_definitions/others/parry_shield_umbrella_green_lime_handler.xml"))
		end,
		ondrop_function			= function()
			local entity_children = EntityGetAllChildren(getPlayerEntity())
			if entity_children ~= nil then
				for i,child_id in ipairs(entity_children) do
					if(EntityGetName(child_id) == "parry_shield_umbrella_green_lime_handler" and not EntityHasTag(child_id, "relic")) then
						local x, y = EntityGetTransform(child_id)
						EntityKill(child_id)
					end
				end
			end
		end,
		excludes_relics_desc	= "It can't be used along with other Parry Shield Relic variations",
		excludes_relics_tags	= "parry_shield_variation",
		combine_exclusive		= true,
		combine_requires_relics	= "parry_shield,little_cloud,ball_of_goo",
	},
	{
		name 					= "parry_shield_umbrella_yellow",
		ui_name					= "Yellow Umbrella",
		categories				= "player",
		effect_desc				= "You hold an umbrella that scatters enemy projectiles, embuing them with the \nurine cloud effect and also inflicting this effect on enemies that touch it. \nIf you point it up while mid-air, you are able to glide.",
		effect_desc_short		= "You hold an umbrella that scatters enemy projectiles, embuing them with the urine cloud effect.",
        precursor_types			= "gadget,magic_artifact",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			EntityAddChild(getPlayerEntity(), EntityLoad("mods/ArcaneReliquary/files/entity_definitions/others/parry_shield_umbrella_yellow_handler.xml"))
		end,
		ondrop_function			= function()
			local entity_children = EntityGetAllChildren(getPlayerEntity())
			if entity_children ~= nil then
				for i,child_id in ipairs(entity_children) do
					if(EntityGetName(child_id) == "parry_shield_umbrella_yellow_handler" and not EntityHasTag(child_id, "relic")) then
						local x, y = EntityGetTransform(child_id)
						EntityKill(child_id)
					end
				end
			end
		end,
		excludes_relics_desc	= "It can't be used along with other Parry Shield Relic variations",
		excludes_relics_tags	= "parry_shield_variation",
		combine_exclusive		= true,
		combine_requires_relics	= "parry_shield,little_cloud,uric_stone",
	},
	{
		name 					= "parry_shield_umbrella_orange",
		ui_name					= "Orange Umbrella",
		categories				= "player",
		effect_desc				= "You hold an umbrella that scatters enemy projectiles, embuing them with the \nlava cloud effect and also inflicting this effect on enemies that touch it. \nIf you point it up while mid-air, you are able to glide.",
		effect_desc_short		= "You hold an umbrella that scatters enemy projectiles, embuing them with the lava cloud effect.",
        precursor_types			= "gadget,magic_artifact",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			EntityAddChild(getPlayerEntity(), EntityLoad("mods/ArcaneReliquary/files/entity_definitions/others/parry_shield_umbrella_orange_handler.xml"))
		end,
		ondrop_function			= function()
			local entity_children = EntityGetAllChildren(getPlayerEntity())
			if entity_children ~= nil then
				for i,child_id in ipairs(entity_children) do
					if(EntityGetName(child_id) == "parry_shield_umbrella_orange_handler" and not EntityHasTag(child_id, "relic")) then
						local x, y = EntityGetTransform(child_id)
						EntityKill(child_id)
					end
				end
			end
		end,
		excludes_relics_desc	= "It can't be used along with other Parry Shield Relic variations",
		excludes_relics_tags	= "parry_shield_variation",
		combine_exclusive		= true,
		combine_requires_relics	= "parry_shield,little_cloud,obsidian",
	},
	{
		name 					= "parry_shield_umbrella_pink_dark",
		ui_name					= "Dark Pink Umbrella",
		categories				= "player",
		effect_desc				= "You hold an umbrella that scatters enemy projectiles, embuing them with the \nslime cloud effect and also inflicting this effect on enemies that touch it. \nIf you point it up while mid-air, you are able to glide.",
		effect_desc_short		= "You hold an umbrella that scatters enemy projectiles, embuing them with the slime cloud effect.",
        precursor_types			= "gadget,magic_artifact",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			EntityAddChild(getPlayerEntity(), EntityLoad("mods/ArcaneReliquary/files/entity_definitions/others/parry_shield_umbrella_pink_dark_handler.xml"))
		end,
		ondrop_function			= function()
			local entity_children = EntityGetAllChildren(getPlayerEntity())
			if entity_children ~= nil then
				for i,child_id in ipairs(entity_children) do
					if(EntityGetName(child_id) == "parry_shield_umbrella_pink_dark_handler" and not EntityHasTag(child_id, "relic")) then
						local x, y = EntityGetTransform(child_id)
						EntityKill(child_id)
					end
				end
			end
		end,
		excludes_relics_desc	= "It can't be used along with other Parry Shield Relic variations",
		excludes_relics_tags	= "parry_shield_variation",
		combine_exclusive		= true,
		combine_requires_relics	= "parry_shield,little_cloud,slime_gem",
	},
	{
		name 					= "parry_shield_metal_bleed",
		ui_name					= "Metal Parry Shield (Bleed)",
		categories				= "player",
		effect_desc				= "You hold a shield that destroys enemy projectiles and inflicts \nthe bleeding effect on their shooter.",
		effect_desc_short		= "You hold a shield that destroys enemy projectiles and inflicts the bleeding effect on their shooter.",
        precursor_types			= "gadget,magic_artifact",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			EntityAddChild(getPlayerEntity(), EntityLoad("mods/ArcaneReliquary/files/entity_definitions/others/parry_shield_metal_bleed_handler.xml"))
		end,
		ondrop_function			= function()
			local entity_children = EntityGetAllChildren(getPlayerEntity())
			if entity_children ~= nil then
				for i,child_id in ipairs(entity_children) do
					if(EntityGetName(child_id) == "parry_shield_metal_bleed_handler" and not EntityHasTag(child_id, "relic")) then
						local x, y = EntityGetTransform(child_id)
						EntityKill(child_id)
					end
				end
			end
		end,
		excludes_relics_desc	= "It can't be used along with other Parry Shield Relic variations",
		excludes_relics_tags	= "parry_shield_variation",
		combine_exclusive		= true,
		combine_requires_relics	= "parry_shield_metal,ancient_dagger",
	},
	{
		name 					= "parry_shield_metal_freeze",
		ui_name					= "Metal Parry Shield (Freeze)",
		categories				= "player",
		effect_desc				= "You hold a shield that destroys enemy projectiles and inflicts \nthe freeze effect on their shooter.",
		effect_desc_short		= "You hold a shield that destroys enemy projectiles and inflicts the freeze effect on their shooter.",
        precursor_types			= "gadget,magic_artifact",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			EntityAddChild(getPlayerEntity(), EntityLoad("mods/ArcaneReliquary/files/entity_definitions/others/parry_shield_metal_freeze_handler.xml"))
		end,
		ondrop_function			= function()
			local entity_children = EntityGetAllChildren(getPlayerEntity())
			if entity_children ~= nil then
				for i,child_id in ipairs(entity_children) do
					if(EntityGetName(child_id) == "parry_shield_metal_freeze_handler" and not EntityHasTag(child_id, "relic")) then
						local x, y = EntityGetTransform(child_id)
						EntityKill(child_id)
					end
				end
			end
		end,
		excludes_relics_desc	= "It can't be used along with other Parry Shield Relic variations",
		excludes_relics_tags	= "parry_shield_variation",
		combine_exclusive		= true,
		combine_requires_relics	= "parry_shield_metal,ice_artifact",
	},
	{
		name 					= "parry_shield_metal_electrocute",
		ui_name					= "Metal Parry Shield (Electrocute)",
		categories				= "player",
		effect_desc				= "You hold a shield that destroys enemy projectiles and inflicts \nthe electrocute effect on their shooter.",
		effect_desc_short		= "You hold a shield that destroys enemy projectiles and inflicts the electrocute effect on their shooter.",
        precursor_types			= "gadget,magic_artifact",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			EntityAddChild(getPlayerEntity(), EntityLoad("mods/ArcaneReliquary/files/entity_definitions/others/parry_shield_metal_electrocute_handler.xml"))
		end,
		ondrop_function			= function()
			local entity_children = EntityGetAllChildren(getPlayerEntity())
			if entity_children ~= nil then
				for i,child_id in ipairs(entity_children) do
					if(EntityGetName(child_id) == "parry_shield_metal_electrocute_handler" and not EntityHasTag(child_id, "relic")) then
						local x, y = EntityGetTransform(child_id)
						EntityKill(child_id)
					end
				end
			end
		end,
		excludes_relics_desc	= "It can't be used along with other Parry Shield Relic variations",
		excludes_relics_tags	= "parry_shield_variation",
		combine_exclusive		= true,
		combine_requires_relics	= "parry_shield_metal,charged_necklace",
	},
	{
		name 					= "parry_shield_metal_poly",
		ui_name					= "Metal Parry Shield (Poly)",
		categories				= "player",
		effect_desc				= "You hold a shield that destroys enemy projectiles and inflicts \nthe polymorph effect on their shooter.",
		effect_desc_short		= "You hold a shield that destroys enemy projectiles and inflicts the polymorph effect on their shooter.",
        precursor_types			= "gadget,magic_artifact",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			EntityAddChild(getPlayerEntity(), EntityLoad("mods/ArcaneReliquary/files/entity_definitions/others/parry_shield_metal_poly_handler.xml"))
		end,
		ondrop_function			= function()
			local entity_children = EntityGetAllChildren(getPlayerEntity())
			if entity_children ~= nil then
				for i,child_id in ipairs(entity_children) do
					if(EntityGetName(child_id) == "parry_shield_metal_poly_handler" and not EntityHasTag(child_id, "relic")) then
						local x, y = EntityGetTransform(child_id)
						EntityKill(child_id)
					end
				end
			end
		end,
		excludes_relics_desc	= "It can't be used along with other Parry Shield Relic variations",
		excludes_relics_tags	= "parry_shield_variation",
		combine_exclusive		= true,
		combine_requires_relics	= "parry_shield_metal,poly_flask",
	},
	{
		name 					= "parry_shield_metal_explode",
		ui_name					= "Metal Parry Shield (Explode)",
		categories				= "player",
		effect_desc				= "You hold a shield that destroys enemy projectiles and inflicts \nthe exploding effect on their shooter.",
		effect_desc_short		= "You hold a shield that destroys enemy projectiles and inflicts the exploding effect on their shooter.",
        precursor_types			= "gadget,magic_artifact",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			EntityAddChild(getPlayerEntity(), EntityLoad("mods/ArcaneReliquary/files/entity_definitions/others/parry_shield_metal_explode_handler.xml"))
		end,
		ondrop_function			= function()
			local entity_children = EntityGetAllChildren(getPlayerEntity())
			if entity_children ~= nil then
				for i,child_id in ipairs(entity_children) do
					if(EntityGetName(child_id) == "parry_shield_metal_explode_handler" and not EntityHasTag(child_id, "relic")) then
						local x, y = EntityGetTransform(child_id)
						EntityKill(child_id)
					end
				end
			end
		end,
		excludes_relics_desc	= "It can't be used along with other Parry Shield Relic variations",
		excludes_relics_tags	= "parry_shield_variation",
		combine_exclusive		= true,
		combine_requires_relics	= "parry_shield_metal,exploding_scepter",
	},
	{
		name 					= "parry_shield_metal_slap",
		ui_name					= "Metal Parry Shield (Slap)",
		categories				= "player",
		effect_desc				= "You hold a shield that destroys enemy projectiles and inflicts \nthe slap effect on their shooter.",
		effect_desc_short		= "You hold a shield that destroys enemy projectiles and inflicts the slap effect on their shooter.",
        precursor_types			= "gadget,magic_artifact",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			EntityAddChild(getPlayerEntity(), EntityLoad("mods/ArcaneReliquary/files/entity_definitions/others/parry_shield_metal_slap_handler.xml"))
		end,
		ondrop_function			= function()
			local entity_children = EntityGetAllChildren(getPlayerEntity())
			if entity_children ~= nil then
				for i,child_id in ipairs(entity_children) do
					if(EntityGetName(child_id) == "parry_shield_metal_slap_handler" and not EntityHasTag(child_id, "relic")) then
						local x, y = EntityGetTransform(child_id)
						EntityKill(child_id)
					end
				end
			end
		end,
		excludes_relics_desc	= "It can't be used along with other Parry Shield Relic variations",
		excludes_relics_tags	= "parry_shield_variation",
		combine_exclusive		= true,
		combine_requires_relics	= "parry_shield_metal,fly_swatter",
	},
	{
		name 					= "parry_shield_metal_wind",
		ui_name					= "Metal Parry Shield (Wind)",
		categories				= "player",
		effect_desc				= "You hold a shield that destroys enemy projectiles and inflicts \nthe heavy wind effect on their shooter.",
		effect_desc_short		= "You hold a shield that destroys enemy projectiles and inflicts the heavy wind effect on their shooter.",
        precursor_types			= "gadget,magic_artifact",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			EntityAddChild(getPlayerEntity(), EntityLoad("mods/ArcaneReliquary/files/entity_definitions/others/parry_shield_metal_wind_handler.xml"))
		end,
		ondrop_function			= function()
			local entity_children = EntityGetAllChildren(getPlayerEntity())
			if entity_children ~= nil then
				for i,child_id in ipairs(entity_children) do
					if(EntityGetName(child_id) == "parry_shield_metal_wind_handler" and not EntityHasTag(child_id, "relic")) then
						local x, y = EntityGetTransform(child_id)
						EntityKill(child_id)
					end
				end
			end
		end,
		excludes_relics_desc	= "It can't be used along with other Parry Shield Relic variations",
		excludes_relics_tags	= "parry_shield_variation",
		combine_exclusive		= true,
		combine_requires_relics	= "parry_shield_metal,little_cloud",
	},
	{
		name 					= "parry_shield_metal_slash",
		ui_name					= "Metal Parry Shield (Slash)",
		categories				= "player",
		effect_desc				= "You hold a shield that destroys enemy projectiles and inflicts \nthe slash effect on their shooter.",
		effect_desc_short		= "You hold a shield that destroys enemy projectiles and inflicts the slash effect on their shooter.",
        precursor_types			= "gadget,magic_artifact",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			EntityAddChild(getPlayerEntity(), EntityLoad("mods/ArcaneReliquary/files/entity_definitions/others/parry_shield_metal_slash_handler.xml"))
		end,
		ondrop_function			= function()
			local entity_children = EntityGetAllChildren(getPlayerEntity())
			if entity_children ~= nil then
				for i,child_id in ipairs(entity_children) do
					if(EntityGetName(child_id) == "parry_shield_metal_slash_handler" and not EntityHasTag(child_id, "relic")) then
						local x, y = EntityGetTransform(child_id)
						EntityKill(child_id)
					end
				end
			end
		end,
		excludes_relics_desc	= "It can't be used along with other Parry Shield Relic variations",
		excludes_relics_tags	= "parry_shield_variation",
		combine_exclusive		= true,
		combine_requires_relics	= "parry_shield_metal,cursed_dagger",
	},
	{
		name 					= "parry_shield_metal_magnet",
		ui_name					= "Metal Parry Shield (Magnet)",
		categories				= "player",
		effect_desc				= "You hold a shield that destroys enemy projectiles and inflicts \nthe attraction effect on their shooter.",
		effect_desc_short		= "You hold a shield that destroys enemy projectiles and inflicts the attraction effect on their shooter.",
        precursor_types			= "gadget,magic_artifact",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			EntityAddChild(getPlayerEntity(), EntityLoad("mods/ArcaneReliquary/files/entity_definitions/others/parry_shield_metal_magnet_handler.xml"))
		end,
		ondrop_function			= function()
			local entity_children = EntityGetAllChildren(getPlayerEntity())
			if entity_children ~= nil then
				for i,child_id in ipairs(entity_children) do
					if(EntityGetName(child_id) == "parry_shield_metal_magnet_handler" and not EntityHasTag(child_id, "relic")) then
						local x, y = EntityGetTransform(child_id)
						EntityKill(child_id)
					end
				end
			end
		end,
		excludes_relics_desc	= "It can't be used along with other Parry Shield Relic variations",
		excludes_relics_tags	= "parry_shield_variation",
		combine_exclusive		= true,
		combine_requires_relics	= "parry_shield_metal,magnet",
	},
	{
		name 					= "parry_shield_metal_teleport",
		ui_name					= "Metal Parry Shield (Teleport)",
		categories				= "player",
		effect_desc				= "You hold a shield that destroys enemy projectiles and inflicts \nthe teleportation effect on their shooter.",
		effect_desc_short		= "You hold a shield that destroys enemy projectiles and inflicts the teleportation effect on their shooter.",
        precursor_types			= "gadget,magic_artifact",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			EntityAddChild(getPlayerEntity(), EntityLoad("mods/ArcaneReliquary/files/entity_definitions/others/parry_shield_metal_teleport_handler.xml"))
		end,
		ondrop_function			= function()
			local entity_children = EntityGetAllChildren(getPlayerEntity())
			if entity_children ~= nil then
				for i,child_id in ipairs(entity_children) do
					if(EntityGetName(child_id) == "parry_shield_metal_teleport_handler" and not EntityHasTag(child_id, "relic")) then
						local x, y = EntityGetTransform(child_id)
						EntityKill(child_id)
					end
				end
			end
		end,
		excludes_relics_desc	= "It can't be used along with other Parry Shield Relic variations",
		excludes_relics_tags	= "parry_shield_variation",
		combine_exclusive		= true,
		combine_requires_relics	= "parry_shield_metal,displacing_stone",
	},
	{
		name 					= "stone_hand_ornament_overheated",
		ui_name					= "Overheated Stone Hand Ornament",
		categories				= "projectiles,status_effects",
		effect_desc				= "Your spells have a 30% chance of creating a mystic target on \ncreatures you hit, making them summon overheated stone punch hands towards themselves.",
		effect_desc_short		= "Your spells create mystic targets on creatures, summoning overheated stone punch hands towards them.",
        precursor_types			= "magic_artifact,ring",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("stone_hand_ornament_overheated") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 30 + (5 * getRelicLevel("stone_hand_ornament_overheated"))
				if(chance >= 1 and chance <= odds) then
					attachLoadEffectOntoEntitiestoProjectile(projectile_entity_id, getCustomEffect("effect_projectile_target_punch_hand_overheated").path, "fire")
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "stone_hand_overheated,diamond_ring",
	},
	{
		name 					= "stone_hand_ornament_frozen",
		ui_name					= "Frozen Stone Hand Ornament",
		categories				= "projectiles,status_effects",
		effect_desc				= "Your spells have a 30% chance of creating a mystic target on \ncreatures you hit, making them summon frozen stone punch hands towards themselves.",
		effect_desc_short		= "Your spells create mystic targets on creatures, summoning frozen stone punch hands towards them.",
        precursor_types			= "magic_artifact,ring",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("stone_hand_ornament_frozen") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 30 + (5 * getRelicLevel("stone_hand_ornament_frozen"))
				if(chance >= 1 and chance <= odds) then
					attachLoadEffectOntoEntitiestoProjectile(projectile_entity_id, getCustomEffect("effect_projectile_target_punch_hand_frozen").path, "diamond")
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "stone_hand_frozen,diamond_ring",
	},
	{
		name 					= "stone_hand_ornament_charged",
		ui_name					= "Charged Stone Hand Ornament",
		categories				= "projectiles,status_effects",
		effect_desc				= "Your spells have a 30% chance of creating a mystic target on \ncreatures you hit, making them summon charged stone punch hands towards themselves.",
		effect_desc_short		= "Your spells create mystic targets on creatures, summoning charged stone punch hands towards them.",
        precursor_types			= "magic_artifact,ring",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("stone_hand_ornament_charged") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 30 + (5 * getRelicLevel("stone_hand_ornament_charged"))
				if(chance >= 1 and chance <= odds) then
					attachLoadEffectOntoEntitiestoProjectile(projectile_entity_id, getCustomEffect("effect_projectile_target_punch_hand_charged").path, "spark_blue_dark")
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "stone_hand_charged,diamond_ring",
	},
	{
		name 					= "stone_hand_ornament_humid",
		ui_name					= "Humid Stone Hand Ornament",
		categories				= "projectiles,status_effects",
		effect_desc				= "Your spells have a 30% chance of creating a mystic target on \ncreatures you hit, making them summon humid stone punch hands towards themselves.",
		effect_desc_short		= "Your spells create mystic targets on creatures, summoning humid stone punch hands towards them.",
        precursor_types			= "magic_artifact,ring",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("stone_hand_ornament_humid") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 30 + (5 * getRelicLevel("stone_hand_ornament_humid"))
				if(chance >= 1 and chance <= odds) then
					attachLoadEffectOntoEntitiestoProjectile(projectile_entity_id, getCustomEffect("effect_projectile_target_punch_hand_humid").path, "water")
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "stone_hand_humid,diamond_ring",
	},
	{
		name 					= "stone_hand_ornament_venomous",
		ui_name					= "Venomous Stone Hand Ornament",
		categories				= "projectiles,status_effects",
		effect_desc				= "Your spells have a 30% chance of creating a mystic target on \ncreatures you hit, making them summon venomous stone punch hands towards themselves.",
		effect_desc_short		= "Your spells create mystic targets on creatures, summoning venomous stone punch hands towards them.",
        precursor_types			= "magic_artifact,ring",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("stone_hand_ornament_venomous") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 30 + (5 * getRelicLevel("stone_hand_ornament_venomous"))
				if(chance >= 1 and chance <= odds) then
					attachLoadEffectOntoEntitiestoProjectile(projectile_entity_id, getCustomEffect("effect_projectile_target_punch_hand_venomous").path, "spark_purple_bright")
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "stone_hand_venomous,diamond_ring",
	},
	{
		name 					= "stone_hand_ornament_uric",
		ui_name					= "Uric Stone Hand Ornament",
		categories				= "projectiles,status_effects",
		effect_desc				= "Your spells have a 30% chance of creating a mystic target on \ncreatures you hit, making them summon uric stone punch hands towards themselves.",
		effect_desc_short		= "Your spells create mystic targets on creatures, summoning uric stone punch hands towards them.",
        precursor_types			= "magic_artifact,ring",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("stone_hand_ornament_uric") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 30 + (5 * getRelicLevel("stone_hand_ornament_uric"))
				if(chance >= 1 and chance <= odds) then
					attachLoadEffectOntoEntitiestoProjectile(projectile_entity_id, getCustomEffect("effect_projectile_target_punch_hand_uric").path, "spark_yellow")
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "stone_hand_uric,diamond_ring",
	},
	{
		name 					= "stone_hand_ornament_pungent",
		ui_name					= "Pungent Stone Hand Ornament",
		categories				= "projectiles,status_effects",
		effect_desc				= "Your spells have a 30% chance of creating a mystic target on \ncreatures you hit, making them summon pungent stone punch hands towards themselves.",
		effect_desc_short		= "Your spells create mystic targets on creatures, summoning pungent stone punch hands towards them.",
        precursor_types			= "magic_artifact,ring",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("stone_hand_ornament_pungent") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 30 + (5 * getRelicLevel("stone_hand_ornament_pungent"))
				if(chance >= 1 and chance <= odds) then
					attachLoadEffectOntoEntitiestoProjectile(projectile_entity_id, getCustomEffect("effect_projectile_target_punch_hand_pungent").path, "plasma_fading_green")
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "stone_hand_pungent,diamond_ring",
	},
	{
		name 					= "stone_hand_ornament_mutating",
		ui_name					= "Mutating Stone Hand Ornament",
		categories				= "projectiles,status_effects",
		effect_desc				= "Your spells have a 30% chance of creating a mystic target on \ncreatures you hit, making them summon mutating stone punch hands towards themselves.",
		effect_desc_short		= "Your spells create mystic targets on creatures, summoning mutating stone punch hands towards them.",
        precursor_types			= "magic_artifact,ring",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("stone_hand_ornament_mutating") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 30 + (5 * getRelicLevel("stone_hand_ornament_mutating"))
				if(chance >= 1 and chance <= odds) then
					attachLoadEffectOntoEntitiestoProjectile(projectile_entity_id, getCustomEffect("effect_projectile_target_punch_hand_mutating").path, "plasma_fading_pink")
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "stone_hand_mutating,diamond_ring",
	},
	{
		name 					= "stone_hand_ornament_gory",
		ui_name					= "Gory Stone Hand Ornament",
		categories				= "projectiles,status_effects",
		effect_desc				= "Your spells have a 30% chance of creating a mystic target on \ncreatures you hit, making them summon gory stone punch hands towards themselves.",
		effect_desc_short		= "Your spells create mystic targets on creatures, summoning gory stone punch hands towards them.",
        precursor_types			= "magic_artifact,ring",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("stone_hand_ornament_gory") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 30 + (5 * getRelicLevel("stone_hand_ornament_gory"))
				if(chance >= 1 and chance <= odds) then
					attachLoadEffectOntoEntitiestoProjectile(projectile_entity_id, getCustomEffect("effect_projectile_target_punch_hand_gory").path, "blood")
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "stone_hand_gory,diamond_ring",
	},
	{
		name 					= "stone_hand_ornament_toxic",
		ui_name					= "Toxic Stone Hand Ornament",
		categories				= "projectiles,status_effects",
		effect_desc				= "Your spells have a 30% chance of creating a mystic target on \ncreatures you hit, making them summon toxic stone punch hands towards themselves.",
		effect_desc_short		= "Your spells create mystic targets on creatures, summoning toxic stone punch hands towards them.",
        precursor_types			= "magic_artifact,ring",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("stone_hand_ornament_toxic") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 30 + (5 * getRelicLevel("stone_hand_ornament_toxic"))
				if(chance >= 1 and chance <= odds) then
					attachLoadEffectOntoEntitiestoProjectile(projectile_entity_id, getCustomEffect("effect_projectile_target_punch_hand_toxic").path, "radioactive_gas")
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "stone_hand_toxic,diamond_ring",
	},
	{
		name 					= "stone_hand_ornament_igneous",
		ui_name					= "Igneous Stone Hand Ornament",
		categories				= "projectiles,status_effects",
		effect_desc				= "Your spells have a 30% chance of creating a mystic target on \ncreatures you hit, making them summon igneous stone punch hands towards themselves.",
		effect_desc_short		= "Your spells create mystic targets on creatures, summoning igneous stone punch hands towards them.",
        precursor_types			= "magic_artifact,ring",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("stone_hand_ornament_igneous") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 30 + (5 * getRelicLevel("stone_hand_ornament_igneous"))
				if(chance >= 1 and chance <= odds) then
					attachLoadEffectOntoEntitiestoProjectile(projectile_entity_id, getCustomEffect("effect_projectile_target_punch_hand_igneous").path, "lava")
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "stone_hand_igneous,diamond_ring",
	},
	{
		name 					= "stone_hand_ornament_slimy",
		ui_name					= "Slimy Stone Hand Ornament",
		categories				= "projectiles,status_effects",
		effect_desc				= "Your spells have a 30% chance of creating a mystic target on \ncreatures you hit, making them summon slimy stone punch hands towards themselves.",
		effect_desc_short		= "Your spells create mystic targets on creatures, summoning slimy stone punch hands towards them.",
        precursor_types			= "magic_artifact,ring",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("stone_hand_ornament_slimy") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 30 + (5 * getRelicLevel("stone_hand_ornament_slimy"))
				if(chance >= 1 and chance <= odds) then
					attachLoadEffectOntoEntitiestoProjectile(projectile_entity_id, getCustomEffect("effect_projectile_target_punch_hand_slimy").path, "slime")
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "stone_hand_slimy,diamond_ring",
	},
	{
		name 					= "magic_conduit",
		ui_name					= "Magic Conduit",
		categories				= "projectiles,status_effects",
		effect_desc				= "Your spells have a 30% chance of creating a mystic target on \ncreatures you hit, making them summon duplicates of the last spell you cast towards themselves.",
		effect_desc_short		= "Your spells create mystic targets on creatures, summoning duplicates of the last spell you cast towards them.",
        precursor_types			= "magic_artifact,ring",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("magic_conduit") >= 1) then
				
				local projectile_definition = getProjectileDefinitionPath(projectile_entity_id)
				if(projectile_definition ~= nil) then
					if(not entityHasInternalVariableValue(getPlayerEntity(), "last_spell_cast_definition")) then
						addNewInternalVariable(getPlayerEntity(), "last_spell_cast_definition", "value_string", getProjectileDefinitionPath(projectile_entity_id))
					else
						setInternalVariableValue(getPlayerEntity(), "last_spell_cast_definition", "value_string", getProjectileDefinitionPath(projectile_entity_id))
					end
				else
					removeInternalVariable(getPlayerEntity(), "last_spell_cast_definition")
				end
				
			
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 30 + (5 * getRelicLevel("magic_conduit"))
				if(chance >= 1 and chance <= odds) then
					attachLoadEffectOntoEntitiestoProjectile(projectile_entity_id, getCustomEffect("effect_projectile_target_last_spell_cast").path, "material_rainbow")
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "feather_band,diamond_ring",
	},
	{
		name 					= "magic_conduit_charged",
		ui_name					= "Charged Magic Conduit",
		categories				= "projectiles,status_effects",
		effect_desc				= "Your spells have a 30% chance of creating a mystic target on \ncreatures you hit, making them summon duplicates of all spells from your current wand towards themselves.",
		effect_desc_short		= "Your spells create mystic targets on creatures, summoning duplicates of all spells from your current wand towards them.",
        precursor_types			= "magic_artifact,ring",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("magic_conduit_charged") >= 1) then
				
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 30 + (5 * getRelicLevel("magic_conduit_charged"))
				if(chance >= 1 and chance <= odds) then
					attachLoadEffectOntoEntitiestoProjectile(projectile_entity_id, getCustomEffect("effect_projectile_target_all_wand_spells").path, "material_rainbow")
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "magic_conduit,explosive_mana_crystal",
	},
	{
		name 					= "stone_hand_igneous",
		ui_name					= "Igneous Stone Hand",
		categories				= "companions",
		effect_desc				= "A pair of lava stone hands that launch themselves towards enemies aids you.",
		effect_desc_short		= "A pair of lava stone hands that launch themselves towards enemies aids you.",
        precursor_types			= "companion",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			
			attachMonkHandstoEntity(getPlayerEntity(), "monk_hand_igneous", "monk_arm_fx_igneous", "mortal")
		end,
		ondrop_function			= function()
			
			detachMonkHandsfromEntity(getPlayerEntity(), "monk_hand_igneous")
		end,
		overtime_function		= function()
			if(isRelicOwned("stone_hand_igneous") >= 1) then
				attachMonkHandstoEntity(getPlayerEntity(), "monk_hand_igneous", "monk_arm_fx_igneous", "mortal")
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "stone_hand_melee,lava_gem",
	},
	{
		name 					= "stone_hand_gory",
		ui_name					= "Gory Stone Hand",
		categories				= "companions",
		effect_desc				= "A pair of blood stone hands that launch themselves towards enemies aids you.",
		effect_desc_short		= "A pair of blood stone hands that launch themselves towards enemies aids you.",
        precursor_types			= "companion",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			
			attachMonkHandstoEntity(getPlayerEntity(), "monk_hand_gory", "monk_arm_fx_gory", "mortal")
		end,
		ondrop_function			= function()
			
			detachMonkHandsfromEntity(getPlayerEntity(), "monk_hand_gory")
		end,
		overtime_function		= function()
			if(isRelicOwned("stone_hand_gory") >= 1) then
				attachMonkHandstoEntity(getPlayerEntity(), "monk_hand_gory", "monk_arm_fx_gory", "mortal")
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "stone_hand_melee,corrosive_blood_gem",
	},
	{
		name 					= "stone_hand_toxic",
		ui_name					= "Toxic Stone Hand",
		categories				= "companions",
		effect_desc				= "A pair of toxic stone hands that launch themselves towards enemies aids you.",
		effect_desc_short		= "A pair of toxic stone hands that launch themselves towards enemies aids you.",
        precursor_types			= "companion",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			
			attachMonkHandstoEntity(getPlayerEntity(), "monk_hand_toxic", "monk_arm_fx_toxic", "mortal")
		end,
		ondrop_function			= function()
			
			detachMonkHandsfromEntity(getPlayerEntity(), "monk_hand_toxic")
		end,
		overtime_function		= function()
			if(isRelicOwned("stone_hand_toxic") >= 1) then
				attachMonkHandstoEntity(getPlayerEntity(), "monk_hand_toxic", "monk_arm_fx_toxic", "mortal")
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "stone_hand_melee,toxic_gem",
	},
	{
		name 					= "stone_hand_slimy",
		ui_name					= "Slimy Stone Hand",
		categories				= "companions",
		effect_desc				= "A pair of slimy stone hands that launch themselves towards enemies aids you.",
		effect_desc_short		= "A pair of slimy stone hands that launch themselves towards enemies aids you.",
        precursor_types			= "companion",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			
			attachMonkHandstoEntity(getPlayerEntity(), "monk_hand_slimy", "monk_arm_fx_slimy", "mortal")
		end,
		ondrop_function			= function()
			
			detachMonkHandsfromEntity(getPlayerEntity(), "monk_hand_slimy")
		end,
		overtime_function		= function()
			if(isRelicOwned("stone_hand_slimy") >= 1) then
				attachMonkHandstoEntity(getPlayerEntity(), "monk_hand_slimy", "monk_arm_fx_slimy", "mortal")
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "stone_hand_melee,slime_gem",
	},
	{
		name 					= "fireworks_acid",
		ui_name					= "Acid Fireworks",
		categories				= "projectiles,materials",
		effect_desc				= "When you hit an enemy with your spells, there's a 50% chance an acid fireworks rocket will \nbe attached to them, sending them upwards and exploding.",
		effect_desc_short		= "Enemies hit by your spells are attached with acid fireworks.",
        precursor_types			= "gadget",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onenemydamaged_function	= function(damage, message, entity_thats_responsible, is_fatal, projectile_thats_responsible )
			if(isRelicOwned("fireworks_acid") >= 1) then
				if(message == "$damage_projectile" and not is_fatal) then
					SetRandomSeed( GameGetFrameNum(), math.random(1,50))
					local chance = Random(1,100)
					local odds = 50 + (5 * getRelicLevel("fireworks_acid"))
					if(chance >= 1 and chance <= odds) then
						local enemy_x, enemy_y = EntityGetTransform(GetUpdatedEntityID())
						EntityLoad("mods/ArcaneReliquary/files/entity_definitions/custom_projectiles/fireworks_acid.xml", enemy_x, enemy_y)
					end
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "fireworks,acid_gem",
	},
	{
		name 					= "fireworks_water",
		ui_name					= "Water Fireworks",
		categories				= "projectiles,materials",
		effect_desc				= "When you hit an enemy with your spells, there's a 50% chance a water fireworks rocket will \nbe attached to them, sending them upwards and exploding.",
		effect_desc_short		= "Enemies hit by your spells are attached with water fireworks.",
        precursor_types			= "gadget",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onenemydamaged_function	= function(damage, message, entity_thats_responsible, is_fatal, projectile_thats_responsible )
			if(isRelicOwned("fireworks_water") >= 1) then
				if(message == "$damage_projectile" and not is_fatal) then
					SetRandomSeed( GameGetFrameNum(), math.random(1,50))
					local chance = Random(1,100)
					local odds = 50 + (5 * getRelicLevel("fireworks_water"))
					if(chance >= 1 and chance <= odds) then
						local enemy_x, enemy_y = EntityGetTransform(GetUpdatedEntityID())
						EntityLoad("mods/ArcaneReliquary/files/entity_definitions/custom_projectiles/fireworks_water.xml", enemy_x, enemy_y)
					end
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "fireworks,humid_gem",
	},
	{
		name 					= "fireworks_poison",
		ui_name					= "Poison Fireworks",
		categories				= "projectiles,materials",
		effect_desc				= "When you hit an enemy with your spells, there's a 50% chance a poison fireworks rocket will \nbe attached to them, sending them upwards and exploding.",
		effect_desc_short		= "Enemies hit by your spells are attached with poison fireworks.",
        precursor_types			= "gadget",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onenemydamaged_function	= function(damage, message, entity_thats_responsible, is_fatal, projectile_thats_responsible )
			if(isRelicOwned("fireworks_poison") >= 1) then
				if(message == "$damage_projectile" and not is_fatal) then
					SetRandomSeed( GameGetFrameNum(), math.random(1,50))
					local chance = Random(1,100)
					local odds = 50 + (5 * getRelicLevel("fireworks_poison"))
					if(chance >= 1 and chance <= odds) then
						local enemy_x, enemy_y = EntityGetTransform(GetUpdatedEntityID())
						EntityLoad("mods/ArcaneReliquary/files/entity_definitions/custom_projectiles/fireworks_poison.xml", enemy_x, enemy_y)
					end
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "fireworks,poison_gem",
	},
	{
		name 					= "fireworks_lava",
		ui_name					= "Lava Fireworks",
		categories				= "projectiles,materials",
		effect_desc				= "When you hit an enemy with your spells, there's a 50% chance a lava fireworks rocket will \nbe attached to them, sending them upwards and exploding.",
		effect_desc_short		= "Enemies hit by your spells are attached with lava fireworks.",
        precursor_types			= "gadget",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onenemydamaged_function	= function(damage, message, entity_thats_responsible, is_fatal, projectile_thats_responsible )
			if(isRelicOwned("fireworks_lava") >= 1) then
				if(message == "$damage_projectile" and not is_fatal) then
					SetRandomSeed( GameGetFrameNum(), math.random(1,50))
					local chance = Random(1,100)
					local odds = 50 + (5 * getRelicLevel("fireworks_lava"))
					if(chance >= 1 and chance <= odds) then
						local enemy_x, enemy_y = EntityGetTransform(GetUpdatedEntityID())
						EntityLoad("mods/ArcaneReliquary/files/entity_definitions/custom_projectiles/fireworks_lava.xml", enemy_x, enemy_y)
					end
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "fireworks,lava_gem",
	},
	{
		name 					= "fireworks_freezing_liquid",
		ui_name					= "Freezing Liquid Fireworks",
		categories				= "projectiles,materials",
		effect_desc				= "When you hit an enemy with your spells, there's a 50% chance a freezing liquid fireworks rocket will \nbe attached to them, sending them upwards and exploding.",
		effect_desc_short		= "Enemies hit by your spells are attached with freezing liquid fireworks.",
        precursor_types			= "gadget",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onenemydamaged_function	= function(damage, message, entity_thats_responsible, is_fatal, projectile_thats_responsible )
			if(isRelicOwned("fireworks_freezing_liquid") >= 1) then
				if(message == "$damage_projectile" and not is_fatal) then
					SetRandomSeed( GameGetFrameNum(), math.random(1,50))
					local chance = Random(1,100)
					local odds = 50 + (5 * getRelicLevel("fireworks_freezing_liquid"))
					if(chance >= 1 and chance <= odds) then
						local enemy_x, enemy_y = EntityGetTransform(GetUpdatedEntityID())
						EntityLoad("mods/ArcaneReliquary/files/entity_definitions/custom_projectiles/fireworks_freezing_liquid.xml", enemy_x, enemy_y)
					end
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "fireworks,freezing_liquid_gem",
	},
	{
		name 					= "fireworks_poly",
		ui_name					= "Poly Fireworks",
		categories				= "projectiles,materials",
		effect_desc				= "When you hit an enemy with your spells, there's a 50% chance a polymorphine fireworks rocket will \nbe attached to them, sending them upwards and exploding.",
		effect_desc_short		= "Enemies hit by your spells are attached with polymorphine fireworks.",
        precursor_types			= "gadget",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onenemydamaged_function	= function(damage, message, entity_thats_responsible, is_fatal, projectile_thats_responsible )
			if(isRelicOwned("fireworks_poly") >= 1) then
				if(message == "$damage_projectile" and not is_fatal) then
					SetRandomSeed( GameGetFrameNum(), math.random(1,50))
					local chance = Random(1,100)
					local odds = 50 + (5 * getRelicLevel("fireworks_poly"))
					if(chance >= 1 and chance <= odds) then
						local enemy_x, enemy_y = EntityGetTransform(GetUpdatedEntityID())
						EntityLoad("mods/ArcaneReliquary/files/entity_definitions/custom_projectiles/fireworks_poly.xml", enemy_x, enemy_y)
					end
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "fireworks,poly_gem",
	},
	{
		name 					= "fireworks_blood",
		ui_name					= "Blood Fireworks",
		categories				= "projectiles,materials",
		effect_desc				= "When you hit an enemy with your spells, there's a 50% chance a corrosive blood fireworks rocket will \nbe attached to them, sending them upwards and exploding.",
		effect_desc_short		= "Enemies hit by your spells are attached with corrosive blood fireworks.",
        precursor_types			= "gadget",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onenemydamaged_function	= function(damage, message, entity_thats_responsible, is_fatal, projectile_thats_responsible )
			if(isRelicOwned("fireworks_blood") >= 1) then
				if(message == "$damage_projectile" and not is_fatal) then
					SetRandomSeed( GameGetFrameNum(), math.random(1,50))
					local chance = Random(1,100)
					local odds = 50 + (5 * getRelicLevel("fireworks_blood"))
					if(chance >= 1 and chance <= odds) then
						local enemy_x, enemy_y = EntityGetTransform(GetUpdatedEntityID())
						EntityLoad("mods/ArcaneReliquary/files/entity_definitions/custom_projectiles/fireworks_blood.xml", enemy_x, enemy_y)
					end
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "fireworks,corrosive_blood_gem",
	},
	{
		name 					= "fireworks_slime",
		ui_name					= "Slime Fireworks",
		categories				= "projectiles,materials",
		effect_desc				= "When you hit an enemy with your spells, there's a 50% chance a slime fireworks rocket will \nbe attached to them, sending them upwards and exploding.",
		effect_desc_short		= "Enemies hit by your spells are attached with slime fireworks.",
        precursor_types			= "gadget",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onenemydamaged_function	= function(damage, message, entity_thats_responsible, is_fatal, projectile_thats_responsible )
			if(isRelicOwned("fireworks_slime") >= 1) then
				if(message == "$damage_projectile" and not is_fatal) then
					SetRandomSeed( GameGetFrameNum(), math.random(1,50))
					local chance = Random(1,100)
					local odds = 50 + (5 * getRelicLevel("fireworks_slime"))
					if(chance >= 1 and chance <= odds) then
						local enemy_x, enemy_y = EntityGetTransform(GetUpdatedEntityID())
						EntityLoad("mods/ArcaneReliquary/files/entity_definitions/custom_projectiles/fireworks_slime.xml", enemy_x, enemy_y)
					end
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "fireworks,slime_gem",
	},
	{
		name 					= "fireworks_toxic",
		ui_name					= "Toxic Fireworks",
		categories				= "projectiles,materials",
		effect_desc				= "When you hit an enemy with your spells, there's a 50% chance a toxic sludge fireworks rocket will \nbe attached to them, sending them upwards and exploding.",
		effect_desc_short		= "Enemies hit by your spells are attached with toxic sludge fireworks.",
        precursor_types			= "gadget",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onenemydamaged_function	= function(damage, message, entity_thats_responsible, is_fatal, projectile_thats_responsible )
			if(isRelicOwned("fireworks_toxic") >= 1) then
				if(message == "$damage_projectile" and not is_fatal) then
					SetRandomSeed( GameGetFrameNum(), math.random(1,50))
					local chance = Random(1,100)
					local odds = 50 + (5 * getRelicLevel("fireworks_toxic"))
					if(chance >= 1 and chance <= odds) then
						local enemy_x, enemy_y = EntityGetTransform(GetUpdatedEntityID())
						EntityLoad("mods/ArcaneReliquary/files/entity_definitions/custom_projectiles/fireworks_toxic.xml", enemy_x, enemy_y)
					end
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "fireworks,toxic_gem",
	},
	{
		name 					= "fireworks_urine",
		ui_name					= "Urine Fireworks",
		categories				= "projectiles,materials",
		effect_desc				= "When you hit an enemy with your spells, there's a 50% chance an urine fireworks rocket will \nbe attached to them, sending them upwards and exploding.",
		effect_desc_short		= "Enemies hit by your spells are attached with urine fireworks.",
        precursor_types			= "gadget",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onenemydamaged_function	= function(damage, message, entity_thats_responsible, is_fatal, projectile_thats_responsible )
			if(isRelicOwned("fireworks_urine") >= 1) then
				if(message == "$damage_projectile" and not is_fatal) then
					SetRandomSeed( GameGetFrameNum(), math.random(1,50))
					local chance = Random(1,100)
					local odds = 50 + (5 * getRelicLevel("fireworks_urine"))
					if(chance >= 1 and chance <= odds) then
						local enemy_x, enemy_y = EntityGetTransform(GetUpdatedEntityID())
						EntityLoad("mods/ArcaneReliquary/files/entity_definitions/custom_projectiles/fireworks_urine.xml", enemy_x, enemy_y)
					end
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "fireworks,urine_gem",
	},
	{
		name 					= "fireworks_oil",
		ui_name					= "Oil Fireworks",
		categories				= "projectiles,materials",
		effect_desc				= "When you hit an enemy with your spells, there's a 50% chance an oil fireworks rocket will \nbe attached to them, sending them upwards and exploding.",
		effect_desc_short		= "Enemies hit by your spells are attached with oil fireworks.",
        precursor_types			= "gadget",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onenemydamaged_function	= function(damage, message, entity_thats_responsible, is_fatal, projectile_thats_responsible )
			if(isRelicOwned("fireworks_oil") >= 1) then
				if(message == "$damage_projectile" and not is_fatal) then
					SetRandomSeed( GameGetFrameNum(), math.random(1,50))
					local chance = Random(1,100)
					local odds = 50 + (5 * getRelicLevel("fireworks_oil"))
					if(chance >= 1 and chance <= odds) then
						local enemy_x, enemy_y = EntityGetTransform(GetUpdatedEntityID())
						EntityLoad("mods/ArcaneReliquary/files/entity_definitions/custom_projectiles/fireworks_oil.xml", enemy_x, enemy_y)
					end
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "fireworks,oil_gem",
	},
	{
		name 					= "fireworks_alcohol",
		ui_name					= "Whiskey Fireworks",
		categories				= "projectiles,materials",
		effect_desc				= "When you hit an enemy with your spells, there's a 50% chance a whiskey fireworks rocket will \nbe attached to them, sending them upwards and exploding.",
		effect_desc_short		= "Enemies hit by your spells are attached with whiskey fireworks.",
        precursor_types			= "gadget",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onenemydamaged_function	= function(damage, message, entity_thats_responsible, is_fatal, projectile_thats_responsible )
			if(isRelicOwned("fireworks_alcohol") >= 1) then
				if(message == "$damage_projectile" and not is_fatal) then
					SetRandomSeed( GameGetFrameNum(), math.random(1,50))
					local chance = Random(1,100)
					local odds = 50 + (5 * getRelicLevel("fireworks_alcohol"))
					if(chance >= 1 and chance <= odds) then
						local enemy_x, enemy_y = EntityGetTransform(GetUpdatedEntityID())
						EntityLoad("mods/ArcaneReliquary/files/entity_definitions/custom_projectiles/fireworks_alcohol.xml", enemy_x, enemy_y)
					end
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "fireworks,alcohol_gem",
	},
	{
		name 					= "antipoly_gem",
		ui_name					= "Anti-Polymorphine Gem",
		categories				= "projectiles,materials",
		effect_desc				= "Spells you cast that explode on impact have a 20% chance of releasing anti-polymorphine on impact.",
		effect_desc_short		= "Your spells release anti-polymorphine.",
        precursor_types			= "elemental_gem",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("antipoly_gem") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 20 + (5 * getRelicLevel("antipoly_gem"))
				if(chance >= 1 and chance <= odds) then
					attachLoadEntityOnExplosiontoProjectile(projectile_entity_id, getLoadOnExplosion("launch_elemental_particles_antipoly").path, "anti_polymorphine")
				end
			end
		end,
	},
	{
		name 					= "materialcrackers_water",
		ui_name					= "Water Crackers",
		categories				= "projectiles",
		effect_desc				= "Your spells that explode on impact have a 30% chance of creating water pops on impact.",
		effect_desc_short		= "Your spells create water pops on impact.",
        precursor_types			= "magic_artifact",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("materialcrackers_water") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 30 + (5 * getRelicLevel("materialcrackers_water"))
				if(chance >= 1 and chance <= odds) then
					addBuffVisualIndicator_Projectile(projectile_entity_id, "water")
					attachLoadEntityOnExplosiontoProjectile(projectile_entity_id, "mods/ArcaneReliquary/files/entity_definitions/load_on_explosion/explosion_sequence_water_particles.xml")
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "firecrackers,humid_gem",
	},
	{
		name 					= "materialcrackers_acid",
		ui_name					= "Acid Crackers",
		categories				= "projectiles",
		effect_desc				= "Your spells that explode on impact have a 30% chance of creating acid pops on impact.",
		effect_desc_short		= "Your spells create acid pops on impact.",
        precursor_types			= "magic_artifact",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("materialcrackers_acid") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 30 + (5 * getRelicLevel("materialcrackers_acid"))
				if(chance >= 1 and chance <= odds) then
					addBuffVisualIndicator_Projectile(projectile_entity_id, "acid")
					attachLoadEntityOnExplosiontoProjectile(projectile_entity_id, "mods/ArcaneReliquary/files/entity_definitions/load_on_explosion/explosion_sequence_acid_particles.xml")
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "firecrackers,acid_gem",
	},
	{
		name 					= "materialcrackers_lava",
		ui_name					= "Lava Crackers",
		categories				= "projectiles",
		effect_desc				= "Your spells that explode on impact have a 30% chance of creating lava pops on impact.",
		effect_desc_short		= "Your spells create lava pops on impact.",
        precursor_types			= "magic_artifact",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("materialcrackers_lava") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 30 + (5 * getRelicLevel("materialcrackers_lava"))
				if(chance >= 1 and chance <= odds) then
					addBuffVisualIndicator_Projectile(projectile_entity_id, "acid")
					attachLoadEntityOnExplosiontoProjectile(projectile_entity_id, "mods/ArcaneReliquary/files/entity_definitions/load_on_explosion/explosion_sequence_lava_particles.xml")
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "firecrackers,lava_gem",
	},
	{
		name 					= "materialcrackers_poison",
		ui_name					= "Poison Crackers",
		categories				= "projectiles",
		effect_desc				= "Your spells that explode on impact have a 30% chance of creating poison pops on impact.",
		effect_desc_short		= "Your spells create poison pops on impact.",
        precursor_types			= "magic_artifact",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("materialcrackers_poison") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 30 + (5 * getRelicLevel("materialcrackers_poison"))
				if(chance >= 1 and chance <= odds) then
					addBuffVisualIndicator_Projectile(projectile_entity_id, "poison")
					attachLoadEntityOnExplosiontoProjectile(projectile_entity_id, "mods/ArcaneReliquary/files/entity_definitions/load_on_explosion/explosion_sequence_poison_particles.xml")
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "firecrackers,poison_gem",
	},
	{
		name 					= "materialcrackers_toxic_gas",
		ui_name					= "Toxic Gas Crackers",
		categories				= "projectiles",
		effect_desc				= "Your spells that explode on impact have a 30% chance of creating toxic gas pops on impact.",
		effect_desc_short		= "Your spells create toxic gas pops on impact.",
        precursor_types			= "magic_artifact",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("materialcrackers_toxic_gas") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 30 + (5 * getRelicLevel("materialcrackers_toxic_gas"))
				if(chance >= 1 and chance <= odds) then
					addBuffVisualIndicator_Projectile(projectile_entity_id, "radioactive_gas")
					attachLoadEntityOnExplosiontoProjectile(projectile_entity_id, "mods/ArcaneReliquary/files/entity_definitions/load_on_explosion/explosion_sequence_radioactive_gas_particles.xml")
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "firecrackers,canned_beans",
	},
	{
		name 					= "materialcrackers_toxic_sludge",
		ui_name					= "Toxic Sludge Crackers",
		categories				= "projectiles",
		effect_desc				= "Your spells that explode on impact have a 30% chance of creating toxic sludge pops on impact.",
		effect_desc_short		= "Your spells create toxic sludge pops on impact.",
        precursor_types			= "magic_artifact",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("materialcrackers_toxic_sludge") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 30 + (5 * getRelicLevel("materialcrackers_toxic_sludge"))
				if(chance >= 1 and chance <= odds) then
					addBuffVisualIndicator_Projectile(projectile_entity_id, "radioactive_liquid")
					attachLoadEntityOnExplosiontoProjectile(projectile_entity_id, "mods/ArcaneReliquary/files/entity_definitions/load_on_explosion/explosion_sequence_radioactive_liquid_particles.xml")
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "firecrackers,toxic_gem",
	},
	{
		name 					= "materialcrackers_poly",
		ui_name					= "Poly Crackers",
		categories				= "projectiles",
		effect_desc				= "Your spells that explode on impact have a 30% chance of creating polymorphine pops on impact.",
		effect_desc_short		= "Your spells create polymorphine pops on impact.",
        precursor_types			= "magic_artifact",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("materialcrackers_poly") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 30 + (5 * getRelicLevel("materialcrackers_poly"))
				if(chance >= 1 and chance <= odds) then
					addBuffVisualIndicator_Projectile(projectile_entity_id, "magic_liquid_polymorph")
					attachLoadEntityOnExplosiontoProjectile(projectile_entity_id, "mods/ArcaneReliquary/files/entity_definitions/load_on_explosion/explosion_sequence_poly_particles.xml")
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "firecrackers,poly_gem",
	},
	{
		name 					= "materialcrackers_antipoly",
		ui_name					= "Anti-Poly Crackers",
		categories				= "projectiles",
		effect_desc				= "Your spells that explode on impact have a 30% chance of creating anti-polymorphine pops on impact.",
		effect_desc_short		= "Your spells create anti-polymorphine pops on impact.",
        precursor_types			= "magic_artifact",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("materialcrackers_antipoly") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 30 + (5 * getRelicLevel("materialcrackers_antipoly"))
				if(chance >= 1 and chance <= odds) then
					addBuffVisualIndicator_Projectile(projectile_entity_id, "magic_liquid_polymorph")
					attachLoadEntityOnExplosiontoProjectile(projectile_entity_id, "mods/ArcaneReliquary/files/entity_definitions/load_on_explosion/explosion_sequence_antipoly_particles.xml")
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "firecrackers,antipoly_gem",
	},
	{
		name 					= "materialcrackers_slime",
		ui_name					= "Slime Crackers",
		categories				= "projectiles",
		effect_desc				= "Your spells that explode on impact have a 30% chance of creating slime pops on impact.",
		effect_desc_short		= "Your spells create slime pops on impact.",
        precursor_types			= "magic_artifact",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("materialcrackers_slime") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 30 + (5 * getRelicLevel("materialcrackers_slime"))
				if(chance >= 1 and chance <= odds) then
					addBuffVisualIndicator_Projectile(projectile_entity_id, "slime")
					attachLoadEntityOnExplosiontoProjectile(projectile_entity_id, "mods/ArcaneReliquary/files/entity_definitions/load_on_explosion/explosion_sequence_slime_particles.xml")
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "firecrackers,slime_gem",
	},
	{
		name 					= "materialcrackers_blood",
		ui_name					= "Blood Crackers",
		categories				= "projectiles",
		effect_desc				= "Your spells that explode on impact have a 30% chance of creating corrosive blood pops on impact.",
		effect_desc_short		= "Your spells create corrosive blood pops on impact.",
        precursor_types			= "magic_artifact",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("materialcrackers_blood") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 30 + (5 * getRelicLevel("materialcrackers_blood"))
				if(chance >= 1 and chance <= odds) then
					addBuffVisualIndicator_Projectile(projectile_entity_id, "blood")
					attachLoadEntityOnExplosiontoProjectile(projectile_entity_id, "mods/ArcaneReliquary/files/entity_definitions/load_on_explosion/explosion_sequence_corrosive_blood_particles.xml")
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "firecrackers,corrosive_blood_gem",
	},
	{
		name 					= "materialcrackers_freezing_liquid",
		ui_name					= "Freezing Liquid Crackers",
		categories				= "projectiles",
		effect_desc				= "Your spells that explode on impact have a 30% chance of creating freezing liquid pops on impact.",
		effect_desc_short		= "Your spells create freezing liquid pops on impact.",
        precursor_types			= "magic_artifact",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("materialcrackers_freezing_liquid") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 30 + (5 * getRelicLevel("materialcrackers_freezing_liquid"))
				if(chance >= 1 and chance <= odds) then
					addBuffVisualIndicator_Projectile(projectile_entity_id, "blood_cold_vapour")
					attachLoadEntityOnExplosiontoProjectile(projectile_entity_id, "mods/ArcaneReliquary/files/entity_definitions/load_on_explosion/explosion_sequence_freezing_liquid_particles.xml")
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "firecrackers,freezing_liquid_gem",
	},
	{
		name 					= "materialcrackers_alcohol",
		ui_name					= "Whiskey Crackers",
		categories				= "projectiles",
		effect_desc				= "Your spells that explode on impact have a 30% chance of creating whiskey pops on impact.",
		effect_desc_short		= "Your spells create whiskey pops on impact.",
        precursor_types			= "magic_artifact",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("materialcrackers_alcohol") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 30 + (5 * getRelicLevel("materialcrackers_alcohol"))
				if(chance >= 1 and chance <= odds) then
					addBuffVisualIndicator_Projectile(projectile_entity_id, "alcohol")
					attachLoadEntityOnExplosiontoProjectile(projectile_entity_id, "mods/ArcaneReliquary/files/entity_definitions/load_on_explosion/explosion_sequence_alcohol_particles.xml")
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "firecrackers,alcohol_gem",
	},
	{
		name 					= "materialcrackers_oil",
		ui_name					= "Oil Crackers",
		categories				= "projectiles",
		effect_desc				= "Your spells that explode on impact have a 30% chance of creating oil pops on impact.",
		effect_desc_short		= "Your spells create oil pops on impact.",
        precursor_types			= "magic_artifact",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("materialcrackers_oil") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 30 + (5 * getRelicLevel("materialcrackers_oil"))
				if(chance >= 1 and chance <= odds) then
					addBuffVisualIndicator_Projectile(projectile_entity_id, "oil")
					attachLoadEntityOnExplosiontoProjectile(projectile_entity_id, "mods/ArcaneReliquary/files/entity_definitions/load_on_explosion/explosion_sequence_oil_particles.xml")
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "firecrackers,oil_gem",
	},
	{
		name 					= "materialcrackers_urine",
		ui_name					= "Urine Crackers",
		categories				= "projectiles",
		effect_desc				= "Your spells that explode on impact have a 30% chance of creating urine pops on impact.",
		effect_desc_short		= "Your spells create urine pops on impact.",
        precursor_types			= "magic_artifact",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5%.",
		is_upgradable 			= true,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("materialcrackers_urine") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 30 + (5 * getRelicLevel("materialcrackers_urine"))
				if(chance >= 1 and chance <= odds) then
					addBuffVisualIndicator_Projectile(projectile_entity_id, "urine")
					attachLoadEntityOnExplosiontoProjectile(projectile_entity_id, "mods/ArcaneReliquary/files/entity_definitions/load_on_explosion/explosion_sequence_urine_particles.xml")
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "firecrackers,urine_gem",
	},
	{
		name 					= "wand_improve_essence_mana",
		ui_name					= "Mana Essence",
		categories				= "enemies,items",
		effect_desc				= "Enemies you kill have a 20% chance of dropping blue nuggets that permanently \nupgrade the maximum mana capacity of your current wand.",
		effect_desc_short		= "Killed enemies drop mana increasing nuggets.",
        precursor_types			= "flask",
		upgrade_effect_desc		= "For each upgrade level, drop chance increases by 5%.",
		is_upgradable 			= true,
		onenemykill_function	= function(damage_type_bit_field, damage_message, entity_thats_responsible, drop_items)
			if(isRelicOwned("wand_improve_essence_mana") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 20 + (5 * getRelicLevel("wand_improve_essence_mana"))
				if(chance >= 1 and chance <= odds) then
					local death_x, death_y = EntityGetTransform(GetUpdatedEntityID())
					EntityLoad("mods/ArcaneReliquary/files/entity_definitions/items/wandimprovenugget_mana.xml", death_x, death_y)
				end
			end
		end,
	},
	{
		name 					= "wand_improve_essence_reload_time",
		ui_name					= "Reload Time Essence",
		categories				= "enemies,items",
		effect_desc				= "Enemies you kill have a 20% chance of dropping blue nuggets that permanently \ndecrease the reload time of your current wand.",
		effect_desc_short		= "Killed enemies drop reload time decreasing nuggets.",
        precursor_types			= "flask",
		upgrade_effect_desc		= "For each upgrade level, drop chance increases by 5%.",
		is_upgradable 			= true,
		onenemykill_function	= function(damage_type_bit_field, damage_message, entity_thats_responsible, drop_items)
			if(isRelicOwned("wand_improve_essence_reload_time") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 20 + (5 * getRelicLevel("wand_improve_essence_reload_time"))
				if(chance >= 1 and chance <= odds) then
					local death_x, death_y = EntityGetTransform(GetUpdatedEntityID())
					EntityLoad("mods/ArcaneReliquary/files/entity_definitions/items/wandimprovenugget_reloadtime.xml", death_x, death_y)
				end
			end
		end,
	},
	{
		name 					= "wand_improve_essence_cast_delay",
		ui_name					= "Cast Delay Essence",
		categories				= "enemies,items",
		effect_desc				= "Enemies you kill have a 20% chance of dropping blue nuggets that permanently \ndecrease the cast delay of your current wand.",
		effect_desc_short		= "Killed enemies drop cast delay decreasing nuggets.",
        precursor_types			= "flask",
		upgrade_effect_desc		= "For each upgrade level, drop chance increases by 5%.",
		is_upgradable 			= true,
		onenemykill_function	= function(damage_type_bit_field, damage_message, entity_thats_responsible, drop_items)
			if(isRelicOwned("wand_improve_essence_cast_delay") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 20 + (5 * getRelicLevel("wand_improve_essence_cast_delay"))
				if(chance >= 1 and chance <= odds) then
					local death_x, death_y = EntityGetTransform(GetUpdatedEntityID())
					EntityLoad("mods/ArcaneReliquary/files/entity_definitions/items/wandimprovenugget_castdelay.xml", death_x, death_y)
				end
			end
		end,
	},
	{
		name 					= "risk_reward_essence_pink",
		ui_name					= "Pink Risk Reward Essence",
		categories				= "enemies,items",
		effect_desc				= "Enemies you kill have a 30% chance of dropping pink nuggets that yield 80 hp \nonce picked, but when you pick them up there is a chance \nyou'll be inflicted with the polymorphine spit effect.",
		effect_desc_short		= "Killed enemies drop pink nuggets that yield 80 hp once picked but you might be inflicted with a negative effect.",
        precursor_types			= "flask",
		upgrade_effect_desc		= "For each upgrade level, drop chance increases by 5%.",
		is_upgradable 			= true,
		onenemykill_function	= function(damage_type_bit_field, damage_message, entity_thats_responsible, drop_items)
			if(isRelicOwned("risk_reward_essence_pink") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 30 + (5 * getRelicLevel("risk_reward_essence_pink"))
				if(chance >= 1 and chance <= odds) then
					local death_x, death_y = EntityGetTransform(GetUpdatedEntityID())
					EntityLoad("mods/ArcaneReliquary/files/entity_definitions/items/riskrewardnugget_pink.xml", death_x, death_y)
				end
			end
		end,
	},
	{
		name 					= "risk_reward_essence_orange",
		ui_name					= "Orange Risk Reward Essence",
		categories				= "enemies,items",
		effect_desc				= "Enemies you kill have a 30% chance of dropping orange nuggets that yield 600 gold \nonce picked, but when you pick them up there is a chance \nyou'll be inflicted with the lava spit effect.",
		effect_desc_short		= "Killed enemies drop orange nuggets that yield 600 gold once picked but you might be inflicted with a negative effect.",
        precursor_types			= "flask",
		upgrade_effect_desc		= "For each upgrade level, drop chance increases by 5%.",
		is_upgradable 			= true,
		onenemykill_function	= function(damage_type_bit_field, damage_message, entity_thats_responsible, drop_items)
			if(isRelicOwned("risk_reward_essence_orange") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 30 + (5 * getRelicLevel("risk_reward_essence_orange"))
				if(chance >= 1 and chance <= odds) then
					local death_x, death_y = EntityGetTransform(GetUpdatedEntityID())
					EntityLoad("mods/ArcaneReliquary/files/entity_definitions/items/riskrewardnugget_orange.xml", death_x, death_y)
				end
			end
		end,
	},
	{
		name 					= "risk_reward_essence_purple",
		ui_name					= "Purple Risk Reward Essence",
		categories				= "enemies,items",
		effect_desc				= "Enemies you kill have a 30% chance of dropping purple nuggets that yield 300 gold and 40 hp \nonce picked, but when you pick them up there is a chance \nyou'll be inflicted with the poison spit effect.",
		effect_desc_short		= "Killed enemies drop purple nuggets that yield 300 gold and 40 hp once picked but you might be inflicted with a negative effect.",
        precursor_types			= "flask",
		upgrade_effect_desc		= "For each upgrade level, drop chance increases by 5%.",
		is_upgradable 			= true,
		onenemykill_function	= function(damage_type_bit_field, damage_message, entity_thats_responsible, drop_items)
			if(isRelicOwned("risk_reward_essence_purple") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 30 + (5 * getRelicLevel("risk_reward_essence_purple"))
				if(chance >= 1 and chance <= odds) then
					local death_x, death_y = EntityGetTransform(GetUpdatedEntityID())
					EntityLoad("mods/ArcaneReliquary/files/entity_definitions/items/riskrewardnugget_purple.xml", death_x, death_y)
				end
			end
		end,
	},
	{
		name 					= "risk_reward_essence_gray",
		ui_name					= "Gray Risk Reward Essence",
		categories				= "enemies,items",
		effect_desc				= "Enemies you kill have a 30% chance of dropping gray nuggets that yield 2000 gold \nonce picked, but when you pick them up you'll be immediately \ninflicted with the bomb laying effect.",
		effect_desc_short		= "Killed enemies drop gray nuggets that yield 2000 gold once picked but you are inflicted with the bomb laying effect.",
        precursor_types			= "flask",
		upgrade_effect_desc		= "For each upgrade level, drop chance increases by 5%.",
		is_upgradable 			= true,
		onenemykill_function	= function(damage_type_bit_field, damage_message, entity_thats_responsible, drop_items)
			if(isRelicOwned("risk_reward_essence_gray") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 30 + (5 * getRelicLevel("risk_reward_essence_gray"))
				if(chance >= 1 and chance <= odds) then
					local death_x, death_y = EntityGetTransform(GetUpdatedEntityID())
					EntityLoad("mods/ArcaneReliquary/files/entity_definitions/items/riskrewardnugget_gray.xml", death_x, death_y)
				end
			end
		end,
	},
	{
		name 					= "risk_reward_essence_lime",
		ui_name					= "Lime Risk Reward Essence",
		categories				= "enemies,items",
		effect_desc				= "Enemies you kill have a 30% chance of dropping lime nuggets that yield 120 hp \nonce picked, but when you pick them up you might trigger a sequence of nuclear explosions.",
		effect_desc_short		= "Killed enemies drop lime nuggets that yield 100 hp once picked but you might trigger nuclear explosions.",
        precursor_types			= "flask",
		upgrade_effect_desc		= "For each upgrade level, drop chance increases by 5%.",
		is_upgradable 			= true,
		onenemykill_function	= function(damage_type_bit_field, damage_message, entity_thats_responsible, drop_items)
			if(isRelicOwned("risk_reward_essence_lime") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 30 + (5 * getRelicLevel("risk_reward_essence_lime"))
				if(chance >= 1 and chance <= odds) then
					local death_x, death_y = EntityGetTransform(GetUpdatedEntityID())
					EntityLoad("mods/ArcaneReliquary/files/entity_definitions/items/riskrewardnugget_lime.xml", death_x, death_y)
				end
			end
		end,
	},
	{
		name 					= "risk_reward_essence_red",
		ui_name					= "Red Risk Reward Essence",
		categories				= "enemies,items",
		effect_desc				= "Enemies you kill have a 30% chance of dropping red nuggets that yield 25 hp \nonce picked, extending your maximum hp if it's \nfull, but when you pick them up there is a chance \nyou'll be inflicted with the heavy bleeding effect.",
		effect_desc_short		= "Killed enemies drop red nuggets that yield 25 hp once picked (extending your max hp up if it's full) but you might be inflicted with a negative effect.",
        precursor_types			= "flask",
		upgrade_effect_desc		= "For each upgrade level, drop chance increases by 5%.",
		is_upgradable 			= true,
		onenemykill_function	= function(damage_type_bit_field, damage_message, entity_thats_responsible, drop_items)
			if(isRelicOwned("risk_reward_essence_red") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 30 + (5 * getRelicLevel("risk_reward_essence_red"))
				if(chance >= 1 and chance <= odds) then
					local death_x, death_y = EntityGetTransform(GetUpdatedEntityID())
					EntityLoad("mods/ArcaneReliquary/files/entity_definitions/items/riskrewardnugget_red.xml", death_x, death_y)
				end
			end
		end,
	},
	{
		name 					= "elemental_mask_urine",
		ui_name					= "Elemental Mask of Emiction",
		categories				= "projectiles,status_effects",
		effect_desc				= "Enemies hit by your spells have a 50% chance of being inflicted with an effect \nthat transforms all liquids around them into urine, but if \nyou take damage from them, this effect is applied to you.",
		effect_desc_short		= "Enemies you hit with spells are inflicted with effect that converts all liquids around them into urine.",
        precursor_types			= "magic_artifact",
		upgrade_effect_desc		= "For each upgrade level, effect chance on enemies increases by 5%.",
		is_upgradable 			= true,
		onenemydamaged_function	= function(damage, message, entity_thats_responsible, is_fatal, projectile_thats_responsible )
			if(isRelicOwned("elemental_mask_urine") >= 1) then
				if(message == "$damage_projectile") then
					SetRandomSeed( GameGetFrameNum(), math.random(1,50))
					local chance = Random(1,100)
					local odds = 50 + (5 * getRelicLevel("elemental_mask_urine"))
					if(chance >= 1 and chance <= odds) then
						LoadGameEffectEntityTo( GetUpdatedEntityID(), getCustomEffect("effect_field_liquids_to_urine").path )
					end
				end
			end
		end,
		ondamaged_function		= function(damage, message, entity_thats_responsible, is_fatal, projectile_thats_responsible )
			if(isRelicOwned("elemental_mask_urine") >= 1) then
				if(EntityHasTag(entity_thats_responsible, "enemy")) then
					LoadGameEffectEntityTo( GetUpdatedEntityID(), getCustomEffect("effect_field_liquids_to_urine").path )
				end
			end
		end,
	},
	{
		name 					= "elemental_mask_radioactive_gas",
		ui_name					= "Elemental Mask of Radioactivity",
		categories				= "projectiles,status_effects",
		effect_desc				= "Enemies hit by your spells have a 50% chance of being inflicted with an effect \nthat transforms all liquids around them into toxic gas, but if \nyou take damage from them, this effect is applied to you.",
		effect_desc_short		= "Enemies you hit with spells are inflicted with effect that converts all liquids around them into toxic gas.",
        precursor_types			= "magic_artifact",
		upgrade_effect_desc		= "For each upgrade level, effect chance on enemies increases by 5%.",
		is_upgradable 			= true,
		onenemydamaged_function	= function(damage, message, entity_thats_responsible, is_fatal, projectile_thats_responsible )
			if(isRelicOwned("elemental_mask_radioactive_gas") >= 1) then
				if(message == "$damage_projectile") then
					SetRandomSeed( GameGetFrameNum(), math.random(1,50))
					local chance = Random(1,100)
					local odds = 50 + (5 * getRelicLevel("elemental_mask_radioactive_gas"))
					if(chance >= 1 and chance <= odds) then
						LoadGameEffectEntityTo( GetUpdatedEntityID(), getCustomEffect("effect_field_liquids_to_radioactive_gas").path )
					end
				end
			end
		end,
		ondamaged_function		= function(damage, message, entity_thats_responsible, is_fatal, projectile_thats_responsible )
			if(isRelicOwned("elemental_mask_radioactive_gas") >= 1) then
				if(EntityHasTag(entity_thats_responsible, "enemy")) then
					LoadGameEffectEntityTo( GetUpdatedEntityID(), getCustomEffect("effect_field_liquids_to_radioactive_gas").path )
				end
			end
		end,
	},
	{
		name 					= "elemental_mask_poly",
		ui_name					= "Elemental Mask of Mutation",
		categories				= "projectiles,status_effects",
		effect_desc				= "Enemies hit by your spells have a 50% chance of being inflicted with an effect \nthat transforms all liquids around them into polymorphine, but if \nyou take damage from them, this effect is applied to you.",
		effect_desc_short		= "Enemies you hit with spells are inflicted with effect that converts all liquids around them into polymorphine.",
        precursor_types			= "magic_artifact",
		upgrade_effect_desc		= "For each upgrade level, effect chance on enemies increases by 5%.",
		is_upgradable 			= true,
		onenemydamaged_function	= function(damage, message, entity_thats_responsible, is_fatal, projectile_thats_responsible )
			if(isRelicOwned("elemental_mask_poly") >= 1) then
				if(message == "$damage_projectile") then
					SetRandomSeed( GameGetFrameNum(), math.random(1,50))
					local chance = Random(1,100)
					local odds = 50 + (5 * getRelicLevel("elemental_mask_poly"))
					if(chance >= 1 and chance <= odds) then
						LoadGameEffectEntityTo( GetUpdatedEntityID(), getCustomEffect("effect_field_liquids_to_polymorphine").path )
					end
				end
			end
		end,
		ondamaged_function		= function(damage, message, entity_thats_responsible, is_fatal, projectile_thats_responsible )
			if(isRelicOwned("elemental_mask_poly") >= 1) then
				if(EntityHasTag(entity_thats_responsible, "enemy")) then
					LoadGameEffectEntityTo( GetUpdatedEntityID(), getCustomEffect("effect_field_liquids_to_polymorphine").path )
				end
			end
		end,
	},
	{
		name 					= "elemental_mask_poison",
		ui_name					= "Elemental Mask of Venom",
		categories				= "projectiles,status_effects",
		effect_desc				= "Enemies hit by your spells have a 50% chance of being inflicted with an effect \nthat transforms all liquids around them into poison, but if \nyou take damage from them, this effect is applied to you.",
		effect_desc_short		= "Enemies you hit with spells are inflicted with effect that converts all liquids around them into poison.",
        precursor_types			= "magic_artifact",
		upgrade_effect_desc		= "For each upgrade level, effect chance on enemies increases by 5%.",
		is_upgradable 			= true,
		onenemydamaged_function	= function(damage, message, entity_thats_responsible, is_fatal, projectile_thats_responsible )
			if(isRelicOwned("elemental_mask_poison") >= 1) then
				if(message == "$damage_projectile") then
					SetRandomSeed( GameGetFrameNum(), math.random(1,50))
					local chance = Random(1,100)
					local odds = 50 + (5 * getRelicLevel("elemental_mask_poison"))
					if(chance >= 1 and chance <= odds) then
						LoadGameEffectEntityTo( GetUpdatedEntityID(), getCustomEffect("effect_field_liquids_to_poison").path )
					end
				end
			end
		end,
		ondamaged_function		= function(damage, message, entity_thats_responsible, is_fatal, projectile_thats_responsible )
			if(isRelicOwned("elemental_mask_poison") >= 1) then
				if(EntityHasTag(entity_thats_responsible, "enemy")) then
					LoadGameEffectEntityTo( GetUpdatedEntityID(), getCustomEffect("effect_field_liquids_to_poison").path )
				end
			end
		end,
	},
	{
		name 					= "elemental_mask_lava",
		ui_name					= "Elemental Mask of Ashes",
		categories				= "projectiles,status_effects",
		effect_desc				= "Enemies hit by your spells have a 50% chance of being inflicted with an effect \nthat transforms all liquids around them into lava, but if \nyou take damage from them, this effect is applied to you.",
		effect_desc_short		= "Enemies you hit with spells are inflicted with effect that converts all liquids around them into lava.",
        precursor_types			= "magic_artifact",
		upgrade_effect_desc		= "For each upgrade level, effect chance on enemies increases by 5%.",
		is_upgradable 			= true,
		onenemydamaged_function	= function(damage, message, entity_thats_responsible, is_fatal, projectile_thats_responsible )
			if(isRelicOwned("elemental_mask_lava") >= 1) then
				if(message == "$damage_projectile") then
					SetRandomSeed( GameGetFrameNum(), math.random(1,50))
					local chance = Random(1,100)
					local odds = 50 + (5 * getRelicLevel("elemental_mask_lava"))
					if(chance >= 1 and chance <= odds) then
						LoadGameEffectEntityTo( GetUpdatedEntityID(), getCustomEffect("effect_field_liquids_to_lava").path )
					end
				end
			end
		end,
		ondamaged_function		= function(damage, message, entity_thats_responsible, is_fatal, projectile_thats_responsible )
			if(isRelicOwned("elemental_mask_lava") >= 1) then
				if(EntityHasTag(entity_thats_responsible, "enemy")) then
					LoadGameEffectEntityTo( GetUpdatedEntityID(), getCustomEffect("effect_field_liquids_to_lava").path )
				end
			end
		end,
	},
	{
		name 					= "elemental_mask_freezing_liquid",
		ui_name					= "Elemental Mask of Blizzard",
		categories				= "projectiles,status_effects",
		effect_desc				= "Enemies hit by your spells have a 50% chance of being inflicted with an effect \nthat transforms all liquids around them into freezing liquid, but if \nyou take damage from them, this effect is applied to you.",
		effect_desc_short		= "Enemies you hit with spells are inflicted with effect that converts all liquids around them into freezing liquid.",
        precursor_types			= "magic_artifact",
		upgrade_effect_desc		= "For each upgrade level, effect chance on enemies increases by 5%.",
		is_upgradable 			= true,
		onenemydamaged_function	= function(damage, message, entity_thats_responsible, is_fatal, projectile_thats_responsible )
			if(isRelicOwned("elemental_mask_freezing_liquid") >= 1) then
				if(message == "$damage_projectile") then
					SetRandomSeed( GameGetFrameNum(), math.random(1,50))
					local chance = Random(1,100)
					local odds = 50 + (5 * getRelicLevel("elemental_mask_freezing_liquid"))
					if(chance >= 1 and chance <= odds) then
						LoadGameEffectEntityTo( GetUpdatedEntityID(), getCustomEffect("effect_field_liquids_to_freezing_liquid").path )
					end
				end
			end
		end,
		ondamaged_function		= function(damage, message, entity_thats_responsible, is_fatal, projectile_thats_responsible )
			if(isRelicOwned("elemental_mask_freezing_liquid") >= 1) then
				if(EntityHasTag(entity_thats_responsible, "enemy")) then
					LoadGameEffectEntityTo( GetUpdatedEntityID(), getCustomEffect("effect_field_liquids_to_freezing_liquid").path )
				end
			end
		end,
	},
	{
		name 					= "elemental_mask_fire",
		ui_name					= "Elemental Mask of Flames",
		categories				= "projectiles,status_effects",
		effect_desc				= "Enemies hit by your spells have a 50% chance of being inflicted with an effect \nthat transforms all liquids around them into fire, but if \nyou take damage from them, this effect is applied to you.",
		effect_desc_short		= "Enemies you hit with spells are inflicted with effect that converts all liquids around them into fire.",
        precursor_types			= "magic_artifact",
		upgrade_effect_desc		= "For each upgrade level, effect chance on enemies increases by 5%.",
		is_upgradable 			= true,
		onenemydamaged_function	= function(damage, message, entity_thats_responsible, is_fatal, projectile_thats_responsible )
			if(isRelicOwned("elemental_mask_fire") >= 1) then
				if(message == "$damage_projectile") then
					SetRandomSeed( GameGetFrameNum(), math.random(1,50))
					local chance = Random(1,100)
					local odds = 50 + (5 * getRelicLevel("elemental_mask_fire"))
					if(chance >= 1 and chance <= odds) then
						LoadGameEffectEntityTo( GetUpdatedEntityID(), getCustomEffect("effect_field_liquids_to_fire").path )
					end
				end
			end
		end,
		ondamaged_function		= function(damage, message, entity_thats_responsible, is_fatal, projectile_thats_responsible )
			if(isRelicOwned("elemental_mask_fire") >= 1) then
				if(EntityHasTag(entity_thats_responsible, "enemy")) then
					LoadGameEffectEntityTo( GetUpdatedEntityID(), getCustomEffect("effect_field_liquids_to_fire").path )
				end
			end
		end,
	},
	{
		name 					= "elemental_mask_acid",
		ui_name					= "Elemental Mask of Corrosion",
		categories				= "projectiles,status_effects",
		effect_desc				= "Enemies hit by your spells have a 50% chance of being inflicted with an effect \nthat transforms all liquids around them into acid, but if \nyou take damage from them, this effect is applied to you.",
		effect_desc_short		= "Enemies you hit with spells are inflicted with effect that converts all liquids around them into acid.",
        precursor_types			= "magic_artifact",
		upgrade_effect_desc		= "For each upgrade level, effect chance on enemies increases by 5%.",
		is_upgradable 			= true,
		onenemydamaged_function	= function(damage, message, entity_thats_responsible, is_fatal, projectile_thats_responsible )
			if(isRelicOwned("elemental_mask_acid") >= 1) then
				if(message == "$damage_projectile") then
					SetRandomSeed( GameGetFrameNum(), math.random(1,50))
					local chance = Random(1,100)
					local odds = 50 + (5 * getRelicLevel("elemental_mask_acid"))
					if(chance >= 1 and chance <= odds) then
						LoadGameEffectEntityTo( GetUpdatedEntityID(), getCustomEffect("effect_field_liquids_to_acid").path )
					end
				end
			end
		end,
		ondamaged_function		= function(damage, message, entity_thats_responsible, is_fatal, projectile_thats_responsible )
			if(isRelicOwned("elemental_mask_acid") >= 1) then
				if(EntityHasTag(entity_thats_responsible, "enemy")) then
					LoadGameEffectEntityTo( GetUpdatedEntityID(), getCustomEffect("effect_field_liquids_to_acid").path )
				end
			end
		end,
	},
	{
		name 					= "elemental_mask_teleportatium",
		ui_name					= "Elemental Mask of Teleportation",
		categories				= "projectiles,status_effects",
		effect_desc				= "Enemies hit by your spells have a 50% chance of being inflicted with an effect \nthat transforms all liquids around them into teleportatium, but if \nyou take damage from them, this effect is applied to you.",
		effect_desc_short		= "Enemies you hit with spells are inflicted with effect that converts all liquids around them into teleportatium.",
        precursor_types			= "magic_artifact",
		upgrade_effect_desc		= "For each upgrade level, effect chance on enemies increases by 5%.",
		is_upgradable 			= true,
		onenemydamaged_function	= function(damage, message, entity_thats_responsible, is_fatal, projectile_thats_responsible )
			if(isRelicOwned("elemental_mask_teleportatium") >= 1) then
				if(message == "$damage_projectile") then
					SetRandomSeed( GameGetFrameNum(), math.random(1,50))
					local chance = Random(1,100)
					local odds = 50 + (5 * getRelicLevel("elemental_mask_teleportatium"))
					if(chance >= 1 and chance <= odds) then
						LoadGameEffectEntityTo( GetUpdatedEntityID(), getCustomEffect("effect_field_liquids_to_teleportatium").path )
					end
				end
			end
		end,
		ondamaged_function		= function(damage, message, entity_thats_responsible, is_fatal, projectile_thats_responsible )
			if(isRelicOwned("elemental_mask_teleportatium") >= 1) then
				if(EntityHasTag(entity_thats_responsible, "enemy")) then
					LoadGameEffectEntityTo( GetUpdatedEntityID(), getCustomEffect("effect_field_liquids_to_teleportatium").path )
				end
			end
		end,
	},
	{
		name 					= "mystic_cube",
		ui_name					= "Mystic Cube",
		categories				= "projectiles,status_effects",
		effect_desc				= "Enemies hit by your spells have a 50% chance of being trapped in a mystic cube.",
		effect_desc_short		= "Enemies you hit with spells are trapped in a mystic cube.",
        precursor_types			= "magic_artifact",
		upgrade_effect_desc		= "For each upgrade level, effect chance on enemies increases by 5%.",
		is_upgradable 			= true,
		onenemydamaged_function	= function(damage, message, entity_thats_responsible, is_fatal, projectile_thats_responsible )
			if(isRelicOwned("mystic_cube") >= 1) then
				if(message == "$damage_projectile" and not entityHasSimplifiedInternalVariable(projectile_thats_responsible, "relic_effect_projectile")) then
					SetRandomSeed( GameGetFrameNum(), math.random(1,50))
					local chance = Random(1,1)
					local odds = 50 + (5 * getRelicLevel("mystic_cube"))
					if(chance >= 1 and chance <= odds) then
						local x, y = EntityGetTransform(GetUpdatedEntityID())
						castMagicBox("spark_green", x, y)
					end
				end
			end
		end,
	},
	{
		name 					= "mystic_cube_red",
		ui_name					= "Red Mystic Cube",
		categories				= "projectiles,status_effects",
		effect_desc				= "Enemies hit by your spells have a 50% chance of being trapped in a mystic cube with a live bomb inside.",
		effect_desc_short		= "Enemies you hit with spells are trapped in a mystic cube with a live bomb inside.",
        precursor_types			= "magic_artifact",
		upgrade_effect_desc		= "For each upgrade level, effect chance on enemies increases by 5%.",
		is_upgradable 			= true,
		onenemydamaged_function	= function(damage, message, entity_thats_responsible, is_fatal, projectile_thats_responsible )
			if(isRelicOwned("mystic_cube_red") >= 1) then
				if(message == "$damage_projectile" and not entityHasSimplifiedInternalVariable(projectile_thats_responsible, "relic_effect_projectile")) then
					SetRandomSeed( GameGetFrameNum(), math.random(1,50))
					local chance = Random(1,100)
					local odds = 50 + (5 * getRelicLevel("mystic_cube_red"))
					if(chance >= 1 and chance <= odds) then
						local x, y = EntityGetTransform(GetUpdatedEntityID())
						castMagicBox("spark_red", x, y, "data/entities/projectiles/bomb.xml")
					end
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "mystic_cube,bomb_charm",
	},
	{
		name 					= "mystic_cube_blue",
		ui_name					= "Cyan Mystic Cube",
		categories				= "projectiles,status_effects",
		effect_desc				= "Enemies hit by your spells have a 50% chance of being trapped in a mystic cube with a thunderball inside.",
		effect_desc_short		= "Enemies you hit with spells are trapped in a mystic cube with a thunderball inside.",
        precursor_types			= "magic_artifact",
		upgrade_effect_desc		= "For each upgrade level, effect chance on enemies increases by 5%.",
		is_upgradable 			= true,
		onenemydamaged_function	= function(damage, message, entity_thats_responsible, is_fatal, projectile_thats_responsible )
			if(isRelicOwned("mystic_cube_blue") >= 1) then
				if(message == "$damage_projectile" and not entityHasSimplifiedInternalVariable(projectile_thats_responsible, "relic_effect_projectile")) then
					SetRandomSeed( GameGetFrameNum(), math.random(1,50))
					local chance = Random(1,100)
					local odds = 50 + (5 * getRelicLevel("mystic_cube_blue"))
					if(chance >= 1 and chance <= odds) then
						local x, y = EntityGetTransform(GetUpdatedEntityID())
						castMagicBox("spark_blue_dark", x, y, "mods/ArcaneReliquary/files/entity_definitions/custom_projectiles/thunderball_custom.xml")
					end
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "mystic_cube,tesla_coil",
	},
	{
		name 					= "mystic_cube_corrosive",
		ui_name					= "Corrosive Mystic Cube",
		categories				= "projectiles,status_effects",
		effect_desc				= "Enemies hit by your spells have a 50% chance of being trapped in a mystic cube that is slowly filled with acid.",
		effect_desc_short		= "Enemies you hit with spells are trapped in a mystic cube that is filled with acid.",
        precursor_types			= "magic_artifact",
		upgrade_effect_desc		= "For each upgrade level, effect chance on enemies increases by 5%.",
		is_upgradable 			= true,
		onenemydamaged_function	= function(damage, message, entity_thats_responsible, is_fatal, projectile_thats_responsible )
			if(isRelicOwned("mystic_cube_corrosive") >= 1) then
				if(message == "$damage_projectile" and not entityHasSimplifiedInternalVariable(projectile_thats_responsible, "relic_effect_projectile")) then
					SetRandomSeed( GameGetFrameNum(), math.random(1,50))
					local chance = Random(1,100)
					local odds = 50 + (5 * getRelicLevel("mystic_cube_corrosive"))
					if(chance >= 1 and chance <= odds) then
						local x, y = EntityGetTransform(GetUpdatedEntityID())
						castMagicBox("acid", x, y, nil, "acid")
					end
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "mystic_cube,acid_gem",
	},
	{
		name 					= "mystic_cube_magmatic",
		ui_name					= "Magmatic Mystic Cube",
		categories				= "projectiles,status_effects",
		effect_desc				= "Enemies hit by your spells have a 50% chance of being trapped in a mystic cube that is slowly filled with lava.",
		effect_desc_short		= "Enemies you hit with spells are trapped in a mystic cube that is filled with lava.",
        precursor_types			= "magic_artifact",
		upgrade_effect_desc		= "For each upgrade level, effect chance on enemies increases by 5%.",
		is_upgradable 			= true,
		onenemydamaged_function	= function(damage, message, entity_thats_responsible, is_fatal, projectile_thats_responsible )
			if(isRelicOwned("mystic_cube_magmatic") >= 1) then
				if(message == "$damage_projectile" and not entityHasSimplifiedInternalVariable(projectile_thats_responsible, "relic_effect_projectile")) then
					SetRandomSeed( GameGetFrameNum(), math.random(1,50))
					local chance = Random(1,100)
					local odds = 50 + (5 * getRelicLevel("mystic_cube_magmatic"))
					if(chance >= 1 and chance <= odds) then
						local x, y = EntityGetTransform(GetUpdatedEntityID())
						castMagicBox("lava", x, y, nil, "lava")
					end
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "mystic_cube,lava_gem",
	},
	{
		name 					= "mystic_cube_toxic",
		ui_name					= "Toxic Mystic Cube",
		categories				= "projectiles,status_effects",
		effect_desc				= "Enemies hit by your spells have a 50% chance of being trapped in a mystic cube that is slowly filled with toxic gas.",
		effect_desc_short		= "Enemies you hit with spells are trapped in a mystic cube that is filled with toxic gas.",
        precursor_types			= "magic_artifact",
		upgrade_effect_desc		= "For each upgrade level, effect chance on enemies increases by 5%.",
		is_upgradable 			= true,
		onenemydamaged_function	= function(damage, message, entity_thats_responsible, is_fatal, projectile_thats_responsible )
			if(isRelicOwned("mystic_cube_toxic") >= 1) then
				if(message == "$damage_projectile" and not entityHasSimplifiedInternalVariable(projectile_thats_responsible, "relic_effect_projectile")) then
					SetRandomSeed( GameGetFrameNum(), math.random(1,50))
					local chance = Random(1,100)
					local odds = 50 + (5 * getRelicLevel("mystic_cube_toxic"))
					if(chance >= 1 and chance <= odds) then
						local x, y = EntityGetTransform(GetUpdatedEntityID())
						castMagicBox("radioactive_gas", x, y, nil, "radioactive_gas")
					end
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "mystic_cube,toxic_gem",
	},
	{
		name 					= "mystic_cube_humid",
		ui_name					= "Humid Mystic Cube",
		categories				= "projectiles,status_effects",
		effect_desc				= "Enemies hit by your spells have a 50% chance of being trapped in a mystic cube that is slowly filled with water.",
		effect_desc_short		= "Enemies you hit with spells are trapped in a mystic cube that is filled with water.",
        precursor_types			= "magic_artifact",
		upgrade_effect_desc		= "For each upgrade level, effect chance on enemies increases by 5%.",
		is_upgradable 			= true,
		onenemydamaged_function	= function(damage, message, entity_thats_responsible, is_fatal, projectile_thats_responsible )
			if(isRelicOwned("mystic_cube_humid") >= 1) then
				if(message == "$damage_projectile" and not entityHasSimplifiedInternalVariable(projectile_thats_responsible, "relic_effect_projectile")) then
					SetRandomSeed( GameGetFrameNum(), math.random(1,50))
					local chance = Random(1,100)
					local odds = 50 + (5 * getRelicLevel("mystic_cube_humid"))
					if(chance >= 1 and chance <= odds) then
						local x, y = EntityGetTransform(GetUpdatedEntityID())
						castMagicBox("water", x, y, nil, "water")
					end
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "mystic_cube,humid_gem",
	},
	{
		name 					= "mystic_cube_venomous",
		ui_name					= "Venomous Mystic Cube",
		categories				= "projectiles,status_effects",
		effect_desc				= "Enemies hit by your spells have a 50% chance of being trapped in a mystic cube that is slowly filled with poison.",
		effect_desc_short		= "Enemies you hit with spells are trapped in a mystic cube that is filled with poison.",
        precursor_types			= "magic_artifact",
		upgrade_effect_desc		= "For each upgrade level, effect chance on enemies increases by 5%.",
		is_upgradable 			= true,
		onenemydamaged_function	= function(damage, message, entity_thats_responsible, is_fatal, projectile_thats_responsible )
			if(isRelicOwned("mystic_cube_venomous") >= 1) then
				if(message == "$damage_projectile" and not entityHasSimplifiedInternalVariable(projectile_thats_responsible, "relic_effect_projectile")) then
					SetRandomSeed( GameGetFrameNum(), math.random(1,50))
					local chance = Random(1,100)
					local odds = 50 + (5 * getRelicLevel("mystic_cube_venomous"))
					if(chance >= 1 and chance <= odds) then
						local x, y = EntityGetTransform(GetUpdatedEntityID())
						castMagicBox("poison", x, y, nil, "poison")
					end
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "mystic_cube,poison_gem",
	},
	{
		name 					= "mystic_cube_frigid",
		ui_name					= "Frigid Mystic Cube",
		categories				= "projectiles,status_effects",
		effect_desc				= "Enemies hit by your spells have a 50% chance of being trapped in a mystic cube that is slowly filled with freezing gas.",
		effect_desc_short		= "Enemies you hit with spells are trapped in a mystic cube that is filled with freezing gas.",
        precursor_types			= "magic_artifact",
		upgrade_effect_desc		= "For each upgrade level, effect chance on enemies increases by 5%.",
		is_upgradable 			= true,
		onenemydamaged_function	= function(damage, message, entity_thats_responsible, is_fatal, projectile_thats_responsible )
			if(isRelicOwned("mystic_cube_frigid") >= 1) then
				if(message == "$damage_projectile" and not entityHasSimplifiedInternalVariable(projectile_thats_responsible, "relic_effect_projectile")) then
					SetRandomSeed( GameGetFrameNum(), math.random(1,50))
					local chance = Random(1,100)
					local odds = 50 + (5 * getRelicLevel("mystic_cube_frigid"))
					if(chance >= 1 and chance <= odds) then
						local x, y = EntityGetTransform(GetUpdatedEntityID())
						castMagicBox("blood_cold_vapour", x, y, nil, "blood_cold_vapour")
					end
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "mystic_cube,freezing_liquid_gem",
	},
	{
		name 					= "mystic_cube_bloody",
		ui_name					= "Bloody Mystic Cube",
		categories				= "projectiles,status_effects",
		effect_desc				= "Enemies hit by your spells have a 50% chance of being trapped in a mystic cube that is slowly filled with corrosive blood.",
		effect_desc_short		= "Enemies you hit with spells are trapped in a mystic cube that is filled with corrosive blood.",
        precursor_types			= "magic_artifact",
		upgrade_effect_desc		= "For each upgrade level, effect chance on enemies increases by 5%.",
		is_upgradable 			= true,
		onenemydamaged_function	= function(damage, message, entity_thats_responsible, is_fatal, projectile_thats_responsible )
			if(isRelicOwned("mystic_cube_bloody") >= 1) then
				if(message == "$damage_projectile" and not entityHasSimplifiedInternalVariable(projectile_thats_responsible, "relic_effect_projectile")) then
					SetRandomSeed( GameGetFrameNum(), math.random(1,50))
					local chance = Random(1,100)
					local odds = 50 + (5 * getRelicLevel("mystic_cube_bloody"))
					if(chance >= 1 and chance <= odds) then
						local x, y = EntityGetTransform(GetUpdatedEntityID())
						castMagicBox("blood", x, y, nil, "corrosive_blood")
					end
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "mystic_cube,corrosive_blood_gem",
	},
	{
		name 					= "mystic_cube_glacial",
		ui_name					= "Glacial Mystic Cube",
		categories				= "projectiles,status_effects",
		effect_desc				= "Enemies hit by your spells have a 50% chance of being trapped in a mystic cube with a portal that spawns freezebombs inside.",
		effect_desc_short		= "Enemies you hit with spells are trapped in a mystic cube with a portal that spawns freezebombs inside.",
        precursor_types			= "magic_artifact",
		upgrade_effect_desc		= "For each upgrade level, effect chance on enemies increases by 5%.",
		is_upgradable 			= true,
		onenemydamaged_function	= function(damage, message, entity_thats_responsible, is_fatal, projectile_thats_responsible )
			if(isRelicOwned("mystic_cube_glacial") >= 1) then
				if(message == "$damage_projectile" and not entityHasSimplifiedInternalVariable(projectile_thats_responsible, "relic_effect_projectile")) then
					SetRandomSeed( GameGetFrameNum(), math.random(1,50))
					local chance = Random(1,100)
					local odds = 50 + (5 * getRelicLevel("mystic_cube_glacial"))
					if(chance >= 1 and chance <= odds) then
						local x, y = EntityGetTransform(GetUpdatedEntityID())
						castMagicBox("spark_white", x, y, "mods/ArcaneReliquary/files/entity_definitions/others/portal_shooter_ice.xml")
					end
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "mystic_cube,popsicle",
	},
	{
		name 					= "mystic_cube_slapping",
		ui_name					= "Slapping Mystic Cube",
		categories				= "projectiles,status_effects",
		effect_desc				= "Enemies hit by your spells have a 50% chance of being trapped in a mystic cube with a portal that spawns slapperbombs inside.",
		effect_desc_short		= "Enemies you hit with spells are trapped in a mystic cube with a portal that spawns slapperbombs inside.",
        precursor_types			= "magic_artifact",
		upgrade_effect_desc		= "For each upgrade level, effect chance on enemies increases by 5%.",
		is_upgradable 			= true,
		onenemydamaged_function	= function(damage, message, entity_thats_responsible, is_fatal, projectile_thats_responsible )
			if(isRelicOwned("mystic_cube_slapping") >= 1) then
				if(message == "$damage_projectile" and not entityHasSimplifiedInternalVariable(projectile_thats_responsible, "relic_effect_projectile")) then
					SetRandomSeed( GameGetFrameNum(), math.random(1,50))
					local chance = Random(1,100)
					local odds = 50 + (5 * getRelicLevel("mystic_cube_slapping"))
					if(chance >= 1 and chance <= odds) then
						local x, y = EntityGetTransform(GetUpdatedEntityID())
						castMagicBox("spark_green", x, y, "mods/ArcaneReliquary/files/entity_definitions/others/portal_shooter_slap.xml")
					end
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "mystic_cube,fly_swatter",
	},
	{
		name 					= "mystic_cube_bleeding",
		ui_name					= "Bleeding Mystic Cube",
		categories				= "projectiles,status_effects",
		effect_desc				= "Enemies hit by your spells have a 50% chance of being trapped in a mystic cube with a portal that spawns bloodbombs inside.",
		effect_desc_short		= "Enemies you hit with spells are trapped in a mystic cube with a portal that spawns bloodbombs inside.",
        precursor_types			= "magic_artifact",
		upgrade_effect_desc		= "For each upgrade level, effect chance on enemies increases by 5%.",
		is_upgradable 			= true,
		onenemydamaged_function	= function(damage, message, entity_thats_responsible, is_fatal, projectile_thats_responsible )
			if(isRelicOwned("mystic_cube_bleeding") >= 1) then
				if(message == "$damage_projectile" and not entityHasSimplifiedInternalVariable(projectile_thats_responsible, "relic_effect_projectile")) then
					SetRandomSeed( GameGetFrameNum(), math.random(1,50))
					local chance = Random(1,100)
					local odds = 50 + (5 * getRelicLevel("mystic_cube_bleeding"))
					if(chance >= 1 and chance <= odds) then
						local x, y = EntityGetTransform(GetUpdatedEntityID())
						castMagicBox("spark_red", x, y, "mods/ArcaneReliquary/files/entity_definitions/others/portal_shooter_bleed.xml")
					end
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "mystic_cube,coagulated_blood",
	},
	{
		name 					= "mystic_cube_windy",
		ui_name					= "Windy Mystic Cube",
		categories				= "projectiles,status_effects",
		effect_desc				= "Enemies hit by your spells have a 50% chance of being trapped in a mystic cube with a portal that spawns windbombs inside.",
		effect_desc_short		= "Enemies you hit with spells are trapped in a mystic cube with a portal that spawns windbombs inside.",
        precursor_types			= "magic_artifact",
		upgrade_effect_desc		= "For each upgrade level, effect chance on enemies increases by 5%.",
		is_upgradable 			= true,
		onenemydamaged_function	= function(damage, message, entity_thats_responsible, is_fatal, projectile_thats_responsible )
			if(isRelicOwned("mystic_cube_windy") >= 1) then
				if(message == "$damage_projectile" and not entityHasSimplifiedInternalVariable(projectile_thats_responsible, "relic_effect_projectile")) then
					SetRandomSeed( GameGetFrameNum(), math.random(1,50))
					local chance = Random(1,100)
					local odds = 50 + (5 * getRelicLevel("mystic_cube_windy"))
					if(chance >= 1 and chance <= odds) then
						local x, y = EntityGetTransform(GetUpdatedEntityID())
						castMagicBox("spark_white", x, y, "mods/ArcaneReliquary/files/entity_definitions/others/portal_shooter_wind.xml")
					end
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "mystic_cube,pinwheel",
	},
	{
		name 					= "mystic_cube_sharp",
		ui_name					= "Sharp Mystic Cube",
		categories				= "projectiles,status_effects",
		effect_desc				= "Enemies hit by your spells have a 50% chance of being trapped in a mystic cube with a portal that spawns slashbombs inside.",
		effect_desc_short		= "Enemies you hit with spells are trapped in a mystic cube with a portal that spawns slashbombs inside.",
        precursor_types			= "magic_artifact",
		upgrade_effect_desc		= "For each upgrade level, effect chance on enemies increases by 5%.",
		is_upgradable 			= true,
		onenemydamaged_function	= function(damage, message, entity_thats_responsible, is_fatal, projectile_thats_responsible )
			if(isRelicOwned("mystic_cube_sharp") >= 1) then
				if(message == "$damage_projectile" and not entityHasSimplifiedInternalVariable(projectile_thats_responsible, "relic_effect_projectile")) then
					SetRandomSeed( GameGetFrameNum(), math.random(1,50))
					local chance = Random(1,100)
					local odds = 50 + (5 * getRelicLevel("mystic_cube_sharp"))
					if(chance >= 1 and chance <= odds) then
						local x, y = EntityGetTransform(GetUpdatedEntityID())
						castMagicBox("spark_purple_bright", x, y, "mods/ArcaneReliquary/files/entity_definitions/others/portal_shooter_slash.xml")
					end
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "mystic_cube,cursed_dagger",
	},
	{
		name 					= "mystic_cube_fiery",
		ui_name					= "Fiery Mystic Cube",
		categories				= "projectiles,status_effects",
		effect_desc				= "Enemies hit by your spells have a 50% chance of being trapped in a mystic cube with a portal that spawns explodebombs inside.",
		effect_desc_short		= "Enemies you hit with spells are trapped in a mystic cube with a portal that spawns explodebombs inside.",
        precursor_types			= "magic_artifact",
		upgrade_effect_desc		= "For each upgrade level, effect chance on enemies increases by 5%.",
		is_upgradable 			= true,
		onenemydamaged_function	= function(damage, message, entity_thats_responsible, is_fatal, projectile_thats_responsible )
			if(isRelicOwned("mystic_cube_fiery") >= 1) then
				if(message == "$damage_projectile" and not entityHasSimplifiedInternalVariable(projectile_thats_responsible, "relic_effect_projectile")) then
					SetRandomSeed( GameGetFrameNum(), math.random(1,50))
					local chance = Random(1,100)
					local odds = 50 + (5 * getRelicLevel("mystic_cube_fiery"))
					if(chance >= 1 and chance <= odds) then
						local x, y = EntityGetTransform(GetUpdatedEntityID())
						castMagicBox("fire", x, y, "mods/ArcaneReliquary/files/entity_definitions/others/portal_shooter_explode.xml")
					end
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "mystic_cube,exploding_scepter",
	},
	{
		name 					= "mystic_cube_diuretic",
		ui_name					= "Diuretic Mystic Cube",
		categories				= "projectiles,status_effects",
		effect_desc				= "Enemies hit by your spells have a 50% chance of being trapped in a mystic cube that is slowly filled with urine.",
		effect_desc_short		= "Enemies you hit with spells are trapped in a mystic cube that is filled with urine.",
        precursor_types			= "magic_artifact",
		upgrade_effect_desc		= "For each upgrade level, effect chance on enemies increases by 5%.",
		is_upgradable 			= true,
		onenemydamaged_function	= function(damage, message, entity_thats_responsible, is_fatal, projectile_thats_responsible )
			if(isRelicOwned("mystic_cube_diuretic") >= 1) then
				if(message == "$damage_projectile" and not entityHasSimplifiedInternalVariable(projectile_thats_responsible, "relic_effect_projectile")) then
					SetRandomSeed( GameGetFrameNum(), math.random(1,50))
					local chance = Random(1,100)
					local odds = 50 + (5 * getRelicLevel("mystic_cube_diuretic"))
					if(chance >= 1 and chance <= odds) then
						local x, y = EntityGetTransform(GetUpdatedEntityID())
						castMagicBox("urine", x, y, nil, "urine")
					end
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "mystic_cube,urine_gem",
	},
	{
		name 					= "mystic_cube_slimy",
		ui_name					= "Slimy Mystic Cube",
		categories				= "projectiles,status_effects",
		effect_desc				= "Enemies hit by your spells have a 50% chance of being trapped in a mystic cube that is slowly filled with slime.",
		effect_desc_short		= "Enemies you hit with spells are trapped in a mystic cube that is filled with slime.",
        precursor_types			= "magic_artifact",
		upgrade_effect_desc		= "For each upgrade level, effect chance on enemies increases by 5%.",
		is_upgradable 			= true,
		onenemydamaged_function	= function(damage, message, entity_thats_responsible, is_fatal, projectile_thats_responsible )
			if(isRelicOwned("mystic_cube_slimy") >= 1) then
				if(message == "$damage_projectile" and not entityHasSimplifiedInternalVariable(projectile_thats_responsible, "relic_effect_projectile")) then
					SetRandomSeed( GameGetFrameNum(), math.random(1,50))
					local chance = Random(1,100)
					local odds = 50 + (5 * getRelicLevel("mystic_cube_slimy"))
					if(chance >= 1 and chance <= odds) then
						local x, y = EntityGetTransform(GetUpdatedEntityID())
						castMagicBox("slime", x, y, nil, "slime")
					end
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "mystic_cube,slime_gem",
	},
	{
		name 					= "mystic_cube_lubricated",
		ui_name					= "Lubricated Mystic Cube",
		categories				= "projectiles,status_effects",
		effect_desc				= "Enemies hit by your spells have a 50% chance of being trapped in a mystic cube that is slowly filled with oil.",
		effect_desc_short		= "Enemies you hit with spells are trapped in a mystic cube that is filled with oil.",
        precursor_types			= "magic_artifact",
		upgrade_effect_desc		= "For each upgrade level, effect chance on enemies increases by 5%.",
		is_upgradable 			= true,
		onenemydamaged_function	= function(damage, message, entity_thats_responsible, is_fatal, projectile_thats_responsible )
			if(isRelicOwned("mystic_cube_lubricated") >= 1) then
				if(message == "$damage_projectile" and not entityHasSimplifiedInternalVariable(projectile_thats_responsible, "relic_effect_projectile")) then
					SetRandomSeed( GameGetFrameNum(), math.random(1,50))
					local chance = Random(1,100)
					local odds = 50 + (5 * getRelicLevel("mystic_cube_lubricated"))
					if(chance >= 1 and chance <= odds) then
						local x, y = EntityGetTransform(GetUpdatedEntityID())
						castMagicBox("oil", x, y, nil, "oil")
					end
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "mystic_cube,oil_gem",
	},
	{
		name 					= "mystic_cube_alcoholic",
		ui_name					= "Alcoholic Mystic Cube",
		categories				= "projectiles,status_effects",
		effect_desc				= "Enemies hit by your spells have a 50% chance of being trapped in a mystic cube that is slowly filled with whiskey.",
		effect_desc_short		= "Enemies you hit with spells are trapped in a mystic cube that is filled with whiskey.",
        precursor_types			= "magic_artifact",
		upgrade_effect_desc		= "For each upgrade level, effect chance on enemies increases by 5%.",
		is_upgradable 			= true,
		onenemydamaged_function	= function(damage, message, entity_thats_responsible, is_fatal, projectile_thats_responsible )
			if(isRelicOwned("mystic_cube_alcoholic") >= 1) then
				if(message == "$damage_projectile" and not entityHasSimplifiedInternalVariable(projectile_thats_responsible, "relic_effect_projectile")) then
					SetRandomSeed( GameGetFrameNum(), math.random(1,50))
					local chance = Random(1,100)
					local odds = 50 + (5 * getRelicLevel("mystic_cube_alcoholic"))
					if(chance >= 1 and chance <= odds) then
						local x, y = EntityGetTransform(GetUpdatedEntityID())
						castMagicBox("alcohol", x, y, nil, "alcohol")
					end
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "mystic_cube,oil_gem",
	},
	{
		name 					= "mystic_cube_infernal",
		ui_name					= "Infernal Mystic Cube",
		categories				= "projectiles,status_effects",
		effect_desc				= "Enemies hit by your spells have a 50% chance of being trapped in a mystic cube with a portal that spawns firebombs inside.",
		effect_desc_short		= "Enemies you hit with spells are trapped in a mystic cube with a portal that spawns firebombs inside.",
        precursor_types			= "magic_artifact",
		upgrade_effect_desc		= "For each upgrade level, effect chance on enemies increases by 5%.",
		is_upgradable 			= true,
		onenemydamaged_function	= function(damage, message, entity_thats_responsible, is_fatal, projectile_thats_responsible )
			if(isRelicOwned("mystic_cube_infernal") >= 1) then
				if(message == "$damage_projectile" and not entityHasSimplifiedInternalVariable(projectile_thats_responsible, "relic_effect_projectile")) then
					SetRandomSeed( GameGetFrameNum(), math.random(1,50))
					local chance = Random(1,100)
					local odds = 50 + (5 * getRelicLevel("mystic_cube_infernal"))
					if(chance >= 1 and chance <= odds) then
						local x, y = EntityGetTransform(GetUpdatedEntityID())
						castMagicBox("spark_yellow", x, y, "mods/ArcaneReliquary/files/entity_definitions/others/portal_shooter_firebomb.xml")
					end
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "mystic_cube,solar_artifact",
	},
	{
		name 					= "mystic_cube_sticky",
		ui_name					= "Sticky Mystic Cube",
		categories				= "projectiles,status_effects",
		effect_desc				= "Enemies hit by your spells have a 50% chance of being trapped in a mystic cube with a portal that spawns slime bolts inside.",
		effect_desc_short		= "Enemies you hit with spells are trapped in a mystic cube with a portal that spawns slime bolts inside.",
        precursor_types			= "magic_artifact",
		upgrade_effect_desc		= "For each upgrade level, effect chance on enemies increases by 5%.",
		is_upgradable 			= true,
		onenemydamaged_function	= function(damage, message, entity_thats_responsible, is_fatal, projectile_thats_responsible )
			if(isRelicOwned("mystic_cube_sticky") >= 1) then
				if(message == "$damage_projectile" and not entityHasSimplifiedInternalVariable(projectile_thats_responsible, "relic_effect_projectile")) then
					SetRandomSeed( GameGetFrameNum(), math.random(1,50))
					local chance = Random(1,1)
					local odds = 50 + (5 * getRelicLevel("mystic_cube_sticky"))
					if(chance >= 1 and chance <= odds) then
						local x, y = EntityGetTransform(GetUpdatedEntityID())
						castMagicBox("radioactive_gas", x, y, "mods/ArcaneReliquary/files/entity_definitions/others/portal_shooter_slime.xml")
					end
				end
			end
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "mystic_cube,ball_of_goo",
	},
	{
		name 					= "parry_shield_steel",
		ui_name					= "Steel Parry Shield",
		categories				= "player",
		effect_desc				= "You hold a shield that destroys enemy projectiles and gives you immunity to slice attacks.",
		effect_desc_short		= "You hold a shield that destroys enemy projectiles and gives you immunity to slice attacks.",
        precursor_types			= "gadget,magic_artifact",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			EntityAddChild(getPlayerEntity(), EntityLoad("mods/ArcaneReliquary/files/entity_definitions/others/parry_shield_steel_handler.xml"))
		end,
		ondrop_function			= function()
			local entity_children = EntityGetAllChildren(getPlayerEntity())
			if entity_children ~= nil then
				for i,child_id in ipairs(entity_children) do
					if(EntityGetName(child_id) == "parry_shield_steel_handler" and not EntityHasTag(child_id, "relic")) then
						local x, y = EntityGetTransform(child_id)
						EntityKill(child_id)
					end
				end
			end
			
			setBasicDamageMultiplier(getPlayerEntity(), "slice", getInternalVariableValue(getPlayerEntity(), "slice_multiplier", "value_float"))
		end,
		overtime_function		= function()
			-- provides immunity to slice damage as long as the shield is active
			if(isRelicOwned("parry_shield_steel") >= 1) then
				if(entityHasSimplifiedInternalVariable(getPlayerEntity(), "parry_shield_steel_active")) then
					setBasicDamageMultiplier(getPlayerEntity(), "slice", 0)
				else
					setBasicDamageMultiplier(getPlayerEntity(), "slice", getInternalVariableValue(getPlayerEntity(), "slice_multiplier", "value_float"))
				end
			end
		end,
		excludes_relics_desc	= "It can't be used along with other Parry Shield Relic variations or other shields that provide resistance to slice damage",
		excludes_relics_tags	= "parry_shield_variation,slice_proof_shield",
		combine_exclusive		= true,
		combine_requires_relics	= "parry_shield,steel_shield",
	},
	{
		name 					= "parry_shield_rubber",
		ui_name					= "Rubber Parry Shield",
		categories				= "player",
		effect_desc				= "You hold a shield that deflects enemy projectiles and gives you immunity to melee attacks.",
		effect_desc_short		= "You hold a shield that deflects enemy projectiles and gives you immunity to melee attacks.",
        precursor_types			= "gadget,magic_artifact",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			EntityAddChild(getPlayerEntity(), EntityLoad("mods/ArcaneReliquary/files/entity_definitions/others/parry_shield_rubber_handler.xml"))
		end,
		ondrop_function			= function()
			local entity_children = EntityGetAllChildren(getPlayerEntity())
			if entity_children ~= nil then
				for i,child_id in ipairs(entity_children) do
					if(EntityGetName(child_id) == "parry_shield_rubber_handler" and not EntityHasTag(child_id, "relic")) then
						local x, y = EntityGetTransform(child_id)
						EntityKill(child_id)
					end
				end
			end
			
			if(not playerHasPerk("PROTECTION_MELEE")) then
				setBasicDamageMultiplier(getPlayerEntity(), "melee", getInternalVariableValue(getPlayerEntity(), "melee_multiplier", "value_float"))
			end
		end,
		overtime_function		= function()
			-- provides immunity to slice damage as long as the shield is active
			if(isRelicOwned("parry_shield_rubber") >= 1) then
				if(entityHasSimplifiedInternalVariable(getPlayerEntity(), "parry_shield_rubber_active")) then
					setBasicDamageMultiplier(getPlayerEntity(), "melee", 0)
				else
					if(not playerHasPerk("PROTECTION_MELEE")) then
						setBasicDamageMultiplier(getPlayerEntity(), "melee", getInternalVariableValue(getPlayerEntity(), "melee_multiplier", "value_float"))
					end
				end
			end
		end,
		excludes_relics_desc	= "It can't be used along with other Parry Shield Relic variations or other shields that provide resistance to melee damage",
		excludes_relics_tags	= "parry_shield_variation,melee_proof_shield",
		combine_exclusive		= true,
		combine_requires_relics	= "parry_shield,rubber_shield",
	},
	{
		name 					= "parry_shield_riot",
		ui_name					= "Riot Parry Shield",
		categories				= "player",
		effect_desc				= "You hold a shield that deflects enemy projectiles, gives you immunity to melee and slice attacks and damages enemies who get too close.",
		effect_desc_short		= "You hold a shield that deflects enemy projectiles, gives you immunity to melee and slice attacks and damages enemies who get too close.",
        precursor_types			= "gadget,magic_artifact",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			EntityAddChild(getPlayerEntity(), EntityLoad("mods/ArcaneReliquary/files/entity_definitions/others/parry_shield_riot_handler.xml"))
		end,
		ondrop_function			= function()
			local entity_children = EntityGetAllChildren(getPlayerEntity())
			if entity_children ~= nil then
				for i,child_id in ipairs(entity_children) do
					if(EntityGetName(child_id) == "parry_shield_riot_handler" and not EntityHasTag(child_id, "relic")) then
						local x, y = EntityGetTransform(child_id)
						EntityKill(child_id)
					end
				end
			end
			
			if(not playerHasPerk("PROTECTION_MELEE")) then
				setBasicDamageMultiplier(getPlayerEntity(), "melee", getInternalVariableValue(getPlayerEntity(), "melee_multiplier", "value_float"))
			end
			setBasicDamageMultiplier(getPlayerEntity(), "slice", getInternalVariableValue(getPlayerEntity(), "slice_multiplier", "value_float"))
		end,
		overtime_function		= function()
			-- provides immunity to slice damage as long as the shield is active
			if(isRelicOwned("parry_shield_riot") >= 1) then
				if(entityHasSimplifiedInternalVariable(getPlayerEntity(), "parry_shield_riot_active")) then
					setBasicDamageMultiplier(getPlayerEntity(), "melee", 0)
					setBasicDamageMultiplier(getPlayerEntity(), "slice", 0)
				else
					if(not playerHasPerk("PROTECTION_MELEE")) then
						setBasicDamageMultiplier(getPlayerEntity(), "melee", getInternalVariableValue(getPlayerEntity(), "melee_multiplier", "value_float"))
					end
					setBasicDamageMultiplier(getPlayerEntity(), "slice", getInternalVariableValue(getPlayerEntity(), "slice_multiplier", "value_float"))
				end
			end
		end,
		excludes_relics_desc	= "It can't be used along with other Parry Shield Relic variations or other shields that provide resistance to slice and melee damage",
		excludes_relics_tags	= "parry_shield_variation,melee_proof_shield,slice_proof_shield",
		combine_exclusive		= true,
		combine_requires_relics	= "parry_shield,riot_shield",
	},
	{
		name 					= "blast_shield",
		ui_name					= "Blast Shield",
		categories				= "health",
		effect_desc				= "You are now immune to damage caused directly by explosions.",
		effect_desc_short		= "You are now immune to damage caused directly by explosions.",
        precursor_types			= "gadget",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			setBasicDamageMultiplier(getPlayerEntity(), "explosion", 0)
		end,
		ondrop_function			= function()
			if(not playerHasPerk("PROTECTION_EXPLOSION")) then
				setBasicDamageMultiplier(getPlayerEntity(), "explosion", getInternalVariableValue(getPlayerEntity(), "explosion_multiplier", "value_float"))
			end
		end,
		excludes_relics_desc	= "It can't be used along with other relics that provide explosions resistance",
		excludes_relics_tags	= "explosion_resistance",
	},
	{
		name 					= "ceramic_shield",
		ui_name					= "Ceramic Shield",
		categories				= "health",
		effect_desc				= "You are now immune to damage caused directly by fire.",
		effect_desc_short		= "You are now immune to damage caused directly by fire.",
        precursor_types			= "gadget",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			setBasicDamageMultiplier(getPlayerEntity(), "fire", 0)
		end,
		ondrop_function			= function()
			if(not playerHasPerk("PROTECTION_FIRE")) then
				setBasicDamageMultiplier(getPlayerEntity(), "fire", getInternalVariableValue(getPlayerEntity(), "fire_multiplier", "value_float"))
			end
		end,
		excludes_relics_desc	= "It can't be used along with other fire proof shields",
		excludes_relics_tags	= "fire_proof_shield",
	},
	{
		name 					= "lead_shield",
		ui_name					= "Lead Shield",
		categories				= "health",
		effect_desc				= "You are now immune to radiation damage.",
		effect_desc_short		= "You are now immune to radiation damage.",
        precursor_types			= "gadget",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			setBasicDamageMultiplier(getPlayerEntity(), "radioactive", 0)
		end,
		ondrop_function			= function()
			if(not playerHasPerk("PROTECTION_RADIOACTIVITY")) then
				setBasicDamageMultiplier(getPlayerEntity(), "radioactive", getInternalVariableValue(getPlayerEntity(), "radioactive_multiplier", "value_float"))
			end
		end,
		excludes_relics_desc	= "It can't be used along with Riot Shield relic or other radiation proof shields",
		excludes_relics_tags	= "radiation_proof_shield",
	},
	{
		name 					= "plastic_shield",
		ui_name					= "Plastic Shield",
		categories				= "health",
		effect_desc				= "You are now immune to damage caused directly by electricity.",
		effect_desc_short		= "You are now immune to damage caused directly by electricity.",
        precursor_types			= "gadget",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			setBasicDamageMultiplier(getPlayerEntity(), "electricity", 0)
		end,
		ondrop_function			= function()
			if(not playerHasPerk("PROTECTION_ELECTRICITY")) then
				setBasicDamageMultiplier(getPlayerEntity(), "electricity", getInternalVariableValue(getPlayerEntity(), "electricity_multiplier", "value_float"))
			end
		end,
		excludes_relics_desc	= "It can't be used along with other electricity proof shields",
		excludes_relics_tags	= "electricity_proof_shield",
	},
	{
		name 					= "parry_shield_blast",
		ui_name					= "Blast Parry Shield",
		categories				= "player",
		effect_desc				= "You hold a shield that deflects enemy projectiles and gives you immunity to explosion damage.",
		effect_desc_short		= "You hold a shield that deflects enemy projectiles and gives you immunity to explosion damage.",
        precursor_types			= "gadget,magic_artifact",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			EntityAddChild(getPlayerEntity(), EntityLoad("mods/ArcaneReliquary/files/entity_definitions/others/parry_shield_blast_handler.xml"))
		end,
		ondrop_function			= function()
			local entity_children = EntityGetAllChildren(getPlayerEntity())
			if entity_children ~= nil then
				for i,child_id in ipairs(entity_children) do
					if(EntityGetName(child_id) == "parry_shield_blast_handler" and not EntityHasTag(child_id, "relic")) then
						local x, y = EntityGetTransform(child_id)
						EntityKill(child_id)
					end
				end
			end
			if(not playerHasPerk("PROTECTION_EXPLOSION")) then
				setBasicDamageMultiplier(getPlayerEntity(), "explosion", getInternalVariableValue(getPlayerEntity(), "explosion_multiplier", "value_float"))
			end
		end,
		overtime_function		= function()
			-- provides immunity to explosion damage as long as the shield is active
			if(isRelicOwned("parry_shield_blast") >= 1) then
				if(entityHasSimplifiedInternalVariable(getPlayerEntity(), "parry_shield_blast_active")) then
					setBasicDamageMultiplier(getPlayerEntity(), "explosion", 0)
				else
					if(not playerHasPerk("PROTECTION_EXPLOSION")) then
						setBasicDamageMultiplier(getPlayerEntity(), "explosion", getInternalVariableValue(getPlayerEntity(), "explosion_multiplier", "value_float"))
					end
				end
			end
		end,
		excludes_relics_desc	= "It can't be used along with other Parry Shield Relic variations or other relics that provide resistance to explosions",
		excludes_relics_tags	= "parry_shield_variation,explosion_resistance",
		combine_exclusive		= true,
		combine_requires_relics	= "parry_shield,blast_shield",
	},
	{
		name 					= "parry_shield_ceramic",
		ui_name					= "Ceramic Parry Shield",
		categories				= "player",
		effect_desc				= "You hold a shield that deflects enemy projectiles and gives you immunity to fire damage.",
		effect_desc_short		= "You hold a shield that deflects enemy projectiles and gives you immunity to fire damage.",
        precursor_types			= "gadget,magic_artifact",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			EntityAddChild(getPlayerEntity(), EntityLoad("mods/ArcaneReliquary/files/entity_definitions/others/parry_shield_ceramic_handler.xml"))
		end,
		ondrop_function			= function()
			local entity_children = EntityGetAllChildren(getPlayerEntity())
			if entity_children ~= nil then
				for i,child_id in ipairs(entity_children) do
					if(EntityGetName(child_id) == "parry_shield_ceramic_handler" and not EntityHasTag(child_id, "relic")) then
						local x, y = EntityGetTransform(child_id)
						EntityKill(child_id)
					end
				end
			end
			if(not playerHasPerk("PROTECTION_FIRE")) then
				setBasicDamageMultiplier(getPlayerEntity(), "fire", getInternalVariableValue(getPlayerEntity(), "fire_multiplier", "value_float"))
			end
		end,
		overtime_function		= function()
			-- provides immunity to fire damage as long as the shield is active
			if(isRelicOwned("parry_shield_ceramic") >= 1) then
				if(entityHasSimplifiedInternalVariable(getPlayerEntity(), "parry_shield_ceramic_active")) then
					setBasicDamageMultiplier(getPlayerEntity(), "fire", 0)
				else
					if(not playerHasPerk("PROTECTION_FIRE")) then
						setBasicDamageMultiplier(getPlayerEntity(), "fire", getInternalVariableValue(getPlayerEntity(), "fire_multiplier", "value_float"))
					end
				end
			end
		end,
		excludes_relics_desc	= "It can't be used along with other Parry Shield Relic variations or other shields that provide fire resistance",
		excludes_relics_tags	= "parry_shield_variation,fire_proof_shield",
		combine_exclusive		= true,
		combine_requires_relics	= "parry_shield,ceramic_shield",
	},
	{
		name 					= "parry_shield_lead",
		ui_name					= "Lead Parry Shield",
		categories				= "player",
		effect_desc				= "You hold a shield that deflects enemy projectiles and gives you immunity to radiation damage.",
		effect_desc_short		= "You hold a shield that deflects enemy projectiles and gives you immunity to radiation damage.",
        precursor_types			= "gadget,magic_artifact",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			EntityAddChild(getPlayerEntity(), EntityLoad("mods/ArcaneReliquary/files/entity_definitions/others/parry_shield_lead_handler.xml"))
		end,
		ondrop_function			= function()
			local entity_children = EntityGetAllChildren(getPlayerEntity())
			if entity_children ~= nil then
				for i,child_id in ipairs(entity_children) do
					if(EntityGetName(child_id) == "parry_shield_lead_handler" and not EntityHasTag(child_id, "relic")) then
						local x, y = EntityGetTransform(child_id)
						EntityKill(child_id)
					end
				end
			end
			if(not playerHasPerk("PROTECTION_RADIOACTIVITY")) then
				setBasicDamageMultiplier(getPlayerEntity(), "radioactive", getInternalVariableValue(getPlayerEntity(), "radioactive_multiplier", "value_float"))
			end
		end,
		overtime_function		= function()
			-- provides immunity to fire damage as long as the shield is active
			if(isRelicOwned("parry_shield_lead") >= 1) then
				if(entityHasSimplifiedInternalVariable(getPlayerEntity(), "parry_shield_lead_active")) then
					setBasicDamageMultiplier(getPlayerEntity(), "radioactive", 0)
				else
					if(not playerHasPerk("PROTECTION_RADIOACTIVITY")) then
						setBasicDamageMultiplier(getPlayerEntity(), "radioactive", getInternalVariableValue(getPlayerEntity(), "radioactive_multiplier", "value_float"))
					end
				end
			end
		end,
		excludes_relics_desc	= "It can't be used along with other Parry Shield Relic variations or other shields that provide radiation resistance",
		excludes_relics_tags	= "parry_shield_variation,radiation_proof_shield",
		combine_exclusive		= true,
		combine_requires_relics	= "parry_shield,lead_shield",
	},
	{
		name 					= "parry_shield_plastic",
		ui_name					= "Plastic Parry Shield",
		categories				= "player",
		effect_desc				= "You hold a shield that deflects enemy projectiles and gives you immunity to electricity damage.",
		effect_desc_short		= "You hold a shield that deflects enemy projectiles and gives you immunity to electricity damage.",
        precursor_types			= "gadget,magic_artifact",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			EntityAddChild(getPlayerEntity(), EntityLoad("mods/ArcaneReliquary/files/entity_definitions/others/parry_shield_plastic_handler.xml"))
		end,
		ondrop_function			= function()
			local entity_children = EntityGetAllChildren(getPlayerEntity())
			if entity_children ~= nil then
				for i,child_id in ipairs(entity_children) do
					if(EntityGetName(child_id) == "parry_shield_plastic_handler" and not EntityHasTag(child_id, "relic")) then
						local x, y = EntityGetTransform(child_id)
						EntityKill(child_id)
					end
				end
			end
			if(not playerHasPerk("PROTECTION_ELECTRICITY")) then
				setBasicDamageMultiplier(getPlayerEntity(), "electricity", getInternalVariableValue(getPlayerEntity(), "electricity_multiplier", "value_float"))
			end
		end,
		overtime_function		= function()
			-- provides immunity to fire damage as long as the shield is active
			if(isRelicOwned("parry_shield_plastic") >= 1) then
				if(entityHasSimplifiedInternalVariable(getPlayerEntity(), "parry_shield_plastic_active")) then
					setBasicDamageMultiplier(getPlayerEntity(), "electricity", 0)
				else
					if(not playerHasPerk("PROTECTION_RADIOACTIVITY")) then
						setBasicDamageMultiplier(getPlayerEntity(), "electricity", getInternalVariableValue(getPlayerEntity(), "electricity_multiplier", "value_float"))
					end
				end
			end
		end,
		excludes_relics_desc	= "It can't be used along with other Parry Shield Relic variations or other shields that provide electricity resistance",
		excludes_relics_tags	= "parry_shield_variation,electricity_proof_shield",
		combine_exclusive		= true,
		combine_requires_relics	= "parry_shield,plastic_shield",
	},
	{
		name 					= "first_aid_kit",
		ui_name					= "First Aid Kit",
		categories				= "player",
		effect_desc				= "When you take damage from enemies, there's a 50% chance you'll become invulnerable to all damage \ntypes for a few seconds, but there's a 50% chance your spells will have \ntheir base and explosive damage and speed halved.",
		effect_desc_short		= "Taking damage from enemies makes you invulnerable for a few seconds, but your spells get nerfed.",
        precursor_types			= "gadget",
		upgrade_effect_desc		= "For each upgrade level, effect chance increases by 5% and penalty chance decreases by 5%.",
		is_upgradable 			= true,
		ondamaged_function		= function(damage, message, entity_thats_responsible, is_fatal, projectile_thats_responsible )
			if(isRelicOwned("first_aid_kit") >= 1) then	
				if(EntityHasTag(entity_thats_responsible, "enemy")) then
					
					SetRandomSeed( GameGetFrameNum(), math.random(1,50))
					local chance = Random(1,100)
					local odds = 50 + (5 * getRelicLevel("first_aid_kit"))
					if(chance >= 1 and chance <= odds) then
						local game_effect_comp = GetGameEffectLoadTo( getPlayerEntity(), "PROTECTION_ALL", true )
						if game_effect_comp ~= nil then
							ComponentSetValue( game_effect_comp, "frames", "500" )
						end
					end
				end
			end
		end,
		onspellcast_function	= function(projectile_entity_id)
			if(isRelicOwned("first_aid_kit") >= 1) then
				SetRandomSeed( GameGetFrameNum(), math.random(1,50))
				local chance = Random(1,100)
				local odds = 50 - (5 * getRelicLevel("first_aid_kit"))
				if(chance >= 1 and chance <= odds) then
					fractionProjectileDamage(projectile_entity_id, 2)
					fractionProjectileExplosiveDamage(projectile_entity_id, 2)
					fractionProjectileVelocities(projectile_entity_id, 2)
				end
			end
		end,
	},
	{
		name 					= "wrench",
		ui_name					= "Wrench",
		categories				= "player",
		effect_desc				= "A spinning bladewheel that deals slice damage to enemies \nappears whenever you use your kick while standing on solid ground. \nPerforming this ability consumes jetpack energy.",
		effect_desc_short		= "Kicking while in solid ground conjures a slicing spinning bladewheel.",
        precursor_types			= "gadget",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			if(not entityHasCallback(getPlayerEntity(), "player_conjure_bladewheel_listener")) then -- if no other related relics added required callback, adds it
				addNewLuaCallback(getPlayerEntity(), "overtime", "mods/ArcaneReliquary/files/scripts/player/player_conjure_bladewheel_listener.lua", 1, false, "player_conjure_bladewheel_listener")
			end
			addNewSimplifiedInternalVariable(getPlayerEntity(), "player_conjure_bladewheel_listener") -- for each related relic, a new simplified variable like this is added
			
			addNewSimplifiedInternalVariable(getPlayerEntity(), "wrench")
		end,
		ondrop_function			= function()
			removeFirstSimplifiedInternalVariable(getPlayerEntity(), "player_conjure_bladewheel_listener") -- removes just the first variable found
			
			if(countSimplifiedInternalVariable(getPlayerEntity(), "player_conjure_bladewheel_listener") == 0) then -- removes callback only if no other relics are using it
				removeCallback(getPlayerEntity(), "player_conjure_bladewheel_listener")
			end
			
			removeSimplifiedInternalVariable(getPlayerEntity(), "wrench")
		end,
	},
	{
		name 					= "wrench_pink",
		ui_name					= "Wrench with Pink Tape",
		categories				= "player",
		effect_desc				= "A spinning bladewheel that deals slice damage to enemies and deflects enemy projectiles\n appears whenever you use your kick while standing on solid ground. \nPerforming this ability consumes jetpack energy.",
		effect_desc_short		= "Kicking while in solid ground conjures a deflecting spinning bladewheel.",
        precursor_types			= "gadget",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			if(not entityHasCallback(getPlayerEntity(), "player_conjure_bladewheel_listener")) then -- if no other related relics added required callback, adds it
				addNewLuaCallback(getPlayerEntity(), "overtime", "mods/ArcaneReliquary/files/scripts/player/player_conjure_bladewheel_listener.lua", 1, false, "player_conjure_bladewheel_listener")
			end
			addNewSimplifiedInternalVariable(getPlayerEntity(), "player_conjure_bladewheel_listener") -- for each related relic, a new simplified variable like this is added
			
			addNewSimplifiedInternalVariable(getPlayerEntity(), "wrench_pink")
		end,
		ondrop_function			= function()
			removeFirstSimplifiedInternalVariable(getPlayerEntity(), "player_conjure_bladewheel_listener") -- removes just the first variable found
			
			if(countSimplifiedInternalVariable(getPlayerEntity(), "player_conjure_bladewheel_listener") == 0) then -- removes callback only if no other relics are using it
				removeCallback(getPlayerEntity(), "player_conjure_bladewheel_listener")
			end
			
			removeSimplifiedInternalVariable(getPlayerEntity(), "wrench_pink")
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "wrench,piercing_crystal",
	},
	{
		name 					= "wrench_cyan",
		ui_name					= "Wrench with Cyan Tape",
		categories				= "player",
		effect_desc				= "A spinning bladewheel that deals freeze damage to enemies \nappears whenever you use your kick while standing on solid ground. \nPerforming this ability consumes jetpack energy.",
		effect_desc_short		= "Kicking while in solid ground conjures a freezing spinning bladewheel.",
        precursor_types			= "gadget",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			if(not entityHasCallback(getPlayerEntity(), "player_conjure_bladewheel_listener")) then -- if no other related relics added required callback, adds it
				addNewLuaCallback(getPlayerEntity(), "overtime", "mods/ArcaneReliquary/files/scripts/player/player_conjure_bladewheel_listener.lua", 1, false, "player_conjure_bladewheel_listener")
			end
			addNewSimplifiedInternalVariable(getPlayerEntity(), "player_conjure_bladewheel_listener") -- for each related relic, a new simplified variable like this is added
			
			addNewSimplifiedInternalVariable(getPlayerEntity(), "wrench_cyan")
		end,
		ondrop_function			= function()
			removeFirstSimplifiedInternalVariable(getPlayerEntity(), "player_conjure_bladewheel_listener") -- removes just the first variable found
			
			if(countSimplifiedInternalVariable(getPlayerEntity(), "player_conjure_bladewheel_listener") == 0) then -- removes callback only if no other relics are using it
				removeCallback(getPlayerEntity(), "player_conjure_bladewheel_listener")
			end
			
			removeSimplifiedInternalVariable(getPlayerEntity(), "wrench_cyan")
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "wrench,ice_artifact",
	},
	{
		name 					= "wrench_red",
		ui_name					= "Wrench with Red Tape",
		categories				= "player",
		effect_desc				= "A spinning bladewheel that inflicts the bleeding effect to enemies \nappears whenever you use your kick while standing on solid ground. \nPerforming this ability consumes jetpack energy.",
		effect_desc_short		= "Kicking while in solid ground conjures a bleeding spinning bladewheel.",
        precursor_types			= "gadget",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			if(not entityHasCallback(getPlayerEntity(), "player_conjure_bladewheel_listener")) then -- if no other related relics added required callback, adds it
				addNewLuaCallback(getPlayerEntity(), "overtime", "mods/ArcaneReliquary/files/scripts/player/player_conjure_bladewheel_listener.lua", 1, false, "player_conjure_bladewheel_listener")
			end
			addNewSimplifiedInternalVariable(getPlayerEntity(), "player_conjure_bladewheel_listener") -- for each related relic, a new simplified variable like this is added
			
			addNewSimplifiedInternalVariable(getPlayerEntity(), "wrench_red")
		end,
		ondrop_function			= function()
			removeFirstSimplifiedInternalVariable(getPlayerEntity(), "player_conjure_bladewheel_listener") -- removes just the first variable found
			
			if(countSimplifiedInternalVariable(getPlayerEntity(), "player_conjure_bladewheel_listener") == 0) then -- removes callback only if no other relics are using it
				removeCallback(getPlayerEntity(), "player_conjure_bladewheel_listener")
			end
			
			removeSimplifiedInternalVariable(getPlayerEntity(), "wrench_red")
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "wrench,ancient_dagger",
	},
	{
		name 					= "wrench_white",
		ui_name					= "Wrench with White Tape",
		categories				= "player",
		effect_desc				= "A spinning bladewheel that inflicts the slash effect to enemies \nappears whenever you use your kick while standing on solid ground. \nPerforming this ability consumes jetpack energy.",
		effect_desc_short		= "Kicking while in solid ground conjures a slashing spinning bladewheel.",
        precursor_types			= "gadget",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			if(not entityHasCallback(getPlayerEntity(), "player_conjure_bladewheel_listener")) then -- if no other related relics added required callback, adds it
				addNewLuaCallback(getPlayerEntity(), "overtime", "mods/ArcaneReliquary/files/scripts/player/player_conjure_bladewheel_listener.lua", 1, false, "player_conjure_bladewheel_listener")
			end
			addNewSimplifiedInternalVariable(getPlayerEntity(), "player_conjure_bladewheel_listener") -- for each related relic, a new simplified variable like this is added
			
			addNewSimplifiedInternalVariable(getPlayerEntity(), "wrench_white")
		end,
		ondrop_function			= function()
			removeFirstSimplifiedInternalVariable(getPlayerEntity(), "player_conjure_bladewheel_listener") -- removes just the first variable found
			
			if(countSimplifiedInternalVariable(getPlayerEntity(), "player_conjure_bladewheel_listener") == 0) then -- removes callback only if no other relics are using it
				removeCallback(getPlayerEntity(), "player_conjure_bladewheel_listener")
			end
			
			removeSimplifiedInternalVariable(getPlayerEntity(), "wrench_white")
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "wrench,cursed_dagger",
	},
	{
		name 					= "wrench_green",
		ui_name					= "Wrench with Green Tape",
		categories				= "player",
		effect_desc				= "A spinning bladewheel that inflicts the slap effect to enemies \nappears whenever you use your kick while standing on solid ground. \nPerforming this ability consumes jetpack energy.",
		effect_desc_short		= "Kicking while in solid ground conjures a slapping spinning bladewheel.",
        precursor_types			= "gadget",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			if(not entityHasCallback(getPlayerEntity(), "player_conjure_bladewheel_listener")) then -- if no other related relics added required callback, adds it
				addNewLuaCallback(getPlayerEntity(), "overtime", "mods/ArcaneReliquary/files/scripts/player/player_conjure_bladewheel_listener.lua", 1, false, "player_conjure_bladewheel_listener")
			end
			addNewSimplifiedInternalVariable(getPlayerEntity(), "player_conjure_bladewheel_listener") -- for each related relic, a new simplified variable like this is added
			
			addNewSimplifiedInternalVariable(getPlayerEntity(), "wrench_green")
		end,
		ondrop_function			= function()
			removeFirstSimplifiedInternalVariable(getPlayerEntity(), "player_conjure_bladewheel_listener") -- removes just the first variable found
			
			if(countSimplifiedInternalVariable(getPlayerEntity(), "player_conjure_bladewheel_listener") == 0) then -- removes callback only if no other relics are using it
				removeCallback(getPlayerEntity(), "player_conjure_bladewheel_listener")
			end
			
			removeSimplifiedInternalVariable(getPlayerEntity(), "wrench_green")
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "wrench,fly_swatter",
	},
	{
		name 					= "wrench_orange",
		ui_name					= "Wrench with Orange Tape",
		categories				= "player",
		effect_desc				= "A spinning bladewheel that inflicts the exploding effect to enemies \nappears whenever you use your kick while standing on solid ground. \nPerforming this ability consumes jetpack energy.",
		effect_desc_short		= "Kicking while in solid ground conjures a exploding spinning bladewheel.",
        precursor_types			= "gadget",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			if(not entityHasCallback(getPlayerEntity(), "player_conjure_bladewheel_listener")) then -- if no other related relics added required callback, adds it
				addNewLuaCallback(getPlayerEntity(), "overtime", "mods/ArcaneReliquary/files/scripts/player/player_conjure_bladewheel_listener.lua", 1, false, "player_conjure_bladewheel_listener")
			end
			addNewSimplifiedInternalVariable(getPlayerEntity(), "player_conjure_bladewheel_listener") -- for each related relic, a new simplified variable like this is added
			
			addNewSimplifiedInternalVariable(getPlayerEntity(), "wrench_orange")
		end,
		ondrop_function			= function()
			removeFirstSimplifiedInternalVariable(getPlayerEntity(), "player_conjure_bladewheel_listener") -- removes just the first variable found
			
			if(countSimplifiedInternalVariable(getPlayerEntity(), "player_conjure_bladewheel_listener") == 0) then -- removes callback only if no other relics are using it
				removeCallback(getPlayerEntity(), "player_conjure_bladewheel_listener")
			end
			
			removeSimplifiedInternalVariable(getPlayerEntity(), "wrench_orange")
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "wrench,exploding_scepter",
	},
	{
		name 					= "wrench_blue",
		ui_name					= "Wrench with Blue Tape",
		categories				= "player",
		effect_desc				= "A spinning bladewheel that deals electricity damage to enemies \nappears whenever you use your kick while standing on solid ground. \nPerforming this ability consumes jetpack energy.",
		effect_desc_short		= "Kicking while in solid ground conjures a electrocuting spinning bladewheel.",
        precursor_types			= "gadget",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			if(not entityHasCallback(getPlayerEntity(), "player_conjure_bladewheel_listener")) then -- if no other related relics added required callback, adds it
				addNewLuaCallback(getPlayerEntity(), "overtime", "mods/ArcaneReliquary/files/scripts/player/player_conjure_bladewheel_listener.lua", 1, false, "player_conjure_bladewheel_listener")
			end
			addNewSimplifiedInternalVariable(getPlayerEntity(), "player_conjure_bladewheel_listener") -- for each related relic, a new simplified variable like this is added
			
			addNewSimplifiedInternalVariable(getPlayerEntity(), "wrench_blue")
		end,
		ondrop_function			= function()
			removeFirstSimplifiedInternalVariable(getPlayerEntity(), "player_conjure_bladewheel_listener") -- removes just the first variable found
			
			if(countSimplifiedInternalVariable(getPlayerEntity(), "player_conjure_bladewheel_listener") == 0) then -- removes callback only if no other relics are using it
				removeCallback(getPlayerEntity(), "player_conjure_bladewheel_listener")
			end
			
			removeSimplifiedInternalVariable(getPlayerEntity(), "wrench_blue")
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "wrench,electrical_plug",
	},
	{
		name 					= "cutting_board",
		ui_name					= "Cutting Board",
		categories				= "player",
		effect_desc				= "Whenever you use your kick while mid air, you release a barrage of bleeding knives. \nPerforming this ability consumes jetpack energy.",
		effect_desc_short		= "Kicking while mid air releases a barrage of bleeding knives.",
        precursor_types			= "gadget",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			if(not entityHasCallback(getPlayerEntity(), "player_release_effectknives_barrage_listener")) then -- if no other related relics added required callback, adds it
				addNewLuaCallback(getPlayerEntity(), "overtime", "mods/ArcaneReliquary/files/scripts/player/player_release_effectknives_barrage_listener.lua", 1, false, "player_release_effectknives_barrage_listener")
			end
			addNewSimplifiedInternalVariable(getPlayerEntity(), "player_release_effectknives_barrage_listener") -- for each related relic, a new simplified variable like this is added
			
			addNewSimplifiedInternalVariable(getPlayerEntity(), "cutting_board")
		end,
		ondrop_function			= function()
			removeFirstSimplifiedInternalVariable(getPlayerEntity(), "player_release_effectknives_barrage_listener") -- removes just the first variable found
			
			if(countSimplifiedInternalVariable(getPlayerEntity(), "player_release_effectknives_barrage_listener") == 0) then -- removes callback only if no other relics are using it
				removeCallback(getPlayerEntity(), "player_release_effectknives_barrage_listener")
			end
			
			removeSimplifiedInternalVariable(getPlayerEntity(), "cutting_board")
		end,
	},
	{
		name 					= "cutting_board_cyan",
		ui_name					= "Cutting Board with Cyan Tape",
		categories				= "player",
		effect_desc				= "Whenever you use your kick while mid air, you release a barrage of freezing knives. \nPerforming this ability consumes jetpack energy.",
		effect_desc_short		= "Kicking while mid air releases a barrage of freezing knives.",
        precursor_types			= "gadget",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			if(not entityHasCallback(getPlayerEntity(), "player_release_effectknives_barrage_listener")) then -- if no other related relics added required callback, adds it
				addNewLuaCallback(getPlayerEntity(), "overtime", "mods/ArcaneReliquary/files/scripts/player/player_release_effectknives_barrage_listener.lua", 1, false, "player_release_effectknives_barrage_listener")
			end
			addNewSimplifiedInternalVariable(getPlayerEntity(), "player_release_effectknives_barrage_listener") -- for each related relic, a new simplified variable like this is added
			
			addNewSimplifiedInternalVariable(getPlayerEntity(), "cutting_board_cyan")
		end,
		ondrop_function			= function()
			removeFirstSimplifiedInternalVariable(getPlayerEntity(), "player_release_effectknives_barrage_listener") -- removes just the first variable found
			
			if(countSimplifiedInternalVariable(getPlayerEntity(), "player_release_effectknives_barrage_listener") == 0) then -- removes callback only if no other relics are using it
				removeCallback(getPlayerEntity(), "player_release_effectknives_barrage_listener")
			end
			
			removeSimplifiedInternalVariable(getPlayerEntity(), "cutting_board_cyan")
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "cutting_board,ice_artifact",
	},
	{
		name 					= "cutting_board_blue",
		ui_name					= "Cutting Board with Blue Tape",
		categories				= "player",
		effect_desc				= "Whenever you use your kick while mid air, you release a barrage of electrocuting knives. \nPerforming this ability consumes jetpack energy.",
		effect_desc_short		= "Kicking while mid air releases a barrage of electrocuting knives.",
        precursor_types			= "gadget",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			if(not entityHasCallback(getPlayerEntity(), "player_release_effectknives_barrage_listener")) then -- if no other related relics added required callback, adds it
				addNewLuaCallback(getPlayerEntity(), "overtime", "mods/ArcaneReliquary/files/scripts/player/player_release_effectknives_barrage_listener.lua", 1, false, "player_release_effectknives_barrage_listener")
			end
			addNewSimplifiedInternalVariable(getPlayerEntity(), "player_release_effectknives_barrage_listener") -- for each related relic, a new simplified variable like this is added
			
			addNewSimplifiedInternalVariable(getPlayerEntity(), "cutting_board_blue")
		end,
		ondrop_function			= function()
			removeFirstSimplifiedInternalVariable(getPlayerEntity(), "player_release_effectknives_barrage_listener") -- removes just the first variable found
			
			if(countSimplifiedInternalVariable(getPlayerEntity(), "player_release_effectknives_barrage_listener") == 0) then -- removes callback only if no other relics are using it
				removeCallback(getPlayerEntity(), "player_release_effectknives_barrage_listener")
			end
			
			removeSimplifiedInternalVariable(getPlayerEntity(), "cutting_board_blue")
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "cutting_board,electrical_plug",
	},
	{
		name 					= "cutting_board_purple",
		ui_name					= "Cutting Board with Purple Tape",
		categories				= "player",
		effect_desc				= "Whenever you use your kick while mid air, you release a barrage of slashing knives. \nPerforming this ability consumes jetpack energy.",
		effect_desc_short		= "Kicking while mid air releases a barrage of slashing knives.",
        precursor_types			= "gadget",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			if(not entityHasCallback(getPlayerEntity(), "player_release_effectknives_barrage_listener")) then -- if no other related relics added required callback, adds it
				addNewLuaCallback(getPlayerEntity(), "overtime", "mods/ArcaneReliquary/files/scripts/player/player_release_effectknives_barrage_listener.lua", 1, false, "player_release_effectknives_barrage_listener")
			end
			addNewSimplifiedInternalVariable(getPlayerEntity(), "player_release_effectknives_barrage_listener") -- for each related relic, a new simplified variable like this is added
			
			addNewSimplifiedInternalVariable(getPlayerEntity(), "cutting_board_purple")
		end,
		ondrop_function			= function()
			removeFirstSimplifiedInternalVariable(getPlayerEntity(), "player_release_effectknives_barrage_listener") -- removes just the first variable found
			
			if(countSimplifiedInternalVariable(getPlayerEntity(), "player_release_effectknives_barrage_listener") == 0) then -- removes callback only if no other relics are using it
				removeCallback(getPlayerEntity(), "player_release_effectknives_barrage_listener")
			end
			
			removeSimplifiedInternalVariable(getPlayerEntity(), "cutting_board_purple")
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "cutting_board,cursed_dagger",
	},
	{
		name 					= "cutting_board_orange",
		ui_name					= "Cutting Board with Orange Tape",
		categories				= "player",
		effect_desc				= "Whenever you use your kick while mid air, you release a barrage of exploding knives. \nPerforming this ability consumes jetpack energy.",
		effect_desc_short		= "Kicking while mid air releases a barrage of exploding knives.",
        precursor_types			= "gadget",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			if(not entityHasCallback(getPlayerEntity(), "player_release_effectknives_barrage_listener")) then -- if no other related relics added required callback, adds it
				addNewLuaCallback(getPlayerEntity(), "overtime", "mods/ArcaneReliquary/files/scripts/player/player_release_effectknives_barrage_listener.lua", 1, false, "player_release_effectknives_barrage_listener")
			end
			addNewSimplifiedInternalVariable(getPlayerEntity(), "player_release_effectknives_barrage_listener") -- for each related relic, a new simplified variable like this is added
			
			addNewSimplifiedInternalVariable(getPlayerEntity(), "cutting_board_orange")
		end,
		ondrop_function			= function()
			removeFirstSimplifiedInternalVariable(getPlayerEntity(), "player_release_effectknives_barrage_listener") -- removes just the first variable found
			
			if(countSimplifiedInternalVariable(getPlayerEntity(), "player_release_effectknives_barrage_listener") == 0) then -- removes callback only if no other relics are using it
				removeCallback(getPlayerEntity(), "player_release_effectknives_barrage_listener")
			end
			
			removeSimplifiedInternalVariable(getPlayerEntity(), "cutting_board_orange")
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "cutting_board,exploding_scepter",
	},
	{
		name 					= "cutting_board_green",
		ui_name					= "Cutting Board with Green Tape",
		categories				= "player",
		effect_desc				= "Whenever you use your kick while mid air, you release a barrage of slapping knives. \nPerforming this ability consumes jetpack energy.",
		effect_desc_short		= "Kicking while mid air releases a barrage of slapping knives.",
        precursor_types			= "gadget",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			if(not entityHasCallback(getPlayerEntity(), "player_release_effectknives_barrage_listener")) then -- if no other related relics added required callback, adds it
				addNewLuaCallback(getPlayerEntity(), "overtime", "mods/ArcaneReliquary/files/scripts/player/player_release_effectknives_barrage_listener.lua", 1, false, "player_release_effectknives_barrage_listener")
			end
			addNewSimplifiedInternalVariable(getPlayerEntity(), "player_release_effectknives_barrage_listener") -- for each related relic, a new simplified variable like this is added
			
			addNewSimplifiedInternalVariable(getPlayerEntity(), "cutting_board_green")
		end,
		ondrop_function			= function()
			removeFirstSimplifiedInternalVariable(getPlayerEntity(), "player_release_effectknives_barrage_listener") -- removes just the first variable found
			
			if(countSimplifiedInternalVariable(getPlayerEntity(), "player_release_effectknives_barrage_listener") == 0) then -- removes callback only if no other relics are using it
				removeCallback(getPlayerEntity(), "player_release_effectknives_barrage_listener")
			end
			
			removeSimplifiedInternalVariable(getPlayerEntity(), "cutting_board_green")
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "cutting_board,fly_swatter",
	},
	{
		name 					= "cutting_board_pink",
		ui_name					= "Cutting Board with Pink Tape",
		categories				= "player",
		effect_desc				= "Whenever you use your kick while mid air, you release a barrage of mutating knives. \nPerforming this ability consumes jetpack energy.",
		effect_desc_short		= "Kicking while mid air releases a barrage of mutating knives.",
        precursor_types			= "gadget",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			if(not entityHasCallback(getPlayerEntity(), "player_release_effectknives_barrage_listener")) then -- if no other related relics added required callback, adds it
				addNewLuaCallback(getPlayerEntity(), "overtime", "mods/ArcaneReliquary/files/scripts/player/player_release_effectknives_barrage_listener.lua", 1, false, "player_release_effectknives_barrage_listener")
			end
			addNewSimplifiedInternalVariable(getPlayerEntity(), "player_release_effectknives_barrage_listener") -- for each related relic, a new simplified variable like this is added
			
			addNewSimplifiedInternalVariable(getPlayerEntity(), "cutting_board_pink")
		end,
		ondrop_function			= function()
			removeFirstSimplifiedInternalVariable(getPlayerEntity(), "player_release_effectknives_barrage_listener") -- removes just the first variable found
			
			if(countSimplifiedInternalVariable(getPlayerEntity(), "player_release_effectknives_barrage_listener") == 0) then -- removes callback only if no other relics are using it
				removeCallback(getPlayerEntity(), "player_release_effectknives_barrage_listener")
			end
			
			removeSimplifiedInternalVariable(getPlayerEntity(), "cutting_board_pink")
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "cutting_board,mutating_bead",
	},
	{
		name 					= "cutting_board_blue_light",
		ui_name					= "Cutting Board with Light Blue Tape",
		categories				= "player",
		effect_desc				= "Whenever you use your kick while mid air, you release a barrage of tele-knives. \nPerforming this ability consumes jetpack energy.",
		effect_desc_short		= "Kicking while mid air releases a barrage of tele-knives.",
        precursor_types			= "gadget",
		upgrade_effect_desc		= "",
		is_upgradable 			= false,
		onpickup_function		= function()
			if(not entityHasCallback(getPlayerEntity(), "player_release_effectknives_barrage_listener")) then -- if no other related relics added required callback, adds it
				addNewLuaCallback(getPlayerEntity(), "overtime", "mods/ArcaneReliquary/files/scripts/player/player_release_effectknives_barrage_listener.lua", 1, false, "player_release_effectknives_barrage_listener")
			end
			addNewSimplifiedInternalVariable(getPlayerEntity(), "player_release_effectknives_barrage_listener") -- for each related relic, a new simplified variable like this is added
			
			addNewSimplifiedInternalVariable(getPlayerEntity(), "cutting_board_blue_light")
		end,
		ondrop_function			= function()
			removeFirstSimplifiedInternalVariable(getPlayerEntity(), "player_release_effectknives_barrage_listener") -- removes just the first variable found
			
			if(countSimplifiedInternalVariable(getPlayerEntity(), "player_release_effectknives_barrage_listener") == 0) then -- removes callback only if no other relics are using it
				removeCallback(getPlayerEntity(), "player_release_effectknives_barrage_listener")
			end
			
			removeSimplifiedInternalVariable(getPlayerEntity(), "cutting_board_blue_light")
		end,
		combine_exclusive		= true,
		combine_requires_relics	= "cutting_board,wizard_hat",
	},
}

-- Returns a random relic taking into account the entire list
function getRandomRelic()
	SetRandomSeed( GameGetFrameNum(), math.random(1,50))
	return relics_list[Random(1,#relics_list)]
end

-- Returns a random basic relic
function getRandomBasicRelic()
	local eligible_relics = {}
	for i,relic in ipairs(relics_list) do
		if(relic.combine_exclusive == nil or relic.combine_exclusive == false) then
			table.insert(eligible_relics, relic)
		end
	end
	
	SetRandomSeed( GameGetFrameNum(), math.random(1,50))
	return eligible_relics[Random(1,#eligible_relics)]
end

-- Returns a random combine exclusive relic
function getRandomCombineExclusiveRelic()
	local eligible_relics = {}
	for i,relic in ipairs(relics_list) do
		if(relic.combine_exclusive ~= nil or relic.combine_exclusive == true) then
			table.insert(eligible_relics, relic)
		end
	end
	
	SetRandomSeed( GameGetFrameNum(), math.random(1,50))
	return eligible_relics[Random(1,#eligible_relics)]
end

-- Returns a random upgradable relic
function getRandomUpgradableRelic()
	local eligible_relics = {}
	for i,relic in ipairs(relics_list) do
		if(relic.is_upgradable == true) then
			table.insert(eligible_relics, relic)
		end
	end
	
	SetRandomSeed( GameGetFrameNum(), math.random(1,50))
	return eligible_relics[Random(1,#eligible_relics)]
end

-- determines whether or not relic with relicName is upgradable
function isRelicUpgradable(relicName)
	local is_relic_upgradable = false
	for i,relic in ipairs(relics_list) do
		if(relic.name == relicName) then
			is_relic_upgradable = relic.is_upgradable
			break
		end
	end
	return is_relic_upgradable
end

-- Obtains a relic from relics_list by its name
function getRelic(relicName)
	local foundRelic = nil
	for i,relic in ipairs(relics_list) do
		if(relic.name == relicName) then
			foundRelic = relic
			break
		end
	end
	return foundRelic
end

-- Obtains a list of relic objects that correspond to supplied precursor_types
function getRelicsByPrecursorType(precursor_type)
	local precursor_types_relics = {}
	for i,relic in ipairs(relics_list) do
		if(string.find(relic.precursor_types, precursor_type)) then
			table.insert(precursor_types_relics, relic)
		end
	end
	return precursor_types_relics
end

-- Obtains a list of relic objects containing only combine exclusive relics
function getCombineExclusiveRelics()
	local combine_exclusive_relics = {}
	for i,relic in ipairs(relics_list) do
		if(relic.combine_exclusive ~= nil and relic.combine_exclusive) then
			table.insert(combine_exclusive_relics, relic)
		end
	end
	return combine_exclusive_relics
end

-- Utility functions to return base dirs for commmon entities used in this script
function getRelicsBaseDir()
	return "mods/ArcaneReliquary/files/entity_definitions/relics/"
end

-- Utility functions to return main variables related to relics
function getMaximumOwnedRelics()
	return maximum_owned_relics
end

function getRelicSalePrice()
	return relic_sale_price
end

function getRelicBuyPrice()
	return relic_buy_price
end

function getRelicSellPrice()
	return relic_sell_price
end

function getRelicUpgradeCosts()
	return relic_material_base_upgrade_cost, relic_material_advanced_upgrade_cost
end

function getRelicConstructCosts()
	return relic_material_base_construct_cost, relic_precursor_material_construct_cost
end

function getRelicUpgradeMaterialName()
	return relic_upgrade_material_name, relic_advanced_upgrade_material_name
end

function getRelicMaxUpgradeLevel()
	return relic_max_upgrade_level
end

function getRelicConstructionMaterialName()
	return relic_construction_main_material_name
end

-- utility functions
function getGUIIconsPath()
	return "mods/ArcaneReliquary/files/gui_gfx/"
end

function getItemUIPath()
	return "mods/ArcaneReliquary/files/item_ui_gfx/"
end

-- returns relic by index
function getRelicByIndex(index)
	return relics_list[index]
end